export const bk_tooltip = "bk-tooltip";
export const bk_tooltip_arrow = "bk-tooltip-arrow";
export const bk_tooltip_custom = "bk-tooltip-custom";
export const bk_tooltip_row_label = "bk-tooltip-row-label";
export const bk_tooltip_row_value = "bk-tooltip-row-value";
export const bk_tooltip_color_block = "bk-tooltip-color-block";
//# sourceMappingURL=tooltips.js.map