import { Model } from "../../model";
export class TickFormatter extends Model {
    constructor(attrs) {
        super(attrs);
    }
}
TickFormatter.__name__ = "TickFormatter";
//# sourceMappingURL=tick_formatter.js.map