export const bk_toolbar = "bk-toolbar";
export const bk_toolbar_hidden = "bk-toolbar-hidden";
export const bk_toolbar_button = "bk-toolbar-button";
export const bk_button_bar = "bk-button-bar";
export const bk_toolbar_button_custom_action = "bk-toolbar-button-custom-action";
//# sourceMappingURL=toolbar.js.map