/*!
 * Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * Neither the name of Anaconda nor the names of any contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
*/
(function(root, factory) {
  const bokeh = factory();
  bokeh.__bokeh__ = true;
  if (typeof root.Bokeh === "undefined" || typeof root.Bokeh.__bokeh__ === "undefined") {
    root.Bokeh = bokeh;
  }
  const Bokeh = root.Bokeh;
  Bokeh[bokeh.version] = bokeh;
})(this, function() {
  var define;
  var parent_require = typeof require === "function" && require
  return (function(modules, entry, aliases, externals) {
    if (aliases === undefined) aliases = {};
    if (externals === undefined) externals = {};

    var cache = {};

    var normalize = function(name) {
      if (typeof name === "number")
        return name;

      if (name === "bokehjs")
        return entry;

      var prefix = "@bokehjs/"
      if (name.slice(0, prefix.length) === prefix)
        name = name.slice(prefix.length)

      var alias = aliases[name]
      if (alias != null)
        return alias;

      var trailing = name.length > 0 && name[name.lenght-1] === "/";
      var index = aliases[name + (trailing ? "" : "/") + "index"];
      if (index != null)
        return index;

      return name;
    }

    var require = function(name) {
      var mod = cache[name];
      if (!mod) {
        var id = normalize(name);

        mod = cache[id];
        if (!mod) {
          if (!modules[id]) {
            if (externals[id] === false || (externals[id] == true && parent_require)) {
              try {
                mod = {exports: externals[id] ? parent_require(id) : {}};
                cache[id] = cache[name] = mod;
                return mod.exports;
              } catch (e) {}
            }

            var err = new Error("Cannot find module '" + name + "'");
            err.code = 'MODULE_NOT_FOUND';
            throw err;
          }

          mod = {exports: {}};
          cache[id] = cache[name] = mod;
          modules[id].call(mod.exports, require, mod, mod.exports);
        } else
          cache[name] = mod;
      }

      return mod.exports;
    }
    require.resolve = function(name) {
      return ""
    }

    var main = require(entry);
    main.require = require;

    if (typeof Proxy !== "undefined") {
      // allow Bokeh.loader["@bokehjs/module/name"] syntax
      main.loader = new Proxy({}, {
        get: function(_obj, module) {
          return require(module);
        }
      });
    }

    main.register_plugin = function(plugin_modules, plugin_entry, plugin_aliases, plugin_externals) {
      if (plugin_aliases === undefined) plugin_aliases = {};
      if (plugin_externals === undefined) plugin_externals = {};

      for (var name in plugin_modules) {
        modules[name] = plugin_modules[name];
      }

      for (var name in plugin_aliases) {
        aliases[name] = plugin_aliases[name];
      }

      for (var name in plugin_externals) {
        externals[name] = plugin_externals[name];
      }

      var plugin = require(plugin_entry);

      for (var name in plugin) {
        main[name] = plugin[name];
      }

      return plugin;
    }

    return main;
  })
([
/* legacy.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    require(2) /* ./polyfill */;
    tslib_1.__exportStar(require(116) /* ./main */, exports);
},
/* tslib/tslib.es6.js */ function _(require, module, exports) {
    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.
    
    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.
    
    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    Object.defineProperty(exports, "__esModule", { value: true });
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) {
                for (var p in b)
                    if (b.hasOwnProperty(p))
                        d[p] = b[p];
            };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    exports.__extends = __extends;
    exports.__assign = function () {
        exports.__assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return exports.__assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    exports.__rest = __rest;
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    exports.__decorate = __decorate;
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    exports.__param = __param;
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    exports.__metadata = __metadata;
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) {
                try {
                    step(generator.next(value));
                }
                catch (e) {
                    reject(e);
                }
            }
            function rejected(value) {
                try {
                    step(generator["throw"](value));
                }
                catch (e) {
                    reject(e);
                }
            }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    exports.__awaiter = __awaiter;
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () {
                if (t[0] & 1)
                    throw t[1];
                return t[1];
            }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    exports.__generator = __generator;
    function __createBinding(o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    }
    exports.__createBinding = __createBinding;
    function __exportStar(m, exports) {
        for (var p in m)
            if (p !== "default" && !exports.hasOwnProperty(p))
                exports[p] = m[p];
    }
    exports.__exportStar = __exportStar;
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    exports.__values = __values;
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    exports.__read = __read;
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    exports.__spread = __spread;
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    exports.__spreadArrays = __spreadArrays;
    ;
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    exports.__await = __await;
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) {
            if (g[n])
                i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); };
        }
        function resume(n, v) {
            try {
                step(g[n](v));
            }
            catch (e) {
                settle(q[0][3], e);
            }
        }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) {
            if (f(v), q.shift(), q.length)
                resume(q[0][0], q[0][1]);
        }
    }
    exports.__asyncGenerator = __asyncGenerator;
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    exports.__asyncDelegator = __asyncDelegator;
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    exports.__asyncValues = __asyncValues;
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    exports.__makeTemplateObject = __makeTemplateObject;
    ;
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result.default = mod;
        return result;
    }
    exports.__importStar = __importStar;
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    exports.__importDefault = __importDefault;
    function __classPrivateFieldGet(receiver, privateMap) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to get private field on non-instance");
        }
        return privateMap.get(receiver);
    }
    exports.__classPrivateFieldGet = __classPrivateFieldGet;
    function __classPrivateFieldSet(receiver, privateMap, value) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to set private field on non-instance");
        }
        privateMap.set(receiver, value);
        return value;
    }
    exports.__classPrivateFieldSet = __classPrivateFieldSet;
},
/* polyfill.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    require(3) /* es5-ext/object/assign/implement */;
    require(12) /* es5-ext/object/entries/implement */;
    require(15) /* es5-ext/number/is-integer/implement */;
    require(18) /* es5-ext/string/#/repeat/implement */;
    require(25) /* es5-ext/array/from/implement */;
    require(55) /* es5-ext/array/#/fill/implement */;
    require(58) /* es5-ext/math/log10/implement */;
    require(61) /* es5-ext/math/log1p/implement */;
    require(64) /* es6-set/implement */;
    require(100) /* es6-map/implement */;
    require(107) /* es6-weak-map/implement */;
    require(113) /* es6-symbol/implement */;
    require(114) /* es6-promise/auto */;
    if (typeof Object.is === "undefined") {
        Object.is = function (a, b) {
            if (a === b)
                return a !== 0 || 1 / a === 1 / b;
            else
                return a !== a && b !== b;
        };
    }
    if (typeof Object.values === "undefined") {
        Object.values = function (obj) {
            return Object.keys(obj).map(function (key) { return obj[key]; });
        };
    }
    if (typeof Uint8Array.prototype.fill === "undefined") {
        Uint8Array.prototype.fill = Array.prototype.fill;
        Int8Array.prototype.fill = Array.prototype.fill;
        Uint16Array.prototype.fill = Array.prototype.fill;
        Int16Array.prototype.fill = Array.prototype.fill;
        Uint32Array.prototype.fill = Array.prototype.fill;
        Int32Array.prototype.fill = Array.prototype.fill;
        Float32Array.prototype.fill = Array.prototype.fill;
        Float64Array.prototype.fill = Array.prototype.fill; // lgtm [js/automatic-semicolon-insertion]
    }
    if (typeof Array.prototype[Symbol.iterator] === "undefined") {
        function iterator() {
            var i = 0;
            var self = this;
            return {
                next: function () {
                    var done = self.length <= i;
                    var value = done ? undefined : self[i++];
                    return { value: value, done: done };
                },
            };
        }
        (function () {
            arguments.constructor.prototype[Symbol.iterator] = iterator;
            Array.prototype[Symbol.iterator] = iterator;
            Uint8Array.prototype[Symbol.iterator] = iterator;
            Int8Array.prototype[Symbol.iterator] = iterator;
            Uint16Array.prototype[Symbol.iterator] = iterator;
            Int16Array.prototype[Symbol.iterator] = iterator;
            Uint32Array.prototype[Symbol.iterator] = iterator;
            Int32Array.prototype[Symbol.iterator] = iterator;
            Float32Array.prototype[Symbol.iterator] = iterator;
            Float64Array.prototype[Symbol.iterator] = iterator; // lgtm [js/automatic-semicolon-insertion]
        })();
    }
    // fixes up a problem with some versions of IE11
    // ref: http://stackoverflow.com/questions/22062313/imagedata-set-in-internetexplorer
    if (typeof CanvasPixelArray !== "undefined") {
        CanvasPixelArray.prototype.set = function (arr) {
            for (var i = 0; i < this.length; i++) {
                this[i] = arr[i];
            }
        };
    }
},
/* es5-ext/object/assign/implement.js */ function _(require, module, exports) {
    if (!require(4) /* ./is-implemented */()) {
        Object.defineProperty(Object, "assign", {
            value: require(5) /* ./shim */,
            configurable: true,
            enumerable: false,
            writable: true
        });
    }
},
/* es5-ext/object/assign/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        var assign = Object.assign, obj;
        if (typeof assign !== "function")
            return false;
        obj = { foo: "raz" };
        assign(obj, { bar: "dwa" }, { trzy: "trzy" });
        return obj.foo + obj.bar + obj.trzy === "razdwatrzy";
    };
},
/* es5-ext/object/assign/shim.js */ function _(require, module, exports) {
    var keys = require(6) /* ../keys */, value = require(11) /* ../valid-value */, max = Math.max;
    module.exports = function (dest, src /*, …srcn*/) {
        var error, i, length = max(arguments.length, 2), assign;
        dest = Object(value(dest));
        assign = function (key) {
            try {
                dest[key] = src[key];
            }
            catch (e) {
                if (!error)
                    error = e;
            }
        };
        for (i = 1; i < length; ++i) {
            src = arguments[i];
            keys(src).forEach(assign);
        }
        if (error !== undefined)
            throw error;
        return dest;
    };
},
/* es5-ext/object/keys/index.js */ function _(require, module, exports) {
    module.exports = require(7) /* ./is-implemented */() ? Object.keys : require(8) /* ./shim */;
},
/* es5-ext/object/keys/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        try {
            Object.keys("primitive");
            return true;
        }
        catch (e) {
            return false;
        }
    };
},
/* es5-ext/object/keys/shim.js */ function _(require, module, exports) {
    var isValue = require(9) /* ../is-value */;
    var keys = Object.keys;
    module.exports = function (object) { return keys(isValue(object) ? Object(object) : object); };
},
/* es5-ext/object/is-value.js */ function _(require, module, exports) {
    var _undefined = require(10) /* ../function/noop */(); // Support ES3 engines
    module.exports = function (val) { return val !== _undefined && val !== null; };
},
/* es5-ext/function/noop.js */ function _(require, module, exports) {
    // eslint-disable-next-line no-empty-function
    module.exports = function () { };
},
/* es5-ext/object/valid-value.js */ function _(require, module, exports) {
    var isValue = require(9) /* ./is-value */;
    module.exports = function (value) {
        if (!isValue(value))
            throw new TypeError("Cannot use null or undefined");
        return value;
    };
},
/* es5-ext/object/entries/implement.js */ function _(require, module, exports) {
    if (!require(13) /* ./is-implemented */()) {
        Object.defineProperty(Object, "entries", {
            value: require(14) /* ./shim */,
            configurable: true,
            enumerable: false,
            writable: true
        });
    }
},
/* es5-ext/object/entries/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        try {
            return Object.entries({ foo: 12 })[0][0] === "foo";
        }
        catch (e) {
            return false;
        }
    };
},
/* es5-ext/object/entries/shim.js */ function _(require, module, exports) {
    var ensureValue = require(11) /* ../valid-value */;
    module.exports = function (object) {
        ensureValue(object);
        var result = [];
        object = Object(object);
        for (var key in object) {
            if (!propertyIsEnumerable.call(object, key))
                continue;
            result.push([key, object[key]]);
        }
        return result;
    };
},
/* es5-ext/number/is-integer/implement.js */ function _(require, module, exports) {
    if (!require(16) /* ./is-implemented */()) {
        Object.defineProperty(Number, "isInteger", {
            value: require(17) /* ./shim */,
            configurable: true,
            enumerable: false,
            writable: true
        });
    }
},
/* es5-ext/number/is-integer/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        var isInteger = Number.isInteger;
        if (typeof isInteger !== "function")
            return false;
        return !isInteger("23") && isInteger(34) && !isInteger(32.34);
    };
},
/* es5-ext/number/is-integer/shim.js */ function _(require, module, exports) {
    module.exports = function (value) {
        if (typeof value !== "number")
            return false;
        return value % 1 === 0;
    };
},
/* es5-ext/string/#/repeat/implement.js */ function _(require, module, exports) {
    if (!require(19) /* ./is-implemented */()) {
        Object.defineProperty(String.prototype, "repeat", {
            value: require(20) /* ./shim */,
            configurable: true,
            enumerable: false,
            writable: true
        });
    }
},
/* es5-ext/string/#/repeat/is-implemented.js */ function _(require, module, exports) {
    var str = "foo";
    module.exports = function () {
        if (typeof str.repeat !== "function")
            return false;
        return str.repeat(2) === "foofoo";
    };
},
/* es5-ext/string/#/repeat/shim.js */ function _(require, module, exports) {
    var value = require(11) /* ../../../object/valid-value */, toInteger = require(21) /* ../../../number/to-integer */;
    module.exports = function (count) {
        var str = String(value(this)), result;
        count = toInteger(count);
        if (count < 0)
            throw new RangeError("Count must be >= 0");
        if (!isFinite(count))
            throw new RangeError("Count must be < ∞");
        result = "";
        while (count) {
            if (count % 2)
                result += str;
            if (count > 1)
                str += str;
            // eslint-disable-next-line no-bitwise
            count >>= 1;
        }
        return result;
    };
},
/* es5-ext/number/to-integer.js */ function _(require, module, exports) {
    var sign = require(22) /* ../math/sign */, abs = Math.abs, floor = Math.floor;
    module.exports = function (value) {
        if (isNaN(value))
            return 0;
        value = Number(value);
        if (value === 0 || !isFinite(value))
            return value;
        return sign(value) * floor(abs(value));
    };
},
/* es5-ext/math/sign/index.js */ function _(require, module, exports) {
    module.exports = require(23) /* ./is-implemented */() ? Math.sign : require(24) /* ./shim */;
},
/* es5-ext/math/sign/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        var sign = Math.sign;
        if (typeof sign !== "function")
            return false;
        return sign(10) === 1 && sign(-20) === -1;
    };
},
/* es5-ext/math/sign/shim.js */ function _(require, module, exports) {
    module.exports = function (value) {
        value = Number(value);
        if (isNaN(value) || value === 0)
            return value;
        return value > 0 ? 1 : -1;
    };
},
/* es5-ext/array/from/implement.js */ function _(require, module, exports) {
    if (!require(26) /* ./is-implemented */()) {
        Object.defineProperty(Array, "from", {
            value: require(27) /* ./shim */,
            configurable: true,
            enumerable: false,
            writable: true
        });
    }
},
/* es5-ext/array/from/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        var from = Array.from, arr, result;
        if (typeof from !== "function")
            return false;
        arr = ["raz", "dwa"];
        result = from(arr);
        return Boolean(result && result !== arr && result[1] === "dwa");
    };
},
/* es5-ext/array/from/shim.js */ function _(require, module, exports) {
    var iteratorSymbol = require(28) /* es6-symbol */.iterator, isArguments = require(50) /* ../../function/is-arguments */, isFunction = require(51) /* ../../function/is-function */, toPosInt = require(52) /* ../../number/to-pos-integer */, callable = require(53) /* ../../object/valid-callable */, validValue = require(11) /* ../../object/valid-value */, isValue = require(9) /* ../../object/is-value */, isString = require(54) /* ../../string/is-string */, isArray = Array.isArray, call = Function.prototype.call, desc = { configurable: true, enumerable: true, writable: true, value: null }, defineProperty = Object.defineProperty;
    // eslint-disable-next-line complexity, max-lines-per-function
    module.exports = function (arrayLike /*, mapFn, thisArg*/) {
        var mapFn = arguments[1], thisArg = arguments[2], Context, i, j, arr, length, code, iterator, result, getIterator, value;
        arrayLike = Object(validValue(arrayLike));
        if (isValue(mapFn))
            callable(mapFn);
        if (!this || this === Array || !isFunction(this)) {
            // Result: Plain array
            if (!mapFn) {
                if (isArguments(arrayLike)) {
                    // Source: Arguments
                    length = arrayLike.length;
                    if (length !== 1)
                        return Array.apply(null, arrayLike);
                    arr = new Array(1);
                    arr[0] = arrayLike[0];
                    return arr;
                }
                if (isArray(arrayLike)) {
                    // Source: Array
                    arr = new Array((length = arrayLike.length));
                    for (i = 0; i < length; ++i)
                        arr[i] = arrayLike[i];
                    return arr;
                }
            }
            arr = [];
        }
        else {
            // Result: Non plain array
            Context = this;
        }
        if (!isArray(arrayLike)) {
            if ((getIterator = arrayLike[iteratorSymbol]) !== undefined) {
                // Source: Iterator
                iterator = callable(getIterator).call(arrayLike);
                if (Context)
                    arr = new Context();
                result = iterator.next();
                i = 0;
                while (!result.done) {
                    value = mapFn ? call.call(mapFn, thisArg, result.value, i) : result.value;
                    if (Context) {
                        desc.value = value;
                        defineProperty(arr, i, desc);
                    }
                    else {
                        arr[i] = value;
                    }
                    result = iterator.next();
                    ++i;
                }
                length = i;
            }
            else if (isString(arrayLike)) {
                // Source: String
                length = arrayLike.length;
                if (Context)
                    arr = new Context();
                for (i = 0, j = 0; i < length; ++i) {
                    value = arrayLike[i];
                    if (i + 1 < length) {
                        code = value.charCodeAt(0);
                        // eslint-disable-next-line max-depth
                        if (code >= 0xd800 && code <= 0xdbff)
                            value += arrayLike[++i];
                    }
                    value = mapFn ? call.call(mapFn, thisArg, value, j) : value;
                    if (Context) {
                        desc.value = value;
                        defineProperty(arr, j, desc);
                    }
                    else {
                        arr[j] = value;
                    }
                    ++j;
                }
                length = j;
            }
        }
        if (length === undefined) {
            // Source: array or array-like
            length = toPosInt(arrayLike.length);
            if (Context)
                arr = new Context(length);
            for (i = 0; i < length; ++i) {
                value = mapFn ? call.call(mapFn, thisArg, arrayLike[i], i) : arrayLike[i];
                if (Context) {
                    desc.value = value;
                    defineProperty(arr, i, desc);
                }
                else {
                    arr[i] = value;
                }
            }
        }
        if (Context) {
            desc.value = null;
            arr.length = length;
        }
        return arr;
    };
},
/* es6-symbol/index.js */ function _(require, module, exports) {
    module.exports = require(29) /* ./is-implemented */()
        ? require(30) /* ext/global-this */.Symbol
        : require(33) /* ./polyfill */;
},
/* es6-symbol/is-implemented.js */ function _(require, module, exports) {
    var global = require(30) /* ext/global-this */, validTypes = { object: true, symbol: true };
    module.exports = function () {
        var Symbol = global.Symbol;
        var symbol;
        if (typeof Symbol !== "function")
            return false;
        symbol = Symbol("test symbol");
        try {
            String(symbol);
        }
        catch (e) {
            return false;
        }
        // Return 'true' also for polyfills
        if (!validTypes[typeof Symbol.iterator])
            return false;
        if (!validTypes[typeof Symbol.toPrimitive])
            return false;
        if (!validTypes[typeof Symbol.toStringTag])
            return false;
        return true;
    };
},
/* ext/global-this/index.js */ function _(require, module, exports) {
    module.exports = require(31) /* ./is-implemented */() ? globalThis : require(32) /* ./implementation */;
},
/* ext/global-this/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        if (typeof globalThis !== "object")
            return false;
        if (!globalThis)
            return false;
        return globalThis.Array === Array;
    };
},
/* ext/global-this/implementation.js */ function _(require, module, exports) {
    var naiveFallback = function () {
        if (typeof self === "object" && self)
            return self;
        if (typeof window === "object" && window)
            return window;
        throw new Error("Unable to resolve global `this`");
    };
    module.exports = (function () {
        if (this)
            return this;
        // Unexpected strict mode (may happen if e.g. bundled into ESM module)
        // Thanks @mathiasbynens -> https://mathiasbynens.be/notes/globalthis
        // In all ES5+ engines global object inherits from Object.prototype
        // (if you approached one that doesn't please report)
        try {
            Object.defineProperty(Object.prototype, "__global__", {
                get: function () { return this; },
                configurable: true
            });
        }
        catch (error) {
            // Unfortunate case of Object.prototype being sealed (via preventExtensions, seal or freeze)
            return naiveFallback();
        }
        try {
            // Safari case (window.__global__ is resolved with global context, but __global__ does not)
            if (!__global__)
                return naiveFallback();
            return __global__;
        }
        finally {
            delete Object.prototype.__global__;
        }
    })();
},
/* es6-symbol/polyfill.js */ function _(require, module, exports) {
    var d = require(34) /* d */, validateSymbol = require(45) /* ./validate-symbol */, NativeSymbol = require(30) /* ext/global-this */.Symbol, generateName = require(47) /* ./lib/private/generate-name */, setupStandardSymbols = require(48) /* ./lib/private/setup/standard-symbols */, setupSymbolRegistry = require(49) /* ./lib/private/setup/symbol-registry */;
    var create = Object.create, defineProperties = Object.defineProperties, defineProperty = Object.defineProperty;
    var SymbolPolyfill, HiddenSymbol, isNativeSafe;
    if (typeof NativeSymbol === "function") {
        try {
            String(NativeSymbol());
            isNativeSafe = true;
        }
        catch (ignore) { }
    }
    else {
        NativeSymbol = null;
    }
    // Internal constructor (not one exposed) for creating Symbol instances.
    // This one is used to ensure that `someSymbol instanceof Symbol` always return false
    HiddenSymbol = function Symbol(description) {
        if (this instanceof HiddenSymbol)
            throw new TypeError("Symbol is not a constructor");
        return SymbolPolyfill(description);
    };
    // Exposed `Symbol` constructor
    // (returns instances of HiddenSymbol)
    module.exports = SymbolPolyfill = function Symbol(description) {
        var symbol;
        if (this instanceof Symbol)
            throw new TypeError("Symbol is not a constructor");
        if (isNativeSafe)
            return NativeSymbol(description);
        symbol = create(HiddenSymbol.prototype);
        description = description === undefined ? "" : String(description);
        return defineProperties(symbol, {
            __description__: d("", description),
            __name__: d("", generateName(description))
        });
    };
    setupStandardSymbols(SymbolPolyfill);
    setupSymbolRegistry(SymbolPolyfill);
    // Internal tweaks for real symbol producer
    defineProperties(HiddenSymbol.prototype, {
        constructor: d(SymbolPolyfill),
        toString: d("", function () { return this.__name__; })
    });
    // Proper implementation of methods exposed on Symbol.prototype
    // They won't be accessible on produced symbol instances as they derive from HiddenSymbol.prototype
    defineProperties(SymbolPolyfill.prototype, {
        toString: d(function () { return "Symbol (" + validateSymbol(this).__description__ + ")"; }),
        valueOf: d(function () { return validateSymbol(this); })
    });
    defineProperty(SymbolPolyfill.prototype, SymbolPolyfill.toPrimitive, d("", function () {
        var symbol = validateSymbol(this);
        if (typeof symbol === "symbol")
            return symbol;
        return symbol.toString();
    }));
    defineProperty(SymbolPolyfill.prototype, SymbolPolyfill.toStringTag, d("c", "Symbol"));
    // Proper implementaton of toPrimitive and toStringTag for returned symbol instances
    defineProperty(HiddenSymbol.prototype, SymbolPolyfill.toStringTag, d("c", SymbolPolyfill.prototype[SymbolPolyfill.toStringTag]));
    // Note: It's important to define `toPrimitive` as last one, as some implementations
    // implement `toPrimitive` natively without implementing `toStringTag` (or other specified symbols)
    // And that may invoke error in definition flow:
    // See: https://github.com/medikoo/es6-symbol/issues/13#issuecomment-164146149
    defineProperty(HiddenSymbol.prototype, SymbolPolyfill.toPrimitive, d("c", SymbolPolyfill.prototype[SymbolPolyfill.toPrimitive]));
},
/* d/index.js */ function _(require, module, exports) {
    var isValue = require(35) /* type/value/is */, isPlainFunction = require(36) /* type/plain-function/is */, assign = require(40) /* es5-ext/object/assign */, normalizeOpts = require(41) /* es5-ext/object/normalize-options */, contains = require(42) /* es5-ext/string/#/contains */;
    var d = (module.exports = function (dscr, value /*, options*/) {
        var c, e, w, options, desc;
        if (arguments.length < 2 || typeof dscr !== "string") {
            options = value;
            value = dscr;
            dscr = null;
        }
        else {
            options = arguments[2];
        }
        if (isValue(dscr)) {
            c = contains.call(dscr, "c");
            e = contains.call(dscr, "e");
            w = contains.call(dscr, "w");
        }
        else {
            c = w = true;
            e = false;
        }
        desc = { value: value, configurable: c, enumerable: e, writable: w };
        return !options ? desc : assign(normalizeOpts(options), desc);
    });
    d.gs = function (dscr, get, set /*, options*/) {
        var c, e, options, desc;
        if (typeof dscr !== "string") {
            options = set;
            set = get;
            get = dscr;
            dscr = null;
        }
        else {
            options = arguments[3];
        }
        if (!isValue(get)) {
            get = undefined;
        }
        else if (!isPlainFunction(get)) {
            options = get;
            get = set = undefined;
        }
        else if (!isValue(set)) {
            set = undefined;
        }
        else if (!isPlainFunction(set)) {
            options = set;
            set = undefined;
        }
        if (isValue(dscr)) {
            c = contains.call(dscr, "c");
            e = contains.call(dscr, "e");
        }
        else {
            c = true;
            e = false;
        }
        desc = { get: get, set: set, configurable: c, enumerable: e };
        return !options ? desc : assign(normalizeOpts(options), desc);
    };
},
/* type/value/is.js */ function _(require, module, exports) {
    // ES3 safe
    var _undefined = void 0;
    module.exports = function (value) { return value !== _undefined && value !== null; };
},
/* type/plain-function/is.js */ function _(require, module, exports) {
    var isFunction = require(37) /* ../function/is */;
    var classRe = /^\s*class[\s{/}]/, functionToString = Function.prototype.toString;
    module.exports = function (value) {
        if (!isFunction(value))
            return false;
        if (classRe.test(functionToString.call(value)))
            return false;
        return true;
    };
},
/* type/function/is.js */ function _(require, module, exports) {
    var isPrototype = require(38) /* ../prototype/is */;
    module.exports = function (value) {
        if (typeof value !== "function")
            return false;
        if (!hasOwnProperty.call(value, "length"))
            return false;
        try {
            if (typeof value.length !== "number")
                return false;
            if (typeof value.call !== "function")
                return false;
            if (typeof value.apply !== "function")
                return false;
        }
        catch (error) {
            return false;
        }
        return !isPrototype(value);
    };
},
/* type/prototype/is.js */ function _(require, module, exports) {
    var isObject = require(39) /* ../object/is */;
    module.exports = function (value) {
        if (!isObject(value))
            return false;
        try {
            if (!value.constructor)
                return false;
            return value.constructor.prototype === value;
        }
        catch (error) {
            return false;
        }
    };
},
/* type/object/is.js */ function _(require, module, exports) {
    var isValue = require(35) /* ../value/is */;
    // prettier-ignore
    var possibleTypes = { "object": true, "function": true, "undefined": true /* document.all */ };
    module.exports = function (value) {
        if (!isValue(value))
            return false;
        return hasOwnProperty.call(possibleTypes, typeof value);
    };
},
/* es5-ext/object/assign/index.js */ function _(require, module, exports) {
    module.exports = require(4) /* ./is-implemented */() ? Object.assign : require(5) /* ./shim */;
},
/* es5-ext/object/normalize-options.js */ function _(require, module, exports) {
    var isValue = require(9) /* ./is-value */;
    var forEach = Array.prototype.forEach, create = Object.create;
    var process = function (src, obj) {
        var key;
        for (key in src)
            obj[key] = src[key];
    };
    // eslint-disable-next-line no-unused-vars
    module.exports = function (opts1 /*, …options*/) {
        var result = create(null);
        forEach.call(arguments, function (options) {
            if (!isValue(options))
                return;
            process(Object(options), result);
        });
        return result;
    };
},
/* es5-ext/string/#/contains/index.js */ function _(require, module, exports) {
    module.exports = require(43) /* ./is-implemented */() ? String.prototype.contains : require(44) /* ./shim */;
},
/* es5-ext/string/#/contains/is-implemented.js */ function _(require, module, exports) {
    var str = "razdwatrzy";
    module.exports = function () {
        if (typeof str.contains !== "function")
            return false;
        return str.contains("dwa") === true && str.contains("foo") === false;
    };
},
/* es5-ext/string/#/contains/shim.js */ function _(require, module, exports) {
    var indexOf = String.prototype.indexOf;
    module.exports = function (searchString /*, position*/) {
        return indexOf.call(this, searchString, arguments[1]) > -1;
    };
},
/* es6-symbol/validate-symbol.js */ function _(require, module, exports) {
    var isSymbol = require(46) /* ./is-symbol */;
    module.exports = function (value) {
        if (!isSymbol(value))
            throw new TypeError(value + " is not a symbol");
        return value;
    };
},
/* es6-symbol/is-symbol.js */ function _(require, module, exports) {
    module.exports = function (value) {
        if (!value)
            return false;
        if (typeof value === "symbol")
            return true;
        if (!value.constructor)
            return false;
        if (value.constructor.name !== "Symbol")
            return false;
        return value[value.constructor.toStringTag] === "Symbol";
    };
},
/* es6-symbol/lib/private/generate-name.js */ function _(require, module, exports) {
    var d = require(34) /* d */;
    var create = Object.create, defineProperty = Object.defineProperty, objPrototype = Object.prototype;
    var created = create(null);
    module.exports = function (desc) {
        var postfix = 0, name, ie11BugWorkaround;
        while (created[desc + (postfix || "")])
            ++postfix;
        desc += postfix || "";
        created[desc] = true;
        name = "@@" + desc;
        defineProperty(objPrototype, name, d.gs(null, function (value) {
            // For IE11 issue see:
            // https://connect.microsoft.com/IE/feedbackdetail/view/1928508/
            //    ie11-broken-getters-on-dom-objects
            // https://github.com/medikoo/es6-symbol/issues/12
            if (ie11BugWorkaround)
                return;
            ie11BugWorkaround = true;
            defineProperty(this, name, d(value));
            ie11BugWorkaround = false;
        }));
        return name;
    };
},
/* es6-symbol/lib/private/setup/standard-symbols.js */ function _(require, module, exports) {
    var d = require(34) /* d */, NativeSymbol = require(30) /* ext/global-this */.Symbol;
    module.exports = function (SymbolPolyfill) {
        return Object.defineProperties(SymbolPolyfill, {
            // To ensure proper interoperability with other native functions (e.g. Array.from)
            // fallback to eventual native implementation of given symbol
            hasInstance: d("", (NativeSymbol && NativeSymbol.hasInstance) || SymbolPolyfill("hasInstance")),
            isConcatSpreadable: d("", (NativeSymbol && NativeSymbol.isConcatSpreadable) ||
                SymbolPolyfill("isConcatSpreadable")),
            iterator: d("", (NativeSymbol && NativeSymbol.iterator) || SymbolPolyfill("iterator")),
            match: d("", (NativeSymbol && NativeSymbol.match) || SymbolPolyfill("match")),
            replace: d("", (NativeSymbol && NativeSymbol.replace) || SymbolPolyfill("replace")),
            search: d("", (NativeSymbol && NativeSymbol.search) || SymbolPolyfill("search")),
            species: d("", (NativeSymbol && NativeSymbol.species) || SymbolPolyfill("species")),
            split: d("", (NativeSymbol && NativeSymbol.split) || SymbolPolyfill("split")),
            toPrimitive: d("", (NativeSymbol && NativeSymbol.toPrimitive) || SymbolPolyfill("toPrimitive")),
            toStringTag: d("", (NativeSymbol && NativeSymbol.toStringTag) || SymbolPolyfill("toStringTag")),
            unscopables: d("", (NativeSymbol && NativeSymbol.unscopables) || SymbolPolyfill("unscopables"))
        });
    };
},
/* es6-symbol/lib/private/setup/symbol-registry.js */ function _(require, module, exports) {
    var d = require(34) /* d */, validateSymbol = require(45) /* ../../../validate-symbol */;
    var registry = Object.create(null);
    module.exports = function (SymbolPolyfill) {
        return Object.defineProperties(SymbolPolyfill, {
            for: d(function (key) {
                if (registry[key])
                    return registry[key];
                return (registry[key] = SymbolPolyfill(String(key)));
            }),
            keyFor: d(function (symbol) {
                var key;
                validateSymbol(symbol);
                for (key in registry) {
                    if (registry[key] === symbol)
                        return key;
                }
                return undefined;
            })
        });
    };
},
/* es5-ext/function/is-arguments.js */ function _(require, module, exports) {
    var objToString = Object.prototype.toString, id = objToString.call((function () { return arguments; })());
    module.exports = function (value) { return objToString.call(value) === id; };
},
/* es5-ext/function/is-function.js */ function _(require, module, exports) {
    var objToString = Object.prototype.toString, isFunctionStringTag = RegExp.prototype.test.bind(/^[object [A-Za-z0-9]*Function]$/);
    module.exports = function (value) {
        return typeof value === "function" && isFunctionStringTag(objToString.call(value));
    };
},
/* es5-ext/number/to-pos-integer.js */ function _(require, module, exports) {
    var toInteger = require(21) /* ./to-integer */, max = Math.max;
    module.exports = function (value) { return max(0, toInteger(value)); };
},
/* es5-ext/object/valid-callable.js */ function _(require, module, exports) {
    module.exports = function (fn) {
        if (typeof fn !== "function")
            throw new TypeError(fn + " is not a function");
        return fn;
    };
},
/* es5-ext/string/is-string.js */ function _(require, module, exports) {
    var objToString = Object.prototype.toString, id = objToString.call("");
    module.exports = function (value) {
        return (typeof value === "string" ||
            (value &&
                typeof value === "object" &&
                (value instanceof String || objToString.call(value) === id)) ||
            false);
    };
},
/* es5-ext/array/#/fill/implement.js */ function _(require, module, exports) {
    if (!require(56) /* ./is-implemented */()) {
        Object.defineProperty(Array.prototype, "fill", {
            value: require(57) /* ./shim */,
            configurable: true,
            enumerable: false,
            writable: true
        });
    }
},
/* es5-ext/array/#/fill/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        var arr = [1, 2, 3, 4, 5, 6];
        if (typeof arr.fill !== "function")
            return false;
        return String(arr.fill(-1, -3)) === "1,2,3,-1,-1,-1";
    };
},
/* es5-ext/array/#/fill/shim.js */ function _(require, module, exports) {
    var toInteger = require(21) /* ../../../number/to-integer */, toPosInt = require(52) /* ../../../number/to-pos-integer */, validValue = require(11) /* ../../../object/valid-value */, max = Math.max, min = Math.min;
    module.exports = function (value /*, start, end*/) {
        var arr = validValue(this), start = arguments[1], end = arguments[2], length = toPosInt(arr.length), relativeStart, i;
        start = start === undefined ? 0 : toInteger(start);
        end = end === undefined ? length : toInteger(end);
        relativeStart = start < 0 ? max(length + start, 0) : min(start, length);
        for (i = relativeStart; i < length && i < end; ++i)
            arr[i] = value;
        return arr;
    };
},
/* es5-ext/math/log10/implement.js */ function _(require, module, exports) {
    if (!require(59) /* ./is-implemented */()) {
        Object.defineProperty(Math, "log10", {
            value: require(60) /* ./shim */,
            configurable: true,
            enumerable: false,
            writable: true
        });
    }
},
/* es5-ext/math/log10/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        var log10 = Math.log10;
        if (typeof log10 !== "function")
            return false;
        return log10(2) === 0.3010299956639812;
    };
},
/* es5-ext/math/log10/shim.js */ function _(require, module, exports) {
    var log = Math.log, LOG10E = Math.LOG10E;
    module.exports = function (value) {
        if (isNaN(value))
            return NaN;
        value = Number(value);
        if (value < 0)
            return NaN;
        if (value === 0)
            return -Infinity;
        if (value === 1)
            return 0;
        if (value === Infinity)
            return Infinity;
        return log(value) * LOG10E;
    };
},
/* es5-ext/math/log1p/implement.js */ function _(require, module, exports) {
    if (!require(62) /* ./is-implemented */()) {
        Object.defineProperty(Math, "log1p", {
            value: require(63) /* ./shim */,
            configurable: true,
            enumerable: false,
            writable: true
        });
    }
},
/* es5-ext/math/log1p/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        var log1p = Math.log1p;
        if (typeof log1p !== "function")
            return false;
        return log1p(1) === 0.6931471805599453;
    };
},
/* es5-ext/math/log1p/shim.js */ function _(require, module, exports) {
    var log = Math.log;
    module.exports = function (value) {
        if (isNaN(value))
            return NaN;
        value = Number(value);
        if (value < -1)
            return NaN;
        if (value === -1)
            return -Infinity;
        if (value === 0)
            return value;
        if (value === Infinity)
            return Infinity;
        if (value > -1.0e-8 && value < 1.0e-8)
            return value - (value * value) / 2;
        return log(1 + value);
    };
},
/* es6-set/implement.js */ function _(require, module, exports) {
    if (!require(65) /* ./is-implemented */()) {
        Object.defineProperty(require(66) /* es5-ext/global */, 'Set', { value: require(67) /* ./polyfill */, configurable: true, enumerable: false,
            writable: true });
    }
},
/* es6-set/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        var set, iterator, result;
        if (typeof Set !== 'function')
            return false;
        set = new Set(['raz', 'dwa', 'trzy']);
        if (String(set) !== '[object Set]')
            return false;
        if (set.size !== 3)
            return false;
        if (typeof set.add !== 'function')
            return false;
        if (typeof set.clear !== 'function')
            return false;
        if (typeof set.delete !== 'function')
            return false;
        if (typeof set.entries !== 'function')
            return false;
        if (typeof set.forEach !== 'function')
            return false;
        if (typeof set.has !== 'function')
            return false;
        if (typeof set.keys !== 'function')
            return false;
        if (typeof set.values !== 'function')
            return false;
        iterator = set.values();
        result = iterator.next();
        if (result.done !== false)
            return false;
        if (result.value !== 'raz')
            return false;
        return true;
    };
},
/* es5-ext/global.js */ function _(require, module, exports) {
    var naiveFallback = function () {
        if (typeof self === "object" && self)
            return self;
        if (typeof window === "object" && window)
            return window;
        throw new Error("Unable to resolve global `this`");
    };
    module.exports = (function () {
        if (this)
            return this;
        // Unexpected strict mode (may happen if e.g. bundled into ESM module)
        // Fallback to standard globalThis if available
        if (typeof globalThis === "object" && globalThis)
            return globalThis;
        // Thanks @mathiasbynens -> https://mathiasbynens.be/notes/globalthis
        // In all ES5+ engines global object inherits from Object.prototype
        // (if you approached one that doesn't please report)
        try {
            Object.defineProperty(Object.prototype, "__global__", {
                get: function () { return this; },
                configurable: true
            });
        }
        catch (error) {
            // Unfortunate case of updates to Object.prototype being restricted
            // via preventExtensions, seal or freeze
            return naiveFallback();
        }
        try {
            // Safari case (window.__global__ works, but __global__ does not)
            if (!__global__)
                return naiveFallback();
            return __global__;
        }
        finally {
            delete Object.prototype.__global__;
        }
    })();
},
/* es6-set/polyfill.js */ function _(require, module, exports) {
    var clear = require(68) /* es5-ext/array/#/clear */, eIndexOf = require(69) /* es5-ext/array/#/e-index-of */, setPrototypeOf = require(73) /* es5-ext/object/set-prototype-of */, callable = require(53) /* es5-ext/object/valid-callable */, d = require(34) /* d */, ee = require(78) /* event-emitter */, Symbol = require(28) /* es6-symbol */, iterator = require(79) /* es6-iterator/valid-iterable */, forOf = require(81) /* es6-iterator/for-of */, Iterator = require(98) /* ./lib/iterator */, isNative = require(99) /* ./is-native-implemented */, call = Function.prototype.call, defineProperty = Object.defineProperty, getPrototypeOf = Object.getPrototypeOf, SetPoly, getValues, NativeSet;
    if (isNative)
        NativeSet = Set;
    module.exports = SetPoly = function Set( /*iterable*/) {
        var iterable = arguments[0], self;
        if (!(this instanceof SetPoly))
            throw new TypeError('Constructor requires \'new\'');
        if (isNative && setPrototypeOf)
            self = setPrototypeOf(new NativeSet(), getPrototypeOf(this));
        else
            self = this;
        if (iterable != null)
            iterator(iterable);
        defineProperty(self, '__setData__', d('c', []));
        if (!iterable)
            return self;
        forOf(iterable, function (value) {
            if (eIndexOf.call(this, value) !== -1)
                return;
            this.push(value);
        }, self.__setData__);
        return self;
    };
    if (isNative) {
        if (setPrototypeOf)
            setPrototypeOf(SetPoly, NativeSet);
        SetPoly.prototype = Object.create(NativeSet.prototype, { constructor: d(SetPoly) });
    }
    ee(Object.defineProperties(SetPoly.prototype, {
        add: d(function (value) {
            if (this.has(value))
                return this;
            this.emit('_add', this.__setData__.push(value) - 1, value);
            return this;
        }),
        clear: d(function () {
            if (!this.__setData__.length)
                return;
            clear.call(this.__setData__);
            this.emit('_clear');
        }),
        delete: d(function (value) {
            var index = eIndexOf.call(this.__setData__, value);
            if (index === -1)
                return false;
            this.__setData__.splice(index, 1);
            this.emit('_delete', index, value);
            return true;
        }),
        entries: d(function () { return new Iterator(this, 'key+value'); }),
        forEach: d(function (cb /*, thisArg*/) {
            var thisArg = arguments[1], iterator, result, value;
            callable(cb);
            iterator = this.values();
            result = iterator._next();
            while (result !== undefined) {
                value = iterator._resolve(result);
                call.call(cb, thisArg, value, value, this);
                result = iterator._next();
            }
        }),
        has: d(function (value) {
            return (eIndexOf.call(this.__setData__, value) !== -1);
        }),
        keys: d(getValues = function () { return this.values(); }),
        size: d.gs(function () { return this.__setData__.length; }),
        values: d(function () { return new Iterator(this); }),
        toString: d(function () { return '[object Set]'; })
    }));
    defineProperty(SetPoly.prototype, Symbol.iterator, d(getValues));
    defineProperty(SetPoly.prototype, Symbol.toStringTag, d('c', 'Set'));
},
/* es5-ext/array/#/clear.js */ function _(require, module, exports) {
    var value = require(11) /* ../../object/valid-value */;
    module.exports = function () {
        value(this).length = 0;
        return this;
    };
},
/* es5-ext/array/#/e-index-of.js */ function _(require, module, exports) {
    var numberIsNaN = require(70) /* ../../number/is-nan */, toPosInt = require(52) /* ../../number/to-pos-integer */, value = require(11) /* ../../object/valid-value */, indexOf = Array.prototype.indexOf, objHasOwnProperty = Object.prototype.hasOwnProperty, abs = Math.abs, floor = Math.floor;
    module.exports = function (searchElement /*, fromIndex*/) {
        var i, length, fromIndex, val;
        if (!numberIsNaN(searchElement))
            return indexOf.apply(this, arguments);
        length = toPosInt(value(this).length);
        fromIndex = arguments[1];
        if (isNaN(fromIndex))
            fromIndex = 0;
        else if (fromIndex >= 0)
            fromIndex = floor(fromIndex);
        else
            fromIndex = toPosInt(this.length) - floor(abs(fromIndex));
        for (i = fromIndex; i < length; ++i) {
            if (objHasOwnProperty.call(this, i)) {
                val = this[i];
                if (numberIsNaN(val))
                    return i; // Jslint: ignore
            }
        }
        return -1;
    };
},
/* es5-ext/number/is-nan/index.js */ function _(require, module, exports) {
    module.exports = require(71) /* ./is-implemented */() ? Number.isNaN : require(72) /* ./shim */;
},
/* es5-ext/number/is-nan/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        var numberIsNaN = Number.isNaN;
        if (typeof numberIsNaN !== "function")
            return false;
        return !numberIsNaN({}) && numberIsNaN(NaN) && !numberIsNaN(34);
    };
},
/* es5-ext/number/is-nan/shim.js */ function _(require, module, exports) {
    module.exports = function (value) {
        // eslint-disable-next-line no-self-compare
        return value !== value;
    };
},
/* es5-ext/object/set-prototype-of/index.js */ function _(require, module, exports) {
    module.exports = require(74) /* ./is-implemented */() ? Object.setPrototypeOf : require(75) /* ./shim */;
},
/* es5-ext/object/set-prototype-of/is-implemented.js */ function _(require, module, exports) {
    var create = Object.create, getPrototypeOf = Object.getPrototypeOf, plainObject = {};
    module.exports = function ( /* CustomCreate*/) {
        var setPrototypeOf = Object.setPrototypeOf, customCreate = arguments[0] || create;
        if (typeof setPrototypeOf !== "function")
            return false;
        return getPrototypeOf(setPrototypeOf(customCreate(null), plainObject)) === plainObject;
    };
},
/* es5-ext/object/set-prototype-of/shim.js */ function _(require, module, exports) {
    var isObject = require(76) /* ../is-object */, value = require(11) /* ../valid-value */, objIsPrototypeOf = Object.prototype.isPrototypeOf, defineProperty = Object.defineProperty, nullDesc = { configurable: true, enumerable: false, writable: true, value: undefined }, validate;
    validate = function (obj, prototype) {
        value(obj);
        if (prototype === null || isObject(prototype))
            return obj;
        throw new TypeError("Prototype must be null or an object");
    };
    module.exports = (function (status) {
        var fn, set;
        if (!status)
            return null;
        if (status.level === 2) {
            if (status.set) {
                set = status.set;
                fn = function (obj, prototype) {
                    set.call(validate(obj, prototype), prototype);
                    return obj;
                };
            }
            else {
                fn = function (obj, prototype) {
                    validate(obj, prototype).__proto__ = prototype;
                    return obj;
                };
            }
        }
        else {
            fn = function self(obj, prototype) {
                var isNullBase;
                validate(obj, prototype);
                isNullBase = objIsPrototypeOf.call(self.nullPolyfill, obj);
                if (isNullBase)
                    delete self.nullPolyfill.__proto__;
                if (prototype === null)
                    prototype = self.nullPolyfill;
                obj.__proto__ = prototype;
                if (isNullBase)
                    defineProperty(self.nullPolyfill, "__proto__", nullDesc);
                return obj;
            };
        }
        return Object.defineProperty(fn, "level", {
            configurable: false,
            enumerable: false,
            writable: false,
            value: status.level
        });
    })((function () {
        var tmpObj1 = Object.create(null), tmpObj2 = {}, set, desc = Object.getOwnPropertyDescriptor(Object.prototype, "__proto__");
        if (desc) {
            try {
                set = desc.set; // Opera crashes at this point
                set.call(tmpObj1, tmpObj2);
            }
            catch (ignore) { }
            if (Object.getPrototypeOf(tmpObj1) === tmpObj2)
                return { set: set, level: 2 };
        }
        tmpObj1.__proto__ = tmpObj2;
        if (Object.getPrototypeOf(tmpObj1) === tmpObj2)
            return { level: 2 };
        tmpObj1 = {};
        tmpObj1.__proto__ = tmpObj2;
        if (Object.getPrototypeOf(tmpObj1) === tmpObj2)
            return { level: 1 };
        return false;
    })());
    require(77) /* ../create */;
},
/* es5-ext/object/is-object.js */ function _(require, module, exports) {
    var isValue = require(9) /* ./is-value */;
    var map = { function: true, object: true };
    module.exports = function (value) { return (isValue(value) && map[typeof value]) || false; };
},
/* es5-ext/object/create.js */ function _(require, module, exports) {
    var create = Object.create, shim;
    if (!require(74) /* ./set-prototype-of/is-implemented */()) {
        shim = require(75) /* ./set-prototype-of/shim */;
    }
    module.exports = (function () {
        var nullObject, polyProps, desc;
        if (!shim)
            return create;
        if (shim.level !== 1)
            return create;
        nullObject = {};
        polyProps = {};
        desc = { configurable: false, enumerable: false, writable: true, value: undefined };
        Object.getOwnPropertyNames(Object.prototype).forEach(function (name) {
            if (name === "__proto__") {
                polyProps[name] = {
                    configurable: true,
                    enumerable: false,
                    writable: true,
                    value: undefined
                };
                return;
            }
            polyProps[name] = desc;
        });
        Object.defineProperties(nullObject, polyProps);
        Object.defineProperty(shim, "nullPolyfill", {
            configurable: false,
            enumerable: false,
            writable: false,
            value: nullObject
        });
        return function (prototype, props) {
            return create(prototype === null ? nullObject : prototype, props);
        };
    })();
},
/* event-emitter/index.js */ function _(require, module, exports) {
    var d = require(34) /* d */, callable = require(53) /* es5-ext/object/valid-callable */, apply = Function.prototype.apply, call = Function.prototype.call, create = Object.create, defineProperty = Object.defineProperty, defineProperties = Object.defineProperties, hasOwnProperty = Object.prototype.hasOwnProperty, descriptor = { configurable: true, enumerable: false, writable: true }, on, once, off, emit, methods, descriptors, base;
    on = function (type, listener) {
        var data;
        callable(listener);
        if (!hasOwnProperty.call(this, '__ee__')) {
            data = descriptor.value = create(null);
            defineProperty(this, '__ee__', descriptor);
            descriptor.value = null;
        }
        else {
            data = this.__ee__;
        }
        if (!data[type])
            data[type] = listener;
        else if (typeof data[type] === 'object')
            data[type].push(listener);
        else
            data[type] = [data[type], listener];
        return this;
    };
    once = function (type, listener) {
        var once, self;
        callable(listener);
        self = this;
        on.call(this, type, once = function () {
            off.call(self, type, once);
            apply.call(listener, this, arguments);
        });
        once.__eeOnceListener__ = listener;
        return this;
    };
    off = function (type, listener) {
        var data, listeners, candidate, i;
        callable(listener);
        if (!hasOwnProperty.call(this, '__ee__'))
            return this;
        data = this.__ee__;
        if (!data[type])
            return this;
        listeners = data[type];
        if (typeof listeners === 'object') {
            for (i = 0; (candidate = listeners[i]); ++i) {
                if ((candidate === listener) ||
                    (candidate.__eeOnceListener__ === listener)) {
                    if (listeners.length === 2)
                        data[type] = listeners[i ? 0 : 1];
                    else
                        listeners.splice(i, 1);
                }
            }
        }
        else {
            if ((listeners === listener) ||
                (listeners.__eeOnceListener__ === listener)) {
                delete data[type];
            }
        }
        return this;
    };
    emit = function (type) {
        var i, l, listener, listeners, args;
        if (!hasOwnProperty.call(this, '__ee__'))
            return;
        listeners = this.__ee__[type];
        if (!listeners)
            return;
        if (typeof listeners === 'object') {
            l = arguments.length;
            args = new Array(l - 1);
            for (i = 1; i < l; ++i)
                args[i - 1] = arguments[i];
            listeners = listeners.slice();
            for (i = 0; (listener = listeners[i]); ++i) {
                apply.call(listener, this, args);
            }
        }
        else {
            switch (arguments.length) {
                case 1:
                    call.call(listeners, this);
                    break;
                case 2:
                    call.call(listeners, this, arguments[1]);
                    break;
                case 3:
                    call.call(listeners, this, arguments[1], arguments[2]);
                    break;
                default:
                    l = arguments.length;
                    args = new Array(l - 1);
                    for (i = 1; i < l; ++i) {
                        args[i - 1] = arguments[i];
                    }
                    apply.call(listeners, this, args);
            }
        }
    };
    methods = {
        on: on,
        once: once,
        off: off,
        emit: emit
    };
    descriptors = {
        on: d(on),
        once: d(once),
        off: d(off),
        emit: d(emit)
    };
    base = defineProperties({}, descriptors);
    module.exports = exports = function (o) {
        return (o == null) ? create(base) : defineProperties(Object(o), descriptors);
    };
    exports.methods = methods;
},
/* es6-iterator/valid-iterable.js */ function _(require, module, exports) {
    var isIterable = require(80) /* ./is-iterable */;
    module.exports = function (value) {
        if (!isIterable(value))
            throw new TypeError(value + " is not iterable");
        return value;
    };
},
/* es6-iterator/is-iterable.js */ function _(require, module, exports) {
    var isArguments = require(50) /* es5-ext/function/is-arguments */, isValue = require(9) /* es5-ext/object/is-value */, isString = require(54) /* es5-ext/string/is-string */;
    var iteratorSymbol = require(28) /* es6-symbol */.iterator, isArray = Array.isArray;
    module.exports = function (value) {
        if (!isValue(value))
            return false;
        if (isArray(value))
            return true;
        if (isString(value))
            return true;
        if (isArguments(value))
            return true;
        return typeof value[iteratorSymbol] === "function";
    };
},
/* es6-iterator/for-of.js */ function _(require, module, exports) {
    var isArguments = require(50) /* es5-ext/function/is-arguments */, callable = require(53) /* es5-ext/object/valid-callable */, isString = require(54) /* es5-ext/string/is-string */, get = require(82) /* ./get */;
    var isArray = Array.isArray, call = Function.prototype.call, some = Array.prototype.some;
    module.exports = function (iterable, cb /*, thisArg*/) {
        var mode, thisArg = arguments[2], result, doBreak, broken, i, length, char, code;
        if (isArray(iterable) || isArguments(iterable))
            mode = "array";
        else if (isString(iterable))
            mode = "string";
        else
            iterable = get(iterable);
        callable(cb);
        doBreak = function () {
            broken = true;
        };
        if (mode === "array") {
            some.call(iterable, function (value) {
                call.call(cb, thisArg, value, doBreak);
                return broken;
            });
            return;
        }
        if (mode === "string") {
            length = iterable.length;
            for (i = 0; i < length; ++i) {
                char = iterable[i];
                if (i + 1 < length) {
                    code = char.charCodeAt(0);
                    if (code >= 0xd800 && code <= 0xdbff)
                        char += iterable[++i];
                }
                call.call(cb, thisArg, char, doBreak);
                if (broken)
                    break;
            }
            return;
        }
        result = iterable.next();
        while (!result.done) {
            call.call(cb, thisArg, result.value, doBreak);
            if (broken)
                return;
            result = iterable.next();
        }
    };
},
/* es6-iterator/get.js */ function _(require, module, exports) {
    var isArguments = require(50) /* es5-ext/function/is-arguments */, isString = require(54) /* es5-ext/string/is-string */, ArrayIterator = require(83) /* ./array */, StringIterator = require(97) /* ./string */, iterable = require(79) /* ./valid-iterable */, iteratorSymbol = require(28) /* es6-symbol */.iterator;
    module.exports = function (obj) {
        if (typeof iterable(obj)[iteratorSymbol] === "function")
            return obj[iteratorSymbol]();
        if (isArguments(obj))
            return new ArrayIterator(obj);
        if (isString(obj))
            return new StringIterator(obj);
        return new ArrayIterator(obj);
    };
},
/* es6-iterator/array.js */ function _(require, module, exports) {
    var setPrototypeOf = require(73) /* es5-ext/object/set-prototype-of */, contains = require(42) /* es5-ext/string/#/contains */, d = require(34) /* d */, Symbol = require(28) /* es6-symbol */, Iterator = require(84) /* ./ */;
    var defineProperty = Object.defineProperty, ArrayIterator;
    ArrayIterator = module.exports = function (arr, kind) {
        if (!(this instanceof ArrayIterator))
            throw new TypeError("Constructor requires 'new'");
        Iterator.call(this, arr);
        if (!kind)
            kind = "value";
        else if (contains.call(kind, "key+value"))
            kind = "key+value";
        else if (contains.call(kind, "key"))
            kind = "key";
        else
            kind = "value";
        defineProperty(this, "__kind__", d("", kind));
    };
    if (setPrototypeOf)
        setPrototypeOf(ArrayIterator, Iterator);
    // Internal %ArrayIteratorPrototype% doesn't expose its constructor
    delete ArrayIterator.prototype.constructor;
    ArrayIterator.prototype = Object.create(Iterator.prototype, {
        _resolve: d(function (i) {
            if (this.__kind__ === "value")
                return this.__list__[i];
            if (this.__kind__ === "key+value")
                return [i, this.__list__[i]];
            return i;
        })
    });
    defineProperty(ArrayIterator.prototype, Symbol.toStringTag, d("c", "Array Iterator"));
},
/* es6-iterator/index.js */ function _(require, module, exports) {
    var clear = require(68) /* es5-ext/array/#/clear */, assign = require(40) /* es5-ext/object/assign */, callable = require(53) /* es5-ext/object/valid-callable */, value = require(11) /* es5-ext/object/valid-value */, d = require(34) /* d */, autoBind = require(85) /* d/auto-bind */, Symbol = require(28) /* es6-symbol */;
    var defineProperty = Object.defineProperty, defineProperties = Object.defineProperties, Iterator;
    module.exports = Iterator = function (list, context) {
        if (!(this instanceof Iterator))
            throw new TypeError("Constructor requires 'new'");
        defineProperties(this, {
            __list__: d("w", value(list)),
            __context__: d("w", context),
            __nextIndex__: d("w", 0)
        });
        if (!context)
            return;
        callable(context.on);
        context.on("_add", this._onAdd);
        context.on("_delete", this._onDelete);
        context.on("_clear", this._onClear);
    };
    // Internal %IteratorPrototype% doesn't expose its constructor
    delete Iterator.prototype.constructor;
    defineProperties(Iterator.prototype, assign({
        _next: d(function () {
            var i;
            if (!this.__list__)
                return undefined;
            if (this.__redo__) {
                i = this.__redo__.shift();
                if (i !== undefined)
                    return i;
            }
            if (this.__nextIndex__ < this.__list__.length)
                return this.__nextIndex__++;
            this._unBind();
            return undefined;
        }),
        next: d(function () {
            return this._createResult(this._next());
        }),
        _createResult: d(function (i) {
            if (i === undefined)
                return { done: true, value: undefined };
            return { done: false, value: this._resolve(i) };
        }),
        _resolve: d(function (i) {
            return this.__list__[i];
        }),
        _unBind: d(function () {
            this.__list__ = null;
            delete this.__redo__;
            if (!this.__context__)
                return;
            this.__context__.off("_add", this._onAdd);
            this.__context__.off("_delete", this._onDelete);
            this.__context__.off("_clear", this._onClear);
            this.__context__ = null;
        }),
        toString: d(function () {
            return "[object " + (this[Symbol.toStringTag] || "Object") + "]";
        })
    }, autoBind({
        _onAdd: d(function (index) {
            if (index >= this.__nextIndex__)
                return;
            ++this.__nextIndex__;
            if (!this.__redo__) {
                defineProperty(this, "__redo__", d("c", [index]));
                return;
            }
            this.__redo__.forEach(function (redo, i) {
                if (redo >= index)
                    this.__redo__[i] = ++redo;
            }, this);
            this.__redo__.push(index);
        }),
        _onDelete: d(function (index) {
            var i;
            if (index >= this.__nextIndex__)
                return;
            --this.__nextIndex__;
            if (!this.__redo__)
                return;
            i = this.__redo__.indexOf(index);
            if (i !== -1)
                this.__redo__.splice(i, 1);
            this.__redo__.forEach(function (redo, j) {
                if (redo > index)
                    this.__redo__[j] = --redo;
            }, this);
        }),
        _onClear: d(function () {
            if (this.__redo__)
                clear.call(this.__redo__);
            this.__nextIndex__ = 0;
        })
    })));
    defineProperty(Iterator.prototype, Symbol.iterator, d(function () {
        return this;
    }));
},
/* d/auto-bind.js */ function _(require, module, exports) {
    var isValue = require(35) /* type/value/is */, ensureValue = require(86) /* type/value/ensure */, ensurePlainFunction = require(91) /* type/plain-function/ensure */, copy = require(92) /* es5-ext/object/copy */, normalizeOptions = require(41) /* es5-ext/object/normalize-options */, map = require(94) /* es5-ext/object/map */;
    var bind = Function.prototype.bind, defineProperty = Object.defineProperty, hasOwnProperty = Object.prototype.hasOwnProperty, define;
    define = function (name, desc, options) {
        var value = ensureValue(desc) && ensurePlainFunction(desc.value), dgs;
        dgs = copy(desc);
        delete dgs.writable;
        delete dgs.value;
        dgs.get = function () {
            if (!options.overwriteDefinition && hasOwnProperty.call(this, name))
                return value;
            desc.value = bind.call(value, options.resolveContext ? options.resolveContext(this) : this);
            defineProperty(this, name, desc);
            return this[name];
        };
        return dgs;
    };
    module.exports = function (props /*, options*/) {
        var options = normalizeOptions(arguments[1]);
        if (isValue(options.resolveContext))
            ensurePlainFunction(options.resolveContext);
        return map(props, function (desc, name) { return define(name, desc, options); });
    };
},
/* type/value/ensure.js */ function _(require, module, exports) {
    var resolveException = require(87) /* ../lib/resolve-exception */, is = require(35) /* ./is */;
    module.exports = function (value /*, options*/) {
        if (is(value))
            return value;
        return resolveException(value, "Cannot use %v", arguments[1]);
    };
},
/* type/lib/resolve-exception.js */ function _(require, module, exports) {
    var isValue = require(35) /* ../value/is */, isObject = require(39) /* ../object/is */, stringCoerce = require(88) /* ../string/coerce */, toShortString = require(89) /* ./to-short-string */;
    var resolveMessage = function (message, value) {
        return message.replace("%v", toShortString(value));
    };
    module.exports = function (value, defaultMessage, inputOptions) {
        if (!isObject(inputOptions))
            throw new TypeError(resolveMessage(defaultMessage, value));
        if (!isValue(value)) {
            if ("default" in inputOptions)
                return inputOptions["default"];
            if (inputOptions.isOptional)
                return null;
        }
        var errorMessage = stringCoerce(inputOptions.errorMessage);
        if (!isValue(errorMessage))
            errorMessage = defaultMessage;
        throw new TypeError(resolveMessage(errorMessage, value));
    };
},
/* type/string/coerce.js */ function _(require, module, exports) {
    var isValue = require(35) /* ../value/is */, isObject = require(39) /* ../object/is */;
    var objectToString = Object.prototype.toString;
    module.exports = function (value) {
        if (!isValue(value))
            return null;
        if (isObject(value)) {
            // Reject Object.prototype.toString coercion
            var valueToString = value.toString;
            if (typeof valueToString !== "function")
                return null;
            if (valueToString === objectToString)
                return null;
            // Note: It can be object coming from other realm, still as there's no ES3 and CSP compliant
            // way to resolve its realm's Object.prototype.toString it's left as not addressed edge case
        }
        try {
            return "" + value; // Ensure implicit coercion
        }
        catch (error) {
            return null;
        }
    };
},
/* type/lib/to-short-string.js */ function _(require, module, exports) {
    var safeToString = require(90) /* ./safe-to-string */;
    var reNewLine = /[\n\r\u2028\u2029]/g;
    module.exports = function (value) {
        var string = safeToString(value);
        if (string === null)
            return "<Non-coercible to string value>";
        // Trim if too long
        if (string.length > 100)
            string = string.slice(0, 99) + "…";
        // Replace eventual new lines
        string = string.replace(reNewLine, function (char) {
            switch (char) {
                case "\n":
                    return "\\n";
                case "\r":
                    return "\\r";
                case "\u2028":
                    return "\\u2028";
                case "\u2029":
                    return "\\u2029";
                /* istanbul ignore next */
                default:
                    throw new Error("Unexpected character");
            }
        });
        return string;
    };
},
/* type/lib/safe-to-string.js */ function _(require, module, exports) {
    module.exports = function (value) {
        try {
            return value.toString();
        }
        catch (error) {
            try {
                return String(value);
            }
            catch (error2) {
                return null;
            }
        }
    };
},
/* type/plain-function/ensure.js */ function _(require, module, exports) {
    var resolveException = require(87) /* ../lib/resolve-exception */, is = require(36) /* ./is */;
    module.exports = function (value /*, options*/) {
        if (is(value))
            return value;
        return resolveException(value, "%v is not a plain function", arguments[1]);
    };
},
/* es5-ext/object/copy.js */ function _(require, module, exports) {
    var aFrom = require(93) /* ../array/from */, assign = require(40) /* ./assign */, value = require(11) /* ./valid-value */;
    module.exports = function (obj /*, propertyNames, options*/) {
        var copy = Object(value(obj)), propertyNames = arguments[1], options = Object(arguments[2]);
        if (copy !== obj && !propertyNames)
            return copy;
        var result = {};
        if (propertyNames) {
            aFrom(propertyNames, function (propertyName) {
                if (options.ensure || propertyName in obj)
                    result[propertyName] = obj[propertyName];
            });
        }
        else {
            assign(result, obj);
        }
        return result;
    };
},
/* es5-ext/array/from/index.js */ function _(require, module, exports) {
    module.exports = require(26) /* ./is-implemented */() ? Array.from : require(27) /* ./shim */;
},
/* es5-ext/object/map.js */ function _(require, module, exports) {
    var callable = require(53) /* ./valid-callable */, forEach = require(95) /* ./for-each */, call = Function.prototype.call;
    module.exports = function (obj, cb /*, thisArg*/) {
        var result = {}, thisArg = arguments[2];
        callable(cb);
        forEach(obj, function (value, key, targetObj, index) {
            result[key] = call.call(cb, thisArg, value, key, targetObj, index);
        });
        return result;
    };
},
/* es5-ext/object/for-each.js */ function _(require, module, exports) {
    module.exports = require(96) /* ./_iterate */("forEach");
},
/* es5-ext/object/_iterate.js */ function _(require, module, exports) {
    var callable = require(53) /* ./valid-callable */, value = require(11) /* ./valid-value */, bind = Function.prototype.bind, call = Function.prototype.call, keys = Object.keys, objPropertyIsEnumerable = Object.prototype.propertyIsEnumerable;
    module.exports = function (method, defVal) {
        return function (obj, cb /*, thisArg, compareFn*/) {
            var list, thisArg = arguments[2], compareFn = arguments[3];
            obj = Object(value(obj));
            callable(cb);
            list = keys(obj);
            if (compareFn) {
                list.sort(typeof compareFn === "function" ? bind.call(compareFn, obj) : undefined);
            }
            if (typeof method !== "function")
                method = list[method];
            return call.call(method, list, function (key, index) {
                if (!objPropertyIsEnumerable.call(obj, key))
                    return defVal;
                return call.call(cb, thisArg, obj[key], key, obj, index);
            });
        };
    };
},
/* es6-iterator/string.js */ function _(require, module, exports) {
    var setPrototypeOf = require(73) /* es5-ext/object/set-prototype-of */, d = require(34) /* d */, Symbol = require(28) /* es6-symbol */, Iterator = require(84) /* ./ */;
    var defineProperty = Object.defineProperty, StringIterator;
    StringIterator = module.exports = function (str) {
        if (!(this instanceof StringIterator))
            throw new TypeError("Constructor requires 'new'");
        str = String(str);
        Iterator.call(this, str);
        defineProperty(this, "__length__", d("", str.length));
    };
    if (setPrototypeOf)
        setPrototypeOf(StringIterator, Iterator);
    // Internal %ArrayIteratorPrototype% doesn't expose its constructor
    delete StringIterator.prototype.constructor;
    StringIterator.prototype = Object.create(Iterator.prototype, {
        _next: d(function () {
            if (!this.__list__)
                return undefined;
            if (this.__nextIndex__ < this.__length__)
                return this.__nextIndex__++;
            this._unBind();
            return undefined;
        }),
        _resolve: d(function (i) {
            var char = this.__list__[i], code;
            if (this.__nextIndex__ === this.__length__)
                return char;
            code = char.charCodeAt(0);
            if (code >= 0xd800 && code <= 0xdbff)
                return char + this.__list__[this.__nextIndex__++];
            return char;
        })
    });
    defineProperty(StringIterator.prototype, Symbol.toStringTag, d("c", "String Iterator"));
},
/* es6-set/lib/iterator.js */ function _(require, module, exports) {
    var setPrototypeOf = require(73) /* es5-ext/object/set-prototype-of */, contains = require(42) /* es5-ext/string/#/contains */, d = require(34) /* d */, Iterator = require(84) /* es6-iterator */, toStringTagSymbol = require(28) /* es6-symbol */.toStringTag, defineProperty = Object.defineProperty, SetIterator;
    SetIterator = module.exports = function (set, kind) {
        if (!(this instanceof SetIterator))
            return new SetIterator(set, kind);
        Iterator.call(this, set.__setData__, set);
        if (!kind)
            kind = 'value';
        else if (contains.call(kind, 'key+value'))
            kind = 'key+value';
        else
            kind = 'value';
        defineProperty(this, '__kind__', d('', kind));
    };
    if (setPrototypeOf)
        setPrototypeOf(SetIterator, Iterator);
    SetIterator.prototype = Object.create(Iterator.prototype, {
        constructor: d(SetIterator),
        _resolve: d(function (i) {
            if (this.__kind__ === 'value')
                return this.__list__[i];
            return [this.__list__[i], this.__list__[i]];
        }),
        toString: d(function () { return '[object Set Iterator]'; })
    });
    defineProperty(SetIterator.prototype, toStringTagSymbol, d('c', 'Set Iterator'));
},
/* es6-set/is-native-implemented.js */ function _(require, module, exports) {
    module.exports = (function () {
        if (typeof Set === 'undefined')
            return false;
        return (Object.prototype.toString.call(Set.prototype) === '[object Set]');
    }());
},
/* es6-map/implement.js */ function _(require, module, exports) {
    if (!require(101) /* ./is-implemented */()) {
        Object.defineProperty(require(66) /* es5-ext/global */, 'Map', { value: require(102) /* ./polyfill */, configurable: true, enumerable: false,
            writable: true });
    }
},
/* es6-map/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        var map, iterator, result;
        if (typeof Map !== 'function')
            return false;
        try {
            // WebKit doesn't support arguments and crashes
            map = new Map([['raz', 'one'], ['dwa', 'two'], ['trzy', 'three']]);
        }
        catch (e) {
            return false;
        }
        if (String(map) !== '[object Map]')
            return false;
        if (map.size !== 3)
            return false;
        if (typeof map.clear !== 'function')
            return false;
        if (typeof map.delete !== 'function')
            return false;
        if (typeof map.entries !== 'function')
            return false;
        if (typeof map.forEach !== 'function')
            return false;
        if (typeof map.get !== 'function')
            return false;
        if (typeof map.has !== 'function')
            return false;
        if (typeof map.keys !== 'function')
            return false;
        if (typeof map.set !== 'function')
            return false;
        if (typeof map.values !== 'function')
            return false;
        iterator = map.entries();
        result = iterator.next();
        if (result.done !== false)
            return false;
        if (!result.value)
            return false;
        if (result.value[0] !== 'raz')
            return false;
        if (result.value[1] !== 'one')
            return false;
        return true;
    };
},
/* es6-map/polyfill.js */ function _(require, module, exports) {
    var clear = require(68) /* es5-ext/array/#/clear */, eIndexOf = require(69) /* es5-ext/array/#/e-index-of */, setPrototypeOf = require(73) /* es5-ext/object/set-prototype-of */, callable = require(53) /* es5-ext/object/valid-callable */, validValue = require(11) /* es5-ext/object/valid-value */, d = require(34) /* d */, ee = require(78) /* event-emitter */, Symbol = require(28) /* es6-symbol */, iterator = require(79) /* es6-iterator/valid-iterable */, forOf = require(81) /* es6-iterator/for-of */, Iterator = require(103) /* ./lib/iterator */, isNative = require(106) /* ./is-native-implemented */, call = Function.prototype.call, defineProperties = Object.defineProperties, getPrototypeOf = Object.getPrototypeOf, MapPoly;
    module.exports = MapPoly = function ( /*iterable*/) {
        var iterable = arguments[0], keys, values, self;
        if (!(this instanceof MapPoly))
            throw new TypeError('Constructor requires \'new\'');
        if (isNative && setPrototypeOf && (Map !== MapPoly)) {
            self = setPrototypeOf(new Map(), getPrototypeOf(this));
        }
        else {
            self = this;
        }
        if (iterable != null)
            iterator(iterable);
        defineProperties(self, {
            __mapKeysData__: d('c', keys = []),
            __mapValuesData__: d('c', values = [])
        });
        if (!iterable)
            return self;
        forOf(iterable, function (value) {
            var key = validValue(value)[0];
            value = value[1];
            if (eIndexOf.call(keys, key) !== -1)
                return;
            keys.push(key);
            values.push(value);
        }, self);
        return self;
    };
    if (isNative) {
        if (setPrototypeOf)
            setPrototypeOf(MapPoly, Map);
        MapPoly.prototype = Object.create(Map.prototype, {
            constructor: d(MapPoly)
        });
    }
    ee(defineProperties(MapPoly.prototype, {
        clear: d(function () {
            if (!this.__mapKeysData__.length)
                return;
            clear.call(this.__mapKeysData__);
            clear.call(this.__mapValuesData__);
            this.emit('_clear');
        }),
        delete: d(function (key) {
            var index = eIndexOf.call(this.__mapKeysData__, key);
            if (index === -1)
                return false;
            this.__mapKeysData__.splice(index, 1);
            this.__mapValuesData__.splice(index, 1);
            this.emit('_delete', index, key);
            return true;
        }),
        entries: d(function () { return new Iterator(this, 'key+value'); }),
        forEach: d(function (cb /*, thisArg*/) {
            var thisArg = arguments[1], iterator, result;
            callable(cb);
            iterator = this.entries();
            result = iterator._next();
            while (result !== undefined) {
                call.call(cb, thisArg, this.__mapValuesData__[result], this.__mapKeysData__[result], this);
                result = iterator._next();
            }
        }),
        get: d(function (key) {
            var index = eIndexOf.call(this.__mapKeysData__, key);
            if (index === -1)
                return;
            return this.__mapValuesData__[index];
        }),
        has: d(function (key) {
            return (eIndexOf.call(this.__mapKeysData__, key) !== -1);
        }),
        keys: d(function () { return new Iterator(this, 'key'); }),
        set: d(function (key, value) {
            var index = eIndexOf.call(this.__mapKeysData__, key), emit;
            if (index === -1) {
                index = this.__mapKeysData__.push(key) - 1;
                emit = true;
            }
            this.__mapValuesData__[index] = value;
            if (emit)
                this.emit('_add', index, key);
            return this;
        }),
        size: d.gs(function () { return this.__mapKeysData__.length; }),
        values: d(function () { return new Iterator(this, 'value'); }),
        toString: d(function () { return '[object Map]'; })
    }));
    Object.defineProperty(MapPoly.prototype, Symbol.iterator, d(function () {
        return this.entries();
    }));
    Object.defineProperty(MapPoly.prototype, Symbol.toStringTag, d('c', 'Map'));
},
/* es6-map/lib/iterator.js */ function _(require, module, exports) {
    var setPrototypeOf = require(73) /* es5-ext/object/set-prototype-of */, d = require(34) /* d */, Iterator = require(84) /* es6-iterator */, toStringTagSymbol = require(28) /* es6-symbol */.toStringTag, kinds = require(104) /* ./iterator-kinds */, defineProperties = Object.defineProperties, unBind = Iterator.prototype._unBind, MapIterator;
    MapIterator = module.exports = function (map, kind) {
        if (!(this instanceof MapIterator))
            return new MapIterator(map, kind);
        Iterator.call(this, map.__mapKeysData__, map);
        if (!kind || !kinds[kind])
            kind = 'key+value';
        defineProperties(this, {
            __kind__: d('', kind),
            __values__: d('w', map.__mapValuesData__)
        });
    };
    if (setPrototypeOf)
        setPrototypeOf(MapIterator, Iterator);
    MapIterator.prototype = Object.create(Iterator.prototype, {
        constructor: d(MapIterator),
        _resolve: d(function (i) {
            if (this.__kind__ === 'value')
                return this.__values__[i];
            if (this.__kind__ === 'key')
                return this.__list__[i];
            return [this.__list__[i], this.__values__[i]];
        }),
        _unBind: d(function () {
            this.__values__ = null;
            unBind.call(this);
        }),
        toString: d(function () { return '[object Map Iterator]'; })
    });
    Object.defineProperty(MapIterator.prototype, toStringTagSymbol, d('c', 'Map Iterator'));
},
/* es6-map/lib/iterator-kinds.js */ function _(require, module, exports) {
    module.exports = require(105) /* es5-ext/object/primitive-set */('key', 'value', 'key+value');
},
/* es5-ext/object/primitive-set.js */ function _(require, module, exports) {
    var forEach = Array.prototype.forEach, create = Object.create;
    // eslint-disable-next-line no-unused-vars
    module.exports = function (arg /*, …args*/) {
        var set = create(null);
        forEach.call(arguments, function (name) { set[name] = true; });
        return set;
    };
},
/* es6-map/is-native-implemented.js */ function _(require, module, exports) {
    module.exports = (function () {
        if (typeof Map === 'undefined')
            return false;
        return (Object.prototype.toString.call(new Map()) === '[object Map]');
    }());
},
/* es6-weak-map/implement.js */ function _(require, module, exports) {
    if (!require(108) /* ./is-implemented */()) {
        Object.defineProperty(require(66) /* es5-ext/global */, "WeakMap", {
            value: require(109) /* ./polyfill */,
            configurable: true,
            enumerable: false,
            writable: true
        });
    }
},
/* es6-weak-map/is-implemented.js */ function _(require, module, exports) {
    module.exports = function () {
        var weakMap, obj;
        if (typeof WeakMap !== "function")
            return false;
        try {
            // WebKit doesn't support arguments and crashes
            weakMap = new WeakMap([[obj = {}, "one"], [{}, "two"], [{}, "three"]]);
        }
        catch (e) {
            return false;
        }
        if (String(weakMap) !== "[object WeakMap]")
            return false;
        if (typeof weakMap.set !== "function")
            return false;
        if (weakMap.set({}, 1) !== weakMap)
            return false;
        if (typeof weakMap.delete !== "function")
            return false;
        if (typeof weakMap.has !== "function")
            return false;
        if (weakMap.get(obj) !== "one")
            return false;
        return true;
    };
},
/* es6-weak-map/polyfill.js */ function _(require, module, exports) {
    var isValue = require(9) /* es5-ext/object/is-value */, setPrototypeOf = require(73) /* es5-ext/object/set-prototype-of */, object = require(110) /* es5-ext/object/valid-object */, ensureValue = require(11) /* es5-ext/object/valid-value */, randomUniq = require(111) /* es5-ext/string/random-uniq */, d = require(34) /* d */, getIterator = require(82) /* es6-iterator/get */, forOf = require(81) /* es6-iterator/for-of */, toStringTagSymbol = require(28) /* es6-symbol */.toStringTag, isNative = require(112) /* ./is-native-implemented */, isArray = Array.isArray, defineProperty = Object.defineProperty, objHasOwnProperty = Object.prototype.hasOwnProperty, getPrototypeOf = Object.getPrototypeOf, WeakMapPoly;
    module.exports = WeakMapPoly = function ( /* Iterable*/) {
        var iterable = arguments[0], self;
        if (!(this instanceof WeakMapPoly))
            throw new TypeError("Constructor requires 'new'");
        self = isNative && setPrototypeOf && (WeakMap !== WeakMapPoly)
            ? setPrototypeOf(new WeakMap(), getPrototypeOf(this)) : this;
        if (isValue(iterable)) {
            if (!isArray(iterable))
                iterable = getIterator(iterable);
        }
        defineProperty(self, "__weakMapData__", d("c", "$weakMap$" + randomUniq()));
        if (!iterable)
            return self;
        forOf(iterable, function (val) {
            ensureValue(val);
            self.set(val[0], val[1]);
        });
        return self;
    };
    if (isNative) {
        if (setPrototypeOf)
            setPrototypeOf(WeakMapPoly, WeakMap);
        WeakMapPoly.prototype = Object.create(WeakMap.prototype, { constructor: d(WeakMapPoly) });
    }
    Object.defineProperties(WeakMapPoly.prototype, {
        delete: d(function (key) {
            if (objHasOwnProperty.call(object(key), this.__weakMapData__)) {
                delete key[this.__weakMapData__];
                return true;
            }
            return false;
        }),
        get: d(function (key) {
            if (!objHasOwnProperty.call(object(key), this.__weakMapData__))
                return undefined;
            return key[this.__weakMapData__];
        }),
        has: d(function (key) {
            return objHasOwnProperty.call(object(key), this.__weakMapData__);
        }),
        set: d(function (key, value) {
            defineProperty(object(key), this.__weakMapData__, d("c", value));
            return this;
        }),
        toString: d(function () {
            return "[object WeakMap]";
        })
    });
    defineProperty(WeakMapPoly.prototype, toStringTagSymbol, d("c", "WeakMap"));
},
/* es5-ext/object/valid-object.js */ function _(require, module, exports) {
    var isObject = require(76) /* ./is-object */;
    module.exports = function (value) {
        if (!isObject(value))
            throw new TypeError(value + " is not an Object");
        return value;
    };
},
/* es5-ext/string/random-uniq.js */ function _(require, module, exports) {
    var generated = Object.create(null), random = Math.random;
    module.exports = function () {
        var str;
        do {
            str = random().toString(36).slice(2);
        } while (generated[str]);
        return str;
    };
},
/* es6-weak-map/is-native-implemented.js */ function _(require, module, exports) {
    module.exports = (function () {
        if (typeof WeakMap !== "function")
            return false;
        return Object.prototype.toString.call(new WeakMap()) === "[object WeakMap]";
    }());
},
/* es6-symbol/implement.js */ function _(require, module, exports) {
    if (!require(29) /* ./is-implemented */()) {
        Object.defineProperty(require(30) /* ext/global-this */, "Symbol", {
            value: require(33) /* ./polyfill */,
            configurable: true,
            enumerable: false,
            writable: true
        });
    }
},
/* es6-promise/auto.js */ function _(require, module, exports) {
    module.exports = require(115) /* ./ */.polyfill();
},
/* es6-promise/dist/es6-promise.js */ function _(require, module, exports) {
    /*!
     * @overview es6-promise - a tiny implementation of Promises/A+.
     * @copyright Copyright (c) 2014 Yehuda Katz, Tom Dale, Stefan Penner and contributors (Conversion to ES6 API by Jake Archibald)
     * @license   Licensed under MIT license
     *            See https://raw.githubusercontent.com/stefanpenner/es6-promise/master/LICENSE
     * @version   v4.2.8+1e68dce6
     */
    (function (global, factory) {
        typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
            typeof define === 'function' && define.amd ? define(factory) :
                (global.ES6Promise = factory());
    }(this, (function () {
        'use strict';
        function objectOrFunction(x) {
            var type = typeof x;
            return x !== null && (type === 'object' || type === 'function');
        }
        function isFunction(x) {
            return typeof x === 'function';
        }
        var _isArray = void 0;
        if (Array.isArray) {
            _isArray = Array.isArray;
        }
        else {
            _isArray = function (x) {
                return Object.prototype.toString.call(x) === '[object Array]';
            };
        }
        var isArray = _isArray;
        var len = 0;
        var vertxNext = void 0;
        var customSchedulerFn = void 0;
        var asap = function asap(callback, arg) {
            queue[len] = callback;
            queue[len + 1] = arg;
            len += 2;
            if (len === 2) {
                // If len is 2, that means that we need to schedule an async flush.
                // If additional callbacks are queued before the queue is flushed, they
                // will be processed by this flush that we are scheduling.
                if (customSchedulerFn) {
                    customSchedulerFn(flush);
                }
                else {
                    scheduleFlush();
                }
            }
        };
        function setScheduler(scheduleFn) {
            customSchedulerFn = scheduleFn;
        }
        function setAsap(asapFn) {
            asap = asapFn;
        }
        var browserWindow = typeof window !== 'undefined' ? window : undefined;
        var browserGlobal = browserWindow || {};
        var BrowserMutationObserver = browserGlobal.MutationObserver || browserGlobal.WebKitMutationObserver;
        var isNode = typeof self === 'undefined' && typeof process !== 'undefined' && {}.toString.call(process) === '[object process]';
        // test for web worker but not in IE10
        var isWorker = typeof Uint8ClampedArray !== 'undefined' && typeof importScripts !== 'undefined' && typeof MessageChannel !== 'undefined';
        // node
        function useNextTick() {
            // node version 0.10.x displays a deprecation warning when nextTick is used recursively
            // see https://github.com/cujojs/when/issues/410 for details
            return function () {
                return process.nextTick(flush);
            };
        }
        // vertx
        function useVertxTimer() {
            if (typeof vertxNext !== 'undefined') {
                return function () {
                    vertxNext(flush);
                };
            }
            return useSetTimeout();
        }
        function useMutationObserver() {
            var iterations = 0;
            var observer = new BrowserMutationObserver(flush);
            var node = document.createTextNode('');
            observer.observe(node, { characterData: true });
            return function () {
                node.data = iterations = ++iterations % 2;
            };
        }
        // web worker
        function useMessageChannel() {
            var channel = new MessageChannel();
            channel.port1.onmessage = flush;
            return function () {
                return channel.port2.postMessage(0);
            };
        }
        function useSetTimeout() {
            // Store setTimeout reference so es6-promise will be unaffected by
            // other code modifying setTimeout (like sinon.useFakeTimers())
            var globalSetTimeout = setTimeout;
            return function () {
                return globalSetTimeout(flush, 1);
            };
        }
        var queue = new Array(1000);
        function flush() {
            for (var i = 0; i < len; i += 2) {
                var callback = queue[i];
                var arg = queue[i + 1];
                callback(arg);
                queue[i] = undefined;
                queue[i + 1] = undefined;
            }
            len = 0;
        }
        function attemptVertx() {
            try {
                var vertx = Function('return this')().require('vertx');
                vertxNext = vertx.runOnLoop || vertx.runOnContext;
                return useVertxTimer();
            }
            catch (e) {
                return useSetTimeout();
            }
        }
        var scheduleFlush = void 0;
        // Decide what async method to use to triggering processing of queued callbacks:
        if (isNode) {
            scheduleFlush = useNextTick();
        }
        else if (BrowserMutationObserver) {
            scheduleFlush = useMutationObserver();
        }
        else if (isWorker) {
            scheduleFlush = useMessageChannel();
        }
        else if (browserWindow === undefined && typeof require === 'function') {
            scheduleFlush = attemptVertx();
        }
        else {
            scheduleFlush = useSetTimeout();
        }
        function then(onFulfillment, onRejection) {
            var parent = this;
            var child = new this.constructor(noop);
            if (child[PROMISE_ID] === undefined) {
                makePromise(child);
            }
            var _state = parent._state;
            if (_state) {
                var callback = arguments[_state - 1];
                asap(function () {
                    return invokeCallback(_state, child, callback, parent._result);
                });
            }
            else {
                subscribe(parent, child, onFulfillment, onRejection);
            }
            return child;
        }
        /**
          `Promise.resolve` returns a promise that will become resolved with the
          passed `value`. It is shorthand for the following:
        
          ```javascript
          let promise = new Promise(function(resolve, reject){
            resolve(1);
          });
        
          promise.then(function(value){
            // value === 1
          });
          ```
        
          Instead of writing the above, your code now simply becomes the following:
        
          ```javascript
          let promise = Promise.resolve(1);
        
          promise.then(function(value){
            // value === 1
          });
          ```
        
          @method resolve
          @static
          @param {Any} value value that the returned promise will be resolved with
          Useful for tooling.
          @return {Promise} a promise that will become fulfilled with the given
          `value`
        */
        function resolve$1(object) {
            /*jshint validthis:true */
            var Constructor = this;
            if (object && typeof object === 'object' && object.constructor === Constructor) {
                return object;
            }
            var promise = new Constructor(noop);
            resolve(promise, object);
            return promise;
        }
        var PROMISE_ID = Math.random().toString(36).substring(2);
        function noop() { }
        var PENDING = void 0;
        var FULFILLED = 1;
        var REJECTED = 2;
        function selfFulfillment() {
            return new TypeError("You cannot resolve a promise with itself");
        }
        function cannotReturnOwn() {
            return new TypeError('A promises callback cannot return that same promise.');
        }
        function tryThen(then$$1, value, fulfillmentHandler, rejectionHandler) {
            try {
                then$$1.call(value, fulfillmentHandler, rejectionHandler);
            }
            catch (e) {
                return e;
            }
        }
        function handleForeignThenable(promise, thenable, then$$1) {
            asap(function (promise) {
                var sealed = false;
                var error = tryThen(then$$1, thenable, function (value) {
                    if (sealed) {
                        return;
                    }
                    sealed = true;
                    if (thenable !== value) {
                        resolve(promise, value);
                    }
                    else {
                        fulfill(promise, value);
                    }
                }, function (reason) {
                    if (sealed) {
                        return;
                    }
                    sealed = true;
                    reject(promise, reason);
                }, 'Settle: ' + (promise._label || ' unknown promise'));
                if (!sealed && error) {
                    sealed = true;
                    reject(promise, error);
                }
            }, promise);
        }
        function handleOwnThenable(promise, thenable) {
            if (thenable._state === FULFILLED) {
                fulfill(promise, thenable._result);
            }
            else if (thenable._state === REJECTED) {
                reject(promise, thenable._result);
            }
            else {
                subscribe(thenable, undefined, function (value) {
                    return resolve(promise, value);
                }, function (reason) {
                    return reject(promise, reason);
                });
            }
        }
        function handleMaybeThenable(promise, maybeThenable, then$$1) {
            if (maybeThenable.constructor === promise.constructor && then$$1 === then && maybeThenable.constructor.resolve === resolve$1) {
                handleOwnThenable(promise, maybeThenable);
            }
            else {
                if (then$$1 === undefined) {
                    fulfill(promise, maybeThenable);
                }
                else if (isFunction(then$$1)) {
                    handleForeignThenable(promise, maybeThenable, then$$1);
                }
                else {
                    fulfill(promise, maybeThenable);
                }
            }
        }
        function resolve(promise, value) {
            if (promise === value) {
                reject(promise, selfFulfillment());
            }
            else if (objectOrFunction(value)) {
                var then$$1 = void 0;
                try {
                    then$$1 = value.then;
                }
                catch (error) {
                    reject(promise, error);
                    return;
                }
                handleMaybeThenable(promise, value, then$$1);
            }
            else {
                fulfill(promise, value);
            }
        }
        function publishRejection(promise) {
            if (promise._onerror) {
                promise._onerror(promise._result);
            }
            publish(promise);
        }
        function fulfill(promise, value) {
            if (promise._state !== PENDING) {
                return;
            }
            promise._result = value;
            promise._state = FULFILLED;
            if (promise._subscribers.length !== 0) {
                asap(publish, promise);
            }
        }
        function reject(promise, reason) {
            if (promise._state !== PENDING) {
                return;
            }
            promise._state = REJECTED;
            promise._result = reason;
            asap(publishRejection, promise);
        }
        function subscribe(parent, child, onFulfillment, onRejection) {
            var _subscribers = parent._subscribers;
            var length = _subscribers.length;
            parent._onerror = null;
            _subscribers[length] = child;
            _subscribers[length + FULFILLED] = onFulfillment;
            _subscribers[length + REJECTED] = onRejection;
            if (length === 0 && parent._state) {
                asap(publish, parent);
            }
        }
        function publish(promise) {
            var subscribers = promise._subscribers;
            var settled = promise._state;
            if (subscribers.length === 0) {
                return;
            }
            var child = void 0, callback = void 0, detail = promise._result;
            for (var i = 0; i < subscribers.length; i += 3) {
                child = subscribers[i];
                callback = subscribers[i + settled];
                if (child) {
                    invokeCallback(settled, child, callback, detail);
                }
                else {
                    callback(detail);
                }
            }
            promise._subscribers.length = 0;
        }
        function invokeCallback(settled, promise, callback, detail) {
            var hasCallback = isFunction(callback), value = void 0, error = void 0, succeeded = true;
            if (hasCallback) {
                try {
                    value = callback(detail);
                }
                catch (e) {
                    succeeded = false;
                    error = e;
                }
                if (promise === value) {
                    reject(promise, cannotReturnOwn());
                    return;
                }
            }
            else {
                value = detail;
            }
            if (promise._state !== PENDING) {
                // noop
            }
            else if (hasCallback && succeeded) {
                resolve(promise, value);
            }
            else if (succeeded === false) {
                reject(promise, error);
            }
            else if (settled === FULFILLED) {
                fulfill(promise, value);
            }
            else if (settled === REJECTED) {
                reject(promise, value);
            }
        }
        function initializePromise(promise, resolver) {
            try {
                resolver(function resolvePromise(value) {
                    resolve(promise, value);
                }, function rejectPromise(reason) {
                    reject(promise, reason);
                });
            }
            catch (e) {
                reject(promise, e);
            }
        }
        var id = 0;
        function nextId() {
            return id++;
        }
        function makePromise(promise) {
            promise[PROMISE_ID] = id++;
            promise._state = undefined;
            promise._result = undefined;
            promise._subscribers = [];
        }
        function validationError() {
            return new Error('Array Methods must be provided an Array');
        }
        var Enumerator = function () {
            function Enumerator(Constructor, input) {
                this._instanceConstructor = Constructor;
                this.promise = new Constructor(noop);
                if (!this.promise[PROMISE_ID]) {
                    makePromise(this.promise);
                }
                if (isArray(input)) {
                    this.length = input.length;
                    this._remaining = input.length;
                    this._result = new Array(this.length);
                    if (this.length === 0) {
                        fulfill(this.promise, this._result);
                    }
                    else {
                        this.length = this.length || 0;
                        this._enumerate(input);
                        if (this._remaining === 0) {
                            fulfill(this.promise, this._result);
                        }
                    }
                }
                else {
                    reject(this.promise, validationError());
                }
            }
            Enumerator.prototype._enumerate = function _enumerate(input) {
                for (var i = 0; this._state === PENDING && i < input.length; i++) {
                    this._eachEntry(input[i], i);
                }
            };
            Enumerator.prototype._eachEntry = function _eachEntry(entry, i) {
                var c = this._instanceConstructor;
                var resolve$$1 = c.resolve;
                if (resolve$$1 === resolve$1) {
                    var _then = void 0;
                    var error = void 0;
                    var didError = false;
                    try {
                        _then = entry.then;
                    }
                    catch (e) {
                        didError = true;
                        error = e;
                    }
                    if (_then === then && entry._state !== PENDING) {
                        this._settledAt(entry._state, i, entry._result);
                    }
                    else if (typeof _then !== 'function') {
                        this._remaining--;
                        this._result[i] = entry;
                    }
                    else if (c === Promise$1) {
                        var promise = new c(noop);
                        if (didError) {
                            reject(promise, error);
                        }
                        else {
                            handleMaybeThenable(promise, entry, _then);
                        }
                        this._willSettleAt(promise, i);
                    }
                    else {
                        this._willSettleAt(new c(function (resolve$$1) {
                            return resolve$$1(entry);
                        }), i);
                    }
                }
                else {
                    this._willSettleAt(resolve$$1(entry), i);
                }
            };
            Enumerator.prototype._settledAt = function _settledAt(state, i, value) {
                var promise = this.promise;
                if (promise._state === PENDING) {
                    this._remaining--;
                    if (state === REJECTED) {
                        reject(promise, value);
                    }
                    else {
                        this._result[i] = value;
                    }
                }
                if (this._remaining === 0) {
                    fulfill(promise, this._result);
                }
            };
            Enumerator.prototype._willSettleAt = function _willSettleAt(promise, i) {
                var enumerator = this;
                subscribe(promise, undefined, function (value) {
                    return enumerator._settledAt(FULFILLED, i, value);
                }, function (reason) {
                    return enumerator._settledAt(REJECTED, i, reason);
                });
            };
            return Enumerator;
        }();
        /**
          `Promise.all` accepts an array of promises, and returns a new promise which
          is fulfilled with an array of fulfillment values for the passed promises, or
          rejected with the reason of the first passed promise to be rejected. It casts all
          elements of the passed iterable to promises as it runs this algorithm.
        
          Example:
        
          ```javascript
          let promise1 = resolve(1);
          let promise2 = resolve(2);
          let promise3 = resolve(3);
          let promises = [ promise1, promise2, promise3 ];
        
          Promise.all(promises).then(function(array){
            // The array here would be [ 1, 2, 3 ];
          });
          ```
        
          If any of the `promises` given to `all` are rejected, the first promise
          that is rejected will be given as an argument to the returned promises's
          rejection handler. For example:
        
          Example:
        
          ```javascript
          let promise1 = resolve(1);
          let promise2 = reject(new Error("2"));
          let promise3 = reject(new Error("3"));
          let promises = [ promise1, promise2, promise3 ];
        
          Promise.all(promises).then(function(array){
            // Code here never runs because there are rejected promises!
          }, function(error) {
            // error.message === "2"
          });
          ```
        
          @method all
          @static
          @param {Array} entries array of promises
          @param {String} label optional string for labeling the promise.
          Useful for tooling.
          @return {Promise} promise that is fulfilled when all `promises` have been
          fulfilled, or rejected if any of them become rejected.
          @static
        */
        function all(entries) {
            return new Enumerator(this, entries).promise;
        }
        /**
          `Promise.race` returns a new promise which is settled in the same way as the
          first passed promise to settle.
        
          Example:
        
          ```javascript
          let promise1 = new Promise(function(resolve, reject){
            setTimeout(function(){
              resolve('promise 1');
            }, 200);
          });
        
          let promise2 = new Promise(function(resolve, reject){
            setTimeout(function(){
              resolve('promise 2');
            }, 100);
          });
        
          Promise.race([promise1, promise2]).then(function(result){
            // result === 'promise 2' because it was resolved before promise1
            // was resolved.
          });
          ```
        
          `Promise.race` is deterministic in that only the state of the first
          settled promise matters. For example, even if other promises given to the
          `promises` array argument are resolved, but the first settled promise has
          become rejected before the other promises became fulfilled, the returned
          promise will become rejected:
        
          ```javascript
          let promise1 = new Promise(function(resolve, reject){
            setTimeout(function(){
              resolve('promise 1');
            }, 200);
          });
        
          let promise2 = new Promise(function(resolve, reject){
            setTimeout(function(){
              reject(new Error('promise 2'));
            }, 100);
          });
        
          Promise.race([promise1, promise2]).then(function(result){
            // Code here never runs
          }, function(reason){
            // reason.message === 'promise 2' because promise 2 became rejected before
            // promise 1 became fulfilled
          });
          ```
        
          An example real-world use case is implementing timeouts:
        
          ```javascript
          Promise.race([ajax('foo.json'), timeout(5000)])
          ```
        
          @method race
          @static
          @param {Array} promises array of promises to observe
          Useful for tooling.
          @return {Promise} a promise which settles in the same way as the first passed
          promise to settle.
        */
        function race(entries) {
            /*jshint validthis:true */
            var Constructor = this;
            if (!isArray(entries)) {
                return new Constructor(function (_, reject) {
                    return reject(new TypeError('You must pass an array to race.'));
                });
            }
            else {
                return new Constructor(function (resolve, reject) {
                    var length = entries.length;
                    for (var i = 0; i < length; i++) {
                        Constructor.resolve(entries[i]).then(resolve, reject);
                    }
                });
            }
        }
        /**
          `Promise.reject` returns a promise rejected with the passed `reason`.
          It is shorthand for the following:
        
          ```javascript
          let promise = new Promise(function(resolve, reject){
            reject(new Error('WHOOPS'));
          });
        
          promise.then(function(value){
            // Code here doesn't run because the promise is rejected!
          }, function(reason){
            // reason.message === 'WHOOPS'
          });
          ```
        
          Instead of writing the above, your code now simply becomes the following:
        
          ```javascript
          let promise = Promise.reject(new Error('WHOOPS'));
        
          promise.then(function(value){
            // Code here doesn't run because the promise is rejected!
          }, function(reason){
            // reason.message === 'WHOOPS'
          });
          ```
        
          @method reject
          @static
          @param {Any} reason value that the returned promise will be rejected with.
          Useful for tooling.
          @return {Promise} a promise rejected with the given `reason`.
        */
        function reject$1(reason) {
            /*jshint validthis:true */
            var Constructor = this;
            var promise = new Constructor(noop);
            reject(promise, reason);
            return promise;
        }
        function needsResolver() {
            throw new TypeError('You must pass a resolver function as the first argument to the promise constructor');
        }
        function needsNew() {
            throw new TypeError("Failed to construct 'Promise': Please use the 'new' operator, this object constructor cannot be called as a function.");
        }
        /**
          Promise objects represent the eventual result of an asynchronous operation. The
          primary way of interacting with a promise is through its `then` method, which
          registers callbacks to receive either a promise's eventual value or the reason
          why the promise cannot be fulfilled.
        
          Terminology
          -----------
        
          - `promise` is an object or function with a `then` method whose behavior conforms to this specification.
          - `thenable` is an object or function that defines a `then` method.
          - `value` is any legal JavaScript value (including undefined, a thenable, or a promise).
          - `exception` is a value that is thrown using the throw statement.
          - `reason` is a value that indicates why a promise was rejected.
          - `settled` the final resting state of a promise, fulfilled or rejected.
        
          A promise can be in one of three states: pending, fulfilled, or rejected.
        
          Promises that are fulfilled have a fulfillment value and are in the fulfilled
          state.  Promises that are rejected have a rejection reason and are in the
          rejected state.  A fulfillment value is never a thenable.
        
          Promises can also be said to *resolve* a value.  If this value is also a
          promise, then the original promise's settled state will match the value's
          settled state.  So a promise that *resolves* a promise that rejects will
          itself reject, and a promise that *resolves* a promise that fulfills will
          itself fulfill.
        
        
          Basic Usage:
          ------------
        
          ```js
          let promise = new Promise(function(resolve, reject) {
            // on success
            resolve(value);
        
            // on failure
            reject(reason);
          });
        
          promise.then(function(value) {
            // on fulfillment
          }, function(reason) {
            // on rejection
          });
          ```
        
          Advanced Usage:
          ---------------
        
          Promises shine when abstracting away asynchronous interactions such as
          `XMLHttpRequest`s.
        
          ```js
          function getJSON(url) {
            return new Promise(function(resolve, reject){
              let xhr = new XMLHttpRequest();
        
              xhr.open('GET', url);
              xhr.onreadystatechange = handler;
              xhr.responseType = 'json';
              xhr.setRequestHeader('Accept', 'application/json');
              xhr.send();
        
              function handler() {
                if (this.readyState === this.DONE) {
                  if (this.status === 200) {
                    resolve(this.response);
                  } else {
                    reject(new Error('getJSON: `' + url + '` failed with status: [' + this.status + ']'));
                  }
                }
              };
            });
          }
        
          getJSON('/posts.json').then(function(json) {
            // on fulfillment
          }, function(reason) {
            // on rejection
          });
          ```
        
          Unlike callbacks, promises are great composable primitives.
        
          ```js
          Promise.all([
            getJSON('/posts'),
            getJSON('/comments')
          ]).then(function(values){
            values[0] // => postsJSON
            values[1] // => commentsJSON
        
            return values;
          });
          ```
        
          @class Promise
          @param {Function} resolver
          Useful for tooling.
          @constructor
        */
        var Promise$1 = function () {
            function Promise(resolver) {
                this[PROMISE_ID] = nextId();
                this._result = this._state = undefined;
                this._subscribers = [];
                if (noop !== resolver) {
                    typeof resolver !== 'function' && needsResolver();
                    this instanceof Promise ? initializePromise(this, resolver) : needsNew();
                }
            }
            /**
            The primary way of interacting with a promise is through its `then` method,
            which registers callbacks to receive either a promise's eventual value or the
            reason why the promise cannot be fulfilled.
             ```js
            findUser().then(function(user){
              // user is available
            }, function(reason){
              // user is unavailable, and you are given the reason why
            });
            ```
             Chaining
            --------
             The return value of `then` is itself a promise.  This second, 'downstream'
            promise is resolved with the return value of the first promise's fulfillment
            or rejection handler, or rejected if the handler throws an exception.
             ```js
            findUser().then(function (user) {
              return user.name;
            }, function (reason) {
              return 'default name';
            }).then(function (userName) {
              // If `findUser` fulfilled, `userName` will be the user's name, otherwise it
              // will be `'default name'`
            });
             findUser().then(function (user) {
              throw new Error('Found user, but still unhappy');
            }, function (reason) {
              throw new Error('`findUser` rejected and we're unhappy');
            }).then(function (value) {
              // never reached
            }, function (reason) {
              // if `findUser` fulfilled, `reason` will be 'Found user, but still unhappy'.
              // If `findUser` rejected, `reason` will be '`findUser` rejected and we're unhappy'.
            });
            ```
            If the downstream promise does not specify a rejection handler, rejection reasons will be propagated further downstream.
             ```js
            findUser().then(function (user) {
              throw new PedagogicalException('Upstream error');
            }).then(function (value) {
              // never reached
            }).then(function (value) {
              // never reached
            }, function (reason) {
              // The `PedgagocialException` is propagated all the way down to here
            });
            ```
             Assimilation
            ------------
             Sometimes the value you want to propagate to a downstream promise can only be
            retrieved asynchronously. This can be achieved by returning a promise in the
            fulfillment or rejection handler. The downstream promise will then be pending
            until the returned promise is settled. This is called *assimilation*.
             ```js
            findUser().then(function (user) {
              return findCommentsByAuthor(user);
            }).then(function (comments) {
              // The user's comments are now available
            });
            ```
             If the assimliated promise rejects, then the downstream promise will also reject.
             ```js
            findUser().then(function (user) {
              return findCommentsByAuthor(user);
            }).then(function (comments) {
              // If `findCommentsByAuthor` fulfills, we'll have the value here
            }, function (reason) {
              // If `findCommentsByAuthor` rejects, we'll have the reason here
            });
            ```
             Simple Example
            --------------
             Synchronous Example
             ```javascript
            let result;
             try {
              result = findResult();
              // success
            } catch(reason) {
              // failure
            }
            ```
             Errback Example
             ```js
            findResult(function(result, err){
              if (err) {
                // failure
              } else {
                // success
              }
            });
            ```
             Promise Example;
             ```javascript
            findResult().then(function(result){
              // success
            }, function(reason){
              // failure
            });
            ```
             Advanced Example
            --------------
             Synchronous Example
             ```javascript
            let author, books;
             try {
              author = findAuthor();
              books  = findBooksByAuthor(author);
              // success
            } catch(reason) {
              // failure
            }
            ```
             Errback Example
             ```js
             function foundBooks(books) {
             }
             function failure(reason) {
             }
             findAuthor(function(author, err){
              if (err) {
                failure(err);
                // failure
              } else {
                try {
                  findBoooksByAuthor(author, function(books, err) {
                    if (err) {
                      failure(err);
                    } else {
                      try {
                        foundBooks(books);
                      } catch(reason) {
                        failure(reason);
                      }
                    }
                  });
                } catch(error) {
                  failure(err);
                }
                // success
              }
            });
            ```
             Promise Example;
             ```javascript
            findAuthor().
              then(findBooksByAuthor).
              then(function(books){
                // found books
            }).catch(function(reason){
              // something went wrong
            });
            ```
             @method then
            @param {Function} onFulfilled
            @param {Function} onRejected
            Useful for tooling.
            @return {Promise}
            */
            /**
            `catch` is simply sugar for `then(undefined, onRejection)` which makes it the same
            as the catch block of a try/catch statement.
            ```js
            function findAuthor(){
            throw new Error('couldn't find that author');
            }
            // synchronous
            try {
            findAuthor();
            } catch(reason) {
            // something went wrong
            }
            // async with promises
            findAuthor().catch(function(reason){
            // something went wrong
            });
            ```
            @method catch
            @param {Function} onRejection
            Useful for tooling.
            @return {Promise}
            */
            Promise.prototype.catch = function _catch(onRejection) {
                return this.then(null, onRejection);
            };
            /**
              `finally` will be invoked regardless of the promise's fate just as native
              try/catch/finally behaves
            
              Synchronous example:
            
              ```js
              findAuthor() {
                if (Math.random() > 0.5) {
                  throw new Error();
                }
                return new Author();
              }
            
              try {
                return findAuthor(); // succeed or fail
              } catch(error) {
                return findOtherAuther();
              } finally {
                // always runs
                // doesn't affect the return value
              }
              ```
            
              Asynchronous example:
            
              ```js
              findAuthor().catch(function(reason){
                return findOtherAuther();
              }).finally(function(){
                // author was either found, or not
              });
              ```
            
              @method finally
              @param {Function} callback
              @return {Promise}
            */
            Promise.prototype.finally = function _finally(callback) {
                var promise = this;
                var constructor = promise.constructor;
                if (isFunction(callback)) {
                    return promise.then(function (value) {
                        return constructor.resolve(callback()).then(function () {
                            return value;
                        });
                    }, function (reason) {
                        return constructor.resolve(callback()).then(function () {
                            throw reason;
                        });
                    });
                }
                return promise.then(callback, callback);
            };
            return Promise;
        }();
        Promise$1.prototype.then = then;
        Promise$1.all = all;
        Promise$1.race = race;
        Promise$1.resolve = resolve$1;
        Promise$1.reject = reject$1;
        Promise$1._setScheduler = setScheduler;
        Promise$1._setAsap = setAsap;
        Promise$1._asap = asap;
        /*global self*/
        function polyfill() {
            var local = void 0;
            if (typeof global !== 'undefined') {
                local = global;
            }
            else if (typeof self !== 'undefined') {
                local = self;
            }
            else {
                try {
                    local = Function('return this')();
                }
                catch (e) {
                    throw new Error('polyfill failed because global object is unavailable in this environment');
                }
            }
            var P = local.Promise;
            if (P) {
                var promiseToString = null;
                try {
                    promiseToString = Object.prototype.toString.call(P.resolve());
                }
                catch (e) {
                    // silently ignored
                }
                if (promiseToString === '[object Promise]' && !P.cast) {
                    return;
                }
            }
            local.Promise = Promise$1;
        }
        // Strange compat..
        Promise$1.polyfill = polyfill;
        Promise$1.Promise = Promise$1;
        return Promise$1;
    })));
},
/* main.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    tslib_1.__exportStar(require(117) /* ./index */, exports);
},
/* index.js */ function _(require, module, exports) {
    var __importStar = (this && this.__importStar) || function (mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (Object.hasOwnProperty.call(mod, k))
                    result[k] = mod[k];
        result["default"] = mod;
        return result;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    var version_1 = require(118) /* ./version */;
    exports.version = version_1.version;
    var embed_1 = require(119) /* ./embed */;
    exports.index = embed_1.index;
    exports.embed = __importStar(require(119) /* ./embed */);
    exports.protocol = __importStar(require(505) /* ./protocol */);
    exports._testing = __importStar(require(506) /* ./testing */);
    var logging_1 = require(134) /* ./core/logging */;
    exports.logger = logging_1.logger;
    exports.set_log_level = logging_1.set_log_level;
    var settings_1 = require(142) /* ./core/settings */;
    exports.settings = settings_1.settings;
    var base_1 = require(122) /* ./base */;
    exports.Models = base_1.Models;
    var document_1 = require(120) /* ./document */;
    exports.documents = document_1.documents;
    var safely_1 = require(507) /* ./safely */;
    exports.safely = safely_1.safely;
},
/* version.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.version = "2.2.0";
},
/* embed/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var document_1 = require(120) /* ../document */;
    var logging_1 = require(134) /* ../core/logging */;
    var string_1 = require(144) /* ../core/util/string */;
    var object_1 = require(128) /* ../core/util/object */;
    var types_1 = require(123) /* ../core/util/types */;
    var callback_1 = require(131) /* ../core/util/callback */;
    var standalone_1 = require(496) /* ./standalone */;
    var server_1 = require(498) /* ./server */;
    var dom_1 = require(497) /* ./dom */;
    var standalone_2 = require(496) /* ./standalone */;
    exports.add_document_standalone = standalone_2.add_document_standalone;
    exports.index = standalone_2.index;
    var server_2 = require(498) /* ./server */;
    exports.add_document_from_session = server_2.add_document_from_session;
    var notebook_1 = require(503) /* ./notebook */;
    exports.embed_items_notebook = notebook_1.embed_items_notebook;
    exports.kernels = notebook_1.kernels;
    var dom_2 = require(497) /* ./dom */;
    exports.BOKEH_ROOT = dom_2.BOKEH_ROOT;
    function embed_item(item, target_id) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var docs_json, doc_id, element, roots, render_item, _a, views;
            var _b;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        docs_json = {};
                        doc_id = string_1.uuid4();
                        docs_json[doc_id] = item.doc;
                        if (target_id == null)
                            target_id = item.target_id;
                        element = document.getElementById(target_id);
                        if (element != null)
                            element.classList.add(dom_1.BOKEH_ROOT);
                        roots = (_b = {}, _b[item.root_id] = target_id, _b);
                        render_item = { roots: roots, root_ids: [item.root_id], docid: doc_id };
                        return [4 /*yield*/, callback_1.defer(function () { return _embed_items(docs_json, [render_item]); })];
                    case 1:
                        _a = tslib_1.__read.apply(void 0, [_c.sent(), 1]), views = _a[0];
                        return [2 /*return*/, views];
                }
            });
        });
    }
    exports.embed_item = embed_item;
    // TODO (bev) this is currently clunky. Standalone embeds only provide
    // the first two args, whereas server provide the app_app, and *may* prove and
    // absolute_url as well if non-relative links are needed for resources. This function
    // should probably be split in to two pieces to reflect the different usage patterns
    function embed_items(docs_json, render_items, app_path, absolute_url) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, callback_1.defer(function () { return _embed_items(docs_json, render_items, app_path, absolute_url); })];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    }
    exports.embed_items = embed_items;
    function _embed_items(docs_json, render_items, app_path, absolute_url) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var docs, _a, _b, _c, docid, doc_json, views, render_items_1, render_items_1_1, item, element, roots, _d, _e, websocket_url, _f, _g, error_1, e_1_1;
            var e_2, _h, e_1, _j;
            return tslib_1.__generator(this, function (_k) {
                switch (_k.label) {
                    case 0:
                        if (types_1.isString(docs_json))
                            docs_json = JSON.parse(string_1.unescape(docs_json));
                        docs = {};
                        try {
                            for (_a = tslib_1.__values(object_1.entries(docs_json)), _b = _a.next(); !_b.done; _b = _a.next()) {
                                _c = tslib_1.__read(_b.value, 2), docid = _c[0], doc_json = _c[1];
                                docs[docid] = document_1.Document.from_json(doc_json);
                            }
                        }
                        catch (e_2_1) {
                            e_2 = { error: e_2_1 };
                        }
                        finally {
                            try {
                                if (_b && !_b.done && (_h = _a.return))
                                    _h.call(_a);
                            }
                            finally {
                                if (e_2)
                                    throw e_2.error;
                            }
                        }
                        views = [];
                        _k.label = 1;
                    case 1:
                        _k.trys.push([1, 12, 13, 14]);
                        render_items_1 = tslib_1.__values(render_items), render_items_1_1 = render_items_1.next();
                        _k.label = 2;
                    case 2:
                        if (!!render_items_1_1.done)
                            return [3 /*break*/, 11];
                        item = render_items_1_1.value;
                        element = dom_1._resolve_element(item);
                        roots = dom_1._resolve_root_elements(item);
                        if (!(item.docid != null))
                            return [3 /*break*/, 4];
                        _e = (_d = views).push;
                        return [4 /*yield*/, standalone_1.add_document_standalone(docs[item.docid], element, roots, item.use_for_title)];
                    case 3:
                        _e.apply(_d, [_k.sent()]);
                        return [3 /*break*/, 10];
                    case 4:
                        if (!(item.token != null))
                            return [3 /*break*/, 9];
                        websocket_url = server_1._get_ws_url(app_path, absolute_url);
                        logging_1.logger.debug("embed: computed ws url: " + websocket_url);
                        _k.label = 5;
                    case 5:
                        _k.trys.push([5, 7, , 8]);
                        _g = (_f = views).push;
                        return [4 /*yield*/, server_1.add_document_from_session(websocket_url, item.token, element, roots, item.use_for_title)];
                    case 6:
                        _g.apply(_f, [_k.sent()]);
                        console.log("Bokeh items were rendered successfully");
                        return [3 /*break*/, 8];
                    case 7:
                        error_1 = _k.sent();
                        console.log("Error rendering Bokeh items:", error_1);
                        return [3 /*break*/, 8];
                    case 8: return [3 /*break*/, 10];
                    case 9: throw new Error("Error rendering Bokeh items: either 'docid' or 'token' was expected.");
                    case 10:
                        render_items_1_1 = render_items_1.next();
                        return [3 /*break*/, 2];
                    case 11: return [3 /*break*/, 14];
                    case 12:
                        e_1_1 = _k.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 14];
                    case 13:
                        try {
                            if (render_items_1_1 && !render_items_1_1.done && (_j = render_items_1.return))
                                _j.call(render_items_1);
                        }
                        finally {
                            if (e_1)
                                throw e_1.error;
                        }
                        return [7 /*endfinally*/];
                    case 14: return [2 /*return*/, views];
                }
            });
        });
    }
},
/* document/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    tslib_1.__exportStar(require(121) /* ./document */, exports);
    tslib_1.__exportStar(require(236) /* ./events */, exports);
},
/* document/document.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var base_1 = require(122) /* ../base */;
    var version_1 = require(118) /* ../version */;
    var logging_1 = require(134) /* ../core/logging */;
    var bokeh_events_1 = require(428) /* ../core/bokeh_events */;
    var has_props_1 = require(129) /* ../core/has_props */;
    var signaling_1 = require(130) /* ../core/signaling */;
    var refs_1 = require(132) /* ../core/util/refs */;
    var serialization_1 = require(146) /* ../core/util/serialization */;
    var array_1 = require(124) /* ../core/util/array */;
    var object_1 = require(128) /* ../core/util/object */;
    var sets = tslib_1.__importStar(require(235) /* ../core/util/set */);
    var eq_1 = require(140) /* ../core/util/eq */;
    var types_1 = require(123) /* ../core/util/types */;
    var layout_dom_1 = require(387) /* ../models/layouts/layout_dom */;
    var column_data_source_1 = require(200) /* ../models/sources/column_data_source */;
    var model_1 = require(196) /* ../model */;
    var events_1 = require(236) /* ./events */;
    // Dispatches events to the subscribed models
    var EventManager = /** @class */ (function () {
        function EventManager(document) {
            this.document = document;
            this.session = null;
            this.subscribed_models = new Set();
        }
        EventManager.prototype.send_event = function (bokeh_event) {
            var event = new events_1.MessageSentEvent(this.document, "bokeh_event", bokeh_event.to_json());
            this.document._trigger_on_change(event);
        };
        EventManager.prototype.trigger = function (event) {
            var e_1, _b;
            try {
                for (var _c = tslib_1.__values(this.subscribed_models), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var model = _d.value;
                    if (event.origin != null && event.origin != model)
                        continue;
                    model._process_event(event);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        return EventManager;
    }());
    exports.EventManager = EventManager;
    EventManager.__name__ = "EventManager";
    exports.documents = [];
    exports.DEFAULT_TITLE = "Bokeh Application";
    // This class should match the API of the Python Document class
    // as much as possible.
    var Document = /** @class */ (function () {
        function Document() {
            exports.documents.push(this);
            this._init_timestamp = Date.now();
            this._title = exports.DEFAULT_TITLE;
            this._roots = [];
            this._all_models = new Map();
            this._all_models_freeze_count = 0;
            this._callbacks = new Map();
            this._message_callbacks = new Map();
            this.event_manager = new EventManager(this);
            this.idle = new signaling_1.Signal0(this, "idle");
            this._idle_roots = new WeakMap(); // TODO: WeakSet would be better
            this._interactive_timestamp = null;
            this._interactive_plot = null;
        }
        Object.defineProperty(Document.prototype, "layoutables", {
            get: function () {
                return this._roots.filter(function (root) { return root instanceof layout_dom_1.LayoutDOM; });
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Document.prototype, "is_idle", {
            get: function () {
                var e_2, _b;
                try {
                    for (var _c = tslib_1.__values(this.layoutables), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var root = _d.value;
                        if (!this._idle_roots.has(root))
                            return false;
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (_d && !_d.done && (_b = _c.return))
                            _b.call(_c);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
                return true;
            },
            enumerable: true,
            configurable: true
        });
        Document.prototype.notify_idle = function (model) {
            this._idle_roots.set(model, true);
            if (this.is_idle) {
                logging_1.logger.info("document idle at " + (Date.now() - this._init_timestamp) + " ms");
                this.event_manager.send_event(new bokeh_events_1.DocumentReady());
                this.idle.emit();
            }
        };
        Document.prototype.clear = function () {
            this._push_all_models_freeze();
            try {
                while (this._roots.length > 0) {
                    this.remove_root(this._roots[0]);
                }
            }
            finally {
                this._pop_all_models_freeze();
            }
        };
        Document.prototype.interactive_start = function (plot) {
            if (this._interactive_plot == null) {
                this._interactive_plot = plot;
                this._interactive_plot.trigger_event(new bokeh_events_1.LODStart());
            }
            this._interactive_timestamp = Date.now();
        };
        Document.prototype.interactive_stop = function () {
            if (this._interactive_plot != null) {
                this._interactive_plot.trigger_event(new bokeh_events_1.LODEnd());
            }
            this._interactive_plot = null;
            this._interactive_timestamp = null;
        };
        Document.prototype.interactive_duration = function () {
            if (this._interactive_timestamp == null)
                return -1;
            else
                return Date.now() - this._interactive_timestamp;
        };
        Document.prototype.destructively_move = function (dest_doc) {
            var e_3, _b, e_4, _c;
            if (dest_doc === this) {
                throw new Error("Attempted to overwrite a document with itself");
            }
            dest_doc.clear();
            // we have to remove ALL roots before adding any
            // to the new doc or else models referenced from multiple
            // roots could be in both docs at once, which isn't allowed.
            var roots = array_1.copy(this._roots);
            this.clear();
            try {
                for (var roots_1 = tslib_1.__values(roots), roots_1_1 = roots_1.next(); !roots_1_1.done; roots_1_1 = roots_1.next()) {
                    var root = roots_1_1.value;
                    if (root.document != null)
                        throw new Error("Somehow we didn't detach " + root);
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (roots_1_1 && !roots_1_1.done && (_b = roots_1.return))
                        _b.call(roots_1);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            if (this._all_models.size != 0) {
                throw new Error("this._all_models still had stuff in it: " + this._all_models);
            }
            try {
                for (var roots_2 = tslib_1.__values(roots), roots_2_1 = roots_2.next(); !roots_2_1.done; roots_2_1 = roots_2.next()) {
                    var root = roots_2_1.value;
                    dest_doc.add_root(root);
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (roots_2_1 && !roots_2_1.done && (_c = roots_2.return))
                        _c.call(roots_2);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            dest_doc.set_title(this._title);
        };
        // TODO other fields of doc
        Document.prototype._push_all_models_freeze = function () {
            this._all_models_freeze_count += 1;
        };
        Document.prototype._pop_all_models_freeze = function () {
            this._all_models_freeze_count -= 1;
            if (this._all_models_freeze_count === 0) {
                this._recompute_all_models();
            }
        };
        /*protected*/ Document.prototype._invalidate_all_models = function () {
            logging_1.logger.debug("invalidating document models");
            // if freeze count is > 0, we'll recompute on unfreeze
            if (this._all_models_freeze_count === 0) {
                this._recompute_all_models();
            }
        };
        Document.prototype._recompute_all_models = function () {
            var e_5, _b, e_6, _c, e_7, _d, e_8, _e;
            var new_all_models_set = new Set();
            try {
                for (var _f = tslib_1.__values(this._roots), _g = _f.next(); !_g.done; _g = _f.next()) {
                    var r = _g.value;
                    new_all_models_set = sets.union(new_all_models_set, r.references());
                }
            }
            catch (e_5_1) {
                e_5 = { error: e_5_1 };
            }
            finally {
                try {
                    if (_g && !_g.done && (_b = _f.return))
                        _b.call(_f);
                }
                finally {
                    if (e_5)
                        throw e_5.error;
                }
            }
            var old_all_models_set = new Set(this._all_models.values());
            var to_detach = sets.difference(old_all_models_set, new_all_models_set);
            var to_attach = sets.difference(new_all_models_set, old_all_models_set);
            var recomputed = new Map();
            try {
                for (var new_all_models_set_1 = tslib_1.__values(new_all_models_set), new_all_models_set_1_1 = new_all_models_set_1.next(); !new_all_models_set_1_1.done; new_all_models_set_1_1 = new_all_models_set_1.next()) {
                    var model = new_all_models_set_1_1.value;
                    recomputed.set(model.id, model);
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (new_all_models_set_1_1 && !new_all_models_set_1_1.done && (_c = new_all_models_set_1.return))
                        _c.call(new_all_models_set_1);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
            try {
                for (var to_detach_1 = tslib_1.__values(to_detach), to_detach_1_1 = to_detach_1.next(); !to_detach_1_1.done; to_detach_1_1 = to_detach_1.next()) {
                    var d = to_detach_1_1.value;
                    d.detach_document();
                }
            }
            catch (e_7_1) {
                e_7 = { error: e_7_1 };
            }
            finally {
                try {
                    if (to_detach_1_1 && !to_detach_1_1.done && (_d = to_detach_1.return))
                        _d.call(to_detach_1);
                }
                finally {
                    if (e_7)
                        throw e_7.error;
                }
            }
            try {
                for (var to_attach_1 = tslib_1.__values(to_attach), to_attach_1_1 = to_attach_1.next(); !to_attach_1_1.done; to_attach_1_1 = to_attach_1.next()) {
                    var a = to_attach_1_1.value;
                    a.attach_document(this);
                }
            }
            catch (e_8_1) {
                e_8 = { error: e_8_1 };
            }
            finally {
                try {
                    if (to_attach_1_1 && !to_attach_1_1.done && (_e = to_attach_1.return))
                        _e.call(to_attach_1);
                }
                finally {
                    if (e_8)
                        throw e_8.error;
                }
            }
            this._all_models = recomputed;
        };
        Document.prototype.roots = function () {
            return this._roots;
        };
        Document.prototype.add_root = function (model, setter_id) {
            logging_1.logger.debug("Adding root: " + model);
            if (array_1.includes(this._roots, model))
                return;
            this._push_all_models_freeze();
            try {
                this._roots.push(model);
            }
            finally {
                this._pop_all_models_freeze();
            }
            this._trigger_on_change(new events_1.RootAddedEvent(this, model, setter_id));
        };
        Document.prototype.remove_root = function (model, setter_id) {
            var i = this._roots.indexOf(model);
            if (i < 0)
                return;
            this._push_all_models_freeze();
            try {
                this._roots.splice(i, 1);
            }
            finally {
                this._pop_all_models_freeze();
            }
            this._trigger_on_change(new events_1.RootRemovedEvent(this, model, setter_id));
        };
        Document.prototype.title = function () {
            return this._title;
        };
        Document.prototype.set_title = function (title, setter_id) {
            if (title !== this._title) {
                this._title = title;
                this._trigger_on_change(new events_1.TitleChangedEvent(this, title, setter_id));
            }
        };
        Document.prototype.get_model_by_id = function (model_id) {
            var _a;
            return (_a = this._all_models.get(model_id)) !== null && _a !== void 0 ? _a : null;
        };
        Document.prototype.get_model_by_name = function (name) {
            var e_9, _b;
            var found = [];
            try {
                for (var _c = tslib_1.__values(this._all_models.values()), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var model = _d.value;
                    if (model instanceof model_1.Model && model.name == name)
                        found.push(model);
                }
            }
            catch (e_9_1) {
                e_9 = { error: e_9_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_9)
                        throw e_9.error;
                }
            }
            switch (found.length) {
                case 0:
                    return null;
                case 1:
                    return found[0];
                default:
                    throw new Error("Multiple models are named '" + name + "'");
            }
        };
        Document.prototype.on_message = function (msg_type, callback) {
            var message_callbacks = this._message_callbacks.get(msg_type);
            if (message_callbacks == null)
                this._message_callbacks.set(msg_type, new Set([callback]));
            else
                message_callbacks.add(callback);
        };
        Document.prototype.remove_on_message = function (msg_type, callback) {
            var _a;
            (_a = this._message_callbacks.get(msg_type)) === null || _a === void 0 ? void 0 : _a.delete(callback);
        };
        Document.prototype._trigger_on_message = function (msg_type, msg_data) {
            var e_10, _b;
            var message_callbacks = this._message_callbacks.get(msg_type);
            if (message_callbacks != null) {
                try {
                    for (var message_callbacks_1 = tslib_1.__values(message_callbacks), message_callbacks_1_1 = message_callbacks_1.next(); !message_callbacks_1_1.done; message_callbacks_1_1 = message_callbacks_1.next()) {
                        var cb = message_callbacks_1_1.value;
                        cb(msg_data);
                    }
                }
                catch (e_10_1) {
                    e_10 = { error: e_10_1 };
                }
                finally {
                    try {
                        if (message_callbacks_1_1 && !message_callbacks_1_1.done && (_b = message_callbacks_1.return))
                            _b.call(message_callbacks_1);
                    }
                    finally {
                        if (e_10)
                            throw e_10.error;
                    }
                }
            }
        };
        Document.prototype.on_change = function (callback, allow_batches) {
            if (allow_batches === void 0) {
                allow_batches = false;
            }
            if (!this._callbacks.has(callback)) {
                this._callbacks.set(callback, allow_batches);
            }
        };
        Document.prototype.remove_on_change = function (callback) {
            this._callbacks.delete(callback);
        };
        Document.prototype._trigger_on_change = function (event) {
            var e_11, _b, e_12, _c;
            try {
                for (var _d = tslib_1.__values(this._callbacks), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var _f = tslib_1.__read(_e.value, 2), callback = _f[0], allow_batches = _f[1];
                    if (!allow_batches && event instanceof events_1.DocumentEventBatch) {
                        try {
                            for (var _g = (e_12 = void 0, tslib_1.__values(event.events)), _h = _g.next(); !_h.done; _h = _g.next()) {
                                var ev = _h.value;
                                callback(ev);
                            }
                        }
                        catch (e_12_1) {
                            e_12 = { error: e_12_1 };
                        }
                        finally {
                            try {
                                if (_h && !_h.done && (_c = _g.return))
                                    _c.call(_g);
                            }
                            finally {
                                if (e_12)
                                    throw e_12.error;
                            }
                        }
                    }
                    else {
                        callback(event);
                    }
                }
            }
            catch (e_11_1) {
                e_11 = { error: e_11_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_b = _d.return))
                        _b.call(_d);
                }
                finally {
                    if (e_11)
                        throw e_11.error;
                }
            }
        };
        Document.prototype._notify_change = function (model, attr, old_value, new_value, options) {
            this._trigger_on_change(new events_1.ModelChangedEvent(this, model, attr, old_value, new_value, options === null || options === void 0 ? void 0 : options.setter_id, options === null || options === void 0 ? void 0 : options.hint));
        };
        Document._references_json = function (references, include_defaults) {
            var e_13, _b;
            if (include_defaults === void 0) {
                include_defaults = true;
            }
            var references_json = [];
            try {
                for (var references_1 = tslib_1.__values(references), references_1_1 = references_1.next(); !references_1_1.done; references_1_1 = references_1.next()) {
                    var r = references_1_1.value;
                    var struct = r.struct();
                    struct.attributes = r.attributes_as_json(include_defaults);
                    // server doesn't want id in here since it's already in ref above
                    delete struct.attributes.id;
                    references_json.push(struct);
                }
            }
            catch (e_13_1) {
                e_13 = { error: e_13_1 };
            }
            finally {
                try {
                    if (references_1_1 && !references_1_1.done && (_b = references_1.return))
                        _b.call(references_1);
                }
                finally {
                    if (e_13)
                        throw e_13.error;
                }
            }
            return references_json;
        };
        Document._instantiate_object = function (obj_id, obj_type, obj_attrs) {
            var full_attrs = Object.assign(Object.assign({}, obj_attrs), { id: obj_id, __deferred__: true });
            var model = base_1.Models(obj_type);
            return new model(full_attrs);
        };
        // given a JSON representation of all models in a graph, return a
        // dict of new model objects
        Document._instantiate_references_json = function (references_json, existing_models) {
            var e_14, _b;
            // Create all instances, but without setting their props
            var references = new Map();
            try {
                for (var references_json_1 = tslib_1.__values(references_json), references_json_1_1 = references_json_1.next(); !references_json_1_1.done; references_json_1_1 = references_json_1.next()) {
                    var obj = references_json_1_1.value;
                    var obj_id = obj.id;
                    var obj_type = obj.type;
                    var obj_attrs = obj.attributes || {};
                    var instance = existing_models.get(obj_id);
                    if (instance == null) {
                        instance = Document._instantiate_object(obj_id, obj_type, obj_attrs);
                        if (obj.subtype != null)
                            instance.set_subtype(obj.subtype);
                    }
                    references.set(instance.id, instance);
                }
            }
            catch (e_14_1) {
                e_14 = { error: e_14_1 };
            }
            finally {
                try {
                    if (references_json_1_1 && !references_json_1_1.done && (_b = references_json_1.return))
                        _b.call(references_json_1);
                }
                finally {
                    if (e_14)
                        throw e_14.error;
                }
            }
            return references;
        };
        // if v looks like a ref, or a collection, resolve it, otherwise return it unchanged
        // recurse into collections but not into HasProps
        Document._resolve_refs = function (value, old_references, new_references, buffers) {
            function resolve_ref(v) {
                if (refs_1.is_ref(v)) {
                    if (old_references.has(v.id))
                        return old_references.get(v.id);
                    else if (new_references.has(v.id))
                        return new_references.get(v.id);
                    else
                        throw new Error("reference " + JSON.stringify(v) + " isn't known (not in Document?)");
                }
                else if (serialization_1.is_NDArray_ref(v)) {
                    return serialization_1.decode_NDArray(v, buffers);
                }
                else if (types_1.isArray(v))
                    return resolve_array(v);
                else if (types_1.isPlainObject(v))
                    return resolve_dict(v);
                else
                    return v;
            }
            function resolve_array(array) {
                var e_15, _b;
                var results = [];
                try {
                    for (var array_2 = tslib_1.__values(array), array_2_1 = array_2.next(); !array_2_1.done; array_2_1 = array_2.next()) {
                        var v = array_2_1.value;
                        results.push(resolve_ref(v));
                    }
                }
                catch (e_15_1) {
                    e_15 = { error: e_15_1 };
                }
                finally {
                    try {
                        if (array_2_1 && !array_2_1.done && (_b = array_2.return))
                            _b.call(array_2);
                    }
                    finally {
                        if (e_15)
                            throw e_15.error;
                    }
                }
                return results;
            }
            function resolve_dict(dict) {
                var e_16, _b;
                var resolved = {};
                try {
                    for (var _c = tslib_1.__values(object_1.entries(dict)), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var _e = tslib_1.__read(_d.value, 2), k = _e[0], v = _e[1];
                        resolved[k] = resolve_ref(v);
                    }
                }
                catch (e_16_1) {
                    e_16 = { error: e_16_1 };
                }
                finally {
                    try {
                        if (_d && !_d.done && (_b = _c.return))
                            _b.call(_c);
                    }
                    finally {
                        if (e_16)
                            throw e_16.error;
                    }
                }
                return resolved;
            }
            return resolve_ref(value);
        };
        // given a JSON representation of all models in a graph and new
        // model instances, set the properties on the models from the
        // JSON
        Document._initialize_references_json = function (references_json, old_references, new_references, buffers) {
            var e_17, _b, e_18, _c, e_19, _d;
            var to_update = new Map();
            try {
                for (var references_json_2 = tslib_1.__values(references_json), references_json_2_1 = references_json_2.next(); !references_json_2_1.done; references_json_2_1 = references_json_2.next()) {
                    var _e = references_json_2_1.value, id = _e.id, attributes = _e.attributes;
                    var is_new = !old_references.has(id);
                    var instance = is_new ? new_references.get(id) : old_references.get(id);
                    // replace references with actual instances in obj_attrs
                    var resolved_attrs = Document._resolve_refs(attributes, old_references, new_references, buffers);
                    instance.setv(resolved_attrs, { silent: true });
                    to_update.set(id, { instance: instance, is_new: is_new });
                }
            }
            catch (e_17_1) {
                e_17 = { error: e_17_1 };
            }
            finally {
                try {
                    if (references_json_2_1 && !references_json_2_1.done && (_b = references_json_2.return))
                        _b.call(references_json_2);
                }
                finally {
                    if (e_17)
                        throw e_17.error;
                }
            }
            var ordered_instances = [];
            var handled = new Set();
            function finalize_all_by_dfs(v) {
                var e_20, _b, e_21, _c, e_22, _d;
                if (v instanceof has_props_1.HasProps) {
                    // note that we ignore instances that aren't updated (not in to_update)
                    if (to_update.has(v.id) && !handled.has(v.id)) {
                        handled.add(v.id);
                        var _e = to_update.get(v.id), instance = _e.instance, is_new = _e.is_new;
                        var attributes = instance.attributes;
                        try {
                            for (var _f = tslib_1.__values(object_1.values(attributes)), _g = _f.next(); !_g.done; _g = _f.next()) {
                                var value = _g.value;
                                finalize_all_by_dfs(value);
                            }
                        }
                        catch (e_20_1) {
                            e_20 = { error: e_20_1 };
                        }
                        finally {
                            try {
                                if (_g && !_g.done && (_b = _f.return))
                                    _b.call(_f);
                            }
                            finally {
                                if (e_20)
                                    throw e_20.error;
                            }
                        }
                        if (is_new) {
                            // Finalizing here just to avoid iterating
                            // over `ordered_instances` twice.
                            instance.finalize();
                            // Preserving an ordered collection of instances
                            // to avoid having to go through DFS again.
                            ordered_instances.push(instance);
                        }
                    }
                }
                else if (types_1.isArray(v)) {
                    try {
                        for (var v_1 = tslib_1.__values(v), v_1_1 = v_1.next(); !v_1_1.done; v_1_1 = v_1.next()) {
                            var e = v_1_1.value;
                            finalize_all_by_dfs(e);
                        }
                    }
                    catch (e_21_1) {
                        e_21 = { error: e_21_1 };
                    }
                    finally {
                        try {
                            if (v_1_1 && !v_1_1.done && (_c = v_1.return))
                                _c.call(v_1);
                        }
                        finally {
                            if (e_21)
                                throw e_21.error;
                        }
                    }
                }
                else if (types_1.isPlainObject(v)) {
                    try {
                        for (var _h = tslib_1.__values(object_1.values(v)), _j = _h.next(); !_j.done; _j = _h.next()) {
                            var value = _j.value;
                            finalize_all_by_dfs(value);
                        }
                    }
                    catch (e_22_1) {
                        e_22 = { error: e_22_1 };
                    }
                    finally {
                        try {
                            if (_j && !_j.done && (_d = _h.return))
                                _d.call(_h);
                        }
                        finally {
                            if (e_22)
                                throw e_22.error;
                        }
                    }
                }
            }
            try {
                for (var _f = tslib_1.__values(to_update.values()), _g = _f.next(); !_g.done; _g = _f.next()) {
                    var item = _g.value;
                    finalize_all_by_dfs(item.instance);
                }
            }
            catch (e_18_1) {
                e_18 = { error: e_18_1 };
            }
            finally {
                try {
                    if (_g && !_g.done && (_c = _f.return))
                        _c.call(_f);
                }
                finally {
                    if (e_18)
                        throw e_18.error;
                }
            }
            try {
                // `connect_signals` has to be executed last because it
                // may rely on properties of dependencies that are initialized
                // only in `finalize`. It's a problem that appears when
                // there are circular references, e.g. as in
                // CDS -> CustomJS (on data change) -> GlyphRenderer (in args) -> CDS.
                for (var ordered_instances_1 = tslib_1.__values(ordered_instances), ordered_instances_1_1 = ordered_instances_1.next(); !ordered_instances_1_1.done; ordered_instances_1_1 = ordered_instances_1.next()) {
                    var instance = ordered_instances_1_1.value;
                    instance.connect_signals();
                }
            }
            catch (e_19_1) {
                e_19 = { error: e_19_1 };
            }
            finally {
                try {
                    if (ordered_instances_1_1 && !ordered_instances_1_1.done && (_d = ordered_instances_1.return))
                        _d.call(ordered_instances_1);
                }
                finally {
                    if (e_19)
                        throw e_19.error;
                }
            }
        };
        Document._event_for_attribute_change = function (changed_obj, key, new_value, doc, value_refs) {
            var changed_model = doc.get_model_by_id(changed_obj.id); // XXX!
            if (!changed_model.property(key).syncable)
                return null;
            else {
                var event = {
                    kind: "ModelChanged",
                    model: { id: changed_obj.id },
                    attr: key,
                    new: new_value,
                };
                has_props_1.HasProps._json_record_references(doc, new_value, value_refs, { recursive: true });
                return event;
            }
        };
        Document._events_to_sync_objects = function (from_obj, to_obj, to_doc, value_refs) {
            var e_23, _b, e_24, _c, e_25, _d;
            var from_keys = Object.keys(from_obj.attributes); //XXX!
            var to_keys = Object.keys(to_obj.attributes); //XXX!
            var removed = array_1.difference(from_keys, to_keys);
            var added = array_1.difference(to_keys, from_keys);
            var shared = array_1.intersection(from_keys, to_keys);
            var events = [];
            try {
                for (var removed_1 = tslib_1.__values(removed), removed_1_1 = removed_1.next(); !removed_1_1.done; removed_1_1 = removed_1.next()) {
                    var key = removed_1_1.value;
                    // we don't really have a "remove" event - not sure this ever
                    // happens even. One way this could happen is if the server
                    // does include_defaults=True and we do
                    // include_defaults=false ... in that case it'd be best to
                    // just ignore this probably. Warn about it, could mean
                    // there's a bug if we don't have a key that the server sent.
                    logging_1.logger.warn("Server sent key " + key + " but we don't seem to have it in our JSON");
                }
            }
            catch (e_23_1) {
                e_23 = { error: e_23_1 };
            }
            finally {
                try {
                    if (removed_1_1 && !removed_1_1.done && (_b = removed_1.return))
                        _b.call(removed_1);
                }
                finally {
                    if (e_23)
                        throw e_23.error;
                }
            }
            try {
                for (var added_1 = tslib_1.__values(added), added_1_1 = added_1.next(); !added_1_1.done; added_1_1 = added_1.next()) {
                    var key = added_1_1.value;
                    var new_value = to_obj.attributes[key]; // XXX!
                    events.push(Document._event_for_attribute_change(from_obj, key, new_value, to_doc, value_refs));
                }
            }
            catch (e_24_1) {
                e_24 = { error: e_24_1 };
            }
            finally {
                try {
                    if (added_1_1 && !added_1_1.done && (_c = added_1.return))
                        _c.call(added_1);
                }
                finally {
                    if (e_24)
                        throw e_24.error;
                }
            }
            try {
                for (var shared_1 = tslib_1.__values(shared), shared_1_1 = shared_1.next(); !shared_1_1.done; shared_1_1 = shared_1.next()) {
                    var key = shared_1_1.value;
                    var old_value = from_obj.attributes[key]; // XXX!
                    var new_value = to_obj.attributes[key]; // XXX!
                    if (old_value == null && new_value == null) {
                    }
                    else if (old_value == null || new_value == null) {
                        events.push(Document._event_for_attribute_change(from_obj, key, new_value, to_doc, value_refs));
                    }
                    else {
                        if (!eq_1.isEqual(old_value, new_value))
                            events.push(Document._event_for_attribute_change(from_obj, key, new_value, to_doc, value_refs));
                    }
                }
            }
            catch (e_25_1) {
                e_25 = { error: e_25_1 };
            }
            finally {
                try {
                    if (shared_1_1 && !shared_1_1.done && (_d = shared_1.return))
                        _d.call(shared_1);
                }
                finally {
                    if (e_25)
                        throw e_25.error;
                }
            }
            return events.filter(function (e) { return e != null; });
        };
        // we use this to detect changes during document deserialization
        // (in model constructors and initializers)
        Document._compute_patch_since_json = function (from_json, to_doc) {
            var e_26, _b, e_27, _c, e_28, _d;
            var to_json = to_doc.to_json(false); // include_defaults=false
            function refs(json) {
                var e_29, _b;
                var result = new Map();
                try {
                    for (var _c = tslib_1.__values(json.roots.references), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var obj = _d.value;
                        result.set(obj.id, obj);
                    }
                }
                catch (e_29_1) {
                    e_29 = { error: e_29_1 };
                }
                finally {
                    try {
                        if (_d && !_d.done && (_b = _c.return))
                            _b.call(_c);
                    }
                    finally {
                        if (e_29)
                            throw e_29.error;
                    }
                }
                return result;
            }
            var from_references = refs(from_json);
            var from_roots = new Map();
            var from_root_ids = [];
            try {
                for (var _e = tslib_1.__values(from_json.roots.root_ids), _f = _e.next(); !_f.done; _f = _e.next()) {
                    var r = _f.value;
                    from_roots.set(r, from_references.get(r));
                    from_root_ids.push(r);
                }
            }
            catch (e_26_1) {
                e_26 = { error: e_26_1 };
            }
            finally {
                try {
                    if (_f && !_f.done && (_b = _e.return))
                        _b.call(_e);
                }
                finally {
                    if (e_26)
                        throw e_26.error;
                }
            }
            var to_references = refs(to_json);
            var to_roots = new Map();
            var to_root_ids = [];
            try {
                for (var _g = tslib_1.__values(to_json.roots.root_ids), _h = _g.next(); !_h.done; _h = _g.next()) {
                    var r = _h.value;
                    to_roots.set(r, to_references.get(r));
                    to_root_ids.push(r);
                }
            }
            catch (e_27_1) {
                e_27 = { error: e_27_1 };
            }
            finally {
                try {
                    if (_h && !_h.done && (_c = _g.return))
                        _c.call(_g);
                }
                finally {
                    if (e_27)
                        throw e_27.error;
                }
            }
            from_root_ids.sort();
            to_root_ids.sort();
            if (array_1.difference(from_root_ids, to_root_ids).length > 0 ||
                array_1.difference(to_root_ids, from_root_ids).length > 0) {
                // this would arise if someone does add_root/remove_root during
                // document deserialization, hopefully they won't ever do so.
                throw new Error("Not implemented: computing add/remove of document roots");
            }
            var value_refs = new Set();
            var events = [];
            try {
                for (var _j = tslib_1.__values(to_doc._all_models.keys()), _k = _j.next(); !_k.done; _k = _j.next()) {
                    var id = _k.value;
                    if (from_references.has(id)) {
                        var update_model_events = Document._events_to_sync_objects(from_references.get(id), to_references.get(id), to_doc, value_refs);
                        events = events.concat(update_model_events);
                    }
                }
            }
            catch (e_28_1) {
                e_28 = { error: e_28_1 };
            }
            finally {
                try {
                    if (_k && !_k.done && (_d = _j.return))
                        _d.call(_j);
                }
                finally {
                    if (e_28)
                        throw e_28.error;
                }
            }
            return {
                references: Document._references_json(value_refs, false),
                events: events,
            };
        };
        Document.prototype.to_json_string = function (include_defaults) {
            if (include_defaults === void 0) {
                include_defaults = true;
            }
            return JSON.stringify(this.to_json(include_defaults));
        };
        Document.prototype.to_json = function (include_defaults) {
            if (include_defaults === void 0) {
                include_defaults = true;
            }
            var root_ids = this._roots.map(function (r) { return r.id; });
            var root_references = this._all_models.values();
            return {
                version: version_1.version,
                title: this._title,
                roots: {
                    root_ids: root_ids,
                    references: Document._references_json(root_references, include_defaults),
                },
            };
        };
        Document.from_json_string = function (s) {
            var json = JSON.parse(s);
            return Document.from_json(json);
        };
        Document.from_json = function (json) {
            var e_30, _b;
            logging_1.logger.debug("Creating Document from JSON");
            function pyify(version) {
                return version.replace(/-(dev|rc)\./, "$1");
            }
            var py_version = json.version; // XXX!
            var is_dev = py_version.indexOf('+') !== -1 || py_version.indexOf('-') !== -1;
            var versions_string = "Library versions: JS (" + version_1.version + ") / Python (" + py_version + ")";
            if (!is_dev && pyify(version_1.version) != py_version) {
                logging_1.logger.warn("JS/Python version mismatch");
                logging_1.logger.warn(versions_string);
            }
            else
                logging_1.logger.debug(versions_string);
            var roots_json = json.roots;
            var root_ids = roots_json.root_ids;
            var references_json = roots_json.references;
            var references = Document._instantiate_references_json(references_json, new Map());
            Document._initialize_references_json(references_json, new Map(), references, new Map());
            var doc = new Document();
            try {
                for (var root_ids_1 = tslib_1.__values(root_ids), root_ids_1_1 = root_ids_1.next(); !root_ids_1_1.done; root_ids_1_1 = root_ids_1.next()) {
                    var id = root_ids_1_1.value;
                    var root = references.get(id);
                    if (root != null) {
                        doc.add_root(root); // XXX: HasProps
                    }
                }
            }
            catch (e_30_1) {
                e_30 = { error: e_30_1 };
            }
            finally {
                try {
                    if (root_ids_1_1 && !root_ids_1_1.done && (_b = root_ids_1.return))
                        _b.call(root_ids_1);
                }
                finally {
                    if (e_30)
                        throw e_30.error;
                }
            }
            doc.set_title(json.title); // XXX!
            return doc;
        };
        Document.prototype.replace_with_json = function (json) {
            var replacement = Document.from_json(json);
            replacement.destructively_move(this);
        };
        Document.prototype.create_json_patch_string = function (events) {
            return JSON.stringify(this.create_json_patch(events));
        };
        Document.prototype.create_json_patch = function (events) {
            var e_31, _b;
            var references = new Set();
            var json_events = [];
            try {
                for (var events_2 = tslib_1.__values(events), events_2_1 = events_2.next(); !events_2_1.done; events_2_1 = events_2.next()) {
                    var event = events_2_1.value;
                    if (event.document !== this) {
                        logging_1.logger.warn("Cannot create a patch using events from a different document, event had ", event.document, " we are ", this);
                        throw new Error("Cannot create a patch using events from a different document");
                    }
                    json_events.push(event.json(references));
                }
            }
            catch (e_31_1) {
                e_31 = { error: e_31_1 };
            }
            finally {
                try {
                    if (events_2_1 && !events_2_1.done && (_b = events_2.return))
                        _b.call(events_2);
                }
                finally {
                    if (e_31)
                        throw e_31.error;
                }
            }
            return {
                events: json_events,
                references: Document._references_json(references),
            };
        };
        Document.prototype.apply_json_patch = function (patch, buffers, setter_id) {
            var e_32, _b, e_33, _c, e_34, _d, _e;
            if (buffers === void 0) {
                buffers = new Map();
            }
            var references_json = patch.references;
            var events_json = patch.events;
            var references = Document._instantiate_references_json(references_json, this._all_models);
            if (!(buffers instanceof Map)) {
                buffers = new Map(buffers);
            }
            try {
                // The model being changed isn't always in references so add it in
                for (var events_json_1 = tslib_1.__values(events_json), events_json_1_1 = events_json_1.next(); !events_json_1_1.done; events_json_1_1 = events_json_1.next()) {
                    var event_json = events_json_1_1.value;
                    switch (event_json.kind) {
                        case "RootAdded":
                        case "RootRemoved":
                        case "ModelChanged": {
                            var model_id = event_json.model.id;
                            var model = this._all_models.get(model_id);
                            if (model != null) {
                                references.set(model_id, model);
                            }
                            else if (!references.has(model_id)) {
                                logging_1.logger.warn("Got an event for unknown model " + event_json.model + "\"");
                                throw new Error("event model wasn't known");
                            }
                            break;
                        }
                    }
                }
            }
            catch (e_32_1) {
                e_32 = { error: e_32_1 };
            }
            finally {
                try {
                    if (events_json_1_1 && !events_json_1_1.done && (_b = events_json_1.return))
                        _b.call(events_json_1);
                }
                finally {
                    if (e_32)
                        throw e_32.error;
                }
            }
            // split references into old and new so we know whether to initialize or update
            var old_references = new Map();
            var new_references = new Map();
            try {
                for (var references_2 = tslib_1.__values(references), references_2_1 = references_2.next(); !references_2_1.done; references_2_1 = references_2.next()) {
                    var _f = tslib_1.__read(references_2_1.value, 2), id = _f[0], value = _f[1];
                    if (this._all_models.has(id))
                        old_references.set(id, value);
                    else
                        new_references.set(id, value);
                }
            }
            catch (e_33_1) {
                e_33 = { error: e_33_1 };
            }
            finally {
                try {
                    if (references_2_1 && !references_2_1.done && (_c = references_2.return))
                        _c.call(references_2);
                }
                finally {
                    if (e_33)
                        throw e_33.error;
                }
            }
            Document._initialize_references_json(references_json, old_references, new_references, buffers);
            try {
                for (var events_json_2 = tslib_1.__values(events_json), events_json_2_1 = events_json_2.next(); !events_json_2_1.done; events_json_2_1 = events_json_2.next()) {
                    var event_json = events_json_2_1.value;
                    switch (event_json.kind) {
                        case 'MessageSent': {
                            var msg_type = event_json.msg_type, msg_data = event_json.msg_data;
                            var data = void 0;
                            if (msg_data === undefined) {
                                if (buffers.size == 1) {
                                    var _g = tslib_1.__read(buffers, 1), _h = tslib_1.__read(_g[0], 2), buffer = _h[1];
                                    data = buffer;
                                }
                                else {
                                    throw new Error("expected exactly one buffer");
                                }
                            }
                            else {
                                data = Document._resolve_refs(msg_data, old_references, new_references, buffers);
                            }
                            this._trigger_on_message(msg_type, data);
                            break;
                        }
                        case 'ModelChanged': {
                            var patched_id = event_json.model.id;
                            var patched_obj = this._all_models.get(patched_id);
                            if (patched_obj == null) {
                                throw new Error("Cannot apply patch to " + patched_id + " which is not in the document");
                            }
                            var attr = event_json.attr;
                            var value = Document._resolve_refs(event_json.new, old_references, new_references, buffers);
                            patched_obj.setv((_e = {}, _e[attr] = value, _e), { setter_id: setter_id });
                            break;
                        }
                        case 'ColumnDataChanged': {
                            var column_source_id = event_json.column_source.id;
                            var column_source = this._all_models.get(column_source_id);
                            if (column_source == null) {
                                throw new Error("Cannot stream to " + column_source_id + " which is not in the document");
                            }
                            var data = Document._resolve_refs(event_json.new, new Map(), new Map(), buffers);
                            if (event_json.cols != null) {
                                for (var k in column_source.data) {
                                    if (!(k in data)) {
                                        data[k] = column_source.data[k];
                                    }
                                }
                            }
                            column_source.setv({ data: data }, { setter_id: setter_id, check_eq: false });
                            break;
                        }
                        case 'ColumnsStreamed': {
                            var column_source_id = event_json.column_source.id;
                            var column_source = this._all_models.get(column_source_id);
                            if (column_source == null) {
                                throw new Error("Cannot stream to " + column_source_id + " which is not in the document");
                            }
                            if (!(column_source instanceof column_data_source_1.ColumnDataSource)) {
                                throw new Error("Cannot stream to non-ColumnDataSource");
                            }
                            var data = event_json.data;
                            var rollover = event_json.rollover;
                            column_source.stream(data, rollover, setter_id);
                            break;
                        }
                        case 'ColumnsPatched': {
                            var column_source_id = event_json.column_source.id;
                            var column_source = this._all_models.get(column_source_id);
                            if (column_source == null) {
                                throw new Error("Cannot patch " + column_source_id + " which is not in the document");
                            }
                            if (!(column_source instanceof column_data_source_1.ColumnDataSource)) {
                                throw new Error("Cannot patch non-ColumnDataSource");
                            }
                            var patches = event_json.patches;
                            column_source.patch(patches, setter_id);
                            break;
                        }
                        case 'RootAdded': {
                            var root_id = event_json.model.id;
                            var root_obj = references.get(root_id);
                            this.add_root(root_obj, setter_id); // XXX: HasProps
                            break;
                        }
                        case 'RootRemoved': {
                            var root_id = event_json.model.id;
                            var root_obj = references.get(root_id);
                            this.remove_root(root_obj, setter_id); // XXX: HasProps
                            break;
                        }
                        case 'TitleChanged': {
                            this.set_title(event_json.title, setter_id);
                            break;
                        }
                        default:
                            throw new Error("Unknown patch event " + JSON.stringify(event_json));
                    }
                }
            }
            catch (e_34_1) {
                e_34 = { error: e_34_1 };
            }
            finally {
                try {
                    if (events_json_2_1 && !events_json_2_1.done && (_d = events_json_2.return))
                        _d.call(events_json_2);
                }
                finally {
                    if (e_34)
                        throw e_34.error;
                }
            }
        };
        return Document;
    }());
    exports.Document = Document;
    Document.__name__ = "Document";
},
/* base.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(123) /* ./core/util/types */;
    var object_1 = require(128) /* ./core/util/object */;
    var has_props_1 = require(129) /* ./core/has_props */;
    exports.overrides = {};
    var _all_models = new Map();
    exports.Models = (function (name) {
        var model = exports.overrides[name] || _all_models.get(name);
        if (model == null) {
            throw new Error("Model '" + name + "' does not exist. This could be due to a widget or a custom model not being registered before first usage.");
        }
        return model;
    });
    exports.Models.register = function (name, model) {
        exports.overrides[name] = model;
    };
    exports.Models.unregister = function (name) {
        delete exports.overrides[name];
    };
    function is_HasProps(obj) {
        return types_1.isObject(obj) && obj.prototype instanceof has_props_1.HasProps;
    }
    exports.Models.register_models = function (models, force, errorFn) {
        var e_1, _a;
        if (force === void 0) {
            force = false;
        }
        if (models == null)
            return;
        try {
            for (var _b = tslib_1.__values(object_1.values(models)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var model = _c.value;
                if (is_HasProps(model)) {
                    var qualified = model.__qualified__;
                    if (force || !_all_models.has(qualified))
                        _all_models.set(qualified, model);
                    else if (errorFn != null)
                        errorFn(qualified);
                    else
                        console.warn("Model '" + qualified + "' was already registered");
                }
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return))
                    _a.call(_b);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
    };
    exports.register_models = exports.Models.register_models;
    exports.Models.registered_names = function () { return Array.from(_all_models.keys()); };
    // TODO: this doesn't belong here, but it's easier this way for backwards compatibility
    var AllModels = tslib_1.__importStar(require(149) /* ./models */);
    exports.register_models(AllModels);
},
/* core/util/types.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    //     Underscore.js 1.8.3
    //     http://underscorejs.org
    //     (c) 2009-2015 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
    //     Underscore may be freely distributed under the MIT license.
    var array_1 = require(124) /* ./array */;
    var toString = Object.prototype.toString;
    function isBoolean(obj) {
        return obj === true || obj === false || toString.call(obj) === '[object Boolean]';
    }
    exports.isBoolean = isBoolean;
    function isNumber(obj) {
        return toString.call(obj) === "[object Number]";
    }
    exports.isNumber = isNumber;
    function isInteger(obj) {
        return isNumber(obj) && Number.isInteger(obj);
    }
    exports.isInteger = isInteger;
    function isString(obj) {
        return toString.call(obj) === "[object String]";
    }
    exports.isString = isString;
    function isFunction(obj) {
        return toString.call(obj) === "[object Function]";
    }
    exports.isFunction = isFunction;
    function isArray(obj) {
        return Array.isArray(obj);
    }
    exports.isArray = isArray;
    function isArrayOf(arr, predicate) {
        return array_1.every(arr, predicate);
    }
    exports.isArrayOf = isArrayOf;
    function isArrayableOf(arr, predicate) {
        for (var i = 0, end = arr.length; i < end; i++) {
            if (!predicate(arr[i]))
                return false;
        }
        return true;
    }
    exports.isArrayableOf = isArrayableOf;
    function isTypedArray(obj) {
        return ArrayBuffer.isView(obj) && !(obj instanceof DataView);
    }
    exports.isTypedArray = isTypedArray;
    function isObject(obj) {
        var tp = typeof obj;
        return tp === 'function' || tp === 'object' && !!obj;
    }
    exports.isObject = isObject;
    function isPlainObject(obj) {
        return isObject(obj) && (obj.constructor == null || obj.constructor === Object);
    }
    exports.isPlainObject = isPlainObject;
    function isIterable(obj) {
        return Symbol.iterator in Object(obj);
    }
    exports.isIterable = isIterable;
},
/* core/util/array.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    //     Underscore.js 1.8.3
    //     http://underscorejs.org
    //     (c) 2009-2015 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
    //     Underscore may be freely distributed under the MIT license.
    var math_1 = require(125) /* ./math */;
    var assert_1 = require(126) /* ./assert */;
    var arrayable_1 = require(127) /* ./arrayable */;
    exports.map = arrayable_1.map;
    exports.reduce = arrayable_1.reduce;
    exports.min = arrayable_1.min;
    exports.min_by = arrayable_1.min_by;
    exports.max = arrayable_1.max;
    exports.max_by = arrayable_1.max_by;
    exports.sum = arrayable_1.sum;
    exports.cumsum = arrayable_1.cumsum;
    exports.every = arrayable_1.every;
    exports.some = arrayable_1.some;
    exports.find = arrayable_1.find;
    exports.find_last = arrayable_1.find_last;
    exports.find_index = arrayable_1.find_index;
    exports.find_last_index = arrayable_1.find_last_index;
    exports.sorted_index = arrayable_1.sorted_index;
    exports.is_empty = arrayable_1.is_empty;
    var slice = Array.prototype.slice;
    function head(array) {
        return array[0];
    }
    exports.head = head;
    function tail(array) {
        return array[array.length - 1];
    }
    exports.tail = tail;
    function last(array) {
        return array[array.length - 1];
    }
    exports.last = last;
    function copy(array) {
        return slice.call(array);
    }
    exports.copy = copy;
    function concat(arrays) {
        return [].concat.apply([], tslib_1.__spread(arrays));
    }
    exports.concat = concat;
    function includes(array, value) {
        return array.indexOf(value) !== -1;
    }
    exports.includes = includes;
    exports.contains = includes;
    function nth(array, index) {
        return array[index >= 0 ? index : array.length + index];
    }
    exports.nth = nth;
    function zip() {
        var arrays = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            arrays[_i] = arguments[_i];
        }
        if (arrays.length == 0)
            return [];
        var n = arrayable_1.min(arrays.map(function (a) { return a.length; }));
        var k = arrays.length;
        var result = new Array(n);
        for (var i = 0; i < n; i++) {
            result[i] = new Array(k);
            for (var j = 0; j < k; j++)
                result[i][j] = arrays[j][i];
        }
        return result;
    }
    exports.zip = zip;
    function unzip(array) {
        var n = array.length;
        var k = arrayable_1.min(array.map(function (a) { return a.length; }));
        var results = Array(k);
        for (var j = 0; j < k; j++)
            results[j] = new Array(n);
        for (var i = 0; i < n; i++) {
            for (var j = 0; j < k; j++)
                results[j][i] = array[i][j];
        }
        return results;
    }
    exports.unzip = unzip;
    function range(start, stop, step) {
        if (step === void 0) {
            step = 1;
        }
        assert_1.assert(step > 0, "'step' must be a positive number");
        if (stop == null) {
            stop = start;
            start = 0;
        }
        var max = Math.max, ceil = Math.ceil, abs = Math.abs;
        var delta = start <= stop ? step : -step;
        var length = max(ceil(abs(stop - start) / step), 0);
        var range = new Array(length);
        for (var i = 0; i < length; i++, start += delta) {
            range[i] = start;
        }
        return range;
    }
    exports.range = range;
    function linspace(start, stop, num) {
        if (num === void 0) {
            num = 100;
        }
        var step = (stop - start) / (num - 1);
        var array = new Array(num);
        for (var i = 0; i < num; i++) {
            array[i] = start + step * i;
        }
        return array;
    }
    exports.linspace = linspace;
    function transpose(array) {
        var rows = array.length;
        var cols = array[0].length;
        var transposed = [];
        for (var j = 0; j < cols; j++) {
            transposed[j] = [];
            for (var i = 0; i < rows; i++) {
                transposed[j][i] = array[i][j];
            }
        }
        return transposed;
    }
    exports.transpose = transpose;
    function argmin(array) {
        return arrayable_1.min_by(range(array.length), function (i) { return array[i]; });
    }
    exports.argmin = argmin;
    function argmax(array) {
        return arrayable_1.max_by(range(array.length), function (i) { return array[i]; });
    }
    exports.argmax = argmax;
    function sort_by(array, key) {
        var tmp = array.map(function (value, index) {
            return { value: value, index: index, key: key(value) };
        });
        tmp.sort(function (left, right) {
            var a = left.key;
            var b = right.key;
            if (a !== b) {
                if (a > b || a === undefined)
                    return 1;
                if (a < b || b === undefined)
                    return -1;
            }
            return left.index - right.index;
        });
        return tmp.map(function (item) { return item.value; });
    }
    exports.sort_by = sort_by;
    function uniq(array) {
        var e_1, _a;
        var result = new Set();
        try {
            for (var array_1 = tslib_1.__values(array), array_1_1 = array_1.next(); !array_1_1.done; array_1_1 = array_1.next()) {
                var value = array_1_1.value;
                result.add(value);
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (array_1_1 && !array_1_1.done && (_a = array_1.return))
                    _a.call(array_1);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        return tslib_1.__spread(result);
    }
    exports.uniq = uniq;
    function uniq_by(array, key) {
        var e_2, _a;
        var result = [];
        var seen = [];
        try {
            for (var array_2 = tslib_1.__values(array), array_2_1 = array_2.next(); !array_2_1.done; array_2_1 = array_2.next()) {
                var value = array_2_1.value;
                var computed = key(value);
                if (!includes(seen, computed)) {
                    seen.push(computed);
                    result.push(value);
                }
            }
        }
        catch (e_2_1) {
            e_2 = { error: e_2_1 };
        }
        finally {
            try {
                if (array_2_1 && !array_2_1.done && (_a = array_2.return))
                    _a.call(array_2);
            }
            finally {
                if (e_2)
                    throw e_2.error;
            }
        }
        return result;
    }
    exports.uniq_by = uniq_by;
    function union() {
        var e_3, _a, e_4, _b;
        var arrays = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            arrays[_i] = arguments[_i];
        }
        var result = new Set();
        try {
            for (var arrays_1 = tslib_1.__values(arrays), arrays_1_1 = arrays_1.next(); !arrays_1_1.done; arrays_1_1 = arrays_1.next()) {
                var array = arrays_1_1.value;
                try {
                    for (var array_3 = (e_4 = void 0, tslib_1.__values(array)), array_3_1 = array_3.next(); !array_3_1.done; array_3_1 = array_3.next()) {
                        var value = array_3_1.value;
                        result.add(value);
                    }
                }
                catch (e_4_1) {
                    e_4 = { error: e_4_1 };
                }
                finally {
                    try {
                        if (array_3_1 && !array_3_1.done && (_b = array_3.return))
                            _b.call(array_3);
                    }
                    finally {
                        if (e_4)
                            throw e_4.error;
                    }
                }
            }
        }
        catch (e_3_1) {
            e_3 = { error: e_3_1 };
        }
        finally {
            try {
                if (arrays_1_1 && !arrays_1_1.done && (_a = arrays_1.return))
                    _a.call(arrays_1);
            }
            finally {
                if (e_3)
                    throw e_3.error;
            }
        }
        return tslib_1.__spread(result);
    }
    exports.union = union;
    function intersection(array) {
        var e_5, _a, e_6, _b;
        var arrays = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            arrays[_i - 1] = arguments[_i];
        }
        var result = [];
        try {
            top: for (var array_4 = tslib_1.__values(array), array_4_1 = array_4.next(); !array_4_1.done; array_4_1 = array_4.next()) {
                var item = array_4_1.value;
                if (includes(result, item))
                    continue;
                try {
                    for (var arrays_2 = (e_6 = void 0, tslib_1.__values(arrays)), arrays_2_1 = arrays_2.next(); !arrays_2_1.done; arrays_2_1 = arrays_2.next()) {
                        var other = arrays_2_1.value;
                        if (!includes(other, item))
                            continue top;
                    }
                }
                catch (e_6_1) {
                    e_6 = { error: e_6_1 };
                }
                finally {
                    try {
                        if (arrays_2_1 && !arrays_2_1.done && (_b = arrays_2.return))
                            _b.call(arrays_2);
                    }
                    finally {
                        if (e_6)
                            throw e_6.error;
                    }
                }
                result.push(item);
            }
        }
        catch (e_5_1) {
            e_5 = { error: e_5_1 };
        }
        finally {
            try {
                if (array_4_1 && !array_4_1.done && (_a = array_4.return))
                    _a.call(array_4);
            }
            finally {
                if (e_5)
                    throw e_5.error;
            }
        }
        return result;
    }
    exports.intersection = intersection;
    function difference(array) {
        var arrays = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            arrays[_i - 1] = arguments[_i];
        }
        var rest = concat(arrays);
        return array.filter(function (value) { return !includes(rest, value); });
    }
    exports.difference = difference;
    function remove_at(array, i) {
        var result = copy(array);
        result.splice(i, 1);
        return result;
    }
    exports.remove_at = remove_at;
    function remove_by(array, key) {
        for (var i = 0; i < array.length;) {
            if (key(array[i]))
                array.splice(i, 1);
            else
                i++;
        }
    }
    exports.remove_by = remove_by;
    // Shuffle a collection, using the modern version of the
    // [Fisher-Yates shuffle](http://en.wikipedia.org/wiki/Fisher–Yates_shuffle).
    function shuffle(array) {
        var length = array.length;
        var shuffled = new Array(length);
        for (var i = 0; i < length; i++) {
            var rand = math_1.randomIn(0, i);
            if (rand !== i)
                shuffled[i] = shuffled[rand];
            shuffled[rand] = array[i];
        }
        return shuffled;
    }
    exports.shuffle = shuffle;
    function pairwise(array, fn) {
        var n = array.length;
        var result = new Array(n - 1);
        for (var i = 0; i < n - 1; i++) {
            result[i] = fn(array[i], array[i + 1]);
        }
        return result;
    }
    exports.pairwise = pairwise;
    function reversed(array) {
        var n = array.length;
        var result = new Array(n);
        for (var i = 0; i < n; i++) {
            result[n - i - 1] = array[i];
        }
        return result;
    }
    exports.reversed = reversed;
    function repeat(value, n) {
        var result = new Array(n);
        for (var i = 0; i < n; i++) {
            result[i] = value;
        }
        return result;
    }
    exports.repeat = repeat;
},
/* core/util/math.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function angle_norm(angle) {
        if (angle == 0) {
            return 0;
        }
        while (angle <= 0) {
            angle += 2 * Math.PI;
        }
        while (angle > 2 * Math.PI) {
            angle -= 2 * Math.PI;
        }
        return angle;
    }
    exports.angle_norm = angle_norm;
    function angle_dist(lhs, rhs) {
        return angle_norm(lhs - rhs);
    }
    exports.angle_dist = angle_dist;
    function angle_between(mid, lhs, rhs, direction) {
        var d = angle_dist(lhs, rhs);
        if (d == 0)
            return false;
        if (d == 2 * Math.PI)
            return true;
        var norm_mid = angle_norm(mid);
        var cond = angle_dist(lhs, norm_mid) <= d && angle_dist(norm_mid, rhs) <= d;
        return (direction == 0 /* clock */) ? cond : !cond;
    }
    exports.angle_between = angle_between;
    function random() {
        return Math.random();
    }
    exports.random = random;
    function randomIn(min, max) {
        if (max == null) {
            max = min;
            min = 0;
        }
        return min + Math.floor(Math.random() * (max - min + 1));
    }
    exports.randomIn = randomIn;
    function atan2(start, end) {
        /*
         * Calculate the angle between a line containing start and end points (composed
         * of [x, y] arrays) and the positive x-axis.
         */
        return Math.atan2(end[1] - start[1], end[0] - start[0]);
    }
    exports.atan2 = atan2;
    function radians(degrees) {
        return degrees * (Math.PI / 180);
    }
    exports.radians = radians;
    function degrees(radians) {
        return radians / (Math.PI / 180);
    }
    exports.degrees = degrees;
    // http://www2.econ.osaka-u.ac.jp/~tanizaki/class/2013/econome3/13.pdf (Page 432)
    function rnorm(mu, sigma) {
        // Generate a random normal with a mean of 0 and a sigma of 1
        var r1;
        var r2;
        while (true) {
            r1 = random();
            r2 = random();
            r2 = (2 * r2 - 1) * Math.sqrt(2 * (1 / Math.E));
            if (-4 * r1 * r1 * Math.log(r1) >= r2 * r2)
                break;
        }
        var rn = r2 / r1;
        // Transform the standard normal to meet the characteristics that we want (mu, sigma)
        rn = mu + sigma * rn;
        return rn;
    }
    exports.rnorm = rnorm;
    function clamp(val, min, max) {
        if (val < min)
            return min;
        if (val > max)
            return max;
        return val;
    }
    exports.clamp = clamp;
},
/* core/util/assert.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var AssertionError = /** @class */ (function (_super) {
        tslib_1.__extends(AssertionError, _super);
        function AssertionError() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return AssertionError;
    }(Error));
    exports.AssertionError = AssertionError;
    AssertionError.__name__ = "AssertionError";
    function assert(condition, message) {
        if (condition === true || (condition !== false && condition()))
            return;
        throw new AssertionError(message !== null && message !== void 0 ? message : "Assertion failed");
    }
    exports.assert = assert;
    function unreachable() {
        throw new Error("unreachable code");
    }
    exports.unreachable = unreachable;
},
/* core/util/arrayable.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(123) /* ./types */;
    var math_1 = require(125) /* ./math */;
    function is_empty(array) {
        return array.length == 0;
    }
    exports.is_empty = is_empty;
    function copy(array) {
        if (types_1.isArray(array))
            return array.slice();
        else
            return new array.constructor(array);
    }
    exports.copy = copy;
    function splice(array, start, k) {
        var e_1, _a;
        var items = [];
        for (var _i = 3; _i < arguments.length; _i++) {
            items[_i - 3] = arguments[_i];
        }
        var len = array.length;
        if (start < 0)
            start += len;
        if (start < 0)
            start = 0;
        else if (start > len)
            start = len;
        if (k == null || k > len - start)
            k = len - start;
        else if (k < 0)
            k = 0;
        var n = len - k + items.length;
        var result = new array.constructor(n);
        var i = 0;
        for (; i < start; i++) {
            result[i] = array[i];
        }
        try {
            for (var items_1 = tslib_1.__values(items), items_1_1 = items_1.next(); !items_1_1.done; items_1_1 = items_1.next()) {
                var item = items_1_1.value;
                result[i++] = item;
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (items_1_1 && !items_1_1.done && (_a = items_1.return))
                    _a.call(items_1);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        for (var j = start + k; j < len; j++) {
            result[i++] = array[j];
        }
        return result;
    }
    exports.splice = splice;
    function head(array, n) {
        return splice(array, n, array.length - n);
    }
    exports.head = head;
    function insert(array, item, i) {
        return splice(array, i, 0, item);
    }
    exports.insert = insert;
    function append(array, item) {
        return splice(array, array.length, 0, item);
    }
    exports.append = append;
    function prepend(array, item) {
        return splice(array, 0, 0, item);
    }
    exports.prepend = prepend;
    function indexOf(array, item) {
        for (var i = 0, n = array.length; i < n; i++) {
            if (array[i] === item)
                return i;
        }
        return -1;
    }
    exports.indexOf = indexOf;
    function subselect(array, indices) {
        var n = indices.length;
        var result = new array.constructor(n);
        for (var i = 0; i < n; i++) {
            result[i] = array[indices[i]];
        }
        return result;
    }
    exports.subselect = subselect;
    function map(array, fn) {
        var n = array.length;
        var result = new array.constructor(n);
        for (var i = 0; i < n; i++) {
            result[i] = fn(array[i], i, array);
        }
        return result;
    }
    exports.map = map;
    function filter(array, pred) {
        var n = array.length;
        var result = new array.constructor(n);
        var k = 0;
        for (var i = 0; i < n; i++) {
            var value = array[i];
            if (pred(value, i, array))
                result[k++] = value;
        }
        return head(result, k);
    }
    exports.filter = filter;
    function reduce(array, fn, initial) {
        var n = array.length;
        if (initial === undefined && n == 0)
            throw new Error("can't reduce an empty array without an initial value");
        var value;
        var i;
        if (initial === undefined) {
            value = array[0];
            i = 1;
        }
        else {
            value = initial;
            i = 0;
        }
        for (; i < n; i++) {
            value = fn(value, array[i], i, array);
        }
        return value;
    }
    exports.reduce = reduce;
    function min(array) {
        var value;
        var result = Infinity;
        for (var i = 0, length = array.length; i < length; i++) {
            value = array[i];
            if (!isNaN(value) && value < result) {
                result = value;
            }
        }
        return result;
    }
    exports.min = min;
    function max(array) {
        var value;
        var result = -Infinity;
        for (var i = 0, length = array.length; i < length; i++) {
            value = array[i];
            if (!isNaN(value) && value > result) {
                result = value;
            }
        }
        return result;
    }
    exports.max = max;
    function minmax(array) {
        var value;
        var min = +Infinity;
        var max = -Infinity;
        for (var i = 0, length = array.length; i < length; i++) {
            value = array[i];
            if (!isNaN(value)) {
                if (value < min) {
                    min = value;
                }
                if (value > max) {
                    max = value;
                }
            }
        }
        return [min, max];
    }
    exports.minmax = minmax;
    function min_by(array, key) {
        if (array.length == 0)
            throw new Error("min_by() called with an empty array");
        var result = array[0];
        var resultComputed = key(result);
        for (var i = 1, length = array.length; i < length; i++) {
            var value = array[i];
            var computed = key(value);
            if (computed < resultComputed) {
                result = value;
                resultComputed = computed;
            }
        }
        return result;
    }
    exports.min_by = min_by;
    function max_by(array, key) {
        if (array.length == 0)
            throw new Error("max_by() called with an empty array");
        var result = array[0];
        var resultComputed = key(result);
        for (var i = 1, length = array.length; i < length; i++) {
            var value = array[i];
            var computed = key(value);
            if (computed > resultComputed) {
                result = value;
                resultComputed = computed;
            }
        }
        return result;
    }
    exports.max_by = max_by;
    function sum(array) {
        var result = 0;
        for (var i = 0, n = array.length; i < n; i++) {
            result += array[i];
        }
        return result;
    }
    exports.sum = sum;
    function cumsum(array) {
        var result = new array.constructor(array.length);
        reduce(array, function (a, b, i) { return result[i] = a + b; }, 0);
        return result;
    }
    exports.cumsum = cumsum;
    function every(array, predicate) {
        for (var i = 0, length = array.length; i < length; i++) {
            if (!predicate(array[i]))
                return false;
        }
        return true;
    }
    exports.every = every;
    function some(array, predicate) {
        for (var i = 0, length = array.length; i < length; i++) {
            if (predicate(array[i]))
                return true;
        }
        return false;
    }
    exports.some = some;
    function index_of(array, value) {
        for (var i = 0, length = array.length; i < length; i++) {
            if (array[i] === value)
                return i;
        }
        return -1;
    }
    exports.index_of = index_of;
    function _find_index(dir) {
        return function (array, predicate) {
            var length = array.length;
            var index = dir > 0 ? 0 : length - 1;
            for (; index >= 0 && index < length; index += dir) {
                if (predicate(array[index]))
                    return index;
            }
            return -1;
        };
    }
    exports.find_index = _find_index(1);
    exports.find_last_index = _find_index(-1);
    function find(array, predicate) {
        var index = exports.find_index(array, predicate);
        return index == -1 ? undefined : array[index];
    }
    exports.find = find;
    function find_last(array, predicate) {
        var index = exports.find_last_index(array, predicate);
        return index == -1 ? undefined : array[index];
    }
    exports.find_last = find_last;
    function sorted_index(array, value) {
        var low = 0;
        var high = array.length;
        while (low < high) {
            var mid = Math.floor((low + high) / 2);
            if (array[mid] < value)
                low = mid + 1;
            else
                high = mid;
        }
        return low;
    }
    exports.sorted_index = sorted_index;
    function bin_counts(data, bin_edges) {
        var nbins = bin_edges.length - 1;
        var counts = Array(nbins).fill(0);
        for (var i = 0; i < data.length; i++) {
            var sample = data[i];
            var index = sorted_index(bin_edges, sample);
            var bin = math_1.clamp(index - 1, 0, nbins - 1);
            counts[bin] += 1;
        }
        return counts;
    }
    exports.bin_counts = bin_counts;
    function interpolate(points, x_values, y_values) {
        // Implementation ported from np.interp
        var n = points.length;
        var results = new Array(n);
        for (var i = 0; i < n; i++) {
            var point = points[i];
            if (isNaN(point)) {
                results[i] = point;
                continue;
            }
            var index = left_edge_index(point, x_values);
            if (index == -1)
                results[i] = y_values[0];
            else if (index == x_values.length)
                results[i] = y_values[y_values.length - 1];
            else if (index == x_values.length - 1 || x_values[index] == point) {
                results[i] = y_values[index];
            }
            else {
                var x0 = x_values[index];
                var y0 = y_values[index];
                var x1 = x_values[index + 1];
                var y1 = y_values[index + 1];
                results[i] = lerp(point, x0, y0, x1, y1);
            }
        }
        return results;
    }
    exports.interpolate = interpolate;
    function lerp(x, x0, y0, x1, y1) {
        var slope = (y1 - y0) / (x1 - x0);
        var res = slope * (x - x0) + y0;
        if (!isFinite(res)) {
            res = slope * (x - x1) + y1;
            if (!isFinite(res) && (y0 == y1))
                res = y0;
        }
        return res;
    }
    function left_edge_index(point, intervals) {
        if (point < intervals[0])
            return -1;
        if (point > intervals[intervals.length - 1])
            return intervals.length;
        var leftEdgeIndex = 0;
        var rightEdgeIndex = intervals.length - 1;
        while (rightEdgeIndex - leftEdgeIndex != 1) {
            var indexOfNumberToCompare = leftEdgeIndex + Math.floor((rightEdgeIndex - leftEdgeIndex) / 2);
            if (point >= intervals[indexOfNumberToCompare])
                leftEdgeIndex = indexOfNumberToCompare;
            else
                rightEdgeIndex = indexOfNumberToCompare;
        }
        return leftEdgeIndex;
    }
    exports.left_edge_index = left_edge_index;
    function norm(array, start, end) {
        var span = end - start;
        return map(array, function (x) { return (x - start) / span; });
    }
    exports.norm = norm;
},
/* core/util/object.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var array_1 = require(124) /* ./array */;
    exports.keys = Object.keys, exports.values = Object.values, exports.entries = Object.entries, exports.extend = Object.assign;
    function clone(obj) {
        return Object.assign({}, obj);
    }
    exports.clone = clone;
    function merge(obj1, obj2) {
        var e_1, _a;
        /*
         * Returns an object with the array values for obj1 and obj2 unioned by key.
         */
        var result = Object.create(Object.prototype);
        var keys = array_1.concat([Object.keys(obj1), Object.keys(obj2)]);
        try {
            for (var keys_1 = tslib_1.__values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                var key = keys_1_1.value;
                var arr1 = obj1.hasOwnProperty(key) ? obj1[key] : [];
                var arr2 = obj2.hasOwnProperty(key) ? obj2[key] : [];
                result[key] = array_1.union(arr1, arr2);
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (keys_1_1 && !keys_1_1.done && (_a = keys_1.return))
                    _a.call(keys_1);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        return result;
    }
    exports.merge = merge;
    function size(obj) {
        return Object.keys(obj).length;
    }
    exports.size = size;
    function isEmpty(obj) {
        return size(obj) == 0;
    }
    exports.isEmpty = isEmpty;
    function to_object(map) {
        var e_2, _a;
        var obj = {};
        try {
            for (var map_1 = tslib_1.__values(map), map_1_1 = map_1.next(); !map_1_1.done; map_1_1 = map_1.next()) {
                var _b = tslib_1.__read(map_1_1.value, 2), key = _b[0], val = _b[1];
                obj[key] = val;
            }
        }
        catch (e_2_1) {
            e_2 = { error: e_2_1 };
        }
        finally {
            try {
                if (map_1_1 && !map_1_1.done && (_a = map_1.return))
                    _a.call(map_1);
            }
            finally {
                if (e_2)
                    throw e_2.error;
            }
        }
        return obj;
    }
    exports.to_object = to_object;
},
/* core/has_props.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var signaling_1 = require(130) /* ./signaling */;
    var refs_1 = require(132) /* ./util/refs */;
    var p = tslib_1.__importStar(require(133) /* ./properties */);
    var k = tslib_1.__importStar(require(136) /* ./kinds */);
    var mixins = tslib_1.__importStar(require(143) /* ./property_mixins */);
    var string_1 = require(144) /* ./util/string */;
    var array_1 = require(124) /* ./util/array */;
    var object_1 = require(128) /* ./util/object */;
    var types_1 = require(123) /* ./util/types */;
    var eq_1 = require(140) /* ./util/eq */;
    var document_1 = require(120) /* ../document */;
    var ndarray_1 = require(145) /* ./util/ndarray */;
    var serialization_1 = require(146) /* ./util/serialization */;
    var eq_2 = require(140) /* ./util/eq */;
    var pretty_1 = require(148) /* ./util/pretty */;
    var kinds = tslib_1.__importStar(require(136) /* ./kinds */);
    var HasProps = /** @class */ (function (_super) {
        tslib_1.__extends(HasProps, _super);
        function HasProps(attrs) {
            var e_1, _b;
            if (attrs === void 0) {
                attrs = {};
            }
            var _this = this;
            var _a;
            _this = _super.call(this) || this;
            _this._subtype = undefined;
            _this.document = null;
            _this.destroyed = new signaling_1.Signal0(_this, "destroyed");
            _this.change = new signaling_1.Signal0(_this, "change");
            _this.transformchange = new signaling_1.Signal0(_this, "transformchange");
            _this.properties = {}; // Object.create(null)
            _this._pending = false;
            _this._changing = false;
            var get = attrs instanceof Map ? attrs.get : function (name) { return attrs[name]; };
            try {
                for (var _c = tslib_1.__values(object_1.entries(_this._props)), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = tslib_1.__read(_d.value, 2), name = _e[0], _f = _e[1], type = _f.type, default_value = _f.default_value, options = _f.options;
                    var property = void 0;
                    if (type instanceof k.Kind)
                        property = new p.PrimitiveProperty(_this, name, type, default_value, get(name), options);
                    else
                        property = new type(_this, name, k.Any, default_value, get(name), options);
                    _this.properties[name] = property;
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            // allowing us to defer initialization when loading many models
            // when loading a bunch of models, we want to do initialization as a second pass
            // because other objects that this one depends on might not be loaded yet
            if (!((_a = get("__deferred__")) !== null && _a !== void 0 ? _a : false)) {
                _this.finalize();
                _this.connect_signals();
            }
            return _this;
        }
        Object.defineProperty(HasProps.prototype, "type", {
            get: function () {
                return this.constructor.__qualified__;
            },
            // XXX: setter is only required for backwards compatibility
            set: function (name) {
                console.warn("prototype.type = 'ModelName' is deprecated, use static __name__ instead");
                this.constructor.__name__ = name;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(HasProps, "__qualified__", {
            get: function () {
                var _b = this, __module__ = _b.__module__, __name__ = _b.__name__;
                return __module__ != null ? __module__ + "." + __name__ : __name__;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(HasProps, Symbol.toStringTag, {
            get: function () {
                return this.__name__;
            },
            enumerable: true,
            configurable: true
        });
        HasProps.init_HasProps = function () {
            this.prototype._props = {};
            this.prototype._mixins = [];
            this.define({
                id: [p.String, function () { return string_1.uniqueId(); }],
            });
        };
        HasProps._fix_default = function (default_value, _attr) {
            if (default_value === undefined)
                return undefined;
            else if (types_1.isFunction(default_value))
                return default_value;
            else if (types_1.isArray(default_value))
                return function () { return array_1.copy(default_value); };
            else if (types_1.isPlainObject(default_value))
                return function () { return object_1.clone(default_value); };
            else if (!types_1.isObject(default_value))
                return function () { return default_value; };
            else
                throw new Error(default_value + " must be explicitly wrapped in a function");
        };
        // TODO: don't use Partial<>, but exclude inherited properties
        HasProps.define = function (obj) {
            var e_2, _b;
            var _loop_1 = function (name, prop) {
                if (this_1.prototype._props[name] != null)
                    throw new Error("attempted to redefine property '" + this_1.prototype.type + "." + name + "'");
                if (this_1.prototype[name] != null)
                    throw new Error("attempted to redefine attribute '" + this_1.prototype.type + "." + name + "'");
                Object.defineProperty(this_1.prototype, name, {
                    // XXX: don't use tail calls in getters/setters due to https://bugs.webkit.org/show_bug.cgi?id=164306
                    get: function () {
                        var value = this.properties[name].get_value();
                        return value;
                    },
                    set: function (value) {
                        var _b;
                        this.setv((_b = {}, _b[name] = value, _b));
                        return this;
                    },
                    configurable: false,
                    enumerable: true,
                });
                var _b = tslib_1.__read(prop, 3), type = _b[0], default_value = _b[1], options = _b[2];
                var refined_prop = {
                    type: type,
                    default_value: this_1._fix_default(default_value, name),
                    options: options,
                };
                var props = object_1.clone(this_1.prototype._props);
                props[name] = refined_prop;
                this_1.prototype._props = props;
            };
            var this_1 = this;
            try {
                for (var _c = tslib_1.__values(object_1.entries(types_1.isFunction(obj) ? obj(kinds) : obj)), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = tslib_1.__read(_d.value, 2), name = _e[0], prop = _e[1];
                    _loop_1(name, prop);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
        };
        HasProps.internal = function (obj) {
            var e_3, _b;
            var _object = {};
            try {
                for (var _c = tslib_1.__values(object_1.entries(obj)), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = tslib_1.__read(_d.value, 2), name = _e[0], entry = _e[1];
                    var _f = tslib_1.__read(entry, 3), type = _f[0], default_value = _f[1], _g = _f[2], options = _g === void 0 ? {} : _g;
                    _object[name] = [type, default_value, Object.assign(Object.assign({}, options), { internal: true })];
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            this.define(_object);
        };
        HasProps.mixins = function (defs) {
            var e_4, _b;
            if (!types_1.isArray(defs))
                defs = [defs];
            function resolve(kind) {
                switch (kind) {
                    case "line": return mixins.LineVector;
                    case "fill": return mixins.FillVector;
                    case "hatch": return mixins.HatchVector;
                    case "text": return mixins.TextVector;
                    default:
                        throw new Error("Unknown property mixin kind '" + kind + "'");
                }
            }
            function rename(prefix, mixin) {
                var e_5, _b;
                var result = {};
                try {
                    for (var _c = tslib_1.__values(object_1.entries(mixin)), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var _e = tslib_1.__read(_d.value, 2), name = _e[0], prop = _e[1];
                        result[prefix + name] = prop;
                    }
                }
                catch (e_5_1) {
                    e_5 = { error: e_5_1 };
                }
                finally {
                    try {
                        if (_d && !_d.done && (_b = _c.return))
                            _b.call(_c);
                    }
                    finally {
                        if (e_5)
                            throw e_5.error;
                    }
                }
                return result;
            }
            function kind_of(mixin) {
                var _b = tslib_1.__read(Object.keys(mixin), 1), key = _b[0];
                var _c = tslib_1.__read(key.split("_", 1), 1), kind = _c[0];
                return kind;
            }
            var mixin_defs = {};
            var names = [];
            try {
                for (var defs_1 = tslib_1.__values(defs), defs_1_1 = defs_1.next(); !defs_1_1.done; defs_1_1 = defs_1.next()) {
                    var def = defs_1_1.value;
                    if (types_1.isString(def)) {
                        // TODO: remove this branch in 3.0
                        var _c = tslib_1.__read(def.split(":"), 2), kind = _c[0], _d = _c[1], prefix = _d === void 0 ? "" : _d;
                        var mixin = resolve(kind);
                        names.push(def);
                        object_1.extend(mixin_defs, rename(prefix, mixin));
                    }
                    else if (types_1.isArray(def)) {
                        var _e = tslib_1.__read(def, 2), prefix = _e[0], mixin = _e[1];
                        names.push(kind_of(mixin) + ":" + prefix);
                        object_1.extend(mixin_defs, rename(prefix, mixin));
                    }
                    else {
                        var mixin = def;
                        names.push(kind_of(mixin));
                        object_1.extend(mixin_defs, mixin);
                    }
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (defs_1_1 && !defs_1_1.done && (_b = defs_1.return))
                        _b.call(defs_1);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            this.define(mixin_defs);
            this.prototype._mixins = tslib_1.__spread(this.prototype._mixins, names);
        };
        HasProps.override = function (obj) {
            var e_6, _b;
            try {
                for (var _c = tslib_1.__values(object_1.entries(obj)), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = tslib_1.__read(_d.value, 2), name = _e[0], prop = _e[1];
                    var default_value = this._fix_default(prop, name);
                    var value = this.prototype._props[name];
                    if (value == null)
                        throw new Error("attempted to override nonexistent '" + this.prototype.type + "." + name + "'");
                    var props = object_1.clone(this.prototype._props);
                    props[name] = Object.assign(Object.assign({}, value), { default_value: default_value });
                    this.prototype._props = props;
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
        };
        HasProps.prototype.toString = function () {
            return this.type + "(" + this.id + ")";
        };
        HasProps.prototype.property = function (name) {
            var prop = this.properties[name];
            if (prop != null)
                return prop;
            else
                throw new Error("unknown property " + this.type + "." + name);
        };
        Object.defineProperty(HasProps.prototype, "attributes", {
            get: function () {
                var e_7, _b;
                var attrs = {}; // Object.create(null)
                try {
                    for (var _c = tslib_1.__values(this), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var prop = _d.value;
                        attrs[prop.attr] = prop.get_value();
                    }
                }
                catch (e_7_1) {
                    e_7 = { error: e_7_1 };
                }
                finally {
                    try {
                        if (_d && !_d.done && (_b = _c.return))
                            _b.call(_c);
                    }
                    finally {
                        if (e_7)
                            throw e_7.error;
                    }
                }
                return attrs;
            },
            enumerable: true,
            configurable: true
        });
        HasProps.prototype[eq_2.equals] = function (that, cmp) {
            var e_8, _b;
            try {
                for (var _c = tslib_1.__values(this), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var p0 = _d.value;
                    var p1 = that.property(p0.attr);
                    if (cmp.eq(p0.get_value(), p1.get_value()))
                        return false;
                }
            }
            catch (e_8_1) {
                e_8 = { error: e_8_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_8)
                        throw e_8.error;
                }
            }
            return true;
        };
        HasProps.prototype[pretty_1.pretty] = function (printer) {
            var e_9, _b;
            var T = printer.token;
            var items = [];
            try {
                for (var _c = tslib_1.__values(this), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var prop = _d.value;
                    if (prop.dirty) {
                        var value = prop.get_value();
                        items.push("" + prop.attr + T(":") + " " + printer.to_string(value));
                    }
                }
            }
            catch (e_9_1) {
                e_9 = { error: e_9_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_9)
                        throw e_9.error;
                }
            }
            var cls = this.constructor.__qualified__;
            return "" + cls + T("(") + T("{") + items.join(T(",") + " ") + T("}") + T(")");
        };
        HasProps.prototype.finalize = function () {
            var e_10, _b;
            var _this = this;
            try {
                for (var _c = tslib_1.__values(this), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var prop = _d.value;
                    if (prop.spec.transform != null)
                        this.connect(prop.spec.transform.change, function () { return _this.transformchange.emit(); });
                }
            }
            catch (e_10_1) {
                e_10 = { error: e_10_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_10)
                        throw e_10.error;
                }
            }
            this.initialize();
        };
        HasProps.prototype.initialize = function () { };
        HasProps.prototype.connect_signals = function () { };
        HasProps.prototype.disconnect_signals = function () {
            signaling_1.Signal.disconnectReceiver(this);
        };
        HasProps.prototype.destroy = function () {
            this.disconnect_signals();
            this.destroyed.emit();
        };
        // Create a new model with identical attributes to this one.
        HasProps.prototype.clone = function () {
            return new this.constructor(this.attributes);
        };
        // Set a hash of model attributes on the object, firing `"change"`. This is
        // the core primitive operation of a model, updating the data and notifying
        // anyone who needs to know about the change in state. The heart of the beast.
        HasProps.prototype._setv = function (changes, options) {
            var e_11, _b, e_12, _c;
            // Extract attributes and options.
            var check_eq = options.check_eq;
            var changed = [];
            var changing = this._changing;
            this._changing = true;
            try {
                for (var changes_1 = tslib_1.__values(changes), changes_1_1 = changes_1.next(); !changes_1_1.done; changes_1_1 = changes_1.next()) {
                    var _d = tslib_1.__read(changes_1_1.value, 2), prop = _d[0], value = _d[1];
                    if (check_eq === false || !eq_1.isEqual(prop.get_value(), value)) {
                        prop.set_value(value);
                        changed.push(prop);
                    }
                }
            }
            catch (e_11_1) {
                e_11 = { error: e_11_1 };
            }
            finally {
                try {
                    if (changes_1_1 && !changes_1_1.done && (_b = changes_1.return))
                        _b.call(changes_1);
                }
                finally {
                    if (e_11)
                        throw e_11.error;
                }
            }
            // Trigger all relevant attribute changes.
            if (changed.length > 0)
                this._pending = true;
            try {
                for (var changed_1 = tslib_1.__values(changed), changed_1_1 = changed_1.next(); !changed_1_1.done; changed_1_1 = changed_1.next()) {
                    var prop = changed_1_1.value;
                    prop.change.emit();
                }
            }
            catch (e_12_1) {
                e_12 = { error: e_12_1 };
            }
            finally {
                try {
                    if (changed_1_1 && !changed_1_1.done && (_c = changed_1.return))
                        _c.call(changed_1);
                }
                finally {
                    if (e_12)
                        throw e_12.error;
                }
            }
            // You might be wondering why there's a `while` loop here. Changes can
            // be recursively nested within `"change"` events.
            if (changing)
                return;
            if (!options.no_change) {
                while (this._pending) {
                    this._pending = false;
                    this.change.emit();
                }
            }
            this._pending = false;
            this._changing = false;
        };
        HasProps.prototype.setv = function (changed_attrs, options) {
            var e_13, _b, e_14, _c, e_15, _d, e_16, _e;
            if (options === void 0) {
                options = {};
            }
            var changes = object_1.entries(changed_attrs);
            if (changes.length == 0)
                return;
            if (options.silent === true) {
                try {
                    for (var changes_2 = tslib_1.__values(changes), changes_2_1 = changes_2.next(); !changes_2_1.done; changes_2_1 = changes_2.next()) {
                        var _f = tslib_1.__read(changes_2_1.value, 2), attr = _f[0], value = _f[1];
                        this.properties[attr].set_value(value);
                    }
                }
                catch (e_13_1) {
                    e_13 = { error: e_13_1 };
                }
                finally {
                    try {
                        if (changes_2_1 && !changes_2_1.done && (_b = changes_2.return))
                            _b.call(changes_2);
                    }
                    finally {
                        if (e_13)
                            throw e_13.error;
                    }
                }
                return;
            }
            var changed = new Map();
            var previous = new Map();
            try {
                for (var changes_3 = tslib_1.__values(changes), changes_3_1 = changes_3.next(); !changes_3_1.done; changes_3_1 = changes_3.next()) {
                    var _g = tslib_1.__read(changes_3_1.value, 2), attr = _g[0], value = _g[1];
                    var prop = this.properties[attr];
                    changed.set(prop, value);
                    previous.set(prop, prop.get_value());
                }
            }
            catch (e_14_1) {
                e_14 = { error: e_14_1 };
            }
            finally {
                try {
                    if (changes_3_1 && !changes_3_1.done && (_c = changes_3.return))
                        _c.call(changes_3);
                }
                finally {
                    if (e_14)
                        throw e_14.error;
                }
            }
            this._setv(changed, options);
            var document = this.document;
            if (document != null) {
                var changed_3 = [];
                try {
                    for (var previous_1 = tslib_1.__values(previous), previous_1_1 = previous_1.next(); !previous_1_1.done; previous_1_1 = previous_1.next()) {
                        var _h = tslib_1.__read(previous_1_1.value, 2), prop = _h[0], value = _h[1];
                        changed_3.push([prop, value, prop.get_value()]);
                    }
                }
                catch (e_15_1) {
                    e_15 = { error: e_15_1 };
                }
                finally {
                    try {
                        if (previous_1_1 && !previous_1_1.done && (_d = previous_1.return))
                            _d.call(previous_1);
                    }
                    finally {
                        if (e_15)
                            throw e_15.error;
                    }
                }
                try {
                    for (var changed_2 = tslib_1.__values(changed_3), changed_2_1 = changed_2.next(); !changed_2_1.done; changed_2_1 = changed_2.next()) {
                        var _j = tslib_1.__read(changed_2_1.value, 3), old_value = _j[1], new_value = _j[2];
                        if (this._needs_invalidate(old_value, new_value)) {
                            document._invalidate_all_models();
                            break;
                        }
                    }
                }
                catch (e_16_1) {
                    e_16 = { error: e_16_1 };
                }
                finally {
                    try {
                        if (changed_2_1 && !changed_2_1.done && (_e = changed_2.return))
                            _e.call(changed_2);
                    }
                    finally {
                        if (e_16)
                            throw e_16.error;
                    }
                }
                this._push_changes(changed_3, options);
            }
        };
        /** @deprecated */
        HasProps.prototype.getv = function (name) {
            return this.property(name).get_value();
        };
        HasProps.prototype.ref = function () {
            return { id: this.id };
        };
        HasProps.prototype.struct = function () {
            var struct = {
                type: this.type,
                id: this.id,
                attributes: {},
            };
            if (this._subtype != null) {
                struct.subtype = this._subtype;
            }
            return struct;
        };
        // we only keep the subtype so we match Python;
        // only Python cares about this
        HasProps.prototype.set_subtype = function (subtype) {
            this._subtype = subtype;
        };
        HasProps.prototype[Symbol.iterator] = function () {
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [5 /*yield**/, tslib_1.__values(object_1.values(this.properties))];
                    case 1:
                        _b.sent();
                        return [2 /*return*/];
                }
            });
        };
        HasProps.prototype.syncable_properties = function () {
            var _b, _c, prop, e_17_1;
            var e_17, _d;
            return tslib_1.__generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        _e.trys.push([0, 5, 6, 7]);
                        _b = tslib_1.__values(this), _c = _b.next();
                        _e.label = 1;
                    case 1:
                        if (!!_c.done)
                            return [3 /*break*/, 4];
                        prop = _c.value;
                        if (!prop.syncable)
                            return [3 /*break*/, 3];
                        return [4 /*yield*/, prop];
                    case 2:
                        _e.sent();
                        _e.label = 3;
                    case 3:
                        _c = _b.next();
                        return [3 /*break*/, 1];
                    case 4: return [3 /*break*/, 7];
                    case 5:
                        e_17_1 = _e.sent();
                        e_17 = { error: e_17_1 };
                        return [3 /*break*/, 7];
                    case 6:
                        try {
                            if (_c && !_c.done && (_d = _b.return))
                                _d.call(_b);
                        }
                        finally {
                            if (e_17)
                                throw e_17.error;
                        }
                        return [7 /*endfinally*/];
                    case 7: return [2 /*return*/];
                }
            });
        };
        /** @deprecated */
        HasProps.prototype.serializable_attributes = function () {
            var e_18, _b;
            var attrs = {};
            try {
                for (var _c = tslib_1.__values(this.syncable_properties()), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var prop = _d.value;
                    attrs[prop.attr] = prop.get_value();
                }
            }
            catch (e_18_1) {
                e_18 = { error: e_18_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_18)
                        throw e_18.error;
                }
            }
            return attrs;
        };
        HasProps._value_to_json = function (value) {
            var e_19, _b;
            if (value instanceof HasProps)
                return value.ref();
            else if (ndarray_1.is_NDArray(value))
                return serialization_1.encode_NDArray(value);
            else if (types_1.isArray(value) || types_1.isTypedArray(value)) {
                var n = value.length;
                var ref_array = new Array(n);
                for (var i = 0; i < n; i++) {
                    var v = value[i];
                    ref_array[i] = HasProps._value_to_json(v);
                }
                return ref_array;
            }
            else if (types_1.isPlainObject(value)) {
                var ref_obj = {};
                try {
                    for (var _c = tslib_1.__values(object_1.entries(value)), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var _e = tslib_1.__read(_d.value, 2), subkey = _e[0], subvalue = _e[1];
                        ref_obj[subkey] = HasProps._value_to_json(subvalue);
                    }
                }
                catch (e_19_1) {
                    e_19 = { error: e_19_1 };
                }
                finally {
                    try {
                        if (_d && !_d.done && (_b = _c.return))
                            _b.call(_c);
                    }
                    finally {
                        if (e_19)
                            throw e_19.error;
                    }
                }
                return ref_obj;
            }
            else
                return value;
        };
        // Convert attributes to "shallow" JSON (values which are themselves models
        // are included as just references)
        HasProps.prototype.attributes_as_json = function (include_defaults, value_to_json) {
            var e_20, _b;
            if (include_defaults === void 0) {
                include_defaults = true;
            }
            if (value_to_json === void 0) {
                value_to_json = HasProps._value_to_json;
            }
            var attributes = {}; // Object.create(null)
            try {
                for (var _c = tslib_1.__values(this), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var prop = _d.value;
                    if (prop.syncable && (include_defaults || prop.dirty)) {
                        attributes[prop.attr] = value_to_json(prop.get_value());
                    }
                }
            }
            catch (e_20_1) {
                e_20 = { error: e_20_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_20)
                        throw e_20.error;
                }
            }
            return attributes;
        };
        // this is like _value_record_references but expects to find refs
        // instead of models, and takes a doc to look up the refs in
        HasProps._json_record_references = function (doc, v, refs, options) {
            var e_21, _b, e_22, _c;
            var recursive = options.recursive;
            if (refs_1.is_ref(v)) {
                var model = doc.get_model_by_id(v.id);
                if (model != null && !refs.has(model)) {
                    HasProps._value_record_references(model, refs, { recursive: recursive });
                }
            }
            else if (types_1.isArray(v)) {
                try {
                    for (var v_1 = tslib_1.__values(v), v_1_1 = v_1.next(); !v_1_1.done; v_1_1 = v_1.next()) {
                        var elem = v_1_1.value;
                        HasProps._json_record_references(doc, elem, refs, { recursive: recursive });
                    }
                }
                catch (e_21_1) {
                    e_21 = { error: e_21_1 };
                }
                finally {
                    try {
                        if (v_1_1 && !v_1_1.done && (_b = v_1.return))
                            _b.call(v_1);
                    }
                    finally {
                        if (e_21)
                            throw e_21.error;
                    }
                }
            }
            else if (types_1.isPlainObject(v)) {
                try {
                    for (var _d = tslib_1.__values(object_1.values(v)), _e = _d.next(); !_e.done; _e = _d.next()) {
                        var elem = _e.value;
                        HasProps._json_record_references(doc, elem, refs, { recursive: recursive });
                    }
                }
                catch (e_22_1) {
                    e_22 = { error: e_22_1 };
                }
                finally {
                    try {
                        if (_e && !_e.done && (_c = _d.return))
                            _c.call(_d);
                    }
                    finally {
                        if (e_22)
                            throw e_22.error;
                    }
                }
            }
        };
        // add all references from 'v' to 'result', if recurse
        // is true then descend into refs, if false only
        // descend into non-refs
        HasProps._value_record_references = function (v, refs, options) {
            var e_23, _b, e_24, _c, e_25, _d;
            var recursive = options.recursive;
            if (v instanceof HasProps) {
                if (!refs.has(v)) {
                    refs.add(v);
                    if (recursive) {
                        var immediate = v._immediate_references();
                        try {
                            for (var immediate_1 = tslib_1.__values(immediate), immediate_1_1 = immediate_1.next(); !immediate_1_1.done; immediate_1_1 = immediate_1.next()) {
                                var obj = immediate_1_1.value;
                                HasProps._value_record_references(obj, refs, { recursive: true });
                            }
                        }
                        catch (e_23_1) {
                            e_23 = { error: e_23_1 };
                        }
                        finally {
                            try {
                                if (immediate_1_1 && !immediate_1_1.done && (_b = immediate_1.return))
                                    _b.call(immediate_1);
                            }
                            finally {
                                if (e_23)
                                    throw e_23.error;
                            }
                        }
                    }
                }
            }
            else if (types_1.isArray(v)) {
                try {
                    for (var v_2 = tslib_1.__values(v), v_2_1 = v_2.next(); !v_2_1.done; v_2_1 = v_2.next()) {
                        var elem = v_2_1.value;
                        HasProps._value_record_references(elem, refs, { recursive: recursive });
                    }
                }
                catch (e_24_1) {
                    e_24 = { error: e_24_1 };
                }
                finally {
                    try {
                        if (v_2_1 && !v_2_1.done && (_c = v_2.return))
                            _c.call(v_2);
                    }
                    finally {
                        if (e_24)
                            throw e_24.error;
                    }
                }
            }
            else if (types_1.isPlainObject(v)) {
                try {
                    for (var _e = tslib_1.__values(object_1.values(v)), _f = _e.next(); !_f.done; _f = _e.next()) {
                        var elem = _f.value;
                        HasProps._value_record_references(elem, refs, { recursive: recursive });
                    }
                }
                catch (e_25_1) {
                    e_25 = { error: e_25_1 };
                }
                finally {
                    try {
                        if (_f && !_f.done && (_d = _e.return))
                            _d.call(_e);
                    }
                    finally {
                        if (e_25)
                            throw e_25.error;
                    }
                }
            }
        };
        // Get models that are immediately referenced by our properties
        // (do not recurse, do not include ourselves)
        HasProps.prototype._immediate_references = function () {
            var e_26, _b;
            var refs = new Set();
            try {
                for (var _c = tslib_1.__values(this.syncable_properties()), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var prop = _d.value;
                    var value = prop.get_value();
                    HasProps._value_record_references(value, refs, { recursive: false });
                }
            }
            catch (e_26_1) {
                e_26 = { error: e_26_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_26)
                        throw e_26.error;
                }
            }
            return refs;
        };
        HasProps.prototype.references = function () {
            var refs = new Set();
            HasProps._value_record_references(this, refs, { recursive: true });
            return refs;
        };
        HasProps.prototype._doc_attached = function () { };
        HasProps.prototype._doc_detached = function () { };
        HasProps.prototype.attach_document = function (doc) {
            // This should only be called by the Document implementation to set the document field
            if (this.document != null && this.document != doc)
                throw new Error("models must be owned by only a single document");
            this.document = doc;
            this._doc_attached();
        };
        HasProps.prototype.detach_document = function () {
            // This should only be called by the Document implementation to unset the document field
            this._doc_detached();
            this.document = null;
        };
        HasProps.prototype._needs_invalidate = function (old_value, new_value) {
            var e_27, _b, e_28, _c;
            var new_refs = new Set();
            HasProps._value_record_references(new_value, new_refs, { recursive: false });
            var old_refs = new Set();
            HasProps._value_record_references(old_value, old_refs, { recursive: false });
            try {
                for (var new_refs_1 = tslib_1.__values(new_refs), new_refs_1_1 = new_refs_1.next(); !new_refs_1_1.done; new_refs_1_1 = new_refs_1.next()) {
                    var new_id = new_refs_1_1.value;
                    if (!old_refs.has(new_id))
                        return true;
                }
            }
            catch (e_27_1) {
                e_27 = { error: e_27_1 };
            }
            finally {
                try {
                    if (new_refs_1_1 && !new_refs_1_1.done && (_b = new_refs_1.return))
                        _b.call(new_refs_1);
                }
                finally {
                    if (e_27)
                        throw e_27.error;
                }
            }
            try {
                for (var old_refs_1 = tslib_1.__values(old_refs), old_refs_1_1 = old_refs_1.next(); !old_refs_1_1.done; old_refs_1_1 = old_refs_1.next()) {
                    var old_id = old_refs_1_1.value;
                    if (!new_refs.has(old_id))
                        return true;
                }
            }
            catch (e_28_1) {
                e_28 = { error: e_28_1 };
            }
            finally {
                try {
                    if (old_refs_1_1 && !old_refs_1_1.done && (_c = old_refs_1.return))
                        _c.call(old_refs_1);
                }
                finally {
                    if (e_28)
                        throw e_28.error;
                }
            }
            return false;
        };
        HasProps.prototype._push_changes = function (changes, options) {
            var e_29, _b, _c;
            if (options === void 0) {
                options = {};
            }
            var document = this.document;
            if (document == null)
                return;
            var setter_id = options.setter_id;
            var events = [];
            try {
                for (var changes_4 = tslib_1.__values(changes), changes_4_1 = changes_4.next(); !changes_4_1.done; changes_4_1 = changes_4.next()) {
                    var _d = tslib_1.__read(changes_4_1.value, 3), prop = _d[0], old_value = _d[1], new_value = _d[2];
                    if (prop.syncable)
                        events.push(new document_1.ModelChangedEvent(document, this, prop.attr, old_value, new_value, setter_id));
                }
            }
            catch (e_29_1) {
                e_29 = { error: e_29_1 };
            }
            finally {
                try {
                    if (changes_4_1 && !changes_4_1.done && (_b = changes_4.return))
                        _b.call(changes_4);
                }
                finally {
                    if (e_29)
                        throw e_29.error;
                }
            }
            if (events.length != 0) {
                var event = void 0;
                if (events.length == 1)
                    _c = tslib_1.__read(events, 1), event = _c[0];
                else
                    event = new document_1.DocumentEventBatch(document, events, setter_id);
                document._trigger_on_change(event);
            }
        };
        HasProps.prototype.materialize_dataspecs = function (source) {
            var e_30, _b;
            // Note: this should be moved to a function separate from HasProps
            var data = {};
            try {
                for (var _c = tslib_1.__values(this), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var prop = _d.value;
                    if (!(prop instanceof p.VectorSpec))
                        continue;
                    // this skips optional properties like radius for circles
                    if (prop.optional && prop.spec.value == null && !prop.dirty)
                        continue;
                    var name = prop.attr;
                    var array = prop.array(source);
                    data["_" + name] = array;
                    if (prop instanceof p.DistanceSpec)
                        data["max_" + name] = array_1.max(array);
                }
            }
            catch (e_30_1) {
                e_30 = { error: e_30_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_30)
                        throw e_30.error;
                }
            }
            return data;
        };
        HasProps.prototype.on_change = function (properties, fn) {
            var e_31, _b;
            try {
                for (var _c = tslib_1.__values(types_1.isArray(properties) ? properties : [properties]), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var property = _d.value;
                    this.connect(property.change, fn);
                }
            }
            catch (e_31_1) {
                e_31 = { error: e_31_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_31)
                        throw e_31.error;
                }
            }
        };
        return HasProps;
    }(signaling_1.Signalable()));
    exports.HasProps = HasProps;
    HasProps.init_HasProps();
},
/* core/signaling.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    // Based on https://github.com/phosphorjs/phosphor/blob/master/packages/signaling/src/index.ts
    var callback_1 = require(131) /* ./util/callback */;
    var array_1 = require(124) /* ./util/array */;
    var Signal = /** @class */ (function () {
        function Signal(sender, name) {
            this.sender = sender;
            this.name = name;
        }
        Signal.prototype.connect = function (slot, context) {
            if (context === void 0) {
                context = null;
            }
            if (!receiversForSender.has(this.sender)) {
                receiversForSender.set(this.sender, []);
            }
            var receivers = receiversForSender.get(this.sender);
            if (findConnection(receivers, this, slot, context) != null) {
                return false;
            }
            var receiver = context !== null && context !== void 0 ? context : slot;
            if (!sendersForReceiver.has(receiver)) {
                sendersForReceiver.set(receiver, []);
            }
            var senders = sendersForReceiver.get(receiver);
            var connection = { signal: this, slot: slot, context: context };
            receivers.push(connection);
            senders.push(connection);
            return true;
        };
        Signal.prototype.disconnect = function (slot, context) {
            if (context === void 0) {
                context = null;
            }
            var receivers = receiversForSender.get(this.sender);
            if (receivers == null || receivers.length === 0) {
                return false;
            }
            var connection = findConnection(receivers, this, slot, context);
            if (connection == null) {
                return false;
            }
            var receiver = context !== null && context !== void 0 ? context : slot;
            var senders = sendersForReceiver.get(receiver);
            connection.signal = null;
            scheduleCleanup(receivers);
            scheduleCleanup(senders);
            return true;
        };
        Signal.prototype.emit = function (args) {
            var e_1, _b;
            var _a;
            var receivers = (_a = receiversForSender.get(this.sender)) !== null && _a !== void 0 ? _a : [];
            try {
                for (var receivers_1 = tslib_1.__values(receivers), receivers_1_1 = receivers_1.next(); !receivers_1_1.done; receivers_1_1 = receivers_1.next()) {
                    var _c = receivers_1_1.value, signal = _c.signal, slot = _c.slot, context = _c.context;
                    if (signal === this) {
                        slot.call(context, args, this.sender);
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (receivers_1_1 && !receivers_1_1.done && (_b = receivers_1.return))
                        _b.call(receivers_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        return Signal;
    }());
    exports.Signal = Signal;
    Signal.__name__ = "Signal";
    var Signal0 = /** @class */ (function (_super) {
        tslib_1.__extends(Signal0, _super);
        function Signal0() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Signal0.prototype.emit = function () {
            _super.prototype.emit.call(this, undefined);
        };
        return Signal0;
    }(Signal));
    exports.Signal0 = Signal0;
    Signal0.__name__ = "Signal0";
    (function (Signal) {
        function disconnectBetween(sender, receiver) {
            var e_2, _b;
            var receivers = receiversForSender.get(sender);
            if (receivers == null || receivers.length === 0)
                return;
            var senders = sendersForReceiver.get(receiver);
            if (senders == null || senders.length === 0)
                return;
            try {
                for (var senders_1 = tslib_1.__values(senders), senders_1_1 = senders_1.next(); !senders_1_1.done; senders_1_1 = senders_1.next()) {
                    var connection = senders_1_1.value;
                    if (connection.signal == null)
                        return;
                    if (connection.signal.sender === sender)
                        connection.signal = null;
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (senders_1_1 && !senders_1_1.done && (_b = senders_1.return))
                        _b.call(senders_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            scheduleCleanup(receivers);
            scheduleCleanup(senders);
        }
        Signal.disconnectBetween = disconnectBetween;
        function disconnectSender(sender) {
            var e_3, _b;
            var _a;
            var receivers = receiversForSender.get(sender);
            if (receivers == null || receivers.length === 0)
                return;
            try {
                for (var receivers_2 = tslib_1.__values(receivers), receivers_2_1 = receivers_2.next(); !receivers_2_1.done; receivers_2_1 = receivers_2.next()) {
                    var connection = receivers_2_1.value;
                    if (connection.signal == null)
                        return;
                    var receiver = (_a = connection.context) !== null && _a !== void 0 ? _a : connection.slot;
                    connection.signal = null;
                    scheduleCleanup(sendersForReceiver.get(receiver));
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (receivers_2_1 && !receivers_2_1.done && (_b = receivers_2.return))
                        _b.call(receivers_2);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            scheduleCleanup(receivers);
        }
        Signal.disconnectSender = disconnectSender;
        function disconnectReceiver(receiver) {
            var e_4, _b;
            var senders = sendersForReceiver.get(receiver);
            if (senders == null || senders.length === 0)
                return;
            try {
                for (var senders_2 = tslib_1.__values(senders), senders_2_1 = senders_2.next(); !senders_2_1.done; senders_2_1 = senders_2.next()) {
                    var connection = senders_2_1.value;
                    if (connection.signal == null)
                        return;
                    var sender = connection.signal.sender;
                    connection.signal = null;
                    scheduleCleanup(receiversForSender.get(sender));
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (senders_2_1 && !senders_2_1.done && (_b = senders_2.return))
                        _b.call(senders_2);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            scheduleCleanup(senders);
        }
        Signal.disconnectReceiver = disconnectReceiver;
        function disconnectAll(obj) {
            var e_5, _b, e_6, _c;
            var receivers = receiversForSender.get(obj);
            if (receivers != null && receivers.length !== 0) {
                try {
                    for (var receivers_3 = tslib_1.__values(receivers), receivers_3_1 = receivers_3.next(); !receivers_3_1.done; receivers_3_1 = receivers_3.next()) {
                        var connection = receivers_3_1.value;
                        connection.signal = null;
                    }
                }
                catch (e_5_1) {
                    e_5 = { error: e_5_1 };
                }
                finally {
                    try {
                        if (receivers_3_1 && !receivers_3_1.done && (_b = receivers_3.return))
                            _b.call(receivers_3);
                    }
                    finally {
                        if (e_5)
                            throw e_5.error;
                    }
                }
                scheduleCleanup(receivers);
            }
            var senders = sendersForReceiver.get(obj);
            if (senders != null && senders.length !== 0) {
                try {
                    for (var senders_3 = tslib_1.__values(senders), senders_3_1 = senders_3.next(); !senders_3_1.done; senders_3_1 = senders_3.next()) {
                        var connection = senders_3_1.value;
                        connection.signal = null;
                    }
                }
                catch (e_6_1) {
                    e_6 = { error: e_6_1 };
                }
                finally {
                    try {
                        if (senders_3_1 && !senders_3_1.done && (_c = senders_3.return))
                            _c.call(senders_3);
                    }
                    finally {
                        if (e_6)
                            throw e_6.error;
                    }
                }
                scheduleCleanup(senders);
            }
        }
        Signal.disconnectAll = disconnectAll;
    })(Signal || (exports.Signal = Signal = {}));
    function Signalable() {
        return /** @class */ (function () {
            function class_1() {
            }
            class_1.prototype.connect = function (signal, slot) {
                return signal.connect(slot, this);
            };
            class_1.prototype.disconnect = function (signal, slot) {
                return signal.disconnect(slot, this);
            };
            return class_1;
        }());
    }
    exports.Signalable = Signalable;
    var receiversForSender = new WeakMap();
    var sendersForReceiver = new WeakMap();
    function findConnection(conns, signal, slot, context) {
        return array_1.find(conns, function (conn) { return conn.signal === signal && conn.slot === slot && conn.context === context; });
    }
    var dirty_set = new Set();
    function scheduleCleanup(connections) {
        if (dirty_set.size === 0) {
            callback_1.defer(cleanup_dirty_set);
        }
        dirty_set.add(connections);
    }
    function cleanup_dirty_set() {
        var e_7, _b;
        try {
            for (var dirty_set_1 = tslib_1.__values(dirty_set), dirty_set_1_1 = dirty_set_1.next(); !dirty_set_1_1.done; dirty_set_1_1 = dirty_set_1.next()) {
                var connections = dirty_set_1_1.value;
                array_1.remove_by(connections, function (connection) { return connection.signal == null; });
            }
        }
        catch (e_7_1) {
            e_7 = { error: e_7_1 };
        }
        finally {
            try {
                if (dirty_set_1_1 && !dirty_set_1_1.done && (_b = dirty_set_1.return))
                    _b.call(dirty_set_1);
            }
            finally {
                if (e_7)
                    throw e_7.error;
            }
        }
        dirty_set.clear();
    }
},
/* core/util/callback.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    //     Underscore.js 1.8.3
    //     http://underscorejs.org
    //     (c) 2009-2015 Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
    //     Underscore may be freely distributed under the MIT license.
    function delay(func, wait) {
        return setTimeout(func, wait);
    }
    exports.delay = delay;
    var _defer = typeof requestAnimationFrame === "function" ? requestAnimationFrame : setImmediate;
    function defer(func) {
        return new Promise(function (resolve) {
            _defer(function () { return resolve(func()); });
        });
    }
    exports.defer = defer;
    function throttle(func, wait, options) {
        if (options === void 0) {
            options = {};
        }
        var context, args, result;
        var timeout = null;
        var previous = 0;
        var later = function () {
            previous = options.leading === false ? 0 : Date.now();
            timeout = null;
            result = func.apply(context, args);
            if (!timeout)
                context = args = null;
        };
        return function () {
            var now = Date.now();
            if (!previous && options.leading === false)
                previous = now;
            var remaining = wait - (now - previous);
            context = this;
            args = arguments;
            if (remaining <= 0 || remaining > wait) {
                if (timeout) {
                    clearTimeout(timeout);
                    timeout = null;
                }
                previous = now;
                result = func.apply(context, args);
                if (!timeout)
                    context = args = null;
            }
            else if (!timeout && options.trailing !== false) {
                timeout = setTimeout(later, remaining);
            }
            return result;
        };
    }
    exports.throttle = throttle;
    function once(func) {
        var done = false;
        var memo;
        return function () {
            if (!done) {
                done = true;
                memo = func();
            }
            return memo;
        };
    }
    exports.once = once;
},
/* core/util/refs.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var types_1 = require(123) /* ./types */;
    var object_1 = require(128) /* ./object */;
    // Determine whether an object has the proper format of a Bokeh reference
    //
    // @param arg [Object] the object to test
    // @return [bool] whether the object is a reference
    //
    // @note this function does not check that the id and types are valid,
    //   only that the format is correct (all required keys are present)
    //
    function is_ref(arg) {
        if (types_1.isPlainObject(arg)) {
            var attrs = object_1.keys(arg);
            return attrs.length == 1 && attrs[0] == "id";
        }
        return false;
    }
    exports.is_ref = is_ref;
},
/* core/properties.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var signaling_1 = require(130) /* ./signaling */;
    var logging_1 = require(134) /* ./logging */;
    var enums = tslib_1.__importStar(require(135) /* ./enums */);
    var types_1 = require(139) /* ./types */;
    var array_1 = require(124) /* ./util/array */;
    var arrayable_1 = require(127) /* ./util/arrayable */;
    var color_1 = require(137) /* ./util/color */;
    var types_2 = require(123) /* ./util/types */;
    var settings_1 = require(142) /* ./settings */;
    function valueToString(value) {
        try {
            return JSON.stringify(value);
        }
        catch (_a) {
            return value.toString();
        }
    }
    function isSpec(obj) {
        return types_2.isPlainObject(obj) &&
            ((obj.value === undefined ? 0 : 1) +
                (obj.field === undefined ? 0 : 1) +
                (obj.expr === undefined ? 0 : 1) == 1); // garbage JS XOR
    }
    exports.isSpec = isSpec;
    var Property = /** @class */ (function () {
        function Property(obj, attr, kind, default_value, initial_value, options) {
            if (options === void 0) {
                options = {};
            }
            var _a, _b;
            this.obj = obj;
            this.attr = attr;
            this.kind = kind;
            this.default_value = default_value;
            this._dirty = false;
            this.change = new signaling_1.Signal0(this.obj, "change");
            this.internal = (_a = options.internal) !== null && _a !== void 0 ? _a : false;
            this.optional = (_b = options.optional) !== null && _b !== void 0 ? _b : false;
            var attr_value;
            if (initial_value !== undefined) {
                attr_value = initial_value;
                this._dirty = true;
            }
            else {
                var value = this._default_override();
                if (value !== undefined)
                    attr_value = value;
                else if (default_value !== undefined)
                    attr_value = default_value(obj);
                else {
                    //throw new Error("no default")
                    attr_value = null; // XXX: nullable properties
                }
            }
            this._update(attr_value);
        }
        Object.defineProperty(Property.prototype, "is_value", {
            get: function () {
                return this.spec.value !== undefined;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Property.prototype, "syncable", {
            get: function () {
                return !this.internal;
            },
            enumerable: true,
            configurable: true
        });
        Property.prototype.get_value = function () {
            return this.spec.value;
        };
        Property.prototype.set_value = function (val) {
            this._update(val);
            this._dirty = true;
        };
        // abstract _intrinsic_default(): T
        Property.prototype._default_override = function () {
            return undefined;
        };
        Object.defineProperty(Property.prototype, "dirty", {
            get: function () {
                return this._dirty;
            },
            enumerable: true,
            configurable: true
        });
        //protected abstract _update(attr_value: T): void
        Property.prototype._update = function (attr_value) {
            if (attr_value != null) // XXX: non-nullalble types
                this.validate(attr_value);
            this.spec = { value: attr_value };
        };
        Property.prototype.toString = function () {
            /*${this.name}*/
            return "Prop(" + this.obj + "." + this.attr + ", spec: " + valueToString(this.spec) + ")";
        };
        // ----- customizable policies
        Property.prototype.normalize = function (values) {
            return values;
        };
        Property.prototype.validate = function (value) {
            if (!this.valid(value))
                throw new Error(this.obj.type + "." + this.attr + " given invalid value: " + valueToString(value));
        };
        Property.prototype.valid = function (value) {
            return this.kind.valid(value);
        };
        // ----- property accessors
        Property.prototype.value = function (do_spec_transform) {
            if (do_spec_transform === void 0) {
                do_spec_transform = true;
            }
            if (!this.is_value)
                throw new Error("attempted to retrieve property value for property without value specification");
            var ret = this.normalize([this.spec.value])[0];
            if (this.spec.transform != null && do_spec_transform)
                ret = this.spec.transform.compute(ret);
            return ret;
        };
        return Property;
    }());
    exports.Property = Property;
    Property.__name__ = "Property";
    //
    // Primitive Properties
    //
    var PrimitiveProperty = /** @class */ (function (_super) {
        tslib_1.__extends(PrimitiveProperty, _super);
        function PrimitiveProperty() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return PrimitiveProperty;
    }(Property));
    exports.PrimitiveProperty = PrimitiveProperty;
    PrimitiveProperty.__name__ = "PrimitiveProperty";
    var Any = /** @class */ (function (_super) {
        tslib_1.__extends(Any, _super);
        function Any() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return Any;
    }(Property));
    exports.Any = Any;
    Any.__name__ = "Any";
    var Array = /** @class */ (function (_super) {
        tslib_1.__extends(Array, _super);
        function Array() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Array.prototype.valid = function (value) {
            return types_2.isArray(value) || value instanceof Float32Array || value instanceof Float64Array;
        };
        return Array;
    }(Property));
    exports.Array = Array;
    Array.__name__ = "Array";
    var Boolean = /** @class */ (function (_super) {
        tslib_1.__extends(Boolean, _super);
        function Boolean() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Boolean.prototype.valid = function (value) {
            return types_2.isBoolean(value);
        };
        return Boolean;
    }(Property));
    exports.Boolean = Boolean;
    Boolean.__name__ = "Boolean";
    var Color = /** @class */ (function (_super) {
        tslib_1.__extends(Color, _super);
        function Color() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Color.prototype.valid = function (value) {
            return types_2.isString(value) && color_1.is_color(value);
        };
        return Color;
    }(Property));
    exports.Color = Color;
    Color.__name__ = "Color";
    var Instance = /** @class */ (function (_super) {
        tslib_1.__extends(Instance, _super);
        function Instance() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return Instance;
    }(Property));
    exports.Instance = Instance;
    Instance.__name__ = "Instance";
    var Number = /** @class */ (function (_super) {
        tslib_1.__extends(Number, _super);
        function Number() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Number.prototype.valid = function (value) {
            return types_2.isNumber(value);
        };
        return Number;
    }(Property));
    exports.Number = Number;
    Number.__name__ = "Number";
    var Int = /** @class */ (function (_super) {
        tslib_1.__extends(Int, _super);
        function Int() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Int.prototype.valid = function (value) {
            return types_2.isNumber(value) && (value | 0) == value;
        };
        return Int;
    }(Number));
    exports.Int = Int;
    Int.__name__ = "Int";
    var Angle = /** @class */ (function (_super) {
        tslib_1.__extends(Angle, _super);
        function Angle() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return Angle;
    }(Number));
    exports.Angle = Angle;
    Angle.__name__ = "Angle";
    var Percent = /** @class */ (function (_super) {
        tslib_1.__extends(Percent, _super);
        function Percent() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Percent.prototype.valid = function (value) {
            return types_2.isNumber(value) && 0 <= value && value <= 1.0;
        };
        return Percent;
    }(Number));
    exports.Percent = Percent;
    Percent.__name__ = "Percent";
    var String = /** @class */ (function (_super) {
        tslib_1.__extends(String, _super);
        function String() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        String.prototype.valid = function (value) {
            return types_2.isString(value);
        };
        return String;
    }(Property));
    exports.String = String;
    String.__name__ = "String";
    var NullString = /** @class */ (function (_super) {
        tslib_1.__extends(NullString, _super);
        function NullString() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        NullString.prototype.valid = function (value) {
            return value === null || types_2.isString(value);
        };
        return NullString;
    }(Property));
    exports.NullString = NullString;
    NullString.__name__ = "NullString";
    var FontSize = /** @class */ (function (_super) {
        tslib_1.__extends(FontSize, _super);
        function FontSize() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return FontSize;
    }(String));
    exports.FontSize = FontSize;
    FontSize.__name__ = "FontSize";
    var Font = /** @class */ (function (_super) {
        tslib_1.__extends(Font, _super);
        function Font() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Font.prototype._default_override = function () {
            return settings_1.settings.dev ? "Bokeh" : undefined;
        };
        return Font;
    }(String));
    exports.Font = Font;
    Font.__name__ = "Font";
    //
    // Enum properties
    //
    var EnumProperty = /** @class */ (function (_super) {
        tslib_1.__extends(EnumProperty, _super);
        function EnumProperty() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        EnumProperty.prototype.valid = function (value) {
            return types_2.isString(value) && array_1.includes(this.enum_values, value);
        };
        return EnumProperty;
    }(Property));
    exports.EnumProperty = EnumProperty;
    EnumProperty.__name__ = "EnumProperty";
    function Enum(values) {
        return /** @class */ (function (_super) {
            tslib_1.__extends(class_1, _super);
            function class_1() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            Object.defineProperty(class_1.prototype, "enum_values", {
                get: function () {
                    return tslib_1.__spread(values);
                },
                enumerable: true,
                configurable: true
            });
            return class_1;
        }(EnumProperty));
    }
    exports.Enum = Enum;
    var Direction = /** @class */ (function (_super) {
        tslib_1.__extends(Direction, _super);
        function Direction() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Object.defineProperty(Direction.prototype, "enum_values", {
            get: function () {
                return tslib_1.__spread(enums.Direction);
            },
            enumerable: true,
            configurable: true
        });
        Direction.prototype.normalize = function (values) {
            var result = new Uint8Array(values.length);
            for (var i = 0; i < values.length; i++) {
                switch (values[i]) {
                    case "clock":
                        result[i] = 0;
                        break;
                    case "anticlock":
                        result[i] = 1;
                        break;
                }
            }
            return result;
        };
        return Direction;
    }(EnumProperty));
    exports.Direction = Direction;
    Direction.__name__ = "Direction";
    /* TODO: remove this {{{ */
    exports.Anchor = Enum(enums.Anchor);
    exports.AngleUnits = Enum(enums.AngleUnits);
    exports.BoxOrigin = Enum(enums.BoxOrigin);
    exports.ButtonType = Enum(enums.ButtonType);
    exports.CalendarPosition = Enum(enums.CalendarPosition);
    exports.Dimension = Enum(enums.Dimension);
    exports.Dimensions = Enum(enums.Dimensions);
    exports.Distribution = Enum(enums.Distribution);
    exports.FontStyle = Enum(enums.FontStyle);
    exports.HatchPatternType = Enum(enums.HatchPatternType);
    exports.HTTPMethod = Enum(enums.HTTPMethod);
    exports.HexTileOrientation = Enum(enums.HexTileOrientation);
    exports.HoverMode = Enum(enums.HoverMode);
    exports.LatLon = Enum(enums.LatLon);
    exports.LegendClickPolicy = Enum(enums.LegendClickPolicy);
    exports.LegendLocation = Enum(enums.LegendLocation);
    exports.LineCap = Enum(enums.LineCap);
    exports.LineJoin = Enum(enums.LineJoin);
    exports.LinePolicy = Enum(enums.LinePolicy);
    exports.Location = Enum(enums.Location);
    exports.Logo = Enum(enums.Logo);
    exports.MarkerType = Enum(enums.MarkerType);
    exports.MutedPolicy = Enum(enums.MutedPolicy);
    exports.Orientation = Enum(enums.Orientation);
    exports.OutputBackend = Enum(enums.OutputBackend);
    exports.PaddingUnits = Enum(enums.PaddingUnits);
    exports.Place = Enum(enums.Place);
    exports.PointPolicy = Enum(enums.PointPolicy);
    exports.RadiusDimension = Enum(enums.RadiusDimension);
    exports.RenderLevel = Enum(enums.RenderLevel);
    exports.RenderMode = Enum(enums.RenderMode);
    exports.ResetPolicy = Enum(enums.ResetPolicy);
    exports.RoundingFunction = Enum(enums.RoundingFunction);
    exports.Side = Enum(enums.Side);
    exports.SizingMode = Enum(enums.SizingMode);
    exports.Sort = Enum(enums.Sort);
    exports.SpatialUnits = Enum(enums.SpatialUnits);
    exports.StartEnd = Enum(enums.StartEnd);
    exports.StepMode = Enum(enums.StepMode);
    exports.TapBehavior = Enum(enums.TapBehavior);
    exports.TextAlign = Enum(enums.TextAlign);
    exports.TextBaseline = Enum(enums.TextBaseline);
    exports.TextureRepetition = Enum(enums.TextureRepetition);
    exports.TickLabelOrientation = Enum(enums.TickLabelOrientation);
    exports.TooltipAttachment = Enum(enums.TooltipAttachment);
    exports.UpdateMode = Enum(enums.UpdateMode);
    exports.VerticalAlign = Enum(enums.VerticalAlign);
    /* }}} */
    //
    // DataSpec properties
    //
    var ScalarSpec = /** @class */ (function (_super) {
        tslib_1.__extends(ScalarSpec, _super);
        function ScalarSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ScalarSpec.prototype.get_value = function () {
            // XXX: allow obj.x = null; obj.x == null
            return this.spec.value === null ? null : this.spec;
        };
        ScalarSpec.prototype._update = function (attr_value) {
            if (isSpec(attr_value))
                this.spec = attr_value;
            else
                this.spec = { value: attr_value };
            if (this.spec.value != null)
                this.validate(this.spec.value);
        };
        return ScalarSpec;
    }(Property));
    exports.ScalarSpec = ScalarSpec;
    ScalarSpec.__name__ = "ScalarSpec";
    var AnyScalar = /** @class */ (function (_super) {
        tslib_1.__extends(AnyScalar, _super);
        function AnyScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return AnyScalar;
    }(ScalarSpec));
    exports.AnyScalar = AnyScalar;
    AnyScalar.__name__ = "AnyScalar";
    var ColorScalar = /** @class */ (function (_super) {
        tslib_1.__extends(ColorScalar, _super);
        function ColorScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return ColorScalar;
    }(ScalarSpec));
    exports.ColorScalar = ColorScalar;
    ColorScalar.__name__ = "ColorScalar";
    var NumberScalar = /** @class */ (function (_super) {
        tslib_1.__extends(NumberScalar, _super);
        function NumberScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return NumberScalar;
    }(ScalarSpec));
    exports.NumberScalar = NumberScalar;
    NumberScalar.__name__ = "NumberScalar";
    var StringScalar = /** @class */ (function (_super) {
        tslib_1.__extends(StringScalar, _super);
        function StringScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return StringScalar;
    }(ScalarSpec));
    exports.StringScalar = StringScalar;
    StringScalar.__name__ = "StringScalar";
    var NullStringScalar = /** @class */ (function (_super) {
        tslib_1.__extends(NullStringScalar, _super);
        function NullStringScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return NullStringScalar;
    }(ScalarSpec));
    exports.NullStringScalar = NullStringScalar;
    NullStringScalar.__name__ = "NullStringScalar";
    var ArrayScalar = /** @class */ (function (_super) {
        tslib_1.__extends(ArrayScalar, _super);
        function ArrayScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return ArrayScalar;
    }(ScalarSpec));
    exports.ArrayScalar = ArrayScalar;
    ArrayScalar.__name__ = "ArrayScalar";
    var LineJoinScalar = /** @class */ (function (_super) {
        tslib_1.__extends(LineJoinScalar, _super);
        function LineJoinScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return LineJoinScalar;
    }(ScalarSpec));
    exports.LineJoinScalar = LineJoinScalar;
    LineJoinScalar.__name__ = "LineJoinScalar";
    var LineCapScalar = /** @class */ (function (_super) {
        tslib_1.__extends(LineCapScalar, _super);
        function LineCapScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return LineCapScalar;
    }(ScalarSpec));
    exports.LineCapScalar = LineCapScalar;
    LineCapScalar.__name__ = "LineCapScalar";
    var FontSizeScalar = /** @class */ (function (_super) {
        tslib_1.__extends(FontSizeScalar, _super);
        function FontSizeScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return FontSizeScalar;
    }(ScalarSpec));
    exports.FontSizeScalar = FontSizeScalar;
    FontSizeScalar.__name__ = "FontSizeScalar";
    var FontStyleScalar = /** @class */ (function (_super) {
        tslib_1.__extends(FontStyleScalar, _super);
        function FontStyleScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return FontStyleScalar;
    }(ScalarSpec));
    exports.FontStyleScalar = FontStyleScalar;
    FontStyleScalar.__name__ = "FontStyleScalar";
    var TextAlignScalar = /** @class */ (function (_super) {
        tslib_1.__extends(TextAlignScalar, _super);
        function TextAlignScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return TextAlignScalar;
    }(ScalarSpec));
    exports.TextAlignScalar = TextAlignScalar;
    TextAlignScalar.__name__ = "TextAlignScalar";
    var TextBaselineScalar = /** @class */ (function (_super) {
        tslib_1.__extends(TextBaselineScalar, _super);
        function TextBaselineScalar() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return TextBaselineScalar;
    }(ScalarSpec));
    exports.TextBaselineScalar = TextBaselineScalar;
    TextBaselineScalar.__name__ = "TextBaselineScalar";
    var VectorSpec = /** @class */ (function (_super) {
        tslib_1.__extends(VectorSpec, _super);
        function VectorSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        VectorSpec.prototype.get_value = function () {
            // XXX: allow obj.x = null; obj.x == null
            return this.spec.value === null ? null : this.spec;
        };
        VectorSpec.prototype._update = function (attr_value) {
            if (isSpec(attr_value))
                this.spec = attr_value;
            else
                this.spec = { value: attr_value };
            if (this.spec.value != null)
                this.validate(this.spec.value);
        };
        VectorSpec.prototype.array = function (source) {
            var _a;
            var array;
            var length = (_a = source.get_length()) !== null && _a !== void 0 ? _a : 1;
            if (this.spec.field != null) {
                var column = source.get_column(this.spec.field);
                if (column != null)
                    array = this.normalize(column);
                else {
                    logging_1.logger.warn("attempted to retrieve property array for nonexistent field '" + this.spec.field + "'");
                    var missing = new types_1.NumberArray(length);
                    missing.fill(NaN);
                    array = missing;
                }
            }
            else if (this.spec.expr != null) {
                array = this.normalize(this.spec.expr.v_compute(source));
            }
            else {
                var value = this.value(false); // don't apply any spec transform
                if (types_2.isNumber(value)) {
                    var values = new types_1.NumberArray(length);
                    values.fill(value);
                    array = values;
                }
                else
                    array = array_1.repeat(value, length);
            }
            if (this.spec.transform != null)
                array = this.spec.transform.v_compute(array);
            return array;
        };
        return VectorSpec;
    }(Property));
    exports.VectorSpec = VectorSpec;
    VectorSpec.__name__ = "VectorSpec";
    var DataSpec = /** @class */ (function (_super) {
        tslib_1.__extends(DataSpec, _super);
        function DataSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return DataSpec;
    }(VectorSpec));
    exports.DataSpec = DataSpec;
    DataSpec.__name__ = "DataSpec";
    var UnitsSpec = /** @class */ (function (_super) {
        tslib_1.__extends(UnitsSpec, _super);
        function UnitsSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        UnitsSpec.prototype._update = function (attr_value) {
            _super.prototype._update.call(this, attr_value);
            if (this.spec.units == null)
                this.spec.units = this.default_units;
            var units = this.spec.units;
            if (!array_1.includes(this.valid_units, units))
                throw new Error("units must be one of " + this.valid_units.join(", ") + "; got: " + units);
        };
        Object.defineProperty(UnitsSpec.prototype, "units", {
            get: function () {
                return this.spec.units;
            },
            set: function (units) {
                this.spec.units = units;
            },
            enumerable: true,
            configurable: true
        });
        return UnitsSpec;
    }(VectorSpec));
    exports.UnitsSpec = UnitsSpec;
    UnitsSpec.__name__ = "UnitsSpec";
    var NumberUnitsSpec = /** @class */ (function (_super) {
        tslib_1.__extends(NumberUnitsSpec, _super);
        function NumberUnitsSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        NumberUnitsSpec.prototype.array = function (source) {
            return new types_1.NumberArray(_super.prototype.array.call(this, source));
        };
        return NumberUnitsSpec;
    }(UnitsSpec));
    exports.NumberUnitsSpec = NumberUnitsSpec;
    NumberUnitsSpec.__name__ = "NumberUnitsSpec";
    var BaseCoordinateSpec = /** @class */ (function (_super) {
        tslib_1.__extends(BaseCoordinateSpec, _super);
        function BaseCoordinateSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return BaseCoordinateSpec;
    }(DataSpec));
    exports.BaseCoordinateSpec = BaseCoordinateSpec;
    BaseCoordinateSpec.__name__ = "BaseCoordinateSpec";
    var CoordinateSpec = /** @class */ (function (_super) {
        tslib_1.__extends(CoordinateSpec, _super);
        function CoordinateSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return CoordinateSpec;
    }(BaseCoordinateSpec));
    exports.CoordinateSpec = CoordinateSpec;
    CoordinateSpec.__name__ = "CoordinateSpec";
    var CoordinateSeqSpec = /** @class */ (function (_super) {
        tslib_1.__extends(CoordinateSeqSpec, _super);
        function CoordinateSeqSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return CoordinateSeqSpec;
    }(BaseCoordinateSpec));
    exports.CoordinateSeqSpec = CoordinateSeqSpec;
    CoordinateSeqSpec.__name__ = "CoordinateSeqSpec";
    var CoordinateSeqSeqSeqSpec = /** @class */ (function (_super) {
        tslib_1.__extends(CoordinateSeqSeqSeqSpec, _super);
        function CoordinateSeqSeqSeqSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return CoordinateSeqSeqSeqSpec;
    }(BaseCoordinateSpec));
    exports.CoordinateSeqSeqSeqSpec = CoordinateSeqSeqSeqSpec;
    CoordinateSeqSeqSeqSpec.__name__ = "CoordinateSeqSeqSeqSpec";
    var XCoordinateSpec = /** @class */ (function (_super) {
        tslib_1.__extends(XCoordinateSpec, _super);
        function XCoordinateSpec() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.dimension = "x";
            return _this;
        }
        return XCoordinateSpec;
    }(CoordinateSpec));
    exports.XCoordinateSpec = XCoordinateSpec;
    XCoordinateSpec.__name__ = "XCoordinateSpec";
    var YCoordinateSpec = /** @class */ (function (_super) {
        tslib_1.__extends(YCoordinateSpec, _super);
        function YCoordinateSpec() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.dimension = "y";
            return _this;
        }
        return YCoordinateSpec;
    }(CoordinateSpec));
    exports.YCoordinateSpec = YCoordinateSpec;
    YCoordinateSpec.__name__ = "YCoordinateSpec";
    var XCoordinateSeqSpec = /** @class */ (function (_super) {
        tslib_1.__extends(XCoordinateSeqSpec, _super);
        function XCoordinateSeqSpec() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.dimension = "x";
            return _this;
        }
        return XCoordinateSeqSpec;
    }(CoordinateSeqSpec));
    exports.XCoordinateSeqSpec = XCoordinateSeqSpec;
    XCoordinateSeqSpec.__name__ = "XCoordinateSeqSpec";
    var YCoordinateSeqSpec = /** @class */ (function (_super) {
        tslib_1.__extends(YCoordinateSeqSpec, _super);
        function YCoordinateSeqSpec() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.dimension = "y";
            return _this;
        }
        return YCoordinateSeqSpec;
    }(CoordinateSeqSpec));
    exports.YCoordinateSeqSpec = YCoordinateSeqSpec;
    YCoordinateSeqSpec.__name__ = "YCoordinateSeqSpec";
    var XCoordinateSeqSeqSeqSpec = /** @class */ (function (_super) {
        tslib_1.__extends(XCoordinateSeqSeqSeqSpec, _super);
        function XCoordinateSeqSeqSeqSpec() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.dimension = "x";
            return _this;
        }
        return XCoordinateSeqSeqSeqSpec;
    }(CoordinateSeqSeqSeqSpec));
    exports.XCoordinateSeqSeqSeqSpec = XCoordinateSeqSeqSeqSpec;
    XCoordinateSeqSeqSeqSpec.__name__ = "XCoordinateSeqSeqSeqSpec";
    var YCoordinateSeqSeqSeqSpec = /** @class */ (function (_super) {
        tslib_1.__extends(YCoordinateSeqSeqSeqSpec, _super);
        function YCoordinateSeqSeqSeqSpec() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.dimension = "y";
            return _this;
        }
        return YCoordinateSeqSeqSeqSpec;
    }(CoordinateSeqSeqSeqSpec));
    exports.YCoordinateSeqSeqSeqSpec = YCoordinateSeqSeqSeqSpec;
    YCoordinateSeqSeqSeqSpec.__name__ = "YCoordinateSeqSeqSeqSpec";
    var AngleSpec = /** @class */ (function (_super) {
        tslib_1.__extends(AngleSpec, _super);
        function AngleSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Object.defineProperty(AngleSpec.prototype, "default_units", {
            get: function () { return "rad"; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AngleSpec.prototype, "valid_units", {
            get: function () { return tslib_1.__spread(enums.AngleUnits); },
            enumerable: true,
            configurable: true
        });
        AngleSpec.prototype.normalize = function (values) {
            if (this.spec.units == "deg")
                values = arrayable_1.map(values, function (x) { return x * Math.PI / 180.0; });
            values = arrayable_1.map(values, function (x) { return -x; });
            return _super.prototype.normalize.call(this, values);
        };
        return AngleSpec;
    }(NumberUnitsSpec));
    exports.AngleSpec = AngleSpec;
    AngleSpec.__name__ = "AngleSpec";
    var DistanceSpec = /** @class */ (function (_super) {
        tslib_1.__extends(DistanceSpec, _super);
        function DistanceSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Object.defineProperty(DistanceSpec.prototype, "default_units", {
            get: function () { return "data"; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DistanceSpec.prototype, "valid_units", {
            get: function () { return tslib_1.__spread(enums.SpatialUnits); },
            enumerable: true,
            configurable: true
        });
        return DistanceSpec;
    }(NumberUnitsSpec));
    exports.DistanceSpec = DistanceSpec;
    DistanceSpec.__name__ = "DistanceSpec";
    var BooleanSpec = /** @class */ (function (_super) {
        tslib_1.__extends(BooleanSpec, _super);
        function BooleanSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BooleanSpec.prototype.array = function (source) {
            return new Uint8Array(_super.prototype.array.call(this, source));
        };
        return BooleanSpec;
    }(DataSpec));
    exports.BooleanSpec = BooleanSpec;
    BooleanSpec.__name__ = "BooleanSpec";
    var NumberSpec = /** @class */ (function (_super) {
        tslib_1.__extends(NumberSpec, _super);
        function NumberSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        NumberSpec.prototype.array = function (source) {
            return new types_1.NumberArray(_super.prototype.array.call(this, source));
        };
        return NumberSpec;
    }(DataSpec));
    exports.NumberSpec = NumberSpec;
    NumberSpec.__name__ = "NumberSpec";
    var ColorSpec = /** @class */ (function (_super) {
        tslib_1.__extends(ColorSpec, _super);
        function ColorSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ColorSpec.prototype.array = function (source) {
            var colors = _super.prototype.array.call(this, source);
            var n = colors.length;
            var array = new types_1.ColorArray(n);
            for (var i = 0; i < n; i++) {
                var color = colors[i];
                if (types_2.isNumber(color))
                    array[i] = color;
                else {
                    var rgba = color_1.color2rgba(color);
                    array[i] = color_1.encode_rgba(rgba);
                }
            }
            return array;
        };
        return ColorSpec;
    }(DataSpec));
    exports.ColorSpec = ColorSpec;
    ColorSpec.__name__ = "ColorSpec";
    var FontSizeSpec = /** @class */ (function (_super) {
        tslib_1.__extends(FontSizeSpec, _super);
        function FontSizeSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return FontSizeSpec;
    }(DataSpec));
    exports.FontSizeSpec = FontSizeSpec;
    FontSizeSpec.__name__ = "FontSizeSpec";
    var MarkerSpec = /** @class */ (function (_super) {
        tslib_1.__extends(MarkerSpec, _super);
        function MarkerSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return MarkerSpec;
    }(DataSpec));
    exports.MarkerSpec = MarkerSpec;
    MarkerSpec.__name__ = "MarkerSpec";
    var StringSpec = /** @class */ (function (_super) {
        tslib_1.__extends(StringSpec, _super);
        function StringSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return StringSpec;
    }(DataSpec));
    exports.StringSpec = StringSpec;
    StringSpec.__name__ = "StringSpec";
    var NullStringSpec = /** @class */ (function (_super) {
        tslib_1.__extends(NullStringSpec, _super);
        function NullStringSpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return NullStringSpec;
    }(DataSpec));
    exports.NullStringSpec = NullStringSpec;
    NullStringSpec.__name__ = "NullStringSpec";
    var NDArraySpec = /** @class */ (function (_super) {
        tslib_1.__extends(NDArraySpec, _super);
        function NDArraySpec() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return NDArraySpec;
    }(DataSpec));
    exports.NDArraySpec = NDArraySpec;
    NDArraySpec.__name__ = "NDArraySpec";
},
/* core/logging.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    // This is based on https://github.com/pimterry/loglevel
    var types_1 = require(123) /* ./util/types */;
    var object_1 = require(128) /* ./util/object */;
    var _loggers = {};
    var LogLevel = /** @class */ (function () {
        function LogLevel(name, level) {
            this.name = name;
            this.level = level;
        }
        return LogLevel;
    }());
    exports.LogLevel = LogLevel;
    LogLevel.__name__ = "LogLevel";
    var Logger = /** @class */ (function () {
        function Logger(name, level) {
            if (level === void 0) {
                level = Logger.INFO;
            }
            this._name = name;
            this.set_level(level);
        }
        Object.defineProperty(Logger, "levels", {
            get: function () {
                return Object.keys(Logger.log_levels);
            },
            enumerable: true,
            configurable: true
        });
        Logger.get = function (name, level) {
            if (level === void 0) {
                level = Logger.INFO;
            }
            if (name.length > 0) {
                var logger_1 = _loggers[name];
                if (logger_1 == null)
                    _loggers[name] = logger_1 = new Logger(name, level);
                return logger_1;
            }
            else
                throw new TypeError("Logger.get() expects a non-empty string name and an optional log-level");
        };
        Object.defineProperty(Logger.prototype, "level", {
            get: function () {
                return this.get_level();
            },
            enumerable: true,
            configurable: true
        });
        Logger.prototype.get_level = function () {
            return this._log_level;
        };
        Logger.prototype.set_level = function (log_level) {
            var e_1, _a;
            if (log_level instanceof LogLevel)
                this._log_level = log_level;
            else if (types_1.isString(log_level) && Logger.log_levels[log_level] != null)
                this._log_level = Logger.log_levels[log_level];
            else
                throw new Error("Logger.set_level() expects a log-level object or a string name of a log-level");
            var logger_name = "[" + this._name + "]";
            try {
                for (var _b = tslib_1.__values(object_1.entries(Logger.log_levels)), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var _d = tslib_1.__read(_c.value, 2), name = _d[0], log_level_1 = _d[1];
                    if (log_level_1.level < this._log_level.level || this._log_level.level === Logger.OFF.level)
                        this[name] = function () { };
                    else
                        this[name] = _method_factory(name, logger_name);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        Logger.prototype.trace = function () {
            var _args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                _args[_i] = arguments[_i];
            }
        };
        Logger.prototype.debug = function () {
            var _args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                _args[_i] = arguments[_i];
            }
        };
        Logger.prototype.info = function () {
            var _args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                _args[_i] = arguments[_i];
            }
        };
        Logger.prototype.warn = function () {
            var _args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                _args[_i] = arguments[_i];
            }
        };
        Logger.prototype.error = function () {
            var _args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                _args[_i] = arguments[_i];
            }
        };
        return Logger;
    }());
    exports.Logger = Logger;
    Logger.__name__ = "Logger";
    Logger.TRACE = new LogLevel("trace", 0);
    Logger.DEBUG = new LogLevel("debug", 1);
    Logger.INFO = new LogLevel("info", 2);
    Logger.WARN = new LogLevel("warn", 6);
    Logger.ERROR = new LogLevel("error", 7);
    Logger.FATAL = new LogLevel("fatal", 8);
    Logger.OFF = new LogLevel("off", 9);
    Logger.log_levels = {
        trace: Logger.TRACE,
        debug: Logger.DEBUG,
        info: Logger.INFO,
        warn: Logger.WARN,
        error: Logger.ERROR,
        fatal: Logger.FATAL,
        off: Logger.OFF,
    };
    function _method_factory(method_name, logger_name) {
        if (console[method_name] != null)
            return console[method_name].bind(console, logger_name);
        else if (console.log != null)
            return console.log.bind(console, logger_name);
        else
            return function () { };
    }
    exports.logger = Logger.get("bokeh");
    function set_log_level(level) {
        var previous_level = exports.logger.level;
        if (types_1.isString(level) && Logger.log_levels[level] == null) {
            console.log("[bokeh] unrecognized logging level '" + level + "' passed to Bokeh.set_log_level(), ignoring");
            console.log("[bokeh] valid log levels are: " + Logger.levels.join(', '));
        }
        else {
            console.log("[bokeh] setting log level to: '" + (types_1.isString(level) ? level : level.level) + "'");
            exports.logger.set_level(level);
        }
        return previous_level;
    }
    exports.set_log_level = set_log_level;
    function with_log_level(level, fn) {
        var original = set_log_level(level);
        try {
            fn();
        }
        finally {
            set_log_level(original);
        }
    }
    exports.with_log_level = with_log_level;
},
/* core/enums.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var kinds_1 = require(136) /* ./kinds */;
    exports.Align = kinds_1.Enum("start", "center", "end");
    exports.Anchor = kinds_1.Enum("top_left", "top_center", "top_right", "center_left", "center", "center_right", "bottom_left", "bottom_center", "bottom_right");
    exports.AngleUnits = kinds_1.Enum("deg", "rad");
    exports.BoxOrigin = kinds_1.Enum("corner", "center");
    exports.ButtonType = kinds_1.Enum("default", "primary", "success", "warning", "danger");
    exports.CalendarPosition = kinds_1.Enum("auto", "above", "below");
    exports.Dimension = kinds_1.Enum("width", "height");
    exports.Dimensions = kinds_1.Enum("width", "height", "both");
    exports.Direction = kinds_1.Enum("clock", "anticlock");
    exports.Distribution = kinds_1.Enum("uniform", "normal");
    exports.FontStyle = kinds_1.Enum("normal", "italic", "bold", "bold italic");
    exports.HatchPatternType = kinds_1.Enum('blank', 'dot', 'ring', 'horizontal_line', 'vertical_line', 'cross', 'horizontal_dash', 'vertical_dash', 'spiral', 'right_diagonal_line', 'left_diagonal_line', 'diagonal_cross', 'right_diagonal_dash', 'left_diagonal_dash', 'horizontal_wave', 'vertical_wave', 'criss_cross', ' ', '.', 'o', '-', '|', '+', '"', ':', '@', '/', '\\', 'x', ',', '`', 'v', '>', '*');
    exports.HTTPMethod = kinds_1.Enum("POST", "GET");
    exports.HexTileOrientation = kinds_1.Enum("pointytop", "flattop");
    exports.HoverMode = kinds_1.Enum("mouse", "hline", "vline");
    exports.LatLon = kinds_1.Enum("lat", "lon");
    exports.LegendClickPolicy = kinds_1.Enum("none", "hide", "mute");
    exports.LegendLocation = exports.Anchor;
    exports.LineCap = kinds_1.Enum("butt", "round", "square");
    exports.LineJoin = kinds_1.Enum("miter", "round", "bevel");
    exports.LinePolicy = kinds_1.Enum("prev", "next", "nearest", "interp", "none");
    exports.Location = kinds_1.Enum("above", "below", "left", "right");
    exports.Logo = kinds_1.Enum("normal", "grey");
    exports.MarkerType = kinds_1.Enum("asterisk", "circle", "circle_cross", "circle_dot", "circle_x", "circle_y", "cross", "dash", "diamond", "diamond_cross", "diamond_dot", "dot", "hex", "hex_dot", "inverted_triangle", "plus", "square", "square_cross", "square_dot", "square_pin", "square_x", "triangle", "triangle_dot", "triangle_pin", "x", "y");
    exports.MutedPolicy = kinds_1.Enum("show", "ignore");
    exports.Orientation = kinds_1.Enum("vertical", "horizontal");
    exports.OutputBackend = kinds_1.Enum("canvas", "svg", "webgl");
    exports.PaddingUnits = kinds_1.Enum("percent", "absolute");
    exports.Place = kinds_1.Enum("above", "below", "left", "right", "center");
    exports.PointPolicy = kinds_1.Enum("snap_to_data", "follow_mouse", "none");
    exports.RadiusDimension = kinds_1.Enum("x", "y", "max", "min");
    exports.RenderLevel = kinds_1.Enum("image", "underlay", "glyph", "guide", "annotation", "overlay");
    exports.RenderMode = kinds_1.Enum("canvas", "css");
    exports.ResetPolicy = kinds_1.Enum("standard", "event_only");
    exports.RoundingFunction = kinds_1.Enum("round", "nearest", "floor", "rounddown", "ceil", "roundup");
    exports.SelectionMode = kinds_1.Enum("replace", "append", "intersect", "subtract");
    exports.Side = kinds_1.Enum("above", "below", "left", "right");
    exports.SizingMode = kinds_1.Enum("stretch_width", "stretch_height", "stretch_both", "scale_width", "scale_height", "scale_both", "fixed");
    exports.Sort = kinds_1.Enum("ascending", "descending");
    exports.SpatialUnits = kinds_1.Enum("screen", "data");
    exports.StartEnd = kinds_1.Enum("start", "end");
    exports.StepMode = kinds_1.Enum("after", "before", "center");
    exports.TapBehavior = kinds_1.Enum("select", "inspect");
    exports.TextAlign = kinds_1.Enum("left", "right", "center");
    exports.TextBaseline = kinds_1.Enum("top", "middle", "bottom", "alphabetic", "hanging", "ideographic");
    exports.TextureRepetition = kinds_1.Enum("repeat", "repeat_x", "repeat_y", "no_repeat");
    exports.TickLabelOrientation = kinds_1.Enum("vertical", "horizontal", "parallel", "normal");
    exports.TooltipAttachment = kinds_1.Enum("horizontal", "vertical", "left", "right", "above", "below");
    exports.UpdateMode = kinds_1.Enum("replace", "append");
    exports.VerticalAlign = kinds_1.Enum("top", "middle", "bottom");
},
/* core/kinds.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var tp = tslib_1.__importStar(require(123) /* ./util/types */);
    var color_1 = require(137) /* ./util/color */;
    var Kind = /** @class */ (function () {
        function Kind() {
        }
        return Kind;
    }());
    exports.Kind = Kind;
    Kind.__name__ = "Kind";
    (function (Kinds) {
        var Any = /** @class */ (function (_super) {
            tslib_1.__extends(Any, _super);
            function Any() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            Any.prototype.valid = function (_value) {
                return true;
            };
            return Any;
        }(Kind));
        Any.__name__ = "Any";
        Kinds.Any = Any;
        var Unknown = /** @class */ (function (_super) {
            tslib_1.__extends(Unknown, _super);
            function Unknown() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            Unknown.prototype.valid = function (_value) {
                return true;
            };
            return Unknown;
        }(Kind));
        Unknown.__name__ = "Unknown";
        Kinds.Unknown = Unknown;
        var Boolean = /** @class */ (function (_super) {
            tslib_1.__extends(Boolean, _super);
            function Boolean() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            Boolean.prototype.valid = function (value) {
                return tp.isBoolean(value);
            };
            return Boolean;
        }(Kind));
        Boolean.__name__ = "Boolean";
        Kinds.Boolean = Boolean;
        var Ref = /** @class */ (function (_super) {
            tslib_1.__extends(Ref, _super);
            function Ref(obj_type) {
                var _this = _super.call(this) || this;
                _this.obj_type = obj_type;
                return _this;
            }
            Ref.prototype.valid = function (value) {
                // XXX: disable validation for now, because object graph
                // initialization depends on this.
                value;
                return true;
                //return value instanceof this.obj_type
            };
            return Ref;
        }(Kind));
        Ref.__name__ = "Ref";
        Kinds.Ref = Ref;
        var Number = /** @class */ (function (_super) {
            tslib_1.__extends(Number, _super);
            function Number() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            Number.prototype.valid = function (value) {
                return tp.isNumber(value);
            };
            return Number;
        }(Kind));
        Number.__name__ = "Number";
        Kinds.Number = Number;
        var Int = /** @class */ (function (_super) {
            tslib_1.__extends(Int, _super);
            function Int() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            Int.prototype.valid = function (value) {
                return _super.prototype.valid.call(this, value) && tp.isInteger(value);
            };
            return Int;
        }(Number));
        Int.__name__ = "Int";
        Kinds.Int = Int;
        var Or = /** @class */ (function (_super) {
            tslib_1.__extends(Or, _super);
            function Or(types) {
                var _this = _super.call(this) || this;
                _this.types = types;
                _this.types = types;
                return _this;
            }
            Or.prototype.valid = function (value) {
                return this.types.some(function (type) { return type.valid(value); });
            };
            return Or;
        }(Kind));
        Or.__name__ = "Or";
        Kinds.Or = Or;
        var Tuple = /** @class */ (function (_super) {
            tslib_1.__extends(Tuple, _super);
            function Tuple(types) {
                var _this = _super.call(this) || this;
                _this.types = types;
                _this.types = types;
                return _this;
            }
            Tuple.prototype.valid = function (value) {
                if (!tp.isArray(value))
                    return false;
                for (var i = 0; i < this.types.length; i++) {
                    var type = this.types[i];
                    var item = value[i];
                    if (!type.valid(item))
                        return false;
                }
                return true;
            };
            return Tuple;
        }(Kind));
        Tuple.__name__ = "Tuple";
        Kinds.Tuple = Tuple;
        var Array = /** @class */ (function (_super) {
            tslib_1.__extends(Array, _super);
            function Array(item_type) {
                var _this = _super.call(this) || this;
                _this.item_type = item_type;
                return _this;
            }
            Array.prototype.valid = function (value) {
                var _this = this;
                return tp.isArray(value) && value.every(function (item) { return _this.item_type.valid(item); });
            };
            return Array;
        }(Kind));
        Array.__name__ = "Array";
        Kinds.Array = Array;
        var Null = /** @class */ (function (_super) {
            tslib_1.__extends(Null, _super);
            function Null() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            Null.prototype.valid = function (value) {
                return value === null;
            };
            return Null;
        }(Kind));
        Null.__name__ = "Null";
        Kinds.Null = Null;
        var Nullable = /** @class */ (function (_super) {
            tslib_1.__extends(Nullable, _super);
            function Nullable(base_type) {
                var _this = _super.call(this) || this;
                _this.base_type = base_type;
                return _this;
            }
            Nullable.prototype.valid = function (value) {
                return value === null || this.base_type.valid(value);
            };
            return Nullable;
        }(Kind));
        Nullable.__name__ = "Nullable";
        Kinds.Nullable = Nullable;
        var String = /** @class */ (function (_super) {
            tslib_1.__extends(String, _super);
            function String() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            String.prototype.valid = function (value) {
                return tp.isString(value);
            };
            return String;
        }(Kind));
        String.__name__ = "String";
        Kinds.String = String;
        var Enum = /** @class */ (function (_super) {
            tslib_1.__extends(Enum, _super);
            function Enum(values) {
                var _this = _super.call(this) || this;
                _this.values = new Set(values);
                return _this;
            }
            Enum.prototype.valid = function (value) {
                return this.values.has(value);
            };
            Enum.prototype[Symbol.iterator] = function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [5 /*yield**/, tslib_1.__values(this.values)];
                        case 1:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            };
            return Enum;
        }(Kind));
        Enum.__name__ = "Enum";
        Kinds.Enum = Enum;
        var Struct = /** @class */ (function (_super) {
            tslib_1.__extends(Struct, _super);
            function Struct(item_type) {
                var _this = _super.call(this) || this;
                _this.item_type = item_type;
                return _this;
            }
            Struct.prototype.valid = function (value) {
                if (!tp.isPlainObject(value))
                    return false;
                for (var key in value) {
                    if (value.hasOwnProperty(key)) {
                        var item = value[key];
                        if (!this.item_type.valid(item))
                            return false;
                    }
                }
                return true;
            };
            return Struct;
        }(Kind));
        Struct.__name__ = "Struct";
        Kinds.Struct = Struct;
        var Dict = /** @class */ (function (_super) {
            tslib_1.__extends(Dict, _super);
            function Dict(key_type, item_type) {
                var _this = _super.call(this) || this;
                _this.key_type = key_type;
                _this.item_type = item_type;
                return _this;
            }
            Dict.prototype.valid = function (value) {
                var e_1, _a;
                if (!(value instanceof Map))
                    return false;
                try {
                    for (var _b = tslib_1.__values(value.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var _d = tslib_1.__read(_c.value, 2), key = _d[0], item = _d[1];
                        if (!(this.key_type.valid(key) && this.item_type.valid(item)))
                            return false;
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return))
                            _a.call(_b);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
                return true;
            };
            return Dict;
        }(Kind));
        Dict.__name__ = "Dict";
        Kinds.Dict = Dict;
        var Color = /** @class */ (function (_super) {
            tslib_1.__extends(Color, _super);
            function Color() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            Color.prototype.valid = function (value) {
                return tp.isString(value) && color_1.is_color(value);
            };
            return Color;
        }(Kind));
        Color.__name__ = "Color";
        Kinds.Color = Color;
        var Percent = /** @class */ (function (_super) {
            tslib_1.__extends(Percent, _super);
            function Percent() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            Percent.prototype.valid = function (value) {
                return _super.prototype.valid.call(this, value) && 0 <= value && value <= 1.0;
            };
            return Percent;
        }(Number));
        Percent.__name__ = "Percent";
        Kinds.Percent = Percent;
    })(exports.Kinds || (exports.Kinds = {}));
    exports.Any = new exports.Kinds.Any();
    exports.Unknown = new exports.Kinds.Unknown();
    exports.Boolean = new exports.Kinds.Boolean();
    exports.Number = new exports.Kinds.Number();
    exports.Int = new exports.Kinds.Int();
    exports.String = new exports.Kinds.String();
    exports.Null = new exports.Kinds.Null();
    exports.Nullable = function (base_type) { return new exports.Kinds.Nullable(base_type); };
    exports.Or = function () {
        var types = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            types[_i] = arguments[_i];
        }
        return new exports.Kinds.Or(types);
    };
    exports.Tuple = function () {
        var types = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            types[_i] = arguments[_i];
        }
        return new exports.Kinds.Tuple(types);
    };
    exports.Array = function (item_type) { return new exports.Kinds.Array(item_type); };
    exports.Struct = function (item_type) { return new exports.Kinds.Struct(item_type); };
    exports.Dict = function (key_type, item_type) { return new exports.Kinds.Dict(key_type, item_type); };
    exports.Enum = function () {
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        return new exports.Kinds.Enum(values);
    };
    exports.Ref = function (obj_type) { return new exports.Kinds.Ref(obj_type); };
    exports.Percent = new exports.Kinds.Percent();
    exports.Color = new exports.Kinds.Color();
    exports.Auto = exports.Enum("auto");
    exports.FontSize = exports.String;
    exports.Font = exports.String;
    exports.Angle = exports.Number;
},
/* core/util/color.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var svg_colors_1 = require(138) /* ./svg_colors */;
    var array_1 = require(124) /* ./array */;
    function is_color(value) {
        return svg_colors_1.is_svg_color(value.toLowerCase()) || value.substring(0, 1) == "#" || valid_rgb(value);
    }
    exports.is_color = is_color;
    function _component2hex(v) {
        var h = Number(v).toString(16);
        return h.length == 1 ? "0" + h : h;
    }
    function rgb2hex(r, g, b) {
        var R = _component2hex(r & 0xFF);
        var G = _component2hex(g & 0xFF);
        var B = _component2hex(b & 0xFF);
        return "#" + R + G + B;
    }
    exports.rgb2hex = rgb2hex;
    function color2hex(color) {
        color = color + '';
        if (color.indexOf('#') == 0)
            return color;
        else if (svg_colors_1.is_svg_color(color))
            return svg_colors_1.svg_colors[color];
        else if (color.indexOf('rgb') == 0) {
            var rgb = color.replace(/^rgba?\(|\s+|\)$/g, '').split(',');
            var hex = rgb.slice(0, 3).map(_component2hex).join('');
            if (rgb.length == 4)
                hex += _component2hex(Math.floor(parseFloat(rgb[3]) * 255));
            return "#" + hex.slice(0, 8); // can also be rgba
        }
        else
            return color;
    }
    exports.color2hex = color2hex;
    function encode_rgba(_a) {
        var _b = tslib_1.__read(_a, 4), r = _b[0], g = _b[1], b = _b[2], a = _b[3];
        return (r * 255 | 0) << 24 | (g * 255 | 0) << 16 | (b * 255 | 0) << 8 | (a * 255 | 0);
    }
    exports.encode_rgba = encode_rgba;
    function decode_rgba(rgba) {
        var r = ((rgba >> 24) & 0xff) / 255;
        var g = ((rgba >> 16) & 0xff) / 255;
        var b = ((rgba >> 8) & 0xff) / 255;
        var a = ((rgba >> 0) & 0xff) / 255;
        return [r, g, b, a];
    }
    exports.decode_rgba = decode_rgba;
    function color2rgba(color, alpha) {
        if (alpha === void 0) {
            alpha = 1.0;
        }
        if (!color) // NaN, null, '', etc.
            return [0, 0, 0, 0]; // transparent
        // Convert to hex and then to clean version of 6 or 8 chars
        var hex = color2hex(color);
        hex = hex.replace(/ |#/g, '');
        if (hex.length <= 4) {
            hex = hex.replace(/(.)/g, '$1$1');
        }
        // Convert pairs to numbers
        var rgba = hex.match(/../g).map(function (i) { return parseInt(i, 16) / 255; });
        // Ensure correct length, add alpha if necessary
        while (rgba.length < 3)
            rgba.push(0);
        if (rgba.length < 4)
            rgba.push(alpha);
        return rgba.slice(0, 4);
    }
    exports.color2rgba = color2rgba;
    function valid_rgb(value) {
        var params;
        switch (value.substring(0, 4)) {
            case "rgba": {
                params = { start: "rgba(", len: 4, alpha: true };
                break;
            }
            case "rgb(": {
                params = { start: "rgb(", len: 3, alpha: false };
                break;
            }
            default:
                return false;
        }
        // if '.' and then ',' found, we know decimals are used on rgb
        if (new RegExp(".*?(\\.).*(,)").test(value))
            return false;
        // throw new Error(`color expects integers for rgb in rgb/rgba tuple, received ${value}`)
        // extract the numerical values from inside parens
        var contents = value.replace(params.start, "").replace(")", "").split(',').map(parseFloat);
        // check length of array based on rgb/rgba
        if (contents.length != params.len)
            return false;
        // throw new Error(`color expects rgba ${params.len}-tuple, received ${value}`)
        // check for valid numerical values for rgba
        if (params.alpha && !(0 <= contents[3] && contents[3] <= 1))
            return false;
        // throw new Error("color expects rgba 4-tuple to have alpha value between 0 and 1")
        if (array_1.includes(contents.slice(0, 3).map(function (rgb) { return 0 <= rgb && rgb <= 255; }), false))
            return false;
        // throw new Error("color expects rgb to have value between 0 and 255")
        return true;
    }
    exports.valid_rgb = valid_rgb;
},
/* core/util/svg_colors.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.svg_colors = {
        indianred: "#CD5C5C",
        lightcoral: "#F08080",
        salmon: "#FA8072",
        darksalmon: "#E9967A",
        lightsalmon: "#FFA07A",
        crimson: "#DC143C",
        red: "#FF0000",
        firebrick: "#B22222",
        darkred: "#8B0000",
        pink: "#FFC0CB",
        lightpink: "#FFB6C1",
        hotpink: "#FF69B4",
        deeppink: "#FF1493",
        mediumvioletred: "#C71585",
        palevioletred: "#DB7093",
        coral: "#FF7F50",
        tomato: "#FF6347",
        orangered: "#FF4500",
        darkorange: "#FF8C00",
        orange: "#FFA500",
        gold: "#FFD700",
        yellow: "#FFFF00",
        lightyellow: "#FFFFE0",
        lemonchiffon: "#FFFACD",
        lightgoldenrodyellow: "#FAFAD2",
        papayawhip: "#FFEFD5",
        moccasin: "#FFE4B5",
        peachpuff: "#FFDAB9",
        palegoldenrod: "#EEE8AA",
        khaki: "#F0E68C",
        darkkhaki: "#BDB76B",
        lavender: "#E6E6FA",
        thistle: "#D8BFD8",
        plum: "#DDA0DD",
        violet: "#EE82EE",
        orchid: "#DA70D6",
        fuchsia: "#FF00FF",
        magenta: "#FF00FF",
        mediumorchid: "#BA55D3",
        mediumpurple: "#9370DB",
        blueviolet: "#8A2BE2",
        darkviolet: "#9400D3",
        darkorchid: "#9932CC",
        darkmagenta: "#8B008B",
        purple: "#800080",
        indigo: "#4B0082",
        slateblue: "#6A5ACD",
        darkslateblue: "#483D8B",
        mediumslateblue: "#7B68EE",
        greenyellow: "#ADFF2F",
        chartreuse: "#7FFF00",
        lawngreen: "#7CFC00",
        lime: "#00FF00",
        limegreen: "#32CD32",
        palegreen: "#98FB98",
        lightgreen: "#90EE90",
        mediumspringgreen: "#00FA9A",
        springgreen: "#00FF7F",
        mediumseagreen: "#3CB371",
        seagreen: "#2E8B57",
        forestgreen: "#228B22",
        green: "#008000",
        darkgreen: "#006400",
        yellowgreen: "#9ACD32",
        olivedrab: "#6B8E23",
        olive: "#808000",
        darkolivegreen: "#556B2F",
        mediumaquamarine: "#66CDAA",
        darkseagreen: "#8FBC8F",
        lightseagreen: "#20B2AA",
        darkcyan: "#008B8B",
        teal: "#008080",
        aqua: "#00FFFF",
        cyan: "#00FFFF",
        lightcyan: "#E0FFFF",
        paleturquoise: "#AFEEEE",
        aquamarine: "#7FFFD4",
        turquoise: "#40E0D0",
        mediumturquoise: "#48D1CC",
        darkturquoise: "#00CED1",
        cadetblue: "#5F9EA0",
        steelblue: "#4682B4",
        lightsteelblue: "#B0C4DE",
        powderblue: "#B0E0E6",
        lightblue: "#ADD8E6",
        skyblue: "#87CEEB",
        lightskyblue: "#87CEFA",
        deepskyblue: "#00BFFF",
        dodgerblue: "#1E90FF",
        cornflowerblue: "#6495ED",
        royalblue: "#4169E1",
        blue: "#0000FF",
        mediumblue: "#0000CD",
        darkblue: "#00008B",
        navy: "#000080",
        midnightblue: "#191970",
        cornsilk: "#FFF8DC",
        blanchedalmond: "#FFEBCD",
        bisque: "#FFE4C4",
        navajowhite: "#FFDEAD",
        wheat: "#F5DEB3",
        burlywood: "#DEB887",
        tan: "#D2B48C",
        rosybrown: "#BC8F8F",
        sandybrown: "#F4A460",
        goldenrod: "#DAA520",
        darkgoldenrod: "#B8860B",
        peru: "#CD853F",
        chocolate: "#D2691E",
        saddlebrown: "#8B4513",
        sienna: "#A0522D",
        brown: "#A52A2A",
        maroon: "#800000",
        white: "#FFFFFF",
        snow: "#FFFAFA",
        honeydew: "#F0FFF0",
        mintcream: "#F5FFFA",
        azure: "#F0FFFF",
        aliceblue: "#F0F8FF",
        ghostwhite: "#F8F8FF",
        whitesmoke: "#F5F5F5",
        seashell: "#FFF5EE",
        beige: "#F5F5DC",
        oldlace: "#FDF5E6",
        floralwhite: "#FFFAF0",
        ivory: "#FFFFF0",
        antiquewhite: "#FAEBD7",
        linen: "#FAF0E6",
        lavenderblush: "#FFF0F5",
        mistyrose: "#FFE4E1",
        gainsboro: "#DCDCDC",
        lightgray: "#D3D3D3",
        lightgrey: "#D3D3D3",
        silver: "#C0C0C0",
        darkgray: "#A9A9A9",
        darkgrey: "#A9A9A9",
        gray: "#808080",
        grey: "#808080",
        dimgray: "#696969",
        dimgrey: "#696969",
        lightslategray: "#778899",
        lightslategrey: "#778899",
        slategray: "#708090",
        slategrey: "#708090",
        darkslategray: "#2F4F4F",
        darkslategrey: "#2F4F4F",
        black: "#000000",
    };
    function is_svg_color(color) {
        return color in exports.svg_colors;
    }
    exports.is_svg_color = is_svg_color;
},
/* core/types.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    exports.NumberArray = Float32Array;
    exports.ColorArray = Uint32Array;
    var eq_1 = require(140) /* ./util/eq */;
    var RaggedArray = /** @class */ (function () {
        function RaggedArray(offsets, array) {
            this.offsets = offsets;
            this.array = array;
        }
        RaggedArray.prototype[eq_1.equals] = function (that, cmp) {
            return cmp.arrays(this.offsets, that.offsets) && cmp.arrays(this.array, that.array);
        };
        Object.defineProperty(RaggedArray.prototype, "length", {
            get: function () {
                return this.offsets.length;
            },
            enumerable: true,
            configurable: true
        });
        RaggedArray.prototype.clone = function () {
            return new RaggedArray(new Uint32Array(this.offsets), new exports.NumberArray(this.array));
        };
        RaggedArray.from = function (items) {
            var n = items.length;
            var offsets = new Uint32Array(n);
            var offset = 0;
            for (var i = 0; i < n; i++) {
                var length = items[i].length;
                offsets[i] = offset;
                offset += length;
            }
            var array = new exports.NumberArray(offset);
            for (var i = 0; i < n; i++) {
                array.set(items[i], offsets[i]);
            }
            return new RaggedArray(offsets, array);
        };
        RaggedArray.prototype[Symbol.iterator] = function () {
            var _a, offsets, length, i;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this, offsets = _a.offsets, length = _a.length;
                        i = 0;
                        _b.label = 1;
                    case 1:
                        if (!(i < length))
                            return [3 /*break*/, 4];
                        return [4 /*yield*/, this.array.subarray(offsets[i], offsets[i + 1])];
                    case 2:
                        _b.sent();
                        _b.label = 3;
                    case 3:
                        i++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/];
                }
            });
        };
        RaggedArray.prototype.get = function (i) {
            var offsets = this.offsets;
            return this.array.subarray(offsets[i], offsets[i + 1]);
        };
        RaggedArray.prototype.set = function (i, array) {
            this.array.set(array, this.offsets[i]);
        };
        return RaggedArray;
    }());
    exports.RaggedArray = RaggedArray;
    RaggedArray.__name__ = "RaggedArray";
    RaggedArray[Symbol.toStringTag] = "RaggedArray";
    var data_structures_1 = require(141) /* ./util/data_structures */;
    exports.Indices = data_structures_1.BitSet;
},
/* core/util/eq.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    // Based on Underscore.js 1.8.3 (http://underscorejs.org)
    exports.equals = Symbol("equals");
    exports.wildcard = Symbol("wildcard");
    var toString = Object.prototype.toString;
    var Comparator = /** @class */ (function () {
        function Comparator() {
            this.a_stack = [];
            this.b_stack = [];
        }
        Comparator.prototype.eq = function (a, b) {
            var _this = this;
            if (Object.is(a, b))
                return true;
            if (a === exports.wildcard || b === exports.wildcard)
                return true;
            if (a == null || b == null)
                return a === b;
            var class_name = toString.call(a);
            if (class_name != toString.call(b))
                return false;
            switch (class_name) {
                case '[object Number]':
                    return this.numbers(a, b);
                case '[object RegExp]':
                case '[object String]':
                    return "" + a == "" + b;
                case '[object Date]':
                case '[object Boolean]':
                    return +a === +b;
            }
            // Assume equality for cyclic structures. The algorithm for detecting cyclic
            // structures is adapted from ES 5.1 section 15.12.3, abstract operation `JO`.
            // Initializing stack of traversed objects.
            // It's done here since we only need them for objects and arrays comparison.
            var _a = this, a_stack = _a.a_stack, b_stack = _a.b_stack;
            var length = a_stack.length;
            while (length--) {
                // Linear search. Performance is inversely proportional to the number of
                // unique nested structures.
                if (a_stack[length] === a)
                    return b_stack[length] === b;
            }
            a_stack.push(a);
            b_stack.push(b);
            var result = (function () {
                if (a[exports.equals] != null && b[exports.equals] != null) {
                    return a[exports.equals](b, _this);
                }
                switch (class_name) {
                    case "[object Array]":
                    case "[object Uint8Array]":
                    case "[object Int8Array]":
                    case "[object Uint16Array]":
                    case "[object Int16Array]":
                    case "[object Uint32Array]":
                    case "[object Int32Array]":
                    case "[object Float32Array]":
                    case "[object Float64Array]": {
                        return _this.arrays(a, b);
                    }
                    case "[object Map]":
                        return _this.maps(a, b);
                    case "[object Set]":
                        return _this.sets(a, b);
                    case "[object Object]": {
                        if (a.constructor == b.constructor && (a.constructor == null || a.constructor === Object)) {
                            return _this.objects(a, b);
                        }
                    }
                    case "[object Function]": {
                        if (a.constructor == b.constructor && a.constructor === Function) {
                            return _this.eq("" + a, "" + b);
                        }
                    }
                }
                if (a instanceof Node) {
                    return _this.nodes(a, b);
                }
                throw Error("can't compare objects of type " + class_name);
            })();
            a_stack.pop();
            b_stack.pop();
            return result;
        };
        Comparator.prototype.numbers = function (a, b) {
            return Object.is(a, b);
        };
        Comparator.prototype.arrays = function (a, b) {
            var length = a.length;
            if (length != b.length)
                return false;
            for (var i = 0; i < length; i++) {
                if (!this.eq(a[i], b[i]))
                    return false;
            }
            return true;
        };
        Comparator.prototype.iterables = function (a, b) {
            var ai = a[Symbol.iterator]();
            var bi = b[Symbol.iterator]();
            while (true) {
                var an = ai.next();
                var bn = bi.next();
                if (an.done && bn.done)
                    return true;
                if (an.done || bn.done)
                    return false;
                if (!this.eq(an.value, bn.value))
                    return false;
            }
        };
        Comparator.prototype.maps = function (a, b) {
            var e_1, _a;
            if (a.size != b.size)
                return false;
            try {
                for (var a_1 = tslib_1.__values(a), a_1_1 = a_1.next(); !a_1_1.done; a_1_1 = a_1.next()) {
                    var _b = tslib_1.__read(a_1_1.value, 2), key = _b[0], val = _b[1];
                    if (!b.has(key) || !this.eq(val, b.get(key)))
                        return false;
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (a_1_1 && !a_1_1.done && (_a = a_1.return))
                        _a.call(a_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return true;
        };
        Comparator.prototype.sets = function (a, b) {
            var e_2, _a;
            if (a.size != b.size)
                return false;
            try {
                for (var a_2 = tslib_1.__values(a), a_2_1 = a_2.next(); !a_2_1.done; a_2_1 = a_2.next()) {
                    var key = a_2_1.value;
                    if (!b.has(key))
                        return false;
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (a_2_1 && !a_2_1.done && (_a = a_2.return))
                        _a.call(a_2);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return true;
        };
        Comparator.prototype.objects = function (a, b) {
            var e_3, _a;
            var keys = Object.keys(a);
            if (keys.length != Object.keys(b).length)
                return false;
            try {
                for (var keys_1 = tslib_1.__values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                    var key = keys_1_1.value;
                    if (!b.hasOwnProperty(key) || !this.eq(a[key], b[key]))
                        return false;
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (keys_1_1 && !keys_1_1.done && (_a = keys_1.return))
                        _a.call(keys_1);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            return true;
        };
        Comparator.prototype.nodes = function (a, b) {
            if (a.nodeType != b.nodeType)
                return false;
            if (a.textContent != b.textContent)
                return false;
            if (!this.iterables(a.childNodes, b.childNodes))
                return false;
            return true;
        };
        return Comparator;
    }());
    exports.Comparator = Comparator;
    Comparator.__name__ = "Comparator";
    var abs = Math.abs;
    var SimilarComparator = /** @class */ (function (_super) {
        tslib_1.__extends(SimilarComparator, _super);
        function SimilarComparator(tolerance) {
            if (tolerance === void 0) {
                tolerance = 1e-4;
            }
            var _this = _super.call(this) || this;
            _this.tolerance = tolerance;
            return _this;
        }
        SimilarComparator.prototype.numbers = function (a, b) {
            return _super.prototype.numbers.call(this, a, b) || abs(a - b) < this.tolerance;
        };
        return SimilarComparator;
    }(Comparator));
    exports.SimilarComparator = SimilarComparator;
    SimilarComparator.__name__ = "SimilarComparator";
    function is_equal(a, b) {
        var comparator = new Comparator();
        return comparator.eq(a, b);
    }
    exports.is_equal = is_equal;
    function is_similar(a, b, tolerance) {
        var comparator = new SimilarComparator(tolerance);
        return comparator.eq(a, b);
    }
    exports.is_similar = is_similar;
    /** @deprecated */
    exports.isEqual = is_equal;
},
/* core/util/data_structures.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var assert_1 = require(126) /* ./assert */;
    var array_1 = require(124) /* ./array */;
    var eq_1 = require(140) /* ./eq */;
    var BitSet = /** @class */ (function () {
        function BitSet(size, init) {
            if (init === void 0) {
                init = 0;
            }
            this.size = size;
            this[Symbol.toStringTag] = "BitSet";
            this._count = null;
            this._nwords = Math.ceil(size / 32);
            if (init == 0 || init == 1) {
                this._array = new Uint32Array(this._nwords);
                if (init == 1) {
                    this._array.fill(0xffffffff);
                }
            }
            else {
                assert_1.assert(init.length == this._nwords, "Initializer size mismatch");
                this._array = init;
            }
        }
        BitSet.prototype.clone = function () {
            return new BitSet(this.size, new Uint32Array(this._array));
        };
        BitSet.prototype[eq_1.equals] = function (that, cmp) {
            if (!cmp.eq(this.size, that.size))
                return false;
            var _nwords = this._nwords;
            var trailing = this.size % _nwords;
            var n = trailing == 0 ? _nwords : _nwords - 1;
            for (var i = 0; i < n; i++) {
                if (this._array[i] != that._array[i])
                    return false;
            }
            if (trailing == 0)
                return true;
            else {
                var msb = 1 << (trailing - 1);
                var mask = (msb - 1) ^ msb;
                return (this._array[n] & mask) == (that._array[n] & mask);
            }
        };
        BitSet.all_set = function (size) {
            return new BitSet(size, 1);
        };
        BitSet.all_unset = function (size) {
            return new BitSet(size, 0);
        };
        BitSet.from_indices = function (size, indices) {
            var e_1, _a;
            var bits = new BitSet(size);
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    bits.set(i);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_a = indices_1.return))
                        _a.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return bits;
        };
        BitSet.from_booleans = function (size, booleans) {
            var bits = new BitSet(size);
            var n = Math.min(size, booleans.length);
            for (var i = 0; i < n; i++) {
                if (booleans[i])
                    bits.set(i);
            }
            return bits;
        };
        BitSet.prototype._check_bounds = function (k) {
            assert_1.assert(0 <= k && k < this.size, "Out of bounds");
        };
        BitSet.prototype.get = function (k) {
            this._check_bounds(k);
            var i = k >>> 5; // Math.floor(k/32)
            var j = k & 0x1f; // k % 32
            return !!((this._array[i] >> j) & 0x1);
        };
        BitSet.prototype.set = function (k, v) {
            if (v === void 0) {
                v = true;
            }
            this._check_bounds(k);
            this._count = null;
            var i = k >>> 5; // Math.floor(k/32)
            var j = k & 0x1f; // k % 32
            if (v)
                this._array[i] |= 0x1 << j;
            else
                this._array[i] &= ~(0x1 << j);
        };
        BitSet.prototype.unset = function (k) {
            this.set(k, false);
        };
        BitSet.prototype[Symbol.iterator] = function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [5 /*yield**/, tslib_1.__values(this.ones())];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        };
        Object.defineProperty(BitSet.prototype, "count", {
            get: function () {
                var count = this._count;
                if (count == null)
                    this._count = count = this._get_count();
                return count;
            },
            enumerable: true,
            configurable: true
        });
        BitSet.prototype._get_count = function () {
            var _a = this, _array = _a._array, _nwords = _a._nwords, size = _a.size;
            var c = 0;
            for (var k = 0, i = 0; i < _nwords; i++) {
                var word = _array[i];
                if (word == 0) {
                    k += 32;
                }
                else {
                    for (var j = 0; j < 32 && k < size; j++, k++) {
                        if ((word >>> j) & 0x1)
                            c += 1;
                    }
                }
            }
            return c;
        };
        BitSet.prototype.ones = function () {
            var _a, _array, _nwords, size, k, i, word, j;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this, _array = _a._array, _nwords = _a._nwords, size = _a.size;
                        k = 0, i = 0;
                        _b.label = 1;
                    case 1:
                        if (!(i < _nwords))
                            return [3 /*break*/, 6];
                        word = _array[i];
                        if (word == 0) {
                            k += 32;
                            return [3 /*break*/, 5];
                        }
                        j = 0;
                        _b.label = 2;
                    case 2:
                        if (!(j < 32 && k < size))
                            return [3 /*break*/, 5];
                        if (!((word >>> j) & 0x1))
                            return [3 /*break*/, 4];
                        return [4 /*yield*/, k];
                    case 3:
                        _b.sent();
                        _b.label = 4;
                    case 4:
                        j++, k++;
                        return [3 /*break*/, 2];
                    case 5:
                        i++;
                        return [3 /*break*/, 1];
                    case 6: return [2 /*return*/];
                }
            });
        };
        BitSet.prototype.zeros = function () {
            var _a, _array, _nwords, size, k, i, word, j;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this, _array = _a._array, _nwords = _a._nwords, size = _a.size;
                        k = 0, i = 0;
                        _b.label = 1;
                    case 1:
                        if (!(i < _nwords))
                            return [3 /*break*/, 6];
                        word = _array[i];
                        if (word == 0xffffffff) {
                            k += 32;
                            return [3 /*break*/, 5];
                        }
                        j = 0;
                        _b.label = 2;
                    case 2:
                        if (!(j < 32 && k < size))
                            return [3 /*break*/, 5];
                        if (!!((word >>> j) & 0x1))
                            return [3 /*break*/, 4];
                        return [4 /*yield*/, k];
                    case 3:
                        _b.sent();
                        _b.label = 4;
                    case 4:
                        j++, k++;
                        return [3 /*break*/, 2];
                    case 5:
                        i++;
                        return [3 /*break*/, 1];
                    case 6: return [2 /*return*/];
                }
            });
        };
        BitSet.prototype._check_size = function (other) {
            assert_1.assert(this.size == other.size, "Size mismatch");
        };
        BitSet.prototype.add = function (other) {
            this._check_size(other);
            for (var i = 0; i < this._nwords; i++) {
                this._array[i] |= other._array[i];
            }
        };
        BitSet.prototype.intersect = function (other) {
            this._check_size(other);
            for (var i = 0; i < this._nwords; i++) {
                this._array[i] &= other._array[i];
            }
        };
        BitSet.prototype.subtract = function (other) {
            this._check_size(other);
            for (var i = 0; i < this._nwords; i++) {
                var a = this._array[i];
                var b = other._array[i];
                this._array[i] = (a ^ b) & a;
            }
        };
        BitSet.prototype.union = function (other) {
            this._check_size(other);
            var result = this.clone();
            for (var i = 0; i < this._nwords; i++) {
                result._array[i] |= other._array[i];
            }
            return result;
        };
        BitSet.prototype.intersection = function (other) {
            this._check_size(other);
            var result = this.clone();
            for (var i = 0; i < this._nwords; i++) {
                result._array[i] &= other._array[i];
            }
            return result;
        };
        BitSet.prototype.difference = function (other) {
            this._check_size(other);
            var result = this.clone();
            for (var i = 0; i < this._nwords; i++) {
                var a = this._array[i];
                var b = other._array[i];
                result._array[i] = (a ^ b) & a;
            }
            return result;
        };
        BitSet.prototype.select = function (array) {
            var e_2, _a;
            var n = this.count;
            var result = new array.constructor(n);
            var i = 0;
            try {
                for (var _b = tslib_1.__values(this), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var j = _c.value;
                    result[i++] = array[j];
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return result;
        };
        return BitSet;
    }());
    exports.BitSet = BitSet;
    BitSet.__name__ = "BitSet";
    var Matrix = /** @class */ (function () {
        function Matrix(nrows, ncols, init) {
            this.nrows = nrows;
            this.ncols = ncols;
            this._matrix = new Array(nrows);
            for (var y = 0; y < nrows; y++) {
                this._matrix[y] = new Array(ncols);
                for (var x = 0; x < ncols; x++) {
                    this._matrix[y][x] = init(y, x);
                }
            }
        }
        Matrix.prototype.at = function (row, col) {
            return this._matrix[row][col];
        };
        Matrix.prototype[Symbol.iterator] = function () {
            var y, x, value;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        y = 0;
                        _a.label = 1;
                    case 1:
                        if (!(y < this.nrows))
                            return [3 /*break*/, 6];
                        x = 0;
                        _a.label = 2;
                    case 2:
                        if (!(x < this.ncols))
                            return [3 /*break*/, 5];
                        value = this._matrix[y][x];
                        return [4 /*yield*/, [value, y, x]];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4:
                        x++;
                        return [3 /*break*/, 2];
                    case 5:
                        y++;
                        return [3 /*break*/, 1];
                    case 6: return [2 /*return*/];
                }
            });
        };
        Matrix.prototype.values = function () {
            var _a, _b, _c, item, e_3_1;
            var e_3, _d;
            return tslib_1.__generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        _e.trys.push([0, 5, 6, 7]);
                        _a = tslib_1.__values(this), _b = _a.next();
                        _e.label = 1;
                    case 1:
                        if (!!_b.done)
                            return [3 /*break*/, 4];
                        _c = tslib_1.__read(_b.value, 1), item = _c[0];
                        return [4 /*yield*/, item];
                    case 2:
                        _e.sent();
                        _e.label = 3;
                    case 3:
                        _b = _a.next();
                        return [3 /*break*/, 1];
                    case 4: return [3 /*break*/, 7];
                    case 5:
                        e_3_1 = _e.sent();
                        e_3 = { error: e_3_1 };
                        return [3 /*break*/, 7];
                    case 6:
                        try {
                            if (_b && !_b.done && (_d = _a.return))
                                _d.call(_a);
                        }
                        finally {
                            if (e_3)
                                throw e_3.error;
                        }
                        return [7 /*endfinally*/];
                    case 7: return [2 /*return*/];
                }
            });
        };
        Matrix.prototype.map = function (fn) {
            var _this = this;
            return new Matrix(this.nrows, this.ncols, function (row, col) { return fn(_this.at(row, col), row, col); });
        };
        Matrix.prototype.apply = function (obj) {
            var _this = this;
            var fn = Matrix.from(obj);
            var _a = this, nrows = _a.nrows, ncols = _a.ncols;
            if (nrows == fn.nrows && ncols == fn.ncols)
                return new Matrix(nrows, ncols, function (row, col) { return fn.at(row, col)(_this.at(row, col), row, col); });
            else
                throw new Error("dimensions don't match");
        };
        Matrix.prototype.to_sparse = function () {
            return tslib_1.__spread(this);
        };
        Matrix.from = function (obj, ncols) {
            if (obj instanceof Matrix) {
                return obj;
            }
            else if (ncols != null) {
                var entries_1 = obj;
                var nrows = Math.floor(entries_1.length / ncols);
                return new Matrix(nrows, ncols, function (row, col) { return entries_1[row * ncols + col]; });
            }
            else {
                var arrays_1 = obj;
                var nrows = obj.length;
                var ncols_1 = array_1.min(arrays_1.map(function (row) { return row.length; }));
                return new Matrix(nrows, ncols_1, function (row, col) { return arrays_1[row][col]; });
            }
        };
        return Matrix;
    }());
    exports.Matrix = Matrix;
    Matrix.__name__ = "Matrix";
},
/* core/settings.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var Settings = /** @class */ (function () {
        function Settings() {
            this._dev = false;
        }
        Object.defineProperty(Settings.prototype, "dev", {
            get: function () {
                return this._dev;
            },
            set: function (dev) {
                this._dev = dev;
            },
            enumerable: true,
            configurable: true
        });
        return Settings;
    }());
    exports.Settings = Settings;
    Settings.__name__ = "Settings";
    exports.settings = new Settings();
},
/* core/property_mixins.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var p = tslib_1.__importStar(require(133) /* ./properties */);
    exports.Line = {
        line_color: [p.Color, "black"],
        line_alpha: [p.Number, 1.0],
        line_width: [p.Number, 1],
        line_join: [p.LineJoin, "bevel"],
        line_cap: [p.LineCap, "butt"],
        line_dash: [p.Array, []],
        line_dash_offset: [p.Number, 0],
    };
    exports.Fill = {
        fill_color: [p.Color, "gray"],
        fill_alpha: [p.Number, 1.0],
    };
    exports.Hatch = {
        hatch_color: [p.Color, "black"],
        hatch_alpha: [p.Number, 1.0],
        hatch_scale: [p.Number, 12.0],
        hatch_pattern: [p.NullString, null],
        hatch_weight: [p.Number, 1.0],
        hatch_extra: [p.Any, {}],
    };
    exports.Text = {
        text_color: [p.Color, "#444444"],
        text_alpha: [p.Number, 1.0],
        text_font: [p.Font, "helvetica"],
        text_font_size: [p.FontSize, "16px"],
        text_font_style: [p.FontStyle, "normal"],
        text_align: [p.TextAlign, "left"],
        text_baseline: [p.TextBaseline, "bottom"],
        text_line_height: [p.Number, 1.2],
    };
    exports.LineScalar = {
        line_color: [p.ColorScalar, "black"],
        line_alpha: [p.NumberScalar, 1.0],
        line_width: [p.NumberScalar, 1],
        line_join: [p.LineJoinScalar, "bevel"],
        line_cap: [p.LineCapScalar, "butt"],
        line_dash: [p.ArrayScalar, []],
        line_dash_offset: [p.NumberScalar, 0],
    };
    exports.FillScalar = {
        fill_color: [p.ColorScalar, "gray"],
        fill_alpha: [p.NumberScalar, 1.0],
    };
    exports.HatchScalar = {
        hatch_color: [p.ColorScalar, "black"],
        hatch_alpha: [p.NumberScalar, 1.0],
        hatch_scale: [p.NumberScalar, 12.0],
        hatch_pattern: [p.NullStringScalar, null],
        hatch_weight: [p.NumberScalar, 1.0],
        hatch_extra: [p.AnyScalar, {}],
    };
    exports.TextScalar = {
        text_color: [p.ColorScalar, "#444444"],
        text_alpha: [p.NumberScalar, 1.0],
        text_font: [p.Font, "helvetica"],
        text_font_size: [p.FontSizeScalar, "16px"],
        text_font_style: [p.FontStyleScalar, "normal"],
        text_align: [p.TextAlignScalar, "left"],
        text_baseline: [p.TextBaselineScalar, "bottom"],
        text_line_height: [p.NumberScalar, 1.2],
    };
    exports.LineVector = {
        line_color: [p.ColorSpec, "black"],
        line_alpha: [p.NumberSpec, 1.0],
        line_width: [p.NumberSpec, 1],
        line_join: [p.LineJoin, "bevel"],
        line_cap: [p.LineCap, "butt"],
        line_dash: [p.Array, []],
        line_dash_offset: [p.Number, 0],
    };
    exports.FillVector = {
        fill_color: [p.ColorSpec, "gray"],
        fill_alpha: [p.NumberSpec, 1.0],
    };
    exports.HatchVector = {
        hatch_color: [p.ColorSpec, "black"],
        hatch_alpha: [p.NumberSpec, 1.0],
        hatch_scale: [p.NumberSpec, 12.0],
        hatch_pattern: [p.NullStringSpec, null],
        hatch_weight: [p.NumberSpec, 1.0],
        hatch_extra: [p.Any, {}],
    };
    exports.TextVector = {
        text_color: [p.ColorSpec, "#444444"],
        text_alpha: [p.NumberSpec, 1.0],
        text_font: [p.Font, "helvetica"],
        text_font_size: [p.FontSizeSpec, "16px"],
        text_font_style: [p.FontStyle, "normal"],
        text_align: [p.TextAlign, "left"],
        text_baseline: [p.TextBaseline, "bottom"],
        text_line_height: [p.Number, 1.2],
    };
},
/* core/util/string.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var settings_1 = require(142) /* ../settings */;
    function startsWith(str, searchString, position) {
        if (position === void 0) {
            position = 0;
        }
        return str.substr(position, searchString.length) == searchString;
    }
    exports.startsWith = startsWith;
    function uuid4() {
        // from ipython project
        // http://www.ietf.org/rfc/rfc4122.txt
        var s = new Array(32);
        var hexDigits = "0123456789ABCDEF";
        for (var i = 0; i < 32; i++) {
            s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
        }
        s[12] = "4"; // bits 12-15 of the time_hi_and_version field to 0010
        s[16] = hexDigits.substr((s[16].charCodeAt(0) & 0x3) | 0x8, 1); // bits 6-7 of the clock_seq_hi_and_reserved to 01
        return s.join("");
    }
    exports.uuid4 = uuid4;
    var counter = 1000;
    function uniqueId(prefix) {
        var id = settings_1.settings.dev ? "j" + counter++ : uuid4();
        if (prefix != null)
            return prefix + "-" + id;
        else
            return id;
    }
    exports.uniqueId = uniqueId;
    function escape(s) {
        return s.replace(/(?:[&<>"'`])/g, function (ch) {
            switch (ch) {
                case '&': return '&amp;';
                case '<': return '&lt;';
                case '>': return '&gt;';
                case '"': return '&quot;';
                case "'": return '&#x27;';
                case '`': return '&#x60;';
                default: return ch;
            }
        });
    }
    exports.escape = escape;
    function unescape(s) {
        return s.replace(/&(amp|lt|gt|quot|#x27|#x60);/g, function (_, entity) {
            switch (entity) {
                case 'amp': return '&';
                case 'lt': return '<';
                case 'gt': return '>';
                case 'quot': return '"';
                case '#x27': return "'";
                case '#x60': return '`';
                default: return entity;
            }
        });
    }
    exports.unescape = unescape;
    function use_strict(code) {
        return "'use strict';\n" + code;
    }
    exports.use_strict = use_strict;
},
/* core/util/ndarray.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(123) /* ./types */;
    var assert_1 = require(126) /* ./assert */;
    var eq_1 = require(140) /* ./eq */;
    var __ndarray__ = Symbol("__ndarray__");
    var Uint8NDArray = /** @class */ (function (_super) {
        tslib_1.__extends(Uint8NDArray, _super);
        function Uint8NDArray(seq, shape) {
            var _this = new _super(seq);
            _this.__ndarray__ = __ndarray__;
            _this.dtype = "uint8";
            _this.shape = shape !== null && shape !== void 0 ? shape : (is_NDArray(seq) ? seq.shape : [_this.length]);
            _this.dimension = _this.shape.length;
            return _this;
        }
        Uint8NDArray.prototype[eq_1.equals] = function (that, cmp) {
            return cmp.eq(this.shape, that.shape) && cmp.arrays(this, that);
        };
        return Uint8NDArray;
    }(Uint8Array));
    exports.Uint8NDArray = Uint8NDArray;
    Uint8NDArray.__name__ = "Uint8NDArray";
    var Int8NDArray = /** @class */ (function (_super) {
        tslib_1.__extends(Int8NDArray, _super);
        function Int8NDArray(seq, shape) {
            var _this = new _super(seq);
            _this.__ndarray__ = __ndarray__;
            _this.dtype = "int8";
            _this.shape = shape !== null && shape !== void 0 ? shape : (is_NDArray(seq) ? seq.shape : [_this.length]);
            _this.dimension = _this.shape.length;
            return _this;
        }
        Int8NDArray.prototype[eq_1.equals] = function (that, cmp) {
            return cmp.eq(this.shape, that.shape) && cmp.arrays(this, that);
        };
        return Int8NDArray;
    }(Int8Array));
    exports.Int8NDArray = Int8NDArray;
    Int8NDArray.__name__ = "Int8NDArray";
    var Uint16NDArray = /** @class */ (function (_super) {
        tslib_1.__extends(Uint16NDArray, _super);
        function Uint16NDArray(seq, shape) {
            var _this = new _super(seq);
            _this.__ndarray__ = __ndarray__;
            _this.dtype = "uint16";
            _this.shape = shape !== null && shape !== void 0 ? shape : (is_NDArray(seq) ? seq.shape : [_this.length]);
            _this.dimension = _this.shape.length;
            return _this;
        }
        Uint16NDArray.prototype[eq_1.equals] = function (that, cmp) {
            return cmp.eq(this.shape, that.shape) && cmp.arrays(this, that);
        };
        return Uint16NDArray;
    }(Uint16Array));
    exports.Uint16NDArray = Uint16NDArray;
    Uint16NDArray.__name__ = "Uint16NDArray";
    var Int16NDArray = /** @class */ (function (_super) {
        tslib_1.__extends(Int16NDArray, _super);
        function Int16NDArray(seq, shape) {
            var _this = new _super(seq);
            _this.__ndarray__ = __ndarray__;
            _this.dtype = "int16";
            _this.shape = shape !== null && shape !== void 0 ? shape : (is_NDArray(seq) ? seq.shape : [_this.length]);
            _this.dimension = _this.shape.length;
            return _this;
        }
        Int16NDArray.prototype[eq_1.equals] = function (that, cmp) {
            return cmp.eq(this.shape, that.shape) && cmp.arrays(this, that);
        };
        return Int16NDArray;
    }(Int16Array));
    exports.Int16NDArray = Int16NDArray;
    Int16NDArray.__name__ = "Int16NDArray";
    var Uint32NDArray = /** @class */ (function (_super) {
        tslib_1.__extends(Uint32NDArray, _super);
        function Uint32NDArray(seq, shape) {
            var _this = new _super(seq);
            _this.__ndarray__ = __ndarray__;
            _this.dtype = "uint32";
            _this.shape = shape !== null && shape !== void 0 ? shape : (is_NDArray(seq) ? seq.shape : [_this.length]);
            _this.dimension = _this.shape.length;
            return _this;
        }
        Uint32NDArray.prototype[eq_1.equals] = function (that, cmp) {
            return cmp.eq(this.shape, that.shape) && cmp.arrays(this, that);
        };
        return Uint32NDArray;
    }(Uint32Array));
    exports.Uint32NDArray = Uint32NDArray;
    Uint32NDArray.__name__ = "Uint32NDArray";
    var Int32NDArray = /** @class */ (function (_super) {
        tslib_1.__extends(Int32NDArray, _super);
        function Int32NDArray(seq, shape) {
            var _this = new _super(seq);
            _this.__ndarray__ = __ndarray__;
            _this.dtype = "int32";
            _this.shape = shape !== null && shape !== void 0 ? shape : (is_NDArray(seq) ? seq.shape : [_this.length]);
            _this.dimension = _this.shape.length;
            return _this;
        }
        Int32NDArray.prototype[eq_1.equals] = function (that, cmp) {
            return cmp.eq(this.shape, that.shape) && cmp.arrays(this, that);
        };
        return Int32NDArray;
    }(Int32Array));
    exports.Int32NDArray = Int32NDArray;
    Int32NDArray.__name__ = "Int32NDArray";
    var Float32NDArray = /** @class */ (function (_super) {
        tslib_1.__extends(Float32NDArray, _super);
        function Float32NDArray(seq, shape) {
            var _this = new _super(seq);
            _this.__ndarray__ = __ndarray__;
            _this.dtype = "float32";
            _this.shape = shape !== null && shape !== void 0 ? shape : (is_NDArray(seq) ? seq.shape : [_this.length]);
            _this.dimension = _this.shape.length;
            return _this;
        }
        Float32NDArray.prototype[eq_1.equals] = function (that, cmp) {
            return cmp.eq(this.shape, that.shape) && cmp.arrays(this, that);
        };
        return Float32NDArray;
    }(Float32Array));
    exports.Float32NDArray = Float32NDArray;
    Float32NDArray.__name__ = "Float32NDArray";
    var Float64NDArray = /** @class */ (function (_super) {
        tslib_1.__extends(Float64NDArray, _super);
        function Float64NDArray(seq, shape) {
            var _this = new _super(seq);
            _this.__ndarray__ = __ndarray__;
            _this.dtype = "float64";
            _this.shape = shape !== null && shape !== void 0 ? shape : (is_NDArray(seq) ? seq.shape : [_this.length]);
            _this.dimension = _this.shape.length;
            return _this;
        }
        Float64NDArray.prototype[eq_1.equals] = function (that, cmp) {
            return cmp.eq(this.shape, that.shape) && cmp.arrays(this, that);
        };
        return Float64NDArray;
    }(Float64Array));
    exports.Float64NDArray = Float64NDArray;
    Float64NDArray.__name__ = "Float64NDArray";
    function is_NDArray(v) {
        return types_1.isObject(v) && v.__ndarray__ == __ndarray__;
    }
    exports.is_NDArray = is_NDArray;
    function ndarray(array, options) {
        if (options === void 0) {
            options = {};
        }
        var dtype = options.dtype;
        if (dtype == null) {
            if (array instanceof ArrayBuffer || types_1.isArray(array)) {
                dtype = "float32";
            }
            else {
                dtype = (function () {
                    switch (true) {
                        case array instanceof Uint8Array: return "uint8";
                        case array instanceof Int8Array: return "int8";
                        case array instanceof Uint16Array: return "uint16";
                        case array instanceof Int16Array: return "int16";
                        case array instanceof Uint32Array: return "uint32";
                        case array instanceof Int32Array: return "int32";
                        case array instanceof Float32Array: return "float32";
                        case array instanceof Float64Array: return "float64";
                        default:
                            assert_1.unreachable();
                    }
                })();
            }
        }
        var shape = options.shape;
        switch (dtype) {
            case "uint8": return new Uint8NDArray(array, shape);
            case "int8": return new Int8NDArray(array, shape);
            case "uint16": return new Uint16NDArray(array, shape);
            case "int16": return new Int16NDArray(array, shape);
            case "uint32": return new Uint32NDArray(array, shape);
            case "int32": return new Int32NDArray(array, shape);
            case "float32": return new Float32NDArray(array, shape);
            case "float64": return new Float64NDArray(array, shape);
        }
    }
    exports.ndarray = ndarray;
},
/* core/util/serialization.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(123) /* ./types */;
    var compat_1 = require(147) /* ./compat */;
    var ndarray = tslib_1.__importStar(require(145) /* ./ndarray */);
    function buffer_to_base64(buffer) {
        var bytes = new Uint8Array(buffer);
        var chars = Array.from(bytes).map(function (b) { return String.fromCharCode(b); });
        return btoa(chars.join(""));
    }
    exports.buffer_to_base64 = buffer_to_base64;
    function base64_to_buffer(base64) {
        var binary_string = atob(base64);
        var len = binary_string.length;
        var bytes = new Uint8Array(len);
        for (var i = 0, end = len; i < end; i++) {
            bytes[i] = binary_string.charCodeAt(i);
        }
        return bytes.buffer;
    }
    exports.base64_to_buffer = base64_to_buffer;
    exports.BYTE_ORDER = compat_1.is_little_endian ? "little" : "big";
    function swap16(a) {
        var x = new Uint8Array(a.buffer, a.byteOffset, a.length * 2);
        for (var i = 0, end = x.length; i < end; i += 2) {
            var t = x[i];
            x[i] = x[i + 1];
            x[i + 1] = t;
        }
    }
    exports.swap16 = swap16;
    function swap32(a) {
        var x = new Uint8Array(a.buffer, a.byteOffset, a.length * 4);
        for (var i = 0, end = x.length; i < end; i += 4) {
            var t = x[i];
            x[i] = x[i + 3];
            x[i + 3] = t;
            t = x[i + 1];
            x[i + 1] = x[i + 2];
            x[i + 2] = t;
        }
    }
    exports.swap32 = swap32;
    function swap64(a) {
        var x = new Uint8Array(a.buffer, a.byteOffset, a.length * 8);
        for (var i = 0, end = x.length; i < end; i += 8) {
            var t = x[i];
            x[i] = x[i + 7];
            x[i + 7] = t;
            t = x[i + 1];
            x[i + 1] = x[i + 6];
            x[i + 6] = t;
            t = x[i + 2];
            x[i + 2] = x[i + 5];
            x[i + 5] = t;
            t = x[i + 3];
            x[i + 3] = x[i + 4];
            x[i + 4] = t;
        }
    }
    exports.swap64 = swap64;
    function is_NDArray_ref(v) {
        return types_1.isPlainObject(v) && ("__buffer__" in v || "__ndarray__" in v);
    }
    exports.is_NDArray_ref = is_NDArray_ref;
    function decode_NDArray(ref, buffers) {
        var shape = ref.shape, dtype = ref.dtype, order = ref.order;
        var bytes;
        if ("__buffer__" in ref) {
            var buffer = buffers.get(ref.__buffer__);
            if (buffer != null)
                bytes = buffer;
            else
                throw new Error("buffer for " + ref.__buffer__ + " not found");
        }
        else {
            bytes = base64_to_buffer(ref.__ndarray__);
        }
        var array = (function () {
            switch (dtype) {
                case "uint8": return new ndarray.Uint8NDArray(bytes, shape);
                case "int8": return new ndarray.Int8NDArray(bytes, shape);
                case "uint16": return new ndarray.Uint16NDArray(bytes, shape);
                case "int16": return new ndarray.Int16NDArray(bytes, shape);
                case "uint32": return new ndarray.Uint32NDArray(bytes, shape);
                case "int32": return new ndarray.Int32NDArray(bytes, shape);
                case "float32": return new ndarray.Float32NDArray(bytes, shape);
                case "float64": return new ndarray.Float64NDArray(bytes, shape);
            }
        })();
        if (order !== exports.BYTE_ORDER) {
            switch (array.BYTES_PER_ELEMENT) {
                case 2:
                    swap16(array);
                    break;
                case 4:
                    swap32(array);
                    break;
                case 8:
                    swap64(array);
                    break;
            }
        }
        return array;
    }
    exports.decode_NDArray = decode_NDArray;
    function encode_NDArray(array, buffers) {
        var data = {
            order: exports.BYTE_ORDER,
            dtype: array.dtype,
            shape: array.shape,
        };
        if (buffers != null) {
            var __buffer__ = "" + buffers.size;
            buffers.set(__buffer__, array.buffer);
            return Object.assign({ __buffer__: __buffer__ }, data);
        }
        else {
            var __ndarray__ = buffer_to_base64(array.buffer);
            return Object.assign({ __ndarray__: __ndarray__ }, data);
        }
    }
    exports.encode_NDArray = encode_NDArray;
},
/* core/util/compat.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.is_ie = (function () {
        var ua = typeof navigator !== "undefined" ? navigator.userAgent : "";
        return ua.indexOf('MSIE') >= 0 || ua.indexOf('Trident') > 0 || ua.indexOf('Edge') > 0;
    })();
    exports.is_mobile = (function () {
        return typeof window !== "undefined" && ("ontouchstart" in window || navigator.maxTouchPoints > 0);
    })();
    exports.is_little_endian = (function () {
        var buf = new ArrayBuffer(4);
        var buf8 = new Uint8Array(buf);
        var buf32 = new Uint32Array(buf);
        buf32[1] = 0x0a0b0c0d;
        var little_endian = true;
        if (buf8[4] == 0x0a && buf8[5] == 0x0b && buf8[6] == 0x0c && buf8[7] == 0x0d) {
            little_endian = false;
        }
        return little_endian;
    })();
},
/* core/util/pretty.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(123) /* ./types */;
    var object_1 = require(128) /* ./object */;
    exports.pretty = Symbol("pretty");
    function isPrintable(obj) {
        return exports.pretty in Object(obj);
    }
    var Printer = /** @class */ (function () {
        function Printer(options) {
            this.precision = options === null || options === void 0 ? void 0 : options.precision;
        }
        Printer.prototype.to_string = function (obj) {
            if (isPrintable(obj))
                return obj[exports.pretty](this);
            else if (types_1.isBoolean(obj))
                return this.boolean(obj);
            else if (types_1.isNumber(obj))
                return this.number(obj);
            else if (types_1.isString(obj))
                return this.string(obj);
            else if (types_1.isArray(obj))
                return this.array(obj);
            else if (types_1.isIterable(obj))
                return this.iterable(obj);
            else if (types_1.isPlainObject(obj))
                return this.object(obj);
            else
                return "" + obj;
        };
        Printer.prototype.token = function (val) {
            return val;
        };
        Printer.prototype.boolean = function (val) {
            return "" + val;
        };
        Printer.prototype.number = function (val) {
            if (this.precision != null)
                return val.toFixed(this.precision);
            else
                return "" + val;
        };
        Printer.prototype.string = function (val) {
            return "\"" + val.replace(/'/g, "\\'") + "\""; // lgtm [js/incomplete-sanitization]
        };
        Printer.prototype.array = function (obj) {
            var e_1, _b;
            var T = this.token;
            var items = [];
            try {
                for (var obj_1 = tslib_1.__values(obj), obj_1_1 = obj_1.next(); !obj_1_1.done; obj_1_1 = obj_1.next()) {
                    var entry = obj_1_1.value;
                    items.push(this.to_string(entry));
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (obj_1_1 && !obj_1_1.done && (_b = obj_1.return))
                        _b.call(obj_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return "" + T("[") + items.join(T(",") + " ") + T("]");
        };
        Printer.prototype.iterable = function (obj) {
            var _a;
            var T = this.token;
            var tag = (_a = Object(obj)[Symbol.toStringTag]) !== null && _a !== void 0 ? _a : "Object";
            var items = this.array(obj);
            return "" + tag + T("(") + items + T(")");
        };
        Printer.prototype.object = function (obj) {
            var e_2, _b;
            var T = this.token;
            var items = [];
            try {
                for (var _c = tslib_1.__values(object_1.entries(obj)), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = tslib_1.__read(_d.value, 2), key = _e[0], val = _e[1];
                    items.push("" + key + T(":") + " " + this.to_string(val));
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return "" + T("{") + items.join(T(",") + " ") + T("}");
        };
        return Printer;
    }());
    exports.Printer = Printer;
    Printer.__name__ = "Printer";
    function to_string(obj, options) {
        var printer = new Printer(options);
        return printer.to_string(obj);
    }
    exports.to_string = to_string;
},
/* models/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    tslib_1.__exportStar(require(150) /* ./annotations */, exports);
    tslib_1.__exportStar(require(291) /* ./axes */, exports);
    tslib_1.__exportStar(require(318) /* ./callbacks */, exports);
    tslib_1.__exportStar(require(322) /* ./canvas */, exports);
    tslib_1.__exportStar(require(333) /* ./expressions */, exports);
    tslib_1.__exportStar(require(337) /* ./filters */, exports);
    tslib_1.__exportStar(require(343) /* ./formatters */, exports);
    tslib_1.__exportStar(require(347) /* ./glyphs */, exports);
    tslib_1.__exportStar(require(380) /* ./graphs */, exports);
    tslib_1.__exportStar(require(383) /* ./grids */, exports);
    tslib_1.__exportStar(require(385) /* ./layouts */, exports);
    tslib_1.__exportStar(require(247) /* ./mappers */, exports);
    tslib_1.__exportStar(require(263) /* ./transforms */, exports);
    tslib_1.__exportStar(require(402) /* ./markers */, exports);
    tslib_1.__exportStar(require(406) /* ./plots */, exports);
    tslib_1.__exportStar(require(435) /* ./ranges */, exports);
    tslib_1.__exportStar(require(436) /* ./renderers */, exports);
    tslib_1.__exportStar(require(437) /* ./scales */, exports);
    tslib_1.__exportStar(require(438) /* ./selections */, exports);
    tslib_1.__exportStar(require(439) /* ./sources */, exports);
    tslib_1.__exportStar(require(444) /* ./tickers */, exports);
    tslib_1.__exportStar(require(446) /* ./tiles */, exports);
    tslib_1.__exportStar(require(457) /* ./textures */, exports);
    tslib_1.__exportStar(require(461) /* ./tools */, exports);
},
/* models/annotations/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var annotation_1 = require(151) /* ./annotation */;
    exports.Annotation = annotation_1.Annotation;
    var arrow_1 = require(198) /* ./arrow */;
    exports.Arrow = arrow_1.Arrow;
    var arrow_head_1 = require(199) /* ./arrow_head */;
    exports.ArrowHead = arrow_head_1.ArrowHead;
    var arrow_head_2 = require(199) /* ./arrow_head */;
    exports.OpenHead = arrow_head_2.OpenHead;
    var arrow_head_3 = require(199) /* ./arrow_head */;
    exports.NormalHead = arrow_head_3.NormalHead;
    var arrow_head_4 = require(199) /* ./arrow_head */;
    exports.TeeHead = arrow_head_4.TeeHead;
    var arrow_head_5 = require(199) /* ./arrow_head */;
    exports.VeeHead = arrow_head_5.VeeHead;
    var band_1 = require(237) /* ./band */;
    exports.Band = band_1.Band;
    var box_annotation_1 = require(239) /* ./box_annotation */;
    exports.BoxAnnotation = box_annotation_1.BoxAnnotation;
    var color_bar_1 = require(240) /* ./color_bar */;
    exports.ColorBar = color_bar_1.ColorBar;
    var label_1 = require(275) /* ./label */;
    exports.Label = label_1.Label;
    var label_set_1 = require(277) /* ./label_set */;
    exports.LabelSet = label_set_1.LabelSet;
    var legend_1 = require(278) /* ./legend */;
    exports.Legend = legend_1.Legend;
    var legend_item_1 = require(279) /* ./legend_item */;
    exports.LegendItem = legend_item_1.LegendItem;
    var poly_annotation_1 = require(281) /* ./poly_annotation */;
    exports.PolyAnnotation = poly_annotation_1.PolyAnnotation;
    var slope_1 = require(282) /* ./slope */;
    exports.Slope = slope_1.Slope;
    var span_1 = require(283) /* ./span */;
    exports.Span = span_1.Span;
    var text_annotation_1 = require(276) /* ./text_annotation */;
    exports.TextAnnotation = text_annotation_1.TextAnnotation;
    var title_1 = require(284) /* ./title */;
    exports.Title = title_1.Title;
    var toolbar_panel_1 = require(285) /* ./toolbar_panel */;
    exports.ToolbarPanel = toolbar_panel_1.ToolbarPanel;
    var tooltip_1 = require(286) /* ./tooltip */;
    exports.Tooltip = tooltip_1.Tooltip;
    var whisker_1 = require(290) /* ./whisker */;
    exports.Whisker = whisker_1.Whisker;
},
/* models/annotations/annotation.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var proj = tslib_1.__importStar(require(152) /* ../../core/util/projections */);
    var object_1 = require(128) /* ../../core/util/object */;
    var renderer_1 = require(185) /* ../renderers/renderer */;
    var AnnotationView = /** @class */ (function (_super) {
        tslib_1.__extends(AnnotationView, _super);
        function AnnotationView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Object.defineProperty(AnnotationView.prototype, "panel", {
            get: function () {
                return this.layout;
            },
            enumerable: true,
            configurable: true
        });
        AnnotationView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            var p = this.model.properties;
            this.on_change(p.visible, function () { return _this.plot_view.request_layout(); });
        };
        AnnotationView.prototype.get_size = function () {
            if (this.model.visible) {
                var _a = this._get_size(), width = _a.width, height = _a.height;
                return { width: Math.round(width), height: Math.round(height) };
            }
            else
                return { width: 0, height: 0 };
        };
        AnnotationView.prototype._get_size = function () {
            throw new Error("not implemented");
        };
        AnnotationView.prototype.set_data = function (source) {
            var _a, _b;
            var data = this.model.materialize_dataspecs(source);
            object_1.extend(this, data);
            if (this.plot_model.use_map) {
                var self = this;
                if (self._x != null)
                    _a = tslib_1.__read(proj.project_xy(self._x, self._y), 2), self._x = _a[0], self._y = _a[1];
                if (self._xs != null)
                    _b = tslib_1.__read(proj.project_xsys(self._xs, self._ys), 2), self._xs = _b[0], self._ys = _b[1];
            }
        };
        Object.defineProperty(AnnotationView.prototype, "needs_clip", {
            get: function () {
                return this.layout == null; // TODO: change this, when center layout is fully implemented
            },
            enumerable: true,
            configurable: true
        });
        AnnotationView.prototype.serializable_state = function () {
            var state = _super.prototype.serializable_state.call(this);
            return this.layout == null ? state : Object.assign(Object.assign({}, state), { bbox: this.layout.bbox.box });
        };
        return AnnotationView;
    }(renderer_1.RendererView));
    exports.AnnotationView = AnnotationView;
    AnnotationView.__name__ = "AnnotationView";
    var Annotation = /** @class */ (function (_super) {
        tslib_1.__extends(Annotation, _super);
        function Annotation(attrs) {
            return _super.call(this, attrs) || this;
        }
        Annotation.init_Annotation = function () {
            this.override({
                level: 'annotation',
            });
        };
        return Annotation;
    }(renderer_1.Renderer));
    exports.Annotation = Annotation;
    Annotation.__name__ = "Annotation";
    Annotation.init_Annotation();
},
/* core/util/projections.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var core_1 = tslib_1.__importDefault(require(153) /* proj4/lib/core */);
    var Proj_1 = tslib_1.__importDefault(require(154) /* proj4/lib/Proj */);
    var types_1 = require(139) /* ../types */;
    var mercator = new Proj_1.default('GOOGLE');
    var wgs84 = new Proj_1.default('WGS84');
    var _wgs84_mercator = core_1.default(wgs84, mercator);
    exports.wgs84_mercator = {
        compute: function (x, y) {
            if (isFinite(x) && isFinite(y))
                return _wgs84_mercator.forward([x, y]);
            else
                return [NaN, NaN];
        },
        invert: function (merc_x, merc_y) {
            if (isFinite(merc_x) && isFinite(merc_y))
                return _wgs84_mercator.inverse([merc_x, merc_y]);
            else
                return [NaN, NaN];
        },
    };
    var mercator_bounds = {
        lon: [-20026376.39, 20026376.39],
        lat: [-20048966.10, 20048966.10],
    };
    var latlon_bounds = {
        lon: [-180, 180],
        lat: [-85.06, 85.06],
    };
    var min = Math.min, max = Math.max;
    function clip_mercator(low, high, dimension) {
        var _a = tslib_1.__read(mercator_bounds[dimension], 2), vmin = _a[0], vmax = _a[1];
        return [max(low, vmin), min(high, vmax)];
    }
    exports.clip_mercator = clip_mercator;
    function in_bounds(value, dimension) {
        var _a = tslib_1.__read(latlon_bounds[dimension], 2), min = _a[0], max = _a[1];
        return min < value && value < max;
    }
    exports.in_bounds = in_bounds;
    (function (inplace) {
        function project_xy(x, y, merc_x, merc_y) {
            var n = min(x.length, y.length);
            merc_x = merc_x !== null && merc_x !== void 0 ? merc_x : x;
            merc_y = merc_y !== null && merc_y !== void 0 ? merc_y : y;
            for (var i = 0; i < n; i++) {
                var xi = x[i];
                var yi = y[i];
                var _a = tslib_1.__read(exports.wgs84_mercator.compute(xi, yi), 2), merc_xi = _a[0], merc_yi = _a[1];
                merc_x[i] = merc_xi;
                merc_y[i] = merc_yi;
            }
        }
        inplace.project_xy = project_xy;
        function project_xsys(xs, ys, merc_xs, merc_ys) {
            var n = min(xs.length, ys.length);
            merc_xs = merc_xs !== null && merc_xs !== void 0 ? merc_xs : xs;
            merc_ys = merc_ys !== null && merc_ys !== void 0 ? merc_ys : ys;
            for (var i = 0; i < n; i++) {
                project_xy(xs[i], ys[i], merc_xs[i], merc_ys[i]);
            }
        }
        inplace.project_xsys = project_xsys;
    })(exports.inplace || (exports.inplace = {}));
    function project_xy(x, y) {
        var n = min(x.length, y.length);
        var merc_x = new types_1.NumberArray(n);
        var merc_y = new types_1.NumberArray(n);
        exports.inplace.project_xy(x, y, merc_x, merc_y);
        return [merc_x, merc_y];
    }
    exports.project_xy = project_xy;
    function project_xsys(xs, ys) {
        var n = min(xs.length, ys.length);
        var merc_xs = new Array(n);
        var merc_ys = new Array(n);
        for (var i = 0; i < n; i++) {
            var _a = tslib_1.__read(project_xy(xs[i], ys[i]), 2), merc_x = _a[0], merc_y = _a[1];
            merc_xs[i] = merc_x;
            merc_ys[i] = merc_y;
        }
        return [merc_xs, merc_ys];
    }
    exports.project_xsys = project_xsys;
},
/* proj4/lib/core.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var Proj_1 = tslib_1.__importDefault(require(154) /* ./Proj */);
    var transform_1 = tslib_1.__importDefault(require(179) /* ./transform */);
    var wgs84 = Proj_1.default('WGS84');
    function transformer(from, to, coords) {
        var transformedArray, out, keys;
        if (Array.isArray(coords)) {
            transformedArray = transform_1.default(from, to, coords) || { x: NaN, y: NaN };
            if (coords.length > 2) {
                if ((typeof from.name !== 'undefined' && from.name === 'geocent') || (typeof to.name !== 'undefined' && to.name === 'geocent')) {
                    if (typeof transformedArray.z === 'number') {
                        return [transformedArray.x, transformedArray.y, transformedArray.z].concat(coords.splice(3));
                    }
                    else {
                        return [transformedArray.x, transformedArray.y, coords[2]].concat(coords.splice(3));
                    }
                }
                else {
                    return [transformedArray.x, transformedArray.y].concat(coords.splice(2));
                }
            }
            else {
                return [transformedArray.x, transformedArray.y];
            }
        }
        else {
            out = transform_1.default(from, to, coords);
            keys = Object.keys(coords);
            if (keys.length === 2) {
                return out;
            }
            keys.forEach(function (key) {
                if ((typeof from.name !== 'undefined' && from.name === 'geocent') || (typeof to.name !== 'undefined' && to.name === 'geocent')) {
                    if (key === 'x' || key === 'y' || key === 'z') {
                        return;
                    }
                }
                else {
                    if (key === 'x' || key === 'y') {
                        return;
                    }
                }
                out[key] = coords[key];
            });
            return out;
        }
    }
    function checkProj(item) {
        if (item instanceof Proj_1.default) {
            return item;
        }
        if (item.oProj) {
            return item.oProj;
        }
        return Proj_1.default(item);
    }
    function proj4(fromProj, toProj, coord) {
        fromProj = checkProj(fromProj);
        var single = false;
        var obj;
        if (typeof toProj === 'undefined') {
            toProj = fromProj;
            fromProj = wgs84;
            single = true;
        }
        else if (typeof toProj.x !== 'undefined' || Array.isArray(toProj)) {
            coord = toProj;
            toProj = fromProj;
            fromProj = wgs84;
            single = true;
        }
        toProj = checkProj(toProj);
        if (coord) {
            return transformer(fromProj, toProj, coord);
        }
        else {
            obj = {
                forward: function (coords) {
                    return transformer(fromProj, toProj, coords);
                },
                inverse: function (coords) {
                    return transformer(toProj, fromProj, coords);
                }
            };
            if (single) {
                obj.oProj = toProj;
            }
            return obj;
        }
    }
    exports.default = proj4;
},
/* proj4/lib/Proj.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var parseCode_1 = tslib_1.__importDefault(require(155) /* ./parseCode */);
    var extend_1 = tslib_1.__importDefault(require(166) /* ./extend */);
    var projections_1 = tslib_1.__importDefault(require(167) /* ./projections */);
    var deriveConstants_1 = require(175) /* ./deriveConstants */;
    var Datum_1 = tslib_1.__importDefault(require(177) /* ./constants/Datum */);
    var datum_1 = tslib_1.__importDefault(require(178) /* ./datum */);
    var match_1 = tslib_1.__importDefault(require(162) /* ./match */);
    function Projection(srsCode, callback) {
        if (!(this instanceof Projection)) {
            return new Projection(srsCode);
        }
        callback = callback || function (error) {
            if (error) {
                throw error;
            }
        };
        var json = parseCode_1.default(srsCode);
        if (typeof json !== 'object') {
            callback(srsCode);
            return;
        }
        var ourProj = Projection.projections.get(json.projName);
        if (!ourProj) {
            callback(srsCode);
            return;
        }
        if (json.datumCode && json.datumCode !== 'none') {
            var datumDef = match_1.default(Datum_1.default, json.datumCode);
            if (datumDef) {
                json.datum_params = datumDef.towgs84 ? datumDef.towgs84.split(',') : null;
                json.ellps = datumDef.ellipse;
                json.datumName = datumDef.datumName ? datumDef.datumName : json.datumCode;
            }
        }
        json.k0 = json.k0 || 1.0;
        json.axis = json.axis || 'enu';
        json.ellps = json.ellps || 'wgs84';
        var sphere_ = deriveConstants_1.sphere(json.a, json.b, json.rf, json.ellps, json.sphere);
        var ecc = deriveConstants_1.eccentricity(sphere_.a, sphere_.b, sphere_.rf, json.R_A);
        var datumObj = json.datum || datum_1.default(json.datumCode, json.datum_params, sphere_.a, sphere_.b, ecc.es, ecc.ep2);
        extend_1.default(this, json); // transfer everything over from the projection because we don't know what we'll need
        extend_1.default(this, ourProj); // transfer all the methods from the projection
        // copy the 4 things over we calulated in deriveConstants.sphere
        this.a = sphere_.a;
        this.b = sphere_.b;
        this.rf = sphere_.rf;
        this.sphere = sphere_.sphere;
        // copy the 3 things we calculated in deriveConstants.eccentricity
        this.es = ecc.es;
        this.e = ecc.e;
        this.ep2 = ecc.ep2;
        // add in the datum object
        this.datum = datumObj;
        // init the projection
        this.init();
        // legecy callback from back in the day when it went to spatialreference.org
        callback(null, this);
    }
    Projection.projections = projections_1.default;
    Projection.projections.start();
    exports.default = Projection;
},
/* proj4/lib/parseCode.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var defs_1 = tslib_1.__importDefault(require(156) /* ./defs */);
    var wkt_parser_1 = tslib_1.__importDefault(require(163) /* wkt-parser */);
    var projString_1 = tslib_1.__importDefault(require(158) /* ./projString */);
    var match_1 = tslib_1.__importDefault(require(162) /* ./match */);
    function testObj(code) {
        return typeof code === 'string';
    }
    function testDef(code) {
        return code in defs_1.default;
    }
    var codeWords = ['PROJECTEDCRS', 'PROJCRS', 'GEOGCS', 'GEOCCS', 'PROJCS', 'LOCAL_CS', 'GEODCRS', 'GEODETICCRS', 'GEODETICDATUM', 'ENGCRS', 'ENGINEERINGCRS'];
    function testWKT(code) {
        return codeWords.some(function (word) {
            return code.indexOf(word) > -1;
        });
    }
    var codes = ['3857', '900913', '3785', '102113'];
    function checkMercator(item) {
        var auth = match_1.default(item, 'authority');
        if (!auth) {
            return;
        }
        var code = match_1.default(auth, 'epsg');
        return code && codes.indexOf(code) > -1;
    }
    function checkProjStr(item) {
        var ext = match_1.default(item, 'extension');
        if (!ext) {
            return;
        }
        return match_1.default(ext, 'proj4');
    }
    function testProj(code) {
        return code[0] === '+';
    }
    function parse(code) {
        if (testObj(code)) {
            //check to see if this is a WKT string
            if (testDef(code)) {
                return defs_1.default[code];
            }
            if (testWKT(code)) {
                var out = wkt_parser_1.default(code);
                // test of spetial case, due to this being a very common and often malformed
                if (checkMercator(out)) {
                    return defs_1.default['EPSG:3857'];
                }
                var maybeProjStr = checkProjStr(out);
                if (maybeProjStr) {
                    return projString_1.default(maybeProjStr);
                }
                return out;
            }
            if (testProj(code)) {
                return projString_1.default(code);
            }
        }
        else {
            return code;
        }
    }
    exports.default = parse;
},
/* proj4/lib/defs.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var global_1 = tslib_1.__importDefault(require(157) /* ./global */);
    var projString_1 = tslib_1.__importDefault(require(158) /* ./projString */);
    var wkt_parser_1 = tslib_1.__importDefault(require(163) /* wkt-parser */);
    function defs(name) {
        /*global console*/
        var that = this;
        if (arguments.length === 2) {
            var def = arguments[1];
            if (typeof def === 'string') {
                if (def.charAt(0) === '+') {
                    defs[name] = projString_1.default(arguments[1]);
                }
                else {
                    defs[name] = wkt_parser_1.default(arguments[1]);
                }
            }
            else {
                defs[name] = def;
            }
        }
        else if (arguments.length === 1) {
            if (Array.isArray(name)) {
                return name.map(function (v) {
                    if (Array.isArray(v)) {
                        defs.apply(that, v);
                    }
                    else {
                        defs(v);
                    }
                });
            }
            else if (typeof name === 'string') {
                if (name in defs) {
                    return defs[name];
                }
            }
            else if ('EPSG' in name) {
                defs['EPSG:' + name.EPSG] = name;
            }
            else if ('ESRI' in name) {
                defs['ESRI:' + name.ESRI] = name;
            }
            else if ('IAU2000' in name) {
                defs['IAU2000:' + name.IAU2000] = name;
            }
            else {
                console.log(name);
            }
            return;
        }
    }
    global_1.default(defs);
    exports.default = defs;
},
/* proj4/lib/global.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function default_1(defs) {
        defs('EPSG:4326', "+title=WGS 84 (long/lat) +proj=longlat +ellps=WGS84 +datum=WGS84 +units=degrees");
        defs('EPSG:4269', "+title=NAD83 (long/lat) +proj=longlat +a=6378137.0 +b=6356752.31414036 +ellps=GRS80 +datum=NAD83 +units=degrees");
        defs('EPSG:3857', "+title=WGS 84 / Pseudo-Mercator +proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +no_defs");
        defs.WGS84 = defs['EPSG:4326'];
        defs['EPSG:3785'] = defs['EPSG:3857']; // maintain backward compat, official code is 3857
        defs.GOOGLE = defs['EPSG:3857'];
        defs['EPSG:900913'] = defs['EPSG:3857'];
        defs['EPSG:102113'] = defs['EPSG:3857'];
    }
    exports.default = default_1;
},
/* proj4/lib/projString.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var values_1 = require(159) /* ./constants/values */;
    var PrimeMeridian_1 = tslib_1.__importDefault(require(160) /* ./constants/PrimeMeridian */);
    var units_1 = tslib_1.__importDefault(require(161) /* ./constants/units */);
    var match_1 = tslib_1.__importDefault(require(162) /* ./match */);
    function default_1(defData) {
        var self = {};
        var paramObj = defData.split('+').map(function (v) {
            return v.trim();
        }).filter(function (a) {
            return a;
        }).reduce(function (p, a) {
            var split = a.split('=');
            split.push(true);
            p[split[0].toLowerCase()] = split[1];
            return p;
        }, {});
        var paramName, paramVal, paramOutname;
        var params = {
            proj: 'projName',
            datum: 'datumCode',
            rf: function (v) {
                self.rf = parseFloat(v);
            },
            lat_0: function (v) {
                self.lat0 = v * values_1.D2R;
            },
            lat_1: function (v) {
                self.lat1 = v * values_1.D2R;
            },
            lat_2: function (v) {
                self.lat2 = v * values_1.D2R;
            },
            lat_ts: function (v) {
                self.lat_ts = v * values_1.D2R;
            },
            lon_0: function (v) {
                self.long0 = v * values_1.D2R;
            },
            lon_1: function (v) {
                self.long1 = v * values_1.D2R;
            },
            lon_2: function (v) {
                self.long2 = v * values_1.D2R;
            },
            alpha: function (v) {
                self.alpha = parseFloat(v) * values_1.D2R;
            },
            lonc: function (v) {
                self.longc = v * values_1.D2R;
            },
            x_0: function (v) {
                self.x0 = parseFloat(v);
            },
            y_0: function (v) {
                self.y0 = parseFloat(v);
            },
            k_0: function (v) {
                self.k0 = parseFloat(v);
            },
            k: function (v) {
                self.k0 = parseFloat(v);
            },
            a: function (v) {
                self.a = parseFloat(v);
            },
            b: function (v) {
                self.b = parseFloat(v);
            },
            r_a: function () {
                self.R_A = true;
            },
            zone: function (v) {
                self.zone = parseInt(v, 10);
            },
            south: function () {
                self.utmSouth = true;
            },
            towgs84: function (v) {
                self.datum_params = v.split(",").map(function (a) {
                    return parseFloat(a);
                });
            },
            to_meter: function (v) {
                self.to_meter = parseFloat(v);
            },
            units: function (v) {
                self.units = v;
                var unit = match_1.default(units_1.default, v);
                if (unit) {
                    self.to_meter = unit.to_meter;
                }
            },
            from_greenwich: function (v) {
                self.from_greenwich = v * values_1.D2R;
            },
            pm: function (v) {
                var pm = match_1.default(PrimeMeridian_1.default, v);
                self.from_greenwich = (pm ? pm : parseFloat(v)) * values_1.D2R;
            },
            nadgrids: function (v) {
                if (v === '@null') {
                    self.datumCode = 'none';
                }
                else {
                    self.nadgrids = v;
                }
            },
            axis: function (v) {
                var legalAxis = "ewnsud";
                if (v.length === 3 && legalAxis.indexOf(v.substr(0, 1)) !== -1 && legalAxis.indexOf(v.substr(1, 1)) !== -1 && legalAxis.indexOf(v.substr(2, 1)) !== -1) {
                    self.axis = v;
                }
            }
        };
        for (paramName in paramObj) {
            paramVal = paramObj[paramName];
            if (paramName in params) {
                paramOutname = params[paramName];
                if (typeof paramOutname === 'function') {
                    paramOutname(paramVal);
                }
                else {
                    self[paramOutname] = paramVal;
                }
            }
            else {
                self[paramName] = paramVal;
            }
        }
        if (typeof self.datumCode === 'string' && self.datumCode !== "WGS84") {
            self.datumCode = self.datumCode.toLowerCase();
        }
        return self;
    }
    exports.default = default_1;
},
/* proj4/lib/constants/values.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.PJD_3PARAM = 1;
    exports.PJD_7PARAM = 2;
    exports.PJD_WGS84 = 4; // WGS84 or equivalent
    exports.PJD_NODATUM = 5; // WGS84 or equivalent
    exports.SEC_TO_RAD = 4.84813681109535993589914102357e-6;
    exports.HALF_PI = Math.PI / 2;
    // ellipoid pj_set_ell.c
    exports.SIXTH = 0.1666666666666666667;
    /* 1/6 */
    exports.RA4 = 0.04722222222222222222;
    /* 17/360 */
    exports.RA6 = 0.02215608465608465608;
    exports.EPSLN = 1.0e-10;
    // you'd think you could use Number.EPSILON above but that makes
    // Mollweide get into an infinate loop.
    exports.D2R = 0.01745329251994329577;
    exports.R2D = 57.29577951308232088;
    exports.FORTPI = Math.PI / 4;
    exports.TWO_PI = Math.PI * 2;
    // SPI is slightly greater than Math.PI, so values that exceed the -180..180
    // degree range by a tiny amount don't get wrapped. This prevents points that
    // have drifted from their original location along the 180th meridian (due to
    // floating point error) from changing their sign.
    exports.SPI = 3.14159265359;
},
/* proj4/lib/constants/PrimeMeridian.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var exports$1 = {};
    exports.default = exports$1;
    exports$1.greenwich = 0.0; //"0dE",
    exports$1.lisbon = -9.131906111111; //"9d07'54.862\"W",
    exports$1.paris = 2.337229166667; //"2d20'14.025\"E",
    exports$1.bogota = -74.080916666667; //"74d04'51.3\"W",
    exports$1.madrid = -3.687938888889; //"3d41'16.58\"W",
    exports$1.rome = 12.452333333333; //"12d27'8.4\"E",
    exports$1.bern = 7.439583333333; //"7d26'22.5\"E",
    exports$1.jakarta = 106.807719444444; //"106d48'27.79\"E",
    exports$1.ferro = -17.666666666667; //"17d40'W",
    exports$1.brussels = 4.367975; //"4d22'4.71\"E",
    exports$1.stockholm = 18.058277777778; //"18d3'29.8\"E",
    exports$1.athens = 23.7163375; //"23d42'58.815\"E",
    exports$1.oslo = 10.722916666667; //"10d43'22.5\"E"
},
/* proj4/lib/constants/units.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = {
        ft: { to_meter: 0.3048 },
        'us-ft': { to_meter: 1200 / 3937 }
    };
},
/* proj4/lib/match.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var ignoredChar = /[\s_\-\/\(\)]/g;
    function match(obj, key) {
        if (obj[key]) {
            return obj[key];
        }
        var keys = Object.keys(obj);
        var lkey = key.toLowerCase().replace(ignoredChar, '');
        var i = -1;
        var testkey, processedKey;
        while (++i < keys.length) {
            testkey = keys[i];
            processedKey = testkey.toLowerCase().replace(ignoredChar, '');
            if (processedKey === lkey) {
                return obj[testkey];
            }
        }
    }
    exports.default = match;
},
/* wkt-parser/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var D2R = 0.01745329251994329577;
    var parser_1 = tslib_1.__importDefault(require(164) /* ./parser */);
    var process_1 = require(165) /* ./process */;
    function rename(obj, params) {
        var outName = params[0];
        var inName = params[1];
        if (!(outName in obj) && (inName in obj)) {
            obj[outName] = obj[inName];
            if (params.length === 3) {
                obj[outName] = params[2](obj[outName]);
            }
        }
    }
    function d2r(input) {
        return input * D2R;
    }
    function cleanWKT(wkt) {
        if (wkt.type === 'GEOGCS') {
            wkt.projName = 'longlat';
        }
        else if (wkt.type === 'LOCAL_CS') {
            wkt.projName = 'identity';
            wkt.local = true;
        }
        else {
            if (typeof wkt.PROJECTION === 'object') {
                wkt.projName = Object.keys(wkt.PROJECTION)[0];
            }
            else {
                wkt.projName = wkt.PROJECTION;
            }
        }
        if (wkt.AXIS) {
            var axisOrder = '';
            for (var i = 0, ii = wkt.AXIS.length; i < ii; ++i) {
                var axis = wkt.AXIS[i];
                var descriptor = axis[0].toLowerCase();
                if (descriptor.indexOf('north') !== -1) {
                    axisOrder += 'n';
                }
                else if (descriptor.indexOf('south') !== -1) {
                    axisOrder += 's';
                }
                else if (descriptor.indexOf('east') !== -1) {
                    axisOrder += 'e';
                }
                else if (descriptor.indexOf('west') !== -1) {
                    axisOrder += 'w';
                }
            }
            if (axisOrder.length === 2) {
                axisOrder += 'u';
            }
            if (axisOrder.length === 3) {
                wkt.axis = axisOrder;
            }
        }
        if (wkt.UNIT) {
            wkt.units = wkt.UNIT.name.toLowerCase();
            if (wkt.units === 'metre') {
                wkt.units = 'meter';
            }
            if (wkt.UNIT.convert) {
                if (wkt.type === 'GEOGCS') {
                    if (wkt.DATUM && wkt.DATUM.SPHEROID) {
                        wkt.to_meter = wkt.UNIT.convert * wkt.DATUM.SPHEROID.a;
                    }
                }
                else {
                    wkt.to_meter = wkt.UNIT.convert;
                }
            }
        }
        var geogcs = wkt.GEOGCS;
        if (wkt.type === 'GEOGCS') {
            geogcs = wkt;
        }
        if (geogcs) {
            //if(wkt.GEOGCS.PRIMEM&&wkt.GEOGCS.PRIMEM.convert){
            //  wkt.from_greenwich=wkt.GEOGCS.PRIMEM.convert*D2R;
            //}
            if (geogcs.DATUM) {
                wkt.datumCode = geogcs.DATUM.name.toLowerCase();
            }
            else {
                wkt.datumCode = geogcs.name.toLowerCase();
            }
            if (wkt.datumCode.slice(0, 2) === 'd_') {
                wkt.datumCode = wkt.datumCode.slice(2);
            }
            if (wkt.datumCode === 'new_zealand_geodetic_datum_1949' || wkt.datumCode === 'new_zealand_1949') {
                wkt.datumCode = 'nzgd49';
            }
            if (wkt.datumCode === 'wgs_1984' || wkt.datumCode === 'world_geodetic_system_1984') {
                if (wkt.PROJECTION === 'Mercator_Auxiliary_Sphere') {
                    wkt.sphere = true;
                }
                wkt.datumCode = 'wgs84';
            }
            if (wkt.datumCode.slice(-6) === '_ferro') {
                wkt.datumCode = wkt.datumCode.slice(0, -6);
            }
            if (wkt.datumCode.slice(-8) === '_jakarta') {
                wkt.datumCode = wkt.datumCode.slice(0, -8);
            }
            if (~wkt.datumCode.indexOf('belge')) {
                wkt.datumCode = 'rnb72';
            }
            if (geogcs.DATUM && geogcs.DATUM.SPHEROID) {
                wkt.ellps = geogcs.DATUM.SPHEROID.name.replace('_19', '').replace(/[Cc]larke\_18/, 'clrk');
                if (wkt.ellps.toLowerCase().slice(0, 13) === 'international') {
                    wkt.ellps = 'intl';
                }
                wkt.a = geogcs.DATUM.SPHEROID.a;
                wkt.rf = parseFloat(geogcs.DATUM.SPHEROID.rf, 10);
            }
            if (geogcs.DATUM && geogcs.DATUM.TOWGS84) {
                wkt.datum_params = geogcs.DATUM.TOWGS84;
            }
            if (~wkt.datumCode.indexOf('osgb_1936')) {
                wkt.datumCode = 'osgb36';
            }
            if (~wkt.datumCode.indexOf('osni_1952')) {
                wkt.datumCode = 'osni52';
            }
            if (~wkt.datumCode.indexOf('tm65')
                || ~wkt.datumCode.indexOf('geodetic_datum_of_1965')) {
                wkt.datumCode = 'ire65';
            }
            if (wkt.datumCode === 'ch1903+') {
                wkt.datumCode = 'ch1903';
            }
            if (~wkt.datumCode.indexOf('israel')) {
                wkt.datumCode = 'isr93';
            }
        }
        if (wkt.b && !isFinite(wkt.b)) {
            wkt.b = wkt.a;
        }
        function toMeter(input) {
            var ratio = wkt.to_meter || 1;
            return input * ratio;
        }
        var renamer = function (a) {
            return rename(wkt, a);
        };
        var list = [
            ['standard_parallel_1', 'Standard_Parallel_1'],
            ['standard_parallel_2', 'Standard_Parallel_2'],
            ['false_easting', 'False_Easting'],
            ['false_northing', 'False_Northing'],
            ['central_meridian', 'Central_Meridian'],
            ['latitude_of_origin', 'Latitude_Of_Origin'],
            ['latitude_of_origin', 'Central_Parallel'],
            ['scale_factor', 'Scale_Factor'],
            ['k0', 'scale_factor'],
            ['latitude_of_center', 'Latitude_Of_Center'],
            ['latitude_of_center', 'Latitude_of_center'],
            ['lat0', 'latitude_of_center', d2r],
            ['longitude_of_center', 'Longitude_Of_Center'],
            ['longitude_of_center', 'Longitude_of_center'],
            ['longc', 'longitude_of_center', d2r],
            ['x0', 'false_easting', toMeter],
            ['y0', 'false_northing', toMeter],
            ['long0', 'central_meridian', d2r],
            ['lat0', 'latitude_of_origin', d2r],
            ['lat0', 'standard_parallel_1', d2r],
            ['lat1', 'standard_parallel_1', d2r],
            ['lat2', 'standard_parallel_2', d2r],
            ['azimuth', 'Azimuth'],
            ['alpha', 'azimuth', d2r],
            ['srsCode', 'name']
        ];
        list.forEach(renamer);
        if (!wkt.long0 && wkt.longc && (wkt.projName === 'Albers_Conic_Equal_Area' || wkt.projName === 'Lambert_Azimuthal_Equal_Area')) {
            wkt.long0 = wkt.longc;
        }
        if (!wkt.lat_ts && wkt.lat1 && (wkt.projName === 'Stereographic_South_Pole' || wkt.projName === 'Polar Stereographic (variant B)')) {
            wkt.lat0 = d2r(wkt.lat1 > 0 ? 90 : -90);
            wkt.lat_ts = wkt.lat1;
        }
    }
    function default_1(wkt) {
        var lisp = parser_1.default(wkt);
        var type = lisp.shift();
        var name = lisp.shift();
        lisp.unshift(['name', name]);
        lisp.unshift(['type', type]);
        var obj = {};
        process_1.sExpr(lisp, obj);
        cleanWKT(obj);
        return obj;
    }
    exports.default = default_1;
},
/* wkt-parser/parser.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = parseString;
    var NEUTRAL = 1;
    var KEYWORD = 2;
    var NUMBER = 3;
    var QUOTED = 4;
    var AFTERQUOTE = 5;
    var ENDED = -1;
    var whitespace = /\s/;
    var latin = /[A-Za-z]/;
    var keyword = /[A-Za-z84]/;
    var endThings = /[,\]]/;
    var digets = /[\d\.E\-\+]/;
    // const ignoredChar = /[\s_\-\/\(\)]/g;
    function Parser(text) {
        if (typeof text !== 'string') {
            throw new Error('not a string');
        }
        this.text = text.trim();
        this.level = 0;
        this.place = 0;
        this.root = null;
        this.stack = [];
        this.currentObject = null;
        this.state = NEUTRAL;
    }
    Parser.prototype.readCharicter = function () {
        var char = this.text[this.place++];
        if (this.state !== QUOTED) {
            while (whitespace.test(char)) {
                if (this.place >= this.text.length) {
                    return;
                }
                char = this.text[this.place++];
            }
        }
        switch (this.state) {
            case NEUTRAL:
                return this.neutral(char);
            case KEYWORD:
                return this.keyword(char);
            case QUOTED:
                return this.quoted(char);
            case AFTERQUOTE:
                return this.afterquote(char);
            case NUMBER:
                return this.number(char);
            case ENDED:
                return;
        }
    };
    Parser.prototype.afterquote = function (char) {
        if (char === '"') {
            this.word += '"';
            this.state = QUOTED;
            return;
        }
        if (endThings.test(char)) {
            this.word = this.word.trim();
            this.afterItem(char);
            return;
        }
        throw new Error('havn\'t handled "' + char + '" in afterquote yet, index ' + this.place);
    };
    Parser.prototype.afterItem = function (char) {
        if (char === ',') {
            if (this.word !== null) {
                this.currentObject.push(this.word);
            }
            this.word = null;
            this.state = NEUTRAL;
            return;
        }
        if (char === ']') {
            this.level--;
            if (this.word !== null) {
                this.currentObject.push(this.word);
                this.word = null;
            }
            this.state = NEUTRAL;
            this.currentObject = this.stack.pop();
            if (!this.currentObject) {
                this.state = ENDED;
            }
            return;
        }
    };
    Parser.prototype.number = function (char) {
        if (digets.test(char)) {
            this.word += char;
            return;
        }
        if (endThings.test(char)) {
            this.word = parseFloat(this.word);
            this.afterItem(char);
            return;
        }
        throw new Error('havn\'t handled "' + char + '" in number yet, index ' + this.place);
    };
    Parser.prototype.quoted = function (char) {
        if (char === '"') {
            this.state = AFTERQUOTE;
            return;
        }
        this.word += char;
        return;
    };
    Parser.prototype.keyword = function (char) {
        if (keyword.test(char)) {
            this.word += char;
            return;
        }
        if (char === '[') {
            var newObjects = [];
            newObjects.push(this.word);
            this.level++;
            if (this.root === null) {
                this.root = newObjects;
            }
            else {
                this.currentObject.push(newObjects);
            }
            this.stack.push(this.currentObject);
            this.currentObject = newObjects;
            this.state = NEUTRAL;
            return;
        }
        if (endThings.test(char)) {
            this.afterItem(char);
            return;
        }
        throw new Error('havn\'t handled "' + char + '" in keyword yet, index ' + this.place);
    };
    Parser.prototype.neutral = function (char) {
        if (latin.test(char)) {
            this.word = char;
            this.state = KEYWORD;
            return;
        }
        if (char === '"') {
            this.word = '';
            this.state = QUOTED;
            return;
        }
        if (digets.test(char)) {
            this.word = char;
            this.state = NUMBER;
            return;
        }
        if (endThings.test(char)) {
            this.afterItem(char);
            return;
        }
        throw new Error('havn\'t handled "' + char + '" in neutral yet, index ' + this.place);
    };
    Parser.prototype.output = function () {
        while (this.place < this.text.length) {
            this.readCharicter();
        }
        if (this.state === ENDED) {
            return this.root;
        }
        throw new Error('unable to parse string "' + this.text + '". State is ' + this.state);
    };
    function parseString(txt) {
        var parser = new Parser(txt);
        return parser.output();
    }
},
/* wkt-parser/process.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function mapit(obj, key, value) {
        if (Array.isArray(key)) {
            value.unshift(key);
            key = null;
        }
        var thing = key ? {} : obj;
        var out = value.reduce(function (newObj, item) {
            sExpr(item, newObj);
            return newObj;
        }, thing);
        if (key) {
            obj[key] = out;
        }
    }
    function sExpr(v, obj) {
        if (!Array.isArray(v)) {
            obj[v] = true;
            return;
        }
        var key = v.shift();
        if (key === 'PARAMETER') {
            key = v.shift();
        }
        if (v.length === 1) {
            if (Array.isArray(v[0])) {
                obj[key] = {};
                sExpr(v[0], obj[key]);
                return;
            }
            obj[key] = v[0];
            return;
        }
        if (!v.length) {
            obj[key] = true;
            return;
        }
        if (key === 'TOWGS84') {
            obj[key] = v;
            return;
        }
        if (key === 'AXIS') {
            if (!(key in obj)) {
                obj[key] = [];
            }
            obj[key].push(v);
            return;
        }
        if (!Array.isArray(key)) {
            obj[key] = {};
        }
        var i;
        switch (key) {
            case 'UNIT':
            case 'PRIMEM':
            case 'VERT_DATUM':
                obj[key] = {
                    name: v[0].toLowerCase(),
                    convert: v[1]
                };
                if (v.length === 3) {
                    sExpr(v[2], obj[key]);
                }
                return;
            case 'SPHEROID':
            case 'ELLIPSOID':
                obj[key] = {
                    name: v[0],
                    a: v[1],
                    rf: v[2]
                };
                if (v.length === 4) {
                    sExpr(v[3], obj[key]);
                }
                return;
            case 'PROJECTEDCRS':
            case 'PROJCRS':
            case 'GEOGCS':
            case 'GEOCCS':
            case 'PROJCS':
            case 'LOCAL_CS':
            case 'GEODCRS':
            case 'GEODETICCRS':
            case 'GEODETICDATUM':
            case 'EDATUM':
            case 'ENGINEERINGDATUM':
            case 'VERT_CS':
            case 'VERTCRS':
            case 'VERTICALCRS':
            case 'COMPD_CS':
            case 'COMPOUNDCRS':
            case 'ENGINEERINGCRS':
            case 'ENGCRS':
            case 'FITTED_CS':
            case 'LOCAL_DATUM':
            case 'DATUM':
                v[0] = ['name', v[0]];
                mapit(obj, key, v);
                return;
            default:
                i = -1;
                while (++i < v.length) {
                    if (!Array.isArray(v[i])) {
                        return sExpr(v, obj[key]);
                    }
                }
                return mapit(obj, key, v);
        }
    }
    exports.sExpr = sExpr;
},
/* proj4/lib/extend.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function default_1(destination, source) {
        destination = destination || {};
        var value, property;
        if (!source) {
            return destination;
        }
        for (property in source) {
            value = source[property];
            if (value !== undefined) {
                destination[property] = value;
            }
        }
        return destination;
    }
    exports.default = default_1;
},
/* proj4/lib/projections.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var merc_1 = tslib_1.__importDefault(require(168) /* ./projections/merc */);
    var longlat_1 = tslib_1.__importDefault(require(174) /* ./projections/longlat */);
    var projs = [merc_1.default, longlat_1.default];
    var names = {};
    var projStore = [];
    function add(proj, i) {
        var len = projStore.length;
        if (!proj.names) {
            console.log(i);
            return true;
        }
        projStore[len] = proj;
        proj.names.forEach(function (n) {
            names[n.toLowerCase()] = len;
        });
        return this;
    }
    exports.add = add;
    function get(name) {
        if (!name) {
            return false;
        }
        var n = name.toLowerCase();
        if (typeof names[n] !== 'undefined' && projStore[names[n]]) {
            return projStore[names[n]];
        }
    }
    exports.get = get;
    function start() {
        projs.forEach(add);
    }
    exports.start = start;
    exports.default = {
        start: start,
        add: add,
        get: get
    };
},
/* proj4/lib/projections/merc.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var msfnz_1 = tslib_1.__importDefault(require(169) /* ../common/msfnz */);
    var adjust_lon_1 = tslib_1.__importDefault(require(170) /* ../common/adjust_lon */);
    var tsfnz_1 = tslib_1.__importDefault(require(172) /* ../common/tsfnz */);
    var phi2z_1 = tslib_1.__importDefault(require(173) /* ../common/phi2z */);
    var values_1 = require(159) /* ../constants/values */;
    function init() {
        var con = this.b / this.a;
        this.es = 1 - con * con;
        if (!('x0' in this)) {
            this.x0 = 0;
        }
        if (!('y0' in this)) {
            this.y0 = 0;
        }
        this.e = Math.sqrt(this.es);
        if (this.lat_ts) {
            if (this.sphere) {
                this.k0 = Math.cos(this.lat_ts);
            }
            else {
                this.k0 = msfnz_1.default(this.e, Math.sin(this.lat_ts), Math.cos(this.lat_ts));
            }
        }
        else {
            if (!this.k0) {
                if (this.k) {
                    this.k0 = this.k;
                }
                else {
                    this.k0 = 1;
                }
            }
        }
    }
    exports.init = init;
    /* Mercator forward equations--mapping lat,long to x,y
      --------------------------------------------------*/
    function forward(p) {
        var lon = p.x;
        var lat = p.y;
        // convert to radians
        if (lat * values_1.R2D > 90 && lat * values_1.R2D < -90 && lon * values_1.R2D > 180 && lon * values_1.R2D < -180) {
            return null;
        }
        var x, y;
        if (Math.abs(Math.abs(lat) - values_1.HALF_PI) <= values_1.EPSLN) {
            return null;
        }
        else {
            if (this.sphere) {
                x = this.x0 + this.a * this.k0 * adjust_lon_1.default(lon - this.long0);
                y = this.y0 + this.a * this.k0 * Math.log(Math.tan(values_1.FORTPI + 0.5 * lat));
            }
            else {
                var sinphi = Math.sin(lat);
                var ts = tsfnz_1.default(this.e, lat, sinphi);
                x = this.x0 + this.a * this.k0 * adjust_lon_1.default(lon - this.long0);
                y = this.y0 - this.a * this.k0 * Math.log(ts);
            }
            p.x = x;
            p.y = y;
            return p;
        }
    }
    exports.forward = forward;
    /* Mercator inverse equations--mapping x,y to lat/long
      --------------------------------------------------*/
    function inverse(p) {
        var x = p.x - this.x0;
        var y = p.y - this.y0;
        var lon, lat;
        if (this.sphere) {
            lat = values_1.HALF_PI - 2 * Math.atan(Math.exp(-y / (this.a * this.k0)));
        }
        else {
            var ts = Math.exp(-y / (this.a * this.k0));
            lat = phi2z_1.default(this.e, ts);
            if (lat === -9999) {
                return null;
            }
        }
        lon = adjust_lon_1.default(this.long0 + x / (this.a * this.k0));
        p.x = lon;
        p.y = lat;
        return p;
    }
    exports.inverse = inverse;
    exports.names = ["Mercator", "Popular Visualisation Pseudo Mercator", "Mercator_1SP", "Mercator_Auxiliary_Sphere", "merc"];
    exports.default = {
        init: init,
        forward: forward,
        inverse: inverse,
        names: exports.names
    };
},
/* proj4/lib/common/msfnz.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function default_1(eccent, sinphi, cosphi) {
        var con = eccent * sinphi;
        return cosphi / (Math.sqrt(1 - con * con));
    }
    exports.default = default_1;
},
/* proj4/lib/common/adjust_lon.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var values_1 = require(159) /* ../constants/values */;
    var sign_1 = tslib_1.__importDefault(require(171) /* ./sign */);
    function default_1(x) {
        return (Math.abs(x) <= values_1.SPI) ? x : (x - (sign_1.default(x) * values_1.TWO_PI));
    }
    exports.default = default_1;
},
/* proj4/lib/common/sign.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function default_1(x) {
        return x < 0 ? -1 : 1;
    }
    exports.default = default_1;
},
/* proj4/lib/common/tsfnz.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var values_1 = require(159) /* ../constants/values */;
    function default_1(eccent, phi, sinphi) {
        var con = eccent * sinphi;
        var com = 0.5 * eccent;
        con = Math.pow(((1 - con) / (1 + con)), com);
        return (Math.tan(0.5 * (values_1.HALF_PI - phi)) / con);
    }
    exports.default = default_1;
},
/* proj4/lib/common/phi2z.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var values_1 = require(159) /* ../constants/values */;
    function default_1(eccent, ts) {
        var eccnth = 0.5 * eccent;
        var con, dphi;
        var phi = values_1.HALF_PI - 2 * Math.atan(ts);
        for (var i = 0; i <= 15; i++) {
            con = eccent * Math.sin(phi);
            dphi = values_1.HALF_PI - 2 * Math.atan(ts * (Math.pow(((1 - con) / (1 + con)), eccnth))) - phi;
            phi += dphi;
            if (Math.abs(dphi) <= 0.0000000001) {
                return phi;
            }
        }
        //console.log("phi2z has NoConvergence");
        return -9999;
    }
    exports.default = default_1;
},
/* proj4/lib/projections/longlat.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function init() {
        //no-op for longlat
    }
    exports.init = init;
    function identity(pt) {
        return pt;
    }
    exports.forward = identity;
    exports.inverse = identity;
    exports.names = ["longlat", "identity"];
    exports.default = {
        init: init,
        forward: identity,
        inverse: identity,
        names: exports.names
    };
},
/* proj4/lib/deriveConstants.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var values_1 = require(159) /* ./constants/values */;
    var Ellipsoid_1 = tslib_1.__importStar(require(176) /* ./constants/Ellipsoid */);
    var match_1 = tslib_1.__importDefault(require(162) /* ./match */);
    function eccentricity(a, b, rf, R_A) {
        var a2 = a * a; // used in geocentric
        var b2 = b * b; // used in geocentric
        var es = (a2 - b2) / a2; // e ^ 2
        var e = 0;
        if (R_A) {
            a *= 1 - es * (values_1.SIXTH + es * (values_1.RA4 + es * values_1.RA6));
            a2 = a * a;
            es = 0;
        }
        else {
            e = Math.sqrt(es); // eccentricity
        }
        var ep2 = (a2 - b2) / b2; // used in geocentric
        return {
            es: es,
            e: e,
            ep2: ep2
        };
    }
    exports.eccentricity = eccentricity;
    function sphere(a, b, rf, ellps, sphere) {
        if (!a) { // do we have an ellipsoid?
            var ellipse = match_1.default(Ellipsoid_1.default, ellps);
            if (!ellipse) {
                ellipse = Ellipsoid_1.WGS84;
            }
            a = ellipse.a;
            b = ellipse.b;
            rf = ellipse.rf;
        }
        if (rf && !b) {
            b = (1.0 - 1.0 / rf) * a;
        }
        if (rf === 0 || Math.abs(a - b) < values_1.EPSLN) {
            sphere = true;
            b = a;
        }
        return {
            a: a,
            b: b,
            rf: rf,
            sphere: sphere
        };
    }
    exports.sphere = sphere;
},
/* proj4/lib/constants/Ellipsoid.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var exports$1 = {};
    exports.default = exports$1;
    exports$1.MERIT = {
        a: 6378137.0,
        rf: 298.257,
        ellipseName: "MERIT 1983"
    };
    exports$1.SGS85 = {
        a: 6378136.0,
        rf: 298.257,
        ellipseName: "Soviet Geodetic System 85"
    };
    exports$1.GRS80 = {
        a: 6378137.0,
        rf: 298.257222101,
        ellipseName: "GRS 1980(IUGG, 1980)"
    };
    exports$1.IAU76 = {
        a: 6378140.0,
        rf: 298.257,
        ellipseName: "IAU 1976"
    };
    exports$1.airy = {
        a: 6377563.396,
        b: 6356256.910,
        ellipseName: "Airy 1830"
    };
    exports$1.APL4 = {
        a: 6378137,
        rf: 298.25,
        ellipseName: "Appl. Physics. 1965"
    };
    exports$1.NWL9D = {
        a: 6378145.0,
        rf: 298.25,
        ellipseName: "Naval Weapons Lab., 1965"
    };
    exports$1.mod_airy = {
        a: 6377340.189,
        b: 6356034.446,
        ellipseName: "Modified Airy"
    };
    exports$1.andrae = {
        a: 6377104.43,
        rf: 300.0,
        ellipseName: "Andrae 1876 (Den., Iclnd.)"
    };
    exports$1.aust_SA = {
        a: 6378160.0,
        rf: 298.25,
        ellipseName: "Australian Natl & S. Amer. 1969"
    };
    exports$1.GRS67 = {
        a: 6378160.0,
        rf: 298.2471674270,
        ellipseName: "GRS 67(IUGG 1967)"
    };
    exports$1.bessel = {
        a: 6377397.155,
        rf: 299.1528128,
        ellipseName: "Bessel 1841"
    };
    exports$1.bess_nam = {
        a: 6377483.865,
        rf: 299.1528128,
        ellipseName: "Bessel 1841 (Namibia)"
    };
    exports$1.clrk66 = {
        a: 6378206.4,
        b: 6356583.8,
        ellipseName: "Clarke 1866"
    };
    exports$1.clrk80 = {
        a: 6378249.145,
        rf: 293.4663,
        ellipseName: "Clarke 1880 mod."
    };
    exports$1.clrk58 = {
        a: 6378293.645208759,
        rf: 294.2606763692654,
        ellipseName: "Clarke 1858"
    };
    exports$1.CPM = {
        a: 6375738.7,
        rf: 334.29,
        ellipseName: "Comm. des Poids et Mesures 1799"
    };
    exports$1.delmbr = {
        a: 6376428.0,
        rf: 311.5,
        ellipseName: "Delambre 1810 (Belgium)"
    };
    exports$1.engelis = {
        a: 6378136.05,
        rf: 298.2566,
        ellipseName: "Engelis 1985"
    };
    exports$1.evrst30 = {
        a: 6377276.345,
        rf: 300.8017,
        ellipseName: "Everest 1830"
    };
    exports$1.evrst48 = {
        a: 6377304.063,
        rf: 300.8017,
        ellipseName: "Everest 1948"
    };
    exports$1.evrst56 = {
        a: 6377301.243,
        rf: 300.8017,
        ellipseName: "Everest 1956"
    };
    exports$1.evrst69 = {
        a: 6377295.664,
        rf: 300.8017,
        ellipseName: "Everest 1969"
    };
    exports$1.evrstSS = {
        a: 6377298.556,
        rf: 300.8017,
        ellipseName: "Everest (Sabah & Sarawak)"
    };
    exports$1.fschr60 = {
        a: 6378166.0,
        rf: 298.3,
        ellipseName: "Fischer (Mercury Datum) 1960"
    };
    exports$1.fschr60m = {
        a: 6378155.0,
        rf: 298.3,
        ellipseName: "Fischer 1960"
    };
    exports$1.fschr68 = {
        a: 6378150.0,
        rf: 298.3,
        ellipseName: "Fischer 1968"
    };
    exports$1.helmert = {
        a: 6378200.0,
        rf: 298.3,
        ellipseName: "Helmert 1906"
    };
    exports$1.hough = {
        a: 6378270.0,
        rf: 297.0,
        ellipseName: "Hough"
    };
    exports$1.intl = {
        a: 6378388.0,
        rf: 297.0,
        ellipseName: "International 1909 (Hayford)"
    };
    exports$1.kaula = {
        a: 6378163.0,
        rf: 298.24,
        ellipseName: "Kaula 1961"
    };
    exports$1.lerch = {
        a: 6378139.0,
        rf: 298.257,
        ellipseName: "Lerch 1979"
    };
    exports$1.mprts = {
        a: 6397300.0,
        rf: 191.0,
        ellipseName: "Maupertius 1738"
    };
    exports$1.new_intl = {
        a: 6378157.5,
        b: 6356772.2,
        ellipseName: "New International 1967"
    };
    exports$1.plessis = {
        a: 6376523.0,
        rf: 6355863.0,
        ellipseName: "Plessis 1817 (France)"
    };
    exports$1.krass = {
        a: 6378245.0,
        rf: 298.3,
        ellipseName: "Krassovsky, 1942"
    };
    exports$1.SEasia = {
        a: 6378155.0,
        b: 6356773.3205,
        ellipseName: "Southeast Asia"
    };
    exports$1.walbeck = {
        a: 6376896.0,
        b: 6355834.8467,
        ellipseName: "Walbeck"
    };
    exports$1.WGS60 = {
        a: 6378165.0,
        rf: 298.3,
        ellipseName: "WGS 60"
    };
    exports$1.WGS66 = {
        a: 6378145.0,
        rf: 298.25,
        ellipseName: "WGS 66"
    };
    exports$1.WGS7 = {
        a: 6378135.0,
        rf: 298.26,
        ellipseName: "WGS 72"
    };
    exports.WGS84 = exports$1.WGS84 = {
        a: 6378137.0,
        rf: 298.257223563,
        ellipseName: "WGS 84"
    };
    exports$1.sphere = {
        a: 6370997.0,
        b: 6370997.0,
        ellipseName: "Normal Sphere (r=6370997)"
    };
},
/* proj4/lib/constants/Datum.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var exports$1 = {};
    exports.default = exports$1;
    exports$1.wgs84 = {
        towgs84: "0,0,0",
        ellipse: "WGS84",
        datumName: "WGS84"
    };
    exports$1.ch1903 = {
        towgs84: "674.374,15.056,405.346",
        ellipse: "bessel",
        datumName: "swiss"
    };
    exports$1.ggrs87 = {
        towgs84: "-199.87,74.79,246.62",
        ellipse: "GRS80",
        datumName: "Greek_Geodetic_Reference_System_1987"
    };
    exports$1.nad83 = {
        towgs84: "0,0,0",
        ellipse: "GRS80",
        datumName: "North_American_Datum_1983"
    };
    exports$1.nad27 = {
        nadgrids: "@conus,@alaska,@ntv2_0.gsb,@ntv1_can.dat",
        ellipse: "clrk66",
        datumName: "North_American_Datum_1927"
    };
    exports$1.potsdam = {
        towgs84: "606.0,23.0,413.0",
        ellipse: "bessel",
        datumName: "Potsdam Rauenberg 1950 DHDN"
    };
    exports$1.carthage = {
        towgs84: "-263.0,6.0,431.0",
        ellipse: "clark80",
        datumName: "Carthage 1934 Tunisia"
    };
    exports$1.hermannskogel = {
        towgs84: "653.0,-212.0,449.0",
        ellipse: "bessel",
        datumName: "Hermannskogel"
    };
    exports$1.osni52 = {
        towgs84: "482.530,-130.596,564.557,-1.042,-0.214,-0.631,8.15",
        ellipse: "airy",
        datumName: "Irish National"
    };
    exports$1.ire65 = {
        towgs84: "482.530,-130.596,564.557,-1.042,-0.214,-0.631,8.15",
        ellipse: "mod_airy",
        datumName: "Ireland 1965"
    };
    exports$1.rassadiran = {
        towgs84: "-133.63,-157.5,-158.62",
        ellipse: "intl",
        datumName: "Rassadiran"
    };
    exports$1.nzgd49 = {
        towgs84: "59.47,-5.04,187.44,0.47,-0.1,1.024,-4.5993",
        ellipse: "intl",
        datumName: "New Zealand Geodetic Datum 1949"
    };
    exports$1.osgb36 = {
        towgs84: "446.448,-125.157,542.060,0.1502,0.2470,0.8421,-20.4894",
        ellipse: "airy",
        datumName: "Airy 1830"
    };
    exports$1.s_jtsk = {
        towgs84: "589,76,480",
        ellipse: 'bessel',
        datumName: 'S-JTSK (Ferro)'
    };
    exports$1.beduaram = {
        towgs84: '-106,-87,188',
        ellipse: 'clrk80',
        datumName: 'Beduaram'
    };
    exports$1.gunung_segara = {
        towgs84: '-403,684,41',
        ellipse: 'bessel',
        datumName: 'Gunung Segara Jakarta'
    };
    exports$1.rnb72 = {
        towgs84: "106.869,-52.2978,103.724,-0.33657,0.456955,-1.84218,1",
        ellipse: "intl",
        datumName: "Reseau National Belge 1972"
    };
},
/* proj4/lib/datum.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var values_1 = require(159) /* ./constants/values */;
    function datum(datumCode, datum_params, a, b, es, ep2) {
        var out = {};
        if (datumCode === undefined || datumCode === 'none') {
            out.datum_type = values_1.PJD_NODATUM;
        }
        else {
            out.datum_type = values_1.PJD_WGS84;
        }
        if (datum_params) {
            out.datum_params = datum_params.map(parseFloat);
            if (out.datum_params[0] !== 0 || out.datum_params[1] !== 0 || out.datum_params[2] !== 0) {
                out.datum_type = values_1.PJD_3PARAM;
            }
            if (out.datum_params.length > 3) {
                if (out.datum_params[3] !== 0 || out.datum_params[4] !== 0 || out.datum_params[5] !== 0 || out.datum_params[6] !== 0) {
                    out.datum_type = values_1.PJD_7PARAM;
                    out.datum_params[3] *= values_1.SEC_TO_RAD;
                    out.datum_params[4] *= values_1.SEC_TO_RAD;
                    out.datum_params[5] *= values_1.SEC_TO_RAD;
                    out.datum_params[6] = (out.datum_params[6] / 1000000.0) + 1.0;
                }
            }
        }
        out.a = a; //datum object also uses these values
        out.b = b;
        out.es = es;
        out.ep2 = ep2;
        return out;
    }
    exports.default = datum;
},
/* proj4/lib/transform.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var values_1 = require(159) /* ./constants/values */;
    var datum_transform_1 = tslib_1.__importDefault(require(180) /* ./datum_transform */);
    var adjust_axis_1 = tslib_1.__importDefault(require(182) /* ./adjust_axis */);
    var Proj_1 = tslib_1.__importDefault(require(154) /* ./Proj */);
    var toPoint_1 = tslib_1.__importDefault(require(183) /* ./common/toPoint */);
    var checkSanity_1 = tslib_1.__importDefault(require(184) /* ./checkSanity */);
    function checkNotWGS(source, dest) {
        return ((source.datum.datum_type === values_1.PJD_3PARAM || source.datum.datum_type === values_1.PJD_7PARAM) && dest.datumCode !== 'WGS84') || ((dest.datum.datum_type === values_1.PJD_3PARAM || dest.datum.datum_type === values_1.PJD_7PARAM) && source.datumCode !== 'WGS84');
    }
    function transform(source, dest, point) {
        var wgs84;
        if (Array.isArray(point)) {
            point = toPoint_1.default(point);
        }
        checkSanity_1.default(point);
        // Workaround for datum shifts towgs84, if either source or destination projection is not wgs84
        if (source.datum && dest.datum && checkNotWGS(source, dest)) {
            wgs84 = new Proj_1.default('WGS84');
            point = transform(source, wgs84, point);
            source = wgs84;
        }
        // DGR, 2010/11/12
        if (source.axis !== 'enu') {
            point = adjust_axis_1.default(source, false, point);
        }
        // Transform source points to long/lat, if they aren't already.
        if (source.projName === 'longlat') {
            point = {
                x: point.x * values_1.D2R,
                y: point.y * values_1.D2R,
                z: point.z || 0
            };
        }
        else {
            if (source.to_meter) {
                point = {
                    x: point.x * source.to_meter,
                    y: point.y * source.to_meter,
                    z: point.z || 0
                };
            }
            point = source.inverse(point); // Convert Cartesian to longlat
            if (!point) {
                return;
            }
        }
        // Adjust for the prime meridian if necessary
        if (source.from_greenwich) {
            point.x += source.from_greenwich;
        }
        // Convert datums if needed, and if possible.
        point = datum_transform_1.default(source.datum, dest.datum, point);
        // Adjust for the prime meridian if necessary
        if (dest.from_greenwich) {
            point = {
                x: point.x - dest.from_greenwich,
                y: point.y,
                z: point.z || 0
            };
        }
        if (dest.projName === 'longlat') {
            // convert radians to decimal degrees
            point = {
                x: point.x * values_1.R2D,
                y: point.y * values_1.R2D,
                z: point.z || 0
            };
        }
        else { // else project
            point = dest.forward(point);
            if (dest.to_meter) {
                point = {
                    x: point.x / dest.to_meter,
                    y: point.y / dest.to_meter,
                    z: point.z || 0
                };
            }
        }
        // DGR, 2010/11/12
        if (dest.axis !== 'enu') {
            return adjust_axis_1.default(dest, true, point);
        }
        return point;
    }
    exports.default = transform;
},
/* proj4/lib/datum_transform.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var values_1 = require(159) /* ./constants/values */;
    var datumUtils_1 = require(181) /* ./datumUtils */;
    function checkParams(type) {
        return (type === values_1.PJD_3PARAM || type === values_1.PJD_7PARAM);
    }
    function default_1(source, dest, point) {
        // Short cut if the datums are identical.
        if (datumUtils_1.compareDatums(source, dest)) {
            return point; // in this case, zero is sucess,
            // whereas cs_compare_datums returns 1 to indicate TRUE
            // confusing, should fix this
        }
        // Explicitly skip datum transform by setting 'datum=none' as parameter for either source or dest
        if (source.datum_type === values_1.PJD_NODATUM || dest.datum_type === values_1.PJD_NODATUM) {
            return point;
        }
        // If this datum requires grid shifts, then apply it to geodetic coordinates.
        // Do we need to go through geocentric coordinates?
        if (source.es === dest.es && source.a === dest.a && !checkParams(source.datum_type) && !checkParams(dest.datum_type)) {
            return point;
        }
        // Convert to geocentric coordinates.
        point = datumUtils_1.geodeticToGeocentric(point, source.es, source.a);
        // Convert between datums
        if (checkParams(source.datum_type)) {
            point = datumUtils_1.geocentricToWgs84(point, source.datum_type, source.datum_params);
        }
        if (checkParams(dest.datum_type)) {
            point = datumUtils_1.geocentricFromWgs84(point, dest.datum_type, dest.datum_params);
        }
        return datumUtils_1.geocentricToGeodetic(point, dest.es, dest.a, dest.b);
    }
    exports.default = default_1;
},
/* proj4/lib/datumUtils.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var values_1 = require(159) /* ./constants/values */;
    function compareDatums(source, dest) {
        if (source.datum_type !== dest.datum_type) {
            return false; // false, datums are not equal
        }
        else if (source.a !== dest.a || Math.abs(source.es - dest.es) > 0.000000000050) {
            // the tolerance for es is to ensure that GRS80 and WGS84
            // are considered identical
            return false;
        }
        else if (source.datum_type === values_1.PJD_3PARAM) {
            return (source.datum_params[0] === dest.datum_params[0] && source.datum_params[1] === dest.datum_params[1] && source.datum_params[2] === dest.datum_params[2]);
        }
        else if (source.datum_type === values_1.PJD_7PARAM) {
            return (source.datum_params[0] === dest.datum_params[0] && source.datum_params[1] === dest.datum_params[1] && source.datum_params[2] === dest.datum_params[2] && source.datum_params[3] === dest.datum_params[3] && source.datum_params[4] === dest.datum_params[4] && source.datum_params[5] === dest.datum_params[5] && source.datum_params[6] === dest.datum_params[6]);
        }
        else {
            return true; // datums are equal
        }
    } // cs_compare_datums()
    exports.compareDatums = compareDatums;
    /*
     * The function Convert_Geodetic_To_Geocentric converts geodetic coordinates
     * (latitude, longitude, and height) to geocentric coordinates (X, Y, Z),
     * according to the current ellipsoid parameters.
     *
     *    Latitude  : Geodetic latitude in radians                     (input)
     *    Longitude : Geodetic longitude in radians                    (input)
     *    Height    : Geodetic height, in meters                       (input)
     *    X         : Calculated Geocentric X coordinate, in meters    (output)
     *    Y         : Calculated Geocentric Y coordinate, in meters    (output)
     *    Z         : Calculated Geocentric Z coordinate, in meters    (output)
     *
     */
    function geodeticToGeocentric(p, es, a) {
        var Longitude = p.x;
        var Latitude = p.y;
        var Height = p.z ? p.z : 0; //Z value not always supplied
        var Rn; /*  Earth radius at location  */
        var Sin_Lat; /*  Math.sin(Latitude)  */
        var Sin2_Lat; /*  Square of Math.sin(Latitude)  */
        var Cos_Lat; /*  Math.cos(Latitude)  */
        /*
         ** Don't blow up if Latitude is just a little out of the value
         ** range as it may just be a rounding issue.  Also removed longitude
         ** test, it should be wrapped by Math.cos() and Math.sin().  NFW for PROJ.4, Sep/2001.
         */
        if (Latitude < -values_1.HALF_PI && Latitude > -1.001 * values_1.HALF_PI) {
            Latitude = -values_1.HALF_PI;
        }
        else if (Latitude > values_1.HALF_PI && Latitude < 1.001 * values_1.HALF_PI) {
            Latitude = values_1.HALF_PI;
        }
        else if (Latitude < -values_1.HALF_PI) {
            /* Latitude out of range */
            //..reportError('geocent:lat out of range:' + Latitude);
            return { x: -Infinity, y: -Infinity, z: p.z };
        }
        else if (Latitude > values_1.HALF_PI) {
            /* Latitude out of range */
            return { x: Infinity, y: Infinity, z: p.z };
        }
        if (Longitude > Math.PI) {
            Longitude -= (2 * Math.PI);
        }
        Sin_Lat = Math.sin(Latitude);
        Cos_Lat = Math.cos(Latitude);
        Sin2_Lat = Sin_Lat * Sin_Lat;
        Rn = a / (Math.sqrt(1.0e0 - es * Sin2_Lat));
        return {
            x: (Rn + Height) * Cos_Lat * Math.cos(Longitude),
            y: (Rn + Height) * Cos_Lat * Math.sin(Longitude),
            z: ((Rn * (1 - es)) + Height) * Sin_Lat
        };
    } // cs_geodetic_to_geocentric()
    exports.geodeticToGeocentric = geodeticToGeocentric;
    function geocentricToGeodetic(p, es, a, b) {
        /* local defintions and variables */
        /* end-criterium of loop, accuracy of sin(Latitude) */
        var genau = 1e-12;
        var genau2 = (genau * genau);
        var maxiter = 30;
        var P; /* distance between semi-minor axis and location */
        var RR; /* distance between center and location */
        var CT; /* sin of geocentric latitude */
        var ST; /* cos of geocentric latitude */
        var RX;
        var RK;
        var RN; /* Earth radius at location */
        var CPHI0; /* cos of start or old geodetic latitude in iterations */
        var SPHI0; /* sin of start or old geodetic latitude in iterations */
        var CPHI; /* cos of searched geodetic latitude */
        var SPHI; /* sin of searched geodetic latitude */
        var SDPHI; /* end-criterium: addition-theorem of sin(Latitude(iter)-Latitude(iter-1)) */
        var iter; /* # of continous iteration, max. 30 is always enough (s.a.) */
        var X = p.x;
        var Y = p.y;
        var Z = p.z ? p.z : 0.0; //Z value not always supplied
        var Longitude;
        var Latitude;
        var Height;
        P = Math.sqrt(X * X + Y * Y);
        RR = Math.sqrt(X * X + Y * Y + Z * Z);
        /*      special cases for latitude and longitude */
        if (P / a < genau) {
            /*  special case, if P=0. (X=0., Y=0.) */
            Longitude = 0.0;
            /*  if (X,Y,Z)=(0.,0.,0.) then Height becomes semi-minor axis
             *  of ellipsoid (=center of mass), Latitude becomes PI/2 */
            if (RR / a < genau) {
                Latitude = values_1.HALF_PI;
                Height = -b;
                return {
                    x: p.x,
                    y: p.y,
                    z: p.z
                };
            }
        }
        else {
            /*  ellipsoidal (geodetic) longitude
             *  interval: -PI < Longitude <= +PI */
            Longitude = Math.atan2(Y, X);
        }
        /* --------------------------------------------------------------
         * Following iterative algorithm was developped by
         * "Institut for Erdmessung", University of Hannover, July 1988.
         * Internet: www.ife.uni-hannover.de
         * Iterative computation of CPHI,SPHI and Height.
         * Iteration of CPHI and SPHI to 10**-12 radian resp.
         * 2*10**-7 arcsec.
         * --------------------------------------------------------------
         */
        CT = Z / RR;
        ST = P / RR;
        RX = 1.0 / Math.sqrt(1.0 - es * (2.0 - es) * ST * ST);
        CPHI0 = ST * (1.0 - es) * RX;
        SPHI0 = CT * RX;
        iter = 0;
        /* loop to find sin(Latitude) resp. Latitude
         * until |sin(Latitude(iter)-Latitude(iter-1))| < genau */
        do {
            iter++;
            RN = a / Math.sqrt(1.0 - es * SPHI0 * SPHI0);
            /*  ellipsoidal (geodetic) height */
            Height = P * CPHI0 + Z * SPHI0 - RN * (1.0 - es * SPHI0 * SPHI0);
            RK = es * RN / (RN + Height);
            RX = 1.0 / Math.sqrt(1.0 - RK * (2.0 - RK) * ST * ST);
            CPHI = ST * (1.0 - RK) * RX;
            SPHI = CT * RX;
            SDPHI = SPHI * CPHI0 - CPHI * SPHI0;
            CPHI0 = CPHI;
            SPHI0 = SPHI;
        } while (SDPHI * SDPHI > genau2 && iter < maxiter);
        /*      ellipsoidal (geodetic) latitude */
        Latitude = Math.atan(SPHI / Math.abs(CPHI));
        return {
            x: Longitude,
            y: Latitude,
            z: Height
        };
    } // cs_geocentric_to_geodetic()
    exports.geocentricToGeodetic = geocentricToGeodetic;
    /****************************************************************/
    // pj_geocentic_to_wgs84( p )
    //  p = point to transform in geocentric coordinates (x,y,z)
    /** point object, nothing fancy, just allows values to be
        passed back and forth by reference rather than by value.
        Other point classes may be used as long as they have
        x and y properties, which will get modified in the transform method.
    */
    function geocentricToWgs84(p, datum_type, datum_params) {
        if (datum_type === values_1.PJD_3PARAM) {
            // if( x[io] === HUGE_VAL )
            //    continue;
            return {
                x: p.x + datum_params[0],
                y: p.y + datum_params[1],
                z: p.z + datum_params[2],
            };
        }
        else if (datum_type === values_1.PJD_7PARAM) {
            var Dx_BF = datum_params[0];
            var Dy_BF = datum_params[1];
            var Dz_BF = datum_params[2];
            var Rx_BF = datum_params[3];
            var Ry_BF = datum_params[4];
            var Rz_BF = datum_params[5];
            var M_BF = datum_params[6];
            // if( x[io] === HUGE_VAL )
            //    continue;
            return {
                x: M_BF * (p.x - Rz_BF * p.y + Ry_BF * p.z) + Dx_BF,
                y: M_BF * (Rz_BF * p.x + p.y - Rx_BF * p.z) + Dy_BF,
                z: M_BF * (-Ry_BF * p.x + Rx_BF * p.y + p.z) + Dz_BF
            };
        }
    } // cs_geocentric_to_wgs84
    exports.geocentricToWgs84 = geocentricToWgs84;
    /****************************************************************/
    // pj_geocentic_from_wgs84()
    //  coordinate system definition,
    //  point to transform in geocentric coordinates (x,y,z)
    function geocentricFromWgs84(p, datum_type, datum_params) {
        if (datum_type === values_1.PJD_3PARAM) {
            //if( x[io] === HUGE_VAL )
            //    continue;
            return {
                x: p.x - datum_params[0],
                y: p.y - datum_params[1],
                z: p.z - datum_params[2],
            };
        }
        else if (datum_type === values_1.PJD_7PARAM) {
            var Dx_BF = datum_params[0];
            var Dy_BF = datum_params[1];
            var Dz_BF = datum_params[2];
            var Rx_BF = datum_params[3];
            var Ry_BF = datum_params[4];
            var Rz_BF = datum_params[5];
            var M_BF = datum_params[6];
            var x_tmp = (p.x - Dx_BF) / M_BF;
            var y_tmp = (p.y - Dy_BF) / M_BF;
            var z_tmp = (p.z - Dz_BF) / M_BF;
            //if( x[io] === HUGE_VAL )
            //    continue;
            return {
                x: x_tmp + Rz_BF * y_tmp - Ry_BF * z_tmp,
                y: -Rz_BF * x_tmp + y_tmp + Rx_BF * z_tmp,
                z: Ry_BF * x_tmp - Rx_BF * y_tmp + z_tmp
            };
        } //cs_geocentric_from_wgs84()
    }
    exports.geocentricFromWgs84 = geocentricFromWgs84;
},
/* proj4/lib/adjust_axis.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function default_1(crs, denorm, point) {
        var xin = point.x, yin = point.y, zin = point.z || 0.0;
        var v, t, i;
        var out = {};
        for (i = 0; i < 3; i++) {
            if (denorm && i === 2 && point.z === undefined) {
                continue;
            }
            if (i === 0) {
                v = xin;
                if ("ew".indexOf(crs.axis[i]) !== -1) {
                    t = 'x';
                }
                else {
                    t = 'y';
                }
            }
            else if (i === 1) {
                v = yin;
                if ("ns".indexOf(crs.axis[i]) !== -1) {
                    t = 'y';
                }
                else {
                    t = 'x';
                }
            }
            else {
                v = zin;
                t = 'z';
            }
            switch (crs.axis[i]) {
                case 'e':
                case 'w':
                case 'n':
                case 's':
                    out[t] = v;
                    break;
                case 'u':
                    if (point[t] !== undefined) {
                        out.z = v;
                    }
                    break;
                case 'd':
                    if (point[t] !== undefined) {
                        out.z = -v;
                    }
                    break;
                default:
                    //console.log("ERROR: unknow axis ("+crs.axis[i]+") - check definition of "+crs.projName);
                    return null;
            }
        }
        return out;
    }
    exports.default = default_1;
},
/* proj4/lib/common/toPoint.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function default_1(array) {
        var out = {
            x: array[0],
            y: array[1]
        };
        if (array.length > 2) {
            out.z = array[2];
        }
        if (array.length > 3) {
            out.m = array[3];
        }
        return out;
    }
    exports.default = default_1;
},
/* proj4/lib/checkSanity.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function default_1(point) {
        checkCoord(point.x);
        checkCoord(point.y);
    }
    exports.default = default_1;
    function checkCoord(num) {
        if (typeof Number.isFinite === 'function') {
            if (Number.isFinite(num)) {
                return;
            }
            throw new TypeError('coordinates must be finite numbers');
        }
        if (typeof num !== 'number' || num !== num || !isFinite(num)) {
            throw new TypeError('coordinates must be finite numbers');
        }
    }
},
/* models/renderers/renderer.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var view_1 = require(186) /* ../../core/view */;
    var visuals = tslib_1.__importStar(require(189) /* ../../core/visuals */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var model_1 = require(196) /* ../../model */;
    var coordinates_1 = require(197) /* ../canvas/coordinates */;
    var RendererView = /** @class */ (function (_super) {
        tslib_1.__extends(RendererView, _super);
        function RendererView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Object.defineProperty(RendererView.prototype, "coordinates", {
            get: function () {
                return this._coordinates;
            },
            enumerable: true,
            configurable: true
        });
        RendererView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.visuals = new visuals.Visuals(this.model);
            this.needs_webgl_blit = false;
            this._initialize_coordinates();
        };
        RendererView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            var _a = this.model.properties, x_range_name = _a.x_range_name, y_range_name = _a.y_range_name;
            this.on_change([x_range_name, y_range_name], function () { return _this._initialize_coordinates(); });
        };
        RendererView.prototype._initialize_coordinates = function () {
            var _a = this.model, x_range_name = _a.x_range_name, y_range_name = _a.y_range_name;
            var frame = this.plot_view.frame;
            var x_scale = frame.x_scales.get(x_range_name);
            var y_scale = frame.y_scales.get(y_range_name);
            this._coordinates = new coordinates_1.CoordinateTransform(x_scale, y_scale);
        };
        Object.defineProperty(RendererView.prototype, "plot_view", {
            get: function () {
                return this.parent;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RendererView.prototype, "plot_model", {
            get: function () {
                return this.parent.model;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RendererView.prototype, "layer", {
            get: function () {
                var _a = this.plot_view.canvas_view, overlays = _a.overlays, primary = _a.primary;
                return this.model.level == "overlay" ? overlays : primary;
            },
            enumerable: true,
            configurable: true
        });
        RendererView.prototype.request_render = function () {
            this.plot_view.request_render();
        };
        RendererView.prototype.notify_finished = function () {
            this.plot_view.notify_finished();
        };
        Object.defineProperty(RendererView.prototype, "needs_clip", {
            get: function () {
                return false;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RendererView.prototype, "has_webgl", {
            get: function () {
                return false;
            },
            enumerable: true,
            configurable: true
        });
        RendererView.prototype.render = function () {
            if (this.model.visible) {
                this._render();
            }
            this._has_finished = true;
        };
        return RendererView;
    }(view_1.View));
    exports.RendererView = RendererView;
    RendererView.__name__ = "RendererView";
    var Renderer = /** @class */ (function (_super) {
        tslib_1.__extends(Renderer, _super);
        function Renderer(attrs) {
            return _super.call(this, attrs) || this;
        }
        Renderer.init_Renderer = function () {
            this.define({
                level: [p.RenderLevel],
                visible: [p.Boolean, true],
                x_range_name: [p.String, "default"],
                y_range_name: [p.String, "default"],
            });
        };
        return Renderer;
    }(model_1.Model));
    exports.Renderer = Renderer;
    Renderer.__name__ = "Renderer";
    Renderer.init_Renderer();
},
/* core/view.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var signaling_1 = require(130) /* ./signaling */;
    var dom_1 = require(187) /* ./dom */;
    var types_1 = require(123) /* ./util/types */;
    var root_css_1 = tslib_1.__importDefault(require(188) /* ../styles/root.css */);
    var View = /** @class */ (function () {
        function View(options) {
            this.removed = new signaling_1.Signal0(this, "removed");
            this._ready = Promise.resolve(undefined);
            if (options.model != null)
                this.model = options.model;
            else
                throw new Error("model of a view wasn't configured");
            this._parent = options.parent;
        }
        Object.defineProperty(View.prototype, "ready", {
            get: function () {
                return this._ready;
            },
            enumerable: true,
            configurable: true
        });
        View.prototype.connect = function (signal, slot) {
            var _this = this;
            var new_slot = function (args, sender) {
                var promise = Promise.resolve(slot.call(_this, args, sender));
                _this._ready = _this._ready.then(function () { return promise; });
            };
            return signal.connect(new_slot, this);
        };
        View.prototype.disconnect = function (signal, slot) {
            return signal.disconnect(slot, this);
        };
        View.prototype.initialize = function () {
            var e_1, _a;
            this._has_finished = false;
            if (this.is_root) {
                this._stylesheet = dom_1.stylesheet;
            }
            try {
                for (var _b = tslib_1.__values(this.styles()), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var style = _c.value;
                    this.stylesheet.append(style);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        View.prototype.lazy_initialize = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    return [2 /*return*/];
                });
            });
        };
        View.prototype.remove = function () {
            this._parent = undefined;
            this.disconnect_signals();
            this.removed.emit();
        };
        View.prototype.toString = function () {
            return this.model.type + "View(" + this.model.id + ")";
        };
        View.prototype.serializable_state = function () {
            return { type: this.model.type };
        };
        Object.defineProperty(View.prototype, "parent", {
            get: function () {
                if (this._parent !== undefined)
                    return this._parent;
                else
                    throw new Error("parent of a view wasn't configured");
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(View.prototype, "is_root", {
            get: function () {
                return this.parent === null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(View.prototype, "root", {
            get: function () {
                return this.is_root ? this : this.parent.root;
            },
            enumerable: true,
            configurable: true
        });
        View.prototype.assert_root = function () {
            if (!this.is_root)
                throw new Error(this.toString() + " is not a root layout");
        };
        View.prototype.has_finished = function () {
            return this._has_finished;
        };
        Object.defineProperty(View.prototype, "is_idle", {
            get: function () {
                return this.has_finished();
            },
            enumerable: true,
            configurable: true
        });
        View.prototype.connect_signals = function () { };
        View.prototype.disconnect_signals = function () {
            signaling_1.Signal.disconnectReceiver(this);
        };
        View.prototype.on_change = function (properties, fn) {
            var e_2, _a;
            try {
                for (var _b = tslib_1.__values(types_1.isArray(properties) ? properties : [properties]), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var property = _c.value;
                    this.connect(property.change, fn);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
        };
        View.prototype.cursor = function (_sx, _sy) {
            return null;
        };
        Object.defineProperty(View.prototype, "stylesheet", {
            get: function () {
                if (this.is_root)
                    return this._stylesheet;
                else
                    return this.root.stylesheet;
            },
            enumerable: true,
            configurable: true
        });
        View.prototype.styles = function () {
            return [root_css_1.default];
        };
        return View;
    }());
    exports.View = View;
    View.__name__ = "View";
},
/* core/dom.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(123) /* ./util/types */;
    var object_1 = require(128) /* ./util/object */;
    var _createElement = function (tag) {
        return function (attrs) {
            var e_1, _a, e_2, _b, e_3, _c, e_4, _d, e_5, _e, e_6, _f;
            if (attrs === void 0) {
                attrs = {};
            }
            var children = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                children[_i - 1] = arguments[_i];
            }
            var element = document.createElement(tag);
            element.classList.add("bk");
            try {
                for (var _g = tslib_1.__values(object_1.entries(attrs)), _h = _g.next(); !_h.done; _h = _g.next()) {
                    var _j = tslib_1.__read(_h.value, 2), attr = _j[0], value = _j[1];
                    if (value == null || types_1.isBoolean(value) && !value)
                        continue;
                    if (attr === "class") {
                        if (types_1.isString(value))
                            value = value.split(/\s+/);
                        if (types_1.isArray(value)) {
                            try {
                                for (var value_1 = (e_2 = void 0, tslib_1.__values(value)), value_1_1 = value_1.next(); !value_1_1.done; value_1_1 = value_1.next()) {
                                    var cls = value_1_1.value;
                                    if (cls != null)
                                        element.classList.add(cls);
                                }
                            }
                            catch (e_2_1) {
                                e_2 = { error: e_2_1 };
                            }
                            finally {
                                try {
                                    if (value_1_1 && !value_1_1.done && (_b = value_1.return))
                                        _b.call(value_1);
                                }
                                finally {
                                    if (e_2)
                                        throw e_2.error;
                                }
                            }
                            continue;
                        }
                    }
                    if (attr === "style" && types_1.isPlainObject(value)) {
                        try {
                            for (var _k = (e_3 = void 0, tslib_1.__values(object_1.entries(value))), _l = _k.next(); !_l.done; _l = _k.next()) {
                                var _m = tslib_1.__read(_l.value, 2), prop = _m[0], data = _m[1];
                                element.style[prop] = data;
                            }
                        }
                        catch (e_3_1) {
                            e_3 = { error: e_3_1 };
                        }
                        finally {
                            try {
                                if (_l && !_l.done && (_c = _k.return))
                                    _c.call(_k);
                            }
                            finally {
                                if (e_3)
                                    throw e_3.error;
                            }
                        }
                        continue;
                    }
                    if (attr === "data" && types_1.isPlainObject(value)) {
                        try {
                            for (var _o = (e_4 = void 0, tslib_1.__values(object_1.entries(value))), _p = _o.next(); !_p.done; _p = _o.next()) {
                                var _q = tslib_1.__read(_p.value, 2), key = _q[0], data = _q[1];
                                element.dataset[key] = data; // XXX: attrs needs a better type
                            }
                        }
                        catch (e_4_1) {
                            e_4 = { error: e_4_1 };
                        }
                        finally {
                            try {
                                if (_p && !_p.done && (_d = _o.return))
                                    _d.call(_o);
                            }
                            finally {
                                if (e_4)
                                    throw e_4.error;
                            }
                        }
                        continue;
                    }
                    element.setAttribute(attr, value);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_h && !_h.done && (_a = _g.return))
                        _a.call(_g);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            function append(child) {
                var e_7, _a;
                if (types_1.isString(child))
                    element.appendChild(document.createTextNode(child));
                else if (child instanceof Node)
                    element.appendChild(child);
                else if (child instanceof NodeList || child instanceof HTMLCollection) {
                    try {
                        for (var child_2 = tslib_1.__values(child), child_2_1 = child_2.next(); !child_2_1.done; child_2_1 = child_2.next()) {
                            var el = child_2_1.value;
                            element.appendChild(el);
                        }
                    }
                    catch (e_7_1) {
                        e_7 = { error: e_7_1 };
                    }
                    finally {
                        try {
                            if (child_2_1 && !child_2_1.done && (_a = child_2.return))
                                _a.call(child_2);
                        }
                        finally {
                            if (e_7)
                                throw e_7.error;
                        }
                    }
                }
                else if (child != null && child !== false)
                    throw new Error("expected a DOM element, string, false or null, got " + JSON.stringify(child));
            }
            try {
                for (var children_1 = tslib_1.__values(children), children_1_1 = children_1.next(); !children_1_1.done; children_1_1 = children_1.next()) {
                    var child = children_1_1.value;
                    if (types_1.isArray(child)) {
                        try {
                            for (var child_1 = (e_6 = void 0, tslib_1.__values(child)), child_1_1 = child_1.next(); !child_1_1.done; child_1_1 = child_1.next()) {
                                var _child = child_1_1.value;
                                append(_child);
                            }
                        }
                        catch (e_6_1) {
                            e_6 = { error: e_6_1 };
                        }
                        finally {
                            try {
                                if (child_1_1 && !child_1_1.done && (_f = child_1.return))
                                    _f.call(child_1);
                            }
                            finally {
                                if (e_6)
                                    throw e_6.error;
                            }
                        }
                    }
                    else
                        append(child);
                }
            }
            catch (e_5_1) {
                e_5 = { error: e_5_1 };
            }
            finally {
                try {
                    if (children_1_1 && !children_1_1.done && (_e = children_1.return))
                        _e.call(children_1);
                }
                finally {
                    if (e_5)
                        throw e_5.error;
                }
            }
            return element;
        };
    };
    function createElement(tag, attrs) {
        var children = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            children[_i - 2] = arguments[_i];
        }
        return _createElement(tag).apply(void 0, tslib_1.__spread([attrs], children));
    }
    exports.createElement = createElement;
    exports.div = _createElement("div"), exports.span = _createElement("span"), exports.canvas = _createElement("canvas"), exports.link = _createElement("link"), exports.style = _createElement("style"), exports.a = _createElement("a"), exports.p = _createElement("p"), exports.i = _createElement("i"), exports.pre = _createElement("pre"), exports.button = _createElement("button"), exports.label = _createElement("label"), exports.input = _createElement("input"), exports.select = _createElement("select"), exports.option = _createElement("option"), exports.optgroup = _createElement("optgroup"), exports.textarea = _createElement("textarea");
    function nbsp() {
        return document.createTextNode("\u00a0");
    }
    exports.nbsp = nbsp;
    function append(element) {
        var e_8, _a;
        var children = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            children[_i - 1] = arguments[_i];
        }
        try {
            for (var children_2 = tslib_1.__values(children), children_2_1 = children_2.next(); !children_2_1.done; children_2_1 = children_2.next()) {
                var child = children_2_1.value;
                element.appendChild(child);
            }
        }
        catch (e_8_1) {
            e_8 = { error: e_8_1 };
        }
        finally {
            try {
                if (children_2_1 && !children_2_1.done && (_a = children_2.return))
                    _a.call(children_2);
            }
            finally {
                if (e_8)
                    throw e_8.error;
            }
        }
    }
    exports.append = append;
    function remove(element) {
        var parent = element.parentNode;
        if (parent != null) {
            parent.removeChild(element);
        }
    }
    exports.remove = remove;
    exports.removeElement = remove;
    function replaceWith(element, replacement) {
        var parent = element.parentNode;
        if (parent != null) {
            parent.replaceChild(replacement, element);
        }
    }
    exports.replaceWith = replaceWith;
    function prepend(element) {
        var e_9, _a;
        var nodes = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            nodes[_i - 1] = arguments[_i];
        }
        var first = element.firstChild;
        try {
            for (var nodes_1 = tslib_1.__values(nodes), nodes_1_1 = nodes_1.next(); !nodes_1_1.done; nodes_1_1 = nodes_1.next()) {
                var node = nodes_1_1.value;
                element.insertBefore(node, first);
            }
        }
        catch (e_9_1) {
            e_9 = { error: e_9_1 };
        }
        finally {
            try {
                if (nodes_1_1 && !nodes_1_1.done && (_a = nodes_1.return))
                    _a.call(nodes_1);
            }
            finally {
                if (e_9)
                    throw e_9.error;
            }
        }
    }
    exports.prepend = prepend;
    function empty(node, attrs) {
        var e_10, _a;
        if (attrs === void 0) {
            attrs = false;
        }
        var child;
        while (child = node.firstChild) {
            node.removeChild(child);
        }
        if (attrs && node instanceof Element) {
            try {
                for (var _b = tslib_1.__values(node.attributes), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var attr = _c.value;
                    node.removeAttributeNode(attr);
                }
            }
            catch (e_10_1) {
                e_10 = { error: e_10_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_10)
                        throw e_10.error;
                }
            }
        }
    }
    exports.empty = empty;
    function display(element) {
        element.style.display = "";
    }
    exports.display = display;
    function undisplay(element) {
        element.style.display = "none";
    }
    exports.undisplay = undisplay;
    function show(element) {
        element.style.visibility = "";
    }
    exports.show = show;
    function hide(element) {
        element.style.visibility = "hidden";
    }
    exports.hide = hide;
    function offset(element) {
        var rect = element.getBoundingClientRect();
        return {
            top: rect.top + window.pageYOffset - document.documentElement.clientTop,
            left: rect.left + window.pageXOffset - document.documentElement.clientLeft,
        };
    }
    exports.offset = offset;
    function matches(el, selector) {
        var p = Element.prototype;
        var f = p.matches || p.webkitMatchesSelector || p.mozMatchesSelector || p.msMatchesSelector;
        return f.call(el, selector);
    }
    exports.matches = matches;
    function parent(el, selector) {
        var node = el;
        while (node = node.parentElement) {
            if (matches(node, selector))
                return node;
        }
        return null;
    }
    exports.parent = parent;
    function num(value) {
        return parseFloat(value) || 0;
    }
    function extents(el) {
        var style = getComputedStyle(el);
        return {
            border: {
                top: num(style.borderTopWidth),
                bottom: num(style.borderBottomWidth),
                left: num(style.borderLeftWidth),
                right: num(style.borderRightWidth),
            },
            margin: {
                top: num(style.marginTop),
                bottom: num(style.marginBottom),
                left: num(style.marginLeft),
                right: num(style.marginRight),
            },
            padding: {
                top: num(style.paddingTop),
                bottom: num(style.paddingBottom),
                left: num(style.paddingLeft),
                right: num(style.paddingRight),
            },
        };
    }
    exports.extents = extents;
    function size(el) {
        var rect = el.getBoundingClientRect();
        return {
            width: Math.ceil(rect.width),
            height: Math.ceil(rect.height),
        };
    }
    exports.size = size;
    function scroll_size(el) {
        return {
            width: Math.ceil(el.scrollWidth),
            height: Math.ceil(el.scrollHeight),
        };
    }
    exports.scroll_size = scroll_size;
    function outer_size(el) {
        var _a = extents(el).margin, left = _a.left, right = _a.right, top = _a.top, bottom = _a.bottom;
        var _b = size(el), width = _b.width, height = _b.height;
        return {
            width: Math.ceil(width + left + right),
            height: Math.ceil(height + top + bottom),
        };
    }
    exports.outer_size = outer_size;
    function content_size(el) {
        var e_11, _a;
        var _b = el.getBoundingClientRect(), left = _b.left, top = _b.top;
        var padding = extents(el).padding;
        var width = 0;
        var height = 0;
        try {
            for (var _c = tslib_1.__values(el.children), _d = _c.next(); !_d.done; _d = _c.next()) {
                var child = _d.value;
                var rect = child.getBoundingClientRect();
                width = Math.max(width, Math.ceil(rect.left - left - padding.left + rect.width));
                height = Math.max(height, Math.ceil(rect.top - top - padding.top + rect.height));
            }
        }
        catch (e_11_1) {
            e_11 = { error: e_11_1 };
        }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return))
                    _a.call(_c);
            }
            finally {
                if (e_11)
                    throw e_11.error;
            }
        }
        return { width: width, height: height };
    }
    exports.content_size = content_size;
    function position(el, box, margin) {
        var style = el.style;
        style.left = box.x + "px";
        style.top = box.y + "px";
        style.width = box.width + "px";
        style.height = box.height + "px";
        if (margin == null)
            style.margin = "";
        else {
            var top = margin.top, right = margin.right, bottom = margin.bottom, left = margin.left;
            style.margin = top + "px " + right + "px " + bottom + "px " + left + "px";
        }
    }
    exports.position = position;
    function children(el) {
        return Array.from(el.children);
    }
    exports.children = children;
    var ClassList = /** @class */ (function () {
        function ClassList(el) {
            this.el = el;
            this.classList = el.classList;
        }
        Object.defineProperty(ClassList.prototype, "values", {
            get: function () {
                var values = [];
                for (var i_1 = 0; i_1 < this.classList.length; i_1++) {
                    var item = this.classList.item(i_1);
                    if (item != null)
                        values.push(item);
                }
                return values;
            },
            enumerable: true,
            configurable: true
        });
        ClassList.prototype.has = function (cls) {
            return this.classList.contains(cls);
        };
        ClassList.prototype.add = function () {
            var e_12, _a;
            var classes = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                classes[_i] = arguments[_i];
            }
            try {
                for (var classes_1 = tslib_1.__values(classes), classes_1_1 = classes_1.next(); !classes_1_1.done; classes_1_1 = classes_1.next()) {
                    var cls = classes_1_1.value;
                    this.classList.add(cls);
                }
            }
            catch (e_12_1) {
                e_12 = { error: e_12_1 };
            }
            finally {
                try {
                    if (classes_1_1 && !classes_1_1.done && (_a = classes_1.return))
                        _a.call(classes_1);
                }
                finally {
                    if (e_12)
                        throw e_12.error;
                }
            }
            return this;
        };
        ClassList.prototype.remove = function () {
            var e_13, _a;
            var classes = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                classes[_i] = arguments[_i];
            }
            try {
                for (var classes_2 = tslib_1.__values(classes), classes_2_1 = classes_2.next(); !classes_2_1.done; classes_2_1 = classes_2.next()) {
                    var cls = classes_2_1.value;
                    this.classList.remove(cls);
                }
            }
            catch (e_13_1) {
                e_13 = { error: e_13_1 };
            }
            finally {
                try {
                    if (classes_2_1 && !classes_2_1.done && (_a = classes_2.return))
                        _a.call(classes_2);
                }
                finally {
                    if (e_13)
                        throw e_13.error;
                }
            }
            return this;
        };
        ClassList.prototype.clear = function () {
            var e_14, _a;
            try {
                for (var _b = tslib_1.__values(this.values), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var cls = _c.value;
                    if (cls != "bk")
                        this.classList.remove(cls);
                }
            }
            catch (e_14_1) {
                e_14 = { error: e_14_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_14)
                        throw e_14.error;
                }
            }
            return this;
        };
        ClassList.prototype.toggle = function (cls, activate) {
            var add = activate != null ? activate : !this.has(cls);
            if (add)
                this.add(cls);
            else
                this.remove(cls);
            return this;
        };
        return ClassList;
    }());
    exports.ClassList = ClassList;
    ClassList.__name__ = "ClassList";
    function classes(el) {
        return new ClassList(el);
    }
    exports.classes = classes;
    function toggle_attribute(el, attr, state) {
        if (state == null) {
            state = !el.hasAttribute(attr);
        }
        if (state)
            el.setAttribute(attr, "true");
        else
            el.removeAttribute(attr);
    }
    exports.toggle_attribute = toggle_attribute;
    (function (Keys) {
        Keys[Keys["Backspace"] = 8] = "Backspace";
        Keys[Keys["Tab"] = 9] = "Tab";
        Keys[Keys["Enter"] = 13] = "Enter";
        Keys[Keys["Esc"] = 27] = "Esc";
        Keys[Keys["PageUp"] = 33] = "PageUp";
        Keys[Keys["PageDown"] = 34] = "PageDown";
        Keys[Keys["Left"] = 37] = "Left";
        Keys[Keys["Up"] = 38] = "Up";
        Keys[Keys["Right"] = 39] = "Right";
        Keys[Keys["Down"] = 40] = "Down";
        Keys[Keys["Delete"] = 46] = "Delete";
    })(exports.Keys || (exports.Keys = {}));
    function undisplayed(el, fn) {
        var display = el.style.display;
        el.style.display = "none";
        try {
            return fn();
        }
        finally {
            el.style.display = display;
        }
    }
    exports.undisplayed = undisplayed;
    function unsized(el, fn) {
        return sized(el, {}, fn);
    }
    exports.unsized = unsized;
    function sized(el, size, fn) {
        var _a = el.style, width = _a.width, height = _a.height, position = _a.position, display = _a.display;
        el.style.position = "absolute";
        el.style.display = "";
        el.style.width = size.width != null && size.width != Infinity ? size.width + "px" : "auto";
        el.style.height = size.height != null && size.height != Infinity ? size.height + "px" : "auto";
        try {
            return fn();
        }
        finally {
            el.style.position = position;
            el.style.display = display;
            el.style.width = width;
            el.style.height = height;
        }
    }
    exports.sized = sized;
    var StyleSheet = /** @class */ (function () {
        function StyleSheet(root) {
            this.root = root;
            this.known = new Set();
            this.style = exports.style({ type: "text/css" });
            prepend(root, this.style);
        }
        StyleSheet.prototype.append = function (css) {
            if (!this.known.has(css)) {
                this.style.appendChild(document.createTextNode(css));
                this.known.add(css);
            }
        };
        return StyleSheet;
    }());
    exports.StyleSheet = StyleSheet;
    StyleSheet.__name__ = "StyleSheet";
    exports.stylesheet = new StyleSheet(document.head);
},
/* styles/root.css.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var css = "\n.bk-root {\n  position: relative;\n  width: auto;\n  height: auto;\n  z-index: 0;\n  box-sizing: border-box;\n  font-family: Helvetica, Arial, sans-serif;\n  font-size: 13px;\n}\n.bk-root .bk,\n.bk-root .bk:before,\n.bk-root .bk:after {\n  box-sizing: inherit;\n  margin: 0;\n  border: 0;\n  padding: 0;\n  background-image: none;\n  font-family: inherit;\n  font-size: 100%;\n  line-height: 1.42857143;\n}\n.bk-root pre.bk {\n  font-family: Courier, monospace;\n}\n";
    exports.default = css;
},
/* core/visuals.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var mixins = tslib_1.__importStar(require(143) /* ./property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ./properties */);
    var color_1 = require(137) /* ./util/color */;
    var types_1 = require(123) /* ./util/types */;
    var svg_1 = require(190) /* ./util/svg */;
    var canvas_1 = require(192) /* ../models/canvas/canvas */;
    function color2css(color, alpha) {
        var _b = tslib_1.__read(types_1.isString(color) ? color_1.color2rgba(color) : color_1.decode_rgba(color), 4), r = _b[0], g = _b[1], b = _b[2], a = _b[3];
        return "rgba(" + r * 255 + ", " + g * 255 + ", " + b * 255 + ", " + (a == 1.0 ? alpha : a) + ")";
    }
    function _horz(ctx, h, h2) {
        ctx.moveTo(0, h2 + 0.5);
        ctx.lineTo(h, h2 + 0.5);
        ctx.stroke();
    }
    function _vert(ctx, h, h2) {
        ctx.moveTo(h2 + 0.5, 0);
        ctx.lineTo(h2 + 0.5, h);
        ctx.stroke();
    }
    function _x(ctx, h) {
        ctx.moveTo(0, h);
        ctx.lineTo(h, 0);
        ctx.stroke();
        ctx.moveTo(0, 0);
        ctx.lineTo(h, h);
        ctx.stroke();
    }
    exports.hatch_aliases = {
        " ": "blank",
        ".": "dot",
        o: "ring",
        "-": "horizontal_line",
        "|": "vertical_line",
        "+": "cross",
        "\"": "horizontal_dash",
        ":": "vertical_dash",
        "@": "spiral",
        "/": "right_diagonal_line",
        "\\": "left_diagonal_line",
        x: "diagonal_cross",
        ",": "right_diagonal_dash",
        "`": "left_diagonal_dash",
        v: "horizontal_wave",
        ">": "vertical_wave",
        "*": "criss_cross",
    };
    function create_hatch_canvas(ctx, hatch_pattern, hatch_color, hatch_alpha, hatch_scale, hatch_weight) {
        var _a;
        var h = hatch_scale;
        var h2 = h / 2;
        var h4 = h2 / 2;
        ctx.strokeStyle = color2css(hatch_color, hatch_alpha);
        ctx.lineCap = "square";
        ctx.fillStyle = hatch_color;
        ctx.lineWidth = hatch_weight;
        switch ((_a = exports.hatch_aliases[hatch_pattern]) !== null && _a !== void 0 ? _a : hatch_pattern) {
            // we should not need these if code conditions on hatch.doit, but
            // include them here just for completeness
            case "blank":
                break;
            case "dot":
                ctx.arc(h2, h2, h2 / 2, 0, 2 * Math.PI, true);
                ctx.fill();
                break;
            case "ring":
                ctx.arc(h2, h2, h2 / 2, 0, 2 * Math.PI, true);
                ctx.stroke();
                break;
            case "horizontal_line":
                _horz(ctx, h, h2);
                break;
            case "vertical_line":
                _vert(ctx, h, h2);
                break;
            case "cross":
                _horz(ctx, h, h2);
                _vert(ctx, h, h2);
                break;
            case "horizontal_dash":
                _horz(ctx, h2, h2);
                break;
            case "vertical_dash":
                _vert(ctx, h2, h2);
                break;
            case "spiral": {
                var h30 = h / 30;
                ctx.moveTo(h2, h2);
                for (var i = 0; i < 360; i++) {
                    var angle = 0.1 * i;
                    var x = h2 + (h30 * angle) * Math.cos(angle);
                    var y = h2 + (h30 * angle) * Math.sin(angle);
                    ctx.lineTo(x, y);
                }
                ctx.stroke();
                break;
            }
            case "right_diagonal_line":
                ctx.moveTo(-h4 + 0.5, h);
                ctx.lineTo(h4 + 0.5, 0);
                ctx.stroke();
                ctx.moveTo(h4 + 0.5, h);
                ctx.lineTo(3 * h4 + 0.5, 0);
                ctx.stroke();
                ctx.moveTo(3 * h4 + 0.5, h);
                ctx.lineTo(5 * h4 + 0.5, 0);
                ctx.stroke();
                ctx.stroke();
                break;
            case "left_diagonal_line":
                ctx.moveTo(h4 + 0.5, h);
                ctx.lineTo(-h4 + 0.5, 0);
                ctx.stroke();
                ctx.moveTo(3 * h4 + 0.5, h);
                ctx.lineTo(h4 + 0.5, 0);
                ctx.stroke();
                ctx.moveTo(5 * h4 + 0.5, h);
                ctx.lineTo(3 * h4 + 0.5, 0);
                ctx.stroke();
                ctx.stroke();
                break;
            case "diagonal_cross":
                _x(ctx, h);
                break;
            case "right_diagonal_dash":
                ctx.moveTo(h4 + 0.5, 3 * h4 + 0.5);
                ctx.lineTo(3 * h4 + 0.5, h4 + 0.5);
                ctx.stroke();
                break;
            case "left_diagonal_dash":
                ctx.moveTo(h4 + 0.5, h4 + 0.5);
                ctx.lineTo(3 * h4 + 0.5, 3 * h4 + 0.5);
                ctx.stroke();
                break;
            case "horizontal_wave":
                ctx.moveTo(0, h4);
                ctx.lineTo(h2, 3 * h4);
                ctx.lineTo(h, h4);
                ctx.stroke();
                break;
            case "vertical_wave":
                ctx.moveTo(h4, 0);
                ctx.lineTo(3 * h4, h2);
                ctx.lineTo(h4, h);
                ctx.stroke();
                break;
            case "criss_cross":
                _x(ctx, h);
                _horz(ctx, h, h2);
                _vert(ctx, h, h2);
                break;
        }
    }
    var ContextProperties = /** @class */ (function () {
        function ContextProperties(obj, prefix) {
            var e_1, _b;
            if (prefix === void 0) {
                prefix = "";
            }
            this.obj = obj;
            this.prefix = prefix;
            this.cache = {};
            try {
                for (var _c = tslib_1.__values(this.attrs), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var attr = _d.value;
                    this[attr] = obj.properties[prefix + attr];
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        }
        ContextProperties.prototype.warm_cache = function (source, all_indices) {
            var e_2, _b;
            try {
                for (var _c = tslib_1.__values(this.attrs), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var attr = _d.value;
                    var prop = this.obj.properties[this.prefix + attr];
                    if (prop.spec.value !== undefined) // TODO (bev) better test?
                        this.cache[attr] = prop.spec.value;
                    else if (source != null && prop instanceof p.VectorSpec) {
                        var array = prop.array(source);
                        var subarray = all_indices != null ? all_indices.select(array) : array;
                        this.cache[attr + "_array"] = subarray;
                    }
                    else
                        throw new Error("source is required with a vectorized visual property");
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
        };
        ContextProperties.prototype.cache_select = function (attr, i) {
            var prop = this.obj.properties[this.prefix + attr];
            var value;
            if (prop.spec.value !== undefined) // TODO (bev) better test?
                this.cache[attr] = value = prop.spec.value;
            else
                this.cache[attr] = value = this.cache[attr + "_array"][i];
            return value;
        };
        ContextProperties.prototype.get_array = function (attr) {
            return this.cache[attr + "_array"];
        };
        ContextProperties.prototype.set_vectorize = function (ctx, i) {
            this._set_vectorize(ctx, i);
        };
        return ContextProperties;
    }());
    exports.ContextProperties = ContextProperties;
    ContextProperties.__name__ = "ContextProperties";
    var Line = /** @class */ (function (_super) {
        tslib_1.__extends(Line, _super);
        function Line() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Line.prototype.set_value = function (ctx) {
            var color = this.line_color.value();
            var alpha = this.line_alpha.value();
            ctx.strokeStyle = color2css(color, alpha);
            ctx.lineWidth = this.line_width.value();
            ctx.lineJoin = this.line_join.value();
            ctx.lineCap = this.line_cap.value();
            ctx.lineDash = this.line_dash.value();
            ctx.lineDashOffset = this.line_dash_offset.value();
        };
        Object.defineProperty(Line.prototype, "doit", {
            get: function () {
                return !(this.line_color.spec.value === null ||
                    this.line_alpha.spec.value == 0 ||
                    this.line_width.spec.value == 0);
            },
            enumerable: true,
            configurable: true
        });
        Line.prototype._set_vectorize = function (ctx, i) {
            var color = this.cache_select("line_color", i);
            var alpha = this.cache_select("line_alpha", i);
            var width = this.cache_select("line_width", i);
            var join = this.cache_select("line_join", i);
            var cap = this.cache_select("line_cap", i);
            var dash = this.cache_select("line_dash", i);
            var offset = this.cache_select("line_dash_offset", i);
            ctx.strokeStyle = color2css(color, alpha);
            ctx.lineWidth = width;
            ctx.lineJoin = join;
            ctx.lineCap = cap;
            ctx.lineDash = dash;
            ctx.lineDashOffset = offset;
        };
        Line.prototype.color_value = function () {
            return color2css(this.line_color.value(), this.line_alpha.value());
        };
        return Line;
    }(ContextProperties));
    exports.Line = Line;
    Line.__name__ = "Line";
    Line.prototype.attrs = Object.keys(mixins.LineVector);
    var Fill = /** @class */ (function (_super) {
        tslib_1.__extends(Fill, _super);
        function Fill() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Fill.prototype.set_value = function (ctx) {
            var color = this.fill_color.value();
            var alpha = this.fill_alpha.value();
            ctx.fillStyle = color2css(color, alpha);
        };
        Object.defineProperty(Fill.prototype, "doit", {
            get: function () {
                return !(this.fill_color.spec.value === null ||
                    this.fill_alpha.spec.value == 0);
            },
            enumerable: true,
            configurable: true
        });
        Fill.prototype._set_vectorize = function (ctx, i) {
            var color = this.cache_select("fill_color", i);
            var alpha = this.cache_select("fill_alpha", i);
            ctx.fillStyle = color2css(color, alpha);
        };
        Fill.prototype.color_value = function () {
            return color2css(this.fill_color.value(), this.fill_alpha.value());
        };
        return Fill;
    }(ContextProperties));
    exports.Fill = Fill;
    Fill.__name__ = "Fill";
    Fill.prototype.attrs = Object.keys(mixins.FillVector);
    var Hatch = /** @class */ (function (_super) {
        tslib_1.__extends(Hatch, _super);
        function Hatch() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Hatch.prototype.cache_select = function (name, i) {
            var value;
            if (name == "pattern") {
                var color_2 = this.cache_select("hatch_color", i);
                var alpha_1 = this.cache_select("hatch_alpha", i);
                var scale_1 = this.cache_select("hatch_scale", i);
                var pattern_1 = this.cache_select("hatch_pattern", i);
                var weight_1 = this.cache_select("hatch_weight", i);
                var hatch_extra = this.cache.hatch_extra;
                if (hatch_extra != null && hatch_extra.hasOwnProperty(pattern_1)) {
                    var custom = hatch_extra[pattern_1];
                    this.cache.pattern = custom.get_pattern(color_2, alpha_1, scale_1, weight_1);
                }
                else {
                    this.cache.pattern = function (ctx) {
                        // TODO: this needs a canvas provider instead of trying to guess what to use
                        var output_backend = ctx instanceof svg_1.SVGRenderingContext2D ? "svg" : "canvas";
                        var region = new canvas_1.CanvasLayer(output_backend, true);
                        region.resize(scale_1, scale_1);
                        region.prepare();
                        create_hatch_canvas(region.ctx, pattern_1, color_2, alpha_1, scale_1, weight_1);
                        return ctx.createPattern(region.canvas, "repeat");
                    };
                }
            }
            else
                value = _super.prototype.cache_select.call(this, name, i);
            return value;
        };
        Hatch.prototype._try_defer = function (defer_func) {
            var _b = this.cache, hatch_pattern = _b.hatch_pattern, hatch_extra = _b.hatch_extra;
            if (hatch_extra != null && hatch_extra.hasOwnProperty(hatch_pattern)) {
                var custom = hatch_extra[hatch_pattern];
                custom.onload(defer_func);
            }
        };
        Object.defineProperty(Hatch.prototype, "doit", {
            get: function () {
                return !(this.hatch_color.spec.value === null ||
                    this.hatch_alpha.spec.value == 0 ||
                    this.hatch_pattern.spec.value == " " ||
                    this.hatch_pattern.spec.value == "blank" ||
                    this.hatch_pattern.spec.value === null);
            },
            enumerable: true,
            configurable: true
        });
        Hatch.prototype.doit2 = function (ctx, i, ready_func, defer_func) {
            if (!this.doit) {
                return;
            }
            this.cache_select("pattern", i);
            var pattern = this.cache.pattern(ctx);
            if (pattern == null) {
                this._try_defer(defer_func);
            }
            else {
                this.set_vectorize(ctx, i);
                ready_func();
            }
        };
        Hatch.prototype._set_vectorize = function (ctx, i) {
            this.cache_select("pattern", i);
            ctx.fillStyle = this.cache.pattern(ctx);
        };
        Hatch.prototype.color_value = function () {
            return color2css(this.hatch_color.value(), this.hatch_alpha.value());
        };
        return Hatch;
    }(ContextProperties));
    exports.Hatch = Hatch;
    Hatch.__name__ = "Hatch";
    Hatch.prototype.attrs = Object.keys(mixins.HatchVector);
    var Text = /** @class */ (function (_super) {
        tslib_1.__extends(Text, _super);
        function Text() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Text.prototype.color_value = function () {
            return color2css(this.text_color.value(), this.text_alpha.value());
        };
        Text.prototype.font_value = function () {
            var text_font = this.text_font.value();
            var text_font_size = this.text_font_size.value();
            var text_font_style = this.text_font_style.value();
            return text_font_style + " " + text_font_size + " " + text_font;
        };
        Text.prototype.v_font_value = function (i) {
            _super.prototype.cache_select.call(this, "text_font_style", i);
            _super.prototype.cache_select.call(this, "text_font_size", i);
            _super.prototype.cache_select.call(this, "text_font", i);
            var _b = this.cache, text_font_style = _b.text_font_style, text_font_size = _b.text_font_size, text_font = _b.text_font;
            return text_font_style + " " + text_font_size + " " + text_font;
        };
        Text.prototype.cache_select = function (name, i) {
            var value;
            if (name == "font") {
                this.cache.font = value = this.v_font_value(i);
            }
            else
                value = _super.prototype.cache_select.call(this, name, i);
            return value;
        };
        Text.prototype.set_value = function (ctx) {
            var color = this.text_color.value();
            var alpha = this.text_alpha.value();
            ctx.fillStyle = color2css(color, alpha);
            ctx.font = this.font_value();
            ctx.textAlign = this.text_align.value();
            ctx.textBaseline = this.text_baseline.value();
        };
        Object.defineProperty(Text.prototype, "doit", {
            get: function () {
                return !(this.text_color.spec.value === null ||
                    this.text_alpha.spec.value == 0);
            },
            enumerable: true,
            configurable: true
        });
        Text.prototype._set_vectorize = function (ctx, i) {
            var color = this.cache_select("text_color", i);
            var alpha = this.cache_select("text_alpha", i);
            var font = this.cache_select("font", i);
            var align = this.cache_select("text_align", i);
            var baseline = this.cache_select("text_baseline", i);
            ctx.fillStyle = color2css(color, alpha);
            ctx.font = font;
            ctx.textAlign = align;
            ctx.textBaseline = baseline;
        };
        return Text;
    }(ContextProperties));
    exports.Text = Text;
    Text.__name__ = "Text";
    Text.prototype.attrs = Object.keys(mixins.TextVector);
    var Visuals = /** @class */ (function () {
        function Visuals(model) {
            var e_3, _b;
            try {
                for (var _c = tslib_1.__values(model._mixins), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var mixin = _d.value;
                    var _e = tslib_1.__read(mixin.split(":"), 2), name = _e[0], _f = _e[1], prefix = _f === void 0 ? "" : _f;
                    var cls = void 0;
                    switch (name) {
                        case "line":
                            cls = Line;
                            break;
                        case "fill":
                            cls = Fill;
                            break;
                        case "hatch":
                            cls = Hatch;
                            break;
                        case "text":
                            cls = Text;
                            break;
                        default:
                            throw new Error("unknown visual: " + name);
                    }
                    this[prefix + name] = new cls(model, prefix);
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
        }
        Visuals.prototype.warm_cache = function (source, all_indices) {
            for (var name in this) {
                if (this.hasOwnProperty(name)) {
                    var prop = this[name];
                    if (prop instanceof ContextProperties)
                        prop.warm_cache(source, all_indices);
                }
            }
        };
        return Visuals;
    }());
    exports.Visuals = Visuals;
    Visuals.__name__ = "Visuals";
},
/* core/util/svg.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    /**
     * Based on https://github.com/gliffy/canvas2svg
     */
    var affine_1 = require(191) /* ./affine */;
    var types_1 = require(123) /* ./types */;
    var dom_1 = require(187) /* ../dom */;
    // helper function that generates a random string
    function randomString(holder) {
        if (!holder) {
            throw new Error("cannot create a random attribute name for an undefined object");
        }
        var chars = "ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz";
        var randomstring = "";
        do {
            randomstring = "";
            for (var i = 0; i < 12; i++) {
                randomstring += chars[Math.floor(Math.random() * chars.length)];
            }
        } while (holder[randomstring]);
        return randomstring;
    }
    // helper function to map named to numbered entities
    function createNamedToNumberedLookup(input, radix) {
        var lookup = new Map();
        var items = input.split(',');
        radix = radix || 10;
        // Map from named to numbered entities.
        for (var i = 0; i < items.length; i += 2) {
            var entity = '&' + items[i + 1] + ';';
            var base10 = parseInt(items[i], radix);
            lookup.set(entity, '&#' + base10 + ';');
        }
        // FF and IE need to create a regex from hex values ie &nbsp; == \xa0
        lookup.set("\\xa0", '&#160;');
        return lookup;
    }
    // helper function to map canvas-textAlign to svg-textAnchor
    function getTextAnchor(textAlign) {
        // TODO: support rtl languages
        var mapping = { left: "start", right: "end", center: "middle", start: "start", end: "end" };
        return mapping[textAlign] || mapping.start;
    }
    // helper function to map canvas-textBaseline to svg-dominantBaseline
    function getDominantBaseline(textBaseline) {
        // INFO: not supported in all browsers
        var mapping = { alphabetic: "alphabetic", hanging: "hanging", top: "text-before-edge", bottom: "text-after-edge", middle: "central" };
        return mapping[textBaseline] || mapping.alphabetic;
    }
    // Unpack entities lookup where the numbers are in radix 32 to reduce the size
    // entity mapping courtesy of tinymce
    var namedEntities = createNamedToNumberedLookup('50,nbsp,51,iexcl,52,cent,53,pound,54,curren,55,yen,56,brvbar,57,sect,58,uml,59,copy,' +
        '5a,ordf,5b,laquo,5c,not,5d,shy,5e,reg,5f,macr,5g,deg,5h,plusmn,5i,sup2,5j,sup3,5k,acute,' +
        '5l,micro,5m,para,5n,middot,5o,cedil,5p,sup1,5q,ordm,5r,raquo,5s,frac14,5t,frac12,5u,frac34,' +
        '5v,iquest,60,Agrave,61,Aacute,62,Acirc,63,Atilde,64,Auml,65,Aring,66,AElig,67,Ccedil,' +
        '68,Egrave,69,Eacute,6a,Ecirc,6b,Euml,6c,Igrave,6d,Iacute,6e,Icirc,6f,Iuml,6g,ETH,6h,Ntilde,' +
        '6i,Ograve,6j,Oacute,6k,Ocirc,6l,Otilde,6m,Ouml,6n,times,6o,Oslash,6p,Ugrave,6q,Uacute,' +
        '6r,Ucirc,6s,Uuml,6t,Yacute,6u,THORN,6v,szlig,70,agrave,71,aacute,72,acirc,73,atilde,74,auml,' +
        '75,aring,76,aelig,77,ccedil,78,egrave,79,eacute,7a,ecirc,7b,euml,7c,igrave,7d,iacute,7e,icirc,' +
        '7f,iuml,7g,eth,7h,ntilde,7i,ograve,7j,oacute,7k,ocirc,7l,otilde,7m,ouml,7n,divide,7o,oslash,' +
        '7p,ugrave,7q,uacute,7r,ucirc,7s,uuml,7t,yacute,7u,thorn,7v,yuml,ci,fnof,sh,Alpha,si,Beta,' +
        'sj,Gamma,sk,Delta,sl,Epsilon,sm,Zeta,sn,Eta,so,Theta,sp,Iota,sq,Kappa,sr,Lambda,ss,Mu,' +
        'st,Nu,su,Xi,sv,Omicron,t0,Pi,t1,Rho,t3,Sigma,t4,Tau,t5,Upsilon,t6,Phi,t7,Chi,t8,Psi,' +
        't9,Omega,th,alpha,ti,beta,tj,gamma,tk,delta,tl,epsilon,tm,zeta,tn,eta,to,theta,tp,iota,' +
        'tq,kappa,tr,lambda,ts,mu,tt,nu,tu,xi,tv,omicron,u0,pi,u1,rho,u2,sigmaf,u3,sigma,u4,tau,' +
        'u5,upsilon,u6,phi,u7,chi,u8,psi,u9,omega,uh,thetasym,ui,upsih,um,piv,812,bull,816,hellip,' +
        '81i,prime,81j,Prime,81u,oline,824,frasl,88o,weierp,88h,image,88s,real,892,trade,89l,alefsym,' +
        '8cg,larr,8ch,uarr,8ci,rarr,8cj,darr,8ck,harr,8dl,crarr,8eg,lArr,8eh,uArr,8ei,rArr,8ej,dArr,' +
        '8ek,hArr,8g0,forall,8g2,part,8g3,exist,8g5,empty,8g7,nabla,8g8,isin,8g9,notin,8gb,ni,8gf,prod,' +
        '8gh,sum,8gi,minus,8gn,lowast,8gq,radic,8gt,prop,8gu,infin,8h0,ang,8h7,and,8h8,or,8h9,cap,8ha,cup,' +
        '8hb,int,8hk,there4,8hs,sim,8i5,cong,8i8,asymp,8j0,ne,8j1,equiv,8j4,le,8j5,ge,8k2,sub,8k3,sup,8k4,' +
        'nsub,8k6,sube,8k7,supe,8kl,oplus,8kn,otimes,8l5,perp,8m5,sdot,8o8,lceil,8o9,rceil,8oa,lfloor,8ob,' +
        'rfloor,8p9,lang,8pa,rang,9ea,loz,9j0,spades,9j3,clubs,9j5,hearts,9j6,diams,ai,OElig,aj,oelig,b0,' +
        'Scaron,b1,scaron,bo,Yuml,m6,circ,ms,tilde,802,ensp,803,emsp,809,thinsp,80c,zwnj,80d,zwj,80e,lrm,' +
        '80f,rlm,80j,ndash,80k,mdash,80o,lsquo,80p,rsquo,80q,sbquo,80s,ldquo,80t,rdquo,80u,bdquo,810,dagger,' +
        '811,Dagger,81g,permil,81p,lsaquo,81q,rsaquo,85c,euro', 32);
    // Some basic mappings for attributes and default values.
    var STYLES = {
        strokeStyle: {
            svgAttr: "stroke",
            canvas: "#000000",
            svg: "none",
            apply: "stroke",
        },
        fillStyle: {
            svgAttr: "fill",
            canvas: "#000000",
            svg: null,
            apply: "fill",
        },
        lineCap: {
            svgAttr: "stroke-linecap",
            canvas: "butt",
            svg: "butt",
            apply: "stroke",
        },
        lineJoin: {
            svgAttr: "stroke-linejoin",
            canvas: "miter",
            svg: "miter",
            apply: "stroke",
        },
        miterLimit: {
            svgAttr: "stroke-miterlimit",
            canvas: 10,
            svg: 4,
            apply: "stroke",
        },
        lineWidth: {
            svgAttr: "stroke-width",
            canvas: 1,
            svg: 1,
            apply: "stroke",
        },
        globalAlpha: {
            svgAttr: "opacity",
            canvas: 1,
            svg: 1,
            apply: "fill stroke",
        },
        font: {
            // font converts to multiple svg attributes, there is custom logic for this
            canvas: "10px sans-serif",
        },
        shadowColor: {
            canvas: "#000000",
        },
        shadowOffsetX: {
            canvas: 0,
        },
        shadowOffsetY: {
            canvas: 0,
        },
        shadowBlur: {
            canvas: 0,
        },
        textAlign: {
            canvas: "start",
        },
        textBaseline: {
            canvas: "alphabetic",
        },
        lineDash: {
            svgAttr: "stroke-dasharray",
            canvas: [],
            svg: null,
            apply: "stroke",
        },
    };
    var CanvasGradient = /** @class */ (function () {
        function CanvasGradient(gradientNode, ctx) {
            this.__root = gradientNode;
            this.__ctx = ctx;
        }
        /**
         * Adds a color stop to the gradient root
         */
        CanvasGradient.prototype.addColorStop = function (offset, color) {
            var stop = this.__ctx.__createElement("stop");
            stop.setAttribute("offset", "" + offset);
            if (color.indexOf("rgba") !== -1) {
                // separate alpha value, since webkit can't handle it
                var regex = /rgba\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d?\.?\d*)\s*\)/gi;
                var matches = regex.exec(color);
                var _d = tslib_1.__read(matches, 5), r = _d[1], g = _d[2], b = _d[3], a = _d[4];
                stop.setAttribute("stop-color", "rgb(" + r + "," + g + "," + b + ")");
                stop.setAttribute("stop-opacity", a);
            }
            else {
                stop.setAttribute("stop-color", color);
            }
            this.__root.appendChild(stop);
        };
        return CanvasGradient;
    }());
    CanvasGradient.__name__ = "CanvasGradient";
    var CanvasPattern = /** @class */ (function () {
        function CanvasPattern(pattern, ctx) {
            this.__root = pattern;
            this.__ctx = ctx;
        }
        return CanvasPattern;
    }());
    CanvasPattern.__name__ = "CanvasPattern";
    /**
     * The mock canvas context
     * @param o - options include:
     * ctx - existing Context2D to wrap around
     * width - width of your canvas (defaults to 500)
     * height - height of your canvas (defaults to 500)
     * document - the document object (defaults to the current document)
     */
    var SVGRenderingContext2D /*implements CanvasRenderingContext2D*/ = /** @class */ (function () {
        function SVGRenderingContext2D(options) {
            var _a, _b, _c;
            this.__currentPosition = null;
            this.__currentElementsToStyle = null;
            this._transform = new affine_1.AffineTransform();
            this._clip_path = null;
            this.__document = (_a = options === null || options === void 0 ? void 0 : options.document) !== null && _a !== void 0 ? _a : document;
            // allow passing in an existing context to wrap around
            // if a context is passed in, we know a canvas already exist
            if (options === null || options === void 0 ? void 0 : options.ctx) {
                this.__ctx = options.ctx;
            }
            else {
                this.__canvas = this.__document.createElement("canvas");
                this.__ctx = this.__canvas.getContext("2d");
            }
            this.__setDefaultStyles();
            this.__stack = [];
            // the root svg element
            this.__root = this.__document.createElementNS("http://www.w3.org/2000/svg", "svg");
            this.__root.setAttribute("version", "1.1");
            this.__root.setAttribute("xmlns", "http://www.w3.org/2000/svg");
            this.__root.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:xlink", "http://www.w3.org/1999/xlink");
            this.width = (_b = options === null || options === void 0 ? void 0 : options.width) !== null && _b !== void 0 ? _b : 500;
            this.height = (_c = options === null || options === void 0 ? void 0 : options.height) !== null && _c !== void 0 ? _c : 500;
            // make sure we don't generate the same ids in defs
            this.__ids = {};
            // defs tag
            this.__defs = this.__document.createElementNS("http://www.w3.org/2000/svg", "defs");
            this.__root.appendChild(this.__defs);
        }
        Object.defineProperty(SVGRenderingContext2D.prototype, "canvas", {
            get: function () {
                // XXX: point back to this instance
                return this;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SVGRenderingContext2D.prototype, "width", {
            get: function () {
                return this._width;
            },
            set: function (width) {
                this._width = width;
                this.__root.setAttribute("width", "" + width);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SVGRenderingContext2D.prototype, "height", {
            get: function () {
                return this._height;
            },
            set: function (height) {
                this._height = height;
                this.__root.setAttribute("height", "" + height);
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Creates the specified svg element
         */
        SVGRenderingContext2D.prototype.__createElement = function (elementName, properties, resetFill) {
            var e_1, _d;
            if (properties === void 0) {
                properties = {};
            }
            if (resetFill === void 0) {
                resetFill = false;
            }
            var element = this.__document.createElementNS("http://www.w3.org/2000/svg", elementName);
            if (resetFill) {
                // if fill or stroke is not specified, the svg element should not display. By default SVG's fill is black.
                element.setAttribute("fill", "none");
                element.setAttribute("stroke", "none");
            }
            var keys = Object.keys(properties);
            try {
                for (var keys_1 = tslib_1.__values(keys), keys_1_1 = keys_1.next(); !keys_1_1.done; keys_1_1 = keys_1.next()) {
                    var key = keys_1_1.value;
                    element.setAttribute(key, "" + properties[key]);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (keys_1_1 && !keys_1_1.done && (_d = keys_1.return))
                        _d.call(keys_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return element;
        };
        /**
         * Applies default canvas styles to the context
         */
        SVGRenderingContext2D.prototype.__setDefaultStyles = function () {
            // default 2d canvas context properties see:http://www.w3.org/TR/2dcontext/
            var keys = Object.keys(STYLES);
            var self = this;
            for (var i = 0; i < keys.length; i++) {
                var key = keys[i];
                self[key] = STYLES[key].canvas;
            }
        };
        /**
         * Applies styles on restore
         */
        SVGRenderingContext2D.prototype.__applyStyleState = function (styleState) {
            var keys = Object.keys(styleState);
            var self = this;
            for (var i = 0; i < keys.length; i++) {
                var key = keys[i];
                self[key] = styleState[key];
            }
        };
        /**
         * Gets the current style state
         */
        SVGRenderingContext2D.prototype.__getStyleState = function () {
            var keys = Object.keys(STYLES);
            var styleState = {};
            for (var i = 0; i < keys.length; i++) {
                var key = keys[i];
                styleState[key] = this[key];
            }
            return styleState;
        };
        /**
         * Apples the current styles to the current SVG element. On "ctx.fill" or "ctx.stroke"
         */
        SVGRenderingContext2D.prototype.__applyStyleToCurrentElement = function (type) {
            var e_2, _d, e_3, _e;
            var currentElement = this.__currentElement;
            var currentStyleGroup = this.__currentElementsToStyle;
            if (currentStyleGroup != null) {
                currentElement.setAttribute(type, "");
                currentElement = currentStyleGroup.element;
                try {
                    for (var _f = tslib_1.__values(currentStyleGroup.children), _g = _f.next(); !_g.done; _g = _f.next()) {
                        var node = _g.value;
                        node.setAttribute(type, "");
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (_g && !_g.done && (_d = _f.return))
                            _d.call(_f);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
            }
            var keys = Object.keys(STYLES);
            for (var i = 0; i < keys.length; i++) {
                var style = STYLES[keys[i]];
                var value = this[keys[i]];
                if (style.apply) {
                    if (value instanceof CanvasPattern) {
                        try {
                            for (var _h = (e_3 = void 0, tslib_1.__values(tslib_1.__spread(value.__ctx.__defs.childNodes))), _j = _h.next(); !_j.done; _j = _h.next()) {
                                var def = _j.value;
                                if (def instanceof Element) {
                                    var id_1 = def.getAttribute("id");
                                    this.__ids[id_1] = id_1;
                                    this.__defs.appendChild(def);
                                }
                            }
                        }
                        catch (e_3_1) {
                            e_3 = { error: e_3_1 };
                        }
                        finally {
                            try {
                                if (_j && !_j.done && (_e = _h.return))
                                    _e.call(_h);
                            }
                            finally {
                                if (e_3)
                                    throw e_3.error;
                            }
                        }
                        var id = value.__root.getAttribute("id");
                        currentElement.setAttribute(style.apply, "url(#" + id + ")");
                    }
                    else if (value instanceof CanvasGradient) {
                        var id = value.__root.getAttribute("id");
                        currentElement.setAttribute(style.apply, "url(#" + id + ")");
                    }
                    else if (style.apply.indexOf(type) !== -1 && style.svg !== value) {
                        if ((style.svgAttr === "stroke" || style.svgAttr === "fill") && types_1.isString(value) && value.indexOf("rgba") !== -1) {
                            // separate alpha value, since illustrator can't handle it
                            var regex = /rgba\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d?\.?\d*)\s*\)/gi;
                            var matches = regex.exec(value);
                            var _k = tslib_1.__read(matches, 5), r = _k[1], g = _k[2], b = _k[3], a = _k[4];
                            currentElement.setAttribute(style.svgAttr, "rgb(" + r + "," + g + "," + b + ")");
                            // should take globalAlpha here
                            var opacity = parseFloat(a);
                            var globalAlpha = this.globalAlpha;
                            if (globalAlpha != null) {
                                opacity *= globalAlpha;
                            }
                            currentElement.setAttribute(style.svgAttr + "-opacity", "" + opacity);
                        }
                        else {
                            var attr = style.svgAttr;
                            if (keys[i] === 'globalAlpha') {
                                attr = type + '-' + style.svgAttr;
                                if (currentElement.getAttribute(attr)) {
                                    // fill-opacity or stroke-opacity has already been set by stroke or fill.
                                    continue;
                                }
                            }
                            // otherwise only update attribute if right type, and not svg default
                            currentElement.setAttribute(attr, "" + value);
                        }
                    }
                }
            }
        };
        /**
          * Returns the serialized value of the svg so far
          * @param fixNamedEntities - Standalone SVG doesn't support named entities, which document.createTextNode encodes.
          *                           If true, we attempt to find all named entities and encode it as a numeric entity.
          * @return serialized svg
          */
        SVGRenderingContext2D.prototype.get_serialized_svg = function (fixNamedEntities) {
            var e_4, _d;
            if (fixNamedEntities === void 0) {
                fixNamedEntities = false;
            }
            var serialized = new XMLSerializer().serializeToString(this.__root);
            // IE search for a duplicate xmnls because they didn't implement setAttributeNS correctly
            var xmlns = /xmlns="http:\/\/www\.w3\.org\/2000\/svg".+xmlns="http:\/\/www\.w3\.org\/2000\/svg/gi;
            if (xmlns.test(serialized)) {
                serialized = serialized.replace('xmlns="http://www.w3.org/2000/svg', 'xmlns:xlink="http://www.w3.org/1999/xlink');
            }
            if (fixNamedEntities) {
                try {
                    // loop over each named entity and replace with the proper equivalent.
                    for (var namedEntities_1 = tslib_1.__values(namedEntities), namedEntities_1_1 = namedEntities_1.next(); !namedEntities_1_1.done; namedEntities_1_1 = namedEntities_1.next()) {
                        var _e = tslib_1.__read(namedEntities_1_1.value, 2), key = _e[0], value = _e[1];
                        var regexp = new RegExp(key, "gi");
                        if (regexp.test(serialized)) {
                            serialized = serialized.replace(regexp, value);
                        }
                    }
                }
                catch (e_4_1) {
                    e_4 = { error: e_4_1 };
                }
                finally {
                    try {
                        if (namedEntities_1_1 && !namedEntities_1_1.done && (_d = namedEntities_1.return))
                            _d.call(namedEntities_1);
                    }
                    finally {
                        if (e_4)
                            throw e_4.error;
                    }
                }
            }
            return serialized;
        };
        SVGRenderingContext2D.prototype.get_svg = function () {
            return this.__root;
        };
        /**
          * Will generate a group tag.
          */
        SVGRenderingContext2D.prototype.save = function () {
            this.__stack.push({
                transform: this._transform,
                clip_path: this._clip_path,
                attributes: this.__getStyleState(),
            });
            this._transform = this._transform.clone();
        };
        /**
          * Sets current element to parent, or just root if already root
          */
        SVGRenderingContext2D.prototype.restore = function () {
            if (this.__stack.length == 0)
                return;
            var _d = this.__stack.pop(), transform = _d.transform, clip_path = _d.clip_path, attributes = _d.attributes;
            this._transform = transform;
            this._clip_path = clip_path;
            this.__applyStyleState(attributes);
        };
        SVGRenderingContext2D.prototype._apply_transform = function (element, transform) {
            if (transform === void 0) {
                transform = this._transform;
            }
            if (!transform.is_identity) {
                element.setAttribute("transform", transform.toString());
            }
        };
        /**
          *  scales the current element
          */
        SVGRenderingContext2D.prototype.scale = function (x, y) {
            if (!isFinite(x) || (y != null && !isFinite(y)))
                return;
            this._transform.scale(x, y !== null && y !== void 0 ? y : x);
        };
        /**
          * rotates the current element
          */
        SVGRenderingContext2D.prototype.rotate = function (angle) {
            if (!isFinite(angle))
                return;
            this._transform.rotate(angle);
        };
        /**
          * translates the current element
          */
        SVGRenderingContext2D.prototype.translate = function (x, y) {
            if (!isFinite(x + y))
                return;
            this._transform.translate(x, y);
        };
        /**
          * applies a transform to the current element
          */
        SVGRenderingContext2D.prototype.transform = function (a, b, c, d, e, f) {
            if (!isFinite(a + b + c + d + e + f))
                return;
            this._transform.transform(a, b, c, d, e, f);
        };
        /**
          * Create a new Path Element
          */
        SVGRenderingContext2D.prototype.beginPath = function () {
            // Note that there is only one current default path, it is not part of the drawing state.
            // See also: https://html.spec.whatwg.org/multipage/scripting.html#current-default-path
            this.__currentDefaultPath = "";
            this.__currentPosition = null;
            var path = this.__createElement("path", {}, true);
            this.__root.appendChild(path);
            this.__currentElement = path;
        };
        /**
          * Helper function to apply currentDefaultPath to current path element
          */
        SVGRenderingContext2D.prototype.__applyCurrentDefaultPath = function () {
            var currentElement = this.__currentElement;
            if (currentElement.nodeName === "path") {
                currentElement.setAttribute("d", this.__currentDefaultPath);
            }
            else {
                console.error("Attempted to apply path command to node", currentElement.nodeName);
            }
        };
        /**
          * Helper function to add path command
          */
        SVGRenderingContext2D.prototype.__addPathCommand = function (x, y, path) {
            var separator = !this.__currentDefaultPath ? "" : " ";
            this.__currentDefaultPath += separator + path;
            this.__currentPosition = { x: x, y: y };
        };
        Object.defineProperty(SVGRenderingContext2D.prototype, "_hasCurrentDefaultPath", {
            get: function () {
                return !!this.__currentDefaultPath;
            },
            enumerable: true,
            configurable: true
        });
        /**
          * Adds the move command to the current path element,
          * if the currentPathElement is not empty create a new path element
          */
        SVGRenderingContext2D.prototype.moveTo = function (x, y) {
            if (!isFinite(x + y))
                return;
            if (this.__currentElement.nodeName !== "path") {
                this.beginPath();
            }
            // creates a new subpath with the given point
            var _d = tslib_1.__read(this._transform.apply(x, y), 2), tx = _d[0], ty = _d[1];
            this.__addPathCommand(tx, ty, "M " + tx + " " + ty);
        };
        /**
          * Closes the current path
          */
        SVGRenderingContext2D.prototype.closePath = function () {
            if (this._hasCurrentDefaultPath) {
                this.__addPathCommand(NaN, NaN, "Z");
            }
        };
        /**
          * Adds a line to command
          */
        SVGRenderingContext2D.prototype.lineTo = function (x, y) {
            if (!isFinite(x + y))
                return;
            if (!this._hasCurrentDefaultPath)
                this.moveTo(x, y);
            else {
                var _d = tslib_1.__read(this._transform.apply(x, y), 2), tx = _d[0], ty = _d[1];
                this.__addPathCommand(tx, ty, "L " + tx + " " + ty);
            }
        };
        /**
          * Add a bezier command
          */
        SVGRenderingContext2D.prototype.bezierCurveTo = function (cp1x, cp1y, cp2x, cp2y, x, y) {
            if (!isFinite(cp1x + cp1y + cp2x + cp2y + x + y))
                return;
            var _d = tslib_1.__read(this._transform.apply(x, y), 2), tx = _d[0], ty = _d[1];
            var _e = tslib_1.__read(this._transform.apply(cp1x, cp1y), 2), tcp1x = _e[0], tcp1y = _e[1];
            var _f = tslib_1.__read(this._transform.apply(cp2x, cp2y), 2), tcp2x = _f[0], tcp2y = _f[1];
            this.__addPathCommand(tx, ty, "C " + tcp1x + " " + tcp1y + " " + tcp2x + " " + tcp2y + " " + tx + " " + ty);
        };
        /**
          * Adds a quadratic curve to command
          */
        SVGRenderingContext2D.prototype.quadraticCurveTo = function (cpx, cpy, x, y) {
            if (!isFinite(cpx + cpy + x + y))
                return;
            var _d = tslib_1.__read(this._transform.apply(x, y), 2), tx = _d[0], ty = _d[1];
            var _e = tslib_1.__read(this._transform.apply(cpx, cpy), 2), tcpx = _e[0], tcpy = _e[1];
            this.__addPathCommand(tx, ty, "Q " + tcpx + " " + tcpy + " " + tx + " " + ty);
        };
        /**
          * Adds the arcTo to the current path
          *
          * @see http://www.w3.org/TR/2015/WD-2dcontext-20150514/#dom-context-2d-arcto
          */
        SVGRenderingContext2D.prototype.arcTo = function (x1, y1, x2, y2, radius) {
            if (!isFinite(x1 + y1 + x2 + y2 + radius))
                return;
            // Let the point (x0, y0) be the last point in the subpath.
            if (this.__currentPosition == null)
                return;
            var x0 = this.__currentPosition.x;
            var y0 = this.__currentPosition.y;
            // Negative values for radius must cause the implementation to throw an IndexSizeError exception.
            if (radius < 0) {
                throw new Error("IndexSizeError: The radius provided (" + radius + ") is negative.");
            }
            // If the point (x0, y0) is equal to the point (x1, y1),
            // or if the point (x1, y1) is equal to the point (x2, y2),
            // or if the radius radius is zero,
            // then the method must add the point (x1, y1) to the subpath,
            // and connect that point to the previous point (x0, y0) by a straight line.
            if (((x0 === x1) && (y0 === y1)) || ((x1 === x2) && (y1 === y2)) || (radius === 0)) {
                this.lineTo(x1, y1);
                return;
            }
            function normalize(_d) {
                var _e = tslib_1.__read(_d, 2), x = _e[0], y = _e[1];
                var len = Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2));
                return [x / len, y / len];
            }
            // Otherwise, if the points (x0, y0), (x1, y1), and (x2, y2) all lie on a single straight line,
            // then the method must add the point (x1, y1) to the subpath,
            // and connect that point to the previous point (x0, y0) by a straight line.
            var unit_vec_p1_p0 = normalize([x0 - x1, y0 - y1]);
            var unit_vec_p1_p2 = normalize([x2 - x1, y2 - y1]);
            if (unit_vec_p1_p0[0] * unit_vec_p1_p2[1] === unit_vec_p1_p0[1] * unit_vec_p1_p2[0]) {
                this.lineTo(x1, y1);
                return;
            }
            // Otherwise, let The Arc be the shortest arc given by circumference of the circle that has radius radius,
            // and that has one point tangent to the half-infinite line that crosses the point (x0, y0) and ends at the point (x1, y1),
            // and that has a different point tangent to the half-infinite line that ends at the point (x1, y1), and crosses the point (x2, y2).
            // The points at which this circle touches these two lines are called the start and end tangent points respectively.
            // note that both vectors are unit vectors, so the length is 1
            var cos = (unit_vec_p1_p0[0] * unit_vec_p1_p2[0] + unit_vec_p1_p0[1] * unit_vec_p1_p2[1]);
            var theta = Math.acos(Math.abs(cos));
            // Calculate origin
            var unit_vec_p1_origin = normalize([
                unit_vec_p1_p0[0] + unit_vec_p1_p2[0],
                unit_vec_p1_p0[1] + unit_vec_p1_p2[1],
            ]);
            var len_p1_origin = radius / Math.sin(theta / 2);
            var x = x1 + len_p1_origin * unit_vec_p1_origin[0];
            var y = y1 + len_p1_origin * unit_vec_p1_origin[1];
            // Calculate start angle and end angle
            // rotate 90deg clockwise (note that y axis points to its down)
            var unit_vec_origin_start_tangent = [
                -unit_vec_p1_p0[1],
                unit_vec_p1_p0[0],
            ];
            // rotate 90deg counter clockwise (note that y axis points to its down)
            var unit_vec_origin_end_tangent = [
                unit_vec_p1_p2[1],
                -unit_vec_p1_p2[0],
            ];
            function getAngle(vector) {
                // get angle (clockwise) between vector and (1, 0)
                var x = vector[0];
                var y = vector[1];
                if (y >= 0) { // note that y axis points to its down
                    return Math.acos(x);
                }
                else {
                    return -Math.acos(x);
                }
            }
            var startAngle = getAngle(unit_vec_origin_start_tangent);
            var endAngle = getAngle(unit_vec_origin_end_tangent);
            // Connect the point (x0, y0) to the start tangent point by a straight line
            this.lineTo(x + unit_vec_origin_start_tangent[0] * radius, y + unit_vec_origin_start_tangent[1] * radius);
            // Connect the start tangent point to the end tangent point by arc
            // and adding the end tangent point to the subpath.
            this.arc(x, y, radius, startAngle, endAngle);
        };
        /**
          * Sets the stroke property on the current element
          */
        SVGRenderingContext2D.prototype.stroke = function () {
            if (this.__currentElement.nodeName === "path") {
                this.__currentElement.setAttribute("paint-order", "fill");
            }
            this.__applyCurrentDefaultPath();
            this.__applyStyleToCurrentElement("stroke");
            if (this._clip_path != null) {
                this.__currentElement.setAttribute("clip-path", this._clip_path);
            }
        };
        /**
          * Sets fill properties on the current element
          */
        SVGRenderingContext2D.prototype.fill = function () {
            if (this.__currentElement.nodeName === "path") {
                this.__currentElement.setAttribute("paint-order", "stroke");
            }
            this.__applyCurrentDefaultPath();
            this.__applyStyleToCurrentElement("fill");
            if (this._clip_path != null) {
                this.__currentElement.setAttribute("clip-path", this._clip_path);
            }
        };
        /**
          *  Adds a rectangle to the path.
          */
        SVGRenderingContext2D.prototype.rect = function (x, y, width, height) {
            if (!isFinite(x + y + width + height))
                return;
            if (this.__currentElement.nodeName !== "path") {
                this.beginPath();
            }
            this.moveTo(x, y);
            this.lineTo(x + width, y);
            this.lineTo(x + width, y + height);
            this.lineTo(x, y + height);
            this.lineTo(x, y);
        };
        /**
          * adds a rectangle element
          */
        SVGRenderingContext2D.prototype.fillRect = function (x, y, width, height) {
            if (!isFinite(x + y + width + height))
                return;
            this.beginPath();
            this.rect(x, y, width, height);
            this.fill();
        };
        /**
          * Draws a rectangle with no fill
          * @param x
          * @param y
          * @param width
          * @param height
          */
        SVGRenderingContext2D.prototype.strokeRect = function (x, y, width, height) {
            if (!isFinite(x + y + width + height))
                return;
            this.beginPath();
            this.rect(x, y, width, height);
            this.stroke();
        };
        /**
          * Clear entire canvas:
          * 1. save current transforms
          * 2. remove all the childNodes of the root g element
          */
        SVGRenderingContext2D.prototype.__clearCanvas = function () {
            dom_1.empty(this.__defs);
            dom_1.empty(this.__root);
            this.__root.appendChild(this.__defs);
            this.__currentElement = this.__root;
        };
        /**
          * "Clears" a canvas by just drawing a white rectangle in the current group.
          */
        SVGRenderingContext2D.prototype.clearRect = function (x, y, width, height) {
            if (!isFinite(x + y + width + height))
                return;
            if (x === 0 && y === 0 && width === this.width && height === this.height) {
                this.__clearCanvas();
                return;
            }
            var rect = this.__createElement("rect", { x: x, y: y, width: width, height: height, fill: "#FFFFFF" }, true);
            this._apply_transform(rect);
            this.__root.appendChild(rect);
        };
        /**
          * Adds a linear gradient to a defs tag.
          * Returns a canvas gradient object that has a reference to it's parent def
          */
        SVGRenderingContext2D.prototype.createLinearGradient = function (x1, y1, x2, y2) {
            if (!isFinite(x1 + y1 + x2 + y2))
                throw new Error("The provided double value is non-finite");
            var _d = tslib_1.__read(this._transform.apply(x1, y1), 2), tx1 = _d[0], ty1 = _d[1];
            var _e = tslib_1.__read(this._transform.apply(x2, y2), 2), tx2 = _e[0], ty2 = _e[1];
            var grad = this.__createElement("linearGradient", {
                id: randomString(this.__ids),
                x1: tx1 + "px",
                x2: tx2 + "px",
                y1: ty1 + "px",
                y2: ty2 + "px",
                gradientUnits: "userSpaceOnUse",
            }, false);
            this.__defs.appendChild(grad);
            return new CanvasGradient(grad, this);
        };
        /**
          * Adds a radial gradient to a defs tag.
          * Returns a canvas gradient object that has a reference to it's parent def
          */
        SVGRenderingContext2D.prototype.createRadialGradient = function (x0, y0, _r0, x1, y1, r1) {
            if (!isFinite(x0 + y0 + _r0 + x1 + y1 + r1))
                throw new Error("The provided double value is non-finite");
            var _d = tslib_1.__read(this._transform.apply(x0, y0), 2), tx0 = _d[0], ty0 = _d[1];
            var _e = tslib_1.__read(this._transform.apply(x1, y1), 2), tx1 = _e[0], ty1 = _e[1];
            var grad = this.__createElement("radialGradient", {
                id: randomString(this.__ids),
                cx: tx1 + "px",
                cy: ty1 + "px",
                r: r1 + "px",
                fx: tx0 + "px",
                fy: ty0 + "px",
                gradientUnits: "userSpaceOnUse",
            }, false);
            this.__defs.appendChild(grad);
            return new CanvasGradient(grad, this);
        };
        /**
          * Parses the font string and returns svg mapping
          */
        SVGRenderingContext2D.prototype.__parseFont = function () {
            var regex = /^\s*(?=(?:(?:[-a-z]+\s*){0,2}(italic|oblique))?)(?=(?:(?:[-a-z]+\s*){0,2}(small-caps))?)(?=(?:(?:[-a-z]+\s*){0,2}(bold(?:er)?|lighter|[1-9]00))?)(?:(?:normal|\1|\2|\3)\s*){0,3}((?:xx?-)?(?:small|large)|medium|smaller|larger|[.\d]+(?:\%|in|[cem]m|ex|p[ctx]))(?:\s*\/\s*(normal|[.\d]+(?:\%|in|[cem]m|ex|p[ctx])))?\s*([-,\'\"\sa-z0-9]+?)\s*$/i;
            var fontPart = regex.exec(this.font);
            var data = {
                style: fontPart[1] || 'normal',
                size: fontPart[4] || '10px',
                family: fontPart[6] || 'sans-serif',
                weight: fontPart[3] || 'normal',
                decoration: fontPart[2] || 'normal',
            };
            // canvas doesn't support underline natively, but we can pass this attribute
            if (this.__fontUnderline === "underline") {
                data.decoration = "underline";
            }
            // canvas also doesn't support linking, but we can pass this as well
            if (this.__fontHref != null) {
                data.href = this.__fontHref;
            }
            return data;
        };
        /**
          * Helper to link text fragments
          */
        SVGRenderingContext2D.prototype.__wrapTextLink = function (font, element) {
            if (font.href) {
                var a = this.__createElement("a");
                a.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", font.href);
                a.appendChild(element);
                return a;
            }
            return element;
        };
        /**
          * Fills or strokes text
          */
        SVGRenderingContext2D.prototype.__applyText = function (text, x, y, action) {
            var font = this.__parseFont();
            var textElement = this.__createElement("text", {
                "font-family": font.family,
                "font-size": font.size,
                "font-style": font.style,
                "font-weight": font.weight,
                "text-decoration": font.decoration,
                x: x,
                y: y,
                "text-anchor": getTextAnchor(this.textAlign),
                "dominant-baseline": getDominantBaseline(this.textBaseline),
            }, true);
            textElement.appendChild(this.__document.createTextNode(text));
            this._apply_transform(textElement);
            this.__currentElement = textElement;
            this.__applyStyleToCurrentElement(action);
            this.__root.appendChild(this.__wrapTextLink(font, textElement));
        };
        /**
          * Creates a text element
          */
        SVGRenderingContext2D.prototype.fillText = function (text, x, y) {
            if (text == null || !isFinite(x + y))
                return;
            this.__applyText(text, x, y, "fill");
        };
        /**
          * Strokes text
          */
        SVGRenderingContext2D.prototype.strokeText = function (text, x, y) {
            if (text == null || !isFinite(x + y))
                return;
            this.__applyText(text, x, y, "stroke");
        };
        /**
          * No need to implement this for svg.
          */
        SVGRenderingContext2D.prototype.measureText = function (text) {
            this.__ctx.font = this.font;
            return this.__ctx.measureText(text);
        };
        SVGRenderingContext2D.prototype.arc = function (x, y, radius, startAngle, endAngle, counterClockwise) {
            if (counterClockwise === void 0) {
                counterClockwise = false;
            }
            if (!isFinite(x + y + radius + startAngle + endAngle))
                return;
            // in canvas no circle is drawn if no angle is provided.
            if (startAngle === endAngle) {
                return;
            }
            startAngle = startAngle % (2 * Math.PI);
            endAngle = endAngle % (2 * Math.PI);
            if (startAngle === endAngle) {
                // circle time! subtract some of the angle so svg is happy (svg elliptical arc can't draw a full circle)
                endAngle = ((endAngle + (2 * Math.PI)) - 0.001 * (counterClockwise ? -1 : 1)) % (2 * Math.PI);
            }
            var endX = x + radius * Math.cos(endAngle);
            var endY = y + radius * Math.sin(endAngle);
            var startX = x + radius * Math.cos(startAngle);
            var startY = y + radius * Math.sin(startAngle);
            var sweepFlag = counterClockwise ? 0 : 1;
            var largeArcFlag = 0;
            var diff = endAngle - startAngle;
            // https://github.com/gliffy/canvas2svg/issues/4
            if (diff < 0) {
                diff += 2 * Math.PI;
            }
            if (counterClockwise) {
                largeArcFlag = diff > Math.PI ? 0 : 1;
            }
            else {
                largeArcFlag = diff > Math.PI ? 1 : 0;
            }
            this.lineTo(startX, startY);
            var rx = radius;
            var ry = radius;
            var xAxisRotation = 0;
            var _d = tslib_1.__read(this._transform.apply(endX, endY), 2), tendX = _d[0], tendY = _d[1];
            this.__addPathCommand(tendX, tendY, "A " + rx + " " + ry + " " + xAxisRotation + " " + largeArcFlag + " " + sweepFlag + " " + tendX + " " + tendY);
        };
        /**
          * Generates a ClipPath from the clip command.
          */
        SVGRenderingContext2D.prototype.clip = function () {
            var clip_path = this.__createElement("clipPath");
            var id = randomString(this.__ids);
            this.__applyCurrentDefaultPath();
            clip_path.setAttribute("id", id);
            clip_path.appendChild(this.__currentElement);
            this.__defs.appendChild(clip_path);
            this._clip_path = "url(#" + id + ")";
        };
        /**
          * Draws a canvas, image or mock context to this canvas.
          * Note that all svg dom manipulation uses node.childNodes rather than node.children for IE support.
          * http://www.whatwg.org/specs/web-apps/current-work/multipage/the-canvas-element.html#dom-context-2d-drawimage
          */
        SVGRenderingContext2D.prototype.drawImage = function (image) {
            var _d, _e, _f, e_5, _g, e_6, _h;
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var dx, dy;
            var dw, dh;
            var sx, sy;
            var sw, sh;
            if (args.length == 2) {
                _d = tslib_1.__read(args, 2), dx = _d[0], dy = _d[1];
                if (!isFinite(dx + dy))
                    return;
                sx = 0;
                sy = 0;
                sw = image.width;
                sh = image.height;
                dw = sw;
                dh = sh;
            }
            else if (args.length == 4) {
                _e = tslib_1.__read(args, 4), dx = _e[0], dy = _e[1], dw = _e[2], dh = _e[3];
                if (!isFinite(dx + dy + dw + dh))
                    return;
                sx = 0;
                sy = 0;
                sw = image.width;
                sh = image.height;
            }
            else if (args.length === 8) {
                _f = tslib_1.__read(args, 8), sx = _f[0], sy = _f[1], sw = _f[2], sh = _f[3], dx = _f[4], dy = _f[5], dw = _f[6], dh = _f[7];
                if (!isFinite(sx + sy + sw + sh + dx + dy + dw + dh))
                    return;
            }
            else {
                throw new Error("Inavlid number of arguments passed to drawImage: " + arguments.length);
            }
            // parent, svg, defs, group, currentElement, svgImage, canvas, context, id
            var parent = this.__root;
            var translateDirective = "translate(" + dx + ", " + dy + ")";
            var transform = this._transform.clone().translate(dx, dy);
            if (image instanceof SVGRenderingContext2D || image instanceof SVGSVGElement) {
                // In the future we may want to clone nodes instead.
                // also I'm currently ignoring dw, dh, sw, sh, sx, sy for a mock context.
                var svg_node = image instanceof SVGSVGElement ? image : image.get_svg();
                var svg = svg_node.cloneNode(true);
                var scope = void 0;
                if (transform.is_identity)
                    scope = parent;
                else {
                    scope = this.__createElement("g");
                    this._apply_transform(scope, transform);
                    parent.appendChild(scope);
                }
                try {
                    for (var _j = tslib_1.__values(tslib_1.__spread(svg.childNodes)), _k = _j.next(); !_k.done; _k = _j.next()) {
                        var child = _k.value;
                        if (child instanceof SVGDefsElement) {
                            try {
                                for (var _l = (e_6 = void 0, tslib_1.__values(tslib_1.__spread(child.childNodes))), _m = _l.next(); !_m.done; _m = _l.next()) {
                                    var def = _m.value;
                                    if (def instanceof Element) {
                                        var id = def.getAttribute("id");
                                        this.__ids[id] = id;
                                        this.__defs.appendChild(def);
                                    }
                                }
                            }
                            catch (e_6_1) {
                                e_6 = { error: e_6_1 };
                            }
                            finally {
                                try {
                                    if (_m && !_m.done && (_h = _l.return))
                                        _h.call(_l);
                                }
                                finally {
                                    if (e_6)
                                        throw e_6.error;
                                }
                            }
                        }
                        else {
                            scope.appendChild(child);
                        }
                    }
                }
                catch (e_5_1) {
                    e_5 = { error: e_5_1 };
                }
                finally {
                    try {
                        if (_k && !_k.done && (_g = _j.return))
                            _g.call(_j);
                    }
                    finally {
                        if (e_5)
                            throw e_5.error;
                    }
                }
            }
            else if (image instanceof HTMLImageElement || image instanceof SVGImageElement) {
                var svgImage = this.__createElement("image");
                svgImage.setAttribute("width", "" + dw);
                svgImage.setAttribute("height", "" + dh);
                svgImage.setAttribute("preserveAspectRatio", "none");
                if (sx || sy || sw !== image.width || sh !== image.height) {
                    // crop the image using a temporary canvas
                    var canvas = this.__document.createElement("canvas");
                    canvas.width = dw;
                    canvas.height = dh;
                    var context = canvas.getContext("2d");
                    context.drawImage(image, sx, sy, sw, sh, 0, 0, dw, dh);
                    image = canvas;
                }
                svgImage.setAttribute("transform", translateDirective);
                var url = image instanceof HTMLCanvasElement ? image.toDataURL() : image.getAttribute("src");
                svgImage.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", url);
                parent.appendChild(svgImage);
            }
            else if (image instanceof HTMLCanvasElement) {
                var svgImage = this.__createElement("image");
                svgImage.setAttribute("width", "" + dw);
                svgImage.setAttribute("height", "" + dh);
                svgImage.setAttribute("preserveAspectRatio", "none");
                // draw canvas onto temporary canvas so that smoothing can be handled
                var canvas = this.__document.createElement("canvas");
                canvas.width = dw;
                canvas.height = dh;
                var context = canvas.getContext("2d");
                context.imageSmoothingEnabled = false;
                context.drawImage(image, sx, sy, sw, sh, 0, 0, dw, dh);
                image = canvas;
                svgImage.setAttribute("transform", translateDirective);
                svgImage.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", image.toDataURL());
                parent.appendChild(svgImage);
            }
        };
        /**
          * Generates a pattern tag
          */
        SVGRenderingContext2D.prototype.createPattern = function (image, _repetition) {
            var e_7, _d, e_8, _e;
            var pattern = this.__document.createElementNS("http://www.w3.org/2000/svg", "pattern");
            var id = randomString(this.__ids);
            pattern.setAttribute("id", id);
            pattern.setAttribute("width", "" + this._to_number(image.width));
            pattern.setAttribute("height", "" + this._to_number(image.height));
            pattern.setAttribute("patternUnits", "userSpaceOnUse");
            if (image instanceof HTMLCanvasElement || image instanceof HTMLImageElement || image instanceof SVGImageElement) {
                var img = this.__document.createElementNS("http://www.w3.org/2000/svg", "image");
                var url = image instanceof HTMLCanvasElement ? image.toDataURL() : image.getAttribute("src");
                img.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", url);
                pattern.appendChild(img);
                this.__defs.appendChild(pattern);
            }
            else if (image instanceof SVGRenderingContext2D) {
                try {
                    for (var _f = tslib_1.__values(tslib_1.__spread(image.__root.childNodes)), _g = _f.next(); !_g.done; _g = _f.next()) {
                        var child = _g.value;
                        if (!(child instanceof SVGDefsElement)) {
                            pattern.appendChild(child);
                        }
                    }
                }
                catch (e_7_1) {
                    e_7 = { error: e_7_1 };
                }
                finally {
                    try {
                        if (_g && !_g.done && (_d = _f.return))
                            _d.call(_f);
                    }
                    finally {
                        if (e_7)
                            throw e_7.error;
                    }
                }
                //pattern.appendChild(image.__root.childNodes[1])
                this.__defs.appendChild(pattern);
            }
            else if (image instanceof SVGSVGElement) {
                try {
                    for (var _h = tslib_1.__values(tslib_1.__spread(image.childNodes)), _j = _h.next(); !_j.done; _j = _h.next()) {
                        var child = _j.value;
                        if (!(child instanceof SVGDefsElement)) {
                            pattern.appendChild(child);
                        }
                    }
                }
                catch (e_8_1) {
                    e_8 = { error: e_8_1 };
                }
                finally {
                    try {
                        if (_j && !_j.done && (_e = _h.return))
                            _e.call(_h);
                    }
                    finally {
                        if (e_8)
                            throw e_8.error;
                    }
                }
                //pattern.appendChild(image.__root.childNodes[1])
                this.__defs.appendChild(pattern);
            }
            else {
                throw new Error("unsupported");
            }
            return new CanvasPattern(pattern, this);
        };
        SVGRenderingContext2D.prototype.setLineDash = function (dashArray) {
            if (dashArray && dashArray.length > 0) {
                this.lineDash = dashArray.join(",");
            }
            else {
                this.lineDash = null;
            }
        };
        SVGRenderingContext2D.prototype._to_number = function (val) {
            return types_1.isNumber(val) ? val : val.baseVal.value;
        };
        return SVGRenderingContext2D;
    }());
    exports.SVGRenderingContext2D = SVGRenderingContext2D;
    SVGRenderingContext2D.__name__ = "SVGRenderingContext2D";
},
/* core/util/affine.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var sin = Math.sin, cos = Math.cos;
    var AffineTransform = /** @class */ (function () {
        function AffineTransform(a, b, c, d, e, f) {
            if (a === void 0) {
                a = 1;
            }
            if (b === void 0) {
                b = 0;
            }
            if (c === void 0) {
                c = 0;
            }
            if (d === void 0) {
                d = 1;
            }
            if (e === void 0) {
                e = 0;
            }
            if (f === void 0) {
                f = 0;
            }
            this.a = a;
            this.b = b;
            this.c = c;
            this.d = d;
            this.e = e;
            this.f = f;
        }
        AffineTransform.prototype.toString = function () {
            var _a = this, a = _a.a, b = _a.b, c = _a.c, d = _a.d, e = _a.e, f = _a.f;
            return "matrix(" + a + ", " + b + ", " + c + ", " + d + ", " + e + ", " + f + ")";
        };
        AffineTransform.prototype.clone = function () {
            var _a = this, a = _a.a, b = _a.b, c = _a.c, d = _a.d, e = _a.e, f = _a.f;
            return new AffineTransform(a, b, c, d, e, f);
        };
        Object.defineProperty(AffineTransform.prototype, "is_identity", {
            get: function () {
                var _a = this, a = _a.a, b = _a.b, c = _a.c, d = _a.d, e = _a.e, f = _a.f;
                return a == 1 && b == 0 && c == 0 && d == 1 && e == 0 && f == 0;
            },
            enumerable: true,
            configurable: true
        });
        AffineTransform.prototype.apply = function (x, y) {
            var _a = this, a = _a.a, b = _a.b, c = _a.c, d = _a.d, e = _a.e, f = _a.f;
            return [
                a * x + c * y + e,
                b * x + d * y + f,
            ];
        };
        AffineTransform.prototype.iv_apply = function (xs, ys) {
            var _a = this, a = _a.a, b = _a.b, c = _a.c, d = _a.d, e = _a.e, f = _a.f;
            var n = xs.length;
            for (var i = 0; i < n; i++) {
                var x = xs[i];
                var y = ys[i];
                xs[i] = a * x + c * y + e;
                ys[i] = b * x + d * y + f;
            }
        };
        AffineTransform.prototype.transform = function (A, B, C, D, E, F) {
            var _a = this, a = _a.a, b = _a.b, c = _a.c, d = _a.d, e = _a.e, f = _a.f;
            this.a = a * A + c * B;
            this.c = a * C + c * D;
            this.e = a * E + c * F + e;
            this.b = b * A + d * B;
            this.d = b * C + d * D;
            this.f = b * E + d * F + f;
            return this;
        };
        AffineTransform.prototype.translate = function (tx, ty) {
            return this.transform(1, 0, 0, 1, tx, ty);
        };
        AffineTransform.prototype.scale = function (cx, cy) {
            return this.transform(cx, 0, 0, cy, 0, 0);
        };
        AffineTransform.prototype.skew = function (sx, sy) {
            return this.transform(1, sy, sx, 1, 0, 0);
        };
        AffineTransform.prototype.rotate = function (angle) {
            var s = sin(angle);
            var c = cos(angle);
            return this.transform(c, s, -s, c, 0, 0);
        };
        AffineTransform.prototype.rotate_ccw = function (angle) {
            return this.rotate(-angle);
        };
        AffineTransform.prototype.translate_x = function (tx) {
            return this.translate(tx, 0);
        };
        AffineTransform.prototype.translate_y = function (ty) {
            return this.translate(0, ty);
        };
        AffineTransform.prototype.flip = function () {
            return this.scale(-1, -1);
        };
        AffineTransform.prototype.flip_x = function () {
            return this.scale(1, -1);
        };
        AffineTransform.prototype.flip_y = function () {
            return this.scale(-1, 1);
        };
        return AffineTransform;
    }());
    exports.AffineTransform = AffineTransform;
    AffineTransform.__name__ = "AffineTransform";
},
/* models/canvas/canvas.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var has_props_1 = require(129) /* ../../core/has_props */;
    var dom_view_1 = require(193) /* ../../core/dom_view */;
    var logging_1 = require(134) /* ../../core/logging */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var dom_1 = require(187) /* ../../core/dom */;
    var object_1 = require(128) /* ../../core/util/object */;
    var bbox_1 = require(194) /* ../../core/util/bbox */;
    var canvas_1 = require(195) /* ../../core/util/canvas */;
    var svg_1 = require(190) /* ../../core/util/svg */;
    var global_webgl = (function () {
        // We use a global invisible canvas and gl context. By having a global context,
        // we avoid the limitation of max 16 contexts that most browsers have.
        var canvas = document.createElement("canvas");
        var gl = canvas.getContext("webgl", { premultipliedAlpha: true });
        // If WebGL is available, we store a reference to the gl canvas on
        // the ctx object, because that's what gets passed everywhere.
        if (gl != null)
            return { canvas: canvas, gl: gl };
        else {
            logging_1.logger.trace("WebGL is not supported");
            return undefined;
        }
    })();
    var style = {
        position: "absolute",
        top: "0",
        left: "0",
        width: "100%",
        height: "100%",
    };
    var CanvasLayer = /** @class */ (function () {
        function CanvasLayer(backend, hidpi) {
            this.backend = backend;
            this.hidpi = hidpi;
            this.pixel_ratio = 1;
            this.bbox = new bbox_1.BBox();
            switch (backend) {
                case "webgl":
                case "canvas": {
                    this._el = this._canvas = dom_1.canvas({ style: style });
                    var ctx = this.canvas.getContext('2d');
                    if (ctx == null)
                        throw new Error("unable to obtain 2D rendering context");
                    this._ctx = ctx;
                    if (hidpi) {
                        this.pixel_ratio = devicePixelRatio;
                    }
                    break;
                }
                case "svg": {
                    var ctx = new svg_1.SVGRenderingContext2D();
                    this._ctx = ctx;
                    this._canvas = ctx.get_svg();
                    this._el = dom_1.div({ style: style }, this._canvas);
                    break;
                }
            }
            canvas_1.fixup_ctx(this._ctx);
        }
        Object.defineProperty(CanvasLayer.prototype, "canvas", {
            get: function () {
                return this._canvas;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CanvasLayer.prototype, "ctx", {
            get: function () {
                return this._ctx;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CanvasLayer.prototype, "el", {
            get: function () {
                return this._el;
            },
            enumerable: true,
            configurable: true
        });
        CanvasLayer.prototype.resize = function (width, height) {
            this.bbox = new bbox_1.BBox({ left: 0, top: 0, width: width, height: height });
            var target = this._ctx instanceof svg_1.SVGRenderingContext2D ? this._ctx : this.canvas;
            target.width = width * this.pixel_ratio;
            target.height = height * this.pixel_ratio;
        };
        CanvasLayer.prototype.prepare = function () {
            var _a = this, ctx = _a.ctx, hidpi = _a.hidpi, pixel_ratio = _a.pixel_ratio;
            ctx.save();
            if (hidpi) {
                ctx.scale(pixel_ratio, pixel_ratio);
                ctx.translate(0.5, 0.5);
            }
            this.clear();
        };
        CanvasLayer.prototype.clear = function () {
            var _a = this.bbox, x = _a.x, y = _a.y, width = _a.width, height = _a.height;
            this.ctx.clearRect(x, y, width, height);
        };
        CanvasLayer.prototype.finish = function () {
            this.ctx.restore();
        };
        CanvasLayer.prototype.to_blob = function () {
            var _canvas = this._canvas;
            if (_canvas instanceof HTMLCanvasElement) {
                if (_canvas.msToBlob != null) {
                    return Promise.resolve(_canvas.msToBlob());
                }
                else {
                    return new Promise(function (resolve, reject) {
                        _canvas.toBlob(function (blob) { return blob != null ? resolve(blob) : reject(); }, "image/png");
                    });
                }
            }
            else {
                var ctx = this._ctx;
                var svg = ctx.get_serialized_svg(true);
                var blob = new Blob([svg], { type: "image/svg+xml" });
                return Promise.resolve(blob);
            }
        };
        return CanvasLayer;
    }());
    exports.CanvasLayer = CanvasLayer;
    CanvasLayer.__name__ = "CanvasLayer";
    var CanvasView = /** @class */ (function (_super) {
        tslib_1.__extends(CanvasView, _super);
        function CanvasView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.bbox = new bbox_1.BBox();
            return _this;
        }
        CanvasView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            var _a = this.model, output_backend = _a.output_backend, hidpi = _a.hidpi;
            if (output_backend == "webgl") {
                this.webgl = global_webgl;
            }
            this.underlays_el = dom_1.div({ style: style });
            this.primary = new CanvasLayer(output_backend, hidpi);
            this.overlays = new CanvasLayer(output_backend, hidpi);
            this.overlays_el = dom_1.div({ style: style });
            this.events_el = dom_1.div({ class: "bk-canvas-events", style: style });
            var elements = [
                this.underlays_el,
                this.primary.el,
                this.overlays.el,
                this.overlays_el,
                this.events_el,
            ];
            object_1.extend(this.el.style, style);
            dom_1.append.apply(void 0, tslib_1.__spread([this.el], elements));
            logging_1.logger.debug("CanvasView initialized");
        };
        CanvasView.prototype.add_underlay = function (el) {
            this.underlays_el.appendChild(el);
        };
        CanvasView.prototype.add_overlay = function (el) {
            this.overlays_el.appendChild(el);
        };
        CanvasView.prototype.add_event = function (el) {
            this.events_el.appendChild(el);
        };
        Object.defineProperty(CanvasView.prototype, "pixel_ratio", {
            get: function () {
                return this.primary.pixel_ratio; // XXX: primary
            },
            enumerable: true,
            configurable: true
        });
        CanvasView.prototype.resize = function (width, height) {
            this.bbox = new bbox_1.BBox({ left: 0, top: 0, width: width, height: height });
            this.primary.resize(width, height);
            this.overlays.resize(width, height);
        };
        CanvasView.prototype.prepare_webgl = function (frame_box) {
            // Prepare WebGL for a drawing pass
            var webgl = this.webgl;
            if (webgl != null) {
                // Sync canvas size
                var _a = this.bbox, width = _a.width, height = _a.height;
                webgl.canvas.width = this.pixel_ratio * width;
                webgl.canvas.height = this.pixel_ratio * height;
                var gl = webgl.gl;
                // Clipping
                gl.enable(gl.SCISSOR_TEST);
                var _b = tslib_1.__read(frame_box, 4), sx = _b[0], sy = _b[1], w = _b[2], h = _b[3];
                var _c = this.bbox, xview = _c.xview, yview = _c.yview;
                var vx = xview.compute(sx);
                var vy = yview.compute(sy + h);
                var ratio = this.pixel_ratio;
                gl.scissor(ratio * vx, ratio * vy, ratio * w, ratio * h); // lower left corner, width, height
                // Setup blending
                gl.enable(gl.BLEND);
                gl.blendFuncSeparate(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA, gl.ONE_MINUS_DST_ALPHA, gl.ONE); // premultipliedAlpha == true
            }
        };
        CanvasView.prototype.clear_webgl = function () {
            var webgl = this.webgl;
            if (webgl != null) {
                // Prepare GL for drawing
                var gl = webgl.gl, canvas_2 = webgl.canvas;
                gl.viewport(0, 0, canvas_2.width, canvas_2.height);
                gl.clearColor(0, 0, 0, 0);
                gl.clear(gl.COLOR_BUFFER_BIT || gl.DEPTH_BUFFER_BIT);
            }
        };
        CanvasView.prototype.blit_webgl = function (ctx) {
            // This should be called when the ctx has no state except the HIDPI transform
            var webgl = this.webgl;
            if (webgl != null) {
                // Blit gl canvas into the 2D canvas. To do 1-on-1 blitting, we need
                // to remove the hidpi transform, then blit, then restore.
                // ctx.globalCompositeOperation = "source-over"  -> OK; is the default
                logging_1.logger.debug('Blitting WebGL canvas');
                ctx.restore();
                ctx.drawImage(webgl.canvas, 0, 0);
                // Set back hidpi transform
                ctx.save();
                if (this.model.hidpi) {
                    var ratio = this.pixel_ratio;
                    ctx.scale(ratio, ratio);
                    ctx.translate(0.5, 0.5);
                }
            }
        };
        CanvasView.prototype.compose = function () {
            var _a = this.model, output_backend = _a.output_backend, hidpi = _a.hidpi;
            var _b = this.bbox, width = _b.width, height = _b.height;
            var composite = new CanvasLayer(output_backend, hidpi);
            composite.resize(width, height);
            composite.ctx.drawImage(this.primary.canvas, 0, 0);
            composite.ctx.drawImage(this.overlays.canvas, 0, 0);
            return composite;
        };
        CanvasView.prototype.to_blob = function () {
            return this.compose().to_blob();
        };
        return CanvasView;
    }(dom_view_1.DOMView));
    exports.CanvasView = CanvasView;
    CanvasView.__name__ = "CanvasView";
    var Canvas = /** @class */ (function (_super) {
        tslib_1.__extends(Canvas, _super);
        function Canvas(attrs) {
            return _super.call(this, attrs) || this;
        }
        Canvas.init_Canvas = function () {
            this.prototype.default_view = CanvasView;
            this.internal({
                hidpi: [p.Boolean, true],
                output_backend: [p.OutputBackend, "canvas"],
            });
        };
        return Canvas;
    }(has_props_1.HasProps));
    exports.Canvas = Canvas;
    Canvas.__name__ = "Canvas";
    Canvas.init_Canvas();
},
/* core/dom_view.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var view_1 = require(186) /* ./view */;
    var dom_1 = require(187) /* ./dom */;
    var DOMView = /** @class */ (function (_super) {
        tslib_1.__extends(DOMView, _super);
        function DOMView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        DOMView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.el = this._createElement();
        };
        DOMView.prototype.remove = function () {
            dom_1.remove(this.el);
            _super.prototype.remove.call(this);
        };
        DOMView.prototype.css_classes = function () {
            return [];
        };
        DOMView.prototype.render = function () { };
        DOMView.prototype.renderTo = function (element) {
            element.appendChild(this.el);
            this.render();
        };
        DOMView.prototype._createElement = function () {
            return dom_1.createElement(this.tagName, { class: this.css_classes() });
        };
        return DOMView;
    }(view_1.View));
    exports.DOMView = DOMView;
    DOMView.__name__ = "DOMView";
    DOMView.prototype.tagName = "div";
},
/* core/util/bbox.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var types_1 = require(139) /* ../types */;
    var min = Math.min, max = Math.max;
    function empty() {
        return {
            x0: Infinity,
            y0: Infinity,
            x1: -Infinity,
            y1: -Infinity,
        };
    }
    exports.empty = empty;
    function positive_x() {
        return {
            x0: Number.MIN_VALUE,
            y0: -Infinity,
            x1: Infinity,
            y1: Infinity,
        };
    }
    exports.positive_x = positive_x;
    function positive_y() {
        return {
            x0: -Infinity,
            y0: Number.MIN_VALUE,
            x1: Infinity,
            y1: Infinity,
        };
    }
    exports.positive_y = positive_y;
    function union(a, b) {
        return {
            x0: min(a.x0, b.x0),
            x1: max(a.x1, b.x1),
            y0: min(a.y0, b.y0),
            y1: max(a.y1, b.y1),
        };
    }
    exports.union = union;
    var BBox = /** @class */ (function () {
        function BBox(box) {
            if (box == null) {
                this.x0 = 0;
                this.y0 = 0;
                this.x1 = 0;
                this.y1 = 0;
            }
            else if ('x0' in box) {
                var x0 = box.x0, y0 = box.y0, x1 = box.x1, y1 = box.y1;
                if (!(x0 <= x1 && y0 <= y1))
                    throw new Error("invalid bbox {x0: " + x0 + ", y0: " + y0 + ", x1: " + x1 + ", y1: " + y1 + "}");
                this.x0 = x0;
                this.y0 = y0;
                this.x1 = x1;
                this.y1 = y1;
            }
            else if ("x" in box) {
                var x = box.x, y = box.y, width = box.width, height = box.height;
                if (!(width >= 0 && height >= 0))
                    throw new Error("invalid bbox {x: " + x + ", y: " + y + ", width: " + width + ", height: " + height + "}");
                this.x0 = x;
                this.y0 = y;
                this.x1 = x + width;
                this.y1 = y + height;
            }
            else {
                var left = void 0, right = void 0;
                var top = void 0, bottom = void 0;
                if ("width" in box) {
                    if ("left" in box) {
                        left = box.left;
                        right = left + box.width;
                    }
                    else if ("right" in box) {
                        right = box.right;
                        left = right - box.width;
                    }
                    else {
                        var w2 = box.width / 2;
                        left = box.hcenter - w2;
                        right = box.hcenter + w2;
                    }
                }
                else {
                    left = box.left;
                    right = box.right;
                }
                if ("height" in box) {
                    if ("top" in box) {
                        top = box.top;
                        bottom = top + box.height;
                    }
                    else if ("bottom" in box) {
                        bottom = box.bottom;
                        top = bottom - box.height;
                    }
                    else {
                        var h2 = box.height / 2;
                        top = box.vcenter - h2;
                        bottom = box.vcenter + h2;
                    }
                }
                else {
                    top = box.top;
                    bottom = box.bottom;
                }
                if (!(left <= right && top <= bottom))
                    throw new Error("invalid bbox {left: " + left + ", top: " + top + ", right: " + right + ", bottom: " + bottom + "}");
                this.x0 = left;
                this.y0 = top;
                this.x1 = right;
                this.y1 = bottom;
            }
        }
        BBox.prototype.toString = function () {
            return "BBox({left: " + this.left + ", top: " + this.top + ", width: " + this.width + ", height: " + this.height + "})";
        };
        Object.defineProperty(BBox.prototype, "left", {
            get: function () { return this.x0; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "top", {
            get: function () { return this.y0; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "right", {
            get: function () { return this.x1; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "bottom", {
            get: function () { return this.y1; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "p0", {
            get: function () { return [this.x0, this.y0]; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "p1", {
            get: function () { return [this.x1, this.y1]; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "x", {
            get: function () { return this.x0; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "y", {
            get: function () { return this.y0; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "width", {
            get: function () { return this.x1 - this.x0; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "height", {
            get: function () { return this.y1 - this.y0; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "rect", {
            get: function () { return { x0: this.x0, y0: this.y0, x1: this.x1, y1: this.y1 }; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "box", {
            get: function () { return { x: this.x, y: this.y, width: this.width, height: this.height }; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "h_range", {
            get: function () { return { start: this.x0, end: this.x1 }; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "v_range", {
            get: function () { return { start: this.y0, end: this.y1 }; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "ranges", {
            get: function () { return [this.h_range, this.v_range]; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "aspect", {
            get: function () { return this.width / this.height; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "hcenter", {
            get: function () { return (this.left + this.right) / 2; },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "vcenter", {
            get: function () { return (this.top + this.bottom) / 2; },
            enumerable: true,
            configurable: true
        });
        BBox.prototype.relativize = function () {
            var _a = this, width = _a.width, height = _a.height;
            return new BBox({ x: 0, y: 0, width: width, height: height });
        };
        BBox.prototype.contains = function (x, y) {
            return x >= this.x0 && x <= this.x1 && y >= this.y0 && y <= this.y1;
        };
        BBox.prototype.clip = function (x, y) {
            if (x < this.x0)
                x = this.x0;
            else if (x > this.x1)
                x = this.x1;
            if (y < this.y0)
                y = this.y0;
            else if (y > this.y1)
                y = this.y1;
            return [x, y];
        };
        BBox.prototype.union = function (that) {
            return new BBox({
                x0: min(this.x0, that.x0),
                y0: min(this.y0, that.y0),
                x1: max(this.x1, that.x1),
                y1: max(this.y1, that.y1),
            });
        };
        BBox.prototype.equals = function (that) {
            return this.x0 == that.x0 && this.y0 == that.y0 && this.x1 == that.x1 && this.y1 == that.y1;
        };
        Object.defineProperty(BBox.prototype, "xview", {
            get: function () {
                var _this = this;
                return {
                    compute: function (x) {
                        return _this.left + x;
                    },
                    v_compute: function (xx) {
                        var _xx = new types_1.NumberArray(xx.length);
                        var left = _this.left;
                        for (var i = 0; i < xx.length; i++) {
                            _xx[i] = left + xx[i];
                        }
                        return _xx;
                    },
                };
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BBox.prototype, "yview", {
            get: function () {
                var _this = this;
                return {
                    compute: function (y) {
                        return _this.bottom - y;
                    },
                    v_compute: function (yy) {
                        var _yy = new types_1.NumberArray(yy.length);
                        var bottom = _this.bottom;
                        for (var i = 0; i < yy.length; i++) {
                            _yy[i] = bottom - yy[i];
                        }
                        return _yy;
                    },
                };
            },
            enumerable: true,
            configurable: true
        });
        return BBox;
    }());
    exports.BBox = BBox;
    BBox.__name__ = "BBox";
},
/* core/util/canvas.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function fixup_line_dash(ctx) {
        if (typeof ctx.lineDash === "undefined") {
            Object.defineProperty(ctx, "lineDash", {
                get: function () { return ctx.getLineDash(); },
                set: function (segments) { return ctx.setLineDash(segments); },
            });
        }
    }
    function fixup_image_smoothing(ctx) {
        ctx.setImageSmoothingEnabled = function (value) {
            ctx.imageSmoothingEnabled = value;
            ctx.mozImageSmoothingEnabled = value;
            ctx.oImageSmoothingEnabled = value;
            ctx.webkitImageSmoothingEnabled = value;
            ctx.msImageSmoothingEnabled = value;
        };
        ctx.getImageSmoothingEnabled = function () {
            var val = ctx.imageSmoothingEnabled;
            return val != null ? val : true;
        };
    }
    function fixup_measure_text(ctx) {
        if (ctx.measureText && ctx.html5MeasureText == null) {
            ctx.html5MeasureText = ctx.measureText;
            ctx.measureText = function (text) {
                var textMetrics = ctx.html5MeasureText(text);
                // fake it til you make it
                textMetrics.ascent = ctx.html5MeasureText("m").width * 1.6;
                return textMetrics;
            };
        }
    }
    function fixup_ellipse(ctx) {
        // implementing the ctx.ellipse function with bezier curves
        // we don't implement the startAngle, endAngle and anticlockwise arguments.
        function ellipse_bezier(x, y, radiusX, radiusY, rotation, _startAngle, _endAngle, anticlockwise) {
            if (anticlockwise === void 0) {
                anticlockwise = false;
            }
            var c = 0.551784; // see http://www.tinaja.com/glib/ellipse4.pdf
            ctx.translate(x, y);
            ctx.rotate(rotation);
            var rx = radiusX;
            var ry = radiusY;
            if (anticlockwise) {
                rx = -radiusX;
                ry = -radiusY;
            }
            ctx.moveTo(-rx, 0); // start point of first curve
            ctx.bezierCurveTo(-rx, ry * c, -rx * c, ry, 0, ry);
            ctx.bezierCurveTo(rx * c, ry, rx, ry * c, rx, 0);
            ctx.bezierCurveTo(rx, -ry * c, rx * c, -ry, 0, -ry);
            ctx.bezierCurveTo(-rx * c, -ry, -rx, -ry * c, -rx, 0);
            ctx.rotate(-rotation);
            ctx.translate(-x, -y);
        }
        if (!ctx.ellipse)
            ctx.ellipse = ellipse_bezier;
    }
    function fixup_ctx(ctx) {
        fixup_line_dash(ctx);
        fixup_image_smoothing(ctx);
        fixup_measure_text(ctx);
        fixup_ellipse(ctx);
    }
    exports.fixup_ctx = fixup_ctx;
},
/* model.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var has_props_1 = require(129) /* ./core/has_props */;
    var p = tslib_1.__importStar(require(133) /* ./core/properties */);
    var types_1 = require(123) /* ./core/util/types */;
    var object_1 = require(128) /* ./core/util/object */;
    var logging_1 = require(134) /* ./core/logging */;
    var Model = /** @class */ (function (_super) {
        tslib_1.__extends(Model, _super);
        function Model(attrs) {
            return _super.call(this, attrs) || this;
        }
        Model.init_Model = function () {
            this.define({
                tags: [p.Array, []],
                name: [p.String],
                js_property_callbacks: [p.Any, {}],
                js_event_callbacks: [p.Any, {}],
                subscribed_events: [p.Array, []],
            });
        };
        Model.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._js_callbacks = new Map();
        };
        Model.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this._update_property_callbacks();
            this.connect(this.properties.js_property_callbacks.change, function () { return _this._update_property_callbacks(); });
            this.connect(this.properties.js_event_callbacks.change, function () { return _this._update_event_callbacks(); });
            this.connect(this.properties.subscribed_events.change, function () { return _this._update_event_callbacks(); });
        };
        /*protected*/ Model.prototype._process_event = function (event) {
            var e_1, _a;
            try {
                for (var _b = tslib_1.__values(this.js_event_callbacks[event.event_name] || []), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var callback = _c.value;
                    callback.execute(event);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            if (this.document != null && this.subscribed_events.some(function (m) { return m == event.event_name; }))
                this.document.event_manager.send_event(event);
        };
        Model.prototype.trigger_event = function (event) {
            if (this.document != null) {
                event.origin = this;
                this.document.event_manager.trigger(event);
            }
        };
        Model.prototype._update_event_callbacks = function () {
            if (this.document == null) {
                // File an issue: SidePanel in particular seems to have this issue
                logging_1.logger.warn('WARNING: Document not defined for updating event callbacks');
                return;
            }
            this.document.event_manager.subscribed_models.add(this);
        };
        Model.prototype._update_property_callbacks = function () {
            var e_2, _a, e_3, _b, e_4, _c, e_5, _d;
            var _this = this;
            var signal_for = function (event) {
                var _a = tslib_1.__read(event.split(":"), 2), evt = _a[0], _b = _a[1], attr = _b === void 0 ? null : _b;
                return attr != null ? _this.properties[attr][evt] : _this[evt];
            };
            try {
                for (var _e = tslib_1.__values(this._js_callbacks), _f = _e.next(); !_f.done; _f = _e.next()) {
                    var _g = tslib_1.__read(_f.value, 2), event = _g[0], callbacks = _g[1];
                    var signal = signal_for(event);
                    try {
                        for (var callbacks_1 = (e_3 = void 0, tslib_1.__values(callbacks)), callbacks_1_1 = callbacks_1.next(); !callbacks_1_1.done; callbacks_1_1 = callbacks_1.next()) {
                            var cb = callbacks_1_1.value;
                            this.disconnect(signal, cb);
                        }
                    }
                    catch (e_3_1) {
                        e_3 = { error: e_3_1 };
                    }
                    finally {
                        try {
                            if (callbacks_1_1 && !callbacks_1_1.done && (_b = callbacks_1.return))
                                _b.call(callbacks_1);
                        }
                        finally {
                            if (e_3)
                                throw e_3.error;
                        }
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_f && !_f.done && (_a = _e.return))
                        _a.call(_e);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            this._js_callbacks.clear();
            try {
                for (var _h = tslib_1.__values(object_1.entries(this.js_property_callbacks)), _j = _h.next(); !_j.done; _j = _h.next()) {
                    var _k = tslib_1.__read(_j.value, 2), event = _k[0], callbacks = _k[1];
                    var wrappers = callbacks.map(function (cb) { return function () { return cb.execute(_this); }; });
                    this._js_callbacks.set(event, wrappers);
                    var signal = signal_for(event);
                    try {
                        for (var wrappers_1 = (e_5 = void 0, tslib_1.__values(wrappers)), wrappers_1_1 = wrappers_1.next(); !wrappers_1_1.done; wrappers_1_1 = wrappers_1.next()) {
                            var cb = wrappers_1_1.value;
                            this.connect(signal, cb);
                        }
                    }
                    catch (e_5_1) {
                        e_5 = { error: e_5_1 };
                    }
                    finally {
                        try {
                            if (wrappers_1_1 && !wrappers_1_1.done && (_d = wrappers_1.return))
                                _d.call(wrappers_1);
                        }
                        finally {
                            if (e_5)
                                throw e_5.error;
                        }
                    }
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (_j && !_j.done && (_c = _h.return))
                        _c.call(_h);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
        };
        Model.prototype._doc_attached = function () {
            if (!object_1.isEmpty(this.js_event_callbacks) || this.subscribed_events.length != 0)
                this._update_event_callbacks();
        };
        Model.prototype._doc_detached = function () {
            this.document.event_manager.subscribed_models.delete(this);
        };
        Model.prototype.select = function (selector) {
            if (types_1.isString(selector))
                return tslib_1.__spread(this.references()).filter(function (ref) { return ref instanceof Model && ref.name === selector; });
            else if (selector.prototype instanceof has_props_1.HasProps)
                return tslib_1.__spread(this.references()).filter(function (ref) { return ref instanceof selector; });
            else
                throw new Error("invalid selector");
        };
        Model.prototype.select_one = function (selector) {
            var result = this.select(selector);
            switch (result.length) {
                case 0:
                    return null;
                case 1:
                    return result[0];
                default:
                    throw new Error("found more than one object matching given selector");
            }
        };
        return Model;
    }(has_props_1.HasProps));
    exports.Model = Model;
    Model.__name__ = "Model";
    Model.init_Model();
},
/* models/canvas/coordinates.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var CoordinateTransform = /** @class */ (function () {
        function CoordinateTransform(x_scale, y_scale) {
            this.x_scale = x_scale;
            this.y_scale = y_scale;
            this.x_range = this.x_scale.source_range;
            this.y_range = this.y_scale.source_range;
            this.ranges = [this.x_range, this.y_range];
            this.scales = [this.x_scale, this.y_scale];
        }
        CoordinateTransform.prototype.map_to_screen = function (xs, ys) {
            var sxs = this.x_scale.v_compute(xs);
            var sys = this.y_scale.v_compute(ys);
            return [sxs, sys];
        };
        CoordinateTransform.prototype.map_from_screen = function (sxs, sys) {
            var xs = this.x_scale.v_invert(sxs);
            var ys = this.y_scale.v_invert(sys);
            return [xs, ys];
        };
        return CoordinateTransform;
    }());
    exports.CoordinateTransform = CoordinateTransform;
    CoordinateTransform.__name__ = "CoordinateTransform";
},
/* models/annotations/arrow.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var arrow_head_1 = require(199) /* ./arrow_head */;
    var column_data_source_1 = require(200) /* ../sources/column_data_source */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var math_1 = require(125) /* ../../core/util/math */;
    var ArrowView = /** @class */ (function (_super) {
        tslib_1.__extends(ArrowView, _super);
        function ArrowView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ArrowView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            if (this.model.source == null)
                this.model.source = new column_data_source_1.ColumnDataSource();
            this.set_data(this.model.source);
        };
        ArrowView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.change, function () { return _this.set_data(_this.model.source); });
            this.connect(this.model.source.streaming, function () { return _this.set_data(_this.model.source); });
            this.connect(this.model.source.patching, function () { return _this.set_data(_this.model.source); });
            this.connect(this.model.source.change, function () { return _this.set_data(_this.model.source); });
        };
        ArrowView.prototype.set_data = function (source) {
            _super.prototype.set_data.call(this, source);
            this.visuals.warm_cache(source);
            this.plot_view.request_render();
        };
        ArrowView.prototype._map_data = function () {
            var frame = this.plot_view.frame;
            var sx_start, sy_start;
            if (this.model.start_units == 'data') {
                sx_start = this.coordinates.x_scale.v_compute(this._x_start);
                sy_start = this.coordinates.y_scale.v_compute(this._y_start);
            }
            else {
                sx_start = frame.xview.v_compute(this._x_start);
                sy_start = frame.yview.v_compute(this._y_start);
            }
            var sx_end, sy_end;
            if (this.model.end_units == 'data') {
                sx_end = this.coordinates.x_scale.v_compute(this._x_end);
                sy_end = this.coordinates.y_scale.v_compute(this._y_end);
            }
            else {
                sx_end = frame.xview.v_compute(this._x_end);
                sy_end = frame.yview.v_compute(this._y_end);
            }
            return [[sx_start, sy_start], [sx_end, sy_end]];
        };
        ArrowView.prototype._render = function () {
            var ctx = this.layer.ctx;
            ctx.save();
            // Order in this function is important. First we draw all the arrow heads.
            var _a = tslib_1.__read(this._map_data(), 2), start = _a[0], end = _a[1];
            if (this.model.end != null)
                this._arrow_head(ctx, "render", this.model.end, start, end);
            if (this.model.start != null)
                this._arrow_head(ctx, "render", this.model.start, end, start);
            // Next we call .clip on all the arrow heads, inside an initial canvas sized
            // rect, to create an "inverted" clip region for the arrow heads
            ctx.beginPath();
            var _b = this.plot_view.frame.bbox, x = _b.x, y = _b.y, width = _b.width, height = _b.height;
            ctx.rect(x, y, width, height);
            if (this.model.end != null)
                this._arrow_head(ctx, "clip", this.model.end, start, end);
            if (this.model.start != null)
                this._arrow_head(ctx, "clip", this.model.start, end, start);
            ctx.closePath();
            ctx.clip();
            // Finally we draw the arrow body, with the clipping regions set up. This prevents
            // "fat" arrows from overlapping the arrow head in a bad way.
            this._arrow_body(ctx, start, end);
            ctx.restore();
        };
        ArrowView.prototype._arrow_head = function (ctx, action, head, start, end) {
            for (var i = 0, _end = this._x_start.length; i < _end; i++) {
                // arrow head runs orthogonal to arrow body
                var angle = Math.PI / 2 + math_1.atan2([start[0][i], start[1][i]], [end[0][i], end[1][i]]);
                ctx.save();
                ctx.translate(end[0][i], end[1][i]);
                ctx.rotate(angle);
                if (action == "render")
                    head.render(ctx, i);
                else if (action == "clip")
                    head.clip(ctx, i);
                ctx.restore();
            }
        };
        ArrowView.prototype._arrow_body = function (ctx, start, end) {
            if (!this.visuals.line.doit)
                return;
            for (var i = 0, n = this._x_start.length; i < n; i++) {
                this.visuals.line.set_vectorize(ctx, i);
                ctx.beginPath();
                ctx.moveTo(start[0][i], start[1][i]);
                ctx.lineTo(end[0][i], end[1][i]);
                ctx.stroke();
            }
        };
        return ArrowView;
    }(annotation_1.AnnotationView));
    exports.ArrowView = ArrowView;
    ArrowView.__name__ = "ArrowView";
    var Arrow = /** @class */ (function (_super) {
        tslib_1.__extends(Arrow, _super);
        function Arrow(attrs) {
            return _super.call(this, attrs) || this;
        }
        Arrow.init_Arrow = function () {
            this.prototype.default_view = ArrowView;
            this.mixins(property_mixins_1.LineVector);
            this.define({
                x_start: [p.NumberSpec],
                y_start: [p.NumberSpec],
                start_units: [p.SpatialUnits, 'data'],
                start: [p.Instance, null],
                x_end: [p.NumberSpec],
                y_end: [p.NumberSpec],
                end_units: [p.SpatialUnits, 'data'],
                end: [p.Instance, function () { return new arrow_head_1.OpenHead({}); }],
                source: [p.Instance],
            });
        };
        return Arrow;
    }(annotation_1.Annotation));
    exports.Arrow = Arrow;
    Arrow.__name__ = "Arrow";
    Arrow.init_Arrow();
},
/* models/annotations/arrow_head.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var visuals_1 = require(189) /* ../../core/visuals */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var ArrowHead = /** @class */ (function (_super) {
        tslib_1.__extends(ArrowHead, _super);
        function ArrowHead(attrs) {
            return _super.call(this, attrs) || this;
        }
        ArrowHead.init_ArrowHead = function () {
            this.define({
                size: [p.Number, 25],
            });
        };
        ArrowHead.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.visuals = new visuals_1.Visuals(this);
        };
        return ArrowHead;
    }(annotation_1.Annotation));
    exports.ArrowHead = ArrowHead;
    ArrowHead.__name__ = "ArrowHead";
    ArrowHead.init_ArrowHead();
    var OpenHead = /** @class */ (function (_super) {
        tslib_1.__extends(OpenHead, _super);
        function OpenHead(attrs) {
            return _super.call(this, attrs) || this;
        }
        OpenHead.init_OpenHead = function () {
            this.mixins(property_mixins_1.LineVector);
        };
        OpenHead.prototype.clip = function (ctx, i) {
            // This method should not begin or close a path
            this.visuals.line.set_vectorize(ctx, i);
            ctx.moveTo(0.5 * this.size, this.size);
            ctx.lineTo(0.5 * this.size, -2);
            ctx.lineTo(-0.5 * this.size, -2);
            ctx.lineTo(-0.5 * this.size, this.size);
            ctx.lineTo(0, 0);
            ctx.lineTo(0.5 * this.size, this.size);
        };
        OpenHead.prototype.render = function (ctx, i) {
            if (this.visuals.line.doit) {
                this.visuals.line.set_vectorize(ctx, i);
                ctx.beginPath();
                ctx.moveTo(0.5 * this.size, this.size);
                ctx.lineTo(0, 0);
                ctx.lineTo(-0.5 * this.size, this.size);
                ctx.stroke();
            }
        };
        return OpenHead;
    }(ArrowHead));
    exports.OpenHead = OpenHead;
    OpenHead.__name__ = "OpenHead";
    OpenHead.init_OpenHead();
    var NormalHead = /** @class */ (function (_super) {
        tslib_1.__extends(NormalHead, _super);
        function NormalHead(attrs) {
            return _super.call(this, attrs) || this;
        }
        NormalHead.init_NormalHead = function () {
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector]);
            this.override({
                fill_color: 'black',
            });
        };
        NormalHead.prototype.clip = function (ctx, i) {
            // This method should not begin or close a path
            this.visuals.line.set_vectorize(ctx, i);
            ctx.moveTo(0.5 * this.size, this.size);
            ctx.lineTo(0.5 * this.size, -2);
            ctx.lineTo(-0.5 * this.size, -2);
            ctx.lineTo(-0.5 * this.size, this.size);
            ctx.lineTo(0.5 * this.size, this.size);
        };
        NormalHead.prototype.render = function (ctx, i) {
            if (this.visuals.fill.doit) {
                this.visuals.fill.set_vectorize(ctx, i);
                this._normal(ctx, i);
                ctx.fill();
            }
            if (this.visuals.line.doit) {
                this.visuals.line.set_vectorize(ctx, i);
                this._normal(ctx, i);
                ctx.stroke();
            }
        };
        NormalHead.prototype._normal = function (ctx, _i) {
            ctx.beginPath();
            ctx.moveTo(0.5 * this.size, this.size);
            ctx.lineTo(0, 0);
            ctx.lineTo(-0.5 * this.size, this.size);
            ctx.closePath();
        };
        return NormalHead;
    }(ArrowHead));
    exports.NormalHead = NormalHead;
    NormalHead.__name__ = "NormalHead";
    NormalHead.init_NormalHead();
    var VeeHead = /** @class */ (function (_super) {
        tslib_1.__extends(VeeHead, _super);
        function VeeHead(attrs) {
            return _super.call(this, attrs) || this;
        }
        VeeHead.init_VeeHead = function () {
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector]);
            this.override({
                fill_color: 'black',
            });
        };
        VeeHead.prototype.clip = function (ctx, i) {
            // This method should not begin or close a path
            this.visuals.line.set_vectorize(ctx, i);
            ctx.moveTo(0.5 * this.size, this.size);
            ctx.lineTo(0.5 * this.size, -2);
            ctx.lineTo(-0.5 * this.size, -2);
            ctx.lineTo(-0.5 * this.size, this.size);
            ctx.lineTo(0, 0.5 * this.size);
            ctx.lineTo(0.5 * this.size, this.size);
        };
        VeeHead.prototype.render = function (ctx, i) {
            if (this.visuals.fill.doit) {
                this.visuals.fill.set_vectorize(ctx, i);
                this._vee(ctx, i);
                ctx.fill();
            }
            if (this.visuals.line.doit) {
                this.visuals.line.set_vectorize(ctx, i);
                this._vee(ctx, i);
                ctx.stroke();
            }
        };
        VeeHead.prototype._vee = function (ctx, _i) {
            ctx.beginPath();
            ctx.moveTo(0.5 * this.size, this.size);
            ctx.lineTo(0, 0);
            ctx.lineTo(-0.5 * this.size, this.size);
            ctx.lineTo(0, 0.5 * this.size);
            ctx.closePath();
        };
        return VeeHead;
    }(ArrowHead));
    exports.VeeHead = VeeHead;
    VeeHead.__name__ = "VeeHead";
    VeeHead.init_VeeHead();
    var TeeHead = /** @class */ (function (_super) {
        tslib_1.__extends(TeeHead, _super);
        function TeeHead(attrs) {
            return _super.call(this, attrs) || this;
        }
        TeeHead.init_TeeHead = function () {
            this.mixins(property_mixins_1.LineVector);
        };
        TeeHead.prototype.render = function (ctx, i) {
            if (this.visuals.line.doit) {
                this.visuals.line.set_vectorize(ctx, i);
                ctx.beginPath();
                ctx.moveTo(0.5 * this.size, 0);
                ctx.lineTo(-0.5 * this.size, 0);
                ctx.stroke();
            }
        };
        TeeHead.prototype.clip = function (_ctx, _i) { };
        return TeeHead;
    }(ArrowHead));
    exports.TeeHead = TeeHead;
    TeeHead.__name__ = "TeeHead";
    TeeHead.init_TeeHead();
},
/* models/sources/column_data_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var columnar_data_source_1 = require(201) /* ./columnar_data_source */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(123) /* ../../core/util/types */;
    var object_1 = require(128) /* ../../core/util/object */;
    var typed_array = tslib_1.__importStar(require(234) /* ../../core/util/typed_array */);
    var set_1 = require(235) /* ../../core/util/set */;
    var events_1 = require(236) /* ../../document/events */;
    //exported for testing
    function stream_to_column(col, new_col, rollover) {
        if (types_1.isArray(col)) {
            var result = col.concat(new_col);
            if (rollover != null && result.length > rollover)
                return result.slice(-rollover);
            else
                return result;
        }
        else if (types_1.isTypedArray(col)) {
            var total_len = col.length + new_col.length;
            // handle rollover case for typed arrays
            if (rollover != null && total_len > rollover) {
                var start = total_len - rollover;
                var end = col.length;
                // resize col if it is shorter than the rollover length
                var result = void 0;
                if (col.length < rollover) {
                    result = new col.constructor(rollover);
                    result.set(col, 0);
                }
                else
                    result = col;
                // shift values in original col to accommodate new_col
                for (var i = start, endi = end; i < endi; i++) {
                    result[i - start] = result[i];
                }
                // update end values in col with new_col
                for (var i = 0, endi = new_col.length; i < endi; i++) {
                    result[i + (end - start)] = new_col[i];
                }
                return result;
            }
            else {
                var tmp = new col.constructor(new_col);
                return typed_array.concat(col, tmp);
            }
        }
        else
            throw new Error("unsupported array types");
    }
    exports.stream_to_column = stream_to_column;
    // exported for testing
    function slice(ind, length) {
        var start, step, stop;
        if (types_1.isNumber(ind)) {
            start = ind;
            stop = ind + 1;
            step = 1;
        }
        else {
            start = ind.start != null ? ind.start : 0;
            stop = ind.stop != null ? ind.stop : length;
            step = ind.step != null ? ind.step : 1;
        }
        return [start, stop, step];
    }
    exports.slice = slice;
    // exported for testing
    function patch_to_column(col, patch) {
        var e_1, _a;
        var patched = new Set();
        var patched_range = false;
        try {
            for (var patch_1 = tslib_1.__values(patch), patch_1_1 = patch_1.next(); !patch_1_1.done; patch_1_1 = patch_1.next()) {
                var _b = tslib_1.__read(patch_1_1.value, 2), ind = _b[0], val = _b[1];
                // make the single index case look like the length-3 multi-index case
                var shape = void 0;
                var item = void 0;
                var index = void 0;
                var value = void 0;
                if (types_1.isArray(ind)) {
                    var _c = tslib_1.__read(ind, 1), i = _c[0];
                    patched.add(i);
                    shape = col[i].shape;
                    item = col[i];
                    value = val;
                    // this is basically like NumPy's "newaxis", inserting an empty dimension
                    // makes length 2 and 3 multi-index cases uniform, so that the same code
                    // can handle both
                    if (ind.length === 2) {
                        shape = [1, shape[0]];
                        index = [ind[0], 0, ind[1]];
                    }
                    else
                        index = ind;
                }
                else {
                    if (types_1.isNumber(ind)) {
                        value = [val];
                        patched.add(ind);
                    }
                    else {
                        value = val;
                        patched_range = true;
                    }
                    index = [0, 0, ind];
                    shape = [1, col.length];
                    item = col;
                }
                // now this one nested loop handles all cases
                var flat_index = 0;
                var _d = tslib_1.__read(slice(index[1], shape[0]), 3), istart = _d[0], istop = _d[1], istep = _d[2];
                var _e = tslib_1.__read(slice(index[2], shape[1]), 3), jstart = _e[0], jstop = _e[1], jstep = _e[2];
                for (var i = istart; i < istop; i += istep) {
                    for (var j = jstart; j < jstop; j += jstep) {
                        if (patched_range) {
                            patched.add(j);
                        }
                        item[i * shape[1] + j] = value[flat_index];
                        flat_index++;
                    }
                }
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (patch_1_1 && !patch_1_1.done && (_a = patch_1.return))
                    _a.call(patch_1);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        return patched;
    }
    exports.patch_to_column = patch_to_column;
    var ColumnDataSource = /** @class */ (function (_super) {
        tslib_1.__extends(ColumnDataSource, _super);
        function ColumnDataSource(attrs) {
            return _super.call(this, attrs) || this;
        }
        ColumnDataSource.init_ColumnDataSource = function () {
            this.define({
                data: [p.Any, {}],
            });
        };
        ColumnDataSource.prototype.stream = function (new_data, rollover, setter_id) {
            var e_2, _a;
            var data = this.data;
            try {
                for (var _b = tslib_1.__values(object_1.entries(new_data)), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var _d = tslib_1.__read(_c.value, 2), name = _d[0], new_column = _d[1];
                    data[name] = stream_to_column(data[name], new_column, rollover);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            this.setv({ data: data }, { silent: true });
            this.streaming.emit();
            if (this.document != null) {
                var hint = new events_1.ColumnsStreamedEvent(this.document, this.ref(), new_data, rollover);
                this.document._notify_change(this, 'data', null, null, { setter_id: setter_id, hint: hint });
            }
        };
        ColumnDataSource.prototype.patch = function (patches, setter_id) {
            var e_3, _a;
            var data = this.data;
            var patched = new Set();
            try {
                for (var _b = tslib_1.__values(object_1.entries(patches)), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var _d = tslib_1.__read(_c.value, 2), column = _d[0], patch = _d[1];
                    patched = set_1.union(patched, patch_to_column(data[column], patch)); // XXX
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            this.setv({ data: data }, { silent: true });
            this.patching.emit(tslib_1.__spread(patched));
            if (this.document != null) {
                var hint = new events_1.ColumnsPatchedEvent(this.document, this.ref(), patches);
                this.document._notify_change(this, 'data', null, null, { setter_id: setter_id, hint: hint });
            }
        };
        return ColumnDataSource;
    }(columnar_data_source_1.ColumnarDataSource));
    exports.ColumnDataSource = ColumnDataSource;
    ColumnDataSource.__name__ = "ColumnDataSource";
    ColumnDataSource.init_ColumnDataSource();
},
/* models/sources/columnar_data_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var data_source_1 = require(202) /* ./data_source */;
    var signaling_1 = require(130) /* ../../core/signaling */;
    var logging_1 = require(134) /* ../../core/logging */;
    var selection_manager_1 = require(204) /* ../../core/selection_manager */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(123) /* ../../core/util/types */;
    var array_1 = require(124) /* ../../core/util/array */;
    var object_1 = require(128) /* ../../core/util/object */;
    var selection_1 = require(203) /* ../selections/selection */;
    var interaction_policy_1 = require(233) /* ../selections/interaction_policy */;
    var ColumnarDataSource = /** @class */ (function (_super) {
        tslib_1.__extends(ColumnarDataSource, _super);
        function ColumnarDataSource(attrs) {
            return _super.call(this, attrs) || this;
        }
        ColumnarDataSource.prototype.get_array = function (key) {
            var column = this.data[key];
            if (column == null)
                this.data[key] = column = [];
            else if (!types_1.isArray(column))
                this.data[key] = column = Array.from(column);
            return column;
        };
        ColumnarDataSource.init_ColumnarDataSource = function () {
            this.define({
                selection_policy: [p.Instance, function () { return new interaction_policy_1.UnionRenderers(); }],
            });
            this.internal({
                selection_manager: [p.Instance, function (self) { return new selection_manager_1.SelectionManager({ source: self }); }],
                inspected: [p.Instance, function () { return new selection_1.Selection(); }],
            });
        };
        ColumnarDataSource.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._select = new signaling_1.Signal0(this, "select");
            this.inspect = new signaling_1.Signal(this, "inspect"); // XXX: <[indices, tool, renderer-view, source, data], this>
            this.streaming = new signaling_1.Signal0(this, "streaming");
            this.patching = new signaling_1.Signal(this, "patching");
        };
        ColumnarDataSource.prototype.get_column = function (colname) {
            var column = this.data[colname];
            return column != null ? column : null;
        };
        ColumnarDataSource.prototype.columns = function () {
            // return the column names in this data source
            return object_1.keys(this.data);
        };
        ColumnarDataSource.prototype.get_length = function (soft) {
            if (soft === void 0) {
                soft = true;
            }
            var lengths = array_1.uniq(object_1.values(this.data).map(function (v) { return v.length; }));
            switch (lengths.length) {
                case 0: {
                    return null; // XXX: don't guess, treat on case-by-case basis
                }
                case 1: {
                    return lengths[0];
                }
                default: {
                    var msg = "data source has columns of inconsistent lengths";
                    if (soft) {
                        logging_1.logger.warn(msg);
                        return lengths.sort()[0];
                    }
                    else
                        throw new Error(msg);
                }
            }
        };
        Object.defineProperty(ColumnarDataSource.prototype, "length", {
            get: function () {
                var _a;
                return (_a = this.get_length()) !== null && _a !== void 0 ? _a : 0;
            },
            enumerable: true,
            configurable: true
        });
        ColumnarDataSource.prototype.clear = function () {
            var e_1, _b;
            var empty = {};
            try {
                for (var _c = tslib_1.__values(this.columns()), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var col = _d.value;
                    empty[col] = new this.data[col].constructor(0);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            this.data = empty;
        };
        return ColumnarDataSource;
    }(data_source_1.DataSource));
    exports.ColumnarDataSource = ColumnarDataSource;
    ColumnarDataSource.__name__ = "ColumnarDataSource";
    ColumnarDataSource.init_ColumnarDataSource();
},
/* models/sources/data_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var selection_1 = require(203) /* ../selections/selection */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var DataSource = /** @class */ (function (_super) {
        tslib_1.__extends(DataSource, _super);
        function DataSource(attrs) {
            return _super.call(this, attrs) || this;
        }
        DataSource.init_DataSource = function () {
            this.define({
                selected: [p.Instance, function () { return new selection_1.Selection(); }],
            });
        };
        return DataSource;
    }(model_1.Model));
    exports.DataSource = DataSource;
    DataSource.__name__ = "DataSource";
    DataSource.init_DataSource();
},
/* models/selections/selection.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    var object_1 = require(128) /* ../../core/util/object */;
    var Selection = /** @class */ (function (_super) {
        tslib_1.__extends(Selection, _super);
        function Selection(attrs) {
            return _super.call(this, attrs) || this;
        }
        Selection.prototype.get_view = function () {
            return this.view;
        };
        Selection.init_Selection = function () {
            this.define({
                indices: [p.Array, []],
                line_indices: [p.Array, []],
                multiline_indices: [p.Any, {}],
            });
            this.internal({
                selected_glyphs: [p.Array, []],
                view: [p.Any],
                image_indices: [p.Array, []],
            });
        };
        Selection.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
        };
        Object.defineProperty(Selection.prototype, "selected_glyph", {
            get: function () {
                return this.selected_glyphs.length > 0 ? this.selected_glyphs[0] : null;
            },
            enumerable: true,
            configurable: true
        });
        Selection.prototype.add_to_selected_glyphs = function (glyph) {
            this.selected_glyphs.push(glyph);
        };
        Selection.prototype.update = function (selection, _final, mode) {
            if (_final === void 0) {
                _final = true;
            }
            if (mode === void 0) {
                mode = "replace";
            }
            switch (mode) {
                case "replace": {
                    this.indices = selection.indices;
                    this.line_indices = selection.line_indices;
                    this.selected_glyphs = selection.selected_glyphs;
                    this.view = selection.view;
                    this.multiline_indices = selection.multiline_indices;
                    this.image_indices = selection.image_indices;
                    break;
                }
                case "append": {
                    this.update_through_union(selection);
                    break;
                }
                case "intersect": {
                    this.update_through_intersection(selection);
                    break;
                }
                case "subtract": {
                    this.update_through_subtraction(selection);
                    break;
                }
            }
        };
        Selection.prototype.clear = function () {
            this.indices = [];
            this.line_indices = [];
            this.multiline_indices = {};
            this.view = null;
            this.selected_glyphs = [];
        };
        Selection.prototype.is_empty = function () {
            return this.indices.length == 0 && this.line_indices.length == 0 && this.image_indices.length == 0;
        };
        Selection.prototype.update_through_union = function (other) {
            this.indices = array_1.union(this.indices, other.indices);
            this.selected_glyphs = array_1.union(other.selected_glyphs, this.selected_glyphs);
            this.line_indices = array_1.union(other.line_indices, this.line_indices);
            this.view = other.view;
            this.multiline_indices = object_1.merge(other.multiline_indices, this.multiline_indices);
        };
        Selection.prototype.update_through_intersection = function (other) {
            this.indices = array_1.intersection(this.indices, other.indices);
            // TODO: think through and fix any logic below
            this.selected_glyphs = array_1.union(other.selected_glyphs, this.selected_glyphs);
            this.line_indices = array_1.union(other.line_indices, this.line_indices);
            this.view = other.view;
            this.multiline_indices = object_1.merge(other.multiline_indices, this.multiline_indices);
        };
        Selection.prototype.update_through_subtraction = function (other) {
            this.indices = array_1.difference(this.indices, other.indices);
            // TODO: think through and fix any logic below
            this.selected_glyphs = array_1.union(other.selected_glyphs, this.selected_glyphs);
            this.line_indices = array_1.union(other.line_indices, this.line_indices);
            this.view = other.view;
            this.multiline_indices = object_1.merge(other.multiline_indices, this.multiline_indices);
        };
        return Selection;
    }(model_1.Model));
    exports.Selection = Selection;
    Selection.__name__ = "Selection";
    Selection.init_Selection();
},
/* core/selection_manager.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var has_props_1 = require(129) /* ./has_props */;
    var selection_1 = require(203) /* ../models/selections/selection */;
    var glyph_renderer_1 = require(205) /* ../models/renderers/glyph_renderer */;
    var graph_renderer_1 = require(231) /* ../models/renderers/graph_renderer */;
    var p = tslib_1.__importStar(require(133) /* ./properties */);
    var SelectionManager = /** @class */ (function (_super) {
        tslib_1.__extends(SelectionManager, _super);
        function SelectionManager(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.inspectors = new Map();
            return _this;
        }
        SelectionManager.init_SelectionManager = function () {
            this.internal({
                source: [p.Any],
            });
        };
        SelectionManager.prototype.select = function (renderer_views, geometry, final, mode) {
            var e_1, _a, e_2, _b;
            if (mode === void 0) {
                mode = "replace";
            }
            // divide renderers into glyph_renderers or graph_renderers
            var glyph_renderer_views = [];
            var graph_renderer_views = [];
            try {
                for (var renderer_views_1 = tslib_1.__values(renderer_views), renderer_views_1_1 = renderer_views_1.next(); !renderer_views_1_1.done; renderer_views_1_1 = renderer_views_1.next()) {
                    var r = renderer_views_1_1.value;
                    if (r instanceof glyph_renderer_1.GlyphRendererView)
                        glyph_renderer_views.push(r);
                    else if (r instanceof graph_renderer_1.GraphRendererView)
                        graph_renderer_views.push(r);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (renderer_views_1_1 && !renderer_views_1_1.done && (_a = renderer_views_1.return))
                        _a.call(renderer_views_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            var did_hit = false;
            try {
                // graph renderer case
                for (var graph_renderer_views_1 = tslib_1.__values(graph_renderer_views), graph_renderer_views_1_1 = graph_renderer_views_1.next(); !graph_renderer_views_1_1.done; graph_renderer_views_1_1 = graph_renderer_views_1.next()) {
                    var r = graph_renderer_views_1_1.value;
                    var hit_test_result = r.model.selection_policy.hit_test(geometry, r);
                    did_hit = did_hit || r.model.selection_policy.do_selection(hit_test_result, r.model, final, mode);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (graph_renderer_views_1_1 && !graph_renderer_views_1_1.done && (_b = graph_renderer_views_1.return))
                        _b.call(graph_renderer_views_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            // glyph renderers
            if (glyph_renderer_views.length > 0) {
                var hit_test_result = this.source.selection_policy.hit_test(geometry, glyph_renderer_views);
                did_hit = did_hit || this.source.selection_policy.do_selection(hit_test_result, this.source, final, mode);
            }
            return did_hit;
        };
        SelectionManager.prototype.inspect = function (renderer_view, geometry) {
            var did_hit = false;
            if (renderer_view instanceof glyph_renderer_1.GlyphRendererView) {
                var hit_test_result = renderer_view.hit_test(geometry);
                if (hit_test_result != null) {
                    did_hit = !hit_test_result.is_empty();
                    var inspection = this.get_or_create_inspector(renderer_view.model);
                    inspection.update(hit_test_result, true, "replace");
                    this.source.setv({ inspected: inspection }, { silent: true });
                    this.source.inspect.emit([renderer_view, { geometry: geometry }]);
                }
            }
            else if (renderer_view instanceof graph_renderer_1.GraphRendererView) {
                var hit_test_result = renderer_view.model.inspection_policy.hit_test(geometry, renderer_view);
                did_hit = did_hit || renderer_view.model.inspection_policy.do_inspection(hit_test_result, geometry, renderer_view, false, "replace");
            }
            return did_hit;
        };
        SelectionManager.prototype.clear = function (rview) {
            this.source.selected.clear();
            if (rview != null)
                this.get_or_create_inspector(rview.model).clear();
        };
        SelectionManager.prototype.get_or_create_inspector = function (renderer) {
            var selection = this.inspectors.get(renderer);
            if (selection == null) {
                selection = new selection_1.Selection();
                this.inspectors.set(renderer, selection);
            }
            return selection;
        };
        return SelectionManager;
    }(has_props_1.HasProps));
    exports.SelectionManager = SelectionManager;
    SelectionManager.__name__ = "SelectionManager";
    SelectionManager.init_SelectionManager();
},
/* models/renderers/glyph_renderer.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var data_renderer_1 = require(206) /* ./data_renderer */;
    var line_1 = require(207) /* ../glyphs/line */;
    var patch_1 = require(225) /* ../glyphs/patch */;
    var harea_1 = require(226) /* ../glyphs/harea */;
    var varea_1 = require(228) /* ../glyphs/varea */;
    var cds_view_1 = require(229) /* ../sources/cds_view */;
    var types_1 = require(139) /* ../../core/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var array_1 = require(124) /* ../../core/util/array */;
    var object_1 = require(128) /* ../../core/util/object */;
    var build_views_1 = require(230) /* ../../core/build_views */;
    var factor_range_1 = require(213) /* ../ranges/factor_range */;
    var selection_defaults = {
        fill: {},
        line: {},
    };
    var decimated_defaults = {
        fill: { fill_alpha: 0.3, fill_color: "grey" },
        line: { line_alpha: 0.3, line_color: "grey" },
    };
    var nonselection_defaults = {
        fill: { fill_alpha: 0.2 },
        line: {},
    };
    var GlyphRendererView = /** @class */ (function (_super) {
        tslib_1.__extends(GlyphRendererView, _super);
        function GlyphRendererView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        GlyphRendererView.prototype.lazy_initialize = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                function mk_glyph(defaults) {
                    var attrs = object_1.clone(glyph_attrs);
                    if (has_fill)
                        object_1.extend(attrs, defaults.fill);
                    if (has_line)
                        object_1.extend(attrs, defaults.line);
                    return new base_glyph.constructor(attrs);
                }
                var base_glyph, has_fill, has_line, glyph_attrs, _e, selection_glyph, _f, nonselection_glyph, _g, hover_glyph, _h, muted_glyph, _j, decimated_glyph, _k;
                return tslib_1.__generator(this, function (_l) {
                    switch (_l.label) {
                        case 0: return [4 /*yield*/, _super.prototype.lazy_initialize.call(this)];
                        case 1:
                            _l.sent();
                            base_glyph = this.model.glyph;
                            has_fill = array_1.includes(base_glyph._mixins, "fill");
                            has_line = array_1.includes(base_glyph._mixins, "line");
                            glyph_attrs = object_1.clone(base_glyph.attributes);
                            delete glyph_attrs.id;
                            _e = this;
                            return [4 /*yield*/, this.build_glyph_view(base_glyph)];
                        case 2:
                            _e.glyph = _l.sent();
                            selection_glyph = this.model.selection_glyph;
                            if (selection_glyph == null)
                                selection_glyph = mk_glyph({ fill: {}, line: {} });
                            else if (selection_glyph === "auto")
                                selection_glyph = mk_glyph(selection_defaults);
                            _f = this;
                            return [4 /*yield*/, this.build_glyph_view(selection_glyph)];
                        case 3:
                            _f.selection_glyph = _l.sent();
                            nonselection_glyph = this.model.nonselection_glyph;
                            if ((nonselection_glyph == null))
                                nonselection_glyph = mk_glyph({ fill: {}, line: {} });
                            else if (nonselection_glyph === "auto")
                                nonselection_glyph = mk_glyph(nonselection_defaults);
                            _g = this;
                            return [4 /*yield*/, this.build_glyph_view(nonselection_glyph)];
                        case 4:
                            _g.nonselection_glyph = _l.sent();
                            hover_glyph = this.model.hover_glyph;
                            if (!(hover_glyph != null))
                                return [3 /*break*/, 6];
                            _h = this;
                            return [4 /*yield*/, this.build_glyph_view(hover_glyph)];
                        case 5:
                            _h.hover_glyph = _l.sent();
                            _l.label = 6;
                        case 6:
                            muted_glyph = this.model.muted_glyph;
                            if (!(muted_glyph != null))
                                return [3 /*break*/, 8];
                            _j = this;
                            return [4 /*yield*/, this.build_glyph_view(muted_glyph)];
                        case 7:
                            _j.muted_glyph = _l.sent();
                            _l.label = 8;
                        case 8:
                            decimated_glyph = mk_glyph(decimated_defaults);
                            _k = this;
                            return [4 /*yield*/, this.build_glyph_view(decimated_glyph)];
                        case 9:
                            _k.decimated_glyph = _l.sent();
                            this.set_data(false);
                            return [2 /*return*/];
                    }
                });
            });
        };
        GlyphRendererView.prototype.build_glyph_view = function (glyph) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_e) {
                    return [2 /*return*/, build_views_1.build_view(glyph, { parent: this })];
                });
            });
        };
        GlyphRendererView.prototype.remove = function () {
            var _a, _b;
            this.glyph.remove();
            this.selection_glyph.remove();
            this.nonselection_glyph.remove();
            (_a = this.hover_glyph) === null || _a === void 0 ? void 0 : _a.remove();
            (_b = this.muted_glyph) === null || _b === void 0 ? void 0 : _b.remove();
            this.decimated_glyph.remove();
            _super.prototype.remove.call(this);
        };
        GlyphRendererView.prototype.connect_signals = function () {
            var e_1, _e, e_2, _f;
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.change, function () { return _this.request_render(); });
            this.connect(this.model.glyph.change, function () { return _this.set_data(); });
            this.connect(this.model.data_source.change, function () { return _this.set_data(); });
            this.connect(this.model.data_source.streaming, function () { return _this.set_data(); });
            this.connect(this.model.data_source.patching, function (indices /* XXX: WHY? */) { return _this.set_data(true, indices); });
            this.connect(this.model.data_source.selected.change, function () { return _this.request_render(); });
            this.connect(this.model.data_source._select, function () { return _this.request_render(); });
            if (this.hover_glyph != null)
                this.connect(this.model.data_source.inspect, function () { return _this.request_render(); });
            this.connect(this.model.properties.view.change, function () { return _this.set_data(); });
            this.connect(this.model.view.change, function () { return _this.set_data(); });
            this.connect(this.model.properties.visible.change, function () { return _this.plot_view.update_dataranges(); });
            var _g = this.plot_view.frame, x_ranges = _g.x_ranges, y_ranges = _g.y_ranges;
            try {
                for (var x_ranges_1 = tslib_1.__values(x_ranges), x_ranges_1_1 = x_ranges_1.next(); !x_ranges_1_1.done; x_ranges_1_1 = x_ranges_1.next()) {
                    var _h = tslib_1.__read(x_ranges_1_1.value, 2), range = _h[1];
                    if (range instanceof factor_range_1.FactorRange)
                        this.connect(range.change, function () { return _this.set_data(); });
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (x_ranges_1_1 && !x_ranges_1_1.done && (_e = x_ranges_1.return))
                        _e.call(x_ranges_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            try {
                for (var y_ranges_1 = tslib_1.__values(y_ranges), y_ranges_1_1 = y_ranges_1.next(); !y_ranges_1_1.done; y_ranges_1_1 = y_ranges_1.next()) {
                    var _j = tslib_1.__read(y_ranges_1_1.value, 2), range = _j[1];
                    if (range instanceof factor_range_1.FactorRange)
                        this.connect(range.change, function () { return _this.set_data(); });
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (y_ranges_1_1 && !y_ranges_1_1.done && (_f = y_ranges_1.return))
                        _f.call(y_ranges_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            this.connect(this.model.glyph.transformchange, function () { return _this.set_data(); });
        };
        GlyphRendererView.prototype._update_masked_indices = function () {
            var masked = this.glyph.mask_data();
            this.model.view.masked = masked;
            return masked;
        };
        // in case of partial updates like patching, the list of indices that actually
        // changed may be passed as the "indices" parameter to afford any optional optimizations
        GlyphRendererView.prototype.set_data = function (request_render, indices) {
            if (request_render === void 0) {
                request_render = true;
            }
            if (indices === void 0) {
                indices = null;
            }
            var _a, _b, _c, _d;
            var source = this.model.data_source;
            this.all_indices = this.model.view.indices;
            var all_indices = this.all_indices;
            this.glyph.set_data(source, all_indices, indices);
            this.glyph.set_visuals(source, all_indices);
            this.decimated_glyph.set_visuals(source, all_indices);
            (_a = this.selection_glyph) === null || _a === void 0 ? void 0 : _a.set_visuals(source, all_indices);
            (_b = this.nonselection_glyph) === null || _b === void 0 ? void 0 : _b.set_visuals(source, all_indices);
            (_c = this.hover_glyph) === null || _c === void 0 ? void 0 : _c.set_visuals(source, all_indices);
            (_d = this.muted_glyph) === null || _d === void 0 ? void 0 : _d.set_visuals(source, all_indices);
            this._update_masked_indices();
            var lod_factor = this.plot_model.lod_factor;
            var n = this.all_indices.count;
            this.decimated = new types_1.Indices(n);
            for (var i = 0; i < n; i += lod_factor) {
                this.decimated.set(i);
            }
            this.set_data_timestamp = Date.now();
            if (request_render) {
                this.request_render();
            }
        };
        Object.defineProperty(GlyphRendererView.prototype, "has_webgl", {
            get: function () {
                return this.glyph.has_webgl;
            },
            enumerable: true,
            configurable: true
        });
        GlyphRendererView.prototype._render = function () {
            var e_3, _e, e_4, _f, e_5, _g, e_6, _h;
            var _this = this;
            var glsupport = this.has_webgl;
            this.glyph.map_data();
            // all_indices is in full data space, indices is converted to subset space by mask_data (that may use the spatial index)
            var all_indices = tslib_1.__spread(this.all_indices);
            var indices = tslib_1.__spread(this._update_masked_indices());
            var ctx = this.layer.ctx;
            ctx.save();
            // selected is in full set space
            var selected = this.model.data_source.selected;
            var selected_full_indices;
            if (!selected || selected.is_empty())
                selected_full_indices = [];
            else {
                if (this.glyph instanceof line_1.LineView && selected.selected_glyph === this.glyph.model)
                    selected_full_indices = this.model.view.convert_indices_from_subset(indices);
                else
                    selected_full_indices = selected.indices;
            }
            // inspected is in full set space
            var inspected = this.model.data_source.inspected;
            var inspected_full_indices = new Set((function () {
                if (!inspected || inspected.is_empty())
                    return [];
                else {
                    if (inspected.selected_glyph)
                        return _this.model.view.convert_indices_from_subset(indices);
                    else if (inspected.indices.length > 0)
                        return inspected.indices;
                    else {
                        // TODO: return inspected.multiline_indices.keys()
                        return Object.keys(inspected.multiline_indices).map(function (i) { return parseInt(i); });
                    }
                }
            })());
            // inspected is transformed to subset space
            var inspected_subset_indices = arrayable_1.filter(indices, function (i) { return inspected_full_indices.has(all_indices[i]); });
            var lod_threshold = this.plot_model.lod_threshold;
            var glyph;
            var nonselection_glyph;
            var selection_glyph;
            if ((this.model.document != null ? this.model.document.interactive_duration() > 0 : false)
                && !glsupport && lod_threshold != null && all_indices.length > lod_threshold) {
                // Render decimated during interaction if too many elements and not using GL
                indices = tslib_1.__spread(this.decimated);
                glyph = this.decimated_glyph;
                nonselection_glyph = this.decimated_glyph;
                selection_glyph = this.selection_glyph;
            }
            else {
                glyph = this.model.muted && this.muted_glyph != null ? this.muted_glyph : this.glyph;
                nonselection_glyph = this.nonselection_glyph;
                selection_glyph = this.selection_glyph;
            }
            if (this.hover_glyph != null && inspected_subset_indices.length)
                indices = array_1.difference(indices, inspected_subset_indices);
            // Render with no selection
            if (!selected_full_indices.length) {
                if (this.glyph instanceof line_1.LineView) {
                    if (this.hover_glyph && inspected_subset_indices.length)
                        this.hover_glyph.render(ctx, this.model.view.convert_indices_from_subset(inspected_subset_indices), this.glyph);
                    else
                        glyph.render(ctx, all_indices, this.glyph);
                }
                else if (this.glyph instanceof patch_1.PatchView || this.glyph instanceof harea_1.HAreaView || this.glyph instanceof varea_1.VAreaView) {
                    if (inspected.selected_glyphs.length == 0 || this.hover_glyph == null) {
                        glyph.render(ctx, all_indices, this.glyph);
                    }
                    else {
                        try {
                            for (var _j = tslib_1.__values(inspected.selected_glyphs), _k = _j.next(); !_k.done; _k = _j.next()) {
                                var sglyph = _k.value;
                                if (sglyph == this.glyph.model)
                                    this.hover_glyph.render(ctx, all_indices, this.glyph);
                            }
                        }
                        catch (e_3_1) {
                            e_3 = { error: e_3_1 };
                        }
                        finally {
                            try {
                                if (_k && !_k.done && (_e = _j.return))
                                    _e.call(_j);
                            }
                            finally {
                                if (e_3)
                                    throw e_3.error;
                            }
                        }
                    }
                }
                else {
                    glyph.render(ctx, indices, this.glyph);
                    if (this.hover_glyph && inspected_subset_indices.length)
                        this.hover_glyph.render(ctx, inspected_subset_indices, this.glyph);
                }
                // Render with selection
            }
            else {
                // reset the selection mask
                var selected_mask = {};
                try {
                    for (var selected_full_indices_1 = tslib_1.__values(selected_full_indices), selected_full_indices_1_1 = selected_full_indices_1.next(); !selected_full_indices_1_1.done; selected_full_indices_1_1 = selected_full_indices_1.next()) {
                        var i = selected_full_indices_1_1.value;
                        selected_mask[i] = true;
                    }
                }
                catch (e_4_1) {
                    e_4 = { error: e_4_1 };
                }
                finally {
                    try {
                        if (selected_full_indices_1_1 && !selected_full_indices_1_1.done && (_f = selected_full_indices_1.return))
                            _f.call(selected_full_indices_1);
                    }
                    finally {
                        if (e_4)
                            throw e_4.error;
                    }
                }
                // intersect/different selection with render mask
                var selected_subset_indices = new Array();
                var nonselected_subset_indices = new Array();
                // now, selected is changed to subset space, except for Line glyph
                if (this.glyph instanceof line_1.LineView) {
                    try {
                        for (var all_indices_1 = tslib_1.__values(all_indices), all_indices_1_1 = all_indices_1.next(); !all_indices_1_1.done; all_indices_1_1 = all_indices_1.next()) {
                            var i = all_indices_1_1.value;
                            if (selected_mask[i] != null)
                                selected_subset_indices.push(i);
                            else
                                nonselected_subset_indices.push(i);
                        }
                    }
                    catch (e_5_1) {
                        e_5 = { error: e_5_1 };
                    }
                    finally {
                        try {
                            if (all_indices_1_1 && !all_indices_1_1.done && (_g = all_indices_1.return))
                                _g.call(all_indices_1);
                        }
                        finally {
                            if (e_5)
                                throw e_5.error;
                        }
                    }
                }
                else {
                    try {
                        for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                            var i = indices_1_1.value;
                            if (selected_mask[all_indices[i]] != null)
                                selected_subset_indices.push(i);
                            else
                                nonselected_subset_indices.push(i);
                        }
                    }
                    catch (e_6_1) {
                        e_6 = { error: e_6_1 };
                    }
                    finally {
                        try {
                            if (indices_1_1 && !indices_1_1.done && (_h = indices_1.return))
                                _h.call(indices_1);
                        }
                        finally {
                            if (e_6)
                                throw e_6.error;
                        }
                    }
                }
                nonselection_glyph.render(ctx, nonselected_subset_indices, this.glyph);
                selection_glyph.render(ctx, selected_subset_indices, this.glyph);
                if (this.hover_glyph != null) {
                    if (this.glyph instanceof line_1.LineView)
                        this.hover_glyph.render(ctx, this.model.view.convert_indices_from_subset(inspected_subset_indices), this.glyph);
                    else
                        this.hover_glyph.render(ctx, inspected_subset_indices, this.glyph);
                }
            }
            ctx.restore();
        };
        GlyphRendererView.prototype.draw_legend = function (ctx, x0, x1, y0, y1, field, label, index) {
            if (index == null)
                index = this.model.get_reference_point(field, label);
            this.glyph.draw_legend_for_index(ctx, { x0: x0, x1: x1, y0: y0, y1: y1 }, index);
        };
        GlyphRendererView.prototype.hit_test = function (geometry) {
            if (!this.model.visible)
                return null;
            var hit_test_result = this.glyph.hit_test(geometry);
            // glyphs that don't have hit-testing implemented will return null
            if (hit_test_result == null)
                return null;
            return this.model.view.convert_selection_from_subset(hit_test_result);
        };
        return GlyphRendererView;
    }(data_renderer_1.DataRendererView));
    exports.GlyphRendererView = GlyphRendererView;
    GlyphRendererView.__name__ = "GlyphRendererView";
    var GlyphRenderer = /** @class */ (function (_super) {
        tslib_1.__extends(GlyphRenderer, _super);
        function GlyphRenderer(attrs) {
            return _super.call(this, attrs) || this;
        }
        GlyphRenderer.init_GlyphRenderer = function () {
            this.prototype.default_view = GlyphRendererView;
            this.define({
                data_source: [p.Instance],
                view: [p.Instance, function () { return new cds_view_1.CDSView(); }],
                glyph: [p.Instance],
                hover_glyph: [p.Instance],
                nonselection_glyph: [p.Any, 'auto'],
                selection_glyph: [p.Any, 'auto'],
                muted_glyph: [p.Instance],
                muted: [p.Boolean, false],
            });
        };
        GlyphRenderer.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            if (this.view.source == null) {
                this.view.source = this.data_source;
                this.view.compute_indices();
            }
        };
        GlyphRenderer.prototype.get_reference_point = function (field, value) {
            var index = 0;
            if (field != null) {
                var data = this.data_source.get_column(field);
                if (data != null) {
                    var i = arrayable_1.indexOf(data, value);
                    if (i != -1)
                        index = i;
                }
            }
            return index;
        };
        GlyphRenderer.prototype.get_selection_manager = function () {
            return this.data_source.selection_manager;
        };
        return GlyphRenderer;
    }(data_renderer_1.DataRenderer));
    exports.GlyphRenderer = GlyphRenderer;
    GlyphRenderer.__name__ = "GlyphRenderer";
    GlyphRenderer.init_GlyphRenderer();
},
/* models/renderers/data_renderer.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var renderer_1 = require(185) /* ./renderer */;
    var DataRendererView = /** @class */ (function (_super) {
        tslib_1.__extends(DataRendererView, _super);
        function DataRendererView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Object.defineProperty(DataRendererView.prototype, "xscale", {
            get: function () {
                return this.coordinates.x_scale;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DataRendererView.prototype, "yscale", {
            get: function () {
                return this.coordinates.y_scale;
            },
            enumerable: true,
            configurable: true
        });
        return DataRendererView;
    }(renderer_1.RendererView));
    exports.DataRendererView = DataRendererView;
    DataRendererView.__name__ = "DataRendererView";
    var DataRenderer = /** @class */ (function (_super) {
        tslib_1.__extends(DataRenderer, _super);
        function DataRenderer(attrs) {
            return _super.call(this, attrs) || this;
        }
        DataRenderer.init_DataRenderer = function () {
            this.override({
                level: 'glyph',
            });
        };
        return DataRenderer;
    }(renderer_1.Renderer));
    exports.DataRenderer = DataRenderer;
    DataRenderer.__name__ = "DataRenderer";
    DataRenderer.init_DataRenderer();
},
/* models/glyphs/line.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var utils_1 = require(215) /* ./utils */;
    var line_1 = require(217) /* ./webgl/line */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var selection_1 = require(203) /* ../selections/selection */;
    var LineView = /** @class */ (function (_super) {
        tslib_1.__extends(LineView, _super);
        function LineView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        LineView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            var webgl = this.renderer.plot_view.canvas_view.webgl;
            if (webgl != null) {
                this.glglyph = new line_1.LineGL(webgl.gl, this);
            }
        };
        LineView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx = _a.sx, sy = _a.sy;
            var drawing = false;
            var last_index = null;
            this.visuals.line.set_value(ctx);
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (drawing) {
                        if (!isFinite(sx[i] + sy[i])) {
                            ctx.stroke();
                            ctx.beginPath();
                            drawing = false;
                            last_index = i;
                            continue;
                        }
                        if (last_index != null && i - last_index > 1) {
                            ctx.stroke();
                            drawing = false;
                        }
                    }
                    if (drawing)
                        ctx.lineTo(sx[i], sy[i]);
                    else {
                        ctx.beginPath();
                        ctx.moveTo(sx[i], sy[i]);
                        drawing = true;
                    }
                    last_index = i;
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            if (drawing)
                ctx.stroke();
        };
        LineView.prototype._hit_point = function (geometry) {
            /* Check if the point geometry hits this line glyph and return an object
            that describes the hit result:
              Args:
                * geometry (object): object with the following keys
                  * sx (float): screen x coordinate of the point
                  * sy (float): screen y coordinate of the point
                  * type (str): type of geometry (in this case it's a point)
            */
            var result = new selection_1.Selection();
            var point = { x: geometry.sx, y: geometry.sy };
            var shortest = 9999;
            var threshold = Math.max(2, this.visuals.line.line_width.value() / 2);
            for (var i = 0, end = this.sx.length - 1; i < end; i++) {
                var p0 = { x: this.sx[i], y: this.sy[i] };
                var p1 = { x: this.sx[i + 1], y: this.sy[i + 1] };
                var dist = hittest.dist_to_segment(point, p0, p1);
                if (dist < threshold && dist < shortest) {
                    shortest = dist;
                    result.add_to_selected_glyphs(this.model);
                    result.view = this;
                    result.line_indices = [i];
                }
            }
            return result;
        };
        LineView.prototype._hit_span = function (geometry) {
            var sx = geometry.sx, sy = geometry.sy;
            var result = new selection_1.Selection();
            var val;
            var values;
            if (geometry.direction == 'v') {
                val = this.renderer.yscale.invert(sy);
                values = this._y;
            }
            else {
                val = this.renderer.xscale.invert(sx);
                values = this._x;
            }
            for (var i = 0, end = values.length - 1; i < end; i++) {
                if ((values[i] <= val && val <= values[i + 1]) || (values[i + 1] <= val && val <= values[i])) {
                    result.add_to_selected_glyphs(this.model);
                    result.view = this;
                    result.line_indices.push(i);
                }
            }
            return result;
        };
        LineView.prototype.get_interpolation_hit = function (i, geometry) {
            var _a = tslib_1.__read([this._x[i], this._y[i], this._x[i + 1], this._y[i + 1]], 4), x2 = _a[0], y2 = _a[1], x3 = _a[2], y3 = _a[3];
            return utils_1.line_interpolation(this.renderer, geometry, x2, y2, x3, y3);
        };
        LineView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_line_legend(this.visuals, ctx, bbox, index);
        };
        return LineView;
    }(xy_glyph_1.XYGlyphView));
    exports.LineView = LineView;
    LineView.__name__ = "LineView";
    var Line = /** @class */ (function (_super) {
        tslib_1.__extends(Line, _super);
        function Line(attrs) {
            return _super.call(this, attrs) || this;
        }
        Line.init_Line = function () {
            this.prototype.default_view = LineView;
            this.mixins(mixins.Line /*Scalar*/);
        };
        return Line;
    }(xy_glyph_1.XYGlyph));
    exports.Line = Line;
    Line.__name__ = "Line";
    Line.init_Line();
},
/* models/glyphs/xy_glyph.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var projections_1 = require(152) /* ../../core/util/projections */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var glyph_1 = require(209) /* ./glyph */;
    var XYGlyphView = /** @class */ (function (_super) {
        tslib_1.__extends(XYGlyphView, _super);
        function XYGlyphView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        XYGlyphView.prototype._project_data = function () {
            projections_1.inplace.project_xy(this._x, this._y);
        };
        XYGlyphView.prototype._index_data = function (index) {
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                var x = this._x[i];
                var y = this._y[i];
                if (isNaN(x + y) || !isFinite(x + y))
                    index.add_empty();
                else
                    index.add(x, y, x, y);
            }
        };
        XYGlyphView.prototype.scenterxy = function (i) {
            return [this.sx[i], this.sy[i]];
        };
        return XYGlyphView;
    }(glyph_1.GlyphView));
    exports.XYGlyphView = XYGlyphView;
    XYGlyphView.__name__ = "XYGlyphView";
    var XYGlyph = /** @class */ (function (_super) {
        tslib_1.__extends(XYGlyph, _super);
        function XYGlyph(attrs) {
            return _super.call(this, attrs) || this;
        }
        XYGlyph.init_XYGlyph = function () {
            this.define({
                x: [p.XCoordinateSpec, { field: "x" }],
                y: [p.YCoordinateSpec, { field: "y" }],
            });
        };
        return XYGlyph;
    }(glyph_1.Glyph));
    exports.XYGlyph = XYGlyph;
    XYGlyph.__name__ = "XYGlyph";
    XYGlyph.init_XYGlyph();
},
/* models/glyphs/glyph.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var bbox = tslib_1.__importStar(require(194) /* ../../core/util/bbox */);
    var visuals = tslib_1.__importStar(require(189) /* ../../core/visuals */);
    var view_1 = require(186) /* ../../core/view */;
    var model_1 = require(196) /* ../../model */;
    var logging_1 = require(134) /* ../../core/logging */;
    var types_1 = require(139) /* ../../core/types */;
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var spatial_1 = require(210) /* ../../core/util/spatial */;
    var factor_range_1 = require(213) /* ../ranges/factor_range */;
    var selection_1 = require(203) /* ../selections/selection */;
    var GlyphView = /** @class */ (function (_super) {
        tslib_1.__extends(GlyphView, _super);
        function GlyphView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this._index = null;
            _this._data_size = null;
            _this._nohit_warned = new Set();
            return _this;
        }
        Object.defineProperty(GlyphView.prototype, "renderer", {
            get: function () {
                return this.parent;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(GlyphView.prototype, "has_webgl", {
            get: function () {
                return this.glglyph != null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(GlyphView.prototype, "index", {
            get: function () {
                var _index = this._index;
                if (_index != null)
                    return _index;
                else
                    throw new Error(this + ".index_data() wasn't called");
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(GlyphView.prototype, "data_size", {
            get: function () {
                var _data_size = this._data_size;
                if (_data_size != null)
                    return _data_size;
                else
                    throw new Error(this + ".set_data() wasn't called");
            },
            enumerable: true,
            configurable: true
        });
        GlyphView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.visuals = new visuals.Visuals(this.model);
        };
        GlyphView.prototype.set_visuals = function (source, indices) {
            this.visuals.warm_cache(source, indices);
            if (this.glglyph != null)
                this.glglyph.set_visuals_changed();
        };
        GlyphView.prototype.render = function (ctx, indices, data) {
            ctx.beginPath();
            if (this.glglyph != null) {
                this.renderer.needs_webgl_blit = this.glglyph.render(ctx, indices, data);
                if (this.renderer.needs_webgl_blit)
                    return;
            }
            this._render(ctx, indices, data);
        };
        GlyphView.prototype.has_finished = function () {
            return true;
        };
        GlyphView.prototype.notify_finished = function () {
            this.renderer.notify_finished();
        };
        GlyphView.prototype._bounds = function (bounds) {
            return bounds;
        };
        GlyphView.prototype.bounds = function () {
            return this._bounds(this.index.bbox);
        };
        GlyphView.prototype.log_bounds = function () {
            var _c = this.index.bounds(bbox.positive_x()), x0 = _c.x0, x1 = _c.x1;
            var _d = this.index.bounds(bbox.positive_y()), y0 = _d.y0, y1 = _d.y1;
            return this._bounds({ x0: x0, y0: y0, x1: x1, y1: y1 });
        };
        GlyphView.prototype.get_anchor_point = function (anchor, i, _c) {
            var _d = tslib_1.__read(_c, 2), sx = _d[0], sy = _d[1];
            switch (anchor) {
                case "center": {
                    var _e = tslib_1.__read(this.scenterxy(i, sx, sy), 2), x = _e[0], y = _e[1];
                    return { x: x, y: y };
                }
                default:
                    return null;
            }
        };
        /** @deprecated */
        GlyphView.prototype.scenterx = function (i, sx, sy) {
            return this.scenterxy(i, sx, sy)[0];
        };
        /** @deprecated */
        GlyphView.prototype.scentery = function (i, sx, sy) {
            return this.scenterxy(i, sx, sy)[1];
        };
        GlyphView.prototype.sdist = function (scale, pts, spans, pts_location, dilate) {
            if (pts_location === void 0) {
                pts_location = "edge";
            }
            if (dilate === void 0) {
                dilate = false;
            }
            var pt0;
            var pt1;
            var n = pts.length;
            if (pts_location == 'center') {
                var halfspan = arrayable_1.map(spans, function (d) { return d / 2; });
                pt0 = new types_1.NumberArray(n);
                for (var i = 0; i < n; i++) {
                    pt0[i] = pts[i] - halfspan[i];
                }
                pt1 = new types_1.NumberArray(n);
                for (var i = 0; i < n; i++) {
                    pt1[i] = pts[i] + halfspan[i];
                }
            }
            else {
                pt0 = pts;
                pt1 = new types_1.NumberArray(n);
                for (var i = 0; i < n; i++) {
                    pt1[i] = pt0[i] + spans[i];
                }
            }
            var spt0 = scale.v_compute(pt0);
            var spt1 = scale.v_compute(pt1);
            if (dilate)
                return arrayable_1.map(spt0, function (_, i) { return Math.ceil(Math.abs(spt1[i] - spt0[i])); });
            else
                return arrayable_1.map(spt0, function (_, i) { return Math.abs(spt1[i] - spt0[i]); });
        };
        GlyphView.prototype.draw_legend_for_index = function (_ctx, _bbox, _index) { };
        GlyphView.prototype.hit_test = function (geometry) {
            switch (geometry.type) {
                case "point":
                    if (this._hit_point != null)
                        return this._hit_point(geometry);
                    break;
                case "span":
                    if (this._hit_span != null)
                        return this._hit_span(geometry);
                    break;
                case "rect":
                    if (this._hit_rect != null)
                        return this._hit_rect(geometry);
                    break;
                case "poly":
                    if (this._hit_poly != null)
                        return this._hit_poly(geometry);
                    break;
            }
            if (!this._nohit_warned.has(geometry.type)) {
                logging_1.logger.debug("'" + geometry.type + "' selection not available for " + this.model.type);
                this._nohit_warned.add(geometry.type);
            }
            return null;
        };
        GlyphView.prototype._hit_rect_against_index = function (geometry) {
            var sx0 = geometry.sx0, sx1 = geometry.sx1, sy0 = geometry.sy0, sy1 = geometry.sy1;
            var _c = tslib_1.__read(this.renderer.coordinates.x_scale.r_invert(sx0, sx1), 2), x0 = _c[0], x1 = _c[1];
            var _d = tslib_1.__read(this.renderer.coordinates.y_scale.r_invert(sy0, sy1), 2), y0 = _d[0], y1 = _d[1];
            var indices = tslib_1.__spread(this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 }));
            return new selection_1.Selection({ indices: indices });
        };
        GlyphView.prototype._project_data = function () { };
        GlyphView.prototype.set_data = function (source, indices, indices_to_update) {
            var e_1, _c;
            var _a, _b;
            var _d = this.renderer.coordinates, x_range = _d.x_range, y_range = _d.y_range;
            this._data_size = (_a = source.get_length()) !== null && _a !== void 0 ? _a : 1;
            try {
                for (var _e = tslib_1.__values(this.model), _f = _e.next(); !_f.done; _f = _e.next()) {
                    var prop = _f.value;
                    if (!(prop instanceof p.VectorSpec))
                        continue;
                    // this skips optional properties like radius for circles
                    if (prop.optional && prop.spec.value == null && !prop.dirty)
                        continue;
                    var name = prop.attr;
                    var base_array = prop.array(source);
                    var array = indices.select(base_array);
                    if (prop instanceof p.BaseCoordinateSpec) {
                        var range = prop.dimension == "x" ? x_range : y_range;
                        if (range instanceof factor_range_1.FactorRange) {
                            if (prop instanceof p.CoordinateSpec) {
                                array = range.v_synthetic(array);
                            }
                            else if (prop instanceof p.CoordinateSeqSpec) {
                                for (var i = 0; i < array.length; i++) {
                                    array[i] = range.v_synthetic(array[i]);
                                }
                            }
                        }
                        if (prop instanceof p.CoordinateSeqSpec) {
                            array = types_1.RaggedArray.from(array);
                        }
                    }
                    else if (prop instanceof p.DistanceSpec) {
                        this["max_" + name] = arrayable_1.max(array);
                    }
                    this["_" + name] = array;
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_f && !_f.done && (_c = _e.return))
                        _c.call(_e);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            if (this.renderer.plot_view.model.use_map) {
                this._project_data();
            }
            this._set_data(indices_to_update); // TODO doesn't take subset indices into account
            (_b = this.glglyph) === null || _b === void 0 ? void 0 : _b.set_data_changed();
            this.index_data();
        };
        GlyphView.prototype._set_data = function (_indices) { };
        Object.defineProperty(GlyphView.prototype, "_index_size", {
            get: function () {
                return this.data_size;
            },
            enumerable: true,
            configurable: true
        });
        GlyphView.prototype.index_data = function () {
            var index = new spatial_1.SpatialIndex(this._index_size);
            this._index_data(index);
            index.finish();
            this._index = index;
        };
        GlyphView.prototype.mask_data = function () {
            // WebGL can do the clipping much more efficiently
            if (this.glglyph != null || this._mask_data == null)
                return types_1.Indices.all_set(this.data_size);
            else
                return this._mask_data();
        };
        GlyphView.prototype.map_data = function () {
            var e_2, _c;
            var _a;
            var self = this;
            var _d = this.renderer.coordinates, x_scale = _d.x_scale, y_scale = _d.y_scale;
            try {
                for (var _e = tslib_1.__values(this.model), _f = _e.next(); !_f.done; _f = _e.next()) {
                    var prop = _f.value;
                    if (prop instanceof p.BaseCoordinateSpec) {
                        var scale = prop.dimension == "x" ? x_scale : y_scale;
                        var array = self["_" + prop.attr];
                        if (array instanceof types_1.RaggedArray) {
                            var screen = scale.v_compute(array.array);
                            array = new types_1.RaggedArray(array.offsets, screen);
                        }
                        else {
                            array = scale.v_compute(array);
                        }
                        this["s" + prop.attr] = array;
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_f && !_f.done && (_c = _e.return))
                        _c.call(_e);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            this._map_data();
            (_a = this.glglyph) === null || _a === void 0 ? void 0 : _a.set_data_changed();
        };
        // This is where specs not included in coords are computed, e.g. radius.
        GlyphView.prototype._map_data = function () { };
        return GlyphView;
    }(view_1.View));
    exports.GlyphView = GlyphView;
    GlyphView.__name__ = "GlyphView";
    var Glyph = /** @class */ (function (_super) {
        tslib_1.__extends(Glyph, _super);
        function Glyph(attrs) {
            return _super.call(this, attrs) || this;
        }
        Glyph.init_Glyph = function () { };
        return Glyph;
    }(model_1.Model));
    exports.Glyph = Glyph;
    Glyph.__name__ = "Glyph";
    Glyph.init_Glyph();
},
/* core/util/spatial.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var flatbush_1 = tslib_1.__importDefault(require(211) /* flatbush */);
    var types_1 = require(139) /* ../types */;
    var bbox_1 = require(194) /* ./bbox */;
    function upperBound(value, arr) {
        var i = 0;
        var j = arr.length - 1;
        while (i < j) {
            var m = (i + j) >> 1;
            if (arr[m] > value) {
                j = m;
            }
            else {
                i = m + 1;
            }
        }
        return arr[i];
    }
    var _FlatBush = /** @class */ (function (_super) {
        tslib_1.__extends(_FlatBush, _super);
        function _FlatBush() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        _FlatBush.prototype.search_indices = function (minX, minY, maxX, maxY) {
            if (this._pos !== this._boxes.length) {
                throw new Error('Data not yet indexed - call index.finish().');
            }
            var nodeIndex = this._boxes.length - 4;
            var queue = [];
            var results = new types_1.Indices(this.numItems);
            while (nodeIndex !== undefined) {
                // find the end index of the node
                var end = Math.min(nodeIndex + this.nodeSize * 4, upperBound(nodeIndex, this._levelBounds));
                // search through child nodes
                for (var pos = nodeIndex; pos < end; pos += 4) {
                    var index = this._indices[pos >> 2] | 0;
                    // check if node bbox intersects with query bbox
                    if (maxX < this._boxes[pos + 0])
                        continue; // maxX < nodeMinX
                    if (maxY < this._boxes[pos + 1])
                        continue; // maxY < nodeMinY
                    if (minX > this._boxes[pos + 2])
                        continue; // minX > nodeMaxX
                    if (minY > this._boxes[pos + 3])
                        continue; // minY > nodeMaxY
                    if (nodeIndex < this.numItems * 4) {
                        results.set(index); // leaf item
                    }
                    else {
                        queue.push(index); // node; add it to the search queue
                    }
                }
                nodeIndex = queue.pop();
            }
            return results;
        };
        return _FlatBush;
    }(flatbush_1.default));
    _FlatBush.__name__ = "_FlatBush";
    var SpatialIndex = /** @class */ (function () {
        function SpatialIndex(size) {
            this.index = null;
            if (size > 0) {
                this.index = new _FlatBush(size);
            }
        }
        SpatialIndex.prototype.add = function (x0, y0, x1, y1) {
            var _a;
            (_a = this.index) === null || _a === void 0 ? void 0 : _a.add(x0, y0, x1, y1);
        };
        SpatialIndex.prototype.add_empty = function () {
            var _a;
            (_a = this.index) === null || _a === void 0 ? void 0 : _a.add(Infinity, Infinity, -Infinity, -Infinity);
        };
        SpatialIndex.prototype.finish = function () {
            var _a;
            (_a = this.index) === null || _a === void 0 ? void 0 : _a.finish();
        };
        SpatialIndex.prototype._normalize = function (rect) {
            var _b, _c;
            var x0 = rect.x0, y0 = rect.y0, x1 = rect.x1, y1 = rect.y1;
            if (x0 > x1)
                _b = tslib_1.__read([x1, x0], 2), x0 = _b[0], x1 = _b[1];
            if (y0 > y1)
                _c = tslib_1.__read([y1, y0], 2), y0 = _c[0], y1 = _c[1];
            return { x0: x0, y0: y0, x1: x1, y1: y1 };
        };
        Object.defineProperty(SpatialIndex.prototype, "bbox", {
            get: function () {
                if (this.index == null)
                    return bbox_1.empty();
                else {
                    var _b = this.index, minX = _b.minX, minY = _b.minY, maxX = _b.maxX, maxY = _b.maxY;
                    return { x0: minX, y0: minY, x1: maxX, y1: maxY };
                }
            },
            enumerable: true,
            configurable: true
        });
        SpatialIndex.prototype.indices = function (rect) {
            if (this.index == null)
                return new types_1.Indices(0);
            else {
                var _b = this._normalize(rect), x0 = _b.x0, y0 = _b.y0, x1 = _b.x1, y1 = _b.y1;
                return this.index.search_indices(x0, y0, x1, y1);
            }
        };
        SpatialIndex.prototype.bounds = function (rect) {
            var e_1, _b;
            var bounds = bbox_1.empty();
            try {
                for (var _c = tslib_1.__values(this.indices(rect)), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var i = _d.value;
                    var boxes = this.index._boxes;
                    var x1 = boxes[4 * i + 0];
                    var y1 = boxes[4 * i + 1];
                    var x0 = boxes[4 * i + 2];
                    var y0 = boxes[4 * i + 3];
                    if (x0 < bounds.x0)
                        bounds.x0 = x0;
                    if (x1 > bounds.x1)
                        bounds.x1 = x1;
                    if (y0 < bounds.y0)
                        bounds.y0 = y0;
                    if (y1 > bounds.y1)
                        bounds.y1 = y1;
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return bounds;
        };
        return SpatialIndex;
    }());
    exports.SpatialIndex = SpatialIndex;
    SpatialIndex.__name__ = "SpatialIndex";
},
/* flatbush/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var flatqueue_1 = tslib_1.__importDefault(require(212) /* flatqueue */);
    var ARRAY_TYPES = [
        Int8Array, Uint8Array, Uint8ClampedArray, Int16Array, Uint16Array,
        Int32Array, Uint32Array, Float32Array, Float64Array
    ];
    var VERSION = 3; // serialized format version
    var Flatbush = /** @class */ (function () {
        function Flatbush(numItems, nodeSize, ArrayType, data) {
            if (nodeSize === void 0) {
                nodeSize = 16;
            }
            if (ArrayType === void 0) {
                ArrayType = Float64Array;
            }
            if (numItems === undefined)
                throw new Error('Missing required argument: numItems.');
            if (isNaN(numItems) || numItems <= 0)
                throw new Error("Unpexpected numItems value: " + numItems + ".");
            this.numItems = +numItems;
            this.nodeSize = Math.min(Math.max(+nodeSize, 2), 65535);
            // calculate the total number of nodes in the R-tree to allocate space for
            // and the index of each tree level (used in search later)
            var n = numItems;
            var numNodes = n;
            this._levelBounds = [n * 4];
            do {
                n = Math.ceil(n / this.nodeSize);
                numNodes += n;
                this._levelBounds.push(numNodes * 4);
            } while (n !== 1);
            this.ArrayType = ArrayType || Float64Array;
            this.IndexArrayType = numNodes < 16384 ? Uint16Array : Uint32Array;
            var arrayTypeIndex = ARRAY_TYPES.indexOf(this.ArrayType);
            var nodesByteSize = numNodes * 4 * this.ArrayType.BYTES_PER_ELEMENT;
            if (arrayTypeIndex < 0) {
                throw new Error("Unexpected typed array class: " + ArrayType + ".");
            }
            if (data && (data instanceof ArrayBuffer)) {
                this.data = data;
                this._boxes = new this.ArrayType(this.data, 8, numNodes * 4);
                this._indices = new this.IndexArrayType(this.data, 8 + nodesByteSize, numNodes);
                this._pos = numNodes * 4;
                this.minX = this._boxes[this._pos - 4];
                this.minY = this._boxes[this._pos - 3];
                this.maxX = this._boxes[this._pos - 2];
                this.maxY = this._boxes[this._pos - 1];
            }
            else {
                this.data = new ArrayBuffer(8 + nodesByteSize + numNodes * this.IndexArrayType.BYTES_PER_ELEMENT);
                this._boxes = new this.ArrayType(this.data, 8, numNodes * 4);
                this._indices = new this.IndexArrayType(this.data, 8 + nodesByteSize, numNodes);
                this._pos = 0;
                this.minX = Infinity;
                this.minY = Infinity;
                this.maxX = -Infinity;
                this.maxY = -Infinity;
                new Uint8Array(this.data, 0, 2).set([0xfb, (VERSION << 4) + arrayTypeIndex]);
                new Uint16Array(this.data, 2, 1)[0] = nodeSize;
                new Uint32Array(this.data, 4, 1)[0] = numItems;
            }
            // a priority queue for k-nearest-neighbors queries
            this._queue = new flatqueue_1.default();
        }
        Flatbush.from = function (data) {
            if (!(data instanceof ArrayBuffer)) {
                throw new Error('Data must be an instance of ArrayBuffer.');
            }
            var _a = tslib_1.__read(new Uint8Array(data, 0, 2), 2), magic = _a[0], versionAndType = _a[1];
            if (magic !== 0xfb) {
                throw new Error('Data does not appear to be in a Flatbush format.');
            }
            if (versionAndType >> 4 !== VERSION) {
                throw new Error("Got v" + (versionAndType >> 4) + " data when expected v" + VERSION + ".");
            }
            var _b = tslib_1.__read(new Uint16Array(data, 2, 1), 1), nodeSize = _b[0];
            var _c = tslib_1.__read(new Uint32Array(data, 4, 1), 1), numItems = _c[0];
            return new Flatbush(numItems, nodeSize, ARRAY_TYPES[versionAndType & 0x0f], data);
        };
        Flatbush.prototype.add = function (minX, minY, maxX, maxY) {
            var index = this._pos >> 2;
            this._indices[index] = index;
            this._boxes[this._pos++] = minX;
            this._boxes[this._pos++] = minY;
            this._boxes[this._pos++] = maxX;
            this._boxes[this._pos++] = maxY;
            if (minX < this.minX)
                this.minX = minX;
            if (minY < this.minY)
                this.minY = minY;
            if (maxX > this.maxX)
                this.maxX = maxX;
            if (maxY > this.maxY)
                this.maxY = maxY;
            return index;
        };
        Flatbush.prototype.finish = function () {
            if (this._pos >> 2 !== this.numItems) {
                throw new Error("Added " + (this._pos >> 2) + " items when expected " + this.numItems + ".");
            }
            if (this.numItems <= this.nodeSize) {
                // only one node, skip sorting and just fill the root box
                this._boxes[this._pos++] = this.minX;
                this._boxes[this._pos++] = this.minY;
                this._boxes[this._pos++] = this.maxX;
                this._boxes[this._pos++] = this.maxY;
                return;
            }
            var width = this.maxX - this.minX;
            var height = this.maxY - this.minY;
            var hilbertValues = new Uint32Array(this.numItems);
            var hilbertMax = (1 << 16) - 1;
            // map item centers into Hilbert coordinate space and calculate Hilbert values
            for (var i = 0; i < this.numItems; i++) {
                var pos = 4 * i;
                var minX = this._boxes[pos++];
                var minY = this._boxes[pos++];
                var maxX = this._boxes[pos++];
                var maxY = this._boxes[pos++];
                var x = Math.floor(hilbertMax * ((minX + maxX) / 2 - this.minX) / width);
                var y = Math.floor(hilbertMax * ((minY + maxY) / 2 - this.minY) / height);
                hilbertValues[i] = hilbert(x, y);
            }
            // sort items by their Hilbert value (for packing later)
            sort(hilbertValues, this._boxes, this._indices, 0, this.numItems - 1, this.nodeSize);
            // generate nodes at each tree level, bottom-up
            for (var i = 0, pos = 0; i < this._levelBounds.length - 1; i++) {
                var end = this._levelBounds[i];
                // generate a parent node for each block of consecutive <nodeSize> nodes
                while (pos < end) {
                    var nodeIndex = pos;
                    // calculate bbox for the new node
                    var nodeMinX = Infinity;
                    var nodeMinY = Infinity;
                    var nodeMaxX = -Infinity;
                    var nodeMaxY = -Infinity;
                    for (var i_1 = 0; i_1 < this.nodeSize && pos < end; i_1++) {
                        nodeMinX = Math.min(nodeMinX, this._boxes[pos++]);
                        nodeMinY = Math.min(nodeMinY, this._boxes[pos++]);
                        nodeMaxX = Math.max(nodeMaxX, this._boxes[pos++]);
                        nodeMaxY = Math.max(nodeMaxY, this._boxes[pos++]);
                    }
                    // add the new node to the tree data
                    this._indices[this._pos >> 2] = nodeIndex;
                    this._boxes[this._pos++] = nodeMinX;
                    this._boxes[this._pos++] = nodeMinY;
                    this._boxes[this._pos++] = nodeMaxX;
                    this._boxes[this._pos++] = nodeMaxY;
                }
            }
        };
        Flatbush.prototype.search = function (minX, minY, maxX, maxY, filterFn) {
            if (this._pos !== this._boxes.length) {
                throw new Error('Data not yet indexed - call index.finish().');
            }
            var nodeIndex = this._boxes.length - 4;
            var queue = [];
            var results = [];
            while (nodeIndex !== undefined) {
                // find the end index of the node
                var end = Math.min(nodeIndex + this.nodeSize * 4, upperBound(nodeIndex, this._levelBounds));
                // search through child nodes
                for (var pos = nodeIndex; pos < end; pos += 4) {
                    var index = this._indices[pos >> 2] | 0;
                    // check if node bbox intersects with query bbox
                    if (maxX < this._boxes[pos])
                        continue; // maxX < nodeMinX
                    if (maxY < this._boxes[pos + 1])
                        continue; // maxY < nodeMinY
                    if (minX > this._boxes[pos + 2])
                        continue; // minX > nodeMaxX
                    if (minY > this._boxes[pos + 3])
                        continue; // minY > nodeMaxY
                    if (nodeIndex < this.numItems * 4) {
                        if (filterFn === undefined || filterFn(index)) {
                            results.push(index); // leaf item
                        }
                    }
                    else {
                        queue.push(index); // node; add it to the search queue
                    }
                }
                nodeIndex = queue.pop();
            }
            return results;
        };
        Flatbush.prototype.neighbors = function (x, y, maxResults, maxDistance, filterFn) {
            if (maxResults === void 0) {
                maxResults = Infinity;
            }
            if (maxDistance === void 0) {
                maxDistance = Infinity;
            }
            if (this._pos !== this._boxes.length) {
                throw new Error('Data not yet indexed - call index.finish().');
            }
            var nodeIndex = this._boxes.length - 4;
            var q = this._queue;
            var results = [];
            var maxDistSquared = maxDistance * maxDistance;
            while (nodeIndex !== undefined) {
                // find the end index of the node
                var end = Math.min(nodeIndex + this.nodeSize * 4, upperBound(nodeIndex, this._levelBounds));
                // add child nodes to the queue
                for (var pos = nodeIndex; pos < end; pos += 4) {
                    var index = this._indices[pos >> 2] | 0;
                    var dx = axisDist(x, this._boxes[pos], this._boxes[pos + 2]);
                    var dy = axisDist(y, this._boxes[pos + 1], this._boxes[pos + 3]);
                    var dist = dx * dx + dy * dy;
                    if (nodeIndex < this.numItems * 4) { // leaf node
                        if (filterFn === undefined || filterFn(index)) {
                            // put a negative index if it's an item rather than a node, to recognize later
                            q.push(-index - 1, dist);
                        }
                    }
                    else {
                        q.push(index, dist);
                    }
                }
                // pop items from the queue
                while (q.length && q.peek() < 0) {
                    var dist = q.peekValue();
                    if (dist > maxDistSquared) {
                        q.clear();
                        return results;
                    }
                    results.push(-q.pop() - 1);
                    if (results.length === maxResults) {
                        q.clear();
                        return results;
                    }
                }
                nodeIndex = q.pop();
            }
            q.clear();
            return results;
        };
        return Flatbush;
    }());
    exports.default = Flatbush;
    function axisDist(k, min, max) {
        return k < min ? min - k : k <= max ? 0 : k - max;
    }
    // binary search for the first value in the array bigger than the given
    function upperBound(value, arr) {
        var i = 0;
        var j = arr.length - 1;
        while (i < j) {
            var m = (i + j) >> 1;
            if (arr[m] > value) {
                j = m;
            }
            else {
                i = m + 1;
            }
        }
        return arr[i];
    }
    // custom quicksort that partially sorts bbox data alongside the hilbert values
    function sort(values, boxes, indices, left, right, nodeSize) {
        if (Math.floor(left / nodeSize) >= Math.floor(right / nodeSize))
            return;
        var pivot = values[(left + right) >> 1];
        var i = left - 1;
        var j = right + 1;
        while (true) {
            do
                i++;
            while (values[i] < pivot);
            do
                j--;
            while (values[j] > pivot);
            if (i >= j)
                break;
            swap(values, boxes, indices, i, j);
        }
        sort(values, boxes, indices, left, j, nodeSize);
        sort(values, boxes, indices, j + 1, right, nodeSize);
    }
    // swap two values and two corresponding boxes
    function swap(values, boxes, indices, i, j) {
        var temp = values[i];
        values[i] = values[j];
        values[j] = temp;
        var k = 4 * i;
        var m = 4 * j;
        var a = boxes[k];
        var b = boxes[k + 1];
        var c = boxes[k + 2];
        var d = boxes[k + 3];
        boxes[k] = boxes[m];
        boxes[k + 1] = boxes[m + 1];
        boxes[k + 2] = boxes[m + 2];
        boxes[k + 3] = boxes[m + 3];
        boxes[m] = a;
        boxes[m + 1] = b;
        boxes[m + 2] = c;
        boxes[m + 3] = d;
        var e = indices[i];
        indices[i] = indices[j];
        indices[j] = e;
    }
    // Fast Hilbert curve algorithm by http://threadlocalmutex.com/
    // Ported from C++ https://github.com/rawrunprotected/hilbert_curves (public domain)
    function hilbert(x, y) {
        var a = x ^ y;
        var b = 0xFFFF ^ a;
        var c = 0xFFFF ^ (x | y);
        var d = x & (y ^ 0xFFFF);
        var A = a | (b >> 1);
        var B = (a >> 1) ^ a;
        var C = ((c >> 1) ^ (b & (d >> 1))) ^ c;
        var D = ((a & (c >> 1)) ^ (d >> 1)) ^ d;
        a = A;
        b = B;
        c = C;
        d = D;
        A = ((a & (a >> 2)) ^ (b & (b >> 2)));
        B = ((a & (b >> 2)) ^ (b & ((a ^ b) >> 2)));
        C ^= ((a & (c >> 2)) ^ (b & (d >> 2)));
        D ^= ((b & (c >> 2)) ^ ((a ^ b) & (d >> 2)));
        a = A;
        b = B;
        c = C;
        d = D;
        A = ((a & (a >> 4)) ^ (b & (b >> 4)));
        B = ((a & (b >> 4)) ^ (b & ((a ^ b) >> 4)));
        C ^= ((a & (c >> 4)) ^ (b & (d >> 4)));
        D ^= ((b & (c >> 4)) ^ ((a ^ b) & (d >> 4)));
        a = A;
        b = B;
        c = C;
        d = D;
        C ^= ((a & (c >> 8)) ^ (b & (d >> 8)));
        D ^= ((b & (c >> 8)) ^ ((a ^ b) & (d >> 8)));
        a = C ^ (C >> 1);
        b = D ^ (D >> 1);
        var i0 = x ^ y;
        var i1 = b | (0xFFFF ^ (i0 | a));
        i0 = (i0 | (i0 << 8)) & 0x00FF00FF;
        i0 = (i0 | (i0 << 4)) & 0x0F0F0F0F;
        i0 = (i0 | (i0 << 2)) & 0x33333333;
        i0 = (i0 | (i0 << 1)) & 0x55555555;
        i1 = (i1 | (i1 << 8)) & 0x00FF00FF;
        i1 = (i1 | (i1 << 4)) & 0x0F0F0F0F;
        i1 = (i1 | (i1 << 2)) & 0x33333333;
        i1 = (i1 | (i1 << 1)) & 0x55555555;
        return ((i1 << 1) | i0) >>> 0;
    }
},
/* flatqueue/index.mjs */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var FlatQueue = /** @class */ (function () {
        function FlatQueue() {
            this.ids = [];
            this.values = [];
            this.length = 0;
        }
        FlatQueue.prototype.clear = function () {
            this.length = 0;
        };
        FlatQueue.prototype.push = function (id, value) {
            var pos = this.length++;
            this.ids[pos] = id;
            this.values[pos] = value;
            while (pos > 0) {
                var parent = (pos - 1) >> 1;
                var parentValue = this.values[parent];
                if (value >= parentValue)
                    break;
                this.ids[pos] = this.ids[parent];
                this.values[pos] = parentValue;
                pos = parent;
            }
            this.ids[pos] = id;
            this.values[pos] = value;
        };
        FlatQueue.prototype.pop = function () {
            if (this.length === 0)
                return undefined;
            var top = this.ids[0];
            this.length--;
            if (this.length > 0) {
                var id = this.ids[0] = this.ids[this.length];
                var value = this.values[0] = this.values[this.length];
                var halfLength = this.length >> 1;
                var pos = 0;
                while (pos < halfLength) {
                    var left = (pos << 1) + 1;
                    var right = left + 1;
                    var bestIndex = this.ids[left];
                    var bestValue = this.values[left];
                    var rightValue = this.values[right];
                    if (right < this.length && rightValue < bestValue) {
                        left = right;
                        bestIndex = this.ids[right];
                        bestValue = rightValue;
                    }
                    if (bestValue >= value)
                        break;
                    this.ids[pos] = bestIndex;
                    this.values[pos] = bestValue;
                    pos = left;
                }
                this.ids[pos] = id;
                this.values[pos] = value;
            }
            return top;
        };
        FlatQueue.prototype.peek = function () {
            if (this.length === 0)
                return undefined;
            return this.ids[0];
        };
        FlatQueue.prototype.peekValue = function () {
            if (this.length === 0)
                return undefined;
            return this.values[0];
        };
        return FlatQueue;
    }());
    exports.default = FlatQueue;
},
/* models/ranges/factor_range.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var range_1 = require(214) /* ./range */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(139) /* ../../core/types */;
    var array_1 = require(124) /* ../../core/util/array */;
    var types_2 = require(123) /* ../../core/util/types */;
    var assert_1 = require(126) /* ../../core/util/assert */;
    function map_one_level(factors, padding, offset) {
        if (offset === void 0) {
            offset = 0;
        }
        var mapping = new Map();
        for (var i = 0; i < factors.length; i++) {
            var factor = factors[i];
            if (!mapping.has(factor))
                mapping.set(factor, { value: 0.5 + i * (1 + padding) + offset });
            else
                throw new Error("duplicate factor or subfactor: " + factor);
        }
        return [mapping, (factors.length - 1) * padding];
    }
    exports.map_one_level = map_one_level;
    function map_two_levels(factors, outer_pad, factor_pad, offset) {
        var e_1, _b, e_2, _c;
        if (offset === void 0) {
            offset = 0;
        }
        var _a;
        var mapping = new Map();
        var tops = new Map();
        try {
            for (var factors_1 = tslib_1.__values(factors), factors_1_1 = factors_1.next(); !factors_1_1.done; factors_1_1 = factors_1.next()) {
                var _d = tslib_1.__read(factors_1_1.value, 2), f0 = _d[0], f1 = _d[1];
                var top = (_a = tops.get(f0)) !== null && _a !== void 0 ? _a : [];
                tops.set(f0, tslib_1.__spread(top, [f1]));
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (factors_1_1 && !factors_1_1.done && (_b = factors_1.return))
                    _b.call(factors_1);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        var suboffset = offset;
        var total_subpad = 0;
        var _loop_1 = function (f0, top) {
            var n = top.length;
            var _b = tslib_1.__read(map_one_level(top, factor_pad, suboffset), 2), submap = _b[0], subpad = _b[1];
            total_subpad += subpad;
            var subtot = array_1.sum(top.map(function (f1) { return submap.get(f1).value; }));
            mapping.set(f0, { value: subtot / n, mapping: submap });
            suboffset += n + outer_pad + subpad;
        };
        try {
            for (var tops_1 = tslib_1.__values(tops), tops_1_1 = tops_1.next(); !tops_1_1.done; tops_1_1 = tops_1.next()) {
                var _e = tslib_1.__read(tops_1_1.value, 2), f0 = _e[0], top = _e[1];
                _loop_1(f0, top);
            }
        }
        catch (e_2_1) {
            e_2 = { error: e_2_1 };
        }
        finally {
            try {
                if (tops_1_1 && !tops_1_1.done && (_c = tops_1.return))
                    _c.call(tops_1);
            }
            finally {
                if (e_2)
                    throw e_2.error;
            }
        }
        return [mapping, (tops.size - 1) * outer_pad + total_subpad];
    }
    exports.map_two_levels = map_two_levels;
    function map_three_levels(factors, outer_pad, inner_pad, factor_pad, offset) {
        var e_3, _b, e_4, _c;
        if (offset === void 0) {
            offset = 0;
        }
        var _a;
        var mapping = new Map();
        var tops = new Map();
        try {
            for (var factors_2 = tslib_1.__values(factors), factors_2_1 = factors_2.next(); !factors_2_1.done; factors_2_1 = factors_2.next()) {
                var _d = tslib_1.__read(factors_2_1.value, 3), f0 = _d[0], f1 = _d[1], f2 = _d[2];
                var top = (_a = tops.get(f0)) !== null && _a !== void 0 ? _a : [];
                tops.set(f0, tslib_1.__spread(top, [[f1, f2]]));
            }
        }
        catch (e_3_1) {
            e_3 = { error: e_3_1 };
        }
        finally {
            try {
                if (factors_2_1 && !factors_2_1.done && (_b = factors_2.return))
                    _b.call(factors_2);
            }
            finally {
                if (e_3)
                    throw e_3.error;
            }
        }
        var suboffset = offset;
        var total_subpad = 0;
        var _loop_2 = function (f0, top) {
            var n = top.length;
            var _b = tslib_1.__read(map_two_levels(top, inner_pad, factor_pad, suboffset), 2), submap = _b[0], subpad = _b[1];
            total_subpad += subpad;
            var subtot = array_1.sum(top.map(function (_b) {
                var _c = tslib_1.__read(_b, 1), f1 = _c[0];
                return submap.get(f1).value;
            }));
            mapping.set(f0, { value: subtot / n, mapping: submap });
            suboffset += n + outer_pad + subpad;
        };
        try {
            for (var tops_2 = tslib_1.__values(tops), tops_2_1 = tops_2.next(); !tops_2_1.done; tops_2_1 = tops_2.next()) {
                var _e = tslib_1.__read(tops_2_1.value, 2), f0 = _e[0], top = _e[1];
                _loop_2(f0, top);
            }
        }
        catch (e_4_1) {
            e_4 = { error: e_4_1 };
        }
        finally {
            try {
                if (tops_2_1 && !tops_2_1.done && (_c = tops_2.return))
                    _c.call(tops_2);
            }
            finally {
                if (e_4)
                    throw e_4.error;
            }
        }
        return [mapping, (tops.size - 1) * outer_pad + total_subpad];
    }
    exports.map_three_levels = map_three_levels;
    var FactorRange = /** @class */ (function (_super) {
        tslib_1.__extends(FactorRange, _super);
        function FactorRange(attrs) {
            return _super.call(this, attrs) || this;
        }
        FactorRange.init_FactorRange = function () {
            this.define({
                factors: [p.Array, []],
                factor_padding: [p.Number, 0],
                subgroup_padding: [p.Number, 0.8],
                group_padding: [p.Number, 1.4],
                range_padding: [p.Number, 0],
                range_padding_units: [p.PaddingUnits, "percent"],
                start: [p.Number],
                end: [p.Number],
            });
            this.internal({
                levels: [p.Number],
                mids: [p.Array, null],
                tops: [p.Array, null],
            });
        };
        Object.defineProperty(FactorRange.prototype, "min", {
            get: function () {
                return this.start;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FactorRange.prototype, "max", {
            get: function () {
                return this.end;
            },
            enumerable: true,
            configurable: true
        });
        FactorRange.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._init(true);
        };
        FactorRange.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.properties.factors.change, function () { return _this.reset(); });
            this.connect(this.properties.factor_padding.change, function () { return _this.reset(); });
            this.connect(this.properties.group_padding.change, function () { return _this.reset(); });
            this.connect(this.properties.subgroup_padding.change, function () { return _this.reset(); });
            this.connect(this.properties.range_padding.change, function () { return _this.reset(); });
            this.connect(this.properties.range_padding_units.change, function () { return _this.reset(); });
        };
        FactorRange.prototype.reset = function () {
            this._init(false);
            this.change.emit();
        };
        FactorRange.prototype._lookup = function (x) {
            switch (x.length) {
                case 1: {
                    var _b = tslib_1.__read(x, 1), f0 = _b[0];
                    var mapping = this._mapping;
                    var y0 = mapping.get(f0);
                    return y0 != null ? y0.value : NaN;
                }
                case 2: {
                    var _c = tslib_1.__read(x, 2), f0 = _c[0], f1 = _c[1];
                    var mapping = this._mapping;
                    var y0 = mapping.get(f0);
                    if (y0 != null) {
                        var y1 = y0.mapping.get(f1);
                        if (y1 != null)
                            return y1.value;
                    }
                    return NaN;
                }
                case 3: {
                    var _d = tslib_1.__read(x, 3), f0 = _d[0], f1 = _d[1], f2 = _d[2];
                    var mapping = this._mapping;
                    var y0 = mapping.get(f0);
                    if (y0 != null) {
                        var y1 = y0.mapping.get(f1);
                        if (y1 != null) {
                            var y2 = y1.mapping.get(f2);
                            if (y2 != null)
                                return y2.value;
                        }
                    }
                    return NaN;
                }
                default:
                    assert_1.unreachable();
            }
        };
        // convert a string factor into a synthetic coordinate
        FactorRange.prototype.synthetic = function (x) {
            if (types_2.isNumber(x))
                return x;
            if (types_2.isString(x))
                return this._lookup([x]);
            var offset = 0;
            var off = x[x.length - 1];
            if (types_2.isNumber(off)) {
                offset = off;
                x = x.slice(0, -1);
            }
            return this._lookup(x) + offset;
        };
        // convert an array of string factors into synthetic coordinates
        FactorRange.prototype.v_synthetic = function (xs) {
            var n = xs.length;
            var array = new types_1.NumberArray(n);
            for (var i = 0; i < n; i++) {
                array[i] = this.synthetic(xs[i]);
            }
            return array;
        };
        FactorRange.prototype._init = function (silent) {
            var _this = this;
            var _b = (function () {
                var e_5, _b, e_6, _c;
                if (array_1.every(_this.factors, types_2.isString)) {
                    var factors = _this.factors;
                    var _d = tslib_1.__read(map_one_level(factors, _this.factor_padding), 2), mapping_2 = _d[0], inside_padding_1 = _d[1];
                    var tops_3 = null;
                    var mids_1 = null;
                    return { levels: 1, mapping: mapping_2, tops: tops_3, mids: mids_1, inside_padding: inside_padding_1 };
                }
                else if (array_1.every(_this.factors, function (x) { return types_2.isArray(x) && x.length == 2 && types_2.isString(x[0]) && types_2.isString(x[1]); })) {
                    var factors = _this.factors;
                    var _e = tslib_1.__read(map_two_levels(factors, _this.group_padding, _this.factor_padding), 2), mapping_3 = _e[0], inside_padding_2 = _e[1];
                    var tops_4 = tslib_1.__spread(mapping_3.keys());
                    var mids_2 = null;
                    return { levels: 2, mapping: mapping_3, tops: tops_4, mids: mids_2, inside_padding: inside_padding_2 };
                }
                else if (array_1.every(_this.factors, function (x) { return types_2.isArray(x) && x.length == 3 && types_2.isString(x[0]) && types_2.isString(x[1]) && types_2.isString(x[2]); })) {
                    var factors = _this.factors;
                    var _f = tslib_1.__read(map_three_levels(factors, _this.group_padding, _this.subgroup_padding, _this.factor_padding), 2), mapping_4 = _f[0], inside_padding_3 = _f[1];
                    var tops_5 = tslib_1.__spread(mapping_4.keys());
                    var mids_3 = [];
                    try {
                        for (var mapping_1 = tslib_1.__values(mapping_4), mapping_1_1 = mapping_1.next(); !mapping_1_1.done; mapping_1_1 = mapping_1.next()) {
                            var _g = tslib_1.__read(mapping_1_1.value, 2), f0 = _g[0], L2 = _g[1];
                            try {
                                for (var _h = (e_6 = void 0, tslib_1.__values(L2.mapping.keys())), _j = _h.next(); !_j.done; _j = _h.next()) {
                                    var f1 = _j.value;
                                    mids_3.push([f0, f1]);
                                }
                            }
                            catch (e_6_1) {
                                e_6 = { error: e_6_1 };
                            }
                            finally {
                                try {
                                    if (_j && !_j.done && (_c = _h.return))
                                        _c.call(_h);
                                }
                                finally {
                                    if (e_6)
                                        throw e_6.error;
                                }
                            }
                        }
                    }
                    catch (e_5_1) {
                        e_5 = { error: e_5_1 };
                    }
                    finally {
                        try {
                            if (mapping_1_1 && !mapping_1_1.done && (_b = mapping_1.return))
                                _b.call(mapping_1);
                        }
                        finally {
                            if (e_5)
                                throw e_5.error;
                        }
                    }
                    return { levels: 3, mapping: mapping_4, tops: tops_5, mids: mids_3, inside_padding: inside_padding_3 };
                }
                else
                    assert_1.unreachable();
            })(), levels = _b.levels, mapping = _b.mapping, tops = _b.tops, mids = _b.mids, inside_padding = _b.inside_padding;
            this._mapping = mapping;
            this.tops = tops;
            this.mids = mids;
            var start = 0;
            var end = this.factors.length + inside_padding;
            if (this.range_padding_units == "percent") {
                var half_span = (end - start) * this.range_padding / 2;
                start -= half_span;
                end += half_span;
            }
            else {
                start -= this.range_padding;
                end += this.range_padding;
            }
            this.setv({ start: start, end: end, levels: levels }, { silent: silent });
            if (this.bounds == "auto")
                this.setv({ bounds: [start, end] }, { silent: true });
        };
        return FactorRange;
    }(range_1.Range));
    exports.FactorRange = FactorRange;
    FactorRange.__name__ = "FactorRange";
    FactorRange.init_FactorRange();
},
/* models/ranges/range.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var Range = /** @class */ (function (_super) {
        tslib_1.__extends(Range, _super);
        function Range(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.have_updated_interactively = false;
            return _this;
        }
        Range.init_Range = function () {
            this.define({
                bounds: [p.Any],
                min_interval: [p.Any],
                max_interval: [p.Any],
            });
            this.internal({
                plots: [p.Array, []],
            });
        };
        Object.defineProperty(Range.prototype, "is_reversed", {
            get: function () {
                return this.start > this.end;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Range.prototype, "is_valid", {
            get: function () {
                return !isNaN(this.min) && !isNaN(this.max);
            },
            enumerable: true,
            configurable: true
        });
        return Range;
    }(model_1.Model));
    exports.Range = Range;
    Range.__name__ = "Range";
    Range.init_Range();
},
/* models/glyphs/utils.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    function generic_line_legend(visuals, ctx, _a, index) {
        var x0 = _a.x0, x1 = _a.x1, y0 = _a.y0, y1 = _a.y1;
        ctx.save();
        ctx.beginPath();
        ctx.moveTo(x0, (y0 + y1) / 2);
        ctx.lineTo(x1, (y0 + y1) / 2);
        if (visuals.line.doit) {
            visuals.line.set_vectorize(ctx, index);
            ctx.stroke();
        }
        ctx.restore();
    }
    exports.generic_line_legend = generic_line_legend;
    function generic_area_legend(visuals, ctx, _a, index) {
        var x0 = _a.x0, x1 = _a.x1, y0 = _a.y0, y1 = _a.y1;
        var w = Math.abs(x1 - x0);
        var dw = w * 0.1;
        var h = Math.abs(y1 - y0);
        var dh = h * 0.1;
        var sx0 = x0 + dw;
        var sx1 = x1 - dw;
        var sy0 = y0 + dh;
        var sy1 = y1 - dh;
        if (visuals.fill.doit) {
            visuals.fill.set_vectorize(ctx, index);
            ctx.fillRect(sx0, sy0, sx1 - sx0, sy1 - sy0);
        }
        if (visuals.hatch != null && visuals.hatch.doit) {
            visuals.hatch.set_vectorize(ctx, index);
            ctx.fillRect(sx0, sy0, sx1 - sx0, sy1 - sy0);
        }
        if (visuals.line && visuals.line.doit) {
            ctx.beginPath();
            ctx.rect(sx0, sy0, sx1 - sx0, sy1 - sy0);
            visuals.line.set_vectorize(ctx, index);
            ctx.stroke();
        }
    }
    exports.generic_area_legend = generic_area_legend;
    function line_interpolation(renderer, geometry, x2, y2, x3, y3) {
        var _a, _b, _c, _d, _e, _f;
        var sx = geometry.sx, sy = geometry.sy;
        var x0, x1;
        var y0, y1;
        if (geometry.type == 'point') {
            // The +/- adjustments here are to dilate the hit point into a virtual "segment" to use below
            _a = tslib_1.__read(renderer.yscale.r_invert(sy - 1, sy + 1), 2), y0 = _a[0], y1 = _a[1];
            _b = tslib_1.__read(renderer.xscale.r_invert(sx - 1, sx + 1), 2), x0 = _b[0], x1 = _b[1];
        }
        else {
            // The +/- adjustments here are to handle cases such as purely horizontal or vertical lines
            if (geometry.direction == 'v') {
                _c = tslib_1.__read(renderer.yscale.r_invert(sy, sy), 2), y0 = _c[0], y1 = _c[1];
                _d = tslib_1.__read([Math.min(x2 - 1, x3 - 1), Math.max(x2 + 1, x3 + 1)], 2), x0 = _d[0], x1 = _d[1];
            }
            else {
                _e = tslib_1.__read(renderer.xscale.r_invert(sx, sx), 2), x0 = _e[0], x1 = _e[1];
                _f = tslib_1.__read([Math.min(y2 - 1, y3 - 1), Math.max(y2 + 1, y3 + 1)], 2), y0 = _f[0], y1 = _f[1];
            }
        }
        var _g = hittest.check_2_segments_intersect(x0, y0, x1, y1, x2, y2, x3, y3), x = _g.x, y = _g.y;
        return [x, y]; // XXX: null is not handled at use sites
    }
    exports.line_interpolation = line_interpolation;
},
/* core/hittest.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function point_in_poly(x, y, px, py) {
        var inside = false;
        var x1 = px[px.length - 1];
        var y1 = py[py.length - 1];
        for (var i = 0; i < px.length; i++) {
            var x2 = px[i];
            var y2 = py[i];
            if ((y1 < y) != (y2 < y)) {
                if ((x1 + (y - y1) / (y2 - y1) * (x2 - x1)) < x)
                    inside = !inside;
            }
            x1 = x2;
            y1 = y2;
        }
        return inside;
    }
    exports.point_in_poly = point_in_poly;
    function point_in_ellipse(x, y, angle, b, a, x0, y0) {
        var A = (Math.pow((Math.cos(angle) / a), 2) + Math.pow((Math.sin(angle) / b), 2));
        var B = 2 * Math.cos(angle) * Math.sin(angle) * (Math.pow((1 / a), 2) - Math.pow((1 / b), 2));
        var C = (Math.pow((Math.cos(angle) / b), 2) + Math.pow((Math.sin(angle) / a), 2));
        var eqn = A * Math.pow((x - x0), 2) + B * (x - x0) * (y - y0) + C * Math.pow((y - y0), 2);
        var inside = eqn <= 1;
        return inside;
    }
    exports.point_in_ellipse = point_in_ellipse;
    function dist_2_pts(p0, p1) {
        return Math.pow((p0.x - p1.x), 2) + Math.pow((p0.y - p1.y), 2);
    }
    exports.dist_2_pts = dist_2_pts;
    function dist_to_segment_squared(p, v, w) {
        var l2 = dist_2_pts(v, w);
        if (l2 == 0)
            return dist_2_pts(p, v);
        var t = ((p.x - v.x) * (w.x - v.x) + (p.y - v.y) * (w.y - v.y)) / l2;
        if (t < 0)
            return dist_2_pts(p, v);
        if (t > 1)
            return dist_2_pts(p, w);
        var q = { x: v.x + t * (w.x - v.x), y: v.y + t * (w.y - v.y) };
        return dist_2_pts(p, q);
    }
    exports.dist_to_segment_squared = dist_to_segment_squared;
    function dist_to_segment(p, v, w) {
        return Math.sqrt(dist_to_segment_squared(p, v, w));
    }
    exports.dist_to_segment = dist_to_segment;
    function check_2_segments_intersect(l0_x0, l0_y0, l0_x1, l0_y1, l1_x0, l1_y0, l1_x1, l1_y1) {
        /*
         *  Check if 2 segments (l0 and l1) intersect. Returns a structure with
         *  the following attributes:
         *   * hit (boolean): whether the 2 segments intersect
         *   * x (float): x coordinate of the intersection point
         *   * y (float): y coordinate of the intersection point
         */
        var den = ((l1_y1 - l1_y0) * (l0_x1 - l0_x0)) - ((l1_x1 - l1_x0) * (l0_y1 - l0_y0));
        if (den == 0) {
            return { hit: false, x: null, y: null };
        }
        else {
            var a = l0_y0 - l1_y0;
            var b = l0_x0 - l1_x0;
            var num1 = ((l1_x1 - l1_x0) * a) - ((l1_y1 - l1_y0) * b);
            var num2 = ((l0_x1 - l0_x0) * a) - ((l0_y1 - l0_y0) * b);
            a = num1 / den;
            b = num2 / den;
            var x = l0_x0 + (a * (l0_x1 - l0_x0));
            var y = l0_y0 + (a * (l0_y1 - l0_y0));
            return { hit: (a > 0 && a < 1) && (b > 0 && b < 1), x: x, y: y };
        }
    }
    exports.check_2_segments_intersect = check_2_segments_intersect;
},
/* models/glyphs/webgl/line.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var utils_1 = require(218) /* ./utils */;
    var base_1 = require(222) /* ./base */;
    var line_vert_1 = require(223) /* ./line.vert */;
    var line_frag_1 = require(224) /* ./line.frag */;
    var color_1 = require(137) /* ../../../core/util/color */;
    var DashAtlas = /** @class */ (function () {
        function DashAtlas(gl) {
            this._atlas = new Map();
            this._width = 256;
            this._height = 256;
            // Init texture
            this.tex = new utils_1.Texture2d(gl);
            this.tex.set_wrapping(gl.REPEAT, gl.REPEAT);
            this.tex.set_interpolation(gl.NEAREST, gl.NEAREST);
            this.tex.set_size([this._width, this._height], gl.RGBA);
            this.tex.set_data([0, 0], [this._width, this._height], new Uint8Array(4 * this._width * this._height));
            // Init with solid line (index 0 is reserved for this)
            this.get_atlas_data([1]);
        }
        DashAtlas.prototype.get_atlas_data = function (pattern) {
            var key = pattern.join("-");
            var atlas_data = this._atlas.get(key);
            if (atlas_data == null) {
                var _a = tslib_1.__read(this.make_pattern(pattern), 2), data = _a[0], period = _a[1];
                var index = this._atlas.size;
                this.tex.set_data([0, index], [this._width, 1], new Uint8Array(data.map(function (x) { return x + 10; })));
                atlas_data = [index / this._height, period];
                this._atlas.set(key, atlas_data);
            }
            return atlas_data;
        };
        DashAtlas.prototype.make_pattern = function (pattern) {
            var e_1, _a;
            // A pattern is defined as on/off sequence of segments
            // It must be a multiple of 2
            if (pattern.length > 1 && pattern.length % 2) {
                pattern = pattern.concat(pattern);
            }
            // Period is sum of elements
            var period = 0;
            try {
                for (var pattern_1 = tslib_1.__values(pattern), pattern_1_1 = pattern_1.next(); !pattern_1_1.done; pattern_1_1 = pattern_1.next()) {
                    var v = pattern_1_1.value;
                    period += v;
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (pattern_1_1 && !pattern_1_1.done && (_a = pattern_1.return))
                        _a.call(pattern_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            // Find all start and end of on-segment only
            var C = [];
            var c = 0;
            for (var i = 0, end = pattern.length + 2; i < end; i += 2) {
                var a = Math.max(0.0001, pattern[i % pattern.length]);
                var b = Math.max(0.0001, pattern[(i + 1) % pattern.length]);
                C.push(c, c + a);
                c += a + b;
            }
            // Build pattern
            var n = this._width;
            var Z = new Float32Array(n * 4);
            for (var i = 0, end = n; i < end; i++) {
                var dash_end = void 0, dash_start = void 0, dash_type = void 0;
                var x = (period * i) / (n - 1);
                // get index at min - index = np.argmin(abs(C-(x)))
                var index = 0;
                var val_at_index = 1e16;
                for (var j = 0, endj = C.length; j < endj; j++) {
                    var val = Math.abs(C[j] - x);
                    if (val < val_at_index) {
                        index = j;
                        val_at_index = val;
                    }
                }
                if ((index % 2) === 0) {
                    dash_type = (x <= C[index]) ? +1 : 0;
                    dash_start = C[index];
                    dash_end = C[index + 1];
                }
                else {
                    dash_type = (x > C[index]) ? -1 : 0;
                    dash_start = C[index - 1];
                    dash_end = C[index];
                }
                Z[(i * 4) + 0] = C[index];
                Z[(i * 4) + 1] = dash_type;
                Z[(i * 4) + 2] = dash_start;
                Z[(i * 4) + 3] = dash_end;
            }
            return [Z, period];
        };
        return DashAtlas;
    }());
    DashAtlas.__name__ = "DashAtlas";
    var joins = { miter: 0, round: 1, bevel: 2 };
    var caps = {
        '': 0, none: 0, '.': 0,
        round: 1, ')': 1, '(': 1, o: 1,
        'triangle in': 2, '<': 2,
        'triangle out': 3, '>': 3,
        square: 4, '[': 4, ']': 4, '=': 4,
        butt: 5, '|': 5,
    };
    var LineGL = /** @class */ (function (_super) {
        tslib_1.__extends(LineGL, _super);
        function LineGL() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        LineGL.prototype.init = function () {
            var gl = this.gl;
            this._scale_aspect = 0; // keep track, so we know when we need to update segment data
            var vert = line_vert_1.vertex_shader;
            var frag = line_frag_1.fragment_shader;
            // The program
            this.prog = new utils_1.Program(gl);
            this.prog.set_shaders(vert, frag);
            this.index_buffer = new utils_1.IndexBuffer(gl);
            // Buffers
            this.vbo_position = new utils_1.VertexBuffer(gl);
            this.vbo_tangents = new utils_1.VertexBuffer(gl);
            this.vbo_segment = new utils_1.VertexBuffer(gl);
            this.vbo_angles = new utils_1.VertexBuffer(gl);
            this.vbo_texcoord = new utils_1.VertexBuffer(gl);
            // Dash atlas
            this.dash_atlas = new DashAtlas(gl);
        };
        LineGL.prototype.draw = function (indices, mainGlyph, trans) {
            var mainGlGlyph = mainGlyph.glglyph;
            if (mainGlGlyph.data_changed) {
                mainGlGlyph._set_data();
                mainGlGlyph.data_changed = false;
            }
            if (this.visuals_changed) {
                this._set_visuals();
                this.visuals_changed = false;
            }
            mainGlGlyph._update_scale(1, 1);
            this._scale_aspect = 1;
            // Select buffers from main glyph
            // (which may be this glyph but maybe not if this is a (non)selection glyph)
            this.prog.set_attribute('a_position', 'vec2', mainGlGlyph.vbo_position);
            this.prog.set_attribute('a_tangents', 'vec4', mainGlGlyph.vbo_tangents);
            this.prog.set_attribute('a_segment', 'vec2', mainGlGlyph.vbo_segment);
            this.prog.set_attribute('a_angles', 'vec2', mainGlGlyph.vbo_angles);
            this.prog.set_attribute('a_texcoord', 'vec2', mainGlGlyph.vbo_texcoord);
            //
            this.prog.set_uniform('u_length', 'float', [mainGlGlyph.cumsum]);
            this.prog.set_texture('u_dash_atlas', this.dash_atlas.tex);
            // Handle transformation to device coordinates
            this.prog.set_uniform('u_pixel_ratio', 'float', [trans.pixel_ratio]);
            this.prog.set_uniform('u_canvas_size', 'vec2', [trans.width, trans.height]);
            this.prog.set_uniform('u_scale_aspect', 'vec2', [1, 1]);
            this.prog.set_uniform('u_scale_length', 'float', [Math.sqrt(2)]);
            this.I_triangles = mainGlGlyph.I_triangles;
            if (this.I_triangles.length < 65535) {
                // Data is small enough to draw in one pass
                this.index_buffer.set_size(this.I_triangles.length * 2);
                this.index_buffer.set_data(0, new Uint16Array(this.I_triangles));
                this.prog.draw(this.gl.TRIANGLES, this.index_buffer);
                // @prog.draw(@gl.LINE_STRIP, @index_buffer)  # Use this to draw the line skeleton
            }
            else {
                // Work around the limit that the indexbuffer must be uint16. We draw in chunks.
                // First collect indices in chunks
                indices = Array.from(this.I_triangles);
                var nvertices = this.I_triangles.length;
                var chunksize = 64008; // 65536 max. 64008 is divisible by 12
                var chunks = [];
                for (var i = 0, end = Math.ceil(nvertices / chunksize); i < end; i++) {
                    chunks.push([]);
                }
                for (var i = 0, end = indices.length; i < end; i++) {
                    var uint16_index = indices[i] % chunksize;
                    var chunk = Math.floor(indices[i] / chunksize);
                    chunks[chunk].push(uint16_index);
                }
                // Then draw each chunk
                for (var chunk = 0, end = chunks.length; chunk < end; chunk++) {
                    var these_indices = new Uint16Array(chunks[chunk]);
                    var offset = chunk * chunksize * 4;
                    if (these_indices.length === 0) {
                        continue;
                    }
                    this.prog.set_attribute('a_position', 'vec2', mainGlGlyph.vbo_position, 0, offset * 2);
                    this.prog.set_attribute('a_tangents', 'vec4', mainGlGlyph.vbo_tangents, 0, offset * 4);
                    this.prog.set_attribute('a_segment', 'vec2', mainGlGlyph.vbo_segment, 0, offset * 2);
                    this.prog.set_attribute('a_angles', 'vec2', mainGlGlyph.vbo_angles, 0, offset * 2);
                    this.prog.set_attribute('a_texcoord', 'vec2', mainGlGlyph.vbo_texcoord, 0, offset * 2);
                    // The actual drawing
                    this.index_buffer.set_size(these_indices.length * 2);
                    this.index_buffer.set_data(0, these_indices);
                    this.prog.draw(this.gl.TRIANGLES, this.index_buffer);
                }
            }
        };
        LineGL.prototype._set_data = function () {
            this._bake();
            this.vbo_position.set_size(this.V_position.length * 4);
            this.vbo_position.set_data(0, this.V_position);
            this.vbo_tangents.set_size(this.V_tangents.length * 4);
            this.vbo_tangents.set_data(0, this.V_tangents);
            this.vbo_angles.set_size(this.V_angles.length * 4);
            this.vbo_angles.set_data(0, this.V_angles);
            this.vbo_texcoord.set_size(this.V_texcoord.length * 4);
            this.vbo_texcoord.set_data(0, this.V_texcoord);
        };
        LineGL.prototype._set_visuals = function () {
            var _a;
            var color = color_1.color2rgba(this.glyph.visuals.line.line_color.value(), this.glyph.visuals.line.line_alpha.value());
            var cap = caps[this.glyph.visuals.line.line_cap.value()];
            var join = joins[this.glyph.visuals.line.line_join.value()];
            this.prog.set_uniform('u_color', 'vec4', color);
            this.prog.set_uniform('u_linewidth', 'float', [this.glyph.visuals.line.line_width.value()]);
            this.prog.set_uniform('u_antialias', 'float', [0.9]); // Smaller aa-region to obtain crisper images
            this.prog.set_uniform('u_linecaps', 'vec2', [cap, cap]);
            this.prog.set_uniform('u_linejoin', 'float', [join]);
            this.prog.set_uniform('u_miter_limit', 'float', [10.0]); // 10 should be a good value
            // https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-miterlimit
            var dash_pattern = this.glyph.visuals.line.line_dash.value();
            var dash_index = 0;
            var dash_period = 1;
            if (dash_pattern.length) {
                _a = tslib_1.__read(this.dash_atlas.get_atlas_data(dash_pattern), 2), dash_index = _a[0], dash_period = _a[1];
            }
            this.prog.set_uniform('u_dash_index', 'float', [dash_index]); // 0 means solid line
            this.prog.set_uniform('u_dash_phase', 'float', [this.glyph.visuals.line.line_dash_offset.value()]);
            this.prog.set_uniform('u_dash_period', 'float', [dash_period]);
            this.prog.set_uniform('u_dash_caps', 'vec2', [cap, cap]);
            this.prog.set_uniform('u_closed', 'float', [0]); // We dont do closed lines
        };
        LineGL.prototype._bake = function () {
            // This is what you get if you port 50 lines of numpy code to JS.
            // V_segment is handled in another method, because it depends on the aspect
            // ratio of the scale (The original paper/code assumed isotropic scaling).
            //
            // Buffer dtype from the Python implementation:
            //
            // self.vtype = np.dtype( [('a_position', 'f4', 2),
            //                         ('a_segment',  'f4', 2),
            //                         ('a_angles',   'f4', 2),
            //                         ('a_tangents', 'f4', 4),
            //                         ('a_texcoord', 'f4', 2) ])
            // Init array of implicit shape nx2
            var I, T, V_angles2, V_position2, V_tangents2, V_texcoord2, Vp, Vt;
            var n = this.nvertices;
            var sx = this.glyph.sx;
            var sy = this.glyph.sy;
            // Init vertex data
            var V_position = (Vp = new Float32Array(n * 2));
            //V_segment = new Float32Array(n*2)  # Done later
            var V_angles = new Float32Array(n * 2);
            var V_tangents = (Vt = new Float32Array(n * 4)); // mind the 4!
            // Position
            for (var i = 0, end = n; i < end; i++) {
                V_position[(i * 2) + 0] = sx[i];
                V_position[(i * 2) + 1] = sy[i];
            }
            // Tangents & norms (need tangents to calculate segments based on scale)
            this.tangents = (T = new Float32Array((n * 2) - 2));
            for (var i = 0, end = n - 1; i < end; i++) {
                T[(i * 2) + 0] = Vp[((i + 1) * 2) + 0] - Vp[(i * 2) + 0];
                T[(i * 2) + 1] = Vp[((i + 1) * 2) + 1] - Vp[(i * 2) + 1];
            }
            for (var i = 0, end = n - 1; i < end; i++) {
                // V['a_tangents'][+1:, :2] = T
                V_tangents[((i + 1) * 4) + 0] = T[(i * 2) + 0];
                V_tangents[((i + 1) * 4) + 1] = T[(i * 2) + 1];
                // V['a_tangents'][:-1, 2:] = T
                V_tangents[(i * 4) + 2] = T[(i * 2) + 0];
                V_tangents[(i * 4) + 3] = T[(i * 2) + 1];
            }
            // V['a_tangents'][0  , :2] = T[0]
            V_tangents[(0 * 4) + 0] = T[0];
            V_tangents[(0 * 4) + 1] = T[1];
            // V['a_tangents'][ -1, 2:] = T[-1]
            V_tangents[((n - 1) * 4) + 2] = T[((n - 2) * 2) + 0];
            V_tangents[((n - 1) * 4) + 3] = T[((n - 2) * 2) + 1];
            // Angles
            var A = new Float32Array(n);
            for (var i = 0, end = n; i < end; i++) {
                A[i] = Math.atan2((Vt[(i * 4) + 0] * Vt[(i * 4) + 3]) - (Vt[(i * 4) + 1] * Vt[(i * 4) + 2]), (Vt[(i * 4) + 0] * Vt[(i * 4) + 2]) + (Vt[(i * 4) + 1] * Vt[(i * 4) + 3]));
            }
            for (var i = 0, end = n - 1; i < end; i++) {
                V_angles[(i * 2) + 0] = A[i];
                V_angles[(i * 2) + 1] = A[i + 1];
            }
            // Step 1: A -- B -- C  =>  A -- B, B' -- C
            // Repeat our array 4 times
            var m = (4 * n) - 4;
            this.V_position = (V_position2 = new Float32Array(m * 2));
            this.V_angles = (V_angles2 = new Float32Array(m * 2));
            this.V_tangents = (V_tangents2 = new Float32Array(m * 4)); // mind the 4!
            this.V_texcoord = (V_texcoord2 = new Float32Array(m * 2));
            var o = 2;
            //
            // Arg, we really need an ndarray thing in JS :/
            for (var i = 0, end = n; i < end; i++) { // all nodes on the line
                for (var j = 0; j < 4; j++) { // the four quad vertices
                    for (var k = 0; k < 2; k++) { // xy
                        V_position2[((((i * 4) + j) - o) * 2) + k] = V_position[(i * 2) + k];
                        V_angles2[(((i * 4) + j) * 2) + k] = V_angles[(i * 2) + k];
                    } // no offset
                    for (var k = 0; k < 4; k++) {
                        V_tangents2[((((i * 4) + j) - o) * 4) + k] = V_tangents[(i * 4) + k];
                    }
                }
            }
            for (var i = 0, end = n; i < end; i++) {
                V_texcoord2[(((i * 4) + 0) * 2) + 0] = -1;
                V_texcoord2[(((i * 4) + 1) * 2) + 0] = -1;
                V_texcoord2[(((i * 4) + 2) * 2) + 0] = +1;
                V_texcoord2[(((i * 4) + 3) * 2) + 0] = +1;
                //
                V_texcoord2[(((i * 4) + 0) * 2) + 1] = -1;
                V_texcoord2[(((i * 4) + 1) * 2) + 1] = +1;
                V_texcoord2[(((i * 4) + 2) * 2) + 1] = -1;
                V_texcoord2[(((i * 4) + 3) * 2) + 1] = +1;
            }
            // Indices
            //I = np.resize( np.array([0,1,2,1,2,3], dtype=np.uint32), (n-1)*(2*3))
            //I += np.repeat( 4*np.arange(n-1), 6)
            var ni = (n - 1) * 6;
            this.I_triangles = (I = new Uint32Array(ni));
            // Order of indices is such that drawing as line_strip reveals the line skeleton
            // Might have implications on culling, if we ever turn that on.
            // Order in paper was: 0 1 2 1 2 3
            for (var i = 0, end = n; i < end; i++) {
                I[(i * 6) + 0] = 0 + (4 * i);
                I[(i * 6) + 1] = 1 + (4 * i);
                I[(i * 6) + 2] = 3 + (4 * i);
                I[(i * 6) + 3] = 2 + (4 * i);
                I[(i * 6) + 4] = 0 + (4 * i);
                I[(i * 6) + 5] = 3 + (4 * i);
            }
        };
        LineGL.prototype._update_scale = function (sx, sy) {
            // Update segment data and cumsum so the length along the line has the
            // scale aspect ratio in it. In the vertex shader we multiply with the
            // "isotropic part" of the scale.
            var V_segment2;
            var n = this.nvertices;
            var m = (4 * n) - 4;
            // Prepare arrays
            var T = this.tangents;
            var N = new Float32Array(n - 1);
            var V_segment = new Float32Array(n * 2); // Elements are initialized with 0
            this.V_segment = (V_segment2 = new Float32Array(m * 2));
            // Calculate vector lengths - with scale aspect ratio taken into account
            for (var i = 0, end = n - 1; i < end; i++) {
                N[i] = Math.sqrt(Math.pow((T[(i * 2) + 0] * sx), 2) + Math.pow((T[(i * 2) + 1] * sy), 2));
            }
            // Calculate Segments
            var cumsum = 0;
            for (var i = 0, end = n - 1; i < end; i++) {
                cumsum += N[i];
                V_segment[((i + 1) * 2) + 0] = cumsum;
                V_segment[(i * 2) + 1] = cumsum;
            }
            // Upscale (same loop as in _bake())
            for (var i = 0, end = n; i < end; i++) {
                for (var j = 0; j < 4; j++) {
                    for (var k = 0; k < 2; k++) {
                        V_segment2[(((i * 4) + j) * 2) + k] = V_segment[(i * 2) + k];
                    }
                }
            }
            // Update
            this.cumsum = cumsum; // L[-1] in Nico's code
            this.vbo_segment.set_size(this.V_segment.length * 4);
            this.vbo_segment.set_data(0, this.V_segment);
        };
        return LineGL;
    }(base_1.BaseGLGlyph));
    exports.LineGL = LineGL;
    LineGL.__name__ = "LineGL";
},
/* models/glyphs/webgl/utils/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var program_1 = require(219) /* ./program */;
    exports.Program = program_1.Program;
    var texture_1 = require(221) /* ./texture */;
    exports.Texture2d = texture_1.Texture2d;
    var buffer_1 = require(220) /* ./buffer */;
    exports.IndexBuffer = buffer_1.IndexBuffer;
    exports.VertexBuffer = buffer_1.VertexBuffer;
},
/* models/glyphs/webgl/utils/program.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var buffer_1 = require(220) /* ./buffer */;
    var Program = /** @class */ (function () {
        function Program(gl) {
            this.gl = gl;
            this.UTYPEMAP = {
                float: "uniform1fv",
                vec2: "uniform2fv",
                vec3: "uniform3fv",
                vec4: "uniform4fv",
                int: "uniform1iv",
                ivec2: "uniform2iv",
                ivec3: "uniform3iv",
                ivec4: "uniform4iv",
                bool: "uniform1iv",
                bvec2: "uniform2iv",
                bvec3: "uniform3iv",
                bvec4: "uniform4iv",
                mat2: "uniformMatrix2fv",
                mat3: "uniformMatrix3fv",
                mat4: "uniformMatrix4fv",
                sampler1D: "uniform1i",
                sampler2D: "uniform1i",
                sampler3D: "uniform1i",
            };
            this.ATYPEMAP = {
                float: "vertexAttrib1f",
                vec2: "vertexAttrib2f",
                vec3: "vertexAttrib3f",
                vec4: "vertexAttrib4f",
            };
            this.ATYPEINFO = {
                float: [1, 5126],
                vec2: [2, 5126],
                vec3: [3, 5126],
                vec4: [4, 5126],
            };
            this._linked = false;
            this._validated = false;
            this._unset_variables = new Set();
            this._known_invalid = new Set();
            this._locations = new Map();
            this._samplers = new Map();
            this._attributes = new Map();
            this.handle = this.gl.createProgram();
        }
        Program.prototype.delete = function () {
            this.gl.deleteProgram(this.handle);
        };
        Program.prototype.activate = function () {
            this.gl.useProgram(this.handle);
        };
        Program.prototype.deactivate = function () {
            this.gl.useProgram(0);
        };
        Program.prototype.set_shaders = function (vert, frag) {
            var e_1, _b;
            // Set GLSL code for the vertex and fragment shader.
            //
            // This function takes care of setting the shading code and
            // compiling+linking it into a working program object that is ready
            // to use.
            //
            // Parameters
            // ----------
            // vert : str
            //     GLSL code for the vertex shader.
            // frag : str
            //     GLSL code for the fragment shader.
            var gl = this.gl;
            this._linked = false;
            var vert_handle = gl.createShader(gl.VERTEX_SHADER);
            var frag_handle = gl.createShader(gl.FRAGMENT_SHADER);
            var tmp = [
                [vert, vert_handle, "vertex"],
                [frag, frag_handle, "fragment"],
            ];
            try {
                for (var tmp_1 = tslib_1.__values(tmp), tmp_1_1 = tmp_1.next(); !tmp_1_1.done; tmp_1_1 = tmp_1.next()) {
                    var _c = tslib_1.__read(tmp_1_1.value, 3), code = _c[0], handle = _c[1], type = _c[2];
                    gl.shaderSource(handle, code);
                    gl.compileShader(handle);
                    var status = gl.getShaderParameter(handle, gl.COMPILE_STATUS);
                    if (!status) {
                        var errors = gl.getShaderInfoLog(handle);
                        throw new Error("errors in " + type + " shader:\n" + errors);
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (tmp_1_1 && !tmp_1_1.done && (_b = tmp_1.return))
                        _b.call(tmp_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            gl.attachShader(this.handle, vert_handle);
            gl.attachShader(this.handle, frag_handle);
            gl.linkProgram(this.handle);
            if (!gl.getProgramParameter(this.handle, gl.LINK_STATUS)) {
                var logs = gl.getProgramInfoLog(this.handle);
                throw new Error("Program link error:\n" + logs);
            }
            this._unset_variables = this._get_active_attributes_and_uniforms();
            gl.detachShader(this.handle, vert_handle);
            gl.detachShader(this.handle, frag_handle);
            gl.deleteShader(vert_handle);
            gl.deleteShader(frag_handle);
            this._known_invalid.clear();
            this._linked = true;
        };
        Program.prototype._get_active_attributes_and_uniforms = function () {
            var e_2, _b, e_3, _c, e_4, _d;
            // Retrieve active attributes and uniforms to be able to check that
            // all uniforms/attributes are set by the user.
            var gl = this.gl;
            this._locations.clear();
            var regex = new RegExp("(\\w+)\\s*(\\[(\\d+)\\])\\s*");
            var cu = gl.getProgramParameter(this.handle, gl.ACTIVE_UNIFORMS);
            var ca = gl.getProgramParameter(this.handle, gl.ACTIVE_ATTRIBUTES);
            var attributes = [];
            var uniforms = [];
            var stub5_seq = [
                [attributes, ca, gl.getActiveAttrib, gl.getAttribLocation],
                [uniforms, cu, gl.getActiveUniform, gl.getUniformLocation],
            ];
            try {
                for (var stub5_seq_1 = tslib_1.__values(stub5_seq), stub5_seq_1_1 = stub5_seq_1.next(); !stub5_seq_1_1.done; stub5_seq_1_1 = stub5_seq_1.next()) {
                    var _e = tslib_1.__read(stub5_seq_1_1.value, 4), container = _e[0], count = _e[1], getActive = _e[2], getLocation = _e[3];
                    for (var i = 0; i < count; i += 1) {
                        var info = getActive.call(gl, this.handle, i);
                        var name = info.name;
                        var m = name.match(regex);
                        if (m != null) {
                            var name_1 = m[1];
                            for (var j = 0; j < info.size; j += 1) {
                                container.push([name_1 + "[" + j + "]", info.type]);
                            }
                        }
                        else {
                            container.push([name, info.type]);
                        }
                        this._locations.set(name, getLocation.call(gl, this.handle, name));
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (stub5_seq_1_1 && !stub5_seq_1_1.done && (_b = stub5_seq_1.return))
                        _b.call(stub5_seq_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            var attrs_and_uniforms = new Set();
            try {
                for (var attributes_1 = tslib_1.__values(attributes), attributes_1_1 = attributes_1.next(); !attributes_1_1.done; attributes_1_1 = attributes_1.next()) {
                    var _f = tslib_1.__read(attributes_1_1.value, 1), name = _f[0];
                    attrs_and_uniforms.add(name);
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (attributes_1_1 && !attributes_1_1.done && (_c = attributes_1.return))
                        _c.call(attributes_1);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            try {
                for (var uniforms_1 = tslib_1.__values(uniforms), uniforms_1_1 = uniforms_1.next(); !uniforms_1_1.done; uniforms_1_1 = uniforms_1.next()) {
                    var _g = tslib_1.__read(uniforms_1_1.value, 1), name = _g[0];
                    attrs_and_uniforms.add(name);
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (uniforms_1_1 && !uniforms_1_1.done && (_d = uniforms_1.return))
                        _d.call(uniforms_1);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            return attrs_and_uniforms;
        };
        Program.prototype.set_texture = function (name, value) {
            var _a;
            // Set a texture sampler.
            //
            // A texture is a 2 dimensional grid of colors/intensities that
            // can be applied to a face (or used for other means by providing
            // a regular grid of data).
            //
            // Parameters
            // ----------
            // name : str
            //     The name by which the texture is known in the GLSL code.
            // value : Texture2d
            //     The Texture2d object to bind.
            if (!this._linked) {
                throw new Error("Cannot set uniform when program has no code");
            }
            var handle = (_a = this._locations.get(name)) !== null && _a !== void 0 ? _a : -1;
            if (handle < 0) {
                if (!this._known_invalid.has(name)) {
                    this._known_invalid.add(name);
                    console.log("\"Variable " + name + " is not an active texture");
                }
                return;
            }
            if (this._unset_variables.has(name)) {
                this._unset_variables.delete(name);
            }
            this.activate();
            if (true) {
                var unit = this._samplers.size;
                if (this._samplers.has(name)) {
                    unit = this._samplers.get(name)[2];
                }
                this._samplers.set(name, [value._target, value.handle, unit]);
                this.gl.uniform1i(handle, unit);
            }
        };
        Program.prototype.set_uniform = function (name, type_, value) {
            var _a;
            // Set a uniform value.
            //
            // A uniform is a value that is global to both the vertex and
            // fragment shader.
            //
            // Parameters
            // ----------
            // name : str
            //     The name by which the uniform is known in the GLSL code.
            // type_ : str
            //     The type of the uniform, e.g. 'float', 'vec2', etc.
            // value : list of scalars
            //     The value for the uniform. Should be a list even for type float.
            if (!this._linked) {
                throw new Error("Cannot set uniform when program has no code");
            }
            var handle = (_a = this._locations.get(name)) !== null && _a !== void 0 ? _a : -1;
            if (handle < 0) {
                if (!this._known_invalid.has(name)) {
                    this._known_invalid.add(name);
                    console.log("Variable " + name + " is not an active uniform");
                }
                return;
            }
            if (this._unset_variables.has(name)) {
                this._unset_variables.delete(name);
            }
            var count = 1;
            if (!type_.startsWith("mat")) {
                var a_type = type_ == "int" || type_ == "bool" ? "float" : type_.replace(/^ib/, "");
                count = Math.floor(value.length / (this.ATYPEINFO[a_type][0]));
            }
            if (count > 1) {
                for (var j = 0; j < count; j += 1) {
                    if (this._unset_variables.has(name + "[" + j + "]")) {
                        var name_ = name + "[" + j + "]";
                        if (this._unset_variables.has(name_)) {
                            this._unset_variables.delete(name_);
                        }
                    }
                }
            }
            var funcname = this.UTYPEMAP[type_];
            this.activate();
            if (type_.startsWith("mat")) {
                this.gl[funcname](handle, false, value);
            }
            else {
                this.gl[funcname](handle, value);
            }
        };
        Program.prototype.set_attribute = function (name, type_, value, stride, offset) {
            if (stride === void 0) {
                stride = 0;
            }
            if (offset === void 0) {
                offset = 0;
            }
            var _a;
            // Set an attribute value.
            //
            // An attribute represents per-vertex data and can only be used
            // in the vertex shader.
            //
            // Parameters
            // ----------
            // name : str
            //     The name by which the attribute is known in the GLSL code.
            // type_ : str
            //     The type of the attribute, e.g. 'float', 'vec2', etc.
            // value : VertexBuffer, array
            //     If value is a VertexBuffer, it is used (with stride and offset)
            //     for the vertex data. If value is an array, its used to set
            //     the value of all vertices (similar to a uniform).
            // stide : int, default 0
            //     The stride to "sample" the vertex data inside the buffer. Unless
            //     multiple vertex data are packed into a single buffer, this should
            //     be zero.
            // offset : int, default 0
            //     The offset to "sample" the vertex data inside the buffer. Unless
            //     multiple vertex data are packed into a single buffer, or only
            //     a part of the data must be used, this should probably be zero.
            if (!this._linked) {
                throw new Error("Cannot set attribute when program has no code");
            }
            var handle = (_a = this._locations.get(name)) !== null && _a !== void 0 ? _a : -1;
            if (handle < 0) {
                if (!this._known_invalid.has(name)) {
                    this._known_invalid.add(name);
                    if (value instanceof buffer_1.VertexBuffer && offset > 0) {
                    }
                    else {
                        console.log("Variable " + name + " is not an active attribute");
                    }
                }
                return;
            }
            if (this._unset_variables.has(name)) {
                this._unset_variables.delete(name);
            }
            this.activate();
            if (!(value instanceof buffer_1.VertexBuffer)) {
                var funcname = this.ATYPEMAP[type_];
                this._attributes.set(name, [null, handle, funcname, value]);
            }
            else {
                var _b = tslib_1.__read(this.ATYPEINFO[type_], 2), size = _b[0], gtype = _b[1];
                var funcname = "vertexAttribPointer";
                var args = [size, gtype, false, stride, offset];
                this._attributes.set(name, [value.handle, handle, funcname, args]);
            }
        };
        Program.prototype._pre_draw = function () {
            var e_5, _b, e_6, _c;
            this.activate();
            try {
                for (var _d = tslib_1.__values(this._samplers.values()), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var _f = tslib_1.__read(_e.value, 3), tex_target = _f[0], tex_handle = _f[1], unit = _f[2];
                    this.gl.activeTexture(this.gl.TEXTURE0 + unit);
                    this.gl.bindTexture(tex_target, tex_handle);
                }
            }
            catch (e_5_1) {
                e_5 = { error: e_5_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_b = _d.return))
                        _b.call(_d);
                }
                finally {
                    if (e_5)
                        throw e_5.error;
                }
            }
            try {
                for (var _g = tslib_1.__values(this._attributes.values()), _h = _g.next(); !_h.done; _h = _g.next()) {
                    var _j = tslib_1.__read(_h.value, 4), vbo_handle = _j[0], attr_handle = _j[1], funcname = _j[2], args = _j[3];
                    if (vbo_handle != null) {
                        this.gl.bindBuffer(this.gl.ARRAY_BUFFER, vbo_handle);
                        this.gl.enableVertexAttribArray(attr_handle);
                        this.gl[funcname].apply(this.gl, tslib_1.__spread([attr_handle], args));
                    }
                    else {
                        this.gl.bindBuffer(this.gl.ARRAY_BUFFER, null);
                        this.gl.disableVertexAttribArray(attr_handle);
                        this.gl[funcname].apply(this.gl, tslib_1.__spread([attr_handle], args));
                    }
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (_h && !_h.done && (_c = _g.return))
                        _c.call(_g);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
            if (!this._validated) {
                this._validated = true;
                this._validate();
            }
        };
        Program.prototype._validate = function () {
            if (this._unset_variables.size) {
                console.log("Program has unset variables: " + this._unset_variables);
            }
            this.gl.validateProgram(this.handle);
            if (!this.gl.getProgramParameter(this.handle, this.gl.VALIDATE_STATUS)) {
                console.log(this.gl.getProgramInfoLog(this.handle));
                throw new Error("Program validation error");
            }
        };
        Program.prototype.draw = function (mode, selection) {
            // Draw the current visualization defined by the program.
            //
            // Parameters
            // ----------
            // mode : GL enum
            //     Can be POINTS, LINES, LINE_LOOP, LINE_STRIP, LINE_FAN, TRIANGLES
            // selection : 2-element tuple or IndexBuffer
            //     The selection to draw, specified either as (first, count) or an
            //     IndexBuffer object.
            if (!this._linked) {
                throw new Error("Cannot draw program if code has not been set");
            }
            if (selection instanceof buffer_1.IndexBuffer) {
                this._pre_draw();
                selection.activate();
                var count = selection.buffer_size / 2;
                var gtype = this.gl.UNSIGNED_SHORT;
                this.gl.drawElements(mode, count, gtype, 0);
                selection.deactivate();
            }
            else {
                var _b = tslib_1.__read(selection, 2), first = _b[0], count = _b[1];
                if (count != 0) {
                    this._pre_draw();
                    this.gl.drawArrays(mode, first, count);
                }
            }
        };
        return Program;
    }());
    exports.Program = Program;
    Program.__name__ = "Program";
},
/* models/glyphs/webgl/utils/buffer.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var Buffer = /** @class */ (function () {
        function Buffer(gl) {
            this.gl = gl;
            this._usage = 35048;
            this.buffer_size = 0;
            this.handle = this.gl.createBuffer();
        }
        Buffer.prototype.delete = function () {
            this.gl.deleteBuffer(this.handle);
        };
        Buffer.prototype.activate = function () {
            this.gl.bindBuffer(this._target, this.handle);
        };
        Buffer.prototype.deactivate = function () {
            this.gl.bindBuffer(this._target, null);
        };
        Buffer.prototype.set_size = function (nbytes) {
            // Set the size of the buffer in bytes.
            //
            // Parameters
            // ----------
            // nbytes : int
            //     The number of bytes that the buffer needs to hold.
            if (nbytes != this.buffer_size) {
                this.activate();
                this.gl.bufferData(this._target, nbytes, this._usage);
                this.buffer_size = nbytes;
            }
        };
        Buffer.prototype.set_data = function (offset, data) {
            // Set the buffer data.
            //
            // Parameters
            // ----------
            // offset : int
            //     The offset in bytes for the new data.
            // data : typed array
            //     The data to upload.
            this.activate();
            this.gl.bufferSubData(this._target, offset, data);
        };
        return Buffer;
    }());
    exports.Buffer = Buffer;
    Buffer.__name__ = "Buffer";
    var VertexBuffer = /** @class */ (function (_super) {
        tslib_1.__extends(VertexBuffer, _super);
        function VertexBuffer() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this._target = 34962;
            return _this;
        }
        return VertexBuffer;
    }(Buffer));
    exports.VertexBuffer = VertexBuffer;
    VertexBuffer.__name__ = "VertexBuffer";
    var IndexBuffer = /** @class */ (function (_super) {
        tslib_1.__extends(IndexBuffer, _super);
        function IndexBuffer() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this._target = 34963;
            return _this;
        }
        return IndexBuffer;
    }(Buffer));
    exports.IndexBuffer = IndexBuffer;
    IndexBuffer.__name__ = "IndexBuffer";
},
/* models/glyphs/webgl/utils/texture.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var assert_1 = require(126) /* ../../../../core/util/assert */;
    var Texture2d = /** @class */ (function () {
        function Texture2d(gl) {
            this.gl = gl;
            this._target = 3553;
            this._types = {
                Int8Array: 5120,
                Uint8Array: 5121,
                Int16Array: 5122,
                Uint16Array: 5123,
                Int32Array: 5124,
                Uint32Array: 5125,
                Float32Array: 5126,
            };
            this.handle = this.gl.createTexture();
        }
        Texture2d.prototype.delete = function () {
            this.gl.deleteTexture(this.handle);
        };
        Texture2d.prototype.activate = function () {
            this.gl.bindTexture(this._target, this.handle);
        };
        Texture2d.prototype.deactivate = function () {
            this.gl.bindTexture(this._target, 0);
        };
        Texture2d.prototype._get_alignment = function (width) {
            var e_1, _d;
            // Determines a textures byte alignment. If the width isn't a
            // power of 2 we need to adjust the byte alignment of the image.
            // The image height is unimportant.
            //
            // www.opengl.org/wiki/Common_Mistakes#Texture_upload_and_pixel_reads
            var alignments = [4, 8, 2, 1];
            try {
                for (var alignments_1 = tslib_1.__values(alignments), alignments_1_1 = alignments_1.next(); !alignments_1_1.done; alignments_1_1 = alignments_1.next()) {
                    var alignment = alignments_1_1.value;
                    if (width % alignment == 0) {
                        return alignment;
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (alignments_1_1 && !alignments_1_1.done && (_d = alignments_1.return))
                        _d.call(alignments_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            assert_1.unreachable();
        };
        Texture2d.prototype.set_wrapping = function (wrap_s, wrap_t) {
            // Set the texture wrapping mode.
            //
            // Parameters
            // ----------
            // wrap_s : GL enum
            //     The mode to wrap the x dimension. Valid values are REPEAT
            //     CLAMP_TO_EDGE MIRRORED_REPEAT
            // wrap_t : GL enum
            //     The mode to wrap the y dimension. Same options as for wrap_s.
            this.activate();
            this.gl.texParameterf(this._target, this.gl.TEXTURE_WRAP_S, wrap_s);
            this.gl.texParameterf(this._target, this.gl.TEXTURE_WRAP_T, wrap_t);
        };
        Texture2d.prototype.set_interpolation = function (min, mag) {
            // Set the texture interpolation mode
            //
            // Parameters
            // ----------
            // min : GL enum
            //     The interpolation mode when minifying (i.e. zoomed out). Valid
            //     values are LINEAR and NEAREST.
            // max : GL enum
            //     The interpolation mode when magnifying (i.e. zoomed in). Valid
            //     values are LINEAR, NEAREST, NEAREST_MIPMAP_NEAREST,
            //     LINEAR_MIPMAP_NEAREST, NEAREST_MIPMAP_LINEAR, LINEAR_MIPMAP_LINEAR.
            this.activate();
            this.gl.texParameterf(this._target, this.gl.TEXTURE_MIN_FILTER, min);
            this.gl.texParameterf(this._target, this.gl.TEXTURE_MAG_FILTER, mag);
        };
        Texture2d.prototype.set_size = function (_d, format) {
            var _e = tslib_1.__read(_d, 2), width = _e[0], height = _e[1];
            var _a, _b, _c;
            // Set the size of the 2D texture.
            //
            // Parameters
            // ----------
            // shape : tuple of ints
            //     The shape of the data to upload
            // format : GL enum
            //     The format of the texture data. Can be LUMINANCE, LUMINANCE_ALPHA,
            //     RGB, and RGBA.
            if (width != ((_a = this._shape_format) === null || _a === void 0 ? void 0 : _a.width) || height != ((_b = this._shape_format) === null || _b === void 0 ? void 0 : _b.height) || format != ((_c = this._shape_format) === null || _c === void 0 ? void 0 : _c.format)) {
                this._shape_format = { width: width, height: height, format: format };
                this.activate();
                this.gl.texImage2D(this._target, 0, format, width, height, 0, format, this.gl.UNSIGNED_BYTE, null);
            }
        };
        Texture2d.prototype.set_data = function (offset, _d, data) {
            var _e = tslib_1.__read(_d, 2), width = _e[0], height = _e[1];
            // Set the 2D texture data.
            //
            // Parameters
            // ----------
            // offset : tuple of ints
            //     Offset in pixels for each dimension.
            // shape : tuple of ints
            //     The shape of the data to upload
            // data : typed array
            //     The actual pixel data. Can be of any type, but on the GPU the
            //     dat is stored in 8 bit precision.
            this.activate();
            var format = this._shape_format.format;
            var _f = tslib_1.__read(offset, 2), x = _f[0], y = _f[1];
            var gtype = this._types[data.constructor.name];
            if (gtype == null) {
                throw new Error("Type " + data.constructor.name + " not allowed for texture");
            }
            var alignment = this._get_alignment(width);
            if (alignment != 4) {
                this.gl.pixelStorei(this.gl.UNPACK_ALIGNMENT, alignment);
            }
            this.gl.texSubImage2D(this._target, 0, x, y, width, height, format, gtype, data);
            if (alignment != 4) {
                this.gl.pixelStorei(this.gl.UNPACK_ALIGNMENT, 4);
            }
        };
        return Texture2d;
    }());
    exports.Texture2d = Texture2d;
    Texture2d.__name__ = "Texture2d";
},
/* models/glyphs/webgl/base.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var BaseGLGlyph = /** @class */ (function () {
        function BaseGLGlyph(gl, glyph) {
            this.gl = gl;
            this.glyph = glyph;
            this.nvertices = 0;
            this.size_changed = false;
            this.data_changed = false;
            this.visuals_changed = false;
            this.init();
        }
        BaseGLGlyph.prototype.set_data_changed = function () {
            var data_size = this.glyph.data_size;
            if (data_size != this.nvertices) {
                this.nvertices = data_size;
                this.size_changed = true;
            }
            this.data_changed = true;
        };
        BaseGLGlyph.prototype.set_visuals_changed = function () {
            this.visuals_changed = true;
        };
        BaseGLGlyph.prototype.render = function (_ctx, indices, mainglyph) {
            if (indices.length == 0) {
                return true;
            }
            var _a = this.glyph.renderer.plot_view.canvas_view.webgl.canvas, width = _a.width, height = _a.height;
            var trans = {
                pixel_ratio: this.glyph.renderer.plot_view.canvas_view.pixel_ratio,
                width: width,
                height: height,
            };
            this.draw(indices, mainglyph, trans);
            return true;
        };
        return BaseGLGlyph;
    }());
    exports.BaseGLGlyph = BaseGLGlyph;
    BaseGLGlyph.__name__ = "BaseGLGlyph";
},
/* models/glyphs/webgl/line.vert.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.vertex_shader = "\nprecision mediump float;\n\nconst float PI = 3.14159265358979323846264;\nconst float THETA = 15.0 * 3.14159265358979323846264/180.0;\n\nuniform float u_pixel_ratio;\nuniform vec2 u_canvas_size, u_offset;\nuniform vec2 u_scale_aspect;\nuniform float u_scale_length;\n\nuniform vec4 u_color;\nuniform float u_antialias;\nuniform float u_length;\nuniform float u_linewidth;\nuniform float u_dash_index;\nuniform float u_closed;\n\nattribute vec2 a_position;\nattribute vec4 a_tangents;\nattribute vec2 a_segment;\nattribute vec2 a_angles;\nattribute vec2 a_texcoord;\n\nvarying vec4  v_color;\nvarying vec2  v_segment;\nvarying vec2  v_angles;\nvarying vec2  v_texcoord;\nvarying vec2  v_miter;\nvarying float v_length;\nvarying float v_linewidth;\n\nfloat cross(in vec2 v1, in vec2 v2)\n{\n    return v1.x*v2.y - v1.y*v2.x;\n}\n\nfloat signed_distance(in vec2 v1, in vec2 v2, in vec2 v3)\n{\n    return cross(v2-v1,v1-v3) / length(v2-v1);\n}\n\nvoid rotate( in vec2 v, in float alpha, out vec2 result )\n{\n    float c = cos(alpha);\n    float s = sin(alpha);\n    result = vec2( c*v.x - s*v.y,\n                   s*v.x + c*v.y );\n}\n\nvoid main()\n{\n    bool closed = (u_closed > 0.0);\n\n    // Attributes and uniforms to varyings\n    v_color = u_color;\n    v_linewidth = u_linewidth;\n    v_segment = a_segment * u_scale_length;\n    v_length = u_length * u_scale_length;\n\n    // Scale to map to pixel coordinates. The original algorithm from the paper\n    // assumed isotropic scale. We obviously do not have this.\n    vec2 abs_scale_aspect = abs(u_scale_aspect);\n    vec2 abs_scale = u_scale_length * abs_scale_aspect;\n\n    // Correct angles for aspect ratio\n    vec2 av;\n    av = vec2(1.0, tan(a_angles.x)) / abs_scale_aspect;\n    v_angles.x = atan(av.y, av.x);\n    av = vec2(1.0, tan(a_angles.y)) / abs_scale_aspect;\n    v_angles.y = atan(av.y, av.x);\n\n    // Thickness below 1 pixel are represented using a 1 pixel thickness\n    // and a modified alpha\n    v_color.a = min(v_linewidth, v_color.a);\n    v_linewidth = max(v_linewidth, 1.0);\n\n    // If color is fully transparent we just will discard the fragment anyway\n    if( v_color.a <= 0.0 ) {\n        gl_Position = vec4(0.0,0.0,0.0,1.0);\n        return;\n    }\n\n    // This is the actual half width of the line\n    float w = ceil(u_antialias+v_linewidth)/2.0;\n\n    vec2 position = a_position;\n\n    vec2 t1 = normalize(a_tangents.xy * abs_scale_aspect);  // note the scaling for aspect ratio here\n    vec2 t2 = normalize(a_tangents.zw * abs_scale_aspect);\n    float u = a_texcoord.x;\n    float v = a_texcoord.y;\n    vec2 o1 = vec2( +t1.y, -t1.x);\n    vec2 o2 = vec2( +t2.y, -t2.x);\n\n    // This is a join\n    // ----------------------------------------------------------------\n    if( t1 != t2 ) {\n        float angle = atan (t1.x*t2.y-t1.y*t2.x, t1.x*t2.x+t1.y*t2.y);  // Angle needs recalculation for some reason\n        vec2 t  = normalize(t1+t2);\n        vec2 o  = vec2( + t.y, - t.x);\n\n        if ( u_dash_index > 0.0 )\n        {\n            // Broken angle\n            // ----------------------------------------------------------------\n            if( (abs(angle) > THETA) ) {\n                position += v * w * o / cos(angle/2.0);\n                float s = sign(angle);\n                if( angle < 0.0 ) {\n                    if( u == +1.0 ) {\n                        u = v_segment.y + v * w * tan(angle/2.0);\n                        if( v == 1.0 ) {\n                            position -= 2.0 * w * t1 / sin(angle);\n                            u -= 2.0 * w / sin(angle);\n                        }\n                    } else {\n                        u = v_segment.x - v * w * tan(angle/2.0);\n                        if( v == 1.0 ) {\n                            position += 2.0 * w * t2 / sin(angle);\n                            u += 2.0*w / sin(angle);\n                        }\n                    }\n                } else {\n                    if( u == +1.0 ) {\n                        u = v_segment.y + v * w * tan(angle/2.0);\n                        if( v == -1.0 ) {\n                            position += 2.0 * w * t1 / sin(angle);\n                            u += 2.0 * w / sin(angle);\n                        }\n                    } else {\n                        u = v_segment.x - v * w * tan(angle/2.0);\n                        if( v == -1.0 ) {\n                            position -= 2.0 * w * t2 / sin(angle);\n                            u -= 2.0*w / sin(angle);\n                        }\n                    }\n                }\n                // Continuous angle\n                // ------------------------------------------------------------\n            } else {\n                position += v * w * o / cos(angle/2.0);\n                if( u == +1.0 ) u = v_segment.y;\n                else            u = v_segment.x;\n            }\n        }\n\n        // Solid line\n        // --------------------------------------------------------------------\n        else\n        {\n            position.xy += v * w * o / cos(angle/2.0);\n            if( angle < 0.0 ) {\n                if( u == +1.0 ) {\n                    u = v_segment.y + v * w * tan(angle/2.0);\n                } else {\n                    u = v_segment.x - v * w * tan(angle/2.0);\n                }\n            } else {\n                if( u == +1.0 ) {\n                    u = v_segment.y + v * w * tan(angle/2.0);\n                } else {\n                    u = v_segment.x - v * w * tan(angle/2.0);\n                }\n            }\n        }\n\n    // This is a line start or end (t1 == t2)\n    // ------------------------------------------------------------------------\n    } else {\n        position += v * w * o1;\n        if( u == -1.0 ) {\n            u = v_segment.x - w;\n            position -= w * t1;\n        } else {\n            u = v_segment.y + w;\n            position += w * t2;\n        }\n    }\n\n    // Miter distance\n    // ------------------------------------------------------------------------\n    vec2 t;\n    vec2 curr = a_position * abs_scale;\n    if( a_texcoord.x < 0.0 ) {\n        vec2 next = curr + t2*(v_segment.y-v_segment.x);\n\n        rotate( t1, +v_angles.x/2.0, t);\n        v_miter.x = signed_distance(curr, curr+t, position);\n\n        rotate( t2, +v_angles.y/2.0, t);\n        v_miter.y = signed_distance(next, next+t, position);\n    } else {\n        vec2 prev = curr - t1*(v_segment.y-v_segment.x);\n\n        rotate( t1, -v_angles.x/2.0,t);\n        v_miter.x = signed_distance(prev, prev+t, position);\n\n        rotate( t2, -v_angles.y/2.0,t);\n        v_miter.y = signed_distance(curr, curr+t, position);\n    }\n\n    if (!closed && v_segment.x <= 0.0) {\n        v_miter.x = 1e10;\n    }\n    if (!closed && v_segment.y >= v_length)\n    {\n        v_miter.y = 1e10;\n    }\n\n    v_texcoord = vec2( u, v*w );\n\n    // Calculate position in device coordinates. Note that we\n    // already scaled with abs scale above.\n    vec2 normpos = position * sign(u_scale_aspect);\n    normpos += 0.5;  // make up for Bokeh's offset\n    normpos /= u_canvas_size / u_pixel_ratio;  // in 0..1\n    gl_Position = vec4(normpos*2.0-1.0, 0.0, 1.0);\n    gl_Position.y *= -1.0;\n}\n";
},
/* models/glyphs/webgl/line.frag.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.fragment_shader = "\nprecision mediump float;\n\nconst float PI = 3.14159265358979323846264;\nconst float THETA = 15.0 * 3.14159265358979323846264/180.0;\n\nuniform sampler2D u_dash_atlas;\n\nuniform vec2 u_linecaps;\nuniform float u_miter_limit;\nuniform float u_linejoin;\nuniform float u_antialias;\nuniform float u_dash_phase;\nuniform float u_dash_period;\nuniform float u_dash_index;\nuniform vec2 u_dash_caps;\nuniform float u_closed;\n\nvarying vec4  v_color;\nvarying vec2  v_segment;\nvarying vec2  v_angles;\nvarying vec2  v_texcoord;\nvarying vec2  v_miter;\nvarying float v_length;\nvarying float v_linewidth;\n\n// Compute distance to cap ----------------------------------------------------\nfloat cap( int type, float dx, float dy, float t, float linewidth )\n{\n    float d = 0.0;\n    dx = abs(dx);\n    dy = abs(dy);\n    if      (type == 0)  discard;  // None\n    else if (type == 1)  d = sqrt(dx*dx+dy*dy);  // Round\n    else if (type == 3)  d = (dx+abs(dy));  // Triangle in\n    else if (type == 2)  d = max(abs(dy),(t+dx-abs(dy)));  // Triangle out\n    else if (type == 4)  d = max(dx,dy);  // Square\n    else if (type == 5)  d = max(dx+t,dy);  // Butt\n    return d;\n}\n\n// Compute distance to join -------------------------------------------------\nfloat join( in int type, in float d, in vec2 segment, in vec2 texcoord, in vec2 miter,\n           in float linewidth )\n{\n    // texcoord.x is distance from start\n    // texcoord.y is distance from centerline\n    // segment.x and y indicate the limits (as for texcoord.x) for this segment\n\n    float dx = texcoord.x;\n\n    // Round join\n    if( type == 1 ) {\n        if (dx < segment.x) {\n            d = max(d,length( texcoord - vec2(segment.x,0.0)));\n            //d = length( texcoord - vec2(segment.x,0.0));\n        } else if (dx > segment.y) {\n            d = max(d,length( texcoord - vec2(segment.y,0.0)));\n            //d = length( texcoord - vec2(segment.y,0.0));\n        }\n    }\n    // Bevel join\n    else if ( type == 2 ) {\n        if (dx < segment.x) {\n            vec2 x = texcoord - vec2(segment.x,0.0);\n            d = max(d, max(abs(x.x), abs(x.y)));\n\n        } else if (dx > segment.y) {\n            vec2 x = texcoord - vec2(segment.y,0.0);\n            d = max(d, max(abs(x.x), abs(x.y)));\n        }\n        /*  Original code for bevel which does not work for us\n        if( (dx < segment.x) ||  (dx > segment.y) )\n            d = max(d, min(abs(x.x),abs(x.y)));\n        */\n    }\n\n    return d;\n}\n\nvoid main()\n{\n    // If color is fully transparent we just discard the fragment\n    if( v_color.a <= 0.0 ) {\n        discard;\n    }\n\n    // Test if dash pattern is the solid one (0)\n    bool solid =  (u_dash_index == 0.0);\n\n    // Test if path is closed\n    bool closed = (u_closed > 0.0);\n\n    vec4 color = v_color;\n    float dx = v_texcoord.x;\n    float dy = v_texcoord.y;\n    float t = v_linewidth/2.0-u_antialias;\n    float width = 1.0;  //v_linewidth; original code had dashes scale with line width, we do not\n    float d = 0.0;\n\n    vec2 linecaps = u_linecaps;\n    vec2 dash_caps = u_dash_caps;\n    float line_start = 0.0;\n    float line_stop = v_length;\n\n    // Apply miter limit; fragments too far into the miter are simply discarded\n    if( (dx < v_segment.x) || (dx > v_segment.y) ) {\n        float into_miter = max(v_segment.x - dx, dx - v_segment.y);\n        if (into_miter > u_miter_limit*v_linewidth/2.0)\n          discard;\n    }\n\n    // Solid line --------------------------------------------------------------\n    if( solid ) {\n        d = abs(dy);\n        if( (!closed) && (dx < line_start) ) {\n            d = cap( int(u_linecaps.x), abs(dx), abs(dy), t, v_linewidth );\n        }\n        else if( (!closed) &&  (dx > line_stop) ) {\n            d = cap( int(u_linecaps.y), abs(dx)-line_stop, abs(dy), t, v_linewidth );\n        }\n        else {\n            d = join( int(u_linejoin), abs(dy), v_segment, v_texcoord, v_miter, v_linewidth );\n        }\n\n    // Dash line --------------------------------------------------------------\n    } else {\n        float segment_start = v_segment.x;\n        float segment_stop  = v_segment.y;\n        float segment_center= (segment_start+segment_stop)/2.0;\n        float freq          = u_dash_period*width;\n        float u = mod( dx + u_dash_phase*width, freq);\n        vec4 tex = texture2D(u_dash_atlas, vec2(u/freq, u_dash_index)) * 255.0 -10.0;  // conversion to int-like\n        float dash_center= tex.x * width;\n        float dash_type  = tex.y;\n        float _start = tex.z * width;\n        float _stop  = tex.a * width;\n        float dash_start = dx - u + _start;\n        float dash_stop  = dx - u + _stop;\n\n        // Compute extents of the first dash (the one relative to v_segment.x)\n        // Note: this could be computed in the vertex shader\n        if( (dash_stop < segment_start) && (dash_caps.x != 5.0) ) {\n            float u = mod(segment_start + u_dash_phase*width, freq);\n            vec4 tex = texture2D(u_dash_atlas, vec2(u/freq, u_dash_index)) * 255.0 -10.0;  // conversion to int-like\n            dash_center= tex.x * width;\n            //dash_type  = tex.y;\n            float _start = tex.z * width;\n            float _stop  = tex.a * width;\n            dash_start = segment_start - u + _start;\n            dash_stop = segment_start - u + _stop;\n        }\n\n        // Compute extents of the last dash (the one relatives to v_segment.y)\n        // Note: This could be computed in the vertex shader\n        else if( (dash_start > segment_stop)  && (dash_caps.y != 5.0) ) {\n            float u = mod(segment_stop + u_dash_phase*width, freq);\n            vec4 tex = texture2D(u_dash_atlas, vec2(u/freq, u_dash_index)) * 255.0 -10.0;  // conversion to int-like\n            dash_center= tex.x * width;\n            //dash_type  = tex.y;\n            float _start = tex.z * width;\n            float _stop  = tex.a * width;\n            dash_start = segment_stop - u + _start;\n            dash_stop  = segment_stop - u + _stop;\n        }\n\n        // This test if the we are dealing with a discontinuous angle\n        bool discontinuous = ((dx <  segment_center) && abs(v_angles.x) > THETA) ||\n                             ((dx >= segment_center) && abs(v_angles.y) > THETA);\n        //if( dx < line_start) discontinuous = false;\n        //if( dx > line_stop)  discontinuous = false;\n\n        float d_join = join( int(u_linejoin), abs(dy),\n                            v_segment, v_texcoord, v_miter, v_linewidth );\n\n        // When path is closed, we do not have room for linecaps, so we make room\n        // by shortening the total length\n        if (closed) {\n             line_start += v_linewidth/2.0;\n             line_stop  -= v_linewidth/2.0;\n        }\n\n        // We also need to take antialias area into account\n        //line_start += u_antialias;\n        //line_stop  -= u_antialias;\n\n        // Check is dash stop is before line start\n        if( dash_stop <= line_start ) {\n            discard;\n        }\n        // Check is dash start is beyond line stop\n        if( dash_start >= line_stop ) {\n            discard;\n        }\n\n        // Check if current dash start is beyond segment stop\n        if( discontinuous ) {\n            // Dash start is beyond segment, we discard\n            if( (dash_start > segment_stop) ) {\n                discard;\n                //gl_FragColor = vec4(1.0,0.0,0.0,.25); return;\n            }\n\n            // Dash stop is before segment, we discard\n            if( (dash_stop < segment_start) ) {\n                discard;  //gl_FragColor = vec4(0.0,1.0,0.0,.25); return;\n            }\n\n            // Special case for round caps (nicer with this)\n            if( dash_caps.x == 1.0 ) {\n                if( (u > _stop) && (dash_stop > segment_stop )  && (abs(v_angles.y) < PI/2.0)) {\n                    discard;\n                }\n            }\n\n            // Special case for round caps  (nicer with this)\n            if( dash_caps.y == 1.0 ) {\n                if( (u < _start) && (dash_start < segment_start )  && (abs(v_angles.x) < PI/2.0)) {\n                    discard;\n                }\n            }\n\n            // Special case for triangle caps (in & out) and square\n            // We make sure the cap stop at crossing frontier\n            if( (dash_caps.x != 1.0) && (dash_caps.x != 5.0) ) {\n                if( (dash_start < segment_start )  && (abs(v_angles.x) < PI/2.0) ) {\n                    float a = v_angles.x/2.0;\n                    float x = (segment_start-dx)*cos(a) - dy*sin(a);\n                    float y = (segment_start-dx)*sin(a) + dy*cos(a);\n                    if( x > 0.0 ) discard;\n                    // We transform the cap into square to avoid holes\n                    dash_caps.x = 4.0;\n                }\n            }\n\n            // Special case for triangle caps (in & out) and square\n            // We make sure the cap stop at crossing frontier\n            if( (dash_caps.y != 1.0) && (dash_caps.y != 5.0) ) {\n                if( (dash_stop > segment_stop )  && (abs(v_angles.y) < PI/2.0) ) {\n                    float a = v_angles.y/2.0;\n                    float x = (dx-segment_stop)*cos(a) - dy*sin(a);\n                    float y = (dx-segment_stop)*sin(a) + dy*cos(a);\n                    if( x > 0.0 ) discard;\n                    // We transform the caps into square to avoid holes\n                    dash_caps.y = 4.0;\n                }\n            }\n        }\n\n        // Line cap at start\n        if( (dx < line_start) && (dash_start < line_start) && (dash_stop > line_start) ) {\n            d = cap( int(linecaps.x), dx-line_start, dy, t, v_linewidth);\n        }\n        // Line cap at stop\n        else if( (dx > line_stop) && (dash_stop > line_stop) && (dash_start < line_stop) ) {\n            d = cap( int(linecaps.y), dx-line_stop, dy, t, v_linewidth);\n        }\n        // Dash cap left - dash_type = -1, 0 or 1, but there may be roundoff errors\n        else if( dash_type < -0.5 ) {\n            d = cap( int(dash_caps.y), abs(u-dash_center), dy, t, v_linewidth);\n            if( (dx > line_start) && (dx < line_stop) )\n                d = max(d,d_join);\n        }\n        // Dash cap right\n        else if( dash_type > 0.5 ) {\n            d = cap( int(dash_caps.x), abs(dash_center-u), dy, t, v_linewidth);\n            if( (dx > line_start) && (dx < line_stop) )\n                d = max(d,d_join);\n        }\n        // Dash body (plain)\n        else {// if( dash_type > -0.5 &&  dash_type < 0.5) {\n            d = abs(dy);\n        }\n\n        // Line join\n        if( (dx > line_start) && (dx < line_stop)) {\n            if( (dx <= segment_start) && (dash_start <= segment_start)\n                && (dash_stop >= segment_start) ) {\n                d = d_join;\n                // Antialias at outer border\n                float angle = PI/2.+v_angles.x;\n                float f = abs( (segment_start - dx)*cos(angle) - dy*sin(angle));\n                d = max(f,d);\n            }\n            else if( (dx > segment_stop) && (dash_start <= segment_stop)\n                     && (dash_stop >= segment_stop) ) {\n                d = d_join;\n                // Antialias at outer border\n                float angle = PI/2.+v_angles.y;\n                float f = abs((dx - segment_stop)*cos(angle) - dy*sin(angle));\n                d = max(f,d);\n            }\n            else if( dx < (segment_start - v_linewidth/2.)) {\n                discard;\n            }\n            else if( dx > (segment_stop + v_linewidth/2.)) {\n                discard;\n            }\n        }\n        else if( dx < (segment_start - v_linewidth/2.)) {\n            discard;\n        }\n        else if( dx > (segment_stop + v_linewidth/2.)) {\n            discard;\n        }\n    }\n\n    // Distance to border ------------------------------------------------------\n    d = d - t;\n    if( d < 0.0 ) {\n        gl_FragColor = color;\n    } else {\n        d /= u_antialias;\n        gl_FragColor = vec4(color.rgb, exp(-d*d)*color.a);\n    }\n}\n";
},
/* models/glyphs/patch.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var utils_1 = require(215) /* ./utils */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var selection_1 = require(203) /* ../selections/selection */;
    var PatchView = /** @class */ (function (_super) {
        tslib_1.__extends(PatchView, _super);
        function PatchView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        PatchView.prototype._inner_loop = function (ctx, indices, sx, sy, func) {
            var e_1, _a;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (i == 0) {
                        ctx.beginPath();
                        ctx.moveTo(sx[i], sy[i]);
                        continue;
                    }
                    else if (isNaN(sx[i] + sy[i])) {
                        ctx.closePath();
                        func.apply(ctx);
                        ctx.beginPath();
                        continue;
                    }
                    else
                        ctx.lineTo(sx[i], sy[i]);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_a = indices_1.return))
                        _a.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            ctx.closePath();
            func.call(ctx);
        };
        PatchView.prototype._render = function (ctx, indices, _a) {
            var _this = this;
            var sx = _a.sx, sy = _a.sy;
            if (this.visuals.fill.doit) {
                this.visuals.fill.set_value(ctx);
                this._inner_loop(ctx, indices, sx, sy, ctx.fill);
            }
            this.visuals.hatch.doit2(ctx, 0, function () { return _this._inner_loop(ctx, indices, sx, sy, ctx.fill); }, function () { return _this.renderer.request_render(); });
            if (this.visuals.line.doit) {
                this.visuals.line.set_value(ctx);
                this._inner_loop(ctx, indices, sx, sy, ctx.stroke);
            }
        };
        PatchView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_area_legend(this.visuals, ctx, bbox, index);
        };
        PatchView.prototype._hit_point = function (geometry) {
            var result = new selection_1.Selection();
            if (hittest.point_in_poly(geometry.sx, geometry.sy, this.sx, this.sy)) {
                result.add_to_selected_glyphs(this.model);
                result.view = this;
            }
            return result;
        };
        return PatchView;
    }(xy_glyph_1.XYGlyphView));
    exports.PatchView = PatchView;
    PatchView.__name__ = "PatchView";
    var Patch = /** @class */ (function (_super) {
        tslib_1.__extends(Patch, _super);
        function Patch(attrs) {
            return _super.call(this, attrs) || this;
        }
        Patch.init_Patch = function () {
            this.prototype.default_view = PatchView;
            this.mixins([mixins.Line /*Scalar*/, mixins.Fill /*Scalar*/, mixins.Hatch /*Scalar*/]);
        };
        return Patch;
    }(xy_glyph_1.XYGlyph));
    exports.Patch = Patch;
    Patch.__name__ = "Patch";
    Patch.init_Patch();
},
/* models/glyphs/harea.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(139) /* ../../core/types */;
    var area_1 = require(227) /* ./area */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var selection_1 = require(203) /* ../selections/selection */;
    var HAreaView = /** @class */ (function (_super) {
        tslib_1.__extends(HAreaView, _super);
        function HAreaView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        HAreaView.prototype._index_data = function (index) {
            var min = Math.min, max = Math.max;
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                var x1 = this._x1[i];
                var x2 = this._x2[i];
                var y = this._y[i];
                if (isNaN(x1 + x2 + y) || !isFinite(x1 + x2 + y))
                    index.add_empty();
                else
                    index.add(min(x1, x2), y, max(x1, x2), y);
            }
        };
        HAreaView.prototype._inner = function (ctx, sx1, sx2, sy, func) {
            ctx.beginPath();
            for (var i = 0, end = sx1.length; i < end; i++) {
                ctx.lineTo(sx1[i], sy[i]);
            }
            // iterate backwards so that the upper end is below the lower start
            for (var i = sx2.length - 1; i >= 0; i--) {
                ctx.lineTo(sx2[i], sy[i]);
            }
            ctx.closePath();
            func.call(ctx);
        };
        HAreaView.prototype._render = function (ctx, _indices, _a) {
            var _this = this;
            var sx1 = _a.sx1, sx2 = _a.sx2, sy = _a.sy;
            if (this.visuals.fill.doit) {
                this.visuals.fill.set_value(ctx);
                this._inner(ctx, sx1, sx2, sy, ctx.fill);
            }
            this.visuals.hatch.doit2(ctx, 0, function () { return _this._inner(ctx, sx1, sx2, sy, ctx.fill); }, function () { return _this.renderer.request_render(); });
        };
        HAreaView.prototype._hit_point = function (geometry) {
            var L = this.sy.length;
            var sx = new types_1.NumberArray(2 * L);
            var sy = new types_1.NumberArray(2 * L);
            for (var i = 0, end = L; i < end; i++) {
                sx[i] = this.sx1[i];
                sy[i] = this.sy[i];
                sx[L + i] = this.sx2[L - i - 1];
                sy[L + i] = this.sy[L - i - 1];
            }
            var result = new selection_1.Selection();
            if (hittest.point_in_poly(geometry.sx, geometry.sy, sx, sy)) {
                result.add_to_selected_glyphs(this.model);
                result.view = this;
            }
            return result;
        };
        HAreaView.prototype.scenterxy = function (i) {
            var scx = (this.sx1[i] + this.sx2[i]) / 2;
            var scy = this.sy[i];
            return [scx, scy];
        };
        HAreaView.prototype._map_data = function () {
            this.sx1 = this.renderer.xscale.v_compute(this._x1);
            this.sx2 = this.renderer.xscale.v_compute(this._x2);
            this.sy = this.renderer.yscale.v_compute(this._y);
        };
        return HAreaView;
    }(area_1.AreaView));
    exports.HAreaView = HAreaView;
    HAreaView.__name__ = "HAreaView";
    var HArea = /** @class */ (function (_super) {
        tslib_1.__extends(HArea, _super);
        function HArea(attrs) {
            return _super.call(this, attrs) || this;
        }
        HArea.init_HArea = function () {
            this.prototype.default_view = HAreaView;
            this.define({
                x1: [p.XCoordinateSpec, { field: "x1" }],
                x2: [p.XCoordinateSpec, { field: "x2" }],
                y: [p.YCoordinateSpec, { field: "y" }],
            });
        };
        return HArea;
    }(area_1.Area));
    exports.HArea = HArea;
    HArea.__name__ = "HArea";
    HArea.init_HArea();
},
/* models/glyphs/area.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var glyph_1 = require(209) /* ./glyph */;
    var utils_1 = require(215) /* ./utils */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var AreaView = /** @class */ (function (_super) {
        tslib_1.__extends(AreaView, _super);
        function AreaView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        AreaView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_area_legend(this.visuals, ctx, bbox, index);
        };
        return AreaView;
    }(glyph_1.GlyphView));
    exports.AreaView = AreaView;
    AreaView.__name__ = "AreaView";
    var Area = /** @class */ (function (_super) {
        tslib_1.__extends(Area, _super);
        function Area(attrs) {
            return _super.call(this, attrs) || this;
        }
        Area.init_Area = function () {
            this.mixins([mixins.Fill /*Scalar*/, mixins.HatchVector]);
        };
        return Area;
    }(glyph_1.Glyph));
    exports.Area = Area;
    Area.__name__ = "Area";
    Area.init_Area();
},
/* models/glyphs/varea.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(139) /* ../../core/types */;
    var area_1 = require(227) /* ./area */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var selection_1 = require(203) /* ../selections/selection */;
    var VAreaView = /** @class */ (function (_super) {
        tslib_1.__extends(VAreaView, _super);
        function VAreaView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        VAreaView.prototype._index_data = function (index) {
            var min = Math.min, max = Math.max;
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                var x = this._x[i];
                var y1 = this._y1[i];
                var y2 = this._y2[i];
                if (isNaN(x + y1 + y2) || !isFinite(x + y1 + y2))
                    index.add_empty();
                else
                    index.add(x, min(y1, y2), x, max(y1, y2));
            }
        };
        VAreaView.prototype._inner = function (ctx, sx, sy1, sy2, func) {
            ctx.beginPath();
            for (var i = 0, end = sy1.length; i < end; i++) {
                ctx.lineTo(sx[i], sy1[i]);
            }
            // iterate backwards so that the upper end is below the lower start
            for (var i = sy2.length - 1; i >= 0; i--) {
                ctx.lineTo(sx[i], sy2[i]);
            }
            ctx.closePath();
            func.call(ctx);
        };
        VAreaView.prototype._render = function (ctx, _indices, _a) {
            var _this = this;
            var sx = _a.sx, sy1 = _a.sy1, sy2 = _a.sy2;
            if (this.visuals.fill.doit) {
                this.visuals.fill.set_value(ctx);
                this._inner(ctx, sx, sy1, sy2, ctx.fill);
            }
            this.visuals.hatch.doit2(ctx, 0, function () { return _this._inner(ctx, sx, sy1, sy2, ctx.fill); }, function () { return _this.renderer.request_render(); });
        };
        VAreaView.prototype.scenterxy = function (i) {
            var scx = this.sx[i];
            var scy = (this.sy1[i] + this.sy2[i]) / 2;
            return [scx, scy];
        };
        VAreaView.prototype._hit_point = function (geometry) {
            var L = this.sx.length;
            var sx = new types_1.NumberArray(2 * L);
            var sy = new types_1.NumberArray(2 * L);
            for (var i = 0, end = L; i < end; i++) {
                sx[i] = this.sx[i];
                sy[i] = this.sy1[i];
                sx[L + i] = this.sx[L - i - 1];
                sy[L + i] = this.sy2[L - i - 1];
            }
            var result = new selection_1.Selection();
            if (hittest.point_in_poly(geometry.sx, geometry.sy, sx, sy)) {
                result.add_to_selected_glyphs(this.model);
                result.view = this;
            }
            return result;
        };
        VAreaView.prototype._map_data = function () {
            this.sx = this.renderer.xscale.v_compute(this._x);
            this.sy1 = this.renderer.yscale.v_compute(this._y1);
            this.sy2 = this.renderer.yscale.v_compute(this._y2);
        };
        return VAreaView;
    }(area_1.AreaView));
    exports.VAreaView = VAreaView;
    VAreaView.__name__ = "VAreaView";
    var VArea = /** @class */ (function (_super) {
        tslib_1.__extends(VArea, _super);
        function VArea(attrs) {
            return _super.call(this, attrs) || this;
        }
        VArea.init_VArea = function () {
            this.prototype.default_view = VAreaView;
            this.define({
                x: [p.XCoordinateSpec, { field: "x" }],
                y1: [p.YCoordinateSpec, { field: "y1" }],
                y2: [p.YCoordinateSpec, { field: "y2" }],
            });
        };
        return VArea;
    }(area_1.Area));
    exports.VArea = VArea;
    VArea.__name__ = "VArea";
    VArea.init_VArea();
},
/* models/sources/cds_view.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var selection_1 = require(203) /* ../selections/selection */;
    var types_1 = require(139) /* ../../core/types */;
    var columnar_data_source_1 = require(201) /* ./columnar_data_source */;
    var CDSView = /** @class */ (function (_super) {
        tslib_1.__extends(CDSView, _super);
        function CDSView(attrs) {
            return _super.call(this, attrs) || this;
        }
        CDSView.init_CDSView = function () {
            this.define({
                filters: [p.Array, []],
                source: [p.Instance],
            });
            this.internal({
                indices: [p.Any],
                indices_map: [p.Any, {}],
                masked: [p.Any, null],
            });
        };
        CDSView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.compute_indices();
        };
        CDSView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.properties.filters.change, function () {
                _this.compute_indices();
                _this.change.emit();
            });
            var connect_listeners = function () {
                var fn = function () { return _this.compute_indices(); };
                if (_this.source != null) {
                    _this.connect(_this.source.change, fn);
                    if (_this.source instanceof columnar_data_source_1.ColumnarDataSource) {
                        _this.connect(_this.source.streaming, fn);
                        _this.connect(_this.source.patching, fn);
                    }
                }
            };
            var initialized = this.source != null;
            if (initialized)
                connect_listeners();
            else {
                this.connect(this.properties.source.change, function () {
                    if (!initialized) {
                        connect_listeners();
                        initialized = true;
                    }
                });
            }
        };
        CDSView.prototype.compute_indices = function () {
            var e_1, _b;
            var _a;
            var source = this.source;
            if (source == null)
                return;
            // XXX: if the data source is empty, there still may be one
            // index originating from glyph's scalar values.
            var size = (_a = source.get_length()) !== null && _a !== void 0 ? _a : 1;
            var indices = types_1.Indices.all_set(size);
            try {
                for (var _c = tslib_1.__values(this.filters), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var filter = _d.value;
                    indices.intersect(filter.compute_indices(source));
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            this.indices = indices;
            this._indices = tslib_1.__spread(indices);
            this.indices_map_to_subset();
        };
        CDSView.prototype.indices_map_to_subset = function () {
            this.indices_map = {};
            for (var i = 0; i < this._indices.length; i++) {
                this.indices_map[this._indices[i]] = i;
            }
        };
        CDSView.prototype.convert_selection_from_subset = function (selection_subset) {
            var _this = this;
            var indices = selection_subset.indices.map(function (i) { return _this._indices[i]; });
            return new selection_1.Selection(Object.assign(Object.assign({}, selection_subset.attributes), { indices: indices }));
        };
        CDSView.prototype.convert_selection_to_subset = function (selection_full) {
            var _this = this;
            var indices = selection_full.indices.map(function (i) { return _this.indices_map[i]; });
            return new selection_1.Selection(Object.assign(Object.assign({}, selection_full.attributes), { indices: indices }));
        };
        CDSView.prototype.convert_indices_from_subset = function (indices) {
            var _this = this;
            return indices.map(function (i) { return _this._indices[i]; });
        };
        return CDSView;
    }(model_1.Model));
    exports.CDSView = CDSView;
    CDSView.__name__ = "CDSView";
    CDSView.init_CDSView();
},
/* core/build_views.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var array_1 = require(124) /* ./util/array */;
    function _build_view(view_cls, model, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var view;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        view = new view_cls(Object.assign(Object.assign({}, options), { model: model }));
                        view.initialize();
                        return [4 /*yield*/, view.lazy_initialize()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, view];
                }
            });
        });
    }
    function build_view(model, options, cls) {
        if (options === void 0) {
            options = { parent: null };
        }
        if (cls === void 0) {
            cls = function (model) { return model.default_view; };
        }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var view;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, _build_view(cls(model), model, options)];
                    case 1:
                        view = _a.sent();
                        view.connect_signals();
                        return [2 /*return*/, view];
                }
            });
        });
    }
    exports.build_view = build_view;
    function build_views(view_storage, models, options, cls) {
        if (options === void 0) {
            options = { parent: null };
        }
        if (cls === void 0) {
            cls = function (model) { return model.default_view; };
        }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var to_remove, to_remove_1, to_remove_1_1, model, created_views, new_models, new_models_1, new_models_1_1, model, view, e_1_1, created_views_1, created_views_1_1, view;
            var e_2, _a, e_1, _b, e_3, _c;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        to_remove = array_1.difference(tslib_1.__spread(view_storage.keys()), models);
                        try {
                            for (to_remove_1 = tslib_1.__values(to_remove), to_remove_1_1 = to_remove_1.next(); !to_remove_1_1.done; to_remove_1_1 = to_remove_1.next()) {
                                model = to_remove_1_1.value;
                                view_storage.get(model).remove();
                                view_storage.delete(model);
                            }
                        }
                        catch (e_2_1) {
                            e_2 = { error: e_2_1 };
                        }
                        finally {
                            try {
                                if (to_remove_1_1 && !to_remove_1_1.done && (_a = to_remove_1.return))
                                    _a.call(to_remove_1);
                            }
                            finally {
                                if (e_2)
                                    throw e_2.error;
                            }
                        }
                        created_views = [];
                        new_models = models.filter(function (model) { return !view_storage.has(model); });
                        _d.label = 1;
                    case 1:
                        _d.trys.push([1, 6, 7, 8]);
                        new_models_1 = tslib_1.__values(new_models), new_models_1_1 = new_models_1.next();
                        _d.label = 2;
                    case 2:
                        if (!!new_models_1_1.done)
                            return [3 /*break*/, 5];
                        model = new_models_1_1.value;
                        return [4 /*yield*/, _build_view(cls(model), model, options)];
                    case 3:
                        view = _d.sent();
                        view_storage.set(model, view);
                        created_views.push(view);
                        _d.label = 4;
                    case 4:
                        new_models_1_1 = new_models_1.next();
                        return [3 /*break*/, 2];
                    case 5: return [3 /*break*/, 8];
                    case 6:
                        e_1_1 = _d.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 8];
                    case 7:
                        try {
                            if (new_models_1_1 && !new_models_1_1.done && (_b = new_models_1.return))
                                _b.call(new_models_1);
                        }
                        finally {
                            if (e_1)
                                throw e_1.error;
                        }
                        return [7 /*endfinally*/];
                    case 8:
                        try {
                            for (created_views_1 = tslib_1.__values(created_views), created_views_1_1 = created_views_1.next(); !created_views_1_1.done; created_views_1_1 = created_views_1.next()) {
                                view = created_views_1_1.value;
                                view.connect_signals();
                            }
                        }
                        catch (e_3_1) {
                            e_3 = { error: e_3_1 };
                        }
                        finally {
                            try {
                                if (created_views_1_1 && !created_views_1_1.done && (_c = created_views_1.return))
                                    _c.call(created_views_1);
                            }
                            finally {
                                if (e_3)
                                    throw e_3.error;
                            }
                        }
                        return [2 /*return*/, created_views];
                }
            });
        });
    }
    exports.build_views = build_views;
    function remove_views(view_storage) {
        var e_4, _a;
        try {
            for (var view_storage_1 = tslib_1.__values(view_storage), view_storage_1_1 = view_storage_1.next(); !view_storage_1_1.done; view_storage_1_1 = view_storage_1.next()) {
                var _b = tslib_1.__read(view_storage_1_1.value, 2), model = _b[0], view = _b[1];
                view.remove();
                view_storage.delete(model);
            }
        }
        catch (e_4_1) {
            e_4 = { error: e_4_1 };
        }
        finally {
            try {
                if (view_storage_1_1 && !view_storage_1_1.done && (_a = view_storage_1.return))
                    _a.call(view_storage_1);
            }
            finally {
                if (e_4)
                    throw e_4.error;
            }
        }
    }
    exports.remove_views = remove_views;
},
/* models/renderers/graph_renderer.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var data_renderer_1 = require(206) /* ./data_renderer */;
    var graph_hit_test_policy_1 = require(232) /* ../graphs/graph_hit_test_policy */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var build_views_1 = require(230) /* ../../core/build_views */;
    var assert_1 = require(126) /* ../../core/util/assert */;
    var GraphRendererView = /** @class */ (function (_super) {
        tslib_1.__extends(GraphRendererView, _super);
        function GraphRendererView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        GraphRendererView.prototype.lazy_initialize = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var graph, xs_ys, x_y, xs_expr, ys_expr, x_expr, y_expr, _a, edge_renderer, node_renderer, parent, _b, _c;
                return tslib_1.__generator(this, function (_d) {
                    switch (_d.label) {
                        case 0: return [4 /*yield*/, _super.prototype.lazy_initialize.call(this)];
                        case 1:
                            _d.sent();
                            graph = this.model;
                            xs_ys = null;
                            x_y = null;
                            xs_expr = {
                                v_compute: function (source) {
                                    assert_1.assert(xs_ys == null);
                                    var _a = tslib_1.__read(xs_ys = graph.layout_provider.get_edge_coordinates(source), 1), xs = _a[0];
                                    return xs;
                                },
                            };
                            ys_expr = {
                                v_compute: function (_source) {
                                    assert_1.assert(xs_ys != null);
                                    var _a = tslib_1.__read(xs_ys, 2), ys = _a[1];
                                    xs_ys = null;
                                    return ys;
                                },
                            };
                            x_expr = {
                                v_compute: function (source) {
                                    assert_1.assert(x_y == null);
                                    var _a = tslib_1.__read(x_y = graph.layout_provider.get_node_coordinates(source), 1), x = _a[0];
                                    return x;
                                },
                            };
                            y_expr = {
                                v_compute: function (_source) {
                                    assert_1.assert(x_y != null);
                                    var _a = tslib_1.__read(x_y, 2), y = _a[1];
                                    x_y = null;
                                    return y;
                                },
                            };
                            _a = this.model, edge_renderer = _a.edge_renderer, node_renderer = _a.node_renderer;
                            edge_renderer.glyph.xs = { expr: xs_expr };
                            edge_renderer.glyph.ys = { expr: ys_expr };
                            node_renderer.glyph.x = { expr: x_expr };
                            node_renderer.glyph.y = { expr: y_expr };
                            parent = this.parent;
                            _b = this;
                            return [4 /*yield*/, build_views_1.build_view(edge_renderer, { parent: parent })];
                        case 2:
                            _b.edge_view = _d.sent();
                            _c = this;
                            return [4 /*yield*/, build_views_1.build_view(node_renderer, { parent: parent })];
                        case 3:
                            _c.node_view = _d.sent();
                            return [2 /*return*/];
                    }
                });
            });
        };
        GraphRendererView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.layout_provider.change, function () {
                _this.edge_view.set_data(false);
                _this.node_view.set_data(false);
                _this.request_render();
            });
        };
        GraphRendererView.prototype.remove = function () {
            this.edge_view.remove();
            this.node_view.remove();
            _super.prototype.remove.call(this);
        };
        GraphRendererView.prototype._render = function () {
            this.edge_view.render();
            this.node_view.render();
        };
        return GraphRendererView;
    }(data_renderer_1.DataRendererView));
    exports.GraphRendererView = GraphRendererView;
    GraphRendererView.__name__ = "GraphRendererView";
    var GraphRenderer = /** @class */ (function (_super) {
        tslib_1.__extends(GraphRenderer, _super);
        function GraphRenderer(attrs) {
            return _super.call(this, attrs) || this;
        }
        GraphRenderer.init_GraphRenderer = function () {
            this.prototype.default_view = GraphRendererView;
            this.define({
                layout_provider: [p.Instance],
                node_renderer: [p.Instance],
                edge_renderer: [p.Instance],
                selection_policy: [p.Instance, function () { return new graph_hit_test_policy_1.NodesOnly(); }],
                inspection_policy: [p.Instance, function () { return new graph_hit_test_policy_1.NodesOnly(); }],
            });
        };
        GraphRenderer.prototype.get_selection_manager = function () {
            return this.node_renderer.data_source.selection_manager;
        };
        return GraphRenderer;
    }(data_renderer_1.DataRenderer));
    exports.GraphRenderer = GraphRenderer;
    GraphRenderer.__name__ = "GraphRenderer";
    GraphRenderer.init_GraphRenderer();
},
/* models/graphs/graph_hit_test_policy.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var array_1 = require(124) /* ../../core/util/array */;
    var selection_1 = require(203) /* ../selections/selection */;
    var GraphHitTestPolicy = /** @class */ (function (_super) {
        tslib_1.__extends(GraphHitTestPolicy, _super);
        function GraphHitTestPolicy(attrs) {
            return _super.call(this, attrs) || this;
        }
        GraphHitTestPolicy.prototype._hit_test_nodes = function (geometry, graph_view) {
            if (!graph_view.model.visible)
                return null;
            var hit_test_result = graph_view.node_view.glyph.hit_test(geometry);
            if (hit_test_result == null)
                return null;
            else
                return graph_view.node_view.model.view.convert_selection_from_subset(hit_test_result);
        };
        GraphHitTestPolicy.prototype._hit_test_edges = function (geometry, graph_view) {
            if (!graph_view.model.visible)
                return null;
            var hit_test_result = graph_view.edge_view.glyph.hit_test(geometry);
            if (hit_test_result == null)
                return null;
            else
                return graph_view.edge_view.model.view.convert_selection_from_subset(hit_test_result);
        };
        return GraphHitTestPolicy;
    }(model_1.Model));
    exports.GraphHitTestPolicy = GraphHitTestPolicy;
    GraphHitTestPolicy.__name__ = "GraphHitTestPolicy";
    var NodesOnly = /** @class */ (function (_super) {
        tslib_1.__extends(NodesOnly, _super);
        function NodesOnly(attrs) {
            return _super.call(this, attrs) || this;
        }
        NodesOnly.prototype.hit_test = function (geometry, graph_view) {
            return this._hit_test_nodes(geometry, graph_view);
        };
        NodesOnly.prototype.do_selection = function (hit_test_result, graph, final, mode) {
            if (hit_test_result == null)
                return false;
            var node_selection = graph.node_renderer.data_source.selected;
            node_selection.update(hit_test_result, final, mode);
            graph.node_renderer.data_source._select.emit();
            return !node_selection.is_empty();
        };
        NodesOnly.prototype.do_inspection = function (hit_test_result, geometry, graph_view, final, mode) {
            if (hit_test_result == null)
                return false;
            var node_inspection = graph_view.model.get_selection_manager().get_or_create_inspector(graph_view.node_view.model);
            node_inspection.update(hit_test_result, final, mode);
            // silently set inspected attr to avoid triggering data_source.change event and rerender
            graph_view.node_view.model.data_source.setv({ inspected: node_inspection }, { silent: true });
            graph_view.node_view.model.data_source.inspect.emit([graph_view.node_view, { geometry: geometry }]);
            return !node_inspection.is_empty();
        };
        return NodesOnly;
    }(GraphHitTestPolicy));
    exports.NodesOnly = NodesOnly;
    NodesOnly.__name__ = "NodesOnly";
    var NodesAndLinkedEdges = /** @class */ (function (_super) {
        tslib_1.__extends(NodesAndLinkedEdges, _super);
        function NodesAndLinkedEdges(attrs) {
            return _super.call(this, attrs) || this;
        }
        NodesAndLinkedEdges.prototype.hit_test = function (geometry, graph_view) {
            return this._hit_test_nodes(geometry, graph_view);
        };
        NodesAndLinkedEdges.prototype.get_linked_edges = function (node_source, edge_source, mode) {
            var e_1, _a;
            var node_indices = [];
            if (mode == 'selection') {
                node_indices = node_source.selected.indices.map(function (i) { return node_source.data.index[i]; });
            }
            else if (mode == 'inspection') {
                node_indices = node_source.inspected.indices.map(function (i) { return node_source.data.index[i]; });
            }
            var edge_indices = [];
            for (var i = 0; i < edge_source.data.start.length; i++) {
                if (array_1.contains(node_indices, edge_source.data.start[i]) || array_1.contains(node_indices, edge_source.data.end[i]))
                    edge_indices.push(i);
            }
            var linked_edges = new selection_1.Selection();
            try {
                for (var edge_indices_1 = tslib_1.__values(edge_indices), edge_indices_1_1 = edge_indices_1.next(); !edge_indices_1_1.done; edge_indices_1_1 = edge_indices_1.next()) {
                    var i = edge_indices_1_1.value;
                    linked_edges.multiline_indices[i] = [0]; //currently only supports 2-element multilines, so this is all of it
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (edge_indices_1_1 && !edge_indices_1_1.done && (_a = edge_indices_1.return))
                        _a.call(edge_indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            linked_edges.indices = edge_indices;
            return linked_edges;
        };
        NodesAndLinkedEdges.prototype.do_selection = function (hit_test_result, graph, final, mode) {
            if (hit_test_result == null)
                return false;
            var node_selection = graph.node_renderer.data_source.selected;
            node_selection.update(hit_test_result, final, mode);
            var edge_selection = graph.edge_renderer.data_source.selected;
            var linked_edges_selection = this.get_linked_edges(graph.node_renderer.data_source, graph.edge_renderer.data_source, 'selection');
            edge_selection.update(linked_edges_selection, final, mode);
            graph.node_renderer.data_source._select.emit();
            return !node_selection.is_empty();
        };
        NodesAndLinkedEdges.prototype.do_inspection = function (hit_test_result, geometry, graph_view, final, mode) {
            if (hit_test_result == null)
                return false;
            var node_inspection = graph_view.node_view.model.data_source.selection_manager.get_or_create_inspector(graph_view.node_view.model);
            node_inspection.update(hit_test_result, final, mode);
            graph_view.node_view.model.data_source.setv({ inspected: node_inspection }, { silent: true });
            var edge_inspection = graph_view.edge_view.model.data_source.selection_manager.get_or_create_inspector(graph_view.edge_view.model);
            var linked_edges = this.get_linked_edges(graph_view.node_view.model.data_source, graph_view.edge_view.model.data_source, 'inspection');
            edge_inspection.update(linked_edges, final, mode);
            //silently set inspected attr to avoid triggering data_source.change event and rerender
            graph_view.edge_view.model.data_source.setv({ inspected: edge_inspection }, { silent: true });
            graph_view.node_view.model.data_source.inspect.emit([graph_view.node_view, { geometry: geometry }]);
            return !node_inspection.is_empty();
        };
        return NodesAndLinkedEdges;
    }(GraphHitTestPolicy));
    exports.NodesAndLinkedEdges = NodesAndLinkedEdges;
    NodesAndLinkedEdges.__name__ = "NodesAndLinkedEdges";
    var EdgesAndLinkedNodes = /** @class */ (function (_super) {
        tslib_1.__extends(EdgesAndLinkedNodes, _super);
        function EdgesAndLinkedNodes(attrs) {
            return _super.call(this, attrs) || this;
        }
        EdgesAndLinkedNodes.prototype.hit_test = function (geometry, graph_view) {
            return this._hit_test_edges(geometry, graph_view);
        };
        EdgesAndLinkedNodes.prototype.get_linked_nodes = function (node_source, edge_source, mode) {
            var e_2, _a;
            var edge_indices = [];
            if (mode == 'selection')
                edge_indices = edge_source.selected.indices;
            else if (mode == 'inspection')
                edge_indices = edge_source.inspected.indices;
            var nodes = [];
            try {
                for (var edge_indices_2 = tslib_1.__values(edge_indices), edge_indices_2_1 = edge_indices_2.next(); !edge_indices_2_1.done; edge_indices_2_1 = edge_indices_2.next()) {
                    var i = edge_indices_2_1.value;
                    nodes.push(edge_source.data.start[i]);
                    nodes.push(edge_source.data.end[i]);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (edge_indices_2_1 && !edge_indices_2_1.done && (_a = edge_indices_2.return))
                        _a.call(edge_indices_2);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            var node_indices = array_1.uniq(nodes).map(function (i) { return arrayable_1.indexOf(node_source.data.index, i); });
            return new selection_1.Selection({ indices: node_indices });
        };
        EdgesAndLinkedNodes.prototype.do_selection = function (hit_test_result, graph, final, mode) {
            if (hit_test_result == null)
                return false;
            var edge_selection = graph.edge_renderer.data_source.selected;
            edge_selection.update(hit_test_result, final, mode);
            var node_selection = graph.node_renderer.data_source.selected;
            var linked_nodes = this.get_linked_nodes(graph.node_renderer.data_source, graph.edge_renderer.data_source, 'selection');
            node_selection.update(linked_nodes, final, mode);
            graph.edge_renderer.data_source._select.emit();
            return !edge_selection.is_empty();
        };
        EdgesAndLinkedNodes.prototype.do_inspection = function (hit_test_result, geometry, graph_view, final, mode) {
            if (hit_test_result == null)
                return false;
            var edge_inspection = graph_view.edge_view.model.data_source.selection_manager.get_or_create_inspector(graph_view.edge_view.model);
            edge_inspection.update(hit_test_result, final, mode);
            graph_view.edge_view.model.data_source.setv({ inspected: edge_inspection }, { silent: true });
            var node_inspection = graph_view.node_view.model.data_source.selection_manager.get_or_create_inspector(graph_view.node_view.model);
            var linked_nodes = this.get_linked_nodes(graph_view.node_view.model.data_source, graph_view.edge_view.model.data_source, 'inspection');
            node_inspection.update(linked_nodes, final, mode);
            // silently set inspected attr to avoid triggering data_source.change event and rerender
            graph_view.node_view.model.data_source.setv({ inspected: node_inspection }, { silent: true });
            graph_view.edge_view.model.data_source.inspect.emit([graph_view.edge_view, { geometry: geometry }]);
            return !edge_inspection.is_empty();
        };
        return EdgesAndLinkedNodes;
    }(GraphHitTestPolicy));
    exports.EdgesAndLinkedNodes = EdgesAndLinkedNodes;
    EdgesAndLinkedNodes.__name__ = "EdgesAndLinkedNodes";
},
/* models/selections/interaction_policy.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var SelectionPolicy = /** @class */ (function (_super) {
        tslib_1.__extends(SelectionPolicy, _super);
        function SelectionPolicy() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        SelectionPolicy.prototype.do_selection = function (hit_test_result, source, final, mode) {
            if (hit_test_result === null) {
                return false;
            }
            else {
                source.selected.update(hit_test_result, final, mode);
                source._select.emit();
                return !source.selected.is_empty();
            }
        };
        return SelectionPolicy;
    }(model_1.Model));
    exports.SelectionPolicy = SelectionPolicy;
    SelectionPolicy.__name__ = "SelectionPolicy";
    var IntersectRenderers = /** @class */ (function (_super) {
        tslib_1.__extends(IntersectRenderers, _super);
        function IntersectRenderers() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        IntersectRenderers.prototype.hit_test = function (geometry, renderer_views) {
            var e_1, _a, e_2, _b;
            var hit_test_result_renderers = [];
            try {
                for (var renderer_views_1 = tslib_1.__values(renderer_views), renderer_views_1_1 = renderer_views_1.next(); !renderer_views_1_1.done; renderer_views_1_1 = renderer_views_1.next()) {
                    var r = renderer_views_1_1.value;
                    var result = r.hit_test(geometry);
                    if (result !== null)
                        hit_test_result_renderers.push(result);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (renderer_views_1_1 && !renderer_views_1_1.done && (_a = renderer_views_1.return))
                        _a.call(renderer_views_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            if (hit_test_result_renderers.length > 0) {
                var hit_test_result = hit_test_result_renderers[0];
                try {
                    for (var hit_test_result_renderers_1 = tslib_1.__values(hit_test_result_renderers), hit_test_result_renderers_1_1 = hit_test_result_renderers_1.next(); !hit_test_result_renderers_1_1.done; hit_test_result_renderers_1_1 = hit_test_result_renderers_1.next()) {
                        var hit_test_result_other = hit_test_result_renderers_1_1.value;
                        hit_test_result.update_through_intersection(hit_test_result_other);
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (hit_test_result_renderers_1_1 && !hit_test_result_renderers_1_1.done && (_b = hit_test_result_renderers_1.return))
                            _b.call(hit_test_result_renderers_1);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
                return hit_test_result;
            }
            else {
                return null;
            }
        };
        return IntersectRenderers;
    }(SelectionPolicy));
    exports.IntersectRenderers = IntersectRenderers;
    IntersectRenderers.__name__ = "IntersectRenderers";
    var UnionRenderers = /** @class */ (function (_super) {
        tslib_1.__extends(UnionRenderers, _super);
        function UnionRenderers() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        UnionRenderers.prototype.hit_test = function (geometry, renderer_views) {
            var e_3, _a, e_4, _b;
            var hit_test_result_renderers = [];
            try {
                for (var renderer_views_2 = tslib_1.__values(renderer_views), renderer_views_2_1 = renderer_views_2.next(); !renderer_views_2_1.done; renderer_views_2_1 = renderer_views_2.next()) {
                    var r = renderer_views_2_1.value;
                    var result = r.hit_test(geometry);
                    if (result !== null)
                        hit_test_result_renderers.push(result);
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (renderer_views_2_1 && !renderer_views_2_1.done && (_a = renderer_views_2.return))
                        _a.call(renderer_views_2);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            if (hit_test_result_renderers.length > 0) {
                var hit_test_result = hit_test_result_renderers[0];
                try {
                    for (var hit_test_result_renderers_2 = tslib_1.__values(hit_test_result_renderers), hit_test_result_renderers_2_1 = hit_test_result_renderers_2.next(); !hit_test_result_renderers_2_1.done; hit_test_result_renderers_2_1 = hit_test_result_renderers_2.next()) {
                        var hit_test_result_other = hit_test_result_renderers_2_1.value;
                        hit_test_result.update_through_union(hit_test_result_other);
                    }
                }
                catch (e_4_1) {
                    e_4 = { error: e_4_1 };
                }
                finally {
                    try {
                        if (hit_test_result_renderers_2_1 && !hit_test_result_renderers_2_1.done && (_b = hit_test_result_renderers_2.return))
                            _b.call(hit_test_result_renderers_2);
                    }
                    finally {
                        if (e_4)
                            throw e_4.error;
                    }
                }
                return hit_test_result;
            }
            else {
                return null;
            }
        };
        return UnionRenderers;
    }(SelectionPolicy));
    exports.UnionRenderers = UnionRenderers;
    UnionRenderers.__name__ = "UnionRenderers";
},
/* core/util/typed_array.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    function concat(array0) {
        var e_1, _a, e_2, _b;
        var arrays = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            arrays[_i - 1] = arguments[_i];
        }
        var n = array0.length;
        try {
            for (var arrays_1 = tslib_1.__values(arrays), arrays_1_1 = arrays_1.next(); !arrays_1_1.done; arrays_1_1 = arrays_1.next()) {
                var array = arrays_1_1.value;
                n += array.length;
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (arrays_1_1 && !arrays_1_1.done && (_a = arrays_1.return))
                    _a.call(arrays_1);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        var result = new array0.constructor(n);
        result.set(array0, 0);
        var i = array0.length;
        try {
            for (var arrays_2 = tslib_1.__values(arrays), arrays_2_1 = arrays_2.next(); !arrays_2_1.done; arrays_2_1 = arrays_2.next()) {
                var array = arrays_2_1.value;
                result.set(array, i);
                i += array.length;
            }
        }
        catch (e_2_1) {
            e_2 = { error: e_2_1 };
        }
        finally {
            try {
                if (arrays_2_1 && !arrays_2_1.done && (_b = arrays_2.return))
                    _b.call(arrays_2);
            }
            finally {
                if (e_2)
                    throw e_2.error;
            }
        }
        return result;
    }
    exports.concat = concat;
},
/* core/util/set.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    function union() {
        var e_1, _a, e_2, _b;
        var sets = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            sets[_i] = arguments[_i];
        }
        var result = new Set();
        try {
            for (var sets_1 = tslib_1.__values(sets), sets_1_1 = sets_1.next(); !sets_1_1.done; sets_1_1 = sets_1.next()) {
                var set = sets_1_1.value;
                try {
                    for (var set_1 = (e_2 = void 0, tslib_1.__values(set)), set_1_1 = set_1.next(); !set_1_1.done; set_1_1 = set_1.next()) {
                        var item = set_1_1.value;
                        result.add(item);
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (set_1_1 && !set_1_1.done && (_b = set_1.return))
                            _b.call(set_1);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (sets_1_1 && !sets_1_1.done && (_a = sets_1.return))
                    _a.call(sets_1);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        return result;
    }
    exports.union = union;
    function intersection(set) {
        var e_3, _a, e_4, _b;
        var sets = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            sets[_i - 1] = arguments[_i];
        }
        var result = new Set();
        try {
            top: for (var set_2 = tslib_1.__values(set), set_2_1 = set_2.next(); !set_2_1.done; set_2_1 = set_2.next()) {
                var item = set_2_1.value;
                try {
                    for (var sets_2 = (e_4 = void 0, tslib_1.__values(sets)), sets_2_1 = sets_2.next(); !sets_2_1.done; sets_2_1 = sets_2.next()) {
                        var other = sets_2_1.value;
                        if (!other.has(item))
                            continue top;
                    }
                }
                catch (e_4_1) {
                    e_4 = { error: e_4_1 };
                }
                finally {
                    try {
                        if (sets_2_1 && !sets_2_1.done && (_b = sets_2.return))
                            _b.call(sets_2);
                    }
                    finally {
                        if (e_4)
                            throw e_4.error;
                    }
                }
                result.add(item);
            }
        }
        catch (e_3_1) {
            e_3 = { error: e_3_1 };
        }
        finally {
            try {
                if (set_2_1 && !set_2_1.done && (_a = set_2.return))
                    _a.call(set_2);
            }
            finally {
                if (e_3)
                    throw e_3.error;
            }
        }
        return result;
    }
    exports.intersection = intersection;
    function difference(set) {
        var e_5, _a;
        var sets = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            sets[_i - 1] = arguments[_i];
        }
        var result = new Set(set);
        try {
            for (var _b = tslib_1.__values(union.apply(void 0, tslib_1.__spread(sets))), _c = _b.next(); !_c.done; _c = _b.next()) {
                var item = _c.value;
                result.delete(item);
            }
        }
        catch (e_5_1) {
            e_5 = { error: e_5_1 };
        }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return))
                    _a.call(_b);
            }
            finally {
                if (e_5)
                    throw e_5.error;
            }
        }
        return result;
    }
    exports.difference = difference;
},
/* document/events.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var has_props_1 = require(129) /* ../core/has_props */;
    var DocumentEvent = /** @class */ (function () {
        function DocumentEvent(document) {
            this.document = document;
        }
        return DocumentEvent;
    }());
    exports.DocumentEvent = DocumentEvent;
    DocumentEvent.__name__ = "DocumentEvent";
    var DocumentEventBatch = /** @class */ (function (_super) {
        tslib_1.__extends(DocumentEventBatch, _super);
        function DocumentEventBatch(document, events, setter_id) {
            var _this = _super.call(this, document) || this;
            _this.events = events;
            _this.setter_id = setter_id;
            return _this;
        }
        return DocumentEventBatch;
    }(DocumentEvent));
    exports.DocumentEventBatch = DocumentEventBatch;
    DocumentEventBatch.__name__ = "DocumentEventBatch";
    var DocumentChangedEvent = /** @class */ (function (_super) {
        tslib_1.__extends(DocumentChangedEvent, _super);
        function DocumentChangedEvent() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return DocumentChangedEvent;
    }(DocumentEvent));
    exports.DocumentChangedEvent = DocumentChangedEvent;
    DocumentChangedEvent.__name__ = "DocumentChangedEvent";
    var MessageSentEvent = /** @class */ (function (_super) {
        tslib_1.__extends(MessageSentEvent, _super);
        function MessageSentEvent(document, msg_type, msg_data) {
            var _this = _super.call(this, document) || this;
            _this.msg_type = msg_type;
            _this.msg_data = msg_data;
            return _this;
        }
        MessageSentEvent.prototype.json = function (_references) {
            var value = this.msg_data;
            var value_json = has_props_1.HasProps._value_to_json(value);
            var value_refs = new Set();
            has_props_1.HasProps._value_record_references(value, value_refs, { recursive: true });
            /* XXX: this will cause all referenced models to be reinitialized
            for (const id in value_refs) {
              references[id] = value_refs[id]
            }
            */
            return {
                kind: "MessageSent",
                msg_type: this.msg_type,
                msg_data: value_json,
            };
        };
        return MessageSentEvent;
    }(DocumentChangedEvent));
    exports.MessageSentEvent = MessageSentEvent;
    MessageSentEvent.__name__ = "MessageSentEvent";
    var ModelChangedEvent = /** @class */ (function (_super) {
        tslib_1.__extends(ModelChangedEvent, _super);
        function ModelChangedEvent(document, model, attr, old, new_, setter_id, hint) {
            var _this = _super.call(this, document) || this;
            _this.model = model;
            _this.attr = attr;
            _this.old = old;
            _this.new_ = new_;
            _this.setter_id = setter_id;
            _this.hint = hint;
            return _this;
        }
        ModelChangedEvent.prototype.json = function (references) {
            var e_1, _a;
            if (this.attr === "id") {
                throw new Error("'id' field should never change, whatever code just set it is wrong");
            }
            if (this.hint != null)
                return this.hint.json(references);
            var value = this.new_;
            var value_json = has_props_1.HasProps._value_to_json(value);
            var value_refs = new Set();
            has_props_1.HasProps._value_record_references(value, value_refs, { recursive: true });
            if (value_refs.has(this.model) && this.model !== value) {
                // we know we don't want a whole new copy of the obj we're
                // patching unless it's also the value itself
                value_refs.delete(this.model);
            }
            try {
                for (var value_refs_1 = tslib_1.__values(value_refs), value_refs_1_1 = value_refs_1.next(); !value_refs_1_1.done; value_refs_1_1 = value_refs_1.next()) {
                    var ref = value_refs_1_1.value;
                    references.add(ref);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (value_refs_1_1 && !value_refs_1_1.done && (_a = value_refs_1.return))
                        _a.call(value_refs_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return {
                kind: "ModelChanged",
                model: this.model.ref(),
                attr: this.attr,
                new: value_json,
            };
        };
        return ModelChangedEvent;
    }(DocumentChangedEvent));
    exports.ModelChangedEvent = ModelChangedEvent;
    ModelChangedEvent.__name__ = "ModelChangedEvent";
    var ColumnsPatchedEvent = /** @class */ (function (_super) {
        tslib_1.__extends(ColumnsPatchedEvent, _super);
        function ColumnsPatchedEvent(document, column_source, patches) {
            var _this = _super.call(this, document) || this;
            _this.column_source = column_source;
            _this.patches = patches;
            return _this;
        }
        ColumnsPatchedEvent.prototype.json = function (_references) {
            return {
                kind: "ColumnsPatched",
                column_source: this.column_source,
                patches: this.patches,
            };
        };
        return ColumnsPatchedEvent;
    }(DocumentChangedEvent));
    exports.ColumnsPatchedEvent = ColumnsPatchedEvent;
    ColumnsPatchedEvent.__name__ = "ColumnsPatchedEvent";
    var ColumnsStreamedEvent = /** @class */ (function (_super) {
        tslib_1.__extends(ColumnsStreamedEvent, _super);
        function ColumnsStreamedEvent(document, column_source, data, rollover) {
            var _this = _super.call(this, document) || this;
            _this.column_source = column_source;
            _this.data = data;
            _this.rollover = rollover;
            return _this;
        }
        ColumnsStreamedEvent.prototype.json = function (_references) {
            return {
                kind: "ColumnsStreamed",
                column_source: this.column_source,
                data: this.data,
                rollover: this.rollover,
            };
        };
        return ColumnsStreamedEvent;
    }(DocumentChangedEvent));
    exports.ColumnsStreamedEvent = ColumnsStreamedEvent;
    ColumnsStreamedEvent.__name__ = "ColumnsStreamedEvent";
    var TitleChangedEvent = /** @class */ (function (_super) {
        tslib_1.__extends(TitleChangedEvent, _super);
        function TitleChangedEvent(document, title, setter_id) {
            var _this = _super.call(this, document) || this;
            _this.title = title;
            _this.setter_id = setter_id;
            return _this;
        }
        TitleChangedEvent.prototype.json = function (_references) {
            return {
                kind: "TitleChanged",
                title: this.title,
            };
        };
        return TitleChangedEvent;
    }(DocumentChangedEvent));
    exports.TitleChangedEvent = TitleChangedEvent;
    TitleChangedEvent.__name__ = "TitleChangedEvent";
    var RootAddedEvent = /** @class */ (function (_super) {
        tslib_1.__extends(RootAddedEvent, _super);
        function RootAddedEvent(document, model, setter_id) {
            var _this = _super.call(this, document) || this;
            _this.model = model;
            _this.setter_id = setter_id;
            return _this;
        }
        RootAddedEvent.prototype.json = function (references) {
            has_props_1.HasProps._value_record_references(this.model, references, { recursive: true });
            return {
                kind: "RootAdded",
                model: this.model.ref(),
            };
        };
        return RootAddedEvent;
    }(DocumentChangedEvent));
    exports.RootAddedEvent = RootAddedEvent;
    RootAddedEvent.__name__ = "RootAddedEvent";
    var RootRemovedEvent = /** @class */ (function (_super) {
        tslib_1.__extends(RootRemovedEvent, _super);
        function RootRemovedEvent(document, model, setter_id) {
            var _this = _super.call(this, document) || this;
            _this.model = model;
            _this.setter_id = setter_id;
            return _this;
        }
        RootRemovedEvent.prototype.json = function (_references) {
            return {
                kind: "RootRemoved",
                model: this.model.ref(),
            };
        };
        return RootRemovedEvent;
    }(DocumentChangedEvent));
    exports.RootRemovedEvent = RootRemovedEvent;
    RootRemovedEvent.__name__ = "RootRemovedEvent";
},
/* models/annotations/band.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var upper_lower_1 = require(238) /* ./upper_lower */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var BandView = /** @class */ (function (_super) {
        tslib_1.__extends(BandView, _super);
        function BandView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BandView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            var update = function () { return _this.set_data(_this.model.source); };
            this.connect(this.model.change, update);
            this.connect(this.model.source.streaming, update);
            this.connect(this.model.source.patching, update);
            this.connect(this.model.source.change, update);
        };
        BandView.prototype._render = function () {
            this._map_data();
            var ctx = this.layer.ctx;
            // Draw the band body
            ctx.beginPath();
            ctx.moveTo(this._lower_sx[0], this._lower_sy[0]);
            for (var i = 0, end = this._lower_sx.length; i < end; i++) {
                ctx.lineTo(this._lower_sx[i], this._lower_sy[i]);
            }
            // iterate backwards so that the upper end is below the lower start
            for (var i = this._upper_sx.length - 1; i >= 0; i--) {
                ctx.lineTo(this._upper_sx[i], this._upper_sy[i]);
            }
            ctx.closePath();
            if (this.visuals.fill.doit) {
                this.visuals.fill.set_value(ctx);
                ctx.fill();
            }
            // Draw the lower band edge
            ctx.beginPath();
            ctx.moveTo(this._lower_sx[0], this._lower_sy[0]);
            for (var i = 0, end = this._lower_sx.length; i < end; i++) {
                ctx.lineTo(this._lower_sx[i], this._lower_sy[i]);
            }
            if (this.visuals.line.doit) {
                this.visuals.line.set_value(ctx);
                ctx.stroke();
            }
            // Draw the upper band edge
            ctx.beginPath();
            ctx.moveTo(this._upper_sx[0], this._upper_sy[0]);
            for (var i = 0, end = this._upper_sx.length; i < end; i++) {
                ctx.lineTo(this._upper_sx[i], this._upper_sy[i]);
            }
            if (this.visuals.line.doit) {
                this.visuals.line.set_value(ctx);
                ctx.stroke();
            }
        };
        return BandView;
    }(upper_lower_1.UpperLowerView));
    exports.BandView = BandView;
    BandView.__name__ = "BandView";
    var Band = /** @class */ (function (_super) {
        tslib_1.__extends(Band, _super);
        function Band(attrs) {
            return _super.call(this, attrs) || this;
        }
        Band.init_Band = function () {
            this.prototype.default_view = BandView;
            this.mixins([mixins.Line /*Scalar*/, mixins.Fill /*Scalar*/]);
            this.override({
                fill_color: "#fff9ba",
                fill_alpha: 0.4,
                line_color: "#cccccc",
                line_alpha: 0.3,
            });
        };
        return Band;
    }(upper_lower_1.UpperLower));
    exports.Band = Band;
    Band.__name__ = "Band";
    Band.init_Band();
},
/* models/annotations/upper_lower.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var column_data_source_1 = require(200) /* ../sources/column_data_source */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var UpperLowerView = /** @class */ (function (_super) {
        tslib_1.__extends(UpperLowerView, _super);
        function UpperLowerView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        UpperLowerView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.set_data(this.model.source);
        };
        UpperLowerView.prototype.set_data = function (source) {
            _super.prototype.set_data.call(this, source);
            this.visuals.warm_cache(source);
            this.plot_view.request_render();
        };
        UpperLowerView.prototype._map_data = function () {
            var frame = this.plot_view.frame;
            var dim = this.model.dimension;
            var xscale = this.coordinates.x_scale;
            var yscale = this.coordinates.y_scale;
            var limit_scale = dim == "height" ? yscale : xscale;
            var base_scale = dim == "height" ? xscale : yscale;
            var limit_view = dim == "height" ? frame.yview : frame.xview;
            var base_view = dim == "height" ? frame.xview : frame.yview;
            var _lower_sx;
            if (this.model.properties.lower.units == "data")
                _lower_sx = limit_scale.v_compute(this._lower);
            else
                _lower_sx = limit_view.v_compute(this._lower);
            var _upper_sx;
            if (this.model.properties.upper.units == "data")
                _upper_sx = limit_scale.v_compute(this._upper);
            else
                _upper_sx = limit_view.v_compute(this._upper);
            var _base_sx;
            if (this.model.properties.base.units == "data")
                _base_sx = base_scale.v_compute(this._base);
            else
                _base_sx = base_view.v_compute(this._base);
            var _a = tslib_1.__read(dim == 'height' ? [1, 0] : [0, 1], 2), i = _a[0], j = _a[1];
            var _lower = [_lower_sx, _base_sx];
            var _upper = [_upper_sx, _base_sx];
            this._lower_sx = _lower[i];
            this._lower_sy = _lower[j];
            this._upper_sx = _upper[i];
            this._upper_sy = _upper[j];
        };
        return UpperLowerView;
    }(annotation_1.AnnotationView));
    exports.UpperLowerView = UpperLowerView;
    UpperLowerView.__name__ = "UpperLowerView";
    var UpperLower = /** @class */ (function (_super) {
        tslib_1.__extends(UpperLower, _super);
        function UpperLower(attrs) {
            return _super.call(this, attrs) || this;
        }
        UpperLower.init_UpperLower = function () {
            this.define({
                lower: [p.DistanceSpec],
                upper: [p.DistanceSpec],
                base: [p.DistanceSpec],
                dimension: [p.Dimension, 'height'],
                source: [p.Instance, function () { return new column_data_source_1.ColumnDataSource(); }],
            });
        };
        return UpperLower;
    }(annotation_1.Annotation));
    exports.UpperLower = UpperLower;
    UpperLower.__name__ = "UpperLower";
    UpperLower.init_UpperLower();
},
/* models/annotations/box_annotation.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var signaling_1 = require(130) /* ../../core/signaling */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var bbox_1 = require(194) /* ../../core/util/bbox */;
    exports.EDGE_TOLERANCE = 2.5;
    var BoxAnnotationView = /** @class */ (function (_super) {
        tslib_1.__extends(BoxAnnotationView, _super);
        function BoxAnnotationView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BoxAnnotationView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.change, function () { return _this.plot_view.request_paint(_this); });
            this.connect(this.model.data_update, function () { return _this.plot_view.request_paint(_this); });
        };
        BoxAnnotationView.prototype._render = function () {
            var _this = this;
            // don't render if *all* position are null
            if (this.model.left == null && this.model.right == null && this.model.top == null && this.model.bottom == null) {
                return;
            }
            var frame = this.plot_view.frame;
            var xscale = this.coordinates.x_scale;
            var yscale = this.coordinates.y_scale;
            var _calc_dim = function (dim, dim_units, scale, view, frame_extrema) {
                var sdim;
                if (dim != null) {
                    if (_this.model.screen)
                        sdim = dim;
                    else {
                        if (dim_units == 'data')
                            sdim = scale.compute(dim);
                        else
                            sdim = view.compute(dim);
                    }
                }
                else
                    sdim = frame_extrema;
                return sdim;
            };
            this.sleft = _calc_dim(this.model.left, this.model.left_units, xscale, frame.xview, frame.bbox.left);
            this.sright = _calc_dim(this.model.right, this.model.right_units, xscale, frame.xview, frame.bbox.right);
            this.stop = _calc_dim(this.model.top, this.model.top_units, yscale, frame.yview, frame.bbox.top);
            this.sbottom = _calc_dim(this.model.bottom, this.model.bottom_units, yscale, frame.yview, frame.bbox.bottom);
            this._paint_box(this.sleft, this.sright, this.sbottom, this.stop);
        };
        BoxAnnotationView.prototype._paint_box = function (sleft, sright, sbottom, stop) {
            var ctx = this.layer.ctx;
            ctx.save();
            ctx.beginPath();
            ctx.rect(sleft, stop, sright - sleft, sbottom - stop);
            if (this.visuals.fill.doit) {
                this.visuals.fill.set_value(ctx);
                ctx.fill();
            }
            if (this.visuals.line.doit) {
                this.visuals.line.set_value(ctx);
                ctx.stroke();
            }
            ctx.restore();
        };
        BoxAnnotationView.prototype.interactive_bbox = function () {
            var tol = this.model.properties.line_width.value() + exports.EDGE_TOLERANCE;
            return new bbox_1.BBox({
                x0: this.sleft - tol,
                y0: this.stop - tol,
                x1: this.sright + tol,
                y1: this.sbottom + tol,
            });
        };
        BoxAnnotationView.prototype.interactive_hit = function (sx, sy) {
            if (this.model.in_cursor == null)
                return false;
            var bbox = this.interactive_bbox();
            return bbox.contains(sx, sy);
        };
        BoxAnnotationView.prototype.cursor = function (sx, sy) {
            var tol = 3;
            if (Math.abs(sx - this.sleft) < tol || Math.abs(sx - this.sright) < tol)
                return this.model.ew_cursor;
            else if (Math.abs(sy - this.sbottom) < tol || Math.abs(sy - this.stop) < tol)
                return this.model.ns_cursor;
            else if (sx > this.sleft && sx < this.sright && sy > this.stop && sy < this.sbottom)
                return this.model.in_cursor;
            else
                return null;
        };
        return BoxAnnotationView;
    }(annotation_1.AnnotationView));
    exports.BoxAnnotationView = BoxAnnotationView;
    BoxAnnotationView.__name__ = "BoxAnnotationView";
    var BoxAnnotation = /** @class */ (function (_super) {
        tslib_1.__extends(BoxAnnotation, _super);
        function BoxAnnotation(attrs) {
            return _super.call(this, attrs) || this;
        }
        BoxAnnotation.init_BoxAnnotation = function () {
            this.prototype.default_view = BoxAnnotationView;
            this.mixins([mixins.Line /*Scalar*/, mixins.Fill /*Scalar*/]);
            this.define({
                render_mode: [p.RenderMode, 'canvas'],
                top: [p.Number, null],
                top_units: [p.SpatialUnits, 'data'],
                bottom: [p.Number, null],
                bottom_units: [p.SpatialUnits, 'data'],
                left: [p.Number, null],
                left_units: [p.SpatialUnits, 'data'],
                right: [p.Number, null],
                right_units: [p.SpatialUnits, 'data'],
            });
            this.internal({
                screen: [p.Boolean, false],
                ew_cursor: [p.String, null],
                ns_cursor: [p.String, null],
                in_cursor: [p.String, null],
            });
            this.override({
                fill_color: '#fff9ba',
                fill_alpha: 0.4,
                line_color: '#cccccc',
                line_alpha: 0.3,
            });
        };
        BoxAnnotation.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.data_update = new signaling_1.Signal0(this, "data_update");
        };
        BoxAnnotation.prototype.update = function (_a) {
            var left = _a.left, right = _a.right, top = _a.top, bottom = _a.bottom;
            this.setv({ left: left, right: right, top: top, bottom: bottom, screen: true }, { silent: true });
            this.data_update.emit();
        };
        return BoxAnnotation;
    }(annotation_1.Annotation));
    exports.BoxAnnotation = BoxAnnotation;
    BoxAnnotation.__name__ = "BoxAnnotation";
    BoxAnnotation.init_BoxAnnotation();
},
/* models/annotations/color_bar.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var basic_ticker_1 = require(241) /* ../tickers/basic_ticker */;
    var basic_tick_formatter_1 = require(245) /* ../formatters/basic_tick_formatter */;
    var mappers_1 = require(247) /* ../mappers */;
    var linear_scale_1 = require(260) /* ../scales/linear_scale */;
    var linear_interpolation_scale_1 = require(271) /* ../scales/linear_interpolation_scale */;
    var log_scale_1 = require(272) /* ../scales/log_scale */;
    var range1d_1 = require(273) /* ../ranges/range1d */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var text_util = tslib_1.__importStar(require(274) /* ../../core/util/text */);
    var array_1 = require(124) /* ../../core/util/array */;
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var types_1 = require(123) /* ../../core/util/types */;
    var assert_1 = require(126) /* ../../core/util/assert */;
    var SHORT_DIM = 25;
    var LONG_DIM_MIN_SCALAR = 0.3;
    var LONG_DIM_MAX_SCALAR = 0.8;
    var ColorBarView = /** @class */ (function (_super) {
        tslib_1.__extends(ColorBarView, _super);
        function ColorBarView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ColorBarView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._set_canvas_image();
        };
        ColorBarView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.ticker.change, function () { return _this.plot_view.request_render(); });
            this.connect(this.model.formatter.change, function () { return _this.plot_view.request_render(); });
            if (this.model.color_mapper != null) {
                this.connect(this.model.color_mapper.change, function () {
                    _this._set_canvas_image();
                    _this.plot_view.request_render();
                });
            }
        };
        ColorBarView.prototype._get_size = function () {
            if (this.model.color_mapper == null)
                return { width: 0, height: 0 };
            else {
                var _a = this.compute_legend_dimensions(), width = _a.width, height = _a.height;
                return { width: width, height: height };
            }
        };
        ColorBarView.prototype._set_canvas_image = function () {
            var _a, _b;
            if (this.model.color_mapper == null)
                return;
            var palette = this.model.color_mapper.palette;
            if (this.model.orientation == 'vertical')
                palette = array_1.reversed(palette);
            var w, h;
            switch (this.model.orientation) {
                case "vertical": {
                    _a = tslib_1.__read([1, palette.length], 2), w = _a[0], h = _a[1];
                    break;
                }
                case "horizontal": {
                    _b = tslib_1.__read([palette.length, 1], 2), w = _b[0], h = _b[1];
                    break;
                }
            }
            var canvas = document.createElement('canvas');
            canvas.width = w;
            canvas.height = h;
            var image_ctx = canvas.getContext('2d');
            var image_data = image_ctx.getImageData(0, 0, w, h);
            // We always want to draw the entire palette linearly, so we create a new
            // LinearColorMapper instance and map a monotonic range of values with
            // length = palette.length to get each palette color in order.
            var cmap = new mappers_1.LinearColorMapper({ palette: palette }).rgba_mapper;
            var buf8 = cmap.v_compute(array_1.range(0, palette.length));
            image_data.data.set(buf8);
            image_ctx.putImageData(image_data, 0, 0);
            this.image = canvas;
        };
        ColorBarView.prototype.compute_legend_dimensions = function () {
            var image_dimensions = this._computed_image_dimensions();
            var _a = tslib_1.__read([image_dimensions.height, image_dimensions.width], 2), image_height = _a[0], image_width = _a[1];
            var label_extent = this._get_label_extent();
            var title_extent = this._title_extent();
            var tick_extent = this._tick_extent();
            var padding = this.model.padding;
            var legend_height, legend_width;
            switch (this.model.orientation) {
                case "vertical":
                    legend_height = image_height + title_extent + 2 * padding;
                    legend_width = image_width + tick_extent + label_extent + 2 * padding;
                    break;
                case "horizontal":
                    legend_height = image_height + title_extent + tick_extent + label_extent + 2 * padding;
                    legend_width = image_width + 2 * padding;
                    break;
            }
            return { width: legend_width, height: legend_height };
        };
        ColorBarView.prototype.compute_legend_location = function () {
            var legend_dimensions = this.compute_legend_dimensions();
            var _a = tslib_1.__read([legend_dimensions.height, legend_dimensions.width], 2), legend_height = _a[0], legend_width = _a[1];
            var legend_margin = this.model.margin;
            var panel = this.panel != null ? this.panel : this.plot_view.frame;
            var _b = tslib_1.__read(panel.bbox.ranges, 2), hr = _b[0], vr = _b[1];
            var location = this.model.location;
            var sx, sy;
            if (types_1.isString(location)) {
                switch (location) {
                    case 'top_left':
                        sx = hr.start + legend_margin;
                        sy = vr.start + legend_margin;
                        break;
                    case 'top_center':
                        sx = (hr.end + hr.start) / 2 - legend_width / 2;
                        sy = vr.start + legend_margin;
                        break;
                    case 'top_right':
                        sx = hr.end - legend_margin - legend_width;
                        sy = vr.start + legend_margin;
                        break;
                    case 'bottom_right':
                        sx = hr.end - legend_margin - legend_width;
                        sy = vr.end - legend_margin - legend_height;
                        break;
                    case 'bottom_center':
                        sx = (hr.end + hr.start) / 2 - legend_width / 2;
                        sy = vr.end - legend_margin - legend_height;
                        break;
                    case 'bottom_left':
                        sx = hr.start + legend_margin;
                        sy = vr.end - legend_margin - legend_height;
                        break;
                    case 'center_left':
                        sx = hr.start + legend_margin;
                        sy = (vr.end + vr.start) / 2 - legend_height / 2;
                        break;
                    case 'center':
                        sx = (hr.end + hr.start) / 2 - legend_width / 2;
                        sy = (vr.end + vr.start) / 2 - legend_height / 2;
                        break;
                    case 'center_right':
                        sx = hr.end - legend_margin - legend_width;
                        sy = (vr.end + vr.start) / 2 - legend_height / 2;
                        break;
                }
            }
            else if (types_1.isArray(location) && location.length == 2) {
                var _c = tslib_1.__read(location, 2), vx = _c[0], vy = _c[1];
                sx = panel.xview.compute(vx);
                sy = panel.yview.compute(vy) - legend_height;
            }
            else
                assert_1.unreachable();
            return { sx: sx, sy: sy };
        };
        ColorBarView.prototype._render = function () {
            if (this.model.color_mapper == null)
                return;
            var ctx = this.layer.ctx;
            ctx.save();
            var _a = this.compute_legend_location(), sx = _a.sx, sy = _a.sy;
            ctx.translate(sx, sy);
            this._draw_bbox(ctx);
            var image_offset = this._get_image_offset();
            ctx.translate(image_offset.x, image_offset.y);
            this._draw_image(ctx);
            var tick_info = this.tick_info();
            this._draw_major_ticks(ctx, tick_info);
            this._draw_minor_ticks(ctx, tick_info);
            this._draw_major_labels(ctx, tick_info);
            if (this.model.title)
                this._draw_title(ctx);
            ctx.restore();
        };
        ColorBarView.prototype._draw_bbox = function (ctx) {
            var bbox = this.compute_legend_dimensions();
            ctx.save();
            if (this.visuals.background_fill.doit) {
                this.visuals.background_fill.set_value(ctx);
                ctx.fillRect(0, 0, bbox.width, bbox.height);
            }
            if (this.visuals.border_line.doit) {
                this.visuals.border_line.set_value(ctx);
                ctx.strokeRect(0, 0, bbox.width, bbox.height);
            }
            ctx.restore();
        };
        ColorBarView.prototype._draw_image = function (ctx) {
            var image = this._computed_image_dimensions();
            ctx.save();
            ctx.setImageSmoothingEnabled(false);
            ctx.globalAlpha = this.model.scale_alpha;
            ctx.drawImage(this.image, 0, 0, image.width, image.height);
            if (this.visuals.bar_line.doit) {
                this.visuals.bar_line.set_value(ctx);
                ctx.strokeRect(0, 0, image.width, image.height);
            }
            ctx.restore();
        };
        ColorBarView.prototype._draw_major_ticks = function (ctx, tick_info) {
            if (!this.visuals.major_tick_line.doit)
                return;
            var _a = tslib_1.__read(this._normals(), 2), nx = _a[0], ny = _a[1];
            var image = this._computed_image_dimensions();
            var _b = tslib_1.__read([image.width * nx, image.height * ny], 2), x_offset = _b[0], y_offset = _b[1];
            var _c = tslib_1.__read(tick_info.coords.major, 2), sx = _c[0], sy = _c[1];
            var tin = this.model.major_tick_in;
            var tout = this.model.major_tick_out;
            ctx.save();
            ctx.translate(x_offset, y_offset);
            this.visuals.major_tick_line.set_value(ctx);
            for (var i = 0, end = sx.length; i < end; i++) {
                ctx.beginPath();
                ctx.moveTo(Math.round(sx[i] + nx * tout), Math.round(sy[i] + ny * tout));
                ctx.lineTo(Math.round(sx[i] - nx * tin), Math.round(sy[i] - ny * tin));
                ctx.stroke();
            }
            ctx.restore();
        };
        ColorBarView.prototype._draw_minor_ticks = function (ctx, tick_info) {
            if (!this.visuals.minor_tick_line.doit)
                return;
            var _a = tslib_1.__read(this._normals(), 2), nx = _a[0], ny = _a[1];
            var image = this._computed_image_dimensions();
            var _b = tslib_1.__read([image.width * nx, image.height * ny], 2), x_offset = _b[0], y_offset = _b[1];
            var _c = tslib_1.__read(tick_info.coords.minor, 2), sx = _c[0], sy = _c[1];
            var tin = this.model.minor_tick_in;
            var tout = this.model.minor_tick_out;
            ctx.save();
            ctx.translate(x_offset, y_offset);
            this.visuals.minor_tick_line.set_value(ctx);
            for (var i = 0, end = sx.length; i < end; i++) {
                ctx.beginPath();
                ctx.moveTo(Math.round(sx[i] + nx * tout), Math.round(sy[i] + ny * tout));
                ctx.lineTo(Math.round(sx[i] - nx * tin), Math.round(sy[i] - ny * tin));
                ctx.stroke();
            }
            ctx.restore();
        };
        ColorBarView.prototype._draw_major_labels = function (ctx, tick_info) {
            if (!this.visuals.major_label_text.doit)
                return;
            var _a = tslib_1.__read(this._normals(), 2), nx = _a[0], ny = _a[1];
            var image = this._computed_image_dimensions();
            var _b = tslib_1.__read([image.width * nx, image.height * ny], 2), x_offset = _b[0], y_offset = _b[1];
            var standoff = (this.model.label_standoff + this._tick_extent());
            var _c = tslib_1.__read([standoff * nx, standoff * ny], 2), x_standoff = _c[0], y_standoff = _c[1];
            var _d = tslib_1.__read(tick_info.coords.major, 2), sx = _d[0], sy = _d[1];
            var formatted_labels = tick_info.labels.major;
            this.visuals.major_label_text.set_value(ctx);
            ctx.save();
            ctx.translate(x_offset + x_standoff, y_offset + y_standoff);
            for (var i = 0, end = sx.length; i < end; i++) {
                ctx.fillText(formatted_labels[i], Math.round(sx[i] + nx * this.model.label_standoff), Math.round(sy[i] + ny * this.model.label_standoff));
            }
            ctx.restore();
        };
        ColorBarView.prototype._draw_title = function (ctx) {
            if (!this.visuals.title_text.doit)
                return;
            ctx.save();
            this.visuals.title_text.set_value(ctx);
            ctx.fillText(this.model.title, 0, -this.model.title_standoff);
            ctx.restore();
        };
        /*protected*/ ColorBarView.prototype._get_label_extent = function () {
            var major_labels = this.tick_info().labels.major;
            var label_extent;
            if (!array_1.is_empty(major_labels)) {
                var ctx_1 = this.layer.ctx;
                ctx_1.save();
                this.visuals.major_label_text.set_value(ctx_1);
                switch (this.model.orientation) {
                    case "vertical":
                        label_extent = array_1.max((major_labels.map(function (label) { return ctx_1.measureText(label.toString()).width; })));
                        break;
                    case "horizontal":
                        label_extent = text_util.measure_font(this.visuals.major_label_text.font_value()).height;
                        break;
                }
                label_extent += this.model.label_standoff;
                ctx_1.restore();
            }
            else
                label_extent = 0;
            return label_extent;
        };
        /*protected*/ ColorBarView.prototype._get_image_offset = function () {
            // Returns image offset relative to legend bounding box
            var x = this.model.padding;
            var y = this.model.padding + this._title_extent();
            return { x: x, y: y };
        };
        // {{{ TODO: state
        ColorBarView.prototype._normals = function () {
            return this.model.orientation == 'vertical' ? [1, 0] : [0, 1];
        };
        ColorBarView.prototype._title_extent = function () {
            var font_value = this.model.title_text_font + " " + this.model.title_text_font_size + " " + this.model.title_text_font_style;
            var title_extent = this.model.title ? text_util.measure_font(font_value).height + this.model.title_standoff : 0;
            return title_extent;
        };
        ColorBarView.prototype._tick_extent = function () {
            return array_1.max([this.model.major_tick_out, this.model.minor_tick_out]);
        };
        ColorBarView.prototype._computed_image_dimensions = function () {
            /*
            Heuristics to determine ColorBar image dimensions if set to "auto"
        
            Note: Returns the height/width values for the ColorBar's scale image, not
            the dimensions of the entire ColorBar.
        
            If the short dimension (the width of a vertical bar or height of a
            horizontal bar) is set to "auto", the resulting dimension will be set to
            25 px.
        
            For a ColorBar in a side panel with the long dimension (the height of a
            vertical bar or width of a horizontal bar) set to "auto", the
            resulting dimension will be as long as the adjacent frame edge, so that the
            bar "fits" to the plot.
        
            For a ColorBar in the plot frame with the long dimension set to "auto", the
            resulting dimension will be the greater of:
              * The length of the color palette * 25px
              * The parallel frame dimension * 0.30
                (i.e the frame height for a vertical ColorBar)
            But not greater than:
              * The parallel frame dimension * 0.80
            */
            var frame_height = this.plot_view.frame.bbox.height;
            var frame_width = this.plot_view.frame.bbox.width;
            var title_extent = this._title_extent();
            var height, width;
            switch (this.model.orientation) {
                case "vertical": {
                    if (this.model.height == 'auto') {
                        if (this.panel != null)
                            height = frame_height - 2 * this.model.padding - title_extent;
                        else {
                            height = array_1.max([this.model.color_mapper.palette.length * SHORT_DIM, frame_height * LONG_DIM_MIN_SCALAR]);
                            height = array_1.min([height, frame_height * LONG_DIM_MAX_SCALAR - 2 * this.model.padding - title_extent]);
                        }
                    }
                    else
                        height = this.model.height;
                    width = this.model.width == 'auto' ? SHORT_DIM : this.model.width;
                    break;
                }
                case "horizontal": {
                    height = this.model.height == 'auto' ? SHORT_DIM : this.model.height;
                    if (this.model.width == 'auto') {
                        if (this.panel != null)
                            width = frame_width - 2 * this.model.padding;
                        else {
                            width = array_1.max([this.model.color_mapper.palette.length * SHORT_DIM, frame_width * LONG_DIM_MIN_SCALAR]);
                            width = array_1.min([width, frame_width * LONG_DIM_MAX_SCALAR - 2 * this.model.padding]);
                        }
                    }
                    else
                        width = this.model.width;
                    break;
                }
            }
            return { width: width, height: height };
        };
        /*protected*/ ColorBarView.prototype._tick_coordinate_scale = function (scale_length) {
            /*
            Creates and returns a scale instance that maps the `color_mapper` range
            (low to high) to a screen space range equal to the length of the ColorBar's
            scale image. The scale is used to calculate the tick coordinates in screen
            coordinates for plotting purposes.
        
            Note: the type of color_mapper has to match the type of scale (i.e.
            a LinearColorMapper will require a corresponding LinearScale instance).
            */
            var ranges = {
                source_range: new range1d_1.Range1d({
                    start: this.model.color_mapper.metrics.min,
                    end: this.model.color_mapper.metrics.max,
                }),
                target_range: new range1d_1.Range1d({
                    start: 0,
                    end: scale_length,
                }),
            };
            var color_mapper = this.model.color_mapper;
            if (color_mapper instanceof mappers_1.LinearColorMapper)
                return new linear_scale_1.LinearScale(ranges);
            else if (color_mapper instanceof mappers_1.LogColorMapper)
                return new log_scale_1.LogScale(ranges);
            else if (color_mapper instanceof mappers_1.ScanningColorMapper) {
                var binning = color_mapper.metrics.binning;
                return new linear_interpolation_scale_1.LinearInterpolationScale(Object.assign(Object.assign({}, ranges), { binning: binning }));
            }
            else {
                // TODO: Categorical*Mapper needs painters
                assert_1.unreachable();
            }
        };
        ColorBarView.prototype._format_major_labels = function (initial_labels, major_ticks) {
            // XXX: passing null as cross_loc probably means MercatorTickFormatters, etc
            // will not function properly in conjunction with colorbars
            var formatted_labels = this.model.formatter.doFormat(initial_labels, null);
            for (var i = 0, end = major_ticks.length; i < end; i++) {
                if (major_ticks[i] in this.model.major_label_overrides)
                    formatted_labels[i] = this.model.major_label_overrides[major_ticks[i]];
            }
            return formatted_labels;
        };
        ColorBarView.prototype.tick_info = function () {
            var image_dimensions = this._computed_image_dimensions();
            var scale_length;
            switch (this.model.orientation) {
                case "vertical": {
                    scale_length = image_dimensions.height;
                    break;
                }
                case "horizontal": {
                    scale_length = image_dimensions.width;
                    break;
                }
            }
            var scale = this._tick_coordinate_scale(scale_length);
            var _a = tslib_1.__read(this._normals(), 2), i = _a[0], j = _a[1];
            var _b = tslib_1.__read([this.model.color_mapper.metrics.min, this.model.color_mapper.metrics.max], 2), start = _b[0], end = _b[1];
            // XXX: passing null as cross_loc probably means MercatorTickers, etc
            // will not function properly in conjunction with colorbars
            var ticks = this.model.ticker.get_ticks(start, end, null, null, this.model.ticker.desired_num_ticks);
            var majors = ticks.major;
            var minors = ticks.minor;
            var major_coords = [[], []];
            var minor_coords = [[], []];
            for (var ii = 0, _end = majors.length; ii < _end; ii++) {
                if (majors[ii] < start || majors[ii] > end)
                    continue;
                major_coords[i].push(majors[ii]);
                major_coords[j].push(0);
            }
            for (var ii = 0, _end = minors.length; ii < _end; ii++) {
                if (minors[ii] < start || minors[ii] > end)
                    continue;
                minor_coords[i].push(minors[ii]);
                minor_coords[j].push(0);
            }
            var labels = { major: this._format_major_labels(major_coords[i], majors) };
            var coords = {
                major: [[], []],
                minor: [[], []],
            };
            coords.major[i] = scale.v_compute(major_coords[i]);
            coords.minor[i] = scale.v_compute(minor_coords[i]);
            coords.major[j] = major_coords[j];
            coords.minor[j] = minor_coords[j];
            // Because we want the scale to be reversed
            if (this.model.orientation == 'vertical') {
                coords.major[i] = arrayable_1.map(coords.major[i], function (coord) { return scale_length - coord; });
                coords.minor[i] = arrayable_1.map(coords.minor[i], function (coord) { return scale_length - coord; });
            }
            return { coords: coords, labels: labels };
        };
        return ColorBarView;
    }(annotation_1.AnnotationView));
    exports.ColorBarView = ColorBarView;
    ColorBarView.__name__ = "ColorBarView";
    var ColorBar = /** @class */ (function (_super) {
        tslib_1.__extends(ColorBar, _super);
        function ColorBar(attrs) {
            return _super.call(this, attrs) || this;
        }
        ColorBar.init_ColorBar = function () {
            this.prototype.default_view = ColorBarView;
            this.mixins([
                ["major_label_", mixins.Text],
                ["title_", mixins.Text],
                ["major_tick_", mixins.Line],
                ["minor_tick_", mixins.Line],
                ["border_", mixins.Line],
                ["bar_", mixins.Line],
                ["background_", mixins.Fill],
            ]);
            this.define({
                location: [p.Any, 'top_right'],
                orientation: [p.Orientation, 'vertical'],
                title: [p.String],
                title_standoff: [p.Number, 2],
                width: [p.Any, 'auto'],
                height: [p.Any, 'auto'],
                scale_alpha: [p.Number, 1.0],
                ticker: [p.Instance, function () { return new basic_ticker_1.BasicTicker(); }],
                formatter: [p.Instance, function () { return new basic_tick_formatter_1.BasicTickFormatter(); }],
                major_label_overrides: [p.Any, {}],
                color_mapper: [p.Instance],
                label_standoff: [p.Number, 5],
                margin: [p.Number, 30],
                padding: [p.Number, 10],
                major_tick_in: [p.Number, 5],
                major_tick_out: [p.Number, 0],
                minor_tick_in: [p.Number, 0],
                minor_tick_out: [p.Number, 0],
            });
            this.override({
                background_fill_color: "#ffffff",
                background_fill_alpha: 0.95,
                bar_line_color: null,
                border_line_color: null,
                major_label_text_align: "center",
                major_label_text_baseline: "middle",
                major_label_text_font_size: "11px",
                major_tick_line_color: "#ffffff",
                minor_tick_line_color: null,
                title_text_font_size: "13px",
                title_text_font_style: "italic",
            });
        };
        return ColorBar;
    }(annotation_1.Annotation));
    exports.ColorBar = ColorBar;
    ColorBar.__name__ = "ColorBar";
    ColorBar.init_ColorBar();
},
/* models/tickers/basic_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var adaptive_ticker_1 = require(242) /* ./adaptive_ticker */;
    var BasicTicker = /** @class */ (function (_super) {
        tslib_1.__extends(BasicTicker, _super);
        function BasicTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        return BasicTicker;
    }(adaptive_ticker_1.AdaptiveTicker));
    exports.BasicTicker = BasicTicker;
    BasicTicker.__name__ = "BasicTicker";
},
/* models/tickers/adaptive_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var continuous_ticker_1 = require(243) /* ./continuous_ticker */;
    var array_1 = require(124) /* ../../core/util/array */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    // Forces a number x into a specified range [min_val, max_val].
    function clamp(x, min_val, max_val) {
        return Math.max(min_val, Math.min(max_val, x));
    }
    // A log function with an optional base.
    function log(x, base) {
        if (base === void 0) {
            base = Math.E;
        }
        return Math.log(x) / Math.log(base);
    }
    var AdaptiveTicker = /** @class */ (function (_super) {
        tslib_1.__extends(AdaptiveTicker, _super);
        function AdaptiveTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        AdaptiveTicker.init_AdaptiveTicker = function () {
            this.define({
                base: [p.Number, 10.0],
                mantissas: [p.Array, [1, 2, 5]],
                min_interval: [p.Number, 0.0],
                max_interval: [p.Number],
            });
        };
        // These arguments control the range of possible intervals.  The interval I
        // returned by get_interval() will be the one that most closely matches the
        // desired number of ticks, subject to the following constraints:
        // I = (M * B^N), where
        // M is a member of mantissas,
        // B is base,
        // and N is an integer;
        // and min_interval <= I <= max_interval.
        AdaptiveTicker.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            var prefix_mantissa = array_1.nth(this.mantissas, -1) / this.base;
            var suffix_mantissa = array_1.nth(this.mantissas, 0) * this.base;
            this.extended_mantissas = tslib_1.__spread([prefix_mantissa], this.mantissas, [suffix_mantissa]);
            this.base_factor = this.get_min_interval() === 0.0 ? 1.0 : this.get_min_interval();
        };
        AdaptiveTicker.prototype.get_interval = function (data_low, data_high, desired_n_ticks) {
            var data_range = data_high - data_low;
            var ideal_interval = this.get_ideal_interval(data_low, data_high, desired_n_ticks);
            var interval_exponent = Math.floor(log(ideal_interval / this.base_factor, this.base));
            var ideal_magnitude = Math.pow(this.base, interval_exponent) * this.base_factor;
            // An untested optimization.
            //   const ideal_mantissa = ideal_interval / ideal_magnitude
            //   index = sorted_index(this.extended_mantissas, ideal_mantissa)
            //   candidate_mantissas = this.extended_mantissas[index..index + 1]
            var candidate_mantissas = this.extended_mantissas;
            var errors = candidate_mantissas.map(function (mantissa) {
                return Math.abs(desired_n_ticks - (data_range / (mantissa * ideal_magnitude)));
            });
            var best_mantissa = candidate_mantissas[array_1.argmin(errors)];
            var interval = best_mantissa * ideal_magnitude;
            return clamp(interval, this.get_min_interval(), this.get_max_interval());
        };
        return AdaptiveTicker;
    }(continuous_ticker_1.ContinuousTicker));
    exports.AdaptiveTicker = AdaptiveTicker;
    AdaptiveTicker.__name__ = "AdaptiveTicker";
    AdaptiveTicker.init_AdaptiveTicker();
},
/* models/tickers/continuous_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var ticker_1 = require(244) /* ./ticker */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    var ContinuousTicker = /** @class */ (function (_super) {
        tslib_1.__extends(ContinuousTicker, _super);
        function ContinuousTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        ContinuousTicker.init_ContinuousTicker = function () {
            this.define({
                num_minor_ticks: [p.Number, 5],
                desired_num_ticks: [p.Number, 6],
            });
        };
        ContinuousTicker.prototype.get_ticks = function (data_low, data_high, _range, cross_loc, _) {
            return this.get_ticks_no_defaults(data_low, data_high, cross_loc, this.desired_num_ticks);
        };
        // The version of get_ticks() that does the work (and the version that
        // should be overridden in subclasses).
        ContinuousTicker.prototype.get_ticks_no_defaults = function (data_low, data_high, _cross_loc, desired_n_ticks) {
            var e_1, _a, e_2, _b, e_3, _c;
            var interval = this.get_interval(data_low, data_high, desired_n_ticks);
            var start_factor = Math.floor(data_low / interval);
            var end_factor = Math.ceil(data_high / interval);
            var factors;
            if (!isFinite(start_factor) || !isFinite(end_factor))
                factors = [];
            else
                factors = array_1.range(start_factor, end_factor + 1);
            var ticks = factors
                .map(function (factor) { return factor * interval; })
                .filter(function (tick) { return data_low <= tick && tick <= data_high; });
            var num_minor_ticks = this.num_minor_ticks;
            var minor_ticks = [];
            if (num_minor_ticks > 0 && ticks.length > 0) {
                var minor_interval_1 = interval / num_minor_ticks;
                var minor_offsets = array_1.range(0, num_minor_ticks).map(function (i) { return i * minor_interval_1; });
                try {
                    for (var _d = tslib_1.__values(minor_offsets.slice(1)), _e = _d.next(); !_e.done; _e = _d.next()) {
                        var x = _e.value;
                        var mt = ticks[0] - x;
                        if (data_low <= mt && mt <= data_high) {
                            minor_ticks.push(mt);
                        }
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (_e && !_e.done && (_a = _d.return))
                            _a.call(_d);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
                try {
                    for (var ticks_1 = tslib_1.__values(ticks), ticks_1_1 = ticks_1.next(); !ticks_1_1.done; ticks_1_1 = ticks_1.next()) {
                        var tick = ticks_1_1.value;
                        try {
                            for (var minor_offsets_1 = (e_3 = void 0, tslib_1.__values(minor_offsets)), minor_offsets_1_1 = minor_offsets_1.next(); !minor_offsets_1_1.done; minor_offsets_1_1 = minor_offsets_1.next()) {
                                var x = minor_offsets_1_1.value;
                                var mt = tick + x;
                                if (data_low <= mt && mt <= data_high) {
                                    minor_ticks.push(mt);
                                }
                            }
                        }
                        catch (e_3_1) {
                            e_3 = { error: e_3_1 };
                        }
                        finally {
                            try {
                                if (minor_offsets_1_1 && !minor_offsets_1_1.done && (_c = minor_offsets_1.return))
                                    _c.call(minor_offsets_1);
                            }
                            finally {
                                if (e_3)
                                    throw e_3.error;
                            }
                        }
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (ticks_1_1 && !ticks_1_1.done && (_b = ticks_1.return))
                            _b.call(ticks_1);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
            }
            return {
                major: ticks,
                minor: minor_ticks,
            };
        };
        // Returns the smallest interval that can be returned by get_interval().
        ContinuousTicker.prototype.get_min_interval = function () {
            return this.min_interval;
        };
        // Returns the largest interval that can be returned by get_interval().
        ContinuousTicker.prototype.get_max_interval = function () {
            return this.max_interval != null ? this.max_interval : Infinity;
        };
        // Returns the interval size that would produce exactly the number of
        // desired ticks.  (In general we won't use exactly this interval, because
        // we want the ticks to be round numbers.)
        ContinuousTicker.prototype.get_ideal_interval = function (data_low, data_high, desired_n_ticks) {
            var data_range = data_high - data_low;
            return data_range / desired_n_ticks;
        };
        return ContinuousTicker;
    }(ticker_1.Ticker));
    exports.ContinuousTicker = ContinuousTicker;
    ContinuousTicker.__name__ = "ContinuousTicker";
    ContinuousTicker.init_ContinuousTicker();
},
/* models/tickers/ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var Ticker = /** @class */ (function (_super) {
        tslib_1.__extends(Ticker, _super);
        function Ticker(attrs) {
            return _super.call(this, attrs) || this;
        }
        return Ticker;
    }(model_1.Model));
    exports.Ticker = Ticker;
    Ticker.__name__ = "Ticker";
},
/* models/formatters/basic_tick_formatter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var tick_formatter_1 = require(246) /* ./tick_formatter */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var BasicTickFormatter = /** @class */ (function (_super) {
        tslib_1.__extends(BasicTickFormatter, _super);
        function BasicTickFormatter(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.last_precision = 3;
            return _this;
        }
        BasicTickFormatter.init_BasicTickFormatter = function () {
            this.define({
                precision: [p.Any, 'auto'],
                use_scientific: [p.Boolean, true],
                power_limit_high: [p.Number, 5],
                power_limit_low: [p.Number, -3],
            });
        };
        Object.defineProperty(BasicTickFormatter.prototype, "scientific_limit_low", {
            get: function () {
                return Math.pow(10.0, this.power_limit_low);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BasicTickFormatter.prototype, "scientific_limit_high", {
            get: function () {
                return Math.pow(10.0, this.power_limit_high);
            },
            enumerable: true,
            configurable: true
        });
        BasicTickFormatter.prototype._need_sci = function (ticks) {
            var e_1, _a;
            if (!this.use_scientific)
                return false;
            var scientific_limit_high = this.scientific_limit_high;
            var scientific_limit_low = this.scientific_limit_low;
            var zeroish = ticks.length < 2 ? 0 : Math.abs(ticks[1] - ticks[0]) / 10000;
            try {
                for (var ticks_1 = tslib_1.__values(ticks), ticks_1_1 = ticks_1.next(); !ticks_1_1.done; ticks_1_1 = ticks_1.next()) {
                    var tick = ticks_1_1.value;
                    var tick_abs = Math.abs(tick);
                    if (tick_abs <= zeroish)
                        continue;
                    if (tick_abs >= scientific_limit_high || tick_abs <= scientific_limit_low) {
                        return true;
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (ticks_1_1 && !ticks_1_1.done && (_a = ticks_1.return))
                        _a.call(ticks_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return false;
        };
        BasicTickFormatter.prototype._format_with_precision = function (ticks, need_sci, precision) {
            var labels = new Array(ticks.length);
            if (need_sci) {
                for (var i = 0, end = ticks.length; i < end; i++) {
                    labels[i] = ticks[i].toExponential(precision);
                }
            }
            else {
                for (var i = 0, end = ticks.length; i < end; i++) {
                    // strip trailing zeros
                    labels[i] = ticks[i].toFixed(precision).replace(/(\.[0-9]*?)0+$/, "$1").replace(/\.$/, "");
                }
            }
            return labels;
        };
        BasicTickFormatter.prototype._auto_precision = function (ticks, need_sci) {
            var labels = new Array(ticks.length);
            var asc = this.last_precision <= 15;
            outer: for (var x = this.last_precision; asc ? x <= 15 : x >= 1; asc ? x++ : x--) {
                if (need_sci) {
                    labels[0] = ticks[0].toExponential(x);
                    for (var i = 1; i < ticks.length; i++) {
                        if (labels[i] == labels[i - 1]) {
                            continue outer;
                        }
                    }
                    this.last_precision = x;
                    break;
                }
                else {
                    labels[0] = ticks[0].toFixed(x).replace(/(\.[0-9]*?)0+$/, "$1").replace(/\.$/, "");
                    for (var i = 1; i < ticks.length; i++) {
                        labels[i] = ticks[i].toFixed(x).replace(/(\.[0-9]*?)0+$/, "$1").replace(/\.$/, "");
                        if (labels[i] == labels[i - 1]) {
                            continue outer;
                        }
                    }
                    this.last_precision = x;
                    break;
                }
            }
            return this.last_precision;
        };
        BasicTickFormatter.prototype.doFormat = function (ticks, _opts) {
            if (ticks.length == 0)
                return [];
            var need_sci = this._need_sci(ticks);
            var precision = this.precision == "auto" ? this._auto_precision(ticks, need_sci) : this.precision;
            return this._format_with_precision(ticks, need_sci, precision);
        };
        return BasicTickFormatter;
    }(tick_formatter_1.TickFormatter));
    exports.BasicTickFormatter = BasicTickFormatter;
    BasicTickFormatter.__name__ = "BasicTickFormatter";
    BasicTickFormatter.init_BasicTickFormatter();
},
/* models/formatters/tick_formatter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var TickFormatter = /** @class */ (function (_super) {
        tslib_1.__extends(TickFormatter, _super);
        function TickFormatter(attrs) {
            return _super.call(this, attrs) || this;
        }
        return TickFormatter;
    }(model_1.Model));
    exports.TickFormatter = TickFormatter;
    TickFormatter.__name__ = "TickFormatter";
},
/* models/mappers/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var categorical_color_mapper_1 = require(248) /* ./categorical_color_mapper */;
    exports.CategoricalColorMapper = categorical_color_mapper_1.CategoricalColorMapper;
    var categorical_marker_mapper_1 = require(253) /* ./categorical_marker_mapper */;
    exports.CategoricalMarkerMapper = categorical_marker_mapper_1.CategoricalMarkerMapper;
    var categorical_pattern_mapper_1 = require(254) /* ./categorical_pattern_mapper */;
    exports.CategoricalPatternMapper = categorical_pattern_mapper_1.CategoricalPatternMapper;
    var continuous_color_mapper_1 = require(255) /* ./continuous_color_mapper */;
    exports.ContinuousColorMapper = continuous_color_mapper_1.ContinuousColorMapper;
    var color_mapper_1 = require(250) /* ./color_mapper */;
    exports.ColorMapper = color_mapper_1.ColorMapper;
    var linear_color_mapper_1 = require(256) /* ./linear_color_mapper */;
    exports.LinearColorMapper = linear_color_mapper_1.LinearColorMapper;
    var log_color_mapper_1 = require(257) /* ./log_color_mapper */;
    exports.LogColorMapper = log_color_mapper_1.LogColorMapper;
    var scanning_color_mapper_1 = require(258) /* ./scanning_color_mapper */;
    exports.ScanningColorMapper = scanning_color_mapper_1.ScanningColorMapper;
    var eqhist_color_mapper_1 = require(259) /* ./eqhist_color_mapper */;
    exports.EqHistColorMapper = eqhist_color_mapper_1.EqHistColorMapper;
},
/* models/mappers/categorical_color_mapper.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var categorical_mapper_1 = require(249) /* ./categorical_mapper */;
    var color_mapper_1 = require(250) /* ./color_mapper */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var CategoricalColorMapper = /** @class */ (function (_super) {
        tslib_1.__extends(CategoricalColorMapper, _super);
        function CategoricalColorMapper(attrs) {
            return _super.call(this, attrs) || this;
        }
        CategoricalColorMapper.init_CategoricalColorMapper = function () {
            this.define({
                factors: [p.Array],
                start: [p.Number, 0],
                end: [p.Number],
            });
        };
        CategoricalColorMapper.prototype._v_compute = function (data, values, palette, _a) {
            var nan_color = _a.nan_color;
            categorical_mapper_1.cat_v_compute(data, this.factors, palette, values, this.start, this.end, nan_color);
        };
        return CategoricalColorMapper;
    }(color_mapper_1.ColorMapper));
    exports.CategoricalColorMapper = CategoricalColorMapper;
    CategoricalColorMapper.__name__ = "CategoricalColorMapper";
    CategoricalColorMapper.init_CategoricalColorMapper();
},
/* models/mappers/categorical_mapper.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var types_1 = require(123) /* ../../core/util/types */;
    function _cat_equals(a, b) {
        if (a.length != b.length)
            return false;
        for (var i = 0, end = a.length; i < end; i++) {
            if (a[i] !== b[i])
                return false;
        }
        return true;
    }
    exports._cat_equals = _cat_equals;
    function cat_v_compute(data, factors, targets, values, start, end, extra_value) {
        var _loop_1 = function (i, N) {
            var d = data[i];
            var key = void 0;
            if (types_1.isString(d))
                key = arrayable_1.index_of(factors, d);
            else {
                if (start != null) {
                    if (end != null)
                        d = d.slice(start, end);
                    else
                        d = d.slice(start);
                }
                else if (end != null)
                    d = d.slice(0, end);
                if (d.length == 1)
                    key = arrayable_1.index_of(factors, d[0]);
                else
                    key = arrayable_1.find_index(factors, function (x) { return _cat_equals(x, d); });
            }
            var value = void 0;
            if (key < 0 || key >= targets.length)
                value = extra_value;
            else
                value = targets[key];
            values[i] = value;
        };
        for (var i = 0, N = data.length; i < N; i++) {
            _loop_1(i, N);
        }
    }
    exports.cat_v_compute = cat_v_compute;
},
/* models/mappers/color_mapper.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var mapper_1 = require(251) /* ./mapper */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(123) /* ../../core/util/types */;
    var color_1 = require(137) /* ../../core/util/color */;
    var compat_1 = require(147) /* ../../core/util/compat */;
    function _convert_color(color) {
        if (types_1.isNumber(color))
            return color;
        if (color[0] != "#")
            color = color_1.color2hex(color);
        if (color.length != 9)
            color = color + 'ff';
        return parseInt(color.slice(1), 16);
    }
    exports._convert_color = _convert_color;
    function _convert_palette(palette) {
        var new_palette = new Uint32Array(palette.length);
        for (var i = 0, end = palette.length; i < end; i++)
            new_palette[i] = _convert_color(palette[i]);
        return new_palette;
    }
    exports._convert_palette = _convert_palette;
    function _uint32_to_rgba(values) {
        if (compat_1.is_little_endian) {
            var view = new DataView(values.buffer);
            for (var i = 0, end = values.length; i < end; i++)
                view.setUint32(i * 4, values[i]);
        }
        return new Uint8Array(values.buffer);
    }
    exports._uint32_to_rgba = _uint32_to_rgba;
    var ColorMapper = /** @class */ (function (_super) {
        tslib_1.__extends(ColorMapper, _super);
        function ColorMapper(attrs) {
            return _super.call(this, attrs) || this;
        }
        ColorMapper.init_ColorMapper = function () {
            this.define({
                palette: [p.Any],
                nan_color: [p.Color, "gray"],
            });
        };
        ColorMapper.prototype.v_compute = function (xs) {
            var values = new Array(xs.length);
            this._v_compute(xs, values, this.palette, this._colors(function (c) { return c; }));
            return values;
        };
        Object.defineProperty(ColorMapper.prototype, "rgba_mapper", {
            get: function () {
                var self = this;
                var palette = _convert_palette(this.palette);
                var colors = this._colors(_convert_color);
                return {
                    v_compute: function (xs) {
                        var values = new Uint32Array(xs.length);
                        self._v_compute(xs, values, palette, colors);
                        return _uint32_to_rgba(values);
                    },
                };
            },
            enumerable: true,
            configurable: true
        });
        ColorMapper.prototype._colors = function (conv) {
            return { nan_color: conv(this.nan_color) };
        };
        return ColorMapper;
    }(mapper_1.Mapper));
    exports.ColorMapper = ColorMapper;
    ColorMapper.__name__ = "ColorMapper";
    ColorMapper.init_ColorMapper();
},
/* models/mappers/mapper.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var transform_1 = require(252) /* ../transforms/transform */;
    var Mapper = /** @class */ (function (_super) {
        tslib_1.__extends(Mapper, _super);
        function Mapper(attrs) {
            return _super.call(this, attrs) || this;
        }
        Mapper.prototype.compute = function (_x) {
            // If it's just a single value, then a mapper doesn't really make sense.
            throw new Error("mapping single values is not supported");
        };
        return Mapper;
    }(transform_1.Transform));
    exports.Mapper = Mapper;
    Mapper.__name__ = "Mapper";
},
/* models/transforms/transform.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var Transform = /** @class */ (function (_super) {
        tslib_1.__extends(Transform, _super);
        function Transform(attrs) {
            return _super.call(this, attrs) || this;
        }
        return Transform;
    }(model_1.Model));
    exports.Transform = Transform;
    Transform.__name__ = "Transform";
},
/* models/mappers/categorical_marker_mapper.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var categorical_mapper_1 = require(249) /* ./categorical_mapper */;
    var mapper_1 = require(251) /* ./mapper */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var CategoricalMarkerMapper = /** @class */ (function (_super) {
        tslib_1.__extends(CategoricalMarkerMapper, _super);
        function CategoricalMarkerMapper(attrs) {
            return _super.call(this, attrs) || this;
        }
        CategoricalMarkerMapper.init_CategoricalMarkerMapper = function () {
            this.define({
                factors: [p.Array],
                markers: [p.Array],
                start: [p.Number, 0],
                end: [p.Number],
                default_value: [p.MarkerType, "circle"],
            });
        };
        CategoricalMarkerMapper.prototype.v_compute = function (xs) {
            var values = new Array(xs.length);
            categorical_mapper_1.cat_v_compute(xs, this.factors, this.markers, values, this.start, this.end, this.default_value);
            return values;
        };
        return CategoricalMarkerMapper;
    }(mapper_1.Mapper));
    exports.CategoricalMarkerMapper = CategoricalMarkerMapper;
    CategoricalMarkerMapper.__name__ = "CategoricalMarkerMapper";
    CategoricalMarkerMapper.init_CategoricalMarkerMapper();
},
/* models/mappers/categorical_pattern_mapper.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var categorical_mapper_1 = require(249) /* ./categorical_mapper */;
    var mapper_1 = require(251) /* ./mapper */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var CategoricalPatternMapper = /** @class */ (function (_super) {
        tslib_1.__extends(CategoricalPatternMapper, _super);
        function CategoricalPatternMapper(attrs) {
            return _super.call(this, attrs) || this;
        }
        CategoricalPatternMapper.init_CategoricalPatternMapper = function () {
            this.define({
                factors: [p.Array],
                patterns: [p.Array],
                start: [p.Number, 0],
                end: [p.Number],
                default_value: [p.HatchPatternType, " "],
            });
        };
        CategoricalPatternMapper.prototype.v_compute = function (xs) {
            var values = new Array(xs.length);
            categorical_mapper_1.cat_v_compute(xs, this.factors, this.patterns, values, this.start, this.end, this.default_value);
            return values;
        };
        return CategoricalPatternMapper;
    }(mapper_1.Mapper));
    exports.CategoricalPatternMapper = CategoricalPatternMapper;
    CategoricalPatternMapper.__name__ = "CategoricalPatternMapper";
    CategoricalPatternMapper.init_CategoricalPatternMapper();
},
/* models/mappers/continuous_color_mapper.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var color_mapper_1 = require(250) /* ./color_mapper */;
    var glyph_renderer_1 = require(205) /* ../renderers/glyph_renderer */;
    var array_1 = require(124) /* ../../core/util/array */;
    var types_1 = require(123) /* ../../core/util/types */;
    var ContinuousColorMapper = /** @class */ (function (_super) {
        tslib_1.__extends(ContinuousColorMapper, _super);
        function ContinuousColorMapper(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this._scan_data = null;
            return _this;
        }
        ContinuousColorMapper.init_ContinuousColorMapper = function () {
            this.define(function (_a) {
                var Number = _a.Number, String = _a.String, Null = _a.Null, Ref = _a.Ref, Color = _a.Color, Or = _a.Or, Tuple = _a.Tuple, Array = _a.Array;
                return {
                    high: [Or(Number, Null), null],
                    low: [Or(Number, Null), null],
                    high_color: [Or(Color, Null), null],
                    low_color: [Or(Color, Null), null],
                    domain: [Array(Tuple(Ref(glyph_renderer_1.GlyphRenderer), Or(String, Array(String)))), []],
                };
            });
        };
        ContinuousColorMapper.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            var connect_renderers = function () {
                var e_1, _a;
                try {
                    // TODO: if already connected this will bail. However, it won't remove old connections.
                    for (var _b = tslib_1.__values(_this.domain), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var _d = tslib_1.__read(_c.value, 1), renderer = _d[0];
                        _this.connect(renderer.view.change, function () { return _this.update_data(); });
                        _this.connect(renderer.data_source.selected.change, function () { return _this.update_data(); });
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return))
                            _a.call(_b);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
            };
            this.connect(this.properties.domain.change, function () { return connect_renderers(); });
            connect_renderers();
        };
        ContinuousColorMapper.prototype.update_data = function () {
            var _a = this, domain = _a.domain, palette = _a.palette;
            var all_data = tslib_1.__spread(this._collect(domain));
            this._scan_data = this.scan(all_data, palette.length);
            this.change.emit();
        };
        Object.defineProperty(ContinuousColorMapper.prototype, "metrics", {
            get: function () {
                if (this._scan_data == null) {
                    this.update_data();
                }
                return this._scan_data;
            },
            enumerable: true,
            configurable: true
        });
        ContinuousColorMapper.prototype._collect = function (domain) {
            var domain_1, domain_1_1, _a, renderer, fields, _loop_1, _b, _c, field, e_2_1, e_3_1;
            var e_3, _d, e_2, _e;
            return tslib_1.__generator(this, function (_f) {
                switch (_f.label) {
                    case 0:
                        _f.trys.push([0, 11, 12, 13]);
                        domain_1 = tslib_1.__values(domain), domain_1_1 = domain_1.next();
                        _f.label = 1;
                    case 1:
                        if (!!domain_1_1.done)
                            return [3 /*break*/, 10];
                        _a = tslib_1.__read(domain_1_1.value, 2), renderer = _a[0], fields = _a[1];
                        _loop_1 = function (field) {
                            var array, masked, selected, subset, array_2, array_2_1, subarray, e_4_1;
                            var e_4, _a;
                            return tslib_1.__generator(this, function (_b) {
                                switch (_b.label) {
                                    case 0:
                                        array = renderer.data_source.get_column(field);
                                        array = renderer.view.indices.select(array);
                                        masked = renderer.view.masked;
                                        selected = renderer.data_source.selected.indices;
                                        subset = void 0;
                                        if (masked != null && selected.length > 0)
                                            subset = array_1.intersection(tslib_1.__spread(masked), selected);
                                        else if (masked != null)
                                            subset = tslib_1.__spread(masked);
                                        else if (selected.length > 0)
                                            subset = selected;
                                        if (subset != null) {
                                            array = array_1.map(subset, function (i) { return array[i]; });
                                        }
                                        if (!(array.length > 0 && !types_1.isNumber(array[0])))
                                            return [3 /*break*/, 9];
                                        _b.label = 1;
                                    case 1:
                                        _b.trys.push([1, 6, 7, 8]);
                                        array_2 = (e_4 = void 0, tslib_1.__values(array)), array_2_1 = array_2.next();
                                        _b.label = 2;
                                    case 2:
                                        if (!!array_2_1.done)
                                            return [3 /*break*/, 5];
                                        subarray = array_2_1.value;
                                        return [5 /*yield**/, tslib_1.__values(subarray)];
                                    case 3:
                                        _b.sent();
                                        _b.label = 4;
                                    case 4:
                                        array_2_1 = array_2.next();
                                        return [3 /*break*/, 2];
                                    case 5: return [3 /*break*/, 8];
                                    case 6:
                                        e_4_1 = _b.sent();
                                        e_4 = { error: e_4_1 };
                                        return [3 /*break*/, 8];
                                    case 7:
                                        try {
                                            if (array_2_1 && !array_2_1.done && (_a = array_2.return))
                                                _a.call(array_2);
                                        }
                                        finally {
                                            if (e_4)
                                                throw e_4.error;
                                        }
                                        return [7 /*endfinally*/];
                                    case 8: return [3 /*break*/, 11];
                                    case 9: return [5 /*yield**/, tslib_1.__values(array)];
                                    case 10:
                                        _b.sent();
                                        _b.label = 11;
                                    case 11: return [2 /*return*/];
                                }
                            });
                        };
                        _f.label = 2;
                    case 2:
                        _f.trys.push([2, 7, 8, 9]);
                        _b = (e_2 = void 0, tslib_1.__values(types_1.isArray(fields) ? fields : [fields])), _c = _b.next();
                        _f.label = 3;
                    case 3:
                        if (!!_c.done)
                            return [3 /*break*/, 6];
                        field = _c.value;
                        return [5 /*yield**/, _loop_1(field)];
                    case 4:
                        _f.sent();
                        _f.label = 5;
                    case 5:
                        _c = _b.next();
                        return [3 /*break*/, 3];
                    case 6: return [3 /*break*/, 9];
                    case 7:
                        e_2_1 = _f.sent();
                        e_2 = { error: e_2_1 };
                        return [3 /*break*/, 9];
                    case 8:
                        try {
                            if (_c && !_c.done && (_e = _b.return))
                                _e.call(_b);
                        }
                        finally {
                            if (e_2)
                                throw e_2.error;
                        }
                        return [7 /*endfinally*/];
                    case 9:
                        domain_1_1 = domain_1.next();
                        return [3 /*break*/, 1];
                    case 10: return [3 /*break*/, 13];
                    case 11:
                        e_3_1 = _f.sent();
                        e_3 = { error: e_3_1 };
                        return [3 /*break*/, 13];
                    case 12:
                        try {
                            if (domain_1_1 && !domain_1_1.done && (_d = domain_1.return))
                                _d.call(domain_1);
                        }
                        finally {
                            if (e_3)
                                throw e_3.error;
                        }
                        return [7 /*endfinally*/];
                    case 13: return [2 /*return*/];
                }
            });
        };
        ContinuousColorMapper.prototype._v_compute = function (data, values, palette, colors) {
            var nan_color = colors.nan_color;
            var low_color = colors.low_color, high_color = colors.high_color;
            if (low_color == null)
                low_color = palette[0];
            if (high_color == null)
                high_color = palette[palette.length - 1];
            var domain = this.domain;
            var all_data = !array_1.is_empty(domain) ? tslib_1.__spread(this._collect(domain)) : data;
            this._scan_data = this.scan(all_data, palette.length);
            for (var i = 0, end = data.length; i < end; i++) {
                var d = data[i];
                if (isNaN(d))
                    values[i] = nan_color;
                else
                    values[i] = this.cmap(d, palette, low_color, high_color, this._scan_data);
            }
        };
        ContinuousColorMapper.prototype._colors = function (conv) {
            return Object.assign(Object.assign({}, _super.prototype._colors.call(this, conv)), { low_color: this.low_color != null ? conv(this.low_color) : undefined, high_color: this.high_color != null ? conv(this.high_color) : undefined });
        };
        return ContinuousColorMapper;
    }(color_mapper_1.ColorMapper));
    exports.ContinuousColorMapper = ContinuousColorMapper;
    ContinuousColorMapper.__name__ = "ContinuousColorMapper";
    ContinuousColorMapper.init_ContinuousColorMapper();
},
/* models/mappers/linear_color_mapper.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var continuous_color_mapper_1 = require(255) /* ./continuous_color_mapper */;
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var LinearColorMapper = /** @class */ (function (_super) {
        tslib_1.__extends(LinearColorMapper, _super);
        function LinearColorMapper(attrs) {
            return _super.call(this, attrs) || this;
        }
        LinearColorMapper.prototype.scan = function (data, n) {
            var low = this.low != null ? this.low : arrayable_1.min(data);
            var high = this.high != null ? this.high : arrayable_1.max(data);
            var norm_factor = 1 / (high - low);
            var normed_interval = 1 / n;
            return { max: high, min: low, norm_factor: norm_factor, normed_interval: normed_interval };
        };
        LinearColorMapper.prototype.cmap = function (d, palette, low_color, high_color, scan_data) {
            // This handles the edge case where d == high, since the code below maps
            // values exactly equal to high to palette.length, which is greater than
            // max_key
            var max_key = palette.length - 1;
            if (d == scan_data.max) {
                return palette[max_key];
            }
            var normed_d = (d - scan_data.min) * scan_data.norm_factor;
            var key = Math.floor(normed_d / scan_data.normed_interval);
            if (key < 0)
                return low_color;
            else if (key > max_key)
                return high_color;
            else
                return palette[key];
        };
        return LinearColorMapper;
    }(continuous_color_mapper_1.ContinuousColorMapper));
    exports.LinearColorMapper = LinearColorMapper;
    LinearColorMapper.__name__ = "LinearColorMapper";
},
/* models/mappers/log_color_mapper.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var continuous_color_mapper_1 = require(255) /* ./continuous_color_mapper */;
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var LogColorMapper = /** @class */ (function (_super) {
        tslib_1.__extends(LogColorMapper, _super);
        function LogColorMapper(attrs) {
            return _super.call(this, attrs) || this;
        }
        LogColorMapper.prototype.scan = function (data, n) {
            var low = this.low != null ? this.low : arrayable_1.min(data);
            var high = this.high != null ? this.high : arrayable_1.max(data);
            var scale = n / (Math.log(high) - Math.log(low)); // subtract the low offset
            return { max: high, min: low, scale: scale };
        };
        LogColorMapper.prototype.cmap = function (d, palette, low_color, high_color, scan_data) {
            var max_key = palette.length - 1;
            if (d > scan_data.max) {
                return high_color;
            }
            // This handles the edge case where d == high, since the code below maps
            // values exactly equal to high to palette.length, which is greater than
            // max_key
            if (d == scan_data.max)
                return palette[max_key];
            else if (d < scan_data.min)
                return low_color;
            // Get the key
            var log = Math.log(d) - Math.log(scan_data.min); // subtract the low offset
            var key = Math.floor(log * scan_data.scale);
            // Deal with upper bound
            if (key > max_key) {
                key = max_key;
            }
            return palette[key];
        };
        return LogColorMapper;
    }(continuous_color_mapper_1.ContinuousColorMapper));
    exports.LogColorMapper = LogColorMapper;
    LogColorMapper.__name__ = "LogColorMapper";
},
/* models/mappers/scanning_color_mapper.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var continuous_color_mapper_1 = require(255) /* ./continuous_color_mapper */;
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var ScanningColorMapper = /** @class */ (function (_super) {
        tslib_1.__extends(ScanningColorMapper, _super);
        function ScanningColorMapper(attrs) {
            return _super.call(this, attrs) || this;
        }
        ScanningColorMapper.prototype.cmap = function (d, palette, low_color, high_color, edges) {
            if (d < edges.binning[0])
                return low_color;
            if (d > edges.binning[edges.binning.length - 1])
                return high_color;
            var key = arrayable_1.left_edge_index(d, edges.binning);
            return palette[key];
        };
        return ScanningColorMapper;
    }(continuous_color_mapper_1.ContinuousColorMapper));
    exports.ScanningColorMapper = ScanningColorMapper;
    ScanningColorMapper.__name__ = "ScanningColorMapper";
},
/* models/mappers/eqhist_color_mapper.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var scanning_color_mapper_1 = require(258) /* ./scanning_color_mapper */;
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var array_1 = require(124) /* ../../core/util/array */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var logging_1 = require(134) /* ../../core/logging */;
    var EqHistColorMapper = /** @class */ (function (_super) {
        tslib_1.__extends(EqHistColorMapper, _super);
        function EqHistColorMapper(attrs) {
            return _super.call(this, attrs) || this;
        }
        EqHistColorMapper.init_EqHistColorMapper = function () {
            this.define({
                bins: [p.Int, 256 * 256],
            });
        };
        EqHistColorMapper.prototype.scan = function (data, n) {
            var low = this.low != null ? this.low : arrayable_1.min(data);
            var high = this.high != null ? this.high : arrayable_1.max(data);
            var nbins = this.bins;
            var eq_bin_edges = array_1.linspace(low, high, nbins + 1);
            var hist = arrayable_1.bin_counts(data, eq_bin_edges);
            var eq_bin_centers = new Array(nbins);
            for (var i = 0, length = eq_bin_edges.length; i < length - 1; i++) {
                var left = eq_bin_edges[i];
                var right = eq_bin_edges[i + 1];
                eq_bin_centers[i] = (left + right) / 2;
            }
            // CDFs
            var cdf = array_1.cumsum(hist);
            var cdf_max = cdf[cdf.length - 1];
            var norm_cdf = arrayable_1.map(cdf, function (x) { return x / cdf_max; });
            // Iteratively find as many finite bins as there are colors
            var finite_bins = n - 1;
            var binning = [];
            var iterations = 0;
            var guess = n * 2;
            while ((finite_bins != n) && (iterations < 4) && (finite_bins != 0)) {
                var ratio = guess / finite_bins;
                if (ratio > 1000) {
                    // Abort if distribution is extremely skewed
                    break;
                }
                guess = Math.round(Math.max(n * ratio, n));
                // Interpolate
                var palette_edges = array_1.range(0, guess);
                var palette_cdf = arrayable_1.map(norm_cdf, function (x) { return x * (guess - 1); });
                binning = arrayable_1.interpolate(palette_edges, palette_cdf, eq_bin_centers);
                // Evaluate binning
                var uniq_bins = array_1.uniq(binning);
                finite_bins = uniq_bins.length - 1;
                iterations++;
            }
            if (finite_bins == 0) {
                binning = [low, high];
                for (var j = 0; j < n - 1; j++)
                    binning.push(high);
            }
            else {
                binning = binning.slice(binning.length - n - 1);
                if (finite_bins != n)
                    logging_1.logger.warn("EqHistColorMapper warning: Histogram equalization did not converge.");
            }
            return { min: low, max: high, binning: binning };
        };
        return EqHistColorMapper;
    }(scanning_color_mapper_1.ScanningColorMapper));
    exports.EqHistColorMapper = EqHistColorMapper;
    EqHistColorMapper.__name__ = "EqHistColorMapper";
    EqHistColorMapper.init_EqHistColorMapper();
},
/* models/scales/linear_scale.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var continuous_scale_1 = require(261) /* ./continuous_scale */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var LinearScale = /** @class */ (function (_super) {
        tslib_1.__extends(LinearScale, _super);
        function LinearScale(attrs) {
            return _super.call(this, attrs) || this;
        }
        LinearScale.init_LinearScale = function () {
            this.internal({ scan_result: [p.Any] });
        };
        LinearScale.prototype.compute = function (x) {
            return this._linear_compute(x);
        };
        LinearScale.prototype.v_compute = function (xs) {
            return this._linear_v_compute(xs);
        };
        LinearScale.prototype.invert = function (xprime) {
            return this._linear_invert(xprime);
        };
        LinearScale.prototype.v_invert = function (xprimes) {
            return this._linear_v_invert(xprimes);
        };
        return LinearScale;
    }(continuous_scale_1.ContinuousScale));
    exports.LinearScale = LinearScale;
    LinearScale.__name__ = "LinearScale";
    LinearScale.init_LinearScale();
},
/* models/scales/continuous_scale.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var scale_1 = require(262) /* ./scale */;
    var ContinuousScale = /** @class */ (function (_super) {
        tslib_1.__extends(ContinuousScale, _super);
        function ContinuousScale(attrs) {
            return _super.call(this, attrs) || this;
        }
        return ContinuousScale;
    }(scale_1.Scale));
    exports.ContinuousScale = ContinuousScale;
    ContinuousScale.__name__ = "ContinuousScale";
},
/* models/scales/scale.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var transforms_1 = require(263) /* ../transforms */;
    var types_1 = require(139) /* ../../core/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var Scale = /** @class */ (function (_super) {
        tslib_1.__extends(Scale, _super);
        function Scale(attrs) {
            return _super.call(this, attrs) || this;
        }
        Scale.init_Scale = function () {
            this.internal({
                source_range: [p.Any],
                target_range: [p.Any],
            });
        };
        Scale.prototype.r_compute = function (x0, x1) {
            if (this.target_range.is_reversed)
                return [this.compute(x1), this.compute(x0)];
            else
                return [this.compute(x0), this.compute(x1)];
        };
        Scale.prototype.r_invert = function (sx0, sx1) {
            if (this.target_range.is_reversed)
                return [this.invert(sx1), this.invert(sx0)];
            else
                return [this.invert(sx0), this.invert(sx1)];
        };
        // These are needed by both LinearScale and CategoricalScale and this is the
        // only common ancestor. TODO: Proper MI/Mixin would be better.
        Scale.prototype._linear_compute = function (x) {
            var _a = tslib_1.__read(this._linear_compute_state(), 2), factor = _a[0], offset = _a[1];
            return factor * x + offset;
        };
        Scale.prototype._linear_v_compute = function (xs) {
            var _a = tslib_1.__read(this._linear_compute_state(), 2), factor = _a[0], offset = _a[1];
            var result = new types_1.NumberArray(xs.length);
            for (var i = 0; i < xs.length; i++)
                result[i] = factor * xs[i] + offset;
            return result;
        };
        Scale.prototype._linear_invert = function (xprime) {
            var _a = tslib_1.__read(this._linear_compute_state(), 2), factor = _a[0], offset = _a[1];
            return (xprime - offset) / factor;
        };
        Scale.prototype._linear_v_invert = function (xprimes) {
            var _a = tslib_1.__read(this._linear_compute_state(), 2), factor = _a[0], offset = _a[1];
            var result = new types_1.NumberArray(xprimes.length);
            for (var i = 0; i < xprimes.length; i++)
                result[i] = (xprimes[i] - offset) / factor;
            return result;
        };
        /*protected*/ Scale.prototype._linear_compute_state = function () {
            //
            //  (t1 - t0)       (t1 - t0)
            //  --------- * x - --------- * s0 + t0
            //  (s1 - s0)       (s1 - s0)
            //
            // [  factor  ]     [    offset    ]
            //
            var source_start = this.source_range.start;
            var source_end = this.source_range.end;
            var target_start = this.target_range.start;
            var target_end = this.target_range.end;
            var factor = (target_end - target_start) / (source_end - source_start);
            var offset = -(factor * source_start) + target_start;
            return [factor, offset];
        };
        return Scale;
    }(transforms_1.Transform));
    exports.Scale = Scale;
    Scale.__name__ = "Scale";
    Scale.init_Scale();
},
/* models/transforms/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var customjs_transform_1 = require(264) /* ./customjs_transform */;
    exports.CustomJSTransform = customjs_transform_1.CustomJSTransform;
    var dodge_1 = require(265) /* ./dodge */;
    exports.Dodge = dodge_1.Dodge;
    var interpolator_1 = require(267) /* ./interpolator */;
    exports.Interpolator = interpolator_1.Interpolator;
    var jitter_1 = require(268) /* ./jitter */;
    exports.Jitter = jitter_1.Jitter;
    var linear_interpolator_1 = require(269) /* ./linear_interpolator */;
    exports.LinearInterpolator = linear_interpolator_1.LinearInterpolator;
    var step_interpolator_1 = require(270) /* ./step_interpolator */;
    exports.StepInterpolator = step_interpolator_1.StepInterpolator;
    var transform_1 = require(252) /* ./transform */;
    exports.Transform = transform_1.Transform;
},
/* models/transforms/customjs_transform.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var transform_1 = require(252) /* ./transform */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var object_1 = require(128) /* ../../core/util/object */;
    var string_1 = require(144) /* ../../core/util/string */;
    var CustomJSTransform = /** @class */ (function (_super) {
        tslib_1.__extends(CustomJSTransform, _super);
        function CustomJSTransform(attrs) {
            return _super.call(this, attrs) || this;
        }
        CustomJSTransform.init_CustomJSTransform = function () {
            this.define({
                args: [p.Any, {}],
                func: [p.String, ""],
                v_func: [p.String, ""],
            });
        };
        Object.defineProperty(CustomJSTransform.prototype, "names", {
            get: function () {
                return object_1.keys(this.args);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CustomJSTransform.prototype, "values", {
            get: function () {
                return object_1.values(this.args);
            },
            enumerable: true,
            configurable: true
        });
        CustomJSTransform.prototype._make_transform = function (name, func) {
            return new (Function.bind.apply(Function, tslib_1.__spread([void 0], this.names, [name, string_1.use_strict(func)])))();
        };
        Object.defineProperty(CustomJSTransform.prototype, "scalar_transform", {
            get: function () {
                return this._make_transform("x", this.func);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CustomJSTransform.prototype, "vector_transform", {
            get: function () {
                return this._make_transform("xs", this.v_func);
            },
            enumerable: true,
            configurable: true
        });
        CustomJSTransform.prototype.compute = function (x) {
            return this.scalar_transform.apply(this, tslib_1.__spread(this.values, [x]));
        };
        CustomJSTransform.prototype.v_compute = function (xs) {
            return this.vector_transform.apply(this, tslib_1.__spread(this.values, [xs]));
        };
        return CustomJSTransform;
    }(transform_1.Transform));
    exports.CustomJSTransform = CustomJSTransform;
    CustomJSTransform.__name__ = "CustomJSTransform";
    CustomJSTransform.init_CustomJSTransform();
},
/* models/transforms/dodge.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var range_transform_1 = require(266) /* ./range_transform */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var Dodge = /** @class */ (function (_super) {
        tslib_1.__extends(Dodge, _super);
        function Dodge(attrs) {
            return _super.call(this, attrs) || this;
        }
        Dodge.init_Dodge = function () {
            this.define({
                value: [p.Number, 0],
            });
        };
        Dodge.prototype._compute = function (x) {
            return x + this.value;
        };
        return Dodge;
    }(range_transform_1.RangeTransform));
    exports.Dodge = Dodge;
    Dodge.__name__ = "Dodge";
    Dodge.init_Dodge();
},
/* models/transforms/range_transform.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var transform_1 = require(252) /* ./transform */;
    var factor_range_1 = require(213) /* ../ranges/factor_range */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(139) /* ../../core/types */;
    var types_2 = require(123) /* ../../core/util/types */;
    var RangeTransform = /** @class */ (function (_super) {
        tslib_1.__extends(RangeTransform, _super);
        function RangeTransform(attrs) {
            return _super.call(this, attrs) || this;
        }
        RangeTransform.init_RangeTransform = function () {
            this.define({
                range: [p.Instance],
            });
        };
        RangeTransform.prototype.v_compute = function (xs0) {
            var xs;
            if (this.range instanceof factor_range_1.FactorRange)
                xs = this.range.v_synthetic(xs0);
            else if (types_2.isArrayableOf(xs0, types_2.isNumber))
                xs = xs0;
            else
                throw new Error("unexpected");
            var result = new types_1.NumberArray(xs.length);
            for (var i = 0; i < xs.length; i++) {
                var x = xs[i];
                result[i] = this._compute(x);
            }
            return result;
        };
        RangeTransform.prototype.compute = function (x) {
            if (this.range instanceof factor_range_1.FactorRange)
                return this._compute(this.range.synthetic(x));
            else if (types_2.isNumber(x))
                return this._compute(x);
            else
                throw new Error("unexpected");
        };
        return RangeTransform;
    }(transform_1.Transform));
    exports.RangeTransform = RangeTransform;
    RangeTransform.__name__ = "RangeTransform";
    RangeTransform.init_RangeTransform();
},
/* models/transforms/interpolator.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var transform_1 = require(252) /* ./transform */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(139) /* ../../core/types */;
    var array_1 = require(124) /* ../../core/util/array */;
    var types_2 = require(123) /* ../../core/util/types */;
    var Interpolator = /** @class */ (function (_super) {
        tslib_1.__extends(Interpolator, _super);
        function Interpolator(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this._sorted_dirty = true;
            return _this;
        }
        Interpolator.init_Interpolator = function () {
            this.define({
                x: [p.Any],
                y: [p.Any],
                data: [p.Any],
                clip: [p.Boolean, true],
            });
        };
        Interpolator.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.change, function () { return _this._sorted_dirty = true; });
        };
        Interpolator.prototype.v_compute = function (xs) {
            var result = new types_1.NumberArray(xs.length);
            for (var i = 0; i < xs.length; i++) {
                var x = xs[i];
                result[i] = this.compute(x);
            }
            return result;
        };
        Interpolator.prototype.sort = function (descending) {
            if (descending === void 0) {
                descending = false;
            }
            if (!this._sorted_dirty)
                return;
            var tsx;
            var tsy;
            if (types_2.isString(this.x) && types_2.isString(this.y) && this.data != null) {
                var column_names = this.data.columns();
                if (!array_1.includes(column_names, this.x))
                    throw new Error("The x parameter does not correspond to a valid column name defined in the data parameter");
                if (!array_1.includes(column_names, this.y))
                    throw new Error("The y parameter does not correspond to a valid column name defined in the data parameter");
                tsx = this.data.get_column(this.x);
                tsy = this.data.get_column(this.y);
            }
            else if (types_2.isArray(this.x) && types_2.isArray(this.y)) {
                tsx = this.x;
                tsy = this.y;
            }
            else {
                throw new Error("parameters 'x' and 'y' must be both either string fields or arrays");
            }
            if (tsx.length !== tsy.length)
                throw new Error("The length for x and y do not match");
            if (tsx.length < 2)
                throw new Error("x and y must have at least two elements to support interpolation");
            var n = tsx.length;
            var index = new Uint32Array(n);
            for (var i = 0; i < n; i++) {
                index[i] = i;
            }
            var sign = descending ? -1 : 1;
            index.sort(function (i, j) { return sign * (tsx[i] - tsx[j]); });
            this._x_sorted = new types_1.NumberArray(n);
            this._y_sorted = new types_1.NumberArray(n);
            for (var i = 0; i < n; i++) {
                this._x_sorted[i] = tsx[index[i]];
                this._y_sorted[i] = tsy[index[i]];
            }
            this._sorted_dirty = false;
        };
        return Interpolator;
    }(transform_1.Transform));
    exports.Interpolator = Interpolator;
    Interpolator.__name__ = "Interpolator";
    Interpolator.init_Interpolator();
},
/* models/transforms/jitter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var range_transform_1 = require(266) /* ./range_transform */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var bokeh_math = tslib_1.__importStar(require(125) /* ../../core/util/math */);
    var Jitter = /** @class */ (function (_super) {
        tslib_1.__extends(Jitter, _super);
        function Jitter(attrs) {
            return _super.call(this, attrs) || this;
        }
        Jitter.init_Jitter = function () {
            this.define({
                mean: [p.Number, 0],
                width: [p.Number, 1],
                distribution: [p.Distribution, 'uniform'],
            });
        };
        Jitter.prototype.v_compute = function (xs0) {
            if (this.previous_values != null && this.previous_values.length == xs0.length)
                return this.previous_values;
            this.previous_values = _super.prototype.v_compute.call(this, xs0);
            return this.previous_values;
        };
        Jitter.prototype._compute = function (x) {
            switch (this.distribution) {
                case "uniform":
                    return x + this.mean + (bokeh_math.random() - 0.5) * this.width;
                case "normal":
                    return x + bokeh_math.rnorm(this.mean, this.width);
            }
        };
        return Jitter;
    }(range_transform_1.RangeTransform));
    exports.Jitter = Jitter;
    Jitter.__name__ = "Jitter";
    Jitter.init_Jitter();
},
/* models/transforms/linear_interpolator.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var array_1 = require(124) /* ../../core/util/array */;
    var interpolator_1 = require(267) /* ./interpolator */;
    var LinearInterpolator = /** @class */ (function (_super) {
        tslib_1.__extends(LinearInterpolator, _super);
        function LinearInterpolator(attrs) {
            return _super.call(this, attrs) || this;
        }
        LinearInterpolator.prototype.compute = function (x) {
            this.sort(false);
            if (this.clip) {
                if (x < this._x_sorted[0] || x > this._x_sorted[this._x_sorted.length - 1])
                    return NaN;
            }
            else {
                if (x < this._x_sorted[0])
                    return this._y_sorted[0];
                if (x > this._x_sorted[this._x_sorted.length - 1])
                    return this._y_sorted[this._y_sorted.length - 1];
            }
            if (x == this._x_sorted[0])
                return this._y_sorted[0];
            var ind = array_1.find_last_index(this._x_sorted, function (num) { return num < x; });
            var x1 = this._x_sorted[ind];
            var x2 = this._x_sorted[ind + 1];
            var y1 = this._y_sorted[ind];
            var y2 = this._y_sorted[ind + 1];
            return y1 + (((x - x1) / (x2 - x1)) * (y2 - y1));
        };
        return LinearInterpolator;
    }(interpolator_1.Interpolator));
    exports.LinearInterpolator = LinearInterpolator;
    LinearInterpolator.__name__ = "LinearInterpolator";
},
/* models/transforms/step_interpolator.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var interpolator_1 = require(267) /* ./interpolator */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    var StepInterpolator = /** @class */ (function (_super) {
        tslib_1.__extends(StepInterpolator, _super);
        function StepInterpolator(attrs) {
            return _super.call(this, attrs) || this;
        }
        StepInterpolator.init_StepInterpolator = function () {
            this.define({
                mode: [p.StepMode, "after"],
            });
        };
        StepInterpolator.prototype.compute = function (x) {
            this.sort(false);
            if (this.clip) {
                if (x < this._x_sorted[0] || x > this._x_sorted[this._x_sorted.length - 1])
                    return NaN;
            }
            else {
                if (x < this._x_sorted[0])
                    return this._y_sorted[0];
                if (x > this._x_sorted[this._x_sorted.length - 1])
                    return this._y_sorted[this._y_sorted.length - 1];
            }
            var ind;
            switch (this.mode) {
                case "after": {
                    ind = array_1.find_last_index(this._x_sorted, function (num) { return x >= num; });
                    break;
                }
                case "before": {
                    ind = array_1.find_index(this._x_sorted, function (num) { return x <= num; });
                    break;
                }
                case "center": {
                    var diffs = this._x_sorted.map(function (tx) { return Math.abs(tx - x); });
                    var mdiff_1 = array_1.min(diffs);
                    ind = array_1.find_index(diffs, function (num) { return mdiff_1 === num; });
                    break;
                }
                default:
                    throw new Error("unknown mode: " + this.mode);
            }
            return ind != -1 ? this._y_sorted[ind] : NaN;
        };
        return StepInterpolator;
    }(interpolator_1.Interpolator));
    exports.StepInterpolator = StepInterpolator;
    StepInterpolator.__name__ = "StepInterpolator";
    StepInterpolator.init_StepInterpolator();
},
/* models/scales/linear_interpolation_scale.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var scale_1 = require(262) /* ./scale */;
    var types_1 = require(139) /* ../../core/types */;
    var array_1 = require(124) /* ../../core/util/array */;
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var LinearInterpolationScale = /** @class */ (function (_super) {
        tslib_1.__extends(LinearInterpolationScale, _super);
        function LinearInterpolationScale(attrs) {
            return _super.call(this, attrs) || this;
        }
        LinearInterpolationScale.init_LinearInterpolationScale = function () {
            this.internal({ binning: [p.Array] });
        };
        LinearInterpolationScale.prototype.compute = function (x) {
            return x;
        };
        LinearInterpolationScale.prototype.v_compute = function (xs) {
            var _this = this;
            var norm_xs = arrayable_1.norm(xs, this.source_range.start, this.source_range.end);
            var edges_norm = array_1.linspace(0, 1, this.binning.length);
            var interpolated = arrayable_1.interpolate(norm_xs, edges_norm, this.binning);
            var norm_interp = arrayable_1.norm(interpolated, this.source_range.start, this.source_range.end);
            var target_span = this.target_range.end - this.target_range.start;
            var sxs = arrayable_1.map(norm_interp, function (x) { return _this.target_range.start + x * target_span; });
            return new types_1.NumberArray(sxs);
        };
        LinearInterpolationScale.prototype.invert = function (xprime) {
            return xprime;
        };
        LinearInterpolationScale.prototype.v_invert = function (xprimes) {
            return new types_1.NumberArray(xprimes);
        };
        return LinearInterpolationScale;
    }(scale_1.Scale));
    exports.LinearInterpolationScale = LinearInterpolationScale;
    LinearInterpolationScale.__name__ = "LinearInterpolationScale";
    LinearInterpolationScale.init_LinearInterpolationScale();
},
/* models/scales/log_scale.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var continuous_scale_1 = require(261) /* ./continuous_scale */;
    var types_1 = require(139) /* ../../core/types */;
    var LogScale = /** @class */ (function (_super) {
        tslib_1.__extends(LogScale, _super);
        function LogScale(attrs) {
            return _super.call(this, attrs) || this;
        }
        LogScale.prototype.compute = function (x) {
            var _a = tslib_1.__read(this._compute_state(), 4), factor = _a[0], offset = _a[1], inter_factor = _a[2], inter_offset = _a[3];
            var value;
            if (inter_factor == 0)
                value = 0;
            else {
                var _x = (Math.log(x) - inter_offset) / inter_factor;
                if (isFinite(_x))
                    value = _x * factor + offset;
                else
                    value = NaN;
            }
            return value;
        };
        LogScale.prototype.v_compute = function (xs) {
            var _a = tslib_1.__read(this._compute_state(), 4), factor = _a[0], offset = _a[1], inter_factor = _a[2], inter_offset = _a[3];
            var result = new types_1.NumberArray(xs.length);
            if (inter_factor == 0) {
                for (var i = 0; i < xs.length; i++)
                    result[i] = 0;
            }
            else {
                for (var i = 0; i < xs.length; i++) {
                    var _x = (Math.log(xs[i]) - inter_offset) / inter_factor;
                    var value = void 0;
                    if (isFinite(_x))
                        value = _x * factor + offset;
                    else
                        value = NaN;
                    result[i] = value;
                }
            }
            return result;
        };
        LogScale.prototype.invert = function (xprime) {
            var _a = tslib_1.__read(this._compute_state(), 4), factor = _a[0], offset = _a[1], inter_factor = _a[2], inter_offset = _a[3];
            var value = (xprime - offset) / factor;
            return Math.exp(inter_factor * value + inter_offset);
        };
        LogScale.prototype.v_invert = function (xprimes) {
            var _a = tslib_1.__read(this._compute_state(), 4), factor = _a[0], offset = _a[1], inter_factor = _a[2], inter_offset = _a[3];
            var result = new types_1.NumberArray(xprimes.length);
            for (var i = 0; i < xprimes.length; i++) {
                var value = (xprimes[i] - offset) / factor;
                result[i] = Math.exp(inter_factor * value + inter_offset);
            }
            return result;
        };
        LogScale.prototype._get_safe_factor = function (orig_start, orig_end) {
            var _a;
            var start = orig_start < 0 ? 0 : orig_start;
            var end = orig_end < 0 ? 0 : orig_end;
            if (start == end) {
                if (start == 0)
                    _a = tslib_1.__read([1, 10], 2), start = _a[0], end = _a[1];
                else {
                    var log_val = Math.log(start) / Math.log(10);
                    start = Math.pow(10, Math.floor(log_val));
                    if (Math.ceil(log_val) != Math.floor(log_val))
                        end = Math.pow(10, Math.ceil(log_val));
                    else
                        end = Math.pow(10, (Math.ceil(log_val) + 1));
                }
            }
            return [start, end];
        };
        /*protected*/ LogScale.prototype._compute_state = function () {
            var source_start = this.source_range.start;
            var source_end = this.source_range.end;
            var target_start = this.target_range.start;
            var target_end = this.target_range.end;
            var screen_range = target_end - target_start;
            var _a = tslib_1.__read(this._get_safe_factor(source_start, source_end), 2), start = _a[0], end = _a[1];
            var inter_factor;
            var inter_offset;
            if (start == 0) {
                inter_factor = Math.log(end);
                inter_offset = 0;
            }
            else {
                inter_factor = Math.log(end) - Math.log(start);
                inter_offset = Math.log(start);
            }
            var factor = screen_range;
            var offset = target_start;
            return [factor, offset, inter_factor, inter_offset];
        };
        return LogScale;
    }(continuous_scale_1.ContinuousScale));
    exports.LogScale = LogScale;
    LogScale.__name__ = "LogScale";
},
/* models/ranges/range1d.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var range_1 = require(214) /* ./range */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var Range1d = /** @class */ (function (_super) {
        tslib_1.__extends(Range1d, _super);
        function Range1d(attrs) {
            return _super.call(this, attrs) || this;
        }
        Range1d.init_Range1d = function () {
            this.define({
                start: [p.Number, 0],
                end: [p.Number, 1],
                reset_start: [p.Number],
                reset_end: [p.Number],
            });
        };
        Range1d.prototype._set_auto_bounds = function () {
            if (this.bounds == 'auto') {
                var min = Math.min(this.reset_start, this.reset_end);
                var max = Math.max(this.reset_start, this.reset_end);
                this.setv({ bounds: [min, max] }, { silent: true });
            }
        };
        Range1d.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            if (this.reset_start == null) {
                this.reset_start = this.start;
            }
            if (this.reset_end == null) {
                this.reset_end = this.end;
            }
            this._set_auto_bounds();
        };
        Object.defineProperty(Range1d.prototype, "min", {
            get: function () {
                return Math.min(this.start, this.end);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Range1d.prototype, "max", {
            get: function () {
                return Math.max(this.start, this.end);
            },
            enumerable: true,
            configurable: true
        });
        Range1d.prototype.reset = function () {
            this._set_auto_bounds();
            if (this.start != this.reset_start || this.end != this.reset_end)
                this.setv({ start: this.reset_start, end: this.reset_end });
            else
                this.change.emit();
        };
        return Range1d;
    }(range_1.Range));
    exports.Range1d = Range1d;
    Range1d.__name__ = "Range1d";
    Range1d.init_Range1d();
},
/* core/util/text.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var dom_1 = require(187) /* ../dom */;
    var _font_cache = new Map();
    function measure_font(font) {
        var metrics = _font_cache.get(font);
        if (metrics != null)
            return metrics;
        var text = dom_1.span({ style: { font: font } }, "Hg");
        var block = dom_1.div({ style: { display: "inline-block", width: "1px", height: "0px" } });
        var elem = dom_1.div({}, text, block);
        document.body.appendChild(elem);
        try {
            block.style.verticalAlign = "baseline";
            var ascent = dom_1.offset(block).top - dom_1.offset(text).top;
            block.style.verticalAlign = "bottom";
            var height = dom_1.offset(block).top - dom_1.offset(text).top;
            var result = { height: height, ascent: ascent, descent: height - ascent };
            _font_cache.set(font, result);
            return result;
        }
        finally {
            document.body.removeChild(elem);
        }
    }
    exports.measure_font = measure_font;
    var _text_cache = new Map();
    function measure_text(text, font) {
        var size_cache = _text_cache.get(font);
        if (size_cache != null) {
            var size = size_cache.get(text);
            if (size != null)
                return size;
        }
        else {
            size_cache = new Map();
            _text_cache.set(font, size_cache);
        }
        var el = dom_1.div({ style: { display: "inline-block", "white-space": "nowrap", font: font } }, text);
        document.body.appendChild(el);
        try {
            var _a = el.getBoundingClientRect(), width = _a.width, height = _a.height;
            size_cache.set(text, { width: width, height: height });
            return { width: width, height: height };
        }
        finally {
            document.body.removeChild(el);
        }
    }
    exports.measure_text = measure_text;
},
/* models/annotations/label.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var text_annotation_1 = require(276) /* ./text_annotation */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var LabelView = /** @class */ (function (_super) {
        tslib_1.__extends(LabelView, _super);
        function LabelView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        LabelView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.visuals.warm_cache();
        };
        LabelView.prototype._get_size = function () {
            var ctx = this.layer.ctx;
            this.visuals.text.set_value(ctx);
            var _a = ctx.measureText(this.model.text), width = _a.width, ascent = _a.ascent;
            return { width: width, height: ascent };
        };
        LabelView.prototype._render = function () {
            // Here because AngleSpec does units transform and label doesn't support specs
            var angle;
            switch (this.model.angle_units) {
                case "rad": {
                    angle = -this.model.angle;
                    break;
                }
                case "deg": {
                    angle = (-this.model.angle * Math.PI) / 180.0;
                    break;
                }
            }
            var panel = this.panel != null ? this.panel : this.plot_view.frame;
            var xscale = this.coordinates.x_scale;
            var yscale = this.coordinates.y_scale;
            var sx = this.model.x_units == "data" ? xscale.compute(this.model.x) : panel.xview.compute(this.model.x);
            var sy = this.model.y_units == "data" ? yscale.compute(this.model.y) : panel.yview.compute(this.model.y);
            sx += this.model.x_offset;
            sy -= this.model.y_offset;
            var draw = this.model.render_mode == 'canvas' ? this._canvas_text.bind(this) : this._css_text.bind(this);
            draw(this.layer.ctx, this.model.text, sx, sy, angle);
        };
        return LabelView;
    }(text_annotation_1.TextAnnotationView));
    exports.LabelView = LabelView;
    LabelView.__name__ = "LabelView";
    var Label = /** @class */ (function (_super) {
        tslib_1.__extends(Label, _super);
        function Label(attrs) {
            return _super.call(this, attrs) || this;
        }
        Label.init_Label = function () {
            this.prototype.default_view = LabelView;
            this.mixins([
                mixins.Text /*Scalar*/,
                ["border_", mixins.Line],
                ["background_", mixins.Fill],
            ]);
            this.define({
                x: [p.Number],
                x_units: [p.SpatialUnits, 'data'],
                y: [p.Number],
                y_units: [p.SpatialUnits, 'data'],
                text: [p.String],
                angle: [p.Angle, 0],
                angle_units: [p.AngleUnits, 'rad'],
                x_offset: [p.Number, 0],
                y_offset: [p.Number, 0],
            });
            this.override({
                background_fill_color: null,
                border_line_color: null,
            });
        };
        return Label;
    }(text_annotation_1.TextAnnotation));
    exports.Label = Label;
    Label.__name__ = "Label";
    Label.init_Label();
},
/* models/annotations/text_annotation.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var dom_1 = require(187) /* ../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var text_1 = require(274) /* ../../core/util/text */;
    var assert_1 = require(126) /* ../../core/util/assert */;
    var TextAnnotationView = /** @class */ (function (_super) {
        tslib_1.__extends(TextAnnotationView, _super);
        function TextAnnotationView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.rotate = true;
            return _this;
        }
        TextAnnotationView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            if (this.model.render_mode == 'css') {
                this.el = dom_1.div();
                this.plot_view.canvas_view.add_overlay(this.el);
            }
        };
        TextAnnotationView.prototype.remove = function () {
            if (this.el != null)
                dom_1.remove(this.el);
            _super.prototype.remove.call(this);
        };
        TextAnnotationView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            if (this.model.render_mode == 'css') {
                // dispatch CSS update immediately
                this.connect(this.model.change, function () { return _this.render(); });
            }
            else {
                this.connect(this.model.change, function () { return _this.plot_view.request_render(); });
            }
        };
        TextAnnotationView.prototype.render = function () {
            if (!this.model.visible && this.model.render_mode == "css")
                dom_1.undisplay(this.el);
            _super.prototype.render.call(this);
        };
        TextAnnotationView.prototype._calculate_text_dimensions = function (ctx, text) {
            var width = ctx.measureText(text).width;
            var height = text_1.measure_font(this.visuals.text.font_value()).height;
            return [width, height];
        };
        TextAnnotationView.prototype._calculate_bounding_box_dimensions = function (ctx, text) {
            var _a = tslib_1.__read(this._calculate_text_dimensions(ctx, text), 2), width = _a[0], height = _a[1];
            var x_offset;
            switch (ctx.textAlign) {
                case 'left':
                    x_offset = 0;
                    break;
                case 'center':
                    x_offset = -width / 2;
                    break;
                case 'right':
                    x_offset = -width;
                    break;
                default:
                    assert_1.unreachable();
            }
            // guestimated from https://www.w3.org/TR/2dcontext/#dom-context-2d-textbaseline
            var y_offset;
            switch (ctx.textBaseline) {
                case 'top':
                    y_offset = 0.0;
                    break;
                case 'middle':
                    y_offset = -0.5 * height;
                    break;
                case 'bottom':
                    y_offset = -1.0 * height;
                    break;
                case 'alphabetic':
                    y_offset = -0.8 * height;
                    break;
                case 'hanging':
                    y_offset = -0.17 * height;
                    break;
                case 'ideographic':
                    y_offset = -0.83 * height;
                    break;
                default:
                    assert_1.unreachable();
            }
            return [x_offset, y_offset, width, height];
        };
        TextAnnotationView.prototype._canvas_text = function (ctx, text, sx, sy, angle) {
            this.visuals.text.set_value(ctx);
            var bbox_dims = this._calculate_bounding_box_dimensions(ctx, text);
            ctx.save();
            ctx.beginPath();
            ctx.translate(sx, sy);
            if (angle)
                ctx.rotate(angle);
            ctx.rect(bbox_dims[0], bbox_dims[1], bbox_dims[2], bbox_dims[3]);
            if (this.visuals.background_fill.doit) {
                this.visuals.background_fill.set_value(ctx);
                ctx.fill();
            }
            if (this.visuals.border_line.doit) {
                this.visuals.border_line.set_value(ctx);
                ctx.stroke();
            }
            if (this.visuals.text.doit) {
                this.visuals.text.set_value(ctx);
                ctx.fillText(text, 0, 0);
            }
            ctx.restore();
        };
        TextAnnotationView.prototype._css_text = function (ctx, text, sx, sy, angle) {
            var el = this.el;
            assert_1.assert(el != null);
            dom_1.undisplay(el);
            this.visuals.text.set_value(ctx);
            var bbox_dims = this._calculate_bounding_box_dimensions(ctx, text);
            // attempt to support vector string-style ("8 4 8") line dashing for css mode
            var ld = this.visuals.border_line.line_dash.value();
            var line_dash = ld.length < 2 ? "solid" : "dashed";
            this.visuals.border_line.set_value(ctx);
            this.visuals.background_fill.set_value(ctx);
            el.style.position = 'absolute';
            el.style.left = sx + bbox_dims[0] + "px";
            el.style.top = sy + bbox_dims[1] + "px";
            el.style.color = "" + this.visuals.text.text_color.value();
            el.style.opacity = "" + this.visuals.text.text_alpha.value();
            el.style.font = "" + this.visuals.text.font_value();
            el.style.lineHeight = "normal"; // needed to prevent ipynb css override
            if (angle) {
                el.style.transform = "rotate(" + angle + "rad)";
            }
            if (this.visuals.background_fill.doit) {
                el.style.backgroundColor = "" + this.visuals.background_fill.color_value();
            }
            if (this.visuals.border_line.doit) {
                el.style.borderStyle = "" + line_dash;
                el.style.borderWidth = this.visuals.border_line.line_width.value() + "px";
                el.style.borderColor = "" + this.visuals.border_line.color_value();
            }
            el.textContent = text;
            dom_1.display(el);
        };
        return TextAnnotationView;
    }(annotation_1.AnnotationView));
    exports.TextAnnotationView = TextAnnotationView;
    TextAnnotationView.__name__ = "TextAnnotationView";
    var TextAnnotation = /** @class */ (function (_super) {
        tslib_1.__extends(TextAnnotation, _super);
        function TextAnnotation(attrs) {
            return _super.call(this, attrs) || this;
        }
        TextAnnotation.init_TextAnnotation = function () {
            this.define({
                render_mode: [p.RenderMode, "canvas"],
            });
        };
        return TextAnnotation;
    }(annotation_1.Annotation));
    exports.TextAnnotation = TextAnnotation;
    TextAnnotation.__name__ = "TextAnnotation";
    TextAnnotation.init_TextAnnotation();
},
/* models/annotations/label_set.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var text_annotation_1 = require(276) /* ./text_annotation */;
    var column_data_source_1 = require(200) /* ../sources/column_data_source */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var dom_1 = require(187) /* ../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var LabelSetView = /** @class */ (function (_super) {
        tslib_1.__extends(LabelSetView, _super);
        function LabelSetView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        LabelSetView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.set_data(this.model.source);
            if (this.model.render_mode == 'css') {
                for (var i = 0, end = this._text.length; i < end; i++) {
                    var el = dom_1.div({ style: { display: "none" } });
                    this.el.appendChild(el);
                }
            }
        };
        LabelSetView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            if (this.model.render_mode == 'css') {
                // dispatch CSS update immediately
                this.connect(this.model.change, function () {
                    _this.set_data(_this.model.source);
                    _this.render();
                });
                this.connect(this.model.source.streaming, function () {
                    _this.set_data(_this.model.source);
                    _this.render();
                });
                this.connect(this.model.source.patching, function () {
                    _this.set_data(_this.model.source);
                    _this.render();
                });
                this.connect(this.model.source.change, function () {
                    _this.set_data(_this.model.source);
                    _this.render();
                });
            }
            else {
                this.connect(this.model.change, function () {
                    _this.set_data(_this.model.source);
                    _this.plot_view.request_render();
                });
                this.connect(this.model.source.streaming, function () {
                    _this.set_data(_this.model.source);
                    _this.plot_view.request_render();
                });
                this.connect(this.model.source.patching, function () {
                    _this.set_data(_this.model.source);
                    _this.plot_view.request_render();
                });
                this.connect(this.model.source.change, function () {
                    _this.set_data(_this.model.source);
                    _this.plot_view.request_render();
                });
            }
        };
        LabelSetView.prototype.set_data = function (source) {
            _super.prototype.set_data.call(this, source);
            this.visuals.warm_cache(source);
        };
        LabelSetView.prototype._map_data = function () {
            var xscale = this.coordinates.x_scale;
            var yscale = this.coordinates.y_scale;
            var panel = this.panel != null ? this.panel : this.plot_view.frame;
            var sx = this.model.x_units == "data" ? xscale.v_compute(this._x) : panel.xview.v_compute(this._x);
            var sy = this.model.y_units == "data" ? yscale.v_compute(this._y) : panel.yview.v_compute(this._y);
            return [sx, sy];
        };
        LabelSetView.prototype._render = function () {
            var draw = this.model.render_mode == 'canvas' ? this._v_canvas_text.bind(this) : this._v_css_text.bind(this);
            var ctx = this.layer.ctx;
            var _a = tslib_1.__read(this._map_data(), 2), sx = _a[0], sy = _a[1];
            for (var i = 0, end = this._text.length; i < end; i++) {
                draw(ctx, i, this._text[i], sx[i] + this._x_offset[i], sy[i] - this._y_offset[i], this._angle[i]);
            }
        };
        LabelSetView.prototype._get_size = function () {
            var ctx = this.layer.ctx;
            this.visuals.text.set_value(ctx);
            var _a = ctx.measureText(this._text[0]), width = _a.width, ascent = _a.ascent;
            return { width: width, height: ascent };
        };
        LabelSetView.prototype._v_canvas_text = function (ctx, i, text, sx, sy, angle) {
            this.visuals.text.set_vectorize(ctx, i);
            var bbox_dims = this._calculate_bounding_box_dimensions(ctx, text);
            ctx.save();
            ctx.beginPath();
            ctx.translate(sx, sy);
            ctx.rotate(angle);
            ctx.rect(bbox_dims[0], bbox_dims[1], bbox_dims[2], bbox_dims[3]);
            if (this.visuals.background_fill.doit) {
                this.visuals.background_fill.set_vectorize(ctx, i);
                ctx.fill();
            }
            if (this.visuals.border_line.doit) {
                this.visuals.border_line.set_vectorize(ctx, i);
                ctx.stroke();
            }
            if (this.visuals.text.doit) {
                this.visuals.text.set_vectorize(ctx, i);
                ctx.fillText(text, 0, 0);
            }
            ctx.restore();
        };
        LabelSetView.prototype._v_css_text = function (ctx, i, text, sx, sy, angle) {
            var el = this.el.children[i];
            el.textContent = text;
            this.visuals.text.set_vectorize(ctx, i);
            var bbox_dims = this._calculate_bounding_box_dimensions(ctx, text);
            // attempt to support vector-style ("8 4 8") line dashing for css mode
            var ld = this.visuals.border_line.line_dash.value();
            var line_dash = ld.length < 2 ? "solid" : "dashed";
            this.visuals.border_line.set_vectorize(ctx, i);
            this.visuals.background_fill.set_vectorize(ctx, i);
            el.style.position = 'absolute';
            el.style.left = sx + bbox_dims[0] + "px";
            el.style.top = sy + bbox_dims[1] + "px";
            el.style.color = "" + this.visuals.text.text_color.value();
            el.style.opacity = "" + this.visuals.text.text_alpha.value();
            el.style.font = "" + this.visuals.text.font_value();
            el.style.lineHeight = "normal"; // needed to prevent ipynb css override
            if (angle) {
                el.style.transform = "rotate(" + angle + "rad)";
            }
            if (this.visuals.background_fill.doit) {
                el.style.backgroundColor = "" + this.visuals.background_fill.color_value();
            }
            if (this.visuals.border_line.doit) {
                el.style.borderStyle = "" + line_dash;
                el.style.borderWidth = this.visuals.border_line.line_width.value() + "px";
                el.style.borderColor = "" + this.visuals.border_line.color_value();
            }
            dom_1.display(el);
        };
        return LabelSetView;
    }(text_annotation_1.TextAnnotationView));
    exports.LabelSetView = LabelSetView;
    LabelSetView.__name__ = "LabelSetView";
    var LabelSet = /** @class */ (function (_super) {
        tslib_1.__extends(LabelSet, _super);
        function LabelSet(attrs) {
            return _super.call(this, attrs) || this;
        }
        LabelSet.init_LabelSet = function () {
            this.prototype.default_view = LabelSetView;
            this.mixins([
                mixins.TextVector,
                ["border_", mixins.LineVector],
                ["background_", mixins.FillVector],
            ]);
            this.define({
                x: [p.NumberSpec],
                y: [p.NumberSpec],
                x_units: [p.SpatialUnits, 'data'],
                y_units: [p.SpatialUnits, 'data'],
                text: [p.StringSpec, { field: "text" }],
                angle: [p.AngleSpec, 0],
                x_offset: [p.NumberSpec, { value: 0 }],
                y_offset: [p.NumberSpec, { value: 0 }],
                source: [p.Instance, function () { return new column_data_source_1.ColumnDataSource(); }],
            });
            this.override({
                background_fill_color: null,
                border_line_color: null,
            });
        };
        return LabelSet;
    }(text_annotation_1.TextAnnotation));
    exports.LabelSet = LabelSet;
    LabelSet.__name__ = "LabelSet";
    LabelSet.init_LabelSet();
},
/* models/annotations/legend.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var signaling_1 = require(130) /* ../../core/signaling */;
    var text_1 = require(274) /* ../../core/util/text */;
    var bbox_1 = require(194) /* ../../core/util/bbox */;
    var array_1 = require(124) /* ../../core/util/array */;
    var types_1 = require(123) /* ../../core/util/types */;
    var assert_1 = require(126) /* ../../core/util/assert */;
    var LegendView = /** @class */ (function (_super) {
        tslib_1.__extends(LegendView, _super);
        function LegendView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        LegendView.prototype.cursor = function (_sx, _sy) {
            return this.model.click_policy == "none" ? null : "pointer";
        };
        Object.defineProperty(LegendView.prototype, "legend_padding", {
            get: function () {
                return this.visuals.border_line.line_color.value() != null ? this.model.padding : 0;
            },
            enumerable: true,
            configurable: true
        });
        LegendView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.change, function () { return _this.plot_view.request_render(); });
            this.connect(this.model.item_change, function () { return _this.plot_view.request_render(); });
        };
        LegendView.prototype.compute_legend_bbox = function () {
            var e_1, _a, e_2, _b;
            var legend_names = this.model.get_legend_names();
            var _c = this.model, glyph_height = _c.glyph_height, glyph_width = _c.glyph_width;
            var _d = this.model, label_height = _d.label_height, label_width = _d.label_width;
            this.max_label_height = array_1.max([text_1.measure_font(this.visuals.label_text.font_value()).height, label_height, glyph_height]);
            // this is to measure text properties
            var ctx = this.layer.ctx;
            ctx.save();
            this.visuals.label_text.set_value(ctx);
            this.text_widths = new Map();
            try {
                for (var legend_names_1 = tslib_1.__values(legend_names), legend_names_1_1 = legend_names_1.next(); !legend_names_1_1.done; legend_names_1_1 = legend_names_1.next()) {
                    var name = legend_names_1_1.value;
                    this.text_widths.set(name, array_1.max([ctx.measureText(name).width, label_width]));
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (legend_names_1_1 && !legend_names_1_1.done && (_a = legend_names_1.return))
                        _a.call(legend_names_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            this.visuals.title_text.set_value(ctx);
            this.title_height = this.model.title ? text_1.measure_font(this.visuals.title_text.font_value()).height + this.model.title_standoff : 0;
            this.title_width = this.model.title ? ctx.measureText(this.model.title).width : 0;
            ctx.restore();
            var max_label_width = Math.max(array_1.max(tslib_1.__spread(this.text_widths.values())), 0);
            var legend_margin = this.model.margin;
            var legend_padding = this.legend_padding;
            var legend_spacing = this.model.spacing;
            var label_standoff = this.model.label_standoff;
            var legend_height, legend_width;
            if (this.model.orientation == "vertical") {
                legend_height = legend_names.length * this.max_label_height + Math.max(legend_names.length - 1, 0) * legend_spacing + 2 * legend_padding + this.title_height;
                legend_width = array_1.max([(max_label_width + glyph_width + label_standoff + 2 * legend_padding), this.title_width + 2 * legend_padding]);
            }
            else {
                var item_width = 2 * legend_padding + Math.max(legend_names.length - 1, 0) * legend_spacing;
                try {
                    for (var _e = tslib_1.__values(this.text_widths), _f = _e.next(); !_f.done; _f = _e.next()) {
                        var _g = tslib_1.__read(_f.value, 2), width = _g[1];
                        item_width += array_1.max([width, label_width]) + glyph_width + label_standoff;
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (_f && !_f.done && (_b = _e.return))
                            _b.call(_e);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
                legend_width = array_1.max([this.title_width + 2 * legend_padding, item_width]);
                legend_height = this.max_label_height + this.title_height + 2 * legend_padding;
            }
            var panel = this.panel != null ? this.panel : this.plot_view.frame;
            var _h = tslib_1.__read(panel.bbox.ranges, 2), hr = _h[0], vr = _h[1];
            var location = this.model.location;
            var sx, sy;
            if (types_1.isString(location)) {
                switch (location) {
                    case 'top_left':
                        sx = hr.start + legend_margin;
                        sy = vr.start + legend_margin;
                        break;
                    case 'top_center':
                        sx = (hr.end + hr.start) / 2 - legend_width / 2;
                        sy = vr.start + legend_margin;
                        break;
                    case 'top_right':
                        sx = hr.end - legend_margin - legend_width;
                        sy = vr.start + legend_margin;
                        break;
                    case 'bottom_right':
                        sx = hr.end - legend_margin - legend_width;
                        sy = vr.end - legend_margin - legend_height;
                        break;
                    case 'bottom_center':
                        sx = (hr.end + hr.start) / 2 - legend_width / 2;
                        sy = vr.end - legend_margin - legend_height;
                        break;
                    case 'bottom_left':
                        sx = hr.start + legend_margin;
                        sy = vr.end - legend_margin - legend_height;
                        break;
                    case 'center_left':
                        sx = hr.start + legend_margin;
                        sy = (vr.end + vr.start) / 2 - legend_height / 2;
                        break;
                    case 'center':
                        sx = (hr.end + hr.start) / 2 - legend_width / 2;
                        sy = (vr.end + vr.start) / 2 - legend_height / 2;
                        break;
                    case 'center_right':
                        sx = hr.end - legend_margin - legend_width;
                        sy = (vr.end + vr.start) / 2 - legend_height / 2;
                        break;
                }
            }
            else if (types_1.isArray(location) && location.length == 2) {
                var _j = tslib_1.__read(location, 2), vx = _j[0], vy = _j[1];
                sx = panel.xview.compute(vx);
                sy = panel.yview.compute(vy) - legend_height;
            }
            else
                assert_1.unreachable();
            return new bbox_1.BBox({ left: sx, top: sy, width: legend_width, height: legend_height });
        };
        LegendView.prototype.interactive_bbox = function () {
            return this.compute_legend_bbox();
        };
        LegendView.prototype.interactive_hit = function (sx, sy) {
            var bbox = this.interactive_bbox();
            return bbox.contains(sx, sy);
        };
        LegendView.prototype.on_hit = function (sx, sy) {
            var e_3, _a, e_4, _b, _c, _d, e_5, _e, e_6, _f;
            var yoffset;
            var glyph_width = this.model.glyph_width;
            var legend_padding = this.legend_padding;
            var legend_spacing = this.model.spacing;
            var label_standoff = this.model.label_standoff;
            var xoffset = (yoffset = legend_padding);
            var legend_bbox = this.compute_legend_bbox();
            var vertical = this.model.orientation == "vertical";
            try {
                for (var _g = tslib_1.__values(this.model.items), _h = _g.next(); !_h.done; _h = _g.next()) {
                    var item = _h.value;
                    var labels = item.get_labels_list_from_label_prop();
                    try {
                        for (var labels_1 = (e_4 = void 0, tslib_1.__values(labels)), labels_1_1 = labels_1.next(); !labels_1_1.done; labels_1_1 = labels_1.next()) {
                            var label = labels_1_1.value;
                            var x1 = legend_bbox.x + xoffset;
                            var y1 = legend_bbox.y + yoffset + this.title_height;
                            var w = void 0, h = void 0;
                            if (vertical)
                                _c = tslib_1.__read([legend_bbox.width - 2 * legend_padding, this.max_label_height], 2), w = _c[0], h = _c[1];
                            else
                                _d = tslib_1.__read([this.text_widths.get(label) + glyph_width + label_standoff, this.max_label_height], 2), w = _d[0], h = _d[1];
                            var bbox = new bbox_1.BBox({ left: x1, top: y1, width: w, height: h });
                            if (bbox.contains(sx, sy)) {
                                switch (this.model.click_policy) {
                                    case "hide": {
                                        try {
                                            for (var _j = (e_5 = void 0, tslib_1.__values(item.renderers)), _k = _j.next(); !_k.done; _k = _j.next()) {
                                                var r = _k.value;
                                                r.visible = !r.visible;
                                            }
                                        }
                                        catch (e_5_1) {
                                            e_5 = { error: e_5_1 };
                                        }
                                        finally {
                                            try {
                                                if (_k && !_k.done && (_e = _j.return))
                                                    _e.call(_j);
                                            }
                                            finally {
                                                if (e_5)
                                                    throw e_5.error;
                                            }
                                        }
                                        break;
                                    }
                                    case "mute": {
                                        try {
                                            for (var _l = (e_6 = void 0, tslib_1.__values(item.renderers)), _m = _l.next(); !_m.done; _m = _l.next()) {
                                                var r = _m.value;
                                                r.muted = !r.muted;
                                            }
                                        }
                                        catch (e_6_1) {
                                            e_6 = { error: e_6_1 };
                                        }
                                        finally {
                                            try {
                                                if (_m && !_m.done && (_f = _l.return))
                                                    _f.call(_l);
                                            }
                                            finally {
                                                if (e_6)
                                                    throw e_6.error;
                                            }
                                        }
                                        break;
                                    }
                                }
                                return true;
                            }
                            if (vertical)
                                yoffset += this.max_label_height + legend_spacing;
                            else
                                xoffset += this.text_widths.get(label) + glyph_width + label_standoff + legend_spacing;
                        }
                    }
                    catch (e_4_1) {
                        e_4 = { error: e_4_1 };
                    }
                    finally {
                        try {
                            if (labels_1_1 && !labels_1_1.done && (_b = labels_1.return))
                                _b.call(labels_1);
                        }
                        finally {
                            if (e_4)
                                throw e_4.error;
                        }
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_h && !_h.done && (_a = _g.return))
                        _a.call(_g);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            return false;
        };
        LegendView.prototype._render = function () {
            var e_7, _a;
            if (this.model.items.length == 0)
                return;
            try {
                // set a backref on render so that items can later signal item_change upates
                // on the model to trigger a re-render
                for (var _b = tslib_1.__values(this.model.items), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var item = _c.value;
                    item.legend = this.model;
                }
            }
            catch (e_7_1) {
                e_7 = { error: e_7_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_7)
                        throw e_7.error;
                }
            }
            var ctx = this.layer.ctx;
            var bbox = this.compute_legend_bbox();
            ctx.save();
            this._draw_legend_box(ctx, bbox);
            this._draw_legend_items(ctx, bbox);
            if (this.model.title)
                this._draw_title(ctx, bbox);
            ctx.restore();
        };
        LegendView.prototype._draw_legend_box = function (ctx, bbox) {
            ctx.beginPath();
            ctx.rect(bbox.x, bbox.y, bbox.width, bbox.height);
            this.visuals.background_fill.set_value(ctx);
            ctx.fill();
            if (this.visuals.border_line.doit) {
                this.visuals.border_line.set_value(ctx);
                ctx.stroke();
            }
        };
        LegendView.prototype._draw_legend_items = function (ctx, bbox) {
            var e_8, _a;
            var _this = this;
            var _b = this.model, glyph_width = _b.glyph_width, glyph_height = _b.glyph_height;
            var legend_padding = this.legend_padding;
            var legend_spacing = this.model.spacing;
            var label_standoff = this.model.label_standoff;
            var xoffset = legend_padding;
            var yoffset = legend_padding;
            var vertical = this.model.orientation == "vertical";
            var _loop_1 = function (item) {
                var e_9, _a, e_10, _b, _c, _d;
                var labels = item.get_labels_list_from_label_prop();
                var field = item.get_field_from_label_prop();
                if (labels.length == 0)
                    return "continue";
                var active = (function () {
                    switch (_this.model.click_policy) {
                        case "none": return true;
                        case "hide": return array_1.every(item.renderers, function (r) { return r.visible; });
                        case "mute": return array_1.every(item.renderers, function (r) { return !r.muted; });
                    }
                })();
                try {
                    for (var labels_2 = (e_9 = void 0, tslib_1.__values(labels)), labels_2_1 = labels_2.next(); !labels_2_1.done; labels_2_1 = labels_2.next()) {
                        var label = labels_2_1.value;
                        var x1 = bbox.x + xoffset;
                        var y1 = bbox.y + yoffset + this_1.title_height;
                        var x2 = x1 + glyph_width;
                        var y2 = y1 + glyph_height;
                        if (vertical)
                            yoffset += this_1.max_label_height + legend_spacing;
                        else
                            xoffset += this_1.text_widths.get(label) + glyph_width + label_standoff + legend_spacing;
                        this_1.visuals.label_text.set_value(ctx);
                        ctx.fillText(label, x2 + label_standoff, y1 + this_1.max_label_height / 2.0);
                        try {
                            for (var _e = (e_10 = void 0, tslib_1.__values(item.renderers)), _f = _e.next(); !_f.done; _f = _e.next()) {
                                var r = _f.value;
                                var view = this_1.plot_view.renderer_views.get(r);
                                view.draw_legend(ctx, x1, x2, y1, y2, field, label, item.index);
                            }
                        }
                        catch (e_10_1) {
                            e_10 = { error: e_10_1 };
                        }
                        finally {
                            try {
                                if (_f && !_f.done && (_b = _e.return))
                                    _b.call(_e);
                            }
                            finally {
                                if (e_10)
                                    throw e_10.error;
                            }
                        }
                        if (!active) {
                            var w = void 0, h = void 0;
                            if (vertical)
                                _c = tslib_1.__read([bbox.width - 2 * legend_padding, this_1.max_label_height], 2), w = _c[0], h = _c[1];
                            else
                                _d = tslib_1.__read([this_1.text_widths.get(label) + glyph_width + label_standoff, this_1.max_label_height], 2), w = _d[0], h = _d[1];
                            ctx.beginPath();
                            ctx.rect(x1, y1, w, h);
                            this_1.visuals.inactive_fill.set_value(ctx);
                            ctx.fill();
                        }
                    }
                }
                catch (e_9_1) {
                    e_9 = { error: e_9_1 };
                }
                finally {
                    try {
                        if (labels_2_1 && !labels_2_1.done && (_a = labels_2.return))
                            _a.call(labels_2);
                    }
                    finally {
                        if (e_9)
                            throw e_9.error;
                    }
                }
            };
            var this_1 = this;
            try {
                for (var _c = tslib_1.__values(this.model.items), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var item = _d.value;
                    _loop_1(item);
                }
            }
            catch (e_8_1) {
                e_8 = { error: e_8_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return))
                        _a.call(_c);
                }
                finally {
                    if (e_8)
                        throw e_8.error;
                }
            }
        };
        LegendView.prototype._draw_title = function (ctx, bbox) {
            if (!this.visuals.title_text.doit)
                return;
            ctx.save();
            ctx.translate(bbox.x0, bbox.y0 + this.title_height);
            this.visuals.title_text.set_value(ctx);
            ctx.fillText(this.model.title, this.legend_padding, this.legend_padding - this.model.title_standoff);
            ctx.restore();
        };
        LegendView.prototype._get_size = function () {
            var _a = this.compute_legend_bbox(), width = _a.width, height = _a.height;
            return {
                width: width + 2 * this.model.margin,
                height: height + 2 * this.model.margin,
            };
        };
        return LegendView;
    }(annotation_1.AnnotationView));
    exports.LegendView = LegendView;
    LegendView.__name__ = "LegendView";
    var Legend = /** @class */ (function (_super) {
        tslib_1.__extends(Legend, _super);
        function Legend(attrs) {
            return _super.call(this, attrs) || this;
        }
        Legend.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.item_change = new signaling_1.Signal0(this, "item_change");
        };
        Legend.init_Legend = function () {
            this.prototype.default_view = LegendView;
            this.mixins([
                ["label_", mixins.Text],
                ["title_", mixins.Text],
                ["inactive_", mixins.Fill],
                ["border_", mixins.Line],
                ["background_", mixins.Fill],
            ]);
            this.define({
                orientation: [p.Orientation, 'vertical'],
                location: [p.Any, 'top_right'],
                title: [p.String],
                title_standoff: [p.Number, 5],
                label_standoff: [p.Number, 5],
                glyph_height: [p.Number, 20],
                glyph_width: [p.Number, 20],
                label_height: [p.Number, 20],
                label_width: [p.Number, 20],
                margin: [p.Number, 10],
                padding: [p.Number, 10],
                spacing: [p.Number, 3],
                items: [p.Array, []],
                click_policy: [p.Any, "none"],
            });
            this.override({
                border_line_color: "#e5e5e5",
                border_line_alpha: 0.5,
                border_line_width: 1,
                background_fill_color: "#ffffff",
                background_fill_alpha: 0.95,
                inactive_fill_color: "white",
                inactive_fill_alpha: 0.7,
                label_text_font_size: "13px",
                label_text_baseline: "middle",
                title_text_font_size: "13px",
                title_text_font_style: "italic",
            });
        };
        Legend.prototype.get_legend_names = function () {
            var e_11, _a;
            var legend_names = [];
            try {
                for (var _b = tslib_1.__values(this.items), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var item = _c.value;
                    var labels = item.get_labels_list_from_label_prop();
                    legend_names.push.apply(legend_names, tslib_1.__spread(labels));
                }
            }
            catch (e_11_1) {
                e_11 = { error: e_11_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_11)
                        throw e_11.error;
                }
            }
            return legend_names;
        };
        return Legend;
    }(annotation_1.Annotation));
    exports.Legend = Legend;
    Legend.__name__ = "Legend";
    Legend.init_Legend();
},
/* models/annotations/legend_item.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var columnar_data_source_1 = require(201) /* ../sources/columnar_data_source */;
    var vectorization_1 = require(280) /* ../../core/vectorization */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var logging_1 = require(134) /* ../../core/logging */;
    var array_1 = require(124) /* ../../core/util/array */;
    var LegendItem = /** @class */ (function (_super) {
        tslib_1.__extends(LegendItem, _super);
        function LegendItem(attrs) {
            return _super.call(this, attrs) || this;
        }
        LegendItem.init_LegendItem = function () {
            this.define({
                label: [p.StringSpec, null],
                renderers: [p.Array, []],
                index: [p.Number, null],
            });
        };
        /*protected*/ LegendItem.prototype._check_data_sources_on_renderers = function () {
            var e_1, _b;
            var field = this.get_field_from_label_prop();
            if (field != null) {
                if (this.renderers.length < 1) {
                    return false;
                }
                var source = this.renderers[0].data_source;
                if (source != null) {
                    try {
                        for (var _c = tslib_1.__values(this.renderers), _d = _c.next(); !_d.done; _d = _c.next()) {
                            var r = _d.value;
                            if (r.data_source != source) {
                                return false;
                            }
                        }
                    }
                    catch (e_1_1) {
                        e_1 = { error: e_1_1 };
                    }
                    finally {
                        try {
                            if (_d && !_d.done && (_b = _c.return))
                                _b.call(_c);
                        }
                        finally {
                            if (e_1)
                                throw e_1.error;
                        }
                    }
                }
            }
            return true;
        };
        /*protected*/ LegendItem.prototype._check_field_label_on_data_source = function () {
            var field = this.get_field_from_label_prop();
            if (field != null) {
                if (this.renderers.length < 1) {
                    return false;
                }
                var source = this.renderers[0].data_source;
                if (source != null && !array_1.includes(source.columns(), field)) {
                    return false;
                }
            }
            return true;
        };
        LegendItem.prototype.initialize = function () {
            var _this = this;
            _super.prototype.initialize.call(this);
            this.legend = null;
            this.connect(this.change, function () { var _a; return (_a = _this.legend) === null || _a === void 0 ? void 0 : _a.item_change.emit(); });
            // Validate data_sources match
            var data_source_validation = this._check_data_sources_on_renderers();
            if (!data_source_validation)
                logging_1.logger.error("Non matching data sources on legend item renderers");
            // Validate label in data_source
            var field_validation = this._check_field_label_on_data_source();
            if (!field_validation)
                logging_1.logger.error("Bad column name on label: " + this.label);
        };
        LegendItem.prototype.get_field_from_label_prop = function () {
            var label = this.label;
            return vectorization_1.isField(label) ? label.field : null;
        };
        LegendItem.prototype.get_labels_list_from_label_prop = function () {
            // Always return a list of the labels
            if (vectorization_1.isValue(this.label)) {
                var value = this.label.value;
                return value != null ? [value] : [];
            }
            var field = this.get_field_from_label_prop();
            if (field != null) {
                var source = void 0;
                if (this.renderers[0] && this.renderers[0].data_source != null)
                    source = this.renderers[0].data_source;
                else
                    return ["No source found"];
                if (source instanceof columnar_data_source_1.ColumnarDataSource) {
                    var data = source.get_column(field);
                    if (data != null)
                        return array_1.uniq(Array.from(data));
                    else
                        return ["Invalid field"];
                }
            }
            return [];
        };
        return LegendItem;
    }(model_1.Model));
    exports.LegendItem = LegendItem;
    LegendItem.__name__ = "LegendItem";
    LegendItem.init_LegendItem();
},
/* core/vectorization.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var types_1 = require(123) /* ./util/types */;
    function isValue(obj) {
        return types_1.isPlainObject(obj) && "value" in obj;
    }
    exports.isValue = isValue;
    function isField(obj) {
        return types_1.isPlainObject(obj) && "field" in obj;
    }
    exports.isField = isField;
},
/* models/annotations/poly_annotation.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var signaling_1 = require(130) /* ../../core/signaling */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var PolyAnnotationView = /** @class */ (function (_super) {
        tslib_1.__extends(PolyAnnotationView, _super);
        function PolyAnnotationView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        PolyAnnotationView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            // need to respond to either normal BB change events or silent
            // "data only updates" that tools might want to use
            this.connect(this.model.change, function () { return _this.plot_view.request_render(); });
            this.connect(this.model.data_update, function () { return _this.plot_view.request_render(); });
        };
        PolyAnnotationView.prototype._render = function () {
            var _a = this.model, xs = _a.xs, ys = _a.ys;
            if (xs.length != ys.length)
                return;
            if (xs.length < 3 || ys.length < 3)
                return;
            var frame = this.plot_view.frame;
            var ctx = this.layer.ctx;
            for (var i = 0, end = xs.length; i < end; i++) {
                var sx = void 0;
                if (this.model.xs_units == 'screen')
                    sx = this.model.screen ? xs[i] : frame.xview.compute(xs[i]);
                else
                    throw new Error("not implemented");
                var sy = void 0;
                if (this.model.ys_units == 'screen')
                    sy = this.model.screen ? ys[i] : frame.yview.compute(ys[i]);
                else
                    throw new Error("not implemented");
                if (i == 0) {
                    ctx.beginPath();
                    ctx.moveTo(sx, sy);
                }
                else {
                    ctx.lineTo(sx, sy);
                }
            }
            ctx.closePath();
            if (this.visuals.line.doit) {
                this.visuals.line.set_value(ctx);
                ctx.stroke();
            }
            if (this.visuals.fill.doit) {
                this.visuals.fill.set_value(ctx);
                ctx.fill();
            }
        };
        return PolyAnnotationView;
    }(annotation_1.AnnotationView));
    exports.PolyAnnotationView = PolyAnnotationView;
    PolyAnnotationView.__name__ = "PolyAnnotationView";
    var PolyAnnotation = /** @class */ (function (_super) {
        tslib_1.__extends(PolyAnnotation, _super);
        function PolyAnnotation(attrs) {
            return _super.call(this, attrs) || this;
        }
        PolyAnnotation.init_PolyAnnotation = function () {
            this.prototype.default_view = PolyAnnotationView;
            this.mixins([mixins.Line /*Scalar*/, mixins.Fill /*Scalar*/]);
            this.define({
                xs: [p.Array, []],
                xs_units: [p.SpatialUnits, 'data'],
                ys: [p.Array, []],
                ys_units: [p.SpatialUnits, 'data'],
            });
            this.internal({
                screen: [p.Boolean, false],
            });
            this.override({
                fill_color: "#fff9ba",
                fill_alpha: 0.4,
                line_color: "#cccccc",
                line_alpha: 0.3,
            });
        };
        PolyAnnotation.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.data_update = new signaling_1.Signal0(this, "data_update");
        };
        PolyAnnotation.prototype.update = function (_a) {
            var xs = _a.xs, ys = _a.ys;
            this.setv({ xs: xs, ys: ys, screen: true }, { silent: true });
            this.data_update.emit();
        };
        return PolyAnnotation;
    }(annotation_1.Annotation));
    exports.PolyAnnotation = PolyAnnotation;
    PolyAnnotation.__name__ = "PolyAnnotation";
    PolyAnnotation.init_PolyAnnotation();
},
/* models/annotations/slope.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var SlopeView = /** @class */ (function (_super) {
        tslib_1.__extends(SlopeView, _super);
        function SlopeView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        SlopeView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
        };
        SlopeView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.change, function () { return _this.plot_view.request_render(); });
        };
        SlopeView.prototype._render = function () {
            var gradient = this.model.gradient;
            var y_intercept = this.model.y_intercept;
            if (gradient == null || y_intercept == null) {
                return;
            }
            var frame = this.plot_view.frame;
            var xscale = this.coordinates.x_scale;
            var yscale = this.coordinates.y_scale;
            var sy_start = frame.bbox.top;
            var sy_end = sy_start + frame.bbox.height;
            var y_start = yscale.invert(sy_start);
            var y_end = yscale.invert(sy_end);
            var x_start = (y_start - y_intercept) / gradient;
            var x_end = (y_end - y_intercept) / gradient;
            var sx_start = xscale.compute(x_start);
            var sx_end = xscale.compute(x_end);
            var ctx = this.layer.ctx;
            ctx.save();
            ctx.beginPath();
            this.visuals.line.set_value(ctx);
            ctx.moveTo(sx_start, sy_start);
            ctx.lineTo(sx_end, sy_end);
            ctx.stroke();
            ctx.restore();
        };
        return SlopeView;
    }(annotation_1.AnnotationView));
    exports.SlopeView = SlopeView;
    SlopeView.__name__ = "SlopeView";
    var Slope = /** @class */ (function (_super) {
        tslib_1.__extends(Slope, _super);
        function Slope(attrs) {
            return _super.call(this, attrs) || this;
        }
        Slope.init_Slope = function () {
            this.prototype.default_view = SlopeView;
            this.mixins(mixins.Line /*Scalar*/);
            this.define({
                gradient: [p.Number, null],
                y_intercept: [p.Number, null],
            });
            this.override({
                line_color: 'black',
            });
        };
        return Slope;
    }(annotation_1.Annotation));
    exports.Slope = Slope;
    Slope.__name__ = "Slope";
    Slope.init_Slope();
},
/* models/annotations/span.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var SpanView = /** @class */ (function (_super) {
        tslib_1.__extends(SpanView, _super);
        function SpanView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        SpanView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.change, function () { return _this.plot_view.request_paint(_this); });
        };
        SpanView.prototype._render = function () {
            var _this = this;
            var location = this.model.location;
            if (location == null) {
                return;
            }
            var frame = this.plot_view.frame;
            var xscale = this.coordinates.x_scale;
            var yscale = this.coordinates.y_scale;
            var _calc_dim = function (scale, view) {
                if (_this.model.location_units == 'data')
                    return scale.compute(location);
                else
                    return _this.model.for_hover ? location : view.compute(location);
            };
            var height, sleft, stop, width;
            if (this.model.dimension == 'width') {
                stop = _calc_dim(yscale, frame.yview);
                sleft = frame.bbox.left;
                width = frame.bbox.width;
                height = this.model.properties.line_width.value();
            }
            else {
                stop = frame.bbox.top;
                sleft = _calc_dim(xscale, frame.xview);
                width = this.model.properties.line_width.value();
                height = frame.bbox.height;
            }
            var ctx = this.layer.ctx;
            ctx.save();
            ctx.beginPath();
            this.visuals.line.set_value(ctx);
            ctx.moveTo(sleft, stop);
            if (this.model.dimension == "width") {
                ctx.lineTo(sleft + width, stop);
            }
            else {
                ctx.lineTo(sleft, stop + height);
            }
            ctx.stroke();
            ctx.restore();
        };
        return SpanView;
    }(annotation_1.AnnotationView));
    exports.SpanView = SpanView;
    SpanView.__name__ = "SpanView";
    var Span = /** @class */ (function (_super) {
        tslib_1.__extends(Span, _super);
        function Span(attrs) {
            return _super.call(this, attrs) || this;
        }
        Span.init_Span = function () {
            this.prototype.default_view = SpanView;
            this.mixins(mixins.Line /*Scalar*/);
            this.define({
                render_mode: [p.RenderMode, 'canvas'],
                location: [p.Number, null],
                location_units: [p.SpatialUnits, 'data'],
                dimension: [p.Dimension, 'width'],
            });
            this.override({
                line_color: 'black',
            });
            this.internal({
                for_hover: [p.Boolean, false],
            });
        };
        return Span;
    }(annotation_1.Annotation));
    exports.Span = Span;
    Span.__name__ = "Span";
    Span.init_Span();
},
/* models/annotations/title.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var text_annotation_1 = require(276) /* ./text_annotation */;
    var visuals_1 = require(189) /* ../../core/visuals */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var TitleView = /** @class */ (function (_super) {
        tslib_1.__extends(TitleView, _super);
        function TitleView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        TitleView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.visuals.text = new visuals_1.Text(this.model);
        };
        TitleView.prototype._get_location = function () {
            var panel = this.panel;
            var hmargin = this.model.offset;
            var vmargin = 5;
            var sx, sy;
            var bbox = panel.bbox;
            switch (panel.side) {
                case 'above':
                case 'below': {
                    switch (this.model.vertical_align) {
                        case 'top':
                            sy = bbox.top + vmargin;
                            break;
                        case 'middle':
                            sy = bbox.vcenter;
                            break;
                        case 'bottom':
                            sy = bbox.bottom - vmargin;
                            break;
                    }
                    switch (this.model.align) {
                        case 'left':
                            sx = bbox.left + hmargin;
                            break;
                        case 'center':
                            sx = bbox.hcenter;
                            break;
                        case 'right':
                            sx = bbox.right - hmargin;
                            break;
                    }
                    break;
                }
                case 'left': {
                    switch (this.model.vertical_align) {
                        case 'top':
                            sx = bbox.left - vmargin;
                            break;
                        case 'middle':
                            sx = bbox.hcenter;
                            break;
                        case 'bottom':
                            sx = bbox.right + vmargin;
                            break;
                    }
                    switch (this.model.align) {
                        case 'left':
                            sy = bbox.bottom - hmargin;
                            break;
                        case 'center':
                            sy = bbox.vcenter;
                            break;
                        case 'right':
                            sy = bbox.top + hmargin;
                            break;
                    }
                    break;
                }
                case 'right': {
                    switch (this.model.vertical_align) {
                        case 'top':
                            sx = bbox.right - vmargin;
                            break;
                        case 'middle':
                            sx = bbox.hcenter;
                            break;
                        case 'bottom':
                            sx = bbox.left + vmargin;
                            break;
                    }
                    switch (this.model.align) {
                        case 'left':
                            sy = bbox.top + hmargin;
                            break;
                        case 'center':
                            sy = bbox.vcenter;
                            break;
                        case 'right':
                            sy = bbox.bottom - hmargin;
                            break;
                    }
                    break;
                }
            }
            return [sx, sy];
        };
        TitleView.prototype._render = function () {
            var text = this.model.text;
            if (text == null || text.length == 0)
                return;
            this.model.text_baseline = this.model.vertical_align;
            this.model.text_align = this.model.align;
            var _a = tslib_1.__read(this._get_location(), 2), sx = _a[0], sy = _a[1];
            var angle = this.panel.get_label_angle_heuristic('parallel');
            var draw = this.model.render_mode == 'canvas' ? this._canvas_text.bind(this) : this._css_text.bind(this);
            draw(this.layer.ctx, text, sx, sy, angle);
        };
        TitleView.prototype._get_size = function () {
            var text = this.model.text;
            if (text == null || text.length == 0)
                return { width: 0, height: 0 };
            else {
                this.visuals.text.set_value(this.layer.ctx);
                var _a = this.layer.ctx.measureText(text), width = _a.width, ascent = _a.ascent;
                return { width: width, height: ascent * this.visuals.text.text_line_height.value() + 10 };
            }
        };
        return TitleView;
    }(text_annotation_1.TextAnnotationView));
    exports.TitleView = TitleView;
    TitleView.__name__ = "TitleView";
    var Title = /** @class */ (function (_super) {
        tslib_1.__extends(Title, _super);
        function Title(attrs) {
            return _super.call(this, attrs) || this;
        }
        Title.init_Title = function () {
            this.prototype.default_view = TitleView;
            this.mixins([
                ["border_", mixins.Line],
                ["background_", mixins.Fill],
            ]);
            this.define({
                text: [p.String],
                text_font: [p.Font, 'helvetica'],
                text_font_size: [p.StringSpec, '13px'],
                text_font_style: [p.FontStyle, 'bold'],
                text_color: [p.ColorSpec, '#444444'],
                text_alpha: [p.NumberSpec, 1.0],
                text_line_height: [p.Number, 1.0],
                vertical_align: [p.VerticalAlign, 'bottom'],
                align: [p.TextAlign, 'left'],
                offset: [p.Number, 0],
            });
            this.override({
                background_fill_color: null,
                border_line_color: null,
            });
            this.internal({
                text_align: [p.TextAlign, 'left'],
                text_baseline: [p.TextBaseline, 'bottom'],
            });
        };
        return Title;
    }(text_annotation_1.TextAnnotation));
    exports.Title = Title;
    Title.__name__ = "Title";
    Title.init_Title();
},
/* models/annotations/toolbar_panel.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var build_views_1 = require(230) /* ../../core/build_views */;
    var dom_1 = require(187) /* ../../core/dom */;
    var bbox_1 = require(194) /* ../../core/util/bbox */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var ToolbarPanelView = /** @class */ (function (_super) {
        tslib_1.__extends(ToolbarPanelView, _super);
        function ToolbarPanelView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.rotate = true;
            _this._invalidate_toolbar = true;
            _this._previous_bbox = new bbox_1.BBox();
            return _this;
        }
        ToolbarPanelView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.el = dom_1.div();
            this.plot_view.canvas_view.add_event(this.el);
        };
        ToolbarPanelView.prototype.lazy_initialize = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var _a;
                var _this = this;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            _a = this;
                            return [4 /*yield*/, build_views_1.build_view(this.model.toolbar, { parent: this })];
                        case 1:
                            _a._toolbar_view = _b.sent();
                            this.plot_view.visibility_callbacks.push(function (visible) { return _this._toolbar_view.set_visibility(visible); });
                            return [2 /*return*/];
                    }
                });
            });
        };
        ToolbarPanelView.prototype.remove = function () {
            this._toolbar_view.remove();
            dom_1.remove(this.el);
            _super.prototype.remove.call(this);
        };
        ToolbarPanelView.prototype.render = function () {
            if (!this.model.visible)
                dom_1.undisplay(this.el);
            _super.prototype.render.call(this);
        };
        ToolbarPanelView.prototype._render = function () {
            // TODO: this should be handled by the layout
            var bbox = this.panel.bbox;
            if (!this._previous_bbox.equals(bbox)) {
                dom_1.position(this.el, bbox);
                this._previous_bbox = bbox;
            }
            if (this._invalidate_toolbar) {
                this.el.style.position = "absolute";
                this.el.style.overflow = "hidden";
                this._toolbar_view.render();
                dom_1.empty(this.el);
                this.el.appendChild(this._toolbar_view.el);
                this._invalidate_toolbar = false;
            }
            dom_1.display(this.el);
        };
        ToolbarPanelView.prototype._get_size = function () {
            var _a = this.model.toolbar, tools = _a.tools, logo = _a.logo;
            return {
                width: tools.length * 30 + (logo != null ? 25 : 0),
                height: 30,
            };
        };
        return ToolbarPanelView;
    }(annotation_1.AnnotationView));
    exports.ToolbarPanelView = ToolbarPanelView;
    ToolbarPanelView.__name__ = "ToolbarPanelView";
    var ToolbarPanel = /** @class */ (function (_super) {
        tslib_1.__extends(ToolbarPanel, _super);
        function ToolbarPanel(attrs) {
            return _super.call(this, attrs) || this;
        }
        ToolbarPanel.init_ToolbarPanel = function () {
            this.prototype.default_view = ToolbarPanelView;
            this.define({
                toolbar: [p.Instance],
            });
        };
        return ToolbarPanel;
    }(annotation_1.Annotation));
    exports.ToolbarPanel = ToolbarPanel;
    ToolbarPanel.__name__ = "ToolbarPanel";
    ToolbarPanel.init_ToolbarPanel();
},
/* models/annotations/tooltip.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var annotation_1 = require(151) /* ./annotation */;
    var dom_1 = require(187) /* ../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var tooltips_1 = require(287) /* ../../styles/tooltips */;
    var mixins_1 = require(288) /* ../../styles/mixins */;
    var tooltips_css_1 = tslib_1.__importDefault(require(289) /* ../../styles/tooltips.css */);
    var arrow_size = 10; // XXX: keep in sync with less
    var TooltipView = /** @class */ (function (_super) {
        tslib_1.__extends(TooltipView, _super);
        function TooltipView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        TooltipView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.el = dom_1.div({ class: tooltips_1.bk_tooltip });
            dom_1.undisplay(this.el);
            this.plot_view.canvas_view.add_overlay(this.el);
        };
        TooltipView.prototype.remove = function () {
            dom_1.remove(this.el);
            _super.prototype.remove.call(this);
        };
        TooltipView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.properties.content.change, function () { return _this.render(); });
            this.connect(this.model.properties.position.change, function () { return _this._reposition(); });
        };
        TooltipView.prototype.styles = function () {
            return tslib_1.__spread(_super.prototype.styles.call(this), [tooltips_css_1.default]);
        };
        TooltipView.prototype.render = function () {
            if (!this.model.visible)
                dom_1.undisplay(this.el);
            _super.prototype.render.call(this);
        };
        TooltipView.prototype._render = function () {
            var content = this.model.content;
            if (content == null) {
                dom_1.undisplay(this.el);
                return;
            }
            dom_1.empty(this.el);
            dom_1.classes(this.el).toggle(tooltips_1.bk_tooltip_custom, this.model.custom);
            this.el.appendChild(content);
            if (this.model.show_arrow)
                this.el.classList.add(tooltips_1.bk_tooltip_arrow);
        };
        TooltipView.prototype._reposition = function () {
            var _this = this;
            var position = this.model.position;
            if (position == null) {
                dom_1.undisplay(this.el);
                return;
            }
            var _a = tslib_1.__read(position, 2), sx = _a[0], sy = _a[1];
            var side = (function () {
                var area = _this.parent.layout.bbox.relativize();
                var attachment = _this.model.attachment;
                switch (attachment) {
                    case "horizontal":
                        return sx < area.hcenter ? "right" : "left";
                    case "vertical":
                        return sy < area.vcenter ? "below" : "above";
                    default:
                        return attachment;
                }
            })();
            this.el.classList.remove(mixins_1.bk_right);
            this.el.classList.remove(mixins_1.bk_left);
            this.el.classList.remove(mixins_1.bk_above);
            this.el.classList.remove(mixins_1.bk_below);
            dom_1.display(this.el); // XXX: {offset,client}Width() gives 0 when display="none"
            // slightly confusing: side "left" (for example) is relative to point that
            // is being annotated but CS class ".bk-left" is relative to the tooltip itself
            var top;
            var left = null;
            var right = null;
            switch (side) {
                case "right":
                    this.el.classList.add(mixins_1.bk_left);
                    left = sx + (this.el.offsetWidth - this.el.clientWidth) + arrow_size;
                    top = sy - this.el.offsetHeight / 2;
                    break;
                case "left":
                    this.el.classList.add(mixins_1.bk_right);
                    right = (this.plot_view.layout.bbox.width - sx) + arrow_size;
                    top = sy - this.el.offsetHeight / 2;
                    break;
                case "below":
                    this.el.classList.add(mixins_1.bk_above);
                    top = sy + (this.el.offsetHeight - this.el.clientHeight) + arrow_size;
                    left = Math.round(sx - this.el.offsetWidth / 2);
                    break;
                case "above":
                    this.el.classList.add(mixins_1.bk_below);
                    top = sy - this.el.offsetHeight - arrow_size;
                    left = Math.round(sx - this.el.offsetWidth / 2);
                    break;
            }
            this.el.style.top = top + "px";
            this.el.style.left = left != null ? left + "px" : "auto";
            this.el.style.right = right != null ? right + "px" : "auto";
        };
        return TooltipView;
    }(annotation_1.AnnotationView));
    exports.TooltipView = TooltipView;
    TooltipView.__name__ = "TooltipView";
    var Tooltip = /** @class */ (function (_super) {
        tslib_1.__extends(Tooltip, _super);
        function Tooltip(attrs) {
            return _super.call(this, attrs) || this;
        }
        Tooltip.init_Tooltip = function () {
            this.prototype.default_view = TooltipView;
            this.define({
                attachment: [p.TooltipAttachment, 'horizontal'],
                inner_only: [p.Boolean, true],
                show_arrow: [p.Boolean, true],
            });
            this.override({
                level: 'overlay',
            });
            this.internal({
                position: [p.Any, null],
                content: [p.Any, function () { return dom_1.div(); }],
                custom: [p.Any],
            });
        };
        Tooltip.prototype.clear = function () {
            this.position = null;
        };
        return Tooltip;
    }(annotation_1.Annotation));
    exports.Tooltip = Tooltip;
    Tooltip.__name__ = "Tooltip";
    Tooltip.init_Tooltip();
},
/* styles/tooltips.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.bk_tooltip = "bk-tooltip";
    exports.bk_tooltip_arrow = "bk-tooltip-arrow";
    exports.bk_tooltip_custom = "bk-tooltip-custom";
    exports.bk_tooltip_row_label = "bk-tooltip-row-label";
    exports.bk_tooltip_row_value = "bk-tooltip-row-value";
    exports.bk_tooltip_color_block = "bk-tooltip-color-block";
},
/* styles/mixins.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.bk_active = "bk-active";
    exports.bk_inline = "bk-inline";
    exports.bk_left = "bk-left";
    exports.bk_right = "bk-right";
    exports.bk_above = "bk-above";
    exports.bk_below = "bk-below";
    exports.bk_up = "bk-up";
    exports.bk_down = "bk-down";
    function bk_side(side) {
        switch (side) {
            case "above": return exports.bk_above;
            case "below": return exports.bk_below;
            case "left": return exports.bk_left;
            case "right": return exports.bk_right;
        }
    }
    exports.bk_side = bk_side;
},
/* styles/tooltips.css.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var css = "\n.bk-root {\n  /* Same border color used everywhere */\n  /* Gray of icons */\n}\n.bk-root .bk-tooltip {\n  font-weight: 300;\n  font-size: 12px;\n  position: absolute;\n  padding: 5px;\n  border: 1px solid #e5e5e5;\n  color: #2f2f2f;\n  background-color: white;\n  pointer-events: none;\n  opacity: 0.95;\n  z-index: 100;\n}\n.bk-root .bk-tooltip > div:not(:first-child) {\n  /* gives space when multiple elements are being hovered over */\n  margin-top: 5px;\n  border-top: #e5e5e5 1px dashed;\n}\n.bk-root .bk-tooltip.bk-left.bk-tooltip-arrow::before {\n  position: absolute;\n  margin: -7px 0 0 0;\n  top: 50%;\n  width: 0;\n  height: 0;\n  border-style: solid;\n  border-width: 7px 0 7px 0;\n  border-color: transparent;\n  content: \" \";\n  display: block;\n  left: -10px;\n  border-right-width: 10px;\n  border-right-color: #909599;\n}\n.bk-root .bk-tooltip.bk-left::before {\n  left: -10px;\n  border-right-width: 10px;\n  border-right-color: #909599;\n}\n.bk-root .bk-tooltip.bk-right.bk-tooltip-arrow::after {\n  position: absolute;\n  margin: -7px 0 0 0;\n  top: 50%;\n  width: 0;\n  height: 0;\n  border-style: solid;\n  border-width: 7px 0 7px 0;\n  border-color: transparent;\n  content: \" \";\n  display: block;\n  right: -10px;\n  border-left-width: 10px;\n  border-left-color: #909599;\n}\n.bk-root .bk-tooltip.bk-right::after {\n  right: -10px;\n  border-left-width: 10px;\n  border-left-color: #909599;\n}\n.bk-root .bk-tooltip.bk-above::before {\n  position: absolute;\n  margin: 0 0 0 -7px;\n  left: 50%;\n  width: 0;\n  height: 0;\n  border-style: solid;\n  border-width: 0 7px 0 7px;\n  border-color: transparent;\n  content: \" \";\n  display: block;\n  top: -10px;\n  border-bottom-width: 10px;\n  border-bottom-color: #909599;\n}\n.bk-root .bk-tooltip.bk-below::after {\n  position: absolute;\n  margin: 0 0 0 -7px;\n  left: 50%;\n  width: 0;\n  height: 0;\n  border-style: solid;\n  border-width: 0 7px 0 7px;\n  border-color: transparent;\n  content: \" \";\n  display: block;\n  bottom: -10px;\n  border-top-width: 10px;\n  border-top-color: #909599;\n}\n.bk-root .bk-tooltip-row-label {\n  text-align: right;\n  color: #26aae1;\n  /* blue from toolbar highlighting */\n}\n.bk-root .bk-tooltip-row-value {\n  color: default;\n  /* seems to be necessary for notebook */\n}\n.bk-root .bk-tooltip-color-block {\n  width: 12px;\n  height: 12px;\n  margin-left: 5px;\n  margin-right: 5px;\n  outline: #dddddd solid 1px;\n  display: inline-block;\n}\n";
    exports.default = css;
},
/* models/annotations/whisker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var upper_lower_1 = require(238) /* ./upper_lower */;
    var arrow_head_1 = require(199) /* ./arrow_head */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var WhiskerView = /** @class */ (function (_super) {
        tslib_1.__extends(WhiskerView, _super);
        function WhiskerView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        WhiskerView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.source.streaming, function () { return _this.set_data(_this.model.source); });
            this.connect(this.model.source.patching, function () { return _this.set_data(_this.model.source); });
            this.connect(this.model.source.change, function () { return _this.set_data(_this.model.source); });
        };
        WhiskerView.prototype._render = function () {
            this._map_data();
            var ctx = this.layer.ctx;
            if (this.visuals.line.doit) {
                for (var i = 0, end = this._lower_sx.length; i < end; i++) {
                    this.visuals.line.set_vectorize(ctx, i);
                    ctx.beginPath();
                    ctx.moveTo(this._lower_sx[i], this._lower_sy[i]);
                    ctx.lineTo(this._upper_sx[i], this._upper_sy[i]);
                    ctx.stroke();
                }
            }
            var angle = this.model.dimension == "height" ? 0 : Math.PI / 2;
            if (this.model.lower_head != null) {
                for (var i = 0, end = this._lower_sx.length; i < end; i++) {
                    ctx.save();
                    ctx.translate(this._lower_sx[i], this._lower_sy[i]);
                    ctx.rotate(angle + Math.PI);
                    this.model.lower_head.render(ctx, i);
                    ctx.restore();
                }
            }
            if (this.model.upper_head != null) {
                for (var i = 0, end = this._upper_sx.length; i < end; i++) {
                    ctx.save();
                    ctx.translate(this._upper_sx[i], this._upper_sy[i]);
                    ctx.rotate(angle);
                    this.model.upper_head.render(ctx, i);
                    ctx.restore();
                }
            }
        };
        return WhiskerView;
    }(upper_lower_1.UpperLowerView));
    exports.WhiskerView = WhiskerView;
    WhiskerView.__name__ = "WhiskerView";
    var Whisker = /** @class */ (function (_super) {
        tslib_1.__extends(Whisker, _super);
        function Whisker(attrs) {
            return _super.call(this, attrs) || this;
        }
        Whisker.init_Whisker = function () {
            this.prototype.default_view = WhiskerView;
            this.mixins(property_mixins_1.LineVector);
            this.define({
                lower_head: [p.Instance, function () { return new arrow_head_1.TeeHead({ level: "underlay", size: 10 }); }],
                upper_head: [p.Instance, function () { return new arrow_head_1.TeeHead({ level: "underlay", size: 10 }); }],
            });
            this.override({
                level: 'underlay',
            });
        };
        return Whisker;
    }(upper_lower_1.UpperLower));
    exports.Whisker = Whisker;
    Whisker.__name__ = "Whisker";
    Whisker.init_Whisker();
},
/* models/axes/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var axis_1 = require(292) /* ./axis */;
    exports.Axis = axis_1.Axis;
    var categorical_axis_1 = require(294) /* ./categorical_axis */;
    exports.CategoricalAxis = categorical_axis_1.CategoricalAxis;
    var continuous_axis_1 = require(297) /* ./continuous_axis */;
    exports.ContinuousAxis = continuous_axis_1.ContinuousAxis;
    var datetime_axis_1 = require(298) /* ./datetime_axis */;
    exports.DatetimeAxis = datetime_axis_1.DatetimeAxis;
    var linear_axis_1 = require(299) /* ./linear_axis */;
    exports.LinearAxis = linear_axis_1.LinearAxis;
    var log_axis_1 = require(312) /* ./log_axis */;
    exports.LogAxis = log_axis_1.LogAxis;
    var mercator_axis_1 = require(315) /* ./mercator_axis */;
    exports.MercatorAxis = mercator_axis_1.MercatorAxis;
},
/* models/axes/axis.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var guide_renderer_1 = require(293) /* ../renderers/guide_renderer */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    var types_1 = require(123) /* ../../core/util/types */;
    var factor_range_1 = require(213) /* ../ranges/factor_range */;
    var abs = Math.abs, min = Math.min, max = Math.max;
    var AxisView = /** @class */ (function (_super) {
        tslib_1.__extends(AxisView, _super);
        function AxisView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.rotate = true;
            return _this;
        }
        Object.defineProperty(AxisView.prototype, "panel", {
            get: function () {
                return this.layout;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AxisView.prototype, "is_renderable", {
            get: function () {
                var _b = tslib_1.__read(this.ranges, 2), range = _b[0], cross_range = _b[1];
                return range.is_valid && cross_range.is_valid;
            },
            enumerable: true,
            configurable: true
        });
        AxisView.prototype._render = function () {
            var _a;
            if (!this.is_renderable)
                return;
            var extents = {
                tick: this._tick_extent(),
                tick_label: this._tick_label_extents(),
                axis_label: this._axis_label_extent(),
            };
            var tick_coords = this.tick_coords;
            var ctx = this.layer.ctx;
            ctx.save();
            this._draw_rule(ctx, extents);
            this._draw_major_ticks(ctx, extents, tick_coords);
            this._draw_minor_ticks(ctx, extents, tick_coords);
            this._draw_major_labels(ctx, extents, tick_coords);
            this._draw_axis_label(ctx, extents, tick_coords);
            (_a = this._paint) === null || _a === void 0 ? void 0 : _a.call(this, ctx, extents, tick_coords);
            ctx.restore();
        };
        AxisView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.change, function () { return _this.plot_view.request_layout(); });
        };
        AxisView.prototype.get_size = function () {
            if (this.model.visible && this.model.fixed_location == null && this.is_renderable) {
                var size = this._get_size();
                return { width: 0 /* max */, height: Math.round(size) };
            }
            else
                return { width: 0, height: 0 };
        };
        AxisView.prototype._get_size = function () {
            return this._tick_extent() + this._tick_label_extent() + this._axis_label_extent();
        };
        Object.defineProperty(AxisView.prototype, "needs_clip", {
            get: function () {
                return this.model.fixed_location != null;
            },
            enumerable: true,
            configurable: true
        });
        // drawing sub functions -----------------------------------------------------
        AxisView.prototype._draw_rule = function (ctx, _extents) {
            if (!this.visuals.axis_line.doit)
                return;
            var _b = tslib_1.__read(this.rule_coords, 2), xs = _b[0], ys = _b[1];
            var _c = tslib_1.__read(this.coordinates.map_to_screen(xs, ys), 2), sxs = _c[0], sys = _c[1];
            var _d = tslib_1.__read(this.normals, 2), nx = _d[0], ny = _d[1];
            var _e = tslib_1.__read(this.offsets, 2), xoff = _e[0], yoff = _e[1];
            this.visuals.axis_line.set_value(ctx);
            ctx.beginPath();
            ctx.moveTo(Math.round(sxs[0] + nx * xoff), Math.round(sys[0] + ny * yoff));
            for (var i = 1; i < sxs.length; i++) {
                var sx = Math.round(sxs[i] + nx * xoff);
                var sy = Math.round(sys[i] + ny * yoff);
                ctx.lineTo(sx, sy);
            }
            ctx.stroke();
        };
        AxisView.prototype._draw_major_ticks = function (ctx, _extents, tick_coords) {
            var tin = this.model.major_tick_in;
            var tout = this.model.major_tick_out;
            var visuals = this.visuals.major_tick_line;
            this._draw_ticks(ctx, tick_coords.major, tin, tout, visuals);
        };
        AxisView.prototype._draw_minor_ticks = function (ctx, _extents, tick_coords) {
            var tin = this.model.minor_tick_in;
            var tout = this.model.minor_tick_out;
            var visuals = this.visuals.minor_tick_line;
            this._draw_ticks(ctx, tick_coords.minor, tin, tout, visuals);
        };
        AxisView.prototype._draw_major_labels = function (ctx, extents, tick_coords) {
            var coords = tick_coords.major;
            var labels = this.compute_labels(coords[this.dimension]);
            var orient = this.model.major_label_orientation;
            var standoff = extents.tick + this.model.major_label_standoff;
            var visuals = this.visuals.major_label_text;
            this._draw_oriented_labels(ctx, labels, coords, orient, this.panel.side, standoff, visuals);
        };
        AxisView.prototype._draw_axis_label = function (ctx, extents, _tick_coords) {
            if (this.model.axis_label == null || this.model.axis_label.length == 0 || this.model.fixed_location != null)
                return;
            var sx;
            var sy;
            var bbox = this.panel.bbox;
            switch (this.panel.side) {
                case "above":
                    sx = bbox.hcenter;
                    sy = bbox.bottom;
                    break;
                case "below":
                    sx = bbox.hcenter;
                    sy = bbox.top;
                    break;
                case "left":
                    sx = bbox.right;
                    sy = bbox.vcenter;
                    break;
                case "right":
                    sx = bbox.left;
                    sy = bbox.vcenter;
                    break;
                default:
                    throw new Error("unknown side: " + this.panel.side);
            }
            var coords = [[sx], [sy]];
            var standoff = extents.tick + array_1.sum(extents.tick_label) + this.model.axis_label_standoff;
            var visuals = this.visuals.axis_label_text;
            this._draw_oriented_labels(ctx, [this.model.axis_label], coords, 'parallel', this.panel.side, standoff, visuals, "screen");
        };
        AxisView.prototype._draw_ticks = function (ctx, coords, tin, tout, visuals) {
            if (!visuals.doit)
                return;
            var _b = tslib_1.__read(coords, 2), x = _b[0], y = _b[1];
            var _c = tslib_1.__read(this.coordinates.map_to_screen(x, y), 2), sxs = _c[0], sys = _c[1];
            var _d = tslib_1.__read(this.normals, 2), nx = _d[0], ny = _d[1];
            var _e = tslib_1.__read(this.offsets, 2), xoff = _e[0], yoff = _e[1];
            var _f = tslib_1.__read([nx * (xoff - tin), ny * (yoff - tin)], 2), nxin = _f[0], nyin = _f[1];
            var _g = tslib_1.__read([nx * (xoff + tout), ny * (yoff + tout)], 2), nxout = _g[0], nyout = _g[1];
            visuals.set_value(ctx);
            ctx.beginPath();
            for (var i = 0; i < sxs.length; i++) {
                var sx0 = Math.round(sxs[i] + nxout);
                var sy0 = Math.round(sys[i] + nyout);
                var sx1 = Math.round(sxs[i] + nxin);
                var sy1 = Math.round(sys[i] + nyin);
                ctx.moveTo(sx0, sy0);
                ctx.lineTo(sx1, sy1);
            }
            ctx.stroke();
        };
        AxisView.prototype._draw_oriented_labels = function (ctx, labels, coords, orient, _side, standoff, visuals, units) {
            var _b, _c, _d, _e;
            if (units === void 0) {
                units = "data";
            }
            if (!visuals.doit || labels.length == 0)
                return;
            var sxs, sys;
            var xoff, yoff;
            if (units == "screen") {
                _b = tslib_1.__read(coords, 2), sxs = _b[0], sys = _b[1];
                _c = tslib_1.__read([0, 0], 2), xoff = _c[0], yoff = _c[1];
            }
            else {
                var _f = tslib_1.__read(coords, 2), dxs = _f[0], dys = _f[1];
                _d = tslib_1.__read(this.coordinates.map_to_screen(dxs, dys), 2), sxs = _d[0], sys = _d[1];
                _e = tslib_1.__read(this.offsets, 2), xoff = _e[0], yoff = _e[1];
            }
            var _g = tslib_1.__read(this.normals, 2), nx = _g[0], ny = _g[1];
            var nxd = nx * (xoff + standoff);
            var nyd = ny * (yoff + standoff);
            visuals.set_value(ctx);
            this.panel.apply_label_text_heuristics(ctx, orient);
            var angle;
            if (types_1.isString(orient))
                angle = this.panel.get_label_angle_heuristic(orient);
            else
                angle = -orient;
            for (var i = 0; i < sxs.length; i++) {
                var sx = Math.round(sxs[i] + nxd);
                var sy = Math.round(sys[i] + nyd);
                ctx.translate(sx, sy);
                ctx.rotate(angle);
                ctx.fillText(labels[i], 0, 0);
                ctx.rotate(-angle);
                ctx.translate(-sx, -sy);
            }
        };
        // extents sub functions -----------------------------------------------------
        /*protected*/ AxisView.prototype._axis_label_extent = function () {
            if (this.model.axis_label == null || this.model.axis_label == "")
                return 0;
            var standoff = this.model.axis_label_standoff;
            var visuals = this.visuals.axis_label_text;
            return this._oriented_labels_extent([this.model.axis_label], "parallel", this.panel.side, standoff, visuals);
        };
        /*protected*/ AxisView.prototype._tick_extent = function () {
            return this.model.major_tick_out;
        };
        /*protected*/ AxisView.prototype._tick_label_extent = function () {
            return array_1.sum(this._tick_label_extents());
        };
        AxisView.prototype._tick_label_extents = function () {
            var coords = this.tick_coords.major;
            var labels = this.compute_labels(coords[this.dimension]);
            var orient = this.model.major_label_orientation;
            var standoff = this.model.major_label_standoff;
            var visuals = this.visuals.major_label_text;
            return [this._oriented_labels_extent(labels, orient, this.panel.side, standoff, visuals)];
        };
        AxisView.prototype._oriented_labels_extent = function (labels, orient, side, standoff, visuals) {
            if (labels.length == 0)
                return 0;
            var ctx = this.layer.ctx;
            visuals.set_value(ctx);
            var hscale;
            var angle;
            if (types_1.isString(orient)) {
                hscale = 1;
                angle = this.panel.get_label_angle_heuristic(orient);
            }
            else {
                hscale = 2;
                angle = -orient;
            }
            angle = Math.abs(angle);
            var c = Math.cos(angle);
            var s = Math.sin(angle);
            var extent = 0;
            for (var i = 0; i < labels.length; i++) {
                var w = ctx.measureText(labels[i]).width * 1.1;
                var h = ctx.measureText(labels[i]).ascent * 0.9;
                var val = void 0;
                if (side == "above" || side == "below")
                    val = w * s + (h / hscale) * c;
                else
                    val = w * c + (h / hscale) * s;
                // update extent if current value is larger
                if (val > extent)
                    extent = val;
            }
            // only apply the standoff if we already have non-zero extent
            if (extent > 0)
                extent += standoff;
            return extent;
        };
        Object.defineProperty(AxisView.prototype, "normals", {
            // {{{ TODO: state
            get: function () {
                return this.panel.normals;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AxisView.prototype, "dimension", {
            get: function () {
                return this.panel.dimension;
            },
            enumerable: true,
            configurable: true
        });
        AxisView.prototype.compute_labels = function (ticks) {
            var labels = this.model.formatter.doFormat(ticks, this);
            for (var i = 0; i < ticks.length; i++) {
                if (ticks[i] in this.model.major_label_overrides)
                    labels[i] = this.model.major_label_overrides[ticks[i]];
            }
            return labels;
        };
        Object.defineProperty(AxisView.prototype, "offsets", {
            get: function () {
                // If we have a fixed_position then we should respect that exactly and
                // not apply any offsets (https://github.com/bokeh/bokeh/issues/8552)
                if (this.model.fixed_location != null)
                    return [0, 0];
                var frame = this.plot_view.frame;
                var _b = tslib_1.__read([0, 0], 2), xoff = _b[0], yoff = _b[1];
                switch (this.panel.side) {
                    case "below":
                        yoff = abs(this.panel.bbox.top - frame.bbox.bottom);
                        break;
                    case "above":
                        yoff = abs(this.panel.bbox.bottom - frame.bbox.top);
                        break;
                    case "right":
                        xoff = abs(this.panel.bbox.left - frame.bbox.right);
                        break;
                    case "left":
                        xoff = abs(this.panel.bbox.right - frame.bbox.left);
                        break;
                }
                return [xoff, yoff];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AxisView.prototype, "ranges", {
            get: function () {
                var i = this.dimension;
                var j = (i + 1) % 2;
                var ranges = this.coordinates.ranges;
                return [ranges[i], ranges[j]];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AxisView.prototype, "computed_bounds", {
            get: function () {
                var _b = tslib_1.__read(this.ranges, 1), range = _b[0];
                var user_bounds = this.model.bounds;
                var range_bounds = [range.min, range.max];
                if (user_bounds == "auto")
                    return [range.min, range.max];
                else {
                    var start = void 0;
                    var end = void 0;
                    var _c = tslib_1.__read(user_bounds, 2), user_start = _c[0], user_end = _c[1];
                    var _d = tslib_1.__read(range_bounds, 2), range_start = _d[0], range_end = _d[1];
                    if (abs(user_start - user_end) > abs(range_start - range_end)) {
                        start = max(min(user_start, user_end), range_start);
                        end = min(max(user_start, user_end), range_end);
                    }
                    else {
                        start = min(user_start, user_end);
                        end = max(user_start, user_end);
                    }
                    return [start, end];
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AxisView.prototype, "rule_coords", {
            get: function () {
                var i = this.dimension;
                var j = (i + 1) % 2;
                var _b = tslib_1.__read(this.ranges, 1), range = _b[0];
                var _c = tslib_1.__read(this.computed_bounds, 2), start = _c[0], end = _c[1];
                var xs = new Array(2);
                var ys = new Array(2);
                var coords = [xs, ys];
                coords[i][0] = Math.max(start, range.min);
                coords[i][1] = Math.min(end, range.max);
                if (coords[i][0] > coords[i][1])
                    coords[i][0] = coords[i][1] = NaN;
                coords[j][0] = this.loc;
                coords[j][1] = this.loc;
                return coords;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AxisView.prototype, "tick_coords", {
            get: function () {
                var i = this.dimension;
                var j = (i + 1) % 2;
                var _b = tslib_1.__read(this.ranges, 1), range = _b[0];
                var _c = tslib_1.__read(this.computed_bounds, 2), start = _c[0], end = _c[1];
                var ticks = this.model.ticker.get_ticks(start, end, range, this.loc, {});
                var majors = ticks.major;
                var minors = ticks.minor;
                var xs = [];
                var ys = [];
                var coords = [xs, ys];
                var minor_xs = [];
                var minor_ys = [];
                var minor_coords = [minor_xs, minor_ys];
                var _d = tslib_1.__read([range.min, range.max], 2), range_min = _d[0], range_max = _d[1];
                for (var ii = 0; ii < majors.length; ii++) {
                    if (majors[ii] < range_min || majors[ii] > range_max)
                        continue;
                    coords[i].push(majors[ii]);
                    coords[j].push(this.loc);
                }
                for (var ii = 0; ii < minors.length; ii++) {
                    if (minors[ii] < range_min || minors[ii] > range_max)
                        continue;
                    minor_coords[i].push(minors[ii]);
                    minor_coords[j].push(this.loc);
                }
                return {
                    major: coords,
                    minor: minor_coords,
                };
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AxisView.prototype, "loc", {
            get: function () {
                var fixed_location = this.model.fixed_location;
                if (fixed_location != null) {
                    if (types_1.isNumber(fixed_location))
                        return fixed_location;
                    var _b = tslib_1.__read(this.ranges, 2), cross_range_1 = _b[1];
                    if (cross_range_1 instanceof factor_range_1.FactorRange)
                        return cross_range_1.synthetic(fixed_location);
                    throw new Error("unexpected");
                }
                var _c = tslib_1.__read(this.ranges, 2), cross_range = _c[1];
                switch (this.panel.side) {
                    case 'left':
                    case 'below':
                        return cross_range.start;
                    case 'right':
                    case 'above':
                        return cross_range.end;
                }
            },
            enumerable: true,
            configurable: true
        });
        // }}}
        AxisView.prototype.serializable_state = function () {
            return Object.assign(Object.assign({}, _super.prototype.serializable_state.call(this)), { bbox: this.layout.bbox.box });
        };
        return AxisView;
    }(guide_renderer_1.GuideRendererView));
    exports.AxisView = AxisView;
    AxisView.__name__ = "AxisView";
    var Axis = /** @class */ (function (_super) {
        tslib_1.__extends(Axis, _super);
        function Axis(attrs) {
            return _super.call(this, attrs) || this;
        }
        Axis.init_Axis = function () {
            this.prototype.default_view = AxisView;
            this.mixins([
                ["axis_", mixins.Line],
                ["major_tick_", mixins.Line],
                ["minor_tick_", mixins.Line],
                ["major_label_", mixins.Text],
                ["axis_label_", mixins.Text],
            ]);
            this.define({
                bounds: [p.Any, 'auto'],
                ticker: [p.Instance],
                formatter: [p.Instance],
                axis_label: [p.String, ''],
                axis_label_standoff: [p.Int, 5],
                major_label_standoff: [p.Int, 5],
                major_label_orientation: [p.Any, "horizontal"],
                major_label_overrides: [p.Any, {}],
                major_tick_in: [p.Number, 2],
                major_tick_out: [p.Number, 6],
                minor_tick_in: [p.Number, 0],
                minor_tick_out: [p.Number, 4],
                fixed_location: [p.Any, null],
            });
            this.override({
                axis_line_color: 'black',
                major_tick_line_color: 'black',
                minor_tick_line_color: 'black',
                major_label_text_font_size: "11px",
                major_label_text_align: "center",
                major_label_text_baseline: "alphabetic",
                axis_label_text_font_size: "13px",
                axis_label_text_font_style: "italic",
            });
        };
        return Axis;
    }(guide_renderer_1.GuideRenderer));
    exports.Axis = Axis;
    Axis.__name__ = "Axis";
    Axis.init_Axis();
},
/* models/renderers/guide_renderer.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var renderer_1 = require(185) /* ./renderer */;
    var GuideRendererView = /** @class */ (function (_super) {
        tslib_1.__extends(GuideRendererView, _super);
        function GuideRendererView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return GuideRendererView;
    }(renderer_1.RendererView));
    exports.GuideRendererView = GuideRendererView;
    GuideRendererView.__name__ = "GuideRendererView";
    var GuideRenderer = /** @class */ (function (_super) {
        tslib_1.__extends(GuideRenderer, _super);
        function GuideRenderer(attrs) {
            return _super.call(this, attrs) || this;
        }
        GuideRenderer.init_GuideRenderer = function () {
            this.override({
                level: "guide",
            });
        };
        return GuideRenderer;
    }(renderer_1.Renderer));
    exports.GuideRenderer = GuideRenderer;
    GuideRenderer.__name__ = "GuideRenderer";
    GuideRenderer.init_GuideRenderer();
},
/* models/axes/categorical_axis.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var axis_1 = require(292) /* ./axis */;
    var categorical_ticker_1 = require(295) /* ../tickers/categorical_ticker */;
    var categorical_tick_formatter_1 = require(296) /* ../formatters/categorical_tick_formatter */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var CategoricalAxisView = /** @class */ (function (_super) {
        tslib_1.__extends(CategoricalAxisView, _super);
        function CategoricalAxisView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        CategoricalAxisView.prototype._paint = function (ctx, extents, tick_coords) {
            this._draw_group_separators(ctx, extents, tick_coords);
        };
        CategoricalAxisView.prototype._draw_group_separators = function (ctx, _extents, _tick_coords) {
            var _a;
            var _b = tslib_1.__read(this.ranges, 1), range = _b[0];
            var _c = tslib_1.__read(this.computed_bounds, 2), start = _c[0], end = _c[1];
            if (!range.tops || range.tops.length < 2 || !this.visuals.separator_line.doit)
                return;
            var dim = this.dimension;
            var alt = (dim + 1) % 2;
            var coords = [[], []];
            var ind = 0;
            for (var i = 0; i < range.tops.length - 1; i++) {
                var first = void 0, last = void 0;
                for (var j = ind; j < range.factors.length; j++) {
                    if (range.factors[j][0] == range.tops[i + 1]) {
                        _a = tslib_1.__read([range.factors[j - 1], range.factors[j]], 2), first = _a[0], last = _a[1];
                        ind = j;
                        break;
                    }
                }
                var pt = (range.synthetic(first) + range.synthetic(last)) / 2;
                if (pt > start && pt < end) {
                    coords[dim].push(pt);
                    coords[alt].push(this.loc);
                }
            }
            var tex = this._tick_label_extent();
            this._draw_ticks(ctx, coords, -3, (tex - 6), this.visuals.separator_line);
        };
        CategoricalAxisView.prototype._draw_major_labels = function (ctx, extents, _tick_coords) {
            var info = this._get_factor_info();
            var standoff = extents.tick + this.model.major_label_standoff;
            for (var i = 0; i < info.length; i++) {
                var _a = tslib_1.__read(info[i], 4), labels = _a[0], coords = _a[1], orient = _a[2], visuals = _a[3];
                this._draw_oriented_labels(ctx, labels, coords, orient, this.panel.side, standoff, visuals);
                standoff += extents.tick_label[i];
            }
        };
        CategoricalAxisView.prototype._tick_label_extents = function () {
            var e_1, _a;
            var info = this._get_factor_info();
            var extents = [];
            try {
                for (var info_1 = tslib_1.__values(info), info_1_1 = info_1.next(); !info_1_1.done; info_1_1 = info_1.next()) {
                    var _b = tslib_1.__read(info_1_1.value, 4), labels = _b[0], orient = _b[2], visuals = _b[3];
                    var extent = this._oriented_labels_extent(labels, orient, this.panel.side, this.model.major_label_standoff, visuals);
                    extents.push(extent);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (info_1_1 && !info_1_1.done && (_a = info_1.return))
                        _a.call(info_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return extents;
        };
        CategoricalAxisView.prototype._get_factor_info = function () {
            var _a = tslib_1.__read(this.ranges, 1), range = _a[0];
            var _b = tslib_1.__read(this.computed_bounds, 2), start = _b[0], end = _b[1];
            var loc = this.loc;
            var ticks = this.model.ticker.get_ticks(start, end, range, loc, {});
            var coords = this.tick_coords;
            var info = [];
            if (range.levels == 1) {
                var major = ticks.major;
                var labels = this.model.formatter.doFormat(major, this);
                info.push([labels, coords.major, this.model.major_label_orientation, this.visuals.major_label_text]);
            }
            else if (range.levels == 2) {
                var major = ticks.major.map(function (x) { return x[1]; });
                var labels = this.model.formatter.doFormat(major, this);
                info.push([labels, coords.major, this.model.major_label_orientation, this.visuals.major_label_text]);
                info.push([ticks.tops, coords.tops, this.model.group_label_orientation, this.visuals.group_text]);
            }
            else if (range.levels == 3) {
                var major = ticks.major.map(function (x) { return x[2]; });
                var labels = this.model.formatter.doFormat(major, this);
                var mid_labels = ticks.mids.map(function (x) { return x[1]; });
                info.push([labels, coords.major, this.model.major_label_orientation, this.visuals.major_label_text]);
                info.push([mid_labels, coords.mids, this.model.subgroup_label_orientation, this.visuals.subgroup_text]);
                info.push([ticks.tops, coords.tops, this.model.group_label_orientation, this.visuals.group_text]);
            }
            return info;
        };
        Object.defineProperty(CategoricalAxisView.prototype, "tick_coords", {
            // {{{ TODO: state
            get: function () {
                var _this = this;
                var i = this.dimension;
                var j = (i + 1) % 2;
                var _a = tslib_1.__read(this.ranges, 1), range = _a[0];
                var _b = tslib_1.__read(this.computed_bounds, 2), start = _b[0], end = _b[1];
                var ticks = this.model.ticker.get_ticks(start, end, range, this.loc, {});
                var coords = {
                    major: [[], []],
                    mids: [[], []],
                    tops: [[], []],
                    minor: [[], []],
                };
                coords.major[i] = ticks.major;
                coords.major[j] = ticks.major.map(function (_x) { return _this.loc; });
                if (range.levels == 3) {
                    coords.mids[i] = ticks.mids;
                    coords.mids[j] = ticks.mids.map(function (_x) { return _this.loc; });
                }
                if (range.levels > 1) {
                    coords.tops[i] = ticks.tops;
                    coords.tops[j] = ticks.tops.map(function (_x) { return _this.loc; });
                }
                return coords;
            },
            enumerable: true,
            configurable: true
        });
        return CategoricalAxisView;
    }(axis_1.AxisView));
    exports.CategoricalAxisView = CategoricalAxisView;
    CategoricalAxisView.__name__ = "CategoricalAxisView";
    var CategoricalAxis = /** @class */ (function (_super) {
        tslib_1.__extends(CategoricalAxis, _super);
        function CategoricalAxis(attrs) {
            return _super.call(this, attrs) || this;
        }
        CategoricalAxis.init_CategoricalAxis = function () {
            this.prototype.default_view = CategoricalAxisView;
            this.mixins([
                ["separator_", mixins.Line],
                ["group_", mixins.Text],
                ["subgroup_", mixins.Text],
            ]);
            this.define({
                group_label_orientation: [p.Any, "parallel"],
                subgroup_label_orientation: [p.Any, "parallel"],
            });
            this.override({
                ticker: function () { return new categorical_ticker_1.CategoricalTicker(); },
                formatter: function () { return new categorical_tick_formatter_1.CategoricalTickFormatter(); },
                separator_line_color: "lightgrey",
                separator_line_width: 2,
                group_text_font_style: "bold",
                group_text_font_size: "11px",
                group_text_color: "grey",
                subgroup_text_font_style: "bold",
                subgroup_text_font_size: "11px",
            });
        };
        return CategoricalAxis;
    }(axis_1.Axis));
    exports.CategoricalAxis = CategoricalAxis;
    CategoricalAxis.__name__ = "CategoricalAxis";
    CategoricalAxis.init_CategoricalAxis();
},
/* models/tickers/categorical_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var ticker_1 = require(244) /* ./ticker */;
    var CategoricalTicker = /** @class */ (function (_super) {
        tslib_1.__extends(CategoricalTicker, _super);
        function CategoricalTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        CategoricalTicker.prototype.get_ticks = function (start, end, range, _cross_loc, _) {
            var majors = this._collect(range.factors, range, start, end);
            var tops = this._collect(range.tops || [], range, start, end);
            var mids = this._collect(range.mids || [], range, start, end);
            return {
                major: majors,
                minor: [],
                tops: tops,
                mids: mids,
            };
        };
        CategoricalTicker.prototype._collect = function (factors, range, start, end) {
            var e_1, _a;
            var result = [];
            try {
                for (var factors_1 = tslib_1.__values(factors), factors_1_1 = factors_1.next(); !factors_1_1.done; factors_1_1 = factors_1.next()) {
                    var factor = factors_1_1.value;
                    var coord = range.synthetic(factor);
                    if (coord > start && coord < end)
                        result.push(factor);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (factors_1_1 && !factors_1_1.done && (_a = factors_1.return))
                        _a.call(factors_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return result;
        };
        return CategoricalTicker;
    }(ticker_1.Ticker));
    exports.CategoricalTicker = CategoricalTicker;
    CategoricalTicker.__name__ = "CategoricalTicker";
},
/* models/formatters/categorical_tick_formatter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var tick_formatter_1 = require(246) /* ./tick_formatter */;
    var array_1 = require(124) /* ../../core/util/array */;
    var CategoricalTickFormatter = /** @class */ (function (_super) {
        tslib_1.__extends(CategoricalTickFormatter, _super);
        function CategoricalTickFormatter(attrs) {
            return _super.call(this, attrs) || this;
        }
        CategoricalTickFormatter.prototype.doFormat = function (ticks, _opts) {
            return array_1.copy(ticks);
        };
        return CategoricalTickFormatter;
    }(tick_formatter_1.TickFormatter));
    exports.CategoricalTickFormatter = CategoricalTickFormatter;
    CategoricalTickFormatter.__name__ = "CategoricalTickFormatter";
},
/* models/axes/continuous_axis.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var axis_1 = require(292) /* ./axis */;
    var ContinuousAxis = /** @class */ (function (_super) {
        tslib_1.__extends(ContinuousAxis, _super);
        // TODO: __view_type__: ContinuousAxisView
        function ContinuousAxis(attrs) {
            return _super.call(this, attrs) || this;
        }
        return ContinuousAxis;
    }(axis_1.Axis));
    exports.ContinuousAxis = ContinuousAxis;
    ContinuousAxis.__name__ = "ContinuousAxis";
},
/* models/axes/datetime_axis.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var linear_axis_1 = require(299) /* ./linear_axis */;
    var datetime_tick_formatter_1 = require(300) /* ../formatters/datetime_tick_formatter */;
    var datetime_ticker_1 = require(305) /* ../tickers/datetime_ticker */;
    var DatetimeAxisView = /** @class */ (function (_super) {
        tslib_1.__extends(DatetimeAxisView, _super);
        function DatetimeAxisView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return DatetimeAxisView;
    }(linear_axis_1.LinearAxisView));
    exports.DatetimeAxisView = DatetimeAxisView;
    DatetimeAxisView.__name__ = "DatetimeAxisView";
    var DatetimeAxis = /** @class */ (function (_super) {
        tslib_1.__extends(DatetimeAxis, _super);
        function DatetimeAxis(attrs) {
            return _super.call(this, attrs) || this;
        }
        DatetimeAxis.init_DatetimeAxis = function () {
            this.prototype.default_view = DatetimeAxisView;
            this.override({
                ticker: function () { return new datetime_ticker_1.DatetimeTicker(); },
                formatter: function () { return new datetime_tick_formatter_1.DatetimeTickFormatter(); },
            });
        };
        return DatetimeAxis;
    }(linear_axis_1.LinearAxis));
    exports.DatetimeAxis = DatetimeAxis;
    DatetimeAxis.__name__ = "DatetimeAxis";
    DatetimeAxis.init_DatetimeAxis();
},
/* models/axes/linear_axis.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var axis_1 = require(292) /* ./axis */;
    var continuous_axis_1 = require(297) /* ./continuous_axis */;
    var basic_tick_formatter_1 = require(245) /* ../formatters/basic_tick_formatter */;
    var basic_ticker_1 = require(241) /* ../tickers/basic_ticker */;
    var LinearAxisView = /** @class */ (function (_super) {
        tslib_1.__extends(LinearAxisView, _super);
        function LinearAxisView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return LinearAxisView;
    }(axis_1.AxisView));
    exports.LinearAxisView = LinearAxisView;
    LinearAxisView.__name__ = "LinearAxisView";
    var LinearAxis = /** @class */ (function (_super) {
        tslib_1.__extends(LinearAxis, _super);
        function LinearAxis(attrs) {
            return _super.call(this, attrs) || this;
        }
        LinearAxis.init_LinearAxis = function () {
            this.prototype.default_view = LinearAxisView;
            this.override({
                ticker: function () { return new basic_ticker_1.BasicTicker(); },
                formatter: function () { return new basic_tick_formatter_1.BasicTickFormatter(); },
            });
        };
        return LinearAxis;
    }(continuous_axis_1.ContinuousAxis));
    exports.LinearAxis = LinearAxis;
    LinearAxis.__name__ = "LinearAxis";
    LinearAxis.init_LinearAxis();
},
/* models/formatters/datetime_tick_formatter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var timezone_1 = tslib_1.__importDefault(require(301) /* timezone */);
    var tick_formatter_1 = require(246) /* ./tick_formatter */;
    var logging_1 = require(134) /* ../../core/logging */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var templating_1 = require(302) /* ../../core/util/templating */;
    var array_1 = require(124) /* ../../core/util/array */;
    var types_1 = require(123) /* ../../core/util/types */;
    function _us(t) {
        // From double-precision unix (millisecond) timestamp get
        // microsecond since last second. Precision seems to run
        // out around the hundreds of nanoseconds scale, so rounding
        // to the nearest microsecond should round to a nice
        // microsecond / millisecond tick.
        return Math.round(((t / 1000) % 1) * 1000000);
    }
    function _array(t) {
        return timezone_1.default(t, "%Y %m %d %H %M %S").split(/\s+/).map(function (e) { return parseInt(e, 10); });
    }
    function _strftime(t, format) {
        if (types_1.isFunction(format)) {
            return format(t);
        }
        else {
            // Python's datetime library augments the microsecond directive %f, which is not
            // supported by the javascript library timezone: http://bigeasy.github.io/timezone/.
            // Use a regular expression to replace %f directive with microseconds.
            // TODO: what should we do for negative microsecond strings?
            var microsecond_replacement_string = templating_1.sprintf("$1%06d", _us(t));
            format = format.replace(/((^|[^%])(%%)*)%f/, microsecond_replacement_string);
            if (format.indexOf("%") == -1) {
                // timezone seems to ignore any strings without any formatting directives,
                // and just return the time argument back instead of the string argument.
                // But we want the string argument, in case a user supplies a format string
                // which doesn't contain a formatting directive or is only using %f.
                return format;
            }
            return timezone_1.default(t, format);
        }
    }
    // Labels of time units, from finest to coarsest.
    var format_order = [
        'microseconds', 'milliseconds', 'seconds', 'minsec', 'minutes', 'hourmin', 'hours', 'days', 'months', 'years',
    ];
    var DatetimeTickFormatter = /** @class */ (function (_super) {
        tslib_1.__extends(DatetimeTickFormatter, _super);
        function DatetimeTickFormatter(attrs) {
            var _this = _super.call(this, attrs) || this;
            // Whether or not to strip the leading zeros on tick labels.
            _this.strip_leading_zeros = true;
            return _this;
        }
        DatetimeTickFormatter.init_DatetimeTickFormatter = function () {
            this.define({
                microseconds: [p.Array, ['%fus']],
                milliseconds: [p.Array, ['%3Nms', '%S.%3Ns']],
                seconds: [p.Array, ['%Ss']],
                minsec: [p.Array, [':%M:%S']],
                minutes: [p.Array, [':%M', '%Mm']],
                hourmin: [p.Array, ['%H:%M']],
                hours: [p.Array, ['%Hh', '%H:%M']],
                days: [p.Array, ['%m/%d', '%a%d']],
                months: [p.Array, ['%m/%Y', '%b %Y']],
                years: [p.Array, ['%Y']],
            });
        };
        DatetimeTickFormatter.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            // TODO (bev) trigger update on format change
            this._update_width_formats();
        };
        DatetimeTickFormatter.prototype._update_width_formats = function () {
            var now = +timezone_1.default(new Date());
            var _widths = function (fmt_strings) {
                var sizes = fmt_strings.map(function (fmt_string) { return _strftime(now, fmt_string).length; });
                var sorted = array_1.sort_by(array_1.zip(sizes, fmt_strings), function (_a) {
                    var _b = tslib_1.__read(_a, 1), size = _b[0];
                    return size;
                });
                return array_1.unzip(sorted);
            };
            this._width_formats = {
                microseconds: _widths(this.microseconds),
                milliseconds: _widths(this.milliseconds),
                seconds: _widths(this.seconds),
                minsec: _widths(this.minsec),
                minutes: _widths(this.minutes),
                hourmin: _widths(this.hourmin),
                hours: _widths(this.hours),
                days: _widths(this.days),
                months: _widths(this.months),
                years: _widths(this.years),
            };
        };
        // FIXME There is some unfortunate flicker when panning/zooming near the
        // span boundaries.
        // FIXME Rounding is weird at the 20-us scale and below.
        DatetimeTickFormatter.prototype._get_resolution_str = function (resolution_secs, span_secs) {
            // Our resolution boundaries should not be round numbers, because we want
            // them to fall between the possible tick intervals (which *are* round
            // numbers, as we've worked hard to ensure).  Consequently, we adjust the
            // resolution upwards a small amount (less than any possible step in
            // scales) to make the effective boundaries slightly lower.
            var adjusted_secs = resolution_secs * 1.1;
            switch (false) {
                case !(adjusted_secs < 1e-3): return "microseconds";
                case !(adjusted_secs < 1.0): return "milliseconds";
                case !(adjusted_secs < 60): return span_secs >= 60 ? "minsec" : "seconds";
                case !(adjusted_secs < 3600): return span_secs >= 3600 ? "hourmin" : "minutes";
                case !(adjusted_secs < (24 * 3600)): return "hours";
                case !(adjusted_secs < (31 * 24 * 3600)): return "days";
                case !(adjusted_secs < (365 * 24 * 3600)): return "months";
                default: return "years";
            }
        };
        DatetimeTickFormatter.prototype.doFormat = function (ticks, _opts) {
            var e_1, _a, e_2, _b;
            // In order to pick the right set of labels, we need to determine
            // the resolution of the ticks.  We can do this using a ticker if
            // it's provided, or by computing the resolution from the actual
            // ticks we've been given.
            if (ticks.length == 0)
                return [];
            var span = Math.abs(ticks[ticks.length - 1] - ticks[0]) / 1000.0;
            var r = span / (ticks.length - 1);
            var resol = this._get_resolution_str(r, span);
            var _c = tslib_1.__read(this._width_formats[resol], 2), _d = tslib_1.__read(_c[1], 1), format = _d[0];
            // Apply the format to the tick values
            var labels = [];
            var resol_ndx = format_order.indexOf(resol);
            // This dictionary maps the name of a time resolution (in @format_order)
            // to its index in a time.localtime() timetuple.  The default is to map
            // everything to index 0, which is year.  This is not ideal; it might cause
            // a problem with the tick at midnight, january 1st, 0 a.d. being incorrectly
            // promoted at certain tick resolutions.
            var time_tuple_ndx_for_resol = {};
            try {
                for (var format_order_1 = tslib_1.__values(format_order), format_order_1_1 = format_order_1.next(); !format_order_1_1.done; format_order_1_1 = format_order_1.next()) {
                    var fmt = format_order_1_1.value;
                    time_tuple_ndx_for_resol[fmt] = 0;
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (format_order_1_1 && !format_order_1_1.done && (_a = format_order_1.return))
                        _a.call(format_order_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            time_tuple_ndx_for_resol.seconds = 5;
            time_tuple_ndx_for_resol.minsec = 4;
            time_tuple_ndx_for_resol.minutes = 4;
            time_tuple_ndx_for_resol.hourmin = 3;
            time_tuple_ndx_for_resol.hours = 3;
            try {
                // As we format each tick, check to see if we are at a boundary of the
                // next higher unit of time.  If so, replace the current format with one
                // from that resolution.  This is not the best heuristic in the world,
                // but it works!  There is some trickiness here due to having to deal
                // with hybrid formats in a reasonable manner.
                for (var ticks_1 = tslib_1.__values(ticks), ticks_1_1 = ticks_1.next(); !ticks_1_1.done; ticks_1_1 = ticks_1.next()) {
                    var t = ticks_1_1.value;
                    var s = void 0, tm = void 0;
                    try {
                        tm = _array(t);
                        s = _strftime(t, format);
                    }
                    catch (error) {
                        logging_1.logger.warn("unable to format tick for timestamp value " + t);
                        logging_1.logger.warn(" - " + error);
                        labels.push("ERR");
                        continue;
                    }
                    var hybrid_handled = false;
                    var next_ndx = resol_ndx;
                    // The way to check that we are at the boundary of the next unit of
                    // time is by checking that we have 0 units of the resolution, i.e.
                    // we are at zero minutes, so display hours, or we are at zero seconds,
                    // so display minutes (and if that is zero as well, then display hours).
                    while (tm[time_tuple_ndx_for_resol[format_order[next_ndx]]] == 0) {
                        var next_format = void 0;
                        next_ndx += 1;
                        if (next_ndx == format_order.length)
                            break;
                        if ((resol == "minsec" || resol == "hourmin") && !hybrid_handled) {
                            if ((resol == "minsec" && tm[4] == 0 && tm[5] != 0) || (resol == "hourmin" && tm[3] == 0 && tm[4] != 0)) {
                                next_format = this._width_formats[format_order[resol_ndx - 1]][1][0];
                                s = _strftime(t, next_format);
                                break;
                            }
                            else {
                                hybrid_handled = true;
                            }
                        }
                        next_format = this._width_formats[format_order[next_ndx]][1][0];
                        s = _strftime(t, next_format);
                    }
                    // TODO: should expose this in api. %H, %d, etc use leading zeros and
                    // users might prefer to see them lined up correctly.
                    if (this.strip_leading_zeros) {
                        var ss = s.replace(/^0+/g, "");
                        if (ss != s && isNaN(parseInt(ss))) {
                            // If the string can now be parsed as starting with an integer, then
                            // leave all zeros stripped, otherwise start with a zero. Hence:
                            // A label such as '000ms' should leave one zero.
                            // A label such as '001ms' or '0-1ms' should not leave a leading zero.
                            ss = "0" + ss;
                        }
                        labels.push(ss);
                    }
                    else
                        labels.push(s);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (ticks_1_1 && !ticks_1_1.done && (_b = ticks_1.return))
                        _b.call(ticks_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return labels;
        };
        return DatetimeTickFormatter;
    }(tick_formatter_1.TickFormatter));
    exports.DatetimeTickFormatter = DatetimeTickFormatter;
    DatetimeTickFormatter.__name__ = "DatetimeTickFormatter";
    DatetimeTickFormatter.init_DatetimeTickFormatter();
},
/* timezone/index.js */ function _(require, module, exports) {
    !function (definition) {
        if (typeof module == "object" && module.exports)
            module.exports = definition();
        else if (typeof define == "function")
            define(definition);
        else
            this.tz = definition();
    }(function () {
        /*
          function die () {
            console.log.apply(console, __slice.call(arguments, 0));
            return process.exit(1);
          }
        
          function say () { return console.log.apply(console, __slice.call(arguments, 0)) }
        */
        function actualize(entry, rule, year) {
            var actualized, date = rule.day[1];
            do {
                actualized = new Date(Date.UTC(year, rule.month, Math.abs(date++)));
            } while (rule.day[0] < 7 && actualized.getUTCDay() != rule.day[0]);
            actualized = {
                clock: rule.clock,
                sort: actualized.getTime(),
                rule: rule,
                save: rule.save * 6e4,
                offset: entry.offset
            };
            actualized[actualized.clock] = actualized.sort + rule.time * 6e4;
            if (actualized.posix) {
                actualized.wallclock = actualized[actualized.clock] + (entry.offset + rule.saved);
            }
            else {
                actualized.posix = actualized[actualized.clock] - (entry.offset + rule.saved);
            }
            return actualized;
        }
        function find(request, clock, time) {
            var i, I, entry, found, zone = request[request.zone], actualized = [], abbrev, rules, j, year = new Date(time).getUTCFullYear(), off = 1;
            for (i = 1, I = zone.length; i < I; i++)
                if (zone[i][clock] <= time)
                    break;
            entry = zone[i];
            if (entry.rules) {
                rules = request[entry.rules];
                for (j = year + 1; j >= year - off; --j)
                    for (i = 0, I = rules.length; i < I; i++)
                        if (rules[i].from <= j && j <= rules[i].to)
                            actualized.push(actualize(entry, rules[i], j));
                        else if (rules[i].to < j && off == 1)
                            off = j - rules[i].to;
                actualized.sort(function (a, b) { return a.sort - b.sort; });
                for (i = 0, I = actualized.length; i < I; i++) {
                    if (time >= actualized[i][clock] && actualized[i][actualized[i].clock] > entry[actualized[i].clock])
                        found = actualized[i];
                }
            }
            if (found) {
                if (abbrev = /^(.*)\/(.*)$/.exec(entry.format)) {
                    found.abbrev = abbrev[found.save ? 2 : 1];
                }
                else {
                    found.abbrev = entry.format.replace(/%s/, found.rule.letter);
                }
            }
            return found || entry;
        }
        function convertToWallclock(request, posix) {
            if (request.zone == "UTC")
                return posix;
            request.entry = find(request, "posix", posix);
            return posix + request.entry.offset + request.entry.save;
        }
        function convertToPOSIX(request, wallclock) {
            if (request.zone == "UTC")
                return wallclock;
            var entry, diff;
            request.entry = entry = find(request, "wallclock", wallclock);
            diff = wallclock - entry.wallclock;
            return 0 < diff && diff < entry.save ? null : wallclock - entry.offset - entry.save;
        }
        function adjust(request, posix, match) {
            var increment = +(match[1] + 1) // conversion necessary for week day addition
            , offset = match[2] * increment, index = UNITS.indexOf(match[3].toLowerCase()), date;
            if (index > 9) {
                posix += offset * TIME[index - 10];
            }
            else {
                date = new Date(convertToWallclock(request, posix));
                if (index < 7) {
                    while (offset) {
                        date.setUTCDate(date.getUTCDate() + increment);
                        if (date.getUTCDay() == index)
                            offset -= increment;
                    }
                }
                else if (index == 7) {
                    date.setUTCFullYear(date.getUTCFullYear() + offset);
                }
                else if (index == 8) {
                    date.setUTCMonth(date.getUTCMonth() + offset);
                }
                else {
                    date.setUTCDate(date.getUTCDate() + offset);
                }
                if ((posix = convertToPOSIX(request, date.getTime())) == null) {
                    posix = convertToPOSIX(request, date.getTime() + 864e5 * increment) - 864e5 * increment;
                }
            }
            return posix;
        }
        function convert(vargs) {
            if (!vargs.length)
                return "1.0.23";
            var request = Object.create(this), adjustments = [], i, I, $, argument, date;
            for (i = 0; i < vargs.length; i++) { // leave the for loop alone, it works.
                argument = vargs[i];
                // https://twitter.com/bigeasy/status/215112186572439552
                if (Array.isArray(argument)) {
                    if (!i && !isNaN(argument[1])) {
                        date = argument;
                    }
                    else {
                        argument.splice.apply(vargs, [i--, 1].concat(argument));
                    }
                }
                else if (isNaN(argument)) {
                    $ = typeof argument;
                    if ($ == "string") {
                        if (~argument.indexOf("%")) {
                            request.format = argument;
                        }
                        else if (!i && argument == "*") {
                            date = argument;
                        }
                        else if (!i && ($ = /^(\d{4})-(\d{2})-(\d{2})(?:[T\s](\d{2}):(\d{2})(?::(\d{2})(?:\.(\d+))?)?(Z|(([+-])(\d{2}(:\d{2}){0,2})))?)?$/.exec(argument))) {
                            date = [];
                            date.push.apply(date, $.slice(1, 8));
                            if ($[9]) {
                                date.push($[10] + 1);
                                date.push.apply(date, $[11].split(/:/));
                            }
                            else if ($[8]) {
                                date.push(1);
                            }
                        }
                        else if (/^\w{2,3}_\w{2}$/.test(argument)) {
                            request.locale = argument;
                        }
                        else if ($ = UNIT_RE.exec(argument)) {
                            adjustments.push($);
                        }
                        else {
                            request.zone = argument;
                        }
                    }
                    else if ($ == "function") {
                        if ($ = argument.call(request))
                            return $;
                    }
                    else if (/^\w{2,3}_\w{2}$/.test(argument.name)) {
                        request[argument.name] = argument;
                    }
                    else if (argument.zones) {
                        for ($ in argument.zones)
                            request[$] = argument.zones[$];
                        for ($ in argument.rules)
                            request[$] = argument.rules[$];
                    }
                }
                else if (!i) {
                    date = argument;
                }
            }
            if (!request[request.locale])
                delete request.locale;
            if (!request[request.zone])
                delete request.zone;
            if (date != null) {
                if (date == "*") {
                    date = request.clock();
                }
                else if (Array.isArray(date)) {
                    $ = [];
                    I = !date[7];
                    for (i = 0; i < 11; i++)
                        $[i] = +(date[i] || 0); // conversion necessary for decrement
                    --$[1]; // Grr..
                    date = Date.UTC.apply(Date.UTC, $) + -$[7] * ($[8] * 36e5 + $[9] * 6e4 + $[10] * 1e3);
                }
                else {
                    date = Math.floor(date);
                }
                if (!isNaN(date)) {
                    if (I)
                        date = convertToPOSIX(request, date);
                    if (date == null)
                        return date;
                    for (i = 0, I = adjustments.length; i < I; i++) {
                        date = adjust(request, date, adjustments[i]);
                    }
                    if (!request.format)
                        return date;
                    $ = new Date(convertToWallclock(request, date));
                    return request.format.replace(/%([-0_^]?)(:{0,3})(\d*)(.)/g, function (value, flag, colons, padding, specifier) {
                        var f, fill = "0", pad;
                        if (f = request[specifier]) {
                            value = String(f.call(request, $, date, flag, colons.length));
                            if ((flag || f.style) == "_")
                                fill = " ";
                            pad = flag == "-" ? 0 : f.pad || 0;
                            while (value.length < pad)
                                value = fill + value;
                            pad = flag == "-" ? 0 : padding || f.pad;
                            while (value.length < pad)
                                value = fill + value;
                            if (specifier == "N" && pad < value.length)
                                value = value.slice(0, pad);
                            if (flag == "^")
                                value = value.toUpperCase();
                        }
                        return value;
                    });
                }
            }
            return function () { return request.convert(arguments); };
        }
        var context = { clock: function () { return +(new Date()); },
            zone: "UTC",
            entry: { abbrev: "UTC", offset: 0, save: 0 },
            UTC: 1,
            z: function (date, posix, flag, delimiters) {
                var offset = this.entry.offset + this.entry.save, seconds = Math.abs(offset / 1000), parts = [], part = 3600, i, z;
                for (i = 0; i < 3; i++) {
                    parts.push(("0" + Math.floor(seconds / part)).slice(-2));
                    seconds %= part;
                    part /= 60;
                }
                if (flag == "^" && !offset)
                    return "Z";
                if (flag == "^")
                    delimiters = 3;
                if (delimiters == 3) {
                    z = parts.join(":");
                    z = z.replace(/:00$/, "");
                    if (flag != "^")
                        z = z.replace(/:00$/, "");
                }
                else if (delimiters) {
                    z = parts.slice(0, delimiters + 1).join(":");
                    if (flag == "^")
                        z = z.replace(/:00$/, "");
                }
                else {
                    z = parts.slice(0, 2).join("");
                }
                z = (offset < 0 ? "-" : "+") + z;
                z = z.replace(/([-+])(0)/, { "_": " $1", "-": "$1" }[flag] || "$1$2");
                return z;
            },
            "%": function (date) { return "%"; },
            n: function (date) { return "\n"; },
            t: function (date) { return "\t"; },
            U: function (date) { return weekOfYear(date, 0); },
            W: function (date) { return weekOfYear(date, 1); },
            V: function (date) { return isoWeek(date)[0]; },
            G: function (date) { return isoWeek(date)[1]; },
            g: function (date) { return isoWeek(date)[1] % 100; },
            j: function (date) { return Math.floor((date.getTime() - Date.UTC(date.getUTCFullYear(), 0)) / 864e5) + 1; },
            s: function (date) { return Math.floor(date.getTime() / 1000); },
            C: function (date) { return Math.floor(date.getUTCFullYear() / 100); },
            N: function (date) { return date.getTime() % 1000 * 1000000; },
            m: function (date) { return date.getUTCMonth() + 1; },
            Y: function (date) { return date.getUTCFullYear(); },
            y: function (date) { return date.getUTCFullYear() % 100; },
            H: function (date) { return date.getUTCHours(); },
            M: function (date) { return date.getUTCMinutes(); },
            S: function (date) { return date.getUTCSeconds(); },
            e: function (date) { return date.getUTCDate(); },
            d: function (date) { return date.getUTCDate(); },
            u: function (date) { return date.getUTCDay() || 7; },
            w: function (date) { return date.getUTCDay(); },
            l: function (date) { return date.getUTCHours() % 12 || 12; },
            I: function (date) { return date.getUTCHours() % 12 || 12; },
            k: function (date) { return date.getUTCHours(); },
            Z: function (date) { return this.entry.abbrev; },
            a: function (date) { return this[this.locale].day.abbrev[date.getUTCDay()]; },
            A: function (date) { return this[this.locale].day.full[date.getUTCDay()]; },
            h: function (date) { return this[this.locale].month.abbrev[date.getUTCMonth()]; },
            b: function (date) { return this[this.locale].month.abbrev[date.getUTCMonth()]; },
            B: function (date) { return this[this.locale].month.full[date.getUTCMonth()]; },
            P: function (date) { return this[this.locale].meridiem[Math.floor(date.getUTCHours() / 12)].toLowerCase(); },
            p: function (date) { return this[this.locale].meridiem[Math.floor(date.getUTCHours() / 12)]; },
            R: function (date, posix) { return this.convert([posix, "%H:%M"]); },
            T: function (date, posix) { return this.convert([posix, "%H:%M:%S"]); },
            D: function (date, posix) { return this.convert([posix, "%m/%d/%y"]); },
            F: function (date, posix) { return this.convert([posix, "%Y-%m-%d"]); },
            x: function (date, posix) { return this.convert([posix, this[this.locale].date]); },
            r: function (date, posix) { return this.convert([posix, this[this.locale].time12 || '%I:%M:%S']); },
            X: function (date, posix) { return this.convert([posix, this[this.locale].time24]); },
            c: function (date, posix) { return this.convert([posix, this[this.locale].dateTime]); },
            convert: convert,
            locale: "en_US",
            en_US: {
                date: "%m/%d/%Y",
                time24: "%I:%M:%S %p",
                time12: "%I:%M:%S %p",
                dateTime: "%a %d %b %Y %I:%M:%S %p %Z",
                meridiem: ["AM", "PM"],
                month: {
                    abbrev: "Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec".split("|"),
                    full: "January|February|March|April|May|June|July|August|September|October|November|December".split("|")
                },
                day: {
                    abbrev: "Sun|Mon|Tue|Wed|Thu|Fri|Sat".split("|"),
                    full: "Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday".split("|")
                }
            }
        };
        var UNITS = "Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday|year|month|day|hour|minute|second|millisecond", UNIT_RE = new RegExp("^\\s*([+-])(\\d+)\\s+(" + UNITS + ")s?\\s*$", "i"), TIME = [36e5, 6e4, 1e3, 1];
        UNITS = UNITS.toLowerCase().split("|");
        "delmHMSUWVgCIky".replace(/./g, function (e) { context[e].pad = 2; });
        context.N.pad = 9;
        context.j.pad = 3;
        context.k.style = "_";
        context.l.style = "_";
        context.e.style = "_";
        function weekOfYear(date, startOfWeek) {
            var diff, nyd, weekStart;
            nyd = new Date(Date.UTC(date.getUTCFullYear(), 0));
            diff = Math.floor((date.getTime() - nyd.getTime()) / 864e5);
            if (nyd.getUTCDay() == startOfWeek) {
                weekStart = 0;
            }
            else {
                weekStart = 7 - nyd.getUTCDay() + startOfWeek;
                if (weekStart == 8) {
                    weekStart = 1;
                }
            }
            return diff >= weekStart ? Math.floor((diff - weekStart) / 7) + 1 : 0;
        }
        function isoWeek(date) {
            var nyd, nyy, week;
            nyy = date.getUTCFullYear();
            nyd = new Date(Date.UTC(nyy, 0)).getUTCDay();
            week = weekOfYear(date, 1) + (nyd > 1 && nyd <= 4 ? 1 : 0);
            if (!week) {
                nyy = date.getUTCFullYear() - 1;
                nyd = new Date(Date.UTC(nyy, 0)).getUTCDay();
                week = nyd == 4 || (nyd == 3 && new Date(nyy, 1, 29).getDate() == 29) ? 53 : 52;
                return [week, date.getUTCFullYear() - 1];
            }
            else if (week == 53 && !(nyd == 4 || (nyd == 3 && new Date(nyy, 1, 29).getDate() == 29))) {
                return [1, date.getUTCFullYear() + 1];
            }
            else {
                return [week, date.getUTCFullYear()];
            }
        }
        return function () { return context.convert(arguments); };
    });
},
/* core/util/templating.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var Numbro = tslib_1.__importStar(require(303) /* @bokeh/numbro */);
    var sprintf_js_1 = require(304) /* sprintf-js */;
    var timezone_1 = tslib_1.__importDefault(require(301) /* timezone */);
    var string_1 = require(144) /* ./string */;
    var types_1 = require(123) /* ./types */;
    exports.DEFAULT_FORMATTERS = {
        numeral: function (value, format, _special_vars) { return Numbro.format(value, format); },
        datetime: function (value, format, _special_vars) { return timezone_1.default(value, format); },
        printf: function (value, format, _special_vars) { return sprintf(format, value); },
    };
    function sprintf(format) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return sprintf_js_1.sprintf.apply(void 0, tslib_1.__spread([format], args));
    }
    exports.sprintf = sprintf;
    function basic_formatter(value, _format, _special_vars) {
        if (types_1.isNumber(value)) {
            var format = (function () {
                switch (false) {
                    case Math.floor(value) != value:
                        return "%d";
                    case !(Math.abs(value) > 0.1) || !(Math.abs(value) < 1000):
                        return "%0.3f";
                    default:
                        return "%0.3e";
                }
            })();
            return sprintf(format, value);
        }
        else
            return "" + value; // get strings for categorical types
    }
    exports.basic_formatter = basic_formatter;
    function get_formatter(raw_spec, format, formatters) {
        // no format, use default built in formatter
        if (format == null)
            return basic_formatter;
        // format spec in the formatters dict, use that
        if (formatters != null && raw_spec in formatters) {
            var formatter_1 = formatters[raw_spec];
            if (types_1.isString(formatter_1)) {
                if (formatter_1 in exports.DEFAULT_FORMATTERS)
                    return exports.DEFAULT_FORMATTERS[formatter_1];
                else
                    throw new Error("Unknown tooltip field formatter type '" + formatter_1 + "'");
            }
            return function (value, format, special_vars) {
                return formatter_1.format(value, format, special_vars);
            };
        }
        // otherwise use "numeral" as default
        return exports.DEFAULT_FORMATTERS.numeral;
    }
    exports.get_formatter = get_formatter;
    function _get_special_value(name, special_vars) {
        if (name in special_vars)
            return special_vars[name];
        else
            throw new Error("Unknown special variable '$" + name + "'");
    }
    function _get_column_value(name, data_source, i) {
        var column = data_source.get_column(name);
        // missing column
        if (column == null)
            return null;
        // typical (non-image) index
        if (types_1.isNumber(i))
            return column[i];
        // image index
        var data = column[i.index];
        if (types_1.isTypedArray(data) || types_1.isArray(data)) {
            // inspect array of arrays
            if (types_1.isArray(data[0])) {
                var row = data[i.dim2];
                return row[i.dim1];
            }
            else
                return data[i.flat_index]; // inspect flat array
        }
        else
            return data; // inspect per-image scalar data
    }
    function get_value(raw_name, data_source, i, special_vars) {
        if (raw_name[0] == "$") {
            var name = raw_name.substring(1);
            return _get_special_value(name, special_vars);
        }
        else {
            var name = raw_name.substring(1).replace(/[{}]/g, "");
            return _get_column_value(name, data_source, i);
        }
    }
    exports.get_value = get_value;
    function replace_placeholders(content, data_source, i, formatters, special_vars) {
        if (special_vars === void 0) {
            special_vars = {};
        }
        var str;
        var has_html;
        if (types_1.isString(content)) {
            str = content;
            has_html = false;
        }
        else {
            str = content.html;
            has_html = true;
        }
        // this handles the special case @$name, replacing it with an @var corresponding to special_vars.name
        str = str.replace(/@\$name/g, function (_match) { return "@{" + special_vars.name + "}"; });
        //
        // (?:\$\w+) - special vars: $x
        // (?:@\w+) - simple names: @foo
        // (?:@{(?:[^{}]+)})) - full names: @{one two}
        //
        // (?:{([^{}]+)})? - (optional) format for all of the above: @foo{fmt}
        //
        str = str.replace(/((?:\$\w+)|(?:@\w+)|(?:@{(?:[^{}]+)}))(?:{([^{}]+)})?/g, function (_match, spec, format) {
            var value = get_value(spec, data_source, i, special_vars);
            // missing value, return ???
            if (value == null)
                return "" + string_1.escape("???");
            // 'safe' format, return the value as-is
            if (format == 'safe') {
                has_html = true;
                return "" + value;
            }
            // format and escape everything else
            var formatter = get_formatter(spec, format, formatters);
            return "" + string_1.escape(formatter(value, format, special_vars));
        });
        if (!has_html)
            return str;
        else {
            var parser = new DOMParser();
            var document = parser.parseFromString(str, "text/html");
            return tslib_1.__spread(document.body.childNodes);
        }
    }
    exports.replace_placeholders = replace_placeholders;
},
/* @bokeh/numbro/numbro.js */ function _(require, module, exports) {
    /*!
     * numbro.js
     * version : 1.6.2
     * author : Företagsplatsen AB
     * license : MIT
     * http://www.foretagsplatsen.se
     */
    /************************************
        Constants
    ************************************/
    var numbro, VERSION = '1.6.2', 
    // internal storage for culture config files
    cultures = {}, 
    // Todo: Remove in 2.0.0
    languages = cultures, currentCulture = 'en-US', zeroFormat = null, defaultFormat = '0,0', defaultCurrencyFormat = '0$', 
    // check for nodeJS
    hasModule = (typeof module !== 'undefined' && module.exports), 
    // default culture
    enUS = {
        delimiters: {
            thousands: ',',
            decimal: '.'
        },
        abbreviations: {
            thousand: 'k',
            million: 'm',
            billion: 'b',
            trillion: 't'
        },
        ordinal: function (number) {
            var b = number % 10;
            return (~~(number % 100 / 10) === 1) ? 'th' :
                (b === 1) ? 'st' :
                    (b === 2) ? 'nd' :
                        (b === 3) ? 'rd' : 'th';
        },
        currency: {
            symbol: '$',
            position: 'prefix'
        },
        defaults: {
            currencyFormat: ',0000 a'
        },
        formats: {
            fourDigits: '0000 a',
            fullWithTwoDecimals: '$ ,0.00',
            fullWithTwoDecimalsNoCurrency: ',0.00'
        }
    };
    /************************************
        Constructors
    ************************************/
    // Numbro prototype object
    function Numbro(number) {
        this._value = number;
    }
    function zeroes(count) {
        var i, ret = '';
        for (i = 0; i < count; i++) {
            ret += '0';
        }
        return ret;
    }
    /**
     * Implementation of toFixed() for numbers with exponent > 21
     *
     *
     */
    function toFixedLarge(value, precision) {
        var mantissa, beforeDec, afterDec, exponent, str;
        str = value.toString();
        mantissa = str.split('e')[0];
        exponent = str.split('e')[1];
        beforeDec = mantissa.split('.')[0];
        afterDec = mantissa.split('.')[1] || '';
        str = beforeDec + afterDec + zeroes(exponent - afterDec.length);
        if (precision > 0) {
            str += '.' + zeroes(precision);
        }
        return str;
    }
    /**
     * Implementation of toFixed() that treats floats more like decimals
     *
     * Fixes binary rounding issues (eg. (0.615).toFixed(2) === '0.61') that present
     * problems for accounting- and finance-related software.
     */
    function toFixed(value, precision, roundingFunction, optionals) {
        var power = Math.pow(10, precision), optionalsRegExp, output;
        if (value.toFixed(0).search('e') > -1) {
            // Above 1e21, toFixed returns scientific notation, which
            // is useless and unexpected
            output = toFixedLarge(value, precision);
        }
        else {
            //roundingFunction = (roundingFunction !== undefined ? roundingFunction : Math.round);
            // Multiply up by precision, round accurately, then divide and use native toFixed():
            output = (roundingFunction(value * power) / power).toFixed(precision);
        }
        if (optionals) {
            optionalsRegExp = new RegExp('0{1,' + optionals + '}$');
            output = output.replace(optionalsRegExp, '');
        }
        return output;
    }
    /************************************
        Formatting
    ************************************/
    // determine what type of formatting we need to do
    function formatNumbro(value, format, roundingFunction) {
        var output;
        // TODO: do something with `language`
        // figure out what kind of format we are dealing with
        if (format.indexOf('$') > -1) { // currency!!!!!
            output = formatCurrency(value, format, roundingFunction);
        }
        else if (format.indexOf('%') > -1) { // percentage
            output = formatPercentage(value, format, roundingFunction);
        }
        else if (format.indexOf(':') > -1) { // time
            output = formatTime(value);
        }
        else { // plain ol' numbers or bytes
            output = formatNumber(value, format, roundingFunction);
        }
        // return string
        return output;
    }
    function formatCurrency(value, originalFormat, roundingFunction) {
        var format = originalFormat, symbolIndex = format.indexOf('$'), openParenIndex = format.indexOf('('), plusSignIndex = format.indexOf('+'), minusSignIndex = format.indexOf('-'), space = '', decimalSeparator = '', spliceIndex, output;
        if (format.indexOf('$') === -1) {
            // Use defaults instead of the format provided
            if (cultures[currentCulture].currency.position === 'infix') {
                decimalSeparator = cultures[currentCulture].currency.symbol;
                if (cultures[currentCulture].currency.spaceSeparated) {
                    decimalSeparator = ' ' + decimalSeparator + ' ';
                }
            }
            else if (cultures[currentCulture].currency.spaceSeparated) {
                space = ' ';
            }
        }
        else {
            // check for space before or after currency
            if (format.indexOf(' $') > -1) {
                space = ' ';
                format = format.replace(' $', '');
            }
            else if (format.indexOf('$ ') > -1) {
                space = ' ';
                format = format.replace('$ ', '');
            }
            else {
                format = format.replace('$', '');
            }
        }
        // Format The Number
        output = formatNumber(value, format, roundingFunction, decimalSeparator);
        if (originalFormat.indexOf('$') === -1) {
            // Use defaults instead of the format provided
            switch (cultures[currentCulture].currency.position) {
                case 'postfix':
                    if (output.indexOf(')') > -1) {
                        output = output.split('');
                        output.splice(-1, 0, space + cultures[currentCulture].currency.symbol);
                        output = output.join('');
                    }
                    else {
                        output = output + space + cultures[currentCulture].currency.symbol;
                    }
                    break;
                case 'infix':
                    break;
                case 'prefix':
                    if (output.indexOf('(') > -1 || output.indexOf('-') > -1) {
                        output = output.split('');
                        spliceIndex = Math.max(openParenIndex, minusSignIndex) + 1;
                        output.splice(spliceIndex, 0, cultures[currentCulture].currency.symbol + space);
                        output = output.join('');
                    }
                    else {
                        output = cultures[currentCulture].currency.symbol + space + output;
                    }
                    break;
                default:
                    throw Error('Currency position should be among ["prefix", "infix", "postfix"]');
            }
        }
        else {
            // position the symbol
            if (symbolIndex <= 1) {
                if (output.indexOf('(') > -1 || output.indexOf('+') > -1 || output.indexOf('-') > -1) {
                    output = output.split('');
                    spliceIndex = 1;
                    if (symbolIndex < openParenIndex || symbolIndex < plusSignIndex || symbolIndex < minusSignIndex) {
                        // the symbol appears before the "(", "+" or "-"
                        spliceIndex = 0;
                    }
                    output.splice(spliceIndex, 0, cultures[currentCulture].currency.symbol + space);
                    output = output.join('');
                }
                else {
                    output = cultures[currentCulture].currency.symbol + space + output;
                }
            }
            else {
                if (output.indexOf(')') > -1) {
                    output = output.split('');
                    output.splice(-1, 0, space + cultures[currentCulture].currency.symbol);
                    output = output.join('');
                }
                else {
                    output = output + space + cultures[currentCulture].currency.symbol;
                }
            }
        }
        return output;
    }
    function formatPercentage(value, format, roundingFunction) {
        var space = '', output;
        value = value * 100;
        // check for space before %
        if (format.indexOf(' %') > -1) {
            space = ' ';
            format = format.replace(' %', '');
        }
        else {
            format = format.replace('%', '');
        }
        output = formatNumber(value, format, roundingFunction);
        if (output.indexOf(')') > -1) {
            output = output.split('');
            output.splice(-1, 0, space + '%');
            output = output.join('');
        }
        else {
            output = output + space + '%';
        }
        return output;
    }
    function formatTime(value) {
        var hours = Math.floor(value / 60 / 60), minutes = Math.floor((value - (hours * 60 * 60)) / 60), seconds = Math.round(value - (hours * 60 * 60) - (minutes * 60));
        return hours + ':' +
            ((minutes < 10) ? '0' + minutes : minutes) + ':' +
            ((seconds < 10) ? '0' + seconds : seconds);
    }
    function formatNumber(value, format, roundingFunction, sep) {
        var negP = false, signed = false, optDec = false, abbr = '', i, abbrK = false, // force abbreviation to thousands
        abbrM = false, // force abbreviation to millions
        abbrB = false, // force abbreviation to billions
        abbrT = false, // force abbreviation to trillions
        abbrForce = false, // force abbreviation
        bytes = '', ord = '', abs = Math.abs(value), binarySuffixes = ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB'], decimalSuffixes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'], min, max, power, totalLength, length, minimumPrecision, pow, w, intPrecision, precision, prefix, postfix, thousands, d = '', forcedNeg = false, neg = false, indexOpenP, size, indexMinus, paren = '', minlen;
        // check if number is zero and a custom zero format has been set
        if (value === 0 && zeroFormat !== null) {
            return zeroFormat;
        }
        if (!isFinite(value)) {
            return '' + value;
        }
        if (format.indexOf('{') === 0) {
            var end = format.indexOf('}');
            if (end === -1) {
                throw Error('Format should also contain a "}"');
            }
            prefix = format.slice(1, end);
            format = format.slice(end + 1);
        }
        else {
            prefix = '';
        }
        if (format.indexOf('}') === format.length - 1) {
            var start = format.indexOf('{');
            if (start === -1) {
                throw Error('Format should also contain a "{"');
            }
            postfix = format.slice(start + 1, -1);
            format = format.slice(0, start + 1);
        }
        else {
            postfix = '';
        }
        // check for min length
        var info;
        if (format.indexOf('.') === -1) {
            info = format.match(/([0-9]+).*/);
        }
        else {
            info = format.match(/([0-9]+)\..*/);
        }
        minlen = info === null ? -1 : info[1].length;
        // see if we should use parentheses for negative number or if we should prefix with a sign
        // if both are present we default to parentheses
        if (format.indexOf('-') !== -1) {
            forcedNeg = true;
        }
        if (format.indexOf('(') > -1) {
            negP = true;
            format = format.slice(1, -1);
        }
        else if (format.indexOf('+') > -1) {
            signed = true;
            format = format.replace(/\+/g, '');
        }
        // see if abbreviation is wanted
        if (format.indexOf('a') > -1) {
            intPrecision = format.split('.')[0].match(/[0-9]+/g) || ['0'];
            intPrecision = parseInt(intPrecision[0], 10);
            // check if abbreviation is specified
            abbrK = format.indexOf('aK') >= 0;
            abbrM = format.indexOf('aM') >= 0;
            abbrB = format.indexOf('aB') >= 0;
            abbrT = format.indexOf('aT') >= 0;
            abbrForce = abbrK || abbrM || abbrB || abbrT;
            // check for space before abbreviation
            if (format.indexOf(' a') > -1) {
                abbr = ' ';
                format = format.replace(' a', '');
            }
            else {
                format = format.replace('a', '');
            }
            totalLength = Math.floor(Math.log(abs) / Math.LN10) + 1;
            minimumPrecision = totalLength % 3;
            minimumPrecision = minimumPrecision === 0 ? 3 : minimumPrecision;
            if (intPrecision && abs !== 0) {
                length = Math.floor(Math.log(abs) / Math.LN10) + 1 - intPrecision;
                pow = 3 * ~~((Math.min(intPrecision, totalLength) - minimumPrecision) / 3);
                abs = abs / Math.pow(10, pow);
                if (format.indexOf('.') === -1 && intPrecision > 3) {
                    format += '[.]';
                    size = length === 0 ? 0 : 3 * ~~(length / 3) - length;
                    size = size < 0 ? size + 3 : size;
                    for (i = 0; i < size; i++) {
                        format += '0';
                    }
                }
            }
            if (Math.floor(Math.log(Math.abs(value)) / Math.LN10) + 1 !== intPrecision) {
                if (abs >= Math.pow(10, 12) && !abbrForce || abbrT) {
                    // trillion
                    abbr = abbr + cultures[currentCulture].abbreviations.trillion;
                    value = value / Math.pow(10, 12);
                }
                else if (abs < Math.pow(10, 12) && abs >= Math.pow(10, 9) && !abbrForce || abbrB) {
                    // billion
                    abbr = abbr + cultures[currentCulture].abbreviations.billion;
                    value = value / Math.pow(10, 9);
                }
                else if (abs < Math.pow(10, 9) && abs >= Math.pow(10, 6) && !abbrForce || abbrM) {
                    // million
                    abbr = abbr + cultures[currentCulture].abbreviations.million;
                    value = value / Math.pow(10, 6);
                }
                else if (abs < Math.pow(10, 6) && abs >= Math.pow(10, 3) && !abbrForce || abbrK) {
                    // thousand
                    abbr = abbr + cultures[currentCulture].abbreviations.thousand;
                    value = value / Math.pow(10, 3);
                }
            }
        }
        // see if we are formatting binary bytes
        if (format.indexOf('b') > -1) {
            // check for space before
            if (format.indexOf(' b') > -1) {
                bytes = ' ';
                format = format.replace(' b', '');
            }
            else {
                format = format.replace('b', '');
            }
            for (power = 0; power <= binarySuffixes.length; power++) {
                min = Math.pow(1024, power);
                max = Math.pow(1024, power + 1);
                if (value >= min && value < max) {
                    bytes = bytes + binarySuffixes[power];
                    if (min > 0) {
                        value = value / min;
                    }
                    break;
                }
            }
        }
        // see if we are formatting decimal bytes
        if (format.indexOf('d') > -1) {
            // check for space before
            if (format.indexOf(' d') > -1) {
                bytes = ' ';
                format = format.replace(' d', '');
            }
            else {
                format = format.replace('d', '');
            }
            for (power = 0; power <= decimalSuffixes.length; power++) {
                min = Math.pow(1000, power);
                max = Math.pow(1000, power + 1);
                if (value >= min && value < max) {
                    bytes = bytes + decimalSuffixes[power];
                    if (min > 0) {
                        value = value / min;
                    }
                    break;
                }
            }
        }
        // see if ordinal is wanted
        if (format.indexOf('o') > -1) {
            // check for space before
            if (format.indexOf(' o') > -1) {
                ord = ' ';
                format = format.replace(' o', '');
            }
            else {
                format = format.replace('o', '');
            }
            if (cultures[currentCulture].ordinal) {
                ord = ord + cultures[currentCulture].ordinal(value);
            }
        }
        if (format.indexOf('[.]') > -1) {
            optDec = true;
            format = format.replace('[.]', '.');
        }
        w = value.toString().split('.')[0];
        precision = format.split('.')[1];
        thousands = format.indexOf(',');
        if (precision) {
            if (precision.indexOf('*') !== -1) {
                d = toFixed(value, value.toString().split('.')[1].length, roundingFunction);
            }
            else {
                if (precision.indexOf('[') > -1) {
                    precision = precision.replace(']', '');
                    precision = precision.split('[');
                    d = toFixed(value, (precision[0].length + precision[1].length), roundingFunction, precision[1].length);
                }
                else {
                    d = toFixed(value, precision.length, roundingFunction);
                }
            }
            w = d.split('.')[0];
            if (d.split('.')[1].length) {
                var p = sep ? abbr + sep : cultures[currentCulture].delimiters.decimal;
                d = p + d.split('.')[1];
            }
            else {
                d = '';
            }
            if (optDec && Number(d.slice(1)) === 0) {
                d = '';
            }
        }
        else {
            w = toFixed(value, null, roundingFunction);
        }
        // format number
        if (w.indexOf('-') > -1) {
            w = w.slice(1);
            neg = true;
        }
        if (w.length < minlen) {
            w = new Array(minlen - w.length + 1).join('0') + w;
        }
        if (thousands > -1) {
            w = w.toString().replace(/(\d)(?=(\d{3})+(?!\d))/g, '$1' +
                cultures[currentCulture].delimiters.thousands);
        }
        if (format.indexOf('.') === 0) {
            w = '';
        }
        indexOpenP = format.indexOf('(');
        indexMinus = format.indexOf('-');
        if (indexOpenP < indexMinus) {
            paren = ((negP && neg) ? '(' : '') + (((forcedNeg && neg) || (!negP && neg)) ? '-' : '');
        }
        else {
            paren = (((forcedNeg && neg) || (!negP && neg)) ? '-' : '') + ((negP && neg) ? '(' : '');
        }
        return prefix +
            paren + ((!neg && signed && value !== 0) ? '+' : '') +
            w + d +
            ((ord) ? ord : '') +
            ((abbr && !sep) ? abbr : '') +
            ((bytes) ? bytes : '') +
            ((negP && neg) ? ')' : '') +
            postfix;
    }
    /************************************
        Top Level Functions
    ************************************/
    numbro = function (input) {
        if (numbro.isNumbro(input)) {
            input = input.value();
        }
        else if (input === 0 || typeof input === 'undefined') {
            input = 0;
        }
        else if (!Number(input)) {
            input = numbro.fn.unformat(input);
        }
        return new Numbro(Number(input));
    };
    // version number
    numbro.version = VERSION;
    // compare numbro object
    numbro.isNumbro = function (obj) {
        return obj instanceof Numbro;
    };
    /**
     * This function allow the user to set a new language with a fallback if
     * the language does not exist. If no fallback language is provided,
     * it fallbacks to english.
     *
     * @deprecated Since in version 1.6.0. It will be deleted in version 2.0
     * `setCulture` should be used instead.
     */
    numbro.setLanguage = function (newLanguage, fallbackLanguage) {
        console.warn('`setLanguage` is deprecated since version 1.6.0. Use `setCulture` instead');
        var key = newLanguage, prefix = newLanguage.split('-')[0], matchingLanguage = null;
        if (!languages[key]) {
            Object.keys(languages).forEach(function (language) {
                if (!matchingLanguage && language.split('-')[0] === prefix) {
                    matchingLanguage = language;
                }
            });
            key = matchingLanguage || fallbackLanguage || 'en-US';
        }
        chooseCulture(key);
    };
    /**
     * This function allow the user to set a new culture with a fallback if
     * the culture does not exist. If no fallback culture is provided,
     * it fallbacks to "en-US".
     */
    numbro.setCulture = function (newCulture, fallbackCulture) {
        var key = newCulture, suffix = newCulture.split('-')[1], matchingCulture = null;
        if (!cultures[key]) {
            if (suffix) {
                Object.keys(cultures).forEach(function (language) {
                    if (!matchingCulture && language.split('-')[1] === suffix) {
                        matchingCulture = language;
                    }
                });
            }
            key = matchingCulture || fallbackCulture || 'en-US';
        }
        chooseCulture(key);
    };
    /**
     * This function will load languages and then set the global language.  If
     * no arguments are passed in, it will simply return the current global
     * language key.
     *
     * @deprecated Since in version 1.6.0. It will be deleted in version 2.0
     * `culture` should be used instead.
     */
    numbro.language = function (key, values) {
        console.warn('`language` is deprecated since version 1.6.0. Use `culture` instead');
        if (!key) {
            return currentCulture;
        }
        if (key && !values) {
            if (!languages[key]) {
                throw new Error('Unknown language : ' + key);
            }
            chooseCulture(key);
        }
        if (values || !languages[key]) {
            setCulture(key, values);
        }
        return numbro;
    };
    /**
     * This function will load cultures and then set the global culture.  If
     * no arguments are passed in, it will simply return the current global
     * culture code.
     */
    numbro.culture = function (code, values) {
        if (!code) {
            return currentCulture;
        }
        if (code && !values) {
            if (!cultures[code]) {
                throw new Error('Unknown culture : ' + code);
            }
            chooseCulture(code);
        }
        if (values || !cultures[code]) {
            setCulture(code, values);
        }
        return numbro;
    };
    /**
     * This function provides access to the loaded language data.  If
     * no arguments are passed in, it will simply return the current
     * global language object.
     *
     * @deprecated Since in version 1.6.0. It will be deleted in version 2.0
     * `culture` should be used instead.
     */
    numbro.languageData = function (key) {
        console.warn('`languageData` is deprecated since version 1.6.0. Use `cultureData` instead');
        if (!key) {
            return languages[currentCulture];
        }
        if (!languages[key]) {
            throw new Error('Unknown language : ' + key);
        }
        return languages[key];
    };
    /**
     * This function provides access to the loaded culture data.  If
     * no arguments are passed in, it will simply return the current
     * global culture object.
     */
    numbro.cultureData = function (code) {
        if (!code) {
            return cultures[currentCulture];
        }
        if (!cultures[code]) {
            throw new Error('Unknown culture : ' + code);
        }
        return cultures[code];
    };
    numbro.culture('en-US', enUS);
    /**
     * @deprecated Since in version 1.6.0. It will be deleted in version 2.0
     * `cultures` should be used instead.
     */
    numbro.languages = function () {
        console.warn('`languages` is deprecated since version 1.6.0. Use `cultures` instead');
        return languages;
    };
    numbro.cultures = function () {
        return cultures;
    };
    numbro.zeroFormat = function (format) {
        zeroFormat = typeof (format) === 'string' ? format : null;
    };
    numbro.defaultFormat = function (format) {
        defaultFormat = typeof (format) === 'string' ? format : '0.0';
    };
    numbro.defaultCurrencyFormat = function (format) {
        defaultCurrencyFormat = typeof (format) === 'string' ? format : '0$';
    };
    numbro.validate = function (val, culture) {
        var _decimalSep, _thousandSep, _currSymbol, _valArray, _abbrObj, _thousandRegEx, cultureData, temp;
        //coerce val to string
        if (typeof val !== 'string') {
            val += '';
            if (console.warn) {
                console.warn('Numbro.js: Value is not string. It has been co-erced to: ', val);
            }
        }
        //trim whitespaces from either sides
        val = val.trim();
        //if val is just digits return true
        if (!!val.match(/^\d+$/)) {
            return true;
        }
        //if val is empty return false
        if (val === '') {
            return false;
        }
        //get the decimal and thousands separator from numbro.cultureData
        try {
            //check if the culture is understood by numbro. if not, default it to current culture
            cultureData = numbro.cultureData(culture);
        }
        catch (e) {
            cultureData = numbro.cultureData(numbro.culture());
        }
        //setup the delimiters and currency symbol based on culture
        _currSymbol = cultureData.currency.symbol;
        _abbrObj = cultureData.abbreviations;
        _decimalSep = cultureData.delimiters.decimal;
        if (cultureData.delimiters.thousands === '.') {
            _thousandSep = '\\.';
        }
        else {
            _thousandSep = cultureData.delimiters.thousands;
        }
        // validating currency symbol
        temp = val.match(/^[^\d]+/);
        if (temp !== null) {
            val = val.substr(1);
            if (temp[0] !== _currSymbol) {
                return false;
            }
        }
        //validating abbreviation symbol
        temp = val.match(/[^\d]+$/);
        if (temp !== null) {
            val = val.slice(0, -1);
            if (temp[0] !== _abbrObj.thousand && temp[0] !== _abbrObj.million &&
                temp[0] !== _abbrObj.billion && temp[0] !== _abbrObj.trillion) {
                return false;
            }
        }
        _thousandRegEx = new RegExp(_thousandSep + '{2}');
        if (!val.match(/[^\d.,]/g)) {
            _valArray = val.split(_decimalSep);
            if (_valArray.length > 2) {
                return false;
            }
            else {
                if (_valArray.length < 2) {
                    return (!!_valArray[0].match(/^\d+.*\d$/) && !_valArray[0].match(_thousandRegEx));
                }
                else {
                    if (_valArray[0].length === 1) {
                        return (!!_valArray[0].match(/^\d+$/) &&
                            !_valArray[0].match(_thousandRegEx) &&
                            !!_valArray[1].match(/^\d+$/));
                    }
                    else {
                        return (!!_valArray[0].match(/^\d+.*\d$/) &&
                            !_valArray[0].match(_thousandRegEx) &&
                            !!_valArray[1].match(/^\d+$/));
                    }
                }
            }
        }
        return false;
    };
    /************************************
        Helpers
    ************************************/
    function setCulture(code, values) {
        cultures[code] = values;
    }
    function chooseCulture(code) {
        currentCulture = code;
        var defaults = cultures[code].defaults;
        if (defaults && defaults.format) {
            numbro.defaultFormat(defaults.format);
        }
        if (defaults && defaults.currencyFormat) {
            numbro.defaultCurrencyFormat(defaults.currencyFormat);
        }
    }
    function format(input, formatString, language, roundingFunction) {
        if (language != null && language !== numbro.culture()) {
            numbro.setCulture(language);
        }
        return formatNumbro(Number(input), formatString != null ? formatString : defaultFormat, roundingFunction == null ? Math.round : roundingFunction);
    }
    module.exports = { "format": format };
},
/* sprintf-js/src/sprintf.js */ function _(require, module, exports) {
    /* global window, exports, define */
    !function () {
        'use strict';
        var re = {
            not_string: /[^s]/,
            not_bool: /[^t]/,
            not_type: /[^T]/,
            not_primitive: /[^v]/,
            number: /[diefg]/,
            numeric_arg: /[bcdiefguxX]/,
            json: /[j]/,
            not_json: /[^j]/,
            text: /^[^\x25]+/,
            modulo: /^\x25{2}/,
            placeholder: /^\x25(?:([1-9]\d*)\$|\(([^)]+)\))?(\+)?(0|'[^$])?(-)?(\d+)?(?:\.(\d+))?([b-gijostTuvxX])/,
            key: /^([a-z_][a-z_\d]*)/i,
            key_access: /^\.([a-z_][a-z_\d]*)/i,
            index_access: /^\[(\d+)\]/,
            sign: /^[+-]/
        };
        function sprintf(key) {
            // `arguments` is not an array, but should be fine for this call
            return sprintf_format(sprintf_parse(key), arguments);
        }
        function vsprintf(fmt, argv) {
            return sprintf.apply(null, [fmt].concat(argv || []));
        }
        function sprintf_format(parse_tree, argv) {
            var cursor = 1, tree_length = parse_tree.length, arg, output = '', i, k, ph, pad, pad_character, pad_length, is_positive, sign;
            for (i = 0; i < tree_length; i++) {
                if (typeof parse_tree[i] === 'string') {
                    output += parse_tree[i];
                }
                else if (typeof parse_tree[i] === 'object') {
                    ph = parse_tree[i]; // convenience purposes only
                    if (ph.keys) { // keyword argument
                        arg = argv[cursor];
                        for (k = 0; k < ph.keys.length; k++) {
                            if (arg == undefined) {
                                throw new Error(sprintf('[sprintf] Cannot access property "%s" of undefined value "%s"', ph.keys[k], ph.keys[k - 1]));
                            }
                            arg = arg[ph.keys[k]];
                        }
                    }
                    else if (ph.param_no) { // positional argument (explicit)
                        arg = argv[ph.param_no];
                    }
                    else { // positional argument (implicit)
                        arg = argv[cursor++];
                    }
                    if (re.not_type.test(ph.type) && re.not_primitive.test(ph.type) && arg instanceof Function) {
                        arg = arg();
                    }
                    if (re.numeric_arg.test(ph.type) && (typeof arg !== 'number' && isNaN(arg))) {
                        throw new TypeError(sprintf('[sprintf] expecting number but found %T', arg));
                    }
                    if (re.number.test(ph.type)) {
                        is_positive = arg >= 0;
                    }
                    switch (ph.type) {
                        case 'b':
                            arg = parseInt(arg, 10).toString(2);
                            break;
                        case 'c':
                            arg = String.fromCharCode(parseInt(arg, 10));
                            break;
                        case 'd':
                        case 'i':
                            arg = parseInt(arg, 10);
                            break;
                        case 'j':
                            arg = JSON.stringify(arg, null, ph.width ? parseInt(ph.width) : 0);
                            break;
                        case 'e':
                            arg = ph.precision ? parseFloat(arg).toExponential(ph.precision) : parseFloat(arg).toExponential();
                            break;
                        case 'f':
                            arg = ph.precision ? parseFloat(arg).toFixed(ph.precision) : parseFloat(arg);
                            break;
                        case 'g':
                            arg = ph.precision ? String(Number(arg.toPrecision(ph.precision))) : parseFloat(arg);
                            break;
                        case 'o':
                            arg = (parseInt(arg, 10) >>> 0).toString(8);
                            break;
                        case 's':
                            arg = String(arg);
                            arg = (ph.precision ? arg.substring(0, ph.precision) : arg);
                            break;
                        case 't':
                            arg = String(!!arg);
                            arg = (ph.precision ? arg.substring(0, ph.precision) : arg);
                            break;
                        case 'T':
                            arg = Object.prototype.toString.call(arg).slice(8, -1).toLowerCase();
                            arg = (ph.precision ? arg.substring(0, ph.precision) : arg);
                            break;
                        case 'u':
                            arg = parseInt(arg, 10) >>> 0;
                            break;
                        case 'v':
                            arg = arg.valueOf();
                            arg = (ph.precision ? arg.substring(0, ph.precision) : arg);
                            break;
                        case 'x':
                            arg = (parseInt(arg, 10) >>> 0).toString(16);
                            break;
                        case 'X':
                            arg = (parseInt(arg, 10) >>> 0).toString(16).toUpperCase();
                            break;
                    }
                    if (re.json.test(ph.type)) {
                        output += arg;
                    }
                    else {
                        if (re.number.test(ph.type) && (!is_positive || ph.sign)) {
                            sign = is_positive ? '+' : '-';
                            arg = arg.toString().replace(re.sign, '');
                        }
                        else {
                            sign = '';
                        }
                        pad_character = ph.pad_char ? ph.pad_char === '0' ? '0' : ph.pad_char.charAt(1) : ' ';
                        pad_length = ph.width - (sign + arg).length;
                        pad = ph.width ? (pad_length > 0 ? pad_character.repeat(pad_length) : '') : '';
                        output += ph.align ? sign + arg + pad : (pad_character === '0' ? sign + pad + arg : pad + sign + arg);
                    }
                }
            }
            return output;
        }
        var sprintf_cache = Object.create(null);
        function sprintf_parse(fmt) {
            if (sprintf_cache[fmt]) {
                return sprintf_cache[fmt];
            }
            var _fmt = fmt, match, parse_tree = [], arg_names = 0;
            while (_fmt) {
                if ((match = re.text.exec(_fmt)) !== null) {
                    parse_tree.push(match[0]);
                }
                else if ((match = re.modulo.exec(_fmt)) !== null) {
                    parse_tree.push('%');
                }
                else if ((match = re.placeholder.exec(_fmt)) !== null) {
                    if (match[2]) {
                        arg_names |= 1;
                        var field_list = [], replacement_field = match[2], field_match = [];
                        if ((field_match = re.key.exec(replacement_field)) !== null) {
                            field_list.push(field_match[1]);
                            while ((replacement_field = replacement_field.substring(field_match[0].length)) !== '') {
                                if ((field_match = re.key_access.exec(replacement_field)) !== null) {
                                    field_list.push(field_match[1]);
                                }
                                else if ((field_match = re.index_access.exec(replacement_field)) !== null) {
                                    field_list.push(field_match[1]);
                                }
                                else {
                                    throw new SyntaxError('[sprintf] failed to parse named argument key');
                                }
                            }
                        }
                        else {
                            throw new SyntaxError('[sprintf] failed to parse named argument key');
                        }
                        match[2] = field_list;
                    }
                    else {
                        arg_names |= 2;
                    }
                    if (arg_names === 3) {
                        throw new Error('[sprintf] mixing positional and named placeholders is not (yet) supported');
                    }
                    parse_tree.push({
                        placeholder: match[0],
                        param_no: match[1],
                        keys: match[2],
                        sign: match[3],
                        pad_char: match[4],
                        align: match[5],
                        width: match[6],
                        precision: match[7],
                        type: match[8]
                    });
                }
                else {
                    throw new SyntaxError('[sprintf] unexpected placeholder');
                }
                _fmt = _fmt.substring(match[0].length);
            }
            return sprintf_cache[fmt] = parse_tree;
        }
        /**
         * export to either browser or node.js
         */
        /* eslint-disable quote-props */
        if (typeof exports !== 'undefined') {
            exports['sprintf'] = sprintf;
            exports['vsprintf'] = vsprintf;
        }
        if (typeof window !== 'undefined') {
            window['sprintf'] = sprintf;
            window['vsprintf'] = vsprintf;
            if (typeof define === 'function' && define['amd']) {
                define(function () {
                    return {
                        'sprintf': sprintf,
                        'vsprintf': vsprintf
                    };
                });
            }
        }
        /* eslint-enable quote-props */
    }(); // eslint-disable-line
},
/* models/tickers/datetime_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var array_1 = require(124) /* ../../core/util/array */;
    var adaptive_ticker_1 = require(242) /* ./adaptive_ticker */;
    var composite_ticker_1 = require(306) /* ./composite_ticker */;
    var days_ticker_1 = require(307) /* ./days_ticker */;
    var months_ticker_1 = require(310) /* ./months_ticker */;
    var years_ticker_1 = require(311) /* ./years_ticker */;
    var util_1 = require(309) /* ./util */;
    var DatetimeTicker = /** @class */ (function (_super) {
        tslib_1.__extends(DatetimeTicker, _super);
        function DatetimeTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        DatetimeTicker.init_DatetimeTicker = function () {
            this.override({
                num_minor_ticks: 0,
                tickers: function () {
                    return [
                        // Sub-second.
                        new adaptive_ticker_1.AdaptiveTicker({
                            mantissas: [1, 2, 5],
                            base: 10,
                            min_interval: 0,
                            max_interval: 500 * util_1.ONE_MILLI,
                            num_minor_ticks: 0,
                        }),
                        // Seconds, minutes.
                        new adaptive_ticker_1.AdaptiveTicker({
                            mantissas: [1, 2, 5, 10, 15, 20, 30],
                            base: 60,
                            min_interval: util_1.ONE_SECOND,
                            max_interval: 30 * util_1.ONE_MINUTE,
                            num_minor_ticks: 0,
                        }),
                        // Hours.
                        new adaptive_ticker_1.AdaptiveTicker({
                            mantissas: [1, 2, 4, 6, 8, 12],
                            base: 24.0,
                            min_interval: util_1.ONE_HOUR,
                            max_interval: 12 * util_1.ONE_HOUR,
                            num_minor_ticks: 0,
                        }),
                        // Days.
                        new days_ticker_1.DaysTicker({ days: array_1.range(1, 32) }),
                        new days_ticker_1.DaysTicker({ days: array_1.range(1, 31, 3) }),
                        new days_ticker_1.DaysTicker({ days: [1, 8, 15, 22] }),
                        new days_ticker_1.DaysTicker({ days: [1, 15] }),
                        // Months.
                        new months_ticker_1.MonthsTicker({ months: array_1.range(0, 12, 1) }),
                        new months_ticker_1.MonthsTicker({ months: array_1.range(0, 12, 2) }),
                        new months_ticker_1.MonthsTicker({ months: array_1.range(0, 12, 4) }),
                        new months_ticker_1.MonthsTicker({ months: array_1.range(0, 12, 6) }),
                        // Years
                        new years_ticker_1.YearsTicker({}),
                    ];
                },
            });
        };
        return DatetimeTicker;
    }(composite_ticker_1.CompositeTicker));
    exports.DatetimeTicker = DatetimeTicker;
    DatetimeTicker.__name__ = "DatetimeTicker";
    DatetimeTicker.init_DatetimeTicker();
},
/* models/tickers/composite_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var continuous_ticker_1 = require(243) /* ./continuous_ticker */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    var CompositeTicker = /** @class */ (function (_super) {
        tslib_1.__extends(CompositeTicker, _super);
        function CompositeTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        CompositeTicker.init_CompositeTicker = function () {
            this.define({
                tickers: [p.Array, []],
            });
        };
        Object.defineProperty(CompositeTicker.prototype, "min_intervals", {
            // The tickers should be in order of increasing interval size; specifically,
            // if S comes before T, then it should be the case that
            // S.get_max_interval() < T.get_min_interval().
            // FIXME Enforce this automatically.
            get: function () {
                return this.tickers.map(function (ticker) { return ticker.get_min_interval(); });
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CompositeTicker.prototype, "max_intervals", {
            get: function () {
                return this.tickers.map(function (ticker) { return ticker.get_max_interval(); });
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CompositeTicker.prototype, "min_interval", {
            get: function () {
                return this.min_intervals[0];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CompositeTicker.prototype, "max_interval", {
            get: function () {
                return this.max_intervals[0];
            },
            enumerable: true,
            configurable: true
        });
        CompositeTicker.prototype.get_best_ticker = function (data_low, data_high, desired_n_ticks) {
            var data_range = data_high - data_low;
            var ideal_interval = this.get_ideal_interval(data_low, data_high, desired_n_ticks);
            var ticker_ndxs = [
                array_1.sorted_index(this.min_intervals, ideal_interval) - 1,
                array_1.sorted_index(this.max_intervals, ideal_interval),
            ];
            var intervals = [
                this.min_intervals[ticker_ndxs[0]],
                this.max_intervals[ticker_ndxs[1]],
            ];
            var errors = intervals.map(function (interval) {
                return Math.abs(desired_n_ticks - (data_range / interval));
            });
            var best_ticker;
            if (array_1.is_empty(errors.filter(function (e) { return !isNaN(e); }))) {
                // this can happen if the data isn't loaded yet, we just default to the first scale
                best_ticker = this.tickers[0];
            }
            else {
                var best_index = array_1.argmin(errors);
                var best_ticker_ndx = ticker_ndxs[best_index];
                best_ticker = this.tickers[best_ticker_ndx];
            }
            return best_ticker;
        };
        CompositeTicker.prototype.get_interval = function (data_low, data_high, desired_n_ticks) {
            var best_ticker = this.get_best_ticker(data_low, data_high, desired_n_ticks);
            return best_ticker.get_interval(data_low, data_high, desired_n_ticks);
        };
        CompositeTicker.prototype.get_ticks_no_defaults = function (data_low, data_high, cross_loc, desired_n_ticks) {
            var best_ticker = this.get_best_ticker(data_low, data_high, desired_n_ticks);
            return best_ticker.get_ticks_no_defaults(data_low, data_high, cross_loc, desired_n_ticks);
        };
        return CompositeTicker;
    }(continuous_ticker_1.ContinuousTicker));
    exports.CompositeTicker = CompositeTicker;
    CompositeTicker.__name__ = "CompositeTicker";
    CompositeTicker.init_CompositeTicker();
},
/* models/tickers/days_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var single_interval_ticker_1 = require(308) /* ./single_interval_ticker */;
    var util_1 = require(309) /* ./util */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    // Given a start and end time in millis, returns the shortest array of
    // consecutive months (as Dates) that surrounds both times.
    function date_range_by_month(start_time, end_time) {
        var start_date = util_1.last_month_no_later_than(new Date(start_time));
        var end_date = util_1.last_month_no_later_than(new Date(end_time));
        // XXX This is not a reliable technique in general, but it should be
        // safe when the day of the month is 1.  (The problem case is this:
        // Mar 31 -> Apr 31, which becomes May 1.)
        end_date.setUTCMonth(end_date.getUTCMonth() + 1);
        var dates = [];
        var date = start_date;
        while (true) {
            dates.push(util_1.copy_date(date));
            date.setUTCMonth(date.getUTCMonth() + 1);
            if (date > end_date)
                break;
        }
        return dates;
    }
    var DaysTicker = /** @class */ (function (_super) {
        tslib_1.__extends(DaysTicker, _super);
        function DaysTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        DaysTicker.init_DaysTicker = function () {
            this.define({
                days: [p.Array, []],
            });
            this.override({
                num_minor_ticks: 0,
            });
        };
        DaysTicker.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            var days = this.days;
            if (days.length > 1)
                this.interval = (days[1] - days[0]) * util_1.ONE_DAY;
            else
                this.interval = 31 * util_1.ONE_DAY;
        };
        DaysTicker.prototype.get_ticks_no_defaults = function (data_low, data_high, _cross_loc, _desired_n_ticks) {
            var month_dates = date_range_by_month(data_low, data_high);
            var days = this.days;
            var days_of_month = function (month_date, interval) {
                var e_1, _a;
                var current_month = month_date.getUTCMonth();
                var dates = [];
                try {
                    for (var days_1 = tslib_1.__values(days), days_1_1 = days_1.next(); !days_1_1.done; days_1_1 = days_1.next()) {
                        var day = days_1_1.value;
                        var day_date = util_1.copy_date(month_date);
                        day_date.setUTCDate(day);
                        // We can't use all of the values in this.days, because they may not
                        // fall within the current month.  In fact, if, e.g., our month is 28 days
                        // and we're marking every third day, we don't want day 28 to show up
                        // because it'll be right next to the 1st of the next month.  So we
                        // make sure we have a bit of room before we include a day.
                        // TODO (bev) The above description does not exactly work because JS Date
                        // is broken and will happily consider "Feb 28 + 3*ONE_DAY" to have month "2"
                        var future_date = new Date(day_date.getTime() + (interval / 2));
                        if (future_date.getUTCMonth() == current_month)
                            dates.push(day_date);
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (days_1_1 && !days_1_1.done && (_a = days_1.return))
                            _a.call(days_1);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
                return dates;
            };
            var interval = this.interval;
            var day_dates = array_1.concat(month_dates.map(function (date) { return days_of_month(date, interval); }));
            var all_ticks = day_dates.map(function (day_date) { return day_date.getTime(); });
            var ticks_in_range = all_ticks.filter(function (tick) { return data_low <= tick && tick <= data_high; });
            return {
                major: ticks_in_range,
                minor: [],
            };
        };
        return DaysTicker;
    }(single_interval_ticker_1.SingleIntervalTicker));
    exports.DaysTicker = DaysTicker;
    DaysTicker.__name__ = "DaysTicker";
    DaysTicker.init_DaysTicker();
},
/* models/tickers/single_interval_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var continuous_ticker_1 = require(243) /* ./continuous_ticker */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var SingleIntervalTicker = /** @class */ (function (_super) {
        tslib_1.__extends(SingleIntervalTicker, _super);
        function SingleIntervalTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        SingleIntervalTicker.init_SingleIntervalTicker = function () {
            this.define({
                interval: [p.Number],
            });
        };
        SingleIntervalTicker.prototype.get_interval = function (_data_low, _data_high, _n_desired_ticks) {
            return this.interval;
        };
        Object.defineProperty(SingleIntervalTicker.prototype, "min_interval", {
            get: function () {
                return this.interval;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SingleIntervalTicker.prototype, "max_interval", {
            get: function () {
                return this.interval;
            },
            enumerable: true,
            configurable: true
        });
        return SingleIntervalTicker;
    }(continuous_ticker_1.ContinuousTicker));
    exports.SingleIntervalTicker = SingleIntervalTicker;
    SingleIntervalTicker.__name__ = "SingleIntervalTicker";
    SingleIntervalTicker.init_SingleIntervalTicker();
},
/* models/tickers/util.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    // TODO Clear out debugging code, etc.
    // TODO Organize helper functions.
    // TODO The years ticker doesn't always use the roundest numbers; it should
    // probably use a special ticker.
    // TODO Add tests.
    // TODO There used to be a TODO: restore memoization.  So.... do that?
    // TODO Instead of a get_ticks() method, there used to be an auto_ticks()
    // function that took a lot of fancy arguments, but those arguments weren't
    // used anywhere.  Should we restore them?
    // Some time constants, in milliseconds.
    exports.ONE_MILLI = 1.0;
    exports.ONE_SECOND = 1000.0;
    exports.ONE_MINUTE = 60.0 * exports.ONE_SECOND;
    exports.ONE_HOUR = 60 * exports.ONE_MINUTE;
    exports.ONE_DAY = 24 * exports.ONE_HOUR;
    exports.ONE_MONTH = 30 * exports.ONE_DAY; // An approximation, obviously.
    exports.ONE_YEAR = 365 * exports.ONE_DAY;
    // ---------------------------------------------------------------------------
    // Date/time utility functions
    // ---------------------------------------------------------------------------
    // Makes a copy of a date object.
    function copy_date(date) {
        return new Date(date.getTime());
    }
    exports.copy_date = copy_date;
    // Rounds a date down to the month.
    function last_month_no_later_than(date) {
        var new_date = copy_date(date);
        new_date.setUTCDate(1);
        new_date.setUTCHours(0);
        new_date.setUTCMinutes(0);
        new_date.setUTCSeconds(0);
        new_date.setUTCMilliseconds(0);
        return new_date;
    }
    exports.last_month_no_later_than = last_month_no_later_than;
    // Rounds a date down to the year.
    function last_year_no_later_than(date) {
        var new_date = last_month_no_later_than(date);
        new_date.setUTCMonth(0);
        return new_date;
    }
    exports.last_year_no_later_than = last_year_no_later_than;
},
/* models/tickers/months_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var single_interval_ticker_1 = require(308) /* ./single_interval_ticker */;
    var util_1 = require(309) /* ./util */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    // Given a start and end time in millis, returns the shortest array of
    // consecutive years (as Dates) that surrounds both times.
    function date_range_by_year(start_time, end_time) {
        var start_date = util_1.last_year_no_later_than(new Date(start_time));
        var end_date = util_1.last_year_no_later_than(new Date(end_time));
        end_date.setUTCFullYear(end_date.getUTCFullYear() + 1);
        var dates = [];
        var date = start_date;
        while (true) {
            dates.push(util_1.copy_date(date));
            date.setUTCFullYear(date.getUTCFullYear() + 1);
            if (date > end_date)
                break;
        }
        return dates;
    }
    var MonthsTicker = /** @class */ (function (_super) {
        tslib_1.__extends(MonthsTicker, _super);
        function MonthsTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        MonthsTicker.init_MonthsTicker = function () {
            this.define({
                months: [p.Array, []],
            });
        };
        MonthsTicker.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            var months = this.months;
            if (months.length > 1)
                this.interval = (months[1] - months[0]) * util_1.ONE_MONTH;
            else
                this.interval = 12 * util_1.ONE_MONTH;
        };
        MonthsTicker.prototype.get_ticks_no_defaults = function (data_low, data_high, _cross_loc, _desired_n_ticks) {
            var year_dates = date_range_by_year(data_low, data_high);
            var months = this.months;
            var months_of_year = function (year_date) {
                return months.map(function (month) {
                    var month_date = util_1.copy_date(year_date);
                    month_date.setUTCMonth(month);
                    return month_date;
                });
            };
            var month_dates = array_1.concat(year_dates.map(months_of_year));
            var all_ticks = month_dates.map(function (month_date) { return month_date.getTime(); });
            var ticks_in_range = all_ticks.filter(function (tick) { return data_low <= tick && tick <= data_high; });
            return {
                major: ticks_in_range,
                minor: [],
            };
        };
        return MonthsTicker;
    }(single_interval_ticker_1.SingleIntervalTicker));
    exports.MonthsTicker = MonthsTicker;
    MonthsTicker.__name__ = "MonthsTicker";
    MonthsTicker.init_MonthsTicker();
},
/* models/tickers/years_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var basic_ticker_1 = require(241) /* ./basic_ticker */;
    var single_interval_ticker_1 = require(308) /* ./single_interval_ticker */;
    var util_1 = require(309) /* ./util */;
    var YearsTicker = /** @class */ (function (_super) {
        tslib_1.__extends(YearsTicker, _super);
        function YearsTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        YearsTicker.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.interval = util_1.ONE_YEAR;
            this.basic_ticker = new basic_ticker_1.BasicTicker({ num_minor_ticks: 0 });
        };
        YearsTicker.prototype.get_ticks_no_defaults = function (data_low, data_high, cross_loc, desired_n_ticks) {
            var start_year = util_1.last_year_no_later_than(new Date(data_low)).getUTCFullYear();
            var end_year = util_1.last_year_no_later_than(new Date(data_high)).getUTCFullYear();
            var years = this.basic_ticker.get_ticks_no_defaults(start_year, end_year, cross_loc, desired_n_ticks).major;
            var all_ticks = years.map(function (year) { return Date.UTC(year, 0, 1); });
            var ticks_in_range = all_ticks.filter(function (tick) { return data_low <= tick && tick <= data_high; });
            return {
                major: ticks_in_range,
                minor: [],
            };
        };
        return YearsTicker;
    }(single_interval_ticker_1.SingleIntervalTicker));
    exports.YearsTicker = YearsTicker;
    YearsTicker.__name__ = "YearsTicker";
},
/* models/axes/log_axis.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var axis_1 = require(292) /* ./axis */;
    var continuous_axis_1 = require(297) /* ./continuous_axis */;
    var log_tick_formatter_1 = require(313) /* ../formatters/log_tick_formatter */;
    var log_ticker_1 = require(314) /* ../tickers/log_ticker */;
    var LogAxisView = /** @class */ (function (_super) {
        tslib_1.__extends(LogAxisView, _super);
        function LogAxisView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return LogAxisView;
    }(axis_1.AxisView));
    exports.LogAxisView = LogAxisView;
    LogAxisView.__name__ = "LogAxisView";
    var LogAxis = /** @class */ (function (_super) {
        tslib_1.__extends(LogAxis, _super);
        function LogAxis(attrs) {
            return _super.call(this, attrs) || this;
        }
        LogAxis.init_LogAxis = function () {
            this.prototype.default_view = LogAxisView;
            this.override({
                ticker: function () { return new log_ticker_1.LogTicker(); },
                formatter: function () { return new log_tick_formatter_1.LogTickFormatter(); },
            });
        };
        return LogAxis;
    }(continuous_axis_1.ContinuousAxis));
    exports.LogAxis = LogAxis;
    LogAxis.__name__ = "LogAxis";
    LogAxis.init_LogAxis();
},
/* models/formatters/log_tick_formatter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var tick_formatter_1 = require(246) /* ./tick_formatter */;
    var basic_tick_formatter_1 = require(245) /* ./basic_tick_formatter */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var LogTickFormatter = /** @class */ (function (_super) {
        tslib_1.__extends(LogTickFormatter, _super);
        function LogTickFormatter(attrs) {
            return _super.call(this, attrs) || this;
        }
        LogTickFormatter.init_LogTickFormatter = function () {
            this.define({
                ticker: [p.Instance, null],
            });
        };
        LogTickFormatter.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.basic_formatter = new basic_tick_formatter_1.BasicTickFormatter();
        };
        LogTickFormatter.prototype.doFormat = function (ticks, opts) {
            if (ticks.length == 0)
                return [];
            var base = this.ticker != null ? this.ticker.base : 10;
            var small_interval = false;
            var labels = new Array(ticks.length);
            for (var i = 0, end = ticks.length; i < end; i++) {
                labels[i] = base + "^" + Math.round(Math.log(ticks[i]) / Math.log(base));
                if (i > 0 && labels[i] == labels[i - 1]) {
                    small_interval = true;
                    break;
                }
            }
            if (small_interval)
                return this.basic_formatter.doFormat(ticks, opts);
            else
                return labels;
        };
        return LogTickFormatter;
    }(tick_formatter_1.TickFormatter));
    exports.LogTickFormatter = LogTickFormatter;
    LogTickFormatter.__name__ = "LogTickFormatter";
    LogTickFormatter.init_LogTickFormatter();
},
/* models/tickers/log_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var adaptive_ticker_1 = require(242) /* ./adaptive_ticker */;
    var array_1 = require(124) /* ../../core/util/array */;
    var LogTicker = /** @class */ (function (_super) {
        tslib_1.__extends(LogTicker, _super);
        function LogTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        LogTicker.init_LogTicker = function () {
            this.override({
                mantissas: [1, 5],
            });
        };
        LogTicker.prototype.get_ticks_no_defaults = function (data_low, data_high, _cross_loc, desired_n_ticks) {
            var e_1, _a, e_2, _b, e_3, _c, e_4, _d, e_5, _e, e_6, _f;
            var num_minor_ticks = this.num_minor_ticks;
            var minor_ticks = [];
            var base = this.base;
            var log_low = Math.log(data_low) / Math.log(base);
            var log_high = Math.log(data_high) / Math.log(base);
            var log_interval = log_high - log_low;
            var ticks;
            if (!isFinite(log_interval)) {
                ticks = [];
            }
            else if (log_interval < 2) { // treat as linear ticker
                var interval_1 = this.get_interval(data_low, data_high, desired_n_ticks);
                var start_factor = Math.floor(data_low / interval_1);
                var end_factor = Math.ceil(data_high / interval_1);
                ticks = array_1.range(start_factor, end_factor + 1)
                    .filter(function (factor) { return factor != 0; })
                    .map(function (factor) { return factor * interval_1; })
                    .filter(function (tick) { return data_low <= tick && tick <= data_high; });
                if (num_minor_ticks > 0 && ticks.length > 0) {
                    var minor_interval_1 = interval_1 / num_minor_ticks;
                    var minor_offsets = array_1.range(0, num_minor_ticks).map(function (i) { return i * minor_interval_1; });
                    try {
                        for (var _g = tslib_1.__values(minor_offsets.slice(1)), _h = _g.next(); !_h.done; _h = _g.next()) {
                            var x = _h.value;
                            minor_ticks.push(ticks[0] - x);
                        }
                    }
                    catch (e_1_1) {
                        e_1 = { error: e_1_1 };
                    }
                    finally {
                        try {
                            if (_h && !_h.done && (_a = _g.return))
                                _a.call(_g);
                        }
                        finally {
                            if (e_1)
                                throw e_1.error;
                        }
                    }
                    try {
                        for (var ticks_1 = tslib_1.__values(ticks), ticks_1_1 = ticks_1.next(); !ticks_1_1.done; ticks_1_1 = ticks_1.next()) {
                            var tick = ticks_1_1.value;
                            try {
                                for (var minor_offsets_1 = (e_3 = void 0, tslib_1.__values(minor_offsets)), minor_offsets_1_1 = minor_offsets_1.next(); !minor_offsets_1_1.done; minor_offsets_1_1 = minor_offsets_1.next()) {
                                    var x = minor_offsets_1_1.value;
                                    minor_ticks.push(tick + x);
                                }
                            }
                            catch (e_3_1) {
                                e_3 = { error: e_3_1 };
                            }
                            finally {
                                try {
                                    if (minor_offsets_1_1 && !minor_offsets_1_1.done && (_c = minor_offsets_1.return))
                                        _c.call(minor_offsets_1);
                                }
                                finally {
                                    if (e_3)
                                        throw e_3.error;
                                }
                            }
                        }
                    }
                    catch (e_2_1) {
                        e_2 = { error: e_2_1 };
                    }
                    finally {
                        try {
                            if (ticks_1_1 && !ticks_1_1.done && (_b = ticks_1.return))
                                _b.call(ticks_1);
                        }
                        finally {
                            if (e_2)
                                throw e_2.error;
                        }
                    }
                }
            }
            else {
                var startlog = Math.ceil(log_low * 0.999999);
                var endlog = Math.floor(log_high * 1.000001);
                var interval = Math.ceil((endlog - startlog) / 9.0);
                ticks = array_1.range(startlog - 1, endlog + 1, interval)
                    .map(function (i) { return Math.pow(base, i); });
                if (num_minor_ticks > 0 && ticks.length > 0) {
                    var minor_interval_2 = Math.pow(base, interval) / num_minor_ticks;
                    var minor_offsets = array_1.range(1, num_minor_ticks + 1).map(function (i) { return i * minor_interval_2; });
                    try {
                        for (var minor_offsets_2 = tslib_1.__values(minor_offsets), minor_offsets_2_1 = minor_offsets_2.next(); !minor_offsets_2_1.done; minor_offsets_2_1 = minor_offsets_2.next()) {
                            var x = minor_offsets_2_1.value;
                            minor_ticks.push(ticks[0] / x);
                        }
                    }
                    catch (e_4_1) {
                        e_4 = { error: e_4_1 };
                    }
                    finally {
                        try {
                            if (minor_offsets_2_1 && !minor_offsets_2_1.done && (_d = minor_offsets_2.return))
                                _d.call(minor_offsets_2);
                        }
                        finally {
                            if (e_4)
                                throw e_4.error;
                        }
                    }
                    minor_ticks.push(ticks[0]);
                    try {
                        for (var ticks_2 = tslib_1.__values(ticks), ticks_2_1 = ticks_2.next(); !ticks_2_1.done; ticks_2_1 = ticks_2.next()) {
                            var tick = ticks_2_1.value;
                            try {
                                for (var minor_offsets_3 = (e_6 = void 0, tslib_1.__values(minor_offsets)), minor_offsets_3_1 = minor_offsets_3.next(); !minor_offsets_3_1.done; minor_offsets_3_1 = minor_offsets_3.next()) {
                                    var x = minor_offsets_3_1.value;
                                    minor_ticks.push(tick * x);
                                }
                            }
                            catch (e_6_1) {
                                e_6 = { error: e_6_1 };
                            }
                            finally {
                                try {
                                    if (minor_offsets_3_1 && !minor_offsets_3_1.done && (_f = minor_offsets_3.return))
                                        _f.call(minor_offsets_3);
                                }
                                finally {
                                    if (e_6)
                                        throw e_6.error;
                                }
                            }
                        }
                    }
                    catch (e_5_1) {
                        e_5 = { error: e_5_1 };
                    }
                    finally {
                        try {
                            if (ticks_2_1 && !ticks_2_1.done && (_e = ticks_2.return))
                                _e.call(ticks_2);
                        }
                        finally {
                            if (e_5)
                                throw e_5.error;
                        }
                    }
                }
            }
            return {
                major: ticks.filter(function (tick) { return data_low <= tick && tick <= data_high; }),
                minor: minor_ticks.filter(function (tick) { return data_low <= tick && tick <= data_high; }),
            };
        };
        return LogTicker;
    }(adaptive_ticker_1.AdaptiveTicker));
    exports.LogTicker = LogTicker;
    LogTicker.__name__ = "LogTicker";
    LogTicker.init_LogTicker();
},
/* models/axes/mercator_axis.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var axis_1 = require(292) /* ./axis */;
    var linear_axis_1 = require(299) /* ./linear_axis */;
    var mercator_tick_formatter_1 = require(316) /* ../formatters/mercator_tick_formatter */;
    var mercator_ticker_1 = require(317) /* ../tickers/mercator_ticker */;
    var MercatorAxisView = /** @class */ (function (_super) {
        tslib_1.__extends(MercatorAxisView, _super);
        function MercatorAxisView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return MercatorAxisView;
    }(axis_1.AxisView));
    exports.MercatorAxisView = MercatorAxisView;
    MercatorAxisView.__name__ = "MercatorAxisView";
    var MercatorAxis = /** @class */ (function (_super) {
        tslib_1.__extends(MercatorAxis, _super);
        function MercatorAxis(attrs) {
            return _super.call(this, attrs) || this;
        }
        MercatorAxis.init_MercatorAxis = function () {
            this.prototype.default_view = MercatorAxisView;
            this.override({
                ticker: function () { return new mercator_ticker_1.MercatorTicker({ dimension: "lat" }); },
                formatter: function () { return new mercator_tick_formatter_1.MercatorTickFormatter({ dimension: "lat" }); },
            });
        };
        return MercatorAxis;
    }(linear_axis_1.LinearAxis));
    exports.MercatorAxis = MercatorAxis;
    MercatorAxis.__name__ = "MercatorAxis";
    MercatorAxis.init_MercatorAxis();
},
/* models/formatters/mercator_tick_formatter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var basic_tick_formatter_1 = require(245) /* ./basic_tick_formatter */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var projections_1 = require(152) /* ../../core/util/projections */;
    var MercatorTickFormatter = /** @class */ (function (_super) {
        tslib_1.__extends(MercatorTickFormatter, _super);
        function MercatorTickFormatter(attrs) {
            return _super.call(this, attrs) || this;
        }
        MercatorTickFormatter.init_MercatorTickFormatter = function () {
            this.define({
                dimension: [p.LatLon],
            });
        };
        MercatorTickFormatter.prototype.doFormat = function (ticks, opts) {
            if (this.dimension == null)
                throw new Error("MercatorTickFormatter.dimension not configured");
            if (ticks.length == 0)
                return [];
            var n = ticks.length;
            var proj_ticks = new Array(n);
            if (this.dimension == "lon") {
                for (var i = 0; i < n; i++) {
                    var _a = tslib_1.__read(projections_1.wgs84_mercator.invert(ticks[i], opts.loc), 1), lon = _a[0];
                    proj_ticks[i] = lon;
                }
            }
            else {
                for (var i = 0; i < n; i++) {
                    var _b = tslib_1.__read(projections_1.wgs84_mercator.invert(opts.loc, ticks[i]), 2), lat = _b[1];
                    proj_ticks[i] = lat;
                }
            }
            return _super.prototype.doFormat.call(this, proj_ticks, opts);
        };
        return MercatorTickFormatter;
    }(basic_tick_formatter_1.BasicTickFormatter));
    exports.MercatorTickFormatter = MercatorTickFormatter;
    MercatorTickFormatter.__name__ = "MercatorTickFormatter";
    MercatorTickFormatter.init_MercatorTickFormatter();
},
/* models/tickers/mercator_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var basic_ticker_1 = require(241) /* ./basic_ticker */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var projections_1 = require(152) /* ../../core/util/projections */;
    var MercatorTicker = /** @class */ (function (_super) {
        tslib_1.__extends(MercatorTicker, _super);
        function MercatorTicker(attrs) {
            return _super.call(this, attrs) || this;
        }
        MercatorTicker.init_MercatorTicker = function () {
            this.define({
                dimension: [p.LatLon],
            });
        };
        MercatorTicker.prototype.get_ticks_no_defaults = function (data_low, data_high, cross_loc, desired_n_ticks) {
            var _a;
            if (this.dimension == null) {
                throw new Error(this + ".dimension wasn't configured");
            }
            _a = tslib_1.__read(projections_1.clip_mercator(data_low, data_high, this.dimension), 2), data_low = _a[0], data_high = _a[1];
            if (this.dimension == "lon")
                return this._get_ticks_lon(data_low, data_high, cross_loc, desired_n_ticks);
            else
                return this._get_ticks_lat(data_low, data_high, cross_loc, desired_n_ticks);
        };
        MercatorTicker.prototype._get_ticks_lon = function (data_low, data_high, cross_loc, desired_n_ticks) {
            var e_1, _a, e_2, _b;
            var _c = tslib_1.__read(projections_1.wgs84_mercator.invert(data_low, cross_loc), 1), proj_low = _c[0];
            var _d = tslib_1.__read(projections_1.wgs84_mercator.invert(data_high, cross_loc), 2), proj_high = _d[0], proj_cross_loc = _d[1];
            var proj_ticks = _super.prototype.get_ticks_no_defaults.call(this, proj_low, proj_high, cross_loc, desired_n_ticks);
            var major = [];
            try {
                for (var _e = tslib_1.__values(proj_ticks.major), _f = _e.next(); !_f.done; _f = _e.next()) {
                    var tick = _f.value;
                    if (projections_1.in_bounds(tick, "lon")) {
                        var _g = tslib_1.__read(projections_1.wgs84_mercator.compute(tick, proj_cross_loc), 1), lon = _g[0];
                        major.push(lon);
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_f && !_f.done && (_a = _e.return))
                        _a.call(_e);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            var minor = [];
            try {
                for (var _h = tslib_1.__values(proj_ticks.minor), _j = _h.next(); !_j.done; _j = _h.next()) {
                    var tick = _j.value;
                    if (projections_1.in_bounds(tick, "lon")) {
                        var _k = tslib_1.__read(projections_1.wgs84_mercator.compute(tick, proj_cross_loc), 1), lon = _k[0];
                        minor.push(lon);
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_j && !_j.done && (_b = _h.return))
                        _b.call(_h);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return { major: major, minor: minor };
        };
        MercatorTicker.prototype._get_ticks_lat = function (data_low, data_high, cross_loc, desired_n_ticks) {
            var e_3, _a, e_4, _b;
            var _c = tslib_1.__read(projections_1.wgs84_mercator.invert(cross_loc, data_low), 2), proj_low = _c[1];
            var _d = tslib_1.__read(projections_1.wgs84_mercator.invert(cross_loc, data_high), 2), proj_cross_loc = _d[0], proj_high = _d[1];
            var proj_ticks = _super.prototype.get_ticks_no_defaults.call(this, proj_low, proj_high, cross_loc, desired_n_ticks);
            var major = [];
            try {
                for (var _e = tslib_1.__values(proj_ticks.major), _f = _e.next(); !_f.done; _f = _e.next()) {
                    var tick = _f.value;
                    if (projections_1.in_bounds(tick, "lat")) {
                        var _g = tslib_1.__read(projections_1.wgs84_mercator.compute(proj_cross_loc, tick), 2), lat = _g[1];
                        major.push(lat);
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_f && !_f.done && (_a = _e.return))
                        _a.call(_e);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            var minor = [];
            try {
                for (var _h = tslib_1.__values(proj_ticks.minor), _j = _h.next(); !_j.done; _j = _h.next()) {
                    var tick = _j.value;
                    if (projections_1.in_bounds(tick, "lat")) {
                        var _k = tslib_1.__read(projections_1.wgs84_mercator.compute(proj_cross_loc, tick), 2), lat = _k[1];
                        minor.push(lat);
                    }
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (_j && !_j.done && (_b = _h.return))
                        _b.call(_h);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            return { major: major, minor: minor };
        };
        return MercatorTicker;
    }(basic_ticker_1.BasicTicker));
    exports.MercatorTicker = MercatorTicker;
    MercatorTicker.__name__ = "MercatorTicker";
    MercatorTicker.init_MercatorTicker();
},
/* models/callbacks/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var customjs_1 = require(319) /* ./customjs */;
    exports.CustomJS = customjs_1.CustomJS;
    var open_url_1 = require(321) /* ./open_url */;
    exports.OpenURL = open_url_1.OpenURL;
},
/* models/callbacks/customjs.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var callback_1 = require(320) /* ./callback */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var object_1 = require(128) /* ../../core/util/object */;
    var string_1 = require(144) /* ../../core/util/string */;
    var CustomJS = /** @class */ (function (_super) {
        tslib_1.__extends(CustomJS, _super);
        function CustomJS(attrs) {
            return _super.call(this, attrs) || this;
        }
        CustomJS.init_CustomJS = function () {
            this.define({
                args: [p.Any, {}],
                code: [p.String, ''],
            });
        };
        Object.defineProperty(CustomJS.prototype, "names", {
            get: function () {
                return object_1.keys(this.args);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CustomJS.prototype, "values", {
            get: function () {
                return object_1.values(this.args);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CustomJS.prototype, "func", {
            get: function () {
                var code = string_1.use_strict(this.code);
                return new (Function.bind.apply(Function, tslib_1.__spread([void 0], this.names, ["cb_obj", "cb_data", code])))();
            },
            enumerable: true,
            configurable: true
        });
        CustomJS.prototype.execute = function (cb_obj, cb_data) {
            if (cb_data === void 0) {
                cb_data = {};
            }
            return this.func.apply(cb_obj, this.values.concat(cb_obj, cb_data));
        };
        return CustomJS;
    }(callback_1.Callback));
    exports.CustomJS = CustomJS;
    CustomJS.__name__ = "CustomJS";
    CustomJS.init_CustomJS();
},
/* models/callbacks/callback.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var Callback = /** @class */ (function (_super) {
        tslib_1.__extends(Callback, _super);
        function Callback(attrs) {
            return _super.call(this, attrs) || this;
        }
        return Callback;
    }(model_1.Model));
    exports.Callback = Callback;
    Callback.__name__ = "Callback";
},
/* models/callbacks/open_url.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var callback_1 = require(320) /* ./callback */;
    var templating_1 = require(302) /* ../../core/util/templating */;
    var types_1 = require(123) /* ../../core/util/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var OpenURL = /** @class */ (function (_super) {
        tslib_1.__extends(OpenURL, _super);
        function OpenURL(attrs) {
            return _super.call(this, attrs) || this;
        }
        OpenURL.init_OpenURL = function () {
            this.define({
                url: [p.String, 'http://'],
                same_tab: [p.Boolean, false],
            });
        };
        OpenURL.prototype.execute = function (_cb_obj, _a) {
            var e_1, _b, e_2, _c;
            var _this = this;
            var source = _a.source;
            var open_url = function (i) {
                var url = templating_1.replace_placeholders(_this.url, source, i);
                if (!types_1.isString(url))
                    throw new Error("HTML output is not supported in this context");
                if (_this.same_tab)
                    window.location.href = url;
                else
                    window.open(url);
            };
            var selected = source.selected;
            try {
                for (var _d = tslib_1.__values(selected.indices), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var i = _e.value;
                    open_url(i);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_b = _d.return))
                        _b.call(_d);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            try {
                for (var _f = tslib_1.__values(selected.line_indices), _g = _f.next(); !_g.done; _g = _f.next()) {
                    var i = _g.value;
                    open_url(i);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_g && !_g.done && (_c = _f.return))
                        _c.call(_f);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            // TODO: multiline_indices: {[key: string]: number[]}
        };
        return OpenURL;
    }(callback_1.Callback));
    exports.OpenURL = OpenURL;
    OpenURL.__name__ = "OpenURL";
    OpenURL.init_OpenURL();
},
/* models/canvas/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var canvas_1 = require(192) /* ./canvas */;
    exports.Canvas = canvas_1.Canvas;
    var cartesian_frame_1 = require(323) /* ./cartesian_frame */;
    exports.CartesianFrame = cartesian_frame_1.CartesianFrame;
},
/* models/canvas/cartesian_frame.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var categorical_scale_1 = require(324) /* ../scales/categorical_scale */;
    var continuous_scale_1 = require(261) /* ../scales/continuous_scale */;
    var log_scale_1 = require(272) /* ../scales/log_scale */;
    var range1d_1 = require(273) /* ../ranges/range1d */;
    var data_range1d_1 = require(325) /* ../ranges/data_range1d */;
    var factor_range_1 = require(213) /* ../ranges/factor_range */;
    var layout_1 = require(327) /* ../../core/layout */;
    var object_1 = require(128) /* ../../core/util/object */;
    var assert_1 = require(126) /* ../../core/util/assert */;
    var CartesianFrame = /** @class */ (function (_super) {
        tslib_1.__extends(CartesianFrame, _super);
        function CartesianFrame(in_x_scale, in_y_scale, x_range, y_range, extra_x_ranges, extra_y_ranges) {
            if (extra_x_ranges === void 0) {
                extra_x_ranges = {};
            }
            if (extra_y_ranges === void 0) {
                extra_y_ranges = {};
            }
            var _this = _super.call(this) || this;
            _this.in_x_scale = in_x_scale;
            _this.in_y_scale = in_y_scale;
            _this.x_range = x_range;
            _this.y_range = y_range;
            _this.extra_x_ranges = extra_x_ranges;
            _this.extra_y_ranges = extra_y_ranges;
            assert_1.assert(in_x_scale.source_range == null && in_x_scale.target_range == null);
            assert_1.assert(in_y_scale.source_range == null && in_y_scale.target_range == null);
            _this._configure_scales();
            return _this;
        }
        CartesianFrame.prototype._get_ranges = function (range, extra_ranges) {
            return new Map(object_1.entries(Object.assign(Object.assign({}, extra_ranges), { default: range })));
        };
        /*protected*/ CartesianFrame.prototype._get_scales = function (scale, ranges, frame_range) {
            var e_1, _a;
            var scales = new Map();
            try {
                for (var ranges_1 = tslib_1.__values(ranges), ranges_1_1 = ranges_1.next(); !ranges_1_1.done; ranges_1_1 = ranges_1.next()) {
                    var _b = tslib_1.__read(ranges_1_1.value, 2), name = _b[0], range = _b[1];
                    if (range instanceof data_range1d_1.DataRange1d || range instanceof range1d_1.Range1d) {
                        if (!(scale instanceof continuous_scale_1.ContinuousScale))
                            throw new Error("Range " + range.type + " is incompatible is Scale " + scale.type);
                    }
                    if (range instanceof factor_range_1.FactorRange) {
                        if (!(scale instanceof categorical_scale_1.CategoricalScale))
                            throw new Error("Range " + range.type + " is incompatible is Scale " + scale.type);
                    }
                    if (scale instanceof log_scale_1.LogScale && range instanceof data_range1d_1.DataRange1d)
                        range.scale_hint = "log";
                    var derived_scale = scale.clone();
                    derived_scale.setv({ source_range: range, target_range: frame_range });
                    scales.set(name, derived_scale);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (ranges_1_1 && !ranges_1_1.done && (_a = ranges_1.return))
                        _a.call(ranges_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return scales;
        };
        CartesianFrame.prototype._configure_frame_ranges = function () {
            // data to/from screen space transform (left-bottom <-> left-top origin)
            var bbox = this.bbox;
            this._x_target = new range1d_1.Range1d({ start: bbox.left, end: bbox.right });
            this._y_target = new range1d_1.Range1d({ start: bbox.bottom, end: bbox.top });
        };
        CartesianFrame.prototype._configure_scales = function () {
            this._configure_frame_ranges();
            this._x_ranges = this._get_ranges(this.x_range, this.extra_x_ranges);
            this._y_ranges = this._get_ranges(this.y_range, this.extra_y_ranges);
            this._x_scales = this._get_scales(this.in_x_scale, this._x_ranges, this._x_target);
            this._y_scales = this._get_scales(this.in_y_scale, this._y_ranges, this._y_target);
        };
        CartesianFrame.prototype._update_scales = function () {
            var e_2, _a, e_3, _b;
            this._configure_frame_ranges();
            try {
                for (var _c = tslib_1.__values(this._x_scales), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = tslib_1.__read(_d.value, 2), scale = _e[1];
                    scale.target_range = this._x_target;
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return))
                        _a.call(_c);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            try {
                for (var _f = tslib_1.__values(this._y_scales), _g = _f.next(); !_g.done; _g = _f.next()) {
                    var _h = tslib_1.__read(_g.value, 2), scale = _h[1];
                    scale.target_range = this._y_target;
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_g && !_g.done && (_b = _f.return))
                        _b.call(_f);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
        };
        CartesianFrame.prototype._set_geometry = function (outer, inner) {
            _super.prototype._set_geometry.call(this, outer, inner);
            this._update_scales();
        };
        Object.defineProperty(CartesianFrame.prototype, "x_ranges", {
            get: function () {
                return this._x_ranges;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CartesianFrame.prototype, "y_ranges", {
            get: function () {
                return this._y_ranges;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CartesianFrame.prototype, "x_scales", {
            get: function () {
                return this._x_scales;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CartesianFrame.prototype, "y_scales", {
            get: function () {
                return this._y_scales;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CartesianFrame.prototype, "x_scale", {
            get: function () {
                return this._x_scales.get("default");
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CartesianFrame.prototype, "y_scale", {
            get: function () {
                return this._y_scales.get("default");
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CartesianFrame.prototype, "xscales", {
            /** @deprecated */
            get: function () {
                return object_1.to_object(this.x_scales);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CartesianFrame.prototype, "yscales", {
            /** @deprecated */
            get: function () {
                return object_1.to_object(this.y_scales);
            },
            enumerable: true,
            configurable: true
        });
        return CartesianFrame;
    }(layout_1.LayoutItem));
    exports.CartesianFrame = CartesianFrame;
    CartesianFrame.__name__ = "CartesianFrame";
},
/* models/scales/categorical_scale.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var scale_1 = require(262) /* ./scale */;
    var CategoricalScale = /** @class */ (function (_super) {
        tslib_1.__extends(CategoricalScale, _super);
        function CategoricalScale(attrs) {
            return _super.call(this, attrs) || this;
        }
        CategoricalScale.prototype.compute = function (x) {
            return _super.prototype._linear_compute.call(this, this.source_range.synthetic(x));
        };
        CategoricalScale.prototype.v_compute = function (xs) {
            return _super.prototype._linear_v_compute.call(this, this.source_range.v_synthetic(xs));
        };
        CategoricalScale.prototype.invert = function (xprime) {
            return this._linear_invert(xprime);
        };
        CategoricalScale.prototype.v_invert = function (xprimes) {
            return this._linear_v_invert(xprimes);
        };
        return CategoricalScale;
    }(scale_1.Scale));
    exports.CategoricalScale = CategoricalScale;
    CategoricalScale.__name__ = "CategoricalScale";
},
/* models/ranges/data_range1d.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var data_range_1 = require(326) /* ./data_range */;
    var glyph_renderer_1 = require(205) /* ../renderers/glyph_renderer */;
    var logging_1 = require(134) /* ../../core/logging */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var bbox = tslib_1.__importStar(require(194) /* ../../core/util/bbox */);
    var array_1 = require(124) /* ../../core/util/array */;
    var DataRange1d = /** @class */ (function (_super) {
        tslib_1.__extends(DataRange1d, _super);
        function DataRange1d(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.have_updated_interactively = false;
            return _this;
        }
        DataRange1d.init_DataRange1d = function () {
            this.define({
                start: [p.Number],
                end: [p.Number],
                range_padding: [p.Number, 0.1],
                range_padding_units: [p.PaddingUnits, "percent"],
                flipped: [p.Boolean, false],
                follow: [p.StartEnd],
                follow_interval: [p.Number],
                default_span: [p.Number, 2],
                only_visible: [p.Boolean, false],
            });
            this.internal({
                scale_hint: [p.String, 'auto'],
            });
        };
        DataRange1d.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._initial_start = this.start;
            this._initial_end = this.end;
            this._initial_range_padding = this.range_padding;
            this._initial_range_padding_units = this.range_padding_units;
            this._initial_follow = this.follow;
            this._initial_follow_interval = this.follow_interval;
            this._initial_default_span = this.default_span;
            this._plot_bounds = new Map();
        };
        Object.defineProperty(DataRange1d.prototype, "min", {
            get: function () {
                return Math.min(this.start, this.end);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DataRange1d.prototype, "max", {
            get: function () {
                return Math.max(this.start, this.end);
            },
            enumerable: true,
            configurable: true
        });
        DataRange1d.prototype.computed_renderers = function () {
            var e_1, _a, e_2, _b;
            // TODO (bev) check that renderers actually configured with this range
            var names = this.names;
            var renderers = this.renderers;
            if (renderers.length == 0) {
                try {
                    for (var _c = tslib_1.__values(this.plots), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var plot = _d.value;
                        var rs = plot.renderers.filter(function (r) { return r instanceof glyph_renderer_1.GlyphRenderer; });
                        renderers = renderers.concat(rs);
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (_d && !_d.done && (_a = _c.return))
                            _a.call(_c);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
            }
            if (names.length > 0)
                renderers = renderers.filter(function (r) { return array_1.includes(names, r.name); });
            logging_1.logger.debug("computed " + renderers.length + " renderers for " + this);
            try {
                for (var renderers_1 = tslib_1.__values(renderers), renderers_1_1 = renderers_1.next(); !renderers_1_1.done; renderers_1_1 = renderers_1.next()) {
                    var renderer = renderers_1_1.value;
                    logging_1.logger.trace(" - " + renderer);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (renderers_1_1 && !renderers_1_1.done && (_b = renderers_1.return))
                        _b.call(renderers_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return renderers;
        };
        /*protected*/ DataRange1d.prototype._compute_plot_bounds = function (renderers, bounds) {
            var e_3, _a;
            var result = bbox.empty();
            try {
                for (var renderers_2 = tslib_1.__values(renderers), renderers_2_1 = renderers_2.next(); !renderers_2_1.done; renderers_2_1 = renderers_2.next()) {
                    var r = renderers_2_1.value;
                    var rect = bounds.get(r);
                    if (rect != null && (r.visible || !this.only_visible)) {
                        result = bbox.union(result, rect);
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (renderers_2_1 && !renderers_2_1.done && (_a = renderers_2.return))
                        _a.call(renderers_2);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            return result;
        };
        DataRange1d.prototype.adjust_bounds_for_aspect = function (bounds, ratio) {
            var result = bbox.empty();
            var width = bounds.x1 - bounds.x0;
            if (width <= 0) {
                width = 1.0;
            }
            var height = bounds.y1 - bounds.y0;
            if (height <= 0) {
                height = 1.0;
            }
            var xcenter = 0.5 * (bounds.x1 + bounds.x0);
            var ycenter = 0.5 * (bounds.y1 + bounds.y0);
            if (width < ratio * height) {
                width = ratio * height;
            }
            else {
                height = width / ratio;
            }
            result.x1 = xcenter + 0.5 * width;
            result.x0 = xcenter - 0.5 * width;
            result.y1 = ycenter + 0.5 * height;
            result.y0 = ycenter - 0.5 * height;
            return result;
        };
        /*protected*/ DataRange1d.prototype._compute_min_max = function (plot_bounds, dimension) {
            var e_4, _a, _b, _c;
            var overall = bbox.empty();
            try {
                for (var plot_bounds_1 = tslib_1.__values(plot_bounds), plot_bounds_1_1 = plot_bounds_1.next(); !plot_bounds_1_1.done; plot_bounds_1_1 = plot_bounds_1.next()) {
                    var rect = plot_bounds_1_1.value;
                    overall = bbox.union(overall, rect);
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (plot_bounds_1_1 && !plot_bounds_1_1.done && (_a = plot_bounds_1.return))
                        _a.call(plot_bounds_1);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            var min, max;
            if (dimension == 0)
                _b = tslib_1.__read([overall.x0, overall.x1], 2), min = _b[0], max = _b[1];
            else
                _c = tslib_1.__read([overall.y0, overall.y1], 2), min = _c[0], max = _c[1];
            return [min, max];
        };
        /*protected*/ DataRange1d.prototype._compute_range = function (min, max) {
            var _a;
            var range_padding = this.range_padding; // XXX: ? 0
            var start, end;
            if (this._initial_start != null)
                min = this._initial_start;
            if (this._initial_end != null)
                max = this._initial_end;
            if (this.scale_hint == "log") {
                if (isNaN(min) || !isFinite(min) || min <= 0) {
                    if (isNaN(max) || !isFinite(max) || max <= 0)
                        min = 0.1;
                    else
                        min = max / 100;
                    logging_1.logger.warn("could not determine minimum data value for log axis, DataRange1d using value " + min);
                }
                if (isNaN(max) || !isFinite(max) || max <= 0) {
                    if (isNaN(min) || !isFinite(min) || min <= 0)
                        max = 10;
                    else
                        max = min * 100;
                    logging_1.logger.warn("could not determine maximum data value for log axis, DataRange1d using value " + max);
                }
                var center = void 0, span = void 0;
                if (max == min) {
                    span = this.default_span + 0.001;
                    center = Math.log(min) / Math.log(10);
                }
                else {
                    var log_min = void 0, log_max = void 0;
                    if (this.range_padding_units == "percent") {
                        log_min = Math.log(min) / Math.log(10);
                        log_max = Math.log(max) / Math.log(10);
                        span = (log_max - log_min) * (1 + range_padding);
                    }
                    else {
                        log_min = Math.log(min - range_padding) / Math.log(10);
                        log_max = Math.log(max + range_padding) / Math.log(10);
                        span = log_max - log_min;
                    }
                    center = (log_min + log_max) / 2.0;
                }
                start = Math.pow(10, (center - span / 2.0));
                end = Math.pow(10, (center + span / 2.0));
            }
            else {
                var span = void 0;
                if (max == min)
                    span = this.default_span;
                else {
                    if (this.range_padding_units == "percent")
                        span = (max - min) * (1 + range_padding);
                    else
                        span = (max - min) + 2 * range_padding;
                }
                var center = (max + min) / 2.0;
                start = center - span / 2.0;
                end = center + span / 2.0;
            }
            var follow_sign = +1;
            if (this.flipped) {
                _a = tslib_1.__read([end, start], 2), start = _a[0], end = _a[1];
                follow_sign = -1;
            }
            var follow_interval = this.follow_interval;
            if (follow_interval != null && Math.abs(start - end) > follow_interval) {
                if (this.follow == 'start')
                    end = start + follow_sign * follow_interval;
                else if (this.follow == 'end')
                    start = end - follow_sign * follow_interval;
            }
            return [start, end];
        };
        DataRange1d.prototype.update = function (bounds, dimension, plot, ratio) {
            if (this.have_updated_interactively)
                return;
            var renderers = this.computed_renderers();
            // update the raw data bounds for all renderers we care about
            var total_bounds = this._compute_plot_bounds(renderers, bounds);
            if (ratio != null)
                total_bounds = this.adjust_bounds_for_aspect(total_bounds, ratio);
            this._plot_bounds.set(plot, total_bounds);
            // compute the min/mix for our specified dimension
            var _a = tslib_1.__read(this._compute_min_max(this._plot_bounds.values(), dimension), 2), min = _a[0], max = _a[1];
            // derive start, end from bounds and data range config
            var _b = tslib_1.__read(this._compute_range(min, max), 2), start = _b[0], end = _b[1];
            if (this._initial_start != null) {
                if (this.scale_hint == "log") {
                    if (this._initial_start > 0)
                        start = this._initial_start;
                }
                else
                    start = this._initial_start;
            }
            if (this._initial_end != null) {
                if (this.scale_hint == "log") {
                    if (this._initial_end > 0)
                        end = this._initial_end;
                }
                else
                    end = this._initial_end;
            }
            // only trigger updates when there are changes
            var _c = tslib_1.__read([this.start, this.end], 2), _start = _c[0], _end = _c[1];
            if (start != _start || end != _end) {
                var new_range = {};
                if (start != _start)
                    new_range.start = start;
                if (end != _end)
                    new_range.end = end;
                this.setv(new_range);
            }
            if (this.bounds == 'auto')
                this.setv({ bounds: [start, end] }, { silent: true });
            this.change.emit();
        };
        DataRange1d.prototype.reset = function () {
            this.have_updated_interactively = false;
            // change events silenced as PlotView.update_dataranges triggers property callbacks
            this.setv({
                range_padding: this._initial_range_padding,
                range_padding_units: this._initial_range_padding_units,
                follow: this._initial_follow,
                follow_interval: this._initial_follow_interval,
                default_span: this._initial_default_span,
            }, { silent: true });
            this.change.emit();
        };
        return DataRange1d;
    }(data_range_1.DataRange));
    exports.DataRange1d = DataRange1d;
    DataRange1d.__name__ = "DataRange1d";
    DataRange1d.init_DataRange1d();
},
/* models/ranges/data_range.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var range_1 = require(214) /* ./range */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var DataRange = /** @class */ (function (_super) {
        tslib_1.__extends(DataRange, _super);
        function DataRange(attrs) {
            return _super.call(this, attrs) || this;
        }
        DataRange.init_DataRange = function () {
            this.define({
                names: [p.Array, []],
                renderers: [p.Array, []],
            });
        };
        return DataRange;
    }(range_1.Range));
    exports.DataRange = DataRange;
    DataRange.__name__ = "DataRange";
    DataRange.init_DataRange();
},
/* core/layout/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var types_1 = require(328) /* ./types */;
    exports.Sizeable = types_1.Sizeable;
    exports.SizingPolicy = types_1.SizingPolicy;
    var layoutable_1 = require(329) /* ./layoutable */;
    exports.Layoutable = layoutable_1.Layoutable;
    exports.LayoutItem = layoutable_1.LayoutItem;
    var alignments_1 = require(330) /* ./alignments */;
    exports.HStack = alignments_1.HStack;
    exports.VStack = alignments_1.VStack;
    exports.AnchorLayout = alignments_1.AnchorLayout;
    var grid_1 = require(331) /* ./grid */;
    exports.Grid = grid_1.Grid;
    exports.Row = grid_1.Row;
    exports.Column = grid_1.Column;
    var html_1 = require(332) /* ./html */;
    exports.ContentBox = html_1.ContentBox;
    exports.VariadicBox = html_1.VariadicBox;
},
/* core/layout/types.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var kinds_1 = require(136) /* ../kinds */;
    var min = Math.min, max = Math.max;
    var Sizeable = /** @class */ (function () {
        function Sizeable(size) {
            if (size === void 0) {
                size = {};
            }
            this.width = size.width != null ? size.width : 0;
            this.height = size.height != null ? size.height : 0;
        }
        Sizeable.prototype.bounded_to = function (_a) {
            var width = _a.width, height = _a.height;
            return new Sizeable({
                width: this.width == Infinity && width != null ? width : this.width,
                height: this.height == Infinity && height != null ? height : this.height,
            });
        };
        Sizeable.prototype.expanded_to = function (_a) {
            var width = _a.width, height = _a.height;
            return new Sizeable({
                width: width != Infinity ? max(this.width, width) : this.width,
                height: height != Infinity ? max(this.height, height) : this.height,
            });
        };
        Sizeable.prototype.expand_to = function (_a) {
            var width = _a.width, height = _a.height;
            this.width = max(this.width, width);
            this.height = max(this.height, height);
        };
        Sizeable.prototype.narrowed_to = function (_a) {
            var width = _a.width, height = _a.height;
            return new Sizeable({
                width: min(this.width, width),
                height: min(this.height, height),
            });
        };
        Sizeable.prototype.narrow_to = function (_a) {
            var width = _a.width, height = _a.height;
            this.width = min(this.width, width);
            this.height = min(this.height, height);
        };
        Sizeable.prototype.grow_by = function (_a) {
            var left = _a.left, right = _a.right, top = _a.top, bottom = _a.bottom;
            var width = this.width + left + right;
            var height = this.height + top + bottom;
            return new Sizeable({ width: width, height: height });
        };
        Sizeable.prototype.shrink_by = function (_a) {
            var left = _a.left, right = _a.right, top = _a.top, bottom = _a.bottom;
            var width = max(this.width - left - right, 0);
            var height = max(this.height - top - bottom, 0);
            return new Sizeable({ width: width, height: height });
        };
        Sizeable.prototype.map = function (w_fn, h_fn) {
            return new Sizeable({
                width: w_fn(this.width),
                height: (h_fn != null ? h_fn : w_fn)(this.height),
            });
        };
        return Sizeable;
    }());
    exports.Sizeable = Sizeable;
    Sizeable.__name__ = "Sizeable";
    exports.SizingPolicy = kinds_1.Enum("fixed", "fit", "min", "max");
},
/* core/layout/layoutable.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(328) /* ./types */;
    var bbox_1 = require(194) /* ../util/bbox */;
    var min = Math.min, max = Math.max, round = Math.round;
    var Layoutable = /** @class */ (function () {
        function Layoutable() {
            this._bbox = new bbox_1.BBox();
            this._inner_bbox = new bbox_1.BBox();
        }
        Object.defineProperty(Layoutable.prototype, "bbox", {
            get: function () {
                return this._bbox;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Layoutable.prototype, "inner_bbox", {
            get: function () {
                return this._inner_bbox;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Layoutable.prototype, "sizing", {
            get: function () {
                return this._sizing;
            },
            enumerable: true,
            configurable: true
        });
        Layoutable.prototype.set_sizing = function (sizing) {
            var width_policy = sizing.width_policy || "fit";
            var width = sizing.width;
            var min_width = sizing.min_width != null ? sizing.min_width : 0;
            var max_width = sizing.max_width != null ? sizing.max_width : Infinity;
            var height_policy = sizing.height_policy || "fit";
            var height = sizing.height;
            var min_height = sizing.min_height != null ? sizing.min_height : 0;
            var max_height = sizing.max_height != null ? sizing.max_height : Infinity;
            var aspect = sizing.aspect;
            var margin = sizing.margin || { top: 0, right: 0, bottom: 0, left: 0 };
            var visible = sizing.visible !== false;
            var halign = sizing.halign || "start";
            var valign = sizing.valign || "start";
            this._sizing = {
                width_policy: width_policy, min_width: min_width, width: width, max_width: max_width,
                height_policy: height_policy, min_height: min_height, height: height, max_height: max_height,
                aspect: aspect,
                margin: margin,
                visible: visible,
                halign: halign,
                valign: valign,
                size: { width: width, height: height },
                min_size: { width: min_width, height: min_height },
                max_size: { width: max_width, height: max_height },
            };
            this._init();
        };
        Layoutable.prototype._init = function () { };
        Layoutable.prototype._set_geometry = function (outer, inner) {
            this._bbox = outer;
            this._inner_bbox = inner;
        };
        Layoutable.prototype.set_geometry = function (outer, inner) {
            this._set_geometry(outer, inner || outer);
        };
        Layoutable.prototype.is_width_expanding = function () {
            return this.sizing.width_policy == "max";
        };
        Layoutable.prototype.is_height_expanding = function () {
            return this.sizing.height_policy == "max";
        };
        Layoutable.prototype.apply_aspect = function (viewport, _a) {
            var width = _a.width, height = _a.height;
            var aspect = this.sizing.aspect;
            if (aspect != null) {
                var _b = this.sizing, width_policy = _b.width_policy, height_policy = _b.height_policy;
                var gt = function (width, height) {
                    var policies = { max: 4, fit: 3, min: 2, fixed: 1 };
                    return policies[width] > policies[height];
                };
                if (width_policy != "fixed" && height_policy != "fixed") {
                    if (width_policy == height_policy) {
                        var w_width = width;
                        var w_height = round(width / aspect);
                        var h_width = round(height * aspect);
                        var h_height = height;
                        var w_diff = Math.abs(viewport.width - w_width) + Math.abs(viewport.height - w_height);
                        var h_diff = Math.abs(viewport.width - h_width) + Math.abs(viewport.height - h_height);
                        if (w_diff <= h_diff) {
                            width = w_width;
                            height = w_height;
                        }
                        else {
                            width = h_width;
                            height = h_height;
                        }
                    }
                    else if (gt(width_policy, height_policy)) {
                        height = round(width / aspect);
                    }
                    else {
                        width = round(height * aspect);
                    }
                }
                else if (width_policy == "fixed") {
                    height = round(width / aspect);
                }
                else if (height_policy == "fixed") {
                    width = round(height * aspect);
                }
            }
            return { width: width, height: height };
        };
        Layoutable.prototype.measure = function (viewport_size) {
            var _this = this;
            if (!this.sizing.visible)
                return { width: 0, height: 0 };
            var exact_width = function (width) {
                return _this.sizing.width_policy == "fixed" && _this.sizing.width != null ? _this.sizing.width : width;
            };
            var exact_height = function (height) {
                return _this.sizing.height_policy == "fixed" && _this.sizing.height != null ? _this.sizing.height : height;
            };
            var viewport = new types_1.Sizeable(viewport_size)
                .shrink_by(this.sizing.margin)
                .map(exact_width, exact_height);
            var computed = this._measure(viewport);
            var clipped = this.clip_size(computed);
            var width = exact_width(clipped.width);
            var height = exact_height(clipped.height);
            var size = this.apply_aspect(viewport, { width: width, height: height });
            return Object.assign(Object.assign({}, computed), size);
        };
        Layoutable.prototype.compute = function (viewport) {
            if (viewport === void 0) {
                viewport = {};
            }
            var size_hint = this.measure({
                width: viewport.width != null && this.is_width_expanding() ? viewport.width : Infinity,
                height: viewport.height != null && this.is_height_expanding() ? viewport.height : Infinity,
            });
            var width = size_hint.width, height = size_hint.height;
            var outer = new bbox_1.BBox({ left: 0, top: 0, width: width, height: height });
            var inner = undefined;
            if (size_hint.inner != null) {
                var _a = size_hint.inner, left = _a.left, top = _a.top, right = _a.right, bottom = _a.bottom;
                inner = new bbox_1.BBox({ left: left, top: top, right: width - right, bottom: height - bottom });
            }
            this.set_geometry(outer, inner);
        };
        Object.defineProperty(Layoutable.prototype, "xview", {
            get: function () {
                return this.bbox.xview;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Layoutable.prototype, "yview", {
            get: function () {
                return this.bbox.yview;
            },
            enumerable: true,
            configurable: true
        });
        Layoutable.prototype.clip_width = function (width) {
            return max(this.sizing.min_width, min(width, this.sizing.max_width));
        };
        Layoutable.prototype.clip_height = function (height) {
            return max(this.sizing.min_height, min(height, this.sizing.max_height));
        };
        Layoutable.prototype.clip_size = function (_a) {
            var width = _a.width, height = _a.height;
            return {
                width: this.clip_width(width),
                height: this.clip_height(height),
            };
        };
        return Layoutable;
    }());
    exports.Layoutable = Layoutable;
    Layoutable.__name__ = "Layoutable";
    var LayoutItem = /** @class */ (function (_super) {
        tslib_1.__extends(LayoutItem, _super);
        function LayoutItem() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        /*
        constructor(readonly measure_fn: (viewport: Size) => Size) {
          super()
        }
        protected _measure(viewport: Size): SizeHint {
          return this.measure_fn(viewport)
        }
        protected _measure(viewport: Size): SizeHint {
          return {
            width: viewport.width != Infinity ? viewport.width : this.sizing.min_width,
            height: viewport.height != Infinity ? viewport.height : this.sizing.min_width,
          }
        }
        */
        LayoutItem.prototype._measure = function (viewport) {
            var _a = this.sizing, width_policy = _a.width_policy, height_policy = _a.height_policy;
            var width;
            if (viewport.width == Infinity) {
                width = this.sizing.width != null ? this.sizing.width : 0;
            }
            else {
                switch (width_policy) {
                    case "fixed": {
                        width = this.sizing.width != null ? this.sizing.width : 0;
                        break;
                    }
                    case "min": {
                        width = this.sizing.width != null ? min(viewport.width, this.sizing.width) : 0;
                        break;
                    }
                    case "fit": {
                        width = this.sizing.width != null ? min(viewport.width, this.sizing.width) : viewport.width;
                        break;
                    }
                    case "max": {
                        width = this.sizing.width != null ? max(viewport.width, this.sizing.width) : viewport.width;
                        break;
                    }
                }
            }
            var height;
            if (viewport.height == Infinity) {
                height = this.sizing.height != null ? this.sizing.height : 0;
            }
            else {
                switch (height_policy) {
                    case "fixed": {
                        height = this.sizing.height != null ? this.sizing.height : 0;
                        break;
                    }
                    case "min": {
                        height = this.sizing.height != null ? min(viewport.height, this.sizing.height) : 0;
                        break;
                    }
                    case "fit": {
                        height = this.sizing.height != null ? min(viewport.height, this.sizing.height) : viewport.height;
                        break;
                    }
                    case "max": {
                        height = this.sizing.height != null ? max(viewport.height, this.sizing.height) : viewport.height;
                        break;
                    }
                }
            }
            return { width: width, height: height };
        };
        return LayoutItem;
    }(Layoutable));
    exports.LayoutItem = LayoutItem;
    LayoutItem.__name__ = "LayoutItem";
    var ContentLayoutable = /** @class */ (function (_super) {
        tslib_1.__extends(ContentLayoutable, _super);
        function ContentLayoutable() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ContentLayoutable.prototype._measure = function (viewport) {
            var _this = this;
            var content_size = this._content_size();
            var bounds = viewport
                .bounded_to(this.sizing.size)
                .bounded_to(content_size);
            var width = (function () {
                switch (_this.sizing.width_policy) {
                    case "fixed":
                        return _this.sizing.width != null ? _this.sizing.width : content_size.width;
                    case "min":
                        return content_size.width;
                    case "fit":
                        return bounds.width;
                    case "max":
                        return Math.max(content_size.width, bounds.width);
                }
            })();
            var height = (function () {
                switch (_this.sizing.height_policy) {
                    case "fixed":
                        return _this.sizing.height != null ? _this.sizing.height : content_size.height;
                    case "min":
                        return content_size.height;
                    case "fit":
                        return bounds.height;
                    case "max":
                        return Math.max(content_size.height, bounds.height);
                }
            })();
            return { width: width, height: height };
        };
        return ContentLayoutable;
    }(Layoutable));
    exports.ContentLayoutable = ContentLayoutable;
    ContentLayoutable.__name__ = "ContentLayoutable";
},
/* core/layout/alignments.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var layoutable_1 = require(329) /* ./layoutable */;
    var bbox_1 = require(194) /* ../util/bbox */;
    var Stack = /** @class */ (function (_super) {
        tslib_1.__extends(Stack, _super);
        function Stack() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.children = [];
            return _this;
        }
        return Stack;
    }(layoutable_1.Layoutable));
    exports.Stack = Stack;
    Stack.__name__ = "Stack";
    var HStack = /** @class */ (function (_super) {
        tslib_1.__extends(HStack, _super);
        function HStack() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        HStack.prototype._measure = function (_viewport) {
            var e_1, _a;
            var width = 0;
            var height = 0;
            try {
                for (var _b = tslib_1.__values(this.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var child = _c.value;
                    var size_hint = child.measure({ width: 0, height: 0 });
                    width += size_hint.width;
                    height = Math.max(height, size_hint.height);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return { width: width, height: height };
        };
        HStack.prototype._set_geometry = function (outer, inner) {
            var e_2, _a;
            _super.prototype._set_geometry.call(this, outer, inner);
            var top = outer.top, bottom = outer.bottom;
            var left = outer.left;
            try {
                for (var _b = tslib_1.__values(this.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var child = _c.value;
                    var width = child.measure({ width: 0, height: 0 }).width;
                    child.set_geometry(new bbox_1.BBox({ left: left, width: width, top: top, bottom: bottom }));
                    left += width;
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
        };
        return HStack;
    }(Stack));
    exports.HStack = HStack;
    HStack.__name__ = "HStack";
    var VStack = /** @class */ (function (_super) {
        tslib_1.__extends(VStack, _super);
        function VStack() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        VStack.prototype._measure = function (_viewport) {
            var e_3, _a;
            var width = 0;
            var height = 0;
            try {
                for (var _b = tslib_1.__values(this.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var child = _c.value;
                    var size_hint = child.measure({ width: 0, height: 0 });
                    width = Math.max(width, size_hint.width);
                    height += size_hint.height;
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            return { width: width, height: height };
        };
        VStack.prototype._set_geometry = function (outer, inner) {
            var e_4, _a;
            _super.prototype._set_geometry.call(this, outer, inner);
            var left = outer.left, right = outer.right;
            var top = outer.top;
            try {
                for (var _b = tslib_1.__values(this.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var child = _c.value;
                    var height = child.measure({ width: 0, height: 0 }).height;
                    child.set_geometry(new bbox_1.BBox({ top: top, height: height, left: left, right: right }));
                    top += height;
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
        };
        return VStack;
    }(Stack));
    exports.VStack = VStack;
    VStack.__name__ = "VStack";
    var AnchorLayout = /** @class */ (function (_super) {
        tslib_1.__extends(AnchorLayout, _super);
        function AnchorLayout() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.children = [];
            return _this;
        }
        AnchorLayout.prototype._measure = function (viewport) {
            var e_5, _a;
            var width = 0;
            var height = 0;
            try {
                for (var _b = tslib_1.__values(this.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var layout = _c.value.layout;
                    var size_hint = layout.measure(viewport);
                    width = Math.max(width, size_hint.width);
                    height = Math.max(height, size_hint.height);
                }
            }
            catch (e_5_1) {
                e_5 = { error: e_5_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_5)
                        throw e_5.error;
                }
            }
            return { width: width, height: height };
        };
        AnchorLayout.prototype._set_geometry = function (outer, inner) {
            var e_6, _a;
            _super.prototype._set_geometry.call(this, outer, inner);
            try {
                for (var _b = tslib_1.__values(this.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var _d = _c.value, layout = _d.layout, anchor = _d.anchor, margin = _d.margin;
                    var left = outer.left, right = outer.right, top = outer.top, bottom = outer.bottom, hcenter = outer.hcenter, vcenter = outer.vcenter;
                    var _e = layout.measure(outer), width = _e.width, height = _e.height;
                    var bbox = void 0;
                    switch (anchor) {
                        case 'top_left':
                            bbox = new bbox_1.BBox({ left: left + margin, top: top + margin, width: width, height: height });
                            break;
                        case 'top_center':
                            bbox = new bbox_1.BBox({ hcenter: hcenter, top: top + margin, width: width, height: height });
                            break;
                        case 'top_right':
                            bbox = new bbox_1.BBox({ right: right - margin, top: top + margin, width: width, height: height });
                            break;
                        case 'bottom_right':
                            bbox = new bbox_1.BBox({ right: right - margin, bottom: bottom - margin, width: width, height: height });
                            break;
                        case 'bottom_center':
                            bbox = new bbox_1.BBox({ hcenter: hcenter, bottom: bottom - margin, width: width, height: height });
                            break;
                        case 'bottom_left':
                            bbox = new bbox_1.BBox({ left: left + margin, bottom: bottom - margin, width: width, height: height });
                            break;
                        case 'center_left':
                            bbox = new bbox_1.BBox({ left: left + margin, vcenter: vcenter, width: width, height: height });
                            break;
                        case 'center':
                            bbox = new bbox_1.BBox({ hcenter: hcenter, vcenter: vcenter, width: width, height: height });
                            break;
                        case 'center_right':
                            bbox = new bbox_1.BBox({ right: right - margin, vcenter: vcenter, width: width, height: height });
                            break;
                    }
                    layout.set_geometry(bbox);
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
        };
        return AnchorLayout;
    }(layoutable_1.Layoutable));
    exports.AnchorLayout = AnchorLayout;
    AnchorLayout.__name__ = "AnchorLayout";
},
/* core/layout/grid.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(328) /* ./types */;
    var layoutable_1 = require(329) /* ./layoutable */;
    var types_2 = require(123) /* ../util/types */;
    var bbox_1 = require(194) /* ../util/bbox */;
    var array_1 = require(124) /* ../util/array */;
    var max = Math.max, round = Math.round;
    var DefaultMap = /** @class */ (function () {
        function DefaultMap(def) {
            this.def = def;
            this._map = new Map();
        }
        DefaultMap.prototype.get = function (key) {
            var value = this._map.get(key);
            if (value === undefined) {
                value = this.def();
                this._map.set(key, value);
            }
            return value;
        };
        DefaultMap.prototype.apply = function (key, fn) {
            var value = this.get(key);
            this._map.set(key, fn(value));
        };
        return DefaultMap;
    }());
    DefaultMap.__name__ = "DefaultMap";
    var Container = /** @class */ (function () {
        function Container() {
            this._items = [];
            this._nrows = 0;
            this._ncols = 0;
        }
        Object.defineProperty(Container.prototype, "nrows", {
            get: function () {
                return this._nrows;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Container.prototype, "ncols", {
            get: function () {
                return this._ncols;
            },
            enumerable: true,
            configurable: true
        });
        Container.prototype.add = function (span, data) {
            var r1 = span.r1, c1 = span.c1;
            this._nrows = max(this._nrows, r1 + 1);
            this._ncols = max(this._ncols, c1 + 1);
            this._items.push({ span: span, data: data });
        };
        Container.prototype.at = function (r, c) {
            var selected = this._items.filter(function (_b) {
                var span = _b.span;
                return span.r0 <= r && r <= span.r1 &&
                    span.c0 <= c && c <= span.c1;
            });
            return selected.map(function (_b) {
                var data = _b.data;
                return data;
            });
        };
        Container.prototype.row = function (r) {
            var selected = this._items.filter(function (_b) {
                var span = _b.span;
                return span.r0 <= r && r <= span.r1;
            });
            return selected.map(function (_b) {
                var data = _b.data;
                return data;
            });
        };
        Container.prototype.col = function (c) {
            var selected = this._items.filter(function (_b) {
                var span = _b.span;
                return span.c0 <= c && c <= span.c1;
            });
            return selected.map(function (_b) {
                var data = _b.data;
                return data;
            });
        };
        Container.prototype.foreach = function (fn) {
            var e_1, _b;
            try {
                for (var _c = tslib_1.__values(this._items), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = _d.value, span = _e.span, data = _e.data;
                    fn(span, data);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        Container.prototype.map = function (fn) {
            var e_2, _b;
            var result = new Container();
            try {
                for (var _c = tslib_1.__values(this._items), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = _d.value, span = _e.span, data = _e.data;
                    result.add(span, fn(span, data));
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return result;
        };
        return Container;
    }());
    Container.__name__ = "Container";
    var Grid = /** @class */ (function (_super) {
        tslib_1.__extends(Grid, _super);
        function Grid(items) {
            if (items === void 0) {
                items = [];
            }
            var _this = _super.call(this) || this;
            _this.items = items;
            _this.rows = "auto";
            _this.cols = "auto";
            _this.spacing = 0;
            _this.absolute = false;
            return _this;
        }
        Grid.prototype.is_width_expanding = function () {
            if (_super.prototype.is_width_expanding.call(this))
                return true;
            if (this.sizing.width_policy == "fixed")
                return false;
            var cols = this._state.cols;
            return array_1.some(cols, function (col) { return col.policy == "max"; });
        };
        Grid.prototype.is_height_expanding = function () {
            if (_super.prototype.is_height_expanding.call(this))
                return true;
            if (this.sizing.height_policy == "fixed")
                return false;
            var rows = this._state.rows;
            return array_1.some(rows, function (row) { return row.policy == "max"; });
        };
        Grid.prototype._init = function () {
            var e_3, _b;
            var _this = this;
            _super.prototype._init.call(this);
            var items = new Container();
            try {
                for (var _c = tslib_1.__values(this.items), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = _d.value, layout = _e.layout, row = _e.row, col = _e.col, row_span = _e.row_span, col_span = _e.col_span;
                    if (layout.sizing.visible) {
                        var r0 = row;
                        var c0 = col;
                        var r1 = row + (row_span != null ? row_span : 1) - 1;
                        var c1 = col + (col_span != null ? col_span : 1) - 1;
                        items.add({ r0: r0, c0: c0, r1: r1, c1: c1 }, layout);
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            var nrows = items.nrows, ncols = items.ncols;
            var rows = new Array(nrows);
            var _loop_1 = function (y) {
                var row = (function () {
                    var sizing = types_2.isPlainObject(_this.rows) ? _this.rows[y] || _this.rows["*"] : _this.rows;
                    if (sizing == null)
                        return { policy: "auto" };
                    else if (types_2.isNumber(sizing))
                        return { policy: "fixed", height: sizing };
                    else if (types_2.isString(sizing))
                        return { policy: sizing };
                    else
                        return sizing;
                })();
                var align = row.align || "auto";
                if (row.policy == "fixed")
                    rows[y] = { policy: "fixed", height: row.height, align: align };
                else if (row.policy == "min")
                    rows[y] = { policy: "min", align: align };
                else if (row.policy == "fit" || row.policy == "max")
                    rows[y] = { policy: row.policy, flex: row.flex || 1, align: align };
                else if (row.policy == "auto") {
                    if (array_1.some(items.row(y), function (layout) { return layout.is_height_expanding(); }))
                        rows[y] = { policy: "max", flex: 1, align: align };
                    else
                        rows[y] = { policy: "min", align: align };
                }
                else
                    throw new Error("unrechable");
            };
            for (var y = 0; y < nrows; y++) {
                _loop_1(y);
            }
            var cols = new Array(ncols);
            var _loop_2 = function (x) {
                var col = (function () {
                    var sizing = types_2.isPlainObject(_this.cols) ? _this.cols[x] || _this.cols["*"] : _this.cols;
                    if (sizing == null)
                        return { policy: "auto" };
                    else if (types_2.isNumber(sizing))
                        return { policy: "fixed", width: sizing };
                    else if (types_2.isString(sizing))
                        return { policy: sizing };
                    else
                        return sizing;
                })();
                var align = col.align || "auto";
                if (col.policy == "fixed")
                    cols[x] = { policy: "fixed", width: col.width, align: align };
                else if (col.policy == "min")
                    cols[x] = { policy: "min", align: align };
                else if (col.policy == "fit" || col.policy == "max")
                    cols[x] = { policy: col.policy, flex: col.flex || 1, align: align };
                else if (col.policy == "auto") {
                    if (array_1.some(items.col(x), function (layout) { return layout.is_width_expanding(); }))
                        cols[x] = { policy: "max", flex: 1, align: align };
                    else
                        cols[x] = { policy: "min", align: align };
                }
                else
                    throw new Error("unrechable");
            };
            for (var x = 0; x < ncols; x++) {
                _loop_2(x);
            }
            var _f = tslib_1.__read(types_2.isNumber(this.spacing) ? [this.spacing, this.spacing] : this.spacing, 2), rspacing = _f[0], cspacing = _f[1];
            this._state = { items: items, nrows: nrows, ncols: ncols, rows: rows, cols: cols, rspacing: rspacing, cspacing: cspacing };
        };
        Grid.prototype._measure_totals = function (row_heights, col_widths) {
            var _b = this._state, nrows = _b.nrows, ncols = _b.ncols, rspacing = _b.rspacing, cspacing = _b.cspacing;
            return {
                height: array_1.sum(row_heights) + (nrows - 1) * rspacing,
                width: array_1.sum(col_widths) + (ncols - 1) * cspacing,
            };
        };
        Grid.prototype._measure_cells = function (cell_viewport) {
            var _b = this._state, items = _b.items, nrows = _b.nrows, ncols = _b.ncols, rows = _b.rows, cols = _b.cols, rspacing = _b.rspacing, cspacing = _b.cspacing;
            var row_heights = new Array(nrows);
            for (var r = 0; r < nrows; r++) {
                var row = rows[r];
                row_heights[r] = row.policy == "fixed" ? row.height : 0;
            }
            var col_widths = new Array(ncols);
            for (var c = 0; c < ncols; c++) {
                var col = cols[c];
                col_widths[c] = col.policy == "fixed" ? col.width : 0;
            }
            var size_hints = new Container();
            items.foreach(function (span, layout) {
                var e_4, _b, e_5, _c;
                var r0 = span.r0, c0 = span.c0, r1 = span.r1, c1 = span.c1;
                var rspace = (r1 - r0) * rspacing;
                var cspace = (c1 - c0) * cspacing;
                var height = 0;
                for (var r = r0; r <= r1; r++) {
                    height += cell_viewport(r, c0).height;
                }
                height += rspace;
                var width = 0;
                for (var c = c0; c <= c1; c++) {
                    width += cell_viewport(r0, c).width;
                }
                width += cspace;
                var size_hint = layout.measure({ width: width, height: height });
                size_hints.add(span, { layout: layout, size_hint: size_hint });
                var size = new types_1.Sizeable(size_hint).grow_by(layout.sizing.margin);
                size.height -= rspace;
                size.width -= cspace;
                var radjustable = [];
                for (var r = r0; r <= r1; r++) {
                    var row = rows[r];
                    if (row.policy == "fixed")
                        size.height -= row.height;
                    else
                        radjustable.push(r);
                }
                if (size.height > 0) {
                    var rheight = round(size.height / radjustable.length);
                    try {
                        for (var radjustable_1 = tslib_1.__values(radjustable), radjustable_1_1 = radjustable_1.next(); !radjustable_1_1.done; radjustable_1_1 = radjustable_1.next()) {
                            var r = radjustable_1_1.value;
                            row_heights[r] = max(row_heights[r], rheight);
                        }
                    }
                    catch (e_4_1) {
                        e_4 = { error: e_4_1 };
                    }
                    finally {
                        try {
                            if (radjustable_1_1 && !radjustable_1_1.done && (_b = radjustable_1.return))
                                _b.call(radjustable_1);
                        }
                        finally {
                            if (e_4)
                                throw e_4.error;
                        }
                    }
                }
                var cadjustable = [];
                for (var c = c0; c <= c1; c++) {
                    var col = cols[c];
                    if (col.policy == "fixed")
                        size.width -= col.width;
                    else
                        cadjustable.push(c);
                }
                if (size.width > 0) {
                    var cwidth = round(size.width / cadjustable.length);
                    try {
                        for (var cadjustable_1 = tslib_1.__values(cadjustable), cadjustable_1_1 = cadjustable_1.next(); !cadjustable_1_1.done; cadjustable_1_1 = cadjustable_1.next()) {
                            var c = cadjustable_1_1.value;
                            col_widths[c] = max(col_widths[c], cwidth);
                        }
                    }
                    catch (e_5_1) {
                        e_5 = { error: e_5_1 };
                    }
                    finally {
                        try {
                            if (cadjustable_1_1 && !cadjustable_1_1.done && (_c = cadjustable_1.return))
                                _c.call(cadjustable_1);
                        }
                        finally {
                            if (e_5)
                                throw e_5.error;
                        }
                    }
                }
            });
            var size = this._measure_totals(row_heights, col_widths);
            return { size: size, row_heights: row_heights, col_widths: col_widths, size_hints: size_hints };
        };
        Grid.prototype._measure_grid = function (viewport) {
            var _b = this._state, nrows = _b.nrows, ncols = _b.ncols, rows = _b.rows, cols = _b.cols, rspacing = _b.rspacing, cspacing = _b.cspacing;
            var preferred = this._measure_cells(function (y, x) {
                var row = rows[y];
                var col = cols[x];
                return {
                    width: col.policy == "fixed" ? col.width : Infinity,
                    height: row.policy == "fixed" ? row.height : Infinity,
                };
            });
            var available_height;
            if (this.sizing.height_policy == "fixed" && this.sizing.height != null)
                available_height = this.sizing.height;
            else if (viewport.height != Infinity && this.is_height_expanding())
                available_height = viewport.height;
            else
                available_height = preferred.size.height;
            var height_flex = 0;
            for (var y = 0; y < nrows; y++) {
                var row = rows[y];
                if (row.policy == "fit" || row.policy == "max")
                    height_flex += row.flex;
                else
                    available_height -= preferred.row_heights[y];
            }
            available_height -= (nrows - 1) * rspacing;
            if (height_flex != 0 && available_height > 0) {
                for (var y = 0; y < nrows; y++) {
                    var row = rows[y];
                    if (row.policy == "fit" || row.policy == "max") {
                        var height = round(available_height * (row.flex / height_flex));
                        available_height -= height;
                        preferred.row_heights[y] = height;
                        height_flex -= row.flex;
                    }
                }
            }
            else if (available_height < 0) {
                var nadjustable = 0;
                for (var y = 0; y < nrows; y++) {
                    var row = rows[y];
                    if (row.policy != "fixed")
                        nadjustable++;
                }
                var overflow_height = -available_height;
                for (var y = 0; y < nrows; y++) {
                    var row = rows[y];
                    if (row.policy != "fixed") {
                        var height = preferred.row_heights[y];
                        var cutoff = round(overflow_height / nadjustable);
                        preferred.row_heights[y] = max(height - cutoff, 0);
                        overflow_height -= cutoff > height ? height : cutoff;
                        nadjustable--;
                    }
                }
            }
            var available_width;
            if (this.sizing.width_policy == "fixed" && this.sizing.width != null)
                available_width = this.sizing.width;
            else if (viewport.width != Infinity && this.is_width_expanding())
                available_width = viewport.width;
            else
                available_width = preferred.size.width;
            var width_flex = 0;
            for (var x = 0; x < ncols; x++) {
                var col = cols[x];
                if (col.policy == "fit" || col.policy == "max")
                    width_flex += col.flex;
                else
                    available_width -= preferred.col_widths[x];
            }
            available_width -= (ncols - 1) * cspacing;
            if (width_flex != 0 && available_width > 0) {
                for (var x = 0; x < ncols; x++) {
                    var col = cols[x];
                    if (col.policy == "fit" || col.policy == "max") {
                        var width = round(available_width * (col.flex / width_flex));
                        available_width -= width;
                        preferred.col_widths[x] = width;
                        width_flex -= col.flex;
                    }
                }
            }
            else if (available_width < 0) {
                var nadjustable = 0;
                for (var x = 0; x < ncols; x++) {
                    var col = cols[x];
                    if (col.policy != "fixed")
                        nadjustable++;
                }
                var overflow_width = -available_width;
                for (var x = 0; x < ncols; x++) {
                    var col = cols[x];
                    if (col.policy != "fixed") {
                        var width = preferred.col_widths[x];
                        var cutoff = round(overflow_width / nadjustable);
                        preferred.col_widths[x] = max(width - cutoff, 0);
                        overflow_width -= cutoff > width ? width : cutoff;
                        nadjustable--;
                    }
                }
            }
            var _c = this._measure_cells(function (y, x) {
                return {
                    width: preferred.col_widths[x],
                    height: preferred.row_heights[y],
                };
            }), row_heights = _c.row_heights, col_widths = _c.col_widths, size_hints = _c.size_hints;
            var size = this._measure_totals(row_heights, col_widths);
            return { size: size, row_heights: row_heights, col_widths: col_widths, size_hints: size_hints };
        };
        Grid.prototype._measure = function (viewport) {
            var size = this._measure_grid(viewport).size;
            return size;
        };
        Grid.prototype._set_geometry = function (outer, inner) {
            _super.prototype._set_geometry.call(this, outer, inner);
            var _b = this._state, nrows = _b.nrows, ncols = _b.ncols, rspacing = _b.rspacing, cspacing = _b.cspacing;
            var _c = this._measure_grid(outer), row_heights = _c.row_heights, col_widths = _c.col_widths, size_hints = _c.size_hints;
            var rows = this._state.rows.map(function (row, r) {
                return Object.assign(Object.assign({}, row), { top: 0, height: row_heights[r], get bottom() { return this.top + this.height; } });
            });
            var cols = this._state.cols.map(function (col, c) {
                return Object.assign(Object.assign({}, col), { left: 0, width: col_widths[c], get right() { return this.left + this.width; } });
            });
            var items = size_hints.map(function (_, item) {
                return Object.assign(Object.assign({}, item), { outer: new bbox_1.BBox(), inner: new bbox_1.BBox() });
            });
            for (var r = 0, top = !this.absolute ? 0 : outer.top; r < nrows; r++) {
                var row = rows[r];
                row.top = top;
                top += row.height + rspacing;
            }
            for (var c = 0, left = !this.absolute ? 0 : outer.left; c < ncols; c++) {
                var col = cols[c];
                col.left = left;
                left += col.width + cspacing;
            }
            function span_width(c0, c1) {
                var width = (c1 - c0) * cspacing;
                for (var c = c0; c <= c1; c++) {
                    width += cols[c].width;
                }
                return width;
            }
            function span_height(r0, r1) {
                var height = (r1 - r0) * rspacing;
                for (var r = r0; r <= r1; r++) {
                    height += rows[r].height;
                }
                return height;
            }
            items.foreach(function (_b, item) {
                var r0 = _b.r0, c0 = _b.c0, r1 = _b.r1, c1 = _b.c1;
                var layout = item.layout, size_hint = item.size_hint;
                var sizing = layout.sizing;
                var width = size_hint.width, height = size_hint.height;
                var span = {
                    width: span_width(c0, c1),
                    height: span_height(r0, r1),
                };
                var halign = c0 == c1 && cols[c0].align != "auto" ? cols[c0].align : sizing.halign;
                var valign = r0 == r1 && rows[r0].align != "auto" ? rows[r0].align : sizing.valign;
                var left = cols[c0].left;
                if (halign == "start")
                    left += sizing.margin.left;
                else if (halign == "center")
                    left += round((span.width - width) / 2);
                else if (halign == "end")
                    left += span.width - sizing.margin.right - width;
                var top = rows[r0].top;
                if (valign == "start")
                    top += sizing.margin.top;
                else if (valign == "center")
                    top += round((span.height - height) / 2);
                else if (valign == "end")
                    top += span.height - sizing.margin.bottom - height;
                item.outer = new bbox_1.BBox({ left: left, top: top, width: width, height: height });
            });
            var row_aligns = rows.map(function () {
                return {
                    start: new DefaultMap(function () { return 0; }),
                    end: new DefaultMap(function () { return 0; }),
                };
            });
            var col_aligns = cols.map(function () {
                return {
                    start: new DefaultMap(function () { return 0; }),
                    end: new DefaultMap(function () { return 0; }),
                };
            });
            items.foreach(function (_b, _c) {
                var r0 = _b.r0, c0 = _b.c0, r1 = _b.r1, c1 = _b.c1;
                var size_hint = _c.size_hint, outer = _c.outer;
                var inner = size_hint.inner;
                if (inner != null) {
                    row_aligns[r0].start.apply(outer.top, function (v) { return max(v, inner.top); });
                    row_aligns[r1].end.apply(rows[r1].bottom - outer.bottom, function (v) { return max(v, inner.bottom); });
                    col_aligns[c0].start.apply(outer.left, function (v) { return max(v, inner.left); });
                    col_aligns[c1].end.apply(cols[c1].right - outer.right, function (v) { return max(v, inner.right); });
                }
            });
            items.foreach(function (_b, item) {
                var r0 = _b.r0, c0 = _b.c0, r1 = _b.r1, c1 = _b.c1;
                var size_hint = item.size_hint, outer = item.outer;
                function inner_bbox(_b) {
                    var left = _b.left, right = _b.right, top = _b.top, bottom = _b.bottom;
                    var width = outer.width - left - right;
                    var height = outer.height - top - bottom;
                    return new bbox_1.BBox({ left: left, top: top, width: width, height: height });
                }
                if (size_hint.inner != null) {
                    var inner_1 = inner_bbox(size_hint.inner);
                    if (size_hint.align !== false) {
                        var top = row_aligns[r0].start.get(outer.top);
                        var bottom = row_aligns[r1].end.get(rows[r1].bottom - outer.bottom);
                        var left = col_aligns[c0].start.get(outer.left);
                        var right = col_aligns[c1].end.get(cols[c1].right - outer.right);
                        try {
                            inner_1 = inner_bbox({ top: top, bottom: bottom, left: left, right: right });
                        }
                        catch (_a) { }
                    }
                    item.inner = inner_1;
                }
                else
                    item.inner = outer;
            });
            items.foreach(function (_, _b) {
                var layout = _b.layout, outer = _b.outer, inner = _b.inner;
                layout.set_geometry(outer, inner);
            });
        };
        return Grid;
    }(layoutable_1.Layoutable));
    exports.Grid = Grid;
    Grid.__name__ = "Grid";
    var Row = /** @class */ (function (_super) {
        tslib_1.__extends(Row, _super);
        function Row(items) {
            var _this = _super.call(this) || this;
            _this.items = items.map(function (item, i) { return ({ layout: item, row: 0, col: i }); });
            _this.rows = "fit";
            return _this;
        }
        return Row;
    }(Grid));
    exports.Row = Row;
    Row.__name__ = "Row";
    var Column = /** @class */ (function (_super) {
        tslib_1.__extends(Column, _super);
        function Column(items) {
            var _this = _super.call(this) || this;
            _this.items = items.map(function (item, i) { return ({ layout: item, row: i, col: 0 }); });
            _this.cols = "fit";
            return _this;
        }
        return Column;
    }(Grid));
    exports.Column = Column;
    Column.__name__ = "Column";
},
/* core/layout/html.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var layoutable_1 = require(329) /* ./layoutable */;
    var types_1 = require(328) /* ./types */;
    var dom_1 = require(187) /* ../dom */;
    var ContentBox = /** @class */ (function (_super) {
        tslib_1.__extends(ContentBox, _super);
        function ContentBox(el) {
            var _this = _super.call(this) || this;
            _this.content_size = dom_1.unsized(el, function () { return new types_1.Sizeable(dom_1.size(el)); });
            return _this;
        }
        ContentBox.prototype._content_size = function () {
            return this.content_size;
        };
        return ContentBox;
    }(layoutable_1.ContentLayoutable));
    exports.ContentBox = ContentBox;
    ContentBox.__name__ = "ContentBox";
    var VariadicBox = /** @class */ (function (_super) {
        tslib_1.__extends(VariadicBox, _super);
        function VariadicBox(el) {
            var _this = _super.call(this) || this;
            _this.el = el;
            return _this;
        }
        VariadicBox.prototype._measure = function (viewport) {
            var _this = this;
            var bounded = new types_1.Sizeable(viewport).bounded_to(this.sizing.size);
            return dom_1.sized(this.el, bounded, function () {
                var content = new types_1.Sizeable(dom_1.content_size(_this.el));
                var _a = dom_1.extents(_this.el), border = _a.border, padding = _a.padding;
                return content.grow_by(border).grow_by(padding).map(Math.ceil);
            });
        };
        return VariadicBox;
    }(layoutable_1.Layoutable));
    exports.VariadicBox = VariadicBox;
    VariadicBox.__name__ = "VariadicBox";
    var CachedVariadicBox = /** @class */ (function (_super) {
        tslib_1.__extends(CachedVariadicBox, _super);
        function CachedVariadicBox(el) {
            var _this = _super.call(this, el) || this;
            _this._cache = new Map();
            return _this;
        }
        CachedVariadicBox.prototype._measure = function (viewport) {
            var width = viewport.width, height = viewport.height;
            var key = width + "," + height;
            var size_hint = this._cache.get(key);
            if (size_hint == null) {
                size_hint = _super.prototype._measure.call(this, viewport);
                this._cache.set(key, size_hint);
            }
            return size_hint;
        };
        CachedVariadicBox.prototype.invalidate_cache = function () {
            this._cache.clear();
        };
        return CachedVariadicBox;
    }(VariadicBox));
    exports.CachedVariadicBox = CachedVariadicBox;
    CachedVariadicBox.__name__ = "CachedVariadicBox";
},
/* models/expressions/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var expression_1 = require(334) /* ./expression */;
    exports.Expression = expression_1.Expression;
    var stack_1 = require(335) /* ./stack */;
    exports.Stack = stack_1.Stack;
    var cumsum_1 = require(336) /* ./cumsum */;
    exports.CumSum = cumsum_1.CumSum;
},
/* models/expressions/expression.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var Expression = /** @class */ (function (_super) {
        tslib_1.__extends(Expression, _super);
        function Expression(attrs) {
            return _super.call(this, attrs) || this;
        }
        Expression.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._connected = new Set();
            this._result = new Map();
        };
        Expression.prototype.v_compute = function (source) {
            var _this = this;
            if (!this._connected.has(source)) {
                this.connect(source.change, function () { return _this._result.delete(source); });
                this.connect(source.patching, function () { return _this._result.delete(source); });
                this.connect(source.streaming, function () { return _this._result.delete(source); });
                this._connected.add(source);
            }
            var result = this._result.get(source);
            if (result == null) {
                result = this._v_compute(source);
                this._result.set(source, result);
            }
            return result;
        };
        return Expression;
    }(model_1.Model));
    exports.Expression = Expression;
    Expression.__name__ = "Expression";
},
/* models/expressions/stack.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var expression_1 = require(334) /* ./expression */;
    var types_1 = require(139) /* ../../core/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var Stack = /** @class */ (function (_super) {
        tslib_1.__extends(Stack, _super);
        function Stack(attrs) {
            return _super.call(this, attrs) || this;
        }
        Stack.init_Stack = function () {
            this.define({
                fields: [p.Array, []],
            });
        };
        Stack.prototype._v_compute = function (source) {
            var e_1, _b;
            var _a;
            var n = (_a = source.get_length()) !== null && _a !== void 0 ? _a : 0;
            var result = new types_1.NumberArray(n);
            try {
                for (var _c = tslib_1.__values(this.fields), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var f = _d.value;
                    var column = source.data[f];
                    if (column != null) {
                        for (var i = 0, k = Math.min(n, column.length); i < k; i++) {
                            result[i] += column[i];
                        }
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return result;
        };
        return Stack;
    }(expression_1.Expression));
    exports.Stack = Stack;
    Stack.__name__ = "Stack";
    Stack.init_Stack();
},
/* models/expressions/cumsum.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var expression_1 = require(334) /* ./expression */;
    var types_1 = require(139) /* ../../core/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var CumSum = /** @class */ (function (_super) {
        tslib_1.__extends(CumSum, _super);
        function CumSum(attrs) {
            return _super.call(this, attrs) || this;
        }
        CumSum.init_CumSum = function () {
            this.define({
                field: [p.String],
                include_zero: [p.Boolean, false],
            });
        };
        CumSum.prototype._v_compute = function (source) {
            var result = new types_1.NumberArray(source.get_length() || 0);
            var col = source.data[this.field];
            var offset = this.include_zero ? 1 : 0;
            result[0] = this.include_zero ? 0 : col[0];
            for (var i = 1; i < result.length; i++) {
                result[i] = result[i - 1] + col[i - offset];
            }
            return result;
        };
        return CumSum;
    }(expression_1.Expression));
    exports.CumSum = CumSum;
    CumSum.__name__ = "CumSum";
    CumSum.init_CumSum();
},
/* models/filters/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var boolean_filter_1 = require(338) /* ./boolean_filter */;
    exports.BooleanFilter = boolean_filter_1.BooleanFilter;
    var customjs_filter_1 = require(340) /* ./customjs_filter */;
    exports.CustomJSFilter = customjs_filter_1.CustomJSFilter;
    var filter_1 = require(339) /* ./filter */;
    exports.Filter = filter_1.Filter;
    var group_filter_1 = require(341) /* ./group_filter */;
    exports.GroupFilter = group_filter_1.GroupFilter;
    var index_filter_1 = require(342) /* ./index_filter */;
    exports.IndexFilter = index_filter_1.IndexFilter;
},
/* models/filters/boolean_filter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var filter_1 = require(339) /* ./filter */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(139) /* ../../core/types */;
    var BooleanFilter = /** @class */ (function (_super) {
        tslib_1.__extends(BooleanFilter, _super);
        function BooleanFilter(attrs) {
            return _super.call(this, attrs) || this;
        }
        BooleanFilter.init_BooleanFilter = function () {
            this.define({
                booleans: [p.Array, null],
            });
        };
        BooleanFilter.prototype.compute_indices = function (source) {
            var size = source.length;
            var booleans = this.booleans;
            if (booleans == null) {
                return types_1.Indices.all_set(size);
            }
            else {
                return types_1.Indices.from_booleans(size, booleans);
            }
        };
        return BooleanFilter;
    }(filter_1.Filter));
    exports.BooleanFilter = BooleanFilter;
    BooleanFilter.__name__ = "BooleanFilter";
    BooleanFilter.init_BooleanFilter();
},
/* models/filters/filter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var Filter = /** @class */ (function (_super) {
        tslib_1.__extends(Filter, _super);
        function Filter(attrs) {
            return _super.call(this, attrs) || this;
        }
        return Filter;
    }(model_1.Model));
    exports.Filter = Filter;
    Filter.__name__ = "Filter";
},
/* models/filters/customjs_filter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var filter_1 = require(339) /* ./filter */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(139) /* ../../core/types */;
    var object_1 = require(128) /* ../../core/util/object */;
    var types_2 = require(123) /* ../../core/util/types */;
    var string_1 = require(144) /* ../../core/util/string */;
    var CustomJSFilter = /** @class */ (function (_super) {
        tslib_1.__extends(CustomJSFilter, _super);
        function CustomJSFilter(attrs) {
            return _super.call(this, attrs) || this;
        }
        CustomJSFilter.init_CustomJSFilter = function () {
            this.define({
                args: [p.Any, {}],
                code: [p.String, ''],
            });
        };
        Object.defineProperty(CustomJSFilter.prototype, "names", {
            get: function () {
                return object_1.keys(this.args);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CustomJSFilter.prototype, "values", {
            get: function () {
                return object_1.values(this.args);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CustomJSFilter.prototype, "func", {
            get: function () {
                var code = string_1.use_strict(this.code);
                return new (Function.bind.apply(Function, tslib_1.__spread([void 0], this.names, ["source", code])))();
            },
            enumerable: true,
            configurable: true
        });
        CustomJSFilter.prototype.compute_indices = function (source) {
            var size = source.length;
            var filter = this.func.apply(this, tslib_1.__spread(this.values, [source]));
            if (filter == null)
                return types_1.Indices.all_set(size);
            else if (types_2.isArrayOf(filter, types_2.isInteger))
                return types_1.Indices.from_indices(size, filter);
            else if (types_2.isArrayOf(filter, types_2.isBoolean))
                return types_1.Indices.from_booleans(size, filter);
            else
                throw new Error("expect an array of integers or booleans, or null, got " + filter);
        };
        return CustomJSFilter;
    }(filter_1.Filter));
    exports.CustomJSFilter = CustomJSFilter;
    CustomJSFilter.__name__ = "CustomJSFilter";
    CustomJSFilter.init_CustomJSFilter();
},
/* models/filters/group_filter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var filter_1 = require(339) /* ./filter */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(139) /* ../../core/types */;
    var logging_1 = require(134) /* ../../core/logging */;
    var GroupFilter = /** @class */ (function (_super) {
        tslib_1.__extends(GroupFilter, _super);
        function GroupFilter(attrs) {
            return _super.call(this, attrs) || this;
        }
        GroupFilter.init_GroupFilter = function () {
            this.define({
                column_name: [p.String],
                group: [p.String],
            });
        };
        GroupFilter.prototype.compute_indices = function (source) {
            var column = source.get_column(this.column_name);
            if (column == null) {
                logging_1.logger.warn(this + ": groupby column '" + this.column_name + "' not found in the data source");
                return new types_1.Indices(source.length, 1);
            }
            else {
                var indices = new types_1.Indices(source.length);
                for (var i = 0; i < indices.size; i++) {
                    if (column[i] === this.group)
                        indices.set(i);
                }
                return indices;
            }
        };
        return GroupFilter;
    }(filter_1.Filter));
    exports.GroupFilter = GroupFilter;
    GroupFilter.__name__ = "GroupFilter";
    GroupFilter.init_GroupFilter();
},
/* models/filters/index_filter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var filter_1 = require(339) /* ./filter */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(139) /* ../../core/types */;
    var IndexFilter = /** @class */ (function (_super) {
        tslib_1.__extends(IndexFilter, _super);
        function IndexFilter(attrs) {
            return _super.call(this, attrs) || this;
        }
        IndexFilter.init_IndexFilter = function () {
            this.define({
                indices: [p.Array, null],
            });
        };
        IndexFilter.prototype.compute_indices = function (source) {
            var size = source.length;
            var indices = this.indices;
            if (indices == null) {
                return types_1.Indices.all_set(size);
            }
            else {
                return types_1.Indices.from_indices(size, indices);
            }
        };
        return IndexFilter;
    }(filter_1.Filter));
    exports.IndexFilter = IndexFilter;
    IndexFilter.__name__ = "IndexFilter";
    IndexFilter.init_IndexFilter();
},
/* models/formatters/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var basic_tick_formatter_1 = require(245) /* ./basic_tick_formatter */;
    exports.BasicTickFormatter = basic_tick_formatter_1.BasicTickFormatter;
    var categorical_tick_formatter_1 = require(296) /* ./categorical_tick_formatter */;
    exports.CategoricalTickFormatter = categorical_tick_formatter_1.CategoricalTickFormatter;
    var datetime_tick_formatter_1 = require(300) /* ./datetime_tick_formatter */;
    exports.DatetimeTickFormatter = datetime_tick_formatter_1.DatetimeTickFormatter;
    var func_tick_formatter_1 = require(344) /* ./func_tick_formatter */;
    exports.FuncTickFormatter = func_tick_formatter_1.FuncTickFormatter;
    var log_tick_formatter_1 = require(313) /* ./log_tick_formatter */;
    exports.LogTickFormatter = log_tick_formatter_1.LogTickFormatter;
    var mercator_tick_formatter_1 = require(316) /* ./mercator_tick_formatter */;
    exports.MercatorTickFormatter = mercator_tick_formatter_1.MercatorTickFormatter;
    var numeral_tick_formatter_1 = require(345) /* ./numeral_tick_formatter */;
    exports.NumeralTickFormatter = numeral_tick_formatter_1.NumeralTickFormatter;
    var printf_tick_formatter_1 = require(346) /* ./printf_tick_formatter */;
    exports.PrintfTickFormatter = printf_tick_formatter_1.PrintfTickFormatter;
    var tick_formatter_1 = require(246) /* ./tick_formatter */;
    exports.TickFormatter = tick_formatter_1.TickFormatter;
},
/* models/formatters/func_tick_formatter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var tick_formatter_1 = require(246) /* ./tick_formatter */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var object_1 = require(128) /* ../../core/util/object */;
    var string_1 = require(144) /* ../../core/util/string */;
    var FuncTickFormatter = /** @class */ (function (_super) {
        tslib_1.__extends(FuncTickFormatter, _super);
        function FuncTickFormatter(attrs) {
            return _super.call(this, attrs) || this;
        }
        FuncTickFormatter.init_FuncTickFormatter = function () {
            this.define({
                args: [p.Any, {}],
                code: [p.String, ''],
            });
        };
        Object.defineProperty(FuncTickFormatter.prototype, "names", {
            get: function () {
                return object_1.keys(this.args);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(FuncTickFormatter.prototype, "values", {
            get: function () {
                return object_1.values(this.args);
            },
            enumerable: true,
            configurable: true
        });
        /*protected*/ FuncTickFormatter.prototype._make_func = function () {
            var code = string_1.use_strict(this.code);
            return new (Function.bind.apply(Function, tslib_1.__spread([void 0, "tick", "index", "ticks"], this.names, [code])))();
        };
        FuncTickFormatter.prototype.doFormat = function (ticks, _opts) {
            var _this = this;
            var cache = {};
            var func = this._make_func().bind(cache);
            return ticks.map(function (tick, index, ticks) { return func.apply(void 0, tslib_1.__spread([tick, index, ticks], _this.values)); });
        };
        return FuncTickFormatter;
    }(tick_formatter_1.TickFormatter));
    exports.FuncTickFormatter = FuncTickFormatter;
    FuncTickFormatter.__name__ = "FuncTickFormatter";
    FuncTickFormatter.init_FuncTickFormatter();
},
/* models/formatters/numeral_tick_formatter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var Numbro = tslib_1.__importStar(require(303) /* @bokeh/numbro */);
    var tick_formatter_1 = require(246) /* ./tick_formatter */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var NumeralTickFormatter = /** @class */ (function (_super) {
        tslib_1.__extends(NumeralTickFormatter, _super);
        function NumeralTickFormatter(attrs) {
            return _super.call(this, attrs) || this;
        }
        NumeralTickFormatter.init_NumeralTickFormatter = function () {
            this.define({
                // TODO (bev) all of these could be tightened up
                format: [p.String, '0,0'],
                language: [p.String, 'en'],
                rounding: [p.RoundingFunction, 'round'],
            });
        };
        Object.defineProperty(NumeralTickFormatter.prototype, "_rounding_fn", {
            get: function () {
                switch (this.rounding) {
                    case "round":
                    case "nearest":
                        return Math.round;
                    case "floor":
                    case "rounddown":
                        return Math.floor;
                    case "ceil":
                    case "roundup":
                        return Math.ceil;
                }
            },
            enumerable: true,
            configurable: true
        });
        NumeralTickFormatter.prototype.doFormat = function (ticks, _opts) {
            var _a = this, format = _a.format, language = _a.language, _rounding_fn = _a._rounding_fn;
            return ticks.map(function (tick) { return Numbro.format(tick, format, language, _rounding_fn); });
        };
        return NumeralTickFormatter;
    }(tick_formatter_1.TickFormatter));
    exports.NumeralTickFormatter = NumeralTickFormatter;
    NumeralTickFormatter.__name__ = "NumeralTickFormatter";
    NumeralTickFormatter.init_NumeralTickFormatter();
},
/* models/formatters/printf_tick_formatter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var tick_formatter_1 = require(246) /* ./tick_formatter */;
    var templating_1 = require(302) /* ../../core/util/templating */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var PrintfTickFormatter = /** @class */ (function (_super) {
        tslib_1.__extends(PrintfTickFormatter, _super);
        function PrintfTickFormatter(attrs) {
            return _super.call(this, attrs) || this;
        }
        PrintfTickFormatter.init_PrintfTickFormatter = function () {
            this.define({
                format: [p.String, '%s'],
            });
        };
        PrintfTickFormatter.prototype.doFormat = function (ticks, _opts) {
            var _this = this;
            return ticks.map(function (tick) { return templating_1.sprintf(_this.format, tick); });
        };
        return PrintfTickFormatter;
    }(tick_formatter_1.TickFormatter));
    exports.PrintfTickFormatter = PrintfTickFormatter;
    PrintfTickFormatter.__name__ = "PrintfTickFormatter";
    PrintfTickFormatter.init_PrintfTickFormatter();
},
/* models/glyphs/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var annular_wedge_1 = require(348) /* ./annular_wedge */;
    exports.AnnularWedge = annular_wedge_1.AnnularWedge;
    var annulus_1 = require(349) /* ./annulus */;
    exports.Annulus = annulus_1.Annulus;
    var arc_1 = require(350) /* ./arc */;
    exports.Arc = arc_1.Arc;
    var bezier_1 = require(351) /* ./bezier */;
    exports.Bezier = bezier_1.Bezier;
    var circle_1 = require(352) /* ./circle */;
    exports.Circle = circle_1.Circle;
    var center_rotatable_1 = require(356) /* ./center_rotatable */;
    exports.CenterRotatable = center_rotatable_1.CenterRotatable;
    var ellipse_1 = require(357) /* ./ellipse */;
    exports.Ellipse = ellipse_1.Ellipse;
    var ellipse_oval_1 = require(358) /* ./ellipse_oval */;
    exports.EllipseOval = ellipse_oval_1.EllipseOval;
    var glyph_1 = require(209) /* ./glyph */;
    exports.Glyph = glyph_1.Glyph;
    var harea_1 = require(226) /* ./harea */;
    exports.HArea = harea_1.HArea;
    var hbar_1 = require(359) /* ./hbar */;
    exports.HBar = hbar_1.HBar;
    var hex_tile_1 = require(361) /* ./hex_tile */;
    exports.HexTile = hex_tile_1.HexTile;
    var image_1 = require(362) /* ./image */;
    exports.Image = image_1.Image;
    var image_rgba_1 = require(364) /* ./image_rgba */;
    exports.ImageRGBA = image_rgba_1.ImageRGBA;
    var image_url_1 = require(365) /* ./image_url */;
    exports.ImageURL = image_url_1.ImageURL;
    var line_1 = require(207) /* ./line */;
    exports.Line = line_1.Line;
    var multi_line_1 = require(367) /* ./multi_line */;
    exports.MultiLine = multi_line_1.MultiLine;
    var multi_polygons_1 = require(368) /* ./multi_polygons */;
    exports.MultiPolygons = multi_polygons_1.MultiPolygons;
    var oval_1 = require(369) /* ./oval */;
    exports.Oval = oval_1.Oval;
    var patch_1 = require(225) /* ./patch */;
    exports.Patch = patch_1.Patch;
    var patches_1 = require(370) /* ./patches */;
    exports.Patches = patches_1.Patches;
    var quad_1 = require(371) /* ./quad */;
    exports.Quad = quad_1.Quad;
    var quadratic_1 = require(372) /* ./quadratic */;
    exports.Quadratic = quadratic_1.Quadratic;
    var ray_1 = require(373) /* ./ray */;
    exports.Ray = ray_1.Ray;
    var rect_1 = require(374) /* ./rect */;
    exports.Rect = rect_1.Rect;
    var segment_1 = require(375) /* ./segment */;
    exports.Segment = segment_1.Segment;
    var step_1 = require(376) /* ./step */;
    exports.Step = step_1.Step;
    var text_1 = require(377) /* ./text */;
    exports.Text = text_1.Text;
    var varea_1 = require(228) /* ./varea */;
    exports.VArea = varea_1.VArea;
    var vbar_1 = require(378) /* ./vbar */;
    exports.VBar = vbar_1.VBar;
    var wedge_1 = require(379) /* ./wedge */;
    exports.Wedge = wedge_1.Wedge;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    exports.XYGlyph = xy_glyph_1.XYGlyph;
},
/* models/glyphs/annular_wedge.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var utils_1 = require(215) /* ./utils */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var types_1 = require(139) /* ../../core/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var math_1 = require(125) /* ../../core/util/math */;
    var selection_1 = require(203) /* ../selections/selection */;
    var AnnularWedgeView = /** @class */ (function (_super) {
        tslib_1.__extends(AnnularWedgeView, _super);
        function AnnularWedgeView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        AnnularWedgeView.prototype._map_data = function () {
            if (this.model.properties.inner_radius.units == "data")
                this.sinner_radius = this.sdist(this.renderer.xscale, this._x, this._inner_radius);
            else
                this.sinner_radius = this._inner_radius;
            if (this.model.properties.outer_radius.units == "data")
                this.souter_radius = this.sdist(this.renderer.xscale, this._x, this._outer_radius);
            else
                this.souter_radius = this._outer_radius;
            this._angle = new types_1.NumberArray(this._start_angle.length);
            for (var i = 0, end = this._start_angle.length; i < end; i++) {
                this._angle[i] = this._end_angle[i] - this._start_angle[i];
            }
        };
        AnnularWedgeView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx = _a.sx, sy = _a.sy, _start_angle = _a._start_angle, _angle = _a._angle, sinner_radius = _a.sinner_radius, souter_radius = _a.souter_radius;
            var direction = this.model.properties.direction.value();
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (isNaN(sx[i] + sy[i] + sinner_radius[i] + souter_radius[i] + _start_angle[i] + _angle[i]))
                        continue;
                    ctx.translate(sx[i], sy[i]);
                    ctx.rotate(_start_angle[i]);
                    ctx.beginPath();
                    ctx.moveTo(souter_radius[i], 0);
                    ctx.arc(0, 0, souter_radius[i], 0, _angle[i], direction);
                    ctx.rotate(_angle[i]);
                    ctx.lineTo(sinner_radius[i], 0);
                    ctx.arc(0, 0, sinner_radius[i], 0, -_angle[i], !direction);
                    ctx.closePath();
                    ctx.rotate(-_angle[i] - _start_angle[i]);
                    ctx.translate(-sx[i], -sy[i]);
                    if (this.visuals.fill.doit) {
                        this.visuals.fill.set_vectorize(ctx, i);
                        ctx.fill();
                    }
                    if (this.visuals.line.doit) {
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.stroke();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        AnnularWedgeView.prototype._hit_point = function (geometry) {
            var _a, _b, e_2, _c, e_3, _d;
            var sx = geometry.sx, sy = geometry.sy;
            var x = this.renderer.xscale.invert(sx);
            var y = this.renderer.yscale.invert(sy);
            // check radius first
            var x0, y0;
            var x1, y1;
            if (this.model.properties.outer_radius.units == "data") {
                x0 = x - this.max_outer_radius;
                x1 = x + this.max_outer_radius;
                y0 = y - this.max_outer_radius;
                y1 = y + this.max_outer_radius;
            }
            else {
                var sx0 = sx - this.max_outer_radius;
                var sx1 = sx + this.max_outer_radius;
                _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
                var sy0 = sy - this.max_outer_radius;
                var sy1 = sy + this.max_outer_radius;
                _b = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _b[0], y1 = _b[1];
            }
            var candidates = [];
            try {
                for (var _e = tslib_1.__values(this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 })), _f = _e.next(); !_f.done; _f = _e.next()) {
                    var i = _f.value;
                    var or2 = Math.pow(this.souter_radius[i], 2);
                    var ir2 = Math.pow(this.sinner_radius[i], 2);
                    var _g = tslib_1.__read(this.renderer.xscale.r_compute(x, this._x[i]), 2), sx0 = _g[0], sx1 = _g[1];
                    var _h = tslib_1.__read(this.renderer.yscale.r_compute(y, this._y[i]), 2), sy0 = _h[0], sy1 = _h[1];
                    var dist = Math.pow((sx0 - sx1), 2) + Math.pow((sy0 - sy1), 2);
                    if (dist <= or2 && dist >= ir2)
                        candidates.push(i);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_f && !_f.done && (_c = _e.return))
                        _c.call(_e);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            var direction = this.model.properties.direction.value();
            var indices = [];
            try {
                for (var candidates_1 = tslib_1.__values(candidates), candidates_1_1 = candidates_1.next(); !candidates_1_1.done; candidates_1_1 = candidates_1.next()) {
                    var i = candidates_1_1.value;
                    // NOTE: minus the angle because JS uses non-mathy convention for angles
                    var angle = Math.atan2(sy - this.sy[i], sx - this.sx[i]);
                    if (math_1.angle_between(-angle, -this._start_angle[i], -this._end_angle[i], direction)) {
                        indices.push(i);
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (candidates_1_1 && !candidates_1_1.done && (_d = candidates_1.return))
                        _d.call(candidates_1);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        AnnularWedgeView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_area_legend(this.visuals, ctx, bbox, index);
        };
        AnnularWedgeView.prototype.scenterxy = function (i) {
            var r = (this.sinner_radius[i] + this.souter_radius[i]) / 2;
            var a = (this._start_angle[i] + this._end_angle[i]) / 2;
            var scx = this.sx[i] + r * Math.cos(a);
            var scy = this.sy[i] + r * Math.sin(a);
            return [scx, scy];
        };
        return AnnularWedgeView;
    }(xy_glyph_1.XYGlyphView));
    exports.AnnularWedgeView = AnnularWedgeView;
    AnnularWedgeView.__name__ = "AnnularWedgeView";
    var AnnularWedge = /** @class */ (function (_super) {
        tslib_1.__extends(AnnularWedge, _super);
        function AnnularWedge(attrs) {
            return _super.call(this, attrs) || this;
        }
        AnnularWedge.init_AnnularWedge = function () {
            this.prototype.default_view = AnnularWedgeView;
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector]);
            this.define({
                direction: [p.Direction, 'anticlock'],
                inner_radius: [p.DistanceSpec],
                outer_radius: [p.DistanceSpec],
                start_angle: [p.AngleSpec],
                end_angle: [p.AngleSpec],
            });
        };
        return AnnularWedge;
    }(xy_glyph_1.XYGlyph));
    exports.AnnularWedge = AnnularWedge;
    AnnularWedge.__name__ = "AnnularWedge";
    AnnularWedge.init_AnnularWedge();
},
/* models/glyphs/annulus.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var compat_1 = require(147) /* ../../core/util/compat */;
    var selection_1 = require(203) /* ../selections/selection */;
    var AnnulusView = /** @class */ (function (_super) {
        tslib_1.__extends(AnnulusView, _super);
        function AnnulusView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        AnnulusView.prototype._map_data = function () {
            if (this.model.properties.inner_radius.units == "data")
                this.sinner_radius = this.sdist(this.renderer.xscale, this._x, this._inner_radius);
            else
                this.sinner_radius = this._inner_radius;
            if (this.model.properties.outer_radius.units == "data")
                this.souter_radius = this.sdist(this.renderer.xscale, this._x, this._outer_radius);
            else
                this.souter_radius = this._outer_radius;
        };
        AnnulusView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b, e_2, _c;
            var sx = _a.sx, sy = _a.sy, sinner_radius = _a.sinner_radius, souter_radius = _a.souter_radius;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (isNaN(sx[i] + sy[i] + sinner_radius[i] + souter_radius[i]))
                        continue;
                    // Because this visual has a whole in it, it proved "challenging"
                    // for some browsers to render if drawn in one go --- i.e. it did not
                    // work on IE. If we render in two parts (upper and lower part),
                    // it is unambiguous what part should be filled. The line is
                    // better drawn in one go though, otherwise the part where the pieces
                    // meet will not be fully closed due to aa.
                    if (this.visuals.fill.doit) {
                        this.visuals.fill.set_vectorize(ctx, i);
                        ctx.beginPath();
                        if (compat_1.is_ie) {
                            try {
                                // Draw two halves of the donut. Works on IE, but causes an aa line on Safari.
                                for (var _d = (e_2 = void 0, tslib_1.__values([false, true])), _e = _d.next(); !_e.done; _e = _d.next()) {
                                    var clockwise = _e.value;
                                    ctx.arc(sx[i], sy[i], sinner_radius[i], 0, Math.PI, clockwise);
                                    ctx.arc(sx[i], sy[i], souter_radius[i], Math.PI, 0, !clockwise);
                                }
                            }
                            catch (e_2_1) {
                                e_2 = { error: e_2_1 };
                            }
                            finally {
                                try {
                                    if (_e && !_e.done && (_c = _d.return))
                                        _c.call(_d);
                                }
                                finally {
                                    if (e_2)
                                        throw e_2.error;
                                }
                            }
                        }
                        else {
                            // Draw donut in one go. Does not work on iE.
                            ctx.arc(sx[i], sy[i], sinner_radius[i], 0, 2 * Math.PI, true);
                            ctx.arc(sx[i], sy[i], souter_radius[i], 2 * Math.PI, 0, false);
                        }
                        ctx.fill();
                    }
                    if (this.visuals.line.doit) {
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.beginPath();
                        ctx.arc(sx[i], sy[i], sinner_radius[i], 0, 2 * Math.PI);
                        ctx.moveTo(sx[i] + souter_radius[i], sy[i]);
                        ctx.arc(sx[i], sy[i], souter_radius[i], 0, 2 * Math.PI);
                        ctx.stroke();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        AnnulusView.prototype._hit_point = function (geometry) {
            var _a, _b, e_3, _c;
            var sx = geometry.sx, sy = geometry.sy;
            var x = this.renderer.xscale.invert(sx);
            var y = this.renderer.yscale.invert(sy);
            var x0, y0;
            var x1, y1;
            if (this.model.properties.outer_radius.units == "data") {
                x0 = x - this.max_outer_radius;
                x1 = x + this.max_outer_radius;
                y0 = y - this.max_outer_radius;
                y1 = y + this.max_outer_radius;
            }
            else {
                var sx0 = sx - this.max_outer_radius;
                var sx1 = sx + this.max_outer_radius;
                _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
                var sy0 = sy - this.max_outer_radius;
                var sy1 = sy + this.max_outer_radius;
                _b = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _b[0], y1 = _b[1];
            }
            var indices = [];
            try {
                for (var _d = tslib_1.__values(this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 })), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var i = _e.value;
                    var or2 = Math.pow(this.souter_radius[i], 2);
                    var ir2 = Math.pow(this.sinner_radius[i], 2);
                    var _f = tslib_1.__read(this.renderer.xscale.r_compute(x, this._x[i]), 2), sx0 = _f[0], sx1 = _f[1];
                    var _g = tslib_1.__read(this.renderer.yscale.r_compute(y, this._y[i]), 2), sy0 = _g[0], sy1 = _g[1];
                    var dist = Math.pow((sx0 - sx1), 2) + Math.pow((sy0 - sy1), 2);
                    if (dist <= or2 && dist >= ir2)
                        indices.push(i);
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_c = _d.return))
                        _c.call(_d);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        AnnulusView.prototype.draw_legend_for_index = function (ctx, _a, index) {
            var x0 = _a.x0, y0 = _a.y0, x1 = _a.x1, y1 = _a.y1;
            var len = index + 1;
            var sx = new Array(len);
            sx[index] = (x0 + x1) / 2;
            var sy = new Array(len);
            sy[index] = (y0 + y1) / 2;
            var r = Math.min(Math.abs(x1 - x0), Math.abs(y1 - y0)) * 0.5;
            var sinner_radius = new Array(len);
            sinner_radius[index] = r * 0.4;
            var souter_radius = new Array(len);
            souter_radius[index] = r * 0.8;
            this._render(ctx, [index], { sx: sx, sy: sy, sinner_radius: sinner_radius, souter_radius: souter_radius }); // XXX
        };
        return AnnulusView;
    }(xy_glyph_1.XYGlyphView));
    exports.AnnulusView = AnnulusView;
    AnnulusView.__name__ = "AnnulusView";
    var Annulus = /** @class */ (function (_super) {
        tslib_1.__extends(Annulus, _super);
        function Annulus(attrs) {
            return _super.call(this, attrs) || this;
        }
        Annulus.init_Annulus = function () {
            this.prototype.default_view = AnnulusView;
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector]);
            this.define({
                inner_radius: [p.DistanceSpec],
                outer_radius: [p.DistanceSpec],
            });
        };
        return Annulus;
    }(xy_glyph_1.XYGlyph));
    exports.Annulus = Annulus;
    Annulus.__name__ = "Annulus";
    Annulus.init_Annulus();
},
/* models/glyphs/arc.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var utils_1 = require(215) /* ./utils */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var ArcView = /** @class */ (function (_super) {
        tslib_1.__extends(ArcView, _super);
        function ArcView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ArcView.prototype._map_data = function () {
            if (this.model.properties.radius.units == "data")
                this.sradius = this.sdist(this.renderer.xscale, this._x, this._radius);
            else
                this.sradius = this._radius;
        };
        ArcView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx = _a.sx, sy = _a.sy, sradius = _a.sradius, _start_angle = _a._start_angle, _end_angle = _a._end_angle;
            if (this.visuals.line.doit) {
                var direction = this.model.properties.direction.value();
                try {
                    for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                        var i = indices_1_1.value;
                        if (isNaN(sx[i] + sy[i] + sradius[i] + _start_angle[i] + _end_angle[i]))
                            continue;
                        ctx.beginPath();
                        ctx.arc(sx[i], sy[i], sradius[i], _start_angle[i], _end_angle[i], direction);
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.stroke();
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                            _b.call(indices_1);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
            }
        };
        ArcView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_line_legend(this.visuals, ctx, bbox, index);
        };
        return ArcView;
    }(xy_glyph_1.XYGlyphView));
    exports.ArcView = ArcView;
    ArcView.__name__ = "ArcView";
    var Arc = /** @class */ (function (_super) {
        tslib_1.__extends(Arc, _super);
        function Arc(attrs) {
            return _super.call(this, attrs) || this;
        }
        Arc.init_Arc = function () {
            this.prototype.default_view = ArcView;
            this.mixins(property_mixins_1.LineVector);
            this.define({
                direction: [p.Direction, 'anticlock'],
                radius: [p.DistanceSpec],
                start_angle: [p.AngleSpec],
                end_angle: [p.AngleSpec],
            });
        };
        return Arc;
    }(xy_glyph_1.XYGlyph));
    exports.Arc = Arc;
    Arc.__name__ = "Arc";
    Arc.init_Arc();
},
/* models/glyphs/bezier.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var glyph_1 = require(209) /* ./glyph */;
    var utils_1 = require(215) /* ./utils */;
    var projections_1 = require(152) /* ../../core/util/projections */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    // algorithm adapted from http://stackoverflow.com/a/14429749/3406693
    function _cbb(x0, y0, x1, y1, x2, y2, x3, y3) {
        var tvalues = [];
        var bounds = [[], []];
        for (var i = 0; i <= 2; i++) {
            var a = void 0, b = void 0, c = void 0;
            if (i === 0) {
                b = ((6 * x0) - (12 * x1)) + (6 * x2);
                a = (((-3 * x0) + (9 * x1)) - (9 * x2)) + (3 * x3);
                c = (3 * x1) - (3 * x0);
            }
            else {
                b = ((6 * y0) - (12 * y1)) + (6 * y2);
                a = (((-3 * y0) + (9 * y1)) - (9 * y2)) + (3 * y3);
                c = (3 * y1) - (3 * y0);
            }
            if (Math.abs(a) < 1e-12) { // Numerical robustness
                if (Math.abs(b) < 1e-12) // Numerical robustness
                    continue;
                var t = -c / b;
                if (0 < t && t < 1)
                    tvalues.push(t);
                continue;
            }
            var b2ac = (b * b) - (4 * c * a);
            var sqrtb2ac = Math.sqrt(b2ac);
            if (b2ac < 0)
                continue;
            var t1 = (-b + sqrtb2ac) / (2 * a);
            if (0 < t1 && t1 < 1)
                tvalues.push(t1);
            var t2 = (-b - sqrtb2ac) / (2 * a);
            if (0 < t2 && t2 < 1)
                tvalues.push(t2);
        }
        var j = tvalues.length;
        var jlen = j;
        while (j--) {
            var t = tvalues[j];
            var mt = 1 - t;
            var x = (mt * mt * mt * x0) + (3 * mt * mt * t * x1) + (3 * mt * t * t * x2) + (t * t * t * x3);
            bounds[0][j] = x;
            var y = (mt * mt * mt * y0) + (3 * mt * mt * t * y1) + (3 * mt * t * t * y2) + (t * t * t * y3);
            bounds[1][j] = y;
        }
        bounds[0][jlen] = x0;
        bounds[1][jlen] = y0;
        bounds[0][jlen + 1] = x3;
        bounds[1][jlen + 1] = y3;
        return [
            Math.min.apply(Math, tslib_1.__spread(bounds[0])),
            Math.max.apply(Math, tslib_1.__spread(bounds[1])),
            Math.max.apply(Math, tslib_1.__spread(bounds[0])),
            Math.min.apply(Math, tslib_1.__spread(bounds[1])),
        ];
    }
    var BezierView = /** @class */ (function (_super) {
        tslib_1.__extends(BezierView, _super);
        function BezierView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BezierView.prototype._project_data = function () {
            projections_1.inplace.project_xy(this._x0, this._y0);
            projections_1.inplace.project_xy(this._x1, this._y1);
        };
        BezierView.prototype._index_data = function (index) {
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                if (isNaN(this._x0[i] + this._x1[i] + this._y0[i] + this._y1[i] + this._cx0[i] + this._cy0[i] + this._cx1[i] + this._cy1[i]))
                    index.add_empty();
                else {
                    var _a = tslib_1.__read(_cbb(this._x0[i], this._y0[i], this._x1[i], this._y1[i], this._cx0[i], this._cy0[i], this._cx1[i], this._cy1[i]), 4), x0 = _a[0], y0 = _a[1], x1 = _a[2], y1 = _a[3];
                    index.add(x0, y0, x1, y1);
                }
            }
        };
        BezierView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx0 = _a.sx0, sy0 = _a.sy0, sx1 = _a.sx1, sy1 = _a.sy1, scx0 = _a.scx0, scy0 = _a.scy0, scx1 = _a.scx1, scy1 = _a.scy1;
            if (this.visuals.line.doit) {
                try {
                    for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                        var i = indices_1_1.value;
                        if (isNaN(sx0[i] + sy0[i] + sx1[i] + sy1[i] + scx0[i] + scy0[i] + scx1[i] + scy1[i]))
                            continue;
                        ctx.beginPath();
                        ctx.moveTo(sx0[i], sy0[i]);
                        ctx.bezierCurveTo(scx0[i], scy0[i], scx1[i], scy1[i], sx1[i], sy1[i]);
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.stroke();
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                            _b.call(indices_1);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
            }
        };
        BezierView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_line_legend(this.visuals, ctx, bbox, index);
        };
        BezierView.prototype.scenterxy = function () {
            throw new Error(this + ".scenterxy() is not implemented");
        };
        return BezierView;
    }(glyph_1.GlyphView));
    exports.BezierView = BezierView;
    BezierView.__name__ = "BezierView";
    var Bezier = /** @class */ (function (_super) {
        tslib_1.__extends(Bezier, _super);
        function Bezier(attrs) {
            return _super.call(this, attrs) || this;
        }
        Bezier.init_Bezier = function () {
            this.prototype.default_view = BezierView;
            this.define({
                x0: [p.XCoordinateSpec, { field: "x0" }],
                y0: [p.YCoordinateSpec, { field: "y0" }],
                x1: [p.XCoordinateSpec, { field: "x1" }],
                y1: [p.YCoordinateSpec, { field: "y1" }],
                cx0: [p.XCoordinateSpec, { field: "cx0" }],
                cy0: [p.YCoordinateSpec, { field: "cy0" }],
                cx1: [p.XCoordinateSpec, { field: "cx1" }],
                cy1: [p.YCoordinateSpec, { field: "cy1" }],
            });
            this.mixins(property_mixins_1.LineVector);
        };
        return Bezier;
    }(glyph_1.Glyph));
    exports.Bezier = Bezier;
    Bezier.__name__ = "Bezier";
    Bezier.init_Bezier();
},
/* models/glyphs/circle.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var markers_1 = require(353) /* ./webgl/markers */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var selection_1 = require(203) /* ../selections/selection */;
    var CircleView = /** @class */ (function (_super) {
        tslib_1.__extends(CircleView, _super);
        function CircleView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        CircleView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            var webgl = this.renderer.plot_view.canvas_view.webgl;
            if (webgl != null) {
                this.glglyph = new markers_1.CircleGL(webgl.gl, this);
            }
        };
        CircleView.prototype._map_data = function () {
            // XXX: Order is important here: size is always present (at least
            // a default), but radius is only present if a user specifies it.
            if (this._radius != null) {
                if (this.model.properties.radius.units == "data") {
                    switch (this.model.radius_dimension) {
                        case "x": {
                            this.sradius = this.sdist(this.renderer.xscale, this._x, this._radius);
                            break;
                        }
                        case "y": {
                            this.sradius = this.sdist(this.renderer.yscale, this._y, this._radius);
                            break;
                        }
                        case "max": {
                            var sradius_x = this.sdist(this.renderer.xscale, this._x, this._radius);
                            var sradius_y_1 = this.sdist(this.renderer.yscale, this._y, this._radius);
                            this.sradius = arrayable_1.map(sradius_x, function (s, i) { return Math.max(s, sradius_y_1[i]); });
                            break;
                        }
                        case "min": {
                            var sradius_x = this.sdist(this.renderer.xscale, this._x, this._radius);
                            var sradius_y_2 = this.sdist(this.renderer.yscale, this._y, this._radius);
                            this.sradius = arrayable_1.map(sradius_x, function (s, i) { return Math.min(s, sradius_y_2[i]); });
                            break;
                        }
                    }
                }
                else {
                    this.sradius = this._radius;
                    this.max_size = 2 * this.max_radius;
                }
            }
            else
                this.sradius = arrayable_1.map(this._size, function (s) { return s / 2; });
        };
        CircleView.prototype._mask_data = function () {
            var _a, _b, _c, _d;
            var _e = tslib_1.__read(this.renderer.plot_view.frame.bbox.ranges, 2), hr = _e[0], vr = _e[1];
            var x0, y0;
            var x1, y1;
            if (this._radius != null && this.model.properties.radius.units == "data") {
                var sx0 = hr.start;
                var sx1 = hr.end;
                _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
                x0 -= this.max_radius;
                x1 += this.max_radius;
                var sy0 = vr.start;
                var sy1 = vr.end;
                _b = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _b[0], y1 = _b[1];
                y0 -= this.max_radius;
                y1 += this.max_radius;
            }
            else {
                var sx0 = hr.start - this.max_size;
                var sx1 = hr.end + this.max_size;
                _c = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _c[0], x1 = _c[1];
                var sy0 = vr.start - this.max_size;
                var sy1 = vr.end + this.max_size;
                _d = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _d[0], y1 = _d[1];
            }
            return this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 });
        };
        CircleView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx = _a.sx, sy = _a.sy, sradius = _a.sradius;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (isNaN(sx[i] + sy[i] + sradius[i]))
                        continue;
                    ctx.beginPath();
                    ctx.arc(sx[i], sy[i], sradius[i], 0, 2 * Math.PI, false);
                    if (this.visuals.fill.doit) {
                        this.visuals.fill.set_vectorize(ctx, i);
                        ctx.fill();
                    }
                    if (this.visuals.line.doit) {
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.stroke();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        CircleView.prototype._hit_point = function (geometry) {
            var _a, _b, e_2, _c, e_3, _d;
            var sx = geometry.sx, sy = geometry.sy;
            var x = this.renderer.xscale.invert(sx);
            var y = this.renderer.yscale.invert(sy);
            var x0, x1, y0, y1;
            if (this._radius != null && this.model.properties.radius.units == "data") {
                x0 = x - this.max_radius;
                x1 = x + this.max_radius;
                y0 = y - this.max_radius;
                y1 = y + this.max_radius;
            }
            else {
                var sx0 = sx - this.max_size;
                var sx1 = sx + this.max_size;
                _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
                var sy0 = sy - this.max_size;
                var sy1 = sy + this.max_size;
                _b = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _b[0], y1 = _b[1];
            }
            var candidates = this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 });
            var indices = [];
            if (this._radius != null && this.model.properties.radius.units == "data") {
                try {
                    for (var candidates_1 = tslib_1.__values(candidates), candidates_1_1 = candidates_1.next(); !candidates_1_1.done; candidates_1_1 = candidates_1.next()) {
                        var i = candidates_1_1.value;
                        var r2 = Math.pow(this.sradius[i], 2);
                        var _e = tslib_1.__read(this.renderer.xscale.r_compute(x, this._x[i]), 2), sx0 = _e[0], sx1 = _e[1];
                        var _f = tslib_1.__read(this.renderer.yscale.r_compute(y, this._y[i]), 2), sy0 = _f[0], sy1 = _f[1];
                        var dist = Math.pow((sx0 - sx1), 2) + Math.pow((sy0 - sy1), 2);
                        if (dist <= r2) {
                            indices.push(i);
                        }
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (candidates_1_1 && !candidates_1_1.done && (_c = candidates_1.return))
                            _c.call(candidates_1);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
            }
            else {
                try {
                    for (var candidates_2 = tslib_1.__values(candidates), candidates_2_1 = candidates_2.next(); !candidates_2_1.done; candidates_2_1 = candidates_2.next()) {
                        var i = candidates_2_1.value;
                        var r2 = Math.pow(this.sradius[i], 2);
                        var dist = Math.pow((this.sx[i] - sx), 2) + Math.pow((this.sy[i] - sy), 2);
                        if (dist <= r2) {
                            indices.push(i);
                        }
                    }
                }
                catch (e_3_1) {
                    e_3 = { error: e_3_1 };
                }
                finally {
                    try {
                        if (candidates_2_1 && !candidates_2_1.done && (_d = candidates_2.return))
                            _d.call(candidates_2);
                    }
                    finally {
                        if (e_3)
                            throw e_3.error;
                    }
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        CircleView.prototype._hit_span = function (geometry) {
            var _a, _b, _c, _d;
            var sx = geometry.sx, sy = geometry.sy;
            var bounds = this.bounds();
            var x0, x1, y0, y1;
            if (geometry.direction == 'h') {
                // use circle bounds instead of current pointer y coordinates
                var sx0 = void 0, sx1 = void 0;
                y0 = bounds.y0;
                y1 = bounds.y1;
                if (this._radius != null && this.model.properties.radius.units == "data") {
                    sx0 = sx - this.max_radius;
                    sx1 = sx + this.max_radius;
                    _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
                }
                else {
                    var ms = this.max_size / 2;
                    sx0 = sx - ms;
                    sx1 = sx + ms;
                    _b = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _b[0], x1 = _b[1];
                }
            }
            else {
                // use circle bounds instead of current pointer x coordinates
                var sy0 = void 0, sy1 = void 0;
                x0 = bounds.x0;
                x1 = bounds.x1;
                if (this._radius != null && this.model.properties.radius.units == "data") {
                    sy0 = sy - this.max_radius;
                    sy1 = sy + this.max_radius;
                    _c = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _c[0], y1 = _c[1];
                }
                else {
                    var ms = this.max_size / 2;
                    sy0 = sy - ms;
                    sy1 = sy + ms;
                    _d = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _d[0], y1 = _d[1];
                }
            }
            var indices = tslib_1.__spread(this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 }));
            return new selection_1.Selection({ indices: indices });
        };
        CircleView.prototype._hit_rect = function (geometry) {
            var sx0 = geometry.sx0, sx1 = geometry.sx1, sy0 = geometry.sy0, sy1 = geometry.sy1;
            var _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
            var _b = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _b[0], y1 = _b[1];
            var indices = tslib_1.__spread(this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 }));
            return new selection_1.Selection({ indices: indices });
        };
        CircleView.prototype._hit_poly = function (geometry) {
            var sx = geometry.sx, sy = geometry.sy;
            // TODO (bev) use spatial index to pare candidate list
            var candidates = array_1.range(0, this.sx.length);
            var indices = [];
            for (var i = 0, end = candidates.length; i < end; i++) {
                var index = candidates[i];
                if (hittest.point_in_poly(this.sx[i], this.sy[i], sx, sy)) {
                    indices.push(index);
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        // circle does not inherit from marker (since it also accepts radius) so we
        // must supply a draw_legend for it  here
        CircleView.prototype.draw_legend_for_index = function (ctx, _a, index) {
            var x0 = _a.x0, y0 = _a.y0, x1 = _a.x1, y1 = _a.y1;
            // using objects like this seems a little wonky, since the keys are coerced to
            // stings, but it works
            var len = index + 1;
            var sx = new Array(len);
            sx[index] = (x0 + x1) / 2;
            var sy = new Array(len);
            sy[index] = (y0 + y1) / 2;
            var sradius = new Array(len);
            sradius[index] = Math.min(Math.abs(x1 - x0), Math.abs(y1 - y0)) * 0.2;
            this._render(ctx, [index], { sx: sx, sy: sy, sradius: sradius }); // XXX
        };
        return CircleView;
    }(xy_glyph_1.XYGlyphView));
    exports.CircleView = CircleView;
    CircleView.__name__ = "CircleView";
    var Circle = /** @class */ (function (_super) {
        tslib_1.__extends(Circle, _super);
        function Circle(attrs) {
            return _super.call(this, attrs) || this;
        }
        Circle.init_Circle = function () {
            this.prototype.default_view = CircleView;
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector]);
            this.define({
                angle: [p.AngleSpec, 0],
                size: [p.DistanceSpec, { units: "screen", value: 4 }],
                radius: [p.DistanceSpec, undefined, { optional: true }],
                radius_dimension: [p.RadiusDimension, 'x'],
            });
        };
        return Circle;
    }(xy_glyph_1.XYGlyph));
    exports.Circle = Circle;
    Circle.__name__ = "Circle";
    Circle.init_Circle();
},
/* models/glyphs/webgl/markers.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var utils_1 = require(218) /* ./utils */;
    var base_1 = require(222) /* ./base */;
    var markers_vert_1 = require(354) /* ./markers.vert */;
    var markers_frag_1 = require(355) /* ./markers.frag */;
    var circle_1 = require(352) /* ../circle */;
    var arrayable_1 = require(127) /* ../../../core/util/arrayable */;
    var logging_1 = require(134) /* ../../../core/logging */;
    var color_1 = require(137) /* ../../../core/util/color */;
    function attach_float(prog, vbo, att_name, n, visual, name) {
        // Attach a float attribute to the program. Use singleton value if we can,
        // otherwise use VBO to apply array.
        if (!visual.doit) {
            vbo.used = false;
            prog.set_attribute(att_name, 'float', [0]);
        }
        else if (visual[name].is_value) {
            vbo.used = false;
            prog.set_attribute(att_name, 'float', [visual[name].value()]);
        }
        else {
            vbo.used = true;
            var a = new Float32Array(visual.get_array(name));
            vbo.set_size(n * 4);
            vbo.set_data(0, a);
            prog.set_attribute(att_name, 'float', vbo);
        }
    }
    exports.attach_float = attach_float;
    function attach_color(prog, vbo, att_name, n, visual, prefix) {
        // Attach the color attribute to the program. If there's just one color,
        // then use this single color for all vertices (no VBO). Otherwise we
        // create an array and upload that to the VBO, which we attahce to the prog.
        var m = 4;
        var colorname = prefix + '_color';
        var alphaname = prefix + '_alpha';
        if (!visual.doit) {
            // Don't draw (draw transparent)
            vbo.used = false;
            prog.set_attribute(att_name, 'vec4', [0, 0, 0, 0]);
        }
        else {
            // Use vbo; we need an array for both the color and the alpha
            vbo.used = true;
            var colors = void 0;
            if (visual[colorname].is_value) {
                var val = color_1.encode_rgba(color_1.color2rgba(visual[colorname].value()));
                var array = new Uint32Array(n);
                array.fill(val);
                colors = array;
            }
            else
                colors = visual.get_array(colorname);
            var alphas = void 0;
            if (visual[alphaname].is_value) {
                var val = visual[alphaname].value();
                var array = new Float32Array(n);
                array.fill(val);
                alphas = array;
            }
            else
                alphas = visual.get_array(alphaname);
            // Create array of rgbs
            var a = new Float32Array(n * m);
            for (var i = 0, end = n; i < end; i++) {
                var rgba = color_1.decode_rgba(colors[i]);
                if (rgba[3] == 1.0)
                    rgba[3] = alphas[i];
                a.set(rgba, i * m);
            }
            // Attach vbo
            vbo.set_size(n * m * 4);
            vbo.set_data(0, a);
            prog.set_attribute(att_name, 'vec4', vbo);
        }
    }
    exports.attach_color = attach_color;
    // Base class for markers. All markers share the same GLSL, except for one
    // function that defines the marker geometry.
    var MarkerGL = /** @class */ (function (_super) {
        tslib_1.__extends(MarkerGL, _super);
        function MarkerGL() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        MarkerGL.prototype.init = function () {
            var gl = this.gl;
            var vert = markers_vert_1.vertex_shader;
            var frag = markers_frag_1.fragment_shader(this._marker_code);
            // The program
            this.prog = new utils_1.Program(gl);
            this.prog.set_shaders(vert, frag);
            // Real attributes
            this.vbo_sx = new utils_1.VertexBuffer(gl);
            this.prog.set_attribute('a_sx', 'float', this.vbo_sx);
            this.vbo_sy = new utils_1.VertexBuffer(gl);
            this.prog.set_attribute('a_sy', 'float', this.vbo_sy);
            this.vbo_s = new utils_1.VertexBuffer(gl);
            this.prog.set_attribute('a_size', 'float', this.vbo_s);
            this.vbo_a = new utils_1.VertexBuffer(gl);
            this.prog.set_attribute('a_angle', 'float', this.vbo_a);
            // VBO's for attributes (they may not be used if value is singleton)
            this.vbo_linewidth = new utils_1.VertexBuffer(gl);
            this.vbo_fg_color = new utils_1.VertexBuffer(gl);
            this.vbo_bg_color = new utils_1.VertexBuffer(gl);
            this.index_buffer = new utils_1.IndexBuffer(gl);
        };
        MarkerGL.prototype.draw = function (indices, mainGlyph, trans) {
            // The main glyph has the data, *this* glyph has the visuals.
            var mainGlGlyph = mainGlyph.glglyph;
            var nvertices = mainGlGlyph.nvertices;
            // Upload data if we must. Only happens for main glyph.
            if (mainGlGlyph.data_changed) {
                mainGlGlyph._set_data(nvertices);
                if (this.glyph instanceof circle_1.CircleView && this.glyph._radius != null) {
                    // Keep screen radius up-to-date for circle glyph. Only happens when a radius is given
                    this.vbo_s.set_data(0, arrayable_1.map(this.glyph.sradius, function (s) { return s * 2; }));
                }
                mainGlGlyph.data_changed = false;
            }
            // Update visuals if we must. Can happen for all glyphs.
            if (this.visuals_changed) {
                this._set_visuals(nvertices);
                this.visuals_changed = false;
            }
            // Handle transformation to device coordinates
            this.prog.set_uniform('u_pixel_ratio', 'float', [trans.pixel_ratio]);
            this.prog.set_uniform('u_canvas_size', 'vec2', [trans.width, trans.height]);
            // Select buffers from main glyph
            // (which may be this glyph but maybe not if this is a (non)selection glyph)
            this.prog.set_attribute('a_sx', 'float', mainGlGlyph.vbo_sx);
            this.prog.set_attribute('a_sy', 'float', mainGlGlyph.vbo_sy);
            this.prog.set_attribute('a_size', 'float', mainGlGlyph.vbo_s);
            this.prog.set_attribute('a_angle', 'float', mainGlGlyph.vbo_a);
            // Draw directly or using indices. Do not handle indices if they do not
            // fit in a uint16; WebGL 1.0 does not support uint32.
            if (indices.length == 0)
                return;
            else if (indices.length === nvertices)
                this.prog.draw(this.gl.POINTS, [0, nvertices]);
            else if (nvertices < 65535) {
                // On IE the marker size is reduced to 1 px when using an index buffer
                // A MS Edge dev on Twitter said on 24-04-2014: "gl_PointSize > 1.0 works
                // in DrawArrays; gl_PointSize > 1.0 in DrawElements is coming soon in the
                // next renderer update.
                var ua = window.navigator.userAgent;
                if ((ua.indexOf("MSIE ") + ua.indexOf("Trident/") + ua.indexOf("Edge/")) > 0) {
                    logging_1.logger.warn('WebGL warning: IE is known to produce 1px sprites whith selections.');
                }
                this.index_buffer.set_size(indices.length * 2);
                this.index_buffer.set_data(0, new Uint16Array(indices));
                this.prog.draw(this.gl.POINTS, this.index_buffer);
            }
            else {
                // Work around the limit that the indexbuffer must be uint16. We draw in chunks.
                // First collect indices in chunks
                var chunksize = 64000; // 65536
                var chunks = [];
                for (var i = 0, end = Math.ceil(nvertices / chunksize); i < end; i++) {
                    chunks.push([]);
                }
                for (var i = 0, end = indices.length; i < end; i++) {
                    var uint16_index = indices[i] % chunksize;
                    var chunk = Math.floor(indices[i] / chunksize);
                    chunks[chunk].push(uint16_index);
                }
                // Then draw each chunk
                for (var chunk = 0, end = chunks.length; chunk < end; chunk++) {
                    var these_indices = new Uint16Array(chunks[chunk]);
                    var offset = chunk * chunksize * 4;
                    if (these_indices.length === 0) {
                        continue;
                    }
                    this.prog.set_attribute('a_sx', 'float', mainGlGlyph.vbo_sx, 0, offset);
                    this.prog.set_attribute('a_sy', 'float', mainGlGlyph.vbo_sy, 0, offset);
                    this.prog.set_attribute('a_size', 'float', mainGlGlyph.vbo_s, 0, offset);
                    this.prog.set_attribute('a_angle', 'float', mainGlGlyph.vbo_a, 0, offset);
                    if (this.vbo_linewidth.used) {
                        this.prog.set_attribute('a_linewidth', 'float', this.vbo_linewidth, 0, offset);
                    }
                    if (this.vbo_fg_color.used) {
                        this.prog.set_attribute('a_fg_color', 'vec4', this.vbo_fg_color, 0, offset * 4);
                    }
                    if (this.vbo_bg_color.used) {
                        this.prog.set_attribute('a_bg_color', 'vec4', this.vbo_bg_color, 0, offset * 4);
                    }
                    // The actual drawing
                    this.index_buffer.set_size(these_indices.length * 2);
                    this.index_buffer.set_data(0, these_indices);
                    this.prog.draw(this.gl.POINTS, this.index_buffer);
                }
            }
        };
        MarkerGL.prototype._set_data = function (nvertices) {
            var n = nvertices * 4; // in bytes
            // Set buffer size
            this.vbo_sx.set_size(n);
            this.vbo_sy.set_size(n);
            this.vbo_a.set_size(n);
            this.vbo_s.set_size(n);
            this.vbo_sx.set_data(0, this.glyph.sx);
            this.vbo_sy.set_data(0, this.glyph.sy);
            if (this.glyph._angle != null) {
                this.vbo_a.set_data(0, new Float32Array(this.glyph._angle));
            }
            if (this.glyph instanceof circle_1.CircleView && this.glyph._radius != null)
                this.vbo_s.set_data(0, arrayable_1.map(this.glyph.sradius, function (s) { return s * 2; }));
            else
                this.vbo_s.set_data(0, new Float32Array(this.glyph._size));
        };
        MarkerGL.prototype._set_visuals = function (nvertices) {
            attach_float(this.prog, this.vbo_linewidth, 'a_linewidth', nvertices, this.glyph.visuals.line, 'line_width');
            attach_color(this.prog, this.vbo_fg_color, 'a_fg_color', nvertices, this.glyph.visuals.line, 'line');
            attach_color(this.prog, this.vbo_bg_color, 'a_bg_color', nvertices, this.glyph.visuals.fill, 'fill');
            // Static value for antialias. Smaller aa-region to obtain crisper images
            this.prog.set_uniform('u_antialias', 'float', [0.8]);
        };
        return MarkerGL;
    }(base_1.BaseGLGlyph));
    exports.MarkerGL = MarkerGL;
    MarkerGL.__name__ = "MarkerGL";
    function mk_marker(code) {
        return /** @class */ (function (_super) {
            tslib_1.__extends(class_1, _super);
            function class_1() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            Object.defineProperty(class_1.prototype, "_marker_code", {
                get: function () {
                    return code;
                },
                enumerable: true,
                configurable: true
            });
            return class_1;
        }(MarkerGL));
    }
    var glsl = tslib_1.__importStar(require(355) /* ./markers.frag */);
    exports.AsteriskGL = mk_marker(glsl.asterisk);
    exports.CircleGL = mk_marker(glsl.circle);
    exports.CircleCrossGL = mk_marker(glsl.circlecross);
    exports.CircleXGL = mk_marker(glsl.circlex);
    exports.CrossGL = mk_marker(glsl.cross);
    exports.DiamondGL = mk_marker(glsl.diamond);
    exports.DiamondCrossGL = mk_marker(glsl.diamondcross);
    exports.HexGL = mk_marker(glsl.hex);
    exports.InvertedTriangleGL = mk_marker(glsl.invertedtriangle);
    exports.SquareGL = mk_marker(glsl.square);
    exports.SquareCrossGL = mk_marker(glsl.squarecross);
    exports.SquareXGL = mk_marker(glsl.squarex);
    exports.TriangleGL = mk_marker(glsl.triangle);
    exports.XGL = mk_marker(glsl.x);
},
/* models/glyphs/webgl/markers.vert.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.vertex_shader = "\nprecision mediump float;\nconst float SQRT_2 = 1.4142135623730951;\n//\nuniform float u_pixel_ratio;\nuniform vec2 u_canvas_size;\nuniform vec2 u_offset;\nuniform vec2 u_scale;\nuniform float u_antialias;\n//\nattribute float a_sx;\nattribute float a_sy;\nattribute float a_size;\nattribute float a_angle;  // in radians\nattribute float a_linewidth;\nattribute vec4  a_fg_color;\nattribute vec4  a_bg_color;\n//\nvarying float v_linewidth;\nvarying float v_size;\nvarying vec4  v_fg_color;\nvarying vec4  v_bg_color;\nvarying vec2  v_rotation;\n\nvoid main (void)\n{\n    v_size = a_size * u_pixel_ratio;\n    v_linewidth = a_linewidth * u_pixel_ratio;\n    v_fg_color = a_fg_color;\n    v_bg_color = a_bg_color;\n    v_rotation = vec2(cos(-a_angle), sin(-a_angle));\n    vec2 pos = vec2(a_sx, a_sy);  // in pixels\n    pos += 0.5;  // make up for Bokeh's offset\n    pos /= u_canvas_size / u_pixel_ratio;  // in 0..1\n    gl_Position = vec4(pos*2.0-1.0, 0.0, 1.0);\n    gl_Position.y *= -1.0;\n    gl_PointSize = SQRT_2 * v_size + 2.0 * (v_linewidth + 1.5*u_antialias);\n}\n";
},
/* models/glyphs/webgl/markers.frag.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.fragment_shader = function (marker_code) { return "\nprecision mediump float;\nconst float SQRT_2 = 1.4142135623730951;\nconst float PI = 3.14159265358979323846264;\n//\nuniform float u_antialias;\n//\nvarying vec4  v_fg_color;\nvarying vec4  v_bg_color;\nvarying float v_linewidth;\nvarying float v_size;\nvarying vec2  v_rotation;\n\n" + marker_code + "\n\nvec4 outline(float distance, float linewidth, float antialias, vec4 fg_color, vec4 bg_color)\n{\n    vec4 frag_color;\n    float t = linewidth/2.0 - antialias;\n    float signed_distance = distance;\n    float border_distance = abs(signed_distance) - t;\n    float alpha = border_distance/antialias;\n    alpha = exp(-alpha*alpha);\n\n    // If fg alpha is zero, it probably means no outline. To avoid a dark outline\n    // shining through due to aa, we set the fg color to the bg color. Avoid if (i.e. branching).\n    float select = float(bool(fg_color.a));\n    fg_color.rgb = select * fg_color.rgb + (1.0  - select) * bg_color.rgb;\n    // Similarly, if we want a transparent bg\n    select = float(bool(bg_color.a));\n    bg_color.rgb = select * bg_color.rgb + (1.0  - select) * fg_color.rgb;\n\n    if( border_distance < 0.0)\n        frag_color = fg_color;\n    else if( signed_distance < 0.0 ) {\n        frag_color = mix(bg_color, fg_color, sqrt(alpha));\n    } else {\n        if( abs(signed_distance) < (linewidth/2.0 + antialias) ) {\n            frag_color = vec4(fg_color.rgb, fg_color.a * alpha);\n        } else {\n            discard;\n        }\n    }\n    return frag_color;\n}\n\nvoid main()\n{\n    vec2 P = gl_PointCoord.xy - vec2(0.5, 0.5);\n    P = vec2(v_rotation.x*P.x - v_rotation.y*P.y,\n             v_rotation.y*P.x + v_rotation.x*P.y);\n    float point_size = SQRT_2*v_size  + 2.0 * (v_linewidth + 1.5*u_antialias);\n    float distance = marker(P*point_size, v_size);\n    gl_FragColor = outline(distance, v_linewidth, u_antialias, v_fg_color, v_bg_color);\n}\n"; };
    exports.circle = "\nfloat marker(vec2 P, float size)\n{\n    return length(P) - size/2.0;\n}\n";
    exports.square = "\nfloat marker(vec2 P, float size)\n{\n    return max(abs(P.x), abs(P.y)) - size/2.0;\n}\n";
    exports.diamond = "\nfloat marker(vec2 P, float size)\n{\n    float x = SQRT_2 / 2.0 * (P.x * 1.5 - P.y);\n    float y = SQRT_2 / 2.0 * (P.x * 1.5 + P.y);\n    float r1 = max(abs(x), abs(y)) - size / (2.0 * SQRT_2);\n    return r1 / SQRT_2;\n}\n";
    exports.hex = "\nfloat marker(vec2 P, float size)\n{\n    vec2 q = abs(P);\n    return max(q.y * 0.57735 + q.x - 1.0 * size/2.0, q.y - 0.866 * size/2.0);\n}\n";
    exports.triangle = "\nfloat marker(vec2 P, float size)\n{\n    P.y -= size * 0.3;\n    float x = SQRT_2 / 2.0 * (P.x * 1.7 - P.y);\n    float y = SQRT_2 / 2.0 * (P.x * 1.7 + P.y);\n    float r1 = max(abs(x), abs(y)) - size / 1.6;\n    float r2 = P.y;\n    return max(r1 / SQRT_2, r2);  // Intersect diamond with rectangle\n}\n";
    exports.invertedtriangle = "\nfloat marker(vec2 P, float size)\n{\n    P.y += size * 0.3;\n    float x = SQRT_2 / 2.0 * (P.x * 1.7 - P.y);\n    float y = SQRT_2 / 2.0 * (P.x * 1.7 + P.y);\n    float r1 = max(abs(x), abs(y)) - size / 1.6;\n    float r2 = - P.y;\n    return max(r1 / SQRT_2, r2);  // Intersect diamond with rectangle\n}\n";
    exports.cross = "\nfloat marker(vec2 P, float size)\n{\n    float square = max(abs(P.x), abs(P.y)) - size / 2.5;   // 2.5 is a tweak\n    float cross = min(abs(P.x), abs(P.y)) - size / 100.0;  // bit of \"width\" for aa\n    return max(square, cross);\n}\n";
    exports.circlecross = "\nfloat marker(vec2 P, float size)\n{\n    // Define quadrants\n    float qs = size / 2.0;  // quadrant size\n    float s1 = max(abs(P.x - qs), abs(P.y - qs)) - qs;\n    float s2 = max(abs(P.x + qs), abs(P.y - qs)) - qs;\n    float s3 = max(abs(P.x - qs), abs(P.y + qs)) - qs;\n    float s4 = max(abs(P.x + qs), abs(P.y + qs)) - qs;\n    // Intersect main shape with quadrants (to form cross)\n    float circle = length(P) - size/2.0;\n    float c1 = max(circle, s1);\n    float c2 = max(circle, s2);\n    float c3 = max(circle, s3);\n    float c4 = max(circle, s4);\n    // Union\n    return min(min(min(c1, c2), c3), c4);\n}\n";
    exports.squarecross = "\nfloat marker(vec2 P, float size)\n{\n    // Define quadrants\n    float qs = size / 2.0;  // quadrant size\n    float s1 = max(abs(P.x - qs), abs(P.y - qs)) - qs;\n    float s2 = max(abs(P.x + qs), abs(P.y - qs)) - qs;\n    float s3 = max(abs(P.x - qs), abs(P.y + qs)) - qs;\n    float s4 = max(abs(P.x + qs), abs(P.y + qs)) - qs;\n    // Intersect main shape with quadrants (to form cross)\n    float square = max(abs(P.x), abs(P.y)) - size/2.0;\n    float c1 = max(square, s1);\n    float c2 = max(square, s2);\n    float c3 = max(square, s3);\n    float c4 = max(square, s4);\n    // Union\n    return min(min(min(c1, c2), c3), c4);\n}\n";
    exports.diamondcross = "\nfloat marker(vec2 P, float size)\n{\n    // Define quadrants\n    float qs = size / 2.0;  // quadrant size\n    float s1 = max(abs(P.x - qs), abs(P.y - qs)) - qs;\n    float s2 = max(abs(P.x + qs), abs(P.y - qs)) - qs;\n    float s3 = max(abs(P.x - qs), abs(P.y + qs)) - qs;\n    float s4 = max(abs(P.x + qs), abs(P.y + qs)) - qs;\n    // Intersect main shape with quadrants (to form cross)\n    float x = SQRT_2 / 2.0 * (P.x * 1.5 - P.y);\n    float y = SQRT_2 / 2.0 * (P.x * 1.5 + P.y);\n    float diamond = max(abs(x), abs(y)) - size / (2.0 * SQRT_2);\n    diamond /= SQRT_2;\n    float c1 = max(diamond, s1);\n    float c2 = max(diamond, s2);\n    float c3 = max(diamond, s3);\n    float c4 = max(diamond, s4);\n    // Union\n    return min(min(min(c1, c2), c3), c4);\n}\n";
    exports.x = "\nfloat marker(vec2 P, float size)\n{\n    float circle = length(P) - size / 1.6;\n    float X = min(abs(P.x - P.y), abs(P.x + P.y)) - size / 100.0;  // bit of \"width\" for aa\n    return max(circle, X);\n}\n";
    exports.circlex = "\nfloat marker(vec2 P, float size)\n{\n    float x = P.x - P.y;\n    float y = P.x + P.y;\n    // Define quadrants\n    float qs = size / 2.0;  // quadrant size\n    float s1 = max(abs(x - qs), abs(y - qs)) - qs;\n    float s2 = max(abs(x + qs), abs(y - qs)) - qs;\n    float s3 = max(abs(x - qs), abs(y + qs)) - qs;\n    float s4 = max(abs(x + qs), abs(y + qs)) - qs;\n    // Intersect main shape with quadrants (to form cross)\n    float circle = length(P) - size/2.0;\n    float c1 = max(circle, s1);\n    float c2 = max(circle, s2);\n    float c3 = max(circle, s3);\n    float c4 = max(circle, s4);\n    // Union\n    float almost = min(min(min(c1, c2), c3), c4);\n    // In this case, the X is also outside of the main shape\n    float Xmask = length(P) - size / 1.6;  // a circle\n    float X = min(abs(P.x - P.y), abs(P.x + P.y)) - size / 100.0;  // bit of \"width\" for aa\n    return min(max(X, Xmask), almost);\n}\n";
    exports.squarex = "\nfloat marker(vec2 P, float size)\n{\n    float x = P.x - P.y;\n    float y = P.x + P.y;\n    // Define quadrants\n    float qs = size / 2.0;  // quadrant size\n    float s1 = max(abs(x - qs), abs(y - qs)) - qs;\n    float s2 = max(abs(x + qs), abs(y - qs)) - qs;\n    float s3 = max(abs(x - qs), abs(y + qs)) - qs;\n    float s4 = max(abs(x + qs), abs(y + qs)) - qs;\n    // Intersect main shape with quadrants (to form cross)\n    float square = max(abs(P.x), abs(P.y)) - size/2.0;\n    float c1 = max(square, s1);\n    float c2 = max(square, s2);\n    float c3 = max(square, s3);\n    float c4 = max(square, s4);\n    // Union\n    return min(min(min(c1, c2), c3), c4);\n}\n";
    exports.asterisk = "\nfloat marker(vec2 P, float size)\n{\n    // Masks\n    float diamond = max(abs(SQRT_2 / 2.0 * (P.x - P.y)), abs(SQRT_2 / 2.0 * (P.x + P.y))) - size / (2.0 * SQRT_2);\n    float square = max(abs(P.x), abs(P.y)) - size / (2.0 * SQRT_2);\n    // Shapes\n    float X = min(abs(P.x - P.y), abs(P.x + P.y)) - size / 100.0;  // bit of \"width\" for aa\n    float cross = min(abs(P.x), abs(P.y)) - size / 100.0;  // bit of \"width\" for aa\n    // Result is union of masked shapes\n    return min(max(X, diamond), max(cross, square));\n}\n";
},
/* models/glyphs/center_rotatable.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var CenterRotatableView = /** @class */ (function (_super) {
        tslib_1.__extends(CenterRotatableView, _super);
        function CenterRotatableView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return CenterRotatableView;
    }(xy_glyph_1.XYGlyphView));
    exports.CenterRotatableView = CenterRotatableView;
    CenterRotatableView.__name__ = "CenterRotatableView";
    var CenterRotatable = /** @class */ (function (_super) {
        tslib_1.__extends(CenterRotatable, _super);
        function CenterRotatable(attrs) {
            return _super.call(this, attrs) || this;
        }
        CenterRotatable.init_CenterRotatable = function () {
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector]);
            this.define({
                angle: [p.AngleSpec, 0],
                width: [p.DistanceSpec],
                height: [p.DistanceSpec],
            });
        };
        return CenterRotatable;
    }(xy_glyph_1.XYGlyph));
    exports.CenterRotatable = CenterRotatable;
    CenterRotatable.__name__ = "CenterRotatable";
    CenterRotatable.init_CenterRotatable();
},
/* models/glyphs/ellipse.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var ellipse_oval_1 = require(358) /* ./ellipse_oval */;
    var EllipseView = /** @class */ (function (_super) {
        tslib_1.__extends(EllipseView, _super);
        function EllipseView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return EllipseView;
    }(ellipse_oval_1.EllipseOvalView));
    exports.EllipseView = EllipseView;
    EllipseView.__name__ = "EllipseView";
    var Ellipse = /** @class */ (function (_super) {
        tslib_1.__extends(Ellipse, _super);
        function Ellipse(attrs) {
            return _super.call(this, attrs) || this;
        }
        Ellipse.init_Ellipse = function () {
            this.prototype.default_view = EllipseView;
        };
        return Ellipse;
    }(ellipse_oval_1.EllipseOval));
    exports.Ellipse = Ellipse;
    Ellipse.__name__ = "Ellipse";
    Ellipse.init_Ellipse();
},
/* models/glyphs/ellipse_oval.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var center_rotatable_1 = require(356) /* ./center_rotatable */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var selection_1 = require(203) /* ../selections/selection */;
    var EllipseOvalView = /** @class */ (function (_super) {
        tslib_1.__extends(EllipseOvalView, _super);
        function EllipseOvalView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        EllipseOvalView.prototype._set_data = function () {
            this.max_w2 = 0;
            if (this.model.properties.width.units == "data")
                this.max_w2 = this.max_width / 2;
            this.max_h2 = 0;
            if (this.model.properties.height.units == "data")
                this.max_h2 = this.max_height / 2;
        };
        EllipseOvalView.prototype._map_data = function () {
            if (this.model.properties.width.units == "data")
                this.sw = this.sdist(this.renderer.xscale, this._x, this._width, 'center');
            else
                this.sw = this._width;
            if (this.model.properties.height.units == "data")
                this.sh = this.sdist(this.renderer.yscale, this._y, this._height, 'center');
            else
                this.sh = this._height;
        };
        EllipseOvalView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx = _a.sx, sy = _a.sy, sw = _a.sw, sh = _a.sh, _angle = _a._angle;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (isNaN(sx[i] + sy[i] + sw[i] + sh[i] + _angle[i]))
                        continue;
                    ctx.beginPath();
                    ctx.ellipse(sx[i], sy[i], sw[i] / 2.0, sh[i] / 2.0, _angle[i], 0, 2 * Math.PI);
                    if (this.visuals.fill.doit) {
                        this.visuals.fill.set_vectorize(ctx, i);
                        ctx.fill();
                    }
                    if (this.visuals.line.doit) {
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.stroke();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        EllipseOvalView.prototype._hit_point = function (geometry) {
            var _a, _b, e_2, _c;
            var x0, x1, y0, y1, cond, sx0, sx1, sy0, sy1;
            var sx = geometry.sx, sy = geometry.sy;
            var x = this.renderer.xscale.invert(sx);
            var y = this.renderer.yscale.invert(sy);
            if (this.model.properties.width.units == "data") {
                x0 = x - this.max_width;
                x1 = x + this.max_width;
            }
            else {
                sx0 = sx - this.max_width;
                sx1 = sx + this.max_width;
                _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
            }
            if (this.model.properties.height.units == "data") {
                y0 = y - this.max_height;
                y1 = y + this.max_height;
            }
            else {
                sy0 = sy - this.max_height;
                sy1 = sy + this.max_height;
                _b = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _b[0], y1 = _b[1];
            }
            var candidates = this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 });
            var indices = [];
            try {
                for (var candidates_1 = tslib_1.__values(candidates), candidates_1_1 = candidates_1.next(); !candidates_1_1.done; candidates_1_1 = candidates_1.next()) {
                    var i = candidates_1_1.value;
                    cond = hittest.point_in_ellipse(sx, sy, this._angle[i], this.sh[i] / 2, this.sw[i] / 2, this.sx[i], this.sy[i]);
                    if (cond) {
                        indices.push(i);
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (candidates_1_1 && !candidates_1_1.done && (_c = candidates_1.return))
                        _c.call(candidates_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        EllipseOvalView.prototype.draw_legend_for_index = function (ctx, _a, index) {
            var x0 = _a.x0, y0 = _a.y0, x1 = _a.x1, y1 = _a.y1;
            var len = index + 1;
            var sx = new Array(len);
            sx[index] = (x0 + x1) / 2;
            var sy = new Array(len);
            sy[index] = (y0 + y1) / 2;
            var scale = this.sw[index] / this.sh[index];
            var d = Math.min(Math.abs(x1 - x0), Math.abs(y1 - y0)) * 0.8;
            var sw = new Array(len);
            var sh = new Array(len);
            if (scale > 1) {
                sw[index] = d;
                sh[index] = d / scale;
            }
            else {
                sw[index] = d * scale;
                sh[index] = d;
            }
            this._render(ctx, [index], { sx: sx, sy: sy, sw: sw, sh: sh, _angle: [0] }); // XXX
        };
        EllipseOvalView.prototype._bounds = function (_a) {
            var x0 = _a.x0, x1 = _a.x1, y0 = _a.y0, y1 = _a.y1;
            return {
                x0: x0 - this.max_w2,
                x1: x1 + this.max_w2,
                y0: y0 - this.max_h2,
                y1: y1 + this.max_h2,
            };
        };
        return EllipseOvalView;
    }(center_rotatable_1.CenterRotatableView));
    exports.EllipseOvalView = EllipseOvalView;
    EllipseOvalView.__name__ = "EllipseOvalView";
    var EllipseOval = /** @class */ (function (_super) {
        tslib_1.__extends(EllipseOval, _super);
        function EllipseOval(attrs) {
            return _super.call(this, attrs) || this;
        }
        return EllipseOval;
    }(center_rotatable_1.CenterRotatable));
    exports.EllipseOval = EllipseOval;
    EllipseOval.__name__ = "EllipseOval";
},
/* models/glyphs/hbar.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var box_1 = require(360) /* ./box */;
    var types_1 = require(139) /* ../../core/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var HBarView = /** @class */ (function (_super) {
        tslib_1.__extends(HBarView, _super);
        function HBarView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        HBarView.prototype.scenterxy = function (i) {
            var scx = (this.sleft[i] + this.sright[i]) / 2;
            var scy = this.sy[i];
            return [scx, scy];
        };
        HBarView.prototype._lrtb = function (i) {
            var l = Math.min(this._left[i], this._right[i]);
            var r = Math.max(this._left[i], this._right[i]);
            var t = this._y[i] + 0.5 * this._height[i];
            var b = this._y[i] - 0.5 * this._height[i];
            return [l, r, t, b];
        };
        HBarView.prototype._map_data = function () {
            this.sy = this.renderer.yscale.v_compute(this._y);
            this.sh = this.sdist(this.renderer.yscale, this._y, this._height, "center");
            this.sleft = this.renderer.xscale.v_compute(this._left);
            this.sright = this.renderer.xscale.v_compute(this._right);
            var n = this.sy.length;
            this.stop = new types_1.NumberArray(n);
            this.sbottom = new types_1.NumberArray(n);
            for (var i = 0; i < n; i++) {
                this.stop[i] = this.sy[i] - this.sh[i] / 2;
                this.sbottom[i] = this.sy[i] + this.sh[i] / 2;
            }
            this._clamp_viewport();
        };
        return HBarView;
    }(box_1.BoxView));
    exports.HBarView = HBarView;
    HBarView.__name__ = "HBarView";
    var HBar = /** @class */ (function (_super) {
        tslib_1.__extends(HBar, _super);
        function HBar(attrs) {
            return _super.call(this, attrs) || this;
        }
        HBar.init_HBar = function () {
            this.prototype.default_view = HBarView;
            this.define({
                left: [p.XCoordinateSpec, { value: 0 }],
                y: [p.YCoordinateSpec, { field: "y" }],
                height: [p.NumberSpec, { value: 1 }],
                right: [p.XCoordinateSpec, { field: "right" }],
            });
        };
        return HBar;
    }(box_1.Box));
    exports.HBar = HBar;
    HBar.__name__ = "HBar";
    HBar.init_HBar();
},
/* models/glyphs/box.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var glyph_1 = require(209) /* ./glyph */;
    var utils_1 = require(215) /* ./utils */;
    var selection_1 = require(203) /* ../selections/selection */;
    var BoxView = /** @class */ (function (_super) {
        tslib_1.__extends(BoxView, _super);
        function BoxView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BoxView.prototype.get_anchor_point = function (anchor, i, _spt) {
            var left = Math.min(this.sleft[i], this.sright[i]);
            var right = Math.max(this.sright[i], this.sleft[i]);
            var top = Math.min(this.stop[i], this.sbottom[i]); // screen coordinates !!!
            var bottom = Math.max(this.sbottom[i], this.stop[i]); //
            switch (anchor) {
                case "top_left": return { x: left, y: top };
                case "top_center": return { x: (left + right) / 2, y: top };
                case "top_right": return { x: right, y: top };
                case "bottom_left": return { x: left, y: bottom };
                case "bottom_center": return { x: (left + right) / 2, y: bottom };
                case "bottom_right": return { x: right, y: bottom };
                case "center_left": return { x: left, y: (top + bottom) / 2 };
                case "center": return { x: (left + right) / 2, y: (top + bottom) / 2 };
                case "center_right": return { x: right, y: (top + bottom) / 2 };
                default: return null;
            }
        };
        BoxView.prototype._index_data = function (index) {
            var min = Math.min, max = Math.max;
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                var _a = tslib_1.__read(this._lrtb(i), 4), l = _a[0], r = _a[1], t = _a[2], b = _a[3];
                if (isNaN(l + r + t + b) || !isFinite(l + r + t + b))
                    index.add_empty();
                else
                    index.add(min(l, r), min(t, b), max(r, l), max(t, b));
            }
        };
        BoxView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var _this = this;
            var sleft = _a.sleft, sright = _a.sright, stop = _a.stop, sbottom = _a.sbottom;
            var _loop_1 = function (i) {
                if (isNaN(sleft[i] + stop[i] + sright[i] + sbottom[i]))
                    return "continue";
                ctx.rect(sleft[i], stop[i], sright[i] - sleft[i], sbottom[i] - stop[i]);
                if (this_1.visuals.fill.doit) {
                    this_1.visuals.fill.set_vectorize(ctx, i);
                    ctx.beginPath();
                    ctx.rect(sleft[i], stop[i], sright[i] - sleft[i], sbottom[i] - stop[i]);
                    ctx.fill();
                }
                this_1.visuals.hatch.doit2(ctx, i, function () {
                    ctx.beginPath();
                    ctx.rect(sleft[i], stop[i], sright[i] - sleft[i], sbottom[i] - stop[i]);
                    ctx.fill();
                }, function () { return _this.renderer.request_render(); });
                if (this_1.visuals.line.doit) {
                    this_1.visuals.line.set_vectorize(ctx, i);
                    ctx.beginPath();
                    ctx.rect(sleft[i], stop[i], sright[i] - sleft[i], sbottom[i] - stop[i]);
                    ctx.stroke();
                }
            };
            var this_1 = this;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    _loop_1(i);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        // We need to clamp the endpoints inside the viewport, because various browser canvas
        // implementations have issues drawing rects with enpoints far outside the viewport
        BoxView.prototype._clamp_viewport = function () {
            var hr = this.renderer.plot_view.frame.bbox.h_range;
            var vr = this.renderer.plot_view.frame.bbox.v_range;
            var n = this.stop.length;
            for (var i = 0; i < n; i++) {
                this.stop[i] = Math.max(this.stop[i], vr.start);
                this.sbottom[i] = Math.min(this.sbottom[i], vr.end);
                this.sleft[i] = Math.max(this.sleft[i], hr.start);
                this.sright[i] = Math.min(this.sright[i], hr.end);
            }
        };
        BoxView.prototype._hit_rect = function (geometry) {
            return this._hit_rect_against_index(geometry);
        };
        BoxView.prototype._hit_point = function (geometry) {
            var sx = geometry.sx, sy = geometry.sy;
            var x = this.renderer.xscale.invert(sx);
            var y = this.renderer.yscale.invert(sy);
            var indices = tslib_1.__spread(this.index.indices({ x0: x, y0: y, x1: x, y1: y }));
            return new selection_1.Selection({ indices: indices });
        };
        BoxView.prototype._hit_span = function (geometry) {
            var sx = geometry.sx, sy = geometry.sy;
            var indices;
            if (geometry.direction == 'v') {
                var y = this.renderer.yscale.invert(sy);
                var hr = this.renderer.plot_view.frame.bbox.h_range;
                var _a = tslib_1.__read(this.renderer.xscale.r_invert(hr.start, hr.end), 2), x0 = _a[0], x1 = _a[1];
                indices = tslib_1.__spread(this.index.indices({ x0: x0, y0: y, x1: x1, y1: y }));
            }
            else {
                var x = this.renderer.xscale.invert(sx);
                var vr = this.renderer.plot_view.frame.bbox.v_range;
                var _b = tslib_1.__read(this.renderer.yscale.r_invert(vr.start, vr.end), 2), y0 = _b[0], y1 = _b[1];
                indices = tslib_1.__spread(this.index.indices({ x0: x, y0: y0, x1: x, y1: y1 }));
            }
            return new selection_1.Selection({ indices: indices });
        };
        BoxView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_area_legend(this.visuals, ctx, bbox, index);
        };
        return BoxView;
    }(glyph_1.GlyphView));
    exports.BoxView = BoxView;
    BoxView.__name__ = "BoxView";
    var Box = /** @class */ (function (_super) {
        tslib_1.__extends(Box, _super);
        function Box(attrs) {
            return _super.call(this, attrs) || this;
        }
        Box.init_Box = function () {
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector, property_mixins_1.HatchVector]);
        };
        return Box;
    }(glyph_1.Glyph));
    exports.Box = Box;
    Box.__name__ = "Box";
    Box.init_Box();
},
/* models/glyphs/hex_tile.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var glyph_1 = require(209) /* ./glyph */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var types_1 = require(139) /* ../../core/types */;
    var projections_1 = require(152) /* ../../core/util/projections */;
    var utils_1 = require(215) /* ./utils */;
    var selection_1 = require(203) /* ../selections/selection */;
    var HexTileView = /** @class */ (function (_super) {
        tslib_1.__extends(HexTileView, _super);
        function HexTileView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        HexTileView.prototype.scenterxy = function (i) {
            var scx = this.sx[i];
            var scy = this.sy[i];
            return [scx, scy];
        };
        HexTileView.prototype._set_data = function () {
            var n = this._q.length;
            var _a = this.model, orientation = _a.orientation, size = _a.size, aspect_scale = _a.aspect_scale;
            this._x = new types_1.NumberArray(n);
            this._y = new types_1.NumberArray(n);
            var sqrt3 = Math.sqrt(3);
            if (orientation == "pointytop") {
                for (var i = 0; i < n; i++) {
                    this._x[i] = size * sqrt3 * (this._q[i] + this._r[i] / 2) / aspect_scale;
                    this._y[i] = -size * 3 / 2 * this._r[i];
                }
            }
            else {
                for (var i = 0; i < n; i++) {
                    this._x[i] = size * 3 / 2 * this._q[i];
                    this._y[i] = -size * sqrt3 * (this._r[i] + this._q[i] / 2) * aspect_scale;
                }
            }
        };
        HexTileView.prototype._project_data = function () {
            projections_1.inplace.project_xy(this._x, this._y);
        };
        HexTileView.prototype._index_data = function (index) {
            var _a;
            var ysize = this.model.size;
            var xsize = Math.sqrt(3) * ysize / 2;
            if (this.model.orientation == "flattop") {
                _a = tslib_1.__read([ysize, xsize], 2), xsize = _a[0], ysize = _a[1];
                ysize *= this.model.aspect_scale;
            }
            else
                xsize /= this.model.aspect_scale;
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                var x = this._x[i];
                var y = this._y[i];
                if (isNaN(x + y) || !isFinite(x + y))
                    index.add_empty();
                else
                    index.add(x - xsize, y - ysize, x + xsize, y + ysize);
            }
        };
        // overriding map_data instead of _map_data because the default automatic mappings
        // for other glyphs (with cartesian coordinates) is not useful
        HexTileView.prototype.map_data = function () {
            var _a, _b;
            _a = tslib_1.__read(this.renderer.coordinates.map_to_screen(this._x, this._y), 2), this.sx = _a[0], this.sy = _a[1];
            _b = tslib_1.__read(this._get_unscaled_vertices(), 2), this.svx = _b[0], this.svy = _b[1];
        };
        HexTileView.prototype._get_unscaled_vertices = function () {
            var size = this.model.size;
            var aspect_scale = this.model.aspect_scale;
            if (this.model.orientation == "pointytop") {
                var rscale = this.renderer.yscale;
                var hscale = this.renderer.xscale;
                var r = Math.abs(rscale.compute(0) - rscale.compute(size)); // assumes linear scale
                var h = Math.sqrt(3) / 2 * Math.abs(hscale.compute(0) - hscale.compute(size)) / aspect_scale; // assumes linear scale
                var r2 = r / 2.0;
                var svx = [0, -h, -h, 0, h, h];
                var svy = [r, r2, -r2, -r, -r2, r2];
                return [svx, svy];
            }
            else {
                var rscale = this.renderer.xscale;
                var hscale = this.renderer.yscale;
                var r = Math.abs(rscale.compute(0) - rscale.compute(size)); // assumes linear scale
                var h = Math.sqrt(3) / 2 * Math.abs(hscale.compute(0) - hscale.compute(size)) * aspect_scale; // assumes linear scale
                var r2 = r / 2.0;
                var svx = [r, r2, -r2, -r, -r2, r2];
                var svy = [0, -h, -h, 0, h, h];
                return [svx, svy];
            }
        };
        HexTileView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx = _a.sx, sy = _a.sy, svx = _a.svx, svy = _a.svy, _scale = _a._scale;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (isNaN(sx[i] + sy[i] + _scale[i]))
                        continue;
                    ctx.translate(sx[i], sy[i]);
                    ctx.beginPath();
                    for (var j = 0; j < 6; j++) {
                        ctx.lineTo(svx[j] * _scale[i], svy[j] * _scale[i]);
                    }
                    ctx.closePath();
                    ctx.translate(-sx[i], -sy[i]);
                    if (this.visuals.fill.doit) {
                        this.visuals.fill.set_vectorize(ctx, i);
                        ctx.fill();
                    }
                    if (this.visuals.line.doit) {
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.stroke();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        HexTileView.prototype._hit_point = function (geometry) {
            var e_2, _a;
            var sx = geometry.sx, sy = geometry.sy;
            var x = this.renderer.xscale.invert(sx);
            var y = this.renderer.yscale.invert(sy);
            var candidates = this.index.indices({ x0: x, y0: y, x1: x, y1: y });
            var indices = [];
            try {
                for (var candidates_1 = tslib_1.__values(candidates), candidates_1_1 = candidates_1.next(); !candidates_1_1.done; candidates_1_1 = candidates_1.next()) {
                    var i = candidates_1_1.value;
                    if (hittest.point_in_poly(sx - this.sx[i], sy - this.sy[i], this.svx, this.svy)) {
                        indices.push(i);
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (candidates_1_1 && !candidates_1_1.done && (_a = candidates_1.return))
                        _a.call(candidates_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        HexTileView.prototype._hit_span = function (geometry) {
            var sx = geometry.sx, sy = geometry.sy;
            var indices;
            if (geometry.direction == 'v') {
                var y = this.renderer.yscale.invert(sy);
                var hr = this.renderer.plot_view.frame.bbox.h_range;
                var _a = tslib_1.__read(this.renderer.xscale.r_invert(hr.start, hr.end), 2), x0 = _a[0], x1 = _a[1];
                indices = tslib_1.__spread(this.index.indices({ x0: x0, y0: y, x1: x1, y1: y }));
            }
            else {
                var x = this.renderer.xscale.invert(sx);
                var vr = this.renderer.plot_view.frame.bbox.v_range;
                var _b = tslib_1.__read(this.renderer.yscale.r_invert(vr.start, vr.end), 2), y0 = _b[0], y1 = _b[1];
                indices = tslib_1.__spread(this.index.indices({ x0: x, y0: y0, x1: x, y1: y1 }));
            }
            return new selection_1.Selection({ indices: indices });
        };
        HexTileView.prototype._hit_rect = function (geometry) {
            var sx0 = geometry.sx0, sx1 = geometry.sx1, sy0 = geometry.sy0, sy1 = geometry.sy1;
            var _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
            var _b = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _b[0], y1 = _b[1];
            var indices = tslib_1.__spread(this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 }));
            return new selection_1.Selection({ indices: indices });
        };
        HexTileView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_area_legend(this.visuals, ctx, bbox, index);
        };
        return HexTileView;
    }(glyph_1.GlyphView));
    exports.HexTileView = HexTileView;
    HexTileView.__name__ = "HexTileView";
    var HexTile = /** @class */ (function (_super) {
        tslib_1.__extends(HexTile, _super);
        function HexTile(attrs) {
            return _super.call(this, attrs) || this;
        }
        HexTile.init_HexTile = function () {
            this.prototype.default_view = HexTileView;
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector]);
            this.define({
                r: [p.NumberSpec],
                q: [p.NumberSpec],
                size: [p.Number, 1.0],
                aspect_scale: [p.Number, 1.0],
                scale: [p.NumberSpec, 1.0],
                orientation: [p.HexTileOrientation, "pointytop"],
            });
            this.override({ line_color: null });
        };
        return HexTile;
    }(glyph_1.Glyph));
    exports.HexTile = HexTile;
    HexTile.__name__ = "HexTile";
    HexTile.init_HexTile();
},
/* models/glyphs/image.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var image_base_1 = require(363) /* ./image_base */;
    var linear_color_mapper_1 = require(256) /* ../mappers/linear_color_mapper */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var ImageView = /** @class */ (function (_super) {
        tslib_1.__extends(ImageView, _super);
        function ImageView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ImageView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.color_mapper.change, function () { return _this._update_image(); });
        };
        ImageView.prototype._update_image = function () {
            // Only reset image_data if already initialized
            if (this.image_data != null) {
                this._set_data(null);
                this.renderer.plot_view.request_render();
            }
        };
        ImageView.prototype._flat_img_to_buf8 = function (img) {
            var cmap = this.model.color_mapper.rgba_mapper;
            return cmap.v_compute(img);
        };
        return ImageView;
    }(image_base_1.ImageBaseView));
    exports.ImageView = ImageView;
    ImageView.__name__ = "ImageView";
    // NOTE: this needs to be redefined here, because palettes are located in bokeh-api.js bundle
    var Greys9 = function () { return ["#000000", "#252525", "#525252", "#737373", "#969696", "#bdbdbd", "#d9d9d9", "#f0f0f0", "#ffffff"]; };
    var Image = /** @class */ (function (_super) {
        tslib_1.__extends(Image, _super);
        function Image(attrs) {
            return _super.call(this, attrs) || this;
        }
        Image.init_Image = function () {
            this.prototype.default_view = ImageView;
            this.define({
                color_mapper: [p.Instance, function () { return new linear_color_mapper_1.LinearColorMapper({ palette: Greys9() }); }],
            });
        };
        return Image;
    }(image_base_1.ImageBase));
    exports.Image = Image;
    Image.__name__ = "Image";
    Image.init_Image();
},
/* models/glyphs/image_base.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var types_1 = require(139) /* ../../core/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var selection_1 = require(203) /* ../selections/selection */;
    var array_1 = require(124) /* ../../core/util/array */;
    var ndarray_1 = require(145) /* ../../core/util/ndarray */;
    var assert_1 = require(126) /* ../../core/util/assert */;
    var ImageBaseView = /** @class */ (function (_super) {
        tslib_1.__extends(ImageBaseView, _super);
        function ImageBaseView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ImageBaseView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.properties.global_alpha.change, function () { return _this.renderer.request_render(); });
        };
        ImageBaseView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var image_data = _a.image_data, sx = _a.sx, sy = _a.sy, sw = _a.sw, sh = _a.sh;
            var old_smoothing = ctx.getImageSmoothingEnabled();
            ctx.setImageSmoothingEnabled(false);
            ctx.globalAlpha = this.model.global_alpha;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (image_data[i] == null || isNaN(sx[i] + sy[i] + sw[i] + sh[i]))
                        continue;
                    var y_offset = sy[i];
                    ctx.translate(0, y_offset);
                    ctx.scale(1, -1);
                    ctx.translate(0, -y_offset);
                    ctx.drawImage(image_data[i], sx[i] | 0, sy[i] | 0, sw[i], sh[i]);
                    ctx.translate(0, y_offset);
                    ctx.scale(1, -1);
                    ctx.translate(0, -y_offset);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            ctx.setImageSmoothingEnabled(old_smoothing);
        };
        ImageBaseView.prototype._set_data = function (indices) {
            this._set_width_heigh_data();
            for (var i = 0, end = this._image.length; i < end; i++) {
                if (indices != null && indices.indexOf(i) < 0)
                    continue;
                var img = this._image[i];
                var flat_img = void 0;
                if (ndarray_1.is_NDArray(img)) {
                    assert_1.assert(img.dimension == 2, "expected a 2D array");
                    flat_img = img;
                    this._height[i] = img.shape[0];
                    this._width[i] = img.shape[1];
                }
                else {
                    flat_img = array_1.concat(img);
                    this._height[i] = img.length;
                    this._width[i] = img[0].length;
                }
                var buf8 = this._flat_img_to_buf8(flat_img);
                this._set_image_data_from_buffer(i, buf8);
            }
        };
        ImageBaseView.prototype._index_data = function (index) {
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                var _a = tslib_1.__read(this._lrtb(i), 4), l = _a[0], r = _a[1], t = _a[2], b = _a[3];
                if (isNaN(l + r + t + b) || !isFinite(l + r + t + b))
                    index.add_empty();
                else
                    index.add(l, b, r, t);
            }
        };
        ImageBaseView.prototype._lrtb = function (i) {
            var xr = this.renderer.xscale.source_range;
            var x1 = this._x[i];
            var x2 = xr.is_reversed ? x1 - this._dw[i] : x1 + this._dw[i];
            var yr = this.renderer.yscale.source_range;
            var y1 = this._y[i];
            var y2 = yr.is_reversed ? y1 - this._dh[i] : y1 + this._dh[i];
            var _a = tslib_1.__read(x1 < x2 ? [x1, x2] : [x2, x1], 2), l = _a[0], r = _a[1];
            var _b = tslib_1.__read(y1 < y2 ? [y1, y2] : [y2, y1], 2), b = _b[0], t = _b[1];
            return [l, r, t, b];
        };
        ImageBaseView.prototype._set_width_heigh_data = function () {
            if (this.image_data == null || this.image_data.length != this._image.length)
                this.image_data = new Array(this._image.length);
            if (this._width == null || this._width.length != this._image.length)
                this._width = new types_1.NumberArray(this._image.length);
            if (this._height == null || this._height.length != this._image.length)
                this._height = new types_1.NumberArray(this._image.length);
        };
        ImageBaseView.prototype._get_or_create_canvas = function (i) {
            var _image_data = this.image_data[i];
            if (_image_data != null && _image_data.width == this._width[i] &&
                _image_data.height == this._height[i])
                return _image_data;
            else {
                var canvas = document.createElement('canvas');
                canvas.width = this._width[i];
                canvas.height = this._height[i];
                return canvas;
            }
        };
        ImageBaseView.prototype._set_image_data_from_buffer = function (i, buf8) {
            var canvas = this._get_or_create_canvas(i);
            var ctx = canvas.getContext('2d');
            var image_data = ctx.getImageData(0, 0, this._width[i], this._height[i]);
            image_data.data.set(buf8);
            ctx.putImageData(image_data, 0, 0);
            this.image_data[i] = canvas;
        };
        ImageBaseView.prototype._map_data = function () {
            switch (this.model.properties.dw.units) {
                case "data": {
                    this.sw = this.sdist(this.renderer.xscale, this._x, this._dw, 'edge', this.model.dilate);
                    break;
                }
                case "screen": {
                    this.sw = this._dw;
                    break;
                }
            }
            switch (this.model.properties.dh.units) {
                case "data": {
                    this.sh = this.sdist(this.renderer.yscale, this._y, this._dh, 'edge', this.model.dilate);
                    break;
                }
                case "screen": {
                    this.sh = this._dh;
                    break;
                }
            }
        };
        ImageBaseView.prototype._image_index = function (index, x, y) {
            var _a = tslib_1.__read(this._lrtb(index), 4), l = _a[0], r = _a[1], t = _a[2], b = _a[3];
            var width = this._width[index];
            var height = this._height[index];
            var dx = (r - l) / width;
            var dy = (t - b) / height;
            var dim1 = Math.floor((x - l) / dx);
            var dim2 = Math.floor((y - b) / dy);
            if (this.renderer.xscale.source_range.is_reversed)
                dim1 = width - dim1 - 1;
            if (this.renderer.yscale.source_range.is_reversed)
                dim2 = height - dim2 - 1;
            return { index: index, dim1: dim1, dim2: dim2, flat_index: dim2 * width + dim1 };
        };
        ImageBaseView.prototype._hit_point = function (geometry) {
            var e_2, _a;
            var sx = geometry.sx, sy = geometry.sy;
            var x = this.renderer.xscale.invert(sx);
            var y = this.renderer.yscale.invert(sy);
            var candidates = this.index.indices({ x0: x, x1: x, y0: y, y1: y });
            var result = new selection_1.Selection();
            try {
                for (var candidates_1 = tslib_1.__values(candidates), candidates_1_1 = candidates_1.next(); !candidates_1_1.done; candidates_1_1 = candidates_1.next()) {
                    var index = candidates_1_1.value;
                    if (sx != Infinity && sy != Infinity) {
                        result.image_indices.push(this._image_index(index, x, y));
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (candidates_1_1 && !candidates_1_1.done && (_a = candidates_1.return))
                        _a.call(candidates_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return result;
        };
        return ImageBaseView;
    }(xy_glyph_1.XYGlyphView));
    exports.ImageBaseView = ImageBaseView;
    ImageBaseView.__name__ = "ImageBaseView";
    var ImageBase = /** @class */ (function (_super) {
        tslib_1.__extends(ImageBase, _super);
        function ImageBase(attrs) {
            return _super.call(this, attrs) || this;
        }
        ImageBase.init_ImageBase = function () {
            this.define({
                image: [p.NDArraySpec],
                dw: [p.DistanceSpec],
                dh: [p.DistanceSpec],
                dilate: [p.Boolean, false],
                global_alpha: [p.Number, 1.0],
            });
        };
        return ImageBase;
    }(xy_glyph_1.XYGlyph));
    exports.ImageBase = ImageBase;
    ImageBase.__name__ = "ImageBase";
    ImageBase.init_ImageBase();
},
/* models/glyphs/image_rgba.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var image_base_1 = require(363) /* ./image_base */;
    var types_1 = require(123) /* ../../core/util/types */;
    var ImageRGBAView = /** @class */ (function (_super) {
        tslib_1.__extends(ImageRGBAView, _super);
        function ImageRGBAView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ImageRGBAView.prototype._flat_img_to_buf8 = function (img) {
            var array;
            if (types_1.isArray(img)) {
                array = new Uint32Array(img);
            }
            else {
                array = img;
            }
            return new Uint8Array(array.buffer);
        };
        return ImageRGBAView;
    }(image_base_1.ImageBaseView));
    exports.ImageRGBAView = ImageRGBAView;
    ImageRGBAView.__name__ = "ImageRGBAView";
    var ImageRGBA = /** @class */ (function (_super) {
        tslib_1.__extends(ImageRGBA, _super);
        function ImageRGBA(attrs) {
            return _super.call(this, attrs) || this;
        }
        ImageRGBA.init_ImageRGBA = function () {
            this.prototype.default_view = ImageRGBAView;
        };
        return ImageRGBA;
    }(image_base_1.ImageBase));
    exports.ImageRGBA = ImageRGBA;
    ImageRGBA.__name__ = "ImageRGBA";
    ImageRGBA.init_ImageRGBA();
},
/* models/glyphs/image_url.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var types_1 = require(139) /* ../../core/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var image_1 = require(366) /* ../../core/util/image */;
    var ImageURLView = /** @class */ (function (_super) {
        tslib_1.__extends(ImageURLView, _super);
        function ImageURLView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this._images_rendered = false;
            return _this;
        }
        ImageURLView.prototype.initialize = function () {
            var _this = this;
            _super.prototype.initialize.call(this);
            this.connect(this.model.properties.global_alpha.change, function () { return _this.renderer.request_render(); });
        };
        ImageURLView.prototype._index_data = function (index) {
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                // TODO: add a proper implementation (same as ImageBase?)
                index.add_empty();
            }
        };
        ImageURLView.prototype._set_data = function () {
            var _a, _b;
            var _this = this;
            if (this.image == null || this.image.length != this._url.length)
                this.image = arrayable_1.map(this._url, function () { return null; });
            var _c = this.model, retry_attempts = _c.retry_attempts, retry_timeout = _c.retry_timeout;
            var _loop_1 = function (i, end) {
                var url = this_1._url[i];
                if (url == null || url == "")
                    return "continue";
                new image_1.ImageLoader(url, {
                    loaded: function (image) {
                        _this.image[i] = image;
                        _this.renderer.request_render();
                    },
                    attempts: retry_attempts + 1,
                    timeout: retry_timeout,
                });
            };
            var this_1 = this;
            for (var i = 0, end = this._url.length; i < end; i++) {
                _loop_1(i, end);
            }
            var w_data = this.model.properties.w.units == "data";
            var h_data = this.model.properties.h.units == "data";
            var n = this._x.length;
            var xs = new types_1.NumberArray(w_data ? 2 * n : n);
            var ys = new types_1.NumberArray(h_data ? 2 * n : n);
            var anchor = this.model.anchor;
            function x0x1(x, w) {
                switch (anchor) {
                    case "top_left":
                    case "bottom_left":
                    case "center_left":
                        return [x, x + w];
                    case "top_center":
                    case "bottom_center":
                    case "center":
                        return [x - w / 2, x + w / 2];
                    case "top_right":
                    case "bottom_right":
                    case "center_right":
                        return [x - w, x];
                }
            }
            function y0y1(y, h) {
                switch (anchor) {
                    case "top_left":
                    case "top_center":
                    case "top_right":
                        return [y, y - h];
                    case "bottom_left":
                    case "bottom_center":
                    case "bottom_right":
                        return [y + h, y];
                    case "center_left":
                    case "center":
                    case "center_right":
                        return [y + h / 2, y - h / 2];
                }
            }
            // if the width/height are in screen units, don't try to include them in bounds
            if (w_data) {
                for (var i = 0; i < n; i++) {
                    _a = tslib_1.__read(x0x1(this._x[i], this._w[i]), 2), xs[i] = _a[0], xs[n + i] = _a[1];
                }
            }
            else
                xs.set(this._x, 0);
            if (h_data) {
                for (var i = 0; i < n; i++) {
                    _b = tslib_1.__read(y0y1(this._y[i], this._h[i]), 2), ys[i] = _b[0], ys[n + i] = _b[1];
                }
            }
            else
                ys.set(this._y, 0);
            var _d = tslib_1.__read(arrayable_1.minmax(xs), 2), x0 = _d[0], x1 = _d[1];
            var _e = tslib_1.__read(arrayable_1.minmax(ys), 2), y0 = _e[0], y1 = _e[1];
            this._bounds_rect = { x0: x0, x1: x1, y0: y0, y1: y1 };
        };
        ImageURLView.prototype.has_finished = function () {
            return _super.prototype.has_finished.call(this) && this._images_rendered == true;
        };
        ImageURLView.prototype._map_data = function () {
            // Better to check this.model.w and this.model.h for null since the set_data
            // machinery will have converted this._w and this._w to lists of null
            var ws = this.model.w != null ? this._w : arrayable_1.map(this._x, function () { return NaN; });
            var hs = this.model.h != null ? this._h : arrayable_1.map(this._x, function () { return NaN; });
            switch (this.model.properties.w.units) {
                case "data": {
                    this.sw = this.sdist(this.renderer.xscale, this._x, ws, "edge", this.model.dilate);
                    break;
                }
                case "screen": {
                    this.sw = ws;
                    break;
                }
            }
            switch (this.model.properties.h.units) {
                case "data": {
                    this.sh = this.sdist(this.renderer.yscale, this._y, hs, "edge", this.model.dilate);
                    break;
                }
                case "screen": {
                    this.sh = hs;
                    break;
                }
            }
        };
        ImageURLView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var image = _a.image, sx = _a.sx, sy = _a.sy, sw = _a.sw, sh = _a.sh, _angle = _a._angle;
            // TODO (bev): take actual border width into account when clipping
            var frame = this.renderer.plot_view.frame;
            ctx.rect(frame.bbox.left + 1, frame.bbox.top + 1, frame.bbox.width - 2, frame.bbox.height - 2);
            ctx.clip();
            var finished = true;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (isNaN(sx[i] + sy[i] + _angle[i]))
                        continue;
                    var img = image[i];
                    if (img == null) {
                        finished = false;
                        continue;
                    }
                    this._render_image(ctx, i, img, sx, sy, sw, sh, _angle);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            if (finished && !this._images_rendered) {
                this._images_rendered = true;
                this.notify_finished();
            }
        };
        ImageURLView.prototype._final_sx_sy = function (anchor, sx, sy, sw, sh) {
            switch (anchor) {
                case 'top_left': return [sx, sy];
                case 'top_center': return [sx - (sw / 2), sy];
                case 'top_right': return [sx - sw, sy];
                case 'center_right': return [sx - sw, sy - (sh / 2)];
                case 'bottom_right': return [sx - sw, sy - sh];
                case 'bottom_center': return [sx - (sw / 2), sy - sh];
                case 'bottom_left': return [sx, sy - sh];
                case 'center_left': return [sx, sy - (sh / 2)];
                case 'center': return [sx - (sw / 2), sy - (sh / 2)];
            }
        };
        ImageURLView.prototype._render_image = function (ctx, i, image, sx, sy, sw, sh, angle) {
            if (isNaN(sw[i]))
                sw[i] = image.width;
            if (isNaN(sh[i]))
                sh[i] = image.height;
            var anchor = this.model.anchor;
            var _a = tslib_1.__read(this._final_sx_sy(anchor, sx[i], sy[i], sw[i], sh[i]), 2), sxi = _a[0], syi = _a[1];
            ctx.save();
            ctx.globalAlpha = this.model.global_alpha;
            var sw2 = sw[i] / 2;
            var sh2 = sh[i] / 2;
            if (angle[i]) {
                ctx.translate(sxi, syi);
                //rotation about center of image
                ctx.translate(sw2, sh2);
                ctx.rotate(angle[i]);
                ctx.translate(-sw2, -sh2);
                ctx.drawImage(image, 0, 0, sw[i], sh[i]);
                ctx.translate(sw2, sh2);
                ctx.rotate(-angle[i]);
                ctx.translate(-sw2, -sh2);
                ctx.translate(-sxi, -syi);
            }
            else
                ctx.drawImage(image, sxi, syi, sw[i], sh[i]);
            ctx.restore();
        };
        ImageURLView.prototype.bounds = function () {
            return this._bounds_rect;
        };
        return ImageURLView;
    }(xy_glyph_1.XYGlyphView));
    exports.ImageURLView = ImageURLView;
    ImageURLView.__name__ = "ImageURLView";
    var ImageURL = /** @class */ (function (_super) {
        tslib_1.__extends(ImageURL, _super);
        function ImageURL(attrs) {
            return _super.call(this, attrs) || this;
        }
        ImageURL.init_ImageURL = function () {
            this.prototype.default_view = ImageURLView;
            this.define({
                url: [p.StringSpec],
                anchor: [p.Anchor, 'top_left'],
                global_alpha: [p.Number, 1.0],
                angle: [p.AngleSpec, 0],
                w: [p.DistanceSpec],
                h: [p.DistanceSpec],
                dilate: [p.Boolean, false],
                retry_attempts: [p.Number, 0],
                retry_timeout: [p.Number, 0],
            });
        };
        return ImageURL;
    }(xy_glyph_1.XYGlyph));
    exports.ImageURL = ImageURL;
    ImageURL.__name__ = "ImageURL";
    ImageURL.init_ImageURL();
},
/* core/util/image.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var logging_1 = require(134) /* ../logging */;
    var ImageLoader = /** @class */ (function () {
        function ImageLoader(url, options) {
            var _this = this;
            if (options === void 0) {
                options = {};
            }
            this._image = new Image();
            this._finished = false;
            var _a = options.attempts, attempts = _a === void 0 ? 1 : _a, _b = options.timeout, timeout = _b === void 0 ? 1 : _b;
            this.promise = new Promise(function (resolve, _reject) {
                _this._image.crossOrigin = "anonymous";
                var retries = 0;
                _this._image.onerror = function () {
                    if (++retries == attempts) {
                        var message = "unable to load " + url + " image after " + attempts + " attempts";
                        logging_1.logger.warn(message);
                        if (_this._image.crossOrigin != null) {
                            logging_1.logger.warn("attempting to load " + url + " without a cross origin policy");
                            _this._image.crossOrigin = null;
                            retries = 0;
                        }
                        else {
                            if (options.failed != null)
                                options.failed();
                            return; // XXX reject(new Error(message))
                        }
                    }
                    setTimeout(function () { return _this._image.src = url; }, timeout);
                };
                _this._image.onload = function () {
                    _this._finished = true;
                    if (options.loaded != null)
                        options.loaded(_this._image);
                    resolve(_this._image);
                };
                _this._image.src = url;
            });
        }
        Object.defineProperty(ImageLoader.prototype, "finished", {
            get: function () {
                return this._finished;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ImageLoader.prototype, "image", {
            get: function () {
                return this._image;
            },
            enumerable: true,
            configurable: true
        });
        return ImageLoader;
    }());
    exports.ImageLoader = ImageLoader;
    ImageLoader.__name__ = "ImageLoader";
},
/* models/glyphs/multi_line.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var projections_1 = require(152) /* ../../core/util/projections */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var object_1 = require(128) /* ../../core/util/object */;
    var glyph_1 = require(209) /* ./glyph */;
    var utils_1 = require(215) /* ./utils */;
    var selection_1 = require(203) /* ../selections/selection */;
    var MultiLineView = /** @class */ (function (_super) {
        tslib_1.__extends(MultiLineView, _super);
        function MultiLineView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        MultiLineView.prototype._project_data = function () {
            projections_1.inplace.project_xy(this._xs.array, this._ys.array);
        };
        MultiLineView.prototype._index_data = function (index) {
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                var xsi = this._xs.get(i);
                if (xsi.length == 0) {
                    index.add_empty();
                    continue;
                }
                var ysi = this._ys.get(i);
                if (ysi.length == 0) {
                    index.add_empty();
                    continue;
                }
                var _a = tslib_1.__read(arrayable_1.minmax(xsi), 2), x0 = _a[0], x1 = _a[1];
                var _b = tslib_1.__read(arrayable_1.minmax(ysi), 2), y0 = _b[0], y1 = _b[1];
                index.add(x0, y0, x1, y1);
            }
        };
        MultiLineView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sxs = _a.sxs, sys = _a.sys;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    var sx = sxs.get(i);
                    var sy = sys.get(i);
                    this.visuals.line.set_vectorize(ctx, i);
                    for (var j = 0, end = sx.length; j < end; j++) {
                        if (j == 0) {
                            ctx.beginPath();
                            ctx.moveTo(sx[j], sy[j]);
                            continue;
                        }
                        else if (isNaN(sx[j]) || isNaN(sy[j])) {
                            ctx.stroke();
                            ctx.beginPath();
                            continue;
                        }
                        else
                            ctx.lineTo(sx[j], sy[j]);
                    }
                    ctx.stroke();
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        MultiLineView.prototype._hit_point = function (geometry) {
            var point = { x: geometry.sx, y: geometry.sy };
            var shortest = 9999;
            var hits = new Map();
            for (var i = 0, end = this.sxs.length; i < end; i++) {
                var threshold = Math.max(2, this.visuals.line.cache_select('line_width', i) / 2);
                var sxsi = this.sxs.get(i);
                var sysi = this.sys.get(i);
                var points = null;
                for (var j = 0, endj = sxsi.length - 1; j < endj; j++) {
                    var p0 = { x: sxsi[j], y: sysi[j] };
                    var p1 = { x: sxsi[j + 1], y: sysi[j + 1] };
                    var dist = hittest.dist_to_segment(point, p0, p1);
                    if (dist < threshold && dist < shortest) {
                        shortest = dist;
                        points = [j];
                    }
                }
                if (points != null) {
                    hits.set(i, points);
                }
            }
            return new selection_1.Selection({
                indices: tslib_1.__spread(hits.keys()),
                multiline_indices: object_1.to_object(hits),
            });
        };
        MultiLineView.prototype._hit_span = function (geometry) {
            var sx = geometry.sx, sy = geometry.sy;
            var val;
            var vs;
            if (geometry.direction == 'v') {
                val = this.renderer.yscale.invert(sy);
                vs = this._ys;
            }
            else {
                val = this.renderer.xscale.invert(sx);
                vs = this._xs;
            }
            var hits = new Map();
            for (var i = 0, end = vs.length; i < end; i++) {
                var vsi = vs.get(i);
                var points = [];
                for (var j = 0, endj = vsi.length - 1; j < endj; j++) {
                    if (vsi[j] <= val && val <= vsi[j + 1])
                        points.push(j);
                }
                if (points.length > 0) {
                    hits.set(i, points);
                }
            }
            return new selection_1.Selection({
                indices: tslib_1.__spread(hits.keys()),
                multiline_indices: object_1.to_object(hits),
            });
        };
        MultiLineView.prototype.get_interpolation_hit = function (i, point_i, geometry) {
            var xsi = this._xs.get(i);
            var ysi = this._ys.get(i);
            var x2 = xsi[point_i];
            var y2 = ysi[point_i];
            var x3 = xsi[point_i + 1];
            var y3 = ysi[point_i + 1];
            return utils_1.line_interpolation(this.renderer, geometry, x2, y2, x3, y3);
        };
        MultiLineView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_line_legend(this.visuals, ctx, bbox, index);
        };
        MultiLineView.prototype.scenterxy = function () {
            throw new Error(this + ".scenterxy() is not implemented");
        };
        return MultiLineView;
    }(glyph_1.GlyphView));
    exports.MultiLineView = MultiLineView;
    MultiLineView.__name__ = "MultiLineView";
    var MultiLine = /** @class */ (function (_super) {
        tslib_1.__extends(MultiLine, _super);
        function MultiLine(attrs) {
            return _super.call(this, attrs) || this;
        }
        MultiLine.init_MultiLine = function () {
            this.prototype.default_view = MultiLineView;
            this.define({
                xs: [p.XCoordinateSeqSpec, { field: "xs" }],
                ys: [p.YCoordinateSeqSpec, { field: "ys" }],
            });
            this.mixins(property_mixins_1.LineVector);
        };
        return MultiLine;
    }(glyph_1.Glyph));
    exports.MultiLine = MultiLine;
    MultiLine.__name__ = "MultiLine";
    MultiLine.init_MultiLine();
},
/* models/glyphs/multi_polygons.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var spatial_1 = require(210) /* ../../core/util/spatial */;
    var glyph_1 = require(209) /* ./glyph */;
    var utils_1 = require(215) /* ./utils */;
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var arrayable_2 = require(127) /* ../../core/util/arrayable */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var selection_1 = require(203) /* ../selections/selection */;
    var assert_1 = require(126) /* ../../core/util/assert */;
    var MultiPolygonsView = /** @class */ (function (_super) {
        tslib_1.__extends(MultiPolygonsView, _super);
        function MultiPolygonsView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        MultiPolygonsView.prototype._project_data = function () {
            // TODO
        };
        MultiPolygonsView.prototype._index_data = function (index) {
            var min = Math.min, max = Math.max;
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                var xsi = this._xs[i];
                var ysi = this._ys[i];
                if (xsi.length == 0 || ysi.length == 0) {
                    index.add_empty();
                    continue;
                }
                var xi0 = +Infinity;
                var xi1 = -Infinity;
                var yi0 = +Infinity;
                var yi1 = -Infinity;
                for (var j = 0, endj = xsi.length; j < endj; j++) {
                    var xsij = xsi[j][0]; // do not use holes
                    var ysij = ysi[j][0]; // do not use holes
                    if (xsij.length != 0 && ysij.length != 0) {
                        var _a = tslib_1.__read(arrayable_1.minmax(xsij), 2), xij0 = _a[0], xij1 = _a[1];
                        var _b = tslib_1.__read(arrayable_1.minmax(ysij), 2), yij0 = _b[0], yij1 = _b[1];
                        xi0 = min(xi0, xij0);
                        xi1 = max(xi1, xij1);
                        yi0 = min(yi0, yij0);
                        yi1 = max(yi1, yij1);
                    }
                }
                if (!isFinite(xi0 + xi1 + yi0 + yi1))
                    index.add_empty();
                else
                    index.add(xi0, yi0, xi1, yi1);
            }
            this._hole_index = this._index_hole_data();
        };
        MultiPolygonsView.prototype._index_hole_data = function () {
            var min = Math.min, max = Math.max;
            var data_size = this.data_size;
            var index = new spatial_1.SpatialIndex(data_size);
            for (var i = 0; i < data_size; i++) {
                var xsi = this._xs[i];
                var ysi = this._ys[i];
                if (xsi.length == 0 || ysi.length == 0) {
                    index.add_empty();
                    continue;
                }
                var xi0 = +Infinity;
                var xi1 = -Infinity;
                var yi0 = +Infinity;
                var yi1 = -Infinity;
                for (var j = 0, endj = xsi.length; j < endj; j++) {
                    var xsij = xsi[j];
                    var ysij = ysi[j];
                    if (xsij.length > 1 && ysij.length > 1) {
                        for (var k = 1, endk = xsij.length; k < endk; k++) {
                            var _a = tslib_1.__read(arrayable_1.minmax(xsij[k]), 2), xij0 = _a[0], xij1 = _a[1];
                            var _b = tslib_1.__read(arrayable_1.minmax(ysij[k]), 2), yij0 = _b[0], yij1 = _b[1];
                            xi0 = min(xi0, xij0);
                            xi1 = max(xi1, xij1);
                            yi0 = min(yi0, yij0);
                            yi1 = max(yi1, yij1);
                        }
                    }
                }
                if (!isFinite(xi0 + xi1 + yi0 + yi1))
                    index.add_empty();
                else
                    index.add(xi0, yi0, xi1, yi1);
            }
            index.finish();
            return index;
        };
        MultiPolygonsView.prototype._mask_data = function () {
            var xr = this.renderer.plot_view.frame.x_range;
            var _a = tslib_1.__read([xr.min, xr.max], 2), x0 = _a[0], x1 = _a[1];
            var yr = this.renderer.plot_view.frame.y_range;
            var _b = tslib_1.__read([yr.min, yr.max], 2), y0 = _b[0], y1 = _b[1];
            return this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 });
        };
        MultiPolygonsView.prototype._inner_loop = function (ctx, sx, sy) {
            ctx.beginPath();
            for (var j = 0, endj = sx.length; j < endj; j++) {
                for (var k = 0, endk = sx[j].length; k < endk; k++) {
                    var _sx = sx[j][k];
                    var _sy = sy[j][k];
                    for (var l = 0, endl = _sx.length; l < endl; l++) {
                        if (l == 0) {
                            ctx.moveTo(_sx[l], _sy[l]);
                            continue;
                        }
                        else
                            ctx.lineTo(_sx[l], _sy[l]);
                    }
                    ctx.closePath();
                }
            }
        };
        MultiPolygonsView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var _this = this;
            var sxs = _a.sxs, sys = _a.sys;
            if (this.visuals.fill.doit || this.visuals.line.doit) {
                var _loop_1 = function (i) {
                    var _a = tslib_1.__read([sxs[i], sys[i]], 2), sx = _a[0], sy = _a[1];
                    if (this_1.visuals.fill.doit) {
                        this_1.visuals.fill.set_vectorize(ctx, i);
                        this_1._inner_loop(ctx, sx, sy);
                        ctx.fill("evenodd");
                    }
                    this_1.visuals.hatch.doit2(ctx, i, function () {
                        _this._inner_loop(ctx, sx, sy);
                        ctx.fill("evenodd");
                    }, function () { return _this.renderer.request_render(); });
                    if (this_1.visuals.line.doit) {
                        this_1.visuals.line.set_vectorize(ctx, i);
                        this_1._inner_loop(ctx, sx, sy);
                        ctx.stroke();
                    }
                };
                var this_1 = this;
                try {
                    for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                        var i = indices_1_1.value;
                        _loop_1(i);
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                            _b.call(indices_1);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
            }
        };
        MultiPolygonsView.prototype._hit_rect = function (geometry) {
            var e_2, _a;
            var sx0 = geometry.sx0, sx1 = geometry.sx1, sy0 = geometry.sy0, sy1 = geometry.sy1;
            var xs = [sx0, sx1, sx1, sx0];
            var ys = [sy0, sy0, sy1, sy1];
            var _b = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _b[0], x1 = _b[1];
            var _c = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _c[0], y1 = _c[1];
            var candidates = this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 });
            var indices = [];
            try {
                for (var candidates_1 = tslib_1.__values(candidates), candidates_1_1 = candidates_1.next(); !candidates_1_1.done; candidates_1_1 = candidates_1.next()) {
                    var index = candidates_1_1.value;
                    var sxss = this.sxs[index];
                    var syss = this.sys[index];
                    var hit = true;
                    for (var j = 0, endj = sxss.length; j < endj; j++) {
                        for (var k = 0, endk = sxss[j][0].length; k < endk; k++) {
                            var sx = sxss[j][0][k];
                            var sy = syss[j][0][k];
                            if (!hittest.point_in_poly(sx, sy, xs, ys)) {
                                hit = false;
                                break;
                            }
                        }
                        if (!hit)
                            break;
                    }
                    if (hit) {
                        indices.push(index);
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (candidates_1_1 && !candidates_1_1.done && (_a = candidates_1.return))
                        _a.call(candidates_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        MultiPolygonsView.prototype._hit_point = function (geometry) {
            var e_3, _a;
            var sx = geometry.sx, sy = geometry.sy;
            var x = this.renderer.xscale.invert(sx);
            var y = this.renderer.yscale.invert(sy);
            var candidates = this.index.indices({ x0: x, y0: y, x1: x, y1: y });
            var hole_candidates = this._hole_index.indices({ x0: x, y0: y, x1: x, y1: y });
            var indices = [];
            try {
                for (var candidates_2 = tslib_1.__values(candidates), candidates_2_1 = candidates_2.next(); !candidates_2_1.done; candidates_2_1 = candidates_2.next()) {
                    var index = candidates_2_1.value;
                    var sxs = this.sxs[index];
                    var sys = this.sys[index];
                    for (var j = 0, endj = sxs.length; j < endj; j++) {
                        var nk = sxs[j].length;
                        if (hittest.point_in_poly(sx, sy, sxs[j][0], sys[j][0])) {
                            if (nk == 1) {
                                indices.push(index);
                            }
                            else if (!hole_candidates.get(index)) {
                                indices.push(index);
                            }
                            else if (nk > 1) {
                                var in_a_hole = false;
                                for (var k = 1; k < nk; k++) {
                                    var sxs_k = sxs[j][k];
                                    var sys_k = sys[j][k];
                                    if (hittest.point_in_poly(sx, sy, sxs_k, sys_k)) {
                                        in_a_hole = true;
                                        break;
                                    }
                                    else {
                                        continue;
                                    }
                                }
                                if (!in_a_hole) {
                                    indices.push(index);
                                }
                            }
                        }
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (candidates_2_1 && !candidates_2_1.done && (_a = candidates_2.return))
                        _a.call(candidates_2);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        MultiPolygonsView.prototype._get_snap_coord = function (array) {
            return arrayable_2.sum(array) / array.length;
        };
        MultiPolygonsView.prototype.scenterxy = function (i, sx, sy) {
            if (this.sxs[i].length == 1) {
                // We don't have discontinuous objects so we're ok
                var scx = this._get_snap_coord(this.sxs[i][0][0]);
                var scy = this._get_snap_coord(this.sys[i][0][0]);
                return [scx, scy];
            }
            else {
                // We have discontinuous objects, so we need to find which
                // one we're in, we can use point_in_poly again
                var sxs = this.sxs[i];
                var sys = this.sys[i];
                for (var j = 0, end = sxs.length; j < end; j++) {
                    if (hittest.point_in_poly(sx, sy, sxs[j][0], sys[j][0])) {
                        var scx = this._get_snap_coord(sxs[j][0]);
                        var scy = this._get_snap_coord(sys[j][0]);
                        return [scx, scy];
                    }
                }
            }
            assert_1.unreachable();
        };
        MultiPolygonsView.prototype.map_data = function () {
            var n_i = this._xs.length;
            this.sxs = new Array(n_i);
            this.sys = new Array(n_i);
            for (var i = 0; i < n_i; i++) {
                var n_j = this._xs[i].length;
                this.sxs[i] = new Array(n_j);
                this.sys[i] = new Array(n_j);
                for (var j = 0; j < n_j; j++) {
                    var n_k = this._xs[i][j].length;
                    this.sxs[i][j] = new Array(n_k);
                    this.sys[i][j] = new Array(n_k);
                    for (var k = 0; k < n_k; k++) {
                        var _a = tslib_1.__read(this.renderer.coordinates.map_to_screen(this._xs[i][j][k], this._ys[i][j][k]), 2), sx = _a[0], sy = _a[1];
                        this.sxs[i][j][k] = sx;
                        this.sys[i][j][k] = sy;
                    }
                }
            }
        };
        MultiPolygonsView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_area_legend(this.visuals, ctx, bbox, index);
        };
        return MultiPolygonsView;
    }(glyph_1.GlyphView));
    exports.MultiPolygonsView = MultiPolygonsView;
    MultiPolygonsView.__name__ = "MultiPolygonsView";
    var MultiPolygons = /** @class */ (function (_super) {
        tslib_1.__extends(MultiPolygons, _super);
        function MultiPolygons(attrs) {
            return _super.call(this, attrs) || this;
        }
        MultiPolygons.init_MultiPolygons = function () {
            this.prototype.default_view = MultiPolygonsView;
            this.define({
                xs: [p.XCoordinateSeqSeqSeqSpec, { field: "xs" }],
                ys: [p.YCoordinateSeqSeqSeqSpec, { field: "ys" }],
            });
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector, property_mixins_1.HatchVector]);
        };
        return MultiPolygons;
    }(glyph_1.Glyph));
    exports.MultiPolygons = MultiPolygons;
    MultiPolygons.__name__ = "MultiPolygons";
    MultiPolygons.init_MultiPolygons();
},
/* models/glyphs/oval.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var ellipse_oval_1 = require(358) /* ./ellipse_oval */;
    var OvalView = /** @class */ (function (_super) {
        tslib_1.__extends(OvalView, _super);
        function OvalView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        OvalView.prototype._map_data = function () {
            _super.prototype._map_data.call(this);
            // oval drawn from bezier curves = ellipse with width reduced by 3/4
            var sw = this.sw;
            var n = sw.length;
            for (var i = 0; i < n; i++) {
                sw[i] *= 0.75;
            }
        };
        return OvalView;
    }(ellipse_oval_1.EllipseOvalView));
    exports.OvalView = OvalView;
    OvalView.__name__ = "OvalView";
    var Oval = /** @class */ (function (_super) {
        tslib_1.__extends(Oval, _super);
        function Oval(attrs) {
            return _super.call(this, attrs) || this;
        }
        Oval.init_Oval = function () {
            this.prototype.default_view = OvalView;
        };
        return Oval;
    }(ellipse_oval_1.EllipseOval));
    exports.Oval = Oval;
    Oval.__name__ = "Oval";
    Oval.init_Oval();
},
/* models/glyphs/patches.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var glyph_1 = require(209) /* ./glyph */;
    var utils_1 = require(215) /* ./utils */;
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var selection_1 = require(203) /* ../selections/selection */;
    var assert_1 = require(126) /* ../../core/util/assert */;
    var projections_1 = require(152) /* ../../core/util/projections */;
    var PatchesView = /** @class */ (function (_super) {
        tslib_1.__extends(PatchesView, _super);
        function PatchesView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        PatchesView.prototype._project_data = function () {
            projections_1.inplace.project_xy(this._xs.array, this._ys.array);
        };
        PatchesView.prototype._index_data = function (index) {
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                var xsi = this._xs.get(i);
                var ysi = this._ys.get(i);
                if (xsi.length == 0)
                    index.add_empty();
                else {
                    var _a = tslib_1.__read(arrayable_1.minmax(xsi), 2), x0 = _a[0], x1 = _a[1];
                    var _b = tslib_1.__read(arrayable_1.minmax(ysi), 2), y0 = _b[0], y1 = _b[1];
                    index.add(x0, y0, x1, y1);
                }
            }
        };
        PatchesView.prototype._mask_data = function () {
            var xr = this.renderer.plot_view.frame.x_range;
            var _a = tslib_1.__read([xr.min, xr.max], 2), x0 = _a[0], x1 = _a[1];
            var yr = this.renderer.plot_view.frame.y_range;
            var _b = tslib_1.__read([yr.min, yr.max], 2), y0 = _b[0], y1 = _b[1];
            return this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 });
        };
        PatchesView.prototype._inner_loop = function (ctx, sx, sy, func) {
            for (var j = 0, end = sx.length; j < end; j++) {
                if (j == 0) {
                    ctx.beginPath();
                    ctx.moveTo(sx[j], sy[j]);
                    continue;
                }
                else if (isNaN(sx[j] + sy[j])) {
                    ctx.closePath();
                    func.apply(ctx);
                    ctx.beginPath();
                    continue;
                }
                else
                    ctx.lineTo(sx[j], sy[j]);
            }
            ctx.closePath();
            func.call(ctx);
        };
        PatchesView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var _this = this;
            var sxs = _a.sxs, sys = _a.sys;
            var _loop_1 = function (i) {
                var sx = sxs.get(i);
                var sy = sys.get(i);
                if (this_1.visuals.fill.doit) {
                    this_1.visuals.fill.set_vectorize(ctx, i);
                    this_1._inner_loop(ctx, sx, sy, ctx.fill);
                }
                this_1.visuals.hatch.doit2(ctx, i, function () { return _this._inner_loop(ctx, sx, sy, ctx.fill); }, function () { return _this.renderer.request_render(); });
                if (this_1.visuals.line.doit) {
                    this_1.visuals.line.set_vectorize(ctx, i);
                    this_1._inner_loop(ctx, sx, sy, ctx.stroke);
                }
            };
            var this_1 = this;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    _loop_1(i);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        PatchesView.prototype._hit_rect = function (geometry) {
            var e_2, _a;
            var sx0 = geometry.sx0, sx1 = geometry.sx1, sy0 = geometry.sy0, sy1 = geometry.sy1;
            var xs = [sx0, sx1, sx1, sx0];
            var ys = [sy0, sy0, sy1, sy1];
            var _b = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _b[0], x1 = _b[1];
            var _c = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _c[0], y1 = _c[1];
            var candidates = this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 });
            var indices = [];
            try {
                for (var candidates_1 = tslib_1.__values(candidates), candidates_1_1 = candidates_1.next(); !candidates_1_1.done; candidates_1_1 = candidates_1.next()) {
                    var index = candidates_1_1.value;
                    var sxss = this.sxs.get(index);
                    var syss = this.sys.get(index);
                    var hit = true;
                    for (var j = 0, endj = sxss.length; j < endj; j++) {
                        var sx = sxss[j];
                        var sy = syss[j];
                        if (!hittest.point_in_poly(sx, sy, xs, ys)) {
                            hit = false;
                            break;
                        }
                    }
                    if (hit) {
                        indices.push(index);
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (candidates_1_1 && !candidates_1_1.done && (_a = candidates_1.return))
                        _a.call(candidates_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        PatchesView.prototype._hit_point = function (geometry) {
            var e_3, _a;
            var sx = geometry.sx, sy = geometry.sy;
            var x = this.renderer.xscale.invert(sx);
            var y = this.renderer.yscale.invert(sy);
            var candidates = this.index.indices({ x0: x, y0: y, x1: x, y1: y });
            var indices = [];
            try {
                for (var candidates_2 = tslib_1.__values(candidates), candidates_2_1 = candidates_2.next(); !candidates_2_1.done; candidates_2_1 = candidates_2.next()) {
                    var index = candidates_2_1.value;
                    var sxsi = this.sxs.get(index);
                    var sysi = this.sys.get(index);
                    var n = sxsi.length;
                    for (var k = 0, j = 0;; j++) {
                        if (isNaN(sxsi[j]) || j == n) {
                            var sxsi_kj = sxsi.subarray(k, j);
                            var sysi_kj = sysi.subarray(k, j);
                            if (hittest.point_in_poly(sx, sy, sxsi_kj, sysi_kj)) {
                                indices.push(index);
                                break;
                            }
                            k = j + 1;
                        }
                        if (j == n)
                            break;
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (candidates_2_1 && !candidates_2_1.done && (_a = candidates_2.return))
                        _a.call(candidates_2);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        PatchesView.prototype._get_snap_coord = function (array) {
            return arrayable_1.sum(array) / array.length;
        };
        PatchesView.prototype.scenterxy = function (i, sx, sy) {
            var sxsi = this.sxs.get(i);
            var sysi = this.sys.get(i);
            var n = sxsi.length;
            var has_nan = false;
            for (var k = 0, j = 0;; j++) {
                var this_nan = isNaN(sxsi[j]);
                has_nan = has_nan || this_nan;
                if (j == n && !has_nan) {
                    var scx = this._get_snap_coord(sxsi);
                    var scy = this._get_snap_coord(sysi);
                    return [scx, scy];
                }
                if (this_nan || j == n) {
                    var sxsi_kj = sxsi.subarray(k, j);
                    var sysi_kj = sysi.subarray(k, j);
                    if (hittest.point_in_poly(sx, sy, sxsi_kj, sysi_kj)) {
                        var scx = this._get_snap_coord(sxsi_kj);
                        var scy = this._get_snap_coord(sysi_kj);
                        return [scx, scy];
                    }
                    k = j + 1;
                }
                if (j == n)
                    break;
            }
            assert_1.unreachable();
        };
        PatchesView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_area_legend(this.visuals, ctx, bbox, index);
        };
        return PatchesView;
    }(glyph_1.GlyphView));
    exports.PatchesView = PatchesView;
    PatchesView.__name__ = "PatchesView";
    var Patches = /** @class */ (function (_super) {
        tslib_1.__extends(Patches, _super);
        function Patches(attrs) {
            return _super.call(this, attrs) || this;
        }
        Patches.init_Patches = function () {
            this.prototype.default_view = PatchesView;
            this.define({
                xs: [p.XCoordinateSeqSpec, { field: "xs" }],
                ys: [p.YCoordinateSeqSpec, { field: "ys" }],
            });
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector, property_mixins_1.HatchVector]);
        };
        return Patches;
    }(glyph_1.Glyph));
    exports.Patches = Patches;
    Patches.__name__ = "Patches";
    Patches.init_Patches();
},
/* models/glyphs/quad.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var box_1 = require(360) /* ./box */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var QuadView = /** @class */ (function (_super) {
        tslib_1.__extends(QuadView, _super);
        function QuadView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        QuadView.prototype.scenterxy = function (i) {
            var scx = (this.sleft[i] + this.sright[i]) / 2;
            var scy = (this.stop[i] + this.sbottom[i]) / 2;
            return [scx, scy];
        };
        QuadView.prototype._lrtb = function (i) {
            var l = this._left[i];
            var r = this._right[i];
            var t = this._top[i];
            var b = this._bottom[i];
            return [l, r, t, b];
        };
        return QuadView;
    }(box_1.BoxView));
    exports.QuadView = QuadView;
    QuadView.__name__ = "QuadView";
    var Quad = /** @class */ (function (_super) {
        tslib_1.__extends(Quad, _super);
        function Quad(attrs) {
            return _super.call(this, attrs) || this;
        }
        Quad.init_Quad = function () {
            this.prototype.default_view = QuadView;
            this.define({
                right: [p.XCoordinateSpec, { field: "right" }],
                bottom: [p.YCoordinateSpec, { field: "bottom" }],
                left: [p.XCoordinateSpec, { field: "left" }],
                top: [p.YCoordinateSpec, { field: "top" }],
            });
        };
        return Quad;
    }(box_1.Box));
    exports.Quad = Quad;
    Quad.__name__ = "Quad";
    Quad.init_Quad();
},
/* models/glyphs/quadratic.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var projections_1 = require(152) /* ../../core/util/projections */;
    var glyph_1 = require(209) /* ./glyph */;
    var utils_1 = require(215) /* ./utils */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    // Formula from: http://pomax.nihongoresources.com/pages/bezier/
    //
    // if segment is quadratic bezier do:
    //   for both directions do:
    //     if control between start and end, compute linear bounding box
    //     otherwise, compute
    //       bound = u(1-t)^2 + 2v(1-t)t + wt^2
    //         (with t = ((u-v) / (u-2v+w)), with {u = start, v = control, w = end})
    //       if control precedes start, min = bound, otherwise max = bound
    function _qbb(u, v, w) {
        if (v == (u + w) / 2)
            return [u, w];
        else {
            var t = (u - v) / ((u - (2 * v)) + w);
            var bd = (u * Math.pow((1 - t), 2)) + (2 * v * (1 - t) * t) + (w * Math.pow(t, 2));
            return [Math.min(u, w, bd), Math.max(u, w, bd)];
        }
    }
    var QuadraticView = /** @class */ (function (_super) {
        tslib_1.__extends(QuadraticView, _super);
        function QuadraticView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        QuadraticView.prototype._project_data = function () {
            projections_1.inplace.project_xy(this._x0, this._y0);
            projections_1.inplace.project_xy(this._x1, this._y1);
        };
        QuadraticView.prototype._index_data = function (index) {
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                if (isNaN(this._x0[i] + this._x1[i] + this._y0[i] + this._y1[i] + this._cx[i] + this._cy[i]))
                    index.add_empty();
                else {
                    var _a = tslib_1.__read(_qbb(this._x0[i], this._cx[i], this._x1[i]), 2), x0 = _a[0], x1 = _a[1];
                    var _b = tslib_1.__read(_qbb(this._y0[i], this._cy[i], this._y1[i]), 2), y0 = _b[0], y1 = _b[1];
                    index.add(x0, y0, x1, y1);
                }
            }
        };
        QuadraticView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx0 = _a.sx0, sy0 = _a.sy0, sx1 = _a.sx1, sy1 = _a.sy1, scx = _a.scx, scy = _a.scy;
            if (this.visuals.line.doit) {
                try {
                    for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                        var i = indices_1_1.value;
                        if (isNaN(sx0[i] + sy0[i] + sx1[i] + sy1[i] + scx[i] + scy[i]))
                            continue;
                        ctx.beginPath();
                        ctx.moveTo(sx0[i], sy0[i]);
                        ctx.quadraticCurveTo(scx[i], scy[i], sx1[i], sy1[i]);
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.stroke();
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                            _b.call(indices_1);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
            }
        };
        QuadraticView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_line_legend(this.visuals, ctx, bbox, index);
        };
        QuadraticView.prototype.scenterxy = function () {
            throw new Error(this + ".scenterxy() is not implemented");
        };
        return QuadraticView;
    }(glyph_1.GlyphView));
    exports.QuadraticView = QuadraticView;
    QuadraticView.__name__ = "QuadraticView";
    var Quadratic = /** @class */ (function (_super) {
        tslib_1.__extends(Quadratic, _super);
        function Quadratic(attrs) {
            return _super.call(this, attrs) || this;
        }
        Quadratic.init_Quadratic = function () {
            this.prototype.default_view = QuadraticView;
            this.define({
                x0: [p.XCoordinateSpec, { field: "x0" }],
                y0: [p.YCoordinateSpec, { field: "y0" }],
                x1: [p.XCoordinateSpec, { field: "x1" }],
                y1: [p.YCoordinateSpec, { field: "y1" }],
                cx: [p.XCoordinateSpec, { field: "cx" }],
                cy: [p.YCoordinateSpec, { field: "cy" }],
            });
            this.mixins(property_mixins_1.LineVector);
        };
        return Quadratic;
    }(glyph_1.Glyph));
    exports.Quadratic = Quadratic;
    Quadratic.__name__ = "Quadratic";
    Quadratic.init_Quadratic();
},
/* models/glyphs/ray.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var utils_1 = require(215) /* ./utils */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var RayView = /** @class */ (function (_super) {
        tslib_1.__extends(RayView, _super);
        function RayView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        RayView.prototype._map_data = function () {
            if (this.model.properties.length.units == "data")
                this.slength = this.sdist(this.renderer.xscale, this._x, this._length);
            else
                this.slength = this._length;
        };
        RayView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx = _a.sx, sy = _a.sy, slength = _a.slength, _angle = _a._angle;
            if (this.visuals.line.doit) {
                var width = this.renderer.plot_view.frame.bbox.width;
                var height = this.renderer.plot_view.frame.bbox.height;
                var inf_len = 2 * (width + height);
                for (var i = 0, end = slength.length; i < end; i++) {
                    if (slength[i] == 0)
                        slength[i] = inf_len;
                }
                try {
                    for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                        var i = indices_1_1.value;
                        if (isNaN(sx[i] + sy[i] + _angle[i] + slength[i]))
                            continue;
                        ctx.translate(sx[i], sy[i]);
                        ctx.rotate(_angle[i]);
                        ctx.beginPath();
                        ctx.moveTo(0, 0);
                        ctx.lineTo(slength[i], 0);
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.stroke();
                        ctx.rotate(-_angle[i]);
                        ctx.translate(-sx[i], -sy[i]);
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                            _b.call(indices_1);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
            }
        };
        RayView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_line_legend(this.visuals, ctx, bbox, index);
        };
        return RayView;
    }(xy_glyph_1.XYGlyphView));
    exports.RayView = RayView;
    RayView.__name__ = "RayView";
    var Ray = /** @class */ (function (_super) {
        tslib_1.__extends(Ray, _super);
        function Ray(attrs) {
            return _super.call(this, attrs) || this;
        }
        Ray.init_Ray = function () {
            this.prototype.default_view = RayView;
            this.mixins(property_mixins_1.LineVector);
            this.define({
                length: [p.DistanceSpec],
                angle: [p.AngleSpec],
            });
        };
        return Ray;
    }(xy_glyph_1.XYGlyph));
    exports.Ray = Ray;
    Ray.__name__ = "Ray";
    Ray.init_Ray();
},
/* models/glyphs/rect.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var center_rotatable_1 = require(356) /* ./center_rotatable */;
    var utils_1 = require(215) /* ./utils */;
    var types_1 = require(139) /* ../../core/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var arrayable_1 = require(127) /* ../../core/util/arrayable */;
    var selection_1 = require(203) /* ../selections/selection */;
    var RectView = /** @class */ (function (_super) {
        tslib_1.__extends(RectView, _super);
        function RectView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        RectView.prototype._set_data = function () {
            this.max_w2 = 0;
            if (this.model.properties.width.units == "data")
                this.max_w2 = this.max_width / 2;
            this.max_h2 = 0;
            if (this.model.properties.height.units == "data")
                this.max_h2 = this.max_height / 2;
        };
        RectView.prototype._map_data = function () {
            var _a, _b;
            if (this.model.properties.width.units == "data")
                _a = tslib_1.__read(this._map_dist_corner_for_data_side_length(this._x, this._width, this.renderer.xscale), 2), this.sw = _a[0], this.sx0 = _a[1];
            else {
                this.sw = this._width;
                var n_1 = this.sx.length;
                this.sx0 = new types_1.NumberArray(n_1);
                for (var i = 0; i < n_1; i++)
                    this.sx0[i] = this.sx[i] - this.sw[i] / 2;
            }
            if (this.model.properties.height.units == "data")
                _b = tslib_1.__read(this._map_dist_corner_for_data_side_length(this._y, this._height, this.renderer.yscale), 2), this.sh = _b[0], this.sy1 = _b[1];
            else {
                this.sh = this._height;
                var n_2 = this.sy.length;
                this.sy1 = new types_1.NumberArray(n_2);
                for (var i = 0; i < n_2; i++)
                    this.sy1[i] = this.sy[i] - this.sh[i] / 2;
            }
            var n = this.sw.length;
            this.ssemi_diag = new types_1.NumberArray(n);
            for (var i = 0; i < n; i++)
                this.ssemi_diag[i] = Math.sqrt((this.sw[i] / 2 * this.sw[i]) / 2 + (this.sh[i] / 2 * this.sh[i]) / 2);
        };
        RectView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b, e_2, _c;
            var sx = _a.sx, sy = _a.sy, sx0 = _a.sx0, sy1 = _a.sy1, sw = _a.sw, sh = _a.sh, _angle = _a._angle;
            if (this.visuals.fill.doit) {
                try {
                    for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                        var i = indices_1_1.value;
                        if (isNaN(sx[i] + sy[i] + sx0[i] + sy1[i] + sw[i] + sh[i] + _angle[i]))
                            continue;
                        //no need to test the return value, we call fillRect for every glyph anyway
                        this.visuals.fill.set_vectorize(ctx, i);
                        if (_angle[i]) {
                            ctx.translate(sx[i], sy[i]);
                            ctx.rotate(_angle[i]);
                            ctx.fillRect(-sw[i] / 2, -sh[i] / 2, sw[i], sh[i]);
                            ctx.rotate(-_angle[i]);
                            ctx.translate(-sx[i], -sy[i]);
                        }
                        else
                            ctx.fillRect(sx0[i], sy1[i], sw[i], sh[i]);
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                            _b.call(indices_1);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
            }
            if (this.visuals.line.doit) {
                ctx.beginPath();
                try {
                    for (var indices_2 = tslib_1.__values(indices), indices_2_1 = indices_2.next(); !indices_2_1.done; indices_2_1 = indices_2.next()) {
                        var i = indices_2_1.value;
                        if (isNaN(sx[i] + sy[i] + sx0[i] + sy1[i] + sw[i] + sh[i] + _angle[i]))
                            continue;
                        // fillRect does not fill zero-height or -width rects, but rect(...)
                        // does seem to stroke them (1px wide or tall). Explicitly ignore rects
                        // with zero width or height to be consistent
                        if (sw[i] == 0 || sh[i] == 0)
                            continue;
                        if (_angle[i]) {
                            ctx.translate(sx[i], sy[i]);
                            ctx.rotate(_angle[i]);
                            ctx.rect(-sw[i] / 2, -sh[i] / 2, sw[i], sh[i]);
                            ctx.rotate(-_angle[i]);
                            ctx.translate(-sx[i], -sy[i]);
                        }
                        else
                            ctx.rect(sx0[i], sy1[i], sw[i], sh[i]);
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.stroke();
                        ctx.beginPath();
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (indices_2_1 && !indices_2_1.done && (_c = indices_2.return))
                            _c.call(indices_2);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
                ctx.stroke();
            }
        };
        RectView.prototype._hit_rect = function (geometry) {
            return this._hit_rect_against_index(geometry);
        };
        RectView.prototype._hit_point = function (geometry) {
            var e_3, _a;
            var sx = geometry.sx, sy = geometry.sy;
            var x = this.renderer.xscale.invert(sx);
            var y = this.renderer.yscale.invert(sy);
            var n = this.sx0.length;
            var scenter_x = new types_1.NumberArray(n);
            for (var i = 0; i < n; i++) {
                scenter_x[i] = this.sx0[i] + this.sw[i] / 2;
            }
            var scenter_y = new types_1.NumberArray(n);
            for (var i = 0; i < n; i++) {
                scenter_y[i] = this.sy1[i] + this.sh[i] / 2;
            }
            var max_x2_ddist = arrayable_1.max(this._ddist(0, scenter_x, this.ssemi_diag));
            var max_y2_ddist = arrayable_1.max(this._ddist(1, scenter_y, this.ssemi_diag));
            var x0 = x - max_x2_ddist;
            var x1 = x + max_x2_ddist;
            var y0 = y - max_y2_ddist;
            var y1 = y + max_y2_ddist;
            var width_in;
            var height_in;
            var indices = [];
            try {
                for (var _b = tslib_1.__values(this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 })), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var i = _c.value;
                    if (this._angle[i]) {
                        var s = Math.sin(-this._angle[i]);
                        var c = Math.cos(-this._angle[i]);
                        var px = c * (sx - this.sx[i]) - s * (sy - this.sy[i]) + this.sx[i];
                        var py = s * (sx - this.sx[i]) + c * (sy - this.sy[i]) + this.sy[i];
                        sx = px;
                        sy = py;
                        width_in = Math.abs(this.sx[i] - sx) <= this.sw[i] / 2;
                        height_in = Math.abs(this.sy[i] - sy) <= this.sh[i] / 2;
                    }
                    else {
                        var dx = sx - this.sx0[i];
                        var dy = sy - this.sy1[i];
                        width_in = 0 <= dx && dx <= this.sw[i];
                        height_in = 0 <= dy && dy <= this.sh[i];
                    }
                    if (width_in && height_in) {
                        indices.push(i);
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        RectView.prototype._map_dist_corner_for_data_side_length = function (coord, side_length, scale) {
            var n = coord.length;
            var pt0 = new types_1.NumberArray(n);
            var pt1 = new types_1.NumberArray(n);
            for (var i = 0; i < n; i++) {
                pt0[i] = coord[i] - side_length[i] / 2;
                pt1[i] = coord[i] + side_length[i] / 2;
            }
            var spt0 = scale.v_compute(pt0);
            var spt1 = scale.v_compute(pt1);
            var sside_length = this.sdist(scale, pt0, side_length, 'edge', this.model.dilate);
            var spt_corner = spt0;
            for (var i = 0; i < n; i++) {
                var spt0i = spt0[i];
                var spt1i = spt1[i];
                if (!isNaN(spt0i + spt1i) && spt0i != spt1i) {
                    spt_corner = spt0i < spt1i ? spt0 : spt1;
                    break;
                }
            }
            return [sside_length, spt_corner];
        };
        RectView.prototype._ddist = function (dim, spts, spans) {
            var scale = dim == 0 ? this.renderer.xscale : this.renderer.yscale;
            var spt0 = spts;
            var m = spt0.length;
            var spt1 = new types_1.NumberArray(m);
            for (var i = 0; i < m; i++)
                spt1[i] = spt0[i] + spans[i];
            var pt0 = scale.v_invert(spt0);
            var pt1 = scale.v_invert(spt1);
            var n = pt0.length;
            var ddist = new types_1.NumberArray(n);
            for (var i = 0; i < n; i++)
                ddist[i] = Math.abs(pt1[i] - pt0[i]);
            return ddist;
        };
        RectView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_area_legend(this.visuals, ctx, bbox, index);
        };
        RectView.prototype._bounds = function (_a) {
            var x0 = _a.x0, x1 = _a.x1, y0 = _a.y0, y1 = _a.y1;
            return {
                x0: x0 - this.max_w2,
                x1: x1 + this.max_w2,
                y0: y0 - this.max_h2,
                y1: y1 + this.max_h2,
            };
        };
        return RectView;
    }(center_rotatable_1.CenterRotatableView));
    exports.RectView = RectView;
    RectView.__name__ = "RectView";
    var Rect = /** @class */ (function (_super) {
        tslib_1.__extends(Rect, _super);
        function Rect(attrs) {
            return _super.call(this, attrs) || this;
        }
        Rect.init_Rect = function () {
            this.prototype.default_view = RectView;
            this.define({
                dilate: [p.Boolean, false],
            });
        };
        return Rect;
    }(center_rotatable_1.CenterRotatable));
    exports.Rect = Rect;
    Rect.__name__ = "Rect";
    Rect.init_Rect();
},
/* models/glyphs/segment.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var projections_1 = require(152) /* ../../core/util/projections */;
    var glyph_1 = require(209) /* ./glyph */;
    var utils_1 = require(215) /* ./utils */;
    var selection_1 = require(203) /* ../selections/selection */;
    var SegmentView = /** @class */ (function (_super) {
        tslib_1.__extends(SegmentView, _super);
        function SegmentView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        SegmentView.prototype._project_data = function () {
            projections_1.inplace.project_xy(this._x0, this._y0);
            projections_1.inplace.project_xy(this._x1, this._y1);
        };
        SegmentView.prototype._index_data = function (index) {
            var min = Math.min, max = Math.max;
            var data_size = this.data_size;
            for (var i = 0; i < data_size; i++) {
                var x0 = this._x0[i];
                var x1 = this._x1[i];
                var y0 = this._y0[i];
                var y1 = this._y1[i];
                if (isNaN(x0 + x1 + y0 + y1))
                    index.add_empty();
                else
                    index.add(min(x0, x1), min(y0, y1), max(x0, x1), max(y0, y1));
            }
        };
        SegmentView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx0 = _a.sx0, sy0 = _a.sy0, sx1 = _a.sx1, sy1 = _a.sy1;
            if (this.visuals.line.doit) {
                try {
                    for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                        var i = indices_1_1.value;
                        if (isNaN(sx0[i] + sy0[i] + sx1[i] + sy1[i]))
                            continue;
                        ctx.beginPath();
                        ctx.moveTo(sx0[i], sy0[i]);
                        ctx.lineTo(sx1[i], sy1[i]);
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.stroke();
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                            _b.call(indices_1);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
            }
        };
        SegmentView.prototype._hit_point = function (geometry) {
            var e_2, _a;
            var sx = geometry.sx, sy = geometry.sy;
            var point = { x: sx, y: sy };
            var lw_voffset = 2; // FIXME: Use maximum of segments line_width/2 instead of magic constant 2
            var _b = tslib_1.__read(this.renderer.xscale.r_invert(sx - lw_voffset, sx + lw_voffset), 2), x0 = _b[0], x1 = _b[1];
            var _c = tslib_1.__read(this.renderer.yscale.r_invert(sy - lw_voffset, sy + lw_voffset), 2), y0 = _c[0], y1 = _c[1];
            var candidates = this.index.indices({ x0: x0, y0: y0, x1: x1, y1: y1 });
            var indices = [];
            try {
                for (var candidates_1 = tslib_1.__values(candidates), candidates_1_1 = candidates_1.next(); !candidates_1_1.done; candidates_1_1 = candidates_1.next()) {
                    var i = candidates_1_1.value;
                    var threshold2 = Math.pow(Math.max(2, this.visuals.line.cache_select('line_width', i) / 2), 2);
                    var p0 = { x: this.sx0[i], y: this.sy0[i] };
                    var p1 = { x: this.sx1[i], y: this.sy1[i] };
                    var dist2 = hittest.dist_to_segment_squared(point, p0, p1);
                    if (dist2 < threshold2) {
                        indices.push(i);
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (candidates_1_1 && !candidates_1_1.done && (_a = candidates_1.return))
                        _a.call(candidates_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        SegmentView.prototype._hit_span = function (geometry) {
            var _a, _b, e_3, _c;
            var _d = tslib_1.__read(this.renderer.plot_view.frame.bbox.ranges, 2), hr = _d[0], vr = _d[1];
            var sx = geometry.sx, sy = geometry.sy;
            var v0;
            var v1;
            var val;
            if (geometry.direction == 'v') {
                val = this.renderer.yscale.invert(sy);
                _a = tslib_1.__read([this._y0, this._y1], 2), v0 = _a[0], v1 = _a[1];
            }
            else {
                val = this.renderer.xscale.invert(sx);
                _b = tslib_1.__read([this._x0, this._x1], 2), v0 = _b[0], v1 = _b[1];
            }
            var indices = [];
            var _e = tslib_1.__read(this.renderer.xscale.r_invert(hr.start, hr.end), 2), x0 = _e[0], x1 = _e[1];
            var _f = tslib_1.__read(this.renderer.yscale.r_invert(vr.start, vr.end), 2), y0 = _f[0], y1 = _f[1];
            var candidates = this.index.indices({ x0: x0, y0: y0, x1: x1, y1: y1 });
            try {
                for (var candidates_2 = tslib_1.__values(candidates), candidates_2_1 = candidates_2.next(); !candidates_2_1.done; candidates_2_1 = candidates_2.next()) {
                    var i = candidates_2_1.value;
                    if ((v0[i] <= val && val <= v1[i]) || (v1[i] <= val && val <= v0[i]))
                        indices.push(i);
                    var threshold = 1.5 + (this.visuals.line.cache_select('line_width', i) / 2); // Maximum pixel difference to detect hit
                    if (v0[i] == v1[i]) {
                        if (geometry.direction == 'h') {
                            if (Math.abs(this.sx0[i] - sx) <= threshold) {
                                indices.push(i);
                            }
                        }
                        else {
                            if (Math.abs(this.sy0[i] - sy) <= threshold) {
                                indices.push(i);
                            }
                        }
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (candidates_2_1 && !candidates_2_1.done && (_c = candidates_2.return))
                        _c.call(candidates_2);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        SegmentView.prototype.scenterxy = function (i) {
            var scx = (this.sx0[i] + this.sx1[i]) / 2;
            var scy = (this.sy0[i] + this.sy1[i]) / 2;
            return [scx, scy];
        };
        SegmentView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_line_legend(this.visuals, ctx, bbox, index);
        };
        return SegmentView;
    }(glyph_1.GlyphView));
    exports.SegmentView = SegmentView;
    SegmentView.__name__ = "SegmentView";
    var Segment = /** @class */ (function (_super) {
        tslib_1.__extends(Segment, _super);
        function Segment(attrs) {
            return _super.call(this, attrs) || this;
        }
        Segment.init_Segment = function () {
            this.prototype.default_view = SegmentView;
            this.define({
                x0: [p.XCoordinateSpec, { field: "x0" }],
                y0: [p.YCoordinateSpec, { field: "y0" }],
                x1: [p.XCoordinateSpec, { field: "x1" }],
                y1: [p.YCoordinateSpec, { field: "y1" }],
            });
            this.mixins(property_mixins_1.LineVector);
        };
        return Segment;
    }(glyph_1.Glyph));
    exports.Segment = Segment;
    Segment.__name__ = "Segment";
    Segment.init_Segment();
},
/* models/glyphs/step.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var utils_1 = require(215) /* ./utils */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var StepView = /** @class */ (function (_super) {
        tslib_1.__extends(StepView, _super);
        function StepView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        StepView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b, _c, _d, _e, _f, _g, _h;
            var sx = _a.sx, sy = _a.sy;
            var drawing = false;
            var last_index = null;
            this.visuals.line.set_value(ctx);
            var L = indices.length;
            if (L < 2)
                return;
            ctx.beginPath();
            ctx.moveTo(sx[0], sy[0]);
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    var x1 = void 0, x2 = void 0;
                    var y1 = void 0, y2 = void 0;
                    switch (this.model.mode) {
                        case "before": {
                            _c = tslib_1.__read([sx[i - 1], sy[i]], 2), x1 = _c[0], y1 = _c[1];
                            _d = tslib_1.__read([sx[i], sy[i]], 2), x2 = _d[0], y2 = _d[1];
                            break;
                        }
                        case "after": {
                            _e = tslib_1.__read([sx[i], sy[i - 1]], 2), x1 = _e[0], y1 = _e[1];
                            _f = tslib_1.__read([sx[i], sy[i]], 2), x2 = _f[0], y2 = _f[1];
                            break;
                        }
                        case "center": {
                            var xm = (sx[i - 1] + sx[i]) / 2;
                            _g = tslib_1.__read([xm, sy[i - 1]], 2), x1 = _g[0], y1 = _g[1];
                            _h = tslib_1.__read([xm, sy[i]], 2), x2 = _h[0], y2 = _h[1];
                            break;
                        }
                        default:
                            throw new Error("unexpected");
                    }
                    if (drawing) {
                        if (!isFinite(sx[i] + sy[i])) {
                            ctx.stroke();
                            ctx.beginPath();
                            drawing = false;
                            last_index = i;
                            continue;
                        }
                        if (last_index != null && i - last_index > 1) {
                            ctx.stroke();
                            drawing = false;
                        }
                    }
                    if (drawing) {
                        ctx.lineTo(x1, y1);
                        ctx.lineTo(x2, y2);
                    }
                    else {
                        ctx.beginPath();
                        ctx.moveTo(sx[i], sy[i]);
                        drawing = true;
                    }
                    last_index = i;
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            ctx.lineTo(sx[L - 1], sy[L - 1]);
            ctx.stroke();
        };
        StepView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_line_legend(this.visuals, ctx, bbox, index);
        };
        return StepView;
    }(xy_glyph_1.XYGlyphView));
    exports.StepView = StepView;
    StepView.__name__ = "StepView";
    var Step = /** @class */ (function (_super) {
        tslib_1.__extends(Step, _super);
        function Step(attrs) {
            return _super.call(this, attrs) || this;
        }
        Step.init_Step = function () {
            this.prototype.default_view = StepView;
            this.mixins(property_mixins_1.LineVector);
            this.define({
                mode: [p.StepMode, "before"],
            });
        };
        return Step;
    }(xy_glyph_1.XYGlyph));
    exports.Step = Step;
    Step.__name__ = "Step";
    Step.init_Step();
},
/* models/glyphs/text.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var text_1 = require(274) /* ../../core/util/text */;
    var assert_1 = require(126) /* ../../core/util/assert */;
    var selection_1 = require(203) /* ../selections/selection */;
    var TextView = /** @class */ (function (_super) {
        tslib_1.__extends(TextView, _super);
        function TextView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        TextView.prototype._rotate_point = function (x, y, xoff, yoff, angle) {
            var sxr = (x - xoff) * Math.cos(angle) - (y - yoff) * Math.sin(angle) + xoff;
            var syr = (x - xoff) * Math.sin(angle) + (y - yoff) * Math.cos(angle) + yoff;
            return [sxr, syr];
        };
        TextView.prototype._text_bounds = function (x0, y0, width, height) {
            var xvals = [x0, x0 + width, x0 + width, x0, x0];
            var yvals = [y0, y0, y0 - height, y0 - height, y0];
            return [xvals, yvals];
        };
        TextView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b, e_2, _c;
            var sx = _a.sx, sy = _a.sy, _x_offset = _a._x_offset, _y_offset = _a._y_offset, _angle = _a._angle, _text = _a._text;
            this._sys = [];
            this._sxs = [];
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    this._sxs[i] = [];
                    this._sys[i] = [];
                    if (isNaN(sx[i] + sy[i] + _x_offset[i] + _y_offset[i] + _angle[i]) || _text[i] == null)
                        continue;
                    if (this.visuals.text.doit) {
                        var text = "" + _text[i];
                        ctx.save();
                        ctx.translate(sx[i] + _x_offset[i], sy[i] + _y_offset[i]);
                        ctx.rotate(_angle[i]);
                        this.visuals.text.set_vectorize(ctx, i);
                        var font = this.visuals.text.cache_select("font", i);
                        var height = text_1.measure_font(font).height;
                        var line_height = this.visuals.text.text_line_height.value() * height;
                        if (text.indexOf("\n") == -1) {
                            ctx.fillText(text, 0, 0);
                            var x0 = sx[i] + _x_offset[i];
                            var y0 = sy[i] + _y_offset[i];
                            var width = ctx.measureText(text).width;
                            var _d = tslib_1.__read(this._text_bounds(x0, y0, width, line_height), 2), xvalues = _d[0], yvalues = _d[1];
                            this._sxs[i].push(xvalues);
                            this._sys[i].push(yvalues);
                        }
                        else {
                            var lines = text.split("\n");
                            var block_height = line_height * lines.length;
                            var baseline = this.visuals.text.cache_select("text_baseline", i);
                            var y = void 0;
                            switch (baseline) {
                                case "top": {
                                    y = 0;
                                    break;
                                }
                                case "middle": {
                                    y = (-block_height / 2) + (line_height / 2);
                                    break;
                                }
                                case "bottom": {
                                    y = -block_height + line_height;
                                    break;
                                }
                                default: {
                                    y = 0;
                                    console.warn("'" + baseline + "' baseline not supported with multi line text");
                                }
                            }
                            try {
                                for (var lines_1 = (e_2 = void 0, tslib_1.__values(lines)), lines_1_1 = lines_1.next(); !lines_1_1.done; lines_1_1 = lines_1.next()) {
                                    var line = lines_1_1.value;
                                    ctx.fillText(line, 0, y);
                                    var x0 = sx[i] + _x_offset[i];
                                    var y0 = y + sy[i] + _y_offset[i];
                                    var width = ctx.measureText(line).width;
                                    var _e = tslib_1.__read(this._text_bounds(x0, y0, width, line_height), 2), xvalues = _e[0], yvalues = _e[1];
                                    this._sxs[i].push(xvalues);
                                    this._sys[i].push(yvalues);
                                    y += line_height;
                                }
                            }
                            catch (e_2_1) {
                                e_2 = { error: e_2_1 };
                            }
                            finally {
                                try {
                                    if (lines_1_1 && !lines_1_1.done && (_c = lines_1.return))
                                        _c.call(lines_1);
                                }
                                finally {
                                    if (e_2)
                                        throw e_2.error;
                                }
                            }
                        }
                        ctx.restore();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        TextView.prototype._hit_point = function (geometry) {
            var sx = geometry.sx, sy = geometry.sy;
            var indices = [];
            for (var i = 0; i < this._sxs.length; i++) {
                var sxs = this._sxs[i];
                var sys = this._sys[i];
                var n = sxs.length;
                for (var j = 0, endj = n; j < endj; j++) {
                    var _a = tslib_1.__read(this._rotate_point(sx, sy, sxs[n - 1][0], sys[n - 1][0], -this._angle[i]), 2), sxr = _a[0], syr = _a[1];
                    if (hittest.point_in_poly(sxr, syr, sxs[j], sys[j])) {
                        indices.push(i);
                    }
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        TextView.prototype.scenterxy = function (i) {
            var sxs = this._sxs[i];
            var sys = this._sys[i];
            assert_1.assert(sxs.length != 0 && sys.length != 0);
            var sx0 = sxs[0][0];
            var sy0 = sys[0][0];
            var sxc = (sxs[0][2] + sx0) / 2;
            var syc = (sys[0][2] + sy0) / 2;
            var _a = tslib_1.__read(this._rotate_point(sxc, syc, sx0, sy0, this._angle[i]), 2), sxcr = _a[0], sycr = _a[1];
            return [sxcr, sycr];
        };
        return TextView;
    }(xy_glyph_1.XYGlyphView));
    exports.TextView = TextView;
    TextView.__name__ = "TextView";
    var Text = /** @class */ (function (_super) {
        tslib_1.__extends(Text, _super);
        function Text(attrs) {
            return _super.call(this, attrs) || this;
        }
        Text.init_Text = function () {
            this.prototype.default_view = TextView;
            this.mixins(property_mixins_1.TextVector);
            this.define({
                text: [p.NullStringSpec, { field: "text" }],
                angle: [p.AngleSpec, 0],
                x_offset: [p.NumberSpec, 0],
                y_offset: [p.NumberSpec, 0],
            });
        };
        return Text;
    }(xy_glyph_1.XYGlyph));
    exports.Text = Text;
    Text.__name__ = "Text";
    Text.init_Text();
},
/* models/glyphs/vbar.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var box_1 = require(360) /* ./box */;
    var types_1 = require(139) /* ../../core/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var VBarView = /** @class */ (function (_super) {
        tslib_1.__extends(VBarView, _super);
        function VBarView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        VBarView.prototype.scenterxy = function (i) {
            var scx = this.sx[i];
            var scy = (this.stop[i] + this.sbottom[i]) / 2;
            return [scx, scy];
        };
        VBarView.prototype._lrtb = function (i) {
            var l = this._x[i] - (this._width[i] / 2);
            var r = this._x[i] + (this._width[i] / 2);
            var t = Math.max(this._top[i], this._bottom[i]);
            var b = Math.min(this._top[i], this._bottom[i]);
            return [l, r, t, b];
        };
        VBarView.prototype._map_data = function () {
            this.sx = this.renderer.xscale.v_compute(this._x);
            this.sw = this.sdist(this.renderer.xscale, this._x, this._width, "center");
            this.stop = this.renderer.yscale.v_compute(this._top);
            this.sbottom = this.renderer.yscale.v_compute(this._bottom);
            var n = this.sx.length;
            this.sleft = new types_1.NumberArray(n);
            this.sright = new types_1.NumberArray(n);
            for (var i = 0; i < n; i++) {
                this.sleft[i] = this.sx[i] - this.sw[i] / 2;
                this.sright[i] = this.sx[i] + this.sw[i] / 2;
            }
            this._clamp_viewport();
        };
        return VBarView;
    }(box_1.BoxView));
    exports.VBarView = VBarView;
    VBarView.__name__ = "VBarView";
    var VBar = /** @class */ (function (_super) {
        tslib_1.__extends(VBar, _super);
        function VBar(attrs) {
            return _super.call(this, attrs) || this;
        }
        VBar.init_VBar = function () {
            this.prototype.default_view = VBarView;
            this.define({
                x: [p.XCoordinateSpec, { field: "x" }],
                bottom: [p.YCoordinateSpec, { value: 0 }],
                width: [p.NumberSpec, { value: 1 }],
                top: [p.YCoordinateSpec, { field: "top" }],
            });
        };
        return VBar;
    }(box_1.Box));
    exports.VBar = VBar;
    VBar.__name__ = "VBar";
    VBar.init_VBar();
},
/* models/glyphs/wedge.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ./xy_glyph */;
    var utils_1 = require(215) /* ./utils */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var math_1 = require(125) /* ../../core/util/math */;
    var selection_1 = require(203) /* ../selections/selection */;
    var WedgeView = /** @class */ (function (_super) {
        tslib_1.__extends(WedgeView, _super);
        function WedgeView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        WedgeView.prototype._map_data = function () {
            if (this.model.properties.radius.units == "data")
                this.sradius = this.sdist(this.renderer.xscale, this._x, this._radius);
            else
                this.sradius = this._radius;
        };
        WedgeView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx = _a.sx, sy = _a.sy, sradius = _a.sradius, _start_angle = _a._start_angle, _end_angle = _a._end_angle;
            var direction = this.model.properties.direction.value();
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (isNaN(sx[i] + sy[i] + sradius[i] + _start_angle[i] + _end_angle[i]))
                        continue;
                    ctx.beginPath();
                    ctx.arc(sx[i], sy[i], sradius[i], _start_angle[i], _end_angle[i], direction);
                    ctx.lineTo(sx[i], sy[i]);
                    ctx.closePath();
                    if (this.visuals.fill.doit) {
                        this.visuals.fill.set_vectorize(ctx, i);
                        ctx.fill();
                    }
                    if (this.visuals.line.doit) {
                        this.visuals.line.set_vectorize(ctx, i);
                        ctx.stroke();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        WedgeView.prototype._hit_point = function (geometry) {
            var _a, _b, e_2, _c, _d, _e, e_3, _f;
            var dist, sx0, sx1, sy0, sy1, x0, x1, y0, y1;
            var sx = geometry.sx, sy = geometry.sy;
            var x = this.renderer.xscale.invert(sx);
            var y = this.renderer.yscale.invert(sy);
            // check diameter first
            var max_diameter = 2 * this.max_radius;
            if (this.model.properties.radius.units === "data") {
                x0 = x - max_diameter;
                x1 = x + max_diameter;
                y0 = y - max_diameter;
                y1 = y + max_diameter;
            }
            else {
                sx0 = sx - max_diameter;
                sx1 = sx + max_diameter;
                _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
                sy0 = sy - max_diameter;
                sy1 = sy + max_diameter;
                _b = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _b[0], y1 = _b[1];
            }
            var candidates = [];
            try {
                for (var _g = tslib_1.__values(this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 })), _h = _g.next(); !_h.done; _h = _g.next()) {
                    var i = _h.value;
                    var r2 = Math.pow(this.sradius[i], 2);
                    _d = tslib_1.__read(this.renderer.xscale.r_compute(x, this._x[i]), 2), sx0 = _d[0], sx1 = _d[1];
                    _e = tslib_1.__read(this.renderer.yscale.r_compute(y, this._y[i]), 2), sy0 = _e[0], sy1 = _e[1];
                    dist = Math.pow((sx0 - sx1), 2) + Math.pow((sy0 - sy1), 2);
                    if (dist <= r2) {
                        candidates.push(i);
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_h && !_h.done && (_c = _g.return))
                        _c.call(_g);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            var direction = this.model.properties.direction.value();
            var indices = [];
            try {
                for (var candidates_1 = tslib_1.__values(candidates), candidates_1_1 = candidates_1.next(); !candidates_1_1.done; candidates_1_1 = candidates_1.next()) {
                    var i = candidates_1_1.value;
                    // NOTE: minus the angle because JS uses non-mathy convention for angles
                    var angle = Math.atan2(sy - this.sy[i], sx - this.sx[i]);
                    if (math_1.angle_between(-angle, -this._start_angle[i], -this._end_angle[i], direction)) {
                        indices.push(i);
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (candidates_1_1 && !candidates_1_1.done && (_f = candidates_1.return))
                        _f.call(candidates_1);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        WedgeView.prototype.draw_legend_for_index = function (ctx, bbox, index) {
            utils_1.generic_area_legend(this.visuals, ctx, bbox, index);
        };
        WedgeView.prototype.scenterxy = function (i) {
            var r = this.sradius[i] / 2;
            var a = (this._start_angle[i] + this._end_angle[i]) / 2;
            var scx = this.sx[i] + r * Math.cos(a);
            var scy = this.sy[i] + r * Math.sin(a);
            return [scx, scy];
        };
        return WedgeView;
    }(xy_glyph_1.XYGlyphView));
    exports.WedgeView = WedgeView;
    WedgeView.__name__ = "WedgeView";
    var Wedge = /** @class */ (function (_super) {
        tslib_1.__extends(Wedge, _super);
        function Wedge(attrs) {
            return _super.call(this, attrs) || this;
        }
        Wedge.init_Wedge = function () {
            this.prototype.default_view = WedgeView;
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector]);
            this.define({
                direction: [p.Direction, 'anticlock'],
                radius: [p.DistanceSpec],
                start_angle: [p.AngleSpec],
                end_angle: [p.AngleSpec],
            });
        };
        return Wedge;
    }(xy_glyph_1.XYGlyph));
    exports.Wedge = Wedge;
    Wedge.__name__ = "Wedge";
    Wedge.init_Wedge();
},
/* models/graphs/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    tslib_1.__exportStar(require(232) /* ./graph_hit_test_policy */, exports);
    tslib_1.__exportStar(require(381) /* ./layout_provider */, exports);
    tslib_1.__exportStar(require(382) /* ./static_layout_provider */, exports);
},
/* models/graphs/layout_provider.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var LayoutProvider = /** @class */ (function (_super) {
        tslib_1.__extends(LayoutProvider, _super);
        function LayoutProvider(attrs) {
            return _super.call(this, attrs) || this;
        }
        return LayoutProvider;
    }(model_1.Model));
    exports.LayoutProvider = LayoutProvider;
    LayoutProvider.__name__ = "LayoutProvider";
},
/* models/graphs/static_layout_provider.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var layout_provider_1 = require(381) /* ./layout_provider */;
    var types_1 = require(139) /* ../../core/types */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var StaticLayoutProvider = /** @class */ (function (_super) {
        tslib_1.__extends(StaticLayoutProvider, _super);
        function StaticLayoutProvider(attrs) {
            return _super.call(this, attrs) || this;
        }
        StaticLayoutProvider.init_StaticLayoutProvider = function () {
            this.define({
                graph_layout: [p.Any, {}],
            });
        };
        StaticLayoutProvider.prototype.get_node_coordinates = function (node_source) {
            var index = node_source.data.index;
            var n = index.length;
            var xs = new types_1.NumberArray(n);
            var ys = new types_1.NumberArray(n);
            for (var i = 0; i < n; i++) {
                var point = this.graph_layout[index[i]];
                var _a = tslib_1.__read(point !== null && point !== void 0 ? point : [NaN, NaN], 2), x = _a[0], y = _a[1];
                xs[i] = x;
                ys[i] = y;
            }
            return [xs, ys];
        };
        StaticLayoutProvider.prototype.get_edge_coordinates = function (edge_source) {
            var starts = edge_source.data.start;
            var ends = edge_source.data.end;
            var n = starts.length;
            var xs = [];
            var ys = [];
            var has_paths = edge_source.data.xs != null && edge_source.data.ys != null;
            for (var i = 0; i < n; i++) {
                var in_layout = this.graph_layout[starts[i]] != null && this.graph_layout[ends[i]] != null;
                if (has_paths && in_layout) {
                    xs.push(edge_source.data.xs[i]);
                    ys.push(edge_source.data.ys[i]);
                }
                else {
                    var start = void 0, end = void 0;
                    if (in_layout) {
                        start = this.graph_layout[starts[i]];
                        end = this.graph_layout[ends[i]];
                    }
                    else {
                        start = [NaN, NaN];
                        end = [NaN, NaN];
                    }
                    xs.push([start[0], end[0]]);
                    ys.push([start[1], end[1]]);
                }
            }
            return [xs, ys];
        };
        return StaticLayoutProvider;
    }(layout_provider_1.LayoutProvider));
    exports.StaticLayoutProvider = StaticLayoutProvider;
    StaticLayoutProvider.__name__ = "StaticLayoutProvider";
    StaticLayoutProvider.init_StaticLayoutProvider();
},
/* models/grids/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var grid_1 = require(384) /* ./grid */;
    exports.Grid = grid_1.Grid;
},
/* models/grids/grid.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var guide_renderer_1 = require(293) /* ../renderers/guide_renderer */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(123) /* ../../core/util/types */;
    var GridView = /** @class */ (function (_super) {
        tslib_1.__extends(GridView, _super);
        function GridView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        GridView.prototype._render = function () {
            var ctx = this.layer.ctx;
            ctx.save();
            this._draw_regions(ctx);
            this._draw_minor_grids(ctx);
            this._draw_grids(ctx);
            ctx.restore();
        };
        GridView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.change, function () { return _this.request_render(); });
        };
        GridView.prototype._draw_regions = function (ctx) {
            var _this = this;
            if (!this.visuals.band_fill.doit && !this.visuals.band_hatch.doit)
                return;
            this.visuals.band_fill.set_value(ctx);
            var _a = tslib_1.__read(this.grid_coords('major', false), 2), xs = _a[0], ys = _a[1];
            var _loop_1 = function (i) {
                if (i % 2 != 1)
                    return "continue";
                var _a = tslib_1.__read(this_1.coordinates.map_to_screen(xs[i], ys[i]), 2), sx0 = _a[0], sy0 = _a[1];
                var _b = tslib_1.__read(this_1.coordinates.map_to_screen(xs[i + 1], ys[i + 1]), 2), sx1 = _b[0], sy1 = _b[1];
                if (this_1.visuals.band_fill.doit)
                    ctx.fillRect(sx0[0], sy0[0], sx1[1] - sx0[0], sy1[1] - sy0[0]);
                this_1.visuals.band_hatch.doit2(ctx, i, function () {
                    ctx.fillRect(sx0[0], sy0[0], sx1[1] - sx0[0], sy1[1] - sy0[0]);
                }, function () { return _this.request_render(); });
            };
            var this_1 = this;
            for (var i = 0; i < xs.length - 1; i++) {
                _loop_1(i);
            }
        };
        GridView.prototype._draw_grids = function (ctx) {
            if (!this.visuals.grid_line.doit)
                return;
            var _a = tslib_1.__read(this.grid_coords('major'), 2), xs = _a[0], ys = _a[1];
            this._draw_grid_helper(ctx, this.visuals.grid_line, xs, ys);
        };
        GridView.prototype._draw_minor_grids = function (ctx) {
            if (!this.visuals.minor_grid_line.doit)
                return;
            var _a = tslib_1.__read(this.grid_coords('minor'), 2), xs = _a[0], ys = _a[1];
            this._draw_grid_helper(ctx, this.visuals.minor_grid_line, xs, ys);
        };
        GridView.prototype._draw_grid_helper = function (ctx, visuals, xs, ys) {
            visuals.set_value(ctx);
            ctx.beginPath();
            for (var i = 0; i < xs.length; i++) {
                var _a = tslib_1.__read(this.coordinates.map_to_screen(xs[i], ys[i]), 2), sx = _a[0], sy = _a[1];
                ctx.moveTo(Math.round(sx[0]), Math.round(sy[0]));
                for (var i_1 = 1; i_1 < sx.length; i_1++) {
                    ctx.lineTo(Math.round(sx[i_1]), Math.round(sy[i_1]));
                }
            }
            ctx.stroke();
        };
        // {{{ TODO: state
        GridView.prototype.ranges = function () {
            var i = this.model.dimension;
            var j = (i + 1) % 2;
            var ranges = this.coordinates.ranges;
            return [ranges[i], ranges[j]];
        };
        GridView.prototype.computed_bounds = function () {
            var _a, e_1, _b, _c;
            var _d = tslib_1.__read(this.ranges(), 1), range = _d[0];
            var user_bounds = this.model.bounds;
            var range_bounds = [range.min, range.max];
            var start;
            var end;
            if (types_1.isArray(user_bounds)) {
                start = Math.min(user_bounds[0], user_bounds[1]);
                end = Math.max(user_bounds[0], user_bounds[1]);
                if (start < range_bounds[0])
                    start = range_bounds[0];
                // XXX:
                //else if (start > range_bounds[1])
                //  start = null
                if (end > range_bounds[1])
                    end = range_bounds[1];
                // XXX:
                //else if (end < range_bounds[0])
                //  end = null
            }
            else {
                _a = tslib_1.__read(range_bounds, 2), start = _a[0], end = _a[1];
                try {
                    for (var _e = tslib_1.__values(this.plot_view.axis_views), _f = _e.next(); !_f.done; _f = _e.next()) {
                        var axis_view = _f.value;
                        if (axis_view.dimension == this.model.dimension
                            && axis_view.model.x_range_name == this.model.x_range_name
                            && axis_view.model.y_range_name == this.model.y_range_name) {
                            _c = tslib_1.__read(axis_view.computed_bounds, 2), start = _c[0], end = _c[1];
                        }
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (_f && !_f.done && (_b = _e.return))
                            _b.call(_e);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
            }
            return [start, end];
        };
        GridView.prototype.grid_coords = function (location, exclude_ends) {
            var _a;
            if (exclude_ends === void 0) {
                exclude_ends = true;
            }
            var i = this.model.dimension;
            var j = (i + 1) % 2;
            var _b = tslib_1.__read(this.ranges(), 2), range = _b[0], cross_range = _b[1];
            var _c = tslib_1.__read(this.computed_bounds(), 2), start = _c[0], end = _c[1];
            _a = tslib_1.__read([Math.min(start, end), Math.max(start, end)], 2), start = _a[0], end = _a[1];
            var coords = [[], []];
            // TODO: (bev) using cross_range.min for cross_loc is a bit of a cheat. Since we
            // currently only support "straight line" grids, this should be OK for now. If
            // we ever want to support "curved" grids, e.g. for some projections, we may
            // have to communicate more than just a single cross location.
            var ticker = this.model.get_ticker();
            if (ticker == null) {
                return coords;
            }
            var ticks = ticker.get_ticks(start, end, range, cross_range.min, {})[location];
            var min = range.min;
            var max = range.max;
            var cmin = cross_range.min;
            var cmax = cross_range.max;
            if (!exclude_ends) {
                if (ticks[0] != min)
                    ticks.splice(0, 0, min);
                if (ticks[ticks.length - 1] != max)
                    ticks.push(max);
            }
            for (var ii = 0; ii < ticks.length; ii++) {
                if ((ticks[ii] == min || ticks[ii] == max) && exclude_ends)
                    continue;
                var dim_i = [];
                var dim_j = [];
                var N = 2;
                for (var n = 0; n < N; n++) {
                    var loc = cmin + (cmax - cmin) / (N - 1) * n;
                    dim_i.push(ticks[ii]);
                    dim_j.push(loc);
                }
                coords[i].push(dim_i);
                coords[j].push(dim_j);
            }
            return coords;
        };
        return GridView;
    }(guide_renderer_1.GuideRendererView));
    exports.GridView = GridView;
    GridView.__name__ = "GridView";
    var Grid = /** @class */ (function (_super) {
        tslib_1.__extends(Grid, _super);
        function Grid(attrs) {
            return _super.call(this, attrs) || this;
        }
        Grid.init_Grid = function () {
            this.prototype.default_view = GridView;
            this.mixins([
                ["grid_", mixins.Line],
                ["minor_grid_", mixins.Line],
                ["band_", mixins.Fill],
                ["band_", mixins.Hatch],
            ]);
            this.define({
                bounds: [p.Any, 'auto'],
                dimension: [p.Any, 0],
                axis: [p.Instance],
                ticker: [p.Instance],
            });
            this.override({
                level: "underlay",
                band_fill_color: null,
                band_fill_alpha: 0,
                grid_line_color: '#e5e5e5',
                minor_grid_line_color: null,
            });
        };
        Grid.prototype.get_ticker = function () {
            if (this.ticker != null) {
                return this.ticker;
            }
            if (this.axis != null) {
                return this.axis.ticker;
            }
            return null;
        };
        return Grid;
    }(guide_renderer_1.GuideRenderer));
    exports.Grid = Grid;
    Grid.__name__ = "Grid";
    Grid.init_Grid();
},
/* models/layouts/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var box_1 = require(386) /* ./box */;
    exports.Box = box_1.Box;
    var column_1 = require(389) /* ./column */;
    exports.Column = column_1.Column;
    var grid_box_1 = require(390) /* ./grid_box */;
    exports.GridBox = grid_box_1.GridBox;
    var html_box_1 = require(391) /* ./html_box */;
    exports.HTMLBox = html_box_1.HTMLBox;
    var layout_dom_1 = require(387) /* ./layout_dom */;
    exports.LayoutDOM = layout_dom_1.LayoutDOM;
    var row_1 = require(392) /* ./row */;
    exports.Row = row_1.Row;
    var spacer_1 = require(393) /* ./spacer */;
    exports.Spacer = spacer_1.Spacer;
    var tabs_1 = require(394) /* ./tabs */;
    exports.Panel = tabs_1.Panel;
    exports.Tabs = tabs_1.Tabs;
    var widget_box_1 = require(401) /* ./widget_box */;
    exports.WidgetBox = widget_box_1.WidgetBox;
},
/* models/layouts/box.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var layout_dom_1 = require(387) /* ./layout_dom */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var BoxView = /** @class */ (function (_super) {
        tslib_1.__extends(BoxView, _super);
        function BoxView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BoxView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.properties.children.change, function () { return _this.rebuild(); });
        };
        Object.defineProperty(BoxView.prototype, "child_models", {
            get: function () {
                return this.model.children;
            },
            enumerable: true,
            configurable: true
        });
        return BoxView;
    }(layout_dom_1.LayoutDOMView));
    exports.BoxView = BoxView;
    BoxView.__name__ = "BoxView";
    var Box = /** @class */ (function (_super) {
        tslib_1.__extends(Box, _super);
        function Box(attrs) {
            return _super.call(this, attrs) || this;
        }
        Box.init_Box = function () {
            this.define({
                children: [p.Array, []],
                spacing: [p.Number, 0],
            });
        };
        return Box;
    }(layout_dom_1.LayoutDOM));
    exports.Box = Box;
    Box.__name__ = "Box";
    Box.init_Box();
},
/* models/layouts/layout_dom.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var enums_1 = require(135) /* ../../core/enums */;
    var dom_1 = require(187) /* ../../core/dom */;
    var logging_1 = require(134) /* ../../core/logging */;
    var types_1 = require(123) /* ../../core/util/types */;
    var build_views_1 = require(230) /* ../../core/build_views */;
    var dom_view_1 = require(193) /* ../../core/dom_view */;
    var layout_1 = require(327) /* ../../core/layout */;
    var root_1 = require(388) /* ../../styles/root */;
    var canvas_1 = require(192) /* ../canvas/canvas */;
    var LayoutDOMView = /** @class */ (function (_super) {
        tslib_1.__extends(LayoutDOMView, _super);
        function LayoutDOMView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this._idle_notified = false;
            _this._offset_parent = null;
            _this._viewport = {};
            return _this;
        }
        LayoutDOMView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.el.style.position = this.is_root ? "relative" : "absolute";
            this._child_views = new Map();
        };
        LayoutDOMView.prototype.lazy_initialize = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, this.build_child_views()];
                        case 1:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            });
        };
        LayoutDOMView.prototype.remove = function () {
            var e_1, _a;
            try {
                for (var _b = tslib_1.__values(this.child_views), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var child_view = _c.value;
                    child_view.remove();
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            this._child_views.clear();
            _super.prototype.remove.call(this);
        };
        LayoutDOMView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            if (this.is_root) {
                this._on_resize = function () { return _this.resize_layout(); };
                window.addEventListener("resize", this._on_resize);
                this._parent_observer = setInterval(function () {
                    var offset_parent = _this.el.offsetParent;
                    if (_this._offset_parent != offset_parent) {
                        _this._offset_parent = offset_parent;
                        if (offset_parent != null) {
                            _this.compute_viewport();
                            _this.invalidate_layout();
                        }
                    }
                }, 250);
            }
            var p = this.model.properties;
            this.on_change([
                p.width, p.height,
                p.min_width, p.min_height,
                p.max_width, p.max_height,
                p.margin,
                p.width_policy, p.height_policy, p.sizing_mode,
                p.aspect_ratio,
                p.visible,
            ], function () { return _this.invalidate_layout(); });
            this.on_change([
                p.background,
                p.css_classes,
            ], function () { return _this.invalidate_render(); });
        };
        LayoutDOMView.prototype.disconnect_signals = function () {
            if (this._parent_observer != null)
                clearTimeout(this._parent_observer);
            if (this._on_resize != null)
                window.removeEventListener("resize", this._on_resize);
            _super.prototype.disconnect_signals.call(this);
        };
        LayoutDOMView.prototype.css_classes = function () {
            return _super.prototype.css_classes.call(this).concat(this.model.css_classes);
        };
        Object.defineProperty(LayoutDOMView.prototype, "child_views", {
            get: function () {
                var _this = this;
                return this.child_models.map(function (child) { return _this._child_views.get(child); });
            },
            enumerable: true,
            configurable: true
        });
        LayoutDOMView.prototype.build_child_views = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, build_views_1.build_views(this._child_views, this.child_models, { parent: this })];
                        case 1:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            });
        };
        LayoutDOMView.prototype.render = function () {
            var _a, e_2, _b;
            _super.prototype.render.call(this);
            dom_1.empty(this.el); // XXX: this should be in super
            var background = this.model.background;
            this.el.style.backgroundColor = background != null ? background : "";
            (_a = dom_1.classes(this.el).clear()).add.apply(_a, tslib_1.__spread(this.css_classes()));
            try {
                for (var _c = tslib_1.__values(this.child_views), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var child_view = _d.value;
                    this.el.appendChild(child_view.el);
                    child_view.render();
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
        };
        LayoutDOMView.prototype.update_layout = function () {
            var e_3, _a;
            try {
                for (var _b = tslib_1.__values(this.child_views), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var child_view = _c.value;
                    child_view.update_layout();
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            this._update_layout();
        };
        LayoutDOMView.prototype.update_position = function () {
            var e_4, _a;
            this.el.style.display = this.model.visible ? "block" : "none";
            var margin = this.is_root ? this.layout.sizing.margin : undefined;
            dom_1.position(this.el, this.layout.bbox, margin);
            try {
                for (var _b = tslib_1.__values(this.child_views), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var child_view = _c.value;
                    child_view.update_position();
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
        };
        LayoutDOMView.prototype.after_layout = function () {
            var e_5, _a;
            try {
                for (var _b = tslib_1.__values(this.child_views), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var child_view = _c.value;
                    child_view.after_layout();
                }
            }
            catch (e_5_1) {
                e_5 = { error: e_5_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_5)
                        throw e_5.error;
                }
            }
            this._has_finished = true;
        };
        LayoutDOMView.prototype.compute_viewport = function () {
            this._viewport = this._viewport_size();
        };
        LayoutDOMView.prototype.renderTo = function (element) {
            element.appendChild(this.el);
            this._offset_parent = this.el.offsetParent;
            this.compute_viewport();
            this.build();
        };
        LayoutDOMView.prototype.build = function () {
            this.assert_root();
            this.render();
            this.update_layout();
            this.compute_layout();
            return this;
        };
        LayoutDOMView.prototype.rebuild = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, this.build_child_views()];
                        case 1:
                            _a.sent();
                            this.invalidate_render();
                            return [2 /*return*/];
                    }
                });
            });
        };
        LayoutDOMView.prototype.compute_layout = function () {
            var start = Date.now();
            this.layout.compute(this._viewport);
            this.update_position();
            this.after_layout();
            logging_1.logger.debug("layout computed in " + (Date.now() - start) + " ms");
            this.notify_finished();
        };
        LayoutDOMView.prototype.resize_layout = function () {
            this.root.compute_viewport();
            this.root.compute_layout();
        };
        LayoutDOMView.prototype.invalidate_layout = function () {
            this.root.update_layout();
            this.root.compute_layout();
        };
        LayoutDOMView.prototype.invalidate_render = function () {
            this.render();
            this.invalidate_layout();
        };
        LayoutDOMView.prototype.has_finished = function () {
            var e_6, _a;
            if (!_super.prototype.has_finished.call(this))
                return false;
            try {
                for (var _b = tslib_1.__values(this.child_views), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var child_view = _c.value;
                    if (!child_view.has_finished())
                        return false;
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
            return true;
        };
        LayoutDOMView.prototype.notify_finished = function () {
            if (!this.is_root)
                this.root.notify_finished();
            else {
                if (!this._idle_notified && this.has_finished()) {
                    if (this.model.document != null) {
                        this._idle_notified = true;
                        this.model.document.notify_idle(this.model);
                    }
                }
            }
        };
        LayoutDOMView.prototype._width_policy = function () {
            return this.model.width != null ? "fixed" : "fit";
        };
        LayoutDOMView.prototype._height_policy = function () {
            return this.model.height != null ? "fixed" : "fit";
        };
        LayoutDOMView.prototype.box_sizing = function () {
            var _a;
            var _b = this.model, width_policy = _b.width_policy, height_policy = _b.height_policy, aspect_ratio = _b.aspect_ratio;
            if (width_policy == "auto")
                width_policy = this._width_policy();
            if (height_policy == "auto")
                height_policy = this._height_policy();
            var sizing_mode = this.model.sizing_mode;
            if (sizing_mode != null) {
                if (sizing_mode == "fixed")
                    width_policy = height_policy = "fixed";
                else if (sizing_mode == "stretch_both")
                    width_policy = height_policy = "max";
                else if (sizing_mode == "stretch_width")
                    width_policy = "max";
                else if (sizing_mode == "stretch_height")
                    height_policy = "max";
                else {
                    if (aspect_ratio == null)
                        aspect_ratio = "auto";
                    switch (sizing_mode) {
                        case "scale_width":
                            width_policy = "max";
                            height_policy = "min";
                            break;
                        case "scale_height":
                            width_policy = "min";
                            height_policy = "max";
                            break;
                        case "scale_both":
                            width_policy = "max";
                            height_policy = "max";
                            break;
                    }
                }
            }
            var sizing = { width_policy: width_policy, height_policy: height_policy };
            var _c = this.model, min_width = _c.min_width, min_height = _c.min_height;
            if (min_width != null)
                sizing.min_width = min_width;
            if (min_height != null)
                sizing.min_height = min_height;
            var _d = this.model, width = _d.width, height = _d.height;
            if (width != null)
                sizing.width = width;
            if (height != null)
                sizing.height = height;
            var _e = this.model, max_width = _e.max_width, max_height = _e.max_height;
            if (max_width != null)
                sizing.max_width = max_width;
            if (max_height != null)
                sizing.max_height = max_height;
            if (aspect_ratio == "auto" && width != null && height != null)
                sizing.aspect = width / height;
            else if (types_1.isNumber(aspect_ratio))
                sizing.aspect = aspect_ratio;
            var margin = this.model.margin;
            if (margin != null) {
                if (types_1.isNumber(margin))
                    sizing.margin = { top: margin, right: margin, bottom: margin, left: margin };
                else if (margin.length == 2) {
                    var _f = tslib_1.__read(margin, 2), vertical = _f[0], horizontal = _f[1];
                    sizing.margin = { top: vertical, right: horizontal, bottom: vertical, left: horizontal };
                }
                else {
                    var _g = tslib_1.__read(margin, 4), top = _g[0], right = _g[1], bottom = _g[2], left = _g[3];
                    sizing.margin = { top: top, right: right, bottom: bottom, left: left };
                }
            }
            sizing.visible = this.model.visible;
            var align = this.model.align;
            if (types_1.isArray(align))
                _a = tslib_1.__read(align, 2), sizing.halign = _a[0], sizing.valign = _a[1];
            else
                sizing.halign = sizing.valign = align;
            return sizing;
        };
        LayoutDOMView.prototype._viewport_size = function () {
            var _this = this;
            return dom_1.undisplayed(this.el, function () {
                var measuring = _this.el;
                while (measuring = measuring.parentElement) {
                    // .bk-root element doesn't bring any value
                    if (measuring.classList.contains(root_1.bk_root))
                        continue;
                    // we reached <body> element, so use viewport size
                    if (measuring == document.body) {
                        var _a = dom_1.extents(document.body).margin, left_1 = _a.left, right_1 = _a.right, top_1 = _a.top, bottom_1 = _a.bottom;
                        var width_1 = Math.ceil(document.documentElement.clientWidth - left_1 - right_1);
                        var height_1 = Math.ceil(document.documentElement.clientHeight - top_1 - bottom_1);
                        return { width: width_1, height: height_1 };
                    }
                    // stop on first element with sensible dimensions
                    var _b = dom_1.extents(measuring).padding, left = _b.left, right = _b.right, top = _b.top, bottom = _b.bottom;
                    var _c = measuring.getBoundingClientRect(), width = _c.width, height = _c.height;
                    var inner_width = Math.ceil(width - left - right);
                    var inner_height = Math.ceil(height - top - bottom);
                    if (inner_width > 0 || inner_height > 0)
                        return {
                            width: inner_width > 0 ? inner_width : undefined,
                            height: inner_height > 0 ? inner_height : undefined,
                        };
                }
                // this element is detached from DOM
                return {};
            });
        };
        LayoutDOMView.prototype.export = function (type, hidpi) {
            var e_7, _a;
            if (hidpi === void 0) {
                hidpi = true;
            }
            var output_backend = type == "png" ? "canvas" : "svg";
            var composite = new canvas_1.CanvasLayer(output_backend, hidpi);
            var _b = this.layout.bbox, width = _b.width, height = _b.height;
            composite.resize(width, height);
            try {
                for (var _c = tslib_1.__values(this.child_views), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var view = _d.value;
                    var region = view.export(type, hidpi);
                    var _e = view.layout.bbox, x = _e.x, y = _e.y;
                    composite.ctx.drawImage(region.canvas, x, y);
                }
            }
            catch (e_7_1) {
                e_7 = { error: e_7_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return))
                        _a.call(_c);
                }
                finally {
                    if (e_7)
                        throw e_7.error;
                }
            }
            return composite;
        };
        LayoutDOMView.prototype.serializable_state = function () {
            return Object.assign(Object.assign({}, _super.prototype.serializable_state.call(this)), { bbox: this.layout.bbox.box, children: this.child_views.map(function (child) { return child.serializable_state(); }) });
        };
        return LayoutDOMView;
    }(dom_view_1.DOMView));
    exports.LayoutDOMView = LayoutDOMView;
    LayoutDOMView.__name__ = "LayoutDOMView";
    var LayoutDOM = /** @class */ (function (_super) {
        tslib_1.__extends(LayoutDOM, _super);
        function LayoutDOM(attrs) {
            return _super.call(this, attrs) || this;
        }
        LayoutDOM.init_LayoutDOM = function () {
            this.define(function (types) {
                var Boolean = types.Boolean, Number = types.Number, String = types.String, Null = types.Null, Auto = types.Auto, Color = types.Color, Array = types.Array, Tuple = types.Tuple, Or = types.Or;
                var Number2 = Tuple(Number, Number);
                var Number4 = Tuple(Number, Number, Number, Number);
                return {
                    width: [Or(Number, Null), null],
                    height: [Or(Number, Null), null],
                    min_width: [Or(Number, Null), null],
                    min_height: [Or(Number, Null), null],
                    max_width: [Or(Number, Null), null],
                    max_height: [Or(Number, Null), null],
                    margin: [Or(Number, Number2, Number4), [0, 0, 0, 0]],
                    width_policy: [Or(layout_1.SizingPolicy, Auto), "auto"],
                    height_policy: [Or(layout_1.SizingPolicy, Auto), "auto"],
                    aspect_ratio: [Or(Number, Auto, Null), null],
                    sizing_mode: [Or(enums_1.SizingMode, Null), null],
                    visible: [Boolean, true],
                    disabled: [Boolean, false],
                    align: [Or(enums_1.Align, Tuple(enums_1.Align, enums_1.Align)), "start"],
                    background: [Or(Color, Null), null],
                    css_classes: [Array(String), []],
                };
            });
        };
        return LayoutDOM;
    }(model_1.Model));
    exports.LayoutDOM = LayoutDOM;
    LayoutDOM.__name__ = "LayoutDOM";
    LayoutDOM.init_LayoutDOM();
},
/* styles/root.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.bk_root = "bk-root";
},
/* models/layouts/column.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var box_1 = require(386) /* ./box */;
    var grid_1 = require(331) /* ../../core/layout/grid */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var ColumnView = /** @class */ (function (_super) {
        tslib_1.__extends(ColumnView, _super);
        function ColumnView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ColumnView.prototype._update_layout = function () {
            var items = this.child_views.map(function (child) { return child.layout; });
            this.layout = new grid_1.Column(items);
            this.layout.rows = this.model.rows;
            this.layout.spacing = [this.model.spacing, 0];
            this.layout.set_sizing(this.box_sizing());
        };
        return ColumnView;
    }(box_1.BoxView));
    exports.ColumnView = ColumnView;
    ColumnView.__name__ = "ColumnView";
    var Column = /** @class */ (function (_super) {
        tslib_1.__extends(Column, _super);
        function Column(attrs) {
            return _super.call(this, attrs) || this;
        }
        Column.init_Column = function () {
            this.prototype.default_view = ColumnView;
            this.define({
                rows: [p.Any, "auto"],
            });
        };
        return Column;
    }(box_1.Box));
    exports.Column = Column;
    Column.__name__ = "Column";
    Column.init_Column();
},
/* models/layouts/grid_box.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var layout_dom_1 = require(387) /* ./layout_dom */;
    var grid_1 = require(331) /* ../../core/layout/grid */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var GridBoxView = /** @class */ (function (_super) {
        tslib_1.__extends(GridBoxView, _super);
        function GridBoxView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        GridBoxView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.properties.children.change, function () { return _this.rebuild(); });
        };
        Object.defineProperty(GridBoxView.prototype, "child_models", {
            get: function () {
                return this.model.children.map(function (_a) {
                    var _b = tslib_1.__read(_a, 1), child = _b[0];
                    return child;
                });
            },
            enumerable: true,
            configurable: true
        });
        GridBoxView.prototype._update_layout = function () {
            var e_1, _a;
            this.layout = new grid_1.Grid();
            this.layout.rows = this.model.rows;
            this.layout.cols = this.model.cols;
            this.layout.spacing = this.model.spacing;
            try {
                for (var _b = tslib_1.__values(this.model.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var _d = tslib_1.__read(_c.value, 5), child = _d[0], row = _d[1], col = _d[2], row_span = _d[3], col_span = _d[4];
                    var child_view = this._child_views.get(child);
                    this.layout.items.push({ layout: child_view.layout, row: row, col: col, row_span: row_span, col_span: col_span });
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            this.layout.set_sizing(this.box_sizing());
        };
        return GridBoxView;
    }(layout_dom_1.LayoutDOMView));
    exports.GridBoxView = GridBoxView;
    GridBoxView.__name__ = "GridBoxView";
    var GridBox = /** @class */ (function (_super) {
        tslib_1.__extends(GridBox, _super);
        function GridBox(attrs) {
            return _super.call(this, attrs) || this;
        }
        GridBox.init_GridBox = function () {
            this.prototype.default_view = GridBoxView;
            this.define({
                children: [p.Array, []],
                rows: [p.Any, "auto"],
                cols: [p.Any, "auto"],
                spacing: [p.Any, 0],
            });
        };
        return GridBox;
    }(layout_dom_1.LayoutDOM));
    exports.GridBox = GridBox;
    GridBox.__name__ = "GridBox";
    GridBox.init_GridBox();
},
/* models/layouts/html_box.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var layout_dom_1 = require(387) /* ../layouts/layout_dom */;
    var layout_1 = require(327) /* ../../core/layout */;
    var HTMLBoxView = /** @class */ (function (_super) {
        tslib_1.__extends(HTMLBoxView, _super);
        function HTMLBoxView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Object.defineProperty(HTMLBoxView.prototype, "child_models", {
            get: function () {
                return [];
            },
            enumerable: true,
            configurable: true
        });
        HTMLBoxView.prototype._update_layout = function () {
            this.layout = new layout_1.ContentBox(this.el);
            this.layout.set_sizing(this.box_sizing());
        };
        return HTMLBoxView;
    }(layout_dom_1.LayoutDOMView));
    exports.HTMLBoxView = HTMLBoxView;
    HTMLBoxView.__name__ = "HTMLBoxView";
    var HTMLBox = /** @class */ (function (_super) {
        tslib_1.__extends(HTMLBox, _super);
        function HTMLBox(attrs) {
            return _super.call(this, attrs) || this;
        }
        return HTMLBox;
    }(layout_dom_1.LayoutDOM));
    exports.HTMLBox = HTMLBox;
    HTMLBox.__name__ = "HTMLBox";
},
/* models/layouts/row.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var box_1 = require(386) /* ./box */;
    var grid_1 = require(331) /* ../../core/layout/grid */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var RowView = /** @class */ (function (_super) {
        tslib_1.__extends(RowView, _super);
        function RowView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        RowView.prototype._update_layout = function () {
            var items = this.child_views.map(function (child) { return child.layout; });
            this.layout = new grid_1.Row(items);
            this.layout.cols = this.model.cols;
            this.layout.spacing = [0, this.model.spacing];
            this.layout.set_sizing(this.box_sizing());
        };
        return RowView;
    }(box_1.BoxView));
    exports.RowView = RowView;
    RowView.__name__ = "RowView";
    var Row = /** @class */ (function (_super) {
        tslib_1.__extends(Row, _super);
        function Row(attrs) {
            return _super.call(this, attrs) || this;
        }
        Row.init_Row = function () {
            this.prototype.default_view = RowView;
            this.define({
                cols: [p.Any, "auto"],
            });
        };
        return Row;
    }(box_1.Box));
    exports.Row = Row;
    Row.__name__ = "Row";
    Row.init_Row();
},
/* models/layouts/spacer.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var layout_dom_1 = require(387) /* ./layout_dom */;
    var layout_1 = require(327) /* ../../core/layout */;
    var SpacerView = /** @class */ (function (_super) {
        tslib_1.__extends(SpacerView, _super);
        function SpacerView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Object.defineProperty(SpacerView.prototype, "child_models", {
            get: function () {
                return [];
            },
            enumerable: true,
            configurable: true
        });
        SpacerView.prototype._update_layout = function () {
            this.layout = new layout_1.LayoutItem();
            this.layout.set_sizing(this.box_sizing());
        };
        return SpacerView;
    }(layout_dom_1.LayoutDOMView));
    exports.SpacerView = SpacerView;
    SpacerView.__name__ = "SpacerView";
    var Spacer = /** @class */ (function (_super) {
        tslib_1.__extends(Spacer, _super);
        function Spacer(attrs) {
            return _super.call(this, attrs) || this;
        }
        Spacer.init_Spacer = function () {
            this.prototype.default_view = SpacerView;
        };
        return Spacer;
    }(layout_dom_1.LayoutDOM));
    exports.Spacer = Spacer;
    Spacer.__name__ = "Spacer";
    Spacer.init_Spacer();
},
/* models/layouts/tabs.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var layout_1 = require(327) /* ../../core/layout */;
    var dom_1 = require(187) /* ../../core/dom */;
    var array_1 = require(124) /* ../../core/util/array */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var layout_dom_1 = require(387) /* ./layout_dom */;
    var model_1 = require(196) /* ../../model */;
    var mixins_1 = require(288) /* ../../styles/mixins */;
    var tabs_1 = require(395) /* ../../styles/tabs */;
    var buttons_1 = require(396) /* ../../styles/buttons */;
    var menus_1 = require(397) /* ../../styles/menus */;
    var buttons_css_1 = tslib_1.__importDefault(require(398) /* ../../styles/buttons.css */);
    var menus_css_1 = tslib_1.__importDefault(require(399) /* ../../styles/menus.css */);
    var tabs_css_1 = tslib_1.__importDefault(require(400) /* ../../styles/tabs.css */);
    var TabsView = /** @class */ (function (_super) {
        tslib_1.__extends(TabsView, _super);
        function TabsView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        TabsView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.properties.tabs.change, function () { return _this.rebuild(); });
            this.connect(this.model.properties.active.change, function () { return _this.on_active_change(); });
        };
        TabsView.prototype.styles = function () {
            return tslib_1.__spread(_super.prototype.styles.call(this), [buttons_css_1.default, menus_css_1.default, tabs_css_1.default]);
        };
        Object.defineProperty(TabsView.prototype, "child_models", {
            get: function () {
                return this.model.tabs.map(function (tab) { return tab.child; });
            },
            enumerable: true,
            configurable: true
        });
        TabsView.prototype._update_layout = function () {
            var loc = this.model.tabs_location;
            var vertical = loc == "above" || loc == "below";
            // XXX: this is a hack, this should be handled by "fit" policy in grid layout
            var _a = this, scroll_el = _a.scroll_el, headers_el = _a.headers_el;
            this.header = new /** @class */ (function (_super) {
                tslib_1.__extends(class_1, _super);
                function class_1() {
                    return _super !== null && _super.apply(this, arguments) || this;
                }
                class_1.prototype._measure = function (viewport) {
                    var min_headers = 3;
                    var scroll = dom_1.size(scroll_el);
                    var headers = dom_1.children(headers_el).slice(0, min_headers).map(function (el) { return dom_1.size(el); });
                    var _a = _super.prototype._measure.call(this, viewport), width = _a.width, height = _a.height;
                    if (vertical) {
                        var min_width = scroll.width + array_1.sum(headers.map(function (size) { return size.width; }));
                        return { width: viewport.width != Infinity ? viewport.width : min_width, height: height };
                    }
                    else {
                        var min_height = scroll.height + array_1.sum(headers.map(function (size) { return size.height; }));
                        return { width: width, height: viewport.height != Infinity ? viewport.height : min_height };
                    }
                };
                return class_1;
            }(layout_1.ContentBox))(this.header_el);
            if (vertical)
                this.header.set_sizing({ width_policy: "fit", height_policy: "fixed" });
            else
                this.header.set_sizing({ width_policy: "fixed", height_policy: "fit" });
            var row = 1;
            var col = 1;
            switch (loc) {
                case "above":
                    row -= 1;
                    break;
                case "below":
                    row += 1;
                    break;
                case "left":
                    col -= 1;
                    break;
                case "right":
                    col += 1;
                    break;
            }
            var header = { layout: this.header, row: row, col: col };
            var panels = this.child_views.map(function (child_view) {
                return { layout: child_view.layout, row: 1, col: 1 };
            });
            this.layout = new layout_1.Grid(tslib_1.__spread([header], panels));
            this.layout.set_sizing(this.box_sizing());
        };
        TabsView.prototype.update_position = function () {
            var e_1, _a;
            _super.prototype.update_position.call(this);
            this.header_el.style.position = "absolute"; // XXX: do it in position()
            dom_1.position(this.header_el, this.header.bbox);
            var loc = this.model.tabs_location;
            var vertical = loc == "above" || loc == "below";
            var scroll_el_size = dom_1.size(this.scroll_el);
            var headers_el_size = dom_1.scroll_size(this.headers_el);
            if (vertical) {
                var width = this.header.bbox.width;
                if (headers_el_size.width > width) {
                    this.wrapper_el.style.maxWidth = width - scroll_el_size.width + "px";
                    dom_1.display(this.scroll_el);
                }
                else {
                    this.wrapper_el.style.maxWidth = "";
                    dom_1.undisplay(this.scroll_el);
                }
            }
            else {
                var height = this.header.bbox.height;
                if (headers_el_size.height > height) {
                    this.wrapper_el.style.maxHeight = height - scroll_el_size.height + "px";
                    dom_1.display(this.scroll_el);
                }
                else {
                    this.wrapper_el.style.maxHeight = "";
                    dom_1.undisplay(this.scroll_el);
                }
            }
            var child_views = this.child_views;
            try {
                for (var child_views_1 = tslib_1.__values(child_views), child_views_1_1 = child_views_1.next(); !child_views_1_1.done; child_views_1_1 = child_views_1.next()) {
                    var child_view = child_views_1_1.value;
                    dom_1.hide(child_view.el);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (child_views_1_1 && !child_views_1_1.done && (_a = child_views_1.return))
                        _a.call(child_views_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            var tab = child_views[this.model.active];
            if (tab != null)
                dom_1.show(tab.el);
        };
        TabsView.prototype.render = function () {
            var _this = this;
            _super.prototype.render.call(this);
            var active = this.model.active;
            var loc = this.model.tabs_location;
            var vertical = loc == "above" || loc == "below";
            var headers = this.model.tabs.map(function (tab, i) {
                var el = dom_1.div({ class: [tabs_1.bk_tab, i == active ? mixins_1.bk_active : null] }, tab.title);
                el.addEventListener("click", function (event) {
                    if (event.target == event.currentTarget)
                        _this.change_active(i);
                });
                if (tab.closable) {
                    var close_el = dom_1.div({ class: tabs_1.bk_close });
                    close_el.addEventListener("click", function (event) {
                        if (event.target == event.currentTarget) {
                            _this.model.tabs = array_1.remove_at(_this.model.tabs, i);
                            var ntabs = _this.model.tabs.length;
                            if (_this.model.active > ntabs - 1)
                                _this.model.active = ntabs - 1;
                        }
                    });
                    el.appendChild(close_el);
                }
                return el;
            });
            this.headers_el = dom_1.div({ class: [tabs_1.bk_headers] }, headers);
            this.wrapper_el = dom_1.div({ class: tabs_1.bk_headers_wrapper }, this.headers_el);
            var left_el = dom_1.div({ class: [buttons_1.bk_btn, buttons_1.bk_btn_default], disabled: "" }, dom_1.div({ class: [menus_1.bk_caret, mixins_1.bk_left] }));
            var right_el = dom_1.div({ class: [buttons_1.bk_btn, buttons_1.bk_btn_default] }, dom_1.div({ class: [menus_1.bk_caret, mixins_1.bk_right] }));
            var scroll_index = 0;
            var do_scroll = function (dir) {
                return function () {
                    var ntabs = _this.model.tabs.length;
                    if (dir == "left")
                        scroll_index = Math.max(scroll_index - 1, 0);
                    else
                        scroll_index = Math.min(scroll_index + 1, ntabs - 1);
                    if (scroll_index == 0)
                        left_el.setAttribute("disabled", "");
                    else
                        left_el.removeAttribute("disabled");
                    if (scroll_index == ntabs - 1)
                        right_el.setAttribute("disabled", "");
                    else
                        right_el.removeAttribute("disabled");
                    var sizes = dom_1.children(_this.headers_el)
                        .slice(0, scroll_index)
                        .map(function (el) { return el.getBoundingClientRect(); });
                    if (vertical) {
                        var left = -array_1.sum(sizes.map(function (size) { return size.width; }));
                        _this.headers_el.style.left = left + "px";
                    }
                    else {
                        var top = -array_1.sum(sizes.map(function (size) { return size.height; }));
                        _this.headers_el.style.top = top + "px";
                    }
                };
            };
            left_el.addEventListener("click", do_scroll("left"));
            right_el.addEventListener("click", do_scroll("right"));
            this.scroll_el = dom_1.div({ class: buttons_1.bk_btn_group }, left_el, right_el);
            this.header_el = dom_1.div({ class: [tabs_1.bk_tabs_header, mixins_1.bk_side(loc)] }, this.scroll_el, this.wrapper_el);
            this.el.appendChild(this.header_el);
        };
        TabsView.prototype.change_active = function (i) {
            if (i != this.model.active) {
                this.model.active = i;
            }
        };
        TabsView.prototype.on_active_change = function () {
            var e_2, _a, e_3, _b;
            var i = this.model.active;
            var headers = dom_1.children(this.headers_el);
            try {
                for (var headers_1 = tslib_1.__values(headers), headers_1_1 = headers_1.next(); !headers_1_1.done; headers_1_1 = headers_1.next()) {
                    var el = headers_1_1.value;
                    el.classList.remove(mixins_1.bk_active);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (headers_1_1 && !headers_1_1.done && (_a = headers_1.return))
                        _a.call(headers_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            headers[i].classList.add(mixins_1.bk_active);
            var child_views = this.child_views;
            try {
                for (var child_views_2 = tslib_1.__values(child_views), child_views_2_1 = child_views_2.next(); !child_views_2_1.done; child_views_2_1 = child_views_2.next()) {
                    var child_view = child_views_2_1.value;
                    dom_1.hide(child_view.el);
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (child_views_2_1 && !child_views_2_1.done && (_b = child_views_2.return))
                        _b.call(child_views_2);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            dom_1.show(child_views[i].el);
        };
        return TabsView;
    }(layout_dom_1.LayoutDOMView));
    exports.TabsView = TabsView;
    TabsView.__name__ = "TabsView";
    var Tabs = /** @class */ (function (_super) {
        tslib_1.__extends(Tabs, _super);
        function Tabs(attrs) {
            return _super.call(this, attrs) || this;
        }
        Tabs.init_Tabs = function () {
            this.prototype.default_view = TabsView;
            this.define({
                tabs: [p.Array, []],
                tabs_location: [p.Location, "above"],
                active: [p.Number, 0],
            });
        };
        return Tabs;
    }(layout_dom_1.LayoutDOM));
    exports.Tabs = Tabs;
    Tabs.__name__ = "Tabs";
    Tabs.init_Tabs();
    var Panel = /** @class */ (function (_super) {
        tslib_1.__extends(Panel, _super);
        function Panel(attrs) {
            return _super.call(this, attrs) || this;
        }
        Panel.init_Panel = function () {
            this.define({
                title: [p.String, ""],
                child: [p.Instance],
                closable: [p.Boolean, false],
            });
        };
        return Panel;
    }(model_1.Model));
    exports.Panel = Panel;
    Panel.__name__ = "Panel";
    Panel.init_Panel();
},
/* styles/tabs.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.bk_tabs_header = "bk-tabs-header";
    exports.bk_headers_wrapper = "bk-headers-wrapper";
    exports.bk_headers = "bk-headers";
    exports.bk_tab = "bk-tab";
    exports.bk_close = "bk-close";
},
/* styles/buttons.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.bk_btn = "bk-btn";
    exports.bk_btn_group = "bk-btn-group";
    exports.bk_btn_default = "bk-btn-default";
    exports.bk_btn_primary = "bk-btn-primary";
    exports.bk_btn_success = "bk-btn-success";
    exports.bk_btn_warning = "bk-btn-warning";
    exports.bk_btn_danger = "bk-btn-danger";
    function bk_btn_type(button_type) {
        switch (button_type) {
            case "default": return exports.bk_btn_default;
            case "primary": return exports.bk_btn_primary;
            case "success": return exports.bk_btn_success;
            case "warning": return exports.bk_btn_warning;
            case "danger": return exports.bk_btn_danger;
        }
    }
    exports.bk_btn_type = bk_btn_type;
    exports.bk_dropdown_toggle = "bk-dropdown-toggle";
},
/* styles/menus.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.bk_menu = "bk-menu";
    exports.bk_caret = "bk-caret";
    exports.bk_divider = "bk-divider";
},
/* styles/buttons.css.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var css = "\n.bk-root .bk-btn {\n  height: 100%;\n  display: inline-block;\n  text-align: center;\n  vertical-align: middle;\n  white-space: nowrap;\n  cursor: pointer;\n  padding: 6px 12px;\n  font-size: 12px;\n  border: 1px solid transparent;\n  border-radius: 4px;\n  outline: 0;\n  user-select: none;\n  -ms-user-select: none;\n  -moz-user-select: none;\n  -webkit-user-select: none;\n}\n.bk-root .bk-btn:hover,\n.bk-root .bk-btn:focus {\n  text-decoration: none;\n}\n.bk-root .bk-btn:active,\n.bk-root .bk-btn.bk-active {\n  background-image: none;\n  box-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);\n}\n.bk-root .bk-btn[disabled] {\n  cursor: not-allowed;\n  pointer-events: none;\n  opacity: 0.65;\n  box-shadow: none;\n}\n.bk-root .bk-btn-default {\n  color: #333;\n  background-color: #fff;\n  border-color: #ccc;\n}\n.bk-root .bk-btn-default:hover {\n  background-color: #f5f5f5;\n  border-color: #b8b8b8;\n}\n.bk-root .bk-btn-default.bk-active {\n  background-color: #ebebeb;\n  border-color: #adadad;\n}\n.bk-root .bk-btn-default[disabled],\n.bk-root .bk-btn-default[disabled]:hover,\n.bk-root .bk-btn-default[disabled]:focus,\n.bk-root .bk-btn-default[disabled]:active,\n.bk-root .bk-btn-default[disabled].bk-active {\n  background-color: #e6e6e6;\n  border-color: #ccc;\n}\n.bk-root .bk-btn-primary {\n  color: #fff;\n  background-color: #428bca;\n  border-color: #357ebd;\n}\n.bk-root .bk-btn-primary:hover {\n  background-color: #3681c1;\n  border-color: #2c699e;\n}\n.bk-root .bk-btn-primary.bk-active {\n  background-color: #3276b1;\n  border-color: #285e8e;\n}\n.bk-root .bk-btn-primary[disabled],\n.bk-root .bk-btn-primary[disabled]:hover,\n.bk-root .bk-btn-primary[disabled]:focus,\n.bk-root .bk-btn-primary[disabled]:active,\n.bk-root .bk-btn-primary[disabled].bk-active {\n  background-color: #506f89;\n  border-color: #357ebd;\n}\n.bk-root .bk-btn-success {\n  color: #fff;\n  background-color: #5cb85c;\n  border-color: #4cae4c;\n}\n.bk-root .bk-btn-success:hover {\n  background-color: #4eb24e;\n  border-color: #409240;\n}\n.bk-root .bk-btn-success.bk-active {\n  background-color: #47a447;\n  border-color: #398439;\n}\n.bk-root .bk-btn-success[disabled],\n.bk-root .bk-btn-success[disabled]:hover,\n.bk-root .bk-btn-success[disabled]:focus,\n.bk-root .bk-btn-success[disabled]:active,\n.bk-root .bk-btn-success[disabled].bk-active {\n  background-color: #667b66;\n  border-color: #4cae4c;\n}\n.bk-root .bk-btn-warning {\n  color: #fff;\n  background-color: #f0ad4e;\n  border-color: #eea236;\n}\n.bk-root .bk-btn-warning:hover {\n  background-color: #eea43b;\n  border-color: #e89014;\n}\n.bk-root .bk-btn-warning.bk-active {\n  background-color: #ed9c28;\n  border-color: #d58512;\n}\n.bk-root .bk-btn-warning[disabled],\n.bk-root .bk-btn-warning[disabled]:hover,\n.bk-root .bk-btn-warning[disabled]:focus,\n.bk-root .bk-btn-warning[disabled]:active,\n.bk-root .bk-btn-warning[disabled].bk-active {\n  background-color: #c89143;\n  border-color: #eea236;\n}\n.bk-root .bk-btn-danger {\n  color: #fff;\n  background-color: #d9534f;\n  border-color: #d43f3a;\n}\n.bk-root .bk-btn-danger:hover {\n  background-color: #d5433e;\n  border-color: #bd2d29;\n}\n.bk-root .bk-btn-danger.bk-active {\n  background-color: #d2322d;\n  border-color: #ac2925;\n}\n.bk-root .bk-btn-danger[disabled],\n.bk-root .bk-btn-danger[disabled]:hover,\n.bk-root .bk-btn-danger[disabled]:focus,\n.bk-root .bk-btn-danger[disabled]:active,\n.bk-root .bk-btn-danger[disabled].bk-active {\n  background-color: #a55350;\n  border-color: #d43f3a;\n}\n.bk-root .bk-btn-group {\n  height: 100%;\n  display: flex;\n  display: -webkit-flex;\n  flex-wrap: nowrap;\n  -webkit-flex-wrap: nowrap;\n  align-items: center;\n  -webkit-align-items: center;\n  flex-direction: row;\n  -webkit-flex-direction: row;\n}\n.bk-root .bk-btn-group > .bk-btn {\n  flex-grow: 1;\n  -webkit-flex-grow: 1;\n}\n.bk-root .bk-btn-group > .bk-btn + .bk-btn {\n  margin-left: -1px;\n}\n.bk-root .bk-btn-group > .bk-btn:first-child:not(:last-child) {\n  border-bottom-right-radius: 0;\n  border-top-right-radius: 0;\n}\n.bk-root .bk-btn-group > .bk-btn:not(:first-child):last-child {\n  border-bottom-left-radius: 0;\n  border-top-left-radius: 0;\n}\n.bk-root .bk-btn-group > .bk-btn:not(:first-child):not(:last-child) {\n  border-radius: 0;\n}\n.bk-root .bk-btn-group .bk-dropdown-toggle {\n  flex: 0 0 0;\n  -webkit-flex: 0 0 0;\n  padding: 6px 6px;\n}\n";
    exports.default = css;
},
/* styles/menus.css.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var css = "\n.bk-root .bk-menu-icon {\n  width: 28px;\n  height: 28px;\n  background-size: 60%;\n  background-color: transparent;\n  background-repeat: no-repeat;\n  background-position: center center;\n}\n.bk-root .bk-context-menu {\n  position: absolute;\n  display: inline-flex;\n  display: -webkit-inline-flex;\n  flex-wrap: nowrap;\n  -webkit-flex-wrap: nowrap;\n  user-select: none;\n  -ms-user-select: none;\n  -moz-user-select: none;\n  -webkit-user-select: none;\n  width: auto;\n  height: auto;\n  z-index: 100;\n  cursor: pointer;\n  font-size: 12px;\n  background-color: #fff;\n  border: 1px solid #ccc;\n  border-radius: 4px;\n  box-shadow: 0 6px 12px rgba(0, 0, 0, 0.175);\n}\n.bk-root .bk-context-menu.bk-horizontal {\n  flex-direction: row;\n  -webkit-flex-direction: row;\n}\n.bk-root .bk-context-menu.bk-vertical {\n  flex-direction: column;\n  -webkit-flex-direction: column;\n}\n.bk-root .bk-context-menu > .bk-divider {\n  cursor: default;\n  overflow: hidden;\n  background-color: #e5e5e5;\n}\n.bk-root .bk-context-menu.bk-horizontal > .bk-divider {\n  width: 1px;\n  margin: 5px 0;\n}\n.bk-root .bk-context-menu.bk-vertical > .bk-divider {\n  height: 1px;\n  margin: 0 5px;\n}\n.bk-root .bk-context-menu > :not(.bk-divider) {\n  border: 1px solid transparent;\n}\n.bk-root .bk-context-menu > :not(.bk-divider).bk-active {\n  border-color: #26aae1;\n}\n.bk-root .bk-context-menu > :not(.bk-divider):hover {\n  background-color: #f9f9f9;\n}\n.bk-root .bk-context-menu.bk-horizontal > :not(.bk-divider):first-child {\n  border-top-left-radius: 4px;\n  border-bottom-left-radius: 4px;\n}\n.bk-root .bk-context-menu.bk-horizontal > :not(.bk-divider):last-child {\n  border-top-right-radius: 4px;\n  border-bottom-right-radius: 4px;\n}\n.bk-root .bk-context-menu.bk-vertical > :not(.bk-divider):first-child {\n  border-top-left-radius: 4px;\n  border-top-right-radius: 4px;\n}\n.bk-root .bk-context-menu.bk-vertical > :not(.bk-divider):last-child {\n  border-bottom-left-radius: 4px;\n  border-bottom-right-radius: 4px;\n}\n.bk-root .bk-menu {\n  position: absolute;\n  left: 0;\n  width: 100%;\n  z-index: 100;\n  cursor: pointer;\n  font-size: 12px;\n  background-color: #fff;\n  border: 1px solid #ccc;\n  border-radius: 4px;\n  box-shadow: 0 6px 12px rgba(0, 0, 0, 0.175);\n}\n.bk-root .bk-menu.bk-above {\n  bottom: 100%;\n}\n.bk-root .bk-menu.bk-below {\n  top: 100%;\n}\n.bk-root .bk-menu > .bk-divider {\n  height: 1px;\n  margin: 7.5px 0;\n  overflow: hidden;\n  background-color: #e5e5e5;\n}\n.bk-root .bk-menu > :not(.bk-divider) {\n  padding: 6px 12px;\n}\n.bk-root .bk-menu > :not(.bk-divider):hover,\n.bk-root .bk-menu > :not(.bk-divider).bk-active {\n  background-color: #e6e6e6;\n}\n.bk-root .bk-caret {\n  display: inline-block;\n  vertical-align: middle;\n  width: 0;\n  height: 0;\n  margin: 0 5px;\n}\n.bk-root .bk-caret.bk-down {\n  border-top: 4px solid;\n}\n.bk-root .bk-caret.bk-up {\n  border-bottom: 4px solid;\n}\n.bk-root .bk-caret.bk-down,\n.bk-root .bk-caret.bk-up {\n  border-right: 4px solid transparent;\n  border-left: 4px solid transparent;\n}\n.bk-root .bk-caret.bk-left {\n  border-right: 4px solid;\n}\n.bk-root .bk-caret.bk-right {\n  border-left: 4px solid;\n}\n.bk-root .bk-caret.bk-left,\n.bk-root .bk-caret.bk-right {\n  border-top: 4px solid transparent;\n  border-bottom: 4px solid transparent;\n}\n";
    exports.default = css;
},
/* styles/tabs.css.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var css = "\n.bk-root .bk-tabs-header {\n  display: flex;\n  display: -webkit-flex;\n  flex-wrap: nowrap;\n  -webkit-flex-wrap: nowrap;\n  align-items: center;\n  -webkit-align-items: center;\n  overflow: hidden;\n  user-select: none;\n  -ms-user-select: none;\n  -moz-user-select: none;\n  -webkit-user-select: none;\n}\n.bk-root .bk-tabs-header .bk-btn-group {\n  height: auto;\n  margin-right: 5px;\n}\n.bk-root .bk-tabs-header .bk-btn-group > .bk-btn {\n  flex-grow: 0;\n  -webkit-flex-grow: 0;\n  height: auto;\n  padding: 4px 4px;\n}\n.bk-root .bk-tabs-header .bk-headers-wrapper {\n  flex-grow: 1;\n  -webkit-flex-grow: 1;\n  overflow: hidden;\n  color: #666666;\n}\n.bk-root .bk-tabs-header.bk-above .bk-headers-wrapper {\n  border-bottom: 1px solid #e6e6e6;\n}\n.bk-root .bk-tabs-header.bk-right .bk-headers-wrapper {\n  border-left: 1px solid #e6e6e6;\n}\n.bk-root .bk-tabs-header.bk-below .bk-headers-wrapper {\n  border-top: 1px solid #e6e6e6;\n}\n.bk-root .bk-tabs-header.bk-left .bk-headers-wrapper {\n  border-right: 1px solid #e6e6e6;\n}\n.bk-root .bk-tabs-header.bk-above,\n.bk-root .bk-tabs-header.bk-below {\n  flex-direction: row;\n  -webkit-flex-direction: row;\n}\n.bk-root .bk-tabs-header.bk-above .bk-headers,\n.bk-root .bk-tabs-header.bk-below .bk-headers {\n  flex-direction: row;\n  -webkit-flex-direction: row;\n}\n.bk-root .bk-tabs-header.bk-left,\n.bk-root .bk-tabs-header.bk-right {\n  flex-direction: column;\n  -webkit-flex-direction: column;\n}\n.bk-root .bk-tabs-header.bk-left .bk-headers,\n.bk-root .bk-tabs-header.bk-right .bk-headers {\n  flex-direction: column;\n  -webkit-flex-direction: column;\n}\n.bk-root .bk-tabs-header .bk-headers {\n  position: relative;\n  display: flex;\n  display: -webkit-flex;\n  flex-wrap: nowrap;\n  -webkit-flex-wrap: nowrap;\n  align-items: center;\n  -webkit-align-items: center;\n}\n.bk-root .bk-tabs-header .bk-tab {\n  padding: 4px 8px;\n  border: solid transparent;\n  white-space: nowrap;\n  cursor: pointer;\n}\n.bk-root .bk-tabs-header .bk-tab:hover {\n  background-color: #f2f2f2;\n}\n.bk-root .bk-tabs-header .bk-tab.bk-active {\n  color: #4d4d4d;\n  background-color: white;\n  border-color: #e6e6e6;\n}\n.bk-root .bk-tabs-header .bk-tab .bk-close {\n  margin-left: 10px;\n}\n.bk-root .bk-tabs-header.bk-above .bk-tab {\n  border-width: 3px 1px 0px 1px;\n  border-radius: 4px 4px 0 0;\n}\n.bk-root .bk-tabs-header.bk-right .bk-tab {\n  border-width: 1px 3px 1px 0px;\n  border-radius: 0 4px 4px 0;\n}\n.bk-root .bk-tabs-header.bk-below .bk-tab {\n  border-width: 0px 1px 3px 1px;\n  border-radius: 0 0 4px 4px;\n}\n.bk-root .bk-tabs-header.bk-left .bk-tab {\n  border-width: 1px 0px 1px 3px;\n  border-radius: 4px 0 0 4px;\n}\n.bk-root .bk-close {\n  display: inline-block;\n  width: 10px;\n  height: 10px;\n  vertical-align: middle;\n  background-image: url('data:image/svg+xml;utf8,      <svg viewPort=\"0 0 10 10\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\">        <line x1=\"1\" y1=\"9\" x2=\"9\" y2=\"1\" stroke=\"gray\" stroke-width=\"2\"/>        <line x1=\"1\" y1=\"1\" x2=\"9\" y2=\"9\" stroke=\"gray\" stroke-width=\"2\"/>      </svg>');\n}\n.bk-root .bk-close:hover {\n  background-image: url('data:image/svg+xml;utf8,      <svg viewPort=\"0 0 10 10\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\">        <line x1=\"1\" y1=\"9\" x2=\"9\" y2=\"1\" stroke=\"red\" stroke-width=\"2\"/>        <line x1=\"1\" y1=\"1\" x2=\"9\" y2=\"9\" stroke=\"red\" stroke-width=\"2\"/>      </svg>');\n}\n";
    exports.default = css;
},
/* models/layouts/widget_box.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var column_1 = require(389) /* ./column */;
    var WidgetBoxView = /** @class */ (function (_super) {
        tslib_1.__extends(WidgetBoxView, _super);
        function WidgetBoxView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return WidgetBoxView;
    }(column_1.ColumnView));
    exports.WidgetBoxView = WidgetBoxView;
    WidgetBoxView.__name__ = "WidgetBoxView";
    var WidgetBox = /** @class */ (function (_super) {
        tslib_1.__extends(WidgetBox, _super);
        function WidgetBox(attrs) {
            return _super.call(this, attrs) || this;
        }
        WidgetBox.init_WidgetBox = function () {
            this.prototype.default_view = WidgetBoxView;
        };
        return WidgetBox;
    }(column_1.Column));
    exports.WidgetBox = WidgetBox;
    WidgetBox.__name__ = "WidgetBox";
    WidgetBox.init_WidgetBox();
},
/* models/markers/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    tslib_1.__exportStar(require(403) /* ./defs */, exports);
    var marker_1 = require(404) /* ./marker */;
    exports.Marker = marker_1.Marker;
    var scatter_1 = require(405) /* ./scatter */;
    exports.Scatter = scatter_1.Scatter;
},
/* models/markers/defs.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var marker_1 = require(404) /* ./marker */;
    var glmarks = tslib_1.__importStar(require(353) /* ../glyphs/webgl/markers */);
    var SQ3 = Math.sqrt(3);
    function _one_line(ctx, r) {
        ctx.moveTo(-r, 0);
        ctx.lineTo(r, 0);
    }
    function _one_x(ctx, r) {
        ctx.rotate(Math.PI / 4);
        _one_cross(ctx, r);
        ctx.rotate(-Math.PI / 4);
    }
    function _one_y(ctx, r) {
        var h = r * SQ3;
        var a = h / 3;
        ctx.moveTo(-h / 2, -a);
        ctx.lineTo(0, 0);
        ctx.lineTo(h / 2, -a);
        ctx.lineTo(0, 0);
        ctx.lineTo(0, r);
    }
    function _one_cross(ctx, r) {
        ctx.moveTo(0, r);
        ctx.lineTo(0, -r);
        ctx.moveTo(-r, 0);
        ctx.lineTo(r, 0);
    }
    function _one_dot(ctx, r) {
        ctx.beginPath();
        ctx.arc(0, 0, r / 4, 0, 2 * Math.PI, false);
        ctx.closePath();
    }
    function _one_diamond(ctx, r) {
        ctx.moveTo(0, r);
        ctx.lineTo(r / 1.5, 0);
        ctx.lineTo(0, -r);
        ctx.lineTo(-r / 1.5, 0);
        ctx.closePath();
    }
    function _one_hex(ctx, r) {
        var r2 = r / 2;
        var h = SQ3 * r2;
        ctx.moveTo(r, 0);
        ctx.lineTo(r2, -h);
        ctx.lineTo(-r2, -h);
        ctx.lineTo(-r, 0);
        ctx.lineTo(-r2, h);
        ctx.lineTo(r2, h);
        ctx.closePath();
    }
    function _one_tri(ctx, r) {
        var h = r * SQ3;
        var a = h / 3;
        ctx.moveTo(-r, a);
        ctx.lineTo(r, a);
        ctx.lineTo(0, a - h);
        ctx.closePath();
    }
    function asterisk(ctx, i, r, line, _fill) {
        _one_cross(ctx, r);
        _one_x(ctx, r);
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function circle(ctx, i, r, line, fill) {
        ctx.arc(0, 0, r, 0, 2 * Math.PI, false);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function circle_cross(ctx, i, r, line, fill) {
        ctx.arc(0, 0, r, 0, 2 * Math.PI, false);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            _one_cross(ctx, r);
            ctx.stroke();
        }
    }
    function circle_dot(ctx, i, r, line, fill) {
        circle(ctx, i, r, line, fill);
        dot(ctx, i, r, line, fill);
    }
    function circle_y(ctx, i, r, line, fill) {
        ctx.arc(0, 0, r, 0, 2 * Math.PI, false);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            _one_y(ctx, r);
            ctx.stroke();
        }
    }
    function circle_x(ctx, i, r, line, fill) {
        ctx.arc(0, 0, r, 0, 2 * Math.PI, false);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            _one_x(ctx, r);
            ctx.stroke();
        }
    }
    function cross(ctx, i, r, line, _fill) {
        _one_cross(ctx, r);
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function diamond(ctx, i, r, line, fill) {
        _one_diamond(ctx, r);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function diamond_cross(ctx, i, r, line, fill) {
        _one_diamond(ctx, r);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.moveTo(0, r);
            ctx.lineTo(0, -r);
            ctx.moveTo(-r / 1.5, 0);
            ctx.lineTo(r / 1.5, 0);
            ctx.stroke();
        }
    }
    function diamond_dot(ctx, i, r, line, fill) {
        diamond(ctx, i, r, line, fill);
        dot(ctx, i, r, line, fill);
    }
    function dot(ctx, i, r, line, _fill) {
        _one_dot(ctx, r);
        line.set_vectorize(ctx, i);
        ctx.fillStyle = ctx.strokeStyle; // NOTE: dots use line color for fill to match
        ctx.fill();
    }
    function hex(ctx, i, r, line, fill) {
        _one_hex(ctx, r);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function hex_dot(ctx, i, r, line, fill) {
        hex(ctx, i, r, line, fill);
        dot(ctx, i, r, line, fill);
    }
    function inverted_triangle(ctx, i, r, line, fill) {
        ctx.rotate(Math.PI);
        _one_tri(ctx, r);
        ctx.rotate(-Math.PI);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function plus(ctx, i, r, line, fill) {
        var a = 3 * r / 8;
        var b = r;
        var xs = [a, a, b, b, a, a, -a, -a, -b, -b, -a, -a];
        var ys = [b, a, a, -a, -a, -b, -b, -a, -a, a, a, b];
        ctx.moveTo(xs[0], ys[0]);
        for (i = 1; i < 12; i++)
            ctx.lineTo(xs[i], ys[i]);
        ctx.closePath();
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function square(ctx, i, r, line, fill) {
        var size = 2 * r;
        ctx.rect(-r, -r, size, size);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function square_pin(ctx, i, r, line, fill) {
        var a = 3 * r / 8;
        ctx.moveTo(-r, -r);
        /* eslint-disable space-in-parens */
        ctx.quadraticCurveTo(0, -a, r, -r);
        ctx.quadraticCurveTo(a, 0, r, r);
        ctx.quadraticCurveTo(0, a, -r, r);
        ctx.quadraticCurveTo(-a, 0, -r, -r);
        /* eslint-ensable space-in-parens */
        ctx.closePath();
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function square_cross(ctx, i, r, line, fill) {
        var size = 2 * r;
        ctx.rect(-r, -r, size, size);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            _one_cross(ctx, r);
            ctx.stroke();
        }
    }
    function square_dot(ctx, i, r, line, fill) {
        square(ctx, i, r, line, fill);
        dot(ctx, i, r, line, fill);
    }
    function square_x(ctx, i, r, line, fill) {
        var size = 2 * r;
        ctx.rect(-r, -r, size, size);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.moveTo(-r, r);
            ctx.lineTo(r, -r);
            ctx.moveTo(-r, -r);
            ctx.lineTo(r, r);
            ctx.stroke();
        }
    }
    function triangle(ctx, i, r, line, fill) {
        _one_tri(ctx, r);
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function triangle_dot(ctx, i, r, line, fill) {
        triangle(ctx, i, r, line, fill);
        dot(ctx, i, r, line, fill);
    }
    function triangle_pin(ctx, i, r, line, fill) {
        var h = r * SQ3;
        var a = h / 3;
        var b = 3 * a / 8;
        ctx.moveTo(-r, a);
        ctx.quadraticCurveTo(0, b, r, a);
        ctx.quadraticCurveTo(SQ3 * b / 2, b / 2, 0, a - h);
        ctx.quadraticCurveTo(-SQ3 * b / 2, b / 2, -r, a);
        ctx.closePath();
        if (fill.doit) {
            fill.set_vectorize(ctx, i);
            ctx.fill();
        }
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function dash(ctx, i, r, line, _fill) {
        _one_line(ctx, r);
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function x(ctx, i, r, line, _fill) {
        _one_x(ctx, r);
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function y(ctx, i, r, line, _fill) {
        _one_y(ctx, r);
        if (line.doit) {
            line.set_vectorize(ctx, i);
            ctx.stroke();
        }
    }
    function _mk_model(type, f, glglyph_cls) {
        var _a;
        var view = /** @class */ (function (_super) {
            tslib_1.__extends(view, _super);
            function view() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            view.initClass = function () {
                this.prototype._render_one = f;
                this.prototype.glglyph_cls = glglyph_cls;
            };
            return view;
        }(marker_1.MarkerView));
        view.initClass();
        var model = (_a = /** @class */ (function (_super) {
            tslib_1.__extends(_a, _super);
            function _a() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            _a.initClass = function () {
                this.prototype.default_view = view;
            };
            return _a;
        }(marker_1.Marker)),
            _a.__name__ = type,
            _a);
        model.initClass();
        return model;
    }
    // markers are final, so no need to export views
    exports.Asterisk = _mk_model('Asterisk', asterisk, glmarks.AsteriskGL);
    exports.CircleCross = _mk_model('CircleCross', circle_cross, glmarks.CircleCrossGL);
    exports.CircleDot = _mk_model('CircleDot', circle_dot);
    exports.CircleY = _mk_model('CircleY', circle_y);
    exports.CircleX = _mk_model('CircleX', circle_x, glmarks.CircleXGL);
    exports.Cross = _mk_model('Cross', cross, glmarks.CrossGL);
    exports.Dash = _mk_model('Dash', dash);
    exports.Diamond = _mk_model('Diamond', diamond, glmarks.DiamondGL);
    exports.DiamondCross = _mk_model('DiamondCross', diamond_cross, glmarks.DiamondCrossGL);
    exports.DiamondDot = _mk_model('DiamondDot', diamond_dot);
    exports.Dot = _mk_model('Dot', dot);
    exports.Hex = _mk_model('Hex', hex, glmarks.HexGL);
    exports.HexDot = _mk_model('HexDot', hex_dot);
    exports.InvertedTriangle = _mk_model('InvertedTriangle', inverted_triangle, glmarks.InvertedTriangleGL);
    exports.Plus = _mk_model('Plus', plus);
    exports.Square = _mk_model('Square', square, glmarks.SquareGL);
    exports.SquareCross = _mk_model('SquareCross', square_cross, glmarks.SquareCrossGL);
    exports.SquareDot = _mk_model('SquareDot', square_dot);
    exports.SquarePin = _mk_model('SquarePin', square_pin);
    exports.SquareX = _mk_model('SquareX', square_x, glmarks.SquareXGL);
    exports.Triangle = _mk_model('Triangle', triangle, glmarks.TriangleGL);
    exports.TriangleDot = _mk_model('TriangleDot', triangle_dot);
    exports.TrianglePin = _mk_model('TrianglePin', triangle_pin);
    exports.X = _mk_model('X', x, glmarks.XGL);
    exports.Y = _mk_model('Y', y);
    exports.marker_funcs = {
        asterisk: asterisk,
        circle: circle,
        circle_cross: circle_cross,
        circle_dot: circle_dot,
        circle_y: circle_y,
        circle_x: circle_x,
        cross: cross,
        diamond: diamond,
        diamond_dot: diamond_dot,
        diamond_cross: diamond_cross,
        dot: dot,
        hex: hex,
        hex_dot: hex_dot,
        inverted_triangle: inverted_triangle,
        plus: plus,
        square: square,
        square_cross: square_cross,
        square_dot: square_dot,
        square_pin: square_pin,
        square_x: square_x,
        triangle: triangle,
        triangle_dot: triangle_dot,
        triangle_pin: triangle_pin,
        dash: dash,
        x: x,
        y: y,
    };
},
/* models/markers/marker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var xy_glyph_1 = require(208) /* ../glyphs/xy_glyph */;
    var property_mixins_1 = require(143) /* ../../core/property_mixins */;
    var hittest = tslib_1.__importStar(require(216) /* ../../core/hittest */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    var selection_1 = require(203) /* ../selections/selection */;
    var MarkerView = /** @class */ (function (_super) {
        tslib_1.__extends(MarkerView, _super);
        function MarkerView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        MarkerView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            var webgl = this.renderer.plot_view.canvas_view.webgl;
            if (webgl != null && this.glglyph_cls != null) {
                this.glglyph = new this.glglyph_cls(webgl.gl, this);
            }
        };
        MarkerView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx = _a.sx, sy = _a.sy, _size = _a._size, _angle = _a._angle;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (isNaN(sx[i] + sy[i] + _size[i] + _angle[i]))
                        continue;
                    var r = _size[i] / 2;
                    ctx.beginPath();
                    ctx.translate(sx[i], sy[i]);
                    if (_angle[i])
                        ctx.rotate(_angle[i]);
                    this._render_one(ctx, i, r, this.visuals.line, this.visuals.fill);
                    if (_angle[i])
                        ctx.rotate(-_angle[i]);
                    ctx.translate(-sx[i], -sy[i]);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        MarkerView.prototype._mask_data = function () {
            // dilate the inner screen region by max_size and map back to data space for use in spatial query
            var hr = this.renderer.plot_view.frame.bbox.h_range;
            var sx0 = hr.start - this.max_size;
            var sx1 = hr.end + this.max_size;
            var _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
            var vr = this.renderer.plot_view.frame.bbox.v_range;
            var sy0 = vr.start - this.max_size;
            var sy1 = vr.end + this.max_size;
            var _b = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _b[0], y1 = _b[1];
            return this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 });
        };
        MarkerView.prototype._hit_point = function (geometry) {
            var e_2, _a;
            var sx = geometry.sx, sy = geometry.sy;
            var sx0 = sx - this.max_size;
            var sx1 = sx + this.max_size;
            var _b = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _b[0], x1 = _b[1];
            var sy0 = sy - this.max_size;
            var sy1 = sy + this.max_size;
            var _c = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _c[0], y1 = _c[1];
            var candidates = this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 });
            var indices = [];
            try {
                for (var candidates_1 = tslib_1.__values(candidates), candidates_1_1 = candidates_1.next(); !candidates_1_1.done; candidates_1_1 = candidates_1.next()) {
                    var i = candidates_1_1.value;
                    var s2 = this._size[i] / 2;
                    if (Math.abs(this.sx[i] - sx) <= s2 && Math.abs(this.sy[i] - sy) <= s2) {
                        indices.push(i);
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (candidates_1_1 && !candidates_1_1.done && (_a = candidates_1.return))
                        _a.call(candidates_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        MarkerView.prototype._hit_span = function (geometry) {
            var _a, _b;
            var sx = geometry.sx, sy = geometry.sy;
            var bounds = this.bounds();
            var ms = this.max_size / 2;
            var x0, x1, y0, y1;
            if (geometry.direction == 'h') {
                y0 = bounds.y0;
                y1 = bounds.y1;
                var sx0 = sx - ms;
                var sx1 = sx + ms;
                _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
            }
            else {
                x0 = bounds.x0;
                x1 = bounds.x1;
                var sy0 = sy - ms;
                var sy1 = sy + ms;
                _b = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _b[0], y1 = _b[1];
            }
            var indices = tslib_1.__spread(this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 }));
            return new selection_1.Selection({ indices: indices });
        };
        MarkerView.prototype._hit_rect = function (geometry) {
            var sx0 = geometry.sx0, sx1 = geometry.sx1, sy0 = geometry.sy0, sy1 = geometry.sy1;
            var _a = tslib_1.__read(this.renderer.xscale.r_invert(sx0, sx1), 2), x0 = _a[0], x1 = _a[1];
            var _b = tslib_1.__read(this.renderer.yscale.r_invert(sy0, sy1), 2), y0 = _b[0], y1 = _b[1];
            var indices = tslib_1.__spread(this.index.indices({ x0: x0, x1: x1, y0: y0, y1: y1 }));
            return new selection_1.Selection({ indices: indices });
        };
        MarkerView.prototype._hit_poly = function (geometry) {
            var sx = geometry.sx, sy = geometry.sy;
            // TODO (bev) use spatial index to pare candidate list
            var candidates = array_1.range(0, this.sx.length);
            var indices = [];
            for (var i = 0, end = candidates.length; i < end; i++) {
                var index = candidates[i];
                if (hittest.point_in_poly(this.sx[i], this.sy[i], sx, sy)) {
                    indices.push(index);
                }
            }
            return new selection_1.Selection({ indices: indices });
        };
        MarkerView.prototype._get_legend_args = function (_a, index) {
            var x0 = _a.x0, x1 = _a.x1, y0 = _a.y0, y1 = _a.y1;
            // using objects like this seems a little wonky, since the keys are coerced to strings, but it works
            var len = index + 1;
            var sx = new Array(len);
            sx[index] = (x0 + x1) / 2;
            var sy = new Array(len);
            sy[index] = (y0 + y1) / 2;
            var size = new Array(len);
            size[index] = Math.min(Math.abs(x1 - x0), Math.abs(y1 - y0)) * 0.4;
            var angle = new Array(len);
            angle[index] = 0; // don't attempt to match glyph angle
            return { sx: sx, sy: sy, _size: size, _angle: angle };
        };
        MarkerView.prototype.draw_legend_for_index = function (ctx, _a, index) {
            var x0 = _a.x0, x1 = _a.x1, y0 = _a.y0, y1 = _a.y1;
            var args = this._get_legend_args({ x0: x0, x1: x1, y0: y0, y1: y1 }, index);
            this._render(ctx, [index], args); // XXX
        };
        return MarkerView;
    }(xy_glyph_1.XYGlyphView));
    exports.MarkerView = MarkerView;
    MarkerView.__name__ = "MarkerView";
    var Marker = /** @class */ (function (_super) {
        tslib_1.__extends(Marker, _super);
        function Marker(attrs) {
            return _super.call(this, attrs) || this;
        }
        Marker.init_Marker = function () {
            this.mixins([property_mixins_1.LineVector, property_mixins_1.FillVector]);
            this.define({
                size: [p.DistanceSpec, { units: "screen", value: 4 }],
                angle: [p.AngleSpec, 0],
            });
        };
        return Marker;
    }(xy_glyph_1.XYGlyph));
    exports.Marker = Marker;
    Marker.__name__ = "Marker";
    Marker.init_Marker();
},
/* models/markers/scatter.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var marker_1 = require(404) /* ./marker */;
    var defs_1 = require(403) /* ./defs */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var ScatterView = /** @class */ (function (_super) {
        tslib_1.__extends(ScatterView, _super);
        function ScatterView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ScatterView.prototype._render = function (ctx, indices, _a) {
            var e_1, _b;
            var sx = _a.sx, sy = _a.sy, _size = _a._size, _angle = _a._angle, _marker = _a._marker;
            try {
                for (var indices_1 = tslib_1.__values(indices), indices_1_1 = indices_1.next(); !indices_1_1.done; indices_1_1 = indices_1.next()) {
                    var i = indices_1_1.value;
                    if (isNaN(sx[i] + sy[i] + _size[i] + _angle[i]) || _marker[i] == null)
                        continue;
                    var r = _size[i] / 2;
                    ctx.beginPath();
                    ctx.translate(sx[i], sy[i]);
                    if (_angle[i])
                        ctx.rotate(_angle[i]);
                    defs_1.marker_funcs[_marker[i]](ctx, i, r, this.visuals.line, this.visuals.fill);
                    if (_angle[i])
                        ctx.rotate(-_angle[i]);
                    ctx.translate(-sx[i], -sy[i]);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (indices_1_1 && !indices_1_1.done && (_b = indices_1.return))
                        _b.call(indices_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        ScatterView.prototype.draw_legend_for_index = function (ctx, _a, index) {
            var x0 = _a.x0, x1 = _a.x1, y0 = _a.y0, y1 = _a.y1;
            var args = this._get_legend_args({ x0: x0, x1: x1, y0: y0, y1: y1 }, index);
            var len = index + 1;
            var marker = new Array(len);
            marker[index] = this._marker[index];
            args._marker = marker;
            this._render(ctx, [index], args); // XXX
        };
        return ScatterView;
    }(marker_1.MarkerView));
    exports.ScatterView = ScatterView;
    ScatterView.__name__ = "ScatterView";
    var Scatter = /** @class */ (function (_super) {
        tslib_1.__extends(Scatter, _super);
        function Scatter(attrs) {
            return _super.call(this, attrs) || this;
        }
        Scatter.init_Scatter = function () {
            this.prototype.default_view = ScatterView;
            this.define({
                marker: [p.MarkerSpec, { value: "circle" }],
            });
        };
        return Scatter;
    }(marker_1.Marker));
    exports.Scatter = Scatter;
    Scatter.__name__ = "Scatter";
    Scatter.init_Scatter();
},
/* models/plots/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var gmap_plot_1 = require(407) /* ./gmap_plot */;
    exports.MapOptions = gmap_plot_1.MapOptions;
    var gmap_plot_2 = require(407) /* ./gmap_plot */;
    exports.GMapOptions = gmap_plot_2.GMapOptions;
    var gmap_plot_3 = require(407) /* ./gmap_plot */;
    exports.GMapPlot = gmap_plot_3.GMapPlot;
    var plot_1 = require(408) /* ./plot */;
    exports.Plot = plot_1.Plot;
},
/* models/plots/gmap_plot.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var logging_1 = require(134) /* ../../core/logging */;
    var plot_1 = require(408) /* ./plot */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var model_1 = require(196) /* ../../model */;
    var range1d_1 = require(273) /* ../ranges/range1d */;
    var gmap_plot_canvas_1 = require(434) /* ./gmap_plot_canvas */;
    exports.GMapPlotView = gmap_plot_canvas_1.GMapPlotView;
    var MapOptions = /** @class */ (function (_super) {
        tslib_1.__extends(MapOptions, _super);
        function MapOptions(attrs) {
            return _super.call(this, attrs) || this;
        }
        MapOptions.init_MapOptions = function () {
            this.define({
                lat: [p.Number],
                lng: [p.Number],
                zoom: [p.Number, 12],
            });
        };
        return MapOptions;
    }(model_1.Model));
    exports.MapOptions = MapOptions;
    MapOptions.__name__ = "MapOptions";
    MapOptions.init_MapOptions();
    var GMapOptions = /** @class */ (function (_super) {
        tslib_1.__extends(GMapOptions, _super);
        function GMapOptions(attrs) {
            return _super.call(this, attrs) || this;
        }
        GMapOptions.init_GMapOptions = function () {
            this.define({
                map_type: [p.String, "roadmap"],
                scale_control: [p.Boolean, false],
                styles: [p.String],
                tilt: [p.Int, 45],
            });
        };
        return GMapOptions;
    }(MapOptions));
    exports.GMapOptions = GMapOptions;
    GMapOptions.__name__ = "GMapOptions";
    GMapOptions.init_GMapOptions();
    var GMapPlot = /** @class */ (function (_super) {
        tslib_1.__extends(GMapPlot, _super);
        function GMapPlot(attrs) {
            return _super.call(this, attrs) || this;
        }
        GMapPlot.init_GMapPlot = function () {
            this.prototype.default_view = gmap_plot_canvas_1.GMapPlotView;
            // This seems to be necessary so that everything can initialize.
            // Feels very clumsy, but I'm not sure how the properties system wants
            // to handle something like this situation.
            this.define({
                map_options: [p.Instance],
                api_key: [p.String],
            });
            this.override({
                x_range: function () { return new range1d_1.Range1d(); },
                y_range: function () { return new range1d_1.Range1d(); },
            });
        };
        GMapPlot.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.use_map = true;
            if (!this.api_key)
                logging_1.logger.error("api_key is required. See https://developers.google.com/maps/documentation/javascript/get-api-key for more information on how to obtain your own.");
        };
        return GMapPlot;
    }(plot_1.Plot));
    exports.GMapPlot = GMapPlot;
    GMapPlot.__name__ = "GMapPlot";
    GMapPlot.init_GMapPlot();
},
/* models/plots/plot.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var mixins = tslib_1.__importStar(require(143) /* ../../core/property_mixins */);
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var signaling_1 = require(130) /* ../../core/signaling */;
    var array_1 = require(124) /* ../../core/util/array */;
    var object_1 = require(128) /* ../../core/util/object */;
    var types_1 = require(123) /* ../../core/util/types */;
    var layout_dom_1 = require(387) /* ../layouts/layout_dom */;
    var title_1 = require(284) /* ../annotations/title */;
    var linear_scale_1 = require(260) /* ../scales/linear_scale */;
    var toolbar_1 = require(409) /* ../tools/toolbar */;
    var column_data_source_1 = require(200) /* ../sources/column_data_source */;
    var glyph_renderer_1 = require(205) /* ../renderers/glyph_renderer */;
    var data_range1d_1 = require(325) /* ../ranges/data_range1d */;
    var plot_canvas_1 = require(427) /* ./plot_canvas */;
    exports.PlotView = plot_canvas_1.PlotView;
    var Plot = /** @class */ (function (_super) {
        tslib_1.__extends(Plot, _super);
        function Plot(attrs) {
            return _super.call(this, attrs) || this;
        }
        Plot.init_Plot = function () {
            this.prototype.default_view = plot_canvas_1.PlotView;
            this.mixins([
                ["outline_", mixins.Line],
                ["background_", mixins.Fill],
                ["border_", mixins.Fill],
            ]);
            this.define({
                toolbar: [p.Instance, function () { return new toolbar_1.Toolbar(); }],
                toolbar_location: [p.Location, 'right'],
                toolbar_sticky: [p.Boolean, true],
                plot_width: [p.Number, 600],
                plot_height: [p.Number, 600],
                frame_width: [p.Number, null],
                frame_height: [p.Number, null],
                title: [p.Any, function () { return new title_1.Title({ text: "" }); }],
                title_location: [p.Location, 'above'],
                above: [p.Array, []],
                below: [p.Array, []],
                left: [p.Array, []],
                right: [p.Array, []],
                center: [p.Array, []],
                renderers: [p.Array, []],
                x_range: [p.Instance, function () { return new data_range1d_1.DataRange1d(); }],
                extra_x_ranges: [p.Any, {}],
                y_range: [p.Instance, function () { return new data_range1d_1.DataRange1d(); }],
                extra_y_ranges: [p.Any, {}],
                x_scale: [p.Instance, function () { return new linear_scale_1.LinearScale(); }],
                y_scale: [p.Instance, function () { return new linear_scale_1.LinearScale(); }],
                lod_factor: [p.Number, 10],
                lod_interval: [p.Number, 300],
                lod_threshold: [p.Number, 2000],
                lod_timeout: [p.Number, 500],
                hidpi: [p.Boolean, true],
                output_backend: [p.OutputBackend, "canvas"],
                min_border: [p.Number, 5],
                min_border_top: [p.Number, null],
                min_border_left: [p.Number, null],
                min_border_bottom: [p.Number, null],
                min_border_right: [p.Number, null],
                inner_width: [p.Number],
                inner_height: [p.Number],
                outer_width: [p.Number],
                outer_height: [p.Number],
                match_aspect: [p.Boolean, false],
                aspect_scale: [p.Number, 1],
                reset_policy: [p.ResetPolicy, "standard"],
            });
            this.override({
                outline_line_color: "#e5e5e5",
                border_fill_color: "#ffffff",
                background_fill_color: "#ffffff",
            });
        };
        Object.defineProperty(Plot.prototype, "width", {
            // TODO: change this when we drop ES5 compatibility (https://github.com/microsoft/TypeScript/issues/338)
            get: function () {
                // const width = super.width
                var width = this.properties.width.get_value();
                return width != null ? width : this.plot_width;
            },
            set: function (width) {
                this.setv({ width: width, plot_width: width });
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Plot.prototype, "height", {
            get: function () {
                // const height = super.height
                var height = this.properties.height.get_value();
                return height != null ? height : this.plot_height;
            },
            set: function (height) {
                this.setv({ height: height, plot_height: height });
            },
            enumerable: true,
            configurable: true
        });
        Plot.prototype._doc_attached = function () {
            _super.prototype._doc_attached.call(this);
            this._push_changes([
                [this.properties.inner_height, null, this.inner_height],
                [this.properties.inner_width, null, this.inner_width],
            ]);
        };
        Plot.prototype.initialize = function () {
            var e_1, _a, e_2, _b;
            _super.prototype.initialize.call(this);
            this.reset = new signaling_1.Signal0(this, "reset");
            try {
                for (var _c = tslib_1.__values(object_1.values(this.extra_x_ranges).concat(this.x_range)), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var xr = _d.value;
                    var plots = xr.plots;
                    if (types_1.isArray(plots)) {
                        plots = plots.concat(this);
                        xr.setv({ plots: plots }, { silent: true });
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return))
                        _a.call(_c);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            try {
                for (var _e = tslib_1.__values(object_1.values(this.extra_y_ranges).concat(this.y_range)), _f = _e.next(); !_f.done; _f = _e.next()) {
                    var yr = _f.value;
                    var plots = yr.plots;
                    if (types_1.isArray(plots)) {
                        plots = plots.concat(this);
                        yr.setv({ plots: plots }, { silent: true });
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_f && !_f.done && (_b = _e.return))
                        _b.call(_e);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
        };
        Plot.prototype.add_layout = function (renderer, side) {
            var _a;
            if (side === void 0) {
                side = "center";
            }
            var renderers = this.properties[side].get_value();
            this.setv((_a = {}, _a[side] = tslib_1.__spread(renderers, [renderer]), _a));
        };
        Plot.prototype.remove_layout = function (renderer) {
            var del = function (items) {
                array_1.remove_by(items, function (item) { return item == renderer; });
            };
            del(this.left);
            del(this.right);
            del(this.above);
            del(this.below);
            del(this.center);
        };
        Plot.prototype.add_renderers = function () {
            var renderers = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                renderers[_i] = arguments[_i];
            }
            this.renderers = this.renderers.concat(renderers);
        };
        Plot.prototype.add_glyph = function (glyph, source, extra_attrs) {
            if (source === void 0) {
                source = new column_data_source_1.ColumnDataSource();
            }
            if (extra_attrs === void 0) {
                extra_attrs = {};
            }
            var attrs = Object.assign(Object.assign({}, extra_attrs), { data_source: source, glyph: glyph });
            var renderer = new glyph_renderer_1.GlyphRenderer(attrs);
            this.add_renderers(renderer);
            return renderer;
        };
        Plot.prototype.add_tools = function () {
            var tools = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                tools[_i] = arguments[_i];
            }
            this.toolbar.tools = this.toolbar.tools.concat(tools);
        };
        Object.defineProperty(Plot.prototype, "panels", {
            get: function () {
                return tslib_1.__spread(this.side_panels, this.center);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Plot.prototype, "side_panels", {
            get: function () {
                var _a = this, above = _a.above, below = _a.below, left = _a.left, right = _a.right;
                return array_1.concat([above, below, left, right]);
            },
            enumerable: true,
            configurable: true
        });
        return Plot;
    }(layout_dom_1.LayoutDOM));
    exports.Plot = Plot;
    Plot.__name__ = "Plot";
    Plot.init_Plot();
},
/* models/tools/toolbar.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var types_1 = require(123) /* ../../core/util/types */;
    var array_1 = require(124) /* ../../core/util/array */;
    var object_1 = require(128) /* ../../core/util/object */;
    var inspect_tool_1 = require(410) /* ./inspectors/inspect_tool */;
    var toolbar_base_1 = require(420) /* ./toolbar_base */;
    var _get_active_attr = function (et) {
        switch (et) {
            case 'tap': return 'active_tap';
            case 'pan': return 'active_drag';
            case 'pinch':
            case 'scroll': return 'active_scroll';
            case 'multi': return 'active_multi';
        }
        return null;
    };
    var _supports_auto = function (et) {
        return et == 'tap' || et == 'pan';
    };
    var Toolbar = /** @class */ (function (_super) {
        tslib_1.__extends(Toolbar, _super);
        function Toolbar(attrs) {
            return _super.call(this, attrs) || this;
        }
        Toolbar.init_Toolbar = function () {
            this.prototype.default_view = toolbar_base_1.ToolbarBaseView;
            this.define({
                active_drag: [p.Any, 'auto'],
                active_inspect: [p.Any, 'auto'],
                active_scroll: [p.Any, 'auto'],
                active_tap: [p.Any, 'auto'],
                active_multi: [p.Any, null],
            });
        };
        Toolbar.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            var _a = this.properties, tools = _a.tools, active_drag = _a.active_drag, active_inspect = _a.active_inspect, active_scroll = _a.active_scroll, active_tap = _a.active_tap, active_multi = _a.active_multi;
            this.on_change([tools, active_drag, active_inspect, active_scroll, active_tap, active_multi], function () { return _this._init_tools(); });
        };
        Toolbar.prototype._init_tools = function () {
            var e_1, _a, e_2, _b, e_3, _c, e_4, _d, e_5, _e, e_6, _f;
            var _this = this;
            _super.prototype._init_tools.call(this);
            if (this.active_inspect == 'auto') {
                // do nothing as all tools are active be default
            }
            else if (this.active_inspect instanceof inspect_tool_1.InspectTool) {
                var found = false;
                try {
                    for (var _g = tslib_1.__values(this.inspectors), _h = _g.next(); !_h.done; _h = _g.next()) {
                        var inspector = _h.value;
                        if (inspector != this.active_inspect)
                            inspector.active = false;
                        else
                            found = true;
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (_h && !_h.done && (_a = _g.return))
                            _a.call(_g);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
                if (!found) {
                    this.active_inspect = null;
                }
            }
            else if (types_1.isArray(this.active_inspect)) {
                var active_inspect = array_1.intersection(this.active_inspect, this.inspectors);
                if (active_inspect.length != this.active_inspect.length) {
                    this.active_inspect = active_inspect;
                }
                try {
                    for (var _j = tslib_1.__values(this.inspectors), _k = _j.next(); !_k.done; _k = _j.next()) {
                        var inspector = _k.value;
                        if (!array_1.includes(this.active_inspect, inspector))
                            inspector.active = false;
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (_k && !_k.done && (_b = _j.return))
                            _b.call(_j);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
            }
            else if (this.active_inspect == null) {
                try {
                    for (var _l = tslib_1.__values(this.inspectors), _m = _l.next(); !_m.done; _m = _l.next()) {
                        var inspector = _m.value;
                        inspector.active = false;
                    }
                }
                catch (e_3_1) {
                    e_3 = { error: e_3_1 };
                }
                finally {
                    try {
                        if (_m && !_m.done && (_c = _l.return))
                            _c.call(_l);
                    }
                    finally {
                        if (e_3)
                            throw e_3.error;
                    }
                }
            }
            var _activate_gesture = function (tool) {
                if (tool.active) {
                    // tool was activated by a proxy, but we need to finish configuration manually
                    _this._active_change(tool);
                }
                else
                    tool.active = true;
            };
            try {
                // Connecting signals has to be done before changing the active state of the tools.
                for (var _o = tslib_1.__values(object_1.values(this.gestures)), _p = _o.next(); !_p.done; _p = _o.next()) {
                    var gesture = _p.value;
                    gesture.tools = array_1.sort_by(gesture.tools, function (tool) { return tool.default_order; });
                    var _loop_1 = function (tool) {
                        this_1.connect(tool.properties.active.change, function () { return _this._active_change(tool); });
                    };
                    var this_1 = this;
                    try {
                        for (var _q = (e_5 = void 0, tslib_1.__values(gesture.tools)), _r = _q.next(); !_r.done; _r = _q.next()) {
                            var tool = _r.value;
                            _loop_1(tool);
                        }
                    }
                    catch (e_5_1) {
                        e_5 = { error: e_5_1 };
                    }
                    finally {
                        try {
                            if (_r && !_r.done && (_e = _q.return))
                                _e.call(_q);
                        }
                        finally {
                            if (e_5)
                                throw e_5.error;
                        }
                    }
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (_p && !_p.done && (_d = _o.return))
                        _d.call(_o);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            try {
                for (var _s = tslib_1.__values(object_1.entries(this.gestures)), _t = _s.next(); !_t.done; _t = _s.next()) {
                    var _u = tslib_1.__read(_t.value, 2), et = _u[0], gesture = _u[1];
                    var active_attr = _get_active_attr(et);
                    if (active_attr) {
                        var active_tool = this[active_attr];
                        if (active_tool == 'auto') {
                            if (gesture.tools.length != 0 && _supports_auto(et)) {
                                _activate_gesture(gesture.tools[0]);
                            }
                        }
                        else if (active_tool != null) {
                            if (array_1.includes(this.tools, active_tool)) {
                                _activate_gesture(active_tool);
                            }
                            else {
                                this[active_attr] = null;
                            }
                        }
                    }
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (_t && !_t.done && (_f = _s.return))
                        _f.call(_s);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
        };
        return Toolbar;
    }(toolbar_base_1.ToolbarBase));
    exports.Toolbar = Toolbar;
    Toolbar.__name__ = "Toolbar";
    Toolbar.init_Toolbar();
},
/* models/tools/inspectors/inspect_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var button_tool_1 = require(411) /* ../button_tool */;
    var on_off_button_1 = require(419) /* ../on_off_button */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var InspectToolView = /** @class */ (function (_super) {
        tslib_1.__extends(InspectToolView, _super);
        function InspectToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return InspectToolView;
    }(button_tool_1.ButtonToolView));
    exports.InspectToolView = InspectToolView;
    InspectToolView.__name__ = "InspectToolView";
    var InspectTool = /** @class */ (function (_super) {
        tslib_1.__extends(InspectTool, _super);
        function InspectTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.event_type = "move";
            return _this;
        }
        InspectTool.init_InspectTool = function () {
            this.prototype.button_view = on_off_button_1.OnOffButtonView;
            this.define({
                toggleable: [p.Boolean, true],
            });
            this.override({
                active: true,
            });
        };
        return InspectTool;
    }(button_tool_1.ButtonTool));
    exports.InspectTool = InspectTool;
    InspectTool.__name__ = "InspectTool";
    InspectTool.init_InspectTool();
},
/* models/tools/button_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var hammerjs_1 = tslib_1.__importDefault(require(412) /* hammerjs */);
    var dom_view_1 = require(193) /* ../../core/dom_view */;
    var tool_1 = require(413) /* ./tool */;
    var dom_1 = require(187) /* ../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var string_1 = require(144) /* ../../core/util/string */;
    var types_1 = require(123) /* ../../core/util/types */;
    var array_1 = require(124) /* ../../core/util/array */;
    var toolbar_1 = require(414) /* ../../styles/toolbar */;
    var toolbar_css_1 = tslib_1.__importDefault(require(415) /* ../../styles/toolbar.css */);
    var icons_css_1 = tslib_1.__importDefault(require(416) /* ../../styles/icons.css */);
    var menus_css_1 = tslib_1.__importDefault(require(399) /* ../../styles/menus.css */);
    var menus_1 = require(417) /* ../../core/util/menus */;
    var ButtonToolButtonView = /** @class */ (function (_super) {
        tslib_1.__extends(ButtonToolButtonView, _super);
        function ButtonToolButtonView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ButtonToolButtonView.prototype.initialize = function () {
            var _this = this;
            _super.prototype.initialize.call(this);
            var items = this.model.menu;
            if (items != null) {
                var location = this.parent.model.toolbar_location;
                var reverse = location == "left" || location == "above";
                var orientation = this.parent.model.horizontal ? "vertical" : "horizontal";
                this._menu = new menus_1.ContextMenu(!reverse ? items : array_1.reversed(items), {
                    orientation: orientation,
                    prevent_hide: function (event) { return event.target == _this.el; },
                });
            }
            this._hammer = new hammerjs_1.default(this.el, {
                touchAction: "auto",
                inputClass: hammerjs_1.default.TouchMouseInput,
            });
            this.connect(this.model.change, function () { return _this.render(); });
            this._hammer.on("tap", function (e) {
                var _a;
                if ((_a = _this._menu) === null || _a === void 0 ? void 0 : _a.is_open) {
                    _this._menu.hide();
                    return;
                }
                if (e.target == _this.el) {
                    _this._clicked();
                }
            });
            this._hammer.on("press", function () { return _this._pressed(); });
        };
        ButtonToolButtonView.prototype.remove = function () {
            var _a;
            this._hammer.destroy();
            (_a = this._menu) === null || _a === void 0 ? void 0 : _a.remove();
            _super.prototype.remove.call(this);
        };
        ButtonToolButtonView.prototype.styles = function () {
            return tslib_1.__spread(_super.prototype.styles.call(this), [toolbar_css_1.default, icons_css_1.default, menus_css_1.default]);
        };
        ButtonToolButtonView.prototype.css_classes = function () {
            return _super.prototype.css_classes.call(this).concat(toolbar_1.bk_toolbar_button);
        };
        ButtonToolButtonView.prototype.render = function () {
            dom_1.empty(this.el);
            var icon = this.model.computed_icon;
            if (types_1.isString(icon)) {
                if (string_1.startsWith(icon, "data:image"))
                    this.el.style.backgroundImage = "url('" + icon + "')";
                else
                    this.el.classList.add(icon);
            }
            this.el.title = this.model.tooltip;
            if (this._menu != null) {
                this.root.el.appendChild(this._menu.el);
            }
        };
        ButtonToolButtonView.prototype._pressed = function () {
            var _this = this;
            var _a;
            var _b = this.el.getBoundingClientRect(), left = _b.left, top = _b.top, right = _b.right, bottom = _b.bottom;
            var at = (function () {
                switch (_this.parent.model.toolbar_location) {
                    case "right":
                        return { right: left, top: top };
                    case "left":
                        return { left: right, top: top };
                    case "above":
                        return { left: left, top: bottom };
                    case "below":
                        return { left: left, bottom: top };
                }
            })();
            (_a = this._menu) === null || _a === void 0 ? void 0 : _a.toggle(at);
        };
        return ButtonToolButtonView;
    }(dom_view_1.DOMView));
    exports.ButtonToolButtonView = ButtonToolButtonView;
    ButtonToolButtonView.__name__ = "ButtonToolButtonView";
    var ButtonToolView = /** @class */ (function (_super) {
        tslib_1.__extends(ButtonToolView, _super);
        function ButtonToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return ButtonToolView;
    }(tool_1.ToolView));
    exports.ButtonToolView = ButtonToolView;
    ButtonToolView.__name__ = "ButtonToolView";
    var ButtonTool = /** @class */ (function (_super) {
        tslib_1.__extends(ButtonTool, _super);
        function ButtonTool(attrs) {
            return _super.call(this, attrs) || this;
        }
        ButtonTool.init_ButtonTool = function () {
            this.internal({
                disabled: [p.Boolean, false],
            });
        };
        Object.defineProperty(ButtonTool.prototype, "tooltip", {
            get: function () {
                return this.tool_name;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ButtonTool.prototype, "computed_icon", {
            get: function () {
                return this.icon;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ButtonTool.prototype, "menu", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        return ButtonTool;
    }(tool_1.Tool));
    exports.ButtonTool = ButtonTool;
    ButtonTool.__name__ = "ButtonTool";
    ButtonTool.init_ButtonTool();
},
/* hammerjs/hammer.js */ function _(require, module, exports) {
    /*! Hammer.JS - v2.0.7 - 2016-04-22
     * http://hammerjs.github.io/
     *
     * Copyright (c) 2016 Jorik Tangelder;
     * Licensed under the MIT license */
    (function (window, document, exportName, undefined) {
        'use strict';
        var VENDOR_PREFIXES = ['', 'webkit', 'Moz', 'MS', 'ms', 'o'];
        var TEST_ELEMENT = document.createElement('div');
        var TYPE_FUNCTION = 'function';
        var round = Math.round;
        var abs = Math.abs;
        var now = Date.now;
        /**
         * set a timeout with a given scope
         * @param {Function} fn
         * @param {Number} timeout
         * @param {Object} context
         * @returns {number}
         */
        function setTimeoutContext(fn, timeout, context) {
            return setTimeout(bindFn(fn, context), timeout);
        }
        /**
         * if the argument is an array, we want to execute the fn on each entry
         * if it aint an array we don't want to do a thing.
         * this is used by all the methods that accept a single and array argument.
         * @param {*|Array} arg
         * @param {String} fn
         * @param {Object} [context]
         * @returns {Boolean}
         */
        function invokeArrayArg(arg, fn, context) {
            if (Array.isArray(arg)) {
                each(arg, context[fn], context);
                return true;
            }
            return false;
        }
        /**
         * walk objects and arrays
         * @param {Object} obj
         * @param {Function} iterator
         * @param {Object} context
         */
        function each(obj, iterator, context) {
            var i;
            if (!obj) {
                return;
            }
            if (obj.forEach) {
                obj.forEach(iterator, context);
            }
            else if (obj.length !== undefined) {
                i = 0;
                while (i < obj.length) {
                    iterator.call(context, obj[i], i, obj);
                    i++;
                }
            }
            else {
                for (i in obj) {
                    obj.hasOwnProperty(i) && iterator.call(context, obj[i], i, obj);
                }
            }
        }
        /**
         * wrap a method with a deprecation warning and stack trace
         * @param {Function} method
         * @param {String} name
         * @param {String} message
         * @returns {Function} A new function wrapping the supplied method.
         */
        function deprecate(method, name, message) {
            var deprecationMessage = 'DEPRECATED METHOD: ' + name + '\n' + message + ' AT \n';
            return function () {
                var e = new Error('get-stack-trace');
                var stack = e && e.stack ? e.stack.replace(/^[^\(]+?[\n$]/gm, '')
                    .replace(/^\s+at\s+/gm, '')
                    .replace(/^Object.<anonymous>\s*\(/gm, '{anonymous}()@') : 'Unknown Stack Trace';
                var log = window.console && (window.console.warn || window.console.log);
                if (log) {
                    log.call(window.console, deprecationMessage, stack);
                }
                return method.apply(this, arguments);
            };
        }
        /**
         * extend object.
         * means that properties in dest will be overwritten by the ones in src.
         * @param {Object} target
         * @param {...Object} objects_to_assign
         * @returns {Object} target
         */
        var assign;
        if (typeof Object.assign !== 'function') {
            assign = function assign(target) {
                if (target === undefined || target === null) {
                    throw new TypeError('Cannot convert undefined or null to object');
                }
                var output = Object(target);
                for (var index = 1; index < arguments.length; index++) {
                    var source = arguments[index];
                    if (source !== undefined && source !== null) {
                        for (var nextKey in source) {
                            if (source.hasOwnProperty(nextKey)) {
                                output[nextKey] = source[nextKey];
                            }
                        }
                    }
                }
                return output;
            };
        }
        else {
            assign = Object.assign;
        }
        /**
         * extend object.
         * means that properties in dest will be overwritten by the ones in src.
         * @param {Object} dest
         * @param {Object} src
         * @param {Boolean} [merge=false]
         * @returns {Object} dest
         */
        var extend = deprecate(function extend(dest, src, merge) {
            var keys = Object.keys(src);
            var i = 0;
            while (i < keys.length) {
                if (!merge || (merge && dest[keys[i]] === undefined)) {
                    dest[keys[i]] = src[keys[i]];
                }
                i++;
            }
            return dest;
        }, 'extend', 'Use `assign`.');
        /**
         * merge the values from src in the dest.
         * means that properties that exist in dest will not be overwritten by src
         * @param {Object} dest
         * @param {Object} src
         * @returns {Object} dest
         */
        var merge = deprecate(function merge(dest, src) {
            return extend(dest, src, true);
        }, 'merge', 'Use `assign`.');
        /**
         * simple class inheritance
         * @param {Function} child
         * @param {Function} base
         * @param {Object} [properties]
         */
        function inherit(child, base, properties) {
            var baseP = base.prototype, childP;
            childP = child.prototype = Object.create(baseP);
            childP.constructor = child;
            childP._super = baseP;
            if (properties) {
                assign(childP, properties);
            }
        }
        /**
         * simple function bind
         * @param {Function} fn
         * @param {Object} context
         * @returns {Function}
         */
        function bindFn(fn, context) {
            return function boundFn() {
                return fn.apply(context, arguments);
            };
        }
        /**
         * let a boolean value also be a function that must return a boolean
         * this first item in args will be used as the context
         * @param {Boolean|Function} val
         * @param {Array} [args]
         * @returns {Boolean}
         */
        function boolOrFn(val, args) {
            if (typeof val == TYPE_FUNCTION) {
                return val.apply(args ? args[0] || undefined : undefined, args);
            }
            return val;
        }
        /**
         * use the val2 when val1 is undefined
         * @param {*} val1
         * @param {*} val2
         * @returns {*}
         */
        function ifUndefined(val1, val2) {
            return (val1 === undefined) ? val2 : val1;
        }
        /**
         * addEventListener with multiple events at once
         * @param {EventTarget} target
         * @param {String} types
         * @param {Function} handler
         */
        function addEventListeners(target, types, handler) {
            each(splitStr(types), function (type) {
                target.addEventListener(type, handler, false);
            });
        }
        /**
         * removeEventListener with multiple events at once
         * @param {EventTarget} target
         * @param {String} types
         * @param {Function} handler
         */
        function removeEventListeners(target, types, handler) {
            each(splitStr(types), function (type) {
                target.removeEventListener(type, handler, false);
            });
        }
        /**
         * find if a node is in the given parent
         * @method hasParent
         * @param {HTMLElement} node
         * @param {HTMLElement} parent
         * @return {Boolean} found
         */
        function hasParent(node, parent) {
            while (node) {
                if (node == parent) {
                    return true;
                }
                node = node.parentNode;
            }
            return false;
        }
        /**
         * small indexOf wrapper
         * @param {String} str
         * @param {String} find
         * @returns {Boolean} found
         */
        function inStr(str, find) {
            return str.indexOf(find) > -1;
        }
        /**
         * split string on whitespace
         * @param {String} str
         * @returns {Array} words
         */
        function splitStr(str) {
            return str.trim().split(/\s+/g);
        }
        /**
         * find if a array contains the object using indexOf or a simple polyFill
         * @param {Array} src
         * @param {String} find
         * @param {String} [findByKey]
         * @return {Boolean|Number} false when not found, or the index
         */
        function inArray(src, find, findByKey) {
            if (src.indexOf && !findByKey) {
                return src.indexOf(find);
            }
            else {
                var i = 0;
                while (i < src.length) {
                    if ((findByKey && src[i][findByKey] == find) || (!findByKey && src[i] === find)) {
                        return i;
                    }
                    i++;
                }
                return -1;
            }
        }
        /**
         * convert array-like objects to real arrays
         * @param {Object} obj
         * @returns {Array}
         */
        function toArray(obj) {
            return Array.prototype.slice.call(obj, 0);
        }
        /**
         * unique array with objects based on a key (like 'id') or just by the array's value
         * @param {Array} src [{id:1},{id:2},{id:1}]
         * @param {String} [key]
         * @param {Boolean} [sort=False]
         * @returns {Array} [{id:1},{id:2}]
         */
        function uniqueArray(src, key, sort) {
            var results = [];
            var values = [];
            var i = 0;
            while (i < src.length) {
                var val = key ? src[i][key] : src[i];
                if (inArray(values, val) < 0) {
                    results.push(src[i]);
                }
                values[i] = val;
                i++;
            }
            if (sort) {
                if (!key) {
                    results = results.sort();
                }
                else {
                    results = results.sort(function sortUniqueArray(a, b) {
                        return a[key] > b[key];
                    });
                }
            }
            return results;
        }
        /**
         * get the prefixed property
         * @param {Object} obj
         * @param {String} property
         * @returns {String|Undefined} prefixed
         */
        function prefixed(obj, property) {
            var prefix, prop;
            var camelProp = property[0].toUpperCase() + property.slice(1);
            var i = 0;
            while (i < VENDOR_PREFIXES.length) {
                prefix = VENDOR_PREFIXES[i];
                prop = (prefix) ? prefix + camelProp : property;
                if (prop in obj) {
                    return prop;
                }
                i++;
            }
            return undefined;
        }
        /**
         * get a unique id
         * @returns {number} uniqueId
         */
        var _uniqueId = 1;
        function uniqueId() {
            return _uniqueId++;
        }
        /**
         * get the window object of an element
         * @param {HTMLElement} element
         * @returns {DocumentView|Window}
         */
        function getWindowForElement(element) {
            var doc = element.ownerDocument || element;
            return (doc.defaultView || doc.parentWindow || window);
        }
        var MOBILE_REGEX = /mobile|tablet|ip(ad|hone|od)|android/i;
        var SUPPORT_TOUCH = ('ontouchstart' in window);
        var SUPPORT_POINTER_EVENTS = prefixed(window, 'PointerEvent') !== undefined;
        var SUPPORT_ONLY_TOUCH = SUPPORT_TOUCH && MOBILE_REGEX.test(navigator.userAgent);
        var INPUT_TYPE_TOUCH = 'touch';
        var INPUT_TYPE_PEN = 'pen';
        var INPUT_TYPE_MOUSE = 'mouse';
        var INPUT_TYPE_KINECT = 'kinect';
        var COMPUTE_INTERVAL = 25;
        var INPUT_START = 1;
        var INPUT_MOVE = 2;
        var INPUT_END = 4;
        var INPUT_CANCEL = 8;
        var DIRECTION_NONE = 1;
        var DIRECTION_LEFT = 2;
        var DIRECTION_RIGHT = 4;
        var DIRECTION_UP = 8;
        var DIRECTION_DOWN = 16;
        var DIRECTION_HORIZONTAL = DIRECTION_LEFT | DIRECTION_RIGHT;
        var DIRECTION_VERTICAL = DIRECTION_UP | DIRECTION_DOWN;
        var DIRECTION_ALL = DIRECTION_HORIZONTAL | DIRECTION_VERTICAL;
        var PROPS_XY = ['x', 'y'];
        var PROPS_CLIENT_XY = ['clientX', 'clientY'];
        /**
         * create new input type manager
         * @param {Manager} manager
         * @param {Function} callback
         * @returns {Input}
         * @constructor
         */
        function Input(manager, callback) {
            var self = this;
            this.manager = manager;
            this.callback = callback;
            this.element = manager.element;
            this.target = manager.options.inputTarget;
            // smaller wrapper around the handler, for the scope and the enabled state of the manager,
            // so when disabled the input events are completely bypassed.
            this.domHandler = function (ev) {
                if (boolOrFn(manager.options.enable, [manager])) {
                    self.handler(ev);
                }
            };
            this.init();
        }
        Input.prototype = {
            /**
             * should handle the inputEvent data and trigger the callback
             * @virtual
             */
            handler: function () { },
            /**
             * bind the events
             */
            init: function () {
                this.evEl && addEventListeners(this.element, this.evEl, this.domHandler);
                this.evTarget && addEventListeners(this.target, this.evTarget, this.domHandler);
                this.evWin && addEventListeners(getWindowForElement(this.element), this.evWin, this.domHandler);
            },
            /**
             * unbind the events
             */
            destroy: function () {
                this.evEl && removeEventListeners(this.element, this.evEl, this.domHandler);
                this.evTarget && removeEventListeners(this.target, this.evTarget, this.domHandler);
                this.evWin && removeEventListeners(getWindowForElement(this.element), this.evWin, this.domHandler);
            }
        };
        /**
         * create new input type manager
         * called by the Manager constructor
         * @param {Hammer} manager
         * @returns {Input}
         */
        function createInputInstance(manager) {
            var Type;
            var inputClass = manager.options.inputClass;
            if (inputClass) {
                Type = inputClass;
            }
            else if (SUPPORT_POINTER_EVENTS) {
                Type = PointerEventInput;
            }
            else if (SUPPORT_ONLY_TOUCH) {
                Type = TouchInput;
            }
            else if (!SUPPORT_TOUCH) {
                Type = MouseInput;
            }
            else {
                Type = TouchMouseInput;
            }
            return new (Type)(manager, inputHandler);
        }
        /**
         * handle input events
         * @param {Manager} manager
         * @param {String} eventType
         * @param {Object} input
         */
        function inputHandler(manager, eventType, input) {
            var pointersLen = input.pointers.length;
            var changedPointersLen = input.changedPointers.length;
            var isFirst = (eventType & INPUT_START && (pointersLen - changedPointersLen === 0));
            var isFinal = (eventType & (INPUT_END | INPUT_CANCEL) && (pointersLen - changedPointersLen === 0));
            input.isFirst = !!isFirst;
            input.isFinal = !!isFinal;
            if (isFirst) {
                manager.session = {};
            }
            // source event is the normalized value of the domEvents
            // like 'touchstart, mouseup, pointerdown'
            input.eventType = eventType;
            // compute scale, rotation etc
            computeInputData(manager, input);
            // emit secret event
            manager.emit('hammer.input', input);
            manager.recognize(input);
            manager.session.prevInput = input;
        }
        /**
         * extend the data with some usable properties like scale, rotate, velocity etc
         * @param {Object} manager
         * @param {Object} input
         */
        function computeInputData(manager, input) {
            var session = manager.session;
            var pointers = input.pointers;
            var pointersLength = pointers.length;
            // store the first input to calculate the distance and direction
            if (!session.firstInput) {
                session.firstInput = simpleCloneInputData(input);
            }
            // to compute scale and rotation we need to store the multiple touches
            if (pointersLength > 1 && !session.firstMultiple) {
                session.firstMultiple = simpleCloneInputData(input);
            }
            else if (pointersLength === 1) {
                session.firstMultiple = false;
            }
            var firstInput = session.firstInput;
            var firstMultiple = session.firstMultiple;
            var offsetCenter = firstMultiple ? firstMultiple.center : firstInput.center;
            var center = input.center = getCenter(pointers);
            input.timeStamp = now();
            input.deltaTime = input.timeStamp - firstInput.timeStamp;
            input.angle = getAngle(offsetCenter, center);
            input.distance = getDistance(offsetCenter, center);
            computeDeltaXY(session, input);
            input.offsetDirection = getDirection(input.deltaX, input.deltaY);
            var overallVelocity = getVelocity(input.deltaTime, input.deltaX, input.deltaY);
            input.overallVelocityX = overallVelocity.x;
            input.overallVelocityY = overallVelocity.y;
            input.overallVelocity = (abs(overallVelocity.x) > abs(overallVelocity.y)) ? overallVelocity.x : overallVelocity.y;
            input.scale = firstMultiple ? getScale(firstMultiple.pointers, pointers) : 1;
            input.rotation = firstMultiple ? getRotation(firstMultiple.pointers, pointers) : 0;
            input.maxPointers = !session.prevInput ? input.pointers.length : ((input.pointers.length >
                session.prevInput.maxPointers) ? input.pointers.length : session.prevInput.maxPointers);
            computeIntervalInputData(session, input);
            // find the correct target
            var target = manager.element;
            if (hasParent(input.srcEvent.target, target)) {
                target = input.srcEvent.target;
            }
            input.target = target;
        }
        function computeDeltaXY(session, input) {
            var center = input.center;
            var offset = session.offsetDelta || {};
            var prevDelta = session.prevDelta || {};
            var prevInput = session.prevInput || {};
            if (input.eventType === INPUT_START || prevInput.eventType === INPUT_END) {
                prevDelta = session.prevDelta = {
                    x: prevInput.deltaX || 0,
                    y: prevInput.deltaY || 0
                };
                offset = session.offsetDelta = {
                    x: center.x,
                    y: center.y
                };
            }
            input.deltaX = prevDelta.x + (center.x - offset.x);
            input.deltaY = prevDelta.y + (center.y - offset.y);
        }
        /**
         * velocity is calculated every x ms
         * @param {Object} session
         * @param {Object} input
         */
        function computeIntervalInputData(session, input) {
            var last = session.lastInterval || input, deltaTime = input.timeStamp - last.timeStamp, velocity, velocityX, velocityY, direction;
            if (input.eventType != INPUT_CANCEL && (deltaTime > COMPUTE_INTERVAL || last.velocity === undefined)) {
                var deltaX = input.deltaX - last.deltaX;
                var deltaY = input.deltaY - last.deltaY;
                var v = getVelocity(deltaTime, deltaX, deltaY);
                velocityX = v.x;
                velocityY = v.y;
                velocity = (abs(v.x) > abs(v.y)) ? v.x : v.y;
                direction = getDirection(deltaX, deltaY);
                session.lastInterval = input;
            }
            else {
                // use latest velocity info if it doesn't overtake a minimum period
                velocity = last.velocity;
                velocityX = last.velocityX;
                velocityY = last.velocityY;
                direction = last.direction;
            }
            input.velocity = velocity;
            input.velocityX = velocityX;
            input.velocityY = velocityY;
            input.direction = direction;
        }
        /**
         * create a simple clone from the input used for storage of firstInput and firstMultiple
         * @param {Object} input
         * @returns {Object} clonedInputData
         */
        function simpleCloneInputData(input) {
            // make a simple copy of the pointers because we will get a reference if we don't
            // we only need clientXY for the calculations
            var pointers = [];
            var i = 0;
            while (i < input.pointers.length) {
                pointers[i] = {
                    clientX: round(input.pointers[i].clientX),
                    clientY: round(input.pointers[i].clientY)
                };
                i++;
            }
            return {
                timeStamp: now(),
                pointers: pointers,
                center: getCenter(pointers),
                deltaX: input.deltaX,
                deltaY: input.deltaY
            };
        }
        /**
         * get the center of all the pointers
         * @param {Array} pointers
         * @return {Object} center contains `x` and `y` properties
         */
        function getCenter(pointers) {
            var pointersLength = pointers.length;
            // no need to loop when only one touch
            if (pointersLength === 1) {
                return {
                    x: round(pointers[0].clientX),
                    y: round(pointers[0].clientY)
                };
            }
            var x = 0, y = 0, i = 0;
            while (i < pointersLength) {
                x += pointers[i].clientX;
                y += pointers[i].clientY;
                i++;
            }
            return {
                x: round(x / pointersLength),
                y: round(y / pointersLength)
            };
        }
        /**
         * calculate the velocity between two points. unit is in px per ms.
         * @param {Number} deltaTime
         * @param {Number} x
         * @param {Number} y
         * @return {Object} velocity `x` and `y`
         */
        function getVelocity(deltaTime, x, y) {
            return {
                x: x / deltaTime || 0,
                y: y / deltaTime || 0
            };
        }
        /**
         * get the direction between two points
         * @param {Number} x
         * @param {Number} y
         * @return {Number} direction
         */
        function getDirection(x, y) {
            if (x === y) {
                return DIRECTION_NONE;
            }
            if (abs(x) >= abs(y)) {
                return x < 0 ? DIRECTION_LEFT : DIRECTION_RIGHT;
            }
            return y < 0 ? DIRECTION_UP : DIRECTION_DOWN;
        }
        /**
         * calculate the absolute distance between two points
         * @param {Object} p1 {x, y}
         * @param {Object} p2 {x, y}
         * @param {Array} [props] containing x and y keys
         * @return {Number} distance
         */
        function getDistance(p1, p2, props) {
            if (!props) {
                props = PROPS_XY;
            }
            var x = p2[props[0]] - p1[props[0]], y = p2[props[1]] - p1[props[1]];
            return Math.sqrt((x * x) + (y * y));
        }
        /**
         * calculate the angle between two coordinates
         * @param {Object} p1
         * @param {Object} p2
         * @param {Array} [props] containing x and y keys
         * @return {Number} angle
         */
        function getAngle(p1, p2, props) {
            if (!props) {
                props = PROPS_XY;
            }
            var x = p2[props[0]] - p1[props[0]], y = p2[props[1]] - p1[props[1]];
            return Math.atan2(y, x) * 180 / Math.PI;
        }
        /**
         * calculate the rotation degrees between two pointersets
         * @param {Array} start array of pointers
         * @param {Array} end array of pointers
         * @return {Number} rotation
         */
        function getRotation(start, end) {
            return getAngle(end[1], end[0], PROPS_CLIENT_XY) + getAngle(start[1], start[0], PROPS_CLIENT_XY);
        }
        /**
         * calculate the scale factor between two pointersets
         * no scale is 1, and goes down to 0 when pinched together, and bigger when pinched out
         * @param {Array} start array of pointers
         * @param {Array} end array of pointers
         * @return {Number} scale
         */
        function getScale(start, end) {
            return getDistance(end[0], end[1], PROPS_CLIENT_XY) / getDistance(start[0], start[1], PROPS_CLIENT_XY);
        }
        var MOUSE_INPUT_MAP = {
            mousedown: INPUT_START,
            mousemove: INPUT_MOVE,
            mouseup: INPUT_END
        };
        var MOUSE_ELEMENT_EVENTS = 'mousedown';
        var MOUSE_WINDOW_EVENTS = 'mousemove mouseup';
        /**
         * Mouse events input
         * @constructor
         * @extends Input
         */
        function MouseInput() {
            this.evEl = MOUSE_ELEMENT_EVENTS;
            this.evWin = MOUSE_WINDOW_EVENTS;
            this.pressed = false; // mousedown state
            Input.apply(this, arguments);
        }
        inherit(MouseInput, Input, {
            /**
             * handle mouse events
             * @param {Object} ev
             */
            handler: function MEhandler(ev) {
                var eventType = MOUSE_INPUT_MAP[ev.type];
                // on start we want to have the left mouse button down
                if (eventType & INPUT_START && ev.button === 0) {
                    this.pressed = true;
                }
                if (eventType & INPUT_MOVE && ev.which !== 1) {
                    eventType = INPUT_END;
                }
                // mouse must be down
                if (!this.pressed) {
                    return;
                }
                if (eventType & INPUT_END) {
                    this.pressed = false;
                }
                this.callback(this.manager, eventType, {
                    pointers: [ev],
                    changedPointers: [ev],
                    pointerType: INPUT_TYPE_MOUSE,
                    srcEvent: ev
                });
            }
        });
        var POINTER_INPUT_MAP = {
            pointerdown: INPUT_START,
            pointermove: INPUT_MOVE,
            pointerup: INPUT_END,
            pointercancel: INPUT_CANCEL,
            pointerout: INPUT_CANCEL
        };
        // in IE10 the pointer types is defined as an enum
        var IE10_POINTER_TYPE_ENUM = {
            2: INPUT_TYPE_TOUCH,
            3: INPUT_TYPE_PEN,
            4: INPUT_TYPE_MOUSE,
            5: INPUT_TYPE_KINECT // see https://twitter.com/jacobrossi/status/480596438489890816
        };
        var POINTER_ELEMENT_EVENTS = 'pointerdown';
        var POINTER_WINDOW_EVENTS = 'pointermove pointerup pointercancel';
        // IE10 has prefixed support, and case-sensitive
        if (window.MSPointerEvent && !window.PointerEvent) {
            POINTER_ELEMENT_EVENTS = 'MSPointerDown';
            POINTER_WINDOW_EVENTS = 'MSPointerMove MSPointerUp MSPointerCancel';
        }
        /**
         * Pointer events input
         * @constructor
         * @extends Input
         */
        function PointerEventInput() {
            this.evEl = POINTER_ELEMENT_EVENTS;
            this.evWin = POINTER_WINDOW_EVENTS;
            Input.apply(this, arguments);
            this.store = (this.manager.session.pointerEvents = []);
        }
        inherit(PointerEventInput, Input, {
            /**
             * handle mouse events
             * @param {Object} ev
             */
            handler: function PEhandler(ev) {
                var store = this.store;
                var removePointer = false;
                var eventTypeNormalized = ev.type.toLowerCase().replace('ms', '');
                var eventType = POINTER_INPUT_MAP[eventTypeNormalized];
                var pointerType = IE10_POINTER_TYPE_ENUM[ev.pointerType] || ev.pointerType;
                var isTouch = (pointerType == INPUT_TYPE_TOUCH);
                // get index of the event in the store
                var storeIndex = inArray(store, ev.pointerId, 'pointerId');
                // start and mouse must be down
                if (eventType & INPUT_START && (ev.button === 0 || isTouch)) {
                    if (storeIndex < 0) {
                        store.push(ev);
                        storeIndex = store.length - 1;
                    }
                }
                else if (eventType & (INPUT_END | INPUT_CANCEL)) {
                    removePointer = true;
                }
                // it not found, so the pointer hasn't been down (so it's probably a hover)
                if (storeIndex < 0) {
                    return;
                }
                // update the event in the store
                store[storeIndex] = ev;
                this.callback(this.manager, eventType, {
                    pointers: store,
                    changedPointers: [ev],
                    pointerType: pointerType,
                    srcEvent: ev
                });
                if (removePointer) {
                    // remove from the store
                    store.splice(storeIndex, 1);
                }
            }
        });
        var SINGLE_TOUCH_INPUT_MAP = {
            touchstart: INPUT_START,
            touchmove: INPUT_MOVE,
            touchend: INPUT_END,
            touchcancel: INPUT_CANCEL
        };
        var SINGLE_TOUCH_TARGET_EVENTS = 'touchstart';
        var SINGLE_TOUCH_WINDOW_EVENTS = 'touchstart touchmove touchend touchcancel';
        /**
         * Touch events input
         * @constructor
         * @extends Input
         */
        function SingleTouchInput() {
            this.evTarget = SINGLE_TOUCH_TARGET_EVENTS;
            this.evWin = SINGLE_TOUCH_WINDOW_EVENTS;
            this.started = false;
            Input.apply(this, arguments);
        }
        inherit(SingleTouchInput, Input, {
            handler: function TEhandler(ev) {
                var type = SINGLE_TOUCH_INPUT_MAP[ev.type];
                // should we handle the touch events?
                if (type === INPUT_START) {
                    this.started = true;
                }
                if (!this.started) {
                    return;
                }
                var touches = normalizeSingleTouches.call(this, ev, type);
                // when done, reset the started state
                if (type & (INPUT_END | INPUT_CANCEL) && touches[0].length - touches[1].length === 0) {
                    this.started = false;
                }
                this.callback(this.manager, type, {
                    pointers: touches[0],
                    changedPointers: touches[1],
                    pointerType: INPUT_TYPE_TOUCH,
                    srcEvent: ev
                });
            }
        });
        /**
         * @this {TouchInput}
         * @param {Object} ev
         * @param {Number} type flag
         * @returns {undefined|Array} [all, changed]
         */
        function normalizeSingleTouches(ev, type) {
            var all = toArray(ev.touches);
            var changed = toArray(ev.changedTouches);
            if (type & (INPUT_END | INPUT_CANCEL)) {
                all = uniqueArray(all.concat(changed), 'identifier', true);
            }
            return [all, changed];
        }
        var TOUCH_INPUT_MAP = {
            touchstart: INPUT_START,
            touchmove: INPUT_MOVE,
            touchend: INPUT_END,
            touchcancel: INPUT_CANCEL
        };
        var TOUCH_TARGET_EVENTS = 'touchstart touchmove touchend touchcancel';
        /**
         * Multi-user touch events input
         * @constructor
         * @extends Input
         */
        function TouchInput() {
            this.evTarget = TOUCH_TARGET_EVENTS;
            this.targetIds = {};
            Input.apply(this, arguments);
        }
        inherit(TouchInput, Input, {
            handler: function MTEhandler(ev) {
                var type = TOUCH_INPUT_MAP[ev.type];
                var touches = getTouches.call(this, ev, type);
                if (!touches) {
                    return;
                }
                this.callback(this.manager, type, {
                    pointers: touches[0],
                    changedPointers: touches[1],
                    pointerType: INPUT_TYPE_TOUCH,
                    srcEvent: ev
                });
            }
        });
        /**
         * @this {TouchInput}
         * @param {Object} ev
         * @param {Number} type flag
         * @returns {undefined|Array} [all, changed]
         */
        function getTouches(ev, type) {
            var allTouches = toArray(ev.touches);
            var targetIds = this.targetIds;
            // when there is only one touch, the process can be simplified
            if (type & (INPUT_START | INPUT_MOVE) && allTouches.length === 1) {
                targetIds[allTouches[0].identifier] = true;
                return [allTouches, allTouches];
            }
            var i, targetTouches, changedTouches = toArray(ev.changedTouches), changedTargetTouches = [], target = this.target;
            // get target touches from touches
            targetTouches = allTouches.filter(function (touch) {
                return hasParent(touch.target, target);
            });
            // collect touches
            if (type === INPUT_START) {
                i = 0;
                while (i < targetTouches.length) {
                    targetIds[targetTouches[i].identifier] = true;
                    i++;
                }
            }
            // filter changed touches to only contain touches that exist in the collected target ids
            i = 0;
            while (i < changedTouches.length) {
                if (targetIds[changedTouches[i].identifier]) {
                    changedTargetTouches.push(changedTouches[i]);
                }
                // cleanup removed touches
                if (type & (INPUT_END | INPUT_CANCEL)) {
                    delete targetIds[changedTouches[i].identifier];
                }
                i++;
            }
            if (!changedTargetTouches.length) {
                return;
            }
            return [
                // merge targetTouches with changedTargetTouches so it contains ALL touches, including 'end' and 'cancel'
                uniqueArray(targetTouches.concat(changedTargetTouches), 'identifier', true),
                changedTargetTouches
            ];
        }
        /**
         * Combined touch and mouse input
         *
         * Touch has a higher priority then mouse, and while touching no mouse events are allowed.
         * This because touch devices also emit mouse events while doing a touch.
         *
         * @constructor
         * @extends Input
         */
        var DEDUP_TIMEOUT = 2500;
        var DEDUP_DISTANCE = 25;
        function TouchMouseInput() {
            Input.apply(this, arguments);
            var handler = bindFn(this.handler, this);
            this.touch = new TouchInput(this.manager, handler);
            this.mouse = new MouseInput(this.manager, handler);
            this.primaryTouch = null;
            this.lastTouches = [];
        }
        inherit(TouchMouseInput, Input, {
            /**
             * handle mouse and touch events
             * @param {Hammer} manager
             * @param {String} inputEvent
             * @param {Object} inputData
             */
            handler: function TMEhandler(manager, inputEvent, inputData) {
                var isTouch = (inputData.pointerType == INPUT_TYPE_TOUCH), isMouse = (inputData.pointerType == INPUT_TYPE_MOUSE);
                if (isMouse && inputData.sourceCapabilities && inputData.sourceCapabilities.firesTouchEvents) {
                    return;
                }
                // when we're in a touch event, record touches to  de-dupe synthetic mouse event
                if (isTouch) {
                    recordTouches.call(this, inputEvent, inputData);
                }
                else if (isMouse && isSyntheticEvent.call(this, inputData)) {
                    return;
                }
                this.callback(manager, inputEvent, inputData);
            },
            /**
             * remove the event listeners
             */
            destroy: function destroy() {
                this.touch.destroy();
                this.mouse.destroy();
            }
        });
        function recordTouches(eventType, eventData) {
            if (eventType & INPUT_START) {
                this.primaryTouch = eventData.changedPointers[0].identifier;
                setLastTouch.call(this, eventData);
            }
            else if (eventType & (INPUT_END | INPUT_CANCEL)) {
                setLastTouch.call(this, eventData);
            }
        }
        function setLastTouch(eventData) {
            var touch = eventData.changedPointers[0];
            if (touch.identifier === this.primaryTouch) {
                var lastTouch = { x: touch.clientX, y: touch.clientY };
                this.lastTouches.push(lastTouch);
                var lts = this.lastTouches;
                var removeLastTouch = function () {
                    var i = lts.indexOf(lastTouch);
                    if (i > -1) {
                        lts.splice(i, 1);
                    }
                };
                setTimeout(removeLastTouch, DEDUP_TIMEOUT);
            }
        }
        function isSyntheticEvent(eventData) {
            var x = eventData.srcEvent.clientX, y = eventData.srcEvent.clientY;
            for (var i = 0; i < this.lastTouches.length; i++) {
                var t = this.lastTouches[i];
                var dx = Math.abs(x - t.x), dy = Math.abs(y - t.y);
                if (dx <= DEDUP_DISTANCE && dy <= DEDUP_DISTANCE) {
                    return true;
                }
            }
            return false;
        }
        var PREFIXED_TOUCH_ACTION = prefixed(TEST_ELEMENT.style, 'touchAction');
        var NATIVE_TOUCH_ACTION = PREFIXED_TOUCH_ACTION !== undefined;
        // magical touchAction value
        var TOUCH_ACTION_COMPUTE = 'compute';
        var TOUCH_ACTION_AUTO = 'auto';
        var TOUCH_ACTION_MANIPULATION = 'manipulation'; // not implemented
        var TOUCH_ACTION_NONE = 'none';
        var TOUCH_ACTION_PAN_X = 'pan-x';
        var TOUCH_ACTION_PAN_Y = 'pan-y';
        var TOUCH_ACTION_MAP = getTouchActionProps();
        /**
         * Touch Action
         * sets the touchAction property or uses the js alternative
         * @param {Manager} manager
         * @param {String} value
         * @constructor
         */
        function TouchAction(manager, value) {
            this.manager = manager;
            this.set(value);
        }
        TouchAction.prototype = {
            /**
             * set the touchAction value on the element or enable the polyfill
             * @param {String} value
             */
            set: function (value) {
                // find out the touch-action by the event handlers
                if (value == TOUCH_ACTION_COMPUTE) {
                    value = this.compute();
                }
                if (NATIVE_TOUCH_ACTION && this.manager.element.style && TOUCH_ACTION_MAP[value]) {
                    this.manager.element.style[PREFIXED_TOUCH_ACTION] = value;
                }
                this.actions = value.toLowerCase().trim();
            },
            /**
             * just re-set the touchAction value
             */
            update: function () {
                this.set(this.manager.options.touchAction);
            },
            /**
             * compute the value for the touchAction property based on the recognizer's settings
             * @returns {String} value
             */
            compute: function () {
                var actions = [];
                each(this.manager.recognizers, function (recognizer) {
                    if (boolOrFn(recognizer.options.enable, [recognizer])) {
                        actions = actions.concat(recognizer.getTouchAction());
                    }
                });
                return cleanTouchActions(actions.join(' '));
            },
            /**
             * this method is called on each input cycle and provides the preventing of the browser behavior
             * @param {Object} input
             */
            preventDefaults: function (input) {
                var srcEvent = input.srcEvent;
                var direction = input.offsetDirection;
                // if the touch action did prevented once this session
                if (this.manager.session.prevented) {
                    srcEvent.preventDefault();
                    return;
                }
                var actions = this.actions;
                var hasNone = inStr(actions, TOUCH_ACTION_NONE) && !TOUCH_ACTION_MAP[TOUCH_ACTION_NONE];
                var hasPanY = inStr(actions, TOUCH_ACTION_PAN_Y) && !TOUCH_ACTION_MAP[TOUCH_ACTION_PAN_Y];
                var hasPanX = inStr(actions, TOUCH_ACTION_PAN_X) && !TOUCH_ACTION_MAP[TOUCH_ACTION_PAN_X];
                if (hasNone) {
                    //do not prevent defaults if this is a tap gesture
                    var isTapPointer = input.pointers.length === 1;
                    var isTapMovement = input.distance < 2;
                    var isTapTouchTime = input.deltaTime < 250;
                    if (isTapPointer && isTapMovement && isTapTouchTime) {
                        return;
                    }
                }
                if (hasPanX && hasPanY) {
                    // `pan-x pan-y` means browser handles all scrolling/panning, do not prevent
                    return;
                }
                if (hasNone ||
                    (hasPanY && direction & DIRECTION_HORIZONTAL) ||
                    (hasPanX && direction & DIRECTION_VERTICAL)) {
                    return this.preventSrc(srcEvent);
                }
            },
            /**
             * call preventDefault to prevent the browser's default behavior (scrolling in most cases)
             * @param {Object} srcEvent
             */
            preventSrc: function (srcEvent) {
                this.manager.session.prevented = true;
                srcEvent.preventDefault();
            }
        };
        /**
         * when the touchActions are collected they are not a valid value, so we need to clean things up. *
         * @param {String} actions
         * @returns {*}
         */
        function cleanTouchActions(actions) {
            // none
            if (inStr(actions, TOUCH_ACTION_NONE)) {
                return TOUCH_ACTION_NONE;
            }
            var hasPanX = inStr(actions, TOUCH_ACTION_PAN_X);
            var hasPanY = inStr(actions, TOUCH_ACTION_PAN_Y);
            // if both pan-x and pan-y are set (different recognizers
            // for different directions, e.g. horizontal pan but vertical swipe?)
            // we need none (as otherwise with pan-x pan-y combined none of these
            // recognizers will work, since the browser would handle all panning
            if (hasPanX && hasPanY) {
                return TOUCH_ACTION_NONE;
            }
            // pan-x OR pan-y
            if (hasPanX || hasPanY) {
                return hasPanX ? TOUCH_ACTION_PAN_X : TOUCH_ACTION_PAN_Y;
            }
            // manipulation
            if (inStr(actions, TOUCH_ACTION_MANIPULATION)) {
                return TOUCH_ACTION_MANIPULATION;
            }
            return TOUCH_ACTION_AUTO;
        }
        function getTouchActionProps() {
            if (!NATIVE_TOUCH_ACTION) {
                return false;
            }
            var touchMap = {};
            var cssSupports = window.CSS && window.CSS.supports;
            ['auto', 'manipulation', 'pan-y', 'pan-x', 'pan-x pan-y', 'none'].forEach(function (val) {
                // If css.supports is not supported but there is native touch-action assume it supports
                // all values. This is the case for IE 10 and 11.
                touchMap[val] = cssSupports ? window.CSS.supports('touch-action', val) : true;
            });
            return touchMap;
        }
        /**
         * Recognizer flow explained; *
         * All recognizers have the initial state of POSSIBLE when a input session starts.
         * The definition of a input session is from the first input until the last input, with all it's movement in it. *
         * Example session for mouse-input: mousedown -> mousemove -> mouseup
         *
         * On each recognizing cycle (see Manager.recognize) the .recognize() method is executed
         * which determines with state it should be.
         *
         * If the recognizer has the state FAILED, CANCELLED or RECOGNIZED (equals ENDED), it is reset to
         * POSSIBLE to give it another change on the next cycle.
         *
         *               Possible
         *                  |
         *            +-----+---------------+
         *            |                     |
         *      +-----+-----+               |
         *      |           |               |
         *   Failed      Cancelled          |
         *                          +-------+------+
         *                          |              |
         *                      Recognized       Began
         *                                         |
         *                                      Changed
         *                                         |
         *                                  Ended/Recognized
         */
        var STATE_POSSIBLE = 1;
        var STATE_BEGAN = 2;
        var STATE_CHANGED = 4;
        var STATE_ENDED = 8;
        var STATE_RECOGNIZED = STATE_ENDED;
        var STATE_CANCELLED = 16;
        var STATE_FAILED = 32;
        /**
         * Recognizer
         * Every recognizer needs to extend from this class.
         * @constructor
         * @param {Object} options
         */
        function Recognizer(options) {
            this.options = assign({}, this.defaults, options || {});
            this.id = uniqueId();
            this.manager = null;
            // default is enable true
            this.options.enable = ifUndefined(this.options.enable, true);
            this.state = STATE_POSSIBLE;
            this.simultaneous = {};
            this.requireFail = [];
        }
        Recognizer.prototype = {
            /**
             * @virtual
             * @type {Object}
             */
            defaults: {},
            /**
             * set options
             * @param {Object} options
             * @return {Recognizer}
             */
            set: function (options) {
                assign(this.options, options);
                // also update the touchAction, in case something changed about the directions/enabled state
                this.manager && this.manager.touchAction.update();
                return this;
            },
            /**
             * recognize simultaneous with an other recognizer.
             * @param {Recognizer} otherRecognizer
             * @returns {Recognizer} this
             */
            recognizeWith: function (otherRecognizer) {
                if (invokeArrayArg(otherRecognizer, 'recognizeWith', this)) {
                    return this;
                }
                var simultaneous = this.simultaneous;
                otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);
                if (!simultaneous[otherRecognizer.id]) {
                    simultaneous[otherRecognizer.id] = otherRecognizer;
                    otherRecognizer.recognizeWith(this);
                }
                return this;
            },
            /**
             * drop the simultaneous link. it doesnt remove the link on the other recognizer.
             * @param {Recognizer} otherRecognizer
             * @returns {Recognizer} this
             */
            dropRecognizeWith: function (otherRecognizer) {
                if (invokeArrayArg(otherRecognizer, 'dropRecognizeWith', this)) {
                    return this;
                }
                otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);
                delete this.simultaneous[otherRecognizer.id];
                return this;
            },
            /**
             * recognizer can only run when an other is failing
             * @param {Recognizer} otherRecognizer
             * @returns {Recognizer} this
             */
            requireFailure: function (otherRecognizer) {
                if (invokeArrayArg(otherRecognizer, 'requireFailure', this)) {
                    return this;
                }
                var requireFail = this.requireFail;
                otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);
                if (inArray(requireFail, otherRecognizer) === -1) {
                    requireFail.push(otherRecognizer);
                    otherRecognizer.requireFailure(this);
                }
                return this;
            },
            /**
             * drop the requireFailure link. it does not remove the link on the other recognizer.
             * @param {Recognizer} otherRecognizer
             * @returns {Recognizer} this
             */
            dropRequireFailure: function (otherRecognizer) {
                if (invokeArrayArg(otherRecognizer, 'dropRequireFailure', this)) {
                    return this;
                }
                otherRecognizer = getRecognizerByNameIfManager(otherRecognizer, this);
                var index = inArray(this.requireFail, otherRecognizer);
                if (index > -1) {
                    this.requireFail.splice(index, 1);
                }
                return this;
            },
            /**
             * has require failures boolean
             * @returns {boolean}
             */
            hasRequireFailures: function () {
                return this.requireFail.length > 0;
            },
            /**
             * if the recognizer can recognize simultaneous with an other recognizer
             * @param {Recognizer} otherRecognizer
             * @returns {Boolean}
             */
            canRecognizeWith: function (otherRecognizer) {
                return !!this.simultaneous[otherRecognizer.id];
            },
            /**
             * You should use `tryEmit` instead of `emit` directly to check
             * that all the needed recognizers has failed before emitting.
             * @param {Object} input
             */
            emit: function (input) {
                var self = this;
                var state = this.state;
                function emit(event) {
                    self.manager.emit(event, input);
                }
                // 'panstart' and 'panmove'
                if (state < STATE_ENDED) {
                    emit(self.options.event + stateStr(state));
                }
                emit(self.options.event); // simple 'eventName' events
                if (input.additionalEvent) { // additional event(panleft, panright, pinchin, pinchout...)
                    emit(input.additionalEvent);
                }
                // panend and pancancel
                if (state >= STATE_ENDED) {
                    emit(self.options.event + stateStr(state));
                }
            },
            /**
             * Check that all the require failure recognizers has failed,
             * if true, it emits a gesture event,
             * otherwise, setup the state to FAILED.
             * @param {Object} input
             */
            tryEmit: function (input) {
                if (this.canEmit()) {
                    return this.emit(input);
                }
                // it's failing anyway
                this.state = STATE_FAILED;
            },
            /**
             * can we emit?
             * @returns {boolean}
             */
            canEmit: function () {
                var i = 0;
                while (i < this.requireFail.length) {
                    if (!(this.requireFail[i].state & (STATE_FAILED | STATE_POSSIBLE))) {
                        return false;
                    }
                    i++;
                }
                return true;
            },
            /**
             * update the recognizer
             * @param {Object} inputData
             */
            recognize: function (inputData) {
                // make a new copy of the inputData
                // so we can change the inputData without messing up the other recognizers
                var inputDataClone = assign({}, inputData);
                // is is enabled and allow recognizing?
                if (!boolOrFn(this.options.enable, [this, inputDataClone])) {
                    this.reset();
                    this.state = STATE_FAILED;
                    return;
                }
                // reset when we've reached the end
                if (this.state & (STATE_RECOGNIZED | STATE_CANCELLED | STATE_FAILED)) {
                    this.state = STATE_POSSIBLE;
                }
                this.state = this.process(inputDataClone);
                // the recognizer has recognized a gesture
                // so trigger an event
                if (this.state & (STATE_BEGAN | STATE_CHANGED | STATE_ENDED | STATE_CANCELLED)) {
                    this.tryEmit(inputDataClone);
                }
            },
            /**
             * return the state of the recognizer
             * the actual recognizing happens in this method
             * @virtual
             * @param {Object} inputData
             * @returns {Const} STATE
             */
            process: function (inputData) { },
            /**
             * return the preferred touch-action
             * @virtual
             * @returns {Array}
             */
            getTouchAction: function () { },
            /**
             * called when the gesture isn't allowed to recognize
             * like when another is being recognized or it is disabled
             * @virtual
             */
            reset: function () { }
        };
        /**
         * get a usable string, used as event postfix
         * @param {Const} state
         * @returns {String} state
         */
        function stateStr(state) {
            if (state & STATE_CANCELLED) {
                return 'cancel';
            }
            else if (state & STATE_ENDED) {
                return 'end';
            }
            else if (state & STATE_CHANGED) {
                return 'move';
            }
            else if (state & STATE_BEGAN) {
                return 'start';
            }
            return '';
        }
        /**
         * direction cons to string
         * @param {Const} direction
         * @returns {String}
         */
        function directionStr(direction) {
            if (direction == DIRECTION_DOWN) {
                return 'down';
            }
            else if (direction == DIRECTION_UP) {
                return 'up';
            }
            else if (direction == DIRECTION_LEFT) {
                return 'left';
            }
            else if (direction == DIRECTION_RIGHT) {
                return 'right';
            }
            return '';
        }
        /**
         * get a recognizer by name if it is bound to a manager
         * @param {Recognizer|String} otherRecognizer
         * @param {Recognizer} recognizer
         * @returns {Recognizer}
         */
        function getRecognizerByNameIfManager(otherRecognizer, recognizer) {
            var manager = recognizer.manager;
            if (manager) {
                return manager.get(otherRecognizer);
            }
            return otherRecognizer;
        }
        /**
         * This recognizer is just used as a base for the simple attribute recognizers.
         * @constructor
         * @extends Recognizer
         */
        function AttrRecognizer() {
            Recognizer.apply(this, arguments);
        }
        inherit(AttrRecognizer, Recognizer, {
            /**
             * @namespace
             * @memberof AttrRecognizer
             */
            defaults: {
                /**
                 * @type {Number}
                 * @default 1
                 */
                pointers: 1
            },
            /**
             * Used to check if it the recognizer receives valid input, like input.distance > 10.
             * @memberof AttrRecognizer
             * @param {Object} input
             * @returns {Boolean} recognized
             */
            attrTest: function (input) {
                var optionPointers = this.options.pointers;
                return optionPointers === 0 || input.pointers.length === optionPointers;
            },
            /**
             * Process the input and return the state for the recognizer
             * @memberof AttrRecognizer
             * @param {Object} input
             * @returns {*} State
             */
            process: function (input) {
                var state = this.state;
                var eventType = input.eventType;
                var isRecognized = state & (STATE_BEGAN | STATE_CHANGED);
                var isValid = this.attrTest(input);
                // on cancel input and we've recognized before, return STATE_CANCELLED
                if (isRecognized && (eventType & INPUT_CANCEL || !isValid)) {
                    return state | STATE_CANCELLED;
                }
                else if (isRecognized || isValid) {
                    if (eventType & INPUT_END) {
                        return state | STATE_ENDED;
                    }
                    else if (!(state & STATE_BEGAN)) {
                        return STATE_BEGAN;
                    }
                    return state | STATE_CHANGED;
                }
                return STATE_FAILED;
            }
        });
        /**
         * Pan
         * Recognized when the pointer is down and moved in the allowed direction.
         * @constructor
         * @extends AttrRecognizer
         */
        function PanRecognizer() {
            AttrRecognizer.apply(this, arguments);
            this.pX = null;
            this.pY = null;
        }
        inherit(PanRecognizer, AttrRecognizer, {
            /**
             * @namespace
             * @memberof PanRecognizer
             */
            defaults: {
                event: 'pan',
                threshold: 10,
                pointers: 1,
                direction: DIRECTION_ALL
            },
            getTouchAction: function () {
                var direction = this.options.direction;
                var actions = [];
                if (direction & DIRECTION_HORIZONTAL) {
                    actions.push(TOUCH_ACTION_PAN_Y);
                }
                if (direction & DIRECTION_VERTICAL) {
                    actions.push(TOUCH_ACTION_PAN_X);
                }
                return actions;
            },
            directionTest: function (input) {
                var options = this.options;
                var hasMoved = true;
                var distance = input.distance;
                var direction = input.direction;
                var x = input.deltaX;
                var y = input.deltaY;
                // lock to axis?
                if (!(direction & options.direction)) {
                    if (options.direction & DIRECTION_HORIZONTAL) {
                        direction = (x === 0) ? DIRECTION_NONE : (x < 0) ? DIRECTION_LEFT : DIRECTION_RIGHT;
                        hasMoved = x != this.pX;
                        distance = Math.abs(input.deltaX);
                    }
                    else {
                        direction = (y === 0) ? DIRECTION_NONE : (y < 0) ? DIRECTION_UP : DIRECTION_DOWN;
                        hasMoved = y != this.pY;
                        distance = Math.abs(input.deltaY);
                    }
                }
                input.direction = direction;
                return hasMoved && distance > options.threshold && direction & options.direction;
            },
            attrTest: function (input) {
                return AttrRecognizer.prototype.attrTest.call(this, input) &&
                    (this.state & STATE_BEGAN || (!(this.state & STATE_BEGAN) && this.directionTest(input)));
            },
            emit: function (input) {
                this.pX = input.deltaX;
                this.pY = input.deltaY;
                var direction = directionStr(input.direction);
                if (direction) {
                    input.additionalEvent = this.options.event + direction;
                }
                this._super.emit.call(this, input);
            }
        });
        /**
         * Pinch
         * Recognized when two or more pointers are moving toward (zoom-in) or away from each other (zoom-out).
         * @constructor
         * @extends AttrRecognizer
         */
        function PinchRecognizer() {
            AttrRecognizer.apply(this, arguments);
        }
        inherit(PinchRecognizer, AttrRecognizer, {
            /**
             * @namespace
             * @memberof PinchRecognizer
             */
            defaults: {
                event: 'pinch',
                threshold: 0,
                pointers: 2
            },
            getTouchAction: function () {
                return [TOUCH_ACTION_NONE];
            },
            attrTest: function (input) {
                return this._super.attrTest.call(this, input) &&
                    (Math.abs(input.scale - 1) > this.options.threshold || this.state & STATE_BEGAN);
            },
            emit: function (input) {
                if (input.scale !== 1) {
                    var inOut = input.scale < 1 ? 'in' : 'out';
                    input.additionalEvent = this.options.event + inOut;
                }
                this._super.emit.call(this, input);
            }
        });
        /**
         * Press
         * Recognized when the pointer is down for x ms without any movement.
         * @constructor
         * @extends Recognizer
         */
        function PressRecognizer() {
            Recognizer.apply(this, arguments);
            this._timer = null;
            this._input = null;
        }
        inherit(PressRecognizer, Recognizer, {
            /**
             * @namespace
             * @memberof PressRecognizer
             */
            defaults: {
                event: 'press',
                pointers: 1,
                time: 251,
                threshold: 9 // a minimal movement is ok, but keep it low
            },
            getTouchAction: function () {
                return [TOUCH_ACTION_AUTO];
            },
            process: function (input) {
                var options = this.options;
                var validPointers = input.pointers.length === options.pointers;
                var validMovement = input.distance < options.threshold;
                var validTime = input.deltaTime > options.time;
                this._input = input;
                // we only allow little movement
                // and we've reached an end event, so a tap is possible
                if (!validMovement || !validPointers || (input.eventType & (INPUT_END | INPUT_CANCEL) && !validTime)) {
                    this.reset();
                }
                else if (input.eventType & INPUT_START) {
                    this.reset();
                    this._timer = setTimeoutContext(function () {
                        this.state = STATE_RECOGNIZED;
                        this.tryEmit();
                    }, options.time, this);
                }
                else if (input.eventType & INPUT_END) {
                    return STATE_RECOGNIZED;
                }
                return STATE_FAILED;
            },
            reset: function () {
                clearTimeout(this._timer);
            },
            emit: function (input) {
                if (this.state !== STATE_RECOGNIZED) {
                    return;
                }
                if (input && (input.eventType & INPUT_END)) {
                    this.manager.emit(this.options.event + 'up', input);
                }
                else {
                    this._input.timeStamp = now();
                    this.manager.emit(this.options.event, this._input);
                }
            }
        });
        /**
         * Rotate
         * Recognized when two or more pointer are moving in a circular motion.
         * @constructor
         * @extends AttrRecognizer
         */
        function RotateRecognizer() {
            AttrRecognizer.apply(this, arguments);
        }
        inherit(RotateRecognizer, AttrRecognizer, {
            /**
             * @namespace
             * @memberof RotateRecognizer
             */
            defaults: {
                event: 'rotate',
                threshold: 0,
                pointers: 2
            },
            getTouchAction: function () {
                return [TOUCH_ACTION_NONE];
            },
            attrTest: function (input) {
                return this._super.attrTest.call(this, input) &&
                    (Math.abs(input.rotation) > this.options.threshold || this.state & STATE_BEGAN);
            }
        });
        /**
         * Swipe
         * Recognized when the pointer is moving fast (velocity), with enough distance in the allowed direction.
         * @constructor
         * @extends AttrRecognizer
         */
        function SwipeRecognizer() {
            AttrRecognizer.apply(this, arguments);
        }
        inherit(SwipeRecognizer, AttrRecognizer, {
            /**
             * @namespace
             * @memberof SwipeRecognizer
             */
            defaults: {
                event: 'swipe',
                threshold: 10,
                velocity: 0.3,
                direction: DIRECTION_HORIZONTAL | DIRECTION_VERTICAL,
                pointers: 1
            },
            getTouchAction: function () {
                return PanRecognizer.prototype.getTouchAction.call(this);
            },
            attrTest: function (input) {
                var direction = this.options.direction;
                var velocity;
                if (direction & (DIRECTION_HORIZONTAL | DIRECTION_VERTICAL)) {
                    velocity = input.overallVelocity;
                }
                else if (direction & DIRECTION_HORIZONTAL) {
                    velocity = input.overallVelocityX;
                }
                else if (direction & DIRECTION_VERTICAL) {
                    velocity = input.overallVelocityY;
                }
                return this._super.attrTest.call(this, input) &&
                    direction & input.offsetDirection &&
                    input.distance > this.options.threshold &&
                    input.maxPointers == this.options.pointers &&
                    abs(velocity) > this.options.velocity && input.eventType & INPUT_END;
            },
            emit: function (input) {
                var direction = directionStr(input.offsetDirection);
                if (direction) {
                    this.manager.emit(this.options.event + direction, input);
                }
                this.manager.emit(this.options.event, input);
            }
        });
        /**
         * A tap is ecognized when the pointer is doing a small tap/click. Multiple taps are recognized if they occur
         * between the given interval and position. The delay option can be used to recognize multi-taps without firing
         * a single tap.
         *
         * The eventData from the emitted event contains the property `tapCount`, which contains the amount of
         * multi-taps being recognized.
         * @constructor
         * @extends Recognizer
         */
        function TapRecognizer() {
            Recognizer.apply(this, arguments);
            // previous time and center,
            // used for tap counting
            this.pTime = false;
            this.pCenter = false;
            this._timer = null;
            this._input = null;
            this.count = 0;
        }
        inherit(TapRecognizer, Recognizer, {
            /**
             * @namespace
             * @memberof PinchRecognizer
             */
            defaults: {
                event: 'tap',
                pointers: 1,
                taps: 1,
                interval: 300,
                time: 250,
                threshold: 9,
                posThreshold: 10 // a multi-tap can be a bit off the initial position
            },
            getTouchAction: function () {
                return [TOUCH_ACTION_MANIPULATION];
            },
            process: function (input) {
                var options = this.options;
                var validPointers = input.pointers.length === options.pointers;
                var validMovement = input.distance < options.threshold;
                var validTouchTime = input.deltaTime < options.time;
                this.reset();
                if ((input.eventType & INPUT_START) && (this.count === 0)) {
                    return this.failTimeout();
                }
                // we only allow little movement
                // and we've reached an end event, so a tap is possible
                if (validMovement && validTouchTime && validPointers) {
                    if (input.eventType != INPUT_END) {
                        return this.failTimeout();
                    }
                    var validInterval = this.pTime ? (input.timeStamp - this.pTime < options.interval) : true;
                    var validMultiTap = !this.pCenter || getDistance(this.pCenter, input.center) < options.posThreshold;
                    this.pTime = input.timeStamp;
                    this.pCenter = input.center;
                    if (!validMultiTap || !validInterval) {
                        this.count = 1;
                    }
                    else {
                        this.count += 1;
                    }
                    this._input = input;
                    // if tap count matches we have recognized it,
                    // else it has began recognizing...
                    var tapCount = this.count % options.taps;
                    if (tapCount === 0) {
                        // no failing requirements, immediately trigger the tap event
                        // or wait as long as the multitap interval to trigger
                        if (!this.hasRequireFailures()) {
                            return STATE_RECOGNIZED;
                        }
                        else {
                            this._timer = setTimeoutContext(function () {
                                this.state = STATE_RECOGNIZED;
                                this.tryEmit();
                            }, options.interval, this);
                            return STATE_BEGAN;
                        }
                    }
                }
                return STATE_FAILED;
            },
            failTimeout: function () {
                this._timer = setTimeoutContext(function () {
                    this.state = STATE_FAILED;
                }, this.options.interval, this);
                return STATE_FAILED;
            },
            reset: function () {
                clearTimeout(this._timer);
            },
            emit: function () {
                if (this.state == STATE_RECOGNIZED) {
                    this._input.tapCount = this.count;
                    this.manager.emit(this.options.event, this._input);
                }
            }
        });
        /**
         * Simple way to create a manager with a default set of recognizers.
         * @param {HTMLElement} element
         * @param {Object} [options]
         * @constructor
         */
        function Hammer(element, options) {
            options = options || {};
            options.recognizers = ifUndefined(options.recognizers, Hammer.defaults.preset);
            return new Manager(element, options);
        }
        /**
         * @const {string}
         */
        Hammer.VERSION = '2.0.7';
        /**
         * default settings
         * @namespace
         */
        Hammer.defaults = {
            /**
             * set if DOM events are being triggered.
             * But this is slower and unused by simple implementations, so disabled by default.
             * @type {Boolean}
             * @default false
             */
            domEvents: false,
            /**
             * The value for the touchAction property/fallback.
             * When set to `compute` it will magically set the correct value based on the added recognizers.
             * @type {String}
             * @default compute
             */
            touchAction: TOUCH_ACTION_COMPUTE,
            /**
             * @type {Boolean}
             * @default true
             */
            enable: true,
            /**
             * EXPERIMENTAL FEATURE -- can be removed/changed
             * Change the parent input target element.
             * If Null, then it is being set the to main element.
             * @type {Null|EventTarget}
             * @default null
             */
            inputTarget: null,
            /**
             * force an input class
             * @type {Null|Function}
             * @default null
             */
            inputClass: null,
            /**
             * Default recognizer setup when calling `Hammer()`
             * When creating a new Manager these will be skipped.
             * @type {Array}
             */
            preset: [
                // RecognizerClass, options, [recognizeWith, ...], [requireFailure, ...]
                [RotateRecognizer, { enable: false }],
                [PinchRecognizer, { enable: false }, ['rotate']],
                [SwipeRecognizer, { direction: DIRECTION_HORIZONTAL }],
                [PanRecognizer, { direction: DIRECTION_HORIZONTAL }, ['swipe']],
                [TapRecognizer],
                [TapRecognizer, { event: 'doubletap', taps: 2 }, ['tap']],
                [PressRecognizer]
            ],
            /**
             * Some CSS properties can be used to improve the working of Hammer.
             * Add them to this method and they will be set when creating a new Manager.
             * @namespace
             */
            cssProps: {
                /**
                 * Disables text selection to improve the dragging gesture. Mainly for desktop browsers.
                 * @type {String}
                 * @default 'none'
                 */
                userSelect: 'none',
                /**
                 * Disable the Windows Phone grippers when pressing an element.
                 * @type {String}
                 * @default 'none'
                 */
                touchSelect: 'none',
                /**
                 * Disables the default callout shown when you touch and hold a touch target.
                 * On iOS, when you touch and hold a touch target such as a link, Safari displays
                 * a callout containing information about the link. This property allows you to disable that callout.
                 * @type {String}
                 * @default 'none'
                 */
                touchCallout: 'none',
                /**
                 * Specifies whether zooming is enabled. Used by IE10>
                 * @type {String}
                 * @default 'none'
                 */
                contentZooming: 'none',
                /**
                 * Specifies that an entire element should be draggable instead of its contents. Mainly for desktop browsers.
                 * @type {String}
                 * @default 'none'
                 */
                userDrag: 'none',
                /**
                 * Overrides the highlight color shown when the user taps a link or a JavaScript
                 * clickable element in iOS. This property obeys the alpha value, if specified.
                 * @type {String}
                 * @default 'rgba(0,0,0,0)'
                 */
                tapHighlightColor: 'rgba(0,0,0,0)'
            }
        };
        var STOP = 1;
        var FORCED_STOP = 2;
        /**
         * Manager
         * @param {HTMLElement} element
         * @param {Object} [options]
         * @constructor
         */
        function Manager(element, options) {
            this.options = assign({}, Hammer.defaults, options || {});
            this.options.inputTarget = this.options.inputTarget || element;
            this.handlers = {};
            this.session = {};
            this.recognizers = [];
            this.oldCssProps = {};
            this.element = element;
            this.input = createInputInstance(this);
            this.touchAction = new TouchAction(this, this.options.touchAction);
            toggleCssProps(this, true);
            each(this.options.recognizers, function (item) {
                var recognizer = this.add(new (item[0])(item[1]));
                item[2] && recognizer.recognizeWith(item[2]);
                item[3] && recognizer.requireFailure(item[3]);
            }, this);
        }
        Manager.prototype = {
            /**
             * set options
             * @param {Object} options
             * @returns {Manager}
             */
            set: function (options) {
                assign(this.options, options);
                // Options that need a little more setup
                if (options.touchAction) {
                    this.touchAction.update();
                }
                if (options.inputTarget) {
                    // Clean up existing event listeners and reinitialize
                    this.input.destroy();
                    this.input.target = options.inputTarget;
                    this.input.init();
                }
                return this;
            },
            /**
             * stop recognizing for this session.
             * This session will be discarded, when a new [input]start event is fired.
             * When forced, the recognizer cycle is stopped immediately.
             * @param {Boolean} [force]
             */
            stop: function (force) {
                this.session.stopped = force ? FORCED_STOP : STOP;
            },
            /**
             * run the recognizers!
             * called by the inputHandler function on every movement of the pointers (touches)
             * it walks through all the recognizers and tries to detect the gesture that is being made
             * @param {Object} inputData
             */
            recognize: function (inputData) {
                var session = this.session;
                if (session.stopped) {
                    return;
                }
                // run the touch-action polyfill
                this.touchAction.preventDefaults(inputData);
                var recognizer;
                var recognizers = this.recognizers;
                // this holds the recognizer that is being recognized.
                // so the recognizer's state needs to be BEGAN, CHANGED, ENDED or RECOGNIZED
                // if no recognizer is detecting a thing, it is set to `null`
                var curRecognizer = session.curRecognizer;
                // reset when the last recognizer is recognized
                // or when we're in a new session
                if (!curRecognizer || (curRecognizer && curRecognizer.state & STATE_RECOGNIZED)) {
                    curRecognizer = session.curRecognizer = null;
                }
                var i = 0;
                while (i < recognizers.length) {
                    recognizer = recognizers[i];
                    // find out if we are allowed try to recognize the input for this one.
                    // 1.   allow if the session is NOT forced stopped (see the .stop() method)
                    // 2.   allow if we still haven't recognized a gesture in this session, or the this recognizer is the one
                    //      that is being recognized.
                    // 3.   allow if the recognizer is allowed to run simultaneous with the current recognized recognizer.
                    //      this can be setup with the `recognizeWith()` method on the recognizer.
                    if (session.stopped !== FORCED_STOP && ( // 1
                    !curRecognizer || recognizer == curRecognizer || // 2
                        recognizer.canRecognizeWith(curRecognizer))) { // 3
                        recognizer.recognize(inputData);
                    }
                    else {
                        recognizer.reset();
                    }
                    // if the recognizer has been recognizing the input as a valid gesture, we want to store this one as the
                    // current active recognizer. but only if we don't already have an active recognizer
                    if (!curRecognizer && recognizer.state & (STATE_BEGAN | STATE_CHANGED | STATE_ENDED)) {
                        curRecognizer = session.curRecognizer = recognizer;
                    }
                    i++;
                }
            },
            /**
             * get a recognizer by its event name.
             * @param {Recognizer|String} recognizer
             * @returns {Recognizer|Null}
             */
            get: function (recognizer) {
                if (recognizer instanceof Recognizer) {
                    return recognizer;
                }
                var recognizers = this.recognizers;
                for (var i = 0; i < recognizers.length; i++) {
                    if (recognizers[i].options.event == recognizer) {
                        return recognizers[i];
                    }
                }
                return null;
            },
            /**
             * add a recognizer to the manager
             * existing recognizers with the same event name will be removed
             * @param {Recognizer} recognizer
             * @returns {Recognizer|Manager}
             */
            add: function (recognizer) {
                if (invokeArrayArg(recognizer, 'add', this)) {
                    return this;
                }
                // remove existing
                var existing = this.get(recognizer.options.event);
                if (existing) {
                    this.remove(existing);
                }
                this.recognizers.push(recognizer);
                recognizer.manager = this;
                this.touchAction.update();
                return recognizer;
            },
            /**
             * remove a recognizer by name or instance
             * @param {Recognizer|String} recognizer
             * @returns {Manager}
             */
            remove: function (recognizer) {
                if (invokeArrayArg(recognizer, 'remove', this)) {
                    return this;
                }
                recognizer = this.get(recognizer);
                // let's make sure this recognizer exists
                if (recognizer) {
                    var recognizers = this.recognizers;
                    var index = inArray(recognizers, recognizer);
                    if (index !== -1) {
                        recognizers.splice(index, 1);
                        this.touchAction.update();
                    }
                }
                return this;
            },
            /**
             * bind event
             * @param {String} events
             * @param {Function} handler
             * @returns {EventEmitter} this
             */
            on: function (events, handler) {
                if (events === undefined) {
                    return;
                }
                if (handler === undefined) {
                    return;
                }
                var handlers = this.handlers;
                each(splitStr(events), function (event) {
                    handlers[event] = handlers[event] || [];
                    handlers[event].push(handler);
                });
                return this;
            },
            /**
             * unbind event, leave emit blank to remove all handlers
             * @param {String} events
             * @param {Function} [handler]
             * @returns {EventEmitter} this
             */
            off: function (events, handler) {
                if (events === undefined) {
                    return;
                }
                var handlers = this.handlers;
                each(splitStr(events), function (event) {
                    if (!handler) {
                        delete handlers[event];
                    }
                    else {
                        handlers[event] && handlers[event].splice(inArray(handlers[event], handler), 1);
                    }
                });
                return this;
            },
            /**
             * emit event to the listeners
             * @param {String} event
             * @param {Object} data
             */
            emit: function (event, data) {
                // we also want to trigger dom events
                if (this.options.domEvents) {
                    triggerDomEvent(event, data);
                }
                // no handlers, so skip it all
                var handlers = this.handlers[event] && this.handlers[event].slice();
                if (!handlers || !handlers.length) {
                    return;
                }
                data.type = event;
                data.preventDefault = function () {
                    data.srcEvent.preventDefault();
                };
                var i = 0;
                while (i < handlers.length) {
                    handlers[i](data);
                    i++;
                }
            },
            /**
             * destroy the manager and unbinds all events
             * it doesn't unbind dom events, that is the user own responsibility
             */
            destroy: function () {
                this.element && toggleCssProps(this, false);
                this.handlers = {};
                this.session = {};
                this.input.destroy();
                this.element = null;
            }
        };
        /**
         * add/remove the css properties as defined in manager.options.cssProps
         * @param {Manager} manager
         * @param {Boolean} add
         */
        function toggleCssProps(manager, add) {
            var element = manager.element;
            if (!element.style) {
                return;
            }
            var prop;
            each(manager.options.cssProps, function (value, name) {
                prop = prefixed(element.style, name);
                if (add) {
                    manager.oldCssProps[prop] = element.style[prop];
                    element.style[prop] = value;
                }
                else {
                    element.style[prop] = manager.oldCssProps[prop] || '';
                }
            });
            if (!add) {
                manager.oldCssProps = {};
            }
        }
        /**
         * trigger dom event
         * @param {String} event
         * @param {Object} data
         */
        function triggerDomEvent(event, data) {
            var gestureEvent = document.createEvent('Event');
            gestureEvent.initEvent(event, true, true);
            gestureEvent.gesture = data;
            data.target.dispatchEvent(gestureEvent);
        }
        assign(Hammer, {
            INPUT_START: INPUT_START,
            INPUT_MOVE: INPUT_MOVE,
            INPUT_END: INPUT_END,
            INPUT_CANCEL: INPUT_CANCEL,
            STATE_POSSIBLE: STATE_POSSIBLE,
            STATE_BEGAN: STATE_BEGAN,
            STATE_CHANGED: STATE_CHANGED,
            STATE_ENDED: STATE_ENDED,
            STATE_RECOGNIZED: STATE_RECOGNIZED,
            STATE_CANCELLED: STATE_CANCELLED,
            STATE_FAILED: STATE_FAILED,
            DIRECTION_NONE: DIRECTION_NONE,
            DIRECTION_LEFT: DIRECTION_LEFT,
            DIRECTION_RIGHT: DIRECTION_RIGHT,
            DIRECTION_UP: DIRECTION_UP,
            DIRECTION_DOWN: DIRECTION_DOWN,
            DIRECTION_HORIZONTAL: DIRECTION_HORIZONTAL,
            DIRECTION_VERTICAL: DIRECTION_VERTICAL,
            DIRECTION_ALL: DIRECTION_ALL,
            Manager: Manager,
            Input: Input,
            TouchAction: TouchAction,
            TouchInput: TouchInput,
            MouseInput: MouseInput,
            PointerEventInput: PointerEventInput,
            TouchMouseInput: TouchMouseInput,
            SingleTouchInput: SingleTouchInput,
            Recognizer: Recognizer,
            AttrRecognizer: AttrRecognizer,
            Tap: TapRecognizer,
            Pan: PanRecognizer,
            Swipe: SwipeRecognizer,
            Pinch: PinchRecognizer,
            Rotate: RotateRecognizer,
            Press: PressRecognizer,
            on: addEventListeners,
            off: removeEventListeners,
            each: each,
            merge: merge,
            extend: extend,
            assign: assign,
            inherit: inherit,
            bindFn: bindFn,
            prefixed: prefixed
        });
        // this prevents errors when Hammer is loaded in the presence of an AMD
        //  style loader but by script tag, not by the loader.
        var freeGlobal = (typeof window !== 'undefined' ? window : (typeof self !== 'undefined' ? self : {})); // jshint ignore:line
        freeGlobal.Hammer = Hammer;
        if (typeof define === 'function' && define.amd) {
            define(function () {
                return Hammer;
            });
        }
        else if (typeof module != 'undefined' && module.exports) {
            module.exports = Hammer;
        }
        else {
            window[exportName] = Hammer;
        }
    })(window, document, 'Hammer');
},
/* models/tools/tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var view_1 = require(186) /* ../../core/view */;
    var array_1 = require(124) /* ../../core/util/array */;
    var model_1 = require(196) /* ../../model */;
    var ToolView = /** @class */ (function (_super) {
        tslib_1.__extends(ToolView, _super);
        function ToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Object.defineProperty(ToolView.prototype, "plot_view", {
            get: function () {
                return this.parent;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ToolView.prototype, "plot_model", {
            get: function () {
                return this.parent.model;
            },
            enumerable: true,
            configurable: true
        });
        ToolView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.properties.active.change, function () {
                if (_this.model.active)
                    _this.activate();
                else
                    _this.deactivate();
            });
        };
        // activate is triggered by toolbar ui actions
        ToolView.prototype.activate = function () { };
        // deactivate is triggered by toolbar ui actions
        ToolView.prototype.deactivate = function () { };
        return ToolView;
    }(view_1.View));
    exports.ToolView = ToolView;
    ToolView.__name__ = "ToolView";
    var Tool = /** @class */ (function (_super) {
        tslib_1.__extends(Tool, _super);
        function Tool(attrs) {
            return _super.call(this, attrs) || this;
        }
        Tool.init_Tool = function () {
            this.prototype._known_aliases = new Map();
            this.internal({
                active: [p.Boolean, false],
            });
        };
        Object.defineProperty(Tool.prototype, "synthetic_renderers", {
            get: function () {
                return [];
            },
            enumerable: true,
            configurable: true
        });
        // utility function to return a tool name, modified
        // by the active dimensions. Used by tools that have dimensions
        Tool.prototype._get_dim_tooltip = function (name, dims) {
            switch (dims) {
                case "width": return name + " (x-axis)";
                case "height": return name + " (y-axis)";
                case "both": return name;
            }
        };
        // utility function to get limits along both dimensions, given
        // optional dimensional constraints
        Tool.prototype._get_dim_limits = function (_a, _b, frame, dims) {
            var _c = tslib_1.__read(_a, 2), sx0 = _c[0], sy0 = _c[1];
            var _d = tslib_1.__read(_b, 2), sx1 = _d[0], sy1 = _d[1];
            var hr = frame.bbox.h_range;
            var sxlim;
            if (dims == 'width' || dims == 'both') {
                sxlim = [array_1.min([sx0, sx1]), array_1.max([sx0, sx1])];
                sxlim = [array_1.max([sxlim[0], hr.start]), array_1.min([sxlim[1], hr.end])];
            }
            else
                sxlim = [hr.start, hr.end];
            var vr = frame.bbox.v_range;
            var sylim;
            if (dims == 'height' || dims == 'both') {
                sylim = [array_1.min([sy0, sy1]), array_1.max([sy0, sy1])];
                sylim = [array_1.max([sylim[0], vr.start]), array_1.min([sylim[1], vr.end])];
            }
            else
                sylim = [vr.start, vr.end];
            return [sxlim, sylim];
        };
        Tool.register_alias = function (name, fn) {
            this.prototype._known_aliases.set(name, fn);
        };
        Tool.from_string = function (name) {
            var fn = this.prototype._known_aliases.get(name);
            if (fn != null)
                return fn();
            else {
                var names = tslib_1.__spread(this.prototype._known_aliases.keys());
                throw new Error("unexpected tool name '" + name + "', possible tools are " + names.join(", "));
            }
        };
        return Tool;
    }(model_1.Model));
    exports.Tool = Tool;
    Tool.__name__ = "Tool";
    Tool.init_Tool();
},
/* styles/toolbar.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.bk_toolbar = "bk-toolbar";
    exports.bk_toolbar_hidden = "bk-toolbar-hidden";
    exports.bk_toolbar_button = "bk-toolbar-button";
    exports.bk_button_bar = "bk-button-bar";
    exports.bk_toolbar_button_custom_action = "bk-toolbar-button-custom-action";
},
/* styles/toolbar.css.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var css = "\n.bk-root .bk-toolbar-hidden {\n  visibility: hidden;\n  opacity: 0;\n  transition: visibility 0.3s linear, opacity 0.3s linear;\n}\n.bk-root .bk-toolbar,\n.bk-root .bk-button-bar {\n  display: flex;\n  display: -webkit-flex;\n  flex-wrap: nowrap;\n  -webkit-flex-wrap: nowrap;\n  align-items: center;\n  -webkit-align-items: center;\n  user-select: none;\n  -ms-user-select: none;\n  -moz-user-select: none;\n  -webkit-user-select: none;\n}\n.bk-root .bk-toolbar .bk-logo {\n  flex-shrink: 0;\n  -webkit-flex-shrink: 0;\n}\n.bk-root .bk-toolbar.bk-above,\n.bk-root .bk-toolbar.bk-below {\n  flex-direction: row;\n  -webkit-flex-direction: row;\n  justify-content: flex-end;\n  -webkit-justify-content: flex-end;\n}\n.bk-root .bk-toolbar.bk-above .bk-button-bar,\n.bk-root .bk-toolbar.bk-below .bk-button-bar {\n  display: flex;\n  display: -webkit-flex;\n  flex-direction: row;\n  -webkit-flex-direction: row;\n}\n.bk-root .bk-toolbar.bk-above .bk-logo,\n.bk-root .bk-toolbar.bk-below .bk-logo {\n  order: 1;\n  -webkit-order: 1;\n  margin-left: 5px;\n  margin-right: 0px;\n}\n.bk-root .bk-toolbar.bk-left,\n.bk-root .bk-toolbar.bk-right {\n  flex-direction: column;\n  -webkit-flex-direction: column;\n  justify-content: flex-start;\n  -webkit-justify-content: flex-start;\n}\n.bk-root .bk-toolbar.bk-left .bk-button-bar,\n.bk-root .bk-toolbar.bk-right .bk-button-bar {\n  display: flex;\n  display: -webkit-flex;\n  flex-direction: column;\n  -webkit-flex-direction: column;\n}\n.bk-root .bk-toolbar.bk-left .bk-logo,\n.bk-root .bk-toolbar.bk-right .bk-logo {\n  order: 0;\n  -webkit-order: 0;\n  margin-bottom: 5px;\n  margin-top: 0px;\n}\n.bk-root .bk-toolbar-button {\n  width: 30px;\n  height: 30px;\n  cursor: pointer;\n  background-size: 60% 60%;\n  background-origin: border-box;\n  background-color: transparent;\n  background-repeat: no-repeat;\n  background-position: center center;\n}\n.bk-root .bk-toolbar-button:hover {\n  background-color: rgba(192, 192, 192, 0.15);\n}\n.bk-root .bk-toolbar-button:focus {\n  outline: none;\n}\n.bk-root .bk-toolbar-button::-moz-focus-inner {\n  border: 0;\n}\n.bk-root .bk-toolbar.bk-above .bk-toolbar-button {\n  border-bottom: 2px solid transparent;\n}\n.bk-root .bk-toolbar.bk-above .bk-toolbar-button.bk-active {\n  border-bottom-color: #26aae1;\n}\n.bk-root .bk-toolbar.bk-below .bk-toolbar-button {\n  border-top: 2px solid transparent;\n}\n.bk-root .bk-toolbar.bk-below .bk-toolbar-button.bk-active {\n  border-top-color: #26aae1;\n}\n.bk-root .bk-toolbar.bk-right .bk-toolbar-button {\n  border-left: 2px solid transparent;\n}\n.bk-root .bk-toolbar.bk-right .bk-toolbar-button.bk-active {\n  border-left-color: #26aae1;\n}\n.bk-root .bk-toolbar.bk-left .bk-toolbar-button {\n  border-right: 2px solid transparent;\n}\n.bk-root .bk-toolbar.bk-left .bk-toolbar-button.bk-active {\n  border-right-color: #26aae1;\n}\n.bk-root .bk-button-bar + .bk-button-bar:before {\n  content: \" \";\n  display: inline-block;\n  background-color: lightgray;\n}\n.bk-root .bk-toolbar.bk-above .bk-button-bar + .bk-button-bar:before,\n.bk-root .bk-toolbar.bk-below .bk-button-bar + .bk-button-bar:before {\n  height: 10px;\n  width: 1px;\n}\n.bk-root .bk-toolbar.bk-left .bk-button-bar + .bk-button-bar:before,\n.bk-root .bk-toolbar.bk-right .bk-button-bar + .bk-button-bar:before {\n  height: 1px;\n  width: 10px;\n}\n";
    exports.default = css;
},
/* styles/icons.css.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var css = "\n.bk-root .bk-tool-icon-copy-to-clipboard {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH5AUSDBoBvcHQeQAAAG9JREFUWMNjXLhsJcNAAiaGAQYwB/xHwh/Q+ITEkfHQCwEWND4jmeb8H/JpgBwfI6cNBhLSEkqaGXRpgFRAcZoZsmlg1AGjDhh1wKgDRh0w6gCaVcf/R2wIkNqw+D9s0wADvUNiyIYA47BJAwPuAAAj/Cjd0TCN6wAAAABJRU5ErkJggg==\");\n}\n.bk-root .bk-tool-icon-replace-mode {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH5AUFFxokK3gniQAAAHpJREFUWMNjXLhsJcNAAiaGAQajDhhwB7DgEP+PxmeksvjgDwFcLmYkUh2hkBj8IcBIZXsYh1w2/I8v3sgAOM0bLYhGc8GgrwuICgldfQO88pcvXvg/aOuCUQeM5oLRuoCFCJcTbOMh5XOiW0JDNhdQS3y0IBp1ABwAAF8KGrhC1Eg6AAAAAElFTkSuQmCC\");\n}\n.bk-root .bk-tool-icon-append-mode {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH5AUFFxkZWD04WwAAAB1pVFh0Q29tbWVudAAAAAAAQ3JlYXRlZCB3aXRoIEdJTVBkLmUHAAAAoUlEQVRYw+1WQQ6AIAwrhO8Y/bIXEz9jIMSDr8ETCUEPQzA4pMeFLKNbu4l5WR0CDOMEALBGIzMuQIBEZQjPgP9JLjwTfBjY9sO9lZsFA9IafZng3BlIyVefgd8XQFZBAWe8jfNxwsDhir6rzoCiPiy1K+J8/FRQemv2XfAdFcQ9znU4Viqg9ta1qYJ+D1BnAIBrkgGVOrXNqUA9rbyZm/AEzFh4jEeY/soAAAAASUVORK5CYII=\");\n}\n.bk-root .bk-tool-icon-intersect-mode {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH5AUFFxkrkOpp2wAAAPhJREFUWMPtV1EKwjAMTUavI3oawR/vtn5srJdREfzwMvHHQlcT2mpdMzFfWxiP5r2+JMN+mAiCOB72CABgR1cln4oOGocJnuMTSxWk8jMm7OggYkYXA9gPE3uyd8NXHONJ+eYMdE/NqCJmEZ5ZqlJJ4sUksKN7cYSaPoCZFWR1QI+Xm1fBACU63Cw22x0AAJxudwrffVwvZ+JmQdAHZkw0d4EpAMCw8k87pMdbnwtizQumJYv3nwV6XOA1qbUT/oQLUJgFRbsiNwFVucBIlyR3p0tdMp+XmFjfLKi1LatyAXtCRjPWBdL3Ke3VuACJKFfDr/xFN2fgAR/Go0qaLlmEAAAAAElFTkSuQmCC\");\n}\n.bk-root .bk-tool-icon-subtract-mode {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH5AUFFxgsF5XNOQAAAB1pVFh0Q29tbWVudAAAAAAAQ3JlYXRlZCB3aXRoIEdJTVBkLmUHAAABFUlEQVRYw9VWUQqDMAxNpWfxQxD1MoP97G7zQ5mH2RTZYLtM9lWoMbXtxLXNX4OG9r28l4hrd0PQoqxqAACYpxH25C/nkwCHyCBwSPoS09k1T5Fo+4EiExcC4v584xGFmyIXHBLRISAVZyZufUPVa4rcrwmPDgr93ylo+2GliLRUYHK6th/o/6r7nfLpqaCsagEA8Hh9FmcNKeRmgeYDC+SCq0B6FFi8/BcV6BdR9cL3gCv3ijPKOacsn3rBEcjmaVxpfGcg4wHxzgJJnc6241Hn23DERFRAu1bNcWa3Q0uXi62XR6sCaWoSejbtdLYmU3kTEunNgj0bUbQqYG/IcMaqwPS9jftoVCAQ0ZVDJwf0zQdH4AsyW6fpQu4YegAAAABJRU5ErkJggg==\");\n}\n.bk-root .bk-tool-icon-clear-selection {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH5AUGEhcuan3d3wAAAoRJREFUWMPtlzFP3EAQhd+b3TNSzg0N5TWXLkJQUUaKhIQ4fgP/g5ArrriE/I3opEgRrZtIVJR0FJQ010SioUmEZHtnUpwN9gWHGA5BJCy58MraffvmfZ41v3z9hqe8BE98vQh4cgG+Ydzmnrng8efvQJNi/uN7dznx/B3ggtfhf4ehNdUttRzBDIm/2VTiiWCG1HK0nc+3UWtq8BQIiEEakEQOADBIA4QCQmBqoHBhFNR27ikQSmGdYCdTqCpEHMDZmEKRWUBEv1gBDg5SzRJnpopILWICgWuRYflLamuzxB2BmtYqSRIka5VWU8QduXO+1hRc5YZu5GAwmP2ZJzND0IBu5HCV2+NQcAhAVRsnC2IbPzPdSjzd6to6VtfWkXi6YLaVWr7xoAwkfpb8MnC3SH7rKSMBe4M0jA/OTicFIbtCGRIyNbURhcf3ErCd6YwA1m0HgAxhw1NGQnlXBHG4kylVlSJuH0RfIP2CkL2I/qS1gIAAQiBl1QwFggIHtyxgrxK5PgyfC0JWKoT0HLh8LwoietB4TYKaIl7yeNURxB05UtMxDOcVQlZIrlRKdK6m47gjR/fuBRQihyLArtNeJD50Izcx2Eczu7iFkIug4VM3cpOr3MKDekFED0fWUHv9Zq0kpLnridjhY3XDg7NTN0jDrhO3X7O9Wg7wwyANu4mnayNg3gmbu0tCNoUyBNGv2l4rB9EXynA7082FOxAQLhU6rQVO9T2AvWowFToNCJcPORGxIRcnpjZSKATSU9NxvOQnAPArDSaQoUKnNI4iufkGtD4P3EHIcWZhz4HLceSOyrR3Izf5memPAL2cX3yhAkonysZVaWLBkd9dw1Ivv2a/AYPkK+ty1U1DAAAAAElFTkSuQmCC\");\n}\n.bk-root .bk-tool-icon-box-select {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg0kduFrowAAAIdJREFUWMPtVtEKwCAI9KL//4e9DPZ3+wP3KgOjNZouFYI4C8q7s7DtB1lGIeMoRMRinCLXg/ML3EcFqpjjloOyZxRntxpwQ8HsgHYARKFAtSFrCg3TCdMFCE1BuuALEXJLjC4qENsFVXCESZw38/kWLOkC/K4PcOc/Hj03WkoDT3EaWW9egQul6CUbq90JTwAAAABJRU5ErkJggg==\");\n}\n.bk-root .bk-tool-icon-box-zoom {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg82t254aQAAAkBJREFUWMPN11+E1FEUB/DPTFn2qaeIpcSwr5NlUyJiKWVXWUqvlUh/iE3RY9mUekkPPURtLKNRrFJEeuphGfUUaVliiX1aVjGs6aG7+XX9ZnZ+d2fTl2vmnHvPPfeee/79Sk+may2/UQq/q7Qu+bAJoxjHIKqB/wlfUMcMVqI9bLZ+DGIKwzlzQ2GcxCx2xwvKOUKlaHTiX8bHNspjDONHkOmJBW5jIof/FvPh/06MZOb6cRc7cGn1AKUE5cdzlM/gAr5F/O24H3xkFRfxAbVygvK+cIsspjGWo1zgjeFpxL+BvnLw7laBA4xjIFJwrgu52DoVjKdY4HBEX8dSF3JLYe1fe6UcYCii3xWQjdfuSTnAtoheKCC7GNED5Zx4L4qt61jbTLHA94geKSC7P7ZeShQ0Inoi1IJuEOeORooFXkV0FZNdZs5qvFfKAeqYy7nZ6yg//HG0MBfffh71lFrQDCW2EvEP4mt4okZUDftz9rmGZkotmMxJRtlisy+MTniAWrty3AlXw0hFM2TD89l+oNsoOJXjbIs4EpqNtTCLXbiZ0g+M4mFObj8U3vsNjoZCVcmk60ZwthpepLZkB/AsivWfOJZxtpUQHfWib7KWDwzjeegBZJSdKFiE2qJTFFTwElsi/unQ/awXrU4WGMD7nOJxBY/1EO2iYConq93CHT1GOwucjdqnRyFz+VcHmMNefMY9nNkA3SWUOoXhQviSWQ4huLIRFlirFixnQq/XaKXUgg2xQNGv4V7x/RcW+AXPB3h7H1PaiQAAAABJRU5ErkJggg==\");\n}\n.bk-root .bk-tool-icon-zoom-in {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEgsUBmL8iQAAA2JJREFUWMO9l12IlFUYx3//MzPrLpSjkm5oN4FFIWVEl66IQlFYwtLOzozsjHdGRSCRF0sfBEXRVV0FQuQiLm5CZNBFgRRaRLVFhbJ2EdiN5gbK7toObTPn6eYdPTvNzPvOBz5Xh/ec5/n/n89zXtEHmZqeSXSuXBz/3zfdKvBWJHQrwZuRcP0El+QkbQXeBX6WZEgm6TtJk5lM5o4Lc+cV6qpf4Ga20Tm338zeATItVK9Ker6yvPzp4NDQ3+XieGsCU9MzTYumGbhz7m4ze9/MHgvBgItACrgfGAj2jgAvAYs3wlEujjc13kii8YyZrXXOfWhmo9GnFUlvOOemarVapVqtkslksmb2KjARqL62ecuWN9NxbRInzrldAXhV0uFSIfdew7G/gNLU9MwS8CwSmE3Oz88fcXG5blfpqVRq0Ix8VIAAX0XgrVL7HDCHGcCaWrV60LUBN8Dae58aQIxEqcA592I9M610JL0cpG/U9TIHJNKY3RV5z0R+7Nd4HZ0P1g/2RMBuegLAsRMnb4vT8d5vqKfMzOgtAlADrkmqGywmiMBTwfr3dC9j1Xv/r6Tvg/5/5ejxE6cO7M9faVbQZrYNOFSPmqQvVo9FKexvi5uWX58943aM7DwAfBDY+FbSCxP5sdkGx55GeguzrUEXPaSo2pFkAbiSZQCAzZJOmdkjwd6SpB/M7KykQTPbA2wDhoIzRzcNDx9MJwGNIXdJ0mEzmwbujL7dbma7gd03A7lKfnTOvf74nl0r6bonTUbujRSUCrm2d4L3/kvn3JPe+8+BDW2i9o+kT7z3kxP5sYsA6W47oE64TsR7P9tQL4vA2mh9WdIscKxUyJ0M7aR7acOGzikD65EQLEjaa2ZXzMwDFeB6qZBbbLTRE4EGeSaozNOZgYFf8qP7lmIvs354n0qlHpB0T7B9Ogl4IgJJrmjv/SiQjbrkD+BMUkfSbYATPdckrTOzkciWAXOlQu5cYgLdPEIapud9wMOR9zVJH3ViKx333mtHMJvNuoWFhZ3A+ojMcja77njXBEKwJJfTcqUyCIQ34Mf7nnh0paMnXacFuGoC1mr3AtuDfLzd8Zuyl+rfuGn4HLAD+Az4qZQf+61TAj0Noj8vX6oC35SL43u7teG6rf5+iXppwW7/JUL5D03qaFRvvUe+AAAAAElFTkSuQmCC\");\n}\n.bk-root .bk-tool-icon-zoom-out {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEgsHgty9VwAAA0FJREFUWMO9l09oXFUUxn/fmXlpItppi22k7UJBRSlVkCytSAuKUloIdjKT0El3FXVXdVFKRVAQV7qQohsNwdA0UFvBhYtqUVyIVlRaogtFQVq7qSTVjA3z3nHzBq/jvPmTN/Ss7rv3nvN99/y794kByMzcfE/7picn/jenmwWeRUI3E7wdCRskuCSTdDfwBvCtJEdySV9KOhpF0e0/LF5SqKtBgbv7ZjObcvfXgShD9Zqk5+orKx8Oj4z8NT05kU1gZm6+bdK0Azezu9z9hLs/HoIBvwAF4H5gKFh7B3gBWFY3460kWve4+3oze9fdx9OpVUmvmNlMHMf1RqNBFEUldz8OHAxUX9q6bduryut+Sfvc/Wz62ZD0fK1afjND9y3gGSRwv1GMojstTxUUCoVhdyopEYDzKXjWwZ4FFnEHWBc3Goet00m7lZlZYQixKw0FZnakGZksHUnHgvCN5/KARBH37enpOVg58H13HV0Kxg/kIuD/ngSA2ZMLt3bTSZJkUzNk7k4+D0AM/CGpaXCyBw/sC8Y/qZd2GpZiuL9YLN4Sx/HpoP5/c/exQ1OVq+1yyt13SLoArEsJnMjlgfOffvK3u58Kprab2QezJxfG2iTzUzI70wRPG9jbmpmb95SNB9mpzp7/j2yVdNbdx4K565K+cvfPJQ27+x5gBzAS7Hlvy+jo4WIvoC3kWpcvS3rR3eeAO9K529x9N7C7zX6AC2b28hN7Hl1Vt44niVq13LUjmtlYkiQfA5s6eO+GpDNJkhw9NFX5ueNt2ARodyF1IHIN2JiOl4H16fiKpK+B2Vq1vBAqFAf4IJkGNiIhWJK0192vunsC1IE/a9XycquNXARa5OnApeeioaHvKuP7r3dTGsiLqFAo7JR0T7B8rhfwXARa2us4UEqr5Ffgs151i/08oTNKdIO770ptObBYq5Yv5ibQq/sl3Qc8lJ4+lnSqH1vFfp9koZRKJVtaWnqkWXqSVkqlDe+vmUDWpZMlK/X6MBDegKf3P/nYaj8ErN9fqZBYEsf3Ag8G8Xit33BaniTcvGX0IvAw8BHwTa1y4Md+CeRqRL9fudwAvpienNi7Vhu21uwflOT+L+i1X2TJP57iUvUFtHWsAAAAAElFTkSuQmCC\");\n}\n.bk-root .bk-tool-icon-help {\n  background-image: url(\"data:image/png;base64,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\");\n}\n.bk-root .bk-tool-icon-hover {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg4oVHp0SwAAAQJJREFUWMPtlsENgzAMRb8RQ5VJItFDOgaZAMaAA0iZpN3KPZSoEEHSQBCViI/G8pfNt/KAFFcPshPdoAGgZkYVVYjQAFCyFLN8tlAbXRwAxp61nc9XCkGERpZCxRDvBl0zoxp7K98GAACxxH29srNNmPsK2l7zHoHHXZDr+/9vwDfB3kgeSB5IHkgeOH0DmesJjSXi6pUvkYt5u9teVy6aWREDM0D0BRvmGRV5N6DsQkMzI64FidtI5t3AOKWaFhuioY8dlYf9TO1PREUh/9HVeAqzIThHgWZ6MuNmC1jiL1mK4pAzlKUojEmNsxcmL0J60tazWjLZFpClPbd9BMJfL95145YajN5RHQAAAABJRU5ErkJggg==\");\n}\n.bk-root .bk-tool-icon-crosshair {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAADEUlEQVRYR81XXVIaQRCeHqug8CXmBNETaE4gniDwIgpVspxAbxC9ATkBkCpQ8gKeQDiB5AQxNyAvUlrldr7eHxyGXZi1rMJ5opbp7m++7un+htSGF204vsoMoNXrlzSpfWa1oxQfhAegCZGaEtPorHo8znIoJwCt6+td8uk7ApUQCIHTF4BNAWzImq8ap6cP68CsBdDp9i9ZqXM7ML79g/EnCWD+jgMKENKqWT+tXK0CkQqgNRjs0OxpQIqKhoMxaG6/6JeRnK7T6yO2UvVqhYSlLX+ryORfgKn9ORDFIy7ky41yGcwsr0QAQfDH5zucOswx819fs4egI9OFCcD8DjBF7VNbEX0JzdWEt3NHSSASAcCxBDqMgt/623kvyTgNgNjJIfTjk4D4FqaJR1715MjmYAmA5Bx3AwUXQL+t105KaTlcBSC26XRvhjEIoLiq1yqXpr8FAGG16/ug4IT27fxBWu7EiQuAiImJpEMKE6nYM30uAIDDttSUOPfJP7JzbjPhAiBIh9QE67vIvoOi9WJfCwDavf40ulpjbCqmUf+W753ezURuh7Dg1SqflwAEHU6pgfyBq9Y4qx0LG++2fnZ/eUzcstmdM2AWH+jfc+liWdBJfSENf8Lifi3GVwC9mybOfi5dzatWVrbbLIHNva8p5h/16gkaFiLGGxbufkoE6XguwePiXLF3XmMfCUCUAqtKXU7sumd1CowOuJEi3Pg1FBpjitIGhyvVSfvmjci6ZR+rFQfDiPVE2jFYeICQ+PoewwjC5h7CZld6DBdyu6nDSKgzOyIMhmhK5TTqXYbRorZYM46TmpKAAOrGWwSJJekSB1yqJNOzp1Gs7YJ0EDeySDIMtJbQHh6Kf/uFfNFZkolJICRmz0P8DKWZuIG2g1hpok+Mk0Qphs0h9lzMtWRoNvYLuVImUWrmPJDlBKeRBDfATGOpHkhw670QSHWGLLckmF1PTsMlYqMJpyUbiO0weiMMceqLVTcotnMCYAYJJbcuQrVgZFP0NOOJYpr62pf3AmrHfWUG4O7abefGAfwH7EXSMJafOlYAAAAASUVORK5CYII=\");\n}\n.bk-root .bk-tool-icon-lasso-select {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEgwlGP1qdAAABMBJREFUWMO9V1uIVVUY/r61z57ZMx4DnbzgkbQXL5iCJphlWdpIGY4jpFBkEiU9ZNaDRRcITcIwMwgxoQtU2IMXdAZfMjFvpERXYiSbysyBEXFmyuHMnLP32uvrwT2xnY5nxvHQ93Jg7fWv71/r//7L4a59TRgqJk+Z6v3a+sv0OI5nk5wu6VaSVZImAThHsgjgrKTvM5nMUWvtmf5n8HodCIKgOgzDhc65pSTrJQWDsSNpJX1ljHnDOfdT37oZLLHv+8OMMasKhcIJ59xHAJYMlhwAJGUAzJfUTHLFuFzOG5QDU6dNMyQfs9Yedc5tBpAD4IYYNQGoBrDtQnt7/b0LFrJsCHzfn2itfQfAnZLiazytA3AaQAuAiwDaEgeNpGkkswAWSBqRONB38b88z5uTKePt6iiKXkk8jq+iJC5LOmiMaTLGHLPWhmWeHr7vV0dRtATAapAzIVmSo51zyzIlbm2stesFPA6pKk0r6Ryg93y/ek8YFvPOOTg3cDSiKCoC2OP7/rEoirYm4rUkF12lAWNM1lr7lqQn0+QA8gI2jBg5cj6Aj8OwmB+KAKIoukhyp6SRJAUgl0ndPLDWPi9pJQCbuviXvu+/GIZhW1dnJ24UJFuTjCCA2ADA8sYGWmsXS3qmL94kDYAtkh4Nw7ANlQJ5U6INT1KrAYC9zQdykl7nFSj5fXp5Y8NWVBhy7mUAjqShMYdMXV2dJ2klyRwAJ8lIeuGWCRMP7N7frEqSG2OmAFhKshNAp5wrmO7u7jEAngPQm1S2z2pqapr+OPt7XEly0oxwzq2RdFmSD2AMgKKJouhhAL4kA+Cs53l7e3t7uytJHgRBreTWkXwkKVJnJD0B4GAGwIJE9R6AFufc6UqSZ7PZbD6ff5dkA4CQZEHSqwAOISmXtwGIE+F1SeqqIP8d+Xz+C0mLJYWSAODteXffczjdDQNJ0BWMCoLg5gqIbRTJNwHsljQhUb0luWPM2LE7Thw/9m/5NCT/TByxAOYWi8X6/gdWV1dnfN8fNRBxJpMZTXKdc+6IpFVJWAEgkvSJpA0X2tvtVTaSjgOYBCAEEADYSHK87/sfhmEYA9gShuEDkgzJHyWtB/B1irQ2juP7ADxkrX0wOUOpzmdpzEY590HJ7Ni1r2kSyZOSiv2+hSRjSTXp/QAukzySNJOJkmalyNIl10hqMcasdc61XDNcQRD8BnITgNp+36r6kfcNFMMlLQGwTNLMEuQGQBfJl2bdPru+HDkAZAqFQux53jZHEsC6aw0eg2gylNRBcqcx5v04ji999+03AwsWAOI4Lsy9a94WkisAnE5a5WCJYwCfA1g7LJudI2lTHMeXBm1faiQzxkyRtF3S5CTupeAB+KG2tnZFT0/P30NO2VKLzrmfAbwGMipjG5Oc0dPTc0Md05SZ5U4Q2FxChErtEYD7jTGNQ3UgM8Asv90Yc9I5LSKRlXSI5CxJa0jWSALJjKRnAewfkniT+vwf7N7fXHK9rq7O7+jo+BTA/NRrdBpjnnLOnUrvXd7YMPQXSBunneno6IhIHgYwW1JtkgmBpBkATlVMAwOk3nFJ+VSoqgCMr6gIy2FcLtdKspAedyQN/98caDt/3kpyabUmf8WvG/8A1vODTBVE/0MAAAAASUVORK5CYII=\");\n}\n.bk-root .bk-tool-icon-pan {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg4lKssI9gAAAOtJREFUWMPVll0KwyAMgNPgoc0JzDX2Mtgp3csKErSamGabIEUo/T6bHz0ezxdsjPJ5kvUDaROem7VJAp3gufkbtwtI+JYEOsHNEugIN0mgM1wtsVoF1MnyKtZHZBW4DVxoMh6jaAW0MTfnBAbALyUwCD6UwEB4VyJN4FXx4aqUAACgFLjzrsRP9AECAP4Cm88QtJeJrGivdeNdPpko+j1H7XzUB+6WYHmo4eDk4wj41XFMEfBZGXpK0F/eB+QhVcXslVo7i6eANjF5NYSojCN7wi05MJNgbfKiMaPZA75TBVKCrWWbnGrb3DPePZ9Bcbe/QecAAAAASUVORK5CYII=\");\n}\n.bk-root .bk-tool-icon-xpan {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg4X4hxZdgAAAMpJREFUWMPtlsEKwjAMhr/pwOOedINJe/PobWXCfAIvgo/nA4heOiilZQqN2yE5lpD/I38SWt3uD9aMHSuHAiiAAmwaYCqoM/0KMABtQYDW11wEaHyiEei28bWb8LGOkk5C4iEEgE11YBQWDyHGuAMD0CeS30IQPfACbC3o+Vd2bOIOWMCtoO1mC+ap3CfmoCokFs/SZd6E0ILjnzrhvFbyEJ2FIZzXyB6iZ3AkjITn8WOdSbbAoaD4NSW+tIZdQYBOPyQKoAAKkIsPv0se4A/1UC0AAAAASUVORK5CYII=\");\n}\n.bk-root .bk-tool-icon-ypan {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg4anK0lywAAAMVJREFUWMPtlzEKwzAMRX/S7rlpIMXeOnaLaME36FLo8XqCdNFghGljyc4kgQi2Q/SUj0F/eL7eMMTKz6j9wNlYPGRrFcSoLH4XxQPvdQeYuPOlcLbw2dRTgqvoXEaolWM0aP4LYm0NkHYWzyFSSwlmzjw2sR6OvAXNwgEcwAEcwAEcwAEcoGYk20SiMCHlmVoCzACoojEqjHBmCeJOCOo1lgPA7Q8E8TvdjMmHuzsV3NFD4w+1t+Ai/gTx3qHuOFqdMQB8ASMwJX0IEHOeAAAAAElFTkSuQmCC\");\n}\n.bk-root .bk-tool-icon-range {\n  background-image: url(\"data:image/png;base64,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\");\n}\n.bk-root .bk-tool-icon-polygon-select {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEjc1OfiVKAAAAe1JREFUWMPt1r9rU1EUB/DPK0XbqphFHETo4OCiFhwF0V1KHbRSROLqon+AUMVRRFBwEbRFMBiV+mMW/wIxi5OD1kERRVKRJHUwLvfBTZrU5OWBGXLgQu7Jfe98z/ec7z0vKa88b2q1BDtRHdAPBaylm1NzsxsOjPnPNt6WSWprbft+/c3I3zOAjhT1Y4+fvcjEQJIXnVECSa+AhqIHqlHH5lWCZoe+Gk4GRgDG86j9SAUdlDBSQaZhlOkuHyoVdJmsw98D1S5fM4NYM1LCpqM+Lwa240oLgmZzpVZvzKT75VLZcqksSZKWlQeAy/iORVwIvh31xvotvK7VG3Px4aWHj3Jl4C2uYSvq+Bn8v6LLbaVWb9zsBiKLCvbiNG7gLm7jAYqbPHMJMziZ9lsKoh8GtqCEVVzHftwJn+TFHp4/hg8BSCYVfMOZoPEv2NZGdy9WCGUr9toDR3E2/H4V6nwRe/BmgN65H1ZhvMuB3XiKIyFoGefwO6ysVkUlrNUNsyAK/jli533Q+Y8cJFvAeXyMS1CI/jiMr/gUtD2LQwMGr4R3p7bY3oQHQ5b38CT4D2AXXg6YcQXHpyYnlqKsi5iOAVSwL9zd7zJ09r+Cpwq72omFMazjT9Dnibym0dTkRDUKrrgwH7MwXVyYB38BstaGDfLUTsgAAAAASUVORK5CYII=\");\n}\n.bk-root .bk-tool-icon-redo {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg4itK+dVQAAAaFJREFUWMPt1L1rFFEUBfDfJDaBBSslIFjbaSFp1FJQFMVCHkzhKIqdUYOCoBgErVz8rCwiTDMwBCIKipDWyip/gxAIWAmBgBC0eYFh2Gx2l9lFcA5M8e59782Zc84dWrT435Hs1siLchqn43MS0zgW22vYxjesYjVLw3YjBPKinMUTBOwf8J5fKLGYpWFjJAJ5Uc7gIW6jM6Kim3iNZ1katgYmEL/6I+YasvY7Lg6iRpIX5VF8wuEe/XV8wGf8jN6LWTiAc7iEQ7ucPZ+lYW0vAtfwvlbfwCKW9gpXDOv1mJvZHiSO91MiyYsyiQSuxtpXXM7SsDmM5nlRdrCMMz3sOJWl4Xevc/vwBzdwAl+yNNwZxfRI+GxelK9ikHcwh8d4NNR/YFRES1ZwoTYdR7I0rNf3TzVNIGbmSvR/Bx08mIgCFSVu4l2ltIWD9WxNGR+W8KOynqnZ0rwCeVG+wa0hjrxtWoF5dAfc28V8Mib/n+Nev5dnabg/zgw87aNEN/bHOwVRiRe4Wym9zNKwMKkpgIWKEt24njxiJlq0aPFv4i9ZWXMSPPhE/QAAAABJRU5ErkJggg==\");\n}\n.bk-root .bk-tool-icon-reset {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg4gWqH8eQAABLdJREFUWMPtlktsVGUUx3/nfvfOlLQaY2IiRRMQIRpI0PjamJhoVASDvNpCpYw1vJQYSVwZwIVQF6wwRHmkAUof9ElrI6VqDAXcID4TF0IiYQMkSlTokNCZ+b7jove2t+NMH7rQBWd3v+989/zP+Z8X3Jb/WGQySvUNTQBJESkNguAVYIWqzhaRhwBU9WcR+QXoymazn6jqzUQiMQSQzWZRVdal1vwzAI2tHQBPOuc2AbWTdOyQ53n7nHNfRwee51GzqoIQMCLDpr3x/tLQ0oZzrk5Vj0/BOEBt+KYuOlBVGlrahr0Wob27t3gEjnZ2AyQzmUwHsDgP6J/AYRE553neDwDOuUdU9QngNeCumK4TkRMhZUORcYC1qysLA6iuSQHIwkWLD6lqapQsuSmwTVV3h99I7EcAR462A2xR2Ilq6ehTaejvO1774kuLNALR33eclsaGsQDe3fYegHl43vyNwEeqGl1963mm2jl7YZRTQ82qlWP4HM6ZToC5ztkW4LHQoALru7s6Di5dvlIj/e6ujrEAWoZDn8hmMjXATMACGaAVuBjXTVVXFc/AxhaA+4zvn1DV+eHxVWPMAmvtb5GeMWZyZVhI2rt7qVy2pOh9U1snwIPW2vMi4oWJuBPYHkVAVScPoKmtkzVVK6cEMsyJraHhiCqJqJUwj/JRz7TW1iSSyR2rVyylqa0Ta+24Ic8vXaAEmDFc/l5Z2A/80OibuVyuz/f9ElUdHCmvw82t5HK5h6y1PYhsz2YyGw43t2KtBZHIGwB6+j4rCkBVUdV7gXrggnPuu8h4eP+xMeZS2D0rJYZ6AdAMzAt1b4nI26p6IFZOY8pugijcKSIHVLUK0LyST4vnrVfnWr3mjmP4QTATaERkXkypRFX3isjmuHdRJEK6Ckqquopp06bdKCkp2Sgi7XnGLcg7gzeutwNIiPYc8HixqIrIOlU9ONVIhHPEd851icgSVXUiskVV94gIqoonIt0i8gfQCfwae38e6BWRXuBZz5jZ8VbaOE4EIqlZVUEQBLlkMplS1QER2RwkEnsSyaREDUzyeNsvIhvCMqkH1kdIJ2o+k8iJB1LVVRfjZ6nqqlEAIbdVQGto8Lrv+/dbawcjAL7vc+6bs+zetetfLSHxniIFGofGGsU2oC7eOCbDfZ7nQawBOSAX74SF9oEPImOq+r7nmVmxb5raukZa8UReGmNmhbMkAwwBH467EYVZe49z7kdgenj8k7V2oTHm8kgdWcvrNdVFjR8cHkYzjDH9wLjDaEwEzpwa4MypgWvAjtjxfGNMj4jMiT+M+kFsZI/Q6Pv+HGNMT8w4wI7TAyevxXVPD5z8+zD64tRXAMHVK1eaVLUyVvuDqroV2BOnJF4ZIedviUidqt4Re9s+vbx8zZXLl7PR2+nl5Tz/zNOFp2FzxzGAklw22wUsLLaSKXwf8vhosZUM6PeDYEUum70VHfpBwKsVyyfeikOP6oBNwN1TrLbfgX3A1kKLzKeff8nLLzw38T5wZDgxn1LnNk5lLRfP26/OnR2hwfNYW2Atn9RCsrf+EECyrKysDFimqhXhyjY3VLkAXBKRDqA7nU6nS0tLhyIj6XSaN9bVclv+l/IXAmkwvZc+jNUAAAAASUVORK5CYII=\");\n}\n.bk-root .bk-tool-icon-save {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg4UexUIzAAAAIRJREFUWMNjXLhs5X+GAQRMDAMMWJDYjGhyf7CoIQf8x2H+f0KGM9M7BBio5FNcITo408CoA0YdQM1cwEhtB/ylgqMkCJmFLwrOQguj/xTg50hmkeyARAYGhlNUCIXjDAwM0eREwTUGBgbz0Ww46oBRB4w6YNQBow4YdcCIahP+H5EhAAAH2R8hH3Rg0QAAAABJRU5ErkJggg==\");\n}\n.bk-root .bk-tool-icon-tap-select {\n  background-image: url(\"data:image/png;base64,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\");\n}\n.bk-root .bk-tool-icon-undo {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg4em8Dh0gAAAatJREFUWMPt1rFrFFEQBvDfGhACASshkL/ALpWVrSAKEQV5sIULWlgZNSgIFkGIVQ412gkBt1lYLERREFJqJRaW1oHAoZUQsDqwecWy7N3tbe6C4H2wxc682Zn3zTfvLXPM8b8j6RqYF+UCzsfnHBawGt3fMcAX7GEvS8NgKgXkRbmMxwg41TLsN0psZmnodyogL8pFPMIdLHUk7hA7eJKl4U/rAuKu3+HslFr/FZezNPSTFslX8QErDe4DvMVH/Iq9F7VwGpdwZUjsPtaSFjv/1vCBPjaxO0xcNbHejLpZrrlvJCMCT+JzA+2fcC1Lw+GE4l3CG1yIptfjCtiKoqtiJ0vD3aM0Py/K57iIMxgkQxat4EdN7e9xdRzlk+LEEPvDWvIDXJ928sYxjL36icWK+VaWhlezOIqbGFirJd/H7szugrwoX+D2BDEvszSsT5OBdfRaru/F9dPXQF6U27g/KnmWhgctxqyzBrZGMNGL/rHI0nDkKXiKexXTsywNGx0OnFbFNk3BRoWJXnw//j+ivCi32/S8CxPVNiWOAdUiJtXITIqYY45/Cn8B2D97FYW2H+IAAAAASUVORK5CYII=\");\n}\n.bk-root .bk-tool-icon-wheel-pan {\n  background-image: url(\"data:image/png;base64,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\");\n}\n.bk-root .bk-tool-icon-wheel-zoom {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEgskILvMJQAABTtJREFUWMPdl1+MXVUVxn/fPvf2zrSFmUKnoBCUdjRoVaIxEpO2JhilMYBCtBQS2hejpg1Uo2NUrIFAoyGmtiE+GHwQGtvQJhqDmKYRBv+URFsFDNCSptH60DJTO3dKnX/33rM/H7rvsDu9M20fDMaVnGTvtb69z7fWXmvtc/TEzqd4OyXwNsv/FwFJQVI/sA14SZKRLOlPkr5TrVYXHz70quYkEEK4TtI2YAgYkrQthHDdhV5uuw+43/ZrwCbgRttgY/tjtrc0m83X3/f+D6ydnJhYcB4BSZcBA7aP2d4ELAGW2N5k+xgwkDB0IH19CGGH7R8B1aQeAf4KvAw0ku4K2zu7uru3ApdPEyiKohd4TNKjtjt5h6RHgccSNrddbvuHtm9Jqoak7xVF8WFgdavV+pSk5cCObNmXgK++85prCj3z28HKqZMnH7D9YAY4BvwujT8BvCuL1INX9vVt+dfwcCvNb7f9q2RuSfrGvWu/sL2Nf3LX7pzvj4ENSGBPVarVd4fRkZFltjdmoMGiKO4IIWwIIWwoiuIOYDDzeOPoyMiyFLkum7WJCMDztrcrTTrIRuAQZ6NcK1utL4dWq/VZoC8BhqvV6l1lWb4YYxyLMY6VZflitVq9CxhOmL60hhCKeYiV7WMKIXw9jT1HpXw3c+bOAKzOjJubzebJrKQCQLPZPClpc7bP6rMYKtjXth2OMf7tIkr11Wz8oQDc1Fb09vY+kQw1YAuwJY2nbUluAnCWpKkaFl6IQIzxivaR2SYA89sJVK/Xp2x32R6w/a30DNjuqtfrU0ArYecDCEqgLqm94T0dEm9mBG7PxkdDlkBnkhebgIezNQ8nHcCZPL9ijE1Jf/bZZoPtzbavmqNZLbf9tSxq+yoduuJ+SZ+zXSZyBXCqU+d8fvC5yRUrV+0G2j3g2hDCLyXd/+Su3QdnvP/zCuH72LWsgf2k0oHlH2c2odlkxcpVEdgr6aDtjyb8x20/J+mA7T9I6rL9SWA5dne2/GdXLl58qNJh398An85yTMA+4DOz8Dgu6Zu2dwJXJ91ltm8Gbp7Fgb+EEB4aHhpq5CEtACqVyr3AC0AlPS8k3TSmQ2YPhhBuS/1/LpmS9JTtNTHGfwBU2uUALARotVqniqJYH2Pck85pfavVaufAwnQvnHc0McaDKVptebN94QAnJB0EdtjekydyZXqjs/0ZgLIs/w6sy8bnYGYJ63pgERKC05JutT1kOwITwL9tvzlzUQUYB+Zjs2DBgu6xsbGJZHstByZbezregcBXeCsEz1bnzXt5anLyzLq71zDLxTRdVgemdx0fv2e2w5thO5DbiqL4oKT3ZKpnpyYnz+SY2ZpTAPZmJfdIrVZbNBNUq9UW2X4kU+2dcf53Aj1pj2PA7y/6m1DS00A9za9uNBq7iqJYBuoGdRdFsazRaOzKSqye1rTbaa/tlbYrqXQP2X4FIA9/J1l39xrC0v7+w5IeB8XkwS1lWe6TGJAYKMty31tfO4qSHl/a3384I3CDpI+kzC4lnRfrue6GytEjR8oQwlY73gC0L4qlth/q0M1/LYWtR48cKQF6enrC6dOnVwGLEpnxnp7en4+O1i/tszzGOCTpPmB7ahb57QUwBWyXdF+McWg6MScmuoA8OX8xOlpvXGz422XYTsB/SnpA0h7bX5R0WzI9HUL4qe2XbI+dk3xl+V7gxoztD5jRI+YK/zkEEokx2/uB/RdzIfUtueqVN04cXwF8G3iHY3z9Urw/j8ClyhsnjrcS2Vv/J/8NLxT+/zqBTkcxU/cfEkyEAu3kmjAAAAAASUVORK5CYII=\");\n}\n.bk-root .bk-tool-icon-box-edit {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEg4QfHjM1QAAAGRJREFUWMNjXLhsJcNAAiaGAQYsDAwM/+lsJ+OgCwGsLqMB+D8o08CoA0YdMOqAUQewDFQdMBoFIyoN/B/U7YFRB7DQIc7xyo9GwbBMA4xDqhxgISH1klXbDYk0QOseEeOgDgEAIS0JQleje6IAAAAASUVORK5CYII=\");\n}\n.bk-root .bk-tool-icon-freehand-draw {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAADTElEQVRYCeWWTWwMYRjH/88721X1lZJIGxJxcEE4OOiBgzjXWh8TJKR76kWacOBGxdEJIdk4VChZI/phidRBHMRRIr7DSUiaSCRFRM3u88gz+o7Z6bBTdjmYZPf9eJ55fv/5zzvvDPC/H9QsA66Olo9Ga+/MdR+Ljm2/KQIULsz9FqItGdOfJKLhApLgVkiSCGODjWit7QpKWy+TNrFeXvzKVUT8NiTVaIgDcbiCFJ7GiT8WkARXAdYBK0Lbhi/CenArRNskuM7/tgNp4ArQ42dwjf3WY5gWTqC7O/NbNn2Xkfw/YwdSw/We14HP2IEZwX+y9cZ9SH0LmgFP7UCz4KkENBNeV0Cz4b8U8DfgKiDxMWwUXETqLvJpCQpXZfawbzS7t9v5pL19cHBwfja7YA0y/lyCM0+E5hv5+piZXwKYcF23as+37bTXsQVqgkL0p/34fHR7DcBtbetFsBmGDwMOJCggYG55yw7dMlk6DuC1Bdu2RsCU9TYWQq2IoGbsreZ5NzvEqfSBsIsIy8OTbcdgiRHeh4o8AFAEwDakbY2AaCCpH7V9aGhoUUUy3UyVbkPYFuYLDlUZH8XBpwxkK0Dbgxg5HcVi0ent7a0RULMIozaHBSMfF9b2SzdutFcFB2FkwMIJOG6qfteXOa1nHZ48tyefuwyfT9s6wtzZ3t7eZse2DR2I228TtHXzuWCx9g8MtK5cuHCZTH4tiHEOa4xFngvTyS8f35d6enomiCi4/foEXBkZaQuukChL4FYA2Whd7YcC4gEdW3CpdL3LtGAVCVYJywEyTpAuJKeMOKXZs/Bw947C50KhUFOG4cwz35cjWNBlHGeD53n3xsfHP/T19U1qciggar8Fa4I3PHobIotBWBtc2hSiChyZxVzM53Pv7FVH6Tp3uVy+g0r1ImD2GjIrQGYIxjnfuXTZGICS5k/bBwJoubwEFX4TLah9EXomJGMA3za+f9913Yl4TnzsDQ+vE6YTZOjHh4ngibstt1pzQwd04F0bPStEBpXqRoBeQ/AKghfBnOEKgS+Q7z91Xfdz/HGKg8Ox7z8iYD9z6wqTkZFgnvhMGP9VZ2or1XVkPM9z0mytSfVsHa1RLBZbLoyNzUnK+ydz3wC6I9x+lwbngwAAAABJRU5ErkJggg==\");\n}\n.bk-root .bk-tool-icon-poly-draw {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEjglo9eZgwAAAc5JREFUWMPt1zFrU1EUB/DfS4OmVTGDIChCP4BgnQXRxVHqIJUupp9AB8VBQcRBQUXIB9DWQoMRiXZzcnQSA34A7aAuHSJKkgo2LvfBrU3aJnlYkBy4vHcP557zP/9z3r33JdXa647N0kHSZd5Nn0rSxc8G3cXp85sMcnZZ8vge3osZ+l3vB8CWFA0iL14t79h210swAjACMAIwAjACkB90D/8/GchI9ve4nPwTBh5E9ws7OepzGWb9EddSn51Op9ZstadSg4VK1UKlKkmSDSMLALewiuNh/hVJq71Wxttmqz0dG88vPc+MgWP4grvYG3SLOBrZFFFrttqPe4HIDxh4GSei+98iSlusuYopXEAjBtEPA3tQwUpwluAbDm4TPJUz+BTW9l2Ce6G7L0X/Bw8D3T/7SKKIDzHg7QCcxjvcQAEtXAnrrg/RP0/DKPbqgcN4iVOR7gcO4dcQgRuoh7HSqwlP4n20m63jJu5n8MkWMYfP3UowhzdR8FU8w9iQwevBdyq3/27CMRzAE5yLuvsRLg+ZcR1nJ8YL81HWJUzGAPaFZwe/Q5MdyYDyNHgjzO90YyGHtVDncuiJchaHw8R4oREFV5qdiVmYLM3OgD9k5209/atmIAAAAABJRU5ErkJggg==\");\n}\n.bk-root .bk-tool-icon-point-draw {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gEMEiERGWPELgAAA4RJREFUWMO1lr1uG1cQhb9ztdRSP7AF1QxgwKlcuZSqRC9gWUUUINWqTh5AnaFOnVPEteQmRuhCURqWsSqqc9IolREXdEvQBElxtdw7KURSFEVKu4w8wAKLxdw9Z+bMnRmZGXfZ29//II8th4WwGVNyIoQLYB5vxA9Caq04iUd9A+7ZlsNC2I7TdSd2hZXMJKlnTqp9jtl/GBaqoyQ0noFKpUIzBicYYc+DEFpxkglc4oVJa5gvDn8v1xV2irG3FM4NSVwjUKlUaMcpJhCGmSEJQ6QGD8M5WnHCd8+f3QCXpPLx8WNwv0j6Bm9FMK7FJ3WBE+R/2t7c/GBmFvSBrzRTCsyTDjXrxUgEMtpxynJYmJoBJ4VAybwVARgvL7Oik0okCodnKpVKX7P0leiVMb0VvbJT+upznK4vh0GIeQwwQStJkHQD3MwsCALTJRG7Qrdrj5m/djgYaIa0hlkRdJk26XEgC9txurccBtVW3IudBImmZuACUP+ZlIDBt9FKcubYNTcAH/X0RYM1E7utJPlqe+uZzPxUcEkiSS4sTT95n15Mud0xWC0o2PAWOCdK3KYZlFxfM+tHOcnMzNr1es18ug+cgsVjP4yBU/Ppfrter1m/+l0+zYygML1xRVHU7TSb1cSzBzoBzszsH+AMdJJ49jrNZjWKou6wBnwOzcyndBpNbuueURR1Dw8Pq35p9cc5p/Dy9Dypt7jXrtdGwQECS9NPhr6Gq6txUzNigE6zydLK6lTw12/KT4FGFEUfJX2YJNONq5tVs4ODA7sD/DnwJ/BoADZuE3tHFs12dna6d4C/BI6AlbyzI8ii2TTw12/KK33gb2cdXsNZoAntbZC2SeO4c9592k/5eNQbiwvFd1kJuFGwLJr1wSPg/SwpvyFBHufOeXcFeAlE97U/uCxOY+P3b+Bn4B3Q+L8EdJfD4a+/AbC4UBzPxiPg3wlHZquB28Cn2IuR9x3gr3uV4DbwfvSDOvi4uFA8BDZmIRHkjHpS9Ht9iRqd8+5G3g05mAGcQbsdiX5QJ428G7Kygo8XYdb1/K4NWVmjzkNge2sz84bs+ELmpDDLtqWsNZBXgvmw8CTtpWVMT7x5YWBjLARnwZfKQNYN2U2LPvrh+5nBt7c2M2/It9bArCTKR8eZN+SJ13AScPnoODeRdqNenH+wul5w2gUr2WUjMFAt8bZ/0axX/wNnv4H8vTFb1QAAAABJRU5ErkJggg==\");\n}\n.bk-root .bk-tool-icon-poly-edit {\n  background-image: url(\"data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH4gELFi46qJmxxAAABV9JREFUWMOdl19vFFUYxn9n9u9sCyylUIzWUoMQBAWCMdEEIt6xIRQSLIEKtvHe6AcA4yeQb7CAUNJy0daLeomJN8SEULAC2kBBapBKoLvbmdl/c14vdmY7u91tF95kknPOnHmf95znPc97Ro2OTeBbdjFDT3c32ZxVHUOE9kSMB0/m6ExuoJn1H+ur6Y+OTfD50SMN5168OgrAlyf7CfuD+z7+iDs3p8hkLUQ0iFQ/yFl5Nm/qonfHVva+s32Zw9GxCYILsZ08tpNfBhbs+1YN4OH9+7huGdECSBVfqUosbsllfmauBqiR+cCNwOr7AEo8pPHJnymXykhg5fUWjoQpl0vVvhZhbSzGoUOHqgBlt6B6uruj2Zy1E9jo0fhfeyL2x4Mnc8VErK0KUEOB64JSyptfG4RSytsJjUJVxw2lsFy3urL9nx1Qd25ObctkrVMi+jQivd7U2ZyV/3Hzpq7h3h1b/7p9Y0o8v8rwAbTWrGpSocN/FGDlbAI0Rl23PCBan0Ok158H9Ipwzi25A/Mzc9Gl/BYx/E4kYqC1NKRARNAaDCNUM27Z+Zr+ouXs0q4+LSLBHPYCFkTkC6uU39kwCdsS7WRKmaYUiAhdnZ3MPX2K4+QjQI+C94A93rMzm8ltMwyDeDzWjMZeEb2pYQDdW3vITU2jtUZ5QThOPgm8C7wP7J15OPsBsB3oWpGnVWisCeDS1VHj4vBI92+/3tgB7Ab2AruAXiDBK5oIOkhtkEYRNRuJhObrd8Dl9ewf4D5wG7hVLpen29vb5wzD+BrkbBMaL3d1dk5nsrnlFDTTFWAWmAZueWD3gCemGde2k2fw1Al1YXhEvjozoO49eczdqekrWmsc2zlrmvEKOGoW1GUjFLqSk2KpJrCLwyMCPAP+BO54QL8DM6YZX/ClsP9YnwKkXnIBP4jdIpJRpdJTCYdMwwi98KU0Hjc/dDILNyUcwTCWdOSMJ0TRmBktGRhLugu0xyLk7CIqVNm+0bGJptl1YXikD0grpY4Rjc4a8Fbgdab/6OGbAJeCUuyJnnHmZH9pbSyGuBXV8NUwlUpR1EWyixmSyTWEwqGlJ2Swbo2JXbAAfgDGgGQA9I1A9t1tlq0AxrXxn0ilUpw4fhQqYkH/sT41OTnJJwf2s6FjI5mshdYa7bqVR2uezr9MJmJt14FvGrh/O9D+e6UkM/xyCuCqEKCYnJyUTKFQrZDHjxzGshwWLQcRsOz8Hi85P23id0ug/XilAMLBmm4tPGdoaKjSH5+oAGrhwvBI9SjZTn4QSK9yenoD7dlrExPoJlXW8G8ytpNHxRKk02lGxsdRKFwXLNvx5yY94HQLGhGk4LFCYQSqaE0AwWM1eOoEbR0dKBSW7bC4mKuffxs4D/wCLKwQQPAUzIkslfp6cVomROWSolh0GjldAM4nzDi2k9/i5UAzC9aKfwNJ3zgJg9YEvN6+C7SHgKm69+sD7RfNnKTTaZRPQfAut4oFV//IS7gkcB34VlVo8kGzphlfB+DU+TfNGBpZtRastvrvARJmfMF28ge9sc2B9/PNnCilMIDwK6y8/ow/Ai4kvILTljAXvDvEvrqKSUs60KolzPjBxspavQD2tKqCAGF/Ba+xE/Wbilu54wZV8NEKF5fXzQHl/bh4hUsE0WAXSlDMYcQSrQXgCmsTseXHsJkNnjqBFGwKJaHsKlxtUHYVhbLCzr1kaOA4bcn1y1Swmb+iLpJKpVrfgdpfsiVVCYcgluwgnU7jEgJ4s5UkLFtWYyHyEg0/N1q1tmQH+YXnAMFr97Nmv3p+0QsHQRsF8qpBOE5+rb9Nkaj50tVQKjqh4OU3GNL/1/So3vuUgbAAAAAASUVORK5CYII=\");\n}\n.bk-root .bk-tool-icon-line-edit {\n  background-image: url(\"data:image/png;base64, \");\n}\n";
    exports.default = css;
},
/* core/util/menus.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var dom_1 = require(187) /* ../dom */;
    var iterator_1 = require(418) /* ./iterator */;
    //import menus_css from "styles/menus.css"
    var styles = tslib_1.__importStar(require(397) /* ../../styles/menus */);
    var ContextMenu = /** @class */ (function () {
        function ContextMenu(items, options) {
            var _this = this;
            if (options === void 0) {
                options = {};
            }
            this.items = items;
            this.options = options;
            this.el = dom_1.div();
            this._open = false;
            this._item_click = function (i) {
                var _a;
                (_a = _this.items[i]) === null || _a === void 0 ? void 0 : _a.handler();
                _this.hide();
            };
            this._on_mousedown = function (event) {
                var _a, _b;
                var target = event.target;
                if (target instanceof Node && _this.el.contains(target))
                    return;
                if ((_b = (_a = _this.options).prevent_hide) === null || _b === void 0 ? void 0 : _b.call(_a, event))
                    return;
                _this.hide();
            };
            this._on_keydown = function (event) {
                if (event.keyCode == dom_1.Keys.Esc)
                    _this.hide();
            };
            this._on_blur = function () {
                _this.hide();
            };
            dom_1.undisplay(this.el);
        }
        Object.defineProperty(ContextMenu.prototype, "is_open", {
            get: function () {
                return this._open;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ContextMenu.prototype, "can_open", {
            get: function () {
                return this.items.length != 0;
            },
            enumerable: true,
            configurable: true
        });
        ContextMenu.prototype.remove = function () {
            dom_1.remove(this.el);
            this._unlisten();
        };
        ContextMenu.prototype._listen = function () {
            document.addEventListener("mousedown", this._on_mousedown);
            document.addEventListener("keydown", this._on_keydown);
            window.addEventListener("blur", this._on_blur);
        };
        ContextMenu.prototype._unlisten = function () {
            document.removeEventListener("mousedown", this._on_mousedown);
            document.removeEventListener("keydown", this._on_keydown);
            window.removeEventListener("blur", this._on_blur);
        };
        ContextMenu.prototype._position = function (at) {
            var parent_el = this.el.parentElement;
            if (parent_el != null) {
                var parent = parent_el.getBoundingClientRect();
                this.el.style.left = at.left != null ? at.left - parent.left + "px" : "";
                this.el.style.top = at.top != null ? at.top - parent.top + "px" : "";
                this.el.style.right = at.right != null ? parent.right - at.right + "px" : "";
                this.el.style.bottom = at.bottom != null ? parent.bottom - at.bottom + "px" : "";
            }
        };
        /*
        styles(): string[] {
          return [...super.styles(), menus_css]
        }
        */
        ContextMenu.prototype.render = function () {
            var e_1, _c;
            var _this = this;
            var _a, _b;
            dom_1.empty(this.el, true);
            var orientation = (_a = this.options.orientation) !== null && _a !== void 0 ? _a : "vertical";
            dom_1.classes(this.el).add("bk-context-menu", "bk-" + orientation);
            var _loop_1 = function (item, i) {
                var el = void 0;
                if (item == null) {
                    el = dom_1.div({ class: styles.bk_divider });
                }
                else if (item.if != null && !item.if()) {
                    return "continue";
                }
                else {
                    var icon = item.icon != null ? dom_1.div({ class: ["bk-menu-icon", item.icon] }) : null;
                    el = dom_1.div({ class: ((_b = item.active) === null || _b === void 0 ? void 0 : _b.call(item)) ? "bk-active" : null, title: item.tooltip }, icon, item.label);
                }
                el.addEventListener("click", function () { return _this._item_click(i); });
                this_1.el.appendChild(el);
            };
            var this_1 = this;
            try {
                for (var _d = tslib_1.__values(iterator_1.enumerate(this.items)), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var _f = tslib_1.__read(_e.value, 2), item = _f[0], i = _f[1];
                    _loop_1(item, i);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_c = _d.return))
                        _c.call(_d);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        ContextMenu.prototype.show = function (at) {
            if (this.items.length == 0)
                return;
            if (!this._open) {
                this.render();
                if (this.el.children.length == 0)
                    return;
                this._position(at !== null && at !== void 0 ? at : { left: 0, top: 0 });
                dom_1.display(this.el);
                this._listen();
                this._open = true;
            }
        };
        ContextMenu.prototype.hide = function () {
            if (this._open) {
                this._open = false;
                this._unlisten();
                dom_1.undisplay(this.el);
            }
        };
        ContextMenu.prototype.toggle = function (at) {
            this._open ? this.hide() : this.show(at);
        };
        return ContextMenu;
    }());
    exports.ContextMenu = ContextMenu;
    ContextMenu.__name__ = "ContextMenu";
},
/* core/util/iterator.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var array_1 = require(124) /* ./array */;
    function enumerate(seq) {
        var i, seq_1, seq_1_1, item, e_1_1;
        var e_1, _a;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    i = 0;
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 6, 7, 8]);
                    seq_1 = tslib_1.__values(seq), seq_1_1 = seq_1.next();
                    _b.label = 2;
                case 2:
                    if (!!seq_1_1.done)
                        return [3 /*break*/, 5];
                    item = seq_1_1.value;
                    return [4 /*yield*/, [item, i++]];
                case 3:
                    _b.sent();
                    _b.label = 4;
                case 4:
                    seq_1_1 = seq_1.next();
                    return [3 /*break*/, 2];
                case 5: return [3 /*break*/, 8];
                case 6:
                    e_1_1 = _b.sent();
                    e_1 = { error: e_1_1 };
                    return [3 /*break*/, 8];
                case 7:
                    try {
                        if (seq_1_1 && !seq_1_1.done && (_a = seq_1.return))
                            _a.call(seq_1);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                    return [7 /*endfinally*/];
                case 8: return [2 /*return*/];
            }
        });
    }
    exports.enumerate = enumerate;
    // https://docs.python.org/3.8/library/itertools.html#itertools.combinations
    function combinations(seq, r) {
        var n, indices, k, _a, _b, i, _c, _d, j;
        var e_2, _e, e_3, _f;
        return tslib_1.__generator(this, function (_g) {
            switch (_g.label) {
                case 0:
                    n = seq.length;
                    if (r > n)
                        return [2 /*return*/];
                    indices = array_1.range(r);
                    return [4 /*yield*/, indices.map(function (i) { return seq[i]; })];
                case 1:
                    _g.sent();
                    _g.label = 2;
                case 2:
                    if (!true)
                        return [3 /*break*/, 4];
                    k = void 0;
                    try {
                        for (_a = (e_2 = void 0, tslib_1.__values(array_1.reversed(array_1.range(r)))), _b = _a.next(); !_b.done; _b = _a.next()) {
                            i = _b.value;
                            if (indices[i] != i + n - r) {
                                k = i;
                                break;
                            }
                        }
                    }
                    catch (e_2_1) {
                        e_2 = { error: e_2_1 };
                    }
                    finally {
                        try {
                            if (_b && !_b.done && (_e = _a.return))
                                _e.call(_a);
                        }
                        finally {
                            if (e_2)
                                throw e_2.error;
                        }
                    }
                    if (k == null)
                        return [2 /*return*/];
                    indices[k] += 1;
                    try {
                        for (_c = (e_3 = void 0, tslib_1.__values(array_1.range(k + 1, r))), _d = _c.next(); !_d.done; _d = _c.next()) {
                            j = _d.value;
                            indices[j] = indices[j - 1] + 1;
                        }
                    }
                    catch (e_3_1) {
                        e_3 = { error: e_3_1 };
                    }
                    finally {
                        try {
                            if (_d && !_d.done && (_f = _c.return))
                                _f.call(_c);
                        }
                        finally {
                            if (e_3)
                                throw e_3.error;
                        }
                    }
                    return [4 /*yield*/, indices.map(function (i) { return seq[i]; })];
                case 3:
                    _g.sent();
                    return [3 /*break*/, 2];
                case 4: return [2 /*return*/];
            }
        });
    }
    exports.combinations = combinations;
    function subsets(seq) {
        var _a, _b, k, e_4_1;
        var e_4, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    _d.trys.push([0, 5, 6, 7]);
                    _a = tslib_1.__values(array_1.range(seq.length + 1)), _b = _a.next();
                    _d.label = 1;
                case 1:
                    if (!!_b.done)
                        return [3 /*break*/, 4];
                    k = _b.value;
                    return [5 /*yield**/, tslib_1.__values(combinations(seq, k))];
                case 2:
                    _d.sent();
                    _d.label = 3;
                case 3:
                    _b = _a.next();
                    return [3 /*break*/, 1];
                case 4: return [3 /*break*/, 7];
                case 5:
                    e_4_1 = _d.sent();
                    e_4 = { error: e_4_1 };
                    return [3 /*break*/, 7];
                case 6:
                    try {
                        if (_b && !_b.done && (_c = _a.return))
                            _c.call(_a);
                    }
                    finally {
                        if (e_4)
                            throw e_4.error;
                    }
                    return [7 /*endfinally*/];
                case 7: return [2 /*return*/];
            }
        });
    }
    exports.subsets = subsets;
},
/* models/tools/on_off_button.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var button_tool_1 = require(411) /* ./button_tool */;
    var mixins_1 = require(288) /* ../../styles/mixins */;
    var dom_1 = require(187) /* ../../core/dom */;
    var OnOffButtonView = /** @class */ (function (_super) {
        tslib_1.__extends(OnOffButtonView, _super);
        function OnOffButtonView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        OnOffButtonView.prototype.render = function () {
            _super.prototype.render.call(this);
            dom_1.classes(this.el).toggle(mixins_1.bk_active, this.model.active);
        };
        OnOffButtonView.prototype._clicked = function () {
            var active = this.model.active;
            this.model.active = !active;
        };
        return OnOffButtonView;
    }(button_tool_1.ButtonToolButtonView));
    exports.OnOffButtonView = OnOffButtonView;
    OnOffButtonView.__name__ = "OnOffButtonView";
},
/* models/tools/toolbar_base.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var logging_1 = require(134) /* ../../core/logging */;
    var dom_1 = require(187) /* ../../core/dom */;
    var build_views_1 = require(230) /* ../../core/build_views */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var dom_view_1 = require(193) /* ../../core/dom_view */;
    var array_1 = require(124) /* ../../core/util/array */;
    var object_1 = require(128) /* ../../core/util/object */;
    var types_1 = require(123) /* ../../core/util/types */;
    var model_1 = require(196) /* ../../model */;
    var gesture_tool_1 = require(421) /* ./gestures/gesture_tool */;
    var action_tool_1 = require(422) /* ./actions/action_tool */;
    var help_tool_1 = require(423) /* ./actions/help_tool */;
    var inspect_tool_1 = require(410) /* ./inspectors/inspect_tool */;
    var toolbar_1 = require(414) /* ../../styles/toolbar */;
    var logo_1 = require(425) /* ../../styles/logo */;
    var mixins_1 = require(288) /* ../../styles/mixins */;
    var toolbar_css_1 = tslib_1.__importDefault(require(415) /* ../../styles/toolbar.css */);
    var logo_css_1 = tslib_1.__importDefault(require(426) /* ../../styles/logo.css */);
    var ToolbarViewModel = /** @class */ (function (_super) {
        tslib_1.__extends(ToolbarViewModel, _super);
        function ToolbarViewModel(attrs) {
            return _super.call(this, attrs) || this;
        }
        ToolbarViewModel.init_ToolbarViewModel = function () {
            this.define({
                _visible: [p.Any, null],
                autohide: [p.Boolean, false],
            });
        };
        Object.defineProperty(ToolbarViewModel.prototype, "visible", {
            get: function () {
                return (!this.autohide) ? true : (this._visible == null) ? false : this._visible;
            },
            enumerable: true,
            configurable: true
        });
        return ToolbarViewModel;
    }(model_1.Model));
    exports.ToolbarViewModel = ToolbarViewModel;
    ToolbarViewModel.__name__ = "ToolbarViewModel";
    ToolbarViewModel.init_ToolbarViewModel();
    var ToolbarBaseView = /** @class */ (function (_super) {
        tslib_1.__extends(ToolbarBaseView, _super);
        function ToolbarBaseView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ToolbarBaseView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._tool_button_views = new Map();
            this._toolbar_view_model = new ToolbarViewModel({ autohide: this.model.autohide });
        };
        ToolbarBaseView.prototype.lazy_initialize = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, this._build_tool_button_views()];
                        case 1:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            });
        };
        ToolbarBaseView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.properties.tools.change, function () {
                return tslib_1.__awaiter(_this, void 0, void 0, function () {
                    return tslib_1.__generator(this, function (_a) {
                        switch (_a.label) {
                            case 0: return [4 /*yield*/, this._build_tool_button_views()];
                            case 1:
                                _a.sent();
                                this.render();
                                return [2 /*return*/];
                        }
                    });
                });
            });
            this.connect(this.model.properties.autohide.change, function () {
                _this._toolbar_view_model.autohide = _this.model.autohide;
                _this._on_visible_change();
            });
            this.connect(this._toolbar_view_model.properties._visible.change, function () { return _this._on_visible_change(); });
        };
        ToolbarBaseView.prototype.styles = function () {
            return tslib_1.__spread(_super.prototype.styles.call(this), [toolbar_css_1.default, logo_css_1.default]);
        };
        ToolbarBaseView.prototype.remove = function () {
            build_views_1.remove_views(this._tool_button_views);
            _super.prototype.remove.call(this);
        };
        ToolbarBaseView.prototype._build_tool_button_views = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var tools;
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            tools = (this.model._proxied_tools != null ? this.model._proxied_tools : this.model.tools);
                            return [4 /*yield*/, build_views_1.build_views(this._tool_button_views, tools, { parent: this }, function (tool) { return tool.button_view; })];
                        case 1:
                            _a.sent(); // XXX: no ButtonToolButton model
                            return [2 /*return*/];
                    }
                });
            });
        };
        ToolbarBaseView.prototype.set_visibility = function (visible) {
            if (visible != this._toolbar_view_model._visible) {
                this._toolbar_view_model._visible = visible;
            }
        };
        ToolbarBaseView.prototype._on_visible_change = function () {
            var visible = this._toolbar_view_model.visible;
            var hidden_class = toolbar_1.bk_toolbar_hidden;
            if (this.el.classList.contains(hidden_class) && visible) {
                this.el.classList.remove(hidden_class);
            }
            else if (!visible) {
                this.el.classList.add(hidden_class);
            }
        };
        ToolbarBaseView.prototype.render = function () {
            var e_1, _a, e_2, _b, e_3, _c;
            var _this = this;
            dom_1.empty(this.el);
            this.el.classList.add(toolbar_1.bk_toolbar);
            this.el.classList.add(mixins_1.bk_side(this.model.toolbar_location));
            this._toolbar_view_model.autohide = this.model.autohide;
            this._on_visible_change();
            if (this.model.logo != null) {
                var gray = this.model.logo === "grey" ? logo_1.bk_grey : null;
                var logo = dom_1.a({ href: "https://bokeh.org/", target: "_blank", class: [logo_1.bk_logo, logo_1.bk_logo_small, gray] });
                this.el.appendChild(logo);
            }
            try {
                for (var _d = tslib_1.__values(this._tool_button_views), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var _f = tslib_1.__read(_e.value, 2), button_view = _f[1];
                    button_view.render();
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_a = _d.return))
                        _a.call(_d);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            var bars = [];
            var el = function (tool) {
                return _this._tool_button_views.get(tool).el;
            };
            var gestures = this.model.gestures;
            try {
                for (var _g = tslib_1.__values(object_1.values(gestures)), _h = _g.next(); !_h.done; _h = _g.next()) {
                    var gesture = _h.value;
                    bars.push(gesture.tools.map(el));
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_h && !_h.done && (_b = _g.return))
                        _b.call(_g);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            bars.push(this.model.actions.map(el));
            bars.push(this.model.inspectors.filter(function (tool) { return tool.toggleable; }).map(el));
            try {
                for (var bars_1 = tslib_1.__values(bars), bars_1_1 = bars_1.next(); !bars_1_1.done; bars_1_1 = bars_1.next()) {
                    var bar = bars_1_1.value;
                    if (bar.length !== 0) {
                        var el_1 = dom_1.div({ class: toolbar_1.bk_button_bar }, bar);
                        this.el.appendChild(el_1);
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (bars_1_1 && !bars_1_1.done && (_c = bars_1.return))
                        _c.call(bars_1);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
        };
        ToolbarBaseView.prototype.update_layout = function () { };
        ToolbarBaseView.prototype.update_position = function () { };
        ToolbarBaseView.prototype.after_layout = function () {
            this._has_finished = true;
        };
        return ToolbarBaseView;
    }(dom_view_1.DOMView));
    exports.ToolbarBaseView = ToolbarBaseView;
    ToolbarBaseView.__name__ = "ToolbarBaseView";
    function createGestureMap() {
        return {
            pan: { tools: [], active: null },
            scroll: { tools: [], active: null },
            pinch: { tools: [], active: null },
            tap: { tools: [], active: null },
            doubletap: { tools: [], active: null },
            press: { tools: [], active: null },
            pressup: { tools: [], active: null },
            rotate: { tools: [], active: null },
            move: { tools: [], active: null },
            multi: { tools: [], active: null },
        };
    }
    var ToolbarBase = /** @class */ (function (_super) {
        tslib_1.__extends(ToolbarBase, _super);
        function ToolbarBase(attrs) {
            return _super.call(this, attrs) || this;
        }
        ToolbarBase.init_ToolbarBase = function () {
            this.prototype.default_view = ToolbarBaseView;
            this.define({
                tools: [p.Array, []],
                logo: [p.Logo, 'normal'],
                autohide: [p.Boolean, false],
            });
            this.internal({
                gestures: [p.Any, createGestureMap],
                actions: [p.Array, []],
                inspectors: [p.Array, []],
                help: [p.Array, []],
                toolbar_location: [p.Location, 'right'],
            });
        };
        ToolbarBase.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._init_tools();
        };
        ToolbarBase.prototype._init_tools = function () {
            var e_4, _a, e_5, _b, e_6, _c;
            var _this = this;
            // The only purpose of this function is to avoid unnecessary property churning.
            var tools_changed = function (old_tools, new_tools) {
                if (old_tools.length != new_tools.length) {
                    return true;
                }
                var new_ids = new Set(new_tools.map(function (t) { return t.id; }));
                return array_1.some(old_tools, function (t) { return !new_ids.has(t.id); });
            };
            var new_inspectors = this.tools.filter(function (t) { return t instanceof inspect_tool_1.InspectTool; });
            if (tools_changed(this.inspectors, new_inspectors)) {
                this.inspectors = new_inspectors;
            }
            var new_help = this.tools.filter(function (t) { return t instanceof help_tool_1.HelpTool; });
            if (tools_changed(this.help, new_help)) {
                this.help = new_help;
            }
            var new_actions = this.tools.filter(function (t) { return t instanceof action_tool_1.ActionTool; });
            if (tools_changed(this.actions, new_actions)) {
                this.actions = new_actions;
            }
            var check_event_type = function (et, tool) {
                if (!(et in _this.gestures)) {
                    logging_1.logger.warn("Toolbar: unknown event type '" + et + "' for tool: " + tool);
                }
            };
            var new_gestures = createGestureMap();
            try {
                for (var _d = tslib_1.__values(this.tools), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var tool = _e.value;
                    if (tool instanceof gesture_tool_1.GestureTool && tool.event_type) {
                        if (types_1.isString(tool.event_type)) {
                            new_gestures[tool.event_type].tools.push(tool);
                            check_event_type(tool.event_type, tool);
                        }
                        else {
                            new_gestures.multi.tools.push(tool);
                            try {
                                for (var _f = (e_5 = void 0, tslib_1.__values(tool.event_type)), _g = _f.next(); !_g.done; _g = _f.next()) {
                                    var et = _g.value;
                                    check_event_type(et, tool);
                                }
                            }
                            catch (e_5_1) {
                                e_5 = { error: e_5_1 };
                            }
                            finally {
                                try {
                                    if (_g && !_g.done && (_b = _f.return))
                                        _b.call(_f);
                                }
                                finally {
                                    if (e_5)
                                        throw e_5.error;
                                }
                            }
                        }
                    }
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_a = _d.return))
                        _a.call(_d);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            var _loop_1 = function (et) {
                var gm = this_1.gestures[et];
                if (tools_changed(gm.tools, new_gestures[et].tools)) {
                    gm.tools = new_gestures[et].tools;
                }
                if (gm.active && array_1.every(gm.tools, function (t) { return t.id != gm.active.id; })) {
                    gm.active = null;
                }
            };
            var this_1 = this;
            try {
                for (var _h = tslib_1.__values(Object.keys(new_gestures)), _j = _h.next(); !_j.done; _j = _h.next()) {
                    var et = _j.value;
                    _loop_1(et);
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (_j && !_j.done && (_c = _h.return))
                        _c.call(_h);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
        };
        Object.defineProperty(ToolbarBase.prototype, "horizontal", {
            get: function () {
                return this.toolbar_location === "above" || this.toolbar_location === "below";
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ToolbarBase.prototype, "vertical", {
            get: function () {
                return this.toolbar_location === "left" || this.toolbar_location === "right";
            },
            enumerable: true,
            configurable: true
        });
        ToolbarBase.prototype._active_change = function (tool) {
            var e_7, _a;
            var event_type = tool.event_type;
            if (event_type == null)
                return;
            var event_types = types_1.isString(event_type) ? [event_type] : event_type;
            try {
                for (var event_types_1 = tslib_1.__values(event_types), event_types_1_1 = event_types_1.next(); !event_types_1_1.done; event_types_1_1 = event_types_1.next()) {
                    var et = event_types_1_1.value;
                    if (tool.active) {
                        var currently_active_tool = this.gestures[et].active;
                        if (currently_active_tool != null && tool != currently_active_tool) {
                            logging_1.logger.debug("Toolbar: deactivating tool: " + currently_active_tool + " for event type '" + et + "'");
                            currently_active_tool.active = false;
                        }
                        this.gestures[et].active = tool;
                        logging_1.logger.debug("Toolbar: activating tool: " + tool + " for event type '" + et + "'");
                    }
                    else
                        this.gestures[et].active = null;
                }
            }
            catch (e_7_1) {
                e_7 = { error: e_7_1 };
            }
            finally {
                try {
                    if (event_types_1_1 && !event_types_1_1.done && (_a = event_types_1.return))
                        _a.call(event_types_1);
                }
                finally {
                    if (e_7)
                        throw e_7.error;
                }
            }
        };
        return ToolbarBase;
    }(model_1.Model));
    exports.ToolbarBase = ToolbarBase;
    ToolbarBase.__name__ = "ToolbarBase";
    ToolbarBase.init_ToolbarBase();
},
/* models/tools/gestures/gesture_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var button_tool_1 = require(411) /* ../button_tool */;
    var on_off_button_1 = require(419) /* ../on_off_button */;
    var GestureToolView = /** @class */ (function (_super) {
        tslib_1.__extends(GestureToolView, _super);
        function GestureToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return GestureToolView;
    }(button_tool_1.ButtonToolView));
    exports.GestureToolView = GestureToolView;
    GestureToolView.__name__ = "GestureToolView";
    var GestureTool = /** @class */ (function (_super) {
        tslib_1.__extends(GestureTool, _super);
        function GestureTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.button_view = on_off_button_1.OnOffButtonView;
            return _this;
        }
        return GestureTool;
    }(button_tool_1.ButtonTool));
    exports.GestureTool = GestureTool;
    GestureTool.__name__ = "GestureTool";
},
/* models/tools/actions/action_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var button_tool_1 = require(411) /* ../button_tool */;
    var signaling_1 = require(130) /* ../../../core/signaling */;
    var ActionToolButtonView = /** @class */ (function (_super) {
        tslib_1.__extends(ActionToolButtonView, _super);
        function ActionToolButtonView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ActionToolButtonView.prototype._clicked = function () {
            this.model.do.emit(undefined);
        };
        return ActionToolButtonView;
    }(button_tool_1.ButtonToolButtonView));
    exports.ActionToolButtonView = ActionToolButtonView;
    ActionToolButtonView.__name__ = "ActionToolButtonView";
    var ActionToolView = /** @class */ (function (_super) {
        tslib_1.__extends(ActionToolView, _super);
        function ActionToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ActionToolView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.do, function (arg) { return _this.doit(arg); });
        };
        return ActionToolView;
    }(button_tool_1.ButtonToolView));
    exports.ActionToolView = ActionToolView;
    ActionToolView.__name__ = "ActionToolView";
    var ActionTool = /** @class */ (function (_super) {
        tslib_1.__extends(ActionTool, _super);
        function ActionTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.button_view = ActionToolButtonView;
            _this.do = new signaling_1.Signal(_this, "do");
            return _this;
        }
        return ActionTool;
    }(button_tool_1.ButtonTool));
    exports.ActionTool = ActionTool;
    ActionTool.__name__ = "ActionTool";
},
/* models/tools/actions/help_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var action_tool_1 = require(422) /* ./action_tool */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var icons_1 = require(424) /* ../../../styles/icons */;
    var HelpToolView = /** @class */ (function (_super) {
        tslib_1.__extends(HelpToolView, _super);
        function HelpToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        HelpToolView.prototype.doit = function () {
            window.open(this.model.redirect);
        };
        return HelpToolView;
    }(action_tool_1.ActionToolView));
    exports.HelpToolView = HelpToolView;
    HelpToolView.__name__ = "HelpToolView";
    var HelpTool = /** @class */ (function (_super) {
        tslib_1.__extends(HelpTool, _super);
        function HelpTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Help";
            _this.icon = icons_1.bk_tool_icon_help;
            return _this;
        }
        HelpTool.init_HelpTool = function () {
            this.prototype.default_view = HelpToolView;
            this.define({
                help_tooltip: [p.String, 'Click the question mark to learn more about Bokeh plot tools.'],
                redirect: [p.String, 'https://docs.bokeh.org/en/latest/docs/user_guide/tools.html'],
            });
            this.register_alias("help", function () { return new HelpTool(); });
        };
        Object.defineProperty(HelpTool.prototype, "tooltip", {
            get: function () {
                return this.help_tooltip;
            },
            enumerable: true,
            configurable: true
        });
        return HelpTool;
    }(action_tool_1.ActionTool));
    exports.HelpTool = HelpTool;
    HelpTool.__name__ = "HelpTool";
    HelpTool.init_HelpTool();
},
/* styles/icons.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.bk_tool_icon_box_select = "bk-tool-icon-box-select";
    exports.bk_tool_icon_box_zoom = "bk-tool-icon-box-zoom";
    exports.bk_tool_icon_zoom_in = "bk-tool-icon-zoom-in";
    exports.bk_tool_icon_zoom_out = "bk-tool-icon-zoom-out";
    exports.bk_tool_icon_help = "bk-tool-icon-help";
    exports.bk_tool_icon_hover = "bk-tool-icon-hover";
    exports.bk_tool_icon_crosshair = "bk-tool-icon-crosshair";
    exports.bk_tool_icon_lasso_select = "bk-tool-icon-lasso-select";
    exports.bk_tool_icon_pan = "bk-tool-icon-pan";
    exports.bk_tool_icon_xpan = "bk-tool-icon-xpan";
    exports.bk_tool_icon_ypan = "bk-tool-icon-ypan";
    exports.bk_tool_icon_range = "bk-tool-icon-range";
    exports.bk_tool_icon_polygon_select = "bk-tool-icon-polygon-select";
    exports.bk_tool_icon_redo = "bk-tool-icon-redo";
    exports.bk_tool_icon_reset = "bk-tool-icon-reset";
    exports.bk_tool_icon_save = "bk-tool-icon-save";
    exports.bk_tool_icon_tap_select = "bk-tool-icon-tap-select";
    exports.bk_tool_icon_undo = "bk-tool-icon-undo";
    exports.bk_tool_icon_wheel_pan = "bk-tool-icon-wheel-pan";
    exports.bk_tool_icon_wheel_zoom = "bk-tool-icon-wheel-zoom";
    exports.bk_tool_icon_box_edit = "bk-tool-icon-box-edit";
    exports.bk_tool_icon_freehand_draw = "bk-tool-icon-freehand-draw";
    exports.bk_tool_icon_poly_draw = "bk-tool-icon-poly-draw";
    exports.bk_tool_icon_point_draw = "bk-tool-icon-point-draw";
    exports.bk_tool_icon_poly_edit = "bk-tool-icon-poly-edit";
    exports.bk_tool_icon_line_edit = "bk-tool-icon-line-edit";
},
/* styles/logo.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.bk_logo = "bk-logo";
    exports.bk_logo_notebook = "bk-logo-notebook";
    exports.bk_logo_small = "bk-logo-small";
    exports.bk_grey = "bk-grey";
},
/* styles/logo.css.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var css = "\n.bk-root .bk-logo {\n  margin: 5px;\n  position: relative;\n  display: block;\n  background-repeat: no-repeat;\n}\n.bk-root .bk-logo.bk-grey {\n  filter: url(\"data:image/svg+xml;utf8,<svg xmlns='http://www.w3.org/2000/svg'><filter id='grayscale'><feColorMatrix type='matrix' values='0.3333 0.3333 0.3333 0 0 0.3333 0.3333 0.3333 0 0 0.3333 0.3333 0.3333 0 0 0 0 0 1 0'/></filter></svg>#grayscale\");\n  /* Firefox 10+, Firefox on Android */\n  filter: gray;\n  /* IE6-9 */\n  -webkit-filter: grayscale(100%);\n  /* Chrome 19+, Safari 6+, Safari 6+ iOS */\n}\n.bk-root .bk-logo-small {\n  width: 20px;\n  height: 20px;\n  background-image: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlwSFlzAAALEgAACxIB0t1+/AAAABx0RVh0U29mdHdhcmUAQWRvYmUgRmlyZXdvcmtzIENTNui8sowAAAOkSURBVDiNjZRtaJVlGMd/1/08zzln5zjP1LWcU9N0NkN8m2CYjpgQYQXqSs0I84OLIC0hkEKoPtiH3gmKoiJDU7QpLgoLjLIQCpEsNJ1vqUOdO7ppbuec5+V+rj4ctwzd8IIbbi6u+8f1539dt3A78eXC7QizUF7gyV1fD1Yqg4JWz84yffhm0qkFqBogB9rM8tZdtwVsPUhWhGcFJngGeWrPzHm5oaMmkfEg1usvLFyc8jLRqDOMru7AyC8saQr7GG7f5fvDeH7Ej8CM66nIF+8yngt6HWaKh7k49Soy9nXurCi1o3qUbS3zWfrYeQDTB/Qj6kX6Ybhw4B+bOYoLKCC9H3Nu/leUTZ1JdRWkkn2ldcCamzrcf47KKXdAJllSlxAOkRgyHsGC/zRday5Qld9DyoM4/q/rUoy/CXh3jzOu3bHUVZeU+DEn8FInkPBFlu3+nW3Nw0mk6vCDiWg8CeJaxEwuHS3+z5RgY+YBR6V1Z1nxSOfoaPa4LASWxxdNp+VWTk7+4vzaou8v8PN+xo+KY2xsw6une2frhw05CTYOmQvsEhjhWjn0bmXPjpE1+kplmmkP3suftwTubK9Vq22qKmrBhpY4jvd5afdRA3wGjFAgcnTK2s4hY0/GPNIb0nErGMCRxWOOX64Z8RAC4oCXdklmEvcL8o0BfkNK4lUg9HTl+oPlQxdNo3Mg4Nv175e/1LDGzZen30MEjRUtmXSfiTVu1kK8W4txyV6BMKlbgk3lMwYCiusNy9fVfvvwMxv8Ynl6vxoByANLTWplvuj/nF9m2+PDtt1eiHPBr1oIfhCChQMBw6Aw0UulqTKZdfVvfG7VcfIqLG9bcldL/+pdWTLxLUy8Qq38heUIjh4XlzZxzQm19lLFlr8vdQ97rjZVOLf8nclzckbcD4wxXMidpX30sFd37Fv/GtwwhzhxGVAprjbg0gCAEeIgwCZyTV2Z1REEW8O4py0wsjeloKoMr6iCY6dP92H6Vw/oTyICIthibxjm/DfN9lVz8IqtqKYLUXfoKVMVQVVJOElGjrnnUt9T9wbgp8AyYKaGlqingHZU/uG2NTZSVqwHQTWkx9hxjkpWDaCg6Ckj5qebgBVbT3V3NNXMSiWSDdGV3hrtzla7J+duwPOToIg42ChPQOQjspnSlp1V+Gjdged7+8UN5CRAV7a5EdFNwCjEaBR27b3W890TE7g24NAP/mMDXRWrGoFPQI9ls/MWO2dWFAar/xcOIImbbpA3zgAAAABJRU5ErkJggg==);\n}\n.bk-root .bk-logo-notebook {\n  display: inline-block;\n  vertical-align: middle;\n  margin-right: 5px;\n}\n";
    exports.default = css;
},
/* models/plots/plot_canvas.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var __rest = (this && this.__rest) || function (s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    };
    var cartesian_frame_1 = require(323) /* ../canvas/cartesian_frame */;
    var canvas_1 = require(192) /* ../canvas/canvas */;
    var data_range1d_1 = require(325) /* ../ranges/data_range1d */;
    var glyph_renderer_1 = require(205) /* ../renderers/glyph_renderer */;
    var layout_dom_1 = require(387) /* ../layouts/layout_dom */;
    var title_1 = require(284) /* ../annotations/title */;
    var axis_1 = require(292) /* ../axes/axis */;
    var toolbar_panel_1 = require(285) /* ../annotations/toolbar_panel */;
    var bokeh_events_1 = require(428) /* ../../core/bokeh_events */;
    var signaling_1 = require(130) /* ../../core/signaling */;
    var build_views_1 = require(230) /* ../../core/build_views */;
    var ui_events_1 = require(429) /* ../../core/ui_events */;
    var visuals_1 = require(189) /* ../../core/visuals */;
    var logging_1 = require(134) /* ../../core/logging */;
    var throttle_1 = require(431) /* ../../core/util/throttle */;
    var types_1 = require(123) /* ../../core/util/types */;
    var array_1 = require(124) /* ../../core/util/array */;
    var alignments_1 = require(330) /* ../../core/layout/alignments */;
    var border_1 = require(432) /* ../../core/layout/border */;
    var side_panel_1 = require(433) /* ../../core/layout/side_panel */;
    var grid_1 = require(331) /* ../../core/layout/grid */;
    var bbox_1 = require(194) /* ../../core/util/bbox */;
    var PlotView = /** @class */ (function (_super) {
        tslib_1.__extends(PlotView, _super);
        function PlotView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this._outer_bbox = new bbox_1.BBox();
            _this._inner_bbox = new bbox_1.BBox();
            _this._needs_paint = true;
            _this._needs_layout = false;
            _this._invalidated_painters = new Set();
            _this._invalidate_all = true;
            return _this;
        }
        Object.defineProperty(PlotView.prototype, "is_paused", {
            get: function () {
                return this._is_paused != null && this._is_paused !== 0;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(PlotView.prototype, "child_models", {
            get: function () {
                return [];
            },
            enumerable: true,
            configurable: true
        });
        PlotView.prototype.pause = function () {
            if (this._is_paused == null)
                this._is_paused = 1;
            else
                this._is_paused += 1;
        };
        PlotView.prototype.unpause = function (no_render) {
            if (no_render === void 0) {
                no_render = false;
            }
            if (this._is_paused == null)
                throw new Error("wasn't paused");
            this._is_paused -= 1;
            if (this._is_paused == 0 && !no_render)
                this.request_paint();
        };
        // TODO: this needs to be removed
        PlotView.prototype.request_render = function () {
            this.request_paint();
        };
        PlotView.prototype.request_paint = function (to_invalidate) {
            if (to_invalidate != null) {
                this._invalidated_painters.add(to_invalidate);
            }
            else {
                this._invalidate_all = true;
            }
            if (!this.is_paused) {
                var promise_1 = this.throttled_paint();
                this._ready = this._ready.then(function () { return promise_1; });
            }
        };
        PlotView.prototype.request_layout = function () {
            this._needs_layout = true;
            this.request_paint();
        };
        PlotView.prototype.reset = function () {
            if (this.model.reset_policy == "standard") {
                this.clear_state();
                this.reset_range();
                this.reset_selection();
            }
            this.model.trigger_event(new bokeh_events_1.Reset());
        };
        PlotView.prototype.remove = function () {
            this.ui_event_bus.destroy();
            build_views_1.remove_views(this.renderer_views);
            build_views_1.remove_views(this.tool_views);
            this.canvas_view.remove();
            _super.prototype.remove.call(this);
        };
        PlotView.prototype.render = function () {
            _super.prototype.render.call(this);
            this.el.appendChild(this.canvas_view.el);
            this.canvas_view.render();
        };
        PlotView.prototype.initialize = function () {
            var _this = this;
            this.pause();
            _super.prototype.initialize.call(this);
            this.state_changed = new signaling_1.Signal0(this, "state_changed");
            this.lod_started = false;
            this.visuals = new visuals_1.Visuals(this.model); // XXX
            this._initial_state_info = {
                selection: new Map(),
                dimensions: { width: 0, height: 0 },
            };
            this.visibility_callbacks = [];
            this.state = { history: [], index: -1 };
            var _b = this.model, hidpi = _b.hidpi, output_backend = _b.output_backend;
            this.canvas = new canvas_1.Canvas({ hidpi: hidpi, output_backend: output_backend });
            this.frame = new cartesian_frame_1.CartesianFrame(this.model.x_scale, this.model.y_scale, this.model.x_range, this.model.y_range, this.model.extra_x_ranges, this.model.extra_y_ranges);
            this.throttled_paint = throttle_1.throttle(function () { return _this.repaint(); }, 1000 / 60);
            var _c = this.model, title_location = _c.title_location, title = _c.title;
            if (title_location != null && title != null) {
                this._title = title instanceof title_1.Title ? title : new title_1.Title({ text: title });
            }
            var _d = this.model, toolbar_location = _d.toolbar_location, toolbar = _d.toolbar;
            if (toolbar_location != null && toolbar != null) {
                this._toolbar = new toolbar_panel_1.ToolbarPanel({ toolbar: toolbar });
                toolbar.toolbar_location = toolbar_location;
            }
            this.renderer_views = new Map();
            this.tool_views = new Map();
        };
        PlotView.prototype.lazy_initialize = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var _b;
                return tslib_1.__generator(this, function (_c) {
                    switch (_c.label) {
                        case 0:
                            _b = this;
                            return [4 /*yield*/, build_views_1.build_view(this.canvas, { parent: this })];
                        case 1:
                            _b.canvas_view = _c.sent();
                            this.ui_event_bus = new ui_events_1.UIEvents(this, this.model.toolbar, this.canvas_view.events_el);
                            return [4 /*yield*/, this.build_renderer_views()];
                        case 2:
                            _c.sent();
                            return [4 /*yield*/, this.build_tool_views()];
                        case 3:
                            _c.sent();
                            this.update_dataranges();
                            this.unpause(true);
                            logging_1.logger.debug("PlotView initialized");
                            return [2 /*return*/];
                    }
                });
            });
        };
        PlotView.prototype._width_policy = function () {
            return this.model.frame_width == null ? _super.prototype._width_policy.call(this) : "min";
        };
        PlotView.prototype._height_policy = function () {
            return this.model.frame_height == null ? _super.prototype._height_policy.call(this) : "min";
        };
        PlotView.prototype._update_layout = function () {
            var _this = this;
            this.layout = new border_1.BorderLayout();
            this.layout.set_sizing(this.box_sizing());
            var _b = this.model, frame_width = _b.frame_width, frame_height = _b.frame_height;
            this.layout.center_panel = this.frame;
            this.layout.center_panel.set_sizing(Object.assign(Object.assign({}, (frame_width != null ? { width_policy: "fixed", width: frame_width } : { width_policy: "fit" })), (frame_height != null ? { height_policy: "fixed", height: frame_height } : { height_policy: "fit" })));
            var above = array_1.copy(this.model.above);
            var below = array_1.copy(this.model.below);
            var left = array_1.copy(this.model.left);
            var right = array_1.copy(this.model.right);
            var get_side = function (side) {
                switch (side) {
                    case "above": return above;
                    case "below": return below;
                    case "left": return left;
                    case "right": return right;
                }
            };
            var _c = this.model, title_location = _c.title_location, title = _c.title;
            if (title_location != null && title != null) {
                get_side(title_location).push(this._title);
            }
            var _d = this.model, toolbar_location = _d.toolbar_location, toolbar = _d.toolbar;
            if (toolbar_location != null && toolbar != null) {
                var panels = get_side(toolbar_location);
                var push_toolbar = true;
                if (this.model.toolbar_sticky) {
                    for (var i = 0; i < panels.length; i++) {
                        var panel = panels[i];
                        if (panel instanceof title_1.Title) {
                            if (toolbar_location == "above" || toolbar_location == "below")
                                panels[i] = [panel, this._toolbar];
                            else
                                panels[i] = [this._toolbar, panel];
                            push_toolbar = false;
                            break;
                        }
                    }
                }
                if (push_toolbar)
                    panels.push(this._toolbar);
            }
            var set_layout = function (side, model) {
                var view = _this.renderer_views.get(model);
                return view.layout = new side_panel_1.SidePanel(side, view);
            };
            var set_layouts = function (side, panels) {
                var e_1, _b;
                var horizontal = side == "above" || side == "below";
                var layouts = [];
                try {
                    for (var panels_1 = tslib_1.__values(panels), panels_1_1 = panels_1.next(); !panels_1_1.done; panels_1_1 = panels_1.next()) {
                        var panel = panels_1_1.value;
                        if (types_1.isArray(panel)) {
                            var items = panel.map(function (subpanel) {
                                var _b;
                                var item = set_layout(side, subpanel);
                                if (subpanel instanceof toolbar_panel_1.ToolbarPanel) {
                                    var dim = horizontal ? "width_policy" : "height_policy";
                                    item.set_sizing(Object.assign(Object.assign({}, item.sizing), (_b = {}, _b[dim] = "min", _b)));
                                }
                                return item;
                            });
                            var layout = void 0;
                            if (horizontal) {
                                layout = new grid_1.Row(items);
                                layout.set_sizing({ width_policy: "max", height_policy: "min" });
                            }
                            else {
                                layout = new grid_1.Column(items);
                                layout.set_sizing({ width_policy: "min", height_policy: "max" });
                            }
                            layout.absolute = true;
                            layouts.push(layout);
                        }
                        else
                            layouts.push(set_layout(side, panel));
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (panels_1_1 && !panels_1_1.done && (_b = panels_1.return))
                            _b.call(panels_1);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
                return layouts;
            };
            var min_border = this.model.min_border != null ? this.model.min_border : 0;
            this.layout.min_border = {
                left: this.model.min_border_left != null ? this.model.min_border_left : min_border,
                top: this.model.min_border_top != null ? this.model.min_border_top : min_border,
                right: this.model.min_border_right != null ? this.model.min_border_right : min_border,
                bottom: this.model.min_border_bottom != null ? this.model.min_border_bottom : min_border,
            };
            var top_panel = new alignments_1.VStack();
            var bottom_panel = new alignments_1.VStack();
            var left_panel = new alignments_1.HStack();
            var right_panel = new alignments_1.HStack();
            top_panel.children = array_1.reversed(set_layouts("above", above));
            bottom_panel.children = set_layouts("below", below);
            left_panel.children = array_1.reversed(set_layouts("left", left));
            right_panel.children = set_layouts("right", right);
            top_panel.set_sizing({ width_policy: "fit", height_policy: "min" /*, min_height: this.layout.min_border.top*/ });
            bottom_panel.set_sizing({ width_policy: "fit", height_policy: "min" /*, min_height: this.layout.min_width.bottom*/ });
            left_panel.set_sizing({ width_policy: "min", height_policy: "fit" /*, min_width: this.layout.min_width.left*/ });
            right_panel.set_sizing({ width_policy: "min", height_policy: "fit" /*, min_width: this.layout.min_width.right*/ });
            this.layout.top_panel = top_panel;
            this.layout.bottom_panel = bottom_panel;
            this.layout.left_panel = left_panel;
            this.layout.right_panel = right_panel;
        };
        Object.defineProperty(PlotView.prototype, "axis_views", {
            get: function () {
                var e_2, _b;
                var views = [];
                try {
                    for (var _c = tslib_1.__values(this.renderer_views), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var _e = tslib_1.__read(_d.value, 2), renderer_view = _e[1];
                        if (renderer_view instanceof axis_1.AxisView)
                            views.push(renderer_view);
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (_d && !_d.done && (_b = _c.return))
                            _b.call(_c);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
                return views;
            },
            enumerable: true,
            configurable: true
        });
        PlotView.prototype.set_cursor = function (cursor) {
            if (cursor === void 0) {
                cursor = "default";
            }
            this.canvas_view.el.style.cursor = cursor;
        };
        PlotView.prototype.set_toolbar_visibility = function (visible) {
            var e_3, _b;
            try {
                for (var _c = tslib_1.__values(this.visibility_callbacks), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var callback = _d.value;
                    callback(visible);
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
        };
        PlotView.prototype.update_dataranges = function () {
            var e_4, _b, e_5, _c, e_6, _d, e_7, _e, e_8, _f, e_9, _g, e_10, _h;
            // Update any DataRange1ds here
            var bounds = new Map();
            var log_bounds = new Map();
            var calculate_log_bounds = false;
            try {
                for (var _j = tslib_1.__values(this.frame.x_ranges), _k = _j.next(); !_k.done; _k = _j.next()) {
                    var _l = tslib_1.__read(_k.value, 2), xr = _l[1];
                    if (xr instanceof data_range1d_1.DataRange1d && xr.scale_hint == "log")
                        calculate_log_bounds = true;
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (_k && !_k.done && (_b = _j.return))
                        _b.call(_j);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            try {
                for (var _m = tslib_1.__values(this.frame.y_ranges), _o = _m.next(); !_o.done; _o = _m.next()) {
                    var _p = tslib_1.__read(_o.value, 2), yr = _p[1];
                    if (yr instanceof data_range1d_1.DataRange1d && yr.scale_hint == "log")
                        calculate_log_bounds = true;
                }
            }
            catch (e_5_1) {
                e_5 = { error: e_5_1 };
            }
            finally {
                try {
                    if (_o && !_o.done && (_c = _m.return))
                        _c.call(_m);
                }
                finally {
                    if (e_5)
                        throw e_5.error;
                }
            }
            try {
                for (var _q = tslib_1.__values(this.renderer_views), _r = _q.next(); !_r.done; _r = _q.next()) {
                    var _s = tslib_1.__read(_r.value, 2), renderer = _s[0], renderer_view = _s[1];
                    if (renderer_view instanceof glyph_renderer_1.GlyphRendererView) {
                        var bds = renderer_view.glyph.bounds();
                        if (bds != null)
                            bounds.set(renderer, bds);
                        if (calculate_log_bounds) {
                            var log_bds = renderer_view.glyph.log_bounds();
                            if (log_bds != null)
                                log_bounds.set(renderer, log_bds);
                        }
                    }
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (_r && !_r.done && (_d = _q.return))
                        _d.call(_q);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
            var follow_enabled = false;
            var has_bounds = false;
            var _t = this.frame.bbox, width = _t.width, height = _t.height;
            var r;
            if (this.model.match_aspect !== false && width != 0 && height != 0)
                r = (1 / this.model.aspect_scale) * (width / height);
            try {
                for (var _u = tslib_1.__values(this.frame.x_ranges), _v = _u.next(); !_v.done; _v = _u.next()) {
                    var _w = tslib_1.__read(_v.value, 2), xr = _w[1];
                    if (xr instanceof data_range1d_1.DataRange1d) {
                        var bounds_to_use = xr.scale_hint == "log" ? log_bounds : bounds;
                        xr.update(bounds_to_use, 0, this.model, r);
                        if (xr.follow) {
                            follow_enabled = true;
                        }
                    }
                    if (xr.bounds != null)
                        has_bounds = true;
                }
            }
            catch (e_7_1) {
                e_7 = { error: e_7_1 };
            }
            finally {
                try {
                    if (_v && !_v.done && (_e = _u.return))
                        _e.call(_u);
                }
                finally {
                    if (e_7)
                        throw e_7.error;
                }
            }
            try {
                for (var _x = tslib_1.__values(this.frame.y_ranges), _y = _x.next(); !_y.done; _y = _x.next()) {
                    var _z = tslib_1.__read(_y.value, 2), yr = _z[1];
                    if (yr instanceof data_range1d_1.DataRange1d) {
                        var bounds_to_use = yr.scale_hint == "log" ? log_bounds : bounds;
                        yr.update(bounds_to_use, 1, this.model, r);
                        if (yr.follow) {
                            follow_enabled = true;
                        }
                    }
                    if (yr.bounds != null)
                        has_bounds = true;
                }
            }
            catch (e_8_1) {
                e_8 = { error: e_8_1 };
            }
            finally {
                try {
                    if (_y && !_y.done && (_f = _x.return))
                        _f.call(_x);
                }
                finally {
                    if (e_8)
                        throw e_8.error;
                }
            }
            if (follow_enabled && has_bounds) {
                logging_1.logger.warn('Follow enabled so bounds are unset.');
                try {
                    for (var _0 = tslib_1.__values(this.frame.x_ranges), _1 = _0.next(); !_1.done; _1 = _0.next()) {
                        var _2 = tslib_1.__read(_1.value, 2), xr = _2[1];
                        xr.bounds = null;
                    }
                }
                catch (e_9_1) {
                    e_9 = { error: e_9_1 };
                }
                finally {
                    try {
                        if (_1 && !_1.done && (_g = _0.return))
                            _g.call(_0);
                    }
                    finally {
                        if (e_9)
                            throw e_9.error;
                    }
                }
                try {
                    for (var _3 = tslib_1.__values(this.frame.y_ranges), _4 = _3.next(); !_4.done; _4 = _3.next()) {
                        var _5 = tslib_1.__read(_4.value, 2), yr = _5[1];
                        yr.bounds = null;
                    }
                }
                catch (e_10_1) {
                    e_10 = { error: e_10_1 };
                }
                finally {
                    try {
                        if (_4 && !_4.done && (_h = _3.return))
                            _h.call(_3);
                    }
                    finally {
                        if (e_10)
                            throw e_10.error;
                    }
                }
            }
            this.range_update_timestamp = Date.now();
        };
        PlotView.prototype.push_state = function (type, new_info) {
            var _b = this.state, history = _b.history, index = _b.index;
            var prev_info = history[index] != null ? history[index].info : {};
            var info = Object.assign(Object.assign(Object.assign({}, this._initial_state_info), prev_info), new_info);
            this.state.history = this.state.history.slice(0, this.state.index + 1);
            this.state.history.push({ type: type, info: info });
            this.state.index = this.state.history.length - 1;
            this.state_changed.emit();
        };
        PlotView.prototype.clear_state = function () {
            this.state = { history: [], index: -1 };
            this.state_changed.emit();
        };
        PlotView.prototype.can_undo = function () {
            return this.state.index >= 0;
        };
        PlotView.prototype.can_redo = function () {
            return this.state.index < this.state.history.length - 1;
        };
        PlotView.prototype.undo = function () {
            if (this.can_undo()) {
                this.state.index -= 1;
                this._do_state_change(this.state.index);
                this.state_changed.emit();
            }
        };
        PlotView.prototype.redo = function () {
            if (this.can_redo()) {
                this.state.index += 1;
                this._do_state_change(this.state.index);
                this.state_changed.emit();
            }
        };
        PlotView.prototype._do_state_change = function (index) {
            var info = this.state.history[index] != null ? this.state.history[index].info : this._initial_state_info;
            if (info.range != null)
                this.update_range(info.range);
            if (info.selection != null)
                this.update_selection(info.selection);
        };
        PlotView.prototype.get_selection = function () {
            var e_11, _b;
            var selection = new Map();
            try {
                for (var _c = tslib_1.__values(this.model.renderers), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var renderer = _d.value;
                    if (renderer instanceof glyph_renderer_1.GlyphRenderer) {
                        var selected = renderer.data_source.selected;
                        selection.set(renderer, selected);
                    }
                }
            }
            catch (e_11_1) {
                e_11 = { error: e_11_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_11)
                        throw e_11.error;
                }
            }
            return selection;
        };
        PlotView.prototype.update_selection = function (selections) {
            var e_12, _b;
            try {
                for (var _c = tslib_1.__values(this.model.renderers), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var renderer = _d.value;
                    if (!(renderer instanceof glyph_renderer_1.GlyphRenderer))
                        continue;
                    var ds = renderer.data_source;
                    if (selections != null) {
                        var selection = selections.get(renderer);
                        if (selection != null) {
                            ds.selected.update(selection, true);
                        }
                    }
                    else
                        ds.selection_manager.clear();
                }
            }
            catch (e_12_1) {
                e_12 = { error: e_12_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_12)
                        throw e_12.error;
                }
            }
        };
        PlotView.prototype.reset_selection = function () {
            this.update_selection(null);
        };
        PlotView.prototype._update_ranges_together = function (range_info_iter) {
            var e_13, _b, e_14, _c;
            // Get weight needed to scale the diff of the range to honor interval limits
            var weight = 1.0;
            try {
                for (var range_info_iter_1 = tslib_1.__values(range_info_iter), range_info_iter_1_1 = range_info_iter_1.next(); !range_info_iter_1_1.done; range_info_iter_1_1 = range_info_iter_1.next()) {
                    var _d = tslib_1.__read(range_info_iter_1_1.value, 2), rng = _d[0], range_info = _d[1];
                    weight = Math.min(weight, this._get_weight_to_constrain_interval(rng, range_info));
                }
            }
            catch (e_13_1) {
                e_13 = { error: e_13_1 };
            }
            finally {
                try {
                    if (range_info_iter_1_1 && !range_info_iter_1_1.done && (_b = range_info_iter_1.return))
                        _b.call(range_info_iter_1);
                }
                finally {
                    if (e_13)
                        throw e_13.error;
                }
            }
            // Apply shared weight to all ranges
            if (weight < 1) {
                try {
                    for (var range_info_iter_2 = tslib_1.__values(range_info_iter), range_info_iter_2_1 = range_info_iter_2.next(); !range_info_iter_2_1.done; range_info_iter_2_1 = range_info_iter_2.next()) {
                        var _e = tslib_1.__read(range_info_iter_2_1.value, 2), rng = _e[0], range_info = _e[1];
                        range_info.start = weight * range_info.start + (1 - weight) * rng.start;
                        range_info.end = weight * range_info.end + (1 - weight) * rng.end;
                    }
                }
                catch (e_14_1) {
                    e_14 = { error: e_14_1 };
                }
                finally {
                    try {
                        if (range_info_iter_2_1 && !range_info_iter_2_1.done && (_c = range_info_iter_2.return))
                            _c.call(range_info_iter_2);
                    }
                    finally {
                        if (e_14)
                            throw e_14.error;
                    }
                }
            }
        };
        PlotView.prototype._update_ranges_individually = function (range_info_iter, is_panning, is_scrolling, maintain_focus) {
            var e_15, _b, e_16, _c;
            var hit_bound = false;
            try {
                for (var range_info_iter_3 = tslib_1.__values(range_info_iter), range_info_iter_3_1 = range_info_iter_3.next(); !range_info_iter_3_1.done; range_info_iter_3_1 = range_info_iter_3.next()) {
                    var _d = tslib_1.__read(range_info_iter_3_1.value, 2), rng = _d[0], range_info = _d[1];
                    // Limit range interval first. Note that for scroll events,
                    // the interval has already been limited for all ranges simultaneously
                    if (!is_scrolling) {
                        var weight = this._get_weight_to_constrain_interval(rng, range_info);
                        if (weight < 1) {
                            range_info.start = weight * range_info.start + (1 - weight) * rng.start;
                            range_info.end = weight * range_info.end + (1 - weight) * rng.end;
                        }
                    }
                    // Prevent range from going outside limits
                    // Also ensure that range keeps the same delta when panning/scrolling
                    if (rng.bounds != null && rng.bounds != "auto") { // check `auto` for type-checking purpose
                        var _e = tslib_1.__read(rng.bounds, 2), min = _e[0], max = _e[1];
                        var new_interval = Math.abs(range_info.end - range_info.start);
                        if (rng.is_reversed) {
                            if (min != null) {
                                if (min >= range_info.end) {
                                    hit_bound = true;
                                    range_info.end = min;
                                    if (is_panning || is_scrolling) {
                                        range_info.start = min + new_interval;
                                    }
                                }
                            }
                            if (max != null) {
                                if (max <= range_info.start) {
                                    hit_bound = true;
                                    range_info.start = max;
                                    if (is_panning || is_scrolling) {
                                        range_info.end = max - new_interval;
                                    }
                                }
                            }
                        }
                        else {
                            if (min != null) {
                                if (min >= range_info.start) {
                                    hit_bound = true;
                                    range_info.start = min;
                                    if (is_panning || is_scrolling) {
                                        range_info.end = min + new_interval;
                                    }
                                }
                            }
                            if (max != null) {
                                if (max <= range_info.end) {
                                    hit_bound = true;
                                    range_info.end = max;
                                    if (is_panning || is_scrolling) {
                                        range_info.start = max - new_interval;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (e_15_1) {
                e_15 = { error: e_15_1 };
            }
            finally {
                try {
                    if (range_info_iter_3_1 && !range_info_iter_3_1.done && (_b = range_info_iter_3.return))
                        _b.call(range_info_iter_3);
                }
                finally {
                    if (e_15)
                        throw e_15.error;
                }
            }
            // Cancel the event when hitting a bound while scrolling. This ensures that
            // the scroll-zoom tool maintains its focus position. Setting `maintain_focus`
            // to false results in a more "gliding" behavior, allowing one to
            // zoom out more smoothly, at the cost of losing the focus position.
            if (is_scrolling && hit_bound && maintain_focus)
                return;
            try {
                for (var range_info_iter_4 = tslib_1.__values(range_info_iter), range_info_iter_4_1 = range_info_iter_4.next(); !range_info_iter_4_1.done; range_info_iter_4_1 = range_info_iter_4.next()) {
                    var _f = tslib_1.__read(range_info_iter_4_1.value, 2), rng = _f[0], range_info = _f[1];
                    rng.have_updated_interactively = true;
                    if (rng.start != range_info.start || rng.end != range_info.end)
                        rng.setv(range_info);
                }
            }
            catch (e_16_1) {
                e_16 = { error: e_16_1 };
            }
            finally {
                try {
                    if (range_info_iter_4_1 && !range_info_iter_4_1.done && (_c = range_info_iter_4.return))
                        _c.call(range_info_iter_4);
                }
                finally {
                    if (e_16)
                        throw e_16.error;
                }
            }
        };
        PlotView.prototype._get_weight_to_constrain_interval = function (rng, range_info) {
            // Get the weight by which a range-update can be applied
            // to still honor the interval limits (including the implicit
            // max interval imposed by the bounds)
            var min_interval = rng.min_interval;
            var max_interval = rng.max_interval;
            // Express bounds as a max_interval. By doing this, the application of
            // bounds and interval limits can be applied independent from each-other.
            if (rng.bounds != null && rng.bounds != "auto") { // check `auto` for type-checking purpose
                var _b = tslib_1.__read(rng.bounds, 2), min = _b[0], max = _b[1];
                if (min != null && max != null) {
                    var max_interval2 = Math.abs(max - min);
                    max_interval = max_interval != null ? Math.min(max_interval, max_interval2) : max_interval2;
                }
            }
            var weight = 1.0;
            if (min_interval != null || max_interval != null) {
                var old_interval = Math.abs(rng.end - rng.start);
                var new_interval = Math.abs(range_info.end - range_info.start);
                if (min_interval > 0 && new_interval < min_interval) {
                    weight = (old_interval - min_interval) / (old_interval - new_interval);
                }
                if (max_interval > 0 && new_interval > max_interval) {
                    weight = (max_interval - old_interval) / (new_interval - old_interval);
                }
                weight = Math.max(0.0, Math.min(1.0, weight));
            }
            return weight;
        };
        PlotView.prototype.update_range = function (range_info, is_panning, is_scrolling, maintain_focus) {
            var e_17, _b, e_18, _c, e_19, _d, e_20, _e;
            if (is_panning === void 0) {
                is_panning = false;
            }
            if (is_scrolling === void 0) {
                is_scrolling = false;
            }
            if (maintain_focus === void 0) {
                maintain_focus = true;
            }
            this.pause();
            var _f = this.frame, x_ranges = _f.x_ranges, y_ranges = _f.y_ranges;
            if (range_info == null) {
                try {
                    for (var x_ranges_1 = tslib_1.__values(x_ranges), x_ranges_1_1 = x_ranges_1.next(); !x_ranges_1_1.done; x_ranges_1_1 = x_ranges_1.next()) {
                        var _g = tslib_1.__read(x_ranges_1_1.value, 2), range = _g[1];
                        range.reset();
                    }
                }
                catch (e_17_1) {
                    e_17 = { error: e_17_1 };
                }
                finally {
                    try {
                        if (x_ranges_1_1 && !x_ranges_1_1.done && (_b = x_ranges_1.return))
                            _b.call(x_ranges_1);
                    }
                    finally {
                        if (e_17)
                            throw e_17.error;
                    }
                }
                try {
                    for (var y_ranges_1 = tslib_1.__values(y_ranges), y_ranges_1_1 = y_ranges_1.next(); !y_ranges_1_1.done; y_ranges_1_1 = y_ranges_1.next()) {
                        var _h = tslib_1.__read(y_ranges_1_1.value, 2), range = _h[1];
                        range.reset();
                    }
                }
                catch (e_18_1) {
                    e_18 = { error: e_18_1 };
                }
                finally {
                    try {
                        if (y_ranges_1_1 && !y_ranges_1_1.done && (_c = y_ranges_1.return))
                            _c.call(y_ranges_1);
                    }
                    finally {
                        if (e_18)
                            throw e_18.error;
                    }
                }
                this.update_dataranges();
            }
            else {
                var range_info_iter = [];
                try {
                    for (var x_ranges_2 = tslib_1.__values(x_ranges), x_ranges_2_1 = x_ranges_2.next(); !x_ranges_2_1.done; x_ranges_2_1 = x_ranges_2.next()) {
                        var _j = tslib_1.__read(x_ranges_2_1.value, 2), name = _j[0], range = _j[1];
                        range_info_iter.push([range, range_info.xrs.get(name)]);
                    }
                }
                catch (e_19_1) {
                    e_19 = { error: e_19_1 };
                }
                finally {
                    try {
                        if (x_ranges_2_1 && !x_ranges_2_1.done && (_d = x_ranges_2.return))
                            _d.call(x_ranges_2);
                    }
                    finally {
                        if (e_19)
                            throw e_19.error;
                    }
                }
                try {
                    for (var y_ranges_2 = tslib_1.__values(y_ranges), y_ranges_2_1 = y_ranges_2.next(); !y_ranges_2_1.done; y_ranges_2_1 = y_ranges_2.next()) {
                        var _k = tslib_1.__read(y_ranges_2_1.value, 2), name = _k[0], range = _k[1];
                        range_info_iter.push([range, range_info.yrs.get(name)]);
                    }
                }
                catch (e_20_1) {
                    e_20 = { error: e_20_1 };
                }
                finally {
                    try {
                        if (y_ranges_2_1 && !y_ranges_2_1.done && (_e = y_ranges_2.return))
                            _e.call(y_ranges_2);
                    }
                    finally {
                        if (e_20)
                            throw e_20.error;
                    }
                }
                if (is_scrolling) {
                    this._update_ranges_together(range_info_iter); // apply interval bounds while keeping aspect
                }
                this._update_ranges_individually(range_info_iter, is_panning, is_scrolling, maintain_focus);
            }
            this.unpause();
        };
        PlotView.prototype.reset_range = function () {
            this.update_range(null);
        };
        PlotView.prototype._invalidate_layout = function () {
            var _this = this;
            var needs_layout = function () {
                var e_21, _b;
                try {
                    for (var _c = tslib_1.__values(_this.model.side_panels), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var panel = _d.value;
                        var view = _this.renderer_views.get(panel);
                        if (view.layout.has_size_changed())
                            return true;
                    }
                }
                catch (e_21_1) {
                    e_21 = { error: e_21_1 };
                }
                finally {
                    try {
                        if (_d && !_d.done && (_b = _c.return))
                            _b.call(_c);
                    }
                    finally {
                        if (e_21)
                            throw e_21.error;
                    }
                }
                return false;
            };
            if (needs_layout())
                this.root.compute_layout();
        };
        PlotView.prototype.get_renderer_views = function () {
            var _this = this;
            return this.computed_renderers.map(function (r) { return _this.renderer_views.get(r); });
        };
        PlotView.prototype.build_renderer_views = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var _b, above, below, left, right, center, renderers, _c, _d, tool;
                var _e, e_22, _f, _g;
                return tslib_1.__generator(this, function (_h) {
                    switch (_h.label) {
                        case 0:
                            this.computed_renderers = [];
                            _b = this.model, above = _b.above, below = _b.below, left = _b.left, right = _b.right, center = _b.center, renderers = _b.renderers;
                            (_e = this.computed_renderers).push.apply(_e, tslib_1.__spread(above, below, left, right, center, renderers));
                            if (this._title != null)
                                this.computed_renderers.push(this._title);
                            if (this._toolbar != null)
                                this.computed_renderers.push(this._toolbar);
                            try {
                                for (_c = tslib_1.__values(this.model.toolbar.tools), _d = _c.next(); !_d.done; _d = _c.next()) {
                                    tool = _d.value;
                                    if (tool.overlay != null)
                                        this.computed_renderers.push(tool.overlay);
                                    (_g = this.computed_renderers).push.apply(_g, tslib_1.__spread(tool.synthetic_renderers));
                                }
                            }
                            catch (e_22_1) {
                                e_22 = { error: e_22_1 };
                            }
                            finally {
                                try {
                                    if (_d && !_d.done && (_f = _c.return))
                                        _f.call(_c);
                                }
                                finally {
                                    if (e_22)
                                        throw e_22.error;
                                }
                            }
                            return [4 /*yield*/, build_views_1.build_views(this.renderer_views, this.computed_renderers, { parent: this })];
                        case 1:
                            _h.sent();
                            return [2 /*return*/];
                    }
                });
            });
        };
        PlotView.prototype.build_tool_views = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var tool_models, new_tool_views;
                var _this = this;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            tool_models = this.model.toolbar.tools;
                            return [4 /*yield*/, build_views_1.build_views(this.tool_views, tool_models, { parent: this })];
                        case 1:
                            new_tool_views = _b.sent();
                            new_tool_views.map(function (tool_view) { return _this.ui_event_bus.register_tool(tool_view); });
                            return [2 /*return*/];
                    }
                });
            });
        };
        PlotView.prototype.connect_signals = function () {
            var e_23, _b, e_24, _c;
            var _this = this;
            _super.prototype.connect_signals.call(this);
            var _d = this.frame, x_ranges = _d.x_ranges, y_ranges = _d.y_ranges;
            try {
                for (var x_ranges_3 = tslib_1.__values(x_ranges), x_ranges_3_1 = x_ranges_3.next(); !x_ranges_3_1.done; x_ranges_3_1 = x_ranges_3.next()) {
                    var _e = tslib_1.__read(x_ranges_3_1.value, 2), range = _e[1];
                    this.connect(range.change, function () { _this._needs_layout = true; _this.request_paint(); });
                }
            }
            catch (e_23_1) {
                e_23 = { error: e_23_1 };
            }
            finally {
                try {
                    if (x_ranges_3_1 && !x_ranges_3_1.done && (_b = x_ranges_3.return))
                        _b.call(x_ranges_3);
                }
                finally {
                    if (e_23)
                        throw e_23.error;
                }
            }
            try {
                for (var y_ranges_3 = tslib_1.__values(y_ranges), y_ranges_3_1 = y_ranges_3.next(); !y_ranges_3_1.done; y_ranges_3_1 = y_ranges_3.next()) {
                    var _f = tslib_1.__read(y_ranges_3_1.value, 2), range = _f[1];
                    this.connect(range.change, function () { _this._needs_layout = true; _this.request_paint(); });
                }
            }
            catch (e_24_1) {
                e_24 = { error: e_24_1 };
            }
            finally {
                try {
                    if (y_ranges_3_1 && !y_ranges_3_1.done && (_c = y_ranges_3.return))
                        _c.call(y_ranges_3);
                }
                finally {
                    if (e_24)
                        throw e_24.error;
                }
            }
            var _g = this.model.properties, plot_width = _g.plot_width, plot_height = _g.plot_height;
            this.on_change([plot_width, plot_height], function () { return _this.invalidate_layout(); });
            var _h = this.model.properties, above = _h.above, below = _h.below, left = _h.left, right = _h.right, center = _h.center, renderers = _h.renderers;
            this.on_change([above, below, left, right, center, renderers], function () {
                return tslib_1.__awaiter(_this, void 0, void 0, function () {
                    return tslib_1.__generator(this, function (_b) {
                        switch (_b.label) {
                            case 0: return [4 /*yield*/, this.build_renderer_views()];
                            case 1: return [2 /*return*/, _b.sent()];
                        }
                    });
                });
            });
            this.connect(this.model.toolbar.properties.tools.change, function () {
                return tslib_1.__awaiter(_this, void 0, void 0, function () {
                    return tslib_1.__generator(this, function (_b) {
                        switch (_b.label) {
                            case 0: return [4 /*yield*/, this.build_renderer_views()];
                            case 1:
                                _b.sent();
                                return [4 /*yield*/, this.build_tool_views()];
                            case 2:
                                _b.sent();
                                return [2 /*return*/];
                        }
                    });
                });
            });
            this.connect(this.model.change, function () { return _this.request_paint(); });
            this.connect(this.model.reset, function () { return _this.reset(); });
        };
        PlotView.prototype.set_initial_range = function () {
            var e_25, _b, e_26, _c;
            // check for good values for ranges before setting initial range
            var good_vals = true;
            var _d = this.frame, x_ranges = _d.x_ranges, y_ranges = _d.y_ranges;
            var xrs = new Map();
            var yrs = new Map();
            try {
                for (var x_ranges_4 = tslib_1.__values(x_ranges), x_ranges_4_1 = x_ranges_4.next(); !x_ranges_4_1.done; x_ranges_4_1 = x_ranges_4.next()) {
                    var _e = tslib_1.__read(x_ranges_4_1.value, 2), name = _e[0], range = _e[1];
                    var start = range.start, end = range.end;
                    if (start == null || end == null || isNaN(start + end)) {
                        good_vals = false;
                        break;
                    }
                    xrs.set(name, { start: start, end: end });
                }
            }
            catch (e_25_1) {
                e_25 = { error: e_25_1 };
            }
            finally {
                try {
                    if (x_ranges_4_1 && !x_ranges_4_1.done && (_b = x_ranges_4.return))
                        _b.call(x_ranges_4);
                }
                finally {
                    if (e_25)
                        throw e_25.error;
                }
            }
            if (good_vals) {
                try {
                    for (var y_ranges_4 = tslib_1.__values(y_ranges), y_ranges_4_1 = y_ranges_4.next(); !y_ranges_4_1.done; y_ranges_4_1 = y_ranges_4.next()) {
                        var _f = tslib_1.__read(y_ranges_4_1.value, 2), name = _f[0], range = _f[1];
                        var start = range.start, end = range.end;
                        if (start == null || end == null || isNaN(start + end)) {
                            good_vals = false;
                            break;
                        }
                        yrs.set(name, { start: start, end: end });
                    }
                }
                catch (e_26_1) {
                    e_26 = { error: e_26_1 };
                }
                finally {
                    try {
                        if (y_ranges_4_1 && !y_ranges_4_1.done && (_c = y_ranges_4.return))
                            _c.call(y_ranges_4);
                    }
                    finally {
                        if (e_26)
                            throw e_26.error;
                    }
                }
            }
            if (good_vals) {
                this._initial_state_info.range = { xrs: xrs, yrs: yrs };
                logging_1.logger.debug("initial ranges set");
            }
            else
                logging_1.logger.warn('could not set initial ranges');
        };
        PlotView.prototype.has_finished = function () {
            var e_27, _b;
            if (!_super.prototype.has_finished.call(this))
                return false;
            if (this.model.visible) {
                try {
                    for (var _c = tslib_1.__values(this.renderer_views), _d = _c.next(); !_d.done; _d = _c.next()) {
                        var _e = tslib_1.__read(_d.value, 2), renderer_view = _e[1];
                        if (!renderer_view.has_finished())
                            return false;
                    }
                }
                catch (e_27_1) {
                    e_27 = { error: e_27_1 };
                }
                finally {
                    try {
                        if (_d && !_d.done && (_b = _c.return))
                            _b.call(_c);
                    }
                    finally {
                        if (e_27)
                            throw e_27.error;
                    }
                }
            }
            return true;
        };
        PlotView.prototype.after_layout = function () {
            _super.prototype.after_layout.call(this);
            this._needs_layout = false;
            this.model.setv({
                inner_width: Math.round(this.frame.bbox.width),
                inner_height: Math.round(this.frame.bbox.height),
                outer_width: Math.round(this.layout.bbox.width),
                outer_height: Math.round(this.layout.bbox.height),
            }, { no_change: true });
            if (this.model.match_aspect !== false) {
                this.pause();
                this.update_dataranges();
                this.unpause(true);
            }
            if (!this._outer_bbox.equals(this.layout.bbox)) {
                var _b = this.layout.bbox, width = _b.width, height = _b.height;
                this.canvas_view.resize(width, height);
                this._outer_bbox = this.layout.bbox;
                this._invalidate_all = true;
                this._needs_paint = true;
            }
            if (!this._inner_bbox.equals(this.frame.inner_bbox)) {
                this._inner_bbox = this.layout.inner_bbox;
                this._needs_paint = true;
            }
            if (this._needs_paint) {
                // XXX: can't be this.request_paint(), because it would trigger back-and-forth
                // layout recomputing feedback loop between plots. Plots are also much more
                // responsive this way, especially in interactive mode.
                this.paint();
            }
        };
        PlotView.prototype.repaint = function () {
            if (this._needs_layout)
                this._invalidate_layout();
            this.paint();
        };
        PlotView.prototype.paint = function () {
            var e_28, _b, e_29, _c;
            var _this = this;
            if (this.is_paused || !this.model.visible)
                return;
            logging_1.logger.trace("PlotView.paint() for " + this.model.id);
            var document = this.model.document;
            if (document != null) {
                var interactive_duration = document.interactive_duration();
                if (interactive_duration >= 0 && interactive_duration < this.model.lod_interval) {
                    setTimeout(function () {
                        if (document.interactive_duration() > _this.model.lod_timeout) {
                            document.interactive_stop();
                        }
                        _this.request_paint();
                    }, this.model.lod_timeout);
                }
                else
                    document.interactive_stop();
            }
            try {
                for (var _d = tslib_1.__values(this.renderer_views), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var _f = tslib_1.__read(_e.value, 2), renderer_view = _f[1];
                    if (this.range_update_timestamp == null ||
                        (renderer_view instanceof glyph_renderer_1.GlyphRendererView && renderer_view.set_data_timestamp > this.range_update_timestamp)) {
                        this.update_dataranges();
                        break;
                    }
                }
            }
            catch (e_28_1) {
                e_28 = { error: e_28_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_b = _d.return))
                        _b.call(_d);
                }
                finally {
                    if (e_28)
                        throw e_28.error;
                }
            }
            var do_primary = false;
            var do_overlays = false;
            if (this._invalidate_all) {
                do_primary = true;
                do_overlays = true;
            }
            else {
                try {
                    for (var _g = tslib_1.__values(this._invalidated_painters), _h = _g.next(); !_h.done; _h = _g.next()) {
                        var painter = _h.value;
                        var level = painter.model.level;
                        if (level != "overlay")
                            do_primary = true;
                        else
                            do_overlays = true;
                        if (do_primary && do_overlays)
                            break;
                    }
                }
                catch (e_29_1) {
                    e_29 = { error: e_29_1 };
                }
                finally {
                    try {
                        if (_h && !_h.done && (_c = _g.return))
                            _c.call(_g);
                    }
                    finally {
                        if (e_29)
                            throw e_29.error;
                    }
                }
            }
            this._invalidated_painters.clear();
            this._invalidate_all = false;
            var frame_box = [
                this.frame.bbox.left,
                this.frame.bbox.top,
                this.frame.bbox.width,
                this.frame.bbox.height,
            ];
            var _j = this.canvas_view, primary = _j.primary, overlays = _j.overlays;
            if (do_primary) {
                primary.prepare();
                this.canvas_view.prepare_webgl(frame_box);
                this.canvas_view.clear_webgl();
                this._map_hook(primary.ctx, frame_box);
                this._paint_empty(primary.ctx, frame_box);
                this._paint_outline(primary.ctx, frame_box);
                this._paint_levels(primary.ctx, "image", frame_box, true);
                this._paint_levels(primary.ctx, "underlay", frame_box, true);
                this._paint_levels(primary.ctx, "glyph", frame_box, true);
                this._paint_levels(primary.ctx, "guide", frame_box, false);
                this._paint_levels(primary.ctx, "annotation", frame_box, false);
                primary.finish();
            }
            if (do_overlays) {
                overlays.prepare();
                this._paint_levels(overlays.ctx, "overlay", frame_box, false);
                overlays.finish();
            }
            if (this._initial_state_info.range == null)
                this.set_initial_range();
            this._needs_paint = false;
        };
        PlotView.prototype._paint_levels = function (ctx, level, clip_region, global_clip) {
            var e_30, _b;
            try {
                for (var _c = tslib_1.__values(this.computed_renderers), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var renderer = _d.value;
                    if (renderer.level != level)
                        continue;
                    var renderer_view = this.renderer_views.get(renderer);
                    ctx.save();
                    if (global_clip || renderer_view.needs_clip) {
                        ctx.beginPath();
                        ctx.rect.apply(ctx, tslib_1.__spread(clip_region));
                        ctx.clip();
                    }
                    renderer_view.render();
                    ctx.restore();
                    if (renderer_view.has_webgl && renderer_view.needs_webgl_blit) {
                        this.canvas_view.blit_webgl(ctx);
                        this.canvas_view.clear_webgl();
                    }
                }
            }
            catch (e_30_1) {
                e_30 = { error: e_30_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_30)
                        throw e_30.error;
                }
            }
        };
        PlotView.prototype._map_hook = function (_ctx, _frame_box) { };
        PlotView.prototype._paint_empty = function (ctx, frame_box) {
            var _b = tslib_1.__read([0, 0, this.layout.bbox.width, this.layout.bbox.height], 4), cx = _b[0], cy = _b[1], cw = _b[2], ch = _b[3];
            var _c = tslib_1.__read(frame_box, 4), fx = _c[0], fy = _c[1], fw = _c[2], fh = _c[3];
            if (this.visuals.border_fill.doit) {
                this.visuals.border_fill.set_value(ctx);
                ctx.fillRect(cx, cy, cw, ch);
                ctx.clearRect(fx, fy, fw, fh);
            }
            if (this.visuals.background_fill.doit) {
                this.visuals.background_fill.set_value(ctx);
                ctx.fillRect(fx, fy, fw, fh);
            }
        };
        PlotView.prototype._paint_outline = function (ctx, frame_box) {
            if (this.visuals.outline_line.doit) {
                ctx.save();
                this.visuals.outline_line.set_value(ctx);
                var _b = tslib_1.__read(frame_box, 4), x0 = _b[0], y0 = _b[1], w = _b[2], h = _b[3];
                // XXX: shrink outline region by 1px to make right and bottom lines visible
                // if they are on the edge of the canvas.
                if (x0 + w == this.layout.bbox.width) {
                    w -= 1;
                }
                if (y0 + h == this.layout.bbox.height) {
                    h -= 1;
                }
                ctx.strokeRect(x0, y0, w, h);
                ctx.restore();
            }
        };
        PlotView.prototype.to_blob = function () {
            return this.canvas_view.to_blob();
        };
        PlotView.prototype.export = function (type, hidpi) {
            if (hidpi === void 0) {
                hidpi = true;
            }
            var output_backend = type == "png" ? "canvas" : "svg";
            var composite = new canvas_1.CanvasLayer(output_backend, hidpi);
            var _b = this.layout.bbox, width = _b.width, height = _b.height;
            composite.resize(width, height);
            var canvas = this.canvas_view.compose().canvas;
            composite.ctx.drawImage(canvas, 0, 0);
            return composite;
        };
        PlotView.prototype.serializable_state = function () {
            var _a = _super.prototype.serializable_state.call(this), children = _a.children, state = __rest(_a, ["children"]);
            var renderers = this.get_renderer_views()
                .map(function (view) { return view.serializable_state(); })
                .filter(function (item) { return "bbox" in item; });
            return Object.assign(Object.assign({}, state), { children: tslib_1.__spread(children, renderers) }); // XXX
        };
        return PlotView;
    }(layout_dom_1.LayoutDOMView));
    exports.PlotView = PlotView;
    PlotView.__name__ = "PlotView";
},
/* core/bokeh_events.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    };
    function event(event_name) {
        return function (cls) {
            cls.prototype.event_name = event_name;
        };
    }
    var BokehEvent = /** @class */ (function () {
        function BokehEvent() {
        }
        BokehEvent.prototype.to_json = function () {
            var event_name = this.event_name;
            return { event_name: event_name, event_values: this._to_json() };
        };
        return BokehEvent;
    }());
    exports.BokehEvent = BokehEvent;
    BokehEvent.__name__ = "BokehEvent";
    var ModelEvent = /** @class */ (function (_super) {
        tslib_1.__extends(ModelEvent, _super);
        function ModelEvent() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.origin = null;
            return _this;
        }
        ModelEvent.prototype._to_json = function () {
            return { model: this.origin };
        };
        return ModelEvent;
    }(BokehEvent));
    exports.ModelEvent = ModelEvent;
    ModelEvent.__name__ = "ModelEvent";
    var DocumentReady = /** @class */ (function (_super) {
        tslib_1.__extends(DocumentReady, _super);
        function DocumentReady() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        DocumentReady.prototype._to_json = function () {
            return {};
        };
        return DocumentReady;
    }(BokehEvent));
    exports.DocumentReady = DocumentReady;
    DocumentReady.__name__ = "DocumentReady";
    exports.DocumentReady = DocumentReady = __decorate([
        event("document_ready")
    ], DocumentReady);
    var ButtonClick = /** @class */ (function (_super) {
        tslib_1.__extends(ButtonClick, _super);
        function ButtonClick() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return ButtonClick;
    }(ModelEvent));
    exports.ButtonClick = ButtonClick;
    ButtonClick.__name__ = "ButtonClick";
    exports.ButtonClick = ButtonClick = __decorate([
        event("button_click")
    ], ButtonClick);
    var MenuItemClick = /** @class */ (function (_super) {
        tslib_1.__extends(MenuItemClick, _super);
        function MenuItemClick(item) {
            var _this = _super.call(this) || this;
            _this.item = item;
            return _this;
        }
        MenuItemClick.prototype._to_json = function () {
            var item = this.item;
            return Object.assign(Object.assign({}, _super.prototype._to_json.call(this)), { item: item });
        };
        return MenuItemClick;
    }(ModelEvent));
    exports.MenuItemClick = MenuItemClick;
    MenuItemClick.__name__ = "MenuItemClick";
    exports.MenuItemClick = MenuItemClick = __decorate([
        event("menu_item_click")
    ], MenuItemClick);
    // A UIEvent is an event originating on a canvas this includes.
    // DOM events such as keystrokes as well as hammer events and LOD events.
    var UIEvent = /** @class */ (function (_super) {
        tslib_1.__extends(UIEvent, _super);
        function UIEvent() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return UIEvent;
    }(ModelEvent));
    exports.UIEvent = UIEvent;
    UIEvent.__name__ = "UIEvent";
    var LODStart = /** @class */ (function (_super) {
        tslib_1.__extends(LODStart, _super);
        function LODStart() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return LODStart;
    }(UIEvent));
    exports.LODStart = LODStart;
    LODStart.__name__ = "LODStart";
    exports.LODStart = LODStart = __decorate([
        event("lodstart")
    ], LODStart);
    var LODEnd = /** @class */ (function (_super) {
        tslib_1.__extends(LODEnd, _super);
        function LODEnd() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return LODEnd;
    }(UIEvent));
    exports.LODEnd = LODEnd;
    LODEnd.__name__ = "LODEnd";
    exports.LODEnd = LODEnd = __decorate([
        event("lodend")
    ], LODEnd);
    var SelectionGeometry = /** @class */ (function (_super) {
        tslib_1.__extends(SelectionGeometry, _super);
        function SelectionGeometry(geometry, final) {
            var _this = _super.call(this) || this;
            _this.geometry = geometry;
            _this.final = final;
            return _this;
        }
        SelectionGeometry.prototype._to_json = function () {
            var _a = this, geometry = _a.geometry, final = _a.final;
            return Object.assign(Object.assign({}, _super.prototype._to_json.call(this)), { geometry: geometry, final: final });
        };
        return SelectionGeometry;
    }(UIEvent));
    exports.SelectionGeometry = SelectionGeometry;
    SelectionGeometry.__name__ = "SelectionGeometry";
    exports.SelectionGeometry = SelectionGeometry = __decorate([
        event("selectiongeometry")
    ], SelectionGeometry);
    var Reset = /** @class */ (function (_super) {
        tslib_1.__extends(Reset, _super);
        function Reset() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return Reset;
    }(UIEvent));
    exports.Reset = Reset;
    Reset.__name__ = "Reset";
    exports.Reset = Reset = __decorate([
        event("reset")
    ], Reset);
    var PointEvent = /** @class */ (function (_super) {
        tslib_1.__extends(PointEvent, _super);
        function PointEvent(sx, sy, x, y) {
            var _this = _super.call(this) || this;
            _this.sx = sx;
            _this.sy = sy;
            _this.x = x;
            _this.y = y;
            return _this;
        }
        PointEvent.prototype._to_json = function () {
            var _a = this, sx = _a.sx, sy = _a.sy, x = _a.x, y = _a.y;
            return Object.assign(Object.assign({}, _super.prototype._to_json.call(this)), { sx: sx, sy: sy, x: x, y: y });
        };
        return PointEvent;
    }(UIEvent));
    exports.PointEvent = PointEvent;
    PointEvent.__name__ = "PointEvent";
    var Pan = /** @class */ (function (_super) {
        tslib_1.__extends(Pan, _super);
        /* TODO: direction: -1 | 1 */
        function Pan(sx, sy, x, y, delta_x, delta_y) {
            var _this = _super.call(this, sx, sy, x, y) || this;
            _this.sx = sx;
            _this.sy = sy;
            _this.x = x;
            _this.y = y;
            _this.delta_x = delta_x;
            _this.delta_y = delta_y;
            return _this;
        }
        Pan.prototype._to_json = function () {
            var _a = this, delta_x = _a.delta_x, delta_y = _a.delta_y /*, direction*/;
            return Object.assign(Object.assign({}, _super.prototype._to_json.call(this)), { delta_x: delta_x, delta_y: delta_y /*, direction*/ });
        };
        return Pan;
    }(PointEvent));
    exports.Pan = Pan;
    Pan.__name__ = "Pan";
    exports.Pan = Pan = __decorate([
        event("pan")
    ], Pan);
    var Pinch = /** @class */ (function (_super) {
        tslib_1.__extends(Pinch, _super);
        function Pinch(sx, sy, x, y, scale) {
            var _this = _super.call(this, sx, sy, x, y) || this;
            _this.sx = sx;
            _this.sy = sy;
            _this.x = x;
            _this.y = y;
            _this.scale = scale;
            return _this;
        }
        Pinch.prototype._to_json = function () {
            var scale = this.scale;
            return Object.assign(Object.assign({}, _super.prototype._to_json.call(this)), { scale: scale });
        };
        return Pinch;
    }(PointEvent));
    exports.Pinch = Pinch;
    Pinch.__name__ = "Pinch";
    exports.Pinch = Pinch = __decorate([
        event("pinch")
    ], Pinch);
    var Rotate = /** @class */ (function (_super) {
        tslib_1.__extends(Rotate, _super);
        function Rotate(sx, sy, x, y, rotation) {
            var _this = _super.call(this, sx, sy, x, y) || this;
            _this.sx = sx;
            _this.sy = sy;
            _this.x = x;
            _this.y = y;
            _this.rotation = rotation;
            return _this;
        }
        Rotate.prototype._to_json = function () {
            var rotation = this.rotation;
            return Object.assign(Object.assign({}, _super.prototype._to_json.call(this)), { rotation: rotation });
        };
        return Rotate;
    }(PointEvent));
    exports.Rotate = Rotate;
    Rotate.__name__ = "Rotate";
    exports.Rotate = Rotate = __decorate([
        event("rotate")
    ], Rotate);
    var MouseWheel = /** @class */ (function (_super) {
        tslib_1.__extends(MouseWheel, _super);
        function MouseWheel(sx, sy, x, y, delta) {
            var _this = _super.call(this, sx, sy, x, y) || this;
            _this.sx = sx;
            _this.sy = sy;
            _this.x = x;
            _this.y = y;
            _this.delta = delta;
            return _this;
        }
        MouseWheel.prototype._to_json = function () {
            var delta = this.delta;
            return Object.assign(Object.assign({}, _super.prototype._to_json.call(this)), { delta: delta });
        };
        return MouseWheel;
    }(PointEvent));
    exports.MouseWheel = MouseWheel;
    MouseWheel.__name__ = "MouseWheel";
    exports.MouseWheel = MouseWheel = __decorate([
        event("wheel")
    ], MouseWheel);
    var MouseMove = /** @class */ (function (_super) {
        tslib_1.__extends(MouseMove, _super);
        function MouseMove() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return MouseMove;
    }(PointEvent));
    exports.MouseMove = MouseMove;
    MouseMove.__name__ = "MouseMove";
    exports.MouseMove = MouseMove = __decorate([
        event("mousemove")
    ], MouseMove);
    var MouseEnter = /** @class */ (function (_super) {
        tslib_1.__extends(MouseEnter, _super);
        function MouseEnter() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return MouseEnter;
    }(PointEvent));
    exports.MouseEnter = MouseEnter;
    MouseEnter.__name__ = "MouseEnter";
    exports.MouseEnter = MouseEnter = __decorate([
        event("mouseenter")
    ], MouseEnter);
    var MouseLeave = /** @class */ (function (_super) {
        tslib_1.__extends(MouseLeave, _super);
        function MouseLeave() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return MouseLeave;
    }(PointEvent));
    exports.MouseLeave = MouseLeave;
    MouseLeave.__name__ = "MouseLeave";
    exports.MouseLeave = MouseLeave = __decorate([
        event("mouseleave")
    ], MouseLeave);
    var Tap = /** @class */ (function (_super) {
        tslib_1.__extends(Tap, _super);
        function Tap() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return Tap;
    }(PointEvent));
    exports.Tap = Tap;
    Tap.__name__ = "Tap";
    exports.Tap = Tap = __decorate([
        event("tap")
    ], Tap);
    var DoubleTap = /** @class */ (function (_super) {
        tslib_1.__extends(DoubleTap, _super);
        function DoubleTap() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return DoubleTap;
    }(PointEvent));
    exports.DoubleTap = DoubleTap;
    DoubleTap.__name__ = "DoubleTap";
    exports.DoubleTap = DoubleTap = __decorate([
        event("doubletap")
    ], DoubleTap);
    var Press = /** @class */ (function (_super) {
        tslib_1.__extends(Press, _super);
        function Press() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return Press;
    }(PointEvent));
    exports.Press = Press;
    Press.__name__ = "Press";
    exports.Press = Press = __decorate([
        event("press")
    ], Press);
    var PressUp = /** @class */ (function (_super) {
        tslib_1.__extends(PressUp, _super);
        function PressUp() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return PressUp;
    }(PointEvent));
    exports.PressUp = PressUp;
    PressUp.__name__ = "PressUp";
    exports.PressUp = PressUp = __decorate([
        event("pressup")
    ], PressUp);
    var PanStart = /** @class */ (function (_super) {
        tslib_1.__extends(PanStart, _super);
        function PanStart() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return PanStart;
    }(PointEvent));
    exports.PanStart = PanStart;
    PanStart.__name__ = "PanStart";
    exports.PanStart = PanStart = __decorate([
        event("panstart")
    ], PanStart);
    var PanEnd = /** @class */ (function (_super) {
        tslib_1.__extends(PanEnd, _super);
        function PanEnd() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return PanEnd;
    }(PointEvent));
    exports.PanEnd = PanEnd;
    PanEnd.__name__ = "PanEnd";
    exports.PanEnd = PanEnd = __decorate([
        event("panend")
    ], PanEnd);
    var PinchStart = /** @class */ (function (_super) {
        tslib_1.__extends(PinchStart, _super);
        function PinchStart() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return PinchStart;
    }(PointEvent));
    exports.PinchStart = PinchStart;
    PinchStart.__name__ = "PinchStart";
    exports.PinchStart = PinchStart = __decorate([
        event("pinchstart")
    ], PinchStart);
    var PinchEnd = /** @class */ (function (_super) {
        tslib_1.__extends(PinchEnd, _super);
        function PinchEnd() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return PinchEnd;
    }(PointEvent));
    exports.PinchEnd = PinchEnd;
    PinchEnd.__name__ = "PinchEnd";
    exports.PinchEnd = PinchEnd = __decorate([
        event("pinchend")
    ], PinchEnd);
    var RotateStart = /** @class */ (function (_super) {
        tslib_1.__extends(RotateStart, _super);
        function RotateStart() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return RotateStart;
    }(PointEvent));
    exports.RotateStart = RotateStart;
    RotateStart.__name__ = "RotateStart";
    exports.RotateStart = RotateStart = __decorate([
        event("rotatestart")
    ], RotateStart);
    var RotateEnd = /** @class */ (function (_super) {
        tslib_1.__extends(RotateEnd, _super);
        function RotateEnd() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        return RotateEnd;
    }(PointEvent));
    exports.RotateEnd = RotateEnd;
    RotateEnd.__name__ = "RotateEnd";
    exports.RotateEnd = RotateEnd = __decorate([
        event("rotateend")
    ], RotateEnd);
},
/* core/ui_events.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var hammerjs_1 = tslib_1.__importDefault(require(412) /* hammerjs */);
    var signaling_1 = require(130) /* ./signaling */;
    var logging_1 = require(134) /* ./logging */;
    var dom_1 = require(187) /* ./dom */;
    var events = tslib_1.__importStar(require(428) /* ./bokeh_events */);
    var wheel_1 = require(430) /* ./util/wheel */;
    var array_1 = require(124) /* ./util/array */;
    var types_1 = require(123) /* ./util/types */;
    var compat_1 = require(147) /* ./util/compat */;
    var menus_1 = require(417) /* ./util/menus */;
    function is_touch(event) {
        return typeof TouchEvent !== "undefined" && event instanceof TouchEvent;
    }
    var UIEvents = /** @class */ (function () {
        function UIEvents(plot_view, toolbar, hit_area) {
            var _this = this;
            this.plot_view = plot_view;
            this.toolbar = toolbar;
            this.hit_area = hit_area;
            this.pan_start = new signaling_1.Signal(this, 'pan:start');
            this.pan = new signaling_1.Signal(this, 'pan');
            this.pan_end = new signaling_1.Signal(this, 'pan:end');
            this.pinch_start = new signaling_1.Signal(this, 'pinch:start');
            this.pinch = new signaling_1.Signal(this, 'pinch');
            this.pinch_end = new signaling_1.Signal(this, 'pinch:end');
            this.rotate_start = new signaling_1.Signal(this, 'rotate:start');
            this.rotate = new signaling_1.Signal(this, 'rotate');
            this.rotate_end = new signaling_1.Signal(this, 'rotate:end');
            this.tap = new signaling_1.Signal(this, 'tap');
            this.doubletap = new signaling_1.Signal(this, 'doubletap');
            this.press = new signaling_1.Signal(this, 'press');
            this.pressup = new signaling_1.Signal(this, 'pressup');
            this.move_enter = new signaling_1.Signal(this, 'move:enter');
            this.move = new signaling_1.Signal(this, 'move');
            this.move_exit = new signaling_1.Signal(this, 'move:exit');
            this.scroll = new signaling_1.Signal(this, 'scroll');
            this.keydown = new signaling_1.Signal(this, 'keydown');
            this.keyup = new signaling_1.Signal(this, 'keyup');
            this.hammer = new hammerjs_1.default(this.hit_area, {
                touchAction: 'auto',
                inputClass: hammerjs_1.default.TouchMouseInput,
            });
            this._configure_hammerjs();
            // Mouse & keyboard events not handled through hammerjs
            // We can 'add and forget' these event listeners because this.hit_area is a DOM element
            // that will be thrown away when the view is removed
            this.hit_area.addEventListener("mousemove", function (e) { return _this._mouse_move(e); });
            this.hit_area.addEventListener("mouseenter", function (e) { return _this._mouse_enter(e); });
            this.hit_area.addEventListener("mouseleave", function (e) { return _this._mouse_exit(e); });
            this.hit_area.addEventListener("contextmenu", function (e) { return _this._context_menu(e); });
            this.hit_area.addEventListener("wheel", function (e) { return _this._mouse_wheel(e); });
            // But we MUST remove listeners registered on document or we'll leak memory: register
            // 'this' as the listener (it implements the event listener interface, i.e. handleEvent)
            // instead of an anonymous function so we can easily refer back to it for removing
            document.addEventListener("keydown", this);
            document.addEventListener("keyup", this);
            this.menu = new menus_1.ContextMenu([], {
                prevent_hide: function (event) { return event.button == 2 && event.target == _this.hit_area; },
            });
            this.hit_area.appendChild(this.menu.el);
        }
        UIEvents.prototype.destroy = function () {
            this.menu.remove();
            this.hammer.destroy();
            document.removeEventListener("keydown", this);
            document.removeEventListener("keyup", this);
        };
        UIEvents.prototype.handleEvent = function (e) {
            if (e.type == "keydown")
                this._key_down(e);
            else if (e.type == "keyup")
                this._key_up(e);
        };
        UIEvents.prototype._configure_hammerjs = function () {
            var _this = this;
            // This is to be able to distinguish double taps from single taps
            this.hammer.get('doubletap').recognizeWith('tap');
            this.hammer.get('tap').requireFailure('doubletap');
            this.hammer.get('doubletap').dropRequireFailure('tap');
            this.hammer.on('doubletap', function (e) { return _this._doubletap(e); });
            this.hammer.on('tap', function (e) { return _this._tap(e); });
            this.hammer.on('press', function (e) { return _this._press(e); });
            this.hammer.on('pressup', function (e) { return _this._pressup(e); });
            this.hammer.get('pan').set({ direction: hammerjs_1.default.DIRECTION_ALL });
            this.hammer.on('panstart', function (e) { return _this._pan_start(e); });
            this.hammer.on('pan', function (e) { return _this._pan(e); });
            this.hammer.on('panend', function (e) { return _this._pan_end(e); });
            this.hammer.get('pinch').set({ enable: true });
            this.hammer.on('pinchstart', function (e) { return _this._pinch_start(e); });
            this.hammer.on('pinch', function (e) { return _this._pinch(e); });
            this.hammer.on('pinchend', function (e) { return _this._pinch_end(e); });
            this.hammer.get('rotate').set({ enable: true });
            this.hammer.on('rotatestart', function (e) { return _this._rotate_start(e); });
            this.hammer.on('rotate', function (e) { return _this._rotate(e); });
            this.hammer.on('rotateend', function (e) { return _this._rotate_end(e); });
        };
        UIEvents.prototype.register_tool = function (tool_view) {
            var _this = this;
            var et = tool_view.model.event_type;
            if (et != null) {
                if (types_1.isString(et))
                    this._register_tool(tool_view, et);
                else {
                    // Multi-tools should only registered shared events once
                    et.forEach(function (e, index) { return _this._register_tool(tool_view, e, index < 1); });
                }
            }
        };
        UIEvents.prototype._register_tool = function (tool_view, et, shared) {
            if (shared === void 0) {
                shared = true;
            }
            var v = tool_view;
            var id = v.model.id;
            var conditionally = function (fn) {
                return function (arg) {
                    if (arg.id == id)
                        fn(arg.e);
                };
            };
            var unconditionally = function (fn) {
                return function (arg) {
                    fn(arg.e);
                };
            };
            switch (et) {
                case "pan": {
                    if (v._pan_start != null)
                        v.connect(this.pan_start, conditionally(v._pan_start.bind(v)));
                    if (v._pan != null)
                        v.connect(this.pan, conditionally(v._pan.bind(v)));
                    if (v._pan_end != null)
                        v.connect(this.pan_end, conditionally(v._pan_end.bind(v)));
                    break;
                }
                case "pinch": {
                    if (v._pinch_start != null)
                        v.connect(this.pinch_start, conditionally(v._pinch_start.bind(v)));
                    if (v._pinch != null)
                        v.connect(this.pinch, conditionally(v._pinch.bind(v)));
                    if (v._pinch_end != null)
                        v.connect(this.pinch_end, conditionally(v._pinch_end.bind(v)));
                    break;
                }
                case "rotate": {
                    if (v._rotate_start != null)
                        v.connect(this.rotate_start, conditionally(v._rotate_start.bind(v)));
                    if (v._rotate != null)
                        v.connect(this.rotate, conditionally(v._rotate.bind(v)));
                    if (v._rotate_end != null)
                        v.connect(this.rotate_end, conditionally(v._rotate_end.bind(v)));
                    break;
                }
                case "move": {
                    if (v._move_enter != null)
                        v.connect(this.move_enter, conditionally(v._move_enter.bind(v)));
                    if (v._move != null)
                        v.connect(this.move, conditionally(v._move.bind(v)));
                    if (v._move_exit != null)
                        v.connect(this.move_exit, conditionally(v._move_exit.bind(v)));
                    break;
                }
                case "tap": {
                    if (v._tap != null)
                        v.connect(this.tap, conditionally(v._tap.bind(v)));
                    break;
                }
                case "press": {
                    if (v._press != null)
                        v.connect(this.press, conditionally(v._press.bind(v)));
                    if (v._pressup != null)
                        v.connect(this.pressup, conditionally(v._pressup.bind(v)));
                    break;
                }
                case "scroll": {
                    if (v._scroll != null)
                        v.connect(this.scroll, conditionally(v._scroll.bind(v)));
                    break;
                }
                default:
                    throw new Error("unsupported event_type: " + et);
            }
            // Skip shared events if registering multi-tool
            if (!shared)
                return;
            if (v._doubletap != null)
                v.connect(this.doubletap, unconditionally(v._doubletap.bind(v)));
            if (v._keydown != null)
                v.connect(this.keydown, unconditionally(v._keydown.bind(v)));
            if (v._keyup != null)
                v.connect(this.keyup, unconditionally(v._keyup.bind(v)));
            // Dual touch hack part 1/2
            // This is a hack for laptops with touch screen who may be pinching or scrolling
            // in order to use the wheel zoom tool. If it's a touch screen the WheelZoomTool event
            // will be linked to pinch. But we also want to trigger in the case of a scroll.
            if (compat_1.is_mobile && v._scroll != null && et == 'pinch') {
                logging_1.logger.debug("Registering scroll on touch screen");
                v.connect(this.scroll, conditionally(v._scroll.bind(v)));
            }
        };
        UIEvents.prototype._hit_test_renderers = function (sx, sy) {
            var e_1, _a;
            var views = this.plot_view.get_renderer_views();
            try {
                for (var _b = tslib_1.__values(array_1.reversed(views)), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var view = _c.value;
                    var level = view.model.level;
                    if ((level == 'annotation' || level == 'overlay') && view.interactive_hit != null) {
                        if (view.interactive_hit(sx, sy))
                            return view;
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return null;
        };
        UIEvents.prototype._hit_test_frame = function (sx, sy) {
            return this.plot_view.frame.bbox.contains(sx, sy);
        };
        UIEvents.prototype._hit_test_canvas = function (sx, sy) {
            return this.plot_view.layout.bbox.contains(sx, sy);
        };
        UIEvents.prototype._trigger = function (signal, e, srcEvent) {
            var _this = this;
            var gestures = this.toolbar.gestures;
            var event_type = signal.name;
            var base_type = event_type.split(":")[0];
            var view = this._hit_test_renderers(e.sx, e.sy);
            var on_canvas = this._hit_test_canvas(e.sx, e.sy);
            switch (base_type) {
                case "move": {
                    var active_gesture = gestures[base_type].active;
                    if (active_gesture != null)
                        this.trigger(signal, e, active_gesture.id);
                    var active_inspectors = this.toolbar.inspectors.filter(function (t) { return t.active; });
                    var cursor = "default";
                    // the event happened on a renderer
                    if (view != null) {
                        cursor = view.cursor(e.sx, e.sy) || cursor;
                        if (!array_1.is_empty(active_inspectors)) {
                            // override event_type to cause inspectors to clear overlays
                            signal = this.move_exit; // XXX
                        }
                        // the event happened on the plot frame but off a renderer
                    }
                    else if (this._hit_test_frame(e.sx, e.sy)) {
                        if (!array_1.is_empty(active_inspectors)) {
                            cursor = "crosshair";
                        }
                    }
                    this.plot_view.set_cursor(cursor);
                    this.plot_view.set_toolbar_visibility(on_canvas);
                    active_inspectors.map(function (inspector) { return _this.trigger(signal, e, inspector.id); });
                    break;
                }
                case "tap": {
                    var target = srcEvent.target;
                    if (target != null && target != this.hit_area)
                        return; // don't trigger bokeh events
                    if (view != null && view.on_hit != null)
                        view.on_hit(e.sx, e.sy);
                    var active_gesture = gestures[base_type].active;
                    if (active_gesture != null)
                        this.trigger(signal, e, active_gesture.id);
                    break;
                }
                case "scroll": {
                    // Dual touch hack part 2/2
                    // This is a hack for laptops with touch screen who may be pinching or scrolling
                    // in order to use the wheel zoom tool. If it's a touch screen the WheelZoomTool event
                    // will be linked to pinch. But we also want to trigger in the case of a scroll.
                    var base = compat_1.is_mobile ? "pinch" : "scroll";
                    var active_gesture = gestures[base].active;
                    if (active_gesture != null) {
                        srcEvent.preventDefault();
                        srcEvent.stopPropagation();
                        this.trigger(signal, e, active_gesture.id);
                    }
                    break;
                }
                case "pan": {
                    var active_gesture = gestures[base_type].active;
                    if (active_gesture != null) {
                        srcEvent.preventDefault();
                        this.trigger(signal, e, active_gesture.id);
                    }
                    break;
                }
                default: {
                    var active_gesture = gestures[base_type].active;
                    if (active_gesture != null)
                        this.trigger(signal, e, active_gesture.id);
                }
            }
            this._trigger_bokeh_event(e);
        };
        UIEvents.prototype.trigger = function (signal, e, id) {
            if (id === void 0) {
                id = null;
            }
            signal.emit({ id: id, e: e });
        };
        UIEvents.prototype._trigger_bokeh_event = function (e) {
            var _this = this;
            var ev = (function () {
                var sx = e.sx, sy = e.sy;
                var x = _this.plot_view.frame.x_scale.invert(sx);
                var y = _this.plot_view.frame.y_scale.invert(sy);
                switch (e.type) {
                    case "wheel":
                        return new events.MouseWheel(sx, sy, x, y, e.delta);
                    case "mousemove":
                        return new events.MouseMove(sx, sy, x, y);
                    case "mouseenter":
                        return new events.MouseEnter(sx, sy, x, y);
                    case "mouseleave":
                        return new events.MouseLeave(sx, sy, x, y);
                    case "tap":
                        return new events.Tap(sx, sy, x, y);
                    case "doubletap":
                        return new events.DoubleTap(sx, sy, x, y);
                    case "press":
                        return new events.Press(sx, sy, x, y);
                    case "pressup":
                        return new events.PressUp(sx, sy, x, y);
                    case "pan":
                        return new events.Pan(sx, sy, x, y, e.deltaX, e.deltaY);
                    case "panstart":
                        return new events.PanStart(sx, sy, x, y);
                    case "panend":
                        return new events.PanEnd(sx, sy, x, y);
                    case "pinch":
                        return new events.Pinch(sx, sy, x, y, e.scale);
                    case "pinchstart":
                        return new events.PinchStart(sx, sy, x, y);
                    case "pinchend":
                        return new events.PinchEnd(sx, sy, x, y);
                    case "rotate":
                        return new events.Rotate(sx, sy, x, y, e.rotation);
                    case "rotatestart":
                        return new events.RotateStart(sx, sy, x, y);
                    case "rotateend":
                        return new events.RotateEnd(sx, sy, x, y);
                    default:
                        return undefined;
                }
            })();
            if (ev != null)
                this.plot_view.model.trigger_event(ev);
        };
        UIEvents.prototype._get_sxy = function (event) {
            var _a = is_touch(event) ? (event.touches.length != 0 ? event.touches : event.changedTouches)[0] : event, pageX = _a.pageX, pageY = _a.pageY;
            var _b = dom_1.offset(this.hit_area), left = _b.left, top = _b.top;
            return {
                sx: pageX - left,
                sy: pageY - top,
            };
        };
        UIEvents.prototype._pan_event = function (e) {
            return Object.assign(Object.assign({ type: e.type }, this._get_sxy(e.srcEvent)), { deltaX: e.deltaX, deltaY: e.deltaY, shiftKey: e.srcEvent.shiftKey, ctrlKey: e.srcEvent.ctrlKey });
        };
        UIEvents.prototype._pinch_event = function (e) {
            return Object.assign(Object.assign({ type: e.type }, this._get_sxy(e.srcEvent)), { scale: e.scale, shiftKey: e.srcEvent.shiftKey, ctrlKey: e.srcEvent.ctrlKey });
        };
        UIEvents.prototype._rotate_event = function (e) {
            return Object.assign(Object.assign({ type: e.type }, this._get_sxy(e.srcEvent)), { rotation: e.rotation, shiftKey: e.srcEvent.shiftKey, ctrlKey: e.srcEvent.ctrlKey });
        };
        UIEvents.prototype._tap_event = function (e) {
            return Object.assign(Object.assign({ type: e.type }, this._get_sxy(e.srcEvent)), { shiftKey: e.srcEvent.shiftKey, ctrlKey: e.srcEvent.ctrlKey });
        };
        UIEvents.prototype._move_event = function (e) {
            return Object.assign(Object.assign({ type: e.type }, this._get_sxy(e)), { shiftKey: e.shiftKey, ctrlKey: e.ctrlKey });
        };
        UIEvents.prototype._scroll_event = function (e) {
            return Object.assign(Object.assign({ type: e.type }, this._get_sxy(e)), { delta: wheel_1.getDeltaY(e), shiftKey: e.shiftKey, ctrlKey: e.ctrlKey });
        };
        UIEvents.prototype._key_event = function (e) {
            return {
                type: e.type,
                keyCode: e.keyCode,
            };
        };
        UIEvents.prototype._pan_start = function (e) {
            var ev = this._pan_event(e);
            // back out delta to get original center point
            ev.sx -= e.deltaX;
            ev.sy -= e.deltaY;
            this._trigger(this.pan_start, ev, e.srcEvent);
        };
        UIEvents.prototype._pan = function (e) {
            this._trigger(this.pan, this._pan_event(e), e.srcEvent);
        };
        UIEvents.prototype._pan_end = function (e) {
            this._trigger(this.pan_end, this._pan_event(e), e.srcEvent);
        };
        UIEvents.prototype._pinch_start = function (e) {
            this._trigger(this.pinch_start, this._pinch_event(e), e.srcEvent);
        };
        UIEvents.prototype._pinch = function (e) {
            this._trigger(this.pinch, this._pinch_event(e), e.srcEvent);
        };
        UIEvents.prototype._pinch_end = function (e) {
            this._trigger(this.pinch_end, this._pinch_event(e), e.srcEvent);
        };
        UIEvents.prototype._rotate_start = function (e) {
            this._trigger(this.rotate_start, this._rotate_event(e), e.srcEvent);
        };
        UIEvents.prototype._rotate = function (e) {
            this._trigger(this.rotate, this._rotate_event(e), e.srcEvent);
        };
        UIEvents.prototype._rotate_end = function (e) {
            this._trigger(this.rotate_end, this._rotate_event(e), e.srcEvent);
        };
        UIEvents.prototype._tap = function (e) {
            this._trigger(this.tap, this._tap_event(e), e.srcEvent);
        };
        UIEvents.prototype._doubletap = function (e) {
            // NOTE: doubletap event triggered unconditionally
            var ev = this._tap_event(e);
            this._trigger_bokeh_event(ev);
            this.trigger(this.doubletap, ev);
        };
        UIEvents.prototype._press = function (e) {
            this._trigger(this.press, this._tap_event(e), e.srcEvent);
        };
        UIEvents.prototype._pressup = function (e) {
            this._trigger(this.pressup, this._tap_event(e), e.srcEvent);
        };
        UIEvents.prototype._mouse_enter = function (e) {
            this._trigger(this.move_enter, this._move_event(e), e);
        };
        UIEvents.prototype._mouse_move = function (e) {
            this._trigger(this.move, this._move_event(e), e);
        };
        UIEvents.prototype._mouse_exit = function (e) {
            this._trigger(this.move_exit, this._move_event(e), e);
        };
        UIEvents.prototype._mouse_wheel = function (e) {
            this._trigger(this.scroll, this._scroll_event(e), e);
        };
        UIEvents.prototype._context_menu = function (e) {
            if (!this.menu.is_open && this.menu.can_open) {
                e.preventDefault();
            }
            var _a = this._get_sxy(e), sx = _a.sx, sy = _a.sy;
            this.menu.toggle({ left: sx, top: sy });
        };
        UIEvents.prototype._key_down = function (e) {
            // NOTE: keyup event triggered unconditionally
            this.trigger(this.keydown, this._key_event(e));
        };
        UIEvents.prototype._key_up = function (e) {
            // NOTE: keyup event triggered unconditionally
            this.trigger(this.keyup, this._key_event(e));
        };
        return UIEvents;
    }());
    exports.UIEvents = UIEvents;
    UIEvents.__name__ = "UIEvents";
},
/* core/util/wheel.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    /*!
     * jQuery Mousewheel 3.1.13
     *
     * Copyright jQuery Foundation and other contributors
     * Released under the MIT license
     * http://jquery.org/license
     */
    function fontSize(element) {
        var value = getComputedStyle(element).fontSize;
        if (value != null)
            return parseInt(value, 10);
        return null;
    }
    function lineHeight(element) {
        var parent = element.offsetParent || document.body;
        return fontSize(parent) || fontSize(element) || 16;
    }
    function pageHeight(element) {
        return element.clientHeight; // XXX: should be content height?
    }
    function getDeltaY(event) {
        var deltaY = -event.deltaY;
        if (event.target instanceof HTMLElement) {
            switch (event.deltaMode) {
                case event.DOM_DELTA_LINE:
                    deltaY *= lineHeight(event.target);
                    break;
                case event.DOM_DELTA_PAGE:
                    deltaY *= pageHeight(event.target);
                    break;
            }
        }
        return deltaY;
    }
    exports.getDeltaY = getDeltaY;
},
/* core/util/throttle.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    function _delay_animation(callback) {
        callback(Date.now()); // XXX: performance.now()
        return -1;
    }
    var delay_animation = (typeof window !== 'undefined' ? window.requestAnimationFrame : undefined) ||
        (typeof window !== 'undefined' ? window.webkitRequestAnimationFrame : undefined) ||
        (typeof window !== 'undefined' ? window.mozRequestAnimationFrame : undefined) ||
        (typeof window !== 'undefined' ? window.msRequestAnimationFrame : undefined) || _delay_animation;
    // Returns a function, that, when invoked, will only be triggered at
    // most once during a given window of time.
    //
    // In addition, if the browser supports requestAnimationFrame, the
    // throttled function will be run no more frequently than request
    // animation frame allows.
    //
    // @param func [function] the function to throttle
    // @param wait [number] time in milliseconds to use for window
    // @return [function] throttled function
    //
    function throttle(func, wait) {
        var timeout = null;
        var previous = 0;
        var pending = false;
        return function () {
            return new Promise(function (resolve, reject) {
                var later = function () {
                    previous = Date.now();
                    timeout = null;
                    pending = false;
                    try {
                        func();
                        resolve();
                    }
                    catch (error) {
                        reject(error);
                    }
                };
                var now = Date.now();
                var remaining = wait - (now - previous);
                if (remaining <= 0 && !pending) {
                    if (timeout != null) {
                        clearTimeout(timeout);
                    }
                    pending = true;
                    delay_animation(later);
                }
                else if (!timeout && !pending) {
                    timeout = setTimeout(function () { return delay_animation(later); }, remaining);
                }
                else {
                    resolve();
                }
            });
        };
    }
    exports.throttle = throttle;
},
/* core/layout/border.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(328) /* ./types */;
    var layoutable_1 = require(329) /* ./layoutable */;
    var bbox_1 = require(194) /* ../util/bbox */;
    var BorderLayout = /** @class */ (function (_super) {
        tslib_1.__extends(BorderLayout, _super);
        function BorderLayout() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this.min_border = { left: 0, top: 0, right: 0, bottom: 0 };
            return _this;
        }
        BorderLayout.prototype._measure = function (viewport) {
            var _this = this;
            viewport = new types_1.Sizeable(viewport).bounded_to(this.sizing.size);
            var left_hint = this.left_panel.measure({ width: 0, height: viewport.height });
            var left = Math.max(left_hint.width, this.min_border.left);
            var right_hint = this.right_panel.measure({ width: 0, height: viewport.height });
            var right = Math.max(right_hint.width, this.min_border.right);
            var top_hint = this.top_panel.measure({ width: viewport.width, height: 0 });
            var top = Math.max(top_hint.height, this.min_border.top);
            var bottom_hint = this.bottom_panel.measure({ width: viewport.width, height: 0 });
            var bottom = Math.max(bottom_hint.height, this.min_border.bottom);
            var center_viewport = new types_1.Sizeable(viewport).shrink_by({ left: left, right: right, top: top, bottom: bottom });
            var center = this.center_panel.measure(center_viewport);
            var width = left + center.width + right;
            var height = top + center.height + bottom;
            var align = (function () {
                var _a = _this.center_panel.sizing, width_policy = _a.width_policy, height_policy = _a.height_policy;
                return width_policy != "fixed" && height_policy != "fixed";
            })();
            return { width: width, height: height, inner: { left: left, right: right, top: top, bottom: bottom }, align: align };
        };
        BorderLayout.prototype._set_geometry = function (outer, inner) {
            _super.prototype._set_geometry.call(this, outer, inner);
            this.center_panel.set_geometry(inner);
            var left_hint = this.left_panel.measure({ width: 0, height: outer.height });
            var right_hint = this.right_panel.measure({ width: 0, height: outer.height });
            var top_hint = this.top_panel.measure({ width: outer.width, height: 0 });
            var bottom_hint = this.bottom_panel.measure({ width: outer.width, height: 0 });
            var left = inner.left, top = inner.top, right = inner.right, bottom = inner.bottom;
            this.top_panel.set_geometry(new bbox_1.BBox({ left: left, right: right, bottom: top, height: top_hint.height }));
            this.bottom_panel.set_geometry(new bbox_1.BBox({ left: left, right: right, top: bottom, height: bottom_hint.height }));
            this.left_panel.set_geometry(new bbox_1.BBox({ top: top, bottom: bottom, right: left, width: left_hint.width }));
            this.right_panel.set_geometry(new bbox_1.BBox({ top: top, bottom: bottom, left: right, width: right_hint.width }));
        };
        return BorderLayout;
    }(layoutable_1.Layoutable));
    exports.BorderLayout = BorderLayout;
    BorderLayout.__name__ = "BorderLayout";
},
/* core/layout/side_panel.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var types_1 = require(328) /* ./types */;
    var layoutable_1 = require(329) /* ./layoutable */;
    var types_2 = require(123) /* ../util/types */;
    // This table lays out the rules for configuring the baseline, alignment, etc. of
    // title text, based on it's location and orientation
    //
    // side    orient        baseline   align     angle   normal-dist
    // ------------------------------------------------------------------------------
    // above   parallel      bottom     center    0       height
    //         normal        middle     left      -90     width
    //         horizontal    bottom     center    0       height
    //         [angle > 0]   middle     left              width * sin + height * cos
    //         [angle < 0]   middle     right             width * sin + height * cos
    //
    // below   parallel      top        center    0       height
    //         normal        middle     right     90      width
    //         horizontal    top        center    0       height
    //         [angle > 0]   middle     right             width * sin + height * cos
    //         [angle < 0]   middle     left              width * sin + height * cos
    //
    // left    parallel      bottom     center    90      height
    //         normal        middle     right     0       width
    //         horizontal    middle     right     0       width
    //         [angle > 0]   middle     right             width * cos + height * sin
    //         [angle < 0]   middle     right             width * cos + height + sin
    //
    // right   parallel      bottom     center   -90      height
    //         normal        middle     left     0        width
    //         horizontal    middle     left     0        width
    //         [angle > 0]   middle     left              width * cos + height * sin
    //         [angle < 0]   middle     left              width * cos + height + sin
    var pi2 = Math.PI / 2;
    var ALPHABETIC = 'alphabetic';
    var TOP = 'top';
    var BOTTOM = 'bottom';
    var MIDDLE = 'middle';
    var HANGING = 'hanging';
    var LEFT = 'left';
    var RIGHT = 'right';
    var CENTER = 'center';
    var _angle_lookup = {
        above: {
            parallel: 0,
            normal: -pi2,
            horizontal: 0,
            vertical: -pi2,
        },
        below: {
            parallel: 0,
            normal: pi2,
            horizontal: 0,
            vertical: pi2,
        },
        left: {
            parallel: -pi2,
            normal: 0,
            horizontal: 0,
            vertical: -pi2,
        },
        right: {
            parallel: pi2,
            normal: 0,
            horizontal: 0,
            vertical: pi2,
        },
    };
    var _baseline_lookup = {
        above: {
            justified: TOP,
            parallel: ALPHABETIC,
            normal: MIDDLE,
            horizontal: ALPHABETIC,
            vertical: MIDDLE,
        },
        below: {
            justified: BOTTOM,
            parallel: HANGING,
            normal: MIDDLE,
            horizontal: HANGING,
            vertical: MIDDLE,
        },
        left: {
            justified: TOP,
            parallel: ALPHABETIC,
            normal: MIDDLE,
            horizontal: MIDDLE,
            vertical: ALPHABETIC,
        },
        right: {
            justified: TOP,
            parallel: ALPHABETIC,
            normal: MIDDLE,
            horizontal: MIDDLE,
            vertical: ALPHABETIC,
        },
    };
    var _align_lookup = {
        above: {
            justified: CENTER,
            parallel: CENTER,
            normal: LEFT,
            horizontal: CENTER,
            vertical: LEFT,
        },
        below: {
            justified: CENTER,
            parallel: CENTER,
            normal: LEFT,
            horizontal: CENTER,
            vertical: LEFT,
        },
        left: {
            justified: CENTER,
            parallel: CENTER,
            normal: RIGHT,
            horizontal: RIGHT,
            vertical: CENTER,
        },
        right: {
            justified: CENTER,
            parallel: CENTER,
            normal: LEFT,
            horizontal: LEFT,
            vertical: CENTER,
        },
    };
    var _align_lookup_negative = {
        above: RIGHT,
        below: LEFT,
        left: RIGHT,
        right: LEFT,
    };
    var _align_lookup_positive = {
        above: LEFT,
        below: RIGHT,
        left: RIGHT,
        right: LEFT,
    };
    var SidePanel = /** @class */ (function (_super) {
        tslib_1.__extends(SidePanel, _super);
        function SidePanel(side, obj) {
            var _this = _super.call(this) || this;
            _this.side = side;
            _this.obj = obj;
            switch (_this.side) {
                case "above":
                    _this._dim = 0;
                    _this._normals = [0, -1];
                    break;
                case "below":
                    _this._dim = 0;
                    _this._normals = [0, 1];
                    break;
                case "left":
                    _this._dim = 1;
                    _this._normals = [-1, 0];
                    break;
                case "right":
                    _this._dim = 1;
                    _this._normals = [1, 0];
                    break;
            }
            if (_this.is_horizontal)
                _this.set_sizing({ width_policy: "max", height_policy: "fixed" });
            else
                _this.set_sizing({ width_policy: "fixed", height_policy: "max" });
            return _this;
        }
        SidePanel.prototype._content_size = function () {
            return new types_1.Sizeable(this.get_oriented_size());
        };
        SidePanel.prototype.get_oriented_size = function () {
            var _a = this.obj.get_size(), width = _a.width, height = _a.height;
            if (!this.obj.rotate || this.is_horizontal)
                return { width: width, height: height };
            else
                return { width: height, height: width };
        };
        SidePanel.prototype.has_size_changed = function () {
            var _a = this.get_oriented_size(), width = _a.width, height = _a.height;
            if (this.is_horizontal)
                return this.bbox.height != height;
            else
                return this.bbox.width != width;
        };
        Object.defineProperty(SidePanel.prototype, "dimension", {
            get: function () {
                return this._dim;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SidePanel.prototype, "normals", {
            get: function () {
                return this._normals;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SidePanel.prototype, "is_horizontal", {
            get: function () {
                return this._dim == 0;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SidePanel.prototype, "is_vertical", {
            get: function () {
                return this._dim == 1;
            },
            enumerable: true,
            configurable: true
        });
        SidePanel.prototype.apply_label_text_heuristics = function (ctx, orient) {
            var side = this.side;
            var baseline;
            var align;
            if (types_2.isString(orient)) {
                baseline = _baseline_lookup[side][orient];
                align = _align_lookup[side][orient];
            }
            else {
                if (orient < 0) {
                    baseline = 'middle';
                    align = _align_lookup_negative[side];
                }
                else {
                    baseline = 'middle';
                    align = _align_lookup_positive[side];
                }
            }
            ctx.textBaseline = baseline;
            ctx.textAlign = align;
        };
        SidePanel.prototype.get_label_angle_heuristic = function (orient) {
            return _angle_lookup[this.side][orient];
        };
        return SidePanel;
    }(layoutable_1.ContentLayoutable));
    exports.SidePanel = SidePanel;
    SidePanel.__name__ = "SidePanel";
},
/* models/plots/gmap_plot_canvas.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var signaling_1 = require(130) /* ../../core/signaling */;
    var dom_1 = require(187) /* ../../core/dom */;
    var projections_1 = require(152) /* ../../core/util/projections */;
    var plot_canvas_1 = require(427) /* ./plot_canvas */;
    var gmaps_ready = new signaling_1.Signal0({}, "gmaps_ready");
    var load_google_api = function (api_key) {
        window._bokeh_gmaps_callback = function () { return gmaps_ready.emit(); };
        var script = document.createElement('script');
        script.type = 'text/javascript';
        script.src = "https://maps.googleapis.com/maps/api/js?v=3.36&key=" + api_key + "&callback=_bokeh_gmaps_callback";
        document.body.appendChild(script);
    };
    var GMapPlotView = /** @class */ (function (_super) {
        tslib_1.__extends(GMapPlotView, _super);
        function GMapPlotView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        GMapPlotView.prototype.initialize = function () {
            var _this = this;
            this.pause();
            _super.prototype.initialize.call(this);
            this._tiles_loaded = false;
            this.zoom_count = 0;
            var _a = this.model.map_options, zoom = _a.zoom, lat = _a.lat, lng = _a.lng;
            this.initial_zoom = zoom;
            this.initial_lat = lat;
            this.initial_lng = lng;
            if (typeof google === "undefined" || google.maps == null) {
                if (typeof window._bokeh_gmaps_callback === "undefined") {
                    var decoded_api_key = atob(this.model.api_key);
                    load_google_api(decoded_api_key);
                }
                gmaps_ready.connect(function () { return _this.request_render(); });
            }
            this.unpause();
        };
        GMapPlotView.prototype.remove = function () {
            dom_1.remove(this.map_el);
            _super.prototype.remove.call(this);
        };
        GMapPlotView.prototype.update_range = function (range_info) {
            // RESET -------------------------
            if (range_info == null) {
                this.map.setCenter({ lat: this.initial_lat, lng: this.initial_lng });
                this.map.setOptions({ zoom: this.initial_zoom });
                _super.prototype.update_range.call(this, null);
                // PAN ----------------------------
            }
            else if (range_info.sdx != null || range_info.sdy != null) {
                this.map.panBy(range_info.sdx || 0, range_info.sdy || 0);
                _super.prototype.update_range.call(this, range_info);
                // ZOOM ---------------------------
            }
            else if (range_info.factor != null) {
                // The zoom count decreases the sensitivity of the zoom. (We could make this user configurable)
                if (this.zoom_count !== 10) {
                    this.zoom_count += 1;
                    return;
                }
                this.zoom_count = 0;
                this.pause();
                _super.prototype.update_range.call(this, range_info);
                var zoom_change = range_info.factor < 0 ? -1 : 1;
                var old_map_zoom = this.map.getZoom();
                var new_map_zoom = old_map_zoom + zoom_change;
                // Zooming out too far causes problems
                if (new_map_zoom >= 2) {
                    this.map.setZoom(new_map_zoom);
                    // Check we haven't gone out of bounds, and if we have undo the zoom
                    var _a = tslib_1.__read(this._get_projected_bounds(), 3), proj_xstart = _a[0], proj_xend = _a[1];
                    if (proj_xend - proj_xstart < 0) {
                        this.map.setZoom(old_map_zoom);
                    }
                }
                this.unpause();
            }
            // Finally re-center
            this._set_bokeh_ranges();
        };
        GMapPlotView.prototype._build_map = function () {
            var _this = this;
            var maps = google.maps;
            this.map_types = {
                satellite: maps.MapTypeId.SATELLITE,
                terrain: maps.MapTypeId.TERRAIN,
                roadmap: maps.MapTypeId.ROADMAP,
                hybrid: maps.MapTypeId.HYBRID,
            };
            var mo = this.model.map_options;
            var map_options = {
                center: new maps.LatLng(mo.lat, mo.lng),
                zoom: mo.zoom,
                disableDefaultUI: true,
                mapTypeId: this.map_types[mo.map_type],
                scaleControl: mo.scale_control,
                tilt: mo.tilt,
            };
            if (mo.styles != null)
                map_options.styles = JSON.parse(mo.styles);
            // create the map with above options in div
            this.map_el = dom_1.div({ style: { position: "absolute" } });
            this.canvas_view.add_underlay(this.map_el);
            this.map = new maps.Map(this.map_el, map_options);
            // update bokeh ranges whenever the map idles, which should be after most UI action
            maps.event.addListener(this.map, 'idle', function () { return _this._set_bokeh_ranges(); });
            // also need an event when bounds change so that map resizes trigger renders too
            maps.event.addListener(this.map, 'bounds_changed', function () { return _this._set_bokeh_ranges(); });
            maps.event.addListenerOnce(this.map, 'tilesloaded', function () { return _this._render_finished(); });
            // wire up listeners so that changes to properties are reflected
            this.connect(this.model.properties.map_options.change, function () { return _this._update_options(); });
            this.connect(this.model.map_options.properties.styles.change, function () { return _this._update_styles(); });
            this.connect(this.model.map_options.properties.lat.change, function () { return _this._update_center('lat'); });
            this.connect(this.model.map_options.properties.lng.change, function () { return _this._update_center('lng'); });
            this.connect(this.model.map_options.properties.zoom.change, function () { return _this._update_zoom(); });
            this.connect(this.model.map_options.properties.map_type.change, function () { return _this._update_map_type(); });
            this.connect(this.model.map_options.properties.scale_control.change, function () { return _this._update_scale_control(); });
            this.connect(this.model.map_options.properties.tilt.change, function () { return _this._update_tilt(); });
        };
        GMapPlotView.prototype._render_finished = function () {
            this._tiles_loaded = true;
            this.notify_finished();
        };
        GMapPlotView.prototype.has_finished = function () {
            return _super.prototype.has_finished.call(this) && this._tiles_loaded === true;
        };
        GMapPlotView.prototype._get_latlon_bounds = function () {
            var bounds = this.map.getBounds();
            var top_right = bounds.getNorthEast();
            var bottom_left = bounds.getSouthWest();
            var xstart = bottom_left.lng();
            var xend = top_right.lng();
            var ystart = bottom_left.lat();
            var yend = top_right.lat();
            return [xstart, xend, ystart, yend];
        };
        GMapPlotView.prototype._get_projected_bounds = function () {
            var _a = tslib_1.__read(this._get_latlon_bounds(), 4), xstart = _a[0], xend = _a[1], ystart = _a[2], yend = _a[3];
            var _b = tslib_1.__read(projections_1.wgs84_mercator.compute(xstart, ystart), 2), proj_xstart = _b[0], proj_ystart = _b[1];
            var _c = tslib_1.__read(projections_1.wgs84_mercator.compute(xend, yend), 2), proj_xend = _c[0], proj_yend = _c[1];
            return [proj_xstart, proj_xend, proj_ystart, proj_yend];
        };
        GMapPlotView.prototype._set_bokeh_ranges = function () {
            var _a = tslib_1.__read(this._get_projected_bounds(), 4), proj_xstart = _a[0], proj_xend = _a[1], proj_ystart = _a[2], proj_yend = _a[3];
            this.frame.x_range.setv({ start: proj_xstart, end: proj_xend });
            this.frame.y_range.setv({ start: proj_ystart, end: proj_yend });
        };
        GMapPlotView.prototype._update_center = function (fld) {
            var c = this.map.getCenter().toJSON();
            c[fld] = this.model.map_options[fld];
            this.map.setCenter(c);
            this._set_bokeh_ranges();
        };
        GMapPlotView.prototype._update_map_type = function () {
            this.map.setOptions({ mapTypeId: this.map_types[this.model.map_options.map_type] });
        };
        GMapPlotView.prototype._update_scale_control = function () {
            this.map.setOptions({ scaleControl: this.model.map_options.scale_control });
        };
        GMapPlotView.prototype._update_tilt = function () {
            this.map.setOptions({ tilt: this.model.map_options.tilt });
        };
        GMapPlotView.prototype._update_options = function () {
            this._update_styles();
            this._update_center('lat');
            this._update_center('lng');
            this._update_zoom();
            this._update_map_type();
        };
        GMapPlotView.prototype._update_styles = function () {
            this.map.setOptions({ styles: JSON.parse(this.model.map_options.styles) });
        };
        GMapPlotView.prototype._update_zoom = function () {
            this.map.setOptions({ zoom: this.model.map_options.zoom });
            this._set_bokeh_ranges();
        };
        // this method is expected and called by PlotView.render
        GMapPlotView.prototype._map_hook = function (_ctx, frame_box) {
            if (this.map == null && typeof google !== "undefined" && google.maps != null)
                this._build_map();
            if (this.map_el != null) {
                var _a = tslib_1.__read(frame_box, 4), left = _a[0], top = _a[1], width = _a[2], height = _a[3];
                this.map_el.style.top = top + "px";
                this.map_el.style.left = left + "px";
                this.map_el.style.width = width + "px";
                this.map_el.style.height = height + "px";
            }
        };
        // this overrides the standard _paint_empty to make the inner canvas transparent
        GMapPlotView.prototype._paint_empty = function (ctx, frame_box) {
            var ow = this.layout.bbox.width;
            var oh = this.layout.bbox.height;
            var _a = tslib_1.__read(frame_box, 4), left = _a[0], top = _a[1], iw = _a[2], ih = _a[3];
            ctx.clearRect(0, 0, ow, oh);
            ctx.beginPath();
            ctx.moveTo(0, 0);
            ctx.lineTo(0, oh);
            ctx.lineTo(ow, oh);
            ctx.lineTo(ow, 0);
            ctx.lineTo(0, 0);
            ctx.moveTo(left, top);
            ctx.lineTo(left + iw, top);
            ctx.lineTo(left + iw, top + ih);
            ctx.lineTo(left, top + ih);
            ctx.lineTo(left, top);
            ctx.closePath();
            if (this.model.border_fill_color != null) {
                ctx.fillStyle = this.model.border_fill_color;
                ctx.fill();
            }
        };
        return GMapPlotView;
    }(plot_canvas_1.PlotView));
    exports.GMapPlotView = GMapPlotView;
    GMapPlotView.__name__ = "GMapPlotView";
},
/* models/ranges/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var data_range_1 = require(326) /* ./data_range */;
    exports.DataRange = data_range_1.DataRange;
    var data_range1d_1 = require(325) /* ./data_range1d */;
    exports.DataRange1d = data_range1d_1.DataRange1d;
    var factor_range_1 = require(213) /* ./factor_range */;
    exports.FactorRange = factor_range_1.FactorRange;
    var range_1 = require(214) /* ./range */;
    exports.Range = range_1.Range;
    var range1d_1 = require(273) /* ./range1d */;
    exports.Range1d = range1d_1.Range1d;
},
/* models/renderers/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var glyph_renderer_1 = require(205) /* ./glyph_renderer */;
    exports.GlyphRenderer = glyph_renderer_1.GlyphRenderer;
    var graph_renderer_1 = require(231) /* ./graph_renderer */;
    exports.GraphRenderer = graph_renderer_1.GraphRenderer;
    var guide_renderer_1 = require(293) /* ./guide_renderer */;
    exports.GuideRenderer = guide_renderer_1.GuideRenderer;
    var renderer_1 = require(185) /* ./renderer */;
    exports.Renderer = renderer_1.Renderer;
},
/* models/scales/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var categorical_scale_1 = require(324) /* ./categorical_scale */;
    exports.CategoricalScale = categorical_scale_1.CategoricalScale;
    var continuous_scale_1 = require(261) /* ./continuous_scale */;
    exports.ContinuousScale = continuous_scale_1.ContinuousScale;
    var linear_scale_1 = require(260) /* ./linear_scale */;
    exports.LinearScale = linear_scale_1.LinearScale;
    var linear_interpolation_scale_1 = require(271) /* ./linear_interpolation_scale */;
    exports.LinearInterpolationScale = linear_interpolation_scale_1.LinearInterpolationScale;
    var log_scale_1 = require(272) /* ./log_scale */;
    exports.LogScale = log_scale_1.LogScale;
    var scale_1 = require(262) /* ./scale */;
    exports.Scale = scale_1.Scale;
},
/* models/selections/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    tslib_1.__exportStar(require(233) /* ./interaction_policy */, exports);
    var selection_1 = require(203) /* ./selection */;
    exports.Selection = selection_1.Selection;
},
/* models/sources/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var server_sent_data_source_1 = require(440) /* ./server_sent_data_source */;
    exports.ServerSentDataSource = server_sent_data_source_1.ServerSentDataSource;
    var ajax_data_source_1 = require(442) /* ./ajax_data_source */;
    exports.AjaxDataSource = ajax_data_source_1.AjaxDataSource;
    var column_data_source_1 = require(200) /* ./column_data_source */;
    exports.ColumnDataSource = column_data_source_1.ColumnDataSource;
    var columnar_data_source_1 = require(201) /* ./columnar_data_source */;
    exports.ColumnarDataSource = columnar_data_source_1.ColumnarDataSource;
    var cds_view_1 = require(229) /* ./cds_view */;
    exports.CDSView = cds_view_1.CDSView;
    var data_source_1 = require(202) /* ./data_source */;
    exports.DataSource = data_source_1.DataSource;
    var geojson_data_source_1 = require(443) /* ./geojson_data_source */;
    exports.GeoJSONDataSource = geojson_data_source_1.GeoJSONDataSource;
    var web_data_source_1 = require(441) /* ./web_data_source */;
    exports.WebDataSource = web_data_source_1.WebDataSource;
},
/* models/sources/server_sent_data_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var web_data_source_1 = require(441) /* ./web_data_source */;
    var ServerSentDataSource = /** @class */ (function (_super) {
        tslib_1.__extends(ServerSentDataSource, _super);
        function ServerSentDataSource(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.initialized = false;
            return _this;
        }
        ServerSentDataSource.prototype.destroy = function () {
            _super.prototype.destroy.call(this);
        };
        ServerSentDataSource.prototype.setup = function () {
            var _this = this;
            if (!this.initialized) {
                this.initialized = true;
                var source = new EventSource(this.data_url);
                source.onmessage = function (event) {
                    _this.load_data(JSON.parse(event.data), _this.mode, _this.max_size);
                };
            }
        };
        return ServerSentDataSource;
    }(web_data_source_1.WebDataSource));
    exports.ServerSentDataSource = ServerSentDataSource;
    ServerSentDataSource.__name__ = "ServerSentDataSource";
},
/* models/sources/web_data_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var column_data_source_1 = require(200) /* ./column_data_source */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var WebDataSource = /** @class */ (function (_super) {
        tslib_1.__extends(WebDataSource, _super);
        function WebDataSource(attrs) {
            return _super.call(this, attrs) || this;
        }
        WebDataSource.prototype.get_column = function (colname) {
            var column = this.data[colname];
            return column != null ? column : [];
        };
        WebDataSource.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.setup();
        };
        WebDataSource.prototype.load_data = function (raw_data, mode, max_size) {
            var e_1, _a;
            var adapter = this.adapter;
            var data;
            if (adapter != null)
                data = adapter.execute(this, { response: raw_data });
            else
                data = raw_data;
            switch (mode) {
                case "replace": {
                    this.data = data;
                    break;
                }
                case "append": {
                    var original_data = this.data;
                    try {
                        for (var _b = tslib_1.__values(this.columns()), _c = _b.next(); !_c.done; _c = _b.next()) {
                            var column = _c.value;
                            // XXX: support typed arrays
                            var old_col = Array.from(original_data[column]);
                            var new_col = Array.from(data[column]);
                            data[column] = old_col.concat(new_col).slice(-max_size);
                        }
                    }
                    catch (e_1_1) {
                        e_1 = { error: e_1_1 };
                    }
                    finally {
                        try {
                            if (_c && !_c.done && (_a = _b.return))
                                _a.call(_b);
                        }
                        finally {
                            if (e_1)
                                throw e_1.error;
                        }
                    }
                    this.data = data;
                    break;
                }
            }
        };
        WebDataSource.init_WebDataSource = function () {
            this.define({
                mode: [p.UpdateMode, 'replace'],
                max_size: [p.Number],
                adapter: [p.Any, null],
                data_url: [p.String],
            });
        };
        return WebDataSource;
    }(column_data_source_1.ColumnDataSource));
    exports.WebDataSource = WebDataSource;
    WebDataSource.__name__ = "WebDataSource";
    WebDataSource.init_WebDataSource();
},
/* models/sources/ajax_data_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var web_data_source_1 = require(441) /* ./web_data_source */;
    var logging_1 = require(134) /* ../../core/logging */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var object_1 = require(128) /* ../../core/util/object */;
    var AjaxDataSource = /** @class */ (function (_super) {
        tslib_1.__extends(AjaxDataSource, _super);
        function AjaxDataSource(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.initialized = false;
            return _this;
        }
        AjaxDataSource.init_AjaxDataSource = function () {
            this.define({
                polling_interval: [p.Number],
                content_type: [p.String, 'application/json'],
                http_headers: [p.Any, {}],
                method: [p.HTTPMethod, 'POST'],
                if_modified: [p.Boolean, false],
            });
        };
        AjaxDataSource.prototype.destroy = function () {
            if (this.interval != null)
                clearInterval(this.interval);
            _super.prototype.destroy.call(this);
        };
        AjaxDataSource.prototype.setup = function () {
            var _this = this;
            if (!this.initialized) {
                this.initialized = true;
                this.get_data(this.mode);
                if (this.polling_interval) {
                    var callback = function () { return _this.get_data(_this.mode, _this.max_size, _this.if_modified); };
                    this.interval = setInterval(callback, this.polling_interval);
                }
            }
        };
        AjaxDataSource.prototype.get_data = function (mode, max_size, _if_modified) {
            var _this = this;
            if (max_size === void 0) {
                max_size = 0;
            }
            if (_if_modified === void 0) {
                _if_modified = false;
            }
            var xhr = this.prepare_request();
            // TODO: if_modified
            xhr.addEventListener("load", function () { return _this.do_load(xhr, mode, max_size); });
            xhr.addEventListener("error", function () { return _this.do_error(xhr); });
            xhr.send();
        };
        AjaxDataSource.prototype.prepare_request = function () {
            var e_1, _a;
            var xhr = new XMLHttpRequest();
            xhr.open(this.method, this.data_url, true);
            xhr.withCredentials = false;
            xhr.setRequestHeader("Content-Type", this.content_type);
            var http_headers = this.http_headers;
            try {
                for (var _b = tslib_1.__values(object_1.entries(http_headers)), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var _d = tslib_1.__read(_c.value, 2), name = _d[0], value = _d[1];
                    xhr.setRequestHeader(name, value);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return xhr;
        };
        AjaxDataSource.prototype.do_load = function (xhr, mode, max_size) {
            if (xhr.status === 200) {
                var raw_data = JSON.parse(xhr.responseText);
                this.load_data(raw_data, mode, max_size);
            }
        };
        AjaxDataSource.prototype.do_error = function (xhr) {
            logging_1.logger.error("Failed to fetch JSON from " + this.data_url + " with code " + xhr.status);
        };
        return AjaxDataSource;
    }(web_data_source_1.WebDataSource));
    exports.AjaxDataSource = AjaxDataSource;
    AjaxDataSource.__name__ = "AjaxDataSource";
    AjaxDataSource.init_AjaxDataSource();
},
/* models/sources/geojson_data_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var columnar_data_source_1 = require(201) /* ./columnar_data_source */;
    var logging_1 = require(134) /* ../../core/logging */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    var object_1 = require(128) /* ../../core/util/object */;
    function orNaN(v) {
        return v != null ? v : NaN;
    }
    var GeoJSONDataSource = /** @class */ (function (_super) {
        tslib_1.__extends(GeoJSONDataSource, _super);
        function GeoJSONDataSource(attrs) {
            return _super.call(this, attrs) || this;
        }
        GeoJSONDataSource.init_GeoJSONDataSource = function () {
            this.define({
                geojson: [p.Any],
            });
            this.internal({
                data: [p.Any, {}],
            });
        };
        GeoJSONDataSource.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._update_data();
        };
        GeoJSONDataSource.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.properties.geojson.change, function () { return _this._update_data(); });
        };
        GeoJSONDataSource.prototype._update_data = function () {
            this.data = this.geojson_to_column_data();
        };
        GeoJSONDataSource.prototype._get_new_list_array = function (length) {
            return array_1.range(0, length).map(function (_i) { return []; });
        };
        GeoJSONDataSource.prototype._get_new_nan_array = function (length) {
            return array_1.range(0, length).map(function (_i) { return NaN; });
        };
        GeoJSONDataSource.prototype._add_properties = function (item, data, i, item_count) {
            var e_1, _b;
            var _a;
            var properties = (_a = item.properties) !== null && _a !== void 0 ? _a : {};
            try {
                for (var _c = tslib_1.__values(object_1.entries(properties)), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = tslib_1.__read(_d.value, 2), property = _e[0], value = _e[1];
                    if (!data.hasOwnProperty(property))
                        data[property] = this._get_new_nan_array(item_count);
                    // orNaN necessary here to prevent null values from ending up in the column
                    data[property][i] = orNaN(value);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        GeoJSONDataSource.prototype._add_geometry = function (geometry, data, i) {
            var e_2, _b;
            function flatten(acc, item) {
                return acc.concat([[NaN, NaN, NaN]]).concat(item);
            }
            switch (geometry.type) {
                case "Point": {
                    var _c = tslib_1.__read(geometry.coordinates, 3), x = _c[0], y = _c[1], z = _c[2];
                    data.x[i] = x;
                    data.y[i] = y;
                    data.z[i] = orNaN(z);
                    break;
                }
                case "LineString": {
                    var coordinates = geometry.coordinates;
                    for (var j = 0; j < coordinates.length; j++) {
                        var _d = tslib_1.__read(coordinates[j], 3), x = _d[0], y = _d[1], z = _d[2];
                        data.xs[i][j] = x;
                        data.ys[i][j] = y;
                        data.zs[i][j] = orNaN(z);
                    }
                    break;
                }
                case "Polygon": {
                    if (geometry.coordinates.length > 1)
                        logging_1.logger.warn('Bokeh does not support Polygons with holes in, only exterior ring used.');
                    var exterior_ring = geometry.coordinates[0];
                    for (var j = 0; j < exterior_ring.length; j++) {
                        var _e = tslib_1.__read(exterior_ring[j], 3), x = _e[0], y = _e[1], z = _e[2];
                        data.xs[i][j] = x;
                        data.ys[i][j] = y;
                        data.zs[i][j] = orNaN(z);
                    }
                    break;
                }
                case "MultiPoint": {
                    logging_1.logger.warn('MultiPoint not supported in Bokeh');
                    break;
                }
                case "MultiLineString": {
                    var coordinates = geometry.coordinates.reduce(flatten);
                    for (var j = 0; j < coordinates.length; j++) {
                        var _f = tslib_1.__read(coordinates[j], 3), x = _f[0], y = _f[1], z = _f[2];
                        data.xs[i][j] = x;
                        data.ys[i][j] = y;
                        data.zs[i][j] = orNaN(z);
                    }
                    break;
                }
                case "MultiPolygon": {
                    var exterior_rings = [];
                    try {
                        for (var _g = tslib_1.__values(geometry.coordinates), _h = _g.next(); !_h.done; _h = _g.next()) {
                            var polygon = _h.value;
                            if (polygon.length > 1)
                                logging_1.logger.warn('Bokeh does not support Polygons with holes in, only exterior ring used.');
                            exterior_rings.push(polygon[0]);
                        }
                    }
                    catch (e_2_1) {
                        e_2 = { error: e_2_1 };
                    }
                    finally {
                        try {
                            if (_h && !_h.done && (_b = _g.return))
                                _b.call(_g);
                        }
                        finally {
                            if (e_2)
                                throw e_2.error;
                        }
                    }
                    var coordinates = exterior_rings.reduce(flatten);
                    for (var j = 0; j < coordinates.length; j++) {
                        var _j = tslib_1.__read(coordinates[j], 3), x = _j[0], y = _j[1], z = _j[2];
                        data.xs[i][j] = x;
                        data.ys[i][j] = y;
                        data.zs[i][j] = orNaN(z);
                    }
                    break;
                }
                default:
                    throw new Error("Invalid GeoJSON geometry type: " + geometry.type);
            }
        };
        GeoJSONDataSource.prototype.geojson_to_column_data = function () {
            var e_3, _b, e_4, _c, e_5, _d;
            var geojson = JSON.parse(this.geojson);
            var items;
            switch (geojson.type) {
                case "GeometryCollection": {
                    if (geojson.geometries == null)
                        throw new Error('No geometries found in GeometryCollection');
                    if (geojson.geometries.length === 0)
                        throw new Error('geojson.geometries must have one or more items');
                    items = geojson.geometries;
                    break;
                }
                case "FeatureCollection": {
                    if (geojson.features == null)
                        throw new Error('No features found in FeaturesCollection');
                    if (geojson.features.length == 0)
                        throw new Error('geojson.features must have one or more items');
                    items = geojson.features;
                    break;
                }
                default:
                    throw new Error('Bokeh only supports type GeometryCollection and FeatureCollection at top level');
            }
            var item_count = 0;
            try {
                for (var items_1 = tslib_1.__values(items), items_1_1 = items_1.next(); !items_1_1.done; items_1_1 = items_1.next()) {
                    var item = items_1_1.value;
                    var geometry = item.type === 'Feature' ? item.geometry : item;
                    if (geometry.type == 'GeometryCollection')
                        item_count += geometry.geometries.length;
                    else
                        item_count += 1;
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (items_1_1 && !items_1_1.done && (_b = items_1.return))
                        _b.call(items_1);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            var data = {
                x: this._get_new_nan_array(item_count),
                y: this._get_new_nan_array(item_count),
                z: this._get_new_nan_array(item_count),
                xs: this._get_new_list_array(item_count),
                ys: this._get_new_list_array(item_count),
                zs: this._get_new_list_array(item_count),
            };
            var arr_index = 0;
            try {
                for (var items_2 = tslib_1.__values(items), items_2_1 = items_2.next(); !items_2_1.done; items_2_1 = items_2.next()) {
                    var item = items_2_1.value;
                    var geometry = item.type == 'Feature' ? item.geometry : item;
                    if (geometry.type == "GeometryCollection") {
                        try {
                            for (var _e = (e_5 = void 0, tslib_1.__values(geometry.geometries)), _f = _e.next(); !_f.done; _f = _e.next()) {
                                var g = _f.value;
                                this._add_geometry(g, data, arr_index);
                                if (item.type === 'Feature')
                                    this._add_properties(item, data, arr_index, item_count);
                                arr_index += 1;
                            }
                        }
                        catch (e_5_1) {
                            e_5 = { error: e_5_1 };
                        }
                        finally {
                            try {
                                if (_f && !_f.done && (_d = _e.return))
                                    _d.call(_e);
                            }
                            finally {
                                if (e_5)
                                    throw e_5.error;
                            }
                        }
                    }
                    else {
                        this._add_geometry(geometry, data, arr_index);
                        if (item.type === 'Feature')
                            this._add_properties(item, data, arr_index, item_count);
                        arr_index += 1;
                    }
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (items_2_1 && !items_2_1.done && (_c = items_2.return))
                        _c.call(items_2);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            return data;
        };
        return GeoJSONDataSource;
    }(columnar_data_source_1.ColumnarDataSource));
    exports.GeoJSONDataSource = GeoJSONDataSource;
    GeoJSONDataSource.__name__ = "GeoJSONDataSource";
    GeoJSONDataSource.init_GeoJSONDataSource();
},
/* models/tickers/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var adaptive_ticker_1 = require(242) /* ./adaptive_ticker */;
    exports.AdaptiveTicker = adaptive_ticker_1.AdaptiveTicker;
    var basic_ticker_1 = require(241) /* ./basic_ticker */;
    exports.BasicTicker = basic_ticker_1.BasicTicker;
    var categorical_ticker_1 = require(295) /* ./categorical_ticker */;
    exports.CategoricalTicker = categorical_ticker_1.CategoricalTicker;
    var composite_ticker_1 = require(306) /* ./composite_ticker */;
    exports.CompositeTicker = composite_ticker_1.CompositeTicker;
    var continuous_ticker_1 = require(243) /* ./continuous_ticker */;
    exports.ContinuousTicker = continuous_ticker_1.ContinuousTicker;
    var datetime_ticker_1 = require(305) /* ./datetime_ticker */;
    exports.DatetimeTicker = datetime_ticker_1.DatetimeTicker;
    var days_ticker_1 = require(307) /* ./days_ticker */;
    exports.DaysTicker = days_ticker_1.DaysTicker;
    var fixed_ticker_1 = require(445) /* ./fixed_ticker */;
    exports.FixedTicker = fixed_ticker_1.FixedTicker;
    var log_ticker_1 = require(314) /* ./log_ticker */;
    exports.LogTicker = log_ticker_1.LogTicker;
    var mercator_ticker_1 = require(317) /* ./mercator_ticker */;
    exports.MercatorTicker = mercator_ticker_1.MercatorTicker;
    var months_ticker_1 = require(310) /* ./months_ticker */;
    exports.MonthsTicker = months_ticker_1.MonthsTicker;
    var single_interval_ticker_1 = require(308) /* ./single_interval_ticker */;
    exports.SingleIntervalTicker = single_interval_ticker_1.SingleIntervalTicker;
    var ticker_1 = require(244) /* ./ticker */;
    exports.Ticker = ticker_1.Ticker;
    var years_ticker_1 = require(311) /* ./years_ticker */;
    exports.YearsTicker = years_ticker_1.YearsTicker;
},
/* models/tickers/fixed_ticker.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var continuous_ticker_1 = require(243) /* ./continuous_ticker */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var FixedTicker = /** @class */ (function (_super) {
        tslib_1.__extends(FixedTicker, _super);
        function FixedTicker(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.min_interval = 0;
            _this.max_interval = 0;
            return _this;
        }
        FixedTicker.init_FixedTicker = function () {
            this.define({
                ticks: [p.Array, []],
                minor_ticks: [p.Array, []],
            });
        };
        FixedTicker.prototype.get_ticks_no_defaults = function (_data_low, _data_high, _cross_loc, _desired_n_ticks) {
            return {
                major: this.ticks,
                minor: this.minor_ticks,
            };
        };
        // XXX: whatever, because FixedTicker needs to fulfill the interface somehow
        FixedTicker.prototype.get_interval = function (_data_low, _data_high, _desired_n_ticks) {
            return 0;
        };
        return FixedTicker;
    }(continuous_ticker_1.ContinuousTicker));
    exports.FixedTicker = FixedTicker;
    FixedTicker.__name__ = "FixedTicker";
    FixedTicker.init_FixedTicker();
},
/* models/tiles/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var bbox_tile_source_1 = require(447) /* ./bbox_tile_source */;
    exports.BBoxTileSource = bbox_tile_source_1.BBoxTileSource;
    var mercator_tile_source_1 = require(448) /* ./mercator_tile_source */;
    exports.MercatorTileSource = mercator_tile_source_1.MercatorTileSource;
    var quadkey_tile_source_1 = require(451) /* ./quadkey_tile_source */;
    exports.QUADKEYTileSource = quadkey_tile_source_1.QUADKEYTileSource;
    var tile_renderer_1 = require(452) /* ./tile_renderer */;
    exports.TileRenderer = tile_renderer_1.TileRenderer;
    var tile_source_1 = require(449) /* ./tile_source */;
    exports.TileSource = tile_source_1.TileSource;
    var tms_tile_source_1 = require(456) /* ./tms_tile_source */;
    exports.TMSTileSource = tms_tile_source_1.TMSTileSource;
    var wmts_tile_source_1 = require(453) /* ./wmts_tile_source */;
    exports.WMTSTileSource = wmts_tile_source_1.WMTSTileSource;
},
/* models/tiles/bbox_tile_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var mercator_tile_source_1 = require(448) /* ./mercator_tile_source */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var BBoxTileSource = /** @class */ (function (_super) {
        tslib_1.__extends(BBoxTileSource, _super);
        function BBoxTileSource(attrs) {
            return _super.call(this, attrs) || this;
        }
        BBoxTileSource.init_BBoxTileSource = function () {
            this.define({
                use_latlon: [p.Boolean, false],
            });
        };
        BBoxTileSource.prototype.get_image_url = function (x, y, z) {
            var _a, _b;
            var image_url = this.string_lookup_replace(this.url, this.extra_url_vars);
            var xmax, xmin, ymax, ymin;
            if (this.use_latlon)
                _a = tslib_1.__read(this.get_tile_geographic_bounds(x, y, z), 4), xmin = _a[0], ymin = _a[1], xmax = _a[2], ymax = _a[3];
            else
                _b = tslib_1.__read(this.get_tile_meter_bounds(x, y, z), 4), xmin = _b[0], ymin = _b[1], xmax = _b[2], ymax = _b[3];
            return image_url
                .replace("{XMIN}", xmin.toString())
                .replace("{YMIN}", ymin.toString())
                .replace("{XMAX}", xmax.toString())
                .replace("{YMAX}", ymax.toString());
        };
        return BBoxTileSource;
    }(mercator_tile_source_1.MercatorTileSource));
    exports.BBoxTileSource = BBoxTileSource;
    BBoxTileSource.__name__ = "BBoxTileSource";
    BBoxTileSource.init_BBoxTileSource();
},
/* models/tiles/mercator_tile_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var tile_source_1 = require(449) /* ./tile_source */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    var tile_utils_1 = require(450) /* ./tile_utils */;
    var MercatorTileSource = /** @class */ (function (_super) {
        tslib_1.__extends(MercatorTileSource, _super);
        function MercatorTileSource(attrs) {
            return _super.call(this, attrs) || this;
        }
        MercatorTileSource.init_MercatorTileSource = function () {
            this.define({
                snap_to_zoom: [p.Boolean, false],
                wrap_around: [p.Boolean, true],
            });
            this.override({
                x_origin_offset: 20037508.34,
                y_origin_offset: 20037508.34,
                initial_resolution: 156543.03392804097,
            });
        };
        MercatorTileSource.prototype.initialize = function () {
            var _this = this;
            _super.prototype.initialize.call(this);
            this._resolutions = array_1.range(this.min_zoom, this.max_zoom + 1).map(function (z) { return _this.get_resolution(z); });
        };
        MercatorTileSource.prototype._computed_initial_resolution = function () {
            if (this.initial_resolution != null)
                return this.initial_resolution;
            else {
                // TODO testing 2015-11-17, if this codepath is used it seems
                // to use 100% cpu and wedge Chrome
                return (2 * Math.PI * 6378137) / this.tile_size;
            }
        };
        MercatorTileSource.prototype.is_valid_tile = function (x, y, z) {
            if (!this.wrap_around) {
                if (x < 0 || x >= Math.pow(2, z))
                    return false;
            }
            if (y < 0 || y >= Math.pow(2, z))
                return false;
            return true;
        };
        MercatorTileSource.prototype.parent_by_tile_xyz = function (x, y, z) {
            var quadkey = this.tile_xyz_to_quadkey(x, y, z);
            var parent_quadkey = quadkey.substring(0, quadkey.length - 1);
            return this.quadkey_to_tile_xyz(parent_quadkey);
        };
        MercatorTileSource.prototype.get_resolution = function (level) {
            return this._computed_initial_resolution() / Math.pow(2, level);
        };
        MercatorTileSource.prototype.get_resolution_by_extent = function (extent, height, width) {
            var x_rs = (extent[2] - extent[0]) / width;
            var y_rs = (extent[3] - extent[1]) / height;
            return [x_rs, y_rs];
        };
        MercatorTileSource.prototype.get_level_by_extent = function (extent, height, width) {
            var e_1, _a;
            var x_rs = (extent[2] - extent[0]) / width;
            var y_rs = (extent[3] - extent[1]) / height;
            var resolution = Math.max(x_rs, y_rs);
            var i = 0;
            try {
                for (var _b = tslib_1.__values(this._resolutions), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var r = _c.value;
                    if (resolution > r) {
                        if (i == 0)
                            return 0;
                        if (i > 0)
                            return i - 1;
                    }
                    i += 1;
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            // otherwise return the highest available resolution
            return (i - 1);
        };
        MercatorTileSource.prototype.get_closest_level_by_extent = function (extent, height, width) {
            var x_rs = (extent[2] - extent[0]) / width;
            var y_rs = (extent[3] - extent[1]) / height;
            var resolution = Math.max(x_rs, y_rs);
            var closest = this._resolutions.reduce(function (previous, current) {
                if (Math.abs(current - resolution) < Math.abs(previous - resolution))
                    return current;
                else
                    return previous;
            });
            return this._resolutions.indexOf(closest);
        };
        MercatorTileSource.prototype.snap_to_zoom_level = function (extent, height, width, level) {
            var _a = tslib_1.__read(extent, 4), xmin = _a[0], ymin = _a[1], xmax = _a[2], ymax = _a[3];
            var desired_res = this._resolutions[level];
            var desired_x_delta = width * desired_res;
            var desired_y_delta = height * desired_res;
            if (!this.snap_to_zoom) {
                var xscale = (xmax - xmin) / desired_x_delta;
                var yscale = (ymax - ymin) / desired_y_delta;
                if (xscale > yscale) {
                    desired_x_delta = (xmax - xmin);
                    desired_y_delta = desired_y_delta * xscale;
                }
                else {
                    desired_x_delta = desired_x_delta * yscale;
                    desired_y_delta = (ymax - ymin);
                }
            }
            var x_adjust = (desired_x_delta - (xmax - xmin)) / 2;
            var y_adjust = (desired_y_delta - (ymax - ymin)) / 2;
            return [xmin - x_adjust, ymin - y_adjust, xmax + x_adjust, ymax + y_adjust];
        };
        MercatorTileSource.prototype.tms_to_wmts = function (x, y, z) {
            // Note this works both ways
            return [x, Math.pow(2, z) - 1 - y, z];
        };
        MercatorTileSource.prototype.wmts_to_tms = function (x, y, z) {
            // Note this works both ways
            return [x, Math.pow(2, z) - 1 - y, z];
        };
        MercatorTileSource.prototype.pixels_to_meters = function (px, py, level) {
            var res = this.get_resolution(level);
            var mx = (px * res) - this.x_origin_offset;
            var my = (py * res) - this.y_origin_offset;
            return [mx, my];
        };
        MercatorTileSource.prototype.meters_to_pixels = function (mx, my, level) {
            var res = this.get_resolution(level);
            var px = (mx + this.x_origin_offset) / res;
            var py = (my + this.y_origin_offset) / res;
            return [px, py];
        };
        MercatorTileSource.prototype.pixels_to_tile = function (px, py) {
            var tx = Math.ceil(px / this.tile_size);
            tx = tx === 0 ? tx : tx - 1;
            var ty = Math.max(Math.ceil(py / this.tile_size) - 1, 0);
            return [tx, ty];
        };
        MercatorTileSource.prototype.pixels_to_raster = function (px, py, level) {
            var mapSize = this.tile_size << level;
            return [px, mapSize - py];
        };
        MercatorTileSource.prototype.meters_to_tile = function (mx, my, level) {
            var _a = tslib_1.__read(this.meters_to_pixels(mx, my, level), 2), px = _a[0], py = _a[1];
            return this.pixels_to_tile(px, py);
        };
        MercatorTileSource.prototype.get_tile_meter_bounds = function (tx, ty, level) {
            // expects tms styles coordinates (bottom-left origin)
            var _a = tslib_1.__read(this.pixels_to_meters(tx * this.tile_size, ty * this.tile_size, level), 2), xmin = _a[0], ymin = _a[1];
            var _b = tslib_1.__read(this.pixels_to_meters((tx + 1) * this.tile_size, (ty + 1) * this.tile_size, level), 2), xmax = _b[0], ymax = _b[1];
            return [xmin, ymin, xmax, ymax];
        };
        MercatorTileSource.prototype.get_tile_geographic_bounds = function (tx, ty, level) {
            var bounds = this.get_tile_meter_bounds(tx, ty, level);
            var _a = tslib_1.__read(tile_utils_1.meters_extent_to_geographic(bounds), 4), minLon = _a[0], minLat = _a[1], maxLon = _a[2], maxLat = _a[3];
            return [minLon, minLat, maxLon, maxLat];
        };
        MercatorTileSource.prototype.get_tiles_by_extent = function (extent, level, tile_border) {
            if (tile_border === void 0) {
                tile_border = 1;
            }
            // unpack extent and convert to tile coordinates
            var _a = tslib_1.__read(extent, 4), xmin = _a[0], ymin = _a[1], xmax = _a[2], ymax = _a[3];
            var _b = tslib_1.__read(this.meters_to_tile(xmin, ymin, level), 2), txmin = _b[0], tymin = _b[1];
            var _c = tslib_1.__read(this.meters_to_tile(xmax, ymax, level), 2), txmax = _c[0], tymax = _c[1];
            // add tiles which border
            txmin -= tile_border;
            tymin -= tile_border;
            txmax += tile_border;
            tymax += tile_border;
            var tiles = [];
            for (var ty = tymax; ty >= tymin; ty--) {
                for (var tx = txmin; tx <= txmax; tx++) {
                    if (this.is_valid_tile(tx, ty, level))
                        tiles.push([tx, ty, level, this.get_tile_meter_bounds(tx, ty, level)]);
                }
            }
            this.sort_tiles_from_center(tiles, [txmin, tymin, txmax, tymax]);
            return tiles;
        };
        MercatorTileSource.prototype.quadkey_to_tile_xyz = function (quadKey) {
            /**
             * Computes tile x, y and z values based on quadKey.
             */
            var tileX = 0;
            var tileY = 0;
            var tileZ = quadKey.length;
            for (var i = tileZ; i > 0; i--) {
                var value = quadKey.charAt(tileZ - i);
                var mask = 1 << (i - 1);
                switch (value) {
                    case '0':
                        continue;
                    case '1':
                        tileX |= mask;
                        break;
                    case '2':
                        tileY |= mask;
                        break;
                    case '3':
                        tileX |= mask;
                        tileY |= mask;
                        break;
                    default:
                        throw new TypeError("Invalid Quadkey: " + quadKey);
                }
            }
            return [tileX, tileY, tileZ];
        };
        MercatorTileSource.prototype.tile_xyz_to_quadkey = function (x, y, z) {
            /*
             * Computes quadkey value based on tile x, y and z values.
             */
            var quadkey = "";
            for (var i = z; i > 0; i--) {
                var mask = 1 << (i - 1);
                var digit = 0;
                if ((x & mask) !== 0) {
                    digit += 1;
                }
                if ((y & mask) !== 0) {
                    digit += 2;
                }
                quadkey += digit.toString();
            }
            return quadkey;
        };
        MercatorTileSource.prototype.children_by_tile_xyz = function (x, y, z) {
            var quadkey = this.tile_xyz_to_quadkey(x, y, z);
            var child_tile_xyz = [];
            for (var i = 0; i <= 3; i++) {
                var _a = tslib_1.__read(this.quadkey_to_tile_xyz(quadkey + i.toString()), 3), x_1 = _a[0], y_1 = _a[1], z_1 = _a[2];
                var b = this.get_tile_meter_bounds(x_1, y_1, z_1);
                child_tile_xyz.push([x_1, y_1, z_1, b]);
            }
            return child_tile_xyz;
        };
        MercatorTileSource.prototype.get_closest_parent_by_tile_xyz = function (x, y, z) {
            var _a, _b, _c;
            var world_x = this.calculate_world_x_by_tile_xyz(x, y, z);
            _a = tslib_1.__read(this.normalize_xyz(x, y, z), 3), x = _a[0], y = _a[1], z = _a[2];
            var quadkey = this.tile_xyz_to_quadkey(x, y, z);
            while (quadkey.length > 0) {
                quadkey = quadkey.substring(0, quadkey.length - 1);
                _b = tslib_1.__read(this.quadkey_to_tile_xyz(quadkey), 3), x = _b[0], y = _b[1], z = _b[2];
                _c = tslib_1.__read(this.denormalize_xyz(x, y, z, world_x), 3), x = _c[0], y = _c[1], z = _c[2];
                if (this.tiles.has(this.tile_xyz_to_key(x, y, z)))
                    return [x, y, z];
            }
            return [0, 0, 0];
        };
        MercatorTileSource.prototype.normalize_xyz = function (x, y, z) {
            if (this.wrap_around) {
                var tile_count = Math.pow(2, z);
                return [((x % tile_count) + tile_count) % tile_count, y, z];
            }
            else {
                return [x, y, z];
            }
        };
        MercatorTileSource.prototype.denormalize_xyz = function (x, y, z, world_x) {
            return [x + (world_x * Math.pow(2, z)), y, z];
        };
        MercatorTileSource.prototype.denormalize_meters = function (meters_x, meters_y, _level, world_x) {
            return [meters_x + (world_x * 2 * Math.PI * 6378137), meters_y];
        };
        MercatorTileSource.prototype.calculate_world_x_by_tile_xyz = function (x, _y, z) {
            return Math.floor(x / Math.pow(2, z));
        };
        return MercatorTileSource;
    }(tile_source_1.TileSource));
    exports.MercatorTileSource = MercatorTileSource;
    MercatorTileSource.__name__ = "MercatorTileSource";
    MercatorTileSource.init_MercatorTileSource();
},
/* models/tiles/tile_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var object_1 = require(128) /* ../../core/util/object */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var TileSource = /** @class */ (function (_super) {
        tslib_1.__extends(TileSource, _super);
        function TileSource(attrs) {
            return _super.call(this, attrs) || this;
        }
        TileSource.init_TileSource = function () {
            this.define({
                url: [p.String, ''],
                tile_size: [p.Number, 256],
                max_zoom: [p.Number, 30],
                min_zoom: [p.Number, 0],
                extra_url_vars: [p.Any, {}],
                attribution: [p.String, ''],
                x_origin_offset: [p.Number],
                y_origin_offset: [p.Number],
                initial_resolution: [p.Number],
            });
        };
        TileSource.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.tiles = new Map();
            this._normalize_case();
        };
        TileSource.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.change, function () { return _this._clear_cache(); });
        };
        TileSource.prototype.string_lookup_replace = function (str, lookup) {
            var e_1, _a;
            var result_str = str;
            try {
                for (var _b = tslib_1.__values(object_1.entries(lookup)), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var _d = tslib_1.__read(_c.value, 2), key = _d[0], value = _d[1];
                    result_str = result_str.replace("{" + key + "}", value);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return result_str;
        };
        TileSource.prototype._normalize_case = function () {
            /*
             * Note: should probably be refactored into subclasses.
             */
            var url = this.url
                .replace('{x}', '{X}')
                .replace('{y}', '{Y}')
                .replace('{z}', '{Z}')
                .replace('{q}', '{Q}')
                .replace('{xmin}', '{XMIN}')
                .replace('{ymin}', '{YMIN}')
                .replace('{xmax}', '{XMAX}')
                .replace('{ymax}', '{YMAX}');
            this.url = url;
        };
        TileSource.prototype._clear_cache = function () {
            this.tiles = new Map();
        };
        TileSource.prototype.tile_xyz_to_key = function (x, y, z) {
            return x + ":" + y + ":" + z;
        };
        TileSource.prototype.key_to_tile_xyz = function (key) {
            var _a = tslib_1.__read(key.split(':').map(function (c) { return parseInt(c); }), 3), x = _a[0], y = _a[1], z = _a[2];
            return [x, y, z];
        };
        TileSource.prototype.sort_tiles_from_center = function (tiles, tile_extent) {
            var _a = tslib_1.__read(tile_extent, 4), txmin = _a[0], tymin = _a[1], txmax = _a[2], tymax = _a[3];
            var center_x = ((txmax - txmin) / 2) + txmin;
            var center_y = ((tymax - tymin) / 2) + tymin;
            tiles.sort(function (a, b) {
                var a_distance = Math.sqrt(Math.pow((center_x - a[0]), 2) + Math.pow((center_y - a[1]), 2));
                var b_distance = Math.sqrt(Math.pow((center_x - b[0]), 2) + Math.pow((center_y - b[1]), 2));
                return a_distance - b_distance;
            });
        };
        TileSource.prototype.get_image_url = function (x, y, z) {
            var image_url = this.string_lookup_replace(this.url, this.extra_url_vars);
            return image_url
                .replace("{X}", x.toString())
                .replace('{Y}', y.toString())
                .replace("{Z}", z.toString());
        };
        return TileSource;
    }(model_1.Model));
    exports.TileSource = TileSource;
    TileSource.__name__ = "TileSource";
    TileSource.init_TileSource();
},
/* models/tiles/tile_utils.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var projections_1 = require(152) /* ../../core/util/projections */;
    function geographic_to_meters(x_lon, y_lat) {
        return projections_1.wgs84_mercator.compute(x_lon, y_lat);
    }
    exports.geographic_to_meters = geographic_to_meters;
    function meters_to_geographic(mx, my) {
        return projections_1.wgs84_mercator.invert(mx, my);
    }
    exports.meters_to_geographic = meters_to_geographic;
    function geographic_extent_to_meters(extent) {
        var _a = tslib_1.__read(extent, 4), g_xmin = _a[0], g_ymin = _a[1], g_xmax = _a[2], g_ymax = _a[3];
        var _b = tslib_1.__read(geographic_to_meters(g_xmin, g_ymin), 2), m_xmin = _b[0], m_ymin = _b[1];
        var _c = tslib_1.__read(geographic_to_meters(g_xmax, g_ymax), 2), m_xmax = _c[0], m_ymax = _c[1];
        return [m_xmin, m_ymin, m_xmax, m_ymax];
    }
    exports.geographic_extent_to_meters = geographic_extent_to_meters;
    function meters_extent_to_geographic(extent) {
        var _a = tslib_1.__read(extent, 4), m_xmin = _a[0], m_ymin = _a[1], m_xmax = _a[2], m_ymax = _a[3];
        var _b = tslib_1.__read(meters_to_geographic(m_xmin, m_ymin), 2), g_xmin = _b[0], g_ymin = _b[1];
        var _c = tslib_1.__read(meters_to_geographic(m_xmax, m_ymax), 2), g_xmax = _c[0], g_ymax = _c[1];
        return [g_xmin, g_ymin, g_xmax, g_ymax];
    }
    exports.meters_extent_to_geographic = meters_extent_to_geographic;
},
/* models/tiles/quadkey_tile_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var mercator_tile_source_1 = require(448) /* ./mercator_tile_source */;
    var QUADKEYTileSource = /** @class */ (function (_super) {
        tslib_1.__extends(QUADKEYTileSource, _super);
        function QUADKEYTileSource(attrs) {
            return _super.call(this, attrs) || this;
        }
        QUADKEYTileSource.prototype.get_image_url = function (x, y, z) {
            var image_url = this.string_lookup_replace(this.url, this.extra_url_vars);
            var _a = tslib_1.__read(this.tms_to_wmts(x, y, z), 3), wx = _a[0], wy = _a[1], wz = _a[2];
            var quadKey = this.tile_xyz_to_quadkey(wx, wy, wz);
            return image_url.replace("{Q}", quadKey);
        };
        return QUADKEYTileSource;
    }(mercator_tile_source_1.MercatorTileSource));
    exports.QUADKEYTileSource = QUADKEYTileSource;
    QUADKEYTileSource.__name__ = "QUADKEYTileSource";
},
/* models/tiles/tile_renderer.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var wmts_tile_source_1 = require(453) /* ./wmts_tile_source */;
    var data_renderer_1 = require(206) /* ../renderers/data_renderer */;
    var range1d_1 = require(273) /* ../ranges/range1d */;
    var dom_1 = require(187) /* ../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var image_1 = require(366) /* ../../core/util/image */;
    var array_1 = require(124) /* ../../core/util/array */;
    var types_1 = require(123) /* ../../core/util/types */;
    var selection_manager_1 = require(204) /* ../../core/selection_manager */;
    var column_data_source_1 = require(200) /* ../sources/column_data_source */;
    var tiles_1 = require(454) /* ../../styles/tiles */;
    var tiles_css_1 = tslib_1.__importDefault(require(455) /* ../../styles/tiles.css */);
    var TileRendererView = /** @class */ (function (_super) {
        tslib_1.__extends(TileRendererView, _super);
        function TileRendererView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        TileRendererView.prototype.initialize = function () {
            this._tiles = [];
            _super.prototype.initialize.call(this);
        };
        TileRendererView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.change, function () { return _this.request_render(); });
            this.connect(this.model.tile_source.change, function () { return _this.request_render(); });
        };
        TileRendererView.prototype.styles = function () {
            return tslib_1.__spread(_super.prototype.styles.call(this), [tiles_css_1.default]);
        };
        TileRendererView.prototype.get_extent = function () {
            return [this.x_range.start, this.y_range.start, this.x_range.end, this.y_range.end];
        };
        Object.defineProperty(TileRendererView.prototype, "map_plot", {
            get: function () {
                return this.plot_model;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TileRendererView.prototype, "map_canvas", {
            get: function () {
                return this.layer.ctx;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TileRendererView.prototype, "map_frame", {
            get: function () {
                return this.plot_view.frame;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TileRendererView.prototype, "x_range", {
            get: function () {
                return this.map_plot.x_range;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TileRendererView.prototype, "y_range", {
            get: function () {
                return this.map_plot.y_range;
            },
            enumerable: true,
            configurable: true
        });
        TileRendererView.prototype._set_data = function () {
            this.extent = this.get_extent();
            this._last_height = undefined;
            this._last_width = undefined;
        };
        TileRendererView.prototype._update_attribution = function () {
            if (this.attribution_el != null)
                dom_1.removeElement(this.attribution_el);
            var attribution = this.model.tile_source.attribution;
            if (types_1.isString(attribution) && attribution.length > 0) {
                var _a = this.plot_view, layout = _a.layout, frame = _a.frame;
                var offset_right = layout.bbox.width - frame.bbox.right;
                var offset_bottom = layout.bbox.height - frame.bbox.bottom;
                var max_width = frame.bbox.width;
                this.attribution_el = dom_1.div({
                    class: tiles_1.bk_tile_attribution,
                    style: {
                        position: "absolute",
                        right: offset_right + "px",
                        bottom: offset_bottom + "px",
                        'max-width': max_width - 4 /*padding*/ + "px",
                        padding: "2px",
                        'background-color': 'rgba(255,255,255,0.5)',
                        'font-size': '9px',
                        'line-height': '1.05',
                        'white-space': 'nowrap',
                        overflow: 'hidden',
                        'text-overflow': 'ellipsis',
                    },
                });
                this.plot_view.canvas_view.add_event(this.attribution_el);
                this.attribution_el.innerHTML = attribution;
                this.attribution_el.title = this.attribution_el.textContent.replace(/\s*\n\s*/g, " ");
            }
        };
        TileRendererView.prototype._map_data = function () {
            this.initial_extent = this.get_extent();
            var zoom_level = this.model.tile_source.get_level_by_extent(this.initial_extent, this.map_frame.bbox.height, this.map_frame.bbox.width);
            var new_extent = this.model.tile_source.snap_to_zoom_level(this.initial_extent, this.map_frame.bbox.height, this.map_frame.bbox.width, zoom_level);
            this.x_range.start = new_extent[0];
            this.y_range.start = new_extent[1];
            this.x_range.end = new_extent[2];
            this.y_range.end = new_extent[3];
            if (this.x_range instanceof range1d_1.Range1d) {
                this.x_range.reset_start = new_extent[0];
                this.x_range.reset_end = new_extent[2];
            }
            if (this.y_range instanceof range1d_1.Range1d) {
                this.y_range.reset_start = new_extent[1];
                this.y_range.reset_end = new_extent[3];
            }
            this._update_attribution();
        };
        TileRendererView.prototype._create_tile = function (x, y, z, bounds, cache_only) {
            var _this = this;
            if (cache_only === void 0) {
                cache_only = false;
            }
            var _a = tslib_1.__read(this.model.tile_source.normalize_xyz(x, y, z), 3), nx = _a[0], ny = _a[1], nz = _a[2];
            var tile = {
                img: undefined,
                tile_coords: [x, y, z],
                normalized_coords: [nx, ny, nz],
                quadkey: this.model.tile_source.tile_xyz_to_quadkey(x, y, z),
                cache_key: this.model.tile_source.tile_xyz_to_key(x, y, z),
                bounds: bounds,
                loaded: false,
                finished: false,
                x_coord: bounds[0],
                y_coord: bounds[3],
            };
            var src = this.model.tile_source.get_image_url(nx, ny, nz);
            new image_1.ImageLoader(src, {
                loaded: function (img) {
                    Object.assign(tile, { img: img, loaded: true });
                    if (cache_only) {
                        tile.finished = true;
                        _this.notify_finished();
                    }
                    else
                        _this.request_render();
                },
                failed: function () {
                    tile.finished = true;
                },
            });
            this.model.tile_source.tiles.set(tile.cache_key, tile);
            this._tiles.push(tile);
        };
        TileRendererView.prototype._enforce_aspect_ratio = function () {
            // brute force way of handling resize or sizing_mode event -------------------------------------------------------------
            if ((this._last_height !== this.map_frame.bbox.height) || (this._last_width !== this.map_frame.bbox.width)) {
                var extent = this.get_extent();
                var zoom_level = this.model.tile_source.get_level_by_extent(extent, this.map_frame.bbox.height, this.map_frame.bbox.width);
                var new_extent = this.model.tile_source.snap_to_zoom_level(extent, this.map_frame.bbox.height, this.map_frame.bbox.width, zoom_level);
                this.x_range.setv({ start: new_extent[0], end: new_extent[2] });
                this.y_range.setv({ start: new_extent[1], end: new_extent[3] });
                this.extent = new_extent;
                this._last_height = this.map_frame.bbox.height;
                this._last_width = this.map_frame.bbox.width;
            }
        };
        TileRendererView.prototype.has_finished = function () {
            var e_1, _a;
            if (!_super.prototype.has_finished.call(this)) {
                return false;
            }
            if (this._tiles.length === 0) {
                return false;
            }
            try {
                for (var _b = tslib_1.__values(this._tiles), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var tile = _c.value;
                    if (!tile.finished) {
                        return false;
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return true;
        };
        TileRendererView.prototype._render = function () {
            if (this.map_initialized == null) {
                this._set_data();
                this._map_data();
                this.map_initialized = true;
            }
            this._enforce_aspect_ratio();
            this._update();
            if (this.prefetch_timer != null) {
                clearTimeout(this.prefetch_timer);
            }
            this.prefetch_timer = setTimeout(this._prefetch_tiles.bind(this), 500);
            if (this.has_finished()) {
                this.notify_finished();
            }
        };
        TileRendererView.prototype._draw_tile = function (tile_key) {
            var tile_data = this.model.tile_source.tiles.get(tile_key);
            if (tile_data != null && tile_data.loaded) {
                var _a = tslib_1.__read(this.coordinates.map_to_screen([tile_data.bounds[0]], [tile_data.bounds[3]]), 2), _b = tslib_1.__read(_a[0], 1), sxmin = _b[0], _c = tslib_1.__read(_a[1], 1), symin = _c[0];
                var _d = tslib_1.__read(this.coordinates.map_to_screen([tile_data.bounds[2]], [tile_data.bounds[1]]), 2), _e = tslib_1.__read(_d[0], 1), sxmax = _e[0], _f = tslib_1.__read(_d[1], 1), symax = _f[0];
                var sw = sxmax - sxmin;
                var sh = symax - symin;
                var sx = sxmin;
                var sy = symin;
                var old_smoothing = this.map_canvas.getImageSmoothingEnabled();
                this.map_canvas.setImageSmoothingEnabled(this.model.smoothing);
                this.map_canvas.drawImage(tile_data.img, sx, sy, sw, sh);
                this.map_canvas.setImageSmoothingEnabled(old_smoothing);
                tile_data.finished = true;
            }
        };
        TileRendererView.prototype._set_rect = function () {
            var outline_width = this.plot_model.properties.outline_line_width.value();
            var l = this.map_frame.bbox.left + (outline_width / 2);
            var t = this.map_frame.bbox.top + (outline_width / 2);
            var w = this.map_frame.bbox.width - outline_width;
            var h = this.map_frame.bbox.height - outline_width;
            this.map_canvas.rect(l, t, w, h);
            this.map_canvas.clip();
        };
        TileRendererView.prototype._render_tiles = function (tile_keys) {
            var e_2, _a;
            this.map_canvas.save();
            this._set_rect();
            this.map_canvas.globalAlpha = this.model.alpha;
            try {
                for (var tile_keys_1 = tslib_1.__values(tile_keys), tile_keys_1_1 = tile_keys_1.next(); !tile_keys_1_1.done; tile_keys_1_1 = tile_keys_1.next()) {
                    var tile_key = tile_keys_1_1.value;
                    this._draw_tile(tile_key);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (tile_keys_1_1 && !tile_keys_1_1.done && (_a = tile_keys_1.return))
                        _a.call(tile_keys_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            this.map_canvas.restore();
        };
        TileRendererView.prototype._prefetch_tiles = function () {
            var e_3, _a;
            var tile_source = this.model.tile_source;
            var extent = this.get_extent();
            var h = this.map_frame.bbox.height;
            var w = this.map_frame.bbox.width;
            var zoom_level = this.model.tile_source.get_level_by_extent(extent, h, w);
            var tiles = this.model.tile_source.get_tiles_by_extent(extent, zoom_level);
            for (var t = 0, end = Math.min(10, tiles.length); t < end; t++) {
                var _b = tslib_1.__read(tiles[t], 3), x = _b[0], y = _b[1], z = _b[2];
                var children = this.model.tile_source.children_by_tile_xyz(x, y, z);
                try {
                    for (var children_1 = (e_3 = void 0, tslib_1.__values(children)), children_1_1 = children_1.next(); !children_1_1.done; children_1_1 = children_1.next()) {
                        var c = children_1_1.value;
                        var _c = tslib_1.__read(c, 4), cx = _c[0], cy = _c[1], cz = _c[2], cbounds = _c[3];
                        if (tile_source.tiles.has(tile_source.tile_xyz_to_key(cx, cy, cz))) {
                            continue;
                        }
                        else {
                            this._create_tile(cx, cy, cz, cbounds, true);
                        }
                    }
                }
                catch (e_3_1) {
                    e_3 = { error: e_3_1 };
                }
                finally {
                    try {
                        if (children_1_1 && !children_1_1.done && (_a = children_1.return))
                            _a.call(children_1);
                    }
                    finally {
                        if (e_3)
                            throw e_3.error;
                    }
                }
            }
        };
        TileRendererView.prototype._fetch_tiles = function (tiles) {
            var e_4, _a;
            try {
                for (var tiles_2 = tslib_1.__values(tiles), tiles_2_1 = tiles_2.next(); !tiles_2_1.done; tiles_2_1 = tiles_2.next()) {
                    var tile = tiles_2_1.value;
                    var _b = tslib_1.__read(tile, 4), x = _b[0], y = _b[1], z = _b[2], bounds = _b[3];
                    this._create_tile(x, y, z, bounds);
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (tiles_2_1 && !tiles_2_1.done && (_a = tiles_2.return))
                        _a.call(tiles_2);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
        };
        TileRendererView.prototype._update = function () {
            var e_5, _a, e_6, _b;
            var _this = this;
            var tile_source = this.model.tile_source;
            var min_zoom = tile_source.min_zoom;
            var max_zoom = tile_source.max_zoom;
            var extent = this.get_extent();
            var zooming_out = (this.extent[2] - this.extent[0]) < (extent[2] - extent[0]);
            var h = this.map_frame.bbox.height;
            var w = this.map_frame.bbox.width;
            var zoom_level = tile_source.get_level_by_extent(extent, h, w);
            var snap_back = false;
            if (zoom_level < min_zoom) {
                extent = this.extent;
                zoom_level = min_zoom;
                snap_back = true;
            }
            else if (zoom_level > max_zoom) {
                extent = this.extent;
                zoom_level = max_zoom;
                snap_back = true;
            }
            if (snap_back) {
                this.x_range.setv({ x_range: { start: extent[0], end: extent[2] } });
                this.y_range.setv({ start: extent[1], end: extent[3] });
            }
            this.extent = extent;
            var tiles = tile_source.get_tiles_by_extent(extent, zoom_level);
            var need_load = [];
            var cached = [];
            var parents = [];
            var children = [];
            try {
                for (var tiles_3 = tslib_1.__values(tiles), tiles_3_1 = tiles_3.next(); !tiles_3_1.done; tiles_3_1 = tiles_3.next()) {
                    var t = tiles_3_1.value;
                    var _c = tslib_1.__read(t, 3), x = _c[0], y = _c[1], z = _c[2];
                    var key = tile_source.tile_xyz_to_key(x, y, z);
                    var tile = tile_source.tiles.get(key);
                    if (tile != null && tile.loaded) {
                        cached.push(key);
                    }
                    else {
                        if (this.model.render_parents) {
                            var _d = tslib_1.__read(tile_source.get_closest_parent_by_tile_xyz(x, y, z), 3), px = _d[0], py = _d[1], pz = _d[2];
                            var parent_key = tile_source.tile_xyz_to_key(px, py, pz);
                            var parent_tile = tile_source.tiles.get(parent_key);
                            if ((parent_tile != null) && parent_tile.loaded && !array_1.includes(parents, parent_key)) {
                                parents.push(parent_key);
                            }
                            if (zooming_out) {
                                var child_tiles = tile_source.children_by_tile_xyz(x, y, z);
                                try {
                                    for (var child_tiles_1 = (e_6 = void 0, tslib_1.__values(child_tiles)), child_tiles_1_1 = child_tiles_1.next(); !child_tiles_1_1.done; child_tiles_1_1 = child_tiles_1.next()) {
                                        var _e = tslib_1.__read(child_tiles_1_1.value, 3), cx = _e[0], cy = _e[1], cz = _e[2];
                                        var child_key = tile_source.tile_xyz_to_key(cx, cy, cz);
                                        if (tile_source.tiles.has(child_key))
                                            children.push(child_key);
                                    }
                                }
                                catch (e_6_1) {
                                    e_6 = { error: e_6_1 };
                                }
                                finally {
                                    try {
                                        if (child_tiles_1_1 && !child_tiles_1_1.done && (_b = child_tiles_1.return))
                                            _b.call(child_tiles_1);
                                    }
                                    finally {
                                        if (e_6)
                                            throw e_6.error;
                                    }
                                }
                            }
                        }
                    }
                    if (tile == null)
                        need_load.push(t);
                }
            }
            catch (e_5_1) {
                e_5 = { error: e_5_1 };
            }
            finally {
                try {
                    if (tiles_3_1 && !tiles_3_1.done && (_a = tiles_3.return))
                        _a.call(tiles_3);
                }
                finally {
                    if (e_5)
                        throw e_5.error;
                }
            }
            // draw stand-in parents ----------
            this._render_tiles(parents);
            this._render_tiles(children);
            // draw cached ----------
            this._render_tiles(cached);
            // fetch missing -------
            if (this.render_timer != null) {
                clearTimeout(this.render_timer);
            }
            this.render_timer = setTimeout((function () { return _this._fetch_tiles(need_load); }), 65);
        };
        return TileRendererView;
    }(data_renderer_1.DataRendererView));
    exports.TileRendererView = TileRendererView;
    TileRendererView.__name__ = "TileRendererView";
    var TileRenderer = /** @class */ (function (_super) {
        tslib_1.__extends(TileRenderer, _super);
        function TileRenderer(attrs) {
            var _this = _super.call(this, attrs) || this;
            // XXX: tile renderer doesn't allow selection, but needs to fulfil the APIs
            _this._selection_manager = new selection_manager_1.SelectionManager({
                source: new column_data_source_1.ColumnDataSource(),
            });
            return _this;
        }
        TileRenderer.init_TileRenderer = function () {
            this.prototype.default_view = TileRendererView;
            this.define({
                alpha: [p.Number, 1.0],
                smoothing: [p.Boolean, true],
                tile_source: [p.Instance, function () { return new wmts_tile_source_1.WMTSTileSource(); }],
                render_parents: [p.Boolean, true],
            });
        };
        TileRenderer.prototype.get_selection_manager = function () {
            return this._selection_manager;
        };
        return TileRenderer;
    }(data_renderer_1.DataRenderer));
    exports.TileRenderer = TileRenderer;
    TileRenderer.__name__ = "TileRenderer";
    TileRenderer.init_TileRenderer();
},
/* models/tiles/wmts_tile_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var mercator_tile_source_1 = require(448) /* ./mercator_tile_source */;
    var WMTSTileSource = /** @class */ (function (_super) {
        tslib_1.__extends(WMTSTileSource, _super);
        function WMTSTileSource(attrs) {
            return _super.call(this, attrs) || this;
        }
        WMTSTileSource.prototype.get_image_url = function (x, y, z) {
            var image_url = this.string_lookup_replace(this.url, this.extra_url_vars);
            var _a = tslib_1.__read(this.tms_to_wmts(x, y, z), 3), wx = _a[0], wy = _a[1], wz = _a[2];
            return image_url
                .replace("{X}", wx.toString())
                .replace("{Y}", wy.toString())
                .replace("{Z}", wz.toString());
        };
        return WMTSTileSource;
    }(mercator_tile_source_1.MercatorTileSource));
    exports.WMTSTileSource = WMTSTileSource;
    WMTSTileSource.__name__ = "WMTSTileSource";
},
/* styles/tiles.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.bk_tile_attribution = "bk-tile-attribution";
},
/* styles/tiles.css.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var css = "\n.bk-root .bk-tile-attribution a {\n  color: black;\n}\n";
    exports.default = css;
},
/* models/tiles/tms_tile_source.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var mercator_tile_source_1 = require(448) /* ./mercator_tile_source */;
    var TMSTileSource = /** @class */ (function (_super) {
        tslib_1.__extends(TMSTileSource, _super);
        function TMSTileSource(attrs) {
            return _super.call(this, attrs) || this;
        }
        TMSTileSource.prototype.get_image_url = function (x, y, z) {
            var image_url = this.string_lookup_replace(this.url, this.extra_url_vars);
            return image_url
                .replace("{X}", x.toString())
                .replace('{Y}', y.toString())
                .replace("{Z}", z.toString());
        };
        return TMSTileSource;
    }(mercator_tile_source_1.MercatorTileSource));
    exports.TMSTileSource = TMSTileSource;
    TMSTileSource.__name__ = "TMSTileSource";
},
/* models/textures/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var canvas_texture_1 = require(458) /* ./canvas_texture */;
    exports.CanvasTexture = canvas_texture_1.CanvasTexture;
    var image_url_texture_1 = require(460) /* ./image_url_texture */;
    exports.ImageURLTexture = image_url_texture_1.ImageURLTexture;
    var texture_1 = require(459) /* ./texture */;
    exports.Texture = texture_1.Texture;
},
/* models/textures/canvas_texture.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var texture_1 = require(459) /* ./texture */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var string_1 = require(144) /* ../../core/util/string */;
    var CanvasTexture = /** @class */ (function (_super) {
        tslib_1.__extends(CanvasTexture, _super);
        function CanvasTexture(attrs) {
            return _super.call(this, attrs) || this;
        }
        CanvasTexture.init_CanvasTexture = function () {
            this.define({
                code: [p.String],
            });
        };
        Object.defineProperty(CanvasTexture.prototype, "func", {
            get: function () {
                var code = string_1.use_strict(this.code);
                return new Function("ctx", "color", "scale", "weight", code);
            },
            enumerable: true,
            configurable: true
        });
        CanvasTexture.prototype.get_pattern = function (color, scale, weight) {
            var _this = this;
            return function (ctx) {
                var canvas = document.createElement('canvas');
                canvas.width = scale;
                canvas.height = scale;
                var pattern_ctx = canvas.getContext('2d');
                _this.func.call(_this, pattern_ctx, color, scale, weight);
                return ctx.createPattern(canvas, _this.repetition);
            };
        };
        return CanvasTexture;
    }(texture_1.Texture));
    exports.CanvasTexture = CanvasTexture;
    CanvasTexture.__name__ = "CanvasTexture";
    CanvasTexture.init_CanvasTexture();
},
/* models/textures/texture.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../model */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var Texture = /** @class */ (function (_super) {
        tslib_1.__extends(Texture, _super);
        function Texture(attrs) {
            return _super.call(this, attrs) || this;
        }
        Texture.init_Texture = function () {
            this.define({
                repetition: [p.TextureRepetition, "repeat"],
            });
        };
        Texture.prototype.onload = function (defer_func) {
            defer_func();
        };
        return Texture;
    }(model_1.Model));
    exports.Texture = Texture;
    Texture.__name__ = "Texture";
    Texture.init_Texture();
},
/* models/textures/image_url_texture.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var texture_1 = require(459) /* ./texture */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var image_1 = require(366) /* ../../core/util/image */;
    var ImageURLTexture = /** @class */ (function (_super) {
        tslib_1.__extends(ImageURLTexture, _super);
        function ImageURLTexture(attrs) {
            return _super.call(this, attrs) || this;
        }
        ImageURLTexture.init_ImageURLTexture = function () {
            this.define({
                url: [p.String],
            });
        };
        ImageURLTexture.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._loader = new image_1.ImageLoader(this.url);
        };
        ImageURLTexture.prototype.get_pattern = function (_color, _scale, _weight) {
            var _this = this;
            return function (ctx) {
                if (!_this._loader.finished) {
                    return null;
                }
                return ctx.createPattern(_this._loader.image, _this.repetition);
            };
        };
        ImageURLTexture.prototype.onload = function (defer_func) {
            this._loader.promise.then(function () { return defer_func(); });
        };
        return ImageURLTexture;
    }(texture_1.Texture));
    exports.ImageURLTexture = ImageURLTexture;
    ImageURLTexture.__name__ = "ImageURLTexture";
    ImageURLTexture.init_ImageURLTexture();
},
/* models/tools/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var action_tool_1 = require(422) /* ./actions/action_tool */;
    exports.ActionTool = action_tool_1.ActionTool;
    var custom_action_1 = require(462) /* ./actions/custom_action */;
    exports.CustomAction = custom_action_1.CustomAction;
    var help_tool_1 = require(423) /* ./actions/help_tool */;
    exports.HelpTool = help_tool_1.HelpTool;
    var redo_tool_1 = require(463) /* ./actions/redo_tool */;
    exports.RedoTool = redo_tool_1.RedoTool;
    var reset_tool_1 = require(464) /* ./actions/reset_tool */;
    exports.ResetTool = reset_tool_1.ResetTool;
    var save_tool_1 = require(465) /* ./actions/save_tool */;
    exports.SaveTool = save_tool_1.SaveTool;
    var undo_tool_1 = require(466) /* ./actions/undo_tool */;
    exports.UndoTool = undo_tool_1.UndoTool;
    var zoom_in_tool_1 = require(467) /* ./actions/zoom_in_tool */;
    exports.ZoomInTool = zoom_in_tool_1.ZoomInTool;
    var zoom_out_tool_1 = require(470) /* ./actions/zoom_out_tool */;
    exports.ZoomOutTool = zoom_out_tool_1.ZoomOutTool;
    var button_tool_1 = require(411) /* ./button_tool */;
    exports.ButtonTool = button_tool_1.ButtonTool;
    var edit_tool_1 = require(471) /* ./edit/edit_tool */;
    exports.EditTool = edit_tool_1.EditTool;
    var box_edit_tool_1 = require(472) /* ./edit/box_edit_tool */;
    exports.BoxEditTool = box_edit_tool_1.BoxEditTool;
    var freehand_draw_tool_1 = require(473) /* ./edit/freehand_draw_tool */;
    exports.FreehandDrawTool = freehand_draw_tool_1.FreehandDrawTool;
    var point_draw_tool_1 = require(474) /* ./edit/point_draw_tool */;
    exports.PointDrawTool = point_draw_tool_1.PointDrawTool;
    var poly_draw_tool_1 = require(475) /* ./edit/poly_draw_tool */;
    exports.PolyDrawTool = poly_draw_tool_1.PolyDrawTool;
    var poly_tool_1 = require(476) /* ./edit/poly_tool */;
    exports.PolyTool = poly_tool_1.PolyTool;
    var poly_edit_tool_1 = require(477) /* ./edit/poly_edit_tool */;
    exports.PolyEditTool = poly_edit_tool_1.PolyEditTool;
    var box_select_tool_1 = require(478) /* ./gestures/box_select_tool */;
    exports.BoxSelectTool = box_select_tool_1.BoxSelectTool;
    var box_zoom_tool_1 = require(481) /* ./gestures/box_zoom_tool */;
    exports.BoxZoomTool = box_zoom_tool_1.BoxZoomTool;
    var gesture_tool_1 = require(421) /* ./gestures/gesture_tool */;
    exports.GestureTool = gesture_tool_1.GestureTool;
    var lasso_select_tool_1 = require(482) /* ./gestures/lasso_select_tool */;
    exports.LassoSelectTool = lasso_select_tool_1.LassoSelectTool;
    var line_edit_tool_1 = require(484) /* ./edit/line_edit_tool */;
    exports.LineEditTool = line_edit_tool_1.LineEditTool;
    var pan_tool_1 = require(486) /* ./gestures/pan_tool */;
    exports.PanTool = pan_tool_1.PanTool;
    var poly_select_tool_1 = require(483) /* ./gestures/poly_select_tool */;
    exports.PolySelectTool = poly_select_tool_1.PolySelectTool;
    var range_tool_1 = require(487) /* ./gestures/range_tool */;
    exports.RangeTool = range_tool_1.RangeTool;
    var select_tool_1 = require(479) /* ./gestures/select_tool */;
    exports.SelectTool = select_tool_1.SelectTool;
    var tap_tool_1 = require(488) /* ./gestures/tap_tool */;
    exports.TapTool = tap_tool_1.TapTool;
    var wheel_pan_tool_1 = require(489) /* ./gestures/wheel_pan_tool */;
    exports.WheelPanTool = wheel_pan_tool_1.WheelPanTool;
    var wheel_zoom_tool_1 = require(490) /* ./gestures/wheel_zoom_tool */;
    exports.WheelZoomTool = wheel_zoom_tool_1.WheelZoomTool;
    var crosshair_tool_1 = require(491) /* ./inspectors/crosshair_tool */;
    exports.CrosshairTool = crosshair_tool_1.CrosshairTool;
    var customjs_hover_1 = require(492) /* ./inspectors/customjs_hover */;
    exports.CustomJSHover = customjs_hover_1.CustomJSHover;
    var hover_tool_1 = require(493) /* ./inspectors/hover_tool */;
    exports.HoverTool = hover_tool_1.HoverTool;
    var inspect_tool_1 = require(410) /* ./inspectors/inspect_tool */;
    exports.InspectTool = inspect_tool_1.InspectTool;
    var tool_1 = require(413) /* ./tool */;
    exports.Tool = tool_1.Tool;
    var tool_proxy_1 = require(494) /* ./tool_proxy */;
    exports.ToolProxy = tool_proxy_1.ToolProxy;
    var toolbar_1 = require(409) /* ./toolbar */;
    exports.Toolbar = toolbar_1.Toolbar;
    var toolbar_base_1 = require(420) /* ./toolbar_base */;
    exports.ToolbarBase = toolbar_base_1.ToolbarBase;
    var toolbar_box_1 = require(495) /* ./toolbar_box */;
    exports.ProxyToolbar = toolbar_box_1.ProxyToolbar;
    var toolbar_box_2 = require(495) /* ./toolbar_box */;
    exports.ToolbarBox = toolbar_box_2.ToolbarBox;
},
/* models/tools/actions/custom_action.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var action_tool_1 = require(422) /* ./action_tool */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var toolbar_1 = require(414) /* ../../../styles/toolbar */;
    var CustomActionButtonView = /** @class */ (function (_super) {
        tslib_1.__extends(CustomActionButtonView, _super);
        function CustomActionButtonView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        CustomActionButtonView.prototype.css_classes = function () {
            return _super.prototype.css_classes.call(this).concat(toolbar_1.bk_toolbar_button_custom_action);
        };
        return CustomActionButtonView;
    }(action_tool_1.ActionToolButtonView));
    exports.CustomActionButtonView = CustomActionButtonView;
    CustomActionButtonView.__name__ = "CustomActionButtonView";
    var CustomActionView = /** @class */ (function (_super) {
        tslib_1.__extends(CustomActionView, _super);
        function CustomActionView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        CustomActionView.prototype.doit = function () {
            if (this.model.callback != null)
                this.model.callback.execute(this.model);
        };
        return CustomActionView;
    }(action_tool_1.ActionToolView));
    exports.CustomActionView = CustomActionView;
    CustomActionView.__name__ = "CustomActionView";
    var CustomAction = /** @class */ (function (_super) {
        tslib_1.__extends(CustomAction, _super);
        function CustomAction(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Custom Action";
            _this.button_view = CustomActionButtonView;
            return _this;
        }
        CustomAction.init_CustomAction = function () {
            this.prototype.default_view = CustomActionView;
            this.define({
                action_tooltip: [p.String, 'Perform a Custom Action'],
                callback: [p.Any],
                icon: [p.String],
            });
        };
        Object.defineProperty(CustomAction.prototype, "tooltip", {
            get: function () {
                return this.action_tooltip;
            },
            enumerable: true,
            configurable: true
        });
        return CustomAction;
    }(action_tool_1.ActionTool));
    exports.CustomAction = CustomAction;
    CustomAction.__name__ = "CustomAction";
    CustomAction.init_CustomAction();
},
/* models/tools/actions/redo_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var action_tool_1 = require(422) /* ./action_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var RedoToolView = /** @class */ (function (_super) {
        tslib_1.__extends(RedoToolView, _super);
        function RedoToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        RedoToolView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.plot_view.state_changed, function () { return _this.model.disabled = !_this.plot_view.can_redo(); });
        };
        RedoToolView.prototype.doit = function () {
            this.plot_view.redo();
        };
        return RedoToolView;
    }(action_tool_1.ActionToolView));
    exports.RedoToolView = RedoToolView;
    RedoToolView.__name__ = "RedoToolView";
    var RedoTool = /** @class */ (function (_super) {
        tslib_1.__extends(RedoTool, _super);
        function RedoTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Redo";
            _this.icon = icons_1.bk_tool_icon_redo;
            return _this;
        }
        RedoTool.init_RedoTool = function () {
            this.prototype.default_view = RedoToolView;
            this.override({
                disabled: true,
            });
            this.register_alias("redo", function () { return new RedoTool(); });
        };
        return RedoTool;
    }(action_tool_1.ActionTool));
    exports.RedoTool = RedoTool;
    RedoTool.__name__ = "RedoTool";
    RedoTool.init_RedoTool();
},
/* models/tools/actions/reset_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var action_tool_1 = require(422) /* ./action_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var ResetToolView = /** @class */ (function (_super) {
        tslib_1.__extends(ResetToolView, _super);
        function ResetToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ResetToolView.prototype.doit = function () {
            this.plot_view.reset();
        };
        return ResetToolView;
    }(action_tool_1.ActionToolView));
    exports.ResetToolView = ResetToolView;
    ResetToolView.__name__ = "ResetToolView";
    var ResetTool = /** @class */ (function (_super) {
        tslib_1.__extends(ResetTool, _super);
        function ResetTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Reset";
            _this.icon = icons_1.bk_tool_icon_reset;
            return _this;
        }
        ResetTool.init_ResetTool = function () {
            this.prototype.default_view = ResetToolView;
            this.register_alias("reset", function () { return new ResetTool(); });
        };
        return ResetTool;
    }(action_tool_1.ActionTool));
    exports.ResetTool = ResetTool;
    ResetTool.__name__ = "ResetTool";
    ResetTool.init_ResetTool();
},
/* models/tools/actions/save_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var action_tool_1 = require(422) /* ./action_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var SaveToolView = /** @class */ (function (_super) {
        tslib_1.__extends(SaveToolView, _super);
        function SaveToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        SaveToolView.prototype.copy = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var blob, item;
                var _a;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0: return [4 /*yield*/, this.plot_view.to_blob()];
                        case 1:
                            blob = _b.sent();
                            item = new ClipboardItem((_a = {}, _a[blob.type] = blob, _a));
                            return [4 /*yield*/, navigator.clipboard.write([item])];
                        case 2:
                            _b.sent();
                            return [2 /*return*/];
                    }
                });
            });
        };
        SaveToolView.prototype.save = function (name) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var blob, link;
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, this.plot_view.to_blob()];
                        case 1:
                            blob = _a.sent();
                            link = document.createElement("a");
                            link.href = URL.createObjectURL(blob);
                            link.download = name; // + ".png" | "svg" (inferred from MIME type)
                            link.target = "_blank";
                            link.dispatchEvent(new MouseEvent("click"));
                            return [2 /*return*/];
                    }
                });
            });
        };
        SaveToolView.prototype.doit = function (action) {
            if (action === void 0) {
                action = "save";
            }
            switch (action) {
                case "save":
                    this.save("bokeh_plot");
                    break;
                case "copy":
                    this.copy();
                    break;
            }
        };
        return SaveToolView;
    }(action_tool_1.ActionToolView));
    exports.SaveToolView = SaveToolView;
    SaveToolView.__name__ = "SaveToolView";
    var SaveTool = /** @class */ (function (_super) {
        tslib_1.__extends(SaveTool, _super);
        function SaveTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Save";
            _this.icon = icons_1.bk_tool_icon_save;
            return _this;
        }
        SaveTool.init_SaveTool = function () {
            this.prototype.default_view = SaveToolView;
            this.register_alias("save", function () { return new SaveTool(); });
        };
        Object.defineProperty(SaveTool.prototype, "menu", {
            get: function () {
                var _this = this;
                return [
                    {
                        icon: "bk-tool-icon-copy-to-clipboard",
                        tooltip: "Copy image to clipboard",
                        if: function () { return typeof ClipboardItem !== "undefined"; },
                        handler: function () {
                            _this.do.emit("copy");
                        },
                    },
                ];
            },
            enumerable: true,
            configurable: true
        });
        return SaveTool;
    }(action_tool_1.ActionTool));
    exports.SaveTool = SaveTool;
    SaveTool.__name__ = "SaveTool";
    SaveTool.init_SaveTool();
},
/* models/tools/actions/undo_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var action_tool_1 = require(422) /* ./action_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var UndoToolView = /** @class */ (function (_super) {
        tslib_1.__extends(UndoToolView, _super);
        function UndoToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        UndoToolView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.plot_view.state_changed, function () { return _this.model.disabled = !_this.plot_view.can_undo(); });
        };
        UndoToolView.prototype.doit = function () {
            this.plot_view.undo();
        };
        return UndoToolView;
    }(action_tool_1.ActionToolView));
    exports.UndoToolView = UndoToolView;
    UndoToolView.__name__ = "UndoToolView";
    var UndoTool = /** @class */ (function (_super) {
        tslib_1.__extends(UndoTool, _super);
        function UndoTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Undo";
            _this.icon = icons_1.bk_tool_icon_undo;
            return _this;
        }
        UndoTool.init_UndoTool = function () {
            this.prototype.default_view = UndoToolView;
            this.override({
                disabled: true,
            });
            this.register_alias("undo", function () { return new UndoTool(); });
        };
        return UndoTool;
    }(action_tool_1.ActionTool));
    exports.UndoTool = UndoTool;
    UndoTool.__name__ = "UndoTool";
    UndoTool.init_UndoTool();
},
/* models/tools/actions/zoom_in_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var zoom_base_tool_1 = require(468) /* ./zoom_base_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var ZoomInTool = /** @class */ (function (_super) {
        tslib_1.__extends(ZoomInTool, _super);
        function ZoomInTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.sign = 1;
            _this.tool_name = "Zoom In";
            _this.icon = icons_1.bk_tool_icon_zoom_in;
            return _this;
        }
        ZoomInTool.init_ZoomInTool = function () {
            this.prototype.default_view = zoom_base_tool_1.ZoomBaseToolView;
            this.register_alias("zoom_in", function () { return new ZoomInTool({ dimensions: 'both' }); });
            this.register_alias("xzoom_in", function () { return new ZoomInTool({ dimensions: 'width' }); });
            this.register_alias("yzoom_in", function () { return new ZoomInTool({ dimensions: 'height' }); });
        };
        return ZoomInTool;
    }(zoom_base_tool_1.ZoomBaseTool));
    exports.ZoomInTool = ZoomInTool;
    ZoomInTool.__name__ = "ZoomInTool";
    ZoomInTool.init_ZoomInTool();
},
/* models/tools/actions/zoom_base_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var action_tool_1 = require(422) /* ./action_tool */;
    var zoom_1 = require(469) /* ../../../core/util/zoom */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var ZoomBaseToolView = /** @class */ (function (_super) {
        tslib_1.__extends(ZoomBaseToolView, _super);
        function ZoomBaseToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ZoomBaseToolView.prototype.doit = function () {
            var frame = this.plot_view.frame;
            var dims = this.model.dimensions;
            // restrict to axis configured in tool's dimensions property
            var h_axis = dims == 'width' || dims == 'both';
            var v_axis = dims == 'height' || dims == 'both';
            var zoom_info = zoom_1.scale_range(frame, this.model.sign * this.model.factor, h_axis, v_axis);
            this.plot_view.push_state('zoom_out', { range: zoom_info });
            this.plot_view.update_range(zoom_info, false, true);
            if (this.model.document)
                this.model.document.interactive_start(this.plot_model);
        };
        return ZoomBaseToolView;
    }(action_tool_1.ActionToolView));
    exports.ZoomBaseToolView = ZoomBaseToolView;
    ZoomBaseToolView.__name__ = "ZoomBaseToolView";
    var ZoomBaseTool = /** @class */ (function (_super) {
        tslib_1.__extends(ZoomBaseTool, _super);
        function ZoomBaseTool(attrs) {
            return _super.call(this, attrs) || this;
        }
        ZoomBaseTool.init_ZoomBaseTool = function () {
            this.prototype.default_view = ZoomBaseToolView;
            this.define({
                factor: [p.Percent, 0.1],
                dimensions: [p.Dimensions, "both"],
            });
        };
        Object.defineProperty(ZoomBaseTool.prototype, "tooltip", {
            get: function () {
                return this._get_dim_tooltip(this.tool_name, this.dimensions);
            },
            enumerable: true,
            configurable: true
        });
        return ZoomBaseTool;
    }(action_tool_1.ActionTool));
    exports.ZoomBaseTool = ZoomBaseTool;
    ZoomBaseTool.__name__ = "ZoomBaseTool";
    ZoomBaseTool.init_ZoomBaseTool();
},
/* core/util/zoom.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var math_1 = require(125) /* ./math */;
    // Module for zoom-related functions
    function scale_highlow(range, factor, center) {
        var _a = tslib_1.__read([range.start, range.end], 2), low = _a[0], high = _a[1];
        var x = center != null ? center : (high + low) / 2.0;
        var x0 = low - (low - x) * factor;
        var x1 = high - (high - x) * factor;
        return [x0, x1];
    }
    exports.scale_highlow = scale_highlow;
    function get_info(scales, _a) {
        var e_1, _b;
        var _c = tslib_1.__read(_a, 2), sxy0 = _c[0], sxy1 = _c[1];
        var info = new Map();
        try {
            for (var scales_1 = tslib_1.__values(scales), scales_1_1 = scales_1.next(); !scales_1_1.done; scales_1_1 = scales_1.next()) {
                var _d = tslib_1.__read(scales_1_1.value, 2), name = _d[0], scale = _d[1];
                var _e = tslib_1.__read(scale.r_invert(sxy0, sxy1), 2), start = _e[0], end = _e[1];
                info.set(name, { start: start, end: end });
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (scales_1_1 && !scales_1_1.done && (_b = scales_1.return))
                    _b.call(scales_1);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        return info;
    }
    exports.get_info = get_info;
    function scale_range(frame, factor, h_axis, v_axis, center) {
        if (h_axis === void 0) {
            h_axis = true;
        }
        if (v_axis === void 0) {
            v_axis = true;
        }
        /*
         * Utility function for zoom tools to calculate/create the zoom_info object
         * of the form required by ``PlotView.update_range``
         *
         * Parameters:
         *   frame : CartesianFrame
         *   factor : Number
         *   h_axis : Boolean, optional
         *     whether to zoom the horizontal axis (default = true)
         *   v_axis : Boolean, optional
         *     whether to zoom the horizontal axis (default = true)
         *   center : object, optional
         *     of form {'x': Number, 'y', Number}
         *
         * Returns:
         *   object:
         */
        // clamp the  magnitude of factor, if it is > 1 bad things happen
        factor = math_1.clamp(factor, -0.9, 0.9);
        var hfactor = h_axis ? factor : 0;
        var _a = tslib_1.__read(scale_highlow(frame.bbox.h_range, hfactor, center != null ? center.x : undefined), 2), sx0 = _a[0], sx1 = _a[1];
        var xrs = get_info(frame.x_scales, [sx0, sx1]);
        var vfactor = v_axis ? factor : 0;
        var _b = tslib_1.__read(scale_highlow(frame.bbox.v_range, vfactor, center != null ? center.y : undefined), 2), sy0 = _b[0], sy1 = _b[1];
        var yrs = get_info(frame.y_scales, [sy0, sy1]);
        // OK this sucks we can't set factor independently in each direction. It is used
        // for GMap plots, and GMap plots always preserve aspect, so effective the value
        // of 'dimensions' is ignored.
        return { xrs: xrs, yrs: yrs, factor: factor };
    }
    exports.scale_range = scale_range;
},
/* models/tools/actions/zoom_out_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var zoom_base_tool_1 = require(468) /* ./zoom_base_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var ZoomOutTool = /** @class */ (function (_super) {
        tslib_1.__extends(ZoomOutTool, _super);
        function ZoomOutTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.sign = -1;
            _this.tool_name = "Zoom Out";
            _this.icon = icons_1.bk_tool_icon_zoom_out;
            return _this;
        }
        ZoomOutTool.init_ZoomOutTool = function () {
            this.prototype.default_view = zoom_base_tool_1.ZoomBaseToolView;
            this.register_alias("zoom_out", function () { return new ZoomOutTool({ dimensions: 'both' }); });
            this.register_alias("xzoom_out", function () { return new ZoomOutTool({ dimensions: 'width' }); });
            this.register_alias("yzoom_out", function () { return new ZoomOutTool({ dimensions: 'height' }); });
        };
        return ZoomOutTool;
    }(zoom_base_tool_1.ZoomBaseTool));
    exports.ZoomOutTool = ZoomOutTool;
    ZoomOutTool.__name__ = "ZoomOutTool";
    ZoomOutTool.init_ZoomOutTool();
},
/* models/tools/edit/edit_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var array_1 = require(124) /* ../../../core/util/array */;
    var types_1 = require(123) /* ../../../core/util/types */;
    var assert_1 = require(126) /* ../../../core/util/assert */;
    var gesture_tool_1 = require(421) /* ../gestures/gesture_tool */;
    var EditToolView = /** @class */ (function (_super) {
        tslib_1.__extends(EditToolView, _super);
        function EditToolView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this._mouse_in_frame = true;
            return _this;
        }
        EditToolView.prototype._select_mode = function (ev) {
            var shiftKey = ev.shiftKey, ctrlKey = ev.ctrlKey;
            if (!shiftKey && !ctrlKey)
                return "replace";
            else if (shiftKey && !ctrlKey)
                return "append";
            else if (!shiftKey && ctrlKey)
                return "intersect";
            else if (shiftKey && ctrlKey)
                return "subtract";
            else
                assert_1.unreachable();
        };
        EditToolView.prototype._move_enter = function (_e) {
            this._mouse_in_frame = true;
        };
        EditToolView.prototype._move_exit = function (_e) {
            this._mouse_in_frame = false;
        };
        EditToolView.prototype._map_drag = function (sx, sy, renderer) {
            // Maps screen to data coordinates
            var frame = this.plot_view.frame;
            if (!frame.bbox.contains(sx, sy)) {
                return null;
            }
            var renderer_view = this.plot_view.renderer_views.get(renderer);
            var x = renderer_view.coordinates.x_scale.invert(sx);
            var y = renderer_view.coordinates.y_scale.invert(sy);
            return [x, y];
        };
        EditToolView.prototype._delete_selected = function (renderer) {
            var e_1, _a;
            // Deletes all selected rows in the ColumnDataSource
            var cds = renderer.data_source;
            var indices = cds.selected.indices;
            indices.sort();
            try {
                for (var _b = tslib_1.__values(cds.columns()), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var column = _c.value;
                    var values = cds.get_array(column);
                    for (var index = 0; index < indices.length; index++) {
                        var ind = indices[index];
                        values.splice(ind - index, 1);
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            this._emit_cds_changes(cds);
        };
        EditToolView.prototype._pop_glyphs = function (cds, num_objects) {
            var e_2, _a;
            // Pops rows in the CDS until only num_objects are left
            var columns = cds.columns();
            if (!num_objects || !columns.length)
                return;
            try {
                for (var columns_1 = tslib_1.__values(columns), columns_1_1 = columns_1.next(); !columns_1_1.done; columns_1_1 = columns_1.next()) {
                    var column = columns_1_1.value;
                    var array = cds.get_array(column);
                    var drop = array.length - num_objects + 1;
                    if (drop < 1)
                        continue;
                    if (!types_1.isArray(array)) {
                        array = Array.from(array);
                        cds.data[column] = array;
                    }
                    array.splice(0, drop);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (columns_1_1 && !columns_1_1.done && (_a = columns_1.return))
                        _a.call(columns_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
        };
        EditToolView.prototype._emit_cds_changes = function (cds, redraw, clear, emit) {
            if (redraw === void 0) {
                redraw = true;
            }
            if (clear === void 0) {
                clear = true;
            }
            if (emit === void 0) {
                emit = true;
            }
            if (clear)
                cds.selection_manager.clear();
            if (redraw)
                cds.change.emit();
            if (emit) {
                cds.data = cds.data;
                cds.properties.data.change.emit();
            }
        };
        EditToolView.prototype._drag_points = function (ev, renderers, dim) {
            var e_3, _a, e_4, _b;
            if (dim === void 0) {
                dim = "both";
            }
            if (this._basepoint == null)
                return;
            var _c = tslib_1.__read(this._basepoint, 2), bx = _c[0], by = _c[1];
            try {
                for (var renderers_1 = tslib_1.__values(renderers), renderers_1_1 = renderers_1.next(); !renderers_1_1.done; renderers_1_1 = renderers_1.next()) {
                    var renderer = renderers_1_1.value;
                    var basepoint = this._map_drag(bx, by, renderer);
                    var point = this._map_drag(ev.sx, ev.sy, renderer);
                    if (point == null || basepoint == null) {
                        continue;
                    }
                    var _d = tslib_1.__read(point, 2), x = _d[0], y = _d[1];
                    var _f = tslib_1.__read(basepoint, 2), px = _f[0], py = _f[1];
                    var _g = tslib_1.__read([x - px, y - py], 2), dx = _g[0], dy = _g[1];
                    // Type once dataspecs are typed
                    var glyph = renderer.glyph;
                    var cds = renderer.data_source;
                    var _h = tslib_1.__read([glyph.x.field, glyph.y.field], 2), xkey = _h[0], ykey = _h[1];
                    try {
                        for (var _j = (e_4 = void 0, tslib_1.__values(cds.selected.indices)), _k = _j.next(); !_k.done; _k = _j.next()) {
                            var index = _k.value;
                            if (xkey && (dim == "width" || dim == "both")) {
                                cds.data[xkey][index] += dx;
                            }
                            if (ykey && (dim == "height" || dim == "both")) {
                                cds.data[ykey][index] += dy;
                            }
                        }
                    }
                    catch (e_4_1) {
                        e_4 = { error: e_4_1 };
                    }
                    finally {
                        try {
                            if (_k && !_k.done && (_b = _j.return))
                                _b.call(_j);
                        }
                        finally {
                            if (e_4)
                                throw e_4.error;
                        }
                    }
                    cds.change.emit();
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (renderers_1_1 && !renderers_1_1.done && (_a = renderers_1.return))
                        _a.call(renderers_1);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            this._basepoint = [ev.sx, ev.sy];
        };
        EditToolView.prototype._pad_empty_columns = function (cds, coord_columns) {
            var e_5, _a;
            try {
                // Pad ColumnDataSource non-coordinate columns with empty_value
                for (var _b = tslib_1.__values(cds.columns()), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var column = _c.value;
                    if (!array_1.includes(coord_columns, column))
                        cds.get_array(column).push(this.model.empty_value);
                }
            }
            catch (e_5_1) {
                e_5 = { error: e_5_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_5)
                        throw e_5.error;
                }
            }
        };
        EditToolView.prototype._select_event = function (ev, mode, renderers) {
            var e_6, _a;
            // Process selection event on the supplied renderers and return selected renderers
            var frame = this.plot_view.frame;
            var sx = ev.sx, sy = ev.sy;
            if (!frame.bbox.contains(sx, sy)) {
                return [];
            }
            var geometry = { type: 'point', sx: sx, sy: sy };
            var selected = [];
            try {
                for (var renderers_2 = tslib_1.__values(renderers), renderers_2_1 = renderers_2.next(); !renderers_2_1.done; renderers_2_1 = renderers_2.next()) {
                    var renderer = renderers_2_1.value;
                    var sm = renderer.get_selection_manager();
                    var cds = renderer.data_source;
                    var views = [this.plot_view.renderer_views.get(renderer)];
                    var did_hit = sm.select(views, geometry, true, mode);
                    if (did_hit) {
                        selected.push(renderer);
                    }
                    cds.properties.selected.change.emit();
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (renderers_2_1 && !renderers_2_1.done && (_a = renderers_2.return))
                        _a.call(renderers_2);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
            return selected;
        };
        return EditToolView;
    }(gesture_tool_1.GestureToolView));
    exports.EditToolView = EditToolView;
    EditToolView.__name__ = "EditToolView";
    var EditTool = /** @class */ (function (_super) {
        tslib_1.__extends(EditTool, _super);
        function EditTool(attrs) {
            return _super.call(this, attrs) || this;
        }
        EditTool.init_EditTool = function () {
            this.define({
                custom_icon: [p.String],
                custom_tooltip: [p.String],
                empty_value: [p.Any],
                renderers: [p.Array, []],
            });
        };
        Object.defineProperty(EditTool.prototype, "tooltip", {
            get: function () {
                return this.custom_tooltip || this.tool_name;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EditTool.prototype, "computed_icon", {
            get: function () {
                return this.custom_icon || this.icon;
            },
            enumerable: true,
            configurable: true
        });
        return EditTool;
    }(gesture_tool_1.GestureTool));
    exports.EditTool = EditTool;
    EditTool.__name__ = "EditTool";
    EditTool.init_EditTool();
},
/* models/tools/edit/box_edit_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var dom_1 = require(187) /* ../../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var edit_tool_1 = require(471) /* ./edit_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var BoxEditToolView = /** @class */ (function (_super) {
        tslib_1.__extends(BoxEditToolView, _super);
        function BoxEditToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BoxEditToolView.prototype._tap = function (ev) {
            if ((this._draw_basepoint != null) || (this._basepoint != null))
                return;
            this._select_event(ev, this._select_mode(ev), this.model.renderers);
        };
        BoxEditToolView.prototype._keyup = function (ev) {
            var e_1, _a;
            if (!this.model.active || !this._mouse_in_frame)
                return;
            try {
                for (var _b = tslib_1.__values(this.model.renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var renderer = _c.value;
                    if (ev.keyCode === dom_1.Keys.Backspace) {
                        this._delete_selected(renderer);
                    }
                    else if (ev.keyCode == dom_1.Keys.Esc) {
                        // Type properly once selection_manager is typed
                        var cds = renderer.data_source;
                        cds.selection_manager.clear();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        BoxEditToolView.prototype._set_extent = function (_a, _b, append, emit) {
            var _c = tslib_1.__read(_a, 2), sx0 = _c[0], sx1 = _c[1];
            var _d = tslib_1.__read(_b, 2), sy0 = _d[0], sy1 = _d[1];
            if (emit === void 0) {
                emit = false;
            }
            var renderer = this.model.renderers[0];
            var renderer_view = this.plot_view.renderer_views.get(renderer);
            // Type once dataspecs are typed
            var glyph = renderer.glyph;
            var cds = renderer.data_source;
            var _e = tslib_1.__read(renderer_view.coordinates.x_scale.r_invert(sx0, sx1), 2), x0 = _e[0], x1 = _e[1];
            var _f = tslib_1.__read(renderer_view.coordinates.y_scale.r_invert(sy0, sy1), 2), y0 = _f[0], y1 = _f[1];
            var _g = tslib_1.__read([(x0 + x1) / 2, (y0 + y1) / 2], 2), x = _g[0], y = _g[1];
            var _h = tslib_1.__read([x1 - x0, y1 - y0], 2), w = _h[0], h = _h[1];
            var _j = tslib_1.__read([glyph.x.field, glyph.y.field], 2), xkey = _j[0], ykey = _j[1];
            var _k = tslib_1.__read([glyph.width.field, glyph.height.field], 2), wkey = _k[0], hkey = _k[1];
            if (append) {
                this._pop_glyphs(cds, this.model.num_objects);
                if (xkey)
                    cds.get_array(xkey).push(x);
                if (ykey)
                    cds.get_array(ykey).push(y);
                if (wkey)
                    cds.get_array(wkey).push(w);
                if (hkey)
                    cds.get_array(hkey).push(h);
                this._pad_empty_columns(cds, [xkey, ykey, wkey, hkey]);
            }
            else {
                var index = cds.data[xkey].length - 1;
                if (xkey)
                    cds.data[xkey][index] = x;
                if (ykey)
                    cds.data[ykey][index] = y;
                if (wkey)
                    cds.data[wkey][index] = w;
                if (hkey)
                    cds.data[hkey][index] = h;
            }
            this._emit_cds_changes(cds, true, false, emit);
        };
        BoxEditToolView.prototype._update_box = function (ev, append, emit) {
            if (append === void 0) {
                append = false;
            }
            if (emit === void 0) {
                emit = false;
            }
            if (this._draw_basepoint == null)
                return;
            var curpoint = [ev.sx, ev.sy];
            var frame = this.plot_view.frame;
            var dims = this.model.dimensions;
            var limits = this.model._get_dim_limits(this._draw_basepoint, curpoint, frame, dims);
            if (limits != null) {
                var _a = tslib_1.__read(limits, 2), sxlim = _a[0], sylim = _a[1];
                this._set_extent(sxlim, sylim, append, emit);
            }
        };
        BoxEditToolView.prototype._doubletap = function (ev) {
            if (!this.model.active)
                return;
            if (this._draw_basepoint != null) {
                this._update_box(ev, false, true);
                this._draw_basepoint = null;
            }
            else {
                this._draw_basepoint = [ev.sx, ev.sy];
                this._select_event(ev, "append", this.model.renderers);
                this._update_box(ev, true, false);
            }
        };
        BoxEditToolView.prototype._move = function (ev) {
            this._update_box(ev, false, false);
        };
        BoxEditToolView.prototype._pan_start = function (ev) {
            if (ev.shiftKey) {
                if (this._draw_basepoint != null)
                    return;
                this._draw_basepoint = [ev.sx, ev.sy];
                this._update_box(ev, true, false);
            }
            else {
                if (this._basepoint != null)
                    return;
                this._select_event(ev, "append", this.model.renderers);
                this._basepoint = [ev.sx, ev.sy];
            }
        };
        BoxEditToolView.prototype._pan = function (ev, append, emit) {
            if (append === void 0) {
                append = false;
            }
            if (emit === void 0) {
                emit = false;
            }
            if (ev.shiftKey) {
                if (this._draw_basepoint == null)
                    return;
                this._update_box(ev, append, emit);
            }
            else {
                if (this._basepoint == null)
                    return;
                this._drag_points(ev, this.model.renderers);
            }
        };
        BoxEditToolView.prototype._pan_end = function (ev) {
            var e_2, _a;
            this._pan(ev, false, true);
            if (ev.shiftKey) {
                this._draw_basepoint = null;
            }
            else {
                this._basepoint = null;
                try {
                    for (var _b = tslib_1.__values(this.model.renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var renderer = _c.value;
                        this._emit_cds_changes(renderer.data_source, false, true, true);
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return))
                            _a.call(_b);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
            }
        };
        return BoxEditToolView;
    }(edit_tool_1.EditToolView));
    exports.BoxEditToolView = BoxEditToolView;
    BoxEditToolView.__name__ = "BoxEditToolView";
    var BoxEditTool = /** @class */ (function (_super) {
        tslib_1.__extends(BoxEditTool, _super);
        function BoxEditTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Box Edit Tool";
            _this.icon = icons_1.bk_tool_icon_box_edit;
            _this.event_type = ["tap", "pan", "move"];
            _this.default_order = 1;
            return _this;
        }
        BoxEditTool.init_BoxEditTool = function () {
            this.prototype.default_view = BoxEditToolView;
            this.define({
                dimensions: [p.Dimensions, "both"],
                num_objects: [p.Int, 0],
            });
        };
        return BoxEditTool;
    }(edit_tool_1.EditTool));
    exports.BoxEditTool = BoxEditTool;
    BoxEditTool.__name__ = "BoxEditTool";
    BoxEditTool.init_BoxEditTool();
},
/* models/tools/edit/freehand_draw_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var dom_1 = require(187) /* ../../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var types_1 = require(123) /* ../../../core/util/types */;
    var edit_tool_1 = require(471) /* ./edit_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var FreehandDrawToolView = /** @class */ (function (_super) {
        tslib_1.__extends(FreehandDrawToolView, _super);
        function FreehandDrawToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        FreehandDrawToolView.prototype._draw = function (ev, mode, emit) {
            if (emit === void 0) {
                emit = false;
            }
            if (!this.model.active)
                return;
            var renderer = this.model.renderers[0];
            var point = this._map_drag(ev.sx, ev.sy, renderer);
            if (point == null)
                return;
            var _a = tslib_1.__read(point, 2), x = _a[0], y = _a[1];
            var cds = renderer.data_source;
            var glyph = renderer.glyph;
            var _b = tslib_1.__read([glyph.xs.field, glyph.ys.field], 2), xkey = _b[0], ykey = _b[1];
            if (mode == 'new') {
                this._pop_glyphs(cds, this.model.num_objects);
                if (xkey)
                    cds.get_array(xkey).push([x]);
                if (ykey)
                    cds.get_array(ykey).push([y]);
                this._pad_empty_columns(cds, [xkey, ykey]);
            }
            else if (mode == 'add') {
                if (xkey) {
                    var xidx = cds.data[xkey].length - 1;
                    var xs = cds.get_array(xkey)[xidx];
                    if (!types_1.isArray(xs)) {
                        xs = Array.from(xs);
                        cds.data[xkey][xidx] = xs;
                    }
                    xs.push(x);
                }
                if (ykey) {
                    var yidx = cds.data[ykey].length - 1;
                    var ys = cds.get_array(ykey)[yidx];
                    if (!types_1.isArray(ys)) {
                        ys = Array.from(ys);
                        cds.data[ykey][yidx] = ys;
                    }
                    ys.push(y);
                }
            }
            this._emit_cds_changes(cds, true, true, emit);
        };
        FreehandDrawToolView.prototype._pan_start = function (ev) {
            this._draw(ev, 'new');
        };
        FreehandDrawToolView.prototype._pan = function (ev) {
            this._draw(ev, 'add');
        };
        FreehandDrawToolView.prototype._pan_end = function (ev) {
            this._draw(ev, 'add', true);
        };
        FreehandDrawToolView.prototype._tap = function (ev) {
            this._select_event(ev, this._select_mode(ev), this.model.renderers);
        };
        FreehandDrawToolView.prototype._keyup = function (ev) {
            var e_1, _a;
            if (!this.model.active || !this._mouse_in_frame)
                return;
            try {
                for (var _b = tslib_1.__values(this.model.renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var renderer = _c.value;
                    if (ev.keyCode === dom_1.Keys.Esc) {
                        renderer.data_source.selection_manager.clear();
                    }
                    else if (ev.keyCode === dom_1.Keys.Backspace) {
                        this._delete_selected(renderer);
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        return FreehandDrawToolView;
    }(edit_tool_1.EditToolView));
    exports.FreehandDrawToolView = FreehandDrawToolView;
    FreehandDrawToolView.__name__ = "FreehandDrawToolView";
    var FreehandDrawTool = /** @class */ (function (_super) {
        tslib_1.__extends(FreehandDrawTool, _super);
        function FreehandDrawTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Freehand Draw Tool";
            _this.icon = icons_1.bk_tool_icon_freehand_draw;
            _this.event_type = ["pan", "tap"];
            _this.default_order = 3;
            return _this;
        }
        FreehandDrawTool.init_FreehandDrawTool = function () {
            this.prototype.default_view = FreehandDrawToolView;
            this.define({
                num_objects: [p.Int, 0],
            });
        };
        return FreehandDrawTool;
    }(edit_tool_1.EditTool));
    exports.FreehandDrawTool = FreehandDrawTool;
    FreehandDrawTool.__name__ = "FreehandDrawTool";
    FreehandDrawTool.init_FreehandDrawTool();
},
/* models/tools/edit/point_draw_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var dom_1 = require(187) /* ../../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var edit_tool_1 = require(471) /* ./edit_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var PointDrawToolView = /** @class */ (function (_super) {
        tslib_1.__extends(PointDrawToolView, _super);
        function PointDrawToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        PointDrawToolView.prototype._tap = function (ev) {
            var renderers = this._select_event(ev, this._select_mode(ev), this.model.renderers);
            if (renderers.length || !this.model.add) {
                return;
            }
            var renderer = this.model.renderers[0];
            var point = this._map_drag(ev.sx, ev.sy, renderer);
            if (point == null)
                return;
            // Type once dataspecs are typed
            var glyph = renderer.glyph;
            var cds = renderer.data_source;
            var _a = tslib_1.__read([glyph.x.field, glyph.y.field], 2), xkey = _a[0], ykey = _a[1];
            var _b = tslib_1.__read(point, 2), x = _b[0], y = _b[1];
            this._pop_glyphs(cds, this.model.num_objects);
            if (xkey)
                cds.get_array(xkey).push(x);
            if (ykey)
                cds.get_array(ykey).push(y);
            this._pad_empty_columns(cds, [xkey, ykey]);
            cds.change.emit();
            cds.data = cds.data;
            cds.properties.data.change.emit();
        };
        PointDrawToolView.prototype._keyup = function (ev) {
            var e_1, _a;
            if (!this.model.active || !this._mouse_in_frame)
                return;
            try {
                for (var _b = tslib_1.__values(this.model.renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var renderer = _c.value;
                    if (ev.keyCode === dom_1.Keys.Backspace) {
                        this._delete_selected(renderer);
                    }
                    else if (ev.keyCode == dom_1.Keys.Esc) {
                        renderer.data_source.selection_manager.clear();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        PointDrawToolView.prototype._pan_start = function (ev) {
            if (!this.model.drag)
                return;
            this._select_event(ev, "append", this.model.renderers);
            this._basepoint = [ev.sx, ev.sy];
        };
        PointDrawToolView.prototype._pan = function (ev) {
            if (!this.model.drag || this._basepoint == null)
                return;
            this._drag_points(ev, this.model.renderers);
        };
        PointDrawToolView.prototype._pan_end = function (ev) {
            var e_2, _a;
            if (!this.model.drag)
                return;
            this._pan(ev);
            try {
                for (var _b = tslib_1.__values(this.model.renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var renderer = _c.value;
                    this._emit_cds_changes(renderer.data_source, false, true, true);
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            this._basepoint = null;
        };
        return PointDrawToolView;
    }(edit_tool_1.EditToolView));
    exports.PointDrawToolView = PointDrawToolView;
    PointDrawToolView.__name__ = "PointDrawToolView";
    var PointDrawTool = /** @class */ (function (_super) {
        tslib_1.__extends(PointDrawTool, _super);
        function PointDrawTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Point Draw Tool";
            _this.icon = icons_1.bk_tool_icon_point_draw;
            _this.event_type = ["tap", "pan", "move"];
            _this.default_order = 2;
            return _this;
        }
        PointDrawTool.init_PointDrawTool = function () {
            this.prototype.default_view = PointDrawToolView;
            this.define({
                add: [p.Boolean, true],
                drag: [p.Boolean, true],
                num_objects: [p.Int, 0],
            });
        };
        return PointDrawTool;
    }(edit_tool_1.EditTool));
    exports.PointDrawTool = PointDrawTool;
    PointDrawTool.__name__ = "PointDrawTool";
    PointDrawTool.init_PointDrawTool();
},
/* models/tools/edit/poly_draw_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var dom_1 = require(187) /* ../../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var types_1 = require(123) /* ../../../core/util/types */;
    var poly_tool_1 = require(476) /* ./poly_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var PolyDrawToolView = /** @class */ (function (_super) {
        tslib_1.__extends(PolyDrawToolView, _super);
        function PolyDrawToolView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this._drawing = false;
            _this._initialized = false;
            return _this;
        }
        PolyDrawToolView.prototype._tap = function (ev) {
            if (this._drawing)
                this._draw(ev, 'add', true);
            else
                this._select_event(ev, this._select_mode(ev), this.model.renderers);
        };
        PolyDrawToolView.prototype._draw = function (ev, mode, emit) {
            if (emit === void 0) {
                emit = false;
            }
            var renderer = this.model.renderers[0];
            var point = this._map_drag(ev.sx, ev.sy, renderer);
            if (!this._initialized)
                this.activate(); // Ensure that activate has been called
            if (point == null)
                return;
            var _a = tslib_1.__read(this._snap_to_vertex.apply(this, tslib_1.__spread([ev], point)), 2), x = _a[0], y = _a[1];
            var cds = renderer.data_source;
            var glyph = renderer.glyph;
            var _b = tslib_1.__read([glyph.xs.field, glyph.ys.field], 2), xkey = _b[0], ykey = _b[1];
            if (mode == 'new') {
                this._pop_glyphs(cds, this.model.num_objects);
                if (xkey)
                    cds.get_array(xkey).push([x, x]);
                if (ykey)
                    cds.get_array(ykey).push([y, y]);
                this._pad_empty_columns(cds, [xkey, ykey]);
            }
            else if (mode == 'edit') {
                if (xkey) {
                    var xs = cds.data[xkey][cds.data[xkey].length - 1];
                    xs[xs.length - 1] = x;
                }
                if (ykey) {
                    var ys = cds.data[ykey][cds.data[ykey].length - 1];
                    ys[ys.length - 1] = y;
                }
            }
            else if (mode == 'add') {
                if (xkey) {
                    var xidx = cds.data[xkey].length - 1;
                    var xs = cds.get_array(xkey)[xidx];
                    var nx = xs[xs.length - 1];
                    xs[xs.length - 1] = x;
                    if (!types_1.isArray(xs)) {
                        xs = Array.from(xs);
                        cds.data[xkey][xidx] = xs;
                    }
                    xs.push(nx);
                }
                if (ykey) {
                    var yidx = cds.data[ykey].length - 1;
                    var ys = cds.get_array(ykey)[yidx];
                    var ny = ys[ys.length - 1];
                    ys[ys.length - 1] = y;
                    if (!types_1.isArray(ys)) {
                        ys = Array.from(ys);
                        cds.data[ykey][yidx] = ys;
                    }
                    ys.push(ny);
                }
            }
            this._emit_cds_changes(cds, true, false, emit);
        };
        PolyDrawToolView.prototype._show_vertices = function () {
            var e_1, _a, e_2, _b;
            if (!this.model.active) {
                return;
            }
            var xs = [];
            var ys = [];
            for (var i = 0; i < this.model.renderers.length; i++) {
                var renderer = this.model.renderers[i];
                var cds = renderer.data_source;
                var glyph = renderer.glyph;
                var _c = tslib_1.__read([glyph.xs.field, glyph.ys.field], 2), xkey = _c[0], ykey = _c[1];
                if (xkey) {
                    try {
                        for (var _d = (e_1 = void 0, tslib_1.__values(cds.get_array(xkey))), _e = _d.next(); !_e.done; _e = _d.next()) {
                            var array = _e.value;
                            Array.prototype.push.apply(xs, array);
                        }
                    }
                    catch (e_1_1) {
                        e_1 = { error: e_1_1 };
                    }
                    finally {
                        try {
                            if (_e && !_e.done && (_a = _d.return))
                                _a.call(_d);
                        }
                        finally {
                            if (e_1)
                                throw e_1.error;
                        }
                    }
                }
                if (ykey) {
                    try {
                        for (var _f = (e_2 = void 0, tslib_1.__values(cds.get_array(ykey))), _g = _f.next(); !_g.done; _g = _f.next()) {
                            var array = _g.value;
                            Array.prototype.push.apply(ys, array);
                        }
                    }
                    catch (e_2_1) {
                        e_2 = { error: e_2_1 };
                    }
                    finally {
                        try {
                            if (_g && !_g.done && (_b = _f.return))
                                _b.call(_f);
                        }
                        finally {
                            if (e_2)
                                throw e_2.error;
                        }
                    }
                }
                if (this._drawing && (i == (this.model.renderers.length - 1))) {
                    // Skip currently drawn vertex
                    xs.splice(xs.length - 1, 1);
                    ys.splice(ys.length - 1, 1);
                }
            }
            this._set_vertices(xs, ys);
        };
        PolyDrawToolView.prototype._doubletap = function (ev) {
            if (!this.model.active)
                return;
            if (this._drawing) {
                this._drawing = false;
                this._draw(ev, 'edit', true);
            }
            else {
                this._drawing = true;
                this._draw(ev, 'new', true);
            }
        };
        PolyDrawToolView.prototype._move = function (ev) {
            if (this._drawing) {
                this._draw(ev, 'edit');
            }
        };
        PolyDrawToolView.prototype._remove = function () {
            var renderer = this.model.renderers[0];
            var cds = renderer.data_source;
            var glyph = renderer.glyph;
            var _a = tslib_1.__read([glyph.xs.field, glyph.ys.field], 2), xkey = _a[0], ykey = _a[1];
            if (xkey) {
                var xidx = cds.data[xkey].length - 1;
                var xs = cds.get_array(xkey)[xidx];
                xs.splice(xs.length - 1, 1);
            }
            if (ykey) {
                var yidx = cds.data[ykey].length - 1;
                var ys = cds.get_array(ykey)[yidx];
                ys.splice(ys.length - 1, 1);
            }
            this._emit_cds_changes(cds);
        };
        PolyDrawToolView.prototype._keyup = function (ev) {
            var e_3, _a;
            if (!this.model.active || !this._mouse_in_frame)
                return;
            try {
                for (var _b = tslib_1.__values(this.model.renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var renderer = _c.value;
                    if (ev.keyCode === dom_1.Keys.Backspace) {
                        this._delete_selected(renderer);
                    }
                    else if (ev.keyCode == dom_1.Keys.Esc) {
                        if (this._drawing) {
                            this._remove();
                            this._drawing = false;
                        }
                        renderer.data_source.selection_manager.clear();
                    }
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
        };
        PolyDrawToolView.prototype._pan_start = function (ev) {
            if (!this.model.drag)
                return;
            this._select_event(ev, "append", this.model.renderers);
            this._basepoint = [ev.sx, ev.sy];
        };
        PolyDrawToolView.prototype._pan = function (ev) {
            var e_4, _a, e_5, _b;
            if (this._basepoint == null || !this.model.drag)
                return;
            var _c = tslib_1.__read(this._basepoint, 2), bx = _c[0], by = _c[1];
            try {
                // Process polygon/line dragging
                for (var _d = tslib_1.__values(this.model.renderers), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var renderer = _e.value;
                    var basepoint = this._map_drag(bx, by, renderer);
                    var point = this._map_drag(ev.sx, ev.sy, renderer);
                    if (point == null || basepoint == null)
                        continue;
                    var cds = renderer.data_source;
                    // Type once dataspecs are typed
                    var glyph = renderer.glyph;
                    var _f = tslib_1.__read([glyph.xs.field, glyph.ys.field], 2), xkey = _f[0], ykey = _f[1];
                    if (!xkey && !ykey)
                        continue;
                    var _g = tslib_1.__read(point, 2), x = _g[0], y = _g[1];
                    var _h = tslib_1.__read(basepoint, 2), px = _h[0], py = _h[1];
                    var _j = tslib_1.__read([x - px, y - py], 2), dx = _j[0], dy = _j[1];
                    try {
                        for (var _k = (e_5 = void 0, tslib_1.__values(cds.selected.indices)), _l = _k.next(); !_l.done; _l = _k.next()) {
                            var index = _l.value;
                            var length = void 0, xs = void 0, ys = void 0;
                            if (xkey)
                                xs = cds.data[xkey][index];
                            if (ykey) {
                                ys = cds.data[ykey][index];
                                length = ys.length;
                            }
                            else {
                                length = xs.length;
                            }
                            for (var i = 0; i < length; i++) {
                                if (xs)
                                    xs[i] += dx;
                                if (ys)
                                    ys[i] += dy;
                            }
                        }
                    }
                    catch (e_5_1) {
                        e_5 = { error: e_5_1 };
                    }
                    finally {
                        try {
                            if (_l && !_l.done && (_b = _k.return))
                                _b.call(_k);
                        }
                        finally {
                            if (e_5)
                                throw e_5.error;
                        }
                    }
                    cds.change.emit();
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_a = _d.return))
                        _a.call(_d);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            this._basepoint = [ev.sx, ev.sy];
        };
        PolyDrawToolView.prototype._pan_end = function (ev) {
            var e_6, _a;
            if (!this.model.drag)
                return;
            this._pan(ev);
            try {
                for (var _b = tslib_1.__values(this.model.renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var renderer = _c.value;
                    this._emit_cds_changes(renderer.data_source);
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
            this._basepoint = null;
        };
        PolyDrawToolView.prototype.activate = function () {
            var e_7, _a;
            var _this = this;
            if (!this.model.vertex_renderer || !this.model.active)
                return;
            this._show_vertices();
            if (!this._initialized) {
                try {
                    for (var _b = tslib_1.__values(this.model.renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var renderer = _c.value;
                        var cds = renderer.data_source;
                        cds.connect(cds.properties.data.change, function () { return _this._show_vertices(); });
                    }
                }
                catch (e_7_1) {
                    e_7 = { error: e_7_1 };
                }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return))
                            _a.call(_b);
                    }
                    finally {
                        if (e_7)
                            throw e_7.error;
                    }
                }
            }
            this._initialized = true;
        };
        PolyDrawToolView.prototype.deactivate = function () {
            if (this._drawing) {
                this._remove();
                this._drawing = false;
            }
            if (this.model.vertex_renderer)
                this._hide_vertices();
        };
        return PolyDrawToolView;
    }(poly_tool_1.PolyToolView));
    exports.PolyDrawToolView = PolyDrawToolView;
    PolyDrawToolView.__name__ = "PolyDrawToolView";
    var PolyDrawTool = /** @class */ (function (_super) {
        tslib_1.__extends(PolyDrawTool, _super);
        function PolyDrawTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Polygon Draw Tool";
            _this.icon = icons_1.bk_tool_icon_poly_draw;
            _this.event_type = ["pan", "tap", "move"];
            _this.default_order = 3;
            return _this;
        }
        PolyDrawTool.init_PolyDrawTool = function () {
            this.prototype.default_view = PolyDrawToolView;
            this.define({
                drag: [p.Boolean, true],
                num_objects: [p.Int, 0],
            });
        };
        return PolyDrawTool;
    }(poly_tool_1.PolyTool));
    exports.PolyDrawTool = PolyDrawTool;
    PolyDrawTool.__name__ = "PolyDrawTool";
    PolyDrawTool.init_PolyDrawTool();
},
/* models/tools/edit/poly_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var types_1 = require(123) /* ../../../core/util/types */;
    var edit_tool_1 = require(471) /* ./edit_tool */;
    var PolyToolView = /** @class */ (function (_super) {
        tslib_1.__extends(PolyToolView, _super);
        function PolyToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        PolyToolView.prototype._set_vertices = function (xs, ys) {
            var point_glyph = this.model.vertex_renderer.glyph;
            var point_cds = this.model.vertex_renderer.data_source;
            var _a = tslib_1.__read([point_glyph.x.field, point_glyph.y.field], 2), pxkey = _a[0], pykey = _a[1];
            if (pxkey) {
                if (types_1.isArray(xs))
                    point_cds.data[pxkey] = xs;
                else
                    point_glyph.x = { value: xs };
            }
            if (pykey) {
                if (types_1.isArray(ys))
                    point_cds.data[pykey] = ys;
                else
                    point_glyph.y = { value: ys };
            }
            this._emit_cds_changes(point_cds, true, true, false);
        };
        PolyToolView.prototype._hide_vertices = function () {
            this._set_vertices([], []);
        };
        PolyToolView.prototype._snap_to_vertex = function (ev, x, y) {
            if (this.model.vertex_renderer) {
                // If an existing vertex is hit snap to it
                var vertex_selected = this._select_event(ev, "replace", [this.model.vertex_renderer]);
                var point_ds = this.model.vertex_renderer.data_source;
                // Type once dataspecs are typed
                var point_glyph = this.model.vertex_renderer.glyph;
                var _a = tslib_1.__read([point_glyph.x.field, point_glyph.y.field], 2), pxkey = _a[0], pykey = _a[1];
                if (vertex_selected.length) {
                    var index = point_ds.selected.indices[0];
                    if (pxkey)
                        x = point_ds.data[pxkey][index];
                    if (pykey)
                        y = point_ds.data[pykey][index];
                    point_ds.selection_manager.clear();
                }
            }
            return [x, y];
        };
        return PolyToolView;
    }(edit_tool_1.EditToolView));
    exports.PolyToolView = PolyToolView;
    PolyToolView.__name__ = "PolyToolView";
    var PolyTool = /** @class */ (function (_super) {
        tslib_1.__extends(PolyTool, _super);
        function PolyTool(attrs) {
            return _super.call(this, attrs) || this;
        }
        PolyTool.init_PolyTool = function () {
            this.prototype.default_view = PolyToolView;
            this.define({
                vertex_renderer: [p.Instance],
            });
        };
        return PolyTool;
    }(edit_tool_1.EditTool));
    exports.PolyTool = PolyTool;
    PolyTool.__name__ = "PolyTool";
    PolyTool.init_PolyTool();
},
/* models/tools/edit/poly_edit_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var dom_1 = require(187) /* ../../../core/dom */;
    var types_1 = require(123) /* ../../../core/util/types */;
    var poly_tool_1 = require(476) /* ./poly_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var PolyEditToolView = /** @class */ (function (_super) {
        tslib_1.__extends(PolyEditToolView, _super);
        function PolyEditToolView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this._drawing = false;
            return _this;
        }
        PolyEditToolView.prototype._doubletap = function (ev) {
            if (!this.model.active)
                return;
            var point = this._map_drag(ev.sx, ev.sy, this.model.vertex_renderer);
            if (point == null)
                return;
            var _a = tslib_1.__read(point, 2), x = _a[0], y = _a[1];
            // Perform hit testing
            var vertex_selected = this._select_event(ev, "replace", [this.model.vertex_renderer]);
            var point_cds = this.model.vertex_renderer.data_source;
            // Type once dataspecs are typed
            var point_glyph = this.model.vertex_renderer.glyph;
            var _b = tslib_1.__read([point_glyph.x.field, point_glyph.y.field], 2), pxkey = _b[0], pykey = _b[1];
            if (vertex_selected.length && this._selected_renderer != null) {
                // Insert a new point after the selected vertex and enter draw mode
                var index = point_cds.selected.indices[0];
                if (this._drawing) {
                    this._drawing = false;
                    point_cds.selection_manager.clear();
                }
                else {
                    point_cds.selected.indices = [index + 1];
                    if (pxkey)
                        point_cds.get_array(pxkey).splice(index + 1, 0, x);
                    if (pykey)
                        point_cds.get_array(pykey).splice(index + 1, 0, y);
                    this._drawing = true;
                }
                point_cds.change.emit();
                this._emit_cds_changes(this._selected_renderer.data_source);
            }
            else {
                this._show_vertices(ev);
            }
        };
        PolyEditToolView.prototype._show_vertices = function (ev) {
            if (!this.model.active)
                return;
            var renderers = this._select_event(ev, "replace", this.model.renderers);
            if (!renderers.length) {
                this._set_vertices([], []);
                this._selected_renderer = null;
                this._drawing = false;
                return;
            }
            var renderer = renderers[0];
            var glyph = renderer.glyph;
            var cds = renderer.data_source;
            var index = cds.selected.indices[0];
            var _a = tslib_1.__read([glyph.xs.field, glyph.ys.field], 2), xkey = _a[0], ykey = _a[1];
            var xs;
            var ys;
            if (xkey) {
                xs = cds.data[xkey][index];
                if (!types_1.isArray(xs))
                    cds.data[xkey][index] = xs = Array.from(xs);
            }
            else {
                xs = glyph.xs.value;
            }
            if (ykey) {
                ys = cds.data[ykey][index];
                if (!types_1.isArray(ys))
                    cds.data[ykey][index] = ys = Array.from(ys);
            }
            else {
                ys = glyph.ys.value;
            }
            this._selected_renderer = renderer;
            this._set_vertices(xs, ys);
        };
        PolyEditToolView.prototype._move = function (ev) {
            var _a;
            if (this._drawing && this._selected_renderer != null) {
                var renderer = this.model.vertex_renderer;
                var cds = renderer.data_source;
                var glyph = renderer.glyph;
                var point = this._map_drag(ev.sx, ev.sy, renderer);
                if (point == null)
                    return;
                var _b = tslib_1.__read(point, 2), x = _b[0], y = _b[1];
                var indices = cds.selected.indices;
                _a = tslib_1.__read(this._snap_to_vertex(ev, x, y), 2), x = _a[0], y = _a[1];
                cds.selected.indices = indices;
                var _c = tslib_1.__read([glyph.x.field, glyph.y.field], 2), xkey = _c[0], ykey = _c[1];
                var index = indices[0];
                if (xkey)
                    cds.data[xkey][index] = x;
                if (ykey)
                    cds.data[ykey][index] = y;
                cds.change.emit();
                this._selected_renderer.data_source.change.emit();
            }
        };
        PolyEditToolView.prototype._tap = function (ev) {
            var _a;
            var renderer = this.model.vertex_renderer;
            var point = this._map_drag(ev.sx, ev.sy, renderer);
            if (point == null)
                return;
            else if (this._drawing && this._selected_renderer) {
                var _b = tslib_1.__read(point, 2), x = _b[0], y = _b[1];
                var cds = renderer.data_source;
                // Type once dataspecs are typed
                var glyph = renderer.glyph;
                var _c = tslib_1.__read([glyph.x.field, glyph.y.field], 2), xkey = _c[0], ykey = _c[1];
                var indices = cds.selected.indices;
                _a = tslib_1.__read(this._snap_to_vertex(ev, x, y), 2), x = _a[0], y = _a[1];
                var index = indices[0];
                cds.selected.indices = [index + 1];
                if (xkey) {
                    var xs = cds.get_array(xkey);
                    var nx = xs[index];
                    xs[index] = x;
                    xs.splice(index + 1, 0, nx);
                }
                if (ykey) {
                    var ys = cds.get_array(ykey);
                    var ny = ys[index];
                    ys[index] = y;
                    ys.splice(index + 1, 0, ny);
                }
                cds.change.emit();
                this._emit_cds_changes(this._selected_renderer.data_source, true, false, true);
                return;
            }
            var mode = this._select_mode(ev);
            this._select_event(ev, mode, [renderer]);
            this._select_event(ev, mode, this.model.renderers);
        };
        PolyEditToolView.prototype._remove_vertex = function () {
            if (!this._drawing || !this._selected_renderer)
                return;
            var renderer = this.model.vertex_renderer;
            var cds = renderer.data_source;
            // Type once dataspecs are typed
            var glyph = renderer.glyph;
            var index = cds.selected.indices[0];
            var _a = tslib_1.__read([glyph.x.field, glyph.y.field], 2), xkey = _a[0], ykey = _a[1];
            if (xkey)
                cds.get_array(xkey).splice(index, 1);
            if (ykey)
                cds.get_array(ykey).splice(index, 1);
            cds.change.emit();
            this._emit_cds_changes(this._selected_renderer.data_source);
        };
        PolyEditToolView.prototype._pan_start = function (ev) {
            this._select_event(ev, "append", [this.model.vertex_renderer]);
            this._basepoint = [ev.sx, ev.sy];
        };
        PolyEditToolView.prototype._pan = function (ev) {
            if (this._basepoint == null)
                return;
            this._drag_points(ev, [this.model.vertex_renderer]);
            if (this._selected_renderer)
                this._selected_renderer.data_source.change.emit();
        };
        PolyEditToolView.prototype._pan_end = function (ev) {
            if (this._basepoint == null)
                return;
            this._drag_points(ev, [this.model.vertex_renderer]);
            this._emit_cds_changes(this.model.vertex_renderer.data_source, false, true, true);
            if (this._selected_renderer) {
                this._emit_cds_changes(this._selected_renderer.data_source);
            }
            this._basepoint = null;
        };
        PolyEditToolView.prototype._keyup = function (ev) {
            var e_1, _a;
            if (!this.model.active || !this._mouse_in_frame)
                return;
            var renderers;
            if (this._selected_renderer) {
                renderers = [this.model.vertex_renderer];
            }
            else {
                renderers = this.model.renderers;
            }
            try {
                for (var renderers_1 = tslib_1.__values(renderers), renderers_1_1 = renderers_1.next(); !renderers_1_1.done; renderers_1_1 = renderers_1.next()) {
                    var renderer = renderers_1_1.value;
                    if (ev.keyCode === dom_1.Keys.Backspace) {
                        this._delete_selected(renderer);
                        if (this._selected_renderer) {
                            this._emit_cds_changes(this._selected_renderer.data_source);
                        }
                    }
                    else if (ev.keyCode == dom_1.Keys.Esc) {
                        if (this._drawing) {
                            this._remove_vertex();
                            this._drawing = false;
                        }
                        else if (this._selected_renderer) {
                            this._hide_vertices();
                        }
                        renderer.data_source.selection_manager.clear();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (renderers_1_1 && !renderers_1_1.done && (_a = renderers_1.return))
                        _a.call(renderers_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        PolyEditToolView.prototype.deactivate = function () {
            if (!this._selected_renderer) {
                return;
            }
            else if (this._drawing) {
                this._remove_vertex();
                this._drawing = false;
            }
            this._hide_vertices();
        };
        return PolyEditToolView;
    }(poly_tool_1.PolyToolView));
    exports.PolyEditToolView = PolyEditToolView;
    PolyEditToolView.__name__ = "PolyEditToolView";
    var PolyEditTool = /** @class */ (function (_super) {
        tslib_1.__extends(PolyEditTool, _super);
        function PolyEditTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Poly Edit Tool";
            _this.icon = icons_1.bk_tool_icon_poly_edit;
            _this.event_type = ["tap", "pan", "move"];
            _this.default_order = 4;
            return _this;
        }
        PolyEditTool.init_PolyEditTool = function () {
            this.prototype.default_view = PolyEditToolView;
        };
        return PolyEditTool;
    }(poly_tool_1.PolyTool));
    exports.PolyEditTool = PolyEditTool;
    PolyEditTool.__name__ = "PolyEditTool";
    PolyEditTool.init_PolyEditTool();
},
/* models/tools/gestures/box_select_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var select_tool_1 = require(479) /* ./select_tool */;
    var box_annotation_1 = require(239) /* ../../annotations/box_annotation */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var icons_1 = require(424) /* ../../../styles/icons */;
    var BoxSelectToolView = /** @class */ (function (_super) {
        tslib_1.__extends(BoxSelectToolView, _super);
        function BoxSelectToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BoxSelectToolView.prototype._compute_limits = function (curpoint) {
            var frame = this.plot_view.frame;
            var dims = this.model.dimensions;
            var base_point = this._base_point;
            if (this.model.origin == "center") {
                var _a = tslib_1.__read(base_point, 2), cx = _a[0], cy = _a[1];
                var _b = tslib_1.__read(curpoint, 2), dx = _b[0], dy = _b[1];
                base_point = [cx - (dx - cx), cy - (dy - cy)];
            }
            return this.model._get_dim_limits(base_point, curpoint, frame, dims);
        };
        BoxSelectToolView.prototype._pan_start = function (ev) {
            var sx = ev.sx, sy = ev.sy;
            this._base_point = [sx, sy];
        };
        BoxSelectToolView.prototype._pan = function (ev) {
            var sx = ev.sx, sy = ev.sy;
            var curpoint = [sx, sy];
            var _a = tslib_1.__read(this._compute_limits(curpoint), 2), sxlim = _a[0], sylim = _a[1];
            this.model.overlay.update({ left: sxlim[0], right: sxlim[1], top: sylim[0], bottom: sylim[1] });
            if (this.model.select_every_mousemove) {
                this._do_select(sxlim, sylim, false, this._select_mode(ev));
            }
        };
        BoxSelectToolView.prototype._pan_end = function (ev) {
            var sx = ev.sx, sy = ev.sy;
            var curpoint = [sx, sy];
            var _a = tslib_1.__read(this._compute_limits(curpoint), 2), sxlim = _a[0], sylim = _a[1];
            this._do_select(sxlim, sylim, true, this._select_mode(ev));
            this.model.overlay.update({ left: null, right: null, top: null, bottom: null });
            this._base_point = null;
            this.plot_view.push_state('box_select', { selection: this.plot_view.get_selection() });
        };
        BoxSelectToolView.prototype._do_select = function (_a, _b, final, mode) {
            var _c = tslib_1.__read(_a, 2), sx0 = _c[0], sx1 = _c[1];
            var _d = tslib_1.__read(_b, 2), sy0 = _d[0], sy1 = _d[1];
            if (mode === void 0) {
                mode = "replace";
            }
            var geometry = { type: 'rect', sx0: sx0, sx1: sx1, sy0: sy0, sy1: sy1 };
            this._select(geometry, final, mode);
        };
        return BoxSelectToolView;
    }(select_tool_1.SelectToolView));
    exports.BoxSelectToolView = BoxSelectToolView;
    BoxSelectToolView.__name__ = "BoxSelectToolView";
    var DEFAULT_BOX_OVERLAY = function () {
        return new box_annotation_1.BoxAnnotation({
            level: "overlay",
            top_units: "screen",
            left_units: "screen",
            bottom_units: "screen",
            right_units: "screen",
            fill_color: "lightgrey",
            fill_alpha: 0.5,
            line_color: "black",
            line_alpha: 1.0,
            line_width: 2,
            line_dash: [4, 4],
        });
    };
    var BoxSelectTool = /** @class */ (function (_super) {
        tslib_1.__extends(BoxSelectTool, _super);
        function BoxSelectTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Box Select";
            _this.icon = icons_1.bk_tool_icon_box_select;
            _this.event_type = "pan";
            _this.default_order = 30;
            return _this;
        }
        BoxSelectTool.init_BoxSelectTool = function () {
            this.prototype.default_view = BoxSelectToolView;
            this.define({
                dimensions: [p.Dimensions, "both"],
                select_every_mousemove: [p.Boolean, false],
                overlay: [p.Instance, DEFAULT_BOX_OVERLAY],
                origin: [p.BoxOrigin, "corner"],
            });
            this.register_alias("box_select", function () { return new BoxSelectTool(); });
            this.register_alias("xbox_select", function () { return new BoxSelectTool({ dimensions: 'width' }); });
            this.register_alias("ybox_select", function () { return new BoxSelectTool({ dimensions: 'height' }); });
        };
        Object.defineProperty(BoxSelectTool.prototype, "tooltip", {
            get: function () {
                return this._get_dim_tooltip(this.tool_name, this.dimensions);
            },
            enumerable: true,
            configurable: true
        });
        return BoxSelectTool;
    }(select_tool_1.SelectTool));
    exports.BoxSelectTool = BoxSelectTool;
    BoxSelectTool.__name__ = "BoxSelectTool";
    BoxSelectTool.init_BoxSelectTool();
},
/* models/tools/gestures/select_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var gesture_tool_1 = require(421) /* ./gesture_tool */;
    var glyph_renderer_1 = require(205) /* ../../renderers/glyph_renderer */;
    var graph_renderer_1 = require(231) /* ../../renderers/graph_renderer */;
    var util_1 = require(480) /* ../util */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var dom_1 = require(187) /* ../../../core/dom */;
    var bokeh_events_1 = require(428) /* ../../../core/bokeh_events */;
    var signaling_1 = require(130) /* ../../../core/signaling */;
    var assert_1 = require(126) /* ../../../core/util/assert */;
    var SelectToolView = /** @class */ (function (_super) {
        tslib_1.__extends(SelectToolView, _super);
        function SelectToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        SelectToolView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.model.clear.connect(function () { return _this._clear(); });
        };
        Object.defineProperty(SelectToolView.prototype, "computed_renderers", {
            get: function () {
                var renderers = this.model.renderers;
                var all_renderers = this.plot_model.renderers;
                var names = this.model.names;
                return util_1.compute_renderers(renderers, all_renderers, names);
            },
            enumerable: true,
            configurable: true
        });
        SelectToolView.prototype._computed_renderers_by_data_source = function () {
            var e_1, _b;
            var _a;
            var renderers_by_source = new Map();
            try {
                for (var _c = tslib_1.__values(this.computed_renderers), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var r = _d.value;
                    var source = void 0;
                    if (r instanceof glyph_renderer_1.GlyphRenderer)
                        source = r.data_source;
                    else if (r instanceof graph_renderer_1.GraphRenderer)
                        source = r.node_renderer.data_source;
                    else
                        continue;
                    var renderers = (_a = renderers_by_source.get(source)) !== null && _a !== void 0 ? _a : [];
                    renderers_by_source.set(source, tslib_1.__spread(renderers, [r]));
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            return renderers_by_source;
        };
        SelectToolView.prototype._select_mode = function (ev) {
            var shiftKey = ev.shiftKey, ctrlKey = ev.ctrlKey;
            if (!shiftKey && !ctrlKey)
                return this.model.mode;
            else if (shiftKey && !ctrlKey)
                return "append";
            else if (!shiftKey && ctrlKey)
                return "intersect";
            else if (shiftKey && ctrlKey)
                return "subtract";
            else
                assert_1.unreachable();
        };
        SelectToolView.prototype._keyup = function (ev) {
            if (ev.keyCode == dom_1.Keys.Esc) {
                this._clear();
            }
        };
        SelectToolView.prototype._clear = function () {
            var e_2, _b;
            try {
                for (var _c = tslib_1.__values(this.computed_renderers), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var renderer = _d.value;
                    renderer.get_selection_manager().clear();
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_b = _c.return))
                        _b.call(_c);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            this.plot_view.request_render();
        };
        SelectToolView.prototype._select = function (geometry, final, mode) {
            var e_3, _b, e_4, _c;
            var renderers_by_source = this._computed_renderers_by_data_source();
            try {
                for (var renderers_by_source_1 = tslib_1.__values(renderers_by_source), renderers_by_source_1_1 = renderers_by_source_1.next(); !renderers_by_source_1_1.done; renderers_by_source_1_1 = renderers_by_source_1.next()) {
                    var _d = tslib_1.__read(renderers_by_source_1_1.value, 2), renderers = _d[1];
                    var sm = renderers[0].get_selection_manager();
                    var r_views = [];
                    try {
                        for (var renderers_1 = (e_4 = void 0, tslib_1.__values(renderers)), renderers_1_1 = renderers_1.next(); !renderers_1_1.done; renderers_1_1 = renderers_1.next()) {
                            var r = renderers_1_1.value;
                            var r_view = this.plot_view.renderer_views.get(r);
                            if (r_view != null) {
                                r_views.push(r_view);
                            }
                        }
                    }
                    catch (e_4_1) {
                        e_4 = { error: e_4_1 };
                    }
                    finally {
                        try {
                            if (renderers_1_1 && !renderers_1_1.done && (_c = renderers_1.return))
                                _c.call(renderers_1);
                        }
                        finally {
                            if (e_4)
                                throw e_4.error;
                        }
                    }
                    sm.select(r_views, geometry, final, mode);
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (renderers_by_source_1_1 && !renderers_by_source_1_1.done && (_b = renderers_by_source_1.return))
                        _b.call(renderers_by_source_1);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
            // XXX: messed up class structure
            if (this.model.callback != null)
                this._emit_callback(geometry);
            this._emit_selection_event(geometry, final);
        };
        SelectToolView.prototype._emit_selection_event = function (geometry, final) {
            if (final === void 0) {
                final = true;
            }
            var _b = this.plot_view.frame, x_scale = _b.x_scale, y_scale = _b.y_scale;
            var geometry_data;
            switch (geometry.type) {
                case "point": {
                    var sx = geometry.sx, sy = geometry.sy;
                    var x = x_scale.invert(sx);
                    var y = y_scale.invert(sy);
                    geometry_data = Object.assign(Object.assign({}, geometry), { x: x, y: y });
                    break;
                }
                case "span": {
                    var sx = geometry.sx, sy = geometry.sy;
                    var x = x_scale.invert(sx);
                    var y = y_scale.invert(sy);
                    geometry_data = Object.assign(Object.assign({}, geometry), { x: x, y: y });
                    break;
                }
                case "rect": {
                    var sx0 = geometry.sx0, sx1 = geometry.sx1, sy0 = geometry.sy0, sy1 = geometry.sy1;
                    var _c = tslib_1.__read(x_scale.r_invert(sx0, sx1), 2), x0 = _c[0], x1 = _c[1];
                    var _d = tslib_1.__read(y_scale.r_invert(sy0, sy1), 2), y0 = _d[0], y1 = _d[1];
                    geometry_data = Object.assign(Object.assign({}, geometry), { x0: x0, y0: y0, x1: x1, y1: y1 });
                    break;
                }
                case "poly": {
                    var sx = geometry.sx, sy = geometry.sy;
                    var x = x_scale.v_invert(sx);
                    var y = y_scale.v_invert(sy);
                    geometry_data = Object.assign(Object.assign({}, geometry), { x: x, y: y });
                    break;
                }
            }
            this.plot_model.trigger_event(new bokeh_events_1.SelectionGeometry(geometry_data, final));
        };
        return SelectToolView;
    }(gesture_tool_1.GestureToolView));
    exports.SelectToolView = SelectToolView;
    SelectToolView.__name__ = "SelectToolView";
    var SelectTool = /** @class */ (function (_super) {
        tslib_1.__extends(SelectTool, _super);
        function SelectTool(attrs) {
            return _super.call(this, attrs) || this;
        }
        SelectTool.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.clear = new signaling_1.Signal0(this, "clear");
        };
        SelectTool.init_SelectTool = function () {
            this.define({
                renderers: [p.Any, 'auto'],
                names: [p.Array, []],
                mode: [p.Any, "replace"],
            });
        };
        Object.defineProperty(SelectTool.prototype, "menu", {
            get: function () {
                var _this = this;
                return [
                    {
                        icon: "bk-tool-icon-replace-mode",
                        tooltip: "Replace the current selection",
                        active: function () { return _this.mode == "replace"; },
                        handler: function () {
                            _this.mode = "replace";
                            _this.active = true;
                        },
                    }, {
                        icon: "bk-tool-icon-append-mode",
                        tooltip: "Append to the current selection (Shift)",
                        active: function () { return _this.mode == "append"; },
                        handler: function () {
                            _this.mode = "append";
                            _this.active = true;
                        },
                    }, {
                        icon: "bk-tool-icon-intersect-mode",
                        tooltip: "Intersect with the current selection (Ctrl)",
                        active: function () { return _this.mode == "intersect"; },
                        handler: function () {
                            _this.mode = "intersect";
                            _this.active = true;
                        },
                    }, {
                        icon: "bk-tool-icon-subtract-mode",
                        tooltip: "Subtract from the current selection (Shift+Ctrl)",
                        active: function () { return _this.mode == "subtract"; },
                        handler: function () {
                            _this.mode = "subtract";
                            _this.active = true;
                        },
                    },
                    null,
                    {
                        icon: "bk-tool-icon-clear-selection",
                        tooltip: "Clear the current selection (Esc)",
                        handler: function () {
                            _this.clear.emit();
                        },
                    },
                ];
            },
            enumerable: true,
            configurable: true
        });
        return SelectTool;
    }(gesture_tool_1.GestureTool));
    exports.SelectTool = SelectTool;
    SelectTool.__name__ = "SelectTool";
    SelectTool.init_SelectTool();
},
/* models/tools/util.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var array_1 = require(124) /* ../../core/util/array */;
    function compute_renderers(renderers, all_renderers, names) {
        if (renderers == null)
            return [];
        var result = renderers == 'auto' ? all_renderers : renderers;
        if (names.length > 0)
            result = result.filter(function (r) { return array_1.includes(names, r.name); });
        return result;
    }
    exports.compute_renderers = compute_renderers;
},
/* models/tools/gestures/box_zoom_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var gesture_tool_1 = require(421) /* ./gesture_tool */;
    var box_annotation_1 = require(239) /* ../../annotations/box_annotation */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var icons_1 = require(424) /* ../../../styles/icons */;
    var BoxZoomToolView = /** @class */ (function (_super) {
        tslib_1.__extends(BoxZoomToolView, _super);
        function BoxZoomToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        BoxZoomToolView.prototype._match_aspect = function (base_point, curpoint, frame) {
            // aspect ratio of plot frame
            var a = frame.bbox.aspect;
            var hend = frame.bbox.h_range.end;
            var hstart = frame.bbox.h_range.start;
            var vend = frame.bbox.v_range.end;
            var vstart = frame.bbox.v_range.start;
            // current aspect of cursor-defined box
            var vw = Math.abs(base_point[0] - curpoint[0]);
            var vh = Math.abs(base_point[1] - curpoint[1]);
            var va = vh == 0 ? 0 : vw / vh;
            var _a = tslib_1.__read(va >= a ? [1, va / a] : [a / va, 1], 1), xmod = _a[0];
            // OK the code blocks below merit some explanation. They do:
            //
            // compute left/right, pin to frame if necessary
            // compute top/bottom (based on new left/right), pin to frame if necessary
            // recompute left/right (based on top/bottom), in case top/bottom were pinned
            // base_point[0] is left
            var left;
            var right;
            if (base_point[0] <= curpoint[0]) {
                left = base_point[0];
                right = base_point[0] + vw * xmod;
                if (right > hend)
                    right = hend;
                // base_point[0] is right
            }
            else {
                right = base_point[0];
                left = base_point[0] - vw * xmod;
                if (left < hstart)
                    left = hstart;
            }
            vw = Math.abs(right - left);
            // base_point[1] is bottom
            var top;
            var bottom;
            if (base_point[1] <= curpoint[1]) {
                bottom = base_point[1];
                top = base_point[1] + vw / a;
                if (top > vend)
                    top = vend;
                // base_point[1] is top
            }
            else {
                top = base_point[1];
                bottom = base_point[1] - vw / a;
                if (bottom < vstart)
                    bottom = vstart;
            }
            vh = Math.abs(top - bottom);
            // base_point[0] is left
            if (base_point[0] <= curpoint[0])
                right = base_point[0] + a * vh;
            // base_point[0] is right
            else
                left = base_point[0] - a * vh;
            return [[left, right], [bottom, top]];
        };
        BoxZoomToolView.prototype._compute_limits = function (curpoint) {
            var _a, _b;
            var frame = this.plot_view.frame;
            var dims = this.model.dimensions;
            var base_point = this._base_point;
            if (this.model.origin == "center") {
                var _c = tslib_1.__read(base_point, 2), cx = _c[0], cy = _c[1];
                var _d = tslib_1.__read(curpoint, 2), dx = _d[0], dy = _d[1];
                base_point = [cx - (dx - cx), cy - (dy - cy)];
            }
            var sx;
            var sy;
            if (this.model.match_aspect && dims == 'both')
                _a = tslib_1.__read(this._match_aspect(base_point, curpoint, frame), 2), sx = _a[0], sy = _a[1];
            else
                _b = tslib_1.__read(this.model._get_dim_limits(base_point, curpoint, frame, dims), 2), sx = _b[0], sy = _b[1];
            return [sx, sy];
        };
        BoxZoomToolView.prototype._pan_start = function (ev) {
            this._base_point = [ev.sx, ev.sy];
        };
        BoxZoomToolView.prototype._pan = function (ev) {
            var curpoint = [ev.sx, ev.sy];
            var _a = tslib_1.__read(this._compute_limits(curpoint), 2), sx = _a[0], sy = _a[1];
            this.model.overlay.update({ left: sx[0], right: sx[1], top: sy[0], bottom: sy[1] });
        };
        BoxZoomToolView.prototype._pan_end = function (ev) {
            var curpoint = [ev.sx, ev.sy];
            var _a = tslib_1.__read(this._compute_limits(curpoint), 2), sx = _a[0], sy = _a[1];
            this._update(sx, sy);
            this.model.overlay.update({ left: null, right: null, top: null, bottom: null });
            this._base_point = null;
        };
        BoxZoomToolView.prototype._update = function (_a, _b) {
            var e_1, _c, e_2, _d;
            var _e = tslib_1.__read(_a, 2), sx0 = _e[0], sx1 = _e[1];
            var _f = tslib_1.__read(_b, 2), sy0 = _f[0], sy1 = _f[1];
            // If the viewing window is too small, no-op: it is likely that the user did
            // not intend to make this box zoom and instead was trying to cancel out of the
            // zoom, a la matplotlib's ToolZoom. Like matplotlib, set the threshold at 5 pixels.
            if (Math.abs(sx1 - sx0) <= 5 || Math.abs(sy1 - sy0) <= 5)
                return;
            var _g = this.plot_view.frame, x_scales = _g.x_scales, y_scales = _g.y_scales;
            var xrs = new Map();
            try {
                for (var x_scales_1 = tslib_1.__values(x_scales), x_scales_1_1 = x_scales_1.next(); !x_scales_1_1.done; x_scales_1_1 = x_scales_1.next()) {
                    var _h = tslib_1.__read(x_scales_1_1.value, 2), name = _h[0], scale = _h[1];
                    var _j = tslib_1.__read(scale.r_invert(sx0, sx1), 2), start = _j[0], end = _j[1];
                    xrs.set(name, { start: start, end: end });
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (x_scales_1_1 && !x_scales_1_1.done && (_c = x_scales_1.return))
                        _c.call(x_scales_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            var yrs = new Map();
            try {
                for (var y_scales_1 = tslib_1.__values(y_scales), y_scales_1_1 = y_scales_1.next(); !y_scales_1_1.done; y_scales_1_1 = y_scales_1.next()) {
                    var _k = tslib_1.__read(y_scales_1_1.value, 2), name = _k[0], scale = _k[1];
                    var _l = tslib_1.__read(scale.r_invert(sy0, sy1), 2), start = _l[0], end = _l[1];
                    yrs.set(name, { start: start, end: end });
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (y_scales_1_1 && !y_scales_1_1.done && (_d = y_scales_1.return))
                        _d.call(y_scales_1);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            var zoom_info = { xrs: xrs, yrs: yrs };
            this.plot_view.push_state('box_zoom', { range: zoom_info });
            this.plot_view.update_range(zoom_info);
        };
        return BoxZoomToolView;
    }(gesture_tool_1.GestureToolView));
    exports.BoxZoomToolView = BoxZoomToolView;
    BoxZoomToolView.__name__ = "BoxZoomToolView";
    var DEFAULT_BOX_OVERLAY = function () {
        return new box_annotation_1.BoxAnnotation({
            level: "overlay",
            top_units: "screen",
            left_units: "screen",
            bottom_units: "screen",
            right_units: "screen",
            fill_color: "lightgrey",
            fill_alpha: 0.5,
            line_color: "black",
            line_alpha: 1.0,
            line_width: 2,
            line_dash: [4, 4],
        });
    };
    var BoxZoomTool = /** @class */ (function (_super) {
        tslib_1.__extends(BoxZoomTool, _super);
        function BoxZoomTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Box Zoom";
            _this.icon = icons_1.bk_tool_icon_box_zoom;
            _this.event_type = "pan";
            _this.default_order = 20;
            return _this;
        }
        BoxZoomTool.init_BoxZoomTool = function () {
            this.prototype.default_view = BoxZoomToolView;
            this.define({
                dimensions: [p.Dimensions, "both"],
                overlay: [p.Instance, DEFAULT_BOX_OVERLAY],
                match_aspect: [p.Boolean, false],
                origin: [p.BoxOrigin, "corner"],
            });
            this.register_alias("box_zoom", function () { return new BoxZoomTool({ dimensions: 'both' }); });
            this.register_alias("xbox_zoom", function () { return new BoxZoomTool({ dimensions: 'width' }); });
            this.register_alias("ybox_zoom", function () { return new BoxZoomTool({ dimensions: 'height' }); });
        };
        Object.defineProperty(BoxZoomTool.prototype, "tooltip", {
            get: function () {
                return this._get_dim_tooltip(this.tool_name, this.dimensions);
            },
            enumerable: true,
            configurable: true
        });
        return BoxZoomTool;
    }(gesture_tool_1.GestureTool));
    exports.BoxZoomTool = BoxZoomTool;
    BoxZoomTool.__name__ = "BoxZoomTool";
    BoxZoomTool.init_BoxZoomTool();
},
/* models/tools/gestures/lasso_select_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var select_tool_1 = require(479) /* ./select_tool */;
    var poly_select_tool_1 = require(483) /* ./poly_select_tool */;
    var dom_1 = require(187) /* ../../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var icons_1 = require(424) /* ../../../styles/icons */;
    var LassoSelectToolView = /** @class */ (function (_super) {
        tslib_1.__extends(LassoSelectToolView, _super);
        function LassoSelectToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        LassoSelectToolView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.data = null;
        };
        LassoSelectToolView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.properties.active.change, function () { return _this._active_change(); });
        };
        LassoSelectToolView.prototype._active_change = function () {
            if (!this.model.active)
                this._clear_overlay();
        };
        LassoSelectToolView.prototype._keyup = function (ev) {
            if (ev.keyCode == dom_1.Keys.Enter)
                this._clear_overlay();
        };
        LassoSelectToolView.prototype._pan_start = function (ev) {
            var sx = ev.sx, sy = ev.sy;
            this.data = { sx: [sx], sy: [sy] };
        };
        LassoSelectToolView.prototype._pan = function (ev) {
            var _sx = ev.sx, _sy = ev.sy;
            var _a = tslib_1.__read(this.plot_view.frame.bbox.clip(_sx, _sy), 2), sx = _a[0], sy = _a[1];
            this.data.sx.push(sx);
            this.data.sy.push(sy);
            var overlay = this.model.overlay;
            overlay.update({ xs: this.data.sx, ys: this.data.sy });
            if (this.model.select_every_mousemove) {
                this._do_select(this.data.sx, this.data.sy, false, this._select_mode(ev));
            }
        };
        LassoSelectToolView.prototype._pan_end = function (ev) {
            this._clear_overlay();
            this._do_select(this.data.sx, this.data.sy, true, this._select_mode(ev));
            this.plot_view.push_state('lasso_select', { selection: this.plot_view.get_selection() });
        };
        LassoSelectToolView.prototype._clear_overlay = function () {
            this.model.overlay.update({ xs: [], ys: [] });
        };
        LassoSelectToolView.prototype._do_select = function (sx, sy, final, mode) {
            var geometry = { type: 'poly', sx: sx, sy: sy };
            this._select(geometry, final, mode);
        };
        return LassoSelectToolView;
    }(select_tool_1.SelectToolView));
    exports.LassoSelectToolView = LassoSelectToolView;
    LassoSelectToolView.__name__ = "LassoSelectToolView";
    var LassoSelectTool = /** @class */ (function (_super) {
        tslib_1.__extends(LassoSelectTool, _super);
        function LassoSelectTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Lasso Select";
            _this.icon = icons_1.bk_tool_icon_lasso_select;
            _this.event_type = "pan";
            _this.default_order = 12;
            return _this;
        }
        LassoSelectTool.init_LassoSelectTool = function () {
            this.prototype.default_view = LassoSelectToolView;
            this.define({
                select_every_mousemove: [p.Boolean, true],
                overlay: [p.Instance, poly_select_tool_1.DEFAULT_POLY_OVERLAY],
            });
            this.register_alias("lasso_select", function () { return new LassoSelectTool(); });
        };
        return LassoSelectTool;
    }(select_tool_1.SelectTool));
    exports.LassoSelectTool = LassoSelectTool;
    LassoSelectTool.__name__ = "LassoSelectTool";
    LassoSelectTool.init_LassoSelectTool();
},
/* models/tools/gestures/poly_select_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var select_tool_1 = require(479) /* ./select_tool */;
    var poly_annotation_1 = require(281) /* ../../annotations/poly_annotation */;
    var dom_1 = require(187) /* ../../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var array_1 = require(124) /* ../../../core/util/array */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var PolySelectToolView = /** @class */ (function (_super) {
        tslib_1.__extends(PolySelectToolView, _super);
        function PolySelectToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        PolySelectToolView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.data = { sx: [], sy: [] };
        };
        PolySelectToolView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.model.properties.active.change, function () { return _this._active_change(); });
        };
        PolySelectToolView.prototype._active_change = function () {
            if (!this.model.active)
                this._clear_data();
        };
        PolySelectToolView.prototype._keyup = function (ev) {
            if (ev.keyCode == dom_1.Keys.Enter)
                this._clear_data();
        };
        PolySelectToolView.prototype._doubletap = function (ev) {
            this._do_select(this.data.sx, this.data.sy, true, this._select_mode(ev));
            this.plot_view.push_state('poly_select', { selection: this.plot_view.get_selection() });
            this._clear_data();
        };
        PolySelectToolView.prototype._clear_data = function () {
            this.data = { sx: [], sy: [] };
            this.model.overlay.update({ xs: [], ys: [] });
        };
        PolySelectToolView.prototype._tap = function (ev) {
            var sx = ev.sx, sy = ev.sy;
            var frame = this.plot_view.frame;
            if (!frame.bbox.contains(sx, sy))
                return;
            this.data.sx.push(sx);
            this.data.sy.push(sy);
            this.model.overlay.update({ xs: array_1.copy(this.data.sx), ys: array_1.copy(this.data.sy) });
        };
        PolySelectToolView.prototype._do_select = function (sx, sy, final, mode) {
            var geometry = { type: 'poly', sx: sx, sy: sy };
            this._select(geometry, final, mode);
        };
        return PolySelectToolView;
    }(select_tool_1.SelectToolView));
    exports.PolySelectToolView = PolySelectToolView;
    PolySelectToolView.__name__ = "PolySelectToolView";
    exports.DEFAULT_POLY_OVERLAY = function () {
        return new poly_annotation_1.PolyAnnotation({
            level: "overlay",
            xs_units: "screen",
            ys_units: "screen",
            fill_color: "lightgrey",
            fill_alpha: 0.5,
            line_color: "black",
            line_alpha: 1.0,
            line_width: 2,
            line_dash: [4, 4],
        });
    };
    var PolySelectTool = /** @class */ (function (_super) {
        tslib_1.__extends(PolySelectTool, _super);
        function PolySelectTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Poly Select";
            _this.icon = icons_1.bk_tool_icon_polygon_select;
            _this.event_type = "tap";
            _this.default_order = 11;
            return _this;
        }
        PolySelectTool.init_PolySelectTool = function () {
            this.prototype.default_view = PolySelectToolView;
            this.define({
                overlay: [p.Instance, exports.DEFAULT_POLY_OVERLAY],
            });
            this.register_alias("poly_select", function () { return new PolySelectTool(); });
        };
        return PolySelectTool;
    }(select_tool_1.SelectTool));
    exports.PolySelectTool = PolySelectTool;
    PolySelectTool.__name__ = "PolySelectTool";
    PolySelectTool.init_PolySelectTool();
},
/* models/tools/edit/line_edit_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var line_tool_1 = require(485) /* ./line_tool */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var icons_1 = require(424) /* ../../../styles/icons */;
    var LineEditToolView = /** @class */ (function (_super) {
        tslib_1.__extends(LineEditToolView, _super);
        function LineEditToolView() {
            var _this = _super.apply(this, tslib_1.__spread(arguments)) || this;
            _this._drawing = false;
            return _this;
        }
        LineEditToolView.prototype._doubletap = function (ev) {
            var e_1, _a;
            if (!this.model.active)
                return;
            var renderers = this.model.renderers;
            try {
                for (var renderers_1 = tslib_1.__values(renderers), renderers_1_1 = renderers_1.next(); !renderers_1_1.done; renderers_1_1 = renderers_1.next()) {
                    var renderer = renderers_1_1.value;
                    var line_selected = this._select_event(ev, "replace", [renderer]);
                    if (line_selected.length == 1) {
                        this._selected_renderer = renderer;
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (renderers_1_1 && !renderers_1_1.done && (_a = renderers_1.return))
                        _a.call(renderers_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            this._show_intersections();
            this._update_line_cds();
        };
        LineEditToolView.prototype._show_intersections = function () {
            if (!this.model.active)
                return;
            if (this._selected_renderer == null)
                return;
            var renderers = this.model.renderers;
            if (!renderers.length) {
                this._set_intersection([], []);
                this._selected_renderer = null;
                this._drawing = false;
                return;
            }
            var cds = this._selected_renderer.data_source;
            var glyph = this._selected_renderer.glyph;
            var _a = tslib_1.__read([glyph.x.field, glyph.y.field], 2), xkey = _a[0], ykey = _a[1];
            var x = cds.get_array(xkey);
            var y = cds.get_array(ykey);
            this._set_intersection(x, y);
        };
        LineEditToolView.prototype._tap = function (ev) {
            var renderer = this.model.intersection_renderer;
            var point = this._map_drag(ev.sx, ev.sy, renderer);
            if (point == null)
                return;
            else if (this._drawing && this._selected_renderer) {
                var mode_1 = this._select_mode(ev);
                var selected_points = this._select_event(ev, mode_1, [renderer]);
                if (selected_points.length == 0) {
                    return;
                }
            }
            var mode = this._select_mode(ev);
            this._select_event(ev, mode, [renderer]);
            this._select_event(ev, mode, this.model.renderers);
        };
        LineEditToolView.prototype._update_line_cds = function () {
            if (this._selected_renderer == null)
                return;
            var point_glyph = this.model.intersection_renderer.glyph;
            var point_cds = this.model.intersection_renderer.data_source;
            var _a = tslib_1.__read([point_glyph.x.field, point_glyph.y.field], 2), pxkey = _a[0], pykey = _a[1];
            if (pxkey && pykey) {
                var x = point_cds.data[pxkey];
                var y = point_cds.data[pykey];
                this._selected_renderer.data_source.data[pxkey] = x;
                this._selected_renderer.data_source.data[pykey] = y;
            }
            this._emit_cds_changes(this._selected_renderer.data_source, true, true, false);
        };
        LineEditToolView.prototype._pan_start = function (ev) {
            this._select_event(ev, "append", [this.model.intersection_renderer]);
            this._basepoint = [ev.sx, ev.sy];
        };
        LineEditToolView.prototype._pan = function (ev) {
            if (this._basepoint == null)
                return;
            this._drag_points(ev, [this.model.intersection_renderer], this.model.dimensions);
            if (this._selected_renderer)
                this._selected_renderer.data_source.change.emit();
        };
        LineEditToolView.prototype._pan_end = function (ev) {
            if (this._basepoint == null)
                return;
            this._drag_points(ev, [this.model.intersection_renderer]);
            this._emit_cds_changes(this.model.intersection_renderer.data_source, false, true, true);
            if (this._selected_renderer) {
                this._emit_cds_changes(this._selected_renderer.data_source);
            }
            this._basepoint = null;
        };
        LineEditToolView.prototype.activate = function () {
            this._drawing = true;
        };
        LineEditToolView.prototype.deactivate = function () {
            if (!this._selected_renderer) {
                return;
            }
            else if (this._drawing) {
                this._drawing = false;
            }
            this._hide_intersections();
        };
        return LineEditToolView;
    }(line_tool_1.LineToolView));
    exports.LineEditToolView = LineEditToolView;
    LineEditToolView.__name__ = "LineEditToolView";
    var LineEditTool = /** @class */ (function (_super) {
        tslib_1.__extends(LineEditTool, _super);
        function LineEditTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Line Edit Tool";
            _this.icon = icons_1.bk_tool_icon_line_edit;
            _this.event_type = ["tap", "pan", "move"];
            _this.default_order = 4;
            return _this;
        }
        LineEditTool.init_LineEditTool = function () {
            this.prototype.default_view = LineEditToolView;
            this.define({
                dimensions: [p.Dimensions, "both"],
            });
        };
        Object.defineProperty(LineEditTool.prototype, "tooltip", {
            get: function () {
                return this._get_dim_tooltip(this.tool_name, this.dimensions);
            },
            enumerable: true,
            configurable: true
        });
        return LineEditTool;
    }(line_tool_1.LineTool));
    exports.LineEditTool = LineEditTool;
    LineEditTool.__name__ = "LineEditTool";
    LineEditTool.init_LineEditTool();
},
/* models/tools/edit/line_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var types_1 = require(123) /* ../../../core/util/types */;
    var edit_tool_1 = require(471) /* ./edit_tool */;
    var LineToolView = /** @class */ (function (_super) {
        tslib_1.__extends(LineToolView, _super);
        function LineToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        LineToolView.prototype._set_intersection = function (x, y) {
            var point_glyph = this.model.intersection_renderer.glyph;
            var point_cds = this.model.intersection_renderer.data_source;
            var _a = tslib_1.__read([point_glyph.x.field, point_glyph.y.field], 2), pxkey = _a[0], pykey = _a[1];
            if (pxkey) {
                if (types_1.isArray(x))
                    point_cds.data[pxkey] = x;
                else
                    point_glyph.x = { value: x };
            }
            if (pykey) {
                if (types_1.isArray(y))
                    point_cds.data[pykey] = y;
                else
                    point_glyph.y = { value: y };
            }
            this._emit_cds_changes(point_cds, true, true, false);
        };
        LineToolView.prototype._hide_intersections = function () {
            this._set_intersection([], []);
        };
        return LineToolView;
    }(edit_tool_1.EditToolView));
    exports.LineToolView = LineToolView;
    LineToolView.__name__ = "LineToolView";
    var LineTool = /** @class */ (function (_super) {
        tslib_1.__extends(LineTool, _super);
        function LineTool(attrs) {
            return _super.call(this, attrs) || this;
        }
        LineTool.init_LineTool = function () {
            this.prototype.default_view = LineToolView;
            this.define({
                intersection_renderer: [p.Instance],
            });
        };
        return LineTool;
    }(edit_tool_1.EditTool));
    exports.LineTool = LineTool;
    LineTool.__name__ = "LineTool";
    LineTool.init_LineTool();
},
/* models/tools/gestures/pan_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var gesture_tool_1 = require(421) /* ./gesture_tool */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var icons_1 = require(424) /* ../../../styles/icons */;
    function update_ranges(scales, p0, p1) {
        var e_1, _a;
        var r = new Map();
        try {
            for (var scales_1 = tslib_1.__values(scales), scales_1_1 = scales_1.next(); !scales_1_1.done; scales_1_1 = scales_1.next()) {
                var _b = tslib_1.__read(scales_1_1.value, 2), name = _b[0], scale = _b[1];
                var _c = tslib_1.__read(scale.r_invert(p0, p1), 2), start = _c[0], end = _c[1];
                r.set(name, { start: start, end: end });
            }
        }
        catch (e_1_1) {
            e_1 = { error: e_1_1 };
        }
        finally {
            try {
                if (scales_1_1 && !scales_1_1.done && (_a = scales_1.return))
                    _a.call(scales_1);
            }
            finally {
                if (e_1)
                    throw e_1.error;
            }
        }
        return r;
    }
    exports.update_ranges = update_ranges;
    var PanToolView = /** @class */ (function (_super) {
        tslib_1.__extends(PanToolView, _super);
        function PanToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        PanToolView.prototype._pan_start = function (ev) {
            this.last_dx = 0;
            this.last_dy = 0;
            var sx = ev.sx, sy = ev.sy;
            var bbox = this.plot_view.frame.bbox;
            if (!bbox.contains(sx, sy)) {
                var hr = bbox.h_range;
                var vr = bbox.v_range;
                if (sx < hr.start || sx > hr.end)
                    this.v_axis_only = true;
                if (sy < vr.start || sy > vr.end)
                    this.h_axis_only = true;
            }
            if (this.model.document != null)
                this.model.document.interactive_start(this.plot_model);
        };
        PanToolView.prototype._pan = function (ev) {
            this._update(ev.deltaX, ev.deltaY);
            if (this.model.document != null)
                this.model.document.interactive_start(this.plot_model);
        };
        PanToolView.prototype._pan_end = function (_e) {
            this.h_axis_only = false;
            this.v_axis_only = false;
            if (this.pan_info != null)
                this.plot_view.push_state('pan', { range: this.pan_info });
        };
        PanToolView.prototype._update = function (dx, dy) {
            var frame = this.plot_view.frame;
            var new_dx = dx - this.last_dx;
            var new_dy = dy - this.last_dy;
            var hr = frame.bbox.h_range;
            var sx_low = hr.start - new_dx;
            var sx_high = hr.end - new_dx;
            var vr = frame.bbox.v_range;
            var sy_low = vr.start - new_dy;
            var sy_high = vr.end - new_dy;
            var dims = this.model.dimensions;
            var sx0;
            var sx1;
            var sdx;
            if ((dims == 'width' || dims == 'both') && !this.v_axis_only) {
                sx0 = sx_low;
                sx1 = sx_high;
                sdx = -new_dx;
            }
            else {
                sx0 = hr.start;
                sx1 = hr.end;
                sdx = 0;
            }
            var sy0;
            var sy1;
            var sdy;
            if ((dims == 'height' || dims == 'both') && !this.h_axis_only) {
                sy0 = sy_low;
                sy1 = sy_high;
                sdy = -new_dy;
            }
            else {
                sy0 = vr.start;
                sy1 = vr.end;
                sdy = 0;
            }
            this.last_dx = dx;
            this.last_dy = dy;
            var x_scales = frame.x_scales, y_scales = frame.y_scales;
            var xrs = update_ranges(x_scales, sx0, sx1);
            var yrs = update_ranges(y_scales, sy0, sy1);
            this.pan_info = { xrs: xrs, yrs: yrs, sdx: sdx, sdy: sdy };
            this.plot_view.update_range(this.pan_info, true);
        };
        return PanToolView;
    }(gesture_tool_1.GestureToolView));
    exports.PanToolView = PanToolView;
    PanToolView.__name__ = "PanToolView";
    var PanTool = /** @class */ (function (_super) {
        tslib_1.__extends(PanTool, _super);
        function PanTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Pan";
            _this.event_type = "pan";
            _this.default_order = 10;
            return _this;
        }
        PanTool.init_PanTool = function () {
            this.prototype.default_view = PanToolView;
            this.define({
                dimensions: [p.Dimensions, "both"],
            });
            this.register_alias("pan", function () { return new PanTool({ dimensions: 'both' }); });
            this.register_alias("xpan", function () { return new PanTool({ dimensions: 'width' }); });
            this.register_alias("ypan", function () { return new PanTool({ dimensions: 'height' }); });
        };
        Object.defineProperty(PanTool.prototype, "tooltip", {
            get: function () {
                return this._get_dim_tooltip("Pan", this.dimensions);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(PanTool.prototype, "icon", {
            get: function () {
                switch (this.dimensions) {
                    case "both": return icons_1.bk_tool_icon_pan;
                    case "width": return icons_1.bk_tool_icon_xpan;
                    case "height": return icons_1.bk_tool_icon_ypan;
                }
            },
            enumerable: true,
            configurable: true
        });
        return PanTool;
    }(gesture_tool_1.GestureTool));
    exports.PanTool = PanTool;
    PanTool.__name__ = "PanTool";
    PanTool.init_PanTool();
},
/* models/tools/gestures/range_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var box_annotation_1 = require(239) /* ../../annotations/box_annotation */;
    var logging_1 = require(134) /* ../../../core/logging */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var gesture_tool_1 = require(421) /* ./gesture_tool */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    function flip_side(side) {
        switch (side) {
            case 1 /* Left */: return 2 /* Right */;
            case 2 /* Right */: return 1 /* Left */;
            case 4 /* Bottom */: return 5 /* Top */;
            case 5 /* Top */: return 4 /* Bottom */;
            default: return side;
        }
    }
    exports.flip_side = flip_side;
    // TODO (bev) This would be better directly with BoxAnnotation, but hard
    // to test on a view. Move when "View Models" are implemented
    function is_near(pos, value, scale, tolerance) {
        if (value == null)
            return false;
        var svalue = scale.compute(value);
        return Math.abs(pos - svalue) < tolerance;
    }
    exports.is_near = is_near;
    // TODO (bev) This would be better directly with BoxAnnotation, but hard
    // to test on a view. Move when "View Models" are implemented
    function is_inside(sx, sy, xscale, yscale, overlay) {
        var result = true;
        if (overlay.left != null && overlay.right != null) {
            var x = xscale.invert(sx);
            if (x < overlay.left || x > overlay.right)
                result = false;
        }
        if (overlay.bottom != null && overlay.top != null) {
            var y = yscale.invert(sy);
            if (y < overlay.bottom || y > overlay.top)
                result = false;
        }
        return result;
    }
    exports.is_inside = is_inside;
    function sides_inside(start, end, range) {
        var result = 0;
        if (start >= range.start && start <= range.end)
            result += 1;
        if (end >= range.start && end <= range.end)
            result += 1;
        return result;
    }
    exports.sides_inside = sides_inside;
    function compute_value(value, scale, sdelta, range) {
        var svalue = scale.compute(value);
        var new_value = scale.invert(svalue + sdelta);
        if (new_value >= range.start && new_value <= range.end)
            return new_value;
        return value;
    }
    exports.compute_value = compute_value;
    function update_range_end_side(end, range, side) {
        if (end > range.start) {
            range.end = end;
            return side;
        }
        else {
            range.end = range.start;
            range.start = end;
            return flip_side(side);
        }
    }
    exports.update_range_end_side = update_range_end_side;
    function update_range_start_side(start, range, side) {
        if (start < range.end) {
            range.start = start;
            return side;
        }
        else {
            range.start = range.end;
            range.end = start;
            return flip_side(side);
        }
    }
    exports.update_range_start_side = update_range_start_side;
    function update_range(range, scale, delta, plot_range) {
        var _a = tslib_1.__read(scale.r_compute(range.start, range.end), 2), sstart = _a[0], send = _a[1];
        var _b = tslib_1.__read(scale.r_invert(sstart + delta, send + delta), 2), start = _b[0], end = _b[1];
        var initial_sides_inside = sides_inside(range.start, range.end, plot_range);
        var final_sides_inside = sides_inside(start, end, plot_range);
        // Allow the update as long as the number of sides in-bounds does not decrease
        if (final_sides_inside >= initial_sides_inside) {
            range.start = start;
            range.end = end;
        }
    }
    exports.update_range = update_range;
    var RangeToolView = /** @class */ (function (_super) {
        tslib_1.__extends(RangeToolView, _super);
        function RangeToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        RangeToolView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.side = 0 /* None */;
            this.model.update_overlay_from_ranges();
        };
        RangeToolView.prototype.connect_signals = function () {
            var _this = this;
            _super.prototype.connect_signals.call(this);
            if (this.model.x_range != null)
                this.connect(this.model.x_range.change, function () { return _this.model.update_overlay_from_ranges(); });
            if (this.model.y_range != null)
                this.connect(this.model.y_range.change, function () { return _this.model.update_overlay_from_ranges(); });
        };
        RangeToolView.prototype._pan_start = function (ev) {
            this.last_dx = 0;
            this.last_dy = 0;
            var xr = this.model.x_range;
            var yr = this.model.y_range;
            var frame = this.plot_view.frame;
            var xscale = frame.x_scale;
            var yscale = frame.y_scale;
            var overlay = this.model.overlay;
            var left = overlay.left, right = overlay.right, top = overlay.top, bottom = overlay.bottom;
            var tolerance = this.model.overlay.properties.line_width.value() + box_annotation_1.EDGE_TOLERANCE;
            if (xr != null && this.model.x_interaction) {
                if (is_near(ev.sx, left, xscale, tolerance))
                    this.side = 1 /* Left */;
                else if (is_near(ev.sx, right, xscale, tolerance))
                    this.side = 2 /* Right */;
                else if (is_inside(ev.sx, ev.sy, xscale, yscale, overlay)) {
                    this.side = 3 /* LeftRight */;
                }
            }
            if (yr != null && this.model.y_interaction) {
                if (this.side == 0 /* None */ && is_near(ev.sy, bottom, yscale, tolerance))
                    this.side = 4 /* Bottom */;
                if (this.side == 0 /* None */ && is_near(ev.sy, top, yscale, tolerance))
                    this.side = 5 /* Top */;
                else if (is_inside(ev.sx, ev.sy, xscale, yscale, this.model.overlay)) {
                    if (this.side == 3 /* LeftRight */)
                        this.side = 7 /* LeftRightBottomTop */;
                    else
                        this.side = 6 /* BottomTop */;
                }
            }
        };
        RangeToolView.prototype._pan = function (ev) {
            var frame = this.plot_view.frame;
            var new_dx = ev.deltaX - this.last_dx;
            var new_dy = ev.deltaY - this.last_dy;
            var xr = this.model.x_range;
            var yr = this.model.y_range;
            var xscale = frame.x_scale;
            var yscale = frame.y_scale;
            if (xr != null) {
                if (this.side == 3 /* LeftRight */ || this.side == 7 /* LeftRightBottomTop */)
                    update_range(xr, xscale, new_dx, frame.x_range);
                else if (this.side == 1 /* Left */) {
                    var start = compute_value(xr.start, xscale, new_dx, frame.x_range);
                    this.side = update_range_start_side(start, xr, this.side);
                }
                else if (this.side == 2 /* Right */) {
                    var end = compute_value(xr.end, xscale, new_dx, frame.x_range);
                    this.side = update_range_end_side(end, xr, this.side);
                }
            }
            if (yr != null) {
                if (this.side == 6 /* BottomTop */ || this.side == 7 /* LeftRightBottomTop */)
                    update_range(yr, yscale, new_dy, frame.y_range);
                else if (this.side == 4 /* Bottom */) {
                    var start = compute_value(yr.start, yscale, new_dy, frame.y_range);
                    this.side = update_range_start_side(start, yr, this.side);
                }
                else if (this.side == 5 /* Top */) {
                    var end = compute_value(yr.end, yscale, new_dy, frame.y_range);
                    this.side = update_range_end_side(end, yr, this.side);
                }
            }
            this.last_dx = ev.deltaX;
            this.last_dy = ev.deltaY;
        };
        RangeToolView.prototype._pan_end = function (_ev) {
            this.side = 0 /* None */;
        };
        return RangeToolView;
    }(gesture_tool_1.GestureToolView));
    exports.RangeToolView = RangeToolView;
    RangeToolView.__name__ = "RangeToolView";
    var DEFAULT_RANGE_OVERLAY = function () {
        return new box_annotation_1.BoxAnnotation({
            level: "overlay",
            fill_color: "lightgrey",
            fill_alpha: 0.5,
            line_color: "black",
            line_alpha: 1.0,
            line_width: 0.5,
            line_dash: [2, 2],
        });
    };
    var RangeTool = /** @class */ (function (_super) {
        tslib_1.__extends(RangeTool, _super);
        function RangeTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Range Tool";
            _this.icon = icons_1.bk_tool_icon_range;
            _this.event_type = "pan";
            _this.default_order = 1;
            return _this;
        }
        RangeTool.init_RangeTool = function () {
            this.prototype.default_view = RangeToolView;
            this.define({
                x_range: [p.Instance, null],
                x_interaction: [p.Boolean, true],
                y_range: [p.Instance, null],
                y_interaction: [p.Boolean, true],
                overlay: [p.Instance, DEFAULT_RANGE_OVERLAY],
            });
        };
        RangeTool.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.overlay.in_cursor = "grab";
            this.overlay.ew_cursor = this.x_range != null && this.x_interaction ? "ew-resize" : null;
            this.overlay.ns_cursor = this.y_range != null && this.y_interaction ? "ns-resize" : null;
        };
        RangeTool.prototype.update_overlay_from_ranges = function () {
            if (this.x_range == null && this.y_range == null) {
                this.overlay.left = null;
                this.overlay.right = null;
                this.overlay.bottom = null;
                this.overlay.top = null;
                logging_1.logger.warn('RangeTool not configured with any Ranges.');
            }
            if (this.x_range == null) {
                this.overlay.left = null;
                this.overlay.right = null;
            }
            else {
                this.overlay.left = this.x_range.start;
                this.overlay.right = this.x_range.end;
            }
            if (this.y_range == null) {
                this.overlay.bottom = null;
                this.overlay.top = null;
            }
            else {
                this.overlay.bottom = this.y_range.start;
                this.overlay.top = this.y_range.end;
            }
        };
        return RangeTool;
    }(gesture_tool_1.GestureTool));
    exports.RangeTool = RangeTool;
    RangeTool.__name__ = "RangeTool";
    RangeTool.init_RangeTool();
},
/* models/tools/gestures/tap_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var select_tool_1 = require(479) /* ./select_tool */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var icons_1 = require(424) /* ../../../styles/icons */;
    var TapToolView = /** @class */ (function (_super) {
        tslib_1.__extends(TapToolView, _super);
        function TapToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        TapToolView.prototype._tap = function (ev) {
            var sx = ev.sx, sy = ev.sy;
            var geometry = { type: "point", sx: sx, sy: sy };
            this._select(geometry, true, this._select_mode(ev));
        };
        TapToolView.prototype._select = function (geometry, final, mode) {
            var e_1, _a, e_2, _b;
            var _this = this;
            var callback = this.model.callback;
            if (this.model.behavior == "select") {
                var renderers_by_source = this._computed_renderers_by_data_source();
                try {
                    for (var renderers_by_source_1 = tslib_1.__values(renderers_by_source), renderers_by_source_1_1 = renderers_by_source_1.next(); !renderers_by_source_1_1.done; renderers_by_source_1_1 = renderers_by_source_1.next()) {
                        var _c = tslib_1.__read(renderers_by_source_1_1.value, 2), renderers = _c[1];
                        var sm = renderers[0].get_selection_manager();
                        var r_views = renderers.map(function (r) { return _this.plot_view.renderer_views.get(r); });
                        var did_hit = sm.select(r_views, geometry, final, mode);
                        if (did_hit && callback != null) {
                            var x = r_views[0].coordinates.x_scale.invert(geometry.sx);
                            var y = r_views[0].coordinates.y_scale.invert(geometry.sy);
                            var data = { geometries: Object.assign(Object.assign({}, geometry), { x: x, y: y }), source: sm.source };
                            callback.execute(this.model, data);
                        }
                    }
                }
                catch (e_1_1) {
                    e_1 = { error: e_1_1 };
                }
                finally {
                    try {
                        if (renderers_by_source_1_1 && !renderers_by_source_1_1.done && (_a = renderers_by_source_1.return))
                            _a.call(renderers_by_source_1);
                    }
                    finally {
                        if (e_1)
                            throw e_1.error;
                    }
                }
                this._emit_selection_event(geometry);
                this.plot_view.push_state('tap', { selection: this.plot_view.get_selection() });
            }
            else {
                try {
                    for (var _d = tslib_1.__values(this.computed_renderers), _e = _d.next(); !_e.done; _e = _d.next()) {
                        var r = _e.value;
                        var rv = this.plot_view.renderer_views.get(r);
                        var sm = r.get_selection_manager();
                        var did_hit = sm.inspect(rv, geometry);
                        if (did_hit && callback != null) {
                            var x = rv.coordinates.x_scale.invert(geometry.sx);
                            var y = rv.coordinates.y_scale.invert(geometry.sy);
                            var data = { geometries: Object.assign(Object.assign({}, geometry), { x: x, y: y }), source: sm.source };
                            callback.execute(this.model, data);
                        }
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (_e && !_e.done && (_b = _d.return))
                            _b.call(_d);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
            }
        };
        return TapToolView;
    }(select_tool_1.SelectToolView));
    exports.TapToolView = TapToolView;
    TapToolView.__name__ = "TapToolView";
    var TapTool = /** @class */ (function (_super) {
        tslib_1.__extends(TapTool, _super);
        function TapTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Tap";
            _this.icon = icons_1.bk_tool_icon_tap_select;
            _this.event_type = "tap";
            _this.default_order = 10;
            return _this;
        }
        TapTool.init_TapTool = function () {
            this.prototype.default_view = TapToolView;
            this.define({
                behavior: [p.TapBehavior, "select"],
                callback: [p.Any],
            });
            this.register_alias("click", function () { return new TapTool({ behavior: "inspect" }); });
            this.register_alias("tap", function () { return new TapTool(); });
        };
        return TapTool;
    }(select_tool_1.SelectTool));
    exports.TapTool = TapTool;
    TapTool.__name__ = "TapTool";
    TapTool.init_TapTool();
},
/* models/tools/gestures/wheel_pan_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var gesture_tool_1 = require(421) /* ./gesture_tool */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var icons_1 = require(424) /* ../../../styles/icons */;
    var pan_tool_1 = require(486) /* ./pan_tool */;
    var WheelPanToolView = /** @class */ (function (_super) {
        tslib_1.__extends(WheelPanToolView, _super);
        function WheelPanToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        WheelPanToolView.prototype._scroll = function (ev) {
            var factor = this.model.speed * ev.delta;
            // clamp the magnitude of factor, if it is > 1 bad things happen
            if (factor > 0.9)
                factor = 0.9;
            else if (factor < -0.9)
                factor = -0.9;
            this._update_ranges(factor);
        };
        WheelPanToolView.prototype._update_ranges = function (factor) {
            var frame = this.plot_view.frame;
            var hr = frame.bbox.h_range;
            var vr = frame.bbox.v_range;
            var _a = tslib_1.__read([hr.start, hr.end], 2), sx_low = _a[0], sx_high = _a[1];
            var _b = tslib_1.__read([vr.start, vr.end], 2), sy_low = _b[0], sy_high = _b[1];
            var sx0;
            var sx1;
            var sy0;
            var sy1;
            switch (this.model.dimension) {
                case "height": {
                    var sy_range = Math.abs(sy_high - sy_low);
                    sx0 = sx_low;
                    sx1 = sx_high;
                    sy0 = sy_low - sy_range * factor;
                    sy1 = sy_high - sy_range * factor;
                    break;
                }
                case "width": {
                    var sx_range = Math.abs(sx_high - sx_low);
                    sx0 = sx_low - sx_range * factor;
                    sx1 = sx_high - sx_range * factor;
                    sy0 = sy_low;
                    sy1 = sy_high;
                    break;
                }
                default:
                    throw new Error("this shouldn't have happened");
            }
            var x_scales = frame.x_scales, y_scales = frame.y_scales;
            var xrs = pan_tool_1.update_ranges(x_scales, sx0, sx1);
            var yrs = pan_tool_1.update_ranges(y_scales, sy0, sy1);
            // OK this sucks we can't set factor independently in each direction. It is used
            // for GMap plots, and GMap plots always preserve aspect, so effective the value
            // of 'dimensions' is ignored.
            var pan_info = { xrs: xrs, yrs: yrs, factor: factor };
            this.plot_view.push_state('wheel_pan', { range: pan_info });
            this.plot_view.update_range(pan_info, false, true);
            if (this.model.document != null)
                this.model.document.interactive_start(this.plot_model);
        };
        return WheelPanToolView;
    }(gesture_tool_1.GestureToolView));
    exports.WheelPanToolView = WheelPanToolView;
    WheelPanToolView.__name__ = "WheelPanToolView";
    var WheelPanTool = /** @class */ (function (_super) {
        tslib_1.__extends(WheelPanTool, _super);
        function WheelPanTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Wheel Pan";
            _this.icon = icons_1.bk_tool_icon_wheel_pan;
            _this.event_type = "scroll";
            _this.default_order = 12;
            return _this;
        }
        WheelPanTool.init_WheelPanTool = function () {
            this.prototype.default_view = WheelPanToolView;
            this.define({
                dimension: [p.Dimension, "width"],
            });
            this.internal({
                speed: [p.Number, 1 / 1000],
            });
            this.register_alias("xwheel_pan", function () { return new WheelPanTool({ dimension: "width" }); });
            this.register_alias("ywheel_pan", function () { return new WheelPanTool({ dimension: "height" }); });
        };
        Object.defineProperty(WheelPanTool.prototype, "tooltip", {
            get: function () {
                return this._get_dim_tooltip(this.tool_name, this.dimension);
            },
            enumerable: true,
            configurable: true
        });
        return WheelPanTool;
    }(gesture_tool_1.GestureTool));
    exports.WheelPanTool = WheelPanTool;
    WheelPanTool.__name__ = "WheelPanTool";
    WheelPanTool.init_WheelPanTool();
},
/* models/tools/gestures/wheel_zoom_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var gesture_tool_1 = require(421) /* ./gesture_tool */;
    var zoom_1 = require(469) /* ../../../core/util/zoom */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var compat_1 = require(147) /* ../../../core/util/compat */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var WheelZoomToolView = /** @class */ (function (_super) {
        tslib_1.__extends(WheelZoomToolView, _super);
        function WheelZoomToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        WheelZoomToolView.prototype._pinch = function (ev) {
            // TODO (bev) this can probably be done much better
            var sx = ev.sx, sy = ev.sy, scale = ev.scale, ctrlKey = ev.ctrlKey, shiftKey = ev.shiftKey;
            var delta;
            if (scale >= 1)
                delta = (scale - 1) * 20.0;
            else
                delta = -20.0 / scale;
            this._scroll({ type: "wheel", sx: sx, sy: sy, delta: delta, ctrlKey: ctrlKey, shiftKey: shiftKey });
        };
        WheelZoomToolView.prototype._scroll = function (ev) {
            var frame = this.plot_view.frame;
            var hr = frame.bbox.h_range;
            var vr = frame.bbox.v_range;
            var sx = ev.sx, sy = ev.sy;
            var dims = this.model.dimensions;
            // restrict to axis configured in tool's dimensions property and if
            // zoom origin is inside of frame range/domain
            var h_axis = (dims == 'width' || dims == 'both') && hr.start < sx && sx < hr.end;
            var v_axis = (dims == 'height' || dims == 'both') && vr.start < sy && sy < vr.end;
            if ((!h_axis || !v_axis) && !this.model.zoom_on_axis) {
                return;
            }
            var factor = this.model.speed * ev.delta;
            var zoom_info = zoom_1.scale_range(frame, factor, h_axis, v_axis, { x: sx, y: sy });
            this.plot_view.push_state('wheel_zoom', { range: zoom_info });
            this.plot_view.update_range(zoom_info, false, true, this.model.maintain_focus);
            if (this.model.document != null)
                this.model.document.interactive_start(this.plot_model);
        };
        return WheelZoomToolView;
    }(gesture_tool_1.GestureToolView));
    exports.WheelZoomToolView = WheelZoomToolView;
    WheelZoomToolView.__name__ = "WheelZoomToolView";
    var WheelZoomTool = /** @class */ (function (_super) {
        tslib_1.__extends(WheelZoomTool, _super);
        function WheelZoomTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Wheel Zoom";
            _this.icon = icons_1.bk_tool_icon_wheel_zoom;
            _this.event_type = compat_1.is_mobile ? "pinch" : "scroll";
            _this.default_order = 10;
            return _this;
        }
        WheelZoomTool.init_WheelZoomTool = function () {
            this.prototype.default_view = WheelZoomToolView;
            this.define({
                dimensions: [p.Dimensions, "both"],
                maintain_focus: [p.Boolean, true],
                zoom_on_axis: [p.Boolean, true],
                speed: [p.Number, 1 / 600],
            });
            this.register_alias("wheel_zoom", function () { return new WheelZoomTool({ dimensions: 'both' }); });
            this.register_alias("xwheel_zoom", function () { return new WheelZoomTool({ dimensions: 'width' }); });
            this.register_alias("ywheel_zoom", function () { return new WheelZoomTool({ dimensions: 'height' }); });
        };
        Object.defineProperty(WheelZoomTool.prototype, "tooltip", {
            get: function () {
                return this._get_dim_tooltip(this.tool_name, this.dimensions);
            },
            enumerable: true,
            configurable: true
        });
        return WheelZoomTool;
    }(gesture_tool_1.GestureTool));
    exports.WheelZoomTool = WheelZoomTool;
    WheelZoomTool.__name__ = "WheelZoomTool";
    WheelZoomTool.init_WheelZoomTool();
},
/* models/tools/inspectors/crosshair_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var inspect_tool_1 = require(410) /* ./inspect_tool */;
    var span_1 = require(283) /* ../../annotations/span */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var object_1 = require(128) /* ../../../core/util/object */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var CrosshairToolView = /** @class */ (function (_super) {
        tslib_1.__extends(CrosshairToolView, _super);
        function CrosshairToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        CrosshairToolView.prototype._move = function (ev) {
            if (!this.model.active)
                return;
            var sx = ev.sx, sy = ev.sy;
            if (!this.plot_view.frame.bbox.contains(sx, sy))
                this._update_spans(null, null);
            else
                this._update_spans(sx, sy);
        };
        CrosshairToolView.prototype._move_exit = function (_e) {
            this._update_spans(null, null);
        };
        CrosshairToolView.prototype._update_spans = function (x, y) {
            var dims = this.model.dimensions;
            if (dims == "width" || dims == "both")
                this.model.spans.width.location = y;
            if (dims == "height" || dims == "both")
                this.model.spans.height.location = x;
        };
        return CrosshairToolView;
    }(inspect_tool_1.InspectToolView));
    exports.CrosshairToolView = CrosshairToolView;
    CrosshairToolView.__name__ = "CrosshairToolView";
    var CrosshairTool = /** @class */ (function (_super) {
        tslib_1.__extends(CrosshairTool, _super);
        function CrosshairTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Crosshair";
            _this.icon = icons_1.bk_tool_icon_crosshair;
            return _this;
        }
        CrosshairTool.init_CrosshairTool = function () {
            this.prototype.default_view = CrosshairToolView;
            this.define({
                dimensions: [p.Dimensions, "both"],
                line_color: [p.Color, 'black'],
                line_width: [p.Number, 1],
                line_alpha: [p.Number, 1.0],
            });
            this.internal({
                spans: [p.Any],
            });
            this.register_alias("crosshair", function () { return new CrosshairTool(); });
        };
        Object.defineProperty(CrosshairTool.prototype, "tooltip", {
            get: function () {
                return this._get_dim_tooltip("Crosshair", this.dimensions);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(CrosshairTool.prototype, "synthetic_renderers", {
            get: function () {
                return object_1.values(this.spans);
            },
            enumerable: true,
            configurable: true
        });
        CrosshairTool.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.spans = {
                width: new span_1.Span({
                    for_hover: true,
                    dimension: "width",
                    location_units: "screen",
                    level: "overlay",
                    line_color: this.line_color,
                    line_width: this.line_width,
                    line_alpha: this.line_alpha,
                }),
                height: new span_1.Span({
                    for_hover: true,
                    dimension: "height",
                    location_units: "screen",
                    level: "overlay",
                    line_color: this.line_color,
                    line_width: this.line_width,
                    line_alpha: this.line_alpha,
                }),
            };
        };
        return CrosshairTool;
    }(inspect_tool_1.InspectTool));
    exports.CrosshairTool = CrosshairTool;
    CrosshairTool.__name__ = "CrosshairTool";
    CrosshairTool.init_CrosshairTool();
},
/* models/tools/inspectors/customjs_hover.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var model_1 = require(196) /* ../../../model */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var object_1 = require(128) /* ../../../core/util/object */;
    var string_1 = require(144) /* ../../../core/util/string */;
    var CustomJSHover = /** @class */ (function (_super) {
        tslib_1.__extends(CustomJSHover, _super);
        function CustomJSHover(attrs) {
            return _super.call(this, attrs) || this;
        }
        CustomJSHover.init_CustomJSHover = function () {
            this.define({
                args: [p.Any, {}],
                code: [p.String, ""],
            });
        };
        Object.defineProperty(CustomJSHover.prototype, "values", {
            get: function () {
                return object_1.values(this.args);
            },
            enumerable: true,
            configurable: true
        });
        /*protected*/ CustomJSHover.prototype._make_code = function (valname, formatname, varsname, fn) {
            // this relies on keys(args) and values(args) returning keys and values
            // in the same order
            return new (Function.bind.apply(Function, tslib_1.__spread([void 0], object_1.keys(this.args), [valname, formatname, varsname, string_1.use_strict(fn)])))();
        };
        CustomJSHover.prototype.format = function (value, format, special_vars) {
            var formatter = this._make_code("value", "format", "special_vars", this.code);
            return formatter.apply(void 0, tslib_1.__spread(this.values, [value, format, special_vars]));
        };
        return CustomJSHover;
    }(model_1.Model));
    exports.CustomJSHover = CustomJSHover;
    CustomJSHover.__name__ = "CustomJSHover";
    CustomJSHover.init_CustomJSHover();
},
/* models/tools/inspectors/hover_tool.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var inspect_tool_1 = require(410) /* ./inspect_tool */;
    var tooltip_1 = require(286) /* ../../annotations/tooltip */;
    var glyph_renderer_1 = require(205) /* ../../renderers/glyph_renderer */;
    var graph_renderer_1 = require(231) /* ../../renderers/graph_renderer */;
    var util_1 = require(480) /* ../util */;
    var hittest = tslib_1.__importStar(require(216) /* ../../../core/hittest */);
    var templating_1 = require(302) /* ../../../core/util/templating */;
    var dom_1 = require(187) /* ../../../core/dom */;
    var p = tslib_1.__importStar(require(133) /* ../../../core/properties */);
    var color_1 = require(137) /* ../../../core/util/color */;
    var object_1 = require(128) /* ../../../core/util/object */;
    var iterator_1 = require(418) /* ../../../core/util/iterator */;
    var types_1 = require(123) /* ../../../core/util/types */;
    var build_views_1 = require(230) /* ../../../core/build_views */;
    var icons_1 = require(424) /* ../../../styles/icons */;
    var tooltips_1 = require(287) /* ../../../styles/tooltips */;
    function _nearest_line_hit(i, geometry, sx, sy, dx, dy) {
        var d1 = { x: dx[i], y: dy[i] };
        var d2 = { x: dx[i + 1], y: dy[i + 1] };
        var dist1;
        var dist2;
        if (geometry.type == "span") {
            if (geometry.direction == "h") {
                dist1 = Math.abs(d1.x - sx);
                dist2 = Math.abs(d2.x - sx);
            }
            else {
                dist1 = Math.abs(d1.y - sy);
                dist2 = Math.abs(d2.y - sy);
            }
        }
        else {
            var s = { x: sx, y: sy };
            dist1 = hittest.dist_2_pts(d1, s);
            dist2 = hittest.dist_2_pts(d2, s);
        }
        if (dist1 < dist2)
            return [[d1.x, d1.y], i];
        else
            return [[d2.x, d2.y], i + 1];
    }
    exports._nearest_line_hit = _nearest_line_hit;
    function _line_hit(xs, ys, ind) {
        return [[xs[ind], ys[ind]], ind];
    }
    exports._line_hit = _line_hit;
    var HoverToolView = /** @class */ (function (_super) {
        tslib_1.__extends(HoverToolView, _super);
        function HoverToolView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        HoverToolView.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._ttmodels = null;
            this._ttviews = new Map();
            var tooltips = this.model.tooltips;
            if (types_1.isArray(tooltips)) {
                this._template_el = this._create_template(tooltips);
            }
        };
        HoverToolView.prototype.remove = function () {
            build_views_1.remove_views(this._ttviews);
            _super.prototype.remove.call(this);
        };
        HoverToolView.prototype.connect_signals = function () {
            var e_1, _a;
            var _this = this;
            _super.prototype.connect_signals.call(this);
            try {
                for (var _b = tslib_1.__values(this.computed_renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var r = _c.value;
                    if (r instanceof glyph_renderer_1.GlyphRenderer)
                        this.connect(r.data_source.inspect, this._update);
                    else if (r instanceof graph_renderer_1.GraphRenderer) {
                        this.connect(r.node_renderer.data_source.inspect, this._update);
                        this.connect(r.edge_renderer.data_source.inspect, this._update);
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            // TODO: this.connect(this.plot_model.properties.renderers.change, () => this._computed_renderers = this._ttmodels = null)
            this.connect(this.model.properties.renderers.change, function () { return _this._computed_renderers = _this._ttmodels = null; });
            this.connect(this.model.properties.names.change, function () { return _this._computed_renderers = _this._ttmodels = null; });
            this.connect(this.model.properties.tooltips.change, function () { return _this._ttmodels = null; });
        };
        HoverToolView.prototype._compute_ttmodels = function () {
            var e_2, _a;
            var _this = this;
            var ttmodels = new Map();
            var tooltips = this.model.tooltips;
            if (tooltips != null) {
                try {
                    for (var _b = tslib_1.__values(this.computed_renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var r = _c.value;
                        var tooltip = new tooltip_1.Tooltip({
                            custom: types_1.isString(tooltips) || types_1.isFunction(tooltips),
                            attachment: this.model.attachment,
                            show_arrow: this.model.show_arrow,
                        });
                        if (r instanceof glyph_renderer_1.GlyphRenderer) {
                            ttmodels.set(r, tooltip);
                        }
                        else if (r instanceof graph_renderer_1.GraphRenderer) {
                            ttmodels.set(r.node_renderer, tooltip);
                            ttmodels.set(r.edge_renderer, tooltip);
                        }
                    }
                }
                catch (e_2_1) {
                    e_2 = { error: e_2_1 };
                }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return))
                            _a.call(_b);
                    }
                    finally {
                        if (e_2)
                            throw e_2.error;
                    }
                }
            }
            // XXX: move this to lazy_initialize()
            (function () {
                return tslib_1.__awaiter(_this, void 0, void 0, function () {
                    var views, views_1, views_1_1, ttview;
                    var e_3, _a;
                    return tslib_1.__generator(this, function (_b) {
                        switch (_b.label) {
                            case 0: return [4 /*yield*/, build_views_1.build_views(this._ttviews, tslib_1.__spread(ttmodels.values()), { parent: this.plot_view })];
                            case 1:
                                views = _b.sent();
                                try {
                                    for (views_1 = tslib_1.__values(views), views_1_1 = views_1.next(); !views_1_1.done; views_1_1 = views_1.next()) {
                                        ttview = views_1_1.value;
                                        ttview.render();
                                    }
                                }
                                catch (e_3_1) {
                                    e_3 = { error: e_3_1 };
                                }
                                finally {
                                    try {
                                        if (views_1_1 && !views_1_1.done && (_a = views_1.return))
                                            _a.call(views_1);
                                    }
                                    finally {
                                        if (e_3)
                                            throw e_3.error;
                                    }
                                }
                                return [2 /*return*/];
                        }
                    });
                });
            })();
            return ttmodels;
        };
        Object.defineProperty(HoverToolView.prototype, "computed_renderers", {
            get: function () {
                if (this._computed_renderers == null) {
                    var renderers = this.model.renderers;
                    var all_renderers = this.plot_model.renderers;
                    var names = this.model.names;
                    this._computed_renderers = util_1.compute_renderers(renderers, all_renderers, names);
                }
                return this._computed_renderers;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(HoverToolView.prototype, "ttmodels", {
            get: function () {
                if (this._ttmodels == null)
                    this._ttmodels = this._compute_ttmodels();
                return this._ttmodels;
            },
            enumerable: true,
            configurable: true
        });
        HoverToolView.prototype._clear = function () {
            var e_4, _a;
            this._inspect(Infinity, Infinity);
            try {
                for (var _b = tslib_1.__values(this.ttmodels), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var _d = tslib_1.__read(_c.value, 2), tooltip = _d[1];
                    tooltip.clear();
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
        };
        HoverToolView.prototype._move = function (ev) {
            if (!this.model.active)
                return;
            var sx = ev.sx, sy = ev.sy;
            if (!this.plot_view.frame.bbox.contains(sx, sy))
                this._clear();
            else
                this._inspect(sx, sy);
        };
        HoverToolView.prototype._move_exit = function () {
            this._clear();
        };
        HoverToolView.prototype._inspect = function (sx, sy) {
            var e_5, _a;
            var geometry;
            if (this.model.mode == 'mouse')
                geometry = { type: 'point', sx: sx, sy: sy };
            else {
                var direction = this.model.mode == 'vline' ? 'h' : 'v';
                geometry = { type: 'span', direction: direction, sx: sx, sy: sy };
            }
            try {
                for (var _b = tslib_1.__values(this.computed_renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var r = _c.value;
                    var sm = r.get_selection_manager();
                    sm.inspect(this.plot_view.renderer_views.get(r), geometry);
                }
            }
            catch (e_5_1) {
                e_5 = { error: e_5_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_5)
                        throw e_5.error;
                }
            }
            if (this.model.callback != null)
                this._emit_callback(geometry);
        };
        HoverToolView.prototype._update = function (_a) {
            var e_6, _b, _c, _d, _e, _f, _g, _h, _j, _k, e_7, _l, e_8, _m, e_9, _o, _p, _q, _r, _s, _t, _u, _v, _w, e_10, _z;
            var _0 = tslib_1.__read(_a, 2), renderer_view = _0[0], geometry = _0[1].geometry;
            if (!this.model.active)
                return;
            if (!(renderer_view instanceof glyph_renderer_1.GlyphRendererView)) // || renderer_view instanceof GraphRendererView))
                return;
            var renderer = renderer_view.model;
            if (this.model.muted_policy == 'ignore' && renderer instanceof glyph_renderer_1.GlyphRenderer && renderer.muted)
                return;
            var tooltip = this.ttmodels.get(renderer);
            if (tooltip == null)
                return;
            var selection_manager = renderer.get_selection_manager();
            var indices = selection_manager.inspectors.get(renderer);
            if (renderer instanceof glyph_renderer_1.GlyphRenderer)
                indices = renderer.view.convert_selection_to_subset(indices);
            if (indices.is_empty()) {
                tooltip.clear();
                return;
            }
            var ds = selection_manager.source;
            var sx = geometry.sx, sy = geometry.sy;
            var xscale = renderer_view.coordinates.x_scale;
            var yscale = renderer_view.coordinates.y_scale;
            var x = xscale.invert(sx);
            var y = yscale.invert(sy);
            var glyph = renderer_view.glyph; // XXX
            var tooltips = [];
            try {
                for (var _1 = tslib_1.__values(indices.line_indices), _2 = _1.next(); !_2.done; _2 = _1.next()) {
                    var i = _2.value;
                    var data_x = glyph._x[i + 1];
                    var data_y = glyph._y[i + 1];
                    var ii = i;
                    var rx = void 0;
                    var ry = void 0;
                    switch (this.model.line_policy) {
                        case "interp": { // and renderer.get_interpolation_hit?
                            _c = tslib_1.__read(glyph.get_interpolation_hit(i, geometry), 2), data_x = _c[0], data_y = _c[1];
                            rx = xscale.compute(data_x);
                            ry = yscale.compute(data_y);
                            break;
                        }
                        case "prev": {
                            _d = tslib_1.__read(_line_hit(glyph.sx, glyph.sy, i), 2), _e = tslib_1.__read(_d[0], 2), rx = _e[0], ry = _e[1], ii = _d[1];
                            break;
                        }
                        case "next": {
                            _f = tslib_1.__read(_line_hit(glyph.sx, glyph.sy, i + 1), 2), _g = tslib_1.__read(_f[0], 2), rx = _g[0], ry = _g[1], ii = _f[1];
                            break;
                        }
                        case "nearest": {
                            _h = tslib_1.__read(_nearest_line_hit(i, geometry, sx, sy, glyph.sx, glyph.sy), 2), _j = tslib_1.__read(_h[0], 2), rx = _j[0], ry = _j[1], ii = _h[1];
                            data_x = glyph._x[ii];
                            data_y = glyph._y[ii];
                            break;
                        }
                        default: {
                            _k = tslib_1.__read([sx, sy], 2), rx = _k[0], ry = _k[1];
                        }
                    }
                    var vars = {
                        index: ii,
                        x: x, y: y, sx: sx, sy: sy, data_x: data_x, data_y: data_y, rx: rx, ry: ry,
                        indices: indices.line_indices,
                        name: renderer_view.model.name,
                    };
                    tooltips.push([rx, ry, this._render_tooltips(ds, ii, vars)]);
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (_2 && !_2.done && (_b = _1.return))
                        _b.call(_1);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
            try {
                for (var _3 = tslib_1.__values(indices.image_indices), _4 = _3.next(); !_4.done; _4 = _3.next()) {
                    var struct = _4.value;
                    var vars = {
                        index: struct.index,
                        x: x, y: y, sx: sx, sy: sy,
                        name: renderer_view.model.name,
                    };
                    var rendered = this._render_tooltips(ds, struct, vars);
                    tooltips.push([sx, sy, rendered]);
                }
            }
            catch (e_7_1) {
                e_7 = { error: e_7_1 };
            }
            finally {
                try {
                    if (_4 && !_4.done && (_l = _3.return))
                        _l.call(_3);
                }
                finally {
                    if (e_7)
                        throw e_7.error;
                }
            }
            try {
                for (var _5 = tslib_1.__values(indices.indices), _6 = _5.next(); !_6.done; _6 = _5.next()) {
                    var i = _6.value;
                    // multiglyphs set additional indices, e.g. multiline_indices for different tooltips
                    if (!object_1.isEmpty(indices.multiline_indices)) {
                        try {
                            for (var _7 = (e_9 = void 0, tslib_1.__values(indices.multiline_indices[i.toString()])), _8 = _7.next(); !_8.done; _8 = _7.next()) { // TODO: indices.multiline_indices.get(i)
                                var j = _8.value;
                                var data_x = glyph._xs[i][j];
                                var data_y = glyph._ys[i][j];
                                var jj = j;
                                var rx = void 0;
                                var ry = void 0;
                                switch (this.model.line_policy) {
                                    case "interp": { // and renderer.get_interpolation_hit?
                                        _p = tslib_1.__read(glyph.get_interpolation_hit(i, j, geometry), 2), data_x = _p[0], data_y = _p[1];
                                        rx = xscale.compute(data_x);
                                        ry = yscale.compute(data_y);
                                        break;
                                    }
                                    case "prev": {
                                        _q = tslib_1.__read(_line_hit(glyph.sxs[i], glyph.sys[i], j), 2), _r = tslib_1.__read(_q[0], 2), rx = _r[0], ry = _r[1], jj = _q[1];
                                        break;
                                    }
                                    case "next": {
                                        _s = tslib_1.__read(_line_hit(glyph.sxs[i], glyph.sys[i], j + 1), 2), _t = tslib_1.__read(_s[0], 2), rx = _t[0], ry = _t[1], jj = _s[1];
                                        break;
                                    }
                                    case "nearest": {
                                        _u = tslib_1.__read(_nearest_line_hit(j, geometry, sx, sy, glyph.sxs[i], glyph.sys[i]), 2), _v = tslib_1.__read(_u[0], 2), rx = _v[0], ry = _v[1], jj = _u[1];
                                        data_x = glyph._xs[i][jj];
                                        data_y = glyph._ys[i][jj];
                                        break;
                                    }
                                    default:
                                        throw new Error("should't have happened");
                                }
                                var index = void 0;
                                if (renderer instanceof glyph_renderer_1.GlyphRenderer)
                                    index = renderer.view.convert_indices_from_subset([i])[0];
                                else
                                    index = i;
                                var vars = {
                                    index: index, x: x, y: y, sx: sx, sy: sy, data_x: data_x, data_y: data_y,
                                    segment_index: jj,
                                    indices: indices.multiline_indices,
                                    name: renderer_view.model.name,
                                };
                                tooltips.push([rx, ry, this._render_tooltips(ds, index, vars)]);
                            }
                        }
                        catch (e_9_1) {
                            e_9 = { error: e_9_1 };
                        }
                        finally {
                            try {
                                if (_8 && !_8.done && (_o = _7.return))
                                    _o.call(_7);
                            }
                            finally {
                                if (e_9)
                                    throw e_9.error;
                            }
                        }
                    }
                    else {
                        // handle non-multiglyphs
                        var data_x = glyph._x != null ? glyph._x[i] : undefined;
                        var data_y = glyph._y != null ? glyph._y[i] : undefined;
                        var rx = void 0;
                        var ry = void 0;
                        if (this.model.point_policy == 'snap_to_data') { // and renderer.glyph.sx? and renderer.glyph.sy?
                            // Pass in our screen position so we can determine which patch we're
                            // over if there are discontinuous patches.
                            var pt = glyph.get_anchor_point(this.model.anchor, i, [sx, sy]);
                            if (pt == null)
                                pt = glyph.get_anchor_point("center", i, [sx, sy]);
                            rx = pt.x;
                            ry = pt.y;
                        }
                        else
                            _w = tslib_1.__read([sx, sy], 2), rx = _w[0], ry = _w[1];
                        var index = void 0;
                        if (renderer instanceof glyph_renderer_1.GlyphRenderer)
                            index = renderer.view.convert_indices_from_subset([i])[0];
                        else
                            index = i;
                        var vars = {
                            index: index, x: x, y: y, sx: sx, sy: sy, data_x: data_x, data_y: data_y,
                            indices: indices.indices,
                            name: renderer_view.model.name,
                        };
                        tooltips.push([rx, ry, this._render_tooltips(ds, index, vars)]);
                    }
                }
            }
            catch (e_8_1) {
                e_8 = { error: e_8_1 };
            }
            finally {
                try {
                    if (_6 && !_6.done && (_m = _5.return))
                        _m.call(_5);
                }
                finally {
                    if (e_8)
                        throw e_8.error;
                }
            }
            if (tooltips.length == 0)
                tooltip.clear();
            else {
                var content = tooltip.content;
                dom_1.empty(tooltip.content);
                try {
                    for (var tooltips_2 = tslib_1.__values(tooltips), tooltips_2_1 = tooltips_2.next(); !tooltips_2_1.done; tooltips_2_1 = tooltips_2.next()) {
                        var _9 = tslib_1.__read(tooltips_2_1.value, 3), node = _9[2];
                        content.appendChild(node);
                    }
                }
                catch (e_10_1) {
                    e_10 = { error: e_10_1 };
                }
                finally {
                    try {
                        if (tooltips_2_1 && !tooltips_2_1.done && (_z = tooltips_2.return))
                            _z.call(tooltips_2);
                    }
                    finally {
                        if (e_10)
                            throw e_10.error;
                    }
                }
                var _10 = tslib_1.__read(tooltips[tooltips.length - 1], 2), x_1 = _10[0], y_1 = _10[1];
                tooltip.setv({ position: [x_1, y_1] }, { check_eq: false }); // XXX: force update
            }
        };
        HoverToolView.prototype._emit_callback = function (geometry) {
            var e_11, _a;
            try {
                for (var _b = tslib_1.__values(this.computed_renderers), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var r = _c.value;
                    var rv = this.plot_view.renderer_views.get(r);
                    var x = rv.coordinates.x_scale.invert(geometry.sx);
                    var y = rv.coordinates.y_scale.invert(geometry.sy);
                    var index = r.data_source.inspected;
                    var g = Object.assign({ x: x, y: y }, geometry);
                    this.model.callback.execute(this.model, { index: index, geometry: g, renderer: r });
                }
            }
            catch (e_11_1) {
                e_11 = { error: e_11_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_11)
                        throw e_11.error;
                }
            }
        };
        HoverToolView.prototype._create_template = function (tooltips) {
            var e_12, _a;
            var rows = dom_1.div({ style: { display: "table", borderSpacing: "2px" } });
            try {
                for (var tooltips_3 = tslib_1.__values(tooltips), tooltips_3_1 = tooltips_3.next(); !tooltips_3_1.done; tooltips_3_1 = tooltips_3.next()) {
                    var _b = tslib_1.__read(tooltips_3_1.value, 1), label = _b[0];
                    var row = dom_1.div({ style: { display: "table-row" } });
                    rows.appendChild(row);
                    var label_cell = dom_1.div({ style: { display: "table-cell" }, class: tooltips_1.bk_tooltip_row_label }, label.length != 0 ? label + ": " : "");
                    row.appendChild(label_cell);
                    var value_el = dom_1.span();
                    value_el.dataset.value = "";
                    var swatch_el = dom_1.span({ class: tooltips_1.bk_tooltip_color_block }, " ");
                    swatch_el.dataset.swatch = "";
                    dom_1.undisplay(swatch_el);
                    var value_cell = dom_1.div({ style: { display: "table-cell" }, class: tooltips_1.bk_tooltip_row_value }, value_el, swatch_el);
                    row.appendChild(value_cell);
                }
            }
            catch (e_12_1) {
                e_12 = { error: e_12_1 };
            }
            finally {
                try {
                    if (tooltips_3_1 && !tooltips_3_1.done && (_a = tooltips_3.return))
                        _a.call(tooltips_3);
                }
                finally {
                    if (e_12)
                        throw e_12.error;
                }
            }
            return rows;
        };
        HoverToolView.prototype._render_template = function (template, tooltips, ds, i, vars) {
            var e_13, _a, e_14, _b;
            var el = template.cloneNode(true);
            var value_els = el.querySelectorAll("[data-value]");
            var swatch_els = el.querySelectorAll("[data-swatch]");
            var color_re = /\$color(\[.*\])?:(\w*)/;
            try {
                for (var _c = tslib_1.__values(iterator_1.enumerate(tooltips)), _d = _c.next(); !_d.done; _d = _c.next()) {
                    var _e = tslib_1.__read(_d.value, 2), _f = tslib_1.__read(_e[0], 2), value = _f[1], j = _e[1];
                    var result = value.match(color_re);
                    if (result != null) {
                        var _g = tslib_1.__read(result, 3), _h = _g[1], opts = _h === void 0 ? "" : _h, colname = _g[2];
                        var column = ds.get_column(colname); // XXX: change to columnar ds
                        if (column == null) {
                            value_els[j].textContent = colname + " unknown";
                            continue;
                        }
                        var hex = opts.indexOf("hex") >= 0;
                        var swatch = opts.indexOf("swatch") >= 0;
                        var color = types_1.isNumber(i) ? column[i] : null;
                        if (color == null) {
                            value_els[j].textContent = "(null)";
                            continue;
                        }
                        if (hex)
                            color = color_1.color2hex(color);
                        value_els[j].textContent = color;
                        if (swatch) {
                            swatch_els[j].style.backgroundColor = color;
                            dom_1.display(swatch_els[j]);
                        }
                    }
                    else {
                        var content = templating_1.replace_placeholders(value.replace("$~", "$data_"), ds, i, this.model.formatters, vars);
                        if (types_1.isString(content)) {
                            value_els[j].textContent = content;
                        }
                        else {
                            try {
                                for (var content_1 = (e_14 = void 0, tslib_1.__values(content)), content_1_1 = content_1.next(); !content_1_1.done; content_1_1 = content_1.next()) {
                                    var el_1 = content_1_1.value;
                                    value_els[j].appendChild(el_1);
                                }
                            }
                            catch (e_14_1) {
                                e_14 = { error: e_14_1 };
                            }
                            finally {
                                try {
                                    if (content_1_1 && !content_1_1.done && (_b = content_1.return))
                                        _b.call(content_1);
                                }
                                finally {
                                    if (e_14)
                                        throw e_14.error;
                                }
                            }
                        }
                    }
                }
            }
            catch (e_13_1) {
                e_13 = { error: e_13_1 };
            }
            finally {
                try {
                    if (_d && !_d.done && (_a = _c.return))
                        _a.call(_c);
                }
                finally {
                    if (e_13)
                        throw e_13.error;
                }
            }
            return el;
        };
        HoverToolView.prototype._render_tooltips = function (ds, i, vars) {
            var tooltips = this.model.tooltips;
            if (types_1.isString(tooltips)) {
                var content = templating_1.replace_placeholders({ html: tooltips }, ds, i, this.model.formatters, vars);
                return dom_1.div({}, content);
            }
            else if (types_1.isFunction(tooltips)) {
                return tooltips(ds, vars);
            }
            else {
                return this._render_template(this._template_el, tooltips, ds, i, vars);
            }
        };
        return HoverToolView;
    }(inspect_tool_1.InspectToolView));
    exports.HoverToolView = HoverToolView;
    HoverToolView.__name__ = "HoverToolView";
    var HoverTool = /** @class */ (function (_super) {
        tslib_1.__extends(HoverTool, _super);
        function HoverTool(attrs) {
            var _this = _super.call(this, attrs) || this;
            _this.tool_name = "Hover";
            _this.icon = icons_1.bk_tool_icon_hover;
            return _this;
        }
        HoverTool.init_HoverTool = function () {
            this.prototype.default_view = HoverToolView;
            this.define({
                tooltips: [p.Any, [
                        ["index", "$index"],
                        ["data (x, y)", "($x, $y)"],
                        ["screen (x, y)", "($sx, $sy)"],
                    ]],
                formatters: [p.Any, {}],
                renderers: [p.Any, 'auto'],
                names: [p.Array, []],
                mode: [p.HoverMode, 'mouse'],
                muted_policy: [p.MutedPolicy, 'show'],
                point_policy: [p.PointPolicy, 'snap_to_data'],
                line_policy: [p.LinePolicy, 'nearest'],
                show_arrow: [p.Boolean, true],
                anchor: [p.Anchor, 'center'],
                attachment: [p.TooltipAttachment, 'horizontal'],
                callback: [p.Any],
            });
            this.register_alias("hover", function () { return new HoverTool(); });
        };
        return HoverTool;
    }(inspect_tool_1.InspectTool));
    exports.HoverTool = HoverTool;
    HoverTool.__name__ = "HoverTool";
    HoverTool.init_HoverTool();
},
/* models/tools/tool_proxy.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var signaling_1 = require(130) /* ../../core/signaling */;
    var model_1 = require(196) /* ../../model */;
    var inspect_tool_1 = require(410) /* ./inspectors/inspect_tool */;
    var iterator_1 = require(418) /* ../../core/util/iterator */;
    var ToolProxy = /** @class */ (function (_super) {
        tslib_1.__extends(ToolProxy, _super);
        function ToolProxy(attrs) {
            return _super.call(this, attrs) || this;
        }
        ToolProxy.init_ToolProxy = function () {
            this.define({
                tools: [p.Array, []],
                active: [p.Boolean, false],
                disabled: [p.Boolean, false],
            });
        };
        Object.defineProperty(ToolProxy.prototype, "button_view", {
            // Operates all the tools given only one button
            get: function () {
                return this.tools[0].button_view;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ToolProxy.prototype, "event_type", {
            get: function () {
                return this.tools[0].event_type;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ToolProxy.prototype, "tooltip", {
            get: function () {
                return this.tools[0].tooltip;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ToolProxy.prototype, "tool_name", {
            get: function () {
                return this.tools[0].tool_name;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ToolProxy.prototype, "icon", {
            get: function () {
                return this.tools[0].computed_icon;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ToolProxy.prototype, "computed_icon", {
            get: function () {
                return this.icon;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ToolProxy.prototype, "toggleable", {
            get: function () {
                var tool = this.tools[0];
                return tool instanceof inspect_tool_1.InspectTool && tool.toggleable;
            },
            enumerable: true,
            configurable: true
        });
        ToolProxy.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this.do = new signaling_1.Signal0(this, "do");
        };
        ToolProxy.prototype.connect_signals = function () {
            var e_1, _c;
            var _this = this;
            _super.prototype.connect_signals.call(this);
            this.connect(this.do, function () { return _this.doit(); });
            this.connect(this.properties.active.change, function () { return _this.set_active(); });
            var _loop_1 = function (tool) {
                this_1.connect(tool.properties.active.change, function () {
                    _this.active = tool.active;
                });
            };
            var this_1 = this;
            try {
                for (var _d = tslib_1.__values(this.tools), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var tool = _e.value;
                    _loop_1(tool);
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_c = _d.return))
                        _c.call(_d);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        };
        ToolProxy.prototype.doit = function () {
            var e_2, _c;
            try {
                for (var _d = tslib_1.__values(this.tools), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var tool = _e.value;
                    tool.do.emit();
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_c = _d.return))
                        _c.call(_d);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
        };
        ToolProxy.prototype.set_active = function () {
            var e_3, _c;
            try {
                for (var _d = tslib_1.__values(this.tools), _e = _d.next(); !_e.done; _e = _d.next()) {
                    var tool = _e.value;
                    tool.active = this.active;
                }
            }
            catch (e_3_1) {
                e_3 = { error: e_3_1 };
            }
            finally {
                try {
                    if (_e && !_e.done && (_c = _d.return))
                        _c.call(_d);
                }
                finally {
                    if (e_3)
                        throw e_3.error;
                }
            }
        };
        Object.defineProperty(ToolProxy.prototype, "menu", {
            get: function () {
                var e_4, _c;
                var _this = this;
                var menu = this.tools[0].menu;
                if (menu == null)
                    return null;
                var items = [];
                var _loop_2 = function (item, i) {
                    if (item == null)
                        items.push(null);
                    else {
                        var handler = function () {
                            var e_5, _c;
                            var _a, _b;
                            try {
                                for (var _d = (e_5 = void 0, tslib_1.__values(_this.tools)), _e = _d.next(); !_e.done; _e = _d.next()) {
                                    var tool = _e.value;
                                    (_b = (_a = tool.menu) === null || _a === void 0 ? void 0 : _a[i]) === null || _b === void 0 ? void 0 : _b.handler();
                                }
                            }
                            catch (e_5_1) {
                                e_5 = { error: e_5_1 };
                            }
                            finally {
                                try {
                                    if (_e && !_e.done && (_c = _d.return))
                                        _c.call(_d);
                                }
                                finally {
                                    if (e_5)
                                        throw e_5.error;
                                }
                            }
                        };
                        items.push(Object.assign(Object.assign({}, item), { handler: handler }));
                    }
                };
                try {
                    for (var _d = tslib_1.__values(iterator_1.enumerate(menu)), _e = _d.next(); !_e.done; _e = _d.next()) {
                        var _f = tslib_1.__read(_e.value, 2), item = _f[0], i = _f[1];
                        _loop_2(item, i);
                    }
                }
                catch (e_4_1) {
                    e_4 = { error: e_4_1 };
                }
                finally {
                    try {
                        if (_e && !_e.done && (_c = _d.return))
                            _c.call(_d);
                    }
                    finally {
                        if (e_4)
                            throw e_4.error;
                    }
                }
                return items;
            },
            enumerable: true,
            configurable: true
        });
        return ToolProxy;
    }(model_1.Model));
    exports.ToolProxy = ToolProxy;
    ToolProxy.__name__ = "ToolProxy";
    ToolProxy.init_ToolProxy();
},
/* models/tools/toolbar_box.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var p = tslib_1.__importStar(require(133) /* ../../core/properties */);
    var array_1 = require(124) /* ../../core/util/array */;
    var object_1 = require(128) /* ../../core/util/object */;
    var toolbar_base_1 = require(420) /* ./toolbar_base */;
    var tool_proxy_1 = require(494) /* ./tool_proxy */;
    var layout_dom_1 = require(387) /* ../layouts/layout_dom */;
    var layout_1 = require(327) /* ../../core/layout */;
    var ProxyToolbar = /** @class */ (function (_super) {
        tslib_1.__extends(ProxyToolbar, _super);
        function ProxyToolbar(attrs) {
            return _super.call(this, attrs) || this;
        }
        ProxyToolbar.init_ProxyToolbar = function () {
            this.define({
                toolbars: [p.Array, []],
            });
        };
        ProxyToolbar.prototype.initialize = function () {
            _super.prototype.initialize.call(this);
            this._merge_tools();
        };
        ProxyToolbar.prototype._merge_tools = function () {
            var e_1, _a, _b, e_2, _c, e_3, _d, e_4, _e, e_5, _f, e_6, _g, e_7, _h, e_8, _j, e_9, _k, e_10, _l, e_11, _m;
            var _this = this;
            // Go through all the tools on the toolbar and replace them with
            // a proxy e.g. PanTool, BoxSelectTool, etc.
            this._proxied_tools = [];
            var inspectors = {};
            var actions = {};
            var gestures = {};
            var new_help_tools = [];
            var new_help_urls = [];
            try {
                for (var _o = tslib_1.__values(this.help), _p = _o.next(); !_p.done; _p = _o.next()) {
                    var helptool = _p.value;
                    if (!array_1.includes(new_help_urls, helptool.redirect)) {
                        new_help_tools.push(helptool);
                        new_help_urls.push(helptool.redirect);
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_p && !_p.done && (_a = _o.return))
                        _a.call(_o);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
            (_b = this._proxied_tools).push.apply(_b, tslib_1.__spread(new_help_tools));
            this.help = new_help_tools;
            try {
                for (var _q = tslib_1.__values(object_1.entries(this.gestures)), _r = _q.next(); !_r.done; _r = _q.next()) {
                    var _s = tslib_1.__read(_r.value, 2), event_type = _s[0], gesture = _s[1];
                    if (!(event_type in gestures)) {
                        gestures[event_type] = {};
                    }
                    try {
                        for (var _t = (e_3 = void 0, tslib_1.__values(gesture.tools)), _u = _t.next(); !_u.done; _u = _t.next()) {
                            var tool = _u.value;
                            if (!(tool.type in gestures[event_type])) {
                                gestures[event_type][tool.type] = [];
                            }
                            gestures[event_type][tool.type].push(tool);
                        }
                    }
                    catch (e_3_1) {
                        e_3 = { error: e_3_1 };
                    }
                    finally {
                        try {
                            if (_u && !_u.done && (_d = _t.return))
                                _d.call(_t);
                        }
                        finally {
                            if (e_3)
                                throw e_3.error;
                        }
                    }
                }
            }
            catch (e_2_1) {
                e_2 = { error: e_2_1 };
            }
            finally {
                try {
                    if (_r && !_r.done && (_c = _q.return))
                        _c.call(_q);
                }
                finally {
                    if (e_2)
                        throw e_2.error;
                }
            }
            try {
                for (var _v = tslib_1.__values(this.inspectors), _w = _v.next(); !_w.done; _w = _v.next()) {
                    var tool = _w.value;
                    if (!(tool.type in inspectors)) {
                        inspectors[tool.type] = [];
                    }
                    inspectors[tool.type].push(tool);
                }
            }
            catch (e_4_1) {
                e_4 = { error: e_4_1 };
            }
            finally {
                try {
                    if (_w && !_w.done && (_e = _v.return))
                        _e.call(_v);
                }
                finally {
                    if (e_4)
                        throw e_4.error;
                }
            }
            try {
                for (var _x = tslib_1.__values(this.actions), _y = _x.next(); !_y.done; _y = _x.next()) {
                    var tool = _y.value;
                    if (!(tool.type in actions)) {
                        actions[tool.type] = [];
                    }
                    actions[tool.type].push(tool);
                }
            }
            catch (e_5_1) {
                e_5 = { error: e_5_1 };
            }
            finally {
                try {
                    if (_y && !_y.done && (_f = _x.return))
                        _f.call(_x);
                }
                finally {
                    if (e_5)
                        throw e_5.error;
                }
            }
            // Add a proxy for each of the groups of tools.
            var make_proxy = function (tools, active) {
                if (active === void 0) {
                    active = false;
                }
                var proxy = new tool_proxy_1.ToolProxy({ tools: tools, active: active });
                _this._proxied_tools.push(proxy);
                return proxy;
            };
            try {
                for (var _z = tslib_1.__values(object_1.keys(gestures)), _0 = _z.next(); !_0.done; _0 = _z.next()) {
                    var event_type = _0.value;
                    var gesture = this.gestures[event_type];
                    gesture.tools = [];
                    var _loop_1 = function (tool_type) {
                        var e_12, _a;
                        var tools = gestures[event_type][tool_type];
                        if (tools.length > 0) {
                            if (event_type == 'multi') {
                                var _loop_2 = function (tool) {
                                    var proxy = make_proxy([tool]);
                                    gesture.tools.push(proxy);
                                    this_1.connect(proxy.properties.active.change, function () { return _this._active_change(proxy); });
                                };
                                try {
                                    for (var tools_2 = (e_12 = void 0, tslib_1.__values(tools)), tools_2_1 = tools_2.next(); !tools_2_1.done; tools_2_1 = tools_2.next()) {
                                        var tool = tools_2_1.value;
                                        _loop_2(tool);
                                    }
                                }
                                catch (e_12_1) {
                                    e_12 = { error: e_12_1 };
                                }
                                finally {
                                    try {
                                        if (tools_2_1 && !tools_2_1.done && (_a = tools_2.return))
                                            _a.call(tools_2);
                                    }
                                    finally {
                                        if (e_12)
                                            throw e_12.error;
                                    }
                                }
                            }
                            else {
                                var proxy_1 = make_proxy(tools);
                                gesture.tools.push(proxy_1);
                                this_1.connect(proxy_1.properties.active.change, function () { return _this._active_change(proxy_1); });
                            }
                        }
                    };
                    var this_1 = this;
                    try {
                        for (var _1 = (e_7 = void 0, tslib_1.__values(object_1.keys(gestures[event_type]))), _2 = _1.next(); !_2.done; _2 = _1.next()) {
                            var tool_type = _2.value;
                            _loop_1(tool_type);
                        }
                    }
                    catch (e_7_1) {
                        e_7 = { error: e_7_1 };
                    }
                    finally {
                        try {
                            if (_2 && !_2.done && (_h = _1.return))
                                _h.call(_1);
                        }
                        finally {
                            if (e_7)
                                throw e_7.error;
                        }
                    }
                }
            }
            catch (e_6_1) {
                e_6 = { error: e_6_1 };
            }
            finally {
                try {
                    if (_0 && !_0.done && (_g = _z.return))
                        _g.call(_z);
                }
                finally {
                    if (e_6)
                        throw e_6.error;
                }
            }
            this.actions = [];
            try {
                for (var _3 = tslib_1.__values(object_1.entries(actions)), _4 = _3.next(); !_4.done; _4 = _3.next()) {
                    var _5 = tslib_1.__read(_4.value, 2), tool_type = _5[0], tools = _5[1];
                    if (tool_type == 'CustomAction') {
                        try {
                            for (var tools_1 = (e_9 = void 0, tslib_1.__values(tools)), tools_1_1 = tools_1.next(); !tools_1_1.done; tools_1_1 = tools_1.next()) {
                                var tool = tools_1_1.value;
                                this.actions.push(make_proxy([tool]));
                            }
                        }
                        catch (e_9_1) {
                            e_9 = { error: e_9_1 };
                        }
                        finally {
                            try {
                                if (tools_1_1 && !tools_1_1.done && (_k = tools_1.return))
                                    _k.call(tools_1);
                            }
                            finally {
                                if (e_9)
                                    throw e_9.error;
                            }
                        }
                    }
                    else if (tools.length > 0) {
                        this.actions.push(make_proxy(tools)); // XXX
                    }
                }
            }
            catch (e_8_1) {
                e_8 = { error: e_8_1 };
            }
            finally {
                try {
                    if (_4 && !_4.done && (_j = _3.return))
                        _j.call(_3);
                }
                finally {
                    if (e_8)
                        throw e_8.error;
                }
            }
            this.inspectors = [];
            try {
                for (var _6 = tslib_1.__values(object_1.values(inspectors)), _7 = _6.next(); !_7.done; _7 = _6.next()) {
                    var tools = _7.value;
                    if (tools.length > 0)
                        this.inspectors.push(make_proxy(tools, true)); // XXX
                }
            }
            catch (e_10_1) {
                e_10 = { error: e_10_1 };
            }
            finally {
                try {
                    if (_7 && !_7.done && (_l = _6.return))
                        _l.call(_6);
                }
                finally {
                    if (e_10)
                        throw e_10.error;
                }
            }
            try {
                for (var _8 = tslib_1.__values(object_1.entries(this.gestures)), _9 = _8.next(); !_9.done; _9 = _8.next()) {
                    var _10 = tslib_1.__read(_9.value, 2), et = _10[0], gesture = _10[1];
                    if (gesture.tools.length == 0)
                        continue;
                    gesture.tools = array_1.sort_by(gesture.tools, function (tool) { return tool.default_order; });
                    if (!(et == 'pinch' || et == 'scroll' || et == 'multi'))
                        gesture.tools[0].active = true;
                }
            }
            catch (e_11_1) {
                e_11 = { error: e_11_1 };
            }
            finally {
                try {
                    if (_9 && !_9.done && (_m = _8.return))
                        _m.call(_8);
                }
                finally {
                    if (e_11)
                        throw e_11.error;
                }
            }
        };
        return ProxyToolbar;
    }(toolbar_base_1.ToolbarBase));
    exports.ProxyToolbar = ProxyToolbar;
    ProxyToolbar.__name__ = "ProxyToolbar";
    ProxyToolbar.init_ProxyToolbar();
    var ToolbarBoxView = /** @class */ (function (_super) {
        tslib_1.__extends(ToolbarBoxView, _super);
        function ToolbarBoxView() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ToolbarBoxView.prototype.initialize = function () {
            this.model.toolbar.toolbar_location = this.model.toolbar_location;
            _super.prototype.initialize.call(this);
        };
        Object.defineProperty(ToolbarBoxView.prototype, "child_models", {
            get: function () {
                return [this.model.toolbar]; // XXX
            },
            enumerable: true,
            configurable: true
        });
        ToolbarBoxView.prototype._update_layout = function () {
            this.layout = new layout_1.ContentBox(this.child_views[0].el);
            var toolbar = this.model.toolbar;
            if (toolbar.horizontal) {
                this.layout.set_sizing({
                    width_policy: "fit", min_width: 100, height_policy: "fixed",
                });
            }
            else {
                this.layout.set_sizing({
                    width_policy: "fixed", height_policy: "fit", min_height: 100,
                });
            }
        };
        return ToolbarBoxView;
    }(layout_dom_1.LayoutDOMView));
    exports.ToolbarBoxView = ToolbarBoxView;
    ToolbarBoxView.__name__ = "ToolbarBoxView";
    var ToolbarBox = /** @class */ (function (_super) {
        tslib_1.__extends(ToolbarBox, _super);
        function ToolbarBox(attrs) {
            return _super.call(this, attrs) || this;
        }
        ToolbarBox.init_ToolbarBox = function () {
            this.prototype.default_view = ToolbarBoxView;
            this.define({
                toolbar: [p.Instance],
                toolbar_location: [p.Location, "right"],
            });
        };
        return ToolbarBox;
    }(layout_dom_1.LayoutDOM));
    exports.ToolbarBox = ToolbarBox;
    ToolbarBox.__name__ = "ToolbarBox";
    ToolbarBox.init_ToolbarBox();
},
/* embed/standalone.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var document_1 = require(120) /* ../document */;
    var dom_view_1 = require(193) /* ../core/dom_view */;
    var build_views_1 = require(230) /* ../core/build_views */;
    var dom_1 = require(187) /* ../core/dom */;
    var dom_2 = require(497) /* ./dom */;
    // A map from the root model IDs to their views.
    exports.index = {};
    function add_document_standalone(document, element, roots, use_for_title) {
        if (roots === void 0) {
            roots = [];
        }
        if (use_for_title === void 0) {
            use_for_title = false;
        }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            function render_model(model) {
                return tslib_1.__awaiter(this, void 0, void 0, function () {
                    var root_el, root_models, idx, root, view;
                    return tslib_1.__generator(this, function (_a) {
                        switch (_a.label) {
                            case 0:
                                root_models = document.roots();
                                idx = root_models.indexOf(model);
                                root = roots[idx];
                                if (root != null)
                                    root_el = root;
                                else if (element.classList.contains(dom_2.BOKEH_ROOT))
                                    root_el = element;
                                else {
                                    root_el = dom_1.div({ class: dom_2.BOKEH_ROOT });
                                    element.appendChild(root_el);
                                }
                                return [4 /*yield*/, build_views_1.build_view(model, { parent: null })];
                            case 1:
                                view = _a.sent();
                                if (view instanceof dom_view_1.DOMView)
                                    view.renderTo(root_el);
                                views.set(model, view);
                                exports.index[model.id] = view;
                                return [2 /*return*/, view];
                        }
                    });
                });
            }
            function unrender_model(model) {
                var view = views.get(model);
                if (view != null) {
                    view.remove();
                    views.delete(model);
                    delete exports.index[model.id];
                }
            }
            var views, _a, _b, model, e_1_1;
            var e_1, _c;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        views = new Map();
                        _d.label = 1;
                    case 1:
                        _d.trys.push([1, 6, 7, 8]);
                        _a = tslib_1.__values(document.roots()), _b = _a.next();
                        _d.label = 2;
                    case 2:
                        if (!!_b.done)
                            return [3 /*break*/, 5];
                        model = _b.value;
                        return [4 /*yield*/, render_model(model)];
                    case 3:
                        _d.sent();
                        _d.label = 4;
                    case 4:
                        _b = _a.next();
                        return [3 /*break*/, 2];
                    case 5: return [3 /*break*/, 8];
                    case 6:
                        e_1_1 = _d.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 8];
                    case 7:
                        try {
                            if (_b && !_b.done && (_c = _a.return))
                                _c.call(_a);
                        }
                        finally {
                            if (e_1)
                                throw e_1.error;
                        }
                        return [7 /*endfinally*/];
                    case 8:
                        if (use_for_title)
                            window.document.title = document.title();
                        document.on_change(function (event) {
                            if (event instanceof document_1.RootAddedEvent)
                                render_model(event.model);
                            else if (event instanceof document_1.RootRemovedEvent)
                                unrender_model(event.model);
                            else if (use_for_title && event instanceof document_1.TitleChangedEvent)
                                window.document.title = event.title;
                        });
                        return [2 /*return*/, tslib_1.__spread(views.values())];
                }
            });
        });
    }
    exports.add_document_standalone = add_document_standalone;
},
/* embed/dom.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var dom_1 = require(187) /* ../core/dom */;
    var root_1 = require(388) /* ../styles/root */;
    // Matches Bokeh CSS class selector. Setting all Bokeh parent element class names
    // with this var prevents user configurations where css styling is unset.
    exports.BOKEH_ROOT = root_1.bk_root;
    function _get_element(elementid) {
        var element = document.getElementById(elementid);
        if (element == null)
            throw new Error("Error rendering Bokeh model: could not find #" + elementid + " HTML tag");
        if (!document.body.contains(element))
            throw new Error("Error rendering Bokeh model: element #" + elementid + " must be under <body>");
        // If autoload script, replace script tag with div for embedding.
        if (element.tagName == "SCRIPT") {
            var root_el = dom_1.div({ class: exports.BOKEH_ROOT });
            dom_1.replaceWith(element, root_el);
            element = root_el;
        }
        return element;
    }
    function _resolve_element(item) {
        var elementid = item.elementid;
        if (elementid != null)
            return _get_element(elementid);
        else
            return document.body;
    }
    exports._resolve_element = _resolve_element;
    function _resolve_root_elements(item) {
        var e_1, _a;
        var roots = [];
        if ((item.root_ids != null) && (item.roots != null)) {
            try {
                for (var _b = tslib_1.__values(item.root_ids), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var root_id = _c.value;
                    roots.push(_get_element(item.roots[root_id]));
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return))
                        _a.call(_b);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        }
        return roots;
    }
    exports._resolve_root_elements = _resolve_root_elements;
},
/* embed/server.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var connection_1 = require(499) /* ../client/connection */;
    var logging_1 = require(134) /* ../core/logging */;
    var standalone_1 = require(496) /* ./standalone */;
    // @internal
    function _get_ws_url(app_path, absolute_url) {
        var protocol = 'ws:';
        if (window.location.protocol == 'https:')
            protocol = 'wss:';
        var loc;
        if (absolute_url != null) {
            loc = document.createElement('a');
            loc.href = absolute_url;
        }
        else
            loc = window.location;
        if (app_path != null) {
            if (app_path == "/")
                app_path = "";
        }
        else
            app_path = loc.pathname.replace(/\/+$/, '');
        return protocol + '//' + loc.host + app_path + '/ws';
    }
    exports._get_ws_url = _get_ws_url;
    // map { websocket url to map { session id to promise of ClientSession } }
    var _sessions = {};
    function _get_session(websocket_url, token, args_string) {
        var session_id = connection_1.parse_token(token).session_id;
        if (!(websocket_url in _sessions))
            _sessions[websocket_url] = {};
        var subsessions = _sessions[websocket_url];
        if (!(session_id in subsessions))
            subsessions[session_id] = connection_1.pull_session(websocket_url, token, args_string);
        return subsessions[session_id];
    }
    // Fill element with the roots from token
    function add_document_from_session(websocket_url, token, element, roots, use_for_title) {
        if (roots === void 0) {
            roots = [];
        }
        if (use_for_title === void 0) {
            use_for_title = false;
        }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var args_string, session, error_1, session_id;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        args_string = window.location.search.substr(1);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, _get_session(websocket_url, token, args_string)];
                    case 2:
                        session = _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        session_id = connection_1.parse_token(token).session_id;
                        logging_1.logger.error("Failed to load Bokeh session " + session_id + ": " + error_1);
                        throw error_1;
                    case 4: return [2 /*return*/, standalone_1.add_document_standalone(session.document, element, roots, use_for_title)];
                }
            });
        });
    }
    exports.add_document_from_session = add_document_from_session;
},
/* client/connection.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var logging_1 = require(134) /* ../core/logging */;
    var document_1 = require(120) /* ../document */;
    var message_1 = require(500) /* ../protocol/message */;
    var receiver_1 = require(501) /* ../protocol/receiver */;
    var session_1 = require(502) /* ./session */;
    exports.DEFAULT_SERVER_WEBSOCKET_URL = "ws://localhost:5006/ws";
    exports.DEFAULT_TOKEN = "eyJzZXNzaW9uX2lkIjogImRlZmF1bHQifQ";
    var _connection_count = 0;
    function parse_token(token) {
        var payload = token.split('.')[0];
        var mod = payload.length % 4;
        if (mod != 0)
            payload = payload + "=".repeat(4 - mod);
        return JSON.parse(atob(payload.replace(/_/g, '/').replace(/-/g, '+')));
    }
    exports.parse_token = parse_token;
    var ClientConnection = /** @class */ (function () {
        function ClientConnection(url, token, args_string) {
            if (url === void 0) {
                url = exports.DEFAULT_SERVER_WEBSOCKET_URL;
            }
            if (token === void 0) {
                token = exports.DEFAULT_TOKEN;
            }
            if (args_string === void 0) {
                args_string = null;
            }
            this.url = url;
            this.token = token;
            this.args_string = args_string;
            this._number = _connection_count++;
            this.socket = null;
            this.session = null;
            this.closed_permanently = false;
            this._current_handler = null;
            this._pending_replies = new Map();
            this._pending_messages = [];
            this._receiver = new receiver_1.Receiver();
            this.id = parse_token(token).session_id.split('.')[0];
            logging_1.logger.debug("Creating websocket " + this._number + " to '" + this.url + "' session '" + this.id + "'");
        }
        ClientConnection.prototype.connect = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var versioned_url;
                var _this = this;
                return tslib_1.__generator(this, function (_b) {
                    if (this.closed_permanently)
                        throw new Error("Cannot connect() a closed ClientConnection");
                    if (this.socket != null)
                        throw new Error("Already connected");
                    this._current_handler = null;
                    this._pending_replies.clear();
                    this._pending_messages = [];
                    try {
                        versioned_url = "" + this.url;
                        if (this.args_string != null && this.args_string.length > 0)
                            versioned_url += "?" + this.args_string;
                        this.socket = new WebSocket(versioned_url, ["bokeh", this.token]);
                        return [2 /*return*/, new Promise(function (resolve, reject) {
                                // "arraybuffer" gives us binary data we can look at;
                                // if we just needed an opaque blob we could use "blob"
                                _this.socket.binaryType = "arraybuffer";
                                _this.socket.onopen = function () { return _this._on_open(resolve, reject); };
                                _this.socket.onmessage = function (event) { return _this._on_message(event); };
                                _this.socket.onclose = function (event) { return _this._on_close(event, reject); };
                                _this.socket.onerror = function () { return _this._on_error(reject); };
                            })];
                    }
                    catch (error) {
                        logging_1.logger.error("websocket creation failed to url: " + this.url);
                        logging_1.logger.error(" - " + error);
                        throw error;
                    }
                    return [2 /*return*/];
                });
            });
        };
        ClientConnection.prototype.close = function () {
            if (!this.closed_permanently) {
                logging_1.logger.debug("Permanently closing websocket connection " + this._number);
                this.closed_permanently = true;
                if (this.socket != null)
                    this.socket.close(1000, "close method called on ClientConnection " + this._number);
                this.session._connection_closed();
            }
        };
        ClientConnection.prototype._schedule_reconnect = function (milliseconds) {
            var _this = this;
            var retry = function () {
                // TODO commented code below until we fix reconnection to repull
                // the document when required. Otherwise, we get a lot of
                // confusing errors that are causing trouble when debugging.
                /*
                if (this.closed_permanently) {
                */
                if (!_this.closed_permanently)
                    logging_1.logger.info("Websocket connection " + _this._number + " disconnected, will not attempt to reconnect");
                return;
                /*
                } else {
                  logger.debug(`Attempting to reconnect websocket ${this._number}`)
                  this.connect()
                }
                */
            };
            setTimeout(retry, milliseconds);
        };
        ClientConnection.prototype.send = function (message) {
            if (this.socket == null)
                throw new Error("not connected so cannot send " + message);
            message.send(this.socket);
        };
        ClientConnection.prototype.send_with_reply = function (message) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var reply;
                var _this = this;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0: return [4 /*yield*/, new Promise(function (resolve, reject) {
                                _this._pending_replies.set(message.msgid(), { resolve: resolve, reject: reject });
                                _this.send(message);
                            })];
                        case 1:
                            reply = _b.sent();
                            if (reply.msgtype() === "ERROR")
                                throw new Error("Error reply " + reply.content.text);
                            else
                                return [2 /*return*/, reply];
                            return [2 /*return*/];
                    }
                });
            });
        };
        ClientConnection.prototype._pull_doc_json = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var message, reply;
                return tslib_1.__generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            message = message_1.Message.create("PULL-DOC-REQ", {});
                            return [4 /*yield*/, this.send_with_reply(message)];
                        case 1:
                            reply = _b.sent();
                            if (!("doc" in reply.content))
                                throw new Error("No 'doc' field in PULL-DOC-REPLY");
                            return [2 /*return*/, reply.content.doc];
                    }
                });
            });
        };
        ClientConnection.prototype._repull_session_doc = function (resolve, reject) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var _a, doc_json, document, patch, patch_message, _b, _c, msg, error_1;
                var e_1, _d;
                return tslib_1.__generator(this, function (_e) {
                    switch (_e.label) {
                        case 0:
                            logging_1.logger.debug(this.session ? "Repulling session" : "Pulling session for first time");
                            _e.label = 1;
                        case 1:
                            _e.trys.push([1, 3, , 4]);
                            return [4 /*yield*/, this._pull_doc_json()];
                        case 2:
                            doc_json = _e.sent();
                            if (this.session == null) {
                                if (this.closed_permanently) {
                                    logging_1.logger.debug("Got new document after connection was already closed");
                                    reject(new Error("The connection has been closed"));
                                }
                                else {
                                    document = document_1.Document.from_json(doc_json);
                                    patch = document_1.Document._compute_patch_since_json(doc_json, document);
                                    if (patch.events.length > 0) {
                                        logging_1.logger.debug("Sending " + patch.events.length + " changes from model construction back to server");
                                        patch_message = message_1.Message.create('PATCH-DOC', {}, patch);
                                        this.send(patch_message);
                                    }
                                    this.session = new session_1.ClientSession(this, document, this.id);
                                    try {
                                        for (_b = tslib_1.__values(this._pending_messages), _c = _b.next(); !_c.done; _c = _b.next()) {
                                            msg = _c.value;
                                            this.session.handle(msg);
                                        }
                                    }
                                    catch (e_1_1) {
                                        e_1 = { error: e_1_1 };
                                    }
                                    finally {
                                        try {
                                            if (_c && !_c.done && (_d = _b.return))
                                                _d.call(_b);
                                        }
                                        finally {
                                            if (e_1)
                                                throw e_1.error;
                                        }
                                    }
                                    this._pending_messages = [];
                                    logging_1.logger.debug("Created a new session from new pulled doc");
                                    resolve(this.session);
                                }
                            }
                            else {
                                this.session.document.replace_with_json(doc_json);
                                logging_1.logger.debug("Updated existing session with new pulled doc");
                                // Since the session already exists, we don't need to call `resolve` again.
                            }
                            return [3 /*break*/, 4];
                        case 3:
                            error_1 = _e.sent();
                            (_a = console.trace) === null || _a === void 0 ? void 0 : _a.call(console, error_1);
                            logging_1.logger.error("Failed to repull session " + error_1);
                            reject(error_1);
                            return [3 /*break*/, 4];
                        case 4: return [2 /*return*/];
                    }
                });
            });
        };
        ClientConnection.prototype._on_open = function (resolve, reject) {
            var _this = this;
            logging_1.logger.info("Websocket connection " + this._number + " is now open");
            this._current_handler = function (message) {
                _this._awaiting_ack_handler(message, resolve, reject);
            };
        };
        ClientConnection.prototype._on_message = function (event) {
            if (this._current_handler == null)
                logging_1.logger.error("Got a message with no current handler set");
            try {
                this._receiver.consume(event.data);
            }
            catch (e) {
                this._close_bad_protocol(e.toString());
            }
            var msg = this._receiver.message;
            if (msg != null) {
                var problem = msg.problem();
                if (problem != null)
                    this._close_bad_protocol(problem);
                this._current_handler(msg);
            }
        };
        ClientConnection.prototype._on_close = function (event, reject) {
            logging_1.logger.info("Lost websocket " + this._number + " connection, " + event.code + " (" + event.reason + ")");
            this.socket = null;
            this._pending_replies.forEach(function (pr) { return pr.reject("Disconnected"); });
            this._pending_replies.clear();
            if (!this.closed_permanently)
                this._schedule_reconnect(2000);
            reject(new Error("Lost websocket connection, " + event.code + " (" + event.reason + ")"));
        };
        ClientConnection.prototype._on_error = function (reject) {
            logging_1.logger.debug("Websocket error on socket " + this._number);
            var msg = "Could not open websocket";
            logging_1.logger.error("Failed to connect to Bokeh server: " + msg);
            reject(new Error(msg));
        };
        ClientConnection.prototype._close_bad_protocol = function (detail) {
            logging_1.logger.error("Closing connection: " + detail);
            if (this.socket != null)
                this.socket.close(1002, detail); // 1002 = protocol error
        };
        ClientConnection.prototype._awaiting_ack_handler = function (message, resolve, reject) {
            var _this = this;
            if (message.msgtype() === "ACK") {
                this._current_handler = function (message) { return _this._steady_state_handler(message); };
                // Reload any sessions
                this._repull_session_doc(resolve, reject);
            }
            else
                this._close_bad_protocol("First message was not an ACK");
        };
        ClientConnection.prototype._steady_state_handler = function (message) {
            var reqid = message.reqid();
            var pr = this._pending_replies.get(reqid);
            if (pr) {
                this._pending_replies.delete(reqid);
                pr.resolve(message);
            }
            else if (this.session) {
                this.session.handle(message);
            }
            else if (message.msgtype() != 'PATCH-DOC') {
                // This branch can be executed only before we get the document.
                // When we get the document, all of the patches will already be incorporated.
                // In general, it's not possible to apply patches received before the document,
                // since they may change some models that were removed before serving the document.
                this._pending_messages.push(message);
            }
        };
        return ClientConnection;
    }());
    exports.ClientConnection = ClientConnection;
    ClientConnection.__name__ = "ClientConnection";
    function pull_session(url, token, args_string) {
        var connection = new ClientConnection(url, token, args_string);
        return connection.connect();
    }
    exports.pull_session = pull_session;
},
/* protocol/message.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var string_1 = require(144) /* ../core/util/string */;
    var Message = /** @class */ (function () {
        function Message(header, metadata, content) {
            this.header = header;
            this.metadata = metadata;
            this.content = content;
            this.buffers = new Map();
        }
        Message.assemble = function (header_json, metadata_json, content_json) {
            var header = JSON.parse(header_json);
            var metadata = JSON.parse(metadata_json);
            var content = JSON.parse(content_json);
            return new Message(header, metadata, content);
        };
        Message.prototype.assemble_buffer = function (buf_header, buf_payload) {
            var nb = this.header.num_buffers != null ? this.header.num_buffers : 0;
            if (nb <= this.buffers.size)
                throw new Error("too many buffers received, expecting " + nb);
            var id = JSON.parse(buf_header).id;
            this.buffers.set(id, buf_payload);
        };
        // not defined for BokehJS, only *receiving* buffers is supported
        // add_buffer: (buf_header, buf_payload) ->
        // write_buffers: (socket)
        Message.create = function (msgtype, metadata, content) {
            if (content === void 0) {
                content = {};
            }
            var header = Message.create_header(msgtype);
            return new Message(header, metadata, content);
        };
        Message.create_header = function (msgtype) {
            return {
                msgid: string_1.uniqueId(),
                msgtype: msgtype,
            };
        };
        Message.prototype.complete = function () {
            if (this.header != null && this.metadata != null && this.content != null) {
                if (this.header.num_buffers != undefined)
                    return this.buffers.size == this.header.num_buffers;
                else
                    return true;
            }
            else
                return false;
        };
        Message.prototype.send = function (socket) {
            var nb = this.header.num_buffers != null ? this.header.num_buffers : 0;
            if (nb > 0)
                throw new Error("BokehJS only supports receiving buffers, not sending");
            var header_json = JSON.stringify(this.header);
            var metadata_json = JSON.stringify(this.metadata);
            var content_json = JSON.stringify(this.content);
            socket.send(header_json);
            socket.send(metadata_json);
            socket.send(content_json);
        };
        Message.prototype.msgid = function () {
            return this.header.msgid;
        };
        Message.prototype.msgtype = function () {
            return this.header.msgtype;
        };
        Message.prototype.reqid = function () {
            return this.header.reqid;
        };
        // return the reason we should close on bad protocol, if there is one
        Message.prototype.problem = function () {
            if (!('msgid' in this.header))
                return "No msgid in header";
            else if (!('msgtype' in this.header))
                return "No msgtype in header";
            else
                return null;
        };
        return Message;
    }());
    exports.Message = Message;
    Message.__name__ = "Message";
},
/* protocol/receiver.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var message_1 = require(500) /* ./message */;
    var types_1 = require(123) /* ../core/util/types */;
    var Receiver = /** @class */ (function () {
        function Receiver() {
            this.message = null;
            this._partial = null;
            this._fragments = [];
            this._buf_header = null;
            this._current_consumer = this._HEADER;
        }
        Receiver.prototype.consume = function (fragment) {
            this._current_consumer(fragment);
        };
        Receiver.prototype._HEADER = function (fragment) {
            this._assume_text(fragment);
            this.message = null;
            this._partial = null;
            this._fragments = [fragment];
            this._buf_header = null;
            this._current_consumer = this._METADATA;
        };
        Receiver.prototype._METADATA = function (fragment) {
            this._assume_text(fragment);
            this._fragments.push(fragment);
            this._current_consumer = this._CONTENT;
        };
        Receiver.prototype._CONTENT = function (fragment) {
            this._assume_text(fragment);
            this._fragments.push(fragment);
            var _a = tslib_1.__read(this._fragments.slice(0, 3), 3), header_json = _a[0], metadata_json = _a[1], content_json = _a[2];
            this._partial = message_1.Message.assemble(header_json, metadata_json, content_json);
            this._check_complete();
        };
        Receiver.prototype._BUFFER_HEADER = function (fragment) {
            this._assume_text(fragment);
            this._buf_header = fragment;
            this._current_consumer = this._BUFFER_PAYLOAD;
        };
        Receiver.prototype._BUFFER_PAYLOAD = function (fragment) {
            this._assume_binary(fragment);
            this._partial.assemble_buffer(this._buf_header, fragment);
            this._check_complete();
        };
        Receiver.prototype._assume_text = function (fragment) {
            if (!types_1.isString(fragment))
                throw new Error("Expected text fragment but received binary fragment");
        };
        Receiver.prototype._assume_binary = function (fragment) {
            if (!(fragment instanceof ArrayBuffer))
                throw new Error("Expected binary fragment but received text fragment");
        };
        Receiver.prototype._check_complete = function () {
            if (this._partial.complete()) {
                this.message = this._partial;
                this._current_consumer = this._HEADER;
            }
            else
                this._current_consumer = this._BUFFER_HEADER;
        };
        return Receiver;
    }());
    exports.Receiver = Receiver;
    Receiver.__name__ = "Receiver";
},
/* client/session.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var document_1 = require(120) /* ../document */;
    var message_1 = require(500) /* ../protocol/message */;
    var logging_1 = require(134) /* ../core/logging */;
    var ClientSession = /** @class */ (function () {
        function ClientSession(_connection, document, id) {
            var _this = this;
            this._connection = _connection;
            this.document = document;
            this.id = id;
            this._document_listener = function (event) {
                _this._document_changed(event);
            };
            this.document.on_change(this._document_listener, true);
        }
        ClientSession.prototype.handle = function (message) {
            var msgtype = message.msgtype();
            if (msgtype === 'PATCH-DOC')
                this._handle_patch(message);
            else if (msgtype === 'OK')
                this._handle_ok(message);
            else if (msgtype === 'ERROR')
                this._handle_error(message);
            else
                logging_1.logger.debug("Doing nothing with message " + message.msgtype());
        };
        ClientSession.prototype.close = function () {
            this._connection.close();
        };
        /*protected*/ ClientSession.prototype._connection_closed = function () {
            this.document.remove_on_change(this._document_listener);
        };
        // Sends a request to the server for info about the server, such as its Bokeh
        // version. Returns a promise, the value of the promise is a free-form dictionary
        // of server details.
        ClientSession.prototype.request_server_info = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var message, reply;
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            message = message_1.Message.create('SERVER-INFO-REQ', {});
                            return [4 /*yield*/, this._connection.send_with_reply(message)];
                        case 1:
                            reply = _a.sent();
                            return [2 /*return*/, reply.content];
                    }
                });
            });
        };
        // Sends some request to the server (no guarantee about which one) and returns
        // a promise which is completed when the server replies. The purpose of this
        // is that if you wait for the promise to be completed, you know the server
        // has processed the request. This is useful when writing tests because once
        // the server has processed this request it should also have processed any
        // events or requests you sent previously, which means you can check for the
        // results of that processing without a race condition. (This assumes the
        // server processes events in sequence, which it mostly has to semantically,
        // since reordering events might change the final state.)
        ClientSession.prototype.force_roundtrip = function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, this.request_server_info()];
                        case 1:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            });
        };
        ClientSession.prototype._document_changed = function (event) {
            // Filter out events that were initiated by the ClientSession itself
            if (event.setter_id === this.id) // XXX: not all document events define this
                return;
            var events = event instanceof document_1.DocumentEventBatch ? event.events : [event];
            var patch = this.document.create_json_patch(events);
            // TODO (havocp) the connection may be closed here, which will
            // cause this send to throw an error - need to deal with it more cleanly.
            var message = message_1.Message.create('PATCH-DOC', {}, patch);
            this._connection.send(message);
        };
        ClientSession.prototype._handle_patch = function (message) {
            this.document.apply_json_patch(message.content, message.buffers, this.id);
        };
        ClientSession.prototype._handle_ok = function (message) {
            logging_1.logger.trace("Unhandled OK reply to " + message.reqid());
        };
        ClientSession.prototype._handle_error = function (message) {
            logging_1.logger.error("Unhandled ERROR reply to " + message.reqid() + ": " + message.content.text);
        };
        return ClientSession;
    }());
    exports.ClientSession = ClientSession;
    ClientSession.__name__ = "ClientSession";
},
/* embed/notebook.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    var __asyncValues = (this && this.__asyncValues) || function (o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    };
    var document_1 = require(120) /* ../document */;
    var receiver_1 = require(501) /* ../protocol/receiver */;
    var logging_1 = require(134) /* ../core/logging */;
    var dom_1 = require(187) /* ../core/dom */;
    var object_1 = require(128) /* ../core/util/object */;
    var standalone_1 = require(496) /* ./standalone */;
    var dom_2 = require(497) /* ./dom */;
    var root_css_1 = tslib_1.__importDefault(require(188) /* ../styles/root.css */);
    var logo_css_1 = tslib_1.__importDefault(require(426) /* ../styles/logo.css */);
    var notebook_css_1 = tslib_1.__importDefault(require(504) /* ../styles/notebook.css */);
    dom_1.stylesheet.append(root_css_1.default);
    dom_1.stylesheet.append(logo_css_1.default);
    dom_1.stylesheet.append(notebook_css_1.default);
    // This exists to allow the @bokeh/jupyter_bokeh extension to store the
    // notebook kernel so that _init_comms can register the comms target.
    // This has to be available at Bokeh.embed.kernels in JupyterLab.
    exports.kernels = {};
    function _handle_notebook_comms(receiver, comm_msg) {
        if (comm_msg.buffers.length > 0)
            receiver.consume(comm_msg.buffers[0].buffer);
        else
            receiver.consume(comm_msg.content.data);
        var msg = receiver.message;
        if (msg != null)
            this.apply_json_patch(msg.content, msg.buffers);
    }
    function _init_comms(target, doc) {
        var _this = this;
        if (typeof Jupyter !== 'undefined' && Jupyter.notebook.kernel != null) {
            logging_1.logger.info("Registering Jupyter comms for target " + target);
            var comm_manager = Jupyter.notebook.kernel.comm_manager;
            try {
                comm_manager.register_target(target, function (comm) {
                    logging_1.logger.info("Registering Jupyter comms for target " + target);
                    var r = new receiver_1.Receiver();
                    comm.on_msg(_handle_notebook_comms.bind(doc, r));
                });
            }
            catch (e) {
                logging_1.logger.warn("Jupyter comms failed to register. push_notebook() will not function. (exception reported: " + e + ")");
            }
        }
        else if (doc.roots()[0].id in exports.kernels) {
            logging_1.logger.info("Registering JupyterLab comms for target " + target);
            var kernel = exports.kernels[doc.roots()[0].id];
            try {
                kernel.registerCommTarget(target, function (comm) {
                    logging_1.logger.info("Registering JupyterLab comms for target " + target);
                    var r = new receiver_1.Receiver();
                    comm.onMsg = _handle_notebook_comms.bind(doc, r);
                });
            }
            catch (e) {
                logging_1.logger.warn("Jupyter comms failed to register. push_notebook() will not function. (exception reported: " + e + ")");
            }
        }
        else if (typeof google != 'undefined' && google.colab.kernel != null) {
            logging_1.logger.info("Registering Google Colab comms for target " + target);
            var comm_manager = google.colab.kernel.comms;
            try {
                comm_manager.registerTarget(target, function (comm) {
                    return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        var e_1, _a, _b, r, _c, _d, message, content, buffers, _e, _f, buffer, msg, e_1_1_1;
                        var e_2, _g;
                        return tslib_1.__generator(this, function (_h) {
                            switch (_h.label) {
                                case 0:
                                    logging_1.logger.info("Registering Google Colab comms for target " + target);
                                    r = new receiver_1.Receiver();
                                    _h.label = 1;
                                case 1:
                                    _h.trys.push([1, 6, 7, 12]);
                                    _c = __asyncValues(comm.messages);
                                    _h.label = 2;
                                case 2: return [4 /*yield*/, _c.next()];
                                case 3:
                                    if (!(_d = _h.sent(), !_d.done))
                                        return [3 /*break*/, 5];
                                    message = _d.value;
                                    content = { data: message.data };
                                    buffers = [];
                                    try {
                                        for (_e = (e_2 = void 0, tslib_1.__values((_b = message.buffers) !== null && _b !== void 0 ? _b : [])), _f = _e.next(); !_f.done; _f = _e.next()) {
                                            buffer = _f.value;
                                            buffers.push(new DataView(buffer));
                                        }
                                    }
                                    catch (e_2_1) {
                                        e_2 = { error: e_2_1 };
                                    }
                                    finally {
                                        try {
                                            if (_f && !_f.done && (_g = _e.return))
                                                _g.call(_e);
                                        }
                                        finally {
                                            if (e_2)
                                                throw e_2.error;
                                        }
                                    }
                                    msg = { content: content, buffers: buffers };
                                    _handle_notebook_comms.bind(doc)(r, msg);
                                    _h.label = 4;
                                case 4: return [3 /*break*/, 2];
                                case 5: return [3 /*break*/, 12];
                                case 6:
                                    e_1_1_1 = _h.sent();
                                    e_1 = { error: e_1_1_1 };
                                    return [3 /*break*/, 12];
                                case 7:
                                    _h.trys.push([7, , 10, 11]);
                                    if (!(_d && !_d.done && (_a = _c.return)))
                                        return [3 /*break*/, 9];
                                    return [4 /*yield*/, _a.call(_c)];
                                case 8:
                                    _h.sent();
                                    _h.label = 9;
                                case 9: return [3 /*break*/, 11];
                                case 10:
                                    if (e_1)
                                        throw e_1.error;
                                    return [7 /*endfinally*/];
                                case 11: return [7 /*endfinally*/];
                                case 12: return [2 /*return*/];
                            }
                        });
                    });
                });
            }
            catch (e) {
                logging_1.logger.warn("Google Colab comms failed to register. push_notebook() will not function. (exception reported: " + e + ")");
            }
        }
        else {
            console.warn("Jupyter notebooks comms not available. push_notebook() will not function. If running JupyterLab ensure the latest @bokeh/jupyter_bokeh extension is installed. In an exported notebook this warning is expected.");
        }
    }
    function embed_items_notebook(docs_json, render_items) {
        var e_3, _e;
        if (object_1.size(docs_json) != 1)
            throw new Error("embed_items_notebook expects exactly one document in docs_json");
        var document = document_1.Document.from_json(object_1.values(docs_json)[0]);
        try {
            for (var render_items_1 = tslib_1.__values(render_items), render_items_1_1 = render_items_1.next(); !render_items_1_1.done; render_items_1_1 = render_items_1.next()) {
                var item = render_items_1_1.value;
                if (item.notebook_comms_target != null)
                    _init_comms(item.notebook_comms_target, document);
                var element = dom_2._resolve_element(item);
                var roots = dom_2._resolve_root_elements(item);
                standalone_1.add_document_standalone(document, element, roots);
            }
        }
        catch (e_3_1) {
            e_3 = { error: e_3_1 };
        }
        finally {
            try {
                if (render_items_1_1 && !render_items_1_1.done && (_e = render_items_1.return))
                    _e.call(render_items_1);
            }
            finally {
                if (e_3)
                    throw e_3.error;
            }
        }
    }
    exports.embed_items_notebook = embed_items_notebook;
},
/* styles/notebook.css.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var css = "\n/* notebook specific tweaks so no black outline and matching padding\n/* can't be wrapped inside bk-root. here are the offending jupyter lines:\n/* https://github.com/jupyter/notebook/blob/master/notebook/static/notebook/less/renderedhtml.less#L59-L76 */\n.rendered_html .bk-root .bk-tooltip table,\n.rendered_html .bk-root .bk-tooltip tr,\n.rendered_html .bk-root .bk-tooltip th,\n.rendered_html .bk-root .bk-tooltip td {\n  border: none;\n  padding: 1px;\n}\n";
    exports.default = css;
},
/* protocol/index.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    var tslib_1 = require(1) /* tslib */;
    tslib_1.__exportStar(require(500) /* ./message */, exports);
    tslib_1.__exportStar(require(501) /* ./receiver */, exports);
},
/* testing.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    // Just a dumb key/value record for collecting arbitrary info for tests
    exports.results = {};
    // Selenium has race conditions that make it difficult to read out the
    // results structure. This function deletes/creates a div that can act as
    // a semaphore. Tests should wait for the previous div to be stale, then
    // find the new div. At that point the results should be available
    function update_test_div() {
        var body = document.getElementsByTagName("body")[0];
        var col = document.getElementsByClassName("bokeh-test-div");
        if (col.length == 1) {
            body.removeChild(col[0]);
            delete col[0];
        }
        var box = document.createElement("div");
        box.classList.add("bokeh-test-div");
        box.style.display = "none";
        body.insertBefore(box, body.firstChild);
    }
    function init() {
        update_test_div();
    }
    exports.init = init;
    function record0(key, value) {
        exports.results[key] = value;
    }
    exports.record0 = record0;
    function record(key, value) {
        exports.results[key] = value;
        update_test_div();
    }
    exports.record = record;
    function count(key) {
        if (exports.results[key] == undefined)
            exports.results[key] = 0;
        exports.results[key] += 1;
        update_test_div();
    }
    exports.count = count;
},
/* safely.js */ function _(require, module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    // Keep this code as terse and as close to vanila JS as possible. If we
    // arrived here, it means we should trust no one and need to act properly.
    function _burst_into_flames(error) {
        // Make box
        var box = document.createElement("div");
        box.style.backgroundColor = "#f2dede";
        box.style.border = "1px solid #a94442";
        box.style.borderRadius = "4px";
        box.style.display = "inline-block";
        box.style.fontFamily = "sans-serif";
        box.style.marginTop = "5px";
        box.style.minWidth = "200px";
        box.style.padding = "5px 5px 5px 10px";
        box.classList.add("bokeh-error-box-into-flames");
        // Make button
        var button = document.createElement("span");
        button.style.backgroundColor = "#a94442";
        button.style.borderRadius = "0px 4px 0px 0px";
        button.style.color = "white";
        button.style.cursor = "pointer";
        button.style.cssFloat = "right";
        button.style.fontSize = "0.8em";
        button.style.margin = "-6px -6px 0px 0px";
        button.style.padding = "2px 5px 4px 5px";
        button.title = "close";
        button.setAttribute("aria-label", "close");
        button.appendChild(document.createTextNode("x"));
        button.addEventListener("click", function () { return body.removeChild(box); });
        // Make title
        var title = document.createElement("h3");
        title.style.color = "#a94442";
        title.style.margin = "8px 0px 0px 0px";
        title.style.padding = "0px";
        title.appendChild(document.createTextNode("Bokeh Error"));
        // Make message
        var message = document.createElement("pre");
        message.style.whiteSpace = "unset";
        message.style.overflowX = "auto";
        var text = error instanceof Error ? error.message : error;
        message.appendChild(document.createTextNode(text));
        // Add pieces to box
        box.appendChild(button);
        box.appendChild(title);
        box.appendChild(message);
        // Put box in doc
        var body = document.getElementsByTagName("body")[0];
        body.insertBefore(box, body.firstChild);
    }
    function safely(fn, silent) {
        if (silent === void 0) {
            silent = false;
        }
        try {
            return fn();
        }
        catch (error) {
            _burst_into_flames(error);
            if (!silent)
                throw error;
            else
                return;
        }
    }
    exports.safely = safely;
},
], 0, {"legacy":0,"polyfill":2,"main":116,"index":117,"version":118,"embed/index":119,"document/index":120,"document/document":121,"base":122,"core/util/types":123,"core/util/array":124,"core/util/math":125,"core/util/assert":126,"core/util/arrayable":127,"core/util/object":128,"core/has_props":129,"core/signaling":130,"core/util/callback":131,"core/util/refs":132,"core/properties":133,"core/logging":134,"core/enums":135,"core/kinds":136,"core/util/color":137,"core/util/svg_colors":138,"core/types":139,"core/util/eq":140,"core/util/data_structures":141,"core/settings":142,"core/property_mixins":143,"core/util/string":144,"core/util/ndarray":145,"core/util/serialization":146,"core/util/compat":147,"core/util/pretty":148,"models/index":149,"models/annotations/index":150,"models/annotations/annotation":151,"core/util/projections":152,"models/renderers/renderer":185,"core/view":186,"core/dom":187,"styles/root.css":188,"core/visuals":189,"core/util/svg":190,"core/util/affine":191,"models/canvas/canvas":192,"core/dom_view":193,"core/util/bbox":194,"core/util/canvas":195,"model":196,"models/canvas/coordinates":197,"models/annotations/arrow":198,"models/annotations/arrow_head":199,"models/sources/column_data_source":200,"models/sources/columnar_data_source":201,"models/sources/data_source":202,"models/selections/selection":203,"core/selection_manager":204,"models/renderers/glyph_renderer":205,"models/renderers/data_renderer":206,"models/glyphs/line":207,"models/glyphs/xy_glyph":208,"models/glyphs/glyph":209,"core/util/spatial":210,"models/ranges/factor_range":213,"models/ranges/range":214,"models/glyphs/utils":215,"core/hittest":216,"models/glyphs/webgl/line":217,"models/glyphs/webgl/utils/index":218,"models/glyphs/webgl/utils/program":219,"models/glyphs/webgl/utils/buffer":220,"models/glyphs/webgl/utils/texture":221,"models/glyphs/webgl/base":222,"models/glyphs/webgl/line.vert":223,"models/glyphs/webgl/line.frag":224,"models/glyphs/patch":225,"models/glyphs/harea":226,"models/glyphs/area":227,"models/glyphs/varea":228,"models/sources/cds_view":229,"core/build_views":230,"models/renderers/graph_renderer":231,"models/graphs/graph_hit_test_policy":232,"models/selections/interaction_policy":233,"core/util/typed_array":234,"core/util/set":235,"document/events":236,"models/annotations/band":237,"models/annotations/upper_lower":238,"models/annotations/box_annotation":239,"models/annotations/color_bar":240,"models/tickers/basic_ticker":241,"models/tickers/adaptive_ticker":242,"models/tickers/continuous_ticker":243,"models/tickers/ticker":244,"models/formatters/basic_tick_formatter":245,"models/formatters/tick_formatter":246,"models/mappers/index":247,"models/mappers/categorical_color_mapper":248,"models/mappers/categorical_mapper":249,"models/mappers/color_mapper":250,"models/mappers/mapper":251,"models/transforms/transform":252,"models/mappers/categorical_marker_mapper":253,"models/mappers/categorical_pattern_mapper":254,"models/mappers/continuous_color_mapper":255,"models/mappers/linear_color_mapper":256,"models/mappers/log_color_mapper":257,"models/mappers/scanning_color_mapper":258,"models/mappers/eqhist_color_mapper":259,"models/scales/linear_scale":260,"models/scales/continuous_scale":261,"models/scales/scale":262,"models/transforms/index":263,"models/transforms/customjs_transform":264,"models/transforms/dodge":265,"models/transforms/range_transform":266,"models/transforms/interpolator":267,"models/transforms/jitter":268,"models/transforms/linear_interpolator":269,"models/transforms/step_interpolator":270,"models/scales/linear_interpolation_scale":271,"models/scales/log_scale":272,"models/ranges/range1d":273,"core/util/text":274,"models/annotations/label":275,"models/annotations/text_annotation":276,"models/annotations/label_set":277,"models/annotations/legend":278,"models/annotations/legend_item":279,"core/vectorization":280,"models/annotations/poly_annotation":281,"models/annotations/slope":282,"models/annotations/span":283,"models/annotations/title":284,"models/annotations/toolbar_panel":285,"models/annotations/tooltip":286,"styles/tooltips":287,"styles/mixins":288,"styles/tooltips.css":289,"models/annotations/whisker":290,"models/axes/index":291,"models/axes/axis":292,"models/renderers/guide_renderer":293,"models/axes/categorical_axis":294,"models/tickers/categorical_ticker":295,"models/formatters/categorical_tick_formatter":296,"models/axes/continuous_axis":297,"models/axes/datetime_axis":298,"models/axes/linear_axis":299,"models/formatters/datetime_tick_formatter":300,"core/util/templating":302,"models/tickers/datetime_ticker":305,"models/tickers/composite_ticker":306,"models/tickers/days_ticker":307,"models/tickers/single_interval_ticker":308,"models/tickers/util":309,"models/tickers/months_ticker":310,"models/tickers/years_ticker":311,"models/axes/log_axis":312,"models/formatters/log_tick_formatter":313,"models/tickers/log_ticker":314,"models/axes/mercator_axis":315,"models/formatters/mercator_tick_formatter":316,"models/tickers/mercator_ticker":317,"models/callbacks/index":318,"models/callbacks/customjs":319,"models/callbacks/callback":320,"models/callbacks/open_url":321,"models/canvas/index":322,"models/canvas/cartesian_frame":323,"models/scales/categorical_scale":324,"models/ranges/data_range1d":325,"models/ranges/data_range":326,"core/layout/index":327,"core/layout/types":328,"core/layout/layoutable":329,"core/layout/alignments":330,"core/layout/grid":331,"core/layout/html":332,"models/expressions/index":333,"models/expressions/expression":334,"models/expressions/stack":335,"models/expressions/cumsum":336,"models/filters/index":337,"models/filters/boolean_filter":338,"models/filters/filter":339,"models/filters/customjs_filter":340,"models/filters/group_filter":341,"models/filters/index_filter":342,"models/formatters/index":343,"models/formatters/func_tick_formatter":344,"models/formatters/numeral_tick_formatter":345,"models/formatters/printf_tick_formatter":346,"models/glyphs/index":347,"models/glyphs/annular_wedge":348,"models/glyphs/annulus":349,"models/glyphs/arc":350,"models/glyphs/bezier":351,"models/glyphs/circle":352,"models/glyphs/webgl/markers":353,"models/glyphs/webgl/markers.vert":354,"models/glyphs/webgl/markers.frag":355,"models/glyphs/center_rotatable":356,"models/glyphs/ellipse":357,"models/glyphs/ellipse_oval":358,"models/glyphs/hbar":359,"models/glyphs/box":360,"models/glyphs/hex_tile":361,"models/glyphs/image":362,"models/glyphs/image_base":363,"models/glyphs/image_rgba":364,"models/glyphs/image_url":365,"core/util/image":366,"models/glyphs/multi_line":367,"models/glyphs/multi_polygons":368,"models/glyphs/oval":369,"models/glyphs/patches":370,"models/glyphs/quad":371,"models/glyphs/quadratic":372,"models/glyphs/ray":373,"models/glyphs/rect":374,"models/glyphs/segment":375,"models/glyphs/step":376,"models/glyphs/text":377,"models/glyphs/vbar":378,"models/glyphs/wedge":379,"models/graphs/index":380,"models/graphs/layout_provider":381,"models/graphs/static_layout_provider":382,"models/grids/index":383,"models/grids/grid":384,"models/layouts/index":385,"models/layouts/box":386,"models/layouts/layout_dom":387,"styles/root":388,"models/layouts/column":389,"models/layouts/grid_box":390,"models/layouts/html_box":391,"models/layouts/row":392,"models/layouts/spacer":393,"models/layouts/tabs":394,"styles/tabs":395,"styles/buttons":396,"styles/menus":397,"styles/buttons.css":398,"styles/menus.css":399,"styles/tabs.css":400,"models/layouts/widget_box":401,"models/markers/index":402,"models/markers/defs":403,"models/markers/marker":404,"models/markers/scatter":405,"models/plots/index":406,"models/plots/gmap_plot":407,"models/plots/plot":408,"models/tools/toolbar":409,"models/tools/inspectors/inspect_tool":410,"models/tools/button_tool":411,"models/tools/tool":413,"styles/toolbar":414,"styles/toolbar.css":415,"styles/icons.css":416,"core/util/menus":417,"core/util/iterator":418,"models/tools/on_off_button":419,"models/tools/toolbar_base":420,"models/tools/gestures/gesture_tool":421,"models/tools/actions/action_tool":422,"models/tools/actions/help_tool":423,"styles/icons":424,"styles/logo":425,"styles/logo.css":426,"models/plots/plot_canvas":427,"core/bokeh_events":428,"core/ui_events":429,"core/util/wheel":430,"core/util/throttle":431,"core/layout/border":432,"core/layout/side_panel":433,"models/plots/gmap_plot_canvas":434,"models/ranges/index":435,"models/renderers/index":436,"models/scales/index":437,"models/selections/index":438,"models/sources/index":439,"models/sources/server_sent_data_source":440,"models/sources/web_data_source":441,"models/sources/ajax_data_source":442,"models/sources/geojson_data_source":443,"models/tickers/index":444,"models/tickers/fixed_ticker":445,"models/tiles/index":446,"models/tiles/bbox_tile_source":447,"models/tiles/mercator_tile_source":448,"models/tiles/tile_source":449,"models/tiles/tile_utils":450,"models/tiles/quadkey_tile_source":451,"models/tiles/tile_renderer":452,"models/tiles/wmts_tile_source":453,"styles/tiles":454,"styles/tiles.css":455,"models/tiles/tms_tile_source":456,"models/textures/index":457,"models/textures/canvas_texture":458,"models/textures/texture":459,"models/textures/image_url_texture":460,"models/tools/index":461,"models/tools/actions/custom_action":462,"models/tools/actions/redo_tool":463,"models/tools/actions/reset_tool":464,"models/tools/actions/save_tool":465,"models/tools/actions/undo_tool":466,"models/tools/actions/zoom_in_tool":467,"models/tools/actions/zoom_base_tool":468,"core/util/zoom":469,"models/tools/actions/zoom_out_tool":470,"models/tools/edit/edit_tool":471,"models/tools/edit/box_edit_tool":472,"models/tools/edit/freehand_draw_tool":473,"models/tools/edit/point_draw_tool":474,"models/tools/edit/poly_draw_tool":475,"models/tools/edit/poly_tool":476,"models/tools/edit/poly_edit_tool":477,"models/tools/gestures/box_select_tool":478,"models/tools/gestures/select_tool":479,"models/tools/util":480,"models/tools/gestures/box_zoom_tool":481,"models/tools/gestures/lasso_select_tool":482,"models/tools/gestures/poly_select_tool":483,"models/tools/edit/line_edit_tool":484,"models/tools/edit/line_tool":485,"models/tools/gestures/pan_tool":486,"models/tools/gestures/range_tool":487,"models/tools/gestures/tap_tool":488,"models/tools/gestures/wheel_pan_tool":489,"models/tools/gestures/wheel_zoom_tool":490,"models/tools/inspectors/crosshair_tool":491,"models/tools/inspectors/customjs_hover":492,"models/tools/inspectors/hover_tool":493,"models/tools/tool_proxy":494,"models/tools/toolbar_box":495,"embed/standalone":496,"embed/dom":497,"embed/server":498,"client/connection":499,"protocol/message":500,"protocol/receiver":501,"client/session":502,"embed/notebook":503,"styles/notebook.css":504,"protocol/index":505,"testing":506,"safely":507}, {});
})

//# sourceMappingURL=bokeh.legacy.js.map
